# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `anova` Chapter.

``anova`` - Analysis of Variance

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with methods for analysing the results of
designed experiments.
The range of experiments covered include:

- single factor designs with equal sized blocks such as randomized
  complete block and balanced incomplete block designs,

- row and column designs such as Latin squares, and

- complete factorial designs.

Further designs may be analysed by combining the analyses provided by
multiple calls to functions or by using general linear model functions
provided in submodule ``correg``.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def hier2(y, n, k, lsub, nobs, l, ngp, gbar, sgbar, ss, idf, f, fp):
    u"""
    Two-way analysis of variance, hierarchical classification, subgroups
    of unequal size.

    For full information please refer to the NAG Library document for
    g04ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04agf.html

    Parameters
    ----------
    y : float, shape (`n`)
        The elements of `y` must contain the observations y_mij in the
        following order:

            y_111,y_211,...,y_{n_11 11},y_112,y_212,...,y_{n_12
            12},...,y_{11l_1},...,

y_{n_{1l_1}1l_1},...,y_{1ij},...,y_{n_{ij}ij},...,y_{1kl_k},...,y_{n_{kl_k}kl_k}.

        In words, the ordering is by group, and within each group is by
        subgroup, the members of each subgroup being in consecutive
        locations in `y`.

    n : int
        n, the total number of observations.

    k : int
        k, the number of groups.

    lsub : int, shape (`k`)
        The number of subgroups within group i, l_i, for i = 1,2,...,k.

    nobs : int, shape (`l`)
        The numbers of observations in each subgroup, n_{ij}, in the
        following order:

n_11,n_12,...,n_{1l_1},n_21,...,n_{2l_2},...,n_{k1},...,n_{kl_k}

    l : int
        l, the total number of subgroups.

    ngp : int, shape (`k`), modified in place
        On exit: the total number of observations in group i, n_{i.},
        for i = 1,2,...,k.

    gbar : float, shape (`k`), modified in place
        On exit: the mean for group i, y__{.i.}, for i = 1,2,...,k.

    sgbar : float, shape (`l`), modified in place
        On exit: the subgroup means, y__{.ij}, in the following order:

y__.11,y__.12,...,y__.1l_1,y__.21,y__.22,...,y__.2l_2,...,y__{.k1},y__{.k2},...,y__.kl_k.

    ss : float, shape (4), modified in place
        On exit: contains the sums of squares for the analysis of
        variance, as follows;

        - `ss`[0] = Between group sum of squares, `ss`_g,

        - `ss`[1] = Between subgroup within groups sum of squares,
          `ss`_{sg},

        - `ss`[2] = Residual sum of squares, `ss`_res,

        - `ss`[3] = Corrected total sum of squares, `ss`_tot.

    idf : int, shape (4), modified in place
        On exit: contains the degrees of freedom attributable to each
        sum of squares in the analysis of variance, as follows:

        - `idf`[0] = Degrees of freedom for between group sum of
          squares,

        - `idf`[1] = Degrees of freedom for between subgroup within
          groups sum of squares,

        - `idf`[2] = Degrees of freedom for residual sum of squares,

        - `idf`[3] = Degrees of freedom for corrected total sum of
          squares.

    f : float, shape (2), modified in place
        On exit: contains the mean square ratios, F_1 and F_2, for the
        between groups variation, and the between subgroups within
        groups variation, with respect to the residual, respectively.

    fp : float, shape (2), modified in place
        On exit: contains the significances of the mean square ratios,
        p_1 and p_2 respectively.

    Returns
    -------
    gm : float
        The grand mean, y__....

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` > 1.

        (`errno` 2)
            On entry, i = *<value>* and `lsub`[i-1] = *<value>*.

            Constraint: `lsub`[i-1] > 0.

        (`errno` 3)
            On entry, sum_i`lsub`[i] = *<value>* and `l` = *<value>*.

            Constraint: sum_i`lsub`[i] = `l`.

        (`errno` 4)
            On entry, i = *<value>* and `nobs`[i-1] = *<value>*.

            Constraint: `nobs`[i-1] > 0.

        (`errno` 5)
            On entry, sum_i`nobs`[i-1] = *<value>* and `n` = *<value>*.

            Constraint: sum_i`nobs`[i-1] = `n`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The total corrected sum of squares is zero.

        (`errno` 7)
            The residual sum of squares is zero.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.hier2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineIntArrayType.c_type, # Input, lsub[k].
        utils._EngineIntArrayType.c_type, # Input, nobs[l].
        utils._BYREF_INT, # Input, l.
        utils._EngineIntArrayType.c_type, # Output, ngp[k].
        utils._EngineFloat64ArrayType.c_type, # Output, gbar[k].
        utils._EngineFloat64ArrayType.c_type, # Output, sgbar[l].
        utils._BYREF_FLOAT64, # Output, gm.
        utils._EngineFloat64ArrayType.c_type, # Output, ss[4].
        utils._EngineIntArrayType.c_type, # Output, idf[4].
        utils._EngineFloat64ArrayType.c_type, # Output, f[2].
        utils._EngineFloat64ArrayType.c_type, # Output, fp[2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _lsub = utils._EngineIntArrayType(
        lsub, {'fun_name': fun_name, 'entity': 'lsub'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    _nobs = utils._EngineIntArrayType(
        nobs, {'fun_name': fun_name, 'entity': 'nobs'},
        exp_shape=(_l_py,),
    ).to_ctypes()
    _ngp_cls = utils._EngineIntArrayType(
        ngp, {'fun_name': fun_name, 'entity': 'ngp'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _ngp = _ngp_cls.to_ctypes(can_reference_contents=False)
    _gbar_cls = utils._EngineFloat64ArrayType(
        gbar, {'fun_name': fun_name, 'entity': 'gbar'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _gbar = _gbar_cls.to_ctypes(can_reference_contents=False)
    _sgbar_cls = utils._EngineFloat64ArrayType(
        sgbar, {'fun_name': fun_name, 'entity': 'sgbar'},
        exp_shape=(_l_py,),
        check_contents=False,
    )
    _sgbar = _sgbar_cls.to_ctypes(can_reference_contents=False)
    _gm = utils._EngineFloat64ScalarType.empty_ctypes()
    _ss_cls = utils._EngineFloat64ArrayType(
        ss, {'fun_name': fun_name, 'entity': 'ss'},
        exp_shape=(4,),
        check_contents=False,
    )
    _ss = _ss_cls.to_ctypes(can_reference_contents=False)
    _idf_cls = utils._EngineIntArrayType(
        idf, {'fun_name': fun_name, 'entity': 'idf'},
        exp_shape=(4,),
        check_contents=False,
    )
    _idf = _idf_cls.to_ctypes(can_reference_contents=False)
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(2,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _fp_cls = utils._EngineFloat64ArrayType(
        fp, {'fun_name': fun_name, 'entity': 'fp'},
        exp_shape=(2,),
        check_contents=False,
    )
    _fp = _fp_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _y,
        _n,
        _k,
        _lsub,
        _nobs,
        _l,
        _ngp,
        _gbar,
        _sgbar,
        _gm,
        _ss,
        _idf,
        _f,
        _fp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and lsub[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lsub[i-1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_ilsub[i] = {:s} and l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sum_ilsub[i] = l.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and nobs[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nobs[i-1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_inobs[i-1] = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sum_inobs[i-1] = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The total corrected sum of squares is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual sum of squares is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ngp_cls.refresh_py(_ngp)
        _gbar_cls.refresh_py(_gbar)
        _sgbar_cls.refresh_py(_sgbar)
        _ss_cls.refresh_py(_ss)
        _idf_cls.refresh_py(_idf)
        _f_cls.refresh_py(_f)
        _fp_cls.refresh_py(_fp)
        _gm_py = utils._EngineFloat64ScalarType.to_py(_gm)
        return_data = _gm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['gm'],
            )(
                _gm_py,
            )
    _err_class.manage()
    return return_data

def random(n, y, iblock, nt, it, bmean, tmean, tabl, c, irep, r, ef, tol, irdf):
    u"""
    Analysis of variance, randomized block or completely randomized
    design, treatment means and standard errors.

    For full information please refer to the NAG Library document for
    g04bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04bbf.html

    Parameters
    ----------
    n : int
        The number of observations.

    y : float, shape (`n`)
        The observations in the order as described by `iblock` and `nt`.

    iblock : int
        Indicates the block structure.

        abs(`iblock`) <= 1

          There are no blocks, i.e., it is a completely randomized
          design.

        `iblock` >= 2

          There are `iblock` blocks and the data should be input by
          blocks, i.e., `y` must contain the observations for block 1
          followed by the observations for block 2, etc.

        `iblock` <= -2

          There are abs(`iblock`) blocks and the data is input in
          parallel with respect to blocks, i.e., `y`[0] must contain the
          first observation for block 1, `y`[1] must contain the first
          observation for block 2...`y`[abs(`iblock`)-1] must contain
          the first observation for block
          abs(`iblock`),`y`[abs(`iblock`+1)-1] must contain the second
          observation for block 1, etc.

    nt : int
        The number of treatments. If only blocks are required in the
        analysis then set `nt` = 1.

    it : int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `nt`>=2: `n`; otherwise: 1.

        `it`[i-1] indicates which of the `nt` treatments plot i
        received, for i = 1,2,...,`n`.

        If `nt` = 1, `it` is not referenced.

    bmean : float, shape (|`iblock`|), modified in place
        On exit: if abs(`iblock`) >= 2, `bmean`[j-1] contains the mean
        for the jth block, beta^_j, for j = 1,2,...,b.

    tmean : float, shape (`nt`), modified in place
        On exit: if `nt` >= 2, `tmean`[l-1] contains the (adjusted) mean
        for the lth treatment, mu^*+tau^_l, for l = 1,2,...,t, where
        mu^* is the mean of the treatment adjusted observations,
        y_{ij(l)}-tau^_l.

    tabl : float, shape (4, 5), modified in place
        On exit: the analysis of variance table. Column 1 contains the
        degrees of freedom, column 2 the sum of squares, and where
        appropriate, column 3 the mean squares, column 4 the F-statistic
        and column 5 the significance level of the F-statistic. Row 1 is
        for Blocks, row 2 for Treatments, row 3 for Residual and row 4
        for Total. Mean squares are computed for all but the Total row;
        F-statistics and significance are computed for Treatments and
        Blocks, if present. Any unfilled cells are set to zero.

    c : float, shape (`nt`, `nt`), modified in place
        On exit: if `nt` >= 2, the upper triangular part of `c` contains
        the variance-covariance matrix of the treatment effects, the
        strictly lower triangular part contains the standard errors of
        the difference between two treatment effects (means), i.e.,
        `c`[i-1,j-1] contains the covariance of treatment i and j if j
        >= i and the standard error of the difference between treatment
        i and j if j < i, for j = 1,2,...,t, for i = 1,2,...,t.

    irep : int, shape (`nt`), modified in place
        On exit: if `nt` >= 2, the treatment replications, R_{ll}, for l
        = 1,2,...,`nt`.

    r : float, shape (`n`), modified in place
        On exit: the residuals, r_i, for i = 1,2,...,`n`.

    ef : float, shape (`nt`), modified in place
        On exit: if `nt` >= 2, the canonical efficiency factors.

    tol : float
        The tolerance value used to check for zero eigenvalues of the
        matrix Omega. If `tol` = 0.0 a default value of 10^(-5) is used.

    irdf : int
        An adjustment to the degrees of freedom for the residual and
        total.

        `irdf` >= 1

          The degrees of freedom for the total is set to `n`-`irdf` and
          the residual degrees of freedom adjusted accordingly.

        `irdf` = 0

          The total degrees of freedom for the total is set to `n`-1, as
          usual.

    Returns
    -------
    gmean : float
        The grand mean, mu^.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irdf` = *<value>*.

            Constraint: `irdf` >= 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, no blocks or treatments in model.

        (`errno` 1)
            On entry, `nt` = *<value>*.

            Constraint: `nt` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, `n` = *<value>* and `iblock` = *<value>*.

            Constraint: `n` >= 2 and if abs(`iblock`) >= 2, `n` must be
            a multiple of abs(`iblock`).

        (`errno` 3)
            On entry, at least one treatment is not present. Treatment
            *<value>* does not appear in `it`.

        (`errno` 3)
            On entry, i = *<value>*, `it`[i-1] = *<value>* and `nt` =
            *<value>*.

            Constraint: 1 <= `it`[i-1] <= `nt`.

        (`errno` 4)
            On entry, the values in `y` are constant.

        (`errno` 5)
            The computation of the eigenvalues has failed to converge.

        (`errno` 5)
            A computed standard error is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The treatments are totally confounded with blocks.

        (`errno` 7)
            The residual degrees of freedom is zero.

        (`errno` 7)
            The residual mean square is zero.

        (`errno` 8)
            The design is disconnected.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.random'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, iblock.
        utils._BYREF_INT, # Input, nt.
        utils._EngineIntArrayType.c_type, # Input, it[*].
        utils._BYREF_FLOAT64, # Output, gmean.
        utils._EngineFloat64ArrayType.c_type, # Output, bmean[abs(iblock)].
        utils._EngineFloat64ArrayType.c_type, # Output, tmean[nt].
        utils._EngineFloat64ArrayType.c_type, # Output, tabl[ldtabl, 5].
        utils._BYREF_INT, # Input, ldtabl.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, nt].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineIntArrayType.c_type, # Output, irep[nt].
        utils._EngineFloat64ArrayType.c_type, # Output, r[n].
        utils._EngineFloat64ArrayType.c_type, # Output, ef[nt].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, irdf.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((tabl, 'tabl'), (c, 'c')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _iblock = utils._EngineIntScalarType(
        iblock, {'fun_name': fun_name, 'entity': 'iblock'},
    ).to_ctypes()
    _nt = utils._EngineIntScalarType(
        nt, {'fun_name': fun_name, 'entity': 'nt'},
    ).to_ctypes()
    _nt_py = utils._EngineIntScalarType.to_py(_nt)
    if _nt_py >= 2:
        _it_dim1 = _n_py
    else:
        _it_dim1 = 1
    exp_shape_it = (_it_dim1,)
    _it = utils._EngineIntArrayType(
        it, {'fun_name': fun_name, 'entity': 'it'},
        exp_shape=exp_shape_it,
    ).to_ctypes()
    _gmean = utils._EngineFloat64ScalarType.empty_ctypes()
    _iblock_py = utils._EngineIntScalarType.to_py(_iblock)
    _bmean_cls = utils._EngineFloat64ArrayType(
        bmean, {'fun_name': fun_name, 'entity': 'bmean'},
        exp_shape=(abs(_iblock_py),),
        check_contents=False,
    )
    _bmean = _bmean_cls.to_ctypes(can_reference_contents=False)
    _tmean_cls = utils._EngineFloat64ArrayType(
        tmean, {'fun_name': fun_name, 'entity': 'tmean'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _tmean = _tmean_cls.to_ctypes(can_reference_contents=False)
    _ldtabl_py = utils._EngineFloat64ArrayType.get_nd(tabl, {'fun_name': fun_name, 'entity': 'tabl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldtabl = utils._EngineIntScalarType(
        _ldtabl_py, {'fun_name': fun_name, 'entity': '_ldtabl'},
    ).to_ctypes()
    _tabl_cls = utils._EngineFloat64ArrayType(
        tabl, {'fun_name': fun_name, 'entity': 'tabl'},
        exp_shape=(4, 5),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _tabl = _tabl_cls.to_ctypes(can_reference_contents=False)
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_nt_py, _nt_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _irep_cls = utils._EngineIntArrayType(
        irep, {'fun_name': fun_name, 'entity': 'irep'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _irep = _irep_cls.to_ctypes(can_reference_contents=False)
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _ef_cls = utils._EngineFloat64ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _ef = _ef_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _irdf = utils._EngineIntScalarType(
        irdf, {'fun_name': fun_name, 'entity': 'irdf'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _y,
        _iblock,
        _nt,
        _it,
        _gmean,
        _bmean,
        _tmean,
        _tabl,
        _ldtabl,
        _c,
        _ldc,
        _irep,
        _r,
        _ef,
        _tol,
        _irdf,
        _wk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irdf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irdf >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, no blocks or treatments in model.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nt >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and iblock = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= 2 and if abs(iblock) >= 2,',
                    'n must be a multiple of abs(iblock).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one treatment is not present.',
                    'Treatment {:s} does not appear in it.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, it[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= it[i-1] <= nt.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the values in y are constant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the eigenvalues has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A computed standard error is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The treatments are totally confounded with blocks.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual degrees of freedom is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual mean square is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The design is disconnected.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _bmean_cls.refresh_py(_bmean)
        _tmean_cls.refresh_py(_tmean)
        _tabl_cls.refresh_py(_tabl)
        _c_cls.refresh_py(_c)
        _irep_cls.refresh_py(_irep)
        _r_cls.refresh_py(_r)
        _ef_cls.refresh_py(_ef)
        _gmean_py = utils._EngineFloat64ScalarType.to_py(_gmean)
        return_data = _gmean_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['gmean'],
            )(
                _gmean_py,
            )
    _err_class.manage()
    return return_data

def rowcol(nrep, nrow, ncol, y, nt, it, tmean, tabl, c, irep, rpmean, rmean, cmean, r, ef, tol, irdf):
    u"""
    Analysis of variance, general row and column design, treatment means
    and standard errors.

    For full information please refer to the NAG Library document for
    g04bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04bcf.html

    Parameters
    ----------
    nrep : int
        b, the number of replicates.

    nrow : int
        r, the number of rows per replicate.

    ncol : int
        c, the number of columns per replicate.

    y : float, shape (`nrep`*`nrow`*`ncol`)
        The n = brc observations ordered by columns within rows within
        replicates. That is `y`[ rc(i-1) +r(j-1) +k -1] contains the
        observation from the kth column of the jth row of the ith
        replicate, for k = 1,2,...,c, for j = 1,2,...,r, for i =
        1,2,...,b.

    nt : int
        The number of treatments. If only replicates, rows and columns
        are required in the analysis then set `nt` = 1.

    it : int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `nt`>1: `nrep`*`nrow`*`ncol`; otherwise: 1.

        If `nt` > 1, `it`[i-1] indicates which of the `nt` treatments
        unit i received, for i = 1,2,...,n.

        If `nt` = 1, `it` is not referenced.

    tmean : float, shape (`nt`), modified in place
        On exit: if `nt` >= 2, `tmean`[l-1] contains the (adjusted) mean
        for the lth treatment, mu^*+tau^_l, for l = 1,2,...,t, where
        mu^* is the mean of the treatment adjusted observations
        y_{ijk(l)}-tau^_l. Otherwise `tmean` is not referenced.

    tabl : float, shape (6, 5), modified in place
        On exit: the analysis of variance table. Column 1 contains the
        degrees of freedom, column 2 the sum of squares, and where
        appropriate, column 3 the mean squares, column 4 the F-statistic
        and column 5 the significance level of the F-statistic. Row 1 is
        for replicates, row 2 for rows, row 3 for columns, row 4 for
        treatments (if `nt` > 1), row 5 for residual and row 6 for
        total. Mean squares are computed for all but the total row,
        F-statistics and significance are computed for treatments,
        replicates, rows and columns. Any unfilled cells are set to
        zero.

    c : float, shape (`nt`, `nt`), modified in place
        On exit: the upper triangular part of `c` contains the
        variance-covariance matrix of the treatment effects, the
        strictly lower triangular part contains the standard errors of
        the difference between two treatment effects (means), i.e.,
        `c`[i-1,j-1] contains the covariance of treatment i and j if j
        >= i and the standard error of the difference between treatment
        i and j if j < i, for j = 1,2,...,t, for i = 1,2,...,t.

    irep : int, shape (`nt`), modified in place
        On exit: if `nt` > 1, the treatment replications, R_{ll}, for l
        = 1,2,...,`nt`. Otherwise `irep` is not referenced.

    rpmean : float, shape (`nrep`), modified in place
        On exit: if `nrep` > 1, `rpmean`[i-1] contains the mean for the
        ith replicate, mu^+beta^_i, for i = 1,2,...,b. Otherwise
        `rpmean` is not referenced.

    rmean : float, shape (`nrep`*`nrow`), modified in place
        On exit: `rmean`[j-1] contains the mean for the jth row,
        mu^+rho^_i, for j = 1,2,...,r.

    cmean : float, shape (`nrep`*`ncol`), modified in place
        On exit: `cmean`[k-1] contains the mean for the kth column,
        mu^+gamma^_k, for k = 1,2,...,c.

    r : float, shape (`nrep`*`nrow`*`ncol`), modified in place
        On exit: the residuals, r_i, for i = 1,2,...,n.

    ef : float, shape (`nt`), modified in place
        On exit: if `nt` >= 2, the canonical efficiency factors.
        Otherwise `ef` is not referenced.

    tol : float
        The tolerance value used to check for zero eigenvalues of the
        matrix Omega. If `tol` = 0.0 a default value of 0.00001 is used.

    irdf : int
        An adjustment to the degrees of freedom for the residual and
        total.

        `irdf` >= 1

          The degrees of freedom for the total is set to n-`irdf` and
          the residual degrees of freedom adjusted accordingly.

        `irdf` = 0

          The total degrees of freedom for the total is set to n-1, as
          usual.

    Returns
    -------
    gmean : float
        The grand mean, mu^.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nrep` = *<value>*.

            Constraint: `nrep` >= 1.

        (`errno` 1)
            On entry, `irdf` = *<value>*.

            Constraint: `irdf` >= 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `ncol` = *<value>*.

            Constraint: `ncol` >= 2.

        (`errno` 1)
            On entry, `nt` = *<value>*.

            Constraint: `nt` >= 1.

        (`errno` 1)
            On entry, `nrow` = *<value>*.

            Constraint: `nrow` >= 2.

        (`errno` 2)
            On entry, at least one treatment is not present. Treatment
            *<value>* does not appear in `it`.

        (`errno` 2)
            On entry, i = *<value>*, `it`[i-1] = *<value>* and `nt` =
            *<value>*.

            Constraint: 1 <= `it`[i-1] <= `nt`.

        (`errno` 3)
            On entry, the values in `y` are constant.

        (`errno` 4)
            The computation of the eigenvalues has failed to converge.

        (`errno` 4)
            A computed standard error is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The treatments are totally confounded with blocks.

        (`errno` 6)
            The residual degrees of freedom is zero.

        (`errno` 6)
            The residual mean square is zero.

        (`errno` 7)
            The design is disconnected.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.rowcol'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nrep.
        utils._BYREF_INT, # Input, nrow.
        utils._BYREF_INT, # Input, ncol.
        utils._EngineFloat64ArrayType.c_type, # Input, y[nrep*nrow*ncol].
        utils._BYREF_INT, # Input, nt.
        utils._EngineIntArrayType.c_type, # Input, it[*].
        utils._BYREF_FLOAT64, # Output, gmean.
        utils._EngineFloat64ArrayType.c_type, # Output, tmean[nt].
        utils._EngineFloat64ArrayType.c_type, # Output, tabl[ldtabl, 5].
        utils._BYREF_INT, # Input, ldtabl.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, nt].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineIntArrayType.c_type, # Output, irep[nt].
        utils._EngineFloat64ArrayType.c_type, # Output, rpmean[nrep].
        utils._EngineFloat64ArrayType.c_type, # Output, rmean[nrep*nrow].
        utils._EngineFloat64ArrayType.c_type, # Output, cmean[nrep*ncol].
        utils._EngineFloat64ArrayType.c_type, # Output, r[nrep*nrow*ncol].
        utils._EngineFloat64ArrayType.c_type, # Output, ef[nt].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, irdf.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((tabl, 'tabl'), (c, 'c')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nrep = utils._EngineIntScalarType(
        nrep, {'fun_name': fun_name, 'entity': 'nrep'},
    ).to_ctypes()
    _nrow = utils._EngineIntScalarType(
        nrow, {'fun_name': fun_name, 'entity': 'nrow'},
    ).to_ctypes()
    _ncol = utils._EngineIntScalarType(
        ncol, {'fun_name': fun_name, 'entity': 'ncol'},
    ).to_ctypes()
    _nrep_py = utils._EngineIntScalarType.to_py(_nrep)
    _nrow_py = utils._EngineIntScalarType.to_py(_nrow)
    _ncol_py = utils._EngineIntScalarType.to_py(_ncol)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_nrep_py*_nrow_py*_ncol_py,),
    ).to_ctypes()
    _nt = utils._EngineIntScalarType(
        nt, {'fun_name': fun_name, 'entity': 'nt'},
    ).to_ctypes()
    _nt_py = utils._EngineIntScalarType.to_py(_nt)
    if _nt_py > 1:
        _it_dim1 = _nrep_py*_nrow_py*_ncol_py
    else:
        _it_dim1 = 1
    exp_shape_it = (_it_dim1,)
    _it = utils._EngineIntArrayType(
        it, {'fun_name': fun_name, 'entity': 'it'},
        exp_shape=exp_shape_it,
    ).to_ctypes()
    _gmean = utils._EngineFloat64ScalarType.empty_ctypes()
    _tmean_cls = utils._EngineFloat64ArrayType(
        tmean, {'fun_name': fun_name, 'entity': 'tmean'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _tmean = _tmean_cls.to_ctypes(can_reference_contents=False)
    _ldtabl_py = utils._EngineFloat64ArrayType.get_nd(tabl, {'fun_name': fun_name, 'entity': 'tabl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldtabl = utils._EngineIntScalarType(
        _ldtabl_py, {'fun_name': fun_name, 'entity': '_ldtabl'},
    ).to_ctypes()
    _tabl_cls = utils._EngineFloat64ArrayType(
        tabl, {'fun_name': fun_name, 'entity': 'tabl'},
        exp_shape=(6, 5),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _tabl = _tabl_cls.to_ctypes(can_reference_contents=False)
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_nt_py, _nt_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _irep_cls = utils._EngineIntArrayType(
        irep, {'fun_name': fun_name, 'entity': 'irep'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _irep = _irep_cls.to_ctypes(can_reference_contents=False)
    _rpmean_cls = utils._EngineFloat64ArrayType(
        rpmean, {'fun_name': fun_name, 'entity': 'rpmean'},
        exp_shape=(_nrep_py,),
        check_contents=False,
    )
    _rpmean = _rpmean_cls.to_ctypes(can_reference_contents=False)
    _rmean_cls = utils._EngineFloat64ArrayType(
        rmean, {'fun_name': fun_name, 'entity': 'rmean'},
        exp_shape=(_nrep_py*_nrow_py,),
        check_contents=False,
    )
    _rmean = _rmean_cls.to_ctypes(can_reference_contents=False)
    _cmean_cls = utils._EngineFloat64ArrayType(
        cmean, {'fun_name': fun_name, 'entity': 'cmean'},
        exp_shape=(_nrep_py*_ncol_py,),
        check_contents=False,
    )
    _cmean = _cmean_cls.to_ctypes(can_reference_contents=False)
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_nrep_py*_nrow_py*_ncol_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _ef_cls = utils._EngineFloat64ArrayType(
        ef, {'fun_name': fun_name, 'entity': 'ef'},
        exp_shape=(_nt_py,),
        check_contents=False,
    )
    _ef = _ef_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _irdf = utils._EngineIntScalarType(
        irdf, {'fun_name': fun_name, 'entity': 'irdf'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nrep,
        _nrow,
        _ncol,
        _y,
        _nt,
        _it,
        _gmean,
        _tmean,
        _tabl,
        _ldtabl,
        _c,
        _ldc,
        _irep,
        _rpmean,
        _rmean,
        _cmean,
        _r,
        _ef,
        _tol,
        _irdf,
        _wk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrep >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irdf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irdf >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncol >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nt >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrow = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrow >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one treatment is not present.',
                    'Treatment {:s} does not appear in it.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, it[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= it[i-1] <= nt.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the values in y are constant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of the eigenvalues has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A computed standard error is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The treatments are totally confounded with blocks.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual degrees of freedom is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual mean square is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The design is disconnected.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _tmean_cls.refresh_py(_tmean)
        _tabl_cls.refresh_py(_tabl)
        _c_cls.refresh_py(_c)
        _irep_cls.refresh_py(_irep)
        _rpmean_cls.refresh_py(_rpmean)
        _rmean_cls.refresh_py(_rmean)
        _cmean_cls.refresh_py(_cmean)
        _r_cls.refresh_py(_r)
        _ef_cls.refresh_py(_ef)
        _gmean_py = utils._EngineFloat64ScalarType.to_py(_gmean)
        return_data = _gmean_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['gmean'],
            )(
                _gmean_py,
            )
    _err_class.manage()
    return return_data

def factorial(n, y, nfac, lfac, nblock, inter, irdf, mterm, table, tmean, maxt, e, imean, semean, bmean, r):
    u"""
    Analysis of variance, complete factorial design, treatment means and
    standard errors.

    For full information please refer to the NAG Library document for
    g04ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04caf.html

    Parameters
    ----------
    n : int
        The number of observations.

    y : float, shape (`n`)
        The observations in standard order, see [Description].

    nfac : int
        k, the number of factors.

    lfac : int, shape (`nfac`)
        `lfac`[i] must contain the number of levels for the ith factor,
        for i = 1,2,...,k.

    nblock : int
        The number of blocks. If there are no blocks, set `nblock` = 0
        or 1.

    inter : int
        The maximum number of factors in an interaction term. If no
        interaction terms are to be computed, set `inter` = 0 or 1.

    irdf : int
        The adjustment to the residual and total degrees of freedom. The
        total degrees of freedom are set to `n`-`irdf` and the residual
        degrees of freedom adjusted accordingly. For examples of the use
        of `irdf` see [Further Comments].

    mterm : int
        The maximum number of terms in the analysis of variance table,
        see [Further Comments].

    table : float, shape (`mterm`, 5), modified in place
        On exit: the first `itotal` rows of `table` contain the analysis
        of variance table. The first column contains the degrees of
        freedom, the second column contains the sum of squares, the
        third column (except for the row corresponding to the total sum
        of squares) contains the mean squares, i.e., the sums of squares
        divided by the degrees of freedom, and the fourth and fifth
        columns contain the F ratio and significance level, respectively
        (except for rows corresponding to the total sum of squares, and
        the residual sum of squares). All other cells of the table are
        set to zero.

        The first row corresponds to the blocks and is set to zero if
        there are no blocks.

        The `itotal`th row corresponds to the total sum of squares for
        `y` and the (`itotal`-1)th row corresponds to the residual sum
        of squares.

        The central rows of the table correspond to the main effects
        followed by the interaction if specified by `inter`.

        The main effects are in the order specified by `lfac` and the
        interactions are in lexical order, see [Description].

    tmean : float, shape (`maxt`), modified in place
        On exit: the treatment means. The position of the means for an
        effect is given by the index in `imean`. For a given effect the
        means are in standard order, see [Description].

    maxt : int
        The maximum number of treatment means to be computed, see
        [Further Comments]. If the value of `maxt` is too small for the
        required analysis then the minimum number is returned in
        `imean`[0].

    e : float, shape (`maxt`), modified in place
        On exit: the estimated effects in the same order as for the
        means in `tmean`.

    imean : int, shape (`mterm`), modified in place
        On exit: indicates the position of the effect means in `tmean`.
        The effect means corresponding to the first treatment effect in
        the ANOVA table are stored in `tmean`[0] up to
        `tmean`[`imean`[0]-1]. Other effect means corresponding to the
        ith treatment effect, i = 1,2,...,`itotal`-3, are stored in
        `tmean`[`imean`[i-1]] up to `tmean`[`imean`[i]-1].

    semean : float, shape (`mterm`), modified in place
        On exit: the standard error of the difference between means
        corresponding to the ith treatment effect in the ANOVA table.

    bmean : float, shape (`nblock`+1), modified in place
        On exit: `bmean`[0] contains the grand mean, if `nblock` > 1,
        `bmean`[1] up to `bmean`[`nblock`] contain the block means.

    r : float, shape (`n`), modified in place
        On exit: the residuals.

    Returns
    -------
    itotal : int
        The row in `table` corresponding to the total sum of squares.
        The number of treatment effects is `itotal`-3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irdf` = *<value>*.

            Constraint: `irdf` >= 0.

        (`errno` 1)
            On entry, `inter` = *<value>* and `nfac` = *<value>*.

            Constraint: `inter` <= `nfac`.

        (`errno` 1)
            On entry, `inter` = *<value>*.

            Constraint: `inter` >= 0.

        (`errno` 1)
            On entry, `nblock` = *<value>*.

            Constraint: `nblock` >= 0.

        (`errno` 1)
            On entry, `nfac` = *<value>*.

            Constraint: `nfac` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 4.

        (`errno` 2)
            On entry, i = *<value>* and `lfac`[i-1] = *<value>*

            Constraint: `lfac`[i-1] >= 2.

        (`errno` 2)
            On entry, the number of plots per block is incompatible with
            the number of plot factors.

        (`errno` 2)
            On entry, `n` = *<value>* and `nblock` = *<value>*.

            Constraint: when `nblock` > 1, `n` must be a multiple of
            `nblock`.

        (`errno` 3)
            On entry, the values of `y` are constant.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `mterm` = *<value>*

            Constraint: `mterm` >= *<value>*.

        (`errno` 2)
            On entry, `maxt` = *<value>*

            Constraint: `maxt` >= *<value>*.

        (`errno` 4)
            The residual sum of squares is zero.

        (`errno` 4)
            There are no degrees of freedom for the residual sum of
            squares.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.factorial'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, nfac.
        utils._EngineIntArrayType.c_type, # Input, lfac[nfac].
        utils._BYREF_INT, # Input, nblock.
        utils._BYREF_INT, # Input, inter.
        utils._BYREF_INT, # Input, irdf.
        utils._BYREF_INT, # Input, mterm.
        utils._EngineFloat64ArrayType.c_type, # Output, table[mterm, 5].
        utils._BYREF_INT, # Output, itotal.
        utils._EngineFloat64ArrayType.c_type, # Output, tmean[maxt].
        utils._BYREF_INT, # Input, maxt.
        utils._EngineFloat64ArrayType.c_type, # Output, e[maxt].
        utils._EngineIntArrayType.c_type, # Output, imean[mterm].
        utils._EngineFloat64ArrayType.c_type, # Output, semean[mterm].
        utils._EngineFloat64ArrayType.c_type, # Output, bmean[nblock+1].
        utils._EngineFloat64ArrayType.c_type, # Output, r[n].
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((table, 'table'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _nfac = utils._EngineIntScalarType(
        nfac, {'fun_name': fun_name, 'entity': 'nfac'},
    ).to_ctypes()
    _nfac_py = utils._EngineIntScalarType.to_py(_nfac)
    _lfac = utils._EngineIntArrayType(
        lfac, {'fun_name': fun_name, 'entity': 'lfac'},
        exp_shape=(_nfac_py,),
    ).to_ctypes()
    _nblock = utils._EngineIntScalarType(
        nblock, {'fun_name': fun_name, 'entity': 'nblock'},
    ).to_ctypes()
    _inter = utils._EngineIntScalarType(
        inter, {'fun_name': fun_name, 'entity': 'inter'},
    ).to_ctypes()
    _irdf = utils._EngineIntScalarType(
        irdf, {'fun_name': fun_name, 'entity': 'irdf'},
    ).to_ctypes()
    _mterm = utils._EngineIntScalarType(
        mterm, {'fun_name': fun_name, 'entity': 'mterm'},
    ).to_ctypes()
    _mterm_py = utils._EngineIntScalarType.to_py(_mterm)
    _table_cls = utils._EngineFloat64ArrayType(
        table, {'fun_name': fun_name, 'entity': 'table'},
        exp_shape=(_mterm_py, 5),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _table = _table_cls.to_ctypes(can_reference_contents=False)
    _itotal = utils._EngineIntScalarType.empty_ctypes()
    _maxt = utils._EngineIntScalarType(
        maxt, {'fun_name': fun_name, 'entity': 'maxt'},
    ).to_ctypes()
    _maxt_py = utils._EngineIntScalarType.to_py(_maxt)
    _tmean_cls = utils._EngineFloat64ArrayType(
        tmean, {'fun_name': fun_name, 'entity': 'tmean'},
        exp_shape=(_maxt_py,),
        check_contents=False,
    )
    _tmean = _tmean_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_maxt_py,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _imean_cls = utils._EngineIntArrayType(
        imean, {'fun_name': fun_name, 'entity': 'imean'},
        exp_shape=(_mterm_py,),
        check_contents=False,
    )
    _imean = _imean_cls.to_ctypes(can_reference_contents=False)
    _semean_cls = utils._EngineFloat64ArrayType(
        semean, {'fun_name': fun_name, 'entity': 'semean'},
        exp_shape=(_mterm_py,),
        check_contents=False,
    )
    _semean = _semean_cls.to_ctypes(can_reference_contents=False)
    _nblock_py = utils._EngineIntScalarType.to_py(_nblock)
    _bmean_cls = utils._EngineFloat64ArrayType(
        bmean, {'fun_name': fun_name, 'entity': 'bmean'},
        exp_shape=(_nblock_py+1,),
        check_contents=False,
    )
    _bmean = _bmean_cls.to_ctypes(can_reference_contents=False)
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _y,
        _nfac,
        _lfac,
        _nblock,
        _inter,
        _irdf,
        _mterm,
        _table,
        _itotal,
        _tmean,
        _maxt,
        _e,
        _imean,
        _semean,
        _bmean,
        _r,
        _iwk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irdf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irdf >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inter = {:s} and nfac = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: inter <= nfac.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inter = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inter >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nblock = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nblock >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfac = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nfac >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and lfac[i-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lfac[i-1] >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of plots per block is incompatible',
                    'with the number of plot factors.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nblock = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: when nblock > 1, n must be a multiple of nblock.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mterm = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mterm >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxt = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxt >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the values of y are constant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual sum of squares is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There are no degrees of freedom for the residual sum of squares.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _table_cls.refresh_py(_table)
        _tmean_cls.refresh_py(_tmean)
        _e_cls.refresh_py(_e)
        _imean_cls.refresh_py(_imean)
        _semean_cls.refresh_py(_semean)
        _bmean_cls.refresh_py(_bmean)
        _r_cls.refresh_py(_r)
        _itotal_py = utils._EngineIntScalarType.to_py(_itotal)
        return_data = _itotal_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['itotal'],
            )(
                _itotal_py,
            )
    _err_class.manage()
    return return_data

def contrasts(nt, tmean, irep, rms, rdf, nc, ct, est, tabl, tol, usetx, tx):
    u"""
    Computes sum of squares for contrast between means.

    For full information please refer to the NAG Library document for
    g04da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04daf.html

    Parameters
    ----------
    nt : int
        t, the number of treatment means.

    tmean : float, shape (`nt`)
        The treatment means, tau^_i, for i = 1,2,...,t.

    irep : int, shape (`nt`)
        The replication for each treatment mean, n_i, for i = 1,2,...,t.

    rms : float
        The residual mean square, sigma^^2.

    rdf : float
        The residual degrees of freedom.

    nc : int
        The number of contrasts.

    ct : float, shape (`nt`, `nc`)
        The columns of `ct` must contain the `nc` contrasts, that is
        `ct`[i-1,j-1] must contain lambda_{ij}, for j = 1,2,...,`nc`,
        for i = 1,2,...,t.

    est : float, shape (`nc`), modified in place
        On exit: the estimates of the contrast, Lambda_j, for j =
        1,2,...,`nc`.

    tabl : float, shape (`nc`, 5), modified in place
        On entry: the elements of `tabl` that are not referenced as
        described below remain unchanged.

        On exit: the rows of the analysis of variance table for the
        contrasts. For each row column 1 contains the degrees of
        freedom, column 2 contains the sum of squares, column 3 contains
        the mean square, column 4 the F-statistic and column 5 the
        significance level for the contrast. Note that the degrees of
        freedom are always one and so the mean square equals the sum of
        squares.

    tol : float
        The tolerance, epsilon used to check if the contrasts are
        orthogonal and if they are orthogonal to the mean. If `tol` <=
        0.0 the value machine precision is used.

    usetx : bool
        If `usetx` = True the means tau_i^* are provided in `tx` and the
        formula (2) is used instead of formula (1).

        If `usetx` = False formula (1) is used and `tx` is not
        referenced.

    tx : float, shape (`nt`)
        If `usetx` = True `tx` must contain the means tau_i^*, for i =
        1,2,...,t.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rdf` = *<value>*.

            Constraint: `rdf` >= 1.0.

        (`errno` 1)
            On entry, `rms` = *<value>*.

            Constraint: `rms` > 0.0.

        (`errno` 1)
            On entry, `nt` = *<value>*.

            Constraint: `nt` >= 2.

        (`errno` 1)
            On entry, `nc` = *<value>*.

            Constraint: `nc` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The *<value>**<value>* and *<value>**<value>* contrasts are
            not orthogonal.

        (`errno` 2)
            The *<value>**<value>* contrast is not orthogonal to the
            mean.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.contrasts'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nt.
        utils._EngineFloat64ArrayType.c_type, # Input, tmean[nt].
        utils._EngineIntArrayType.c_type, # Input, irep[nt].
        utils._BYREF_FLOAT64, # Input, rms.
        utils._BYREF_FLOAT64, # Input, rdf.
        utils._BYREF_INT, # Input, nc.
        utils._EngineFloat64ArrayType.c_type, # Input, ct[ldct, nc].
        utils._BYREF_INT, # Input, ldct.
        utils._EngineFloat64ArrayType.c_type, # Output, est[nc].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, tabl[ldtabl, 5].
        utils._BYREF_INT, # Input, ldtabl.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_LOGICAL, # Input, usetx.
        utils._EngineFloat64ArrayType.c_type, # Input, tx[nt].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ct, 'ct'), (tabl, 'tabl')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nt = utils._EngineIntScalarType(
        nt, {'fun_name': fun_name, 'entity': 'nt'},
    ).to_ctypes()
    _nt_py = utils._EngineIntScalarType.to_py(_nt)
    _tmean = utils._EngineFloat64ArrayType(
        tmean, {'fun_name': fun_name, 'entity': 'tmean'},
        exp_shape=(_nt_py,),
    ).to_ctypes()
    _irep = utils._EngineIntArrayType(
        irep, {'fun_name': fun_name, 'entity': 'irep'},
        exp_shape=(_nt_py,),
    ).to_ctypes()
    _rms = utils._EngineFloat64ScalarType(
        rms, {'fun_name': fun_name, 'entity': 'rms'},
    ).to_ctypes()
    _rdf = utils._EngineFloat64ScalarType(
        rdf, {'fun_name': fun_name, 'entity': 'rdf'},
    ).to_ctypes()
    _nc = utils._EngineIntScalarType(
        nc, {'fun_name': fun_name, 'entity': 'nc'},
    ).to_ctypes()
    _ldct_py = utils._EngineFloat64ArrayType.get_nd(ct, {'fun_name': fun_name, 'entity': 'ct'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldct = utils._EngineIntScalarType(
        _ldct_py, {'fun_name': fun_name, 'entity': '_ldct'},
    ).to_ctypes()
    _nc_py = utils._EngineIntScalarType.to_py(_nc)
    _ct = utils._EngineFloat64ArrayType(
        ct, {'fun_name': fun_name, 'entity': 'ct'},
        exp_shape=(_nt_py, _nc_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _est_cls = utils._EngineFloat64ArrayType(
        est, {'fun_name': fun_name, 'entity': 'est'},
        exp_shape=(_nc_py,),
        check_contents=False,
    )
    _est = _est_cls.to_ctypes(can_reference_contents=False)
    _ldtabl_py = utils._EngineFloat64ArrayType.get_nd(tabl, {'fun_name': fun_name, 'entity': 'tabl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldtabl = utils._EngineIntScalarType(
        _ldtabl_py, {'fun_name': fun_name, 'entity': '_ldtabl'},
    ).to_ctypes()
    _tabl_cls = utils._EngineFloat64ArrayType(
        tabl, {'fun_name': fun_name, 'entity': 'tabl'},
        exp_shape=(_nc_py, 5),
        mutual_sorder=_m_sorder,
    )
    _tabl = _tabl_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _usetx = utils._EngineLogicalScalarType(
        usetx, {'fun_name': fun_name, 'entity': 'usetx'},
    ).to_ctypes()
    _tx = utils._EngineFloat64ArrayType(
        tx, {'fun_name': fun_name, 'entity': 'tx'},
        exp_shape=(_nt_py,),
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nt,
        _tmean,
        _irep,
        _rms,
        _rdf,
        _nc,
        _ct,
        _ldct,
        _est,
        _tabl,
        _ldtabl,
        _tol,
        _usetx,
        _tx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rdf = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rdf >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rms = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rms > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nt >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nc >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}{:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    '{:s}{:s} contrasts are not orthogonal.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}{:s} contrast is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'orthogonal to the mean.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _est_cls.refresh_py(_est)
        _tabl_cls.refresh_py(_tabl)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def confidence(typ, nt, tmean, rdf, c, clevel, cil, ciu, isig):
    u"""
    Computes confidence intervals for differences between means computed
    by ``random`` or ``rowcol``.

    For full information please refer to the NAG Library document for
    g04db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04dbf.html

    Parameters
    ----------
    typ : str, length 1
        Indicates which method is to be used.

        `typ` = 'T'

          The Tukey--Kramer method is used.

        `typ` = 'B'

          The Bonferroni method is used.

        `typ` = 'D'

          The Dunn--Sidak method is used.

        `typ` = 'L'

          The Fisher LSD method is used.

        `typ` = 'S'

          The Scheffe's method is used.

    nt : int
        t, the number of treatment means.

    tmean : float, shape (`nt`)
        The treatment means, tau^_i, for i = 1,2,...,t.

    rdf : float
        nu, the residual degrees of freedom.

    c : float, shape (`nt`, `nt`)
        The strictly lower triangular part of `c` must contain the
        standard errors of the differences between the means as returned
        by ``random`` and ``rowcol``. That is `c`[i,j], i > j, contains
        the standard error of the difference between the ith and jth
        mean in `tmean`.

    clevel : float
        The required confidence level for the computed intervals,
        (1-alpha).

    cil : float, shape (`nt`*(`nt`-1)/2), modified in place
        On exit: the ((i-1)(i-2)/2+j)th element contains the lower limit
        to the confidence interval for the difference between ith and
        jth means in `tmean`, for j = 1,2,...,i-1, for i = 2,3,...,t.

    ciu : float, shape (`nt`*(`nt`-1)/2), modified in place
        On exit: the ((i-1)(i-2)/2+j)th element contains the upper limit
        to the confidence interval for the difference between ith and
        jth means in `tmean`, for j = 1,2,...,i-1, for i = 2,3,...,t.

    isig : int, shape (`nt`*(`nt`-1)/2), modified in place
        On exit: the ((i-1)(i-2)/2+j)th element indicates if the
        difference between ith and jth means in `tmean` is significant,
        for j = 1,2,...,i-1, for i = 2,3,...,t. If the difference is
        significant then the returned value is 1; otherwise the returned
        value is 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `typ` = *<value>*.

            Constraint: `typ` = 'T', 'B', 'D', 'L' or 'S'.

        (`errno` 1)
            On entry, `clevel` = *<value>*.

            Constraint: 0.0 < `clevel` < 1.0.

        (`errno` 1)
            On entry, `rdf` = *<value>*.

            Constraint: `rdf` >= 1.0.

        (`errno` 1)
            On entry, `nt` = *<value>*.

            Constraint: `nt` >= 2.

        (`errno` 2)
            On entry, i = *<value>* and j = *<value>*.

            Constraint: `c`[i-1,j-1] > 0.0.

        (`errno` 3)
            There has been a failure in the computation of the
            studentized range statistic.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.confidence'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04dbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, typ(len=1).
        utils._BYREF_INT, # Input, nt.
        utils._EngineFloat64ArrayType.c_type, # Input, tmean[nt].
        utils._BYREF_FLOAT64, # Input, rdf.
        utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, nt].
        utils._BYREF_INT, # Input, ldc.
        utils._BYREF_FLOAT64, # Input, clevel.
        utils._EngineFloat64ArrayType.c_type, # Output, cil[nt*(nt-1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, ciu[nt*(nt-1)//2].
        utils._EngineIntArrayType.c_type, # Output, isig[nt*(nt-1)//2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of typ.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _typ = utils._EngineCharScalarType(
        typ, {'fun_name': fun_name, 'entity': 'typ'}, exp_chrlen=1,
    ).to_ctypes()
    _nt = utils._EngineIntScalarType(
        nt, {'fun_name': fun_name, 'entity': 'nt'},
    ).to_ctypes()
    _nt_py = utils._EngineIntScalarType.to_py(_nt)
    _tmean = utils._EngineFloat64ArrayType(
        tmean, {'fun_name': fun_name, 'entity': 'tmean'},
        exp_shape=(_nt_py,),
    ).to_ctypes()
    _rdf = utils._EngineFloat64ScalarType(
        rdf, {'fun_name': fun_name, 'entity': 'rdf'},
    ).to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_nt_py, _nt_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _clevel = utils._EngineFloat64ScalarType(
        clevel, {'fun_name': fun_name, 'entity': 'clevel'},
    ).to_ctypes()
    _cil_cls = utils._EngineFloat64ArrayType(
        cil, {'fun_name': fun_name, 'entity': 'cil'},
        exp_shape=(_nt_py*(_nt_py-1)//2,),
        check_contents=False,
    )
    _cil = _cil_cls.to_ctypes(can_reference_contents=False)
    _ciu_cls = utils._EngineFloat64ArrayType(
        ciu, {'fun_name': fun_name, 'entity': 'ciu'},
        exp_shape=(_nt_py*(_nt_py-1)//2,),
        check_contents=False,
    )
    _ciu = _ciu_cls.to_ctypes(can_reference_contents=False)
    _isig_cls = utils._EngineIntArrayType(
        isig, {'fun_name': fun_name, 'entity': 'isig'},
        exp_shape=(_nt_py*(_nt_py-1)//2,),
        check_contents=False,
    )
    _isig = _isig_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _typ,
        _nt,
        _tmean,
        _rdf,
        _c,
        _ldc,
        _clevel,
        _cil,
        _ciu,
        _isig,
        _errbuf,
        _ierr,
        _typ._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, typ = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: typ = \'T\', \'B\', \'D\', \'L\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, clevel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < clevel < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rdf = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rdf >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nt >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: c[i-1,j-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There has been a failure in the computation of',
                    'the studentized range statistic.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _cil_cls.refresh_py(_cil)
        _ciu_cls.refresh_py(_ciu)
        _isig_cls.refresh_py(_isig)
        return_data = None
    _err_class.manage()
    return return_data

def dummyvars(typ, n, levels, ifact, x, v, rep):
    u"""
    Computes orthogonal polynomials or dummy variables for
    factor/classification variable.

    For full information please refer to the NAG Library document for
    g04ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04eaf.html

    Parameters
    ----------
    typ : str, length 1
        The type of dummy variable to be computed.

        - If `typ` = 'P', an orthogonal Polynomial representation is
          computed.

        - If `typ` = 'H', a Helmert matrix representation is computed.

        - If `typ` = 'F', the contrasts relative to the First level are
          computed.

        - If `typ` = 'L', the contrasts relative to the Last level are
          computed.

        - If `typ` = 'C', a Complete set of dummy variables is computed.

    n : int
        n, the number of observations for which the dummy variables are
        to be computed.

    levels : int
        k, the number of levels of the factor.

    ifact : int, shape (`n`)
        The n values of the factor.

    x : float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `typ` in ('P', 'H', 'F', 'L'):
        `levels`-1; if `typ`='C': `levels`; otherwise: 0.

        On exit: the n by k* matrix of dummy variables, where k* = k-1
        if `typ` = 'P', 'H', 'F' or 'L' and k* = k if `typ` = 'C'.

    v : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `typ`='P': `levels`; otherwise: 1.

        If `typ` = 'P', the k distinct values of the underlying variable
        for which the orthogonal polynomial is to be computed.

        If `typ` != 'P', `v` is not referenced.

    rep : float, shape (`levels`), modified in place
        On exit: the number of replications for each level of the
        factor, r_i, for i = 1,2,...,k.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `typ` = *<value>*.

            Constraint: `typ` = 'P', 'H', 'F', 'L' or 'C'.

        (`errno` 1)
            On entry, `n` = *<value>* and `levels` = *<value>*.

            Constraint: `n` >= `levels`.

        (`errno` 1)
            On entry, `levels` = *<value>*.

            Constraint: `levels` >= 2.

        (`errno` 2)
            On entry, not all values of `v` are distinct.

        (`errno` 2)
            On entry, not all levels are present in `ifact`.

        (`errno` 2)
            On entry, i = *<value>*, `ifact`[i-1] = *<value>* and
            `levels` = *<value>*.

            Constraint: 1 <= `levels`[i-1] <= `levels`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The *<value>**<value>* polynomial has all elements zero.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.dummyvars'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, typ(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, levels.
        utils._EngineIntArrayType.c_type, # Input, ifact[n].
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input, v[*].
        utils._EngineFloat64ArrayType.c_type, # Output, rep[levels].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of typ.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _typ = utils._EngineCharScalarType(
        typ, {'fun_name': fun_name, 'entity': 'typ'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _levels = utils._EngineIntScalarType(
        levels, {'fun_name': fun_name, 'entity': 'levels'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ifact = utils._EngineIntArrayType(
        ifact, {'fun_name': fun_name, 'entity': 'ifact'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _typ_py = utils._EngineCharScalarType.to_py(_typ)
    _levels_py = utils._EngineIntScalarType.to_py(_levels)
    if _typ_py in ('P', 'p', 'H', 'h', 'F', 'f', 'L', 'l'):
        _x_dim2 = _levels_py-1
    elif _typ_py in ('C', 'c'):
        _x_dim2 = _levels_py
    else:
        _x_dim2 = 0
    exp_shape_x = (_n_py, _x_dim2)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    if _typ_py in ('P', 'p'):
        _v_dim1 = _levels_py
    else:
        _v_dim1 = 1
    exp_shape_v = (_v_dim1,)
    _v = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
    ).to_ctypes()
    _rep_cls = utils._EngineFloat64ArrayType(
        rep, {'fun_name': fun_name, 'entity': 'rep'},
        exp_shape=(_levels_py,),
        check_contents=False,
    )
    _rep = _rep_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _typ,
        _n,
        _levels,
        _ifact,
        _x,
        _ldx,
        _v,
        _rep,
        _errbuf,
        _ierr,
        _typ._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, typ = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: typ = \'P\', \'H\', \'F\', \'L\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and levels = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= levels.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, levels = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: levels >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, not all values of v are distinct.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, not all levels are present in ifact.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, ifact[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'levels = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= levels[i-1] <= levels.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The {:s}{:s} polynomial has all elements'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _rep_cls.refresh_py(_rep)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def icc(mtype, rtype, nrep, nsubj, nrater, score, smiss, alpha):
    u"""
    Intraclass correlation (ICC) for assessing rater reliability.

    For full information please refer to the NAG Library document for
    g04ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g04/g04gaf.html

    Parameters
    ----------
    mtype : int
        Indicates which model is to be used.

        `mtype` = 1

          The reliability study is a one-factor design, ICC(1,1).

        `mtype` = 2

          The reliability study is a random factorial design, ICC(2,1).

        `mtype` = 3

          The reliability study is a mixed factorial design, ICC(3,1).

    rtype : int
        Indicates which type of reliability is required.

        `rtype` = 1

          Interrater reliability is required.

        `rtype` = 2

          Intrarater reliability is required.

    nrep : int
        m, the number of replicates.

    nsubj : int
        n_s, the number of subjects.

    nrater : int
        n_r, the number of raters.

    score : float, shape (`nrep`, `nsubj`, `nrater`)
        The matrix of scores, with `score`[k-1,i-1,j-1] being the score
        given to the ith subject by the jth rater in the kth replicate.

        If rater j did not rate subject i at replication k, the
        corresponding element of `score`, `score`[k-1,i-1,j-1], should
        be set to `smiss`.

    smiss : None or float
        The value used to indicate a missing score.

        Care should be taken in the selection of the value used to
        indicate a missing score. ``icc`` will treat any score in the
        inclusive range (1+/-0.1^(machine.decimal_digits-2))*`smiss` as
        missing.

        Alternatively, a NaN (Not A Number) can be used to indicate
        missing values, in which case the value of `smiss` and any
        missing values of `score` can be set through a call to
        ``ieee.create_nan``.

    alpha : float
        alpha, the significance level used in the construction of the
        confidence intervals for `icc`.

    Returns
    -------
    icc : float
        An estimate of the intraclass correlation to measure either the
        interrater reliability, rho, or intrarater reliability, gamma,
        as specified by `mtype` and `rtype`.

    lci : float
        An approximate lower limit for the 100*(1-alpha)% confidence
        interval for the ICC.

    uci : float
        An approximate upper limit for the 100*(1-alpha%) confidence
        interval for the ICC.

    fstat : float
        f, the F-statistic associated with `icc`.

    df1 : float
        nu_1 and nu_2, the degrees of freedom associated with f.

    df2 : float
        nu_1 and nu_2, the degrees of freedom associated with f.

    pvalue : float
        P(F >= f:nu_1,nu_1), the upper tail probability from an F
        distribution.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `mtype` = *<value>*.

            Constraint: `mtype` = 1, 2 or 3.

        (`errno` 21)
            On entry, `rtype` = *<value>*.

            Constraint: `rtype` = 1 or 2.

        (`errno` 31)
            On entry, `nrep` = *<value>*.

            Constraint: `nrep` >= 1.

        (`errno` 32)
            On entry, `nrep` = *<value>*.

            Constraint: when `mtype` = 2 or 3 and `rtype` = 2, `nrep` >=
            2.

        (`errno` 33)
            On entry, after adjusting for missing data, `nrep` =
            *<value>*.

            Constraint: `nrep` >= 1.

        (`errno` 34)
            On entry, after adjusting for missing data, `nrep` =
            *<value>*.

            Constraint: when `mtype` = 2 or 3 and `rtype` = 2, `nrep` >=
            2.

        (`errno` 41)
            On entry, `nsubj` = *<value>*.

            Constraint: `nsubj` >= 2.

        (`errno` 42)
            On entry, after adjusting for missing data, `nsubj` =
            *<value>*.

            Constraint: `nsubj` >= 2.

        (`errno` 51)
            On entry, `nrater` = *<value>*.

            Constraint: `nrater` >= 2.

        (`errno` 52)
            On entry, after adjusting for missing data, `nrater` =
            *<value>*.

            Constraint: `nrater` >= 2.

        (`errno` 61)
            Unable to calculate the ICC due to a division by zero.

            This is often due to degenerate data, for example all scores
            being the same.

        (`errno` 71)
            On entry, mscore = *<value>*.

            Constraint: mscore = 1 or 2.

        (`errno` 91)
            On entry, `alpha` = *<value>*.

            Constraint: 0 < `alpha` < 1.

        (`errno` 92)
            On entry, `alpha` = *<value>*.

            `alpha` is too close to either zero or one.

            This error is unlikely to occur.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 62)
            On entry, a replicate, subject or rater contained all
            missing data.

            All output quantities have been calculated using the reduced
            problem size.

        (`errno` 101)
            `icc` does not fall into the interval [`lci`,`uci`].

            All output quantities have been calculated.

        (`errno` 102)
            The estimate of at least one variance component was
            negative.

            Negative estimates were set to zero and all output
            quantities calculated as documented.

    See Also
    --------
    :meth:`naginterfaces.library.examples.anova.icc_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.anova.icc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g04gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mtype.
        utils._BYREF_INT, # Input, rtype.
        utils._BYREF_INT, # Input, nrep.
        utils._BYREF_INT, # Input, nsubj.
        utils._BYREF_INT, # Input, nrater.
        utils._EngineFloat64ArrayType.c_type, # Input, score[nrep, nsubj, nrater].
        utils._BYREF_INT, # Input, mscore.
        utils._BYREF_FLOAT64, # Input, smiss.
        utils._BYREF_FLOAT64, # Input, alpha.
        utils._BYREF_FLOAT64, # Output, icc.
        utils._BYREF_FLOAT64, # Output, lci.
        utils._BYREF_FLOAT64, # Output, uci.
        utils._BYREF_FLOAT64, # Output, fstat.
        utils._BYREF_FLOAT64, # Output, df1.
        utils._BYREF_FLOAT64, # Output, df2.
        utils._BYREF_FLOAT64, # Output, pvalue.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((score, 'score'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _mtype = utils._EngineIntScalarType(
        mtype, {'fun_name': fun_name, 'entity': 'mtype'},
    ).to_ctypes()
    _rtype = utils._EngineIntScalarType(
        rtype, {'fun_name': fun_name, 'entity': 'rtype'},
    ).to_ctypes()
    _nrep = utils._EngineIntScalarType(
        nrep, {'fun_name': fun_name, 'entity': 'nrep'},
    ).to_ctypes()
    _nsubj = utils._EngineIntScalarType(
        nsubj, {'fun_name': fun_name, 'entity': 'nsubj'},
    ).to_ctypes()
    _nrater = utils._EngineIntScalarType(
        nrater, {'fun_name': fun_name, 'entity': 'nrater'},
    ).to_ctypes()
    _nrep_py = utils._EngineIntScalarType.to_py(_nrep)
    _nsubj_py = utils._EngineIntScalarType.to_py(_nsubj)
    _nrater_py = utils._EngineIntScalarType.to_py(_nrater)
    _score = utils._EngineFloat64ArrayType(
        score, {'fun_name': fun_name, 'entity': 'score'},
        exp_shape=(_nrep_py, _nsubj_py, _nrater_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    if smiss is not None:
        _mscore = utils._EngineIntScalarType.c_type(2)
    else:
        _mscore = utils._EngineIntScalarType.c_type(1)
    if smiss is not None:
        if smiss is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'smiss'},
                'must not be None when smiss is not None',
            )
    if smiss is None:
        _smiss = utils._EngineFloat64ScalarType.c_type(0.0)
    else:
        _smiss = utils._EngineFloat64ScalarType(
            smiss, {'fun_name': fun_name, 'entity': 'smiss'},
            allow_none=True,
        ).to_ctypes()
    _alpha = utils._EngineFloat64ScalarType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
    ).to_ctypes()
    _icc = utils._EngineFloat64ScalarType.empty_ctypes()
    _lci = utils._EngineFloat64ScalarType.empty_ctypes()
    _uci = utils._EngineFloat64ScalarType.empty_ctypes()
    _fstat = utils._EngineFloat64ScalarType.empty_ctypes()
    _df1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _df2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _pvalue = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mtype,
        _rtype,
        _nrep,
        _nsubj,
        _nrater,
        _score,
        _mscore,
        _smiss,
        _alpha,
        _icc,
        _lci,
        _uci,
        _fstat,
        _df1,
        _df2,
        _pvalue,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mtype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mtype = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtype = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrep >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 32 and _ifmt == 32:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when mtype = 2 or 3 and rtype = 2, nrep >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 33 and _ifmt == 33:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, after adjusting for missing data, nrep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrep >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 34 and _ifmt == 34:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, after adjusting for missing data, nrep = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when mtype = 2 or 3 and rtype = 2, nrep >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 41 and _ifmt == 41:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nsubj = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nsubj >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 42 and _ifmt == 42:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, after adjusting for missing data, nsubj = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nsubj >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 51 and _ifmt == 51:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrater = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrater >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 52 and _ifmt == 52:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, after adjusting for missing data, nrater = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrater >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 61 and _ifmt == 61:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unable to calculate the ICC due to a division by zero.',
                    'This is often due to degenerate data, for example all scores being the same.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 62 and _ifmt == 62:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a replicate, subject or rater contained all missing data.',
                    'All output quantities have been calculated using the reduced problem size.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 71 and _ifmt == 71:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mscore = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mscore = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 91 and _ifmt == 91:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alpha = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 < alpha < 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 92 and _ifmt == 92:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alpha = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'alpha is too close to either zero or one.',
                    'This error is unlikely to occur.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 101 and _ifmt == 101:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'icc does not fall into the interval [lci,uci].',
                    'All output quantities have been calculated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 102 and _ifmt == 102:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The estimate of at least one variance component was negative.',
                    'Negative estimates were set to zero and all output quantities',
                    'calculated as documented.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _icc_py = utils._EngineFloat64ScalarType.to_py(_icc)
        _lci_py = utils._EngineFloat64ScalarType.to_py(_lci)
        _uci_py = utils._EngineFloat64ScalarType.to_py(_uci)
        _fstat_py = utils._EngineFloat64ScalarType.to_py(_fstat)
        _df1_py = utils._EngineFloat64ScalarType.to_py(_df1)
        _df2_py = utils._EngineFloat64ScalarType.to_py(_df2)
        _pvalue_py = utils._EngineFloat64ScalarType.to_py(_pvalue)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['icc', 'lci', 'uci', 'fstat', 'df1', 'df2', 'pvalue'],
        )(
            _icc_py,
            _lci_py,
            _uci_py,
            _fstat_py,
            _df1_py,
            _df2_py,
            _pvalue_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data
