# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `rnla` Chapter.

``rnla`` - Randomized Numerical Linear Algebra

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module covers linear algebra functions that make use of random
projections to reduce problem dimension.
This area is referred to as RNLA (Randomized Numerical Linear Algebra).
The functions can be split into the following categories:

- building blocks that are intended to be used as components in RNLA
  algorithms written by you;

- RNLA algorithms for matrix factorization.

It is envisaged that users of the higher-level functions, such as matrix
factorization, will have some background in linear algebra.
A common use case would be that you have tried solving your problem
using a deterministic linear algebra function, e.g., an LAPACK function
from submodule ``lapackeig``, and are in need of a function that is more
computationally efficient.

Users of the building block functions would be expected to have some
familiarity with the RNLA literature, i.e., a higher level of expertise.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10intro.html
"""

# NAG Copyright 2017-2019.

import sys as _sys

from ..base import utils

def svd_rowext_real(jobu, jobvt, m, n, a, k, rtol_abs, rtol_rel, statecomm, s, u, vt):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors.

    For full information please refer to the NAG Library document for
    f10ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10caf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing part of or none of the matrix U.

        `jobu` = 'S'

          The first k columns of U (the left singular vectors) are
          returned in the array `u`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing part of or none of the matrix
        V^T.

        `jobvt` = 'S'

          The first k rows of V^T (the right singular vectors) are
          returned in the array `vt`.

        `jobvt` = 'N'

          No rows of V^T (no right singular vectors) are computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`)
        The m by n matrix A.

    k : int
        k, number of columns in random projection, Y = A Omega.

    rtol_abs : float
        The absolute tolerance, epsilon_a used in defining the threshold
        on estimating the rank of A. If `rtol_abs` < 0.0 then 0.0 is
        used unless `rtol_rel` <= 0.0 in which case sqrt(machine
        precision) is used.

    rtol_rel : float
        The relative tolerance, epsilon_r used in defining the threshold
        on estimating the rank of A. If `rtol_rel` < 0.0 then 0.0 is
        used unless `rtol_abs` <= 0.0 in which case sqrt(machine
        precision) is used.

    statecomm : dict, RNG communication object, modified in place
        RNG communication structure.

        This argument must have been initialized by a prior call to
        ``rand.init_repeat`` or ``rand.init_nonrepeat``.

    s : float, shape (`k`), modified in place
        On exit: the first `r` elements of `s` contain the `r` largest
        singular values of A in descending order. The remaining values
        are set to zero.

    u : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='S': `m`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='S': `k`; otherwise: 0.

        On exit, if not None on entry: if `jobu` = 'S', `u` contains the
        first `r` columns of U (the left singular vectors, stored
        column-wise); the remaining elements of `u` are set to zero.

        If `jobu` = 'N', `u` is not referenced.

    vt : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvt`='S': `k`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvt`='S': `n`; otherwise: 1.

        On exit, if not None on entry: if `jobvt` = 'S', `vt` contains
        the first `r` rows of V^T (the right singular vectors); the
        remaining elements of `vt` are set to zero.

        If `jobvt` = 'N', `vt` is not referenced.

    Returns
    -------
    r : int
        r, contains estimated rank of array `a`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `jobu` = *<value>*.

            Constraint: `jobu` = 'S' or 'N'.

        (`errno` 2)
            On entry, `jobvt` = *<value>*.

            Constraint: `jobvt` = 'S' or 'N'.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 7)
            On entry, `k` = *<value>*.

            Constraint: 0 < `k` <= min(`m`, `n`).

        (`errno` 8)
            On entry, `statecomm`['state'] vector has been corrupted or
            not initialized.

        (`errno` 9)
            On entry, `jobu` = *<value>* and ldu = *<value>*.

            Constraint: if `jobu` = 'S', ldu >= `m`.

        (`errno` 10)
            On entry, `jobvt` = *<value>* and ldvt = *<value>*.

            Constraint: if `jobvt` = 'S', ldvt >= `k`.

        (`errno` 21)
            On exit, `r` = `k`, the rank of A may be larger than `r`.

            Increase `k` to obtain a more accurate rank estimate.

            Smallest diagonal element of R, from QR of Y, =*<value>*.

            Tolerance used to determine rank =*<value>*.

        (`errno` 22)
            A has effective rank of zero.

            First diagonal element of R, from QR of Y, =*<value>*.

            Tolerance used to determine rank =*<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.rnla.svd_rowext_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f10caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobvt(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, rtol_abs.
        utils._BYREF_FLOAT64, # Input, rtol_rel.
        utils._EngineIntArrayType.c_type, # Input/Output, state[lstate].
        utils._BYREF_INT, # Input, lstate.
        utils._EngineFloat64ArrayType.c_type, # Output, s[k].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._BYREF_INT, # Output, r.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt = utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _rtol_abs = utils._EngineFloat64ScalarType(
        rtol_abs, {'fun_name': fun_name, 'entity': 'rtol_abs'},
    ).to_ctypes()
    _rtol_rel = utils._EngineFloat64ScalarType(
        rtol_rel, {'fun_name': fun_name, 'entity': 'rtol_rel'},
    ).to_ctypes()
    utils._check_comm(
        statecomm, {'fun_name': fun_name, 'entity': 'statecomm'},
        req_keys=('state',),
    )
    _lstate_py = utils._EngineIntArrayType.get_shape(statecomm['state'], {'fun_name': fun_name, 'entity': 'statecomm[\'state\']'}, exp_rank=1)[0]
    _lstate = utils._EngineIntScalarType(
        _lstate_py, {'fun_name': fun_name, 'entity': '_lstate'},
    ).to_ctypes()
    _state = statecomm['state']
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('S', 's'):
        if u is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u'},
                'must not be None when jobu in (\'S\', \'s\')',
            )
    if u is None:
        if _jobu_py in ('S', 's'):
            _u_dim1 = _m_py
        else:
            _u_dim1 = 0
        if _jobu_py in ('S', 's'):
            _u_dim2 = _k_py
        else:
            _u_dim2 = 0
        _u = utils._EngineFloat64ArrayType.empty_ctypes((_u_dim1)*(_u_dim2))
    else:
        if _jobu_py in ('S', 's'):
            _u_dim1 = _m_py
        else:
            _u_dim1 = 0
        if _jobu_py in ('S', 's'):
            _u_dim2 = _k_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u_cls = utils._EngineFloat64ArrayType(
            u, {'fun_name': fun_name, 'entity': 'u'},
            exp_shape=exp_shape_u,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    _jobvt_py = utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('S', 's'):
        if vt is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vt'},
                'must not be None when jobvt in (\'S\', \'s\')',
            )
    if vt is None:
        if _jobvt_py in ('S', 's'):
            _vt_dim1 = _k_py
        else:
            _vt_dim1 = 0
        if _jobvt_py in ('S', 's'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 1
        _vt = utils._EngineFloat64ArrayType.empty_ctypes((_vt_dim1)*(_vt_dim2))
    else:
        if _jobvt_py in ('S', 's'):
            _vt_dim1 = _k_py
        else:
            _vt_dim1 = 0
        if _jobvt_py in ('S', 's'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 1
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt_cls = utils._EngineFloat64ArrayType(
            vt, {'fun_name': fun_name, 'entity': 'vt'},
            exp_shape=exp_shape_vt,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _r = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _jobu,
        _jobvt,
        _m,
        _n,
        _a,
        _lda,
        _k,
        _rtol_abs,
        _rtol_rel,
        _state,
        _lstate,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _r,
        _errbuf,
        _ierr,
        _jobu._length_,
        _jobvt._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobu = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobu = \'S\' or \'N\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvt = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobvt = \'S\' or \'N\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 < k <= min(m, n).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statecomm[\'state\'] vector has been corrupted or not initialized.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobu = \'{:s}\' and ldu = {:s}.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if jobu = \'S\', ldu >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvt = \'{:s}\' and ldvt = {:s}.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if jobvt = \'S\', ldvt >= k.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On exit, r = k, the rank of A may be larger than r.',
                    'Increase k to obtain a more accurate rank estimate.',
                    'Smallest diagonal element of R, from QR of Y, ={:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Tolerance used to determine rank ={:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A has effective rank of zero.',
                    'First diagonal element of R, from QR of Y, ={:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Tolerance used to determine rank ={:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        if u is not None:
            _u_cls.refresh_py(_u)
        if vt is not None:
            _vt_cls.refresh_py(_vt)
        _r_py = utils._EngineIntScalarType.to_py(_r)
        return_data = _r_py
    _err_class.manage()
    return return_data

def randproj_dct_real(trans, m, n, a, k, statecomm):
    u"""
    Computes a fast random projection of a real matrix using a discrete
    cosine transform.

    For full information please refer to the NAG Library document for
    f10da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10daf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether the operation pre-multiplies A or
        post-multiplies A.

        `trans` = 'N'

          Random projection is done by post-multiplication, Y = A Omega.

        `trans` = 'T'

          Random projection is done by pre-multiplication, Y = Omega A.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `trans` = 'N', the first k columns of A are
        overwritten with the m by k random projection of A.

        If `trans` = 'T', the first k rows of A are overwritten with the
        k by n random projection of A.

    k : int
        k, number of columns in the random projection, Y = A Omega.

    statecomm : dict, RNG communication object, modified in place
        RNG communication structure.

        This argument must have been initialized by a prior call to
        ``rand.init_repeat`` or ``rand.init_nonrepeat``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 4)
            On entry, `k` = *<value>* and `trans` = *<value>*.

            Constraint: if `trans` = 'N', 0 < `k` <= `n`, if `trans` =
            'T', 0 < `k` <= `m`.

        (`errno` 6)
            On entry, `statecomm`['state'] vector has been corrupted or
            not initialized.
    """

    excs = []
    fun_name = 'naginterfaces.base.rnla.randproj_dct_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f10daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, k.
        utils._EngineIntArrayType.c_type, # Input/Output, state[lstate].
        utils._BYREF_INT, # Input, lstate.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    utils._check_comm(
        statecomm, {'fun_name': fun_name, 'entity': 'statecomm'},
        req_keys=('state',),
    )
    _lstate_py = utils._EngineIntArrayType.get_shape(statecomm['state'], {'fun_name': fun_name, 'entity': 'statecomm[\'state\']'}, exp_rank=1)[0]
    _lstate = utils._EngineIntScalarType(
        _lstate_py, {'fun_name': fun_name, 'entity': '_lstate'},
    ).to_ctypes()
    _state = statecomm['state']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _trans,
        _m,
        _n,
        _a,
        _lda,
        _k,
        _state,
        _lstate,
        _errbuf,
        _ierr,
        _trans._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, trans = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: trans = \'N\' or \'T\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and trans = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if trans = \'N\', 0 < k <= n, if trans = \'T\', 0 < k <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statecomm[\'state\'] vector has been corrupted or not initialized.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data
