# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `linsys` Chapter.

``linsys`` - Simultaneous Linear Equations

This module is concerned with the solution of the matrix equation AX =
B, where B may be a single vector or a matrix of multiple right-hand
sides.
The matrix A may be real, complex, symmetric, Hermitian, positive
definite, positive definite Toeplitz or banded.
It may also be rectangular, in which case a least squares solution is
obtained.

Much of the functionality of this module has been superseded by
functions from submodule ``lapacklin`` and submodule ``lapackeig``
(LAPACK routines) as those modules have grown and have included driver
and expert driver functions.

For a general introduction to sparse systems of equations, see [the F11
Introduction], which provides functions for large sparse systems. Some
functions for sparse problems are also included in this module; they are
described in [Sparse Matrix Functions].

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

@_b_utils._withdrawal(
    'naginterfaces.library.linsys.withdraw_real_posdef_solve_ref',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dposvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_posdef_solve_ref(a, b, n, m):
    u"""
    Solution of real symmetric positive definite simultaneous linear
    equations with multiple right-hand sides using iterative refinement
    (Black Box).

    Note: ``withdraw_real_posdef_solve_ref`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dposvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04abf.html

    Parameters
    ----------
    a : float, array-like, shape (`n`, `n`)
        The upper triangle of the n by n positive definite symmetric
        matrix A. The elements of the array below the diagonal need not
        be set.

    b : float, array-like, shape (`n`, `m`)
        The n by m right-hand side matrix B.

    n : int
        n, the order of the matrix A.

    m : int
        m, the number of right-hand sides.

    Returns
    -------
    a : float, ndarray, shape (`n`, `n`)
        The elements of the array below the diagonal are overwritten;
        the upper triangle of A is unchanged.

    c : float, ndarray, shape (`n`, `m`)
        The n by m solution matrix X.

    bb : float, ndarray, shape (`n`, `m`)
        The final n by m residual matrix R = B-AX.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is not positive definite.

        (`errno` 2)
            Matrix A is too ill-conditioned; iterative refinement fails
            to improve the solution.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import withdraw_real_posdef_solve_ref as b_withdraw_real_posdef_solve_ref

    fun_name = 'naginterfaces.library.linsys.withdraw_real_posdef_solve_ref'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_m_sorder,
    )
    _bb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_m_sorder,
    )
    b_withdraw_real_posdef_solve_ref(
        _a,
        _b,
        _n,
        _m,
        _c,
        _bb,
    )
    _a_py = _a
    _c_py = _c
    _bb_py = _bb
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'c', 'bb'],
    )(
        _a_py,
        _c_py,
        _bb_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.linsys.withdraw_real_square_solve_ref',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dgesvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_square_solve_ref(a, b):
    u"""
    Solution of real simultaneous linear equations with multiple
    right-hand sides using iterative refinement (Black Box).

    Note: ``withdraw_real_square_solve_ref`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dgesvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04aef.html

    Parameters
    ----------
    a : float, array-like, shape (n, n)
        The n by n matrix A.

    b : float, array-like, shape (n, m)
        The n by m right-hand side matrix B.

    Returns
    -------
    c : float, ndarray, shape (n, m)
        The n by m solution matrix X.

    aa : float, ndarray, shape (n, n)
        The triangular factors L and U, except that the unit diagonal
        elements of U are not stored.

    bb : float, ndarray, shape (n, m)
        The final n by m residual matrix R = B-AX.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is approximately singular.

        (`errno` 2)
            The matrix A is too ill-conditioned.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m >= 0.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import withdraw_real_square_solve_ref as b_withdraw_real_square_solve_ref

    fun_name = 'naginterfaces.library.linsys.withdraw_real_square_solve_ref'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_m_sorder,
    )
    _aa = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    _bb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_m_sorder,
    )
    b_withdraw_real_square_solve_ref(
        _a,
        _b,
        _n,
        _m,
        _c,
        _aa,
        _bb,
    )
    _c_py = _c
    _aa_py = _aa
    _bb_py = _bb
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'aa', 'bb'],
    )(
        _c_py,
        _aa_py,
        _bb_py,
    )
    return _return_data

def real_gen_lsqsol(a, b, eps):
    u"""
    Least squares solution of m real equations in n unknowns, rank = n,
    m >= n using iterative refinement (Black Box).

    For full information please refer to the NAG Library document for
    f04am

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04amf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m by n matrix A.

    b : float, array-like, shape (m, ir)
        The m by r right-hand side matrix B.

    eps : float
        Must be set to the value of the machine precision.

    Returns
    -------
    x : float, ndarray, shape (n, ir)
        The n by r solution matrix X.

    qr : float, ndarray, shape (m, n)
        Details of the QR factorization.

    alpha : float, ndarray, shape (n)
        The diagonal elements of the upper triangular matrix R.

    ipiv : int, ndarray, shape (n)
        Details of the column interchanges.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The rank of `a` is less than n. The problem does not have a
            unique solution.

        (`errno` 2)
            The iterative refinement fails to converge. The matrix A is
            too ill-conditioned.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_gen_lsqsol as b_real_gen_lsqsol

    fun_name = 'naginterfaces.library.linsys.real_gen_lsqsol'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ir_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _ir = _b_utils._EngineIntScalarType(
        _ir_py, {'fun_name': fun_name, 'entity': '_ir'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _ir_py),
        sorder=_m_sorder,
    )
    _qr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _n_py),
        sorder=_m_sorder,
    )
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_real_gen_lsqsol(
            _a,
            _x,
            _b,
            _m,
            _n,
            _ir,
            eps,
            _qr,
            _alpha,
            _ipiv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _x_py = _x
    _qr_py = _qr
    _alpha_py = _alpha
    _ipiv_py = _ipiv
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'qr', 'alpha', 'ipiv'],
    )(
        _x_py,
        _qr_py,
        _alpha_py,
        _ipiv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.linsys.withdraw_real_posdef_solve_1rhs',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dposvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_posdef_solve_1rhs(a, b):
    u"""
    Solution of real symmetric positive definite simultaneous linear
    equations, one right-hand side using iterative refinement (Black
    Box).

    Note: ``withdraw_real_posdef_solve_1rhs`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dposvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04asf.html

    Parameters
    ----------
    a : float, array-like, shape (n, n)
        The upper triangle of the n by n positive definite symmetric
        matrix A. The elements of the array below the diagonal need not
        be set.

    b : float, array-like, shape (max(1, n))
        The right-hand side vector b.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The elements of the array below the diagonal are overwritten;
        the upper triangle of `a` is unchanged.

    c : float, ndarray, shape (max(1, n))
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is not positive definite.

        (`errno` 2)
            Matrix A is too ill-conditioned; iterative refinement fails
            to improve the solution.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import withdraw_real_posdef_solve_1rhs as b_withdraw_real_posdef_solve_1rhs

    fun_name = 'naginterfaces.library.linsys.withdraw_real_posdef_solve_1rhs'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py),),
    )
    b_withdraw_real_posdef_solve_1rhs(
        _a,
        _b,
        _n,
        _c,
    )
    _a_py = _a
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'c'],
    )(
        _a_py,
        _c_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.linsys.withdraw_real_square_solve_1rhs',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dgesvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_square_solve_1rhs(a, b, n):
    u"""
    Solution of real simultaneous linear equations, one right-hand side
    using iterative refinement (Black Box).

    Note: ``withdraw_real_square_solve_1rhs`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dgesvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04atf.html

    Parameters
    ----------
    a : float, array-like, shape (`n`, `n`)
        The n by n matrix A.

    b : float, array-like, shape (`n`)
        The right-hand side vector b.

    n : int
        n, the order of the matrix A.

    Returns
    -------
    c : float, ndarray, shape (`n`)
        The solution vector x.

    aa : float, ndarray, shape (`n`, `n`)
        The triangular factors L and U, except that the unit diagonal
        elements of U are not stored.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is approximately singular.

        (`errno` 2)
            The matrix A is too ill-conditioned.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import withdraw_real_square_solve_1rhs as b_withdraw_real_square_solve_1rhs

    fun_name = 'naginterfaces.library.linsys.withdraw_real_square_solve_1rhs'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _aa = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    b_withdraw_real_square_solve_1rhs(
        _a,
        _b,
        _n,
        _c,
        _aa,
    )
    _c_py = _c
    _aa_py = _aa
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'aa'],
    )(
        _c_py,
        _aa_py,
    )
    return _return_data

def real_sparse_fac_solve(a, comm, rhs, mtype):
    u"""
    Solution of real sparse simultaneous linear equations (coefficient
    matrix already factorized).

    For full information please refer to the NAG Library document for
    f04ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04axf.html

    Parameters
    ----------
    a : float, array-like, shape (licn)
        The nonzero elements in the factorization of the matrix A, as
        returned by ``matop.real_gen_sparse_lu`` or
        ``matop.real_gen_sparse_lu_reuse``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``matop.real_gen_sparse_lu``.

    rhs : float, array-like, shape (n)
        The right-hand side vector b.

    mtype : int
        `mtype` specifies the task to be performed.

        `mtype` = 1

          Solve Ax = b.

        `mtype` != 1

          Solve A^Tx = b.

    Returns
    -------
    rhs : float, ndarray, shape (n)
        This argument is overwritten by the solution vector x.

    resid : float
        The value of the maximum residual, max(|b_i-sum_j^a_{ij}x_j|),
        over all the unsatisfied equations, in case
        ``matop.real_gen_sparse_lu`` or
        ``matop.real_gen_sparse_lu_reuse`` has been used to factorize a
        singular or rectangular matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_sparse_fac_solve as b_real_sparse_fac_solve

    fun_name = 'naginterfaces.library.linsys.real_sparse_fac_solve'

    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icn', 'ikeep', 'idisp'),
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(comm['ikeep'], {'fun_name': fun_name, 'entity': 'comm[\'ikeep\']'}, exp_rank=1, validate=False)[0]//5, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _licn_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _licn = _b_utils._EngineIntScalarType(
        _licn_py, {'fun_name': fun_name, 'entity': '_licn'},
    ).to_ctypes()
    _rhs = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rhs'},
        rhs,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = b_real_sparse_fac_solve(
        _n,
        _a,
        _licn,
        comm,
        _rhs,
        mtype,
    )
    _rhs_py = _rhs
    _resid_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rhs', 'resid'],
    )(
        _rhs_py,
        _resid_py,
    )
    return _return_data

def real_square_solve(n, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real system of linear equations.

    For full information please refer to the NAG Library document for
    f04ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04baf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    a : float, array-like, shape (`n`, `n`)
        The n by n coefficient matrix A.

    b : float, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : float, ndarray, shape (`n`, `n`)
        If no exception is raised, the factors L and U from the
        factorization A = PLU. The unit diagonal elements of L are not
        stored.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i indicates a
        row interchange was not required.

    b : float, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_square_solve as b_real_square_solve

    fun_name = 'naginterfaces.library.linsys.real_square_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_real_square_solve(
            _n,
            _nrhs,
            _a,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_band_solve(n, kl, ku, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real banded system of linear equations.

    For full information please refer to the NAG Library document for
    f04bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bbf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kl : int
        The number of subdiagonals k_l, within the band of A.

    ku : int
        The number of superdiagonals k_u, within the band of A.

    ab : float, array-like, shape (2*`kl`+`ku`+1, `n`)
        The n by n matrix A.

        See [Further Comments] for further details.

    b : float, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ab : float, ndarray, shape (2*`kl`+`ku`+1, `n`)
        If no exception is raised, this argument is overwritten by
        details of the factorization.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i indicates a
        row interchange was not required.

    b : float, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -6)
            On entry, ldab = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` -4)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` -2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_band_solve as b_real_band_solve

    fun_name = 'naginterfaces.library.linsys.real_band_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_real_band_solve(
            _n,
            kl,
            ku,
            _nrhs,
            _ab,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ab_py = _ab
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ab', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _ab_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_tridiag_solve(n, dl, d, du, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f04bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bcf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    dl : float, array-like, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the matrix A.

    d : float, array-like, shape (`n`)
        Must contain the n diagonal elements of the matrix A.

    du : float, array-like, shape (`n`-1)
        Must contain the (n-1) superdiagonal elements of the matrix A

    b : float, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    dl : float, ndarray, shape (`n`-1)
        If no exception is raised, `dl` is overwritten by the (n-1)
        multipliers that define the matrix L from the LU factorization
        of A.

    d : float, ndarray, shape (`n`)
        If no exception is raised, `d` is overwritten by the n diagonal
        elements of the upper triangular matrix U from the LU
        factorization of A.

    du : float, ndarray, shape (`n`-1)
        If no exception is raised, `du` is overwritten by the (n-1)
        elements of the first superdiagonal of U.

    du2 : float, ndarray, shape (`n`-2)
        If no exception is raised, `du2` returns the (n-2) elements of
        the second superdiagonal of U.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will always be
        either i or (i+1); `ipiv`[i-1] = i indicates a row interchange
        was not required.

    b : float, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_tridiag_solve as b_real_tridiag_solve

    fun_name = 'naginterfaces.library.linsys.real_tridiag_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _dl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dl'},
        dl,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _du = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'du'},
        du,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _du2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py-2,),
    )
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_real_tridiag_solve(
            _n,
            _nrhs,
            _dl,
            _d,
            _du,
            _du2,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _dl_py = _dl
    _d_py = _d
    _du_py = _du
    _b_py = _b
    _du2_py = _du2
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dl', 'd', 'du', 'du2', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _dl_py,
        _d_py,
        _du_py,
        _du2_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_posdef_solve(uplo, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite system of linear equations.

    For full information please refer to the NAG Library document for
    f04bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bdf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    a : float, array-like, shape (n, n)
        The n by n symmetric matrix A.

        If `uplo` = 'U', the leading n by n upper triangular part of `a`
        contains the upper triangular part of the matrix A, and the
        strictly lower triangular part of `a` is not referenced.

        If `uplo` = 'L', the leading n by n lower triangular part of `a`
        contains the lower triangular part of the matrix A, and the
        strictly upper triangular part of `a` is not referenced.

    b : float, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^TU or A =
        LL^T.

    b : float, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_posdef_solve as b_real_posdef_solve

    fun_name = 'naginterfaces.library.linsys.real_posdef_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_posdef_solve(
            uplo,
            _n,
            _nrhs,
            _a,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_posdef_packed_solve(uplo, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite system of linear equations,
    packed storage.

    For full information please refer to the NAG Library document for
    f04be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bef.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    ap : float, array-like, shape (n*(n+1)/2)
        The n by n symmetric matrix A. The upper or lower triangular
        part of the symmetric matrix is packed column-wise in a linear
        array. The jth column of A is stored in the array `ap` as
        follows:

    b : float, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ap : float, ndarray, shape (n*(n+1)/2)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^TU or A =
        LL^T, in the same storage format as A.

    b : float, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_posdef_packed_solve as b_real_posdef_packed_solve

    fun_name = 'naginterfaces.library.linsys.real_posdef_packed_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_posdef_packed_solve(
            uplo,
            _n,
            _nrhs,
            _ap,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ap_py = _ap
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ap', 'b', 'rcond', 'errbnd'],
    )(
        _ap_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_posdef_band_solve(uplo, kd, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite banded system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    kd : int
        The number of superdiagonals k (and the number of subdiagonals)
        of the band matrix A.

    ab : float, array-like, shape (`kd`+1, n)
        The n by n symmetric band matrix A. The upper or lower
        triangular part of the symmetric matrix is stored in the first
        `kd`+1 rows of the array. The jth column of A is stored in the
        jth column of the array `ab` as follows:

        See [Further Comments] below for further details.

    b : float, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ab : float, ndarray, shape (`kd`+1, n)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^TU or A =
        LL^T, in the same storage format as A.

    b : float, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -4)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -3)
            On entry, `kd` = *<value>*.

            Constraint: `kd` >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_posdef_band_solve as b_real_posdef_band_solve

    fun_name = 'naginterfaces.library.linsys.real_posdef_band_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_posdef_band_solve(
            uplo,
            _n,
            kd,
            _nrhs,
            _ab,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ab_py = _ab
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ab', 'b', 'rcond', 'errbnd'],
    )(
        _ab_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_posdef_tridiag_solve(d, e, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bgf.html

    Parameters
    ----------
    d : float, array-like, shape (n)
        Must contain the n diagonal elements of the tridiagonal matrix
        A.

    e : float, array-like, shape (n-1)
        Must contain the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

    b : float, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    d : float, ndarray, shape (n)
        If the function exits successfully or `errno` = `n` + 1, `d` is
        overwritten by the n diagonal elements of the diagonal matrix D
        from the LDL^T factorization of A.

    e : float, ndarray, shape (n-1)
        If the function exits successfully or `errno` = `n` + 1, `e` is
        overwritten by the (n-1) subdiagonal elements of the unit lower
        bidiagonal matrix L from the LDL^T factorization of A. (`e` can
        also be regarded as the superdiagonal of the unit upper
        bidiagonal factor U from the U^TDU factorization of A.)

    b : float, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_posdef_tridiag_solve as b_real_posdef_tridiag_solve

    fun_name = 'naginterfaces.library.linsys.real_posdef_tridiag_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_posdef_tridiag_solve(
            _n,
            _nrhs,
            _d,
            _e,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _d_py = _d
    _e_py = _e
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['d', 'e', 'b', 'rcond', 'errbnd'],
    )(
        _d_py,
        _e_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_symm_solve(uplo, n, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f04bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bhf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    a : float, array-like, shape (`n`, `n`)
        The n by n symmetric matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

    b : float, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : float, ndarray, shape (`n`, `n`)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.dsytrf``.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, details of the interchanges and the
        block structure of D, as determined by ``lapacklin.dsytrf``.

        `ipiv`[k-1] > 0

          Rows and columns k and `ipiv`[k-1] were interchanged, and
          d_{kk} is a 1 by 1 diagonal block.

        `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0

          Rows and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block.

        `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0

          Rows and columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : float, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_symm_solve as b_real_symm_solve

    fun_name = 'naginterfaces.library.linsys.real_symm_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_real_symm_solve(
            uplo,
            _n,
            _nrhs,
            _a,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_symm_packed_solve(uplo, n, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bjf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The n by n symmetric matrix A, packed column-wise in a linear
        array. The jth column of the matrix A is stored in the array
        `ap` as follows:

    b : float, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.dsptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, details of the interchanges and the
        block structure of D, as determined by ``lapacklin.dsptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : float, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import real_symm_packed_solve as b_real_symm_packed_solve

    fun_name = 'naginterfaces.library.linsys.real_symm_packed_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_real_symm_packed_solve(
            uplo,
            _n,
            _nrhs,
            _ap,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ap_py = _ap
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ap', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _ap_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_square_solve(n, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex system of linear equations.

    For full information please refer to the NAG Library document for
    f04ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04caf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    a : complex, array-like, shape (`n`, `n`)
        The n by n coefficient matrix A.

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        If no exception is raised, the factors L and U from the
        factorization A = PLU. The unit diagonal elements of L are not
        stored.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i indicates a
        row interchange was not required.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_square_solve as b_complex_square_solve

    fun_name = 'naginterfaces.library.linsys.complex_square_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_square_solve(
            _n,
            _nrhs,
            _a,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_band_solve(n, kl, ku, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex banded system of linear equations.

    For full information please refer to the NAG Library document for
    f04cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cbf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kl : int
        The number of subdiagonals k_l, within the band of A.

    ku : int
        The number of superdiagonals k_u, within the band of A.

    ab : complex, array-like, shape (2*`kl`+`ku`+1, `n`)
        The n by n matrix A.

        See [Further Comments] for further details.

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ab : complex, ndarray, shape (2*`kl`+`ku`+1, `n`)
        If no exception is raised, this argument is overwritten by
        details of the factorization.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i indicates a
        row interchange was not required.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no exception is raised, an estimate of the reciprocal of the
        condition number of the matrix A, computed as `rcond` =
        (||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -6)
            On entry, ldab = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` -4)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` -2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_band_solve as b_complex_band_solve

    fun_name = 'naginterfaces.library.linsys.complex_band_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_band_solve(
            _n,
            kl,
            ku,
            _nrhs,
            _ab,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ab_py = _ab
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ab', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _ab_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_tridiag_solve(n, dl, d, du, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f04cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04ccf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    dl : complex, array-like, shape (`n`-1)
        Must contain the (n-1) subdiagonal elements of the matrix A.

    d : complex, array-like, shape (`n`)
        Must contain the n diagonal elements of the matrix A.

    du : complex, array-like, shape (`n`-1)
        Must contain the (n-1) superdiagonal elements of the matrix A

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    dl : complex, ndarray, shape (`n`-1)
        If no exception is raised, `dl` is overwritten by the (n-1)
        multipliers that define the matrix L from the LU factorization
        of A.

    d : complex, ndarray, shape (`n`)
        If no exception is raised, `d` is overwritten by the n diagonal
        elements of the upper triangular matrix U from the LU
        factorization of A.

    du : complex, ndarray, shape (`n`-1)
        If no exception is raised, `du` is overwritten by the (n-1)
        elements of the first superdiagonal of U.

    du2 : complex, ndarray, shape (`n`-2)
        If no exception is raised, `du2` returns the (n-2) elements of
        the second superdiagonal of U.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, the pivot indices that define the
        permutation matrix P; at the ith step row i of the matrix was
        interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will always be
        either i or (i+1); `ipiv`[i-1] = i indicates a row interchange
        was not required.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_tridiag_solve as b_complex_tridiag_solve

    fun_name = 'naginterfaces.library.linsys.complex_tridiag_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _dl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dl'},
        dl,
        copy=True,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
    )
    _du = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'du'},
        du,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _du2 = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py-2,),
    )
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_tridiag_solve(
            _n,
            _nrhs,
            _dl,
            _d,
            _du,
            _du2,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _dl_py = _dl
    _d_py = _d
    _du_py = _du
    _b_py = _b
    _du2_py = _du2
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dl', 'd', 'du', 'du2', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _dl_py,
        _d_py,
        _du_py,
        _du2_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_posdef_solve(uplo, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite system of linear equations.

    For full information please refer to the NAG Library document for
    f04cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cdf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    a : complex, array-like, shape (n, n)
        The n by n Hermitian matrix A.

        If `uplo` = 'U', the leading n by n upper triangular part of `a`
        contains the upper triangular part of the matrix A, and the
        strictly lower triangular part of `a` is not referenced.

        If `uplo` = 'L', the leading n by n lower triangular part of `a`
        contains the lower triangular part of the matrix A, and the
        strictly upper triangular part of `a` is not referenced.

    b : complex, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^HU or A =
        LL^H.

    b : complex, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_posdef_solve as b_complex_posdef_solve

    fun_name = 'naginterfaces.library.linsys.complex_posdef_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    try:
        _return_scalars = b_complex_posdef_solve(
            uplo,
            _n,
            _nrhs,
            _a,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_posdef_packed_solve(uplo, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite system of linear equations,
    packed storage.

    For full information please refer to the NAG Library document for
    f04ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cef.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    ap : complex, array-like, shape (n*(n+1)/2)
        The n by n Hermitian matrix A. The upper or lower triangular
        part of the Hermitian matrix is packed column-wise in a linear
        array. The jth column of A is stored in the array `ap` as
        follows:

    b : complex, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ap : complex, ndarray, shape (n*(n+1)/2)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^HU or A =
        LL^H, in the same storage format as A.

    b : complex, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_posdef_packed_solve as b_complex_posdef_packed_solve

    fun_name = 'naginterfaces.library.linsys.complex_posdef_packed_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    try:
        _return_scalars = b_complex_posdef_packed_solve(
            uplo,
            _n,
            _nrhs,
            _ap,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ap_py = _ap
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ap', 'b', 'rcond', 'errbnd'],
    )(
        _ap_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_posdef_band_solve(uplo, kd, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite banded system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    kd : int
        The number of superdiagonals k (and the number of subdiagonals)
        of the band matrix A.

    ab : complex, array-like, shape (`kd`+1, n)
        The n by n Hermitian band matrix A. The upper or lower
        triangular part of the Hermitian matrix is stored in the first
        `kd`+1 rows of the array. The jth column of A is stored in the
        jth column of the array `ab` as follows:

        See [Further Comments] below for further details.

    b : complex, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ab : complex, ndarray, shape (`kd`+1, n)
        If the function exits successfully or `errno` = `n` + 1, the
        factor U or L from the Cholesky factorization A = U^HU or A =
        LL^H, in the same storage format as A.

    b : complex, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -4)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -3)
            On entry, `kd` = *<value>*.

            Constraint: `kd` >= 0.

        (`errno` -2)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_posdef_band_solve as b_complex_posdef_band_solve

    fun_name = 'naginterfaces.library.linsys.complex_posdef_band_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    try:
        _return_scalars = b_complex_posdef_band_solve(
            uplo,
            _n,
            kd,
            _nrhs,
            _ab,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ab_py = _ab
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ab', 'b', 'rcond', 'errbnd'],
    )(
        _ab_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_posdef_tridiag_solve(d, e, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cgf.html

    Parameters
    ----------
    d : float, array-like, shape (n)
        Must contain the n diagonal elements of the tridiagonal matrix
        A.

    e : complex, array-like, shape (n-1)
        Must contain the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

    b : complex, array-like, shape (n, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    d : float, ndarray, shape (n)
        If the function exits successfully or `errno` = `n` + 1, `d` is
        overwritten by the n diagonal elements of the diagonal matrix D
        from the LDL^H factorization of A.

    e : complex, ndarray, shape (n-1)
        If the function exits successfully or `errno` = `n` + 1, `e` is
        overwritten by the (n-1) subdiagonal elements of the unit lower
        bidiagonal matrix L from the LDL^H factorization of A. (`e` can
        also be regarded as the conjugate of the superdiagonal of the
        unit upper bidiagonal factor U from the U^HDU factorization of
        A.)

    b : complex, ndarray, shape (n, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1,||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_posdef_tridiag_solve as b_complex_posdef_tridiag_solve

    fun_name = 'naginterfaces.library.linsys.complex_posdef_tridiag_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
    )
    try:
        _return_scalars = b_complex_posdef_tridiag_solve(
            _n,
            _nrhs,
            _d,
            _e,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _d_py = _d
    _e_py = _e
    _b_py = _b
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['d', 'e', 'b', 'rcond', 'errbnd'],
    )(
        _d_py,
        _e_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_herm_solve(uplo, n, a, b):
    u"""
    Computes the solution and error-bound to a complex Hermitian system
    of linear equations.

    For full information please refer to the NAG Library document for
    f04ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04chf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    a : complex, array-like, shape (`n`, `n`)
        The n by n Hermitian matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zhetrf``.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, details of the interchanges and the
        block structure of D, as determined by ``lapacklin.zhetrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_herm_solve as b_complex_herm_solve

    fun_name = 'naginterfaces.library.linsys.complex_herm_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_herm_solve(
            uplo,
            _n,
            _nrhs,
            _a,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_herm_packed_solve(uplo, n, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04cj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cjf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The n by n Hermitian matrix A, packed column-wise in a linear
        array. The jth column of the matrix A is stored in the array
        `ap` as follows:

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zhptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, details of the interchanges and the
        block structure of D, as determined by ``lapacklin.zhptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_herm_packed_solve as b_complex_herm_packed_solve

    fun_name = 'naginterfaces.library.linsys.complex_herm_packed_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_herm_packed_solve(
            uplo,
            _n,
            _nrhs,
            _ap,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ap_py = _ap
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ap', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _ap_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_symm_solve(uplo, n, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f04dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04dhf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    a : complex, array-like, shape (`n`, `n`)
        The n by n complex symmetric matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.zsytrf``.

    ipiv : int, ndarray, shape (`n`)
        If no exception is raised, details of the interchanges and the
        block structure of D, as determined by ``lapacklin.zsytrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_symm_solve as b_complex_symm_solve

    fun_name = 'naginterfaces.library.linsys.complex_symm_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_symm_solve(
            uplo,
            _n,
            _nrhs,
            _a,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _a_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_symm_packed_solve(uplo, n, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex symmetric system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04dj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04djf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The n by n symmetric matrix A, packed column-wise in a linear
        array. The jth column of the matrix A is stored in the array
        `ap` as follows:

    b : complex, array-like, shape (`n`, nrhs)
        The n by r matrix of right-hand sides B.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        If no exception is raised, the block diagonal matrix D and the
        multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zsptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, ndarray, shape (`n`)
        If no constraints are violated, details of the interchanges and
        the block structure of D, as determined by ``lapacklin.zsptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, ndarray, shape (`n`, nrhs)
        If the function exits successfully or `errno` = `n` + 1, the n
        by r solution matrix X.

    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    from ..base.linsys import complex_symm_packed_solve as b_complex_symm_packed_solve

    fun_name = 'naginterfaces.library.linsys.complex_symm_packed_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _ipiv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_complex_symm_packed_solve(
            uplo,
            _n,
            _nrhs,
            _ap,
            _ipiv,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ap_py = _ap
    _b_py = _b
    _ipiv_py = _ipiv
    _rcond_py = _return_scalars.rcond
    _errbnd_py = _return_scalars.errbnd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ap', 'ipiv', 'b', 'rcond', 'errbnd'],
    )(
        _ap_py,
        _ipiv_py,
        _b_py,
        _rcond_py,
        _errbnd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_toeplitz_yule(t, wantp, wantv):
    u"""
    Solution of the Yule--Walker equations for real symmetric positive
    definite Toeplitz matrix, one right-hand side.

    For full information please refer to the NAG Library document for
    f04fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04fef.html

    Parameters
    ----------
    t : float, array-like, shape (n+1)
        `t`[0] must contain the value tau_0 of the diagonal elements of
        T, and the remaining n elements of `t` must contain the elements
        of the vector t.

    wantp : bool
        Must be set to True if the partial (auto)correlation
        coefficients are required, and must be set to False otherwise.

    wantv : bool
        Must be set to True if the mean square prediction errors are
        required, and must be set to False otherwise.

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x.

    p : float, ndarray, shape (:)
        With `wantp` as True, the ith element of `p` contains the
        partial (auto)correlation coefficient, or reflection
        coefficient, p_i for the ith step. (See [Further Comments] and
        submodule ``tsa``.) If `wantp` is False, `p` is not referenced.
        Note that in any case, x_n = p_n.

    v : float, ndarray, shape (:)
        With `wantv` as True, the ith element of `v` contains the mean
        square prediction error, or predictor error variance ratio, v_i,
        for the ith step. (See [Further Comments] and submodule
        ``tsa``.) If `wantv` is False, `v` is not referenced.

    vlast : float
        The value of v_n, the mean square prediction error for the final
        step.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Principal minor *<value>* is not positive definite. Value of
            the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_toeplitz_yule as b_real_toeplitz_yule

    fun_name = 'naginterfaces.library.linsys.real_toeplitz_yule'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wantp = _b_utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = _b_utils._EngineLogicalScalarType.to_py(_wantp)
    if _wantp_py:
        _p_dim1 = _n_py
    else:
        _p_dim1 = 1
    exp_shape_p = (_p_dim1,)
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_p,
    )
    _wantv = _b_utils._EngineLogicalScalarType(
        wantv, {'fun_name': fun_name, 'entity': 'wantv'},
    ).to_ctypes()
    _wantv_py = _b_utils._EngineLogicalScalarType.to_py(_wantv)
    if _wantv_py:
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    exp_shape_v = (_v_dim1,)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
    )
    try:
        _return_scalar = b_real_toeplitz_yule(
            _n,
            _t,
            _x,
            wantp,
            _p,
            wantv,
            _v,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _p_py = _p
    _v_py = _v
    _vlast_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'p', 'v', 'vlast'],
    )(
        _x_py,
        _p_py,
        _v_py,
        _vlast_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_toeplitz_solve(n, t, b, wantp):
    u"""
    Solution of real symmetric positive definite Toeplitz system, one
    right-hand side.

    For full information please refer to the NAG Library document for
    f04ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04fff.html

    Parameters
    ----------
    n : int
        The order of the Toeplitz matrix T.

    t : float, array-like, shape (`n`)
        `t`[i] must contain the value tau_i, for i = 0,1,...,`n`-1.

    b : float, array-like, shape (`n`)
        The right-hand side vector b.

    wantp : bool
        Must be set to True if the reflection coefficients are required,
        and must be set to False otherwise.

    Returns
    -------
    x : float, ndarray, shape (`n`)
        The solution vector x.

    p : float, ndarray, shape (:)
        With `wantp` as True, the ith element of `p` contains the
        reflection coefficient, p_i, for the ith step, for i =
        1,2,...,`n`-1. (See [Further Comments].) If `wantp` is False,
        `p` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Principal minor *<value>* is not positive definite. Value of
            the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_toeplitz_solve as b_real_toeplitz_solve

    fun_name = 'naginterfaces.library.linsys.real_toeplitz_solve'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wantp = _b_utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = _b_utils._EngineLogicalScalarType.to_py(_wantp)
    if _wantp_py:
        _p_dim1 = _n_py-1
    else:
        _p_dim1 = 1
    exp_shape_p = (_p_dim1,)
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_p,
    )
    try:
        b_real_toeplitz_solve(
            _n,
            _t,
            _b,
            _x,
            wantp,
            _p,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _x_py = _x
    _p_py = _p
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'p'],
    )(
        _x_py,
        _p_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_gen_solve(a, b, tol, lwork):
    u"""
    Least squares (if rank = n) or minimal least squares (if rank < n)
    solution of m real equations in n unknowns, m >= n.

    For full information please refer to the NAG Library document for
    f04jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04jgf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m by n matrix A.

    b : float, array-like, shape (m)
        The right-hand side vector b.

    tol : float
        A relative tolerance to be used to determine the rank of A.
        `tol` should be chosen as approximately the largest relative
        error in the elements of A. For example, if the elements of A
        are correct to about 4 significant figures then `tol` should be
        set to about 5*10^(-4). See [Further Comments] for a description
        of how `tol` is used to determine rank. If `tol` is outside the
        range (epsilon,1.0), where epsilon is the machine precision, the
        value epsilon is used in place of `tol`. For most problems this
        is unreasonably small.

    lwork : int
        The dimension of the array `work`.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If `svd` is returned as False, this argument is overwritten by
        details of the QR factorization of A.

        If `svd` is returned as True, the first n rows of `a` are
        overwritten by the right-hand singular vectors, stored by rows;
        and the remaining rows of the array are used as workspace.

    b : float, ndarray, shape (m)
        The first n elements of `b` contain the minimal least squares
        solution vector x. The remaining m-n elements are used for
        workspace.

    svd : bool
        Is returned as False if the least squares solution has been
        obtained from the QR factorization of A. In this case A is of
        full rank. `svd` is returned as True if the least squares
        solution has been obtained from the singular value decomposition
        of A.

    sigma : float
        The standard error, i.e., the value sqrt(r^Tr/(m-k)) when m > k,
        and the value zero when m = k. Here r is the residual vector
        b-Ax and k is the rank of A.

    irank : int
        k, the rank of the matrix A. It should be noted that it is
        possible for `irank` to be returned as n and `svd` to be
        returned as True. This means that the matrix R only just failed
        the test for nonsingularity.

    work : float, ndarray, shape (`lwork`)
        If `svd` is returned as False, then the first n elements of
        `work` contain information on the QR factorization of A (see
        argument `a` above), and `work`[n] contains the condition number
        ||R||_E||R^-1||_E of the upper triangular matrix R.

        If `svd` is returned as True, then the first n elements of
        `work` contain the singular values of A arranged in descending
        order and `work`[n] contains the total number of iterations
        taken by the QR algorithm.

        The rest of `work` is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lwork` is too small. Minimum size required:
            *<value>*.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>* and n = *<value>*.

            Constraint: m >= n.

        (`errno` 2)
            The QR algorithm has failed to converge to the singular
            values in 50*n iterations. This failure can only happen when
            the singular value decomposition is employed, but even then
            it is not likely to occur.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_gen_solve as b_real_gen_solve

    fun_name = 'naginterfaces.library.linsys.real_gen_solve'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lwork = _b_utils._EngineIntScalarType(
        lwork, {'fun_name': fun_name, 'entity': 'lwork'},
    ).to_ctypes()
    _lwork_py = _b_utils._EngineIntScalarType.to_py(_lwork)
    _work = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lwork_py,),
    )
    _return_scalars = b_real_gen_solve(
        _m,
        _n,
        _a,
        _b,
        tol,
        _work,
        _lwork,
    )
    _a_py = _a
    _b_py = _b
    _svd_py = _return_scalars.svd
    _sigma_py = _return_scalars.sigma
    _irank_py = _return_scalars.irank
    _work_py = _work
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'b', 'svd', 'sigma', 'irank', 'work'],
    )(
        _a_py,
        _b_py,
        _svd_py,
        _sigma_py,
        _irank_py,
        _work_py,
    )
    return _return_data

def real_tridiag_fac_solve(job, a, b, c, d, ipiv, y, tol):
    u"""
    Solution of real tridiagonal simultaneous linear equations
    (coefficient matrix already factorized by
    ``matop.real_gen_tridiag_lu``).

    For full information please refer to the NAG Library document for
    f04le

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04lef.html

    Parameters
    ----------
    job : int
        Must specify the equations to be solved.

        `job` = 1

          The equations (T-lambda I)x = y are to be solved, but diagonal
          elements of U are not to be perturbed.

        `job` = -1

          The equations (T-lambda I)x = y are to be solved and, if
          overflow would otherwise occur, diagonal elements of U are to
          be perturbed. See argument `tol`.

        `job` = 2

          The equations (T-lambda I)^Tx = y are to be solved, but
          diagonal elements of U are not to be perturbed.

        `job` = -2

          The equations (T-lambda I)^Tx = y are to be solved and, if
          overflow would otherwise occur, diagonal elements of U are to
          be perturbed. See argument `tol`.

        `job` = 3

          The equations Ux = y are to be solved, but diagonal elements
          of U are not to be perturbed.

        `job` = -3

          The equations Ux = y are to be solved and, if overflow would
          otherwise occur, diagonal elements of U are to be perturbed.
          See argument `tol`.

    a : float, array-like, shape (n)
        The diagonal elements of U as returned by
        ``real_tridiag_fac_solve``.

    b : float, array-like, shape (n)
        The elements of the first superdiagonal of U as returned by
        ``real_tridiag_fac_solve``.

    c : float, array-like, shape (n)
        The subdiagonal elements of L as returned by
        ``real_tridiag_fac_solve``.

    d : float, array-like, shape (n)
        The elements of the second superdiagonal of U as returned by
        ``real_tridiag_fac_solve``.

    ipiv : int, array-like, shape (n)
        Details of the matrix P as returned by
        ``matop.real_gen_tridiag_lu``.

    y : float, array-like, shape (n)
        The right-hand side vector y.

    tol : float
        The minimum perturbation to be made to very small diagonal
        elements of U. `tol` is only referenced when `job` is negative.
        `tol` should normally be chosen as about epsilon||U||, where
        epsilon is the machine precision, but if `tol` is supplied as
        non-positive, it is reset to epsilon max(|u_{ij}|).

    Returns
    -------
    y : float, ndarray, shape (n)
        This argument is overwritten by the solution vector x.

    tol : float
        If on entry `tol` is non-positive, it is reset as just
        described. Otherwise `tol` is unchanged.

    Raises
    ------
    NagValueError
        (`errno` i > 1)
            Overflow would occur computing the element I in the
            solution. I = *<value>*.

        (`errno` 1)
            On entry, `job` = *<value>*.

            Constraint: `job` != 0 and `job` >= -3 and `job` <= 3.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_tridiag_fac_solve as b_real_tridiag_fac_solve

    fun_name = 'naginterfaces.library.linsys.real_tridiag_fac_solve'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = b_real_tridiag_fac_solve(
        job,
        _n,
        _a,
        _b,
        _c,
        _d,
        _ipiv,
        _y,
        tol,
    )
    _y_py = _y
    _tol_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['y', 'tol'],
    )(
        _y_py,
        _tol_py,
    )
    return _return_data

def real_blkdiag_fac_solve(trans, blkstr, a, pivot, b):
    u"""
    Solution of real almost block diagonal simultaneous linear equations
    (coefficient matrix already factorized by
    ``matop.real_gen_blkdiag_lu``).

    For full information please refer to the NAG Library document for
    f04lh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04lhf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved.

        `trans` = 'N'

          Solve AX = B.

        `trans` = 'T'

          Solve A^TX = B.

    blkstr : int, array-like, shape (3, nbloks)
        Information which describes the block structure of A, as
        supplied to ``real_blkdiag_fac_solve``.

    a : float, array-like, shape (lena)
        The elements in the factorization of A, as returned by
        ``real_blkdiag_fac_solve``.

    pivot : int, array-like, shape (n)
        Details of the interchanges in the factorization, as returned by
        ``real_blkdiag_fac_solve``.

    b : float, array-like, shape (n, ir)
        The n by r right-hand side matrix B.

    Returns
    -------
    b : float, ndarray, shape (n, ir)
        This argument is overwritten by the n by r solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lena is too small. lena = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, the following equality does not hold:
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k =
            2,nbloks) = n.

        (`errno` 1)
            On entry, the following equality does not hold:
            sum(`blkstr`[0,k-1]:k = 1,nbloks) = n.

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[2,K-1] = *<value>*,
            `blkstr`[2,K-2] = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1]+`blkstr`[2,K-2] >=
            `blkstr`[1,K-1].

        (`errno` 1)
            On entry, the following inequality was not satisfied for: J
            = *<value>*. sum(`blkstr`[1,k-1]-`blkstr`[2,k-1]:k = 1,J) <=
            sum(`blkstr`[0,k-1]:k = 1,J) <=
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k = 2,J).

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>*,
            `blkstr`[2,K-1] = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1]-`blkstr`[2,K-1] <=
            `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>* and
            `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[2,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1] >= 0.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= 1.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[0,K-1] >= 1.

        (`errno` 1)
            On entry, n = *<value>* and nbloks = *<value>*.

            Constraint: n >= nbloks.

        (`errno` 1)
            On entry, nbloks = *<value>*.

            Constraint: nbloks >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, ir = *<value>*.

            Constraint: ir >= 1.

        (`errno` 1)
            On entry, `trans` != 'N' or 'T': `trans` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_blkdiag_fac_solve as b_real_blkdiag_fac_solve

    fun_name = 'naginterfaces.library.linsys.real_blkdiag_fac_solve'

    _pivot = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'pivot'},
        pivot,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_pivot, {'fun_name': fun_name, 'entity': 'pivot'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _blkstr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'blkstr'},
        blkstr,
        copy=False,
    )
    _nbloks_py = _b_utils._EngineIntArrayType.get_shape(_blkstr, {'fun_name': fun_name, 'entity': 'blkstr'}, exp_rank=2)[1]
    _nbloks = _b_utils._EngineIntScalarType(
        _nbloks_py, {'fun_name': fun_name, 'entity': '_nbloks'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lena_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _lena = _b_utils._EngineIntScalarType(
        _lena_py, {'fun_name': fun_name, 'entity': '_lena'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ir_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _ir = _b_utils._EngineIntScalarType(
        _ir_py, {'fun_name': fun_name, 'entity': '_ir'},
    ).to_ctypes()
    b_real_blkdiag_fac_solve(
        trans,
        _n,
        _nbloks,
        _blkstr,
        _a,
        _lena,
        _pivot,
        _b,
        _ir,
    )
    _b_py = _b
    return _b_py

def real_posdef_vband_solve(al, d, nrow, b, iselct):
    u"""
    Solution of real symmetric positive definite variable-bandwidth
    simultaneous linear equations (coefficient matrix already factorized
    by ``matop.real_vband_posdef_fac``).

    For full information please refer to the NAG Library document for
    f04mc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mcf.html

    Parameters
    ----------
    al : float, array-like, shape (lal)
        The elements within the envelope of the lower triangular matrix
        L, taken in row by row order, as returned by
        ``matop.real_vband_posdef_fac``. The unit diagonal elements of L
        must be stored explicitly.

    d : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `iselct`>=4: 1; otherwise: n.

        The diagonal elements of the diagonal matrix D. `d` is not
        referenced if `iselct` >= 4.

    nrow : int, array-like, shape (n)
        `nrow`[i-1] must contain the width of row i of L, i.e., the
        number of elements between the first (leftmost) nonzero element
        and the element on the diagonal, inclusive.

    b : float, array-like, shape (n, ir)
        The n by r right-hand side matrix B. See also [Further
        Comments].

    iselct : int
        Must specify the type of system to be solved, as follows:

        `iselct` = 1

          Solve LDL^TX = B.

        `iselct` = 2

          Solve LDX = B.

        `iselct` = 3

          Solve DL^TX = B.

        `iselct` = 4

          Solve LL^TX = B.

        `iselct` = 5

          Solve LX = B.

        `iselct` = 6

          Solve L^TX = B.

    Returns
    -------
    x : float, ndarray, shape (n, ir)
        The n by r solution matrix X. See also [Further Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lal = *<value>* and `nrow`[0]+...+`nrow`[n-1] =
            *<value>*.

            Constraint: lal >= `nrow`[0]+...+`nrow`[n-1].

        (`errno` 1)
            On entry, I = *<value>* and `nrow`[I-1] = *<value>*.

            Constraint: `nrow`[I-1] >= 1 and `nrow`[I-1] <= I.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, ir = *<value>*.

            Constraint: ir >= 1.

        (`errno` 3)
            On entry, `iselct` = *<value>*.

            Constraint: `iselct` >= 1 and `iselct` <= 6.

        (`errno` 4)
            On entry, I = *<value>*.

            Constraint: `d`[I-1] != 0.0.

        (`errno` 5)
            At least one diagonal entry of `al` is not unit.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.linsys import real_posdef_vband_solve as b_real_posdef_vband_solve

    fun_name = 'naginterfaces.library.linsys.real_posdef_vband_solve'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_b, 'b'),),
    )
    _nrow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nrow'},
        nrow,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_nrow, {'fun_name': fun_name, 'entity': 'nrow'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _al = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'al'},
        al,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lal_py = _b_utils._EngineFloat64ArrayType.get_shape(_al, {'fun_name': fun_name, 'entity': 'al'}, exp_rank=1)[0]
    _lal = _b_utils._EngineIntScalarType(
        _lal_py, {'fun_name': fun_name, 'entity': '_lal'},
    ).to_ctypes()
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ir_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _ir = _b_utils._EngineIntScalarType(
        _ir_py, {'fun_name': fun_name, 'entity': '_ir'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _ir_py),
        sorder=_m_sorder,
    )
    b_real_posdef_vband_solve(
        _n,
        _al,
        _lal,
        _d,
        _nrow,
        _ir,
        _b,
        iselct,
        _x,
    )
    _x_py = _x
    return _x_py

def real_toeplitz_yule_update(t, x, v):
    u"""
    Update solution of the Yule--Walker equations for real symmetric
    positive definite Toeplitz matrix.

    For full information please refer to the NAG Library document for
    f04me

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mef.html

    Parameters
    ----------
    t : float, array-like, shape (n+1)
        `t`[0] must contain the value tau_0 of the diagonal elements of
        T, and the remaining n elements of `t` must contain the elements
        of the vector t_n.

    x : float, array-like, shape (n)
        With n > 1 the (n-1) elements of the solution vector x_{n-1} as
        returned by a previous call to ``real_toeplitz_yule_update``.
        The element `x`[n-1] need not be specified.

    v : float
        With n > 1 the mean square prediction error for the (n-1)th
        step, as returned by a previous call to
        ``real_toeplitz_yule_update``.

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x_n. The element `x`[n-1] returns the
        partial (auto)correlation coefficient, or reflection
        coefficient, for the nth step. If |`x`[n-1]| >= 1.0, the matrix
        T_{n+1} will not be positive definite to working accuracy.

    v : float
        The mean square prediction error, or predictor error variance
        ratio, nu_n, for the nth step. (See [Further Comments] and [the
        G13 Introduction].)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, n = *<value>* and `x`[n-2] = *<value>*.

            Constraint: if n > 1, abs(`x`[n-2]) < 1.0.

        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Matrix of order *<value>* would not be positive definite.
            Value of the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_toeplitz_yule_update as b_real_toeplitz_yule_update

    fun_name = 'naginterfaces.library.linsys.real_toeplitz_yule_update'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_real_toeplitz_yule_update(
            _n,
            _t,
            _x,
            v,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _v_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'v'],
    )(
        _x_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_toeplitz_update(t, b, x, work):
    u"""
    Update solution of real symmetric positive definite Toeplitz system.

    For full information please refer to the NAG Library document for
    f04mf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mff.html

    Parameters
    ----------
    t : float, array-like, shape (n)
        `t`[i] must contain the value tau_i, for i = 0,1,...,n-1.

    b : float, array-like, shape (n)
        The right-hand side vector b_n.

    x : float, array-like, shape (n)
        With n > 1 the (n-1) elements of the solution vector x_{n-1} as
        returned by a previous call to ``real_toeplitz_update``. The
        element `x`[n-1] need not be specified.

    work : float, array-like, shape (2*n-1)
        With n > 2 the elements of `work` should be as returned from a
        previous call to ``real_toeplitz_update`` with (n-1) as the
        argument n.

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x_n.

    p : float
        The reflection coefficient p_{n-1}. (See [Further Comments].)

    work : float, ndarray, shape (2*n-1)
        The first (n-1) elements of `work` contain the solution to the
        Yule--Walker equations

            T_{n-1}y_{n-1} = -t_{n-1},

        where t_{n-1} = (tau_1tau_2...tau_{n-1})^`t`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The Toeplitz Matrix is not positive definite Value of the
            reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_toeplitz_update as b_real_toeplitz_update

    fun_name = 'naginterfaces.library.linsys.real_toeplitz_update'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _work = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'work'},
        work,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_real_toeplitz_update(
            _n,
            _t,
            _b,
            _x,
            _work,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _work_py = _work
    _p_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'p', 'work'],
    )(
        _x_py,
        _p_py,
        _work_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_gen_sparse_lsqsol(n, b, aprod, damp=0.0, atol=-1.0, btol=-1.0, conlim=0.0, itnlim=0, msglvl=0, data=None, io_manager=None):
    u"""
    Sparse linear least squares problem, m real equations in n unknowns.

    For full information please refer to the NAG Library document for
    f04qa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04qaf.html

    Parameters
    ----------
    n : int
        n, the number of columns of the matrix A.

    b : float, array-like, shape (m)
        The right-hand side vector b.

    aprod : callable (mode, x, y) = aprod(mode, x, y, data=None)
        `aprod` must perform the operations y := y+Ax and x := x+A^Ty
        for given vectors x and y.

        Parameters
        ~~~~~~~~~~
        mode : int
            Specifies which operation is to be performed.

            `mode` = 1

              `aprod` must compute y+Ax.

            `mode` = 2

              `aprod` must compute x+A^Ty.

        x : float, ndarray, shape (n)
            The vector x.

        y : float, ndarray, shape (m)
            The vector y.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        mode : int
            May be used as a flag to indicate a failure in the
            computation of y+Ax or x+A^Ty. If `mode` is negative on exit
            from `aprod`, ``real_gen_sparse_lsqsol`` will exit
            immediately with errno set to `mode`.

        x : float, array-like, shape (n)
            If `mode` = 1, `x` must be unchanged.

            If `mode` = 2, `x` must contain x+A^Ty.

        y : float, array-like, shape (m)
            If `mode` = 1, `y` must contain y+Ax.

            If `mode` = 2, `y` must be unchanged.

    damp : float, optional
        The value lambda. If either problem (1) or problem (2) is to be
        solved, `damp` must be supplied as zero.

    atol : float, optional
        The tolerance, tol_1, of the convergence criteria (6) and (7);
        it should be an estimate of the largest relative error in the
        elements of A. For example, if the elements of A are correct to
        about 4 significant figures, then `atol` should be set to about
        5*10^(-4). If `atol` is supplied as less than epsilon, where
        epsilon is the machine precision, the value epsilon is used
        instead of `atol`.

    btol : float, optional
        The tolerance, tol_2, of the convergence criterion (6); it
        should be an estimate of the largest relative error in the
        elements of B. For example, if the elements of B are correct to
        about 4 significant figures, then `btol` should be set to about
        5*10^(-4). If `btol` is supplied as less than epsilon, the value
        epsilon is used instead of `btol`.

    conlim : float, optional
        The value c_lim of equation (8); it should be an upper limit on
        the condition number of A_. `conlim` should not normally be
        chosen much larger than 1.0/`atol`. If `conlim` is supplied as
        zero, the value 1.0/epsilon is used instead of `conlim`.

    itnlim : int, optional
        An upper limit on the number of iterations. If `itnlim` <= 0,
        the value `n` is used in place of `itnlim`, but for
        ill-conditioned problems a higher value of `itnlim` is likely to
        be necessary.

    msglvl : int, optional
        The level of printing from ``real_gen_sparse_lsqsol``. If
        `msglvl` <= 0, then no printing occurs, but otherwise messages
        will be output on the file object associated with the advisory
        I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`). A
        description of the printed output is given in [Further
        Comments]. The level of printing is determined as follows:

        `msglvl` <= 0

          No printing.

        `msglvl` = 1

          A brief summary is printed just prior to return from
          ``real_gen_sparse_lsqsol``.

        `msglvl` >= 2

          A summary line is printed periodically to monitor the progress
          of ``real_gen_sparse_lsqsol``, together with a brief summary
          just prior to return from ``real_gen_sparse_lsqsol``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    b : float, ndarray, shape (m)
        This argument is overwritten.

    x : float, ndarray, shape (`n`)
        The solution vector x.

    se : float, ndarray, shape (`n`)
        The estimates of the standard errors of the components of x.
        Thus `se`[i-1] contains an estimate of sqrt(nu_{ii}), where
        nu_{ii} is the ith diagonal element of the estimated
        variance-covariance matrix V. The estimates returned in `se`
        will be the lower bounds on the actual estimated standard
        errors, but will usually be correct to at least one significant
        figure.

    itnlim : int
        Unchanged unless `itnlim` <= 0 on entry, in which case it is set
        to `n`.

    itn : int
        The number of iterations performed.

    anorm : float
        An estimate of ||A_|| for the matrix A_ of (4).

    acond : float
        An estimate of cond(A_) which is a lower bound.

    rnorm : float
        An estimate of ||r_|| for the residual, r_, of (5) corresponding
        to the solution x returned in `x`. Note that ||r_|| is the
        function being minimized.

    arnorm : float
        An estimate of the ||A_^Tr_|| corresponding to the solution x
        returned in `x`.

    xnorm : float
        An estimate of ||x|| for the solution x returned in `x`.

    inform : int
        The reason for termination of ``real_gen_sparse_lsqsol``.

        `inform` = 0

          The exact solution is x = 0. No iterations are performed in
          this case.

        `inform` = 1

          The termination criterion of (6) has been satisfied with tol_1
          and tol_2 as the values supplied in `atol` and `btol`
          respectively.

        `inform` = 2

          The termination criterion of (7) has been satisfied with tol_1
          as the value supplied in `atol`.

        `inform` = 3

          The termination criterion of (6) has been satisfied with tol_1
          and/or tol_2 as the value epsilon, where epsilon is the
          machine precision. One or both of the values supplied in
          `atol` and `btol` must have been less than epsilon and was too
          small for this machine.

        `inform` = 4

          The termination criterion of (7) has been satisfied with tol_1
          as the value epsilon. The value supplied in `atol` must have
          been less than epsilon and was too small for this machine.

        The values `inform` = 5, 6 or 7 correspond to failure with
        `errno` = 2, 3 and 4 respectively (see [Errors]) and when errno
        is negative `inform` will be set to the same negative value.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            Termination criteria not satisfied, but the condition number
            bound supplied in `conlim` has been met.

        (`errno` 3)
            Termination criteria not satisfied, but the condition number
            is bounded by 1/``machine.precision``.

        (`errno` 4)
            Maximum number of iterations reached.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i < 0)
            `mode` in `aprod` has been set to: *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_gen_sparse_lsqsol as b_real_gen_sparse_lsqsol

    fun_name = 'naginterfaces.library.linsys.real_gen_sparse_lsqsol'

    cb_aprod_name = fun_name + ':aprod'

    def cb_aprod(mode, m, n, x, y, data=None):
        """Wrapping user's aprod. Called as base aprod."""
        if data is not None:
            _mode_aprod, _x_aprod, _y_aprod = aprod(
                mode,
                x,
                y,
                data,
            )
        else:
            _mode_aprod, _x_aprod, _y_aprod = aprod(
                mode,
                x,
                y,
            )
        _mode_aprod_base = _mode_aprod
        x[:] = _utils._to_ndarray(
            {'fun_name': cb_aprod_name, 'returned': 'x'},
            _x_aprod,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        y[:] = _utils._to_ndarray(
            {'fun_name': cb_aprod_name, 'returned': 'y'},
            _y_aprod,
            copy=False,
            exp_shape=(m,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        return _mode_aprod_base

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_utils._check_callable(aprod, {'fun_name': fun_name, 'entity': 'aprod'})
    _aprod = cb_aprod
    try:
        _return_scalars = b_real_gen_sparse_lsqsol(
            _m,
            _n,
            _b,
            _x,
            _se,
            _aprod,
            damp,
            atol,
            btol,
            conlim,
            itnlim,
            msglvl,
            data,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _b_py = _b
    _itnlim_py = _return_scalars.itnlim
    _x_py = _x
    _se_py = _se
    _itn_py = _return_scalars.itn
    _anorm_py = _return_scalars.anorm
    _acond_py = _return_scalars.acond
    _rnorm_py = _return_scalars.rnorm
    _arnorm_py = _return_scalars.arnorm
    _xnorm_py = _return_scalars.xnorm
    _inform_py = _return_scalars.inform
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b', 'x', 'se', 'itnlim', 'itn', 'anorm', 'acond', 'rnorm', 'arnorm', 'xnorm', 'inform'],
    )(
        _b_py,
        _x_py,
        _se_py,
        _itnlim_py,
        _itn_py,
        _anorm_py,
        _acond_py,
        _rnorm_py,
        _arnorm_py,
        _xnorm_py,
        _inform_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_gen_lsq_covmat(job, sigma, a, svd, irank, sv):
    u"""
    Covariance matrix for linear least squares problems, m real
    equations in n unknowns.

    For full information please refer to the NAG Library document for
    f04ya

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04yaf.html

    Parameters
    ----------
    job : int
        Specifies which elements of C are required.

        `job` = -1

          The upper triangular part of C is required.

        `job` = 0

          The diagonal elements of C are required.

        `job` > 0

          The elements of column `job` of C are required.

    sigma : float
        The standard error of the residual vector.

    a : float, array-like, shape (:, p)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if not((`svd`) or `job`=-1: p; if `svd`=
        True and `job`>=0: `irank`; otherwise: 0.

        If `svd` = False, `a` must contain the upper triangular matrix U
        of the QU factorization of X, or of the Cholesky factorization
        of X^TX; elements of the array below the diagonal need not be
        set.

        If `svd` = True, A must contain the first k rows of the matrix
        V^T, where k is the rank of X and V is the right-hand orthogonal
        matrix of the singular value decomposition of X.

        Thus the ith row must contain the ith right-hand singular vector
        of X.

    svd : bool
        Must be True if the least squares solution was obtained from a
        singular value decomposition of X. `svd` must be False if the
        least squares solution was obtained from either a QU
        factorization of X or a Cholesky factorization of X^TX. In the
        latter case the rank of X is assumed to be p and so is
        applicable only to full rank problems with n >= p.

    irank : int
        If `svd` = True, `irank` must specify the rank k of the matrix
        X.

        If `svd` = False, `irank` is not referenced and the rank of X is
        assumed to be p.

    sv : float, array-like, shape (p)
        If `svd` = True, `sv` must contain the first `irank` singular
        values of X.

        If `svd` = False, `sv` is not referenced.

    Returns
    -------
    a : float, ndarray, shape (:, p)
        If `job` >= 0, A is unchanged.

        If `job` = -1, `a` contains the upper triangle of the symmetric
        matrix C.

        If `svd` = True, elements of the array below the diagonal are
        used as workspace.

        If `svd` = False, they are unchanged.

    cj : float, ndarray, shape (p)
        If `job` = 0, `cj` returns the diagonal elements of C.

        If `job` = j > 0, `cj` returns the jth column of C.

        If `job` = -1, `cj` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lda = *<value>* and p = *<value>*.

            Constraint: if `svd` = False, lda >= p.

        (`errno` 1)
            On entry, lda = *<value>* and p = *<value>*.

            Constraint: if `svd` = True and `job` = -1, lda >= p.

        (`errno` 1)
            On entry, `job` = *<value>*, lda = *<value>* and `irank` =
            *<value>*.

            Constraint: if `svd` = True and `job` >= 0, lda >= max(1,
            `irank`).

        (`errno` 1)
            On entry, `irank` = *<value>* and p = *<value>*.

            Constraint: if `svd` = True, `irank` >= 0 and `irank` <= p.

        (`errno` 1)
            On entry, `job` = *<value>* and p = *<value>*.

            Constraint: `job` >= -1 and `job` <= p.

        (`errno` 1)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` >= 0.0.

        (`errno` 1)
            On entry, p = *<value>*.

            Constraint: p >= 1.

        (`errno` 2)
            On entry, `irank` = *<value>*.

            Constraint: if `svd` = True, `irank` > 0.

        (`errno` 3)
            On entry, `svd` = False and overflow would occur in
            calculating an element of C. The upper triangular matrix U
            must be very nearly singular.

        (`errno` 4)
            On entry, `svd` = False and one of the first `irank`
            singular values is zero. Either the first `irank` singular
            values or `irank` must be incorrect: `irank` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.linsys import real_gen_lsq_covmat as b_real_gen_lsq_covmat

    fun_name = 'naginterfaces.library.linsys.real_gen_lsq_covmat'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _sv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'sv'},
        sv,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _cj = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_p_py,),
    )
    b_real_gen_lsq_covmat(
        job,
        _p,
        sigma,
        _a,
        svd,
        irank,
        _sv,
        _cj,
    )
    _a_py = _a
    _cj_py = _cj
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'cj'],
    )(
        _a_py,
        _cj_py,
    )
    return _return_data

def real_gen_norm_rcomm(irevcm, x, y, estnrm, seed, comm):
    u"""
    Norm estimation (for use in condition estimation), real rectangular
    matrix.

    For full information please refer to the NAG Library document for
    f04yd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04ydf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

        On intermediate entry: `irevcm` must be unchanged.

    x : float, ndarray, shape (n, t), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 1, contains the current
        matrix X.

        On intermediate entry: if `irevcm` = 2, must contain A^TY.

        On final exit: the array is undefined.

    y : float, ndarray, shape (m, t), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current
        matrix Y.

        On intermediate entry: if `irevcm` = 1, must contain AX.

        On final exit: the array is undefined.

    estnrm : float
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    seed : int
        The seed used for random number generation.

        If t = 1, `seed` is not used.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1 or 2, and `x` contains the
        n*t matrix X and `y` contains the m*t matrix Y. The calling
        program must

        - if `irevcm` = 1, evaluate AX and store the result in `y`

          or

          if `irevcm` = 2, evaluate A^TY and store the result in `x`,

        - call ``real_gen_norm_rcomm`` once again, with all the other
          arguments unchanged.

        On final exit: `irevcm` = 0.

    estnrm : float
        On final exit: an estimate (a lower bound) for ||A||_1.

    Raises
    ------
    NagValueError
        (`errno` -10)
            On entry, t = *<value>* and `seed` = *<value>*.

            Constraint: if t > 1, `seed` >= 1.

        (`errno` -9)
            On entry, m = *<value>* and t = *<value>*.

            Constraint: 1 <= t <= m.

        (`errno` -3)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, m = *<value>*.

            Constraint: m >= 0.

        (`errno` -1)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            Internal error; please contact NAG.
    """

    from ..base.linsys import real_gen_norm_rcomm as b_real_gen_norm_rcomm

    fun_name = 'naginterfaces.library.linsys.real_gen_norm_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _t_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _t = _b_utils._EngineIntScalarType(
        _t_py, {'fun_name': fun_name, 'entity': '_t'},
    ).to_ctypes()
    _return_scalars = b_real_gen_norm_rcomm(
        irevcm,
        _m,
        _n,
        x,
        y,
        estnrm,
        _t,
        seed,
        comm,
    )
    _irevcm_py = _return_scalars.irevcm
    _estnrm_py = _return_scalars.estnrm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'estnrm'],
    )(
        _irevcm_py,
        _estnrm_py,
    )
    return _return_data

def complex_gen_norm_rcomm(irevcm, x, y, estnrm, seed, comm):
    u"""
    Norm estimation (for use in condition estimation), complex
    rectangular matrix.

    For full information please refer to the NAG Library document for
    f04zd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04zdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

        On intermediate entry: `irevcm` must be unchanged.

    x : complex, ndarray, shape (n, t), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 1, contains the current
        matrix X.

        On intermediate entry: if `irevcm` = 2, must contain A^HY.

        On final exit: the array is undefined.

    y : complex, ndarray, shape (m, t), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current
        matrix Y.

        On intermediate entry: if `irevcm` = 1, must contain AX.

        On final exit: the array is undefined.

    estnrm : float
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    seed : int
        The seed used for random number generation.

        If t = 1, `seed` is not used.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1 or 2, and `x` contains the
        n*t matrix X and `y` contains the m*t matrix Y. The calling
        program must

        - if `irevcm` = 1, evaluate AX and store the result in `y`

          or

          if `irevcm` = 2, evaluate A^HY and store the result in `x`,
          where A^H is the complex conjugate transpose;

        - call ``complex_gen_norm_rcomm`` once again, with all the
          arguments unchanged.

        On final exit: `irevcm` = 0.

    estnrm : float
        On final exit: an estimate (a lower bound) for ||A||_1.

    Raises
    ------
    NagValueError
        (`errno` -10)
            On entry, t = *<value>* and `seed` = *<value>*.

            Constraint: if t > 1, `seed` >= 1.

        (`errno` -9)
            On entry, m = *<value>* and t = *<value>*.

            Constraint: 1 <= t <= m.

        (`errno` -3)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, m = *<value>*.

            Constraint: m >= 0.

        (`errno` -1)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            Internal error; please contact NAG.
    """

    from ..base.linsys import complex_gen_norm_rcomm as b_complex_gen_norm_rcomm

    fun_name = 'naginterfaces.library.linsys.complex_gen_norm_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _t_py = _b_utils._EngineComplex128ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _t = _b_utils._EngineIntScalarType(
        _t_py, {'fun_name': fun_name, 'entity': '_t'},
    ).to_ctypes()
    _return_scalars = b_complex_gen_norm_rcomm(
        irevcm,
        _m,
        _n,
        x,
        y,
        estnrm,
        _t,
        seed,
        comm,
    )
    _irevcm_py = _return_scalars.irevcm
    _estnrm_py = _return_scalars.estnrm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'estnrm'],
    )(
        _irevcm_py,
        _estnrm_py,
    )
    return _return_data
