# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `mesh` Chapter.

``mesh`` - Mesh Generation

This module is concerned with automatic mesh generation

- with line segments, over the boundary of a closed two-dimensional
  connected polygonal domain;

- with triangles, over a given two-dimensional region using only its
  boundary mesh.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def dim2_gen_inc(edge, coor, bspace, smooth, itrace, coef=0.75, power=0.25, io_manager=None):
    u"""
    Generates a two-dimensional mesh using a simple incremental method.

    For full information please refer to the NAG Library document for
    d06aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06aaf.html

    Parameters
    ----------
    edge : int, array-like, shape (3, nedge)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by ``dim2_gen_inc``.

    coor : float, array-like, shape (2, nvmax)
        `coor`[0,i-1] contains the x coordinate of the ith input
        boundary mesh vertex; while `coor`[1,i-1] contains the
        corresponding y coordinate, for i = 1,...,nvb.

    bspace : float, array-like, shape (nvb)
        The desired mesh spacing (triangle diameter, which is the length
        of the longer edge of the triangle) near the boundary vertices.

    smooth : bool
        Indicates whether or not mesh smoothing should be performed.

        If `smooth` = True, the smoothing is performed; otherwise no
        smoothing is performed.

    itrace : int
        The level of trace information required from ``dim2_gen_inc``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    coef : float, optional
        The coefficient in the stopping criteria for the generation of
        interior vertices. This argument controls the triangle density
        and the number of triangles generated is in O(`coef`^2). The
        mesh will be finer if `coef` is greater than 0.7165 and 0.75 is
        a good value.

    power : float, optional
        Controls the rate of change of the mesh size during the
        generation of interior vertices. The smaller the value of
        `power`, the faster the decrease in element size away from the
        boundary.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If nvb = nvmax, no interior
        vertices will be generated and `nv` = nvb.

    nelt : int
        The number of triangular elements in the mesh.

    coor : float, ndarray, shape (2, nvmax)
        `coor`[0,i-1] will contain the x coordinate of the (i-nvb)th
        generated interior mesh vertex; while `coor`[1,i-1] will contain
        the corresponding y coordinate, for i = nvb+1,...,`nv`. The
        remaining elements are unchanged.

    conn : int, ndarray, shape (3, 2*(nvmax-1))
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,...,`nelt`, for i =
        1,2,...,3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nvb = *<value>* and nvmax = *<value>*.

            Constraint: 3 <= nvb <= nvmax.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nvb = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= nvb.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `bspace`[I-1] = *<value>* and I = *<value>*.

            Constraint: `bspace`[I-1] > 0.0.

        (`errno` 1)
            On entry, `power` = *<value>*.

            Constraint: `power` <= 10.0.

        (`errno` 1)
            On entry, `power` = *<value>*.

            Constraint: `power` >= 0.1.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.
    """

    from ..base.mesh import dim2_gen_inc as b_dim2_gen_inc

    fun_name = 'naginterfaces.library.mesh.dim2_gen_inc'

    _edge = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge'},
        edge,
        copy=False,
    )
    _coor = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor'},
        coor,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_edge, 'edge'), (_coor, 'coor')),
    )
    _bspace = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bspace'},
        bspace,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nvb_py = _b_utils._EngineFloat64ArrayType.get_shape(_bspace, {'fun_name': fun_name, 'entity': 'bspace'}, exp_rank=1)[0]
    _nvb = _b_utils._EngineIntScalarType(
        _nvb_py, {'fun_name': fun_name, 'entity': '_nvb'},
    ).to_ctypes()
    _nvmax_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor, {'fun_name': fun_name, 'entity': 'coor'}, exp_rank=2)[1]
    _nvmax = _b_utils._EngineIntScalarType(
        _nvmax_py, {'fun_name': fun_name, 'entity': '_nvmax'},
    ).to_ctypes()
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edge, {'fun_name': fun_name, 'entity': 'edge'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _conn = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, 2*(_nvmax_py-1)),
        sorder=_m_sorder,
    )
    _return_scalars = b_dim2_gen_inc(
        _nvb,
        _nvmax,
        _nedge,
        _edge,
        _coor,
        _conn,
        _bspace,
        smooth,
        coef,
        power,
        itrace,
        io_manager,
    )
    _coor_py = _coor
    _nv_py = _return_scalars.nv
    _nelt_py = _return_scalars.nelt
    _conn_py = _conn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nv', 'nelt', 'coor', 'conn'],
    )(
        _nv_py,
        _nelt_py,
        _coor_py,
        _conn_py,
    )
    return _return_data

def dim2_gen_delaunay(nvb, edge, coor, weight, npropa, itrace, io_manager=None):
    u"""
    Generates a two-dimensional mesh using a Delaunay--Voronoi process.

    For full information please refer to the NAG Library document for
    d06ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06abf.html

    Parameters
    ----------
    nvb : int
        The number of vertices in the input boundary mesh.

    edge : int, array-like, shape (3, nedge)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by
        ``dim2_gen_delaunay``.

    coor : float, array-like, shape (2, nvmax)
        `coor`[0,i-1] contains the x coordinate of the ith input
        boundary mesh vertex, for i = 1,...,`nvb`. `coor`[0,i-1]
        contains the x coordinate of the (i-`nvb`)th fixed interior
        vertex, for i = `nvb`+1,...,`nvb`+nvint. For boundary and
        interior vertices, `coor`[1,i-1] contains the corresponding y
        coordinate, for i = 1,2,...,`nvb`+nvint.

    weight : float, array-like, shape (nvint)
        The weight of fixed interior vertices. It is the diameter of
        triangles (length of the longer edge) created around each of the
        given interior vertices.

    npropa : int
        The propagation type and coefficient, the argument `npropa` is
        used when the internal points are created. They are distributed
        in a geometric manner if `npropa` is positive and in an
        arithmetic manner if it is negative. For more details see
        [Further Comments].

    itrace : int
        The level of trace information required from
        ``dim2_gen_delaunay``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If `nvb`+nvint = nvmax, no
        interior vertices will be generated and `nv` = nvmax.

    nelt : int
        The number of triangular elements in the mesh.

    coor : float, ndarray, shape (2, nvmax)
        `coor`[0,i-1] will contain the x coordinate of the
        (i-`nvb`-nvint)th generated interior mesh vertex, for i =
        `nvb`+nvint+1,...,`nv`; while `coor`[1,i-1] will contain the
        corresponding y coordinate. The remaining elements are
        unchanged.

    conn : int, ndarray, shape (3, 2*nvmax+5)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,...,`nelt`, for i =
        1,2,...,3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nvb` = *<value>*, nvint = *<value>* and nvmax =
            *<value>*.

            Constraint: nvmax >= `nvb`+nvint.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nvb` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nvb`.

        (`errno` 1)
            On entry, `nvb` = *<value>*.

            Constraint: `nvb` >= 3.

        (`errno` 1)
            On entry, nvint = *<value>*.

            Constraint: nvint >= 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `weight`[I-1] = *<value>* and I = *<value>*.

            Constraint: `weight`[I-1] > 0.0.

        (`errno` 1)
            On entry, `npropa` = 0.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.

        (`errno` 3)
            An error has occurred during the generation of the boundary
            mesh. It appears that nvmax is not large enough: nvmax =
            *<value>*.
    """

    from ..base.mesh import dim2_gen_delaunay as b_dim2_gen_delaunay

    fun_name = 'naginterfaces.library.mesh.dim2_gen_delaunay'

    _edge = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge'},
        edge,
        copy=False,
    )
    _coor = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor'},
        coor,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_edge, 'edge'), (_coor, 'coor')),
    )
    _weight = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'weight'},
        weight,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nvint_py = _b_utils._EngineFloat64ArrayType.get_shape(_weight, {'fun_name': fun_name, 'entity': 'weight'}, exp_rank=1)[0]
    _nvint = _b_utils._EngineIntScalarType(
        _nvint_py, {'fun_name': fun_name, 'entity': '_nvint'},
    ).to_ctypes()
    _nvmax_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor, {'fun_name': fun_name, 'entity': 'coor'}, exp_rank=2)[1]
    _nvmax = _b_utils._EngineIntScalarType(
        _nvmax_py, {'fun_name': fun_name, 'entity': '_nvmax'},
    ).to_ctypes()
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edge, {'fun_name': fun_name, 'entity': 'edge'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _conn = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, 2*_nvmax_py+5),
        sorder=_m_sorder,
    )
    _return_scalars = b_dim2_gen_delaunay(
        nvb,
        _nvint,
        _nvmax,
        _nedge,
        _edge,
        _coor,
        _conn,
        _weight,
        npropa,
        itrace,
        io_manager,
    )
    _coor_py = _coor
    _nv_py = _return_scalars.nv
    _nelt_py = _return_scalars.nelt
    _conn_py = _conn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nv', 'nelt', 'coor', 'conn'],
    )(
        _nv_py,
        _nelt_py,
        _coor_py,
        _conn_py,
    )
    return _return_data

def dim2_gen_front(nvb, edge, coor, weight, itrace, io_manager=None):
    u"""
    Generates a two-dimensional mesh using an Advancing-front method.

    For full information please refer to the NAG Library document for
    d06ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06acf.html

    Parameters
    ----------
    nvb : int
        The number of vertices in the input boundary mesh.

    edge : int, array-like, shape (3, nedge)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] contain the vertex numbers of the two end points
        of the jth boundary edge. `edge`[2,j-1] is a user-supplied tag
        for the jth boundary edge and is not used by ``dim2_gen_front``.

    coor : float, array-like, shape (2, nvmax)
        `coor`[0,i-1] contains the x coordinate of the ith input
        boundary mesh vertex, for i = 1,...,`nvb`. `coor`[0,i-1]
        contains the x coordinate of the (i-`nvb`)th fixed interior
        vertex, for i = `nvb`+1,...,`nvb`+nvint. For boundary and
        interior vertices, `coor`[1,i-1] contains the corresponding y
        coordinate, for i = 1,2,...,`nvb`+nvint.

    weight : float, array-like, shape (nvint)
        The weight of fixed interior vertices. It is the diameter of
        triangles (length of the longer edge) created around each of the
        given interior vertices.

    itrace : int
        The level of trace information required from ``dim2_gen_front``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Output from the meshing solver is printed. This output
          contains details of the vertices and triangles generated by
          the process.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv : int
        The total number of vertices in the output mesh (including both
        boundary and interior vertices). If `nvb`+nvint = nvmax, no
        interior vertices will be generated and `nv` = nvmax.

    nelt : int
        The number of triangular elements in the mesh.

    coor : float, ndarray, shape (2, nvmax)
        `coor`[0,i-1] will contain the x coordinate of the
        (i-`nvb`-nvint)th generated interior mesh vertex, for i =
        `nvb`+nvint+1,...,`nv`; while `coor`[1,i-1] will contain the
        corresponding y coordinate. The remaining elements are
        unchanged.

    conn : int, ndarray, shape (3, 2*nvmax+5)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,...,`nelt`, for i =
        1,2,...,3.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nv` = *<value>*, nvint = *<value>* and nvmax =
            *<value>*.

            Constraint: `nvb`+nvint <= nvmax.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nvb` = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= `nvb`.

        (`errno` 1)
            On entry, `nvb` = *<value>*.

            Constraint: `nvb` >= 3.

        (`errno` 1)
            On entry, nvint = *<value>*.

            Constraint: nvint >= 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `weight`[I-1] = *<value>* and I = *<value>*.

            Constraint: `weight`[I-1] > 0.0.

        (`errno` 2)
            An error has occurred during the generation of the interior
            mesh. Check the definition of the boundary (arguments `coor`
            and `edge`) as well as the orientation of the boundary
            (especially in the case of a multiple connected component
            boundary). Setting `itrace` > 0 may provide more details.
    """

    from ..base.mesh import dim2_gen_front as b_dim2_gen_front

    fun_name = 'naginterfaces.library.mesh.dim2_gen_front'

    _edge = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge'},
        edge,
        copy=False,
    )
    _coor = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor'},
        coor,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_edge, 'edge'), (_coor, 'coor')),
    )
    _weight = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'weight'},
        weight,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nvint_py = _b_utils._EngineFloat64ArrayType.get_shape(_weight, {'fun_name': fun_name, 'entity': 'weight'}, exp_rank=1)[0]
    _nvint = _b_utils._EngineIntScalarType(
        _nvint_py, {'fun_name': fun_name, 'entity': '_nvint'},
    ).to_ctypes()
    _nvmax_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor, {'fun_name': fun_name, 'entity': 'coor'}, exp_rank=2)[1]
    _nvmax = _b_utils._EngineIntScalarType(
        _nvmax_py, {'fun_name': fun_name, 'entity': '_nvmax'},
    ).to_ctypes()
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edge, {'fun_name': fun_name, 'entity': 'edge'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _conn = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, 2*_nvmax_py+5),
        sorder=_m_sorder,
    )
    _return_scalars = b_dim2_gen_front(
        nvb,
        _nvint,
        _nvmax,
        _nedge,
        _edge,
        _coor,
        _conn,
        _weight,
        itrace,
        io_manager,
    )
    _coor_py = _coor
    _nv_py = _return_scalars.nv
    _nelt_py = _return_scalars.nelt
    _conn_py = _conn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nv', 'nelt', 'coor', 'conn'],
    )(
        _nv_py,
        _nelt_py,
        _coor_py,
        _conn_py,
    )
    return _return_data

def dim2_gen_boundary(coorch, lined, crus, rate, nlcomp, lcomp, nvmax, nedmx, itrace, fbnd=None, data=None, io_manager=None):
    u"""
    Generates a boundary mesh.

    For full information please refer to the NAG Library document for
    d06ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06baf.html

    Parameters
    ----------
    coorch : float, array-like, shape (2, nlines)
        `coorch`[0,i-1] contains the x coordinate of the ith
        characteristic point, for i = 1,2,...,nlines; while
        `coorch`[1,i-1] contains the corresponding y coordinate.

    lined : int, array-like, shape (4, nlines)
        The description of the lines that define the boundary domain.
        The line i, for i = 1,...,m, is defined as follows:

        `lined`[0,i-1]

          The number of points on the line, including two end points.

        `lined`[1,i-1]

          The first end point of the line. If `lined`[1,i-1] = j, the
          coordinates of the first end point are those stored in
          `coorch`[0,j-1],`coorch`[1,j-1].

        `lined`[2,i-1]

          The second end point of the line. If `lined`[2,i-1] = k, the
          coordinates of the second end point are those stored in
          `coorch`[0,k-1],`coorch`[1,k-1].

        `lined`[3,i-1]

          This defines the type of line segment connecting the end
          points. Additional information is conveyed by the numerical
          value of `lined`[3,i-1] as follows:

          - `lined`[3,i-1] > 0, the line is described in `fbnd` with
            `lined`[3,i-1] as the index. In this case, the line must be
            described in the trigonometric (anticlockwise) direction;

          - `lined`[3,i-1] = 0, the line is a straight line;

          - if `lined`[3,i-1] < 0, say (-p), then the line is a
            polygonal arc joining the end points and interior points
            specified in `crus`. In this case the line contains the
            points whose coordinates are stored in `coorch`[z,j-1],
            `crus`[z,p-1], `crus`[z,p],...,`crus`[z,p+r-4],
            `coorch`[z,k-1], where z in {0,1}, r = `lined`[0,i-1], j =
            `lined`[1,i-1] and k = `lined`[2,i-1].

    crus : float, array-like, shape (2, sdcrus)
        The coordinates of the intermediate points for polygonal arc
        lines. For a line i defined as a polygonal arc (i.e.,
        `lined`[3,i-1] < 0), if p = -`lined`[3,i-1], `crus`[0,k-1], for
        k = p,...,p+`lined`[0,i-1]-3, must contain the x coordinate of
        the consecutive intermediate points for this line. Similarly
        `crus`[1,k-1], for k = p,...,p+`lined`[0,i-1]-3, must contain
        the corresponding y coordinate.

    rate : float, array-like, shape (nlines)
        `rate`[i-1] is the geometric progression ratio between the
        points to be generated on the line i, for i = 1,2,...,m and
        `lined`[3,i-1] >= 0.

        If `lined`[3,i-1] < 0, `rate`[i-1] is not referenced.

    nlcomp : int, array-like, shape (ncomp)
        |`nlcomp`[k-1]| is the number of line segments in component k of
        the contour. The line i of component k runs in the direction
        `lined`[1,i-1] to `lined`[2,i-1] if `nlcomp`[k-1] > 0, and in
        the opposite direction otherwise; for k = 1,2,...,n.

    lcomp : int, array-like, shape (nlines)
        `lcomp` must contain the list of line numbers for the each
        component of the boundary. Specifically, the line numbers for
        the kth component of the boundary, for k = 1,2,...,ncomp, must
        be in elements l1-1 to l2-1 of `lcomp`, where l2 = sum_i =
        1^k|`nlcomp`[i-1]| and l1 = l2+1-|`nlcomp`[k-1]|.

    nvmax : int
        The maximum number of the boundary mesh vertices to be
        generated.

    nedmx : int
        The maximum number of boundary edges in the boundary mesh to be
        generated.

    itrace : int
        The level of trace information required from
        ``dim2_gen_boundary``.

        `itrace` = 0 or `itrace` < -1

          No output is generated.

        `itrace` = 1

          Output from the boundary mesh generator is printed. This
          output contains the input information of each line and each
          connected component of the boundary.

        `itrace` = -1

          An analysis of the output boundary mesh is printed on the file
          object associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`). This
          analysis includes the orientation (clockwise or anticlockwise)
          of each connected component of the boundary. This information
          could be of interest to you, especially if an interior meshing
          is carried out using the output of this function, calling
          either ``dim2_gen_inc``, ``dim2_gen_delaunay`` or
          ``dim2_gen_front``.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1, but
          the coordinates of the generated vertices on the boundary are
          also output.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element mesh generation.

    fbnd : None or callable retval = fbnd(i, x, y, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `fbnd` must be supplied to calculate the value of the function
        which describes the curve {(x,y) in R^2; such that f(x,y) = 0}
        on segments of the boundary for which `lined`[3,i-1] > 0.

        Parameters
        ~~~~~~~~~~
        i : int
            `lined`[3,i-1], the reference index of the line (portion of
            the contour) i described.

        x : float
            The values of x and y at which f(x,y) is to be evaluated.

        y : float
            The values of x and y at which f(x,y) is to be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x,y) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nvb : int
        The total number of boundary mesh vertices generated.

    coor : float, ndarray, shape (2, `nvmax`)
        `coor`[0,i-1] will contain the x coordinate of the ith boundary
        mesh vertex generated, for i = 1,...,`nvb`; while `coor`[1,i-1]
        will contain the corresponding y coordinate.

    nedge : int
        The total number of boundary edges in the boundary mesh.

    edge : int, ndarray, shape (3, `nedmx`)
        The specification of the boundary edges. `edge`[0,j-1] and
        `edge`[1,j-1] will contain the vertex numbers of the two end
        points of the jth boundary edge. `edge`[2,j-1] is a reference
        number for the jth boundary edge and

        - `edge`[2,j-1] = `lined`[3,i-1], where i and j are such that
          the jth edges is part of the ith line of the boundary and
          `lined`[3,i-1] >= 0;

        - `edge`[2,j-1] = 100+|`lined`[3,i-1]|, where i and j are such
          that the jth edges is part of the ith line of the boundary and
          `lined`[3,i-1] < 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, the indices of the extremities of line *<value>*
            are both equal to *<value>*.

        (`errno` 1)
            On entry, the sum of absolute values of all numbers of line
            segments is different from nlines. The sum of all the
            elements of `nlcomp` = *<value>*. nlines = *<value>*.

        (`errno` 1)
            On entry, ncomp = *<value>*.

            Constraint: ncomp >= 1.

        (`errno` 1)
            On entry, nlines = *<value>*.

            Constraint: nlines >= 1.

        (`errno` 1)
            On entry, the geometric progression ratio between the points
            to be generated on line *<value>* is *<value>*. It should be
            greater than 0 unless the line segment is defined by
            user-supplied points.

        (`errno` 1)
            On entry, the index of the second end point of line
            *<value>* is *<value>*. It should be greater than or equal
            to 1 and less than or equal to nlines = *<value>*.

        (`errno` 1)
            On entry, the index of the first end point of line *<value>*
            is *<value>*. It should be greater than or equal to 1 and
            less than or equal to nlines = *<value>*.

        (`errno` 1)
            On entry, the number of points on line *<value>* is
            *<value>*. It should be greater than or equal to 2.

        (`errno` 1)
            On entry, `nvmax` = *<value>* and nlines = *<value>*.

            Constraint: `nvmax` >= nlines.

        (`errno` 1)
            On entry, `nedmx` = *<value>*.

            Constraint: `nedmx` >= 1.

        (`errno` 1)
            On entry, the line list for the separate connected component
            of the boundary is badly set: `lcomp`[l-1] = *<value>* and l
            = *<value>*. It should be less than or equal to nlines and
            greater than or equal to 1.

        (`errno` 1)
            On entry, end point 1, with index K, does not lie on the
            curve representing line I with index J: K = *<value>*, I =
            *<value>*, J = *<value>*, f(x,y) = *<value>*.

        (`errno` 1)
            On entry, end point 2, with index K, does not lie on the
            curve representing line I with index J: K = *<value>*, I =
            *<value>*, J = *<value>*, f(x,y) = *<value>*.

        (`errno` 1)
            On entry, sdcrus = *<value>* and nusmin = *<value>*.

            Constraint: sdcrus >= nusmin.

        (`errno` 1)
            On entry, there is a correlation problem between the
            user-supplied coordinates and the specification of the
            polygonal arc representing line I = *<value>* with the index
            in `crus` = *<value>*.

        (`errno` 1)
            On entry, the absolute number of line segments in the kth
            component of the contour should be less than or equal to
            nlines and greater than 0. k = *<value>*, `nlcomp`[k-1] =
            *<value>* and nlines = *<value>*.

        (`errno` 2)
            An error has occurred during the generation of the boundary
            mesh. It appears that `nedmx` is not large enough: `nedmx` =
            *<value>*.

        (`errno` 3)
            An error has occurred during the generation of the boundary
            mesh. It appears that `nvmax` is not large enough: `nvmax` =
            *<value>*.

        (`errno` 4)
            An error has occurred during the generation of the boundary
            mesh. Check the definition of each line (the argument
            `lined`) and each connected component of the boundary (the
            arguments `nlcomp`, and `lcomp`, as well as the coordinates
            of the characteristic points. Setting `itrace` > 0 may
            provide more details.
    """

    from ..base.mesh import dim2_gen_boundary as b_dim2_gen_boundary

    def cb_fbnd(i, x, y, data=None):
        """Wrapping user's fbnd. Called as base fbnd."""
        if data is not None:
            _retval_fbnd = fbnd(
                i,
                x,
                y,
                data,
            )
        else:
            _retval_fbnd = fbnd(
                i,
                x,
                y,
            )
        return _retval_fbnd

    fun_name = 'naginterfaces.library.mesh.dim2_gen_boundary'

    _coorch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coorch'},
        coorch,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lined = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lined'},
        lined,
        copy=False,
    )
    _crus = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'crus'},
        crus,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_coorch, 'coorch'), (_lined, 'lined'), (_crus, 'crus')),
    )
    _nlines_py = _b_utils._EngineFloat64ArrayType.get_shape(_coorch, {'fun_name': fun_name, 'entity': 'coorch'}, exp_rank=2)[1]
    _nlines = _b_utils._EngineIntScalarType(
        _nlines_py, {'fun_name': fun_name, 'entity': '_nlines'},
    ).to_ctypes()
    _b_utils._check_callable(fbnd, {'fun_name': fun_name, 'entity': 'fbnd'}, allow_none=True)
    _fbnd = None if fbnd is None else cb_fbnd
    _sdcrus_py = _b_utils._EngineFloat64ArrayType.get_shape(_crus, {'fun_name': fun_name, 'entity': 'crus'}, exp_rank=2)[1]
    _sdcrus = _b_utils._EngineIntScalarType(
        _sdcrus_py, {'fun_name': fun_name, 'entity': '_sdcrus'},
    ).to_ctypes()
    _rate = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rate'},
        rate,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nlcomp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nlcomp'},
        nlcomp,
        copy=False,
    )
    _ncomp_py = _b_utils._EngineIntArrayType.get_shape(_nlcomp, {'fun_name': fun_name, 'entity': 'nlcomp'}, exp_rank=1)[0]
    _ncomp = _b_utils._EngineIntScalarType(
        _ncomp_py, {'fun_name': fun_name, 'entity': '_ncomp'},
    ).to_ctypes()
    _lcomp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lcomp'},
        lcomp,
        copy=False,
    )
    _nvmax = _b_utils._EngineIntScalarType(
        nvmax, {'fun_name': fun_name, 'entity': 'nvmax'},
    ).to_ctypes()
    _nvmax_py = _b_utils._EngineIntScalarType.to_py(_nvmax)
    _coor = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2, _nvmax_py),
        sorder=_m_sorder,
    )
    _nedmx = _b_utils._EngineIntScalarType(
        nedmx, {'fun_name': fun_name, 'entity': 'nedmx'},
    ).to_ctypes()
    _nedmx_py = _b_utils._EngineIntScalarType.to_py(_nedmx)
    _edge = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, _nedmx_py),
        sorder=_m_sorder,
    )
    _return_scalars = b_dim2_gen_boundary(
        _nlines,
        _coorch,
        _lined,
        _fbnd,
        _crus,
        _sdcrus,
        _rate,
        _ncomp,
        _nlcomp,
        _lcomp,
        _nvmax,
        _nedmx,
        _coor,
        _edge,
        itrace,
        data,
        io_manager,
    )
    _nvb_py = _return_scalars.nvb
    _coor_py = _coor
    _nedge_py = _return_scalars.nedge
    _edge_py = _edge
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nvb', 'coor', 'nedge', 'edge'],
    )(
        _nvb_py,
        _coor_py,
        _nedge_py,
        _edge_py,
    )
    return _return_data

def dim2_smooth_bary(coor, edge, conn, numfix, itrace, nqint, io_manager=None):
    u"""
    Uses a barycentering technique to smooth a given mesh.

    For full information please refer to the NAG Library document for
    d06ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06caf.html

    Parameters
    ----------
    coor : float, array-like, shape (2, nv)
        `coor`[0,i-1] contains the x coordinate of the ith input mesh
        vertex, for i = 1,2,...,nv; while `coor`[1,i-1] contains the
        corresponding y coordinate.

    edge : int, array-like, shape (3, nedge)
        The specification of the boundary or interface edges.
        `edge`[0,j-1] and `edge`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge`[2,j-1] is a
        user-supplied tag for the jth boundary or interface edge:
        `edge`[2,j-1] = 0 for an interior edge and has a nonzero tag
        otherwise.

    conn : int, array-like, shape (3, nelt)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,nelt, for i =
        1,2,...,3.

    numfix : int, array-like, shape (nvfix)
        The indices in `coor` of fixed interior vertices of the input
        mesh.

    itrace : int
        The level of trace information required from
        ``dim2_smooth_bary``.

        `itrace` <= 0

          No output is generated.

        `itrace` = 1

          A histogram of the triangular element qualities is printed on
          the file object associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) before and
          after smoothing. This histogram gives the lowest and the
          highest triangle quality as well as the number of elements
          lying in each of the `nqint` equal intervals between the
          extremes.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1 but
          the connectivity between vertices and triangles (for each
          vertex, the list of triangles in which it appears) is given.

        You are advised to set `itrace` = 0, unless you are experienced
        with finite element meshes.

    nqint : int
        The number of intervals between the extreme quality values for
        the input and the smoothed mesh.

        If `itrace` = 0, `nqint` is not referenced.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    coor : float, ndarray, shape (2, nv)
        `coor`[0,i-1] will contain the x coordinate of the ith smoothed
        mesh vertex, for i = 1,2,...,nv; while `coor`[1,i-1] will
        contain the corresponding y coordinate. Note that the
        coordinates of boundary and interface edge vertices, as well as
        those specified by you (see the description of `numfix`), are
        unchanged by the process.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nv = *<value>*.

            Constraint: nv >= 3.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= nv.

        (`errno` 1)
            On entry, nelt = *<value>* and nv = *<value>*.

            Constraint: nelt <= 2*nv-1.

        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= nv.

        (`errno` 1)
            On entry, `numfix`[I-1] = *<value>*, I = *<value>* and nv =
            *<value>*.

            Constraint: `numfix`[I-1] >= 1 and `numfix`[I-1] <= nv.

        (`errno` 1)
            On entry, nv = *<value>* and nvfix = *<value>*.

            Constraint: 0 <= nvfix <= nv.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh especially the
            connectivity. Seek expert help.
    """

    from ..base.mesh import dim2_smooth_bary as b_dim2_smooth_bary

    fun_name = 'naginterfaces.library.mesh.dim2_smooth_bary'

    _coor = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor'},
        coor,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nv_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor, {'fun_name': fun_name, 'entity': 'coor'}, exp_rank=2)[1]
    _nv = _b_utils._EngineIntScalarType(
        _nv_py, {'fun_name': fun_name, 'entity': '_nv'},
    ).to_ctypes()
    _conn = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conn'},
        conn,
        copy=False,
    )
    _nelt_py = _b_utils._EngineIntArrayType.get_shape(_conn, {'fun_name': fun_name, 'entity': 'conn'}, exp_rank=2)[1]
    _nelt = _b_utils._EngineIntScalarType(
        _nelt_py, {'fun_name': fun_name, 'entity': '_nelt'},
    ).to_ctypes()
    _edge = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge'},
        edge,
        copy=False,
    )
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edge, {'fun_name': fun_name, 'entity': 'edge'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _numfix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'numfix'},
        numfix,
        copy=False,
    )
    _nvfix_py = _b_utils._EngineIntArrayType.get_shape(_numfix, {'fun_name': fun_name, 'entity': 'numfix'}, exp_rank=1)[0]
    _nvfix = _b_utils._EngineIntScalarType(
        _nvfix_py, {'fun_name': fun_name, 'entity': '_nvfix'},
    ).to_ctypes()
    b_dim2_smooth_bary(
        _nv,
        _nelt,
        _nedge,
        _coor,
        _edge,
        _conn,
        _nvfix,
        _numfix,
        itrace,
        nqint,
        io_manager,
    )
    _coor_py = _coor
    return _coor_py

def dim2_sparsity(nv, nnzmax, conn):
    u"""
    Generates a sparsity pattern of a Finite Element matrix associated
    with a given mesh.

    For full information please refer to the NAG Library document for
    d06cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06cbf.html

    Parameters
    ----------
    nv : int
        The total number of vertices in the input mesh.

    nnzmax : int
        The maximum number of nonzero entries in the matrix based on the
        input mesh. It is the dimension of the arrays `irow` and `icol`
        as declared in the function from which ``dim2_sparsity`` is
        called.

    conn : int, array-like, shape (3, nelt)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,nelt, for i =
        1,2,...,3.

    Returns
    -------
    nnz : int
        The number of nonzero entries in the matrix associated with the
        input mesh.

    irow : int, ndarray, shape (`nnzmax`)
        The first `nnz` elements contain the row and column indices of
        the nonzero elements supplied in the finite element matrix A.

    icol : int, ndarray, shape (`nnzmax`)
        The first `nnz` elements contain the row and column indices of
        the nonzero elements supplied in the finite element matrix A.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and `nv` = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= `nv`.

        (`errno` 1)
            On entry, nelt = *<value>* and `nv` = *<value>*.

            Constraint: nelt <= 2*`nv`-1.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, `nnzmax` = *<value>* and `nv` = *<value>*.

            Constraint: `nnzmax` <= `nv`^2.

        (`errno` 1)
            On entry, `nnzmax` = *<value>*, nelt = *<value>* and `nv` =
            *<value>*.

            Constraint: `nnzmax` >= (4*nelt+`nv`).

        (`errno` 1)
            On entry, `nv` = *<value>*.

            Constraint: `nv` >= 3.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh, especially the
            connectivity between triangles and vertices (the argument
            `conn`). Array dimensions should also be checked. If the
            problem persists, contact NAG.
    """

    from ..base.mesh import dim2_sparsity as b_dim2_sparsity

    fun_name = 'naginterfaces.library.mesh.dim2_sparsity'

    _conn = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conn'},
        conn,
        copy=False,
    )
    _nelt_py = _b_utils._EngineIntArrayType.get_shape(_conn, {'fun_name': fun_name, 'entity': 'conn'}, exp_rank=2)[1]
    _nelt = _b_utils._EngineIntScalarType(
        _nelt_py, {'fun_name': fun_name, 'entity': '_nelt'},
    ).to_ctypes()
    _nnzmax = _b_utils._EngineIntScalarType(
        nnzmax, {'fun_name': fun_name, 'entity': 'nnzmax'},
    ).to_ctypes()
    _nnzmax_py = _b_utils._EngineIntScalarType.to_py(_nnzmax)
    _irow = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nnzmax_py,),
    )
    _icol = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nnzmax_py,),
    )
    _return_scalar = b_dim2_sparsity(
        nv,
        _nelt,
        _nnzmax,
        _conn,
        _irow,
        _icol,
    )
    _nnz_py = _return_scalar
    _irow_py = _irow
    _icol_py = _icol
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nnz', 'irow', 'icol'],
    )(
        _nnz_py,
        _irow_py,
        _icol_py,
    )
    return _return_data

def dim2_renumber(nnzmax, coor, edge, conn, itrace, io_manager=None):
    u"""
    Renumbers a given mesh using Gibbs method.

    For full information please refer to the NAG Library document for
    d06cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06ccf.html

    Parameters
    ----------
    nnzmax : int
        The maximum number of nonzero entries in the matrix based on the
        input mesh. It is the dimension of the arrays `irow` and `icol`
        as declared in the function from which ``dim2_renumber`` is
        called.

    coor : float, array-like, shape (2, nv)
        `coor`[0,i-1] contains the x coordinate of the ith input mesh
        vertex, for i = 1,...,nv; while `coor`[1,i-1] contains the
        corresponding y coordinate.

    edge : int, array-like, shape (3, nedge)
        The specification of the boundary or interface edges.
        `edge`[0,j-1] and `edge`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge`[2,j-1] is a
        user-supplied tag for the jth boundary or interface edge:
        `edge`[2,j-1] = 0 for an interior edge and has a nonzero tag
        otherwise.

    conn : int, array-like, shape (3, nelt)
        The connectivity of the mesh between triangles and vertices. For
        each triangle j, `conn`[i-1,j-1] gives the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,nelt, for i =
        1,2,...,3.

    itrace : int
        The level of trace information required from ``dim2_renumber``.

        `itrace` <= 0

          No output is generated.

        `itrace` = 1

          Information about the effect of the renumbering on the finite
          element matrix are output. This information includes the half
          bandwidth and the sparsity structure of this matrix before and
          after renumbering.

        `itrace` > 1

          The output is similar to that produced when `itrace` = 1 but
          the sparsities (for each row of the matrix, indices of nonzero
          entries) of the matrix before and after renumbering are also
          output.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nnz : int
        The number of nonzero entries in the matrix based on the input
        mesh.

    coor : float, ndarray, shape (2, nv)
        `coor`[0,i-1] will contain the x coordinate of the ith
        renumbered mesh vertex, for i = 1,...,nv; while `coor`[1,i-1]
        will contain the corresponding y coordinate.

    edge : int, ndarray, shape (3, nedge)
        The renumbered specification of the boundary or interface edges.

    conn : int, ndarray, shape (3, nelt)
        The renumbered connectivity of the mesh between triangles and
        vertices.

    irow : int, ndarray, shape (`nnzmax`)
        The first `nnz` elements contain the row and column indices of
        the nonzero elements supplied in the finite element matrix A.

    icol : int, ndarray, shape (`nnzmax`)
        The first `nnz` elements contain the row and column indices of
        the nonzero elements supplied in the finite element matrix A.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nv = *<value>*.

            Constraint: nv >= 3.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, `edge`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `edge`(I,J) >= 1 and `edge`(I,J) <= nv.

        (`errno` 1)
            On entry, nelt = *<value>* and nv = *<value>*.

            Constraint: nelt <= 2*nv-1.

        (`errno` 1)
            On entry, `conn`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `conn`(I,J) >= 1 and `conn`(I,J) <= nv.

        (`errno` 1)
            On entry, `nnzmax` = *<value>*, nelt = *<value>* and nv =
            *<value>*.

            Constraint: `nnzmax` >= (4*nelt+nv) and `nnzmax` <= nv^2.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            An error has occurred during the computation of the compact
            sparsity of the finite element matrix. Check the
            Triangle/Vertices connectivity.

        (`errno` 3)
            A serious error has occurred in an internal call to the
            renumbering function. Check the input mesh especially the
            connectivity. Seek expert help.
    """

    from ..base.mesh import dim2_renumber as b_dim2_renumber

    fun_name = 'naginterfaces.library.mesh.dim2_renumber'

    _coor = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor'},
        coor,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nv_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor, {'fun_name': fun_name, 'entity': 'coor'}, exp_rank=2)[1]
    _nv = _b_utils._EngineIntScalarType(
        _nv_py, {'fun_name': fun_name, 'entity': '_nv'},
    ).to_ctypes()
    _conn = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conn'},
        conn,
        copy=True,
    )
    _nelt_py = _b_utils._EngineIntArrayType.get_shape(_conn, {'fun_name': fun_name, 'entity': 'conn'}, exp_rank=2)[1]
    _nelt = _b_utils._EngineIntScalarType(
        _nelt_py, {'fun_name': fun_name, 'entity': '_nelt'},
    ).to_ctypes()
    _edge = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge'},
        edge,
        copy=True,
    )
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edge, {'fun_name': fun_name, 'entity': 'edge'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _nnzmax = _b_utils._EngineIntScalarType(
        nnzmax, {'fun_name': fun_name, 'entity': 'nnzmax'},
    ).to_ctypes()
    _nnzmax_py = _b_utils._EngineIntScalarType.to_py(_nnzmax)
    _irow = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nnzmax_py,),
    )
    _icol = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nnzmax_py,),
    )
    _return_scalar = b_dim2_renumber(
        _nv,
        _nelt,
        _nedge,
        _nnzmax,
        _coor,
        _edge,
        _conn,
        _irow,
        _icol,
        itrace,
        io_manager,
    )
    _coor_py = _coor
    _edge_py = _edge
    _conn_py = _conn
    _nnz_py = _return_scalar
    _irow_py = _irow
    _icol_py = _icol
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nnz', 'coor', 'edge', 'conn', 'irow', 'icol'],
    )(
        _nnz_py,
        _coor_py,
        _edge_py,
        _conn_py,
        _irow_py,
        _icol_py,
    )
    return _return_data

def dim2_transform_affine(mode, itype, trans, coori, edgei, conni, itrace, io_manager=None):
    u"""
    Generates a mesh resulting from an affine transformation of a given
    mesh.

    For full information please refer to the NAG Library document for
    d06da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06daf.html

    Parameters
    ----------
    mode : int
        If `mode` = 1, the arguments `coori`, `edgei` and `conni` are
        overwritten on exit by the output values described in `cooro`,
        `edgeo` and `conno` respectively. In this case `cooro`, `edgeo`
        and `conno` are not referenced, and you can save storage space.

        If `mode` != 1, no such aliasing is assumed.

    itype : int, array-like, shape (ntrans)
        `itype`[i-1], for i = 1,2,...,ntrans, indicates the type of each
        transformation as follows:

        `itype`[i-1] = 0

          Identity transformation.

        `itype`[i-1] = 1

          Translation.

        `itype`[i-1] = 2

          Symmetric transformation with respect to a user-supplied line.

        `itype`[i-1] = 3

          Rotation.

        `itype`[i-1] = 4

          Scaling.

        `itype`[i-1] = 10

          User-supplied analytic transformation.

        Note that the transformations are applied in the order described
        in `itype`.

    trans : float, array-like, shape (6, ntrans)
        The arguments for each transformation. For i = 1,2,...,ntrans,
        `trans`[0,i-1] to `trans`[5,i-1] contain the arguments of the
        ith transformation.

        If `itype`[i-1] = 0, elements `trans`[0,i-1] to `trans`[5,i-1]
        are not referenced.

        If `itype`[i-1] = 1, the translation vector is vec(u) = (a;b),
        where a = `trans`[0,i-1] and b = `trans`[1,i-1], while elements
        `trans`[2,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 2, the user-supplied line is the curve {(x,y)
        in R^2; such that ax+by+c = 0}, where a = `trans`[0,i-1], b =
        `trans`[1,i-1] and c = `trans`[2,i-1], while elements
        `trans`[3,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 3, the centre of the rotation is (x_0,y_0)
        where x_0 = `trans`[0,i-1] and y_0 = `trans`[1,i-1], theta =
        `trans`[2,i-1] is its angle in degrees, while elements
        `trans`[3,i-1] to `trans`[5,i-1] are not referenced.

        If `itype`[i-1] = 4, a = `trans`[0,i-1] is the scaling
        coefficient in the x-direction, b = `trans`[1,i-1] is the
        scaling coefficient in the y-direction, and (x_0,y_0) are the
        scaling centre coordinates, with x_0 = `trans`[2,i-1] and y_0 =
        `trans`[3,i-1]; while elements `trans`[4,i-1] to `trans`[5,i-1]
        are not referenced.

        If `itype`[i-1] = 10, the user-supplied analytic affine
        transformation Y = A*X+B is such that A = (a_{kl})_{1 <= k,l <=
        2} and B = (b_k)_{1 <= k <= 2} wherea_{kl} =
        `trans`[2*(k-1)+l-1,i-1], and b_k = `trans`[4+k-1,i-1] with k,l
        = 1,2.

    coori : float, array-like, shape (2, nv)
        `coori`[0,i-1] contains the x coordinate of the ith vertex of
        the input mesh, for i = 1,2,...,nv; while `coori`[1,i-1]
        contains the corresponding y coordinate.

    edgei : int, array-like, shape (3, nedge)
        The specification of the boundary or interface edges.
        `edgei`[0,j-1] and `edgei`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edgei`[2,j-1] is a
        user-supplied tag for the jth boundary edge.

    conni : int, array-like, shape (3, nelt)
        The connectivity of the input mesh between triangles and
        vertices. For each triangle j, `conni`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,nelt, for i = 1,2,...,3.

    itrace : int
        The level of trace information required from
        ``dim2_transform_affine``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Details of each transformation, the matrix A and the vector B
          of the final transformation, which is the composition of all
          the ntrans transformations, are printed.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    coori : float, ndarray, shape (2, nv)
        See [Further Comments].

    edgei : int, ndarray, shape (3, nedge)
        If `mode` = 1, `edgei` holds the output values described in
        `edgeo`.

    conni : int, ndarray, shape (3, nelt)
        If `mode` = 1, `conni` holds the output values described in
        `conno`.

    cooro : None or float, ndarray, shape (:, :)
        `cooro`[0,i-1] will contain the x coordinate of the ith vertex
        of the transformed mesh, for i = 1,2,...,nv; while
        `cooro`[1,i-1] will contain the corresponding y coordinate.

    edgeo : None or int, ndarray, shape (:, :)
        The specification of the boundary or interface edges of the
        transformed mesh. If the number of symmetric transformations is
        even or zero then `edgeo`[i-1,j-1] = `edgei`[i-1,j-1], for j =
        1,2,...,nedge, for i = 1,2,...,3; otherwise `edgeo`[0,j-1] =
        `edgei`[1,j-1],`edgeo`[1,j-1] = `edgei`[0,j-1] and
        `edgeo`[2,j-1] = `edgei`[2,j-1], for j = 1,2,...,nedge.

    conno : None or int, ndarray, shape (:, :)
        The connectivity of the transformed mesh between triangles and
        vertices. If the number of symmetric transformations is even or
        zero then `conno`[i-1,j-1] = `conni`[i-1,j-1], for j =
        1,2,...,nelt, for i = 1,2,...,3; otherwise `conno`[0,j-1] =
        `conni`[0,j-1], `conno`[1,j-1] = `conni`[2,j-1] and
        `conno`[2,j-1] = `conni`[1,j-1], for j = 1,2,...,nelt.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nv = *<value>*.

            Constraint: nv >= 3.

        (`errno` 1)
            On entry, `itype`[I-1] = *<value>* and I = *<value>*.

            Constraint: `itype`[I-1] = 0, 1, 2, 3, 4 or 10.

        (`errno` 1)
            On entry, nedge = *<value>*.

            Constraint: nedge >= 1.

        (`errno` 1)
            On entry, nelt = *<value>* and nv = *<value>*.

            Constraint: nelt <= 2*nv-1.

        (`errno` 1)
            On entry, `edgei`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `edgei`(I,J) >= 1 and `edgei`(I,J) <= nv.

        (`errno` 1)
            On entry, `conni`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv = *<value>*.

            Constraint: `conni`(I,J) >= 1 and `conni`(I,J) <= nv.

        (`errno` 1)
            On entry, ntrans = *<value>*.

            Constraint: ntrans > 0.

        (`errno` 1)
            On entry, the end points of the edge J have the same index
            I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of the triangle K have the same
            index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            A serious error has occurred in an internal call to an
            auxiliary function. Check the input mesh especially the
            connectivities and the details of each transformations.
    """

    from ..base.mesh import dim2_transform_affine as b_dim2_transform_affine

    fun_name = 'naginterfaces.library.mesh.dim2_transform_affine'

    _trans = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'trans'},
        trans,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _coori = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coori'},
        coori,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _edgei = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edgei'},
        edgei,
        copy=True,
    )
    _conni = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conni'},
        conni,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_trans, 'trans'), (_coori, 'coori'), (_edgei, 'edgei'), (_conni, 'conni')),
    )
    _nv_py = _b_utils._EngineFloat64ArrayType.get_shape(_coori, {'fun_name': fun_name, 'entity': 'coori'}, exp_rank=2)[1]
    _nv = _b_utils._EngineIntScalarType(
        _nv_py, {'fun_name': fun_name, 'entity': '_nv'},
    ).to_ctypes()
    _nedge_py = _b_utils._EngineIntArrayType.get_shape(_edgei, {'fun_name': fun_name, 'entity': 'edgei'}, exp_rank=2)[1]
    _nedge = _b_utils._EngineIntScalarType(
        _nedge_py, {'fun_name': fun_name, 'entity': '_nedge'},
    ).to_ctypes()
    _nelt_py = _b_utils._EngineIntArrayType.get_shape(_conni, {'fun_name': fun_name, 'entity': 'conni'}, exp_rank=2)[1]
    _nelt = _b_utils._EngineIntScalarType(
        _nelt_py, {'fun_name': fun_name, 'entity': '_nelt'},
    ).to_ctypes()
    _itype = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'itype'},
        itype,
        copy=False,
    )
    _ntrans_py = _b_utils._EngineIntArrayType.get_shape(_itype, {'fun_name': fun_name, 'entity': 'itype'}, exp_rank=1)[0]
    _ntrans = _b_utils._EngineIntScalarType(
        _ntrans_py, {'fun_name': fun_name, 'entity': '_ntrans'},
    ).to_ctypes()
    _mode = _b_utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _mode_py = _b_utils._EngineIntScalarType.to_py(_mode)
    if _mode_py != 1:
        if _mode_py != 1:
            _cooro_dim1 = 2
        else:
            _cooro_dim1 = 0
        if _mode_py != 1:
            _cooro_dim2 = _nv_py
        else:
            _cooro_dim2 = 0
        exp_shape_cooro = (_cooro_dim1, _cooro_dim2)
        _cooro = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_cooro,
            sorder=_m_sorder,
        )
    else:
        _cooro = None
    if _mode_py != 1:
        if _mode_py != 1:
            _edgeo_dim1 = 3
        else:
            _edgeo_dim1 = 0
        if _mode_py != 1:
            _edgeo_dim2 = _nedge_py
        else:
            _edgeo_dim2 = 0
        exp_shape_edgeo = (_edgeo_dim1, _edgeo_dim2)
        _edgeo = _b_utils._EngineIntArrayType.empty_ndarray(
            exp_shape_edgeo,
            sorder=_m_sorder,
        )
    else:
        _edgeo = None
    if _mode_py != 1:
        if _mode_py != 1:
            _conno_dim1 = 3
        else:
            _conno_dim1 = 0
        if _mode_py != 1:
            _conno_dim2 = _nelt_py
        else:
            _conno_dim2 = 1
        exp_shape_conno = (_conno_dim1, _conno_dim2)
        _conno = _b_utils._EngineIntArrayType.empty_ndarray(
            exp_shape_conno,
            sorder=_m_sorder,
        )
    else:
        _conno = None
    b_dim2_transform_affine(
        _mode,
        _nv,
        _nedge,
        _nelt,
        _ntrans,
        _itype,
        _trans,
        _coori,
        _edgei,
        _conni,
        _cooro,
        _edgeo,
        _conno,
        itrace,
        io_manager,
    )
    _coori_py = _coori
    _edgei_py = _edgei
    _conni_py = _conni
    if _mode_py != 1:
        _cooro_py = _cooro
    else:
        _cooro_py = None
    if _mode_py != 1:
        _edgeo_py = _edgeo
    else:
        _edgeo_py = None
    if _mode_py != 1:
        _conno_py = _conno
    else:
        _conno_py = None
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['coori', 'edgei', 'conni', 'cooro', 'edgeo', 'conno'],
    )(
        _coori_py,
        _edgei_py,
        _conni_py,
        _cooro_py,
        _edgeo_py,
        _conno_py,
    )
    return _return_data

def dim2_join(coor1, edge1, conn1, reft1, coor2, edge2, conn2, reft2, itrace, eps=0.001, io_manager=None):
    u"""
    Joins together two given adjacent (possibly overlapping) meshes.

    For full information please refer to the NAG Library document for
    d06db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d06/d06dbf.html

    Parameters
    ----------
    coor1 : float, array-like, shape (2, nv1)
        `coor1`[0,i-1] contains the x coordinate of the ith vertex of
        the first input mesh, for i = 1,2,...,nv1; while `coor1`[1,i-1]
        contains the corresponding y coordinate.

    edge1 : int, array-like, shape (3, nedge1)
        The specification of the boundary edges of the first input mesh.
        `edge1`[0,j-1] and `edge1`[1,j-1] contain the vertex numbers of
        the two end points of the jth boundary edge. `edge1`[2,j-1] is a
        user-supplied tag for the jth boundary edge.

    conn1 : int, array-like, shape (3, nelt1)
        The connectivity between triangles and vertices of the first
        input mesh. For each triangle j, `conn1`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,nelt1, for i = 1,2,...,3.

    reft1 : int, array-like, shape (nelt1)
        `reft1`[k-1] contains the user-supplied tag of the kth triangle
        from the first input mesh, for k = 1,2,...,nelt1.

    coor2 : float, array-like, shape (2, nv2)
        `coor2`[0,i-1] contains the x coordinate of the ith vertex of
        the second input mesh, for i = 1,2,...,nv2; while `coor2`[1,i-1]
        contains the corresponding y coordinate.

    edge2 : int, array-like, shape (3, nedge2)
        The specification of the boundary edges of the second input
        mesh. `edge2`[0,j-1] and `edge2`[1,j-1] contain the vertex
        numbers of the two end points of the jth boundary edge.
        `edge2`[2,j-1] is a user-supplied tag for the jth boundary edge.

    conn2 : int, array-like, shape (3, nelt2)
        The connectivity between triangles and vertices of the second
        input mesh. For each triangle j, `conn2`[i-1,j-1] gives the
        indices of its three vertices (in anticlockwise order), for j =
        1,2,...,nelt2, for i = 1,2,...,3.

    reft2 : int, array-like, shape (nelt2)
        `reft2`[k-1] contains the user-supplied tag of the kth triangle
        from the second input mesh, for k = 1,2,...,nelt2.

    itrace : int
        The level of trace information required from ``dim2_join``.

        `itrace` <= 0

          No output is generated.

        `itrace` >= 1

          Details about the common vertices, edges and triangles to both
          meshes are printed.

    eps : float, optional
        The relative precision of the restitching of the two input
        meshes (see [Further Comments]).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nv3 : int
        The total number of vertices in the resulting mesh.

    nelt3 : int
        The number of triangular elements in the resulting mesh.

    nedge3 : int
        The number of boundary edges in the resulting mesh.

    coor3 : float, ndarray, shape (2, nv1+nv2)
        `coor3`[0,i-1] will contain the x coordinate of the ith vertex
        of the resulting mesh, for i = 1,2,...,`nv3`; while
        `coor3`[1,i-1] will contain the corresponding y coordinate.

    edge3 : int, ndarray, shape (3, nedge1+nedge2)
        The specification of the boundary edges of the resulting mesh.
        `edge3`[i-1,j-1] will contain the vertex number of the ith end
        point (i = 1,2) of the jth boundary or interface edge.

        If the two meshes overlap, `edge3`[2,j-1] will contain the same
        tag as the corresponding edge belonging to the first and/or the
        second input mesh.

        If the two meshes are adjacent,

        - if the jth edge is part of the partition interface,
          `edge3`[2,j-1] will contain the value 1000*k_1+k_2 where k_1
          and k_2 are the tags for the same edge of the first and the
          second mesh respectively;

        - otherwise, `edge3`[2,j-1] will contain the same tag as the
          corresponding edge belonging to the first and/or the second
          input mesh.

    conn3 : int, ndarray, shape (3, nelt1+nelt2)
        The connectivity between triangles and vertices of the resulting
        mesh. `conn3`[i-1,j-1] will give the indices of its three
        vertices (in anticlockwise order), for j = 1,2,...,`nelt3`, for
        i = 1,2,...,3.

    reft3 : int, ndarray, shape (nelt1+nelt2)
        If the two meshes form a partition, `reft3`[k-1] will contain
        the same tag as the corresponding triangle belonging to the
        first or the second input mesh, for k = 1,2,...,`nelt3`. If the
        two meshes overlap, `reft3`[k-1] will contain the value
        1000*k_1+k_2 where k_1 and k_2 are the user-supplied tags for
        the same triangle of the first and the second mesh respectively,
        for k = 1,2,...,`nelt3`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nv2 = *<value>*.

            Constraint: nv2 >= 3.

        (`errno` 1)
            On entry, nedge2 = *<value>*.

            Constraint: nedge2 >= 1.

        (`errno` 1)
            On entry, `edge2`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv2 = *<value>*.

            Constraint: `edge2`(I,J) >= 1 and `edge2`(I,J) <= nv2.

        (`errno` 1)
            On entry, nelt2 = *<value>* and nv2 = *<value>*.

            Constraint: nelt2 <= (2*nv2-1).

        (`errno` 1)
            On entry, `conn2`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv2 = *<value>*.

            Constraint: `conn2`(I,J) >= 1 and `conn2`(I,J) <= nv2.

        (`errno` 1)
            On entry, nv1 = *<value>*.

            Constraint: nv1 >= 3.

        (`errno` 1)
            On entry, nedge1 = *<value>*.

            Constraint: nedge1 >= 1.

        (`errno` 1)
            On entry, `edge1`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv1 = *<value>*.

            Constraint: `edge1`(I,J) >= 1 and `edge1`(I,J) <= nv1.

        (`errno` 1)
            On entry, nelt1 = *<value>* and nv1 = *<value>*.

            Constraint: nelt1 <= (2*nv1-1).

        (`errno` 1)
            On entry, `conn1`(I,J) = *<value>*, I = *<value>*, J =
            *<value>* and nv1 = *<value>*.

            Constraint: `conn1`(I,J) >= 1 and `conn1`(I,J) <= nv1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, the end points of edge J in the second mesh have
            the same index I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of triangle K in the second mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, the end points of edge J in the first mesh have
            the same index I: J = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 2 and 3 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 3 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 1)
            On entry, vertices 1 and 2 of triangle K in the first mesh
            have the same index I: K = *<value>* and I = *<value>*.

        (`errno` 2)
            The function has detected only *<value>* coincident vertices
            with a precision `eps` = *<value>*. Either `eps` should be
            changed or the two meshes are not restitchable.

        (`errno` 3)
            A serious error has occurred in an internal call to the
            restitching routine. Check the input of the two meshes,
            especially the edges/vertices and/or the triangles/vertices
            connectivities. Seek expert help.

        (`errno` 4)
            The function has detected a different number of coincident
            triangles from the two meshes in the overlapping zone
            *<value>* *<value>*. Check the input of the two meshes,
            especially the triangles/vertices connectivity.

        (`errno` 5)
            The function has detected a different number of coincident
            edges from the two meshes on the partition interface
            *<value>* *<value>*. Check the input of the two meshes,
            especially the edges/vertices connectivity.
    """

    from ..base.mesh import dim2_join as b_dim2_join

    fun_name = 'naginterfaces.library.mesh.dim2_join'

    _coor1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor1'},
        coor1,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _edge1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge1'},
        edge1,
        copy=False,
    )
    _conn1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conn1'},
        conn1,
        copy=False,
    )
    _coor2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'coor2'},
        coor2,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _edge2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'edge2'},
        edge2,
        copy=False,
    )
    _conn2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'conn2'},
        conn2,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_coor1, 'coor1'), (_edge1, 'edge1'), (_conn1, 'conn1'), (_coor2, 'coor2'), (_edge2, 'edge2'), (_conn2, 'conn2')),
    )
    _nv1_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor1, {'fun_name': fun_name, 'entity': 'coor1'}, exp_rank=2)[1]
    _nv1 = _b_utils._EngineIntScalarType(
        _nv1_py, {'fun_name': fun_name, 'entity': '_nv1'},
    ).to_ctypes()
    _nelt1_py = _b_utils._EngineIntArrayType.get_shape(_conn1, {'fun_name': fun_name, 'entity': 'conn1'}, exp_rank=2)[1]
    _nelt1 = _b_utils._EngineIntScalarType(
        _nelt1_py, {'fun_name': fun_name, 'entity': '_nelt1'},
    ).to_ctypes()
    _nedge1_py = _b_utils._EngineIntArrayType.get_shape(_edge1, {'fun_name': fun_name, 'entity': 'edge1'}, exp_rank=2)[1]
    _nedge1 = _b_utils._EngineIntScalarType(
        _nedge1_py, {'fun_name': fun_name, 'entity': '_nedge1'},
    ).to_ctypes()
    _reft1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'reft1'},
        reft1,
        copy=False,
    )
    _nv2_py = _b_utils._EngineFloat64ArrayType.get_shape(_coor2, {'fun_name': fun_name, 'entity': 'coor2'}, exp_rank=2)[1]
    _nv2 = _b_utils._EngineIntScalarType(
        _nv2_py, {'fun_name': fun_name, 'entity': '_nv2'},
    ).to_ctypes()
    _nelt2_py = _b_utils._EngineIntArrayType.get_shape(_conn2, {'fun_name': fun_name, 'entity': 'conn2'}, exp_rank=2)[1]
    _nelt2 = _b_utils._EngineIntScalarType(
        _nelt2_py, {'fun_name': fun_name, 'entity': '_nelt2'},
    ).to_ctypes()
    _nedge2_py = _b_utils._EngineIntArrayType.get_shape(_edge2, {'fun_name': fun_name, 'entity': 'edge2'}, exp_rank=2)[1]
    _nedge2 = _b_utils._EngineIntScalarType(
        _nedge2_py, {'fun_name': fun_name, 'entity': '_nedge2'},
    ).to_ctypes()
    _reft2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'reft2'},
        reft2,
        copy=False,
    )
    _coor3 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2, _nv1_py+_nv2_py),
        sorder=_m_sorder,
    )
    _edge3 = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, _nedge1_py+_nedge2_py),
        sorder=_m_sorder,
    )
    _conn3 = _b_utils._EngineIntArrayType.empty_ndarray(
        (3, _nelt1_py+_nelt2_py),
        sorder=_m_sorder,
    )
    _reft3 = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nelt1_py+_nelt2_py,),
    )
    _return_scalars = b_dim2_join(
        eps,
        _nv1,
        _nelt1,
        _nedge1,
        _coor1,
        _edge1,
        _conn1,
        _reft1,
        _nv2,
        _nelt2,
        _nedge2,
        _coor2,
        _edge2,
        _conn2,
        _reft2,
        _coor3,
        _edge3,
        _conn3,
        _reft3,
        itrace,
        io_manager,
    )
    _nv3_py = _return_scalars.nv3
    _nelt3_py = _return_scalars.nelt3
    _nedge3_py = _return_scalars.nedge3
    _coor3_py = _coor3
    _edge3_py = _edge3
    _conn3_py = _conn3
    _reft3_py = _reft3
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nv3', 'nelt3', 'nedge3', 'coor3', 'edge3', 'conn3', 'reft3'],
    )(
        _nv3_py,
        _nelt3_py,
        _nedge3_py,
        _coor3_py,
        _edge3_py,
        _conn3_py,
        _reft3_py,
    )
    return _return_data
