"""
Deprecated wrappers for the NAG C Library (Mark 26.0) c02 Chapter.

c02 - Zeros of Polynomials.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/C02/c02conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import Complex
from .util import Nag_Boolean
from .util import NagError

_library_cl = get_library()

c02afc_ctypes = _library_cl.c02afc
c02afc_ctypes.restype = None
c02afc_ctypes.argtypes = [Integer, POINTER(Complex), Nag_Boolean,
                          POINTER(Complex), POINTER(NagError)]


def c02afc(n, a, scale, z, fail):
    """
    Zeros of a polynomial with complex coefficients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c02/c02afc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, z
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, z
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('c02afc', a, 'a')
    _z = nag_complex_type_check_and_cast('c02afc', z, 'z')
    c02afc_ctypes(n, _a, scale, _z, fail)

c02agc_ctypes = _library_cl.c02agc
c02agc_ctypes.restype = None
c02agc_ctypes.argtypes = [Integer, POINTER(c_double), Nag_Boolean,
                          POINTER(Complex), POINTER(NagError)]


def c02agc(n, a, scale, z, fail):
    """
    Zeros of a polynomial with real coefficients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c02/c02agc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('c02agc', a, 'a')
    _z = nag_complex_type_check_and_cast('c02agc', z, 'z')
    c02agc_ctypes(n, _a, scale, _z, fail)

c02akc_ctypes = _library_cl.c02akc
c02akc_ctypes.restype = None
c02akc_ctypes.argtypes = [c_double, c_double, c_double, c_double,
                          POINTER(c_double), POINTER(c_double),
                          POINTER(c_double), POINTER(NagError)]


def c02akc(u, r, s, t, zeror, zeroi, errest, fail):
    """
    Zeros of a cubic polynomial with real coefficients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c02/c02akc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        zeror, zeroi, errest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        zeror, zeroi, errest
    """
    from .util import nag_double_type_check_and_cast
    _zeror = nag_double_type_check_and_cast('c02akc', zeror, 'zeror')
    _zeroi = nag_double_type_check_and_cast('c02akc', zeroi, 'zeroi')
    _errest = nag_double_type_check_and_cast('c02akc', errest, 'errest')
    c02akc_ctypes(u, r, s, t, _zeror, _zeroi, _errest, fail)

c02alc_ctypes = _library_cl.c02alc
c02alc_ctypes.restype = None
c02alc_ctypes.argtypes = [c_double, c_double, c_double, c_double, c_double,
                          POINTER(c_double), POINTER(c_double),
                          POINTER(c_double), POINTER(NagError)]


def c02alc(e, a, b, c, d, zeror, zeroi, errest, fail):
    """
    Zeros of a real quartic polynomial with real coefficients.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/c02/c02alc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        zeror, zeroi, errest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        zeror, zeroi, errest
    """
    from .util import nag_double_type_check_and_cast
    _zeror = nag_double_type_check_and_cast('c02alc', zeror, 'zeror')
    _zeroi = nag_double_type_check_and_cast('c02alc', zeroi, 'zeroi')
    _errest = nag_double_type_check_and_cast('c02alc', errest, 'errest')
    c02alc_ctypes(e, a, b, c, d, _zeror, _zeroi, _errest, fail)

nag_zeros_complex_poly = c02afc
nag_zeros_real_poly = c02agc
nag_cubic_roots = c02akc
nag_quartic_roots = c02alc
__all__ = ['c02afc', 'c02agc', 'c02akc', 'c02alc',
           'nag_cubic_roots', 'nag_quartic_roots',
           'nag_zeros_complex_poly',
           'nag_zeros_real_poly']
