"""
Deprecated wrappers for the NAG C Library (Mark 26.0) e02 Chapter.

e02 - Curve and Surface Fitting.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/E02/e02conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from ctypes import c_char_p
from .util import get_library
from .util import Integer
from .util import Nag_Comm
from .util import NagError
from .util import Nag_OrderType
from .util import Nag_Spline
from .util import Nag_DerivType
from .util import Nag_Start
from .util import Nag_SplineVectorSort
from .util import Nag_Boolean
from .util import Nag_2dSpline
from .util import Nag_VariableType

_library_cl = get_library()

STRING = c_char_p

e02adc_ctypes = _library_cl.e02adc
e02adc_ctypes.restype = None
e02adc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e02adc(m, kplus1, tda, x, y, w, a, s, fail):
    """
    Computes the coefficients of a Chebyshev series polynomial for arbitrary
    data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02adc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, w, a, s
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, w, a, s
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02adc', x, 'x')
    _y = nag_double_type_check_and_cast('e02adc', y, 'y')
    _w = nag_double_type_check_and_cast('e02adc', w, 'w')
    _a = nag_double_type_check_and_cast('e02adc', a, 'a')
    _s = nag_double_type_check_and_cast('e02adc', s, 's')
    e02adc_ctypes(m, kplus1, tda, _x, _y, _w, _a, _s, fail)

e02aec_ctypes = _library_cl.e02aec
e02aec_ctypes.restype = None
e02aec_ctypes.argtypes = [Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def e02aec(nplus1, a, xcap, p, fail):
    """
    Evaluates the coefficients of a Chebyshev series polynomial.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, p
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02aec', a, 'a')
    _p = nag_double_type_check_and_cast('e02aec', p, 'p')
    e02aec_ctypes(nplus1, _a, xcap, _p, fail)

e02afc_ctypes = _library_cl.e02afc
e02afc_ctypes.restype = None
e02afc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e02afc(nplus1, f, a, fail):
    """
    Computes the coefficients of a Chebyshev series polynomial for
    interpolated data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02afc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        f, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, a
    """
    from .util import nag_double_type_check_and_cast
    _f = nag_double_type_check_and_cast('e02afc', f, 'f')
    _a = nag_double_type_check_and_cast('e02afc', a, 'a')
    e02afc_ctypes(nplus1, _f, _a, fail)

e02agc_ctypes = _library_cl.e02agc
e02agc_ctypes.restype = None
e02agc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e02agc(order, m, k, xmin, xmax, x, y, w, mf, xf, yf, p, a, s, n, resid, fail):
    """
    Least squares polynomial fit, values and derivatives may be constrained,
    arbitrary data points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02agc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, w, xf, yf, p, a, s, n, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, w, xf, yf, a, s, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        p, n
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02agc', x, 'x')
    _y = nag_double_type_check_and_cast('e02agc', y, 'y')
    _w = nag_double_type_check_and_cast('e02agc', w, 'w')
    _xf = nag_double_type_check_and_cast('e02agc', xf, 'xf')
    _yf = nag_double_type_check_and_cast('e02agc', yf, 'yf')
    _p = nag_integer_type_check_and_cast('e02agc', p, 'p')
    _a = nag_double_type_check_and_cast('e02agc', a, 'a')
    _s = nag_double_type_check_and_cast('e02agc', s, 's')
    _n = nag_integer_type_check_and_cast('e02agc', n, 'n')
    _resid = nag_double_type_check_and_cast('e02agc', resid, 'resid')
    e02agc_ctypes(order, m, k, xmin, xmax, _x, _y, _w, mf, _xf, _yf, _p, _a, _s, _n, _resid, fail)

e02ahc_ctypes = _library_cl.e02ahc
e02ahc_ctypes.restype = None
e02ahc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def e02ahc(n, xmin, xmax, a, ia1, patm1, adif, iadif1, fail):
    """
    Derivative of fitted polynomial in Chebyshev series form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02ahc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, patm1, adif
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, patm1, adif
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02ahc', a, 'a')
    _patm1 = nag_double_type_check_and_cast('e02ahc', patm1, 'patm1')
    _adif = nag_double_type_check_and_cast('e02ahc', adif, 'adif')
    e02ahc_ctypes(n, xmin, xmax, _a, ia1, _patm1, _adif, iadif1, fail)

e02ajc_ctypes = _library_cl.e02ajc
e02ajc_ctypes.restype = None
e02ajc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def e02ajc(n, xmin, xmax, a, ia1, qatm1, aint, iaint1, fail):
    """
    Integral of fitted polynomial in Chebyshev series form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02ajc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, aint
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, aint
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02ajc', a, 'a')
    _aint = nag_double_type_check_and_cast('e02ajc', aint, 'aint')
    e02ajc_ctypes(n, xmin, xmax, _a, ia1, qatm1, _aint, iaint1, fail)

e02akc_ctypes = _library_cl.e02akc
e02akc_ctypes.restype = None
e02akc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def e02akc(n, xmin, xmax, a, ia1, x, result, fail):
    """
    Evaluation of fitted polynomial in one variable from Chebyshev series
    form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02akc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, result
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, result
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02akc', a, 'a')
    _result = nag_double_type_check_and_cast('e02akc', result, 'result')
    e02akc_ctypes(n, xmin, xmax, _a, ia1, x, _result, fail)

e02alc_ctypes = _library_cl.e02alc
e02alc_ctypes.restype = None
e02alc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e02alc(n, x, y, m, a, ref, fail):
    """
    Minimax curve fit by polynomials.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02alc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, a, ref
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, a, ref
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02alc', x, 'x')
    _y = nag_double_type_check_and_cast('e02alc', y, 'y')
    _a = nag_double_type_check_and_cast('e02alc', a, 'a')
    _ref = nag_double_type_check_and_cast('e02alc', ref, 'ref')
    e02alc_ctypes(n, _x, _y, m, _a, _ref, fail)

e02bac_ctypes = _library_cl.e02bac
e02bac_ctypes.restype = None
e02bac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Spline), POINTER(NagError)]


def e02bac(m, x, y, weights, ss, spline, fail):
    """
    Least squares curve cubic spline fit (including interpolation), one
    variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, weights, ss
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, weights, ss
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02bac', x, 'x')
    _y = nag_double_type_check_and_cast('e02bac', y, 'y')
    _weights = nag_double_type_check_and_cast('e02bac', weights, 'weights')
    _ss = nag_double_type_check_and_cast('e02bac', ss, 'ss')
    e02bac_ctypes(m, _x, _y, _weights, _ss, spline, fail)

e02bbc_ctypes = _library_cl.e02bbc
e02bbc_ctypes.restype = None
e02bbc_ctypes.argtypes = [c_double, POINTER(c_double), POINTER(Nag_Spline), POINTER(NagError)]


def e02bbc(x, s, spline, fail):
    """
    Evaluation of fitted cubic spline, function only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        s
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s
    """
    from .util import nag_double_type_check_and_cast
    _s = nag_double_type_check_and_cast('e02bbc', s, 's')
    e02bbc_ctypes(x, _s, spline, fail)

e02bcc_ctypes = _library_cl.e02bcc
e02bcc_ctypes.restype = None
e02bcc_ctypes.argtypes = [Nag_DerivType, c_double, POINTER(c_double), POINTER(Nag_Spline), POINTER(NagError)]


def e02bcc(derivs, x, s, spline, fail):
    """
    Evaluation of fitted cubic spline, function and derivatives.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        s
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s
    """
    from .util import nag_double_type_check_and_cast
    _s = nag_double_type_check_and_cast('e02bcc', s, 's')
    e02bcc_ctypes(derivs, x, _s, spline, fail)

e02bdc_ctypes = _library_cl.e02bdc
e02bdc_ctypes.restype = None
e02bdc_ctypes.argtypes = [POINTER(Nag_Spline), POINTER(c_double), POINTER(NagError)]


def e02bdc(spline, integral, fail):
    """
    Evaluation of fitted cubic spline, definite integral.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        integral
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        integral
    """
    from .util import nag_double_type_check_and_cast
    _integral = nag_double_type_check_and_cast('e02bdc', integral, 'integral')
    e02bdc_ctypes(spline, _integral, fail)

e02bec_ctypes = _library_cl.e02bec
e02bec_ctypes.restype = None
e02bec_ctypes.argtypes = [Nag_Start, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(Nag_Spline), POINTER(NagError)]


def e02bec(start, m, x, y, weights, s, nest, fp, warmstartinf, spline, fail):
    """
    Least squares cubic spline curve fit, automatic knot placement, one
    variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, weights, fp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, weights, fp
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02bec', x, 'x')
    _y = nag_double_type_check_and_cast('e02bec', y, 'y')
    _weights = nag_double_type_check_and_cast('e02bec', weights, 'weights')
    _fp = nag_double_type_check_and_cast('e02bec', fp, 'fp')
    e02bec_ctypes(start, m, _x, _y, _weights, s, nest, _fp, warmstartinf, spline, fail)

e02bfc_ctypes = _library_cl.e02bfc
e02bfc_ctypes.restype = None
e02bfc_ctypes.argtypes = [Nag_SplineVectorSort, POINTER(Nag_Spline), Nag_DerivType, Nag_Boolean, POINTER(c_double), POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(NagError)]


def e02bfc(start, spline, deriv, xord, x, ixloc, nx, s, pds, iwrk, liwrk, fail):
    """
    Evaluation of fitted cubic spline, function and optionally derivatives
    at a vector of points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ixloc, s, iwrk
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, s
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ixloc, iwrk
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02bfc', x, 'x')
    _ixloc = nag_integer_type_check_and_cast('e02bfc', ixloc, 'ixloc')
    _s = nag_double_type_check_and_cast('e02bfc', s, 's')
    _iwrk = nag_integer_type_check_and_cast('e02bfc', iwrk, 'iwrk')
    e02bfc_ctypes(start, spline, deriv, xord, _x, _ixloc, nx, _s, pds, _iwrk, liwrk, fail)

e02cac_ctypes = _library_cl.e02cac
e02cac_ctypes.restype = None
e02cac_ctypes.argtypes = [POINTER(Integer), Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def e02cac(m, n, k, l, x, y, f, w, a, xmin, xmax, nux, inuxp1, nuy, inuyp1, fail):
    """
    Least squares surface fit by polynomials, data on lines parallel to one
    independent coordinate axis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        m, x, y, f, w, a, xmin, xmax, nux, nuy
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, w, a, xmin, xmax, nux, nuy
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _m = nag_integer_type_check_and_cast('e02cac', m, 'm')
    _x = nag_double_type_check_and_cast('e02cac', x, 'x')
    _y = nag_double_type_check_and_cast('e02cac', y, 'y')
    _f = nag_double_type_check_and_cast('e02cac', f, 'f')
    _w = nag_double_type_check_and_cast('e02cac', w, 'w')
    _a = nag_double_type_check_and_cast('e02cac', a, 'a')
    _xmin = nag_double_type_check_and_cast('e02cac', xmin, 'xmin')
    _xmax = nag_double_type_check_and_cast('e02cac', xmax, 'xmax')
    _nux = nag_double_type_check_and_cast('e02cac', nux, 'nux')
    _nuy = nag_double_type_check_and_cast('e02cac', nuy, 'nuy')
    e02cac_ctypes(_m, n, k, l, _x, _y, _f, _w, _a, _xmin, _xmax, _nux, inuxp1, _nuy, inuyp1, fail)

e02cbc_ctypes = _library_cl.e02cbc
e02cbc_ctypes.restype = None
e02cbc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e02cbc(mfirst, mlast, k, l, x, xmin, xmax, y, ymin, ymax, ff, a, fail):
    """
    Evaluation of fitted polynomial in two variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ff, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, ff, a
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02cbc', x, 'x')
    _ff = nag_double_type_check_and_cast('e02cbc', ff, 'ff')
    _a = nag_double_type_check_and_cast('e02cbc', a, 'a')
    e02cbc_ctypes(mfirst, mlast, k, l, _x, xmin, xmax, y, ymin, ymax, _ff, _a, fail)

e02dac_ctypes = _library_cl.e02dac
e02dac_ctypes.restype = None
e02dac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), c_double, POINTER(c_double), POINTER(Integer), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02dac(m, x, y, f, w, point, dl, eps, sigma, rank, spline, fail):
    """
    Least squares surface fit, bicubic splines.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, w, point, dl, sigma, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, w, dl, sigma
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        point, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02dac', x, 'x')
    _y = nag_double_type_check_and_cast('e02dac', y, 'y')
    _f = nag_double_type_check_and_cast('e02dac', f, 'f')
    _w = nag_double_type_check_and_cast('e02dac', w, 'w')
    _point = nag_integer_type_check_and_cast('e02dac', point, 'point')
    _dl = nag_double_type_check_and_cast('e02dac', dl, 'dl')
    _sigma = nag_double_type_check_and_cast('e02dac', sigma, 'sigma')
    _rank = nag_integer_type_check_and_cast('e02dac', rank, 'rank')
    e02dac_ctypes(m, _x, _y, _f, _w, _point, _dl, eps, _sigma, _rank, spline, fail)

e02dcc_ctypes = _library_cl.e02dcc
e02dcc_ctypes.restype = None
e02dcc_ctypes.argtypes = [Nag_Start, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02dcc(start, mx, x, my, y, f, s, nxest, nyest, fp, warmstartinf, spline, fail):
    """
    Least squares bicubic spline fit with automatic knot placement, two
    variables (rectangular grid).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, fp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, fp
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02dcc', x, 'x')
    _y = nag_double_type_check_and_cast('e02dcc', y, 'y')
    _f = nag_double_type_check_and_cast('e02dcc', f, 'f')
    _fp = nag_double_type_check_and_cast('e02dcc', fp, 'fp')
    e02dcc_ctypes(start, mx, _x, my, _y, _f, s, nxest, nyest, _fp, warmstartinf, spline, fail)

e02ddc_ctypes = _library_cl.e02ddc
e02ddc_ctypes.restype = None
e02ddc_ctypes.argtypes = [Nag_Start, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02ddc(start, m, x, y, f, weights, s, nxest, nyest, fp, rank, warmstartinf, spline, fail):
    """
    Least squares bicubic spline fit with automatic knot placement, two
    variables (scattered data).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, weights, fp, rank, warmstartinf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, weights, fp, warmstartinf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02ddc', x, 'x')
    _y = nag_double_type_check_and_cast('e02ddc', y, 'y')
    _f = nag_double_type_check_and_cast('e02ddc', f, 'f')
    _weights = nag_double_type_check_and_cast('e02ddc', weights, 'weights')
    _fp = nag_double_type_check_and_cast('e02ddc', fp, 'fp')
    _rank = nag_integer_type_check_and_cast('e02ddc', rank, 'rank')
    _warmstartinf = nag_double_type_check_and_cast('e02ddc', warmstartinf, 'warmstartinf')
    e02ddc_ctypes(start, m, _x, _y, _f, _weights, s, nxest, nyest, _fp, _rank, _warmstartinf, spline, fail)

e02dec_ctypes = _library_cl.e02dec
e02dec_ctypes.restype = None
e02dec_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02dec(m, x, y, ff, spline, fail):
    """
    Evaluation of bicubic spline, at a set of points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, ff
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, ff
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02dec', x, 'x')
    _y = nag_double_type_check_and_cast('e02dec', y, 'y')
    _ff = nag_double_type_check_and_cast('e02dec', ff, 'ff')
    e02dec_ctypes(m, _x, _y, _ff, spline, fail)

e02dfc_ctypes = _library_cl.e02dfc
e02dfc_ctypes.restype = None
e02dfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02dfc(mx, my, x, y, ff, spline, fail):
    """
    Evaluation of bicubic spline, at a mesh of points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02dfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, ff
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, ff
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02dfc', x, 'x')
    _y = nag_double_type_check_and_cast('e02dfc', y, 'y')
    _ff = nag_double_type_check_and_cast('e02dfc', ff, 'ff')
    e02dfc_ctypes(mx, my, _x, _y, _ff, spline, fail)

e02dhc_ctypes = _library_cl.e02dhc
e02dhc_ctypes.restype = None
e02dhc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(Nag_2dSpline), POINTER(NagError)]


def e02dhc(mx, my, x, y, nux, nuy, z, spline, fail):
    """
    Evaluation of spline surface at mesh of points with derivatives.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02dhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, z
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02dhc', x, 'x')
    _y = nag_double_type_check_and_cast('e02dhc', y, 'y')
    _z = nag_double_type_check_and_cast('e02dhc', z, 'z')
    e02dhc_ctypes(mx, my, _x, _y, nux, nuy, _z, spline, fail)

e02gac_ctypes = _library_cl.e02gac
e02gac_ctypes.restype = None
e02gac_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def e02gac(order, m, a, b, nplus2, toler, x, resid, rank, _iter, fail):
    """
    L_1-approximation by general linear function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, x, resid, rank, _iter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank, _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02gac', a, 'a')
    _b = nag_double_type_check_and_cast('e02gac', b, 'b')
    _x = nag_double_type_check_and_cast('e02gac', x, 'x')
    _resid = nag_double_type_check_and_cast('e02gac', resid, 'resid')
    _rank = nag_integer_type_check_and_cast('e02gac', rank, 'rank')
    __iter = nag_integer_type_check_and_cast('e02gac', _iter, '_iter')
    e02gac_ctypes(order, m, _a, _b, nplus2, toler, _x, _resid, _rank, __iter, fail)

e02gcc_ctypes = _library_cl.e02gcc
e02gcc_ctypes.restype = None
e02gcc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def e02gcc(order, m, n, a, b, tol, relerr, x, resmax, rank, _iter, fail):
    """
    L_infinity-approximation by general linear function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02gcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, relerr, x, resmax, rank, _iter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, relerr, x, resmax
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank, _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02gcc', a, 'a')
    _b = nag_double_type_check_and_cast('e02gcc', b, 'b')
    _relerr = nag_double_type_check_and_cast('e02gcc', relerr, 'relerr')
    _x = nag_double_type_check_and_cast('e02gcc', x, 'x')
    _resmax = nag_double_type_check_and_cast('e02gcc', resmax, 'resmax')
    _rank = nag_integer_type_check_and_cast('e02gcc', rank, 'rank')
    __iter = nag_integer_type_check_and_cast('e02gcc', _iter, '_iter')
    e02gcc_ctypes(order, m, n, _a, _b, tol, _relerr, _x, _resmax, _rank, __iter, fail)

e02jdc_ctypes = _library_cl.e02jdc
e02jdc_ctypes.restype = None
e02jdc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e02jdc(n, x, y, f, lsminp, lsmaxp, nxcels, nycels, lcoefs, coefs, iopts, opts, fail):
    """
    Spline approximation to a set of scattered data using a two-stage
    approximation method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, coefs, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, coefs, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e02jdc', x, 'x')
    _y = nag_double_type_check_and_cast('e02jdc', y, 'y')
    _f = nag_double_type_check_and_cast('e02jdc', f, 'f')
    _coefs = nag_double_type_check_and_cast('e02jdc', coefs, 'coefs')
    _iopts = nag_integer_type_check_and_cast('e02jdc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e02jdc', opts, 'opts')
    e02jdc_ctypes(n, _x, _y, _f, lsminp, lsmaxp, nxcels, nycels, lcoefs, _coefs, _iopts, _opts, fail)

e02jec_ctypes = _library_cl.e02jec
e02jec_ctypes.restype = None
e02jec_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e02jec(nevalv, xevalv, yevalv, coefs, fevalv, iopts, opts, fail):
    """
    Evaluation at a vector of points of a spline computed by e02jdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02jec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xevalv, yevalv, coefs, fevalv, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xevalv, yevalv, coefs, fevalv, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _xevalv = nag_double_type_check_and_cast('e02jec', xevalv, 'xevalv')
    _yevalv = nag_double_type_check_and_cast('e02jec', yevalv, 'yevalv')
    _coefs = nag_double_type_check_and_cast('e02jec', coefs, 'coefs')
    _fevalv = nag_double_type_check_and_cast('e02jec', fevalv, 'fevalv')
    _iopts = nag_integer_type_check_and_cast('e02jec', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e02jec', opts, 'opts')
    e02jec_ctypes(nevalv, _xevalv, _yevalv, _coefs, _fevalv, _iopts, _opts, fail)

e02jfc_ctypes = _library_cl.e02jfc
e02jfc_ctypes.restype = None
e02jfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e02jfc(nxeval, nyeval, xevalm, yevalm, coefs, fevalm, iopts, opts, fail):
    """
    Evaluation at a mesh of points of a spline computed by e02jdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02jfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xevalm, yevalm, coefs, fevalm, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xevalm, yevalm, coefs, fevalm, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _xevalm = nag_double_type_check_and_cast('e02jfc', xevalm, 'xevalm')
    _yevalm = nag_double_type_check_and_cast('e02jfc', yevalm, 'yevalm')
    _coefs = nag_double_type_check_and_cast('e02jfc', coefs, 'coefs')
    _fevalm = nag_double_type_check_and_cast('e02jfc', fevalm, 'fevalm')
    _iopts = nag_integer_type_check_and_cast('e02jfc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e02jfc', opts, 'opts')
    e02jfc_ctypes(nxeval, nyeval, _xevalm, _yevalm, _coefs, _fevalm, _iopts, _opts, fail)

e02rac_ctypes = _library_cl.e02rac
e02rac_ctypes.restype = None
e02rac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e02rac(ia, ib, c, a, b, fail):
    """
    Pade approximants.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        c, a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        c, a, b
    """
    from .util import nag_double_type_check_and_cast
    _c = nag_double_type_check_and_cast('e02rac', c, 'c')
    _a = nag_double_type_check_and_cast('e02rac', a, 'a')
    _b = nag_double_type_check_and_cast('e02rac', b, 'b')
    e02rac_ctypes(ia, ib, _c, _a, _b, fail)

e02rbc_ctypes = _library_cl.e02rbc
e02rbc_ctypes.restype = None
e02rbc_ctypes.argtypes = [POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def e02rbc(a, ia, b, ib, x, ans, fail):
    """
    Evaluation of fitted rational function as computed by e02rac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02rbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, ans
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, ans
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e02rbc', a, 'a')
    _b = nag_double_type_check_and_cast('e02rbc', b, 'b')
    _ans = nag_double_type_check_and_cast('e02rbc', ans, 'ans')
    e02rbc_ctypes(_a, ia, _b, ib, x, _ans, fail)

e02zac_ctypes = _library_cl.e02zac
e02zac_ctypes.restype = None
e02zac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def e02zac(px, py, lamda, mu, m, x, y, point, fail):
    """
    Sort two-dimensional data into panels for fitting bicubic splines.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02zac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lamda, mu, x, y, point
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lamda, mu, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        point
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lamda = nag_double_type_check_and_cast('e02zac', lamda, 'lamda')
    _mu = nag_double_type_check_and_cast('e02zac', mu, 'mu')
    _x = nag_double_type_check_and_cast('e02zac', x, 'x')
    _y = nag_double_type_check_and_cast('e02zac', y, 'y')
    _point = nag_integer_type_check_and_cast('e02zac', point, 'point')
    e02zac_ctypes(px, py, _lamda, _mu, m, _x, _y, _point, fail)

e02zkc_ctypes = _library_cl.e02zkc
e02zkc_ctypes.restype = None
e02zkc_ctypes.argtypes = [STRING, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def e02zkc(optstr, iopts, liopts, opts, lopts, fail):
    """
    Option setting routine.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02zkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iopts = nag_integer_type_check_and_cast('e02zkc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e02zkc', opts, 'opts')
    e02zkc_ctypes(optstr, _iopts, liopts, _opts, lopts, fail)

e02zlc_ctypes = _library_cl.e02zlc
e02zlc_ctypes.restype = None
e02zlc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(c_double), STRING, Integer, POINTER(Nag_VariableType), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e02zlc(optstr, ivalue, rvalue, cvalue, lcvalue, optype, iopts, opts, fail):
    """
    Option getting routine.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e02/e02zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e02zlc', ivalue, 'ivalue')
    _rvalue = nag_double_type_check_and_cast('e02zlc', rvalue, 'rvalue')
    _iopts = nag_integer_type_check_and_cast('e02zlc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e02zlc', opts, 'opts')
    e02zlc_ctypes(optstr, _ivalue, _rvalue, cvalue, lcvalue, optype, _iopts, _opts, fail)

nag_1d_cheb_fit = e02adc
nag_1d_cheb_eval = e02aec
nag_1d_cheb_interp_fit = e02afc
nag_1d_cheb_fit_constr = e02agc
nag_1d_cheb_deriv = e02ahc
nag_1d_cheb_intg = e02ajc
nag_1d_cheb_eval2 = e02akc
nag_1d_minimax_polynomial = e02alc
nag_1d_spline_fit_knots = e02bac
nag_1d_spline_evaluate = e02bbc
nag_1d_spline_deriv = e02bcc
nag_1d_spline_intg = e02bdc
nag_1d_spline_fit = e02bec
nag_fit_1dspline_deriv_vector = e02bfc
nag_2d_cheb_fit_lines = e02cac
nag_2d_cheb_eval = e02cbc
nag_2d_spline_fit_panel = e02dac
nag_2d_spline_fit_grid = e02dcc
nag_2d_spline_fit_scat = e02ddc
nag_2d_spline_eval = e02dec
nag_2d_spline_eval_rect = e02dfc
nag_2d_spline_deriv_rect = e02dhc
nag_lone_fit = e02gac
nag_linf_fit = e02gcc
nag_2d_spline_fit_ts_scat = e02jdc
nag_2d_spline_ts_eval = e02jec
nag_2d_spline_ts_eval_rect = e02jfc
nag_1d_pade = e02rac
nag_1d_pade_eval = e02rbc
nag_2d_panel_sort = e02zac
nag_fit_opt_set = e02zkc
nag_fit_opt_get = e02zlc
__all__ = ['e02adc', 'e02aec', 'e02afc', 'e02agc', 'e02ahc', 'e02ajc',
           'e02akc', 'e02alc', 'e02bac', 'e02bbc', 'e02bcc', 'e02bdc',
           'e02bec', 'e02bfc', 'e02cac', 'e02cbc', 'e02dac', 'e02dcc',
           'e02ddc', 'e02dec', 'e02dfc', 'e02dhc', 'e02gac',
           'e02gcc', 'e02jdc', 'e02jec', 'e02jfc', 'e02rac', 'e02rbc',
           'e02zac', 'e02zkc', 'e02zlc', 'nag_1d_cheb_deriv',
           'nag_1d_cheb_eval', 'nag_1d_cheb_eval2', 'nag_1d_cheb_fit',
           'nag_1d_cheb_fit_constr', 'nag_1d_cheb_interp_fit',
           'nag_1d_cheb_intg', 'nag_1d_minimax_polynomial',
           'nag_1d_pade', 'nag_1d_pade_eval', 'nag_1d_spline_deriv',
           'nag_1d_spline_evaluate', 'nag_1d_spline_fit',
           'nag_1d_spline_fit_knots', 'nag_1d_spline_intg',
           'nag_2d_cheb_eval', 'nag_2d_cheb_fit_lines',
           'nag_2d_panel_sort', 'nag_2d_spline_deriv_rect',
           'nag_2d_spline_eval', 'nag_2d_spline_eval_rect',
           'nag_2d_spline_fit_grid', 'nag_2d_spline_fit_panel',
           'nag_2d_spline_fit_scat', 'nag_2d_spline_fit_ts_scat',
           'nag_2d_spline_ts_eval', 'nag_2d_spline_ts_eval_rect',
           'nag_fit_1dspline_deriv_vector',
           'nag_fit_opt_get', 'nag_fit_opt_set', 'nag_linf_fit',
           'nag_lone_fit']
