"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f04 Chapter.

f04 - Simultaneous Linear Equations.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F04/f04conts.html

NAG Copyright 2017.
"""
from ctypes import POINTER
from ctypes import c_double
from .util import get_library
from .util import Integer
from .util import Complex
from .util import NagError
from .util import Nag_OrderType
from .util import Nag_UploType
from .util import Nag_SolveSystem

_library_cl = get_library()

f04bac_ctypes = _library_cl.f04bac
f04bac_ctypes.restype = None
f04bac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bac(order, n, nrhs, a, pda, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f04bac', a, 'a')
    _ipiv = nag_integer_type_check_and_cast('f04bac', ipiv, 'ipiv')
    _b = nag_double_type_check_and_cast('f04bac', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bac', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bac', errbnd, 'errbnd')
    f04bac_ctypes(order, n, nrhs, _a, pda, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04bbc_ctypes = _library_cl.f04bbc
f04bbc_ctypes.restype = None
f04bbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bbc(order, n, kl, ku, nrhs, ab, pdab, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real banded system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f04bbc', ab, 'ab')
    _ipiv = nag_integer_type_check_and_cast('f04bbc', ipiv, 'ipiv')
    _b = nag_double_type_check_and_cast('f04bbc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bbc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bbc', errbnd, 'errbnd')
    f04bbc_ctypes(order, n, kl, ku, nrhs, _ab, pdab, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04bcc_ctypes = _library_cl.f04bcc
f04bcc_ctypes.restype = None
f04bcc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bcc(order, n, nrhs, dl, d, du, du2, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real tridiagonal system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dl, d, du, du2, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dl, d, du, du2, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dl = nag_double_type_check_and_cast('f04bcc', dl, 'dl')
    _d = nag_double_type_check_and_cast('f04bcc', d, 'd')
    _du = nag_double_type_check_and_cast('f04bcc', du, 'du')
    _du2 = nag_double_type_check_and_cast('f04bcc', du2, 'du2')
    _ipiv = nag_integer_type_check_and_cast('f04bcc', ipiv, 'ipiv')
    _b = nag_double_type_check_and_cast('f04bcc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bcc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bcc', errbnd, 'errbnd')
    f04bcc_ctypes(order, n, nrhs, _dl, _d, _du, _du2, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04bdc_ctypes = _library_cl.f04bdc
f04bdc_ctypes.restype = None
f04bdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bdc(order, uplo, n, nrhs, a, pda, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric positive definite system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, rcond, errbnd
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f04bdc', a, 'a')
    _b = nag_double_type_check_and_cast('f04bdc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bdc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bdc', errbnd, 'errbnd')
    f04bdc_ctypes(order, uplo, n, nrhs, _a, pda, _b, pdb, _rcond, _errbnd, fail)

f04bec_ctypes = _library_cl.f04bec
f04bec_ctypes.restype = None
f04bec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bec(order, uplo, n, nrhs, ap, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric positive definite system of linear equations, packed
    storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, b, rcond, errbnd
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f04bec', ap, 'ap')
    _b = nag_double_type_check_and_cast('f04bec', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bec', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bec', errbnd, 'errbnd')
    f04bec_ctypes(order, uplo, n, nrhs, _ap, _b, pdb, _rcond, _errbnd, fail)

f04bfc_ctypes = _library_cl.f04bfc
f04bfc_ctypes.restype = None
f04bfc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bfc(order, uplo, n, kd, nrhs, ab, pdab, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric positive definite banded system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, b, rcond, errbnd
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f04bfc', ab, 'ab')
    _b = nag_double_type_check_and_cast('f04bfc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bfc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bfc', errbnd, 'errbnd')
    f04bfc_ctypes(order, uplo, n, kd, nrhs, _ab, pdab, _b, pdb, _rcond, _errbnd, fail)

f04bgc_ctypes = _library_cl.f04bgc
f04bgc_ctypes.restype = None
f04bgc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bgc(order, n, nrhs, d, e, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric positive definite tridiagonal system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, b, rcond, errbnd
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f04bgc', d, 'd')
    _e = nag_double_type_check_and_cast('f04bgc', e, 'e')
    _b = nag_double_type_check_and_cast('f04bgc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bgc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bgc', errbnd, 'errbnd')
    f04bgc_ctypes(order, n, nrhs, _d, _e, _b, pdb, _rcond, _errbnd, fail)

f04bhc_ctypes = _library_cl.f04bhc
f04bhc_ctypes.restype = None
f04bhc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bhc(order, uplo, n, nrhs, a, pda, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f04bhc', a, 'a')
    _ipiv = nag_integer_type_check_and_cast('f04bhc', ipiv, 'ipiv')
    _b = nag_double_type_check_and_cast('f04bhc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bhc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bhc', errbnd, 'errbnd')
    f04bhc_ctypes(order, uplo, n, nrhs, _a, pda, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04bjc_ctypes = _library_cl.f04bjc
f04bjc_ctypes.restype = None
f04bjc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04bjc(order, uplo, n, nrhs, ap, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    real symmetric system of linear equations, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04bjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f04bjc', ap, 'ap')
    _ipiv = nag_integer_type_check_and_cast('f04bjc', ipiv, 'ipiv')
    _b = nag_double_type_check_and_cast('f04bjc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04bjc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04bjc', errbnd, 'errbnd')
    f04bjc_ctypes(order, uplo, n, nrhs, _ap, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04cac_ctypes = _library_cl.f04cac
f04cac_ctypes.restype = None
f04cac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cac(order, n, nrhs, a, pda, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f04cac', a, 'a')
    _ipiv = nag_integer_type_check_and_cast('f04cac', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04cac', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cac', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cac', errbnd, 'errbnd')
    f04cac_ctypes(order, n, nrhs, _a, pda, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04cbc_ctypes = _library_cl.f04cbc
f04cbc_ctypes.restype = None
f04cbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cbc(order, n, kl, ku, nrhs, ab, pdab, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex banded system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f04cbc', ab, 'ab')
    _ipiv = nag_integer_type_check_and_cast('f04cbc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04cbc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cbc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cbc', errbnd, 'errbnd')
    f04cbc_ctypes(order, n, kl, ku, nrhs, _ab, pdab, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04ccc_ctypes = _library_cl.f04ccc
f04ccc_ctypes.restype = None
f04ccc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04ccc(order, n, nrhs, dl, d, du, du2, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex tridiagonal system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dl, d, du, du2, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        dl, d, du, du2, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dl = nag_complex_type_check_and_cast('f04ccc', dl, 'dl')
    _d = nag_complex_type_check_and_cast('f04ccc', d, 'd')
    _du = nag_complex_type_check_and_cast('f04ccc', du, 'du')
    _du2 = nag_complex_type_check_and_cast('f04ccc', du2, 'du2')
    _ipiv = nag_integer_type_check_and_cast('f04ccc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04ccc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04ccc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04ccc', errbnd, 'errbnd')
    f04ccc_ctypes(order, n, nrhs, _dl, _d, _du, _du2, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04cdc_ctypes = _library_cl.f04cdc
f04cdc_ctypes.restype = None
f04cdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cdc(order, uplo, n, nrhs, a, pda, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex Hermitian positive definite system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f04cdc', a, 'a')
    _b = nag_complex_type_check_and_cast('f04cdc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cdc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cdc', errbnd, 'errbnd')
    f04cdc_ctypes(order, uplo, n, nrhs, _a, pda, _b, pdb, _rcond, _errbnd, fail)

f04cec_ctypes = _library_cl.f04cec
f04cec_ctypes.restype = None
f04cec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cec(order, uplo, n, nrhs, ap, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex Hermitian positive definite system of linear equations, packed
    storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f04cec', ap, 'ap')
    _b = nag_complex_type_check_and_cast('f04cec', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cec', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cec', errbnd, 'errbnd')
    f04cec_ctypes(order, uplo, n, nrhs, _ap, _b, pdb, _rcond, _errbnd, fail)

f04cfc_ctypes = _library_cl.f04cfc
f04cfc_ctypes.restype = None
f04cfc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cfc(order, uplo, n, kd, nrhs, ab, pdab, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex Hermitian positive definite banded system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f04cfc', ab, 'ab')
    _b = nag_complex_type_check_and_cast('f04cfc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cfc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cfc', errbnd, 'errbnd')
    f04cfc_ctypes(order, uplo, n, kd, nrhs, _ab, pdab, _b, pdb, _rcond, _errbnd, fail)

f04cgc_ctypes = _library_cl.f04cgc
f04cgc_ctypes.restype = None
f04cgc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), POINTER(Complex), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cgc(order, n, nrhs, d, e, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex Hermitian positive definite tridiagonal system of linear
    equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    Complex:
        e, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f04cgc', d, 'd')
    _e = nag_complex_type_check_and_cast('f04cgc', e, 'e')
    _b = nag_complex_type_check_and_cast('f04cgc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cgc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cgc', errbnd, 'errbnd')
    f04cgc_ctypes(order, n, nrhs, _d, _e, _b, pdb, _rcond, _errbnd, fail)

f04chc_ctypes = _library_cl.f04chc
f04chc_ctypes.restype = None
f04chc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04chc(order, uplo, n, nrhs, a, pda, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution and error-bound to a complex Hermitian system of
    linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04chc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f04chc', a, 'a')
    _ipiv = nag_integer_type_check_and_cast('f04chc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04chc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04chc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04chc', errbnd, 'errbnd')
    f04chc_ctypes(order, uplo, n, nrhs, _a, pda, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04cjc_ctypes = _library_cl.f04cjc
f04cjc_ctypes.restype = None
f04cjc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04cjc(order, uplo, n, nrhs, ap, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex Hermitian system of linear equations, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04cjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f04cjc', ap, 'ap')
    _ipiv = nag_integer_type_check_and_cast('f04cjc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04cjc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04cjc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04cjc', errbnd, 'errbnd')
    f04cjc_ctypes(order, uplo, n, nrhs, _ap, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04dhc_ctypes = _library_cl.f04dhc
f04dhc_ctypes.restype = None
f04dhc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04dhc(order, uplo, n, nrhs, a, pda, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex symmetric system of linear equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04dhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f04dhc', a, 'a')
    _ipiv = nag_integer_type_check_and_cast('f04dhc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04dhc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04dhc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04dhc', errbnd, 'errbnd')
    f04dhc_ctypes(order, uplo, n, nrhs, _a, pda, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04djc_ctypes = _library_cl.f04djc
f04djc_ctypes.restype = None
f04djc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f04djc(order, uplo, n, nrhs, ap, ipiv, b, pdb, rcond, errbnd, fail):
    """
    Computes the solution, estimated condition number and error-bound to a
    complex symmetric system of linear equations, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04djc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, ipiv, b, rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rcond, errbnd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ipiv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f04djc', ap, 'ap')
    _ipiv = nag_integer_type_check_and_cast('f04djc', ipiv, 'ipiv')
    _b = nag_complex_type_check_and_cast('f04djc', b, 'b')
    _rcond = nag_double_type_check_and_cast('f04djc', rcond, 'rcond')
    _errbnd = nag_double_type_check_and_cast('f04djc', errbnd, 'errbnd')
    f04djc_ctypes(order, uplo, n, nrhs, _ap, _ipiv, _b, pdb, _rcond, _errbnd, fail)

f04mcc_ctypes = _library_cl.f04mcc
f04mcc_ctypes.restype = None
f04mcc_ctypes.argtypes = [Nag_SolveSystem, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f04mcc(selct, n, nrhs, al, lal, d, row, b, tdb, x, tdx, fail):
    """
    Approximate solution of real symmetric positive definite variable-
    bandwidth simultaneous linear equations (coefficient matrix already
    factorized by f01mcc).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04mcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        al, d, row, b, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        al, d, b, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        row
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _al = nag_double_type_check_and_cast('f04mcc', al, 'al')
    _d = nag_double_type_check_and_cast('f04mcc', d, 'd')
    _row = nag_integer_type_check_and_cast('f04mcc', row, 'row')
    _b = nag_double_type_check_and_cast('f04mcc', b, 'b')
    _x = nag_double_type_check_and_cast('f04mcc', x, 'x')
    f04mcc_ctypes(selct, n, nrhs, _al, lal, _d, _row, _b, tdb, _x, tdx, fail)

f04ydc_ctypes = _library_cl.f04ydc
f04ydc_ctypes.restype = None
f04ydc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f04ydc(irevcm, m, n, x, pdx, y, pdy, estnrm, t, seed, work, iwork, fail):
    """
    Norm estimation (for use in condition estimation), real rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04ydc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, x, y, estnrm, work, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, estnrm, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, iwork
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f04ydc', irevcm, 'irevcm')
    _x = nag_double_type_check_and_cast('f04ydc', x, 'x')
    _y = nag_double_type_check_and_cast('f04ydc', y, 'y')
    _estnrm = nag_double_type_check_and_cast('f04ydc', estnrm, 'estnrm')
    _work = nag_double_type_check_and_cast('f04ydc', work, 'work')
    _iwork = nag_integer_type_check_and_cast('f04ydc', iwork, 'iwork')
    f04ydc_ctypes(_irevcm, m, n, _x, pdx, _y, pdy, _estnrm, t, seed, _work, _iwork, fail)

f04zdc_ctypes = _library_cl.f04zdc
f04zdc_ctypes.restype = None
f04zdc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), Integer, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f04zdc(irevcm, m, n, x, pdx, y, pdy, estnrm, t, seed, work, rwork, iwork, fail):
    """
    Norm estimation (for use in condition estimation), complex rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f04/f04zdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, x, y, estnrm, work, rwork, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        estnrm, rwork
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, iwork
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y, work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f04zdc', irevcm, 'irevcm')
    _x = nag_complex_type_check_and_cast('f04zdc', x, 'x')
    _y = nag_complex_type_check_and_cast('f04zdc', y, 'y')
    _estnrm = nag_double_type_check_and_cast('f04zdc', estnrm, 'estnrm')
    _work = nag_complex_type_check_and_cast('f04zdc', work, 'work')
    _rwork = nag_double_type_check_and_cast('f04zdc', rwork, 'rwork')
    _iwork = nag_integer_type_check_and_cast('f04zdc', iwork, 'iwork')
    f04zdc_ctypes(_irevcm, m, n, _x, pdx, _y, pdy, _estnrm, t, seed, _work, _rwork, _iwork, fail)

nag_real_gen_lin_solve = f04bac
nag_real_band_lin_solve = f04bbc
nag_real_tridiag_lin_solve = f04bcc
nag_real_sym_posdef_lin_solve = f04bdc
nag_real_sym_posdef_packed_lin_solve = f04bec
nag_real_sym_posdef_band_lin_solve = f04bfc
nag_real_sym_posdef_tridiag_lin_solve = f04bgc
nag_real_sym_lin_solve = f04bhc
nag_real_sym_packed_lin_solve = f04bjc
nag_complex_gen_lin_solve = f04cac
nag_complex_band_lin_solve = f04cbc
nag_complex_tridiag_lin_solve = f04ccc
nag_herm_posdef_lin_solve = f04cdc
nag_herm_posdef_packed_lin_solve = f04cec
nag_herm_posdef_band_lin_solve = f04cfc
nag_herm_posdef_tridiag_lin_solve = f04cgc
nag_herm_lin_solve = f04chc
nag_herm_packed_lin_solve = f04cjc
nag_complex_sym_lin_solve = f04dhc
nag_complex_sym_packed_lin_solve = f04djc
nag_real_cholesky_skyline_solve = f04mcc
nag_linsys_real_gen_norm_rcomm = f04ydc
nag_linsys_complex_gen_norm_rcomm = f04zdc
__all__ = ['f04bac', 'f04bbc', 'f04bcc', 'f04bdc', 'f04bec', 'f04bfc',
           'f04bgc', 'f04bhc', 'f04bjc', 'f04cac', 'f04cbc', 'f04ccc',
           'f04cdc', 'f04cec', 'f04cfc', 'f04cgc', 'f04chc', 'f04cjc',
           'f04dhc', 'f04djc', 'f04mcc',
           'f04ydc', 'f04zdc', 'nag_complex_band_lin_solve',
           'nag_complex_gen_lin_solve',
           'nag_complex_sym_lin_solve',
           'nag_complex_sym_packed_lin_solve',
           'nag_complex_tridiag_lin_solve', 'nag_herm_lin_solve',
           'nag_herm_packed_lin_solve',
           'nag_herm_posdef_band_lin_solve',
           'nag_herm_posdef_lin_solve',
           'nag_herm_posdef_packed_lin_solve',
           'nag_herm_posdef_tridiag_lin_solve',
           'nag_linsys_complex_gen_norm_rcomm',
           'nag_linsys_real_gen_norm_rcomm',
           'nag_real_band_lin_solve',
           'nag_real_cholesky_skyline_solve',
           'nag_real_gen_lin_solve',
           'nag_real_sym_lin_solve',
           'nag_real_sym_packed_lin_solve',
           'nag_real_sym_posdef_band_lin_solve',
           'nag_real_sym_posdef_lin_solve',
           'nag_real_sym_posdef_packed_lin_solve',
           'nag_real_sym_posdef_tridiag_lin_solve',
           'nag_real_tridiag_lin_solve']
