"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g07 Chapter.

g07 - Univariate Estimation.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G07/g07conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER

from .util import get_library

from .util import Integer
from .util import NagError
from .util import Nag_CEMethod
from .util import Nag_CestMethod
from .util import Nag_OptimOpt
from .util import Nag_TailProbability
from .util import Nag_PopVar
from .util import Nag_SigmaSimulEst
from .util import Nag_PsiFun
from .util import NAG_G07DCC_CHI
from .util import NAG_G07DCC_PSI
from .util import Nag_Comm
from .util import Nag_RCIMethod
from .util import Nag_Boolean

_library_cl = get_library()

g07aac_ctypes = _library_cl.g07aac
g07aac_ctypes.restype = None
g07aac_ctypes.argtypes = [Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07aac(n, k, clevel, pl, pu, fail):
    """
    Computes confidence interval for the parameter of a binomial
    distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        pl, pu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        pl, pu
    """
    from .util import nag_double_type_check_and_cast
    _pl = nag_double_type_check_and_cast('g07aac', pl, 'pl')
    _pu = nag_double_type_check_and_cast('g07aac', pu, 'pu')
    g07aac_ctypes(n, k, clevel, _pl, _pu, fail)

g07abc_ctypes = _library_cl.g07abc
g07abc_ctypes.restype = None
g07abc_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07abc(n, xmean, clevel, tl, tu, fail):
    """
    Computes confidence interval for the parameter of a Poisson
    distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        tl, tu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        tl, tu
    """
    from .util import nag_double_type_check_and_cast
    _tl = nag_double_type_check_and_cast('g07abc', tl, 'tl')
    _tu = nag_double_type_check_and_cast('g07abc', tu, 'tu')
    g07abc_ctypes(n, xmean, clevel, _tl, _tu, fail)

g07bbc_ctypes = _library_cl.g07bbc
g07bbc_ctypes.restype = None
g07bbc_ctypes.argtypes = [Nag_CEMethod, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def g07bbc(method, n, x, xc, ic, xmu, xsig, tol, maxit, sexmu, sexsig, corr, dev, nobs, nit, fail):
    """
    Computes maximum likelihood estimates for parameters of the Normal
    distribution from grouped and/or censored data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, xc, ic, xmu, xsig, sexmu, sexsig, corr, dev, nobs, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, xc, xmu, xsig, sexmu, sexsig, corr, dev
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ic, nobs, nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07bbc', x, 'x')
    _xc = nag_double_type_check_and_cast('g07bbc', xc, 'xc')
    _ic = nag_integer_type_check_and_cast('g07bbc', ic, 'ic')
    _xmu = nag_double_type_check_and_cast('g07bbc', xmu, 'xmu')
    _xsig = nag_double_type_check_and_cast('g07bbc', xsig, 'xsig')
    _sexmu = nag_double_type_check_and_cast('g07bbc', sexmu, 'sexmu')
    _sexsig = nag_double_type_check_and_cast('g07bbc', sexsig, 'sexsig')
    _corr = nag_double_type_check_and_cast('g07bbc', corr, 'corr')
    _dev = nag_double_type_check_and_cast('g07bbc', dev, 'dev')
    _nobs = nag_integer_type_check_and_cast('g07bbc', nobs, 'nobs')
    _nit = nag_integer_type_check_and_cast('g07bbc', nit, 'nit')
    g07bbc_ctypes(method, n, _x, _xc, _ic, _xmu, _xsig, tol, maxit, _sexmu, _sexsig, _corr, _dev, _nobs, _nit, fail)

g07bec_ctypes = _library_cl.g07bec
g07bec_ctypes.restype = None
g07bec_ctypes.argtypes = [Nag_CestMethod, Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g07bec(cens, n, x, ic, beta, gamma, tol, maxit, sebeta, segam, corr, dev, nit, fail):
    """
    Computes maximum likelihood estimates for parameters of the Weibull
    distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ic, beta, gamma, sebeta, segam, corr, dev, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, beta, gamma, sebeta, segam, corr, dev
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ic, nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07bec', x, 'x')
    _ic = nag_integer_type_check_and_cast('g07bec', ic, 'ic')
    _beta = nag_double_type_check_and_cast('g07bec', beta, 'beta')
    _gamma = nag_double_type_check_and_cast('g07bec', gamma, 'gamma')
    _sebeta = nag_double_type_check_and_cast('g07bec', sebeta, 'sebeta')
    _segam = nag_double_type_check_and_cast('g07bec', segam, 'segam')
    _corr = nag_double_type_check_and_cast('g07bec', corr, 'corr')
    _dev = nag_double_type_check_and_cast('g07bec', dev, 'dev')
    _nit = nag_integer_type_check_and_cast('g07bec', nit, 'nit')
    g07bec_ctypes(cens, n, _x, _ic, _beta, _gamma, tol, maxit, _sebeta, _segam, _corr, _dev, _nit, fail)

g07bfc_ctypes = _library_cl.g07bfc
g07bfc_ctypes.restype = None
g07bfc_ctypes.argtypes = [Integer, POINTER(c_double), Nag_OptimOpt, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07bfc(n, y, optopt, xi, beta, asvc, obsvc, ll, fail):
    """
    Estimates parameter values of the generalized Pareto distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, xi, beta, asvc, obsvc, ll
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, xi, beta, asvc, obsvc, ll
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g07bfc', y, 'y')
    _xi = nag_double_type_check_and_cast('g07bfc', xi, 'xi')
    _beta = nag_double_type_check_and_cast('g07bfc', beta, 'beta')
    _asvc = nag_double_type_check_and_cast('g07bfc', asvc, 'asvc')
    _obsvc = nag_double_type_check_and_cast('g07bfc', obsvc, 'obsvc')
    _ll = nag_double_type_check_and_cast('g07bfc', ll, 'll')
    g07bfc_ctypes(n, _y, optopt, _xi, _beta, _asvc, _obsvc, _ll, fail)

g07cac_ctypes = _library_cl.g07cac
g07cac_ctypes.restype = None
g07cac_ctypes.argtypes = [Nag_TailProbability, Nag_PopVar, Integer, Integer, c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07cac(tail, equal, nx, ny, xmean, ymean, xstd, ystd, clevel, t, df, prob, dl, du, fail):
    """
    Computes t-test statistic for a difference in means between two Normal
    populations, confidence interval.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, df, prob, dl, du
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, df, prob, dl, du
    """
    from .util import nag_double_type_check_and_cast
    _t = nag_double_type_check_and_cast('g07cac', t, 't')
    _df = nag_double_type_check_and_cast('g07cac', df, 'df')
    _prob = nag_double_type_check_and_cast('g07cac', prob, 'prob')
    _dl = nag_double_type_check_and_cast('g07cac', dl, 'dl')
    _du = nag_double_type_check_and_cast('g07cac', du, 'du')
    g07cac_ctypes(tail, equal, nx, ny, xmean, ymean, xstd, ystd, clevel, _t, _df, _prob, _dl, _du, fail)

g07dac_ctypes = _library_cl.g07dac
g07dac_ctypes.restype = None
g07dac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07dac(n, x, y, xme, xmd, xsd, fail):
    """
    Robust estimation, median, median absolute deviation, robust standard
    deviation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, xme, xmd, xsd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, xme, xmd, xsd
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07dac', x, 'x')
    _y = nag_double_type_check_and_cast('g07dac', y, 'y')
    _xme = nag_double_type_check_and_cast('g07dac', xme, 'xme')
    _xmd = nag_double_type_check_and_cast('g07dac', xmd, 'xmd')
    _xsd = nag_double_type_check_and_cast('g07dac', xsd, 'xsd')
    g07dac_ctypes(n, _x, _y, _xme, _xmd, _xsd, fail)

g07dbc_ctypes = _library_cl.g07dbc
g07dbc_ctypes.restype = None
g07dbc_ctypes.argtypes = [Nag_SigmaSimulEst, Integer, POINTER(c_double), Nag_PsiFun, c_double, c_double, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g07dbc(sigmaest, n, x, psifun, c, h1, h2, h3, dchi, theta, sigma, maxit, tol, rs, nit, sortedx, fail):
    """
    Robust estimation, M-estimates for location and scale parameters,
    standard weight functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, theta, sigma, rs, nit, sortedx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, theta, sigma, rs, sortedx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07dbc', x, 'x')
    _theta = nag_double_type_check_and_cast('g07dbc', theta, 'theta')
    _sigma = nag_double_type_check_and_cast('g07dbc', sigma, 'sigma')
    _rs = nag_double_type_check_and_cast('g07dbc', rs, 'rs')
    _nit = nag_integer_type_check_and_cast('g07dbc', nit, 'nit')
    _sortedx = nag_double_type_check_and_cast('g07dbc', sortedx, 'sortedx')
    g07dbc_ctypes(sigmaest, n, _x, psifun, c, h1, h2, h3, dchi, _theta, _sigma, maxit, tol, _rs, _nit, _sortedx, fail)

g07dcc_ctypes = _library_cl.g07dcc
g07dcc_ctypes.restype = None
g07dcc_ctypes.argtypes = [NAG_G07DCC_CHI, NAG_G07DCC_PSI, Integer, Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g07dcc(chi, psi, isigma, n, x, beta, theta, sigma, maxit, tol, rs, nit, comm, fail):
    """
    Robust estimation, M-estimates for location and scale parameters, user-
    defined weight functions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, theta, sigma, rs, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, theta, sigma, rs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07dcc', x, 'x')
    _theta = nag_double_type_check_and_cast('g07dcc', theta, 'theta')
    _sigma = nag_double_type_check_and_cast('g07dcc', sigma, 'sigma')
    _rs = nag_double_type_check_and_cast('g07dcc', rs, 'rs')
    _nit = nag_integer_type_check_and_cast('g07dcc', nit, 'nit')
    g07dcc_ctypes(chi, psi, isigma, n, _x, beta, _theta, _sigma, maxit, tol, _rs, _nit, comm, fail)

g07ddc_ctypes = _library_cl.g07ddc
g07ddc_ctypes.restype = None
g07ddc_ctypes.argtypes = [Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g07ddc(n, x, alpha, tmean, wmean, tvar, wvar, k, sx, fail):
    """
    Trimmed and winsorized mean of a sample with estimates of the variances
    of the two means.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, tmean, wmean, tvar, wvar, k, sx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, tmean, wmean, tvar, wvar, sx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07ddc', x, 'x')
    _tmean = nag_double_type_check_and_cast('g07ddc', tmean, 'tmean')
    _wmean = nag_double_type_check_and_cast('g07ddc', wmean, 'wmean')
    _tvar = nag_double_type_check_and_cast('g07ddc', tvar, 'tvar')
    _wvar = nag_double_type_check_and_cast('g07ddc', wvar, 'wvar')
    _k = nag_integer_type_check_and_cast('g07ddc', k, 'k')
    _sx = nag_double_type_check_and_cast('g07ddc', sx, 'sx')
    g07ddc_ctypes(n, _x, alpha, _tmean, _wmean, _tvar, _wvar, _k, _sx, fail)

g07eac_ctypes = _library_cl.g07eac
g07eac_ctypes.restype = None
g07eac_ctypes.argtypes = [Nag_RCIMethod, Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07eac(method, n, x, clevel, theta, thetal, thetau, estcl, wlower, wupper, fail):
    """
    Robust confidence intervals, one-sample.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, theta, thetal, thetau, estcl, wlower, wupper
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, theta, thetal, thetau, estcl, wlower, wupper
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07eac', x, 'x')
    _theta = nag_double_type_check_and_cast('g07eac', theta, 'theta')
    _thetal = nag_double_type_check_and_cast('g07eac', thetal, 'thetal')
    _thetau = nag_double_type_check_and_cast('g07eac', thetau, 'thetau')
    _estcl = nag_double_type_check_and_cast('g07eac', estcl, 'estcl')
    _wlower = nag_double_type_check_and_cast('g07eac', wlower, 'wlower')
    _wupper = nag_double_type_check_and_cast('g07eac', wupper, 'wupper')
    g07eac_ctypes(method, n, _x, clevel, _theta, _thetal, _thetau, _estcl, _wlower, _wupper, fail)

g07ebc_ctypes = _library_cl.g07ebc
g07ebc_ctypes.restype = None
g07ebc_ctypes.argtypes = [Nag_RCIMethod, Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07ebc(method, n, x, m, y, clevel, theta, thetal, thetau, estcl, ulower, uupper, fail):
    """
    Robust confidence intervals, two-sample.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07ebc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, theta, thetal, thetau, estcl, ulower, uupper
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, theta, thetal, thetau, estcl, ulower, uupper
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07ebc', x, 'x')
    _y = nag_double_type_check_and_cast('g07ebc', y, 'y')
    _theta = nag_double_type_check_and_cast('g07ebc', theta, 'theta')
    _thetal = nag_double_type_check_and_cast('g07ebc', thetal, 'thetal')
    _thetau = nag_double_type_check_and_cast('g07ebc', thetau, 'thetau')
    _estcl = nag_double_type_check_and_cast('g07ebc', estcl, 'estcl')
    _ulower = nag_double_type_check_and_cast('g07ebc', ulower, 'ulower')
    _uupper = nag_double_type_check_and_cast('g07ebc', uupper, 'uupper')
    g07ebc_ctypes(method, n, _x, m, _y, clevel, _theta, _thetal, _thetau, _estcl, _ulower, _uupper, fail)

g07gac_ctypes = _library_cl.g07gac
g07gac_ctypes.restype = None
g07gac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), c_double, c_double, POINTER(Integer), POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07gac(n, p, y, mean, var, iout, niout, ldiff, diff, llamb, fail):
    """
    Outlier detection using method of Peirce, raw data or single variance
    supplied.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, iout, niout, diff, llamb
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, diff, llamb
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iout, niout
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _y = nag_double_type_check_and_cast('g07gac', y, 'y')
    _iout = nag_integer_type_check_and_cast('g07gac', iout, 'iout')
    _niout = nag_integer_type_check_and_cast('g07gac', niout, 'niout')
    _diff = nag_double_type_check_and_cast('g07gac', diff, 'diff')
    _llamb = nag_double_type_check_and_cast('g07gac', llamb, 'llamb')
    g07gac_ctypes(n, p, _y, mean, var, _iout, _niout, ldiff, _diff, _llamb, fail)

g07gbc_ctypes = _library_cl.g07gbc
g07gbc_ctypes.restype = Nag_Boolean
g07gbc_ctypes.argtypes = [Integer, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g07gbc(n, e, var1, var2, x, lx, ux, fail):
    """
    Outlier detection using method of Peirce, two variances supplied.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g07/g07gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, lx, ux
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, lx, ux
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g07gbc', x, 'x')
    _lx = nag_double_type_check_and_cast('g07gbc', lx, 'lx')
    _ux = nag_double_type_check_and_cast('g07gbc', ux, 'ux')
    return_value = g07gbc_ctypes(n, e, var1, var2, _x, _lx, _ux, fail)
    return return_value

nag_binomial_ci = g07aac
nag_poisson_ci = g07abc
nag_censored_normal = g07bbc
nag_estim_weibull = g07bec
nag_estim_gen_pareto = g07bfc
nag_2_sample_t_test = g07cac
nag_median_1var = g07dac
nag_robust_m_estim_1var = g07dbc
nag_robust_m_estim_1var_usr = g07dcc
nag_robust_trimmed_1var = g07ddc
nag_rank_ci_1var = g07eac
nag_rank_ci_2var = g07ebc
nag_outlier_peirce = g07gac
nag_outlier_peirce_two_var = g07gbc
__all__ = ['g07aac', 'g07abc', 'g07bbc', 'g07bec', 'g07bfc', 'g07cac',
           'g07dac', 'g07dbc', 'g07dcc', 'g07ddc', 'g07eac', 'g07ebc',
           'g07gac', 'g07gbc', 'nag_2_sample_t_test',
           'nag_binomial_ci', 'nag_censored_normal',
           'nag_estim_gen_pareto', 'nag_estim_weibull',
           'nag_median_1var', 'nag_outlier_peirce',
           'nag_outlier_peirce_two_var', 'nag_poisson_ci',
           'nag_rank_ci_1var', 'nag_rank_ci_2var',
           'nag_robust_m_estim_1var', 'nag_robust_m_estim_1var_usr',
           'nag_robust_trimmed_1var']
