"""
Deprecated wrappers for the NAG C Library (Mark 26.0) h02 Functions.

h02 - Integer Programming.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/hconts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_ulong
from ctypes import c_double
from ctypes import POINTER
from .util import NAG_H02DAC_CONFUN
from .util import NAG_H02DAC_OBJFUN
from .util import Nag_VariableType
from .util import get_library
from .util import Integer
from .util import Nag_Boolean
from .util import NAG_H02BBC_QPHESS
from .util import Nag_H02_Opt
from .util import Nag_Comm
from .util import NagError

_library_cl = get_library()

STRING = c_char_p


size_t = c_ulong
h02bbc_ctypes = _library_cl.h02bbc
h02bbc_ctypes.restype = None
h02bbc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_Boolean), POINTER(c_double), POINTER(c_double), Integer, NAG_H02BBC_QPHESS, POINTER(c_double), POINTER(c_double), POINTER(Nag_H02_Opt), POINTER(Nag_Comm), POINTER(NagError)]


def h02bbc(n, m, a, tda, bl, bu, intvar, cvec, h, tdh, qphess, x, objf, options, comm, fail):
    """
    Solves integer programming problems using a branch and bound method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, intvar, cvec, h, x, objf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, h, x, objf
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        intvar
    """
    from ctypes import cast
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('h02bbc', a, 'a')
    _bl = nag_double_type_check_and_cast('h02bbc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('h02bbc', bu, 'bu')
    _intvar = nag_enum_type_check_and_cast('h02bbc', intvar, 'intvar', Nag_Boolean)
    _cvec = nag_double_type_check_and_cast('h02bbc', cvec, 'cvec')
    _h = nag_double_type_check_and_cast('h02bbc', h, 'h')
    if qphess is None:
        qphess = cast(None, NAG_H02BBC_QPHESS)
    _x = nag_double_type_check_and_cast('h02bbc', x, 'x')
    _objf = nag_double_type_check_and_cast('h02bbc', objf, 'objf')
    h02bbc_ctypes(n, m, _a, tda, _bl, _bu, _intvar, _cvec, _h, tdh, qphess, _x, _objf, options, comm, fail)

h02buc_ctypes = _library_cl.h02buc
h02buc_ctypes.restype = None
h02buc_ctypes.argtypes = [STRING, Nag_Boolean, POINTER(Integer), POINTER(Integer), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(Nag_Boolean)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(Nag_H02_Opt), POINTER(NagError)]


def h02buc(mpsfile, minimize, n, m, a, bl, bu, intvar, cvec, x, options, fail):
    """
    Read MPSX data for IP, LP or QP problem from a file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02buc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        n, m, a, bl, bu, intvar, cvec, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        n, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        intvar
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _n = nag_integer_type_check_and_cast('h02buc', n, 'n')
    _m = nag_integer_type_check_and_cast('h02buc', m, 'm')
    _a = nag_double_type_check_and_cast('h02buc', a, 'a')
    _bl = nag_double_type_check_and_cast('h02buc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('h02buc', bu, 'bu')
    _intvar = nag_enum_type_check_and_cast('h02buc', intvar, 'intvar', Nag_Boolean)
    _cvec = nag_double_type_check_and_cast('h02buc', cvec, 'cvec')
    _x = nag_double_type_check_and_cast('h02buc', x, 'x')
    h02buc_ctypes(mpsfile, minimize, _n, _m, _a, _bl, _bu, _intvar, _cvec, _x, options, fail)

h02bvc_ctypes = _library_cl.h02bvc
h02bvc_ctypes.restype = None
h02bvc_ctypes.argtypes = [POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double)), POINTER(POINTER(Nag_Boolean)), POINTER(POINTER(c_double)), POINTER(POINTER(c_double))]


def h02bvc(a, bl, bu, intvar, cvec, x):
    """
    Free memory allocated by h02buc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02bvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, intvar, cvec, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, cvec, x
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        intvar
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _a = nag_double_type_check_and_cast('h02bvc', a, 'a')
    _bl = nag_double_type_check_and_cast('h02bvc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('h02bvc', bu, 'bu')
    _intvar = nag_enum_type_check_and_cast('h02bvc', intvar, 'intvar', Nag_Boolean)
    _cvec = nag_double_type_check_and_cast('h02bvc', cvec, 'cvec')
    _x = nag_double_type_check_and_cast('h02bvc', x, 'x')
    h02bvc_ctypes(_a, _bl, _bu, _intvar, _cvec, _x)

h02dac_ctypes = _library_cl.h02dac
h02dac_ctypes.restype = None
h02dac_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), NAG_H02DAC_CONFUN, POINTER(c_double), POINTER(c_double), NAG_H02DAC_OBJFUN, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def h02dac(n, nclin, ncnln, a, pda, d, ax, bl, bu, varcon, x, confun, c, cjac, objfun, objgrd, maxit, acc, objmip, iopts, opts, comm, fail):
    """
    Mixed integer nonlinear programming.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, d, ax, bl, bu, varcon, x, c, cjac, objgrd, objmip, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, d, ax, bl, bu, x, c, cjac, objgrd, objmip, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        varcon, iopts
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('h02dac', a, 'a')
    _d = nag_double_type_check_and_cast('h02dac', d, 'd')
    _ax = nag_double_type_check_and_cast('h02dac', ax, 'ax')
    _bl = nag_double_type_check_and_cast('h02dac', bl, 'bl')
    _bu = nag_double_type_check_and_cast('h02dac', bu, 'bu')
    _x = nag_double_type_check_and_cast('h02dac', x, 'x')
    if confun is None:
        confun = cast(None, NAG_H02DAC_CONFUN)
    _c = nag_double_type_check_and_cast('h02dac', c, 'c')
    _cjac = nag_double_type_check_and_cast('h02dac', cjac, 'cjac')
    _objgrd = nag_double_type_check_and_cast('h02dac', objgrd, 'objgrd')
    _objmip = nag_double_type_check_and_cast('h02dac', objmip, 'objmip')
    _opts = nag_double_type_check_and_cast('h02dac', opts, 'opts')
    _varcon = nag_integer_type_check_and_cast('h02dac', varcon, 'varcon')
    _iopts = nag_integer_type_check_and_cast('h02dac', iopts, 'iopts')
    h02dac_ctypes(n, nclin, ncnln, _a, pda, _d, _ax, _bl, _bu, _varcon, _x, confun, _c, _cjac, objfun, _objgrd, maxit, acc, _objmip, _iopts, _opts, comm, fail)

nag_mip_sqp = h02dac

h02xxc_ctypes = _library_cl.h02xxc
h02xxc_ctypes.restype = None
h02xxc_ctypes.argtypes = [POINTER(Nag_H02_Opt)]


def h02xxc(options):
    """
    Initialize option structure to null values.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02xxc.html
    """
    h02xxc_ctypes(options)

h02xyc_ctypes = _library_cl.h02xyc
h02xyc_ctypes.restype = None
h02xyc_ctypes.argtypes = [STRING, STRING, POINTER(Nag_H02_Opt), Nag_Boolean, STRING, POINTER(NagError)]


def h02xyc(name, optfile, options, _print, outfile, fail):
    """
    Read optional argument values from a file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02xyc.html
    """
    h02xyc_ctypes(name, optfile, options, _print, outfile, fail)

h02xzc_ctypes = _library_cl.h02xzc
h02xzc_ctypes.restype = None
h02xzc_ctypes.argtypes = [POINTER(Nag_H02_Opt), STRING, POINTER(NagError)]


def h02xzc(options, pname, fail):
    """
    Free NAG allocated memory from option structures.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02xzc.html
    """
    h02xzc_ctypes(options, pname, fail)

STRING = c_char_p
h02zkc_ctypes = _library_cl.h02zkc
h02zkc_ctypes.restype = None
h02zkc_ctypes.argtypes = [STRING, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def h02zkc(optstr, iopts, liopts, opts, lopts, fail):
    """
    Option setting routine for h02dac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02zkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _opts = nag_double_type_check_and_cast('h02zkc', opts, 'opts')
    _iopts = nag_integer_type_check_and_cast('h02zkc', iopts, 'iopts')
    h02zkc_ctypes(optstr, _iopts, liopts, _opts, lopts, fail)

nag_mip_opt_set = h02zkc

h02zlc_ctypes = _library_cl.h02zlc
h02zlc_ctypes.restype = None
h02zlc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(c_double), STRING, Integer, POINTER(Nag_VariableType), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def h02zlc(optstr, ivalue, rvalue, cvalue, lcvalue, optype, iopts, opts, fail):
    """
    Option getting routine for h02dac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h02zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _rvalue = nag_double_type_check_and_cast('h02zlc', rvalue, 'rvalue')
    _opts = nag_double_type_check_and_cast('h02zlc', opts, 'opts')
    _ivalue = nag_integer_type_check_and_cast('h02zlc', ivalue, 'ivalue')
    _iopts = nag_integer_type_check_and_cast('h02zlc', iopts, 'iopts')
    h02zlc_ctypes(optstr, _ivalue, _rvalue, cvalue, lcvalue, optype, _iopts, _opts, fail)

nag_mip_opt_get = h02zlc

nag_ip_bb = h02bbc
nag_ip_mps_read = h02buc
nag_ip_mps_free = h02bvc
nag_ip_init = h02xxc
nag_ip_read = h02xyc
nag_ip_free = h02xzc
__all__ = ['h02bbc', 'h02buc', 'h02bvc',
           'h02xxc', 'h02xyc', 'h02xzc',
           'nag_ip_bb', 'nag_ip_free', 'nag_ip_init',
           'nag_ip_mps_free', 'nag_ip_mps_read', 'nag_ip_read']
