"""
Deprecated wrappers for the NAG C Library (Mark 26.0) h03 Functions.

h03 - Transportation and Shortest Path Problems.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/hconts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER

from .util import get_library

from .util import Integer
from .util import NagError

_library_cl = get_library()

h03abc_ctypes = _library_cl.h03abc
h03abc_ctypes.restype = None
h03abc_ctypes.argtypes = [POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def h03abc(cost, tdcost, avail, navail, req, nreq, maxit, numit, optq, source, dest, optcost, unitcost, fail):
    """
    Classical transportation algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h03abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cost, avail, req, numit, optq, source, dest, optcost, unitcost
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cost, avail, req, optq, optcost, unitcost
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        numit, source, dest
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cost = nag_double_type_check_and_cast('h03abc', cost, 'cost')
    _avail = nag_double_type_check_and_cast('h03abc', avail, 'avail')
    _req = nag_double_type_check_and_cast('h03abc', req, 'req')
    _numit = nag_integer_type_check_and_cast('h03abc', numit, 'numit')
    _optq = nag_double_type_check_and_cast('h03abc', optq, 'optq')
    _source = nag_integer_type_check_and_cast('h03abc', source, 'source')
    _dest = nag_integer_type_check_and_cast('h03abc', dest, 'dest')
    _optcost = nag_double_type_check_and_cast('h03abc', optcost, 'optcost')
    _unitcost = nag_double_type_check_and_cast('h03abc', unitcost, 'unitcost')
    h03abc_ctypes(_cost, tdcost, _avail, navail, _req, nreq, maxit, _numit, _optq, _source, _dest, _optcost, _unitcost, fail)

h03bbc_ctypes = _library_cl.h03bbc
h03bbc_ctypes.restype = None
h03bbc_ctypes.argtypes = [Integer, POINTER(c_double), c_double, c_double, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def h03bbc(nc, dm, bound, targc, path, cost, tmode, alg_stats, state, fail):
    """
    Travelling Salesman Problem, simulated annealing.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/h/h03bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dm, path, cost, tmode, alg_stats, state
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dm, cost, alg_stats
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        path, tmode, state
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dm = nag_double_type_check_and_cast('h03bbc', dm, 'dm')
    _cost = nag_double_type_check_and_cast('h03bbc', cost, 'cost')
    _alg_stats = nag_double_type_check_and_cast('h03bbc', alg_stats, 'alg_stats')
    _path = nag_integer_type_check_and_cast('h03bbc', path, 'path')
    _tmode = nag_integer_type_check_and_cast('h03bbc', tmode, 'tmode')
    _state = nag_integer_type_check_and_cast('h03bbc', state, 'state')
    h03bbc_ctypes(nc, _dm, bound, targc, _path, _cost, _tmode, _alg_stats, _state, fail)

nag_mip_tsp_simann = h03bbc

nag_transport = h03abc
__all__ = ['h03abc',
           'nag_transport']
