#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the sparse submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestRealSys(object):
    """
    Abstract suite of utilities for testing the real iterative solver.
    """

    # pylint: disable=too-few-public-methods,too-many-instance-attributes

    def initialize(self):
        """
        Return a comm from the setup routine.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import sparse as chapter
        self.n = 8
        return chapter.real_gen_basic_setup(
            'BiCGSTAB', 'P', self.n, 1, 1.e-8, 20, 0., 0., 0, norm='1',
        )

    def default_matrix(self):
        """
        A default data set.
        """
        # pylint: disable=attribute-defined-outside-init
        self.nnz = 24
        la = 3*self.nnz
        self.a = np.empty(la)
        self.irow = np.empty(la, dtype=int)
        self.icol = np.empty(la, dtype=int)
        self.a[:self.nnz] = [
            2., -1., 1., 4., -3., 2., -7., 2., 3., -4., 5., 5., -1., 8., -3.,
            -6., 5., 2., -5., -1., 6., -1., 2., 3.,
        ]
        self.irow[:self.nnz] = [
            1, 1, 1, 2, 2, 2, 3, 3, 4, 4, 4,
            4, 5, 5, 5, 6, 6, 6, 7, 7, 7, 8, 8, 8,
        ]
        self.icol[:self.nnz] = [
            1, 4, 8, 1, 2, 5, 3, 6, 1, 3, 4,
            7, 2, 5, 7, 1, 3, 6, 3, 5, 7, 2, 6, 8,
        ]
        self.lfill = 0
        self.dtol = 0.
        self.milu = 'N'
        self.ipivp = np.zeros(self.n, dtype=int)
        self.ipivq = np.zeros(self.n, dtype=int)

class TestF11bd(TestRealSys, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11bd wrapper.
    """

class TestF11bdNp(TestF11bd, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11bd.set_up(self)
        self.fun = chapter.real_gen_basic_setup

    def test_bad_method(self):
        """
        Bad method supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Constraint: method',
            self.fun,
            'Spam', 'P', 8, 1, 1.e-8, 20, 0., 0., 0,
        )

    def test_good(self):
        """
        Good call.
        """
        self.assertEqual(
            len(self.initialize()['work']), 151,
        )

class TestF11be(TestRealSys, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11be wrapper.
    """

class TestF11beNp(TestF11be, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11be.set_up(self)
        self.precon = chapter.real_gen_precon_ilu
        self.fun = chapter.real_gen_basic_solver
        self.matvecmult = chapter.real_gen_matvec
        self.ilu_solve = chapter.real_gen_precon_ilu_solve
        self.diagnostic = chapter.real_gen_basic_diag

    def test_good(self):
        """
        Good call.
        """
        comm = self.initialize()
        self.default_matrix()
        ilu = self.precon(
            self.nnz, self.a, self.irow, self.icol, self.lfill,
            self.dtol, self.milu, self.ipivp, self.ipivq,
        )

        wgt = np.empty(self.n)
        v = np.array([6., 8., -9., 46., 17., 21., 22., 34.])
        u = np.zeros(self.n)

        irevcm = 0
        while irevcm != 4:
            irevcm = self.fun(irevcm, u, v, wgt, comm)

            if irevcm in (-1, 1):
                v = self.matvecmult(
                    'T' if irevcm == -1 else 'N',
                    ilu.a[:self.nnz], ilu.irow[:self.nnz],
                    ilu.icol[:self.nnz], u,
                )
            elif irevcm == 2:
                v = self.ilu_solve(
                    'N',
                    ilu.a, ilu.irow, ilu.icol,
                    ilu.ipivp, ilu.ipivq, ilu.istr,
                    ilu.idiag, u,
                )

        diagn = self.diagnostic(comm)
        self.assertEqual(diagn.itn, 3)
        self.assertAlmostEqual(diagn.stplhs, 1.04e-8)
        self.assertAlmostEqual(diagn.stprhs, 5.59e-6)
        self.assertEqual(diagn.anorm, 11.)

class TestComplexSys(object):
    """
    Abstract suite of utilities for testing the complex iterative solver.
    """

    # pylint: disable=too-few-public-methods,too-many-instance-attributes

    def initialize(self):
        """
        Return a comm from the setup routine.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import sparse as chapter
        self.n = 8
        return chapter.complex_gen_basic_setup(
            'TFQMR', 'P', self.n, 1, 1.e-8, 20, 0., 0., 0, lwork=200, norm='1',
        ).comm

    def default_matrix(self):
        """
        A default data set.
        """
        # pylint: disable=attribute-defined-outside-init
        self.nnz = 24
        la = 2*self.nnz
        self.a = np.empty(la, dtype=complex)
        self.irow = np.empty(la, dtype=int)
        self.icol = np.empty(la, dtype=int)
        self.a[:self.nnz] = [
            complex(2., 1.),
            complex(-1., 1.),
            complex(1., -3.),
            complex(4., 7.),
            complex(-3., 0.),
            complex(2., 4.),
            complex(-7., -5.),
            complex(2., 1.),
            complex(3., 2.),
            complex(-4., 2.),
            complex(0., 1.),
            complex(5., -3.),
            complex(-1., 2.),
            complex(8., 6.),
            complex(-3., -4.),
            complex(-6., -2.),
            complex(5., -2.),
            complex(2., 0.),
            complex(0., -5.),
            complex(-1., 5.),
            complex(6., 2.),
            complex(-1., 4.),
            complex(2., 0.),
            complex(3., 3.),
        ]
        self.irow[:self.nnz] = [
            1, 1, 1, 2, 2, 2, 3, 3, 4, 4, 4, 4, 5, 5, 5,
            6, 6, 6, 7, 7, 7, 8, 8, 8,
        ]
        self.icol[:self.nnz] = [
            1, 4, 8, 1, 2, 5, 3, 6, 1, 3, 4, 7, 2, 5, 7,
            1, 3, 6, 3, 5, 7, 2, 6, 8,
        ]
        self.lfill = 0
        self.dtol = 0.
        self.milu = 'N'
        self.ipivp = np.zeros(self.n, dtype=int)
        self.ipivq = np.zeros(self.n, dtype=int)

class TestF11br(TestComplexSys, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11br wrapper.
    """

class TestF11brNp(TestF11br, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def test_good(self):
        """
        Good call.
        """
        self.assertEqual(
            len(self.initialize()['work']), 200,
        )

class TestF11bs(TestComplexSys, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11bs wrapper.
    """

class TestF11bsNp(TestF11bs, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11bs.set_up(self)
        self.precon = chapter.complex_gen_precon_ilu
        self.fun = chapter.complex_gen_basic_solver
        self.matvecmult = chapter.complex_gen_matvec
        self.ilu_solve = chapter.complex_gen_precon_ilu_solve
        self.diagnostic = chapter.complex_gen_basic_diag

    def test_good(self):
        """
        Good call.
        """
        comm = self.initialize()
        self.default_matrix()
        ilu = self.precon(
            self.nnz, self.a, self.irow, self.icol, self.lfill,
            self.dtol, self.milu, self.ipivp, self.ipivq,
        )

        wgt = np.empty(self.n)
        v = np.array([
            complex(7., 11.),
            complex(1., 24.),
            complex(-13., -18.),
            complex(-10., 3.),
            complex(23., 14.),
            complex(17., -7.),
            complex(15., -3.),
            complex(-3., 20.),
        ])
        u = np.zeros(self.n, dtype=complex)

        irevcm = 0

        while irevcm != 4:
            irevcm = self.fun(irevcm, u, v, wgt, comm)

            if irevcm in (-1, 1):
                v = self.matvecmult(
                    'T' if irevcm == -1 else 'N',
                    ilu.a[:self.nnz], ilu.irow[:self.nnz],
                    ilu.icol[:self.nnz], u,
                )
            elif irevcm == 2:
                v = self.ilu_solve(
                    'N',
                    ilu.a, ilu.irow, ilu.icol,
                    ilu.ipivp, ilu.ipivq, ilu.istr,
                    ilu.idiag, u,
                )

        diagn = self.diagnostic(comm)
        self.assertEqual(diagn.itn, 4)
        self.assertAlmostEqual(diagn.stplhs, 4.93e-12)
        self.assertAlmostEqual(diagn.stprhs, 8.91e-6)
        self.assertEqual(diagn.anorm, 27.)

class TestRealGenPreconIlu(object):
    """
    Abstract suite of utilities for testing the real incomplete LU
    preconditioner.
    """

    # pylint: disable=too-few-public-methods,too-many-instance-attributes

    def default_matrix(self):
        """
        A default data set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = 4
        self.nnz = 11
        self.a = np.empty(2*self.nnz)
        self.irow = np.empty(2*self.nnz, dtype=int)
        self.icol = np.empty(2*self.nnz, dtype=int)
        self.a[:self.nnz] = [1., 1., -1., 2., 2., 3., -2., 1., -2., 1., 1.]
        self.irow[:self.nnz] = [1, 1, 2, 2, 2, 3, 3, 4, 4, 4, 4]
        self.icol[:self.nnz] = [2, 3, 1, 3, 4, 1, 4, 1, 2, 3, 4]
        self.lfill = 0
        self.dtol = 0.
        self.milu = 'N'
        self.ipivp = np.zeros(n, dtype=int)
        self.ipivq = np.zeros(n, dtype=int)

    def ilu_factorize(self):
        """
        Return the ILU.
        """
        return self.factorize(
            self.nnz, self.a, self.irow, self.icol, self.lfill,
            self.dtol, self.milu, self.ipivp, self.ipivq,
        )

    def set_up(self):
        """
        Initial state.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import sparse as chapter
        self.factorize = chapter.real_gen_precon_ilu
        self.solve = chapter.real_gen_precon_ilu_solve
        self.default_matrix()

class TestF11da(TestRealGenPreconIlu, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11da wrapper.
    """

class TestF11daNp(TestF11da, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestRealGenPreconIlu.set_up(self)

    def test_good(self):
        """
        Good call.
        """
        ilu = self.ilu_factorize()
        np.testing.assert_equal(ilu.ipivp, [1, 3, 2, 4])
        np.testing.assert_equal(ilu.ipivq, [2, 1, 3, 4])
        self.assertEqual(ilu.nnzc, 11)

class TestF11db(TestRealGenPreconIlu, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11db wrapper.
    """

class TestF11dbNp(TestF11db, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestRealGenPreconIlu.set_up(self)

    def test_good(self):
        """
        Good call.
        """
        ilu = self.ilu_factorize()
        res = self.solve(
            'N', ilu.a, ilu.irow, ilu.icol, ilu.ipivp, ilu.ipivq, ilu.istr,
            ilu.idiag, [5.0, 13.0, -5.0, 4.0],
        )
        np.testing.assert_equal(res, [1., 2., 3., 4.])

class TestF11dg(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11dg wrapper.
    """

class TestF11dgBase(TestF11dg, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import sparse as chapter
        TestF11dg.set_up(self)
        self.fun = chapter.real_symm_basic_setup

    def test_bad_sigtol(self):
        """
        Bad sigtol supplied.

        Coverage for: STR_FMT_FLOAT outputting.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'sigtol = *4.20',
            self.fun,
            'CG', 'N', 'S', '1', 'N', 1, 1, 0., 1, 0., -42., 42., 1, 0, 120,
        )

class TestComplexGenPreconIlu(object):
    """
    Abstract suite of utilities for testing the complex incomplete LU
    preconditioner.
    """

    # pylint: disable=too-few-public-methods,too-many-instance-attributes

    def default_matrix(self):
        """
        A default data set.
        """
        # pylint: disable=attribute-defined-outside-init
        n = 4
        self.nnz = 11
        self.a = np.empty(2*self.nnz, dtype=complex)
        self.irow = np.empty(2*self.nnz, dtype=int)
        self.icol = np.empty(2*self.nnz, dtype=int)
        self.a[:self.nnz] = [
            complex(1., 3.), complex(1., 0.), complex(-1., -2.),
            complex(2., -2.), complex(2., 1.), complex(0., 5.),
            complex(-2., 0.), complex(1., 1.), complex(-2., 4.),
            complex(1., -3.), complex(0., 7.),
        ]
        self.irow[:self.nnz] = [1, 1, 2, 2, 2, 3, 3, 4, 4, 4, 4]
        self.icol[:self.nnz] = [2, 3, 1, 3, 4, 1, 4, 1, 2, 3, 4]
        self.lfill = 0
        self.dtol = 0.
        self.milu = 'N'
        self.ipivp = np.zeros(n, dtype=int)
        self.ipivq = np.zeros(n, dtype=int)

    def ilu_factorize(self):
        """
        Return the ILU.
        """
        return self.factorize(
            self.nnz, self.a, self.irow, self.icol, self.lfill,
            self.dtol, self.milu, self.ipivp, self.ipivq,
        )

    def set_up(self):
        """
        Initial state.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import sparse as chapter
        self.factorize = chapter.complex_gen_precon_ilu
        self.solve = chapter.complex_gen_precon_ilu_solve
        self.default_matrix()

class TestF11dn(TestComplexGenPreconIlu, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11dn wrapper.
    """

class TestF11dnNp(TestF11dn, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestComplexGenPreconIlu.set_up(self)

    def test_good(self):
        """
        Good call.
        """
        ilu = self.ilu_factorize()
        np.testing.assert_equal(ilu.ipivp, [1, 3, 2, 4])
        np.testing.assert_equal(ilu.ipivq, [2, 1, 3, 4])
        self.assertEqual(ilu.nnzc, 11)

class TestF11dp(TestComplexGenPreconIlu, IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11dp wrapper.
    """

class TestF11dpNp(TestF11dp, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        TestComplexGenPreconIlu.set_up(self)

    def test_good(self):
        """
        Good call.
        """
        ilu = self.ilu_factorize()
        res = self.solve(
            'N', ilu.a, ilu.irow, ilu.icol, ilu.ipivp, ilu.ipivq, ilu.istr,
            ilu.idiag,
            [
                complex(5.0, 14.0),
                complex(21.0, 5.0),
                complex(-21.0, 18.0),
                complex(14.0, 4.0),
            ],
        )
        np.testing.assert_almost_equal(
            res,
            [
                complex(0.4545E+01, 0.5313E+01),
                complex(0.1542E+01, -0.7037E+00),
                complex(0.1347E+01, 0.1008E+02),
                complex(-0.2782E+01, 0.2362E+01),
            ],
            decimal=2,
        )

class TestF11xa(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11xa wrapper.
    """

class TestF11xaNp(TestF11xa, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11xa.set_up(self)
        self.fun = chapter.real_gen_matvec

    def test_good(self):
        """
        Good call.
        """
        a = [2., 1., 1., -1., 4., 1., 1., 1., 2., -2., 3.]
        irow = [1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5]
        icol = [1, 2, 3, 4, 1, 3, 5, 4, 5, 2, 5]
        x = [0.70, 0.16, 0.52, 0.77, 0.28]
        np.testing.assert_almost_equal(
            self.fun('N', a, irow, icol, x),
            [1.56, -0.25, 3.6, 1.33, 0.52],
        )

class TestF11xn(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11xn wrapper.
    """

class TestF11xnNp(TestF11xn, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11xn.set_up(self)
        self.fun = chapter.complex_gen_matvec

    def test_good(self):
        """
        Good call.
        """
        a = [
            complex(2., 3.),
            complex(1., -4.),
            complex(1., 0.),
            complex(-1., -2.),
            complex(4., 1.),
            complex(0., 1.),
            complex(1., 3.),
            complex(0., -1.),
            complex(2., -6.),
            complex(-2., 0.),
            complex(3., 1.),
        ]
        irow = [1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5]
        icol = [1, 2, 3, 4, 1, 3, 5, 4, 5, 2, 5]
        x = [
            complex(0.70, 0.21),
            complex(0.16, -0.43),
            complex(0.52, 0.97),
            complex(0.77, 0.00),
            complex(0.28, -0.64),
        ]
        np.testing.assert_almost_equal(
            self.fun('N', a, irow, icol, x),
            [
                complex(-0.7900E+00, 0.1450E+01),
                complex(-0.2500E+00, -0.5700E+00),
                complex(0.3820E+01, 0.2260E+01),
                complex(-0.3280E+01, -0.3730E+01),
                complex(0.1160E+01, -0.7800E+00),
            ]
        )

class TestF11ye(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11ye wrapper.
    """

class TestF11yeNp(TestF11ye, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11ye.set_up(self)
        self.fun = chapter.sym_rcm

    def test_good(self):
        """
        Good call.
        """
        irowix = [
            2, 5, 6, 1, 3, 11, 2, 4, 16,
            3, 5, 21, 1, 4, 26, 1, 7, 10,
            6, 8, 30, 7, 9, 42, 8, 10, 38,
            6, 9, 12, 2, 12, 15, 10, 11, 13,
            12, 14, 37, 13, 15, 33, 11, 14, 17,
            3, 17, 20, 15, 16, 18, 17, 19, 32,
            18, 20, 53, 16, 19, 22, 4, 22, 25,
            20, 21, 23, 22, 24, 52, 23, 25, 48,
            21, 24, 27, 5, 27, 30, 25, 26, 28,
            27, 29, 47, 28, 30, 43, 7, 26, 29,
            32, 35, 54, 18, 31, 33, 14, 32, 34,
            33, 35, 36, 31, 34, 56, 34, 37, 40,
            13, 36, 38, 9, 37, 39, 38, 40, 41,
            36, 39, 57, 39, 42, 45, 8, 41, 43,
            29, 42, 44, 43, 45, 46, 41, 44, 58,
            44, 47, 50, 28, 46, 48, 24, 47, 49,
            48, 50, 51, 46, 49, 59, 49, 52, 55,
            23, 51, 53, 19, 52, 54, 31, 53, 55,
            51, 54, 60, 35, 57, 60, 40, 56, 58,
            45, 57, 59, 50, 58, 60, 55, 56, 59,
        ]
        icolzp = [
            1, 4, 7, 10, 13, 16,
            19, 22, 25, 28, 31, 34,
            37, 40, 43, 46, 49, 52,
            55, 58, 61, 64, 67, 70,
            73, 76, 79, 82, 85, 88,
            91, 94, 97, 100, 103, 106,
            109, 112, 115, 118, 121, 124,
            127, 130, 133, 136, 139, 142,
            145, 148, 151, 154, 157, 160,
            163, 166, 169, 172, 175, 178,
            181,
        ]
        lopts = [False]*5
        np.testing.assert_equal(
            self.fun(icolzp, irowix, lopts).perm,
            [
                1, 5, 2, 6, 4, 3, 26, 7, 30, 11, 12, 10, 21, 16, 27, 25, 8, 29,
                17, 15, 13, 9, 22, 20, 28, 24, 42, 43, 18, 14, 37, 38, 23, 19,
                47, 48, 41, 44, 32, 33, 36, 39, 52, 53, 46, 49, 45, 31, 34, 40,
                51, 54, 50, 58, 35, 57, 55, 59, 56, 60,
            ],
        )

class TestF11zc(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11zc wrapper.
    """

class TestF11zcNp(TestF11zc, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    # pylint: disable=too-many-instance-attributes

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11zc.set_up(self)
        self.fun = chapter.real_rect_sort
        self.m = 3
        self.n = 4
        self.a = [1., 2., 3., 4., 5., 0.]
        self.irow = [3, 2, 2, 1, 3, 1]
        self.icol = [3, 1, 4, 2, 3, 1]
        self.dup = 'S'
        self.zer = 'R'
        self.exp_a = [2., 4., 6., 3.]
        self.exp_irow = [2, 1, 3, 2]
        self.exp_icol = [1, 2, 3, 4]
        self.exp_istc = [1, 2, 3, 4, 5]

    def test_good_defaults(self):
        """
        Good call.
        """
        ret = self.fun(self.m, self.a, self.irow, n=self.n, icol=self.icol)
        np.testing.assert_array_equal(ret.a, self.exp_a)
        np.testing.assert_array_equal(ret.irow, self.exp_irow)
        np.testing.assert_array_equal(ret.icol, self.exp_icol)
        np.testing.assert_array_equal(ret.istc, self.exp_istc)

    def test_good_supplied(self):
        """
        Good call.
        """
        ret = self.fun(
            self.m, self.a, self.irow,
            n=self.n, icol=self.icol, dup=self.dup, zer=self.zer,
        )
        np.testing.assert_array_equal(ret.a, self.exp_a)
        np.testing.assert_array_equal(ret.irow, self.exp_irow)
        np.testing.assert_array_equal(ret.icol, self.exp_icol)
        np.testing.assert_array_equal(ret.istc, self.exp_istc)

    def test_good_ccs(self):
        """
        Good call.
        """
        a = [4., 6., 1., -1., 1., 1., 2., 2.]
        irow = [3, 1, 1, 2, 3, 2, 1, 3]
        istc = [1, 3, 5, 8, 9]
        ret = self.fun(self.m, a, irow, istc=istc)
        np.testing.assert_array_equal(
            ret.a, [6., 4., 1., -1., 2., 1., 1., 2.],
        )
        np.testing.assert_array_equal(
            ret.irow, [1, 3, 1, 2, 1, 2, 3, 3],
        )
        np.testing.assert_array_equal(
            ret.icol, [1, 1, 2, 2, 3, 3, 3, 4],
        )
        np.testing.assert_array_equal(
            ret.istc, istc,
        )

    def test_bad_n(self):
        """
        Bad call, n not supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'n or istc must be provided',
            self.fun,
            self.m, self.a, self.irow, icol=self.icol,
        )

    def test_bad_col_omitted(self):
        """
        Bad call, all column data omitted.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Exactly one of icol and istc must be provided',
            self.fun,
            self.m, self.a, self.irow,
        )

    def test_bad_col_both(self):
        """
        Bad call, all column data supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Exactly one of icol and istc must be provided',
            self.fun,
            self.m, self.a, self.irow,
            icol=self.icol, istc=[42],
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
