# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `fit` Chapter.

``fit`` - Curve and Surface Fitting

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

The main aim of this module is to assist you in finding a function which
approximates a set of data points.
Typically the data contain random errors, as of experimental
measurement, which need to be smoothed out.
To seek an approximation to the data, it is first necessary to specify
for the approximating function a mathematical form (a polynomial, for
example) which contains a number of unspecified coefficients: the
appropriate fitting function then derives for the coefficients the
values which provide the best fit of that particular form.
The module deals mainly with curve and surface fitting when a polynomial
or a cubic spline is used as the fitting function, since these cover the
most common needs.
However, fitting with other functions and/or more variables can be
undertaken by means of general linear or nonlinear functions (some of
which are contained in other modules) depending on whether the
coefficients in the function occur linearly or nonlinearly.
Cases where a graph rather than a set of data points is given can be
treated simply by first reading a suitable set of points from the graph.

The module also contains functions for evaluating, differentiating and
integrating polynomial and spline curves and surfaces, once the
numerical values of their coefficients have been determined.

There is also a function for computing a Padé approximant of a
mathematical function (see [Background to the Problems]).

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dim1_cheb_arb(m, kplus1, x, y, w, a, s):
    u"""
    Least squares curve fit, by polynomials, arbitrary data points.

    For full information please refer to the NAG Library document for
    e02ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02adf.html

    Parameters
    ----------
    m : int
        The number m of data points.

    kplus1 : int
        k+1, where k is the maximum degree required.

    x : float, shape (`m`)
        The values x_r of the independent variable, for r = 1,2,...,m.

    y : float, shape (`m`)
        The values y_r of the dependent variable, for r = 1,2,...,m.

    w : float, shape (`m`)
        The set of weights, w_r, for r = 1,2,...,m. For advice on the
        choice of weights, see [the E02 Introduction].

    a : float, shape (`kplus1`, `kplus1`), modified in place
        On exit: the coefficients of T_j(x_) in the approximating
        polynomial of degree i. `a`[i,j] contains the coefficient
        a_{i+1,j+1}, for j = 0,1,...,i, for i = 0,1,...,k.

    s : float, shape (`kplus1`), modified in place
        On exit: `s`[i] contains the root-mean-square residual s_i, for
        i = 0,1,...,k, as described in [Description]. For the
        interpretation of the values of the s_i and their use in
        selecting an appropriate degree, see [the E02 Introduction].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, i = *<value>* and `w`[i-1] = *<value>*.

            Constraint: `w`[i-1] > 0.0.

        (`errno` 2)
            On entry, i = *<value>*, `x`[i-1] = *<value>* and `x`[i-2] =
            *<value>*.

            Constraint: `x`[i-1] >= `x`[i-2].

        (`errno` 3)
            On entry, all `x`[I-1] have the same value: `x`[0] =
            *<value>*.

        (`errno` 4)
            On entry, `kplus1` = *<value>* and mdist = *<value>*.

            Constraint: `kplus1` <= mdist, where mdist is the number of
            distinct `x` values.

        (`errno` 4)
            On entry, `kplus1` = *<value>* and `m` = *<value>*.

            Constraint: `kplus1` <= `m`.

        (`errno` 4)
            On entry, `kplus1` = *<value>*.

            Constraint: `kplus1` >= 1.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_arb'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, kplus1.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work2.
        utils._EngineFloat64ArrayType.c_type, # Output, a[lda, kplus1].
        utils._EngineFloat64ArrayType.c_type, # Output, s[kplus1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _kplus1 = utils._EngineIntScalarType(
        kplus1, {'fun_name': fun_name, 'entity': 'kplus1'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _work1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _work2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _kplus1_py = utils._EngineIntScalarType.to_py(_kplus1)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_kplus1_py, _kplus1_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_kplus1_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _kplus1,
        _lda,
        _x,
        _y,
        _w,
        _work1,
        _work2,
        _a,
        _s,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and w[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: w[i-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[i-1] >= x[i-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all x[I-1] have the same value:',
                    'x[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kplus1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mdist = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: kplus1 <= mdist, where mdist is the number of distinct x values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kplus1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: kplus1 <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kplus1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kplus1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_cheb_eval(nplus1, a, xcap):
    u"""
    Evaluation of fitted polynomial in one variable from Chebyshev
    series form (simplified parameter list).

    For full information please refer to the NAG Library document for
    e02ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02aef.html

    Parameters
    ----------
    nplus1 : int
        The number n+1 of terms in the series.

    a : float, shape (`nplus1`)
        `a`[i-1] must be set to the value of the ith coefficient in the
        series, for i = 1,2,...,n+1.

    xcap : float
        x_, the argument at which the polynomial is to be evaluated. It
        should lie in the range -1 to +1, but a value just outside this
        range is permitted (see [Errors]) to allow for possible rounding
        errors committed in the transformation from x to x_ discussed in
        [Description]. Provided the recommended form of the
        transformation is used, a successful exit is thus assured
        whenever the value of x lies in the range x_min to x_max.

    Returns
    -------
    p : float
        The value of the polynomial.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `xcap` = *<value>* and EPS = *<value>*.

            Constraint: abs(`xcap`) <= 1+4*EPS, where EPS is machine
            precision.

        (`errno` 2)
            On entry, `nplus1` = *<value>*.

            Constraint: `nplus1` >= 1.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nplus1.
        utils._EngineFloat64ArrayType.c_type, # Input, a[nplus1].
        utils._BYREF_FLOAT64, # Input, xcap.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nplus1 = utils._EngineIntScalarType(
        nplus1, {'fun_name': fun_name, 'entity': 'nplus1'},
    ).to_ctypes()
    _nplus1_py = utils._EngineIntScalarType.to_py(_nplus1)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_nplus1_py,),
    ).to_ctypes()
    _xcap = utils._EngineFloat64ScalarType(
        xcap, {'fun_name': fun_name, 'entity': 'xcap'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nplus1,
        _a,
        _xcap,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xcap = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'EPS = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: abs(xcap) <= 1+4*EPS, where EPS is machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nplus1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nplus1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def dim1_cheb_glp(nplus1, f, a):
    u"""
    Least squares polynomial fit, special data points (including
    interpolation).

    For full information please refer to the NAG Library document for
    e02af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02aff.html

    Parameters
    ----------
    nplus1 : int
        The number n+1 of data points (one greater than the degree n of
        the interpolating polynomial).

    f : float, shape (`nplus1`)
        For r = 1,2,...,n+1, `f`[r-1] must contain f_r the value of the
        dependent variable (ordinate) corresponding to the value

            x__r = cos(pi(r-1)/n)

        of the independent variable (abscissa) x_, or equivalently to
        the value

            x(r) = 1/2(x_max-x_min)cos(pi(r-1)/n)+1/2(x_max+x_min)

        of your original variable x. Here x_max and x_min are
        respectively the upper and lower ends of the range over which
        you wish to interpolate.

    a : float, shape (`nplus1`), modified in place
        On exit: `a`[j-1] is the coefficient a_j in the interpolating
        polynomial, for j = 1,2,...,n+1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nplus1` = *<value>*.

            Constraint: `nplus1` >= 2.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_glp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nplus1.
        utils._EngineFloat64ArrayType.c_type, # Input, f[nplus1].
        utils._EngineFloat64ArrayType.c_type, # Output, a[nplus1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nplus1 = utils._EngineIntScalarType(
        nplus1, {'fun_name': fun_name, 'entity': 'nplus1'},
    ).to_ctypes()
    _nplus1_py = utils._EngineIntScalarType.to_py(_nplus1)
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_nplus1_py,),
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_nplus1_py,),
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nplus1,
        _f,
        _a,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nplus1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nplus1 >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_cheb_con(m, k, xmin, xmax, x, y, w, mf, xf, yf, lyf, ip, a, s, resid):
    u"""
    Least squares polynomial fit, values and derivatives may be
    constrained, arbitrary data points.

    For full information please refer to the NAG Library document for
    e02ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02agf.html

    Parameters
    ----------
    m : int
        m, the number of data points to be fitted.

    k : int
        k, the maximum degree required.

    xmin : float
        The lower and upper end points, respectively, of the interval
        [x_min,x_max]. Unless there are specific reasons to the
        contrary, it is recommended that `xmin` and `xmax` be set
        respectively to the lowest and highest value among the x_r and
        xf_r. This avoids the danger of extrapolation provided there is
        a constraint point or data point with nonzero weight at each end
        point.

    xmax : float
        The lower and upper end points, respectively, of the interval
        [x_min,x_max]. Unless there are specific reasons to the
        contrary, it is recommended that `xmin` and `xmax` be set
        respectively to the lowest and highest value among the x_r and
        xf_r. This avoids the danger of extrapolation provided there is
        a constraint point or data point with nonzero weight at each end
        point.

    x : float, shape (`m`)
        `x`[r-1] must contain the value x_r of the independent variable
        at the rth data point, for r = 1,2,...,m.

    y : float, shape (`m`)
        `y`[r-1] must contain y_r, the value of the dependent variable
        at the rth data point, for r = 1,2,...,m.

    w : float, shape (`m`)
        `w`[r-1] must contain the weight w_r to be applied to the data
        point x_r, for r = 1,2,...,m. For advice on the choice of
        weights see [the E02 Introduction]. Negative weights are treated
        as positive. A zero weight causes the corresponding data point
        to be ignored. Zero weight should be given to any data point
        whose x and y values both coincide with those of a constraint
        (otherwise the denominators involved in the root mean square
        residuals S_i will be slightly in error).

    mf : int
        mf, the number of values of the independent variable at which a
        constraint is specified.

    xf : float, shape (`mf`)
        `xf`[r-1] must contain xf_r, the value of the independent
        variable at which a constraint is specified, for r =
        1,2,...,`mf`.

    yf : float, shape (`lyf`)
        The values which the approximating polynomials and their
        derivatives are required to take at the points specified in
        `xf`. For each value of `xf`[r-1], `yf` contains in successive
        elements the required value of the approximation, its first
        derivative, second derivative, ...,p_rth derivative, for r =
        1,2,...,mf. Thus the value, yf_s, which the kth derivative of
        each approximation (k = 0 referring to the approximation itself)
        is required to take at the point `xf`[r-1] must be contained in
        `yf`[s-1], where

            s = r+k+p_1+p_2+...+p_{r-1},

        where k = 0,1,...,p_r and r = 1,2,...,mf. The derivatives are
        with respect to the independent variable x.

    lyf : int
        The dimension of the array `yf`.

    ip : int, shape (`mf`)
        `ip`[r-1] must contain p_r, the order of the highest-order
        derivative specified at `xf`[r-1], for r = 1,2,...,mf. p_r = 0
        implies that the value of the approximation at `xf`[r-1] is
        specified, but not that of any derivative.

    a : float, shape (`k`+1, `k`+1), modified in place
        On exit: `a`[i,j] contains the coefficient a_{ij} in the
        approximating polynomial of degree i, for j = 0,1,...,i, for i =
        n,...,k.

    s : float, shape (`k`+1), modified in place
        On exit: `s`[i] contains S_i, for i = n,...,k, the root mean
        square residual corresponding to the approximating polynomial of
        degree i. In the case where the number of data points with
        nonzero weight is equal to k+1-n, S_i is indeterminate: the
        function sets it to zero. For the interpretation of the values
        of S_i and their use in selecting an appropriate degree, see
        [the E02 Introduction].

    resid : float, shape (`m`), modified in place
        On exit: contains weighted residuals of the highest degree of
        fit determined (k). The residual at x_r is in element
        `resid`[r-1], for r = 1,2,...,m.

    Returns
    -------
    n : int
        Contains the total number of constraint conditions imposed: `n`
        = `mf`+p_1+p_2+...+p_`mf`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `k` >= `n`.

        (`errno` 1)
            On entry, `lyf` = *<value>* and N = *<value>*.

            Constraint: `lyf` >= N, where N is the total number of
            constraints.

        (`errno` 1)
            On entry, `mf` = *<value>*.

            Constraint: `mf` >= 1.

        (`errno` 2)
            On entry, I = *<value>* and `ip`[I-1] = *<value>*.

            Constraint: `ip`[I-1] >= 0.

        (`errno` 3)
            On entry, I = *<value>*, `xf`[I-1] = *<value>*, J =
            *<value>* and `xf`[J-1] = *<value>*.

            Constraint: `xf`[I-1] != `xf`[J-1].

        (`errno` 3)
            On entry, `xf`[I-1] lies outside interval [`xmin`,`xmax`]: I
            = *<value>*, `xf`[I-1] = *<value>*, `xmin` = *<value>* and
            `xmax` = *<value>*.

        (`errno` 3)
            On entry, `xmin` = *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` < `xmax`.

        (`errno` 4)
            On entry, `x`[I-1] lies outside interval [`xmin`,`xmax`] for
            some I.

        (`errno` 4)
            On entry, `x`[I-1] lies outside interval [`xmin`,`xmax`]: I
            = *<value>*, `x`[I-1] = *<value>*, `xmin` = *<value>* and
            `xmax` = *<value>*.

        (`errno` 5)
            On entry, i = *<value>*, `x`[i-1] = *<value>* and `x`[i-2] =
            *<value>*.

            Constraint: `x`[i-1] >= `x`[i-2].

        (`errno` 6)
            On entry, `k`+1 > m^('')+`n`, where m^('') is the number of
            data points with nonzero weight and distinct abscissae
            different from all `xf`, and `n` is the total number of
            constraints: `k`+1 = *<value>*, m^('') = *<value>* and `n` =
            *<value>*.

        (`errno` 7)
            The polynomials mu(x) and/or nu(x) cannot be found. The
            problem is too ill-conditioned.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_con'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, kplus1.
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._BYREF_INT, # Input, mf.
        utils._EngineFloat64ArrayType.c_type, # Input, xf[mf].
        utils._EngineFloat64ArrayType.c_type, # Input, yf[lyf].
        utils._BYREF_INT, # Input, lyf.
        utils._EngineIntArrayType.c_type, # Input, ip[mf].
        utils._EngineFloat64ArrayType.c_type, # Output, a[lda, k+1].
        utils._EngineFloat64ArrayType.c_type, # Output, s[k+1].
        utils._BYREF_INT, # Output, np1.
        utils._EngineFloat64ArrayType.c_type, # Output, wrk[m].
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._BYREF_INT, # Input, liwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _kplus1_py = utils._EngineFloat64ArrayType.get_shape(s, {'fun_name': fun_name, 'entity': 's'}, exp_rank=1)[0]
    _kplus1 = utils._EngineIntScalarType(
        _kplus1_py, {'fun_name': fun_name, 'entity': '_kplus1'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _mf = utils._EngineIntScalarType(
        mf, {'fun_name': fun_name, 'entity': 'mf'},
    ).to_ctypes()
    _mf_py = utils._EngineIntScalarType.to_py(_mf)
    _xf = utils._EngineFloat64ArrayType(
        xf, {'fun_name': fun_name, 'entity': 'xf'},
        exp_shape=(_mf_py,),
    ).to_ctypes()
    _lyf = utils._EngineIntScalarType(
        lyf, {'fun_name': fun_name, 'entity': 'lyf'},
    ).to_ctypes()
    _ip_cls = utils._EngineIntArrayType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
        exp_shape=(_mf_py,),
    )
    _ip = _ip_cls.to_ctypes()
    _yf = utils._EngineFloat64ArrayType(
        yf, {'fun_name': fun_name, 'entity': 'yf'},
        exp_shape=(_mf_py+_ip_cls.get_sum(0, _mf_py-1),),
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_k_py+1, _k_py+1),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_k_py+1,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _np1 = utils._EngineIntScalarType.empty_ctypes()
    utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(_m_py)
    _lwrk = utils._EngineIntScalarType.c_type(0)
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _liwrk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _m,
        _kplus1,
        _lda,
        _xmin,
        _xmax,
        _x,
        _y,
        _w,
        _mf,
        _xf,
        _yf,
        _lyf,
        _ip,
        _a,
        _s,
        _np1,
        _wrk,
        _lwrk,
        _iwrk,
        _liwrk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lyf = {:s} and N = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lyf >= N, where N is the total number of constraints.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mf >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ip[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ip[I-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, xf[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and xf[J-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xf[I-1] != xf[J-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xf[I-1] lies outside interval [xmin,xmax]:',
                    'I = {:s}, xf[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmin = {:s} and xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmin < xmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[I-1] lies outside interval [xmin,xmax] for some I.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[I-1] lies outside interval [xmin,xmax]:',
                    'I = {:s}, x[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmin = {:s} and xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[i-1] >= x[i-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k+1 > m^(\'\')+n, where m^(\'\') is the number of data',
                    'points with nonzero weight and distinct abscissae different from',
                    'all xf, and n is the total number of constraints:',
                    'k+1 = {:s}, m^(\'\') = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The polynomials mu(x) and/or nu(x) cannot be found.',
                    'The problem is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        resid[:] = utils._EngineFloat64ArrayType.from_pointer(_wrk, (_m_py,))
        utils._unlink_pointer(_wrk)
        _np1_py = utils._EngineIntScalarType.to_py(_np1)
        _n_py = _np1_py-1
        return_data = _n_py
    _err_class.manage()
    return return_data

def dim1_cheb_deriv(n, xmin, xmax, a, ia1, la, adif, iadif1, ladif):
    u"""
    Derivative of fitted polynomial in Chebyshev series form.

    For full information please refer to the NAG Library document for
    e02ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02ahf.html

    Parameters
    ----------
    n : int
        n, the degree of the given polynomial p(x).

    xmin : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    xmax : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    a : float, shape (`la`)
        The Chebyshev coefficients of the polynomial p(x). Specifically,
        element i*`ia1` of `a` must contain the coefficient a_i, for i =
        0,1,...,n. Only these n+1 elements will be accessed.

        Unchanged on exit, but see `adif`, below.

    ia1 : int
        The index increment of `a`. Most frequently the Chebyshev
        coefficients are stored in adjacent elements of `a`, and `ia1`
        must be set to 1. However, if for example, they are stored in
        `a`[0],`a`[3],`a`[6],..., the value of `ia1` must be 3. See also
        [Further Comments].

    la : int
        The dimension of the array `a`.

    adif : float, shape (`ladif`), modified in place
        On exit: the Chebyshev coefficients of the derived polynomial
        q(x). (The differentiation is with respect to the variable x.)
        Specifically, element i*`iadif1` of `adif` contains the
        coefficient a__i, for i = 0,1,...,n-1. Additionally, element
        n*`iadif1` is set to zero. A call of the function may have the
        array name `adif` the same as `a`, provided that note is taken
        of the order in which elements are overwritten, when choosing
        the starting elements and increments `ia1` and `iadif1`, i.e.,
        the coefficients a_0,a_1,...,a_{i-1} must be intact after
        coefficient a__i is stored. In particular, it is possible to
        overwrite the a_i completely by having `ia1` = `iadif1`, and the
        actual arrays for `a` and `adif` identical.

    iadif1 : int
        The index increment of `adif`. Most frequently the Chebyshev
        coefficients are required in adjacent elements of `adif`, and
        `iadif1` must be set to 1. However, if, for example, they are to
        be stored in `adif`[0],`adif`[3],`adif`[6],..., the value of
        `iadif1` must be 3. See [Further Comments].

    ladif : int
        The dimension of the array `adif`.

    Returns
    -------
    patm1 : float
        The value of p(x_min). If this value is passed to the
        integration function ``dim1_cheb_integ`` with the coefficients
        of q(x), the original polynomial p(x) is recovered, including
        its constant coefficient.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ladif` = *<value>*, `n` = *<value>* and `iadif1`
            = *<value>*.

            Constraint: `ladif` > `n`*`iadif1`.

        (`errno` 1)
            On entry, `iadif1` = *<value>*.

            Constraint: `iadif1` >= 1.

        (`errno` 1)
            On entry, `la` = *<value>*, `n` = *<value>* and `ia1` =
            *<value>*.

            Constraint: `la` > `n`*`ia1`.

        (`errno` 1)
            On entry, `ia1` = *<value>*.

            Constraint: `ia1` >= 1.

        (`errno` 1)
            On entry, `xmax` = *<value>* and `xmin` = *<value>*.

            Constraint: `xmax` > `xmin`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, ia1.
        utils._BYREF_INT, # Input, la.
        utils._BYREF_FLOAT64, # Output, patm1.
        utils._EngineFloat64ArrayType.c_type, # Output, adif[ladif].
        utils._BYREF_INT, # Input, iadif1.
        utils._BYREF_INT, # Input, ladif.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ia1 = utils._EngineIntScalarType(
        ia1, {'fun_name': fun_name, 'entity': 'ia1'},
    ).to_ctypes()
    _ia1_py = utils._EngineIntScalarType.to_py(_ia1)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(1+_n_py*_ia1_py,),
    ).to_ctypes()
    _patm1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _ladif = utils._EngineIntScalarType(
        ladif, {'fun_name': fun_name, 'entity': 'ladif'},
    ).to_ctypes()
    _iadif1 = utils._EngineIntScalarType(
        iadif1, {'fun_name': fun_name, 'entity': 'iadif1'},
    ).to_ctypes()
    _iadif1_py = utils._EngineIntScalarType.to_py(_iadif1)
    _adif_cls = utils._EngineFloat64ArrayType(
        adif, {'fun_name': fun_name, 'entity': 'adif'},
        exp_shape=(1+_n_py*_iadif1_py,),
        check_contents=False,
    )
    _adif = _adif_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _xmin,
        _xmax,
        _a,
        _ia1,
        _la,
        _patm1,
        _adif,
        _iadif1,
        _ladif,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ladif = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'iadif1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ladif > n*iadif1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iadif1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iadif1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, la = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: la > n*ia1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmax = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmax > xmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _adif_cls.refresh_py(_adif)
        _patm1_py = utils._EngineFloat64ScalarType.to_py(_patm1)
        return_data = _patm1_py
    _err_class.manage()
    return return_data

def dim1_cheb_integ(n, xmin, xmax, a, ia1, la, qatm1, aintc, iaint1, laint):
    u"""
    Integral of fitted polynomial in Chebyshev series form.

    For full information please refer to the NAG Library document for
    e02aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02ajf.html

    Parameters
    ----------
    n : int
        n, the degree of the given polynomial p(x).

    xmin : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    xmax : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    a : float, shape (`la`)
        The Chebyshev coefficients of the polynomial p(x). Specifically,
        element i*`ia1` of `a` must contain the coefficient a_i, for i =
        0,1,...,n. Only these n+1 elements will be accessed.

        Unchanged on exit, but see `aintc`, below.

    ia1 : int
        The index increment of `a`. Most frequently the Chebyshev
        coefficients are stored in adjacent elements of `a`, and `ia1`
        must be set to 1. However, if for example, they are stored in
        `a`[0],`a`[3],`a`[6],..., the value of `ia1` must be 3. See also
        [Further Comments].

    la : int
        The dimension of the array `a`.

    qatm1 : float
        The value that the integrated polynomial is required to have at
        the lower end point of its interval of definition, i.e., at x_=
        -1 which corresponds to x = x_min. Thus, `qatm1` is a constant
        of integration and will normally be set to zero by you.

    aintc : float, shape (`laint`), modified in place
        On exit: the Chebyshev coefficients of the integral q(x). (The
        integration is with respect to the variable x, and the constant
        coefficient is chosen so that q(x_min) equals `qatm1`).
        Specifically, element i*`iaint1` of `aintc` contains the
        coefficient a'_i, for i = 0,1,...,n+1. A call of the function
        may have the array name `aintc` the same as `a`, provided that
        note is taken of the order in which elements are overwritten
        when choosing starting elements and increments `ia1` and
        `iaint1`: i.e., the coefficients, a_0,a_1,...,a_{i-2} must be
        intact after coefficient a'_i is stored. In particular it is
        possible to overwrite the a_i entirely by having `ia1` =
        `iaint1`, and the actual array for `a` and `aintc` identical.

    iaint1 : int
        The index increment of `aintc`. Most frequently the Chebyshev
        coefficients are required in adjacent elements of `aintc`, and
        `iaint1` must be set to 1. However, if, for example, they are to
        be stored in `aintc`[0],`aintc`[3],`aintc`[6],..., the value of
        `iaint1` must be 3. See also [Further Comments].

    laint : int
        The dimension of the array `aintc`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `laint` = *<value>*, `n` = *<value>* and `iaint1`
            = *<value>*.

            Constraint: `laint` > (`n`+1)*`iaint1`.

        (`errno` 1)
            On entry, `iaint1` = *<value>*.

            Constraint: `iaint1` >= 1.

        (`errno` 1)
            On entry, `la` = *<value>*, `n` = *<value>* and `ia1` =
            *<value>*.

            Constraint: `la` > `n`*`ia1`.

        (`errno` 1)
            On entry, `ia1` = *<value>*.

            Constraint: `ia1` >= 1.

        (`errno` 1)
            On entry, `xmax` = *<value>* and `xmin` = *<value>*.

            Constraint: `xmax` > `xmin`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_integ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, ia1.
        utils._BYREF_INT, # Input, la.
        utils._BYREF_FLOAT64, # Input, qatm1.
        utils._EngineFloat64ArrayType.c_type, # Output, aintc[laint].
        utils._BYREF_INT, # Input, iaint1.
        utils._BYREF_INT, # Input, laint.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ia1 = utils._EngineIntScalarType(
        ia1, {'fun_name': fun_name, 'entity': 'ia1'},
    ).to_ctypes()
    _ia1_py = utils._EngineIntScalarType.to_py(_ia1)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(1+_n_py*_ia1_py,),
    ).to_ctypes()
    _qatm1 = utils._EngineFloat64ScalarType(
        qatm1, {'fun_name': fun_name, 'entity': 'qatm1'},
    ).to_ctypes()
    _laint = utils._EngineIntScalarType(
        laint, {'fun_name': fun_name, 'entity': 'laint'},
    ).to_ctypes()
    _iaint1 = utils._EngineIntScalarType(
        iaint1, {'fun_name': fun_name, 'entity': 'iaint1'},
    ).to_ctypes()
    _iaint1_py = utils._EngineIntScalarType.to_py(_iaint1)
    _aintc_cls = utils._EngineFloat64ArrayType(
        aintc, {'fun_name': fun_name, 'entity': 'aintc'},
        exp_shape=(1+(_n_py+1)*_iaint1_py,),
        check_contents=False,
    )
    _aintc = _aintc_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _xmin,
        _xmax,
        _a,
        _ia1,
        _la,
        _qatm1,
        _aintc,
        _iaint1,
        _laint,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, laint = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'iaint1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: laint > (n+1)*iaint1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iaint1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iaint1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, la = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: la > n*ia1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmax = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmax > xmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _aintc_cls.refresh_py(_aintc)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_cheb_eval2(n, xmin, xmax, a, ia1, la, x):
    u"""
    Evaluation of fitted polynomial in one variable from Chebyshev
    series form.

    For full information please refer to the NAG Library document for
    e02ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02akf.html

    Parameters
    ----------
    n : int
        n, the degree of the given polynomial p(x_).

    xmin : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    xmax : float
        The lower and upper end points respectively of the interval
        [x_min,x_max]. The Chebyshev series representation is in terms
        of the normalized variable x_, where

            x_= (2x-(x_max+x_min))/(x_max-x_min).

    a : float, shape (`la`)
        The Chebyshev coefficients of the polynomial p(x_).
        Specifically, element i*`ia1` must contain the coefficient a_i,
        for i = 0,1,...,n. Only these n+1 elements will be accessed.

    ia1 : int
        The index increment of `a`. Most frequently, the Chebyshev
        coefficients are stored in adjacent elements of `a`, and `ia1`
        must be set to 1. However, if, for example, they are stored in
        `a`[0],`a`[3],`a`[6],..., the value of `ia1` must be 3.

    la : int
        The dimension of the array `a`.

    x : float
        The argument x at which the polynomial is to be evaluated.

    Returns
    -------
    result : float
        The value of the polynomial p(x_).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `xmax` = *<value>* and `xmin` = *<value>*.

            Constraint: `xmax` > `xmin`.

        (`errno` 1)
            On entry, `la` = *<value>*, `n` = *<value>* and `ia1` =
            *<value>*.

            Constraint: `la` > `n`*`ia1`.

        (`errno` 1)
            On entry, `ia1` = *<value>*.

            Constraint: `ia1` >= 1.

        (`errno` 1)
            On entry, `n`+1 = *<value>*>.

            Constraint: `n` >= 0.

        (`errno` 2)
            On entry, `x` does not lie in [`xmin`,`xmax`]: `x` =
            *<value>*, `xmin` = *<value>* and `xmax` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_cheb_eval2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02akft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, ia1.
        utils._BYREF_INT, # Input, la.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, result.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ia1 = utils._EngineIntScalarType(
        ia1, {'fun_name': fun_name, 'entity': 'ia1'},
    ).to_ctypes()
    _ia1_py = utils._EngineIntScalarType.to_py(_ia1)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py*_ia1_py+1,),
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _xmin,
        _xmax,
        _a,
        _ia1,
        _la,
        _x,
        _result,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmax = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmax > xmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, la = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: la > n*ia1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n+1 = {:s}>.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x does not lie in [xmin,xmax]:',
                    'x = {:s}, xmin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        return_data = _result_py
    _err_class.manage()
    return return_data

def dim1_minimax_polynomial(n, x, y, m, a):
    u"""
    Minimax curve fit by polynomials.

    For full information please refer to the NAG Library document for
    e02al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02alf.html

    Parameters
    ----------
    n : int
        n, the number of data points.

    x : float, shape (`n`)
        The values of the x coordinates, x_i, for i = 1,2,...,n.

    y : float, shape (`n`)
        The values of the y coordinates, y_i, for i = 1,2,...,n.

    m : int
        m, where m is the degree of the polynomial to be found.

    a : float, shape (`m`+1), modified in place
        On exit: the coefficients a_i of the minimax polynomial, for i =
        0,1,...,m.

    Returns
    -------
    ref : float
        The final reference deviation, i.e., the maximum deviation of
        the computed polynomial evaluated at x_i from the reference
        values y_i, for i = 1,2,...,n. this argument may return a
        negative value which indicates that the algorithm started to
        cycle due to round-off errors.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` < 100.

        (`errno` 2)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` < `n`-1.

        (`errno` 3)
            On entry, i = *<value>*, `x`[i] = *<value>* and `x`[i-1] =
            *<value>*.

            Constraint: `x`[i] > `x`[i-1].
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_minimax_polynomial'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02alft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Output, a[m+1].
        utils._BYREF_FLOAT64, # Output, ref.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py+1,),
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _ref = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _y,
        _m,
        _a,
        _ref,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m < 100.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m < n-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, x[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[i] > x[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _ref_py = utils._EngineFloat64ScalarType.to_py(_ref)
        return_data = _ref_py
    _err_class.manage()
    return return_data

def dim1_spline_knots(m, ncap7, x, y, w, lamda, c):
    u"""
    Least squares curve cubic spline fit (including interpolation).

    For full information please refer to the NAG Library document for
    e02ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02baf.html

    Parameters
    ----------
    m : int
        The number m of data points.

    ncap7 : int
        n_+7, where n_ is the number of intervals of the spline (which
        is one greater than the number of interior knots, i.e., the
        knots strictly within the range x_1 to x_m) over which the
        spline is defined.

    x : float, shape (`m`)
        The values x_r of the independent variable (abscissa), for r =
        1,2,...,m. The values must satisfy the Schoenberg--Whitney
        conditions (see [Further Comments]).

    y : float, shape (`m`)
        The values y_r of the dependent variable (ordinate), for r =
        1,2,...,m.

    w : float, shape (`m`)
        The values w_r of the weights, for r = 1,2,...,m. For advice on
        the choice of weights, see [the E02 Introduction].

    lamda : float, shape (`ncap7`), modified in place
        On entry: `lamda`[i-1] must be set to the (i-4)th (interior)
        knot, lambda_i, for i = 5,6,...,n_+3.

        On exit: the input values are unchanged, and `lamda`[i-1], for i
        = 1,2,...,4, `ncap7`-3, `ncap7`-2, `ncap7`-1, `ncap7` contains
        the additional (exterior) knots introduced by the function. For
        advice on the choice of knots, see [the E02 Introduction].

    c : float, shape (`ncap7`), modified in place
        On exit: the coefficient c_i of the B-spline N_i(x), for i =
        1,2,...,n_+3. The remaining elements of the array are not used.

    Returns
    -------
    ss : float
        The residual sum of squares, theta.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, J = *<value>*, `lamda`[J-1] = *<value>* and
            `lamda`[J] = *<value>*.

            Constraint: `lamda`[J-1] <= `lamda`[J].

        (`errno` 1)
            On entry, `ncap7` = *<value>*, `lamda`[`ncap7`-5] =
            *<value>*, `m` = *<value>* and `x`[`m`-1] = *<value>*.

            Constraint: `lamda`[`ncap7`-5] < `x`[`m`-1].

        (`errno` 1)
            On entry, `lamda`[4] = *<value>* and `x`[0] = *<value>*.

            Constraint: `lamda`[4] > `x`[0].

        (`errno` 2)
            On entry, i = *<value>* and `w`[i-1] = *<value>*.

            Constraint: `w`[i-1] > 0.0.

        (`errno` 3)
            On entry, the `x` values are not in nondecreasing order. I =
            *<value>*, `x`[I-1] = *<value>*, J = *<value>* and
            xdist[J-1] = *<value>*.

            Constraint: `x`[I-1] >= xdist[J-1], where xdist is the set
            of distinct x-values.

        (`errno` 4)
            On entry, `ncap7` = *<value>* and mdist = *<value>*.

            Constraint: `ncap7` <= mdist+4, where mdist is the number of
            distinct x-values.

        (`errno` 4)
            On entry, `ncap7` = *<value>* and `m` = *<value>*.

            Constraint: `ncap7` <= `m`+4.

        (`errno` 4)
            On entry, `ncap7` = *<value>*.

            Constraint: `ncap7` >= 8.

        (`errno` 5)
            On entry, the Schoenberg--Whitney conditions fail to hold
            for at least one subset of the distinct data abscissae. I =
            *<value>*, xdist[I-1] = *<value>*, J = *<value>* and
            `lamda`[J-1] = *<value>*.

            Constraint: xdist[I-1] < `lamda`[J-1], where xdist is the
            set of distinct x-values.

        (`errno` 5)
            On entry, the Schoenberg--Whitney conditions fail to hold
            for at least one subset of the distinct data abscissae. L =
            *<value>*, xdist[L-1] = *<value>*, I = *<value>* and
            `lamda`[I-1] = *<value>*.

            Constraint: xdist[L-1] > `lamda`[I-1], where xdist is the
            set of distinct x-values.

        (`errno` 5)
            On entry, the Schoenberg--Whitney conditions fail to hold
            for at least one subset of the distinct data abscissae. J =
            *<value>*, xdist[J-1] = *<value>*, J+4 = *<value>* and
            `lamda`[J+3] = *<value>*.

            Constraint: xdist[J-1] < `lamda`[J+3], where xdist is the
            set of distinct x-values.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_knots'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, ncap7.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, lamda[ncap7].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work2.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ncap7].
        utils._BYREF_FLOAT64, # Output, ss.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ncap7 = utils._EngineIntScalarType(
        ncap7, {'fun_name': fun_name, 'entity': 'ncap7'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ncap7_py = utils._EngineIntScalarType.to_py(_ncap7)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ncap7_py,),
    )
    _lamda = _lamda_cls.to_ctypes()
    _work1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _work2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ncap7_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _ss = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _ncap7,
        _x,
        _y,
        _w,
        _lamda,
        _work1,
        _work2,
        _c,
        _ss,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, J = {:s}, lamda[J-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and lamda[J] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[J-1] <= lamda[J].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[ncap7-5] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'm = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'x[m-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: lamda[ncap7-5] < x[m-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[4] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lamda[4] > x[0].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and w[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: w[i-1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the x values are not in nondecreasing order.',
                    'I = {:s}, x[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'J = {:s} and xdist[J-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: x[I-1] >= xdist[J-1], where xdist is the set of',
                    'distinct x-values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mdist = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncap7 <= mdist+4, where mdist is the number of',
                    'distinct x-values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncap7 <= m+4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncap7 >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the Schoenberg--Whitney conditions fail to hold for at least one',
                    'subset of the distinct data abscissae. I = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'xdist[I-1] = {:s}, J = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'lamda[J-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xdist[I-1] < lamda[J-1], where xdist is the set of',
                    'distinct x-values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the Schoenberg--Whitney conditions fail to hold for at least one',
                    'subset of the distinct data abscissae.',
                    'L = {:s}, xdist[L-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'I = {:s} and lamda[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xdist[L-1] > lamda[I-1], where xdist is the set of',
                    'distinct x-values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the Schoenberg--Whitney conditions fail to hold for at least one',
                    'subset of the distinct data abscissae. J = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'xdist[J-1] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'J+4 = {:s} and lamda[J+3] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: xdist[J-1] < lamda[J+3], where xdist is the set of',
                    'distinct x-values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lamda_cls.refresh_py(_lamda)
        _c_cls.refresh_py(_c)
        _ss_py = utils._EngineFloat64ScalarType.to_py(_ss)
        return_data = _ss_py
    _err_class.manage()
    return return_data

def dim1_spline_eval(ncap7, lamda, c, x):
    u"""
    Evaluation of fitted cubic spline, function only.

    For full information please refer to the NAG Library document for
    e02bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02bbf.html

    Parameters
    ----------
    ncap7 : int
        n_+7, where n_ is the number of intervals (one greater than the
        number of interior knots, i.e., the knots strictly within the
        range lambda_4 to lambda_{n_+4}) over which the spline is
        defined.

    lamda : float, shape (`ncap7`)
        `lamda`[j-1] must be set to the value of the jth member of the
        complete set of knots, lambda_j, for j = 1,2,...,n_+7.

    c : float, shape (`ncap7`)
        The coefficient c_i of the B-spline N_i(x), for i =
        1,2,...,n_+3. The remaining elements of the array are not
        referenced.

    x : float
        The argument x at which the cubic spline is to be evaluated.

    Returns
    -------
    s : float
        The value of the spline, s(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*, `ncap7` = *<value>* and
            `lamda`[`ncap7`-4] = *<value>*.

            Constraint: `x` <= `lamda`[`ncap7`-4].

        (`errno` 1)
            On entry, `x` = *<value>* and `lamda`[3] = *<value>*.

            Constraint: `x` >= `lamda`[3].

        (`errno` 2)
            On entry, `ncap7` = *<value>*.

            Constraint: `ncap7` >= 8.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ncap7.
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[ncap7].
        utils._EngineFloat64ArrayType.c_type, # Input, c[ncap7].
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, s.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ncap7 = utils._EngineIntScalarType(
        ncap7, {'fun_name': fun_name, 'entity': 'ncap7'},
    ).to_ctypes()
    _ncap7_py = utils._EngineIntScalarType.to_py(_ncap7)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ncap7,
        _lamda,
        _c,
        _x,
        _s,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, ncap7 = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and lamda[ncap7-4] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x <= lamda[ncap7-4].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[3] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x >= lamda[3].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncap7 >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_py = utils._EngineFloat64ScalarType.to_py(_s)
        return_data = _s_py
    _err_class.manage()
    return return_data

def dim1_spline_deriv(ncap7, lamda, c, x, left, s):
    u"""
    Evaluation of fitted cubic spline, function and derivatives.

    For full information please refer to the NAG Library document for
    e02bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02bcf.html

    Parameters
    ----------
    ncap7 : int
        n_+7, where n_ is the number of intervals of the spline (which
        is one greater than the number of interior knots, i.e., the
        knots strictly within the range lambda_4 to lambda_{n_+4} over
        which the spline is defined).

    lamda : float, shape (`ncap7`)
        `lamda`[j-1] must be set to the value of the jth member of the
        complete set of knots, lambda_j, for j = 1,2,...,n_+7.

    c : float, shape (`ncap7`)
        The coefficient c_i of the B-spline N_i(x), for i =
        1,2,...,n_+3. The remaining elements of the array are not
        referenced.

    x : float
        The argument x at which the cubic spline and its derivatives are
        to be evaluated.

    left : int
        Specifies whether left- or right-hand values of the spline and
        its derivatives are to be computed (see [Description]). Left -
        or right-hand values are formed according to whether `left` is
        equal or not equal to 1.

        If x does not coincide with a knot, the value of `left` is
        immaterial.

        If x = `lamda`[3], right-hand values are computed.

        If x = `lamda`[`ncap7`-4], left-hand values are formed,
        regardless of the value of `left`.

    s : float, shape (4), modified in place
        On exit: `s`[j-1] contains the value of the (j-1)th derivative
        of the spline at the argument x, for j = 1,2,...,4. Note that
        `s`[0] contains the value of the spline.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ncap7` = *<value>*.

            Constraint: `ncap7` >= 8.

        (`errno` 2)
            On entry, `x` = *<value>*, `ncap7` = *<value>* and
            `lamda`[`ncap7`-4] = *<value>*.

            Constraint: `x` <= `lamda`[`ncap7`-4].

        (`errno` 2)
            On entry, `x` = *<value>* and `lamda`[3] = *<value>*.

            Constraint: `x` >= `lamda`[3].

        (`errno` 2)
            On entry, `lamda`[3] = *<value>*, `ncap7` = *<value>* and
            `lamda`[`ncap7`-4] = *<value>*.

            Constraint: `lamda`[3] < `lamda`[`ncap7`-4].

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ncap7.
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[ncap7].
        utils._EngineFloat64ArrayType.c_type, # Input, c[ncap7].
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, left.
        utils._EngineFloat64ArrayType.c_type, # Output, s[4].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ncap7 = utils._EngineIntScalarType(
        ncap7, {'fun_name': fun_name, 'entity': 'ncap7'},
    ).to_ctypes()
    _ncap7_py = utils._EngineIntScalarType.to_py(_ncap7)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _left = utils._EngineIntScalarType(
        left, {'fun_name': fun_name, 'entity': 'left'},
    ).to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(4,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ncap7,
        _lamda,
        _c,
        _x,
        _left,
        _s,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncap7 >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, ncap7 = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and lamda[ncap7-4] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x <= lamda[ncap7-4].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[3] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x >= lamda[3].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[3] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ncap7 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[ncap7-4] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[3] < lamda[ncap7-4].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_spline_integ(ncap7, lamda, c):
    u"""
    Evaluation of fitted cubic spline, definite integral.

    For full information please refer to the NAG Library document for
    e02bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02bdf.html

    Parameters
    ----------
    ncap7 : int
        n_+7, where n_ is the number of intervals of the spline (which
        is one greater than the number of interior knots, i.e., the
        knots strictly within the range a to b) over which the spline is
        defined.

    lamda : float, shape (`ncap7`)
        `lamda`[j-1] must be set to the value of the jth member of the
        complete set of knots, lambda_j, for j = 1,2,...,n_+7.

    c : float, shape (`ncap7`)
        The coefficient c_i of the B-spline N_i(x), for i =
        1,2,...,n_+3. The remaining elements of the array are not
        referenced.

    Returns
    -------
    dint : float
        The value of the definite integral of s(x) between the limits x
        = a and x = b, where a = lambda_4 and b = lambda_{n_+4}.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ncap7` = *<value>*.

            Constraint: `ncap7` >= 8.

        (`errno` 2)
            On entry, J = *<value>*, `lamda`[J-1] = *<value>* and
            `lamda`[J-2] = *<value>*.

            Constraint: `lamda`[J-1] >= `lamda`[J-2].

        (`errno` 2)
            On entry, `ncap7` = *<value>*, `lamda`[`ncap7`-2] =
            *<value>* and `lamda`[`ncap7`-1] = *<value>*.

            Constraint: `lamda`[`ncap7`-2] = `lamda`[`ncap7`-1].

        (`errno` 2)
            On entry, `ncap7` = *<value>*, `lamda`[`ncap7`-3] =
            *<value>* and `lamda`[`ncap7`-2] = *<value>*.

            Constraint: `lamda`[`ncap7`-3] = `lamda`[`ncap7`-2].

        (`errno` 2)
            On entry, `ncap7` = *<value>*, `lamda`[`ncap7`-4] =
            *<value>* and `lamda`[`ncap7`-4] = *<value>*.

            Constraint: `lamda`[`ncap7`-4] = `lamda`[`ncap7`-3].

        (`errno` 2)
            On entry, `lamda`[2] = *<value>* and `lamda`[3] = *<value>*.

            Constraint: `lamda`[2] = `lamda`[3].

        (`errno` 2)
            On entry, `lamda`[1] = *<value>* and `lamda`[2] = *<value>*.

            Constraint: `lamda`[1] = `lamda`[2].

        (`errno` 2)
            On entry, `lamda`[0] = *<value>* and `lamda`[1] = *<value>*.

            Constraint: `lamda`[0] = `lamda`[1].

        (`errno` 2)
            On entry, `lamda`[3] = *<value>*, NCAP4 = *<value>* and
            `lamda`[NCAP4-1] = *<value>*.

            Constraint: `lamda`[3] < `lamda`[NCAP4-1].

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_integ'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ncap7.
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[ncap7].
        utils._EngineFloat64ArrayType.c_type, # Input, c[ncap7].
        utils._BYREF_FLOAT64, # Output, dint.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ncap7 = utils._EngineIntScalarType(
        ncap7, {'fun_name': fun_name, 'entity': 'ncap7'},
    ).to_ctypes()
    _ncap7_py = utils._EngineIntScalarType.to_py(_ncap7)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _dint = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ncap7,
        _lamda,
        _c,
        _dint,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncap7 >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, J = {:s}, lamda[J-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and lamda[J-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[J-1] >= lamda[J-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[ncap7-2] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[ncap7-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[ncap7-2] = lamda[ncap7-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[ncap7-3] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[ncap7-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[ncap7-3] = lamda[ncap7-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[ncap7-4] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[ncap7-4] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[ncap7-4] = lamda[ncap7-3].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[2] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[3] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lamda[2] = lamda[3].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lamda[1] = lamda[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'lamda[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lamda[0] = lamda[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[3] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'NCAP4 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[NCAP4-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[3] < lamda[NCAP4-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _dint_py = utils._EngineFloat64ScalarType.to_py(_dint)
        return_data = _dint_py
    _err_class.manage()
    return return_data

def dim1_spline_auto(start, m, x, y, w, s, nest, n, lamda, c, comm):
    u"""
    Least squares cubic spline curve fit, automatic knot placement.

    For full information please refer to the NAG Library document for
    e02be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02bef.html

    Parameters
    ----------
    start : str, length 1
        Must be set to 'C' or 'W'.

        `start` = 'C'

          The function will build up the knot set starting with no
          interior knots. No values need be assigned to the arguments
          `n`, `lamda`, `comm`['wrk'] or `comm`['iwrk'].

        `start` = 'W'

          The function will restart the knot-placing strategy using the
          knots found in a previous call of the function. In this case,
          the arguments `n`, `lamda`, `comm`['wrk'], and `comm`['iwrk']
          must be unchanged from that previous call. This warm start can
          save much time in searching for a satisfactory value of `s`.

    m : int
        m, the number of data points.

    x : float, shape (`m`)
        The values x_r of the independent variable (abscissa) x, for r =
        1,2,...,m.

    y : float, shape (`m`)
        The values y_r of the dependent variable (ordinate) y, for r =
        1,2,...,m.

    w : float, shape (`m`)
        The values w_r of the weights, for r = 1,2,...,m. For advice on
        the choice of weights, see [the E02 Introduction].

    s : float
        The smoothing factor, S.

        If S = 0.0, the function returns an interpolating spline.

        If S is smaller than machine precision, it is assumed equal to
        zero.

        For advice on the choice of S, see [Description].

    nest : int
        An overestimate for the number, n, of knots required.

    n : int
        If the warm start option is used, the value of `n` must be left
        unchanged from the previous call.

    lamda : float, shape (`nest`), modified in place
        On entry: if the warm start option is used, the values
        `lamda`[0],`lamda`[1],...,`lamda`[ `n` -1] must be left
        unchanged from the previous call.

        On exit: the knots of the spline, i.e., the positions of the
        interior knots `lamda`[4], `lamda`[5],...,`lamda`[ `n`-5] as
        well as the positions of the additional knots

            `lamda`[0] = `lamda`[1] = `lamda`[2] = `lamda`[3] = x_1

        and

            `lamda`[ `n`-4] = `lamda`[ `n`-3] = `lamda`[ `n`-2] =
            `lamda`[ `n` -1] = x_m

        needed for the B-spline representation.

    c : float, shape (`nest`), modified in place
        On exit: the coefficient c_i of the B-spline N_i(x) in the
        spline approximation s(x), for i = 1,2,...,n-4.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    n : int
        The total number, n, of knots of the computed spline.

    fp : float
        The sum of the squared weighted residuals, theta, of the
        computed spline approximation. If `fp` = 0.0, this is an
        interpolating spline. `fp` should equal `s` within a relative
        tolerance of 0.001 unless n = 8 when the spline has no interior
        knots and so is simply a cubic polynomial. For knots to be
        inserted, `s` must be set to a value below the value of `fp`
        produced in this case.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, r = *<value>* and `w`[r-1] = *<value>*.

            Constraint: `w`[r-1] > 0.0 for all r.

        (`errno` 1)
            On entry, `nest` = *<value>*.

            Constraint: `nest` >= 8.

        (`errno` 1)
            On entry, `nest` = *<value>*.

            Constraint: `nest` >= *<value>* when `s` = 0.0.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` >= 0.0.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 4.

        (`errno` 1)
            On entry, `start` != 'W' or 'C': `start` = *<value>*.

        (`errno` 2)
            On entry, r = *<value>* and `w`[r-1] = *<value>*.

            Constraint: `w`[r-1] > 0.0 for all r.

        (`errno` 3)
            On entry, r = *<value>*, `x`[r-2] = *<value>* and `x`[r-1] =
            *<value>*.

            Constraint: `x`[r-2] < `x`[r-1] for all r.

        (`errno` 4)
            The number of knots needed is greater than `nest`: `nest` =
            *<value>*. Possibly `s` is too small: `s` = *<value>*.

        (`errno` 5)
            The iterative process has failed to converge. Possibly `s`
            is too small: `s` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_auto'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, start(len=1).
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_INT, # Input, nest.
        utils._BYREF_INT, # Input/Output, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, lamda[nest].
        utils._EngineFloat64ArrayType.c_type, # Output, c[nest].
        utils._BYREF_FLOAT64, # Output, fp.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrk[lwrk].
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Input/Output, iwrk[nest].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of start.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _start = utils._EngineCharScalarType(
        start, {'fun_name': fun_name, 'entity': 'start'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _nest = utils._EngineIntScalarType(
        nest, {'fun_name': fun_name, 'entity': 'nest'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nest_py = utils._EngineIntScalarType.to_py(_nest)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_nest_py,),
    )
    _lamda = _lamda_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_nest_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _fp = utils._EngineFloat64ScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wrk', 'iwrk'),
    )
    if _comm_needs_init:
        _wrk = utils._EngineFloat64ArrayType.empty_ctypes(4*_m_py+16*_nest_py+41)
        comm['wrk'] = _wrk
    else:
        _wrk = comm['wrk']
    _lwrk_py = utils._EngineFloat64ArrayType.get_shape(comm['wrk'], {'fun_name': fun_name, 'entity': 'comm[\'wrk\']'}, exp_rank=1, validate=False)[0]
    _lwrk = utils._EngineIntScalarType(
        _lwrk_py, {'fun_name': fun_name, 'entity': '_lwrk'},
    ).to_ctypes()
    if _comm_needs_init:
        _iwrk = utils._EngineIntArrayType.empty_ctypes(_nest_py)
        comm['iwrk'] = _iwrk
    else:
        _iwrk = comm['iwrk']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _start,
        _m,
        _x,
        _y,
        _w,
        _s,
        _nest,
        _n,
        _lamda,
        _c,
        _fp,
        _wrk,
        _lwrk,
        _iwrk,
        _errbuf,
        _ierr,
        _start._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s} and w[r-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: w[r-1] > 0.0 for all r.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nest >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nest >= {:s} when s = 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, start != \'W\' or \'C\': start = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s} and w[r-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: w[r-1] > 0.0 for all r.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}, x[r-2] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and x[r-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[r-2] < x[r-1] for all r.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of knots needed is greater than nest:',
                    'nest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative process has failed to converge.',
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _n_py = utils._EngineIntScalarType.to_py(_n)
        _lamda_cls.refresh_py(_lamda)
        _c_cls.refresh_py(_c)
        _fp_py = utils._EngineFloat64ScalarType.to_py(_fp)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['n', 'fp'],
        )(
            _n_py,
            _fp_py,
        )
    _err_class.manage()
    return return_data

def dim1_spline_deriv_vector(start, ncap7, lamda, c, deriv, xord, x, ixloc, nx, s, iwrk, liwrk):
    u"""
    Evaluation of fitted cubic spline, function and optionally
    derivatives at a vector of points.

    For full information please refer to the NAG Library document for
    e02bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02bff.html

    Parameters
    ----------
    start : int
        Indicates the completion state of the first phase of the
        algorithm.

        `start` = 0

          The enclosing interval numbers ixloc_j for the abscissae x_j
          contained in `x` have not been determined, and you wish to use
          the sorted mode of vectorization.

        `start` = 1

          The enclosing interval numbers ixloc_j have been determined
          and are provided in `ixloc`, however the required permutation
          and interval related information has not been determined and
          you wish to use the sorted mode of vectorization.

        `start` = 2

          You wish to use the sorted mode of vectorization, and the
          entire first phase has been completed, with the enclosing
          interval numbers supplied in `ixloc`, and the required
          permutation and interval related information provided in
          `iwrk` (from a previous call to ``dim1_spline_deriv_vector``).

        `start` = 10

          The enclosing interval numbers ixloc_j for the abscissae x_j
          contained in `x` have not been determined, and you wish to use
          the unsorted mode of vectorization.

        `start` = 11

          The enclosing interval numbers ixloc_j for the abscissae x_j
          contained in `x` have been supplied in `ixloc`, and you wish
          to use the unsorted mode of vectorization.

    ncap7 : int
        n_+7, where n_ is the number of intervals of the spline (which
        is one greater than the number of interior knots, i.e., the
        knots strictly within the range lambda_4 to lambda_{n_+4} over
        which the spline is defined). Note that if ``dim1_spline_auto``
        was used to generate the knots and spline coefficients then
        `ncap7` should contain the same value as returned in n by
        ``dim1_spline_auto``.

    lamda : float, shape (`ncap7`)
        `lamda`[j-1] must be set to the value of the jth member of the
        complete set of knots, lambda_j, for j = 1,2,...,n_+7.

    c : float, shape (`ncap7`)
        The coefficient c_i of the B-spline N_i(x), for i =
        1,2,...,n_+3. The remaining elements of the array are not
        referenced.

    deriv : int
        Determines the maximum order of derivatives required, D_ord.

        If `deriv` < 0 left derivatives are calculated, otherwise right
        derivatives are calculated. For abscissae satisfying x_j =
        lambda_4 or x_j = lambda_{n_+4} only right-handed or left-handed
        computation will be used respectively.

        For abscissae which do not coincide exactly with a knot, the
        handedness of the computation is immaterial.

        `deriv` = 0

          No derivatives required.

        `deriv` = +/-1

          Only s(x) and its first derivative are required.

        `deriv` = +/-2

          Only s(x) and its first and second derivatives are required.

        `deriv` = +/-3

          s(x) and its first, second and third derivatives are required.

        Note: if |`deriv`| is greater than 3 only the derivatives up to
        and including 3 will be returned.

    xord : int
        Indicates whether `x` is supplied in a sufficiently ordered
        manner. If `x` is sufficiently ordered
        ``dim1_spline_deriv_vector`` will complete faster.

        `xord` != 0

          The abscissae in `x` are ordered at least by ascending
          interval, in that any two abscissae contained in the same
          interval are only separated by abscissae in the same interval,
          and the intervals are arranged in ascending order. For
          example, x_j < x_{j+1}, for j = 1,2,...,`nx`-1.

        `xord` = 0

          The abscissae in `x` are not sufficiently ordered.

    x : float, shape (`nx`)
        The abscissae x_j, for j = 1,2,...,n_x. If `start` = 0 or 10
        then evaluations will only be performed for these x_j satisfying
        lambda_4 <= x_j <= lambda_{n_+4}. Otherwise evaluation will be
        performed unless the corresponding element of `ixloc` contains
        an invalid interval number. Please note that if the `ixloc`[j]
        is a valid interval number then no check is made that `x`[j]
        actually lies in that interval.

    ixloc : int, shape (`nx`), modified in place
        On entry: if `start` = 1, 2 or 11, if you wish x_j to be
        evaluated, `ixloc`[j-1] must be the enclosing interval number
        ixloc_j of the abscissae x_j (see [equation]). If you do not
        wish x_j to be evaluated, you may set the interval number to be
        either less than 4 or greater than n_+4.

        Otherwise, `ixloc` need not be set.

        On exit: if `start` = 1, 2 or 11, `ixloc` is unchanged on exit.

        Otherwise, `ixloc`[j-1], contains the enclosing interval number
        ixloc_j, for the abscissa supplied in `x`[j-1], for j =
        1,2,...,n_x.

        Evaluations will only be performed for abscissae x_j satisfying
        lambda_4 <= x_j <= lambda_{n_+4}.

        If evaluation is not performed `ixloc`[j-1] is set to 0 if x_j <
        lambda_4 or n_+7 if x_j > lambda_{n_+4}.

    nx : int
        n_x, the total number of abscissae contained in `x`, including
        any that will not be evaluated.

    s : float, shape (`nx`, min(|`deriv`|, 3)+1), modified in place
        On exit: if x_j is valid, `s`[j-1,d-1] will contain the (d-1)th
        derivative of s(x), for j = 1,2,...,n_x, for d =
        1,2,...,D_ord+1. In particular, `s`[j-1,0] will contain the
        approximation of s(x_j) for all legal values in `x`.

    iwrk : None or int, shape (`liwrk`), modified in place
        Optionally, on entry: if `start` = 2, `iwrk` must be unchanged
        from a previous call to ``dim1_spline_deriv_vector`` with
        `start` = 0 or 1.

        Otherwise, `iwrk` need not be set.

        On exit, if not None on entry: if `start` = 10 or 11, `iwrk` is
        unchanged on exit.

        Otherwise, `iwrk` contains the required permutation of elements
        of `x`, if any, and information related to the division of the
        abscissae x_j between the intervals derived from `lamda`.

    liwrk : int
        The dimension of the array `iwrk`.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, all elements of `x` had enclosing interval numbers
            in `ixloc` outside the domain allowed by the provided
            spline.

            *<value>* entries of `x` were indexed below the lower bound
            *<value>*.

            *<value>* entries of `x` were indexed above the upper bound
            *<value>*.

        (`errno` 11)
            On entry, `start` = *<value>*.

            Constraint: `start` = 0, 1, 2, 10 or 11.

        (`errno` 12)
            On entry, `start` = 2 and `nx` is not consistent with the
            previous call to ``dim1_spline_deriv_vector``.

            On entry, `nx` = *<value>*.

            Constraint: `nx` = *<value>*.

        (`errno` 21)
            On entry, `ncap7` = *<value>*.

            Constraint: `ncap7` >= 8.

        (`errno` 31)
            On entry, `lamda`[3] = *<value>*, `ncap7` = *<value>* and
            `lamda`[`ncap7`-4] = *<value>*.

            Constraint: `lamda`[3] < `lamda`[`ncap7`-4].

        (`errno` 91)
            On entry, `nx` = *<value>*.

            Constraint: `nx` >= 1.

        (`errno` 131)
            On entry, `liwrk` = *<value>*.

            Constraint: `liwrk` >= 3*`nx`+3 = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one element of `x` has an enclosing
            interval number in `ixloc` outside the set allowed by the
            provided spline. The spline has been evaluated for all `x`
            with enclosing interval numbers inside the allowable set.

            *<value>* entries of `x` were indexed below the lower bound
            *<value>*.

            *<value>* entries of `x` were indexed above the upper bound
            *<value>*.
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.dim1_spline_deriv_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, start.
        utils._BYREF_INT, # Input, ncap7.
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[ncap7].
        utils._EngineFloat64ArrayType.c_type, # Input, c[ncap7].
        utils._BYREF_INT, # Input, deriv.
        utils._BYREF_INT, # Input, xord.
        utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
        utils._EngineIntArrayType.c_type, # Input/Output, ixloc[nx].
        utils._BYREF_INT, # Input, nx.
        utils._EngineFloat64ArrayType.c_type, # Output, s[lds, min(abs(deriv), 3)+1].
        utils._BYREF_INT, # Input, lds.
        utils._EngineIntArrayType.c_type, # Input/Output, iwrk[liwrk].
        utils._BYREF_INT, # Input, liwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((s, 's'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _start = utils._EngineIntScalarType(
        start, {'fun_name': fun_name, 'entity': 'start'},
    ).to_ctypes()
    _ncap7 = utils._EngineIntScalarType(
        ncap7, {'fun_name': fun_name, 'entity': 'ncap7'},
    ).to_ctypes()
    _ncap7_py = utils._EngineIntScalarType.to_py(_ncap7)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ncap7_py,),
    ).to_ctypes()
    _deriv = utils._EngineIntScalarType(
        deriv, {'fun_name': fun_name, 'entity': 'deriv'},
    ).to_ctypes()
    _xord = utils._EngineIntScalarType(
        xord, {'fun_name': fun_name, 'entity': 'xord'},
    ).to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _nx_py = utils._EngineIntScalarType.to_py(_nx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_nx_py,),
    ).to_ctypes()
    _ixloc_cls = utils._EngineIntArrayType(
        ixloc, {'fun_name': fun_name, 'entity': 'ixloc'},
        exp_shape=(_nx_py,),
    )
    _ixloc = _ixloc_cls.to_ctypes()
    _lds_py = utils._EngineFloat64ArrayType.get_nd(s, {'fun_name': fun_name, 'entity': 's'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lds = utils._EngineIntScalarType(
        _lds_py, {'fun_name': fun_name, 'entity': '_lds'},
    ).to_ctypes()
    _deriv_py = utils._EngineIntScalarType.to_py(_deriv)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_nx_py, min(abs(_deriv_py), 3)+1),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _liwrk = utils._EngineIntScalarType(
        liwrk, {'fun_name': fun_name, 'entity': 'liwrk'},
    ).to_ctypes()
    _start_py = utils._EngineIntScalarType.to_py(_start)
    if _start_py == 2:
        if iwrk is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'iwrk'},
                'must not be None when start = 2',
            )
    if _start_py not in (10, 11):
        if iwrk is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'iwrk'},
                'must not be None when start not in (10, 11)',
            )
    _liwrk_py = utils._EngineIntScalarType.to_py(_liwrk)
    _iwrk_cls = utils._EngineIntArrayType(
        iwrk, {'fun_name': fun_name, 'entity': 'iwrk'},
        exp_shape=(_liwrk_py,),
        check_contents=_start_py == 2,
        allow_none=True,
    )
    _iwrk = _iwrk_cls.to_ctypes(can_reference_contents=_start_py == 2)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _start,
        _ncap7,
        _lamda,
        _c,
        _deriv,
        _xord,
        _x,
        _ixloc,
        _nx,
        _s,
        _lds,
        _iwrk,
        _liwrk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_e_consts.E02BFF_IERR_NTINSP and _ifmt == _nag_e_consts.E02BFF_IFMT_NTINSP:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of x has an enclosing interval number in',
                    'ixloc outside the set allowed by the provided spline. The spline has been',
                    'evaluated for all x with enclosing interval numbers inside the allowable',
                    'set.',
                    '{:s} entries of x were indexed below the lower bound'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} entries of x were indexed above the upper bound'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_NOVALID and _ifmt == _nag_e_consts.E02BFF_IFMT_NOVALID:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all elements of x had enclosing interval numbers in ixloc outside',
                    'the domain allowed by the provided spline.',
                    '{:s} entries of x were indexed below the lower bound'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s} entries of x were indexed above the upper bound'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_START and _ifmt == _nag_e_consts.E02BFF_IFMT_START:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, start = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: start = 0, 1, 2, 10 or 11.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_START_NX and _ifmt == _nag_e_consts.E02BFF_IFMT_START_NX:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, start = 2 and nx is not consistent with',
                    'the previous call to dim1_spline_deriv_vector.',
                    'On entry, nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_NCAP7 and _ifmt == _nag_e_consts.E02BFF_IFMT_NCAP7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncap7 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncap7 >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_LAMDA and _ifmt == _nag_e_consts.E02BFF_IFMT_LAMDA:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[3] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ncap7 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'lamda[ncap7-4] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[3] < lamda[ncap7-4].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_NX and _ifmt == _nag_e_consts.E02BFF_IFMT_NX:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.E02BFF_IERR_LIWRK and _ifmt == _nag_e_consts.E02BFF_IFMT_LIWRK:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liwrk = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liwrk >= 3*nx+3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ixloc_cls.refresh_py(_ixloc)
        _s_cls.refresh_py(_s)
        if iwrk is not None:
            _iwrk_cls.refresh_py(_iwrk)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim2_cheb_lines(m, n, k, l, x, y, f, w, mtot, a, na, xmin, xmax, nux, inuxp1, nuy, inuyp1):
    u"""
    Least squares surface fit by polynomials, data on lines parallel to
    one independent coordinate axis.

    For full information please refer to the NAG Library document for
    e02ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02caf.html

    Parameters
    ----------
    m : int, shape (`n`)
        `m`[s-1] must be set to m_s, the number of data x values on the
        line y = y_s, for s = 1,2,...,n.

    n : int
        The number of lines y = constant on which data points are given.

    k : int
        k, the required degree of x in the fit.

    l : int
        l, the required degree of y in the fit.

    x : float, shape (`mtot`)
        The x values of the data points. The sequence must be

        - all points on y = y_1, followed by

        - all points on y = y_2, followed by

        -

        - all points on y = y_n.

    y : float, shape (`n`)
        `y`[s-1] must contain the y value of line y = y_s, for s =
        1,2,...,n, on which data is given.

    f : float, shape (`mtot`)
        f, the data values of the dependent variable in the same
        sequence as the x values.

    w : float, shape (`mtot`)
        The weights to be assigned to the data points, in the same
        sequence as the x values. These weights should be calculated
        from estimates of the absolute accuracies of the f_r, expressed
        as standard deviations, probable errors or some other measure
        which is of the same dimensions as f_r. Specifically, each w_r
        should be inversely proportional to the accuracy estimate of
        f_r. Often weights all equal to unity will be satisfactory. If a
        particular weight is zero, the corresponding data point is
        omitted from the fit.

    mtot : int
        The dimension of the arrays `x`, `f` and `w`.

    a : float, shape (`na`), modified in place
        On exit: contains the Chebyshev coefficients of the fit.
        `a`[i*(`l`+1)+j-1] is the coefficient a_{ij} of [Description]
        defined according to the standard convention. These coefficients
        are used by ``dim2_cheb_eval`` to calculate values of the fitted
        function.

    na : int
        The dimension of the array `a`.

    xmin : float, shape (`n`)
        `xmin`[s-1] must contain x_min^(s), the lower end of the range
        of x on the line y = y_s, for s = 1,2,...,n. It must not be
        greater than the lowest data value of x on the line. Each
        x_min^(s) is scaled to -1.0 in the fit. (See also [Further
        Comments].)

    xmax : float, shape (`n`)
        `xmax`[s-1] must contain x_max^(s), the upper end of the range
        of x on the line y = y_s, for s = 1,2,...,n. It must not be less
        than the highest data value of x on the line. Each x_max^(s) is
        scaled to +1.0 in the fit. (See also [Further Comments].)

    nux : float, shape (`inuxp1`)
        `nux`[i-1] must contain the coefficient of the Chebyshev
        polynomial of degree (i-1) in x_, in the Chebyshev series
        representation of the polynomial factor in x_ which you require
        the fit to contain, for i = 1,2,...,`inuxp1`. These coefficients
        are defined according to the standard convention of
        [Description].

    inuxp1 : int
        INUX+1, where INUX is the degree of a polynomial factor in x_
        which you require the fit to contain. (See [Description], last
        paragraph.)

        If this option is not required, `inuxp1` should be set equal to
        1.

    nuy : float, shape (`inuyp1`)
        `nuy`[i-1] must contain the coefficient of the Chebyshev
        polynomial of degree (i-1) in y_, in the Chebyshev series
        representation of the polynomial factor which you require the
        fit to contain, for i = 1,2,...,`inuyp1`. These coefficients are
        defined according to the standard convention of [Description].

    inuyp1 : int
        INUY+1, where INUY is the degree of a polynomial factor in y_
        which you require the fit to contain. (See [Description], last
        paragraph.) If this option is not required, `inuyp1` should be
        set equal to 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `na` is too small. `na` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, `mtot` is too small. `mtot` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, I = *<value>*, `m`[I-1] = *<value>*, `k` =
            *<value>* and `inuxp1` = *<value>*.

            Constraint: `m`[I-1] >= `k`-`inuxp1`+2.

        (`errno` 1)
            On entry, `n` = *<value>*, `l` = *<value>* and `inuyp1` =
            *<value>*.

            Constraint: `n` >= `l`-`inuyp1`+2.

        (`errno` 1)
            On entry, `inuyp1` = *<value>* and `l` = *<value>*.

            Constraint: `inuyp1` <= `l`+1.

        (`errno` 1)
            On entry, `inuxp1` = *<value>* and `k` = *<value>*.

            Constraint: `inuxp1` <= `k`+1.

        (`errno` 1)
            On entry, `inuyp1` = *<value>*.

            Constraint: `inuyp1` >= 1.

        (`errno` 1)
            On entry, `inuxp1` = *<value>*.

            Constraint: `inuxp1` >= 1.

        (`errno` 1)
            On entry, `l` = *<value>*.

            Constraint: `l` >= 0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            On entry, `xmin`[I-1] and `xmax`[I-1] do not span the data
            `x` values on `y` = `y`[I-1]: I = *<value>*, `xmin`[I-1] =
            *<value>*, `xmax`[I-1] = *<value>* and `y`[I-1] = *<value>*.

        (`errno` 3)
            On entry, the data `x` values are not nondecreasing for `y`
            = `y`[I-1]: I = *<value>* and `y`[I-1] = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `y`[I-1] = *<value>* and `y`[I-2] =
            *<value>*.

            Constraint: `y`[I-1] > `y`[I-2].

        (`errno` 4)
            On entry, the number of distinct `x` values with nonzero
            weight on `y` = `y`[I-1] is less than `k`-`inuxp1`+2: I =
            *<value>*, `y`[I-1] = *<value>*, `k` = *<value>* and
            `inuxp1` = *<value>*.

        (`errno` 5)
            On entry, `inuxp1` = *<value>*, `nux`[`inuxp1`-1] =
            *<value>*, `inuyp1` = *<value>* and `nuy`[`inuyp1`-1] =
            *<value>*.

            Constraint: if `nux`[`inuxp1`-1] = 0.0, `inuxp1` = 1; if
            `nuy`[`inuyp1`-1] = 0.0, `inuyp1` = 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_cheb_lines'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._EngineIntArrayType.c_type, # Input, m[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineFloat64ArrayType.c_type, # Input, x[mtot].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Input, f[mtot].
        utils._EngineFloat64ArrayType.c_type, # Input, w[mtot].
        utils._BYREF_INT, # Input, mtot.
        utils._EngineFloat64ArrayType.c_type, # Output, a[na].
        utils._BYREF_INT, # Input, na.
        utils._EngineFloat64ArrayType.c_type, # Input, xmin[n].
        utils._EngineFloat64ArrayType.c_type, # Input, xmax[n].
        utils._EngineFloat64ArrayType.c_type, # Input, nux[inuxp1].
        utils._BYREF_INT, # Input, inuxp1.
        utils._EngineFloat64ArrayType.c_type, # Input, nuy[inuyp1].
        utils._BYREF_INT, # Input, inuyp1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, nwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_cls = utils._EngineIntArrayType(
        m, {'fun_name': fun_name, 'entity': 'm'},
        exp_shape=(_n_py,),
    )
    _m = _m_cls.to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _mtot = utils._EngineIntScalarType(
        mtot, {'fun_name': fun_name, 'entity': 'mtot'},
    ).to_ctypes()
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_cls.get_sum(0, _n_py-1),),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_cls.get_sum(0, _n_py-1),),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_cls.get_sum(0, _n_py-1),),
    ).to_ctypes()
    _na = utils._EngineIntScalarType(
        na, {'fun_name': fun_name, 'entity': 'na'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _l_py = utils._EngineIntScalarType.to_py(_l)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=((_k_py+1)*(_l_py+1),),
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _xmin = utils._EngineFloat64ArrayType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _xmax = utils._EngineFloat64ArrayType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _inuxp1 = utils._EngineIntScalarType(
        inuxp1, {'fun_name': fun_name, 'entity': 'inuxp1'},
    ).to_ctypes()
    _inuxp1_py = utils._EngineIntScalarType.to_py(_inuxp1)
    _nux = utils._EngineFloat64ArrayType(
        nux, {'fun_name': fun_name, 'entity': 'nux'},
        exp_shape=(_inuxp1_py,),
    ).to_ctypes()
    _inuyp1 = utils._EngineIntScalarType(
        inuyp1, {'fun_name': fun_name, 'entity': 'inuyp1'},
    ).to_ctypes()
    _inuyp1_py = utils._EngineIntScalarType.to_py(_inuyp1)
    _nuy = utils._EngineFloat64ArrayType(
        nuy, {'fun_name': fun_name, 'entity': 'nuy'},
        exp_shape=(_inuyp1_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _nwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _m,
        _n,
        _k,
        _l,
        _x,
        _y,
        _f,
        _w,
        _mtot,
        _a,
        _na,
        _xmin,
        _xmax,
        _nux,
        _inuxp1,
        _nuy,
        _inuyp1,
        _work,
        _nwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, na is too small.',
                    'na = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mtot is too small.',
                    'mtot = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, m[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'k = {:s} and inuxp1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: m[I-1] >= k-inuxp1+2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, l = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'inuyp1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: n >= l-inuyp1+2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inuyp1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: inuyp1 <= l+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inuxp1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: inuxp1 <= k+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inuyp1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inuyp1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inuxp1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inuxp1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmin[I-1] and xmax[I-1] do not span the data x',
                    'values on y = y[I-1]:',
                    'I = {:s}, xmin[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmax[I-1] = {:s} and y[I-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the data x values are not nondecreasing for',
                    'y = y[I-1]:',
                    'I = {:s} and y[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, y[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'y[I-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: y[I-1] > y[I-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of distinct x values with nonzero weight',
                    'on y = y[I-1] is less than k-inuxp1+2:',
                    'I = {:s}, y[I-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'k = {:s} and inuxp1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inuxp1 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nux[inuxp1-1] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'inuyp1 = {:s} and nuy[inuyp1-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: if nux[inuxp1-1] = 0.0, inuxp1 = 1;',
                    'if nuy[inuyp1-1] = 0.0, inuyp1 = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_cheb_eval(mfirst, mlast, k, l, x, xmin, xmax, y, ymin, ymax, ff, a, na):
    u"""
    Evaluation of fitted polynomial in two variables.

    For full information please refer to the NAG Library document for
    e02cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02cbf.html

    Parameters
    ----------
    mfirst : int
        The index of the first and last x value in the array x at which
        the evaluation is required respectively (see [Further
        Comments]).

    mlast : int
        The index of the first and last x value in the array x at which
        the evaluation is required respectively (see [Further
        Comments]).

    k : int
        The degree k of x and l of y, respectively, in the polynomial.

    l : int
        The degree k of x and l of y, respectively, in the polynomial.

    x : float, shape (`mlast`)
        `x`[i-1], for i = `mfirst`,...,`mlast`, must contain the x
        values at which the evaluation is required.

    xmin : float
        The lower and upper ends, x_min and x_max, of the range of the
        variable x (see [Description]).

        The values of `xmin` and `xmax` may depend on the value of y
        (e.g., when the polynomial has been derived using
        ``dim2_cheb_lines``).

    xmax : float
        The lower and upper ends, x_min and x_max, of the range of the
        variable x (see [Description]).

        The values of `xmin` and `xmax` may depend on the value of y
        (e.g., when the polynomial has been derived using
        ``dim2_cheb_lines``).

    y : float
        The value of the y coordinate of all the points at which the
        evaluation is required.

    ymin : float
        The lower and upper ends, y_min and y_max, of the range of the
        variable y (see [Description]).

    ymax : float
        The lower and upper ends, y_min and y_max, of the range of the
        variable y (see [Description]).

    ff : float, shape (`mlast`), modified in place
        On exit: `ff`[i-1] gives the value of the polynomial at the
        point (x_i,y), for i = `mfirst`,...,`mlast`.

    a : float, shape (`na`)
        The Chebyshev coefficients of the polynomial. The coefficient
        a_{ij} defined according to the standard convention (see
        [Description]) must be in `a`[i*(l+1)+j].

    na : int
        The dimension of the array `a`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `na` = *<value>*, `k` = *<value>* and `l` =
            *<value>*.

            Constraint: `na` >= (`k`+1)*(`l`+1).

        (`errno` 1)
            On entry, `l` = *<value>*.

            Constraint: `l` >= 0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 1)
            On entry, `mfirst` = *<value>* and `mlast` = *<value>*.

            Constraint: `mfirst` <= `mlast`.

        (`errno` 2)
            On entry, `y` = *<value>* and `ymax` = *<value>*.

            Constraint: `y` <= `ymax`.

        (`errno` 2)
            On entry, `y` = *<value>* and `ymin` = *<value>*.

            Constraint: `y` >= `ymin`.

        (`errno` 2)
            On entry, `ymin` = *<value>* and `ymax` = *<value>*.

            Constraint: `ymin` < `ymax`.

        (`errno` 3)
            On entry, I = *<value>*, `x`[I-1] = *<value>* and `xmax` =
            *<value>*.

            Constraint: `x`[I-1] <= `xmax`.

        (`errno` 3)
            On entry, I = *<value>*, `x`[I-1] = *<value>* and `xmin` =
            *<value>*.

            Constraint: `x`[I-1] >= `xmin`.

        (`errno` 3)
            On entry, `xmin` = *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` < `xmax`.

        (`errno` 4)
            Unexpected internal failure when evaluating the polynomial.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_cheb_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, mfirst.
        utils._BYREF_INT, # Input, mlast.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineFloat64ArrayType.c_type, # Input, x[mlast].
        utils._BYREF_FLOAT64, # Input, xmin.
        utils._BYREF_FLOAT64, # Input, xmax.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Input, ymin.
        utils._BYREF_FLOAT64, # Input, ymax.
        utils._EngineFloat64ArrayType.c_type, # Output, ff[mlast].
        utils._EngineFloat64ArrayType.c_type, # Input, a[na].
        utils._BYREF_INT, # Input, na.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, nwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _mfirst = utils._EngineIntScalarType(
        mfirst, {'fun_name': fun_name, 'entity': 'mfirst'},
    ).to_ctypes()
    _mlast = utils._EngineIntScalarType(
        mlast, {'fun_name': fun_name, 'entity': 'mlast'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _mlast_py = utils._EngineIntScalarType.to_py(_mlast)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mlast_py,),
    ).to_ctypes()
    _xmin = utils._EngineFloat64ScalarType(
        xmin, {'fun_name': fun_name, 'entity': 'xmin'},
    ).to_ctypes()
    _xmax = utils._EngineFloat64ScalarType(
        xmax, {'fun_name': fun_name, 'entity': 'xmax'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _ymin = utils._EngineFloat64ScalarType(
        ymin, {'fun_name': fun_name, 'entity': 'ymin'},
    ).to_ctypes()
    _ymax = utils._EngineFloat64ScalarType(
        ymax, {'fun_name': fun_name, 'entity': 'ymax'},
    ).to_ctypes()
    _ff_cls = utils._EngineFloat64ArrayType(
        ff, {'fun_name': fun_name, 'entity': 'ff'},
        exp_shape=(_mlast_py,),
        check_contents=False,
    )
    _ff = _ff_cls.to_ctypes(can_reference_contents=False)
    _na = utils._EngineIntScalarType(
        na, {'fun_name': fun_name, 'entity': 'na'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _l_py = utils._EngineIntScalarType.to_py(_l)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=((_k_py+1)*(_l_py+1),),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _nwork = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _mfirst,
        _mlast,
        _k,
        _l,
        _x,
        _xmin,
        _xmax,
        _y,
        _ymin,
        _ymax,
        _ff,
        _a,
        _na,
        _work,
        _nwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, na = {:s}, k = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: na >= (k+1)*(l+1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mfirst = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mlast = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mfirst <= mlast.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: y <= ymax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, y = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ymin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: y >= ymin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ymin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ymin < ymax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, x[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[I-1] <= xmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, x[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'xmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[I-1] >= xmin.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xmin = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'xmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: xmin < xmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal failure when evaluating the polynomial.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ff_cls.refresh_py(_ff)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_spline_panel(m, px, py, x, y, f, w, lamda, mu, point, npoint, dl, c, nc, eps):
    u"""
    Least squares surface fit, bicubic splines.

    For full information please refer to the NAG Library document for
    e02da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02daf.html

    Parameters
    ----------
    m : int
        m, the number of data points.

    px : int
        The total number of knots lambda and mu associated with the
        variables x and y, respectively.

    py : int
        The total number of knots lambda and mu associated with the
        variables x and y, respectively.

    x : float, shape (`m`)
        The coordinates of the data point (x_r,y_r,f_r), for r =
        1,2,...,m. The order of the data points is immaterial, but see
        the array `point`.

    y : float, shape (`m`)
        The coordinates of the data point (x_r,y_r,f_r), for r =
        1,2,...,m. The order of the data points is immaterial, but see
        the array `point`.

    f : float, shape (`m`)
        The coordinates of the data point (x_r,y_r,f_r), for r =
        1,2,...,m. The order of the data points is immaterial, but see
        the array `point`.

    w : float, shape (`m`)
        The weight w_r of the rth data point. It is important to note
        the definition of weight implied by the equation (1) in
        [Description], since it is also common usage to define weight as
        the square of this weight. In this function, each w_r should be
        chosen inversely proportional to the (absolute) accuracy of the
        corresponding f_r, as expressed, for example, by the standard
        deviation or probable error of the f_r. When the f_r are all of
        the same accuracy, all the w_r may be set equal to 1.0.

    lamda : float, shape (`px`), modified in place
        On entry: `lamda`[i+3] must contain the ith interior knot
        lambda_{i+4} associated with the variable x, for i =
        1,2,...,`px`-8. The knots must be in nondecreasing order and lie
        strictly within the range covered by the data values of x. A
        knot is a value of x at which the spline is allowed to be
        discontinuous in the third derivative with respect to x, though
        continuous up to the second derivative. This degree of
        continuity can be reduced, if you require, by the use of
        coincident knots, provided that no more than four knots are
        chosen to coincide at any point. Two, or three, coincident knots
        allow loss of continuity in, respectively, the second and first
        derivative with respect to x at the value of x at which they
        coincide. Four coincident knots split the spline surface into
        two independent parts. For choice of knots see [Further
        Comments].

        On exit: the interior knots `lamda`[4] to `lamda`[`px`-5] are
        unchanged, and the segments `lamda`[0:4] and
        `lamda`[`px`-4:`px`] contain additional (exterior) knots
        introduced by the function in order to define the full set of
        B-splines required. The four knots in the first segment are all
        set equal to the lowest data value of x and the other four
        additional knots are all set equal to the highest value: there
        is experimental evidence that coincident end-knots are best for
        numerical accuracy. The complete array must be left undisturbed
        if ``dim2_spline_evalv`` or ``dim2_spline_evalm`` is to be used
        subsequently.

    mu : float, shape (`py`), modified in place
        On entry: `mu`[i+3] must contain the ith interior knot mu_{i+4}
        associated with the variable y, for i = 1,2,...,`py`-8.

        On exit: the same remarks apply to `mu` as to `lamda` above,
        with `y` replacing `x`, and y replacing x.

    point : int, shape (`npoint`)
        Indexing information usually provided by ``dim2_spline_sort``
        which enables the data points to be accessed in the order which
        produces the advantageous matrix structure mentioned in
        [Description]. This order is such that, if the (x,y) plane is
        thought of as being divided into rectangular panels by the two
        sets of knots, all data in a panel occur before data in
        succeeding panels, where the panels are numbered from bottom to
        top and then left to right with the usual arrangement of axes,
        as indicated in Figure. [figure omitted]

        A data point lying exactly on one or more panel sides is
        considered to be in the highest numbered panel adjacent to the
        point. ``dim2_spline_sort`` should be called to obtain the array
        `point`, unless it is provided by other means.

    npoint : int
        The dimension of the array `point`.

    dl : float, shape (`nc`), modified in place
        On exit: gives the squares of the diagonal elements of the
        reduced triangular matrix, divided by the mean squared weight.
        It includes those elements, less than epsilon, which are treated
        as zero (see [Description]).

    c : float, shape (`nc`), modified in place
        On exit: gives the coefficients of the fit. `c`[ (`py`-4)* (i-1)
        +j -1] is the coefficient c_{ij} of [Description], for j =
        1,2,...,`py`-4, for i = 1,2,...,`px`-4. These coefficients are
        used by ``dim2_spline_evalv`` or ``dim2_spline_evalm`` to
        calculate values of the fitted function.

    nc : int
        The value (`px`-4)*(`py`-4).

    eps : float
        A threshold epsilon for determining the effective rank of the
        system of linear equations. The rank is determined as the number
        of elements of the array `dl` which are nonzero. An element of
        `dl` is regarded as zero if it is less than epsilon. Machine
        precision is a suitable value for epsilon in most practical
        applications which have only 2 or 3 decimals accurate in data.
        If some coefficients of the fit prove to be very large compared
        with the data ordinates, this suggests that epsilon should be
        increased so as to decrease the rank. The array `dl` will give a
        guide to appropriate values of epsilon to achieve this, as well
        as to the choice of epsilon in other cases where some
        experimentation may be needed to determine a value which leads
        to a satisfactory fit.

    Returns
    -------
    sigma : float
        Sigma, the weighted sum of squares of residuals. This is not
        computed from the individual residuals but from the right-hand
        sides of the orthogonally-transformed linear equations. For
        further details see page 97 of Hayes and Halliday. The two
        methods of computation are theoretically equivalent, but the
        results may differ because of rounding error.

    rank : int
        The rank of the system as determined by the value of the
        threshold epsilon.

        `rank` = `nc`

          The least squares solution is unique.

        `rank` != `nc`

          The minimal least squares solution is computed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            At least one set of knots is not in nondecreasing order.

        (`errno` 2)
            More than four knots coincide at a single point.

        (`errno` 3)
            Array `point` does not indicate the data points in panel
            order.

        (`errno` 4)
            On entry, `npoint` = *<value>*, `m` = *<value>*, `px` =
            *<value>* and `py` = *<value>*.

            Constraint: `npoint` >= `m`+(`px`-7)*(`py`-7).

        (`errno` 4)
            On entry, `nc` = *<value>*, `px` = *<value>* and `py` =
            *<value>*.

            Constraint: `nc` = (`px`-4)*(`py`-4).

        (`errno` 4)
            On entry, `py` = *<value>*.

            Constraint: `py` >= 8.

        (`errno` 4)
            On entry, `px` = *<value>*.

            Constraint: `px` >= 8.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` > 1.

        (`errno` 5)
            All the weights are zero, or rank determined as zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_panel'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, px.
        utils._BYREF_INT, # Input, py.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, f[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, lamda[px].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mu[py].
        utils._EngineIntArrayType.c_type, # Input, point[npoint].
        utils._BYREF_INT, # Input, npoint.
        utils._EngineFloat64ArrayType.c_type, # Output, dl[nc].
        utils._EngineFloat64ArrayType.c_type, # Output, c[nc].
        utils._BYREF_INT, # Input, nc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, ws.
        utils._BYREF_INT, # Input, nws.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Output, sigma.
        utils._BYREF_INT, # Output, rank.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _px = utils._EngineIntScalarType(
        px, {'fun_name': fun_name, 'entity': 'px'},
    ).to_ctypes()
    _py = utils._EngineIntScalarType(
        py, {'fun_name': fun_name, 'entity': 'py'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _px_py = utils._EngineIntScalarType.to_py(_px)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_px_py,),
    )
    _lamda = _lamda_cls.to_ctypes()
    _py_py = utils._EngineIntScalarType.to_py(_py)
    _mu_cls = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_py_py,),
    )
    _mu = _mu_cls.to_ctypes()
    _npoint = utils._EngineIntScalarType(
        npoint, {'fun_name': fun_name, 'entity': 'npoint'},
    ).to_ctypes()
    _npoint_py = utils._EngineIntScalarType.to_py(_npoint)
    _point = utils._EngineIntArrayType(
        point, {'fun_name': fun_name, 'entity': 'point'},
        exp_shape=(_npoint_py,),
    ).to_ctypes()
    _nc = utils._EngineIntScalarType(
        nc, {'fun_name': fun_name, 'entity': 'nc'},
    ).to_ctypes()
    _dl_cls = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=((_px_py-4)*(_py_py-4),),
        check_contents=False,
    )
    _dl = _dl_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_px_py-4)*(_py_py-4),),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _ws = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _nws = utils._EngineIntScalarType.c_type(0)
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType.empty_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _m,
        _px,
        _py,
        _x,
        _y,
        _f,
        _w,
        _lamda,
        _mu,
        _point,
        _npoint,
        _dl,
        _c,
        _nc,
        _ws,
        _nws,
        _eps,
        _sigma,
        _rank,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one set of knots is not in nondecreasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than four knots coincide at a single point.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Array point does not indicate the data points in panel order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoint = {:s}, m = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'px = {:s} and py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: npoint >= m+(px-7)*(py-7).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nc = {:s}, px = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nc = (px-4)*(py-4).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: py >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, px = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: px >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All the weights are zero, or rank determined as zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lamda_cls.refresh_py(_lamda)
        _mu_cls.refresh_py(_mu)
        _dl_cls.refresh_py(_dl)
        _c_cls.refresh_py(_c)
        _sigma_py = utils._EngineFloat64ScalarType.to_py(_sigma)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['sigma', 'rank'],
        )(
            _sigma_py,
            _rank_py,
        )
    _err_class.manage()
    return return_data

def dim2_spline_grid(start, mx, x, my, y, f, s, nxest, nyest, nx, lamda, ny, mu, c, comm):
    u"""
    Least squares surface fit by bicubic splines with automatic knot
    placement, data on rectangular grid.

    For full information please refer to the NAG Library document for
    e02dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02dcf.html

    Parameters
    ----------
    start : str, length 1
        Determines whether calculations are to be performed afresh (Cold
        Start) or whether knots found in previous calls are to be used
        as an initial estimate of knot placement (Warm Start).

        `start` = 'C'

          The function will build up the knot set starting with no
          interior knots. No values need be assigned to the arguments
          `nx`, `ny`, `lamda`, `mu`, `comm`['wrk'] or `comm`['iwrk'].

        `start` = 'W'

          The function will restart the knot-placing strategy using the
          knots found in a previous call of the function. In this case,
          the arguments `nx`, `ny`, `lamda`, `mu`, `comm`['wrk'] and
          `comm`['iwrk'] must be unchanged from that previous call. This
          warm start can save much time in determining a satisfactory
          set of knots for the given value of `s`. This is particularly
          useful when different smoothing factors are used for the same
          dataset.

    mx : int
        m_x, the number of grid points along the x axis.

    x : float, shape (`mx`)
        `x`[q-1] must be set to x_q, the x coordinate of the qth grid
        point along the x axis, for q = 1,2,...,m_x.

    my : int
        m_y, the number of grid points along the y axis.

    y : float, shape (`my`)
        `y`[r-1] must be set to y_r, the y coordinate of the rth grid
        point along the y axis, for r = 1,2,...,m_y.

    f : float, shape (`mx`*`my`)
        `f`[m_y*(q-1)+r-1] must contain the data value f_{q,r}, for r =
        1,2,...,m_y, for q = 1,2,...,m_x.

    s : float
        The smoothing factor, S.

        If `s` = 0.0, the function returns an interpolating spline.

        If `s` is smaller than machine precision, it is assumed equal to
        zero.

        For advice on the choice of `s`, see [Description].

    nxest : int
        An upper bound for the number of knots n_x and n_y required in
        the x- and y-directions respectively.

        In most practical situations, `nxest` = m_x/2 and `nyest` =
        m_y/2 is sufficient. `nxest` and `nyest` never need to be larger
        than m_x+4 and m_y+4 respectively, the numbers of knots needed
        for interpolation (`s` = 0.0).

        See also [Further Comments].

    nyest : int
        An upper bound for the number of knots n_x and n_y required in
        the x- and y-directions respectively.

        In most practical situations, `nxest` = m_x/2 and `nyest` =
        m_y/2 is sufficient. `nxest` and `nyest` never need to be larger
        than m_x+4 and m_y+4 respectively, the numbers of knots needed
        for interpolation (`s` = 0.0).

        See also [Further Comments].

    nx : int
        If the warm start option is used, the value of `nx` must be left
        unchanged from the previous call.

    lamda : float, shape (`nxest`), modified in place
        On entry: if the warm start option is used, the values
        `lamda`[0],`lamda`[1],...,`lamda`[`nx`-1] must be left unchanged
        from the previous call.

        On exit: contains the complete set of knots lambda_i associated
        with the x variable, i.e., the interior knots
        `lamda`[4],`lamda`[5],...,`lamda`[`nx`-5] as well as the
        additional knots

            `lamda`[0] = `lamda`[1] = `lamda`[2] = `lamda`[3] = `x`[0]

        and

            `lamda`[`nx`-4] = `lamda`[`nx`-3] = `lamda`[`nx`-2] =
            `lamda`[`nx`-1] = `x`[`mx`-1]

        needed for the B-spline representation.

    ny : int
        If the warm start option is used, the value of `ny` must be left
        unchanged from the previous call.

    mu : float, shape (`nyest`), modified in place
        On entry: if the warm start option is used, the values
        `mu`[0],`mu`[1],...,`mu`[`ny`-1] must be left unchanged from the
        previous call.

        On exit: contains the complete set of knots mu_i associated with
        the y variable, i.e., the interior knots
        `mu`[4],`mu`[5],...,`mu`[`ny`-5] as well as the additional knots

            `mu`[0] = `mu`[1] = `mu`[2] = `mu`[3] = `y`[0]

        and

            `mu`[`ny`-4] = `mu`[`ny`-3] = `mu`[`ny`-2] = `mu`[`ny`-1] =
            `y`[`my`-1]

        needed for the B-spline representation.

    c : float, shape ((`nxest`-4)*(`nyest`-4)), modified in place
        On exit: the coefficients of the spline approximation.
        `c`[(n_y-4) * (i-1) +j -1] is the coefficient c_{ij} defined in
        [Description].

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    nx : int
        The total number of knots, n_x, of the computed spline with
        respect to the x variable.

    ny : int
        The total number of knots, n_y, of the computed spline with
        respect to the y variable.

    fp : float
        The sum of squared residuals, theta, of the computed spline
        approximation. If `fp` = 0.0, this is an interpolating spline.
        `fp` should equal `s` within a relative tolerance of 0.001
        unless `nx` = `ny` = 8, when the spline has no interior knots
        and so is simply a bicubic polynomial. For knots to be inserted,
        `s` must be set to a value below the value of `fp` produced in
        this case.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nxest` = *<value>*, `mx` = *<value>*, `nyest` =
            *<value>* and `my` = *<value>*.

            Constraint: if `s` = 0.0, `nxest` >= `mx`+4 and `nyest` >=
            `my`+4.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` >= 0.0.

        (`errno` 1)
            On entry, `nyest` = *<value>*.

            Constraint: `nyest` >= *<value>*.

        (`errno` 1)
            On entry, `my` = *<value>*.

            Constraint: `my` >= *<value>*.

        (`errno` 1)
            On entry, `nxest` = *<value>*.

            Constraint: `nxest` >= *<value>*.

        (`errno` 1)
            On entry, `mx` = *<value>*.

            Constraint: `mx` >= *<value>*.

        (`errno` 1)
            On entry, `start` != 'W' or 'C': `start` = *<value>*.

        (`errno` 2)
            On entry, q = *<value>*, `x`[q-2] = *<value>* and `x`[q-1] =
            *<value>*.

            Constraint: `x`[q-2] < `x`[q-1] for all q.

        (`errno` 3)
            On entry, r = *<value>*, `y`[r-2] = *<value>* and `y`[r-1] =
            *<value>*.

            Constraint: `y`[r-2] < `y`[r-1] for all r.

        (`errno` 4)
            The number of knots needed in one direction is greater than
            `nxest` or `nyest`: `nxest`, `nyest` = *<value>*,*<value>*.
            Possibly `s` is too small: `s` = *<value>*.

        (`errno` 5)
            The iterative process has failed to converge. Possibly `s`
            is too small: `s` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_grid'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, start(len=1).
        utils._BYREF_INT, # Input, mx.
        utils._EngineFloat64ArrayType.c_type, # Input, x[mx].
        utils._BYREF_INT, # Input, my.
        utils._EngineFloat64ArrayType.c_type, # Input, y[my].
        utils._EngineFloat64ArrayType.c_type, # Input, f[mx*my].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_INT, # Input, nxest.
        utils._BYREF_INT, # Input, nyest.
        utils._BYREF_INT, # Input/Output, nx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, lamda[nxest].
        utils._BYREF_INT, # Input/Output, ny.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mu[nyest].
        utils._EngineFloat64ArrayType.c_type, # Output, c[(nxest-4)*(nyest-4)].
        utils._BYREF_FLOAT64, # Output, fp.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrk[lwrk].
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Input/Output, iwrk[liwrk].
        utils._BYREF_INT, # Input, liwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of start.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _start = utils._EngineCharScalarType(
        start, {'fun_name': fun_name, 'entity': 'start'}, exp_chrlen=1,
    ).to_ctypes()
    _mx = utils._EngineIntScalarType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
    ).to_ctypes()
    _mx_py = utils._EngineIntScalarType.to_py(_mx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mx_py,),
    ).to_ctypes()
    _my = utils._EngineIntScalarType(
        my, {'fun_name': fun_name, 'entity': 'my'},
    ).to_ctypes()
    _my_py = utils._EngineIntScalarType.to_py(_my)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_my_py,),
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_mx_py*_my_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _nxest = utils._EngineIntScalarType(
        nxest, {'fun_name': fun_name, 'entity': 'nxest'},
    ).to_ctypes()
    _nyest = utils._EngineIntScalarType(
        nyest, {'fun_name': fun_name, 'entity': 'nyest'},
    ).to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _nxest_py = utils._EngineIntScalarType.to_py(_nxest)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_nxest_py,),
    )
    _lamda = _lamda_cls.to_ctypes()
    _ny = utils._EngineIntScalarType(
        ny, {'fun_name': fun_name, 'entity': 'ny'},
    ).to_ctypes()
    _nyest_py = utils._EngineIntScalarType.to_py(_nyest)
    _mu_cls = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_nyest_py,),
    )
    _mu = _mu_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_nxest_py-4)*(_nyest_py-4),),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _fp = utils._EngineFloat64ScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wrk', 'iwrk'),
    )
    if _comm_needs_init:
        _wrk = utils._EngineFloat64ArrayType.empty_ctypes(4*(_mx_py+_my_py)+11*(_nxest_py+_nyest_py)+_nxest_py*_my_py+max(_my_py, _nxest_py)+54)
        comm['wrk'] = _wrk
    else:
        _wrk = comm['wrk']
    _lwrk_py = utils._EngineFloat64ArrayType.get_shape(comm['wrk'], {'fun_name': fun_name, 'entity': 'comm[\'wrk\']'}, exp_rank=1, validate=False)[0]
    _lwrk = utils._EngineIntScalarType(
        _lwrk_py, {'fun_name': fun_name, 'entity': '_lwrk'},
    ).to_ctypes()
    if _comm_needs_init:
        _iwrk = utils._EngineIntArrayType.empty_ctypes(3+_mx_py+_my_py+_nxest_py+_nyest_py)
        comm['iwrk'] = _iwrk
    else:
        _iwrk = comm['iwrk']
    _liwrk_py = utils._EngineIntArrayType.get_shape(comm['iwrk'], {'fun_name': fun_name, 'entity': 'comm[\'iwrk\']'}, exp_rank=1, validate=False)[0]
    _liwrk = utils._EngineIntScalarType(
        _liwrk_py, {'fun_name': fun_name, 'entity': '_liwrk'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _start,
        _mx,
        _x,
        _my,
        _y,
        _f,
        _s,
        _nxest,
        _nyest,
        _nx,
        _lamda,
        _ny,
        _mu,
        _c,
        _fp,
        _wrk,
        _lwrk,
        _iwrk,
        _liwrk,
        _errbuf,
        _ierr,
        _start._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxest = {:s}, mx = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'nyest = {:s} and my = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: if s = 0.0, nxest >= mx+4 and nyest >= my+4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nyest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nyest >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, my = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: my >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nxest >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mx >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, start != \'W\' or \'C\': start = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}, x[q-2] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and x[q-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[q-2] < x[q-1] for all q.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r = {:s}, y[r-2] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and y[r-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: y[r-2] < y[r-1] for all r.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of knots needed in one direction is greater than nxest or nyest:',
                    'nxest, nyest = {:s},{:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative process has failed to converge.',
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nx_py = utils._EngineIntScalarType.to_py(_nx)
        _lamda_cls.refresh_py(_lamda)
        _ny_py = utils._EngineIntScalarType.to_py(_ny)
        _mu_cls.refresh_py(_mu)
        _c_cls.refresh_py(_c)
        _fp_py = utils._EngineFloat64ScalarType.to_py(_fp)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nx', 'ny', 'fp'],
        )(
            _nx_py,
            _ny_py,
            _fp_py,
        )
    _err_class.manage()
    return return_data

def dim2_spline_sctr(start, m, x, y, f, w, s, nxest, nyest, nx, lamda, ny, mu, c, comm):
    u"""
    Least squares surface fit by bicubic splines with automatic knot
    placement, scattered data.

    For full information please refer to the NAG Library document for
    e02dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02ddf.html

    Parameters
    ----------
    start : str, length 1
        Determines whether calculations are to be performed afresh (Cold
        Start) or whether knots found in previous calls are to be used
        as an initial estimate of knot placement (Warm Start).

        `start` = 'C'

          The function will build up the knot set starting with no
          interior knots. No values need be assigned to the arguments
          `nx`, `ny`, `lamda`, `mu` or `comm`['wrk'].

        `start` = 'W'

          The function will restart the knot-placing strategy using the
          knots found in a previous call of the function. In this case,
          the arguments `nx`, `ny`, `lamda`, `mu` and `comm`['wrk'] must
          be unchanged from that previous call. This warm start can save
          much time in determining a satisfactory set of knots for the
          given value of `s`. This is particularly useful when different
          smoothing factors are used for the same dataset.

    m : int
        m, the number of data points.

        The number of data points with nonzero weight (see `w`) must be
        at least 16.

    x : float, shape (`m`), modified in place
        `x`[r-1], `y`[r-1], `f`[r-1] must be set to the coordinates of
        (x_r,y_r,f_r), the rth data point, for r = 1,2,...,m. The order
        of the data points is immaterial.

    y : float, shape (`m`), modified in place
        `x`[r-1], `y`[r-1], `f`[r-1] must be set to the coordinates of
        (x_r,y_r,f_r), the rth data point, for r = 1,2,...,m. The order
        of the data points is immaterial.

    f : float, shape (`m`)
        `x`[r-1], `y`[r-1], `f`[r-1] must be set to the coordinates of
        (x_r,y_r,f_r), the rth data point, for r = 1,2,...,m. The order
        of the data points is immaterial.

    w : float, shape (`m`)
        `w`[r-1] must be set to w_r, the rth value in the set of
        weights, for r = 1,2,...,m. Zero weights are permitted and the
        corresponding points are ignored, except when determining x_min,
        x_max, y_min and y_max (see [Further Comments]). For advice on
        the choice of weights, see [the E02 Introduction].

    s : float
        The smoothing factor, `s`.

        For advice on the choice of `s`, see [Description].

    nxest : int
        An upper bound for the number of knots n_x and n_y required in
        the x- and y-directions respectively.

        In most practical situations, `nxest` = `nyest` = 4+sqrt(m/2) is
        sufficient.

        See also [Further Comments].

    nyest : int
        An upper bound for the number of knots n_x and n_y required in
        the x- and y-directions respectively.

        In most practical situations, `nxest` = `nyest` = 4+sqrt(m/2) is
        sufficient.

        See also [Further Comments].

    nx : int
        If the warm start option is used, the value of `nx` must be left
        unchanged from the previous call.

    lamda : float, shape (`nxest`), modified in place
        On entry: if the warm start option is used, the values
        `lamda`[0],`lamda`[1],...,`lamda`[`nx`-1] must be left unchanged
        from the previous call.

        On exit: contains the complete set of knots lambda_i associated
        with the x variable, i.e., the interior knots
        `lamda`[4],`lamda`[5],...,`lamda`[`nx`-5] as well as the
        additional knots

            `lamda`[0] = `lamda`[1] = `lamda`[2] = `lamda`[3] = x_min

        and

            `lamda`[`nx`-4] = `lamda`[`nx`-3] = `lamda`[`nx`-2] =
            `lamda`[`nx`-1] = x_max

        needed for the B-spline representation (where x_min and x_max
        are as described in [Description]).

    ny : int
        If the warm start option is used, the value of `ny` must be left
        unchanged from the previous call.

    mu : float, shape (`nyest`), modified in place
        On entry: if the warm start option is used, the values
        `mu`[0],`mu`[1],...,`mu`[`ny`-1] must be left unchanged from the
        previous call.

        On exit: contains the complete set of knots mu_i associated with
        the y variable, i.e., the interior knots
        `mu`[4],`mu`[5],...,`mu`[`ny`-5] as well as the additional knots

            `mu`[0] = `mu`[1] = `mu`[2] = `mu`[3] = y_min

        and

            `mu`[`ny`-4] = `mu`[`ny`-3] = `mu`[`ny`-2] = `mu`[`ny`-1] =
            y_max

        needed for the B-spline representation (where y_min and y_max
        are as described in [Description]).

    c : float, shape ((`nxest`-4)*(`nyest`-4)), modified in place
        On exit: the coefficients of the spline approximation.
        `c`[(n_y-4) * (i-1) +j -1] is the coefficient c_{ij} defined in
        [Description].

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    nx : int
        The total number of knots, n_x, of the computed spline with
        respect to the x variable.

    ny : int
        The total number of knots, n_y, of the computed spline with
        respect to the y variable.

    fp : float
        The weighted sum of squared residuals, theta, of the computed
        spline approximation. `fp` should equal `s` within a relative
        tolerance of 0.001 unless `nx` = `ny` = 8, when the spline has
        no interior knots and so is simply a bicubic polynomial. For
        knots to be inserted, `s` must be set to a value below the value
        of `fp` produced in this case.

    rank : int
        Gives the rank of the system of equations used to compute the
        final spline (as determined by a suitable machine-dependent
        threshold). When `rank` = (`nx`-4)*(`ny`-4), the solution is
        unique; otherwise the system is rank-deficient and the
        minimum-norm solution is computed. The latter case may be caused
        by too small a value of `s`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nyest` = *<value>*.

            Constraint: `nyest` >= *<value>*.

        (`errno` 1)
            On entry, `nxest` = *<value>*.

            Constraint: `nxest` >= *<value>*.

        (`errno` 1)
            On entry, `start` != 'W' or 'C': `start` = *<value>*.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` > 0.0.

        (`errno` 1)
            On entry, MM < 16, where MM is the number of points with
            nonzero weight: MM = *<value>*.

        (`errno` 2)
            On entry, all the values in array `y` are equal.

        (`errno` 2)
            On entry, all the values in array `x` are equal.

        (`errno` 3)
            The number of knots needed in one direction is greater than
            `nxest` or `nyest`: `nxest`, `nyest` = *<value>*,*<value>*.
            Possibly `s` is too small: `s` = *<value>*.

        (`errno` 4)
            No more knots added; the number of B-spline coefficients
            already exceeds `m`. Either `m` or `s` is probably too
            small: `m` = *<value>* and `s` = *<value>*.

        (`errno` 5)
            No more knots added; the additional knot would coincide with
            an old one. Possibly an inaccurate data point has too large
            a weight, or `s` is too small: `s` = *<value>*.

        (`errno` 6)
            The iterative process has failed to converge. Possibly `s`
            is too small: `s` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_sctr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02ddft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_STRING, # Input, start(len=1).
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, f[m].
        utils._EngineFloat64ArrayType.c_type, # Input, w[m].
        utils._BYREF_FLOAT64, # Input, s.
        utils._BYREF_INT, # Input, nxest.
        utils._BYREF_INT, # Input, nyest.
        utils._BYREF_INT, # Input/Output, nx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, lamda[nxest].
        utils._BYREF_INT, # Input/Output, ny.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, mu[nyest].
        utils._EngineFloat64ArrayType.c_type, # Output, c[(nxest-4)*(nyest-4)].
        utils._BYREF_FLOAT64, # Output, fp.
        utils._BYREF_INT, # Output, rank.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrk[lwrk].
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._BYREF_INT, # Input, liwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of start.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _start = utils._EngineCharScalarType(
        start, {'fun_name': fun_name, 'entity': 'start'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    )
    _x = _x_cls.to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    )
    _y = _y_cls.to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _s = utils._EngineFloat64ScalarType(
        s, {'fun_name': fun_name, 'entity': 's'},
    ).to_ctypes()
    _nxest = utils._EngineIntScalarType(
        nxest, {'fun_name': fun_name, 'entity': 'nxest'},
    ).to_ctypes()
    _nyest = utils._EngineIntScalarType(
        nyest, {'fun_name': fun_name, 'entity': 'nyest'},
    ).to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _nxest_py = utils._EngineIntScalarType.to_py(_nxest)
    _lamda_cls = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_nxest_py,),
    )
    _lamda = _lamda_cls.to_ctypes()
    _ny = utils._EngineIntScalarType(
        ny, {'fun_name': fun_name, 'entity': 'ny'},
    ).to_ctypes()
    _nyest_py = utils._EngineIntScalarType.to_py(_nyest)
    _mu_cls = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_nyest_py,),
    )
    _mu = _mu_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_nxest_py-4)*(_nyest_py-4),),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _fp = utils._EngineFloat64ScalarType.empty_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wrk',),
    )
    _vl_u_py = _nxest_py-4
    _vl_v_py = _nyest_py-4
    _vl_w_py = max(_vl_u_py, _vl_v_py)
    _vl_lwrk2_py = 4*_vl_u_py*_vl_v_py*_vl_w_py+2*_vl_u_py*_vl_v_py+4*_vl_w_py
    if _comm_needs_init:
        _wrk = utils._EngineFloat64ArrayType.empty_ctypes((7*_vl_u_py*_vl_v_py+25*_vl_w_py)*(_vl_w_py+1)+2*(_vl_u_py+_vl_v_py+4*_m_py)+23*_vl_w_py+56+_vl_lwrk2_py)
        comm['wrk'] = _wrk
    else:
        _wrk = comm['wrk']
    _lwrk_py = utils._EngineFloat64ArrayType.get_shape(comm['wrk'], {'fun_name': fun_name, 'entity': 'comm[\'wrk\']'}, exp_rank=1, validate=False)[0]
    _lwrk = utils._EngineIntScalarType(
        _lwrk_py, {'fun_name': fun_name, 'entity': '_lwrk'},
    ).to_ctypes()
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _liwrk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _start,
        _m,
        _x,
        _y,
        _f,
        _w,
        _s,
        _nxest,
        _nyest,
        _nx,
        _lamda,
        _ny,
        _mu,
        _c,
        _fp,
        _rank,
        _wrk,
        _lwrk,
        _iwrk,
        _liwrk,
        _errbuf,
        _ierr,
        _start._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nyest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nyest >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nxest >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, start != \'W\' or \'C\': start = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: s > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, MM < 16, where MM is the number of points with',
                    'nonzero weight: MM = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the values in array y are equal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the values in array x are equal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of knots needed in one direction is greater than nxest or nyest:',
                    'nxest, nyest = {:s},{:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No more knots added; the number of B-spline coefficients already exceeds m.',
                    'Either m or s is probably too small:',
                    'm = {:s} and s = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No more knots added; the additional knot would coincide with an old one.',
                    'Possibly an inaccurate data point has too large a weight, or s is',
                    'too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative process has failed to converge.',
                    'Possibly s is too small: s = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _nx_py = utils._EngineIntScalarType.to_py(_nx)
        _lamda_cls.refresh_py(_lamda)
        _ny_py = utils._EngineIntScalarType.to_py(_ny)
        _mu_cls.refresh_py(_mu)
        _c_cls.refresh_py(_c)
        _fp_py = utils._EngineFloat64ScalarType.to_py(_fp)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nx', 'ny', 'fp', 'rank'],
        )(
            _nx_py,
            _ny_py,
            _fp_py,
            _rank_py,
        )
    _err_class.manage()
    return return_data

def dim2_spline_evalv(m, px, py, x, y, lamda, mu, c, ff):
    u"""
    Evaluation of fitted bicubic spline at a vector of points.

    For full information please refer to the NAG Library document for
    e02de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02def.html

    Parameters
    ----------
    m : int
        m, the number of points at which values of the spline are
        required.

    px : int
        `px` and `py` must specify the total number of knots associated
        with the variables x and y respectively. They are such that
        `px`-8 and `py`-8 are the corresponding numbers of interior
        knots.

    py : int
        `px` and `py` must specify the total number of knots associated
        with the variables x and y respectively. They are such that
        `px`-8 and `py`-8 are the corresponding numbers of interior
        knots.

    x : float, shape (`m`)
        `x` and `y` must contain x_r and y_r, for r = 1,2,...,m,
        respectively. These are the coordinates of the points at which
        values of the spline are required. The order of the points is
        immaterial.

    y : float, shape (`m`)
        `x` and `y` must contain x_r and y_r, for r = 1,2,...,m,
        respectively. These are the coordinates of the points at which
        values of the spline are required. The order of the points is
        immaterial.

    lamda : float, shape (`px`)
        `lamda` and `mu` must contain the complete sets of knots
        {lambda} and {mu} associated with the x and y variables
        respectively.

    mu : float, shape (`py`)
        `lamda` and `mu` must contain the complete sets of knots
        {lambda} and {mu} associated with the x and y variables
        respectively.

    c : float, shape ((`px`-4)*(`py`-4))
        `c`[ (`py`-4) * (i-1) +j -1] must contain the coefficient c_{ij}
        described in [Description], for j = 1,2,...,`py`-4, for i =
        1,2,...,`px`-4.

    ff : float, shape (`m`), modified in place
        On exit: `ff`[r-1] contains the value of the spline at the point
        (x_r,y_r), for r = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `py` = *<value>*.

            Constraint: `py` >= 8.

        (`errno` 1)
            On entry, `px` = *<value>*.

            Constraint: `px` >= 8.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, the knots in `mu` are not in nondecreasing order.

        (`errno` 2)
            On entry, the knots in `lamda` are not in nondecreasing
            order.

        (`errno` 3)
            On entry, point (`x`[K-1],`y`[K-1]) lies outside the
            rectangle bounded by `lamda`[3], `lamda`[`px`-4], `mu`[3],
            `mu`[`py`-4]: K = *<value>*, `x`[K-1] = *<value>* and
            `y`[K-1] = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_evalv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02deft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, px.
        utils._BYREF_INT, # Input, py.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[px].
        utils._EngineFloat64ArrayType.c_type, # Input, mu[py].
        utils._EngineFloat64ArrayType.c_type, # Input, c[(px-4)*(py-4)].
        utils._EngineFloat64ArrayType.c_type, # Output, ff[m].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _px = utils._EngineIntScalarType(
        px, {'fun_name': fun_name, 'entity': 'px'},
    ).to_ctypes()
    _py = utils._EngineIntScalarType(
        py, {'fun_name': fun_name, 'entity': 'py'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _px_py = utils._EngineIntScalarType.to_py(_px)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_px_py,),
    ).to_ctypes()
    _py_py = utils._EngineIntScalarType.to_py(_py)
    _mu = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_py_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_px_py-4)*(_py_py-4),),
    ).to_ctypes()
    _ff_cls = utils._EngineFloat64ArrayType(
        ff, {'fun_name': fun_name, 'entity': 'ff'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _ff = _ff_cls.to_ctypes(can_reference_contents=False)
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _px,
        _py,
        _x,
        _y,
        _lamda,
        _mu,
        _c,
        _ff,
        _wrk,
        _iwrk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: py >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, px = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: px >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the knots in mu are not in nondecreasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the knots in lamda are not in nondecreasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, point (x[K-1],y[K-1]) lies outside the rectangle bounded by',
                    'lamda[3], lamda[px-4], mu[3], mu[py-4]:',
                    'K = {:s}, x[K-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'y[K-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ff_cls.refresh_py(_ff)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_spline_evalm(mx, my, px, py, x, y, lamda, mu, c, ff):
    u"""
    Evaluation of fitted bicubic spline at a mesh of points.

    For full information please refer to the NAG Library document for
    e02df

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02dff.html

    Parameters
    ----------
    mx : int
        `mx` and `my` must specify m_x and m_y respectively, the number
        of points along the x and y axis that define the rectangular
        grid.

    my : int
        `mx` and `my` must specify m_x and m_y respectively, the number
        of points along the x and y axis that define the rectangular
        grid.

    px : int
        `px` and `py` must specify the total number of knots associated
        with the variables x and y respectively. They are such that
        `px`-8 and `py`-8 are the corresponding numbers of interior
        knots.

    py : int
        `px` and `py` must specify the total number of knots associated
        with the variables x and y respectively. They are such that
        `px`-8 and `py`-8 are the corresponding numbers of interior
        knots.

    x : float, shape (`mx`)
        `x` and `y` must contain x_q, for q = 1,2,...,m_x, and y_r, for
        r = 1,2,...,m_y, respectively. These are the x and y coordinates
        that define the rectangular grid of points at which values of
        the spline are required.

    y : float, shape (`my`)
        `x` and `y` must contain x_q, for q = 1,2,...,m_x, and y_r, for
        r = 1,2,...,m_y, respectively. These are the x and y coordinates
        that define the rectangular grid of points at which values of
        the spline are required.

    lamda : float, shape (`px`)
        `lamda` and `mu` must contain the complete sets of knots
        {lambda} and {mu} associated with the x and y variables
        respectively.

    mu : float, shape (`py`)
        `lamda` and `mu` must contain the complete sets of knots
        {lambda} and {mu} associated with the x and y variables
        respectively.

    c : float, shape ((`px`-4)*(`py`-4))
        `c`[ (`py`-4) * (i-1) +j -1] must contain the coefficient c_{ij}
        described in [Description], for j = 1,2,...,`py`-4, for i =
        1,2,...,`px`-4.

    ff : float, shape (`mx`*`my`), modified in place
        On exit: `ff`[`my`*(q-1)+r-1] contains the value of the spline
        at the point (x_q,y_r), for r = 1,2,...,m_y, for q =
        1,2,...,m_x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `my` = *<value>*.

            Constraint: `my` >= 1.

        (`errno` 1)
            On entry, `mx` = *<value>*.

            Constraint: `mx` >= 1.

        (`errno` 1)
            On entry, `py` = *<value>*.

            Constraint: `py` >= 8.

        (`errno` 1)
            On entry, `px` = *<value>*.

            Constraint: `px` >= 8.

        (`errno` 3)
            On entry, the knots in `mu` are not in nondecreasing order.

        (`errno` 3)
            On entry, the knots in `lamda` are not in nondecreasing
            order.

        (`errno` 4)
            On entry, `mu`[3] <= `y`[0] < ... < `y`[`my`-1] <=
            `mu`[`py`-4] is violated.

        (`errno` 4)
            On entry, `lamda`[3] <= `x`[0] < ... < `x`[`mx`-1] <=
            `lamda`[`px`-4] is violated.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_evalm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02dfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, mx.
        utils._BYREF_INT, # Input, my.
        utils._BYREF_INT, # Input, px.
        utils._BYREF_INT, # Input, py.
        utils._EngineFloat64ArrayType.c_type, # Input, x[mx].
        utils._EngineFloat64ArrayType.c_type, # Input, y[my].
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[px].
        utils._EngineFloat64ArrayType.c_type, # Input, mu[py].
        utils._EngineFloat64ArrayType.c_type, # Input, c[(px-4)*(py-4)].
        utils._EngineFloat64ArrayType.c_type, # Output, ff[mx*my].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._BYREF_INT, # Input, liwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _mx = utils._EngineIntScalarType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
    ).to_ctypes()
    _my = utils._EngineIntScalarType(
        my, {'fun_name': fun_name, 'entity': 'my'},
    ).to_ctypes()
    _px = utils._EngineIntScalarType(
        px, {'fun_name': fun_name, 'entity': 'px'},
    ).to_ctypes()
    _py = utils._EngineIntScalarType(
        py, {'fun_name': fun_name, 'entity': 'py'},
    ).to_ctypes()
    _mx_py = utils._EngineIntScalarType.to_py(_mx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mx_py,),
    ).to_ctypes()
    _my_py = utils._EngineIntScalarType.to_py(_my)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_my_py,),
    ).to_ctypes()
    _px_py = utils._EngineIntScalarType.to_py(_px)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_px_py,),
    ).to_ctypes()
    _py_py = utils._EngineIntScalarType.to_py(_py)
    _mu = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_py_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_px_py-4)*(_py_py-4),),
    ).to_ctypes()
    _ff_cls = utils._EngineFloat64ArrayType(
        ff, {'fun_name': fun_name, 'entity': 'ff'},
        exp_shape=(_mx_py*_my_py,),
        check_contents=False,
    )
    _ff = _ff_cls.to_ctypes(can_reference_contents=False)
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwrk = utils._EngineIntScalarType.c_type(0)
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _liwrk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _mx,
        _my,
        _px,
        _py,
        _x,
        _y,
        _lamda,
        _mu,
        _c,
        _ff,
        _wrk,
        _lwrk,
        _iwrk,
        _liwrk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, my = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: my >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: py >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, px = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: px >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the knots in mu are not in nondecreasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the knots in lamda are not in nondecreasing order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu[3] <= y[0] < ... < y[my-1] <= mu[py-4]',
                    'is violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lamda[3] <= x[0] < ... < x[mx-1] <= lamda[px-4]',
                    'is violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ff_cls.refresh_py(_ff)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_spline_derivm(mx, my, px, py, x, y, lamda, mu, c, nux, nuy, z):
    u"""
    Evaluation of spline surface at mesh of points with derivatives.

    For full information please refer to the NAG Library document for
    e02dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02dhf.html

    Parameters
    ----------
    mx : int
        m_x, the number of grid points along the x axis.

    my : int
        m_y, the number of grid points along the y axis.

    px : int
        The total number of knots in the x-direction of the bicubic
        spline approximation, e.g., the value nx as returned by
        ``dim2_spline_grid``.

    py : int
        The total number of knots in the y-direction of the bicubic
        spline approximation, e.g., the value ny as returned by
        ``dim2_spline_grid``.

    x : float, shape (`mx`)
        `x`[q-1] must be set to x_q, the x coordinate of the qth grid
        point along the x axis, for q = 1,2,...,m_x, on which values of
        the partial derivative are sought.

    y : float, shape (`my`)
        `y`[r-1] must be set to y_r, the y coordinate of the rth grid
        point along the y axis, for r = 1,2,...,m_y on which values of
        the partial derivative are sought.

    lamda : float, shape (`px`)
        Contains the position of the knots in the x-direction of the
        bicubic spline approximation to be differentiated, e.g., lamda
        as returned by ``dim2_spline_grid``.

    mu : float, shape (`py`)
        Contains the position of the knots in the y-direction of the
        bicubic spline approximation to be differentiated, e.g., mu as
        returned by ``dim2_spline_grid``.

    c : float, shape ((`px`-4)*(`py`-4))
        The coefficients of the bicubic spline approximation to be
        differentiated, e.g., c as returned by ``dim2_spline_grid``.

    nux : int
        Specifies the order, nu_x of the partial derivative in the
        x-direction.

    nuy : int
        Specifies the order, nu_y of the partial derivative in the
        y-direction.

    z : float, shape (`mx`*`my`), modified in place
        On exit: `z`[m_y*(q-1)+r-1] contains the derivative
        (d^(nu_x+nu_y))/dx^(nu_x)dy^(nu_y)s(x_q,y_r), for r =
        1,2,...,m_y, for q = 1,2,...,m_x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nux` = *<value>*.

            Constraint: 0 <= `nux` <= 2.

        (`errno` 2)
            On entry, `nuy` = *<value>*.

            Constraint: 0 <= `nuy` <= 2.

        (`errno` 3)
            On entry, `mx` = *<value>*.

            Constraint: `mx` >= 1.

        (`errno` 4)
            On entry, `my` = *<value>*.

            Constraint: `my` >= 1.

        (`errno` 5)
            On entry, for i = *<value>*, `x`[i-2] = *<value>* and
            `x`[i-1] = *<value>*.

            Constraint: `x`[i-2] <= `x`[i-1], for i = 2,3,...,`mx`.

        (`errno` 6)
            On entry, for i = *<value>*, `y`[i-2] = *<value>* and
            `y`[i-1] = *<value>*.

            Constraint: `y`[i-2] <= `y`[i-1], for i = 2,3,...,`my`.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_derivm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02dhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mx.
        utils._BYREF_INT, # Input, my.
        utils._BYREF_INT, # Input, px.
        utils._BYREF_INT, # Input, py.
        utils._EngineFloat64ArrayType.c_type, # Input, x[mx].
        utils._EngineFloat64ArrayType.c_type, # Input, y[my].
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[px].
        utils._EngineFloat64ArrayType.c_type, # Input, mu[py].
        utils._EngineFloat64ArrayType.c_type, # Input, c[(px-4)*(py-4)].
        utils._BYREF_INT, # Input, nux.
        utils._BYREF_INT, # Input, nuy.
        utils._EngineFloat64ArrayType.c_type, # Output, z[mx*my].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _mx = utils._EngineIntScalarType(
        mx, {'fun_name': fun_name, 'entity': 'mx'},
    ).to_ctypes()
    _my = utils._EngineIntScalarType(
        my, {'fun_name': fun_name, 'entity': 'my'},
    ).to_ctypes()
    _px = utils._EngineIntScalarType(
        px, {'fun_name': fun_name, 'entity': 'px'},
    ).to_ctypes()
    _py = utils._EngineIntScalarType(
        py, {'fun_name': fun_name, 'entity': 'py'},
    ).to_ctypes()
    _mx_py = utils._EngineIntScalarType.to_py(_mx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mx_py,),
    ).to_ctypes()
    _my_py = utils._EngineIntScalarType.to_py(_my)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_my_py,),
    ).to_ctypes()
    _px_py = utils._EngineIntScalarType.to_py(_px)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_px_py,),
    ).to_ctypes()
    _py_py = utils._EngineIntScalarType.to_py(_py)
    _mu = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_py_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=((_px_py-4)*(_py_py-4),),
    ).to_ctypes()
    _nux = utils._EngineIntScalarType(
        nux, {'fun_name': fun_name, 'entity': 'nux'},
    ).to_ctypes()
    _nuy = utils._EngineIntScalarType(
        nuy, {'fun_name': fun_name, 'entity': 'nuy'},
    ).to_ctypes()
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_mx_py*_my_py,),
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mx,
        _my,
        _px,
        _py,
        _x,
        _y,
        _lamda,
        _mu,
        _c,
        _nux,
        _nuy,
        _z,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nux = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= nux <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nuy = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= nuy <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mx >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, my = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: my >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'x[i-2] = {:s} and x[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[i-2] <= x[i-1], for i = 2,3,...,mx.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'y[i-2] = {:s} and y[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: y[i-2] <= y[i-1], for i = 2,3,...,my.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def glin_l1sol(m, a, b, nplus2, toler, x):
    u"""
    L_1-approximation by general linear function.

    For full information please refer to the NAG Library document for
    e02ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02gaf.html

    Parameters
    ----------
    m : int
        The number of equations, m (the number of rows of the matrix A).

    a : float, shape (`m`+2, `nplus2`), modified in place
        On entry: `a`[i-1,j-1] must contain a_{ij}, the element in the
        ith row and jth column of the matrix A, for j = 1,2,...,n, for i
        = 1,2,...,m. The remaining elements need not be set.

        On exit: contains the last simplex tableau generated by the
        simplex method.

    b : float, shape (`m`), modified in place
        On entry: `b`[i-1] must contain b_i, the ith element of the
        vector b, for i = 1,2,...,m.

        On exit: the ith residual r_i corresponding to the solution
        vector x, for i = 1,2,...,m.

    nplus2 : int
        n+2, where n is the number of unknowns (the number of columns of
        the matrix A).

    toler : float
        A non-negative value. In general `toler` specifies a threshold
        below which numbers are regarded as zero. The recommended
        threshold value is epsilon^(2/3) where epsilon is the machine
        precision. The recommended value can be computed within the
        function by setting `toler` to zero. If premature termination
        occurs a larger value for `toler` may result in a valid
        solution.

    x : float, shape (`nplus2`), modified in place
        On exit: `x`[j-1] contains the jth element of the solution
        vector x, for j = 1,2,...,n. The elements `x`[n] and `x`[n+1]
        are unused.

    Returns
    -------
    resid : float
        The sum of the absolute values of the residuals for the solution
        vector x.

    irank : int
        The computed rank of the matrix A.

    itera : int
        The number of iterations taken by the simplex method.

    Raises
    ------
    NagValueError
        (`errno` 2)
            Premature termination due to rounding errors. Try using
            larger value of `toler`: `toler` = *<value>*.

        (`errno` 3)
            On entry, `nplus2` = *<value>* and `m` = *<value>*.

            Constraint: 3 <= `nplus2` <= `m`+2.

        (`errno` 3)
            On entry, `nplus2` = *<value>*.

            Constraint: `nplus2` >= 3.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            An optimal solution has been obtained, but may not be
            unique.

        (`errno` 4)
            More than 1000*n iterations were performed. ``glin_l1sol``
            has terminated without calculating a solution. The output
            data from the function is as computed on the last good
            iteration. Consider increasing the value of `toler`.
            Alternatively, A may be ill conditioned---try scaling its
            columns.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.glin_l1sol'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, nplus2].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[m].
        utils._BYREF_INT, # Input, nplus2.
        utils._BYREF_FLOAT64, # Input, toler.
        utils._EngineFloat64ArrayType.c_type, # Output, x[nplus2].
        utils._BYREF_FLOAT64, # Output, resid.
        utils._BYREF_INT, # Output, irank.
        utils._BYREF_INT, # Output, itera.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _nplus2 = utils._EngineIntScalarType(
        nplus2, {'fun_name': fun_name, 'entity': 'nplus2'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _nplus2_py = utils._EngineIntScalarType.to_py(_nplus2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py+2, _nplus2_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py,),
    )
    _b = _b_cls.to_ctypes()
    _toler = utils._EngineFloat64ScalarType(
        toler, {'fun_name': fun_name, 'entity': 'toler'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_nplus2_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _resid = utils._EngineFloat64ScalarType.empty_ctypes()
    _irank = utils._EngineIntScalarType.empty_ctypes()
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _a,
        _lda,
        _b,
        _nplus2,
        _toler,
        _x,
        _resid,
        _irank,
        _itera,
        _iwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An optimal solution has been obtained, but may not be unique.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Premature termination due to rounding errors.',
                    'Try using larger value of toler: toler = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nplus2 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 3 <= nplus2 <= m+2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nplus2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nplus2 >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than 1000*n iterations were performed.',
                    'glin_l1sol has terminated without calculating a solution.',
                    'The output data from the function is as computed on the last good iteration.',
                    'Consider increasing the value of toler.',
                    'Alternatively, A may be ill conditioned---try scaling its columns.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _resid_py = utils._EngineFloat64ScalarType.to_py(_resid)
        _irank_py = utils._EngineIntScalarType.to_py(_irank)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['resid', 'irank', 'itera'],
        )(
            _resid_py,
            _irank_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def glinc_l1sol(m, n, mpl, e, f, x, mxs, monit, iprint, indx, data=None):
    u"""
    L_1-approximation by general linear function subject to linear
    inequality constraints.

    For full information please refer to the NAG Library document for
    e02gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02gbf.html

    Parameters
    ----------
    m : int
        The number of equations in the over-determined system, m (i.e.,
        the number of rows of the matrix A).

    n : int
        The number of unknowns, n (the number of columns of the matrix
        A).

    mpl : int
        m+l, where l is the number of constraints (which may be zero).

    e : float, shape (`n`, `mpl`), modified in place
        On entry: the equation and constraint matrices stored in the
        following manner.

        The first m columns contain the m rows of the matrix A; element
        `e`[i-1,j-1] specifying the element a_{ji} in the jth row and
        ith column of A (the coefficient of the ith unknown in the jth
        equation), for j = 1,2,...,m, for i = 1,2,...,n.

        The next l columns contain the l rows of the constraint matrix
        C; element `e`[i-1,j+m-1] containing the element c_{ji} in the
        jth row and ith column of C (the coefficient of the ith unknown
        in the jth constraint), for j = 1,2,...,l, for i = 1,2,...,n.

        On exit: unchanged, except possibly to the extent of a small
        multiple of the machine precision. (See [Further Comments].)

    f : float, shape (`mpl`)
        `f`[i-1], for i = 1,2,...,m, must contain b_i (the ith element
        of the right-hand side vector of the over-determined system of
        equations) and `f`[m+i-1], for i = 1,2,...,l, must contain d_i
        (the ith element of the right-hand side vector of the
        constraints), where l is the number of constraints.

    x : float, shape (`n`), modified in place
        On entry: `x`[i-1] must contain an estimate of the ith unknown,
        for i = 1,2,...,n. If no better initial estimate for `x`[i-1] is
        available, set `x`[i-1] = 0.0.

        On exit: the latest estimate of the ith unknown, for i =
        1,2,...,n. If no exception or warning is raised on exit, these
        are the solution values.

    mxs : int
        The maximum number of steps to be allowed for the solution of
        the unconstrained problem. Typically this may be a modest
        multiple of n. If, on entry, `mxs` is zero or negative, the
        value returned by ``machine.integer_max`` is used.

    monit : callable monit(n, x, niter, k, el1n, data=None)
        `monit` can be used to print out the current values of any
        selection of its arguments.

        The frequency with which `monit` is called in ``glinc_l1sol`` is
        controlled by `iprint`.

        Parameters
        ~~~~~~~~~~
        n : int
            The number n of unknowns (the number of columns of the
            matrix A).

        x : float, ndarray, shape (`n`)
            The latest estimate of the unknowns.

        niter : int
            The number of iterations so far carried out.

        k : int
            The total number of equations and constraints which are
            currently active.

        el1n : float
            The l_1-norm of the current residuals of the over-determined
            system of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    iprint : int
        The frequency of iteration print out.

        `iprint` > 0

          `monit` is called every `iprint` iterations and at the
          solution.

        `iprint` = 0

          Information is printed out at the solution only. Otherwise
          `monit` is not called.

    indx : int, shape (`mpl`), modified in place
        On exit: specifies which columns of `e` relate to the inactive
        equations and constraints. `indx`[0] up to `indx`[`k`-1] number
        the active columns and `indx`[`k`] up to `indx`[`mpl`-1] number
        the inactive columns.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    k : int
        The total number of equations and constraints which are then
        active.

    el1n : float
        The l_1-norm (sum of absolute values) of the equation residuals.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The constraints cannot all be satisfied simultaneously: they
            are not compatible with one another. Hence no solution is
            possible.

        (`errno` 2)
            The limit imposed by `mxs` has been reached without finding
            a solution. Consider restarting from the current point by
            simply calling ``glinc_l1sol`` again without changing the
            arguments.

        (`errno` 3)
            The function has failed because of numerical difficulties;
            the problem is too ill-conditioned. Consider rescaling the
            unknowns.

        (`errno` 4)
            Elements 1 to `m` of one of the first `mpl` columns of the
            array `e` are all zero -- this corresponds to a zero row in
            either of the matrices A or C.

        (`errno` 4)
            On entry, `mpl` = *<value>* and `m` = *<value>*.

            Constraint: `mpl` >= `m`.

        (`errno` 4)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= `n`.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.glinc_l1sol'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, niter.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, el1n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, n, x, niter, k, el1n, _iuser, _ruser):
        """Helper for monit."""
        try:
            _n_monith = utils._EngineIntScalarType.to_py(n.contents)
            _x_monith = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_monith,),
            )
            _niter_monith = utils._EngineIntScalarType.to_py(niter.contents)
            _k_monith = utils._EngineIntScalarType.to_py(k.contents)
            _el1n_monith = utils._EngineFloat64ScalarType.to_py(el1n.contents)
            if data is not None:
                monit(
                    _n_monith,
                    _x_monith,
                    _niter_monith,
                    _k_monith,
                    _el1n_monith,
                    data,
                )
            else:
                monit(
                    _n_monith,
                    _x_monith,
                    _niter_monith,
                    _k_monith,
                    _el1n_monith,
                )
            utils._unlink_pointer(x)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('e02gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, mpl.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[lde, mpl].
        utils._BYREF_INT, # Input, lde.
        utils._EngineFloat64ArrayType.c_type, # Input, f[mpl].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._BYREF_INT, # Input, mxs.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._BYREF_INT, # Input, iprint.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_FLOAT64, # Output, el1n.
        utils._EngineIntArrayType.c_type, # Output, indx[mpl].
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, iw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((e, 'e'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _mpl = utils._EngineIntScalarType(
        mpl, {'fun_name': fun_name, 'entity': 'mpl'},
    ).to_ctypes()
    _lde_py = utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lde = utils._EngineIntScalarType(
        _lde_py, {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _mpl_py = utils._EngineIntScalarType.to_py(_mpl)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _mpl_py),
        mutual_sorder=_m_sorder,
    )
    _e = _e_cls.to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_mpl_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _mxs = utils._EngineIntScalarType(
        mxs, {'fun_name': fun_name, 'entity': 'mxs'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'})
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    _iprint = utils._EngineIntScalarType(
        iprint, {'fun_name': fun_name, 'entity': 'iprint'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _el1n = utils._EngineFloat64ScalarType.empty_ctypes()
    _indx_cls = utils._EngineIntArrayType(
        indx, {'fun_name': fun_name, 'entity': 'indx'},
        exp_shape=(_mpl_py,),
        check_contents=False,
    )
    _indx = _indx_cls.to_ctypes(can_reference_contents=False)
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _m,
        _n,
        _mpl,
        _e,
        _lde,
        _f,
        _x,
        _mxs,
        _monit,
        _monith,
        _iprint,
        _k,
        _el1n,
        _indx,
        _w,
        _iw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The constraints cannot all be satisfied simultaneously: they are not',
                    'compatible with one another. Hence no solution is possible.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The limit imposed by mxs has been reached without finding a',
                    'solution. Consider restarting from the current point by simply calling',
                    'glinc_l1sol again without changing the arguments.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function has failed because of numerical difficulties; the problem is',
                    'too ill-conditioned. Consider rescaling the unknowns.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Elements 1 to m of one of the first mpl columns of the array',
                    'e are all zero -- this corresponds to a zero row in either of the matrices A',
                    'or C.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mpl = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mpl >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _e_cls.refresh_py(_e)
        _x_cls.refresh_py(_x)
        _indx_cls.refresh_py(_indx)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _el1n_py = utils._EngineFloat64ScalarType.to_py(_el1n)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'el1n'],
        )(
            _k_py,
            _el1n_py,
        )
    _err_class.manage()
    return return_data

def glin_linf(m, n, a, b, tol, relerr, x):
    u"""
    L_infinity-approximation by general linear function.

    For full information please refer to the NAG Library document for
    e02gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02gcf.html

    Parameters
    ----------
    m : int
        The number of equations, m (the number of rows of the matrix A).

    n : int
        The number of unknowns, n (the number of columns of the matrix
        A).

    a : float, shape (`n`+3, `m`+1), modified in place
        On entry: `a`[j-1,i-1] must contain a_{ij}, the element in the
        ith row and jth column of the matrix A, for j = 1,2,...,n, for i
        = 1,2,...,m, (that is, the transpose of the matrix). The
        remaining elements need not be set. Preferably, the columns of
        the matrix A (rows of the argument `a`) should be scaled before
        entry: see [Accuracy].

        On exit: contains the last simplex tableau.

    b : float, shape (`m`), modified in place
        On entry: `b`[i-1] must contain b_i, the ith element of the
        vector b, for i = 1,2,...,m.

        On exit: the ith residual r_i corresponding to the solution
        vector x, for i = 1,2,...,m. Note however that these residuals
        may contain few significant figures, especially when `resmax` is
        within one or two orders of magnitude of `tol`. Indeed if
        `resmax` <= `tol`, the elements `b`[i-1] may all be set to zero.
        It is therefore often advisable to compute the residuals
        directly.

    tol : float
        A threshold below which numbers are regarded as zero. The
        recommended threshold value is 10.0*epsilon, where epsilon is
        the machine precision. If `tol` <= 0.0 on entry, the recommended
        value is used within the function. If premature termination
        occurs, a larger value for `tol` may result in a valid solution.

    relerr : float
        Must be set to a bound on the relative error acceptable in the
        maximum residual at the solution.

        If `relerr` <= 0.0, the l_infinity solution is computed, and
        `relerr` is set to 0.0 on exit.

        If `relerr` > 0.0, the function obtains instead an approximate
        solution for which the largest residual is less than
        1.0+`relerr` times that of the l_infinity solution; on exit,
        `relerr` contains a smaller value such that the above bound
        still applies. (The usual result of this option, say with
        `relerr` = 0.1, is a saving in the number of simplex
        iterations).

    x : float, shape (`n`), modified in place
        On exit: if the function exits successfully or `errno` = 1,
        `x`[j-1] contains the jth element of the solution vector x, for
        j = 1,2,...,n. Whether this is an l_infinity solution or an
        approximation to one, depends on the value of `relerr` on entry.

    Returns
    -------
    relerr : float
        Is altered as described above.

    resmax : float
        If the function exits successfully or `errno` = 1, `resmax`
        contains the absolute value of the largest residual(s) for the
        solution vector x. (See `b`.)

    irank : int
        If the function exits successfully or `errno` = 1, `irank`
        contains the computed rank of the matrix A.

    itera : int
        If the function exits successfully or `errno` = 1, `itera`
        contains the number of iterations taken by the simplex method.

    Raises
    ------
    NagValueError
        (`errno` 2)
            Premature termination due to rounding errors. Try using
            larger value of `tol`: `tol` = *<value>*.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= `n`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            An optimal solution has been obtained, but may not be
            unique.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.glin_linf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, sda.
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, sda].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[m].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Input/Output, relerr.
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._BYREF_FLOAT64, # Output, resmax.
        utils._BYREF_INT, # Output, irank.
        utils._BYREF_INT, # Output, itera.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _sda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _sda = utils._EngineIntScalarType(
        _sda_py, {'fun_name': fun_name, 'entity': '_sda'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py+3, _m_py+1),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py,),
    )
    _b = _b_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _relerr = utils._EngineFloat64ScalarType(
        relerr, {'fun_name': fun_name, 'entity': 'relerr'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _resmax = utils._EngineFloat64ScalarType.empty_ctypes()
    _irank = utils._EngineIntScalarType.empty_ctypes()
    _itera = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _sda,
        _lda,
        _a,
        _b,
        _tol,
        _relerr,
        _x,
        _resmax,
        _irank,
        _itera,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An optimal solution has been obtained, but may not be unique.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Premature termination due to rounding errors.',
                    'Try using larger value of tol: tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _relerr_py = utils._EngineFloat64ScalarType.to_py(_relerr)
        _x_cls.refresh_py(_x)
        _resmax_py = utils._EngineFloat64ScalarType.to_py(_resmax)
        _irank_py = utils._EngineIntScalarType.to_py(_irank)
        _itera_py = utils._EngineIntScalarType.to_py(_itera)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['relerr', 'resmax', 'irank', 'itera'],
        )(
            _relerr_py,
            _resmax_py,
            _irank_py,
            _itera_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim2_spline_ts_sctr(n, x, y, f, lsminp, lsmaxp, nxcels, nycels, comm):
    u"""
    Spline approximation to a set of scattered data using a two-stage
    approximation method.

    Note: this function uses optional algorithmic parameters, see also:
    ``opt_set``, ``opt_get``.

    For full information please refer to the NAG Library document for
    e02jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02jdf.html

    Parameters
    ----------
    n : int
        n, the number of data values to be fitted.

    x : float, shape (`n`)
        The (x_i,y_i,f_i) data values to be fitted.

    y : float, shape (`n`)
        The (x_i,y_i,f_i) data values to be fitted.

    f : float, shape (`n`)
        The (x_i,y_i,f_i) data values to be fitted.

    lsminp : int
        `lsminp` and `lsmaxp` are control parameters for the local
        approximations.

        Each local approximation is computed on a local domain
        containing one of the triangles in the discretization of the
        bounding box.

        The size of each local domain will be adaptively chosen such
        that if it contains fewer than `lsminp` sample points it is
        expanded, else if it contains greater than `lsmaxp` sample
        points a thinning method is applied. `lsmaxp` mainly controls
        computational cost (in that working with a thinned set of points
        is cheaper and may be appropriate if the input data is densely
        distributed), while `lsminp` allows handling of different types
        of scattered data.

        Setting `lsmaxp` < `lsminp`, and therefore forcing either
        expansion or thinning, may be useful for computing initial
        coarse approximations.

        In general smaller values for these arguments reduces cost.

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate values for `lsminp` and `lsmaxp`.

    lsmaxp : int
        `lsminp` and `lsmaxp` are control parameters for the local
        approximations.

        Each local approximation is computed on a local domain
        containing one of the triangles in the discretization of the
        bounding box.

        The size of each local domain will be adaptively chosen such
        that if it contains fewer than `lsminp` sample points it is
        expanded, else if it contains greater than `lsmaxp` sample
        points a thinning method is applied. `lsmaxp` mainly controls
        computational cost (in that working with a thinned set of points
        is cheaper and may be appropriate if the input data is densely
        distributed), while `lsminp` allows handling of different types
        of scattered data.

        Setting `lsmaxp` < `lsminp`, and therefore forcing either
        expansion or thinning, may be useful for computing initial
        coarse approximations.

        In general smaller values for these arguments reduces cost.

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate values for `lsminp` and `lsmaxp`.

    nxcels : int
        `nxcels` (respectively `nycels`) is the number of cells in the x
        (respectively y) direction that will be used to create the
        triangulation of the bounding box of the domain of the function
        to be fitted.

        Greater efficiency generally comes when `nxcels` and `nycels`
        are chosen to be of the same order of magnitude and are such
        that `n` is O(`nxcels`*`nycels`).

        Thus for a 'square' triangulation --- when `nxcels` = `nycels`
        --- the quantities sqrt(`n`) and `nxcels` should be of the same
        order of magnitude.

        See also [Further Comments].

    nycels : int
        `nxcels` (respectively `nycels`) is the number of cells in the x
        (respectively y) direction that will be used to create the
        triangulation of the bounding box of the domain of the function
        to be fitted.

        Greater efficiency generally comes when `nxcels` and `nycels`
        are chosen to be of the same order of magnitude and are such
        that `n` is O(`nxcels`*`nycels`).

        Thus for a 'square' triangulation --- when `nxcels` = `nycels`
        --- the quantities sqrt(`n`) and `nxcels` should be of the same
        order of magnitude.

        See also [Further Comments].

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    Other Parameters
    ----------------
    'Averaged Spline' : str
        Default = 'NO'

        When the bounding box is triangulated there are 8 equivalent
        configurations of the mesh.
        Setting 'Averaged Spline' = 'YES' will use the averaged value of
        the 8 possible local polynomial approximations over each
        triangle in the mesh.
        This usually gives better results but at (about 8 times) higher
        computational cost.

        Constraint: 'Averaged Spline' = 'YES' or 'NO'.

    'Global Smoothing Level' : int
        Default = 1

        The smoothness level for the global spline approximation.

        'Global Smoothing Level' = 1
          Will use C^1 piecewise cubics.

        'Global Smoothing Level' = 2
          Will use C^2 piecewise sextics.

    'Interpolation Only RBF' : str
        Default = 'YES'

        If 'Interpolation Only RBF' = 'YES', each local RBF
        approximation is computed by interpolation.

        If 'Interpolation Only RBF' = 'NO', each local RBF approximation
        is computed by a discrete least squares approach.
        This is likely to be more accurate and more expensive than
        interpolation.

        If 'Local Method' = 'HYBRID' or 'POLYNOMIAL', this option
        setting is ignored.

    'Local Method' : str
        Default = 'POLYNOMIAL'

        The local approximation scheme to use.

        'Local Method' = 'POLYNOMIAL'
          Uses least squares polynomial approximations.

        'Local Method' = 'HYBRID'
          Uses hybrid polynomial and RBF approximations.

        'Local Method' = 'RBF'
          Uses pure RBF approximations.

        In general 'POLYNOMIAL' is less computationally expensive than
        'HYBRID' is less computationally expensive than 'RBF' with the
        reverse ordering holding for accuracy of results.

    'Minimum Singular Value LHA' : float
        Default = 1.0

        A tolerance measure for accepting or rejecting a local hybrid
        approximation (LHA) as reliable.

        The solution of a local least squares problem solved on each
        triangle subdomain is accepted as reliable if the minimum
        singular value sigma of the collocation matrix (of polynomial
        and radial basis function terms) associated with the least
        squares problem satisfies 'Minimum Singular Value LHA' <= sigma.

        In general the approximation power will be reduced as 'Minimum
        Singular Value LHA' is reduced. (A small sigma indicates that
        the local data has hidden redundancies which prevent it from
        carrying enough information for a good approximation to be
        made.) Setting 'Minimum Singular Value LHA' very large may have
        the detrimental effect that only approximations of low degree
        are deemed reliable.

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate value for this parameter.

        If 'Local Method' = 'POLYNOMIAL' or 'RBF', this option setting
        is ignored.

    'Minimum Singular Value LPA' : float
        Default = 1.0

        A tolerance measure for accepting or rejecting a local
        polynomial approximation (LPA) as reliable.
        Clearly this setting is relevant when 'Local Method' =
        'POLYNOMIAL', but it also may be used when 'Local Method' =
        'HYBRID' (see [Further Comments].)

        The solution of a local least squares problem solved on each
        triangle subdomain is accepted as reliable if the minimum
        singular value sigma of the matrix (of Bernstein polynomial
        values) associated with the least squares problem satisfies
        'Minimum Singular Value LPA' <= sigma.

        In general the approximation power will be reduced as 'Minimum
        Singular Value LPA' is reduced. (A small sigma indicates that
        the local data has hidden redundancies which prevent it from
        carrying enough information for a good approximation to be
        made.) Setting 'Minimum Singular Value LPA' very large may have
        the detrimental effect that only approximations of low degree
        are deemed reliable.

        'Minimum Singular Value LPA' will have no effect if 'Polynomial
        Starting Degree' = 0, and it will have little effect if the
        input data is 'smooth' (e.g., from a known function).

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate value for this parameter.

        If 'Local Method' = 'RBF', this option setting is ignored.

    'Polynomial Starting Degree' : int
        Default = 5 if 'Local Method' = 'HYBRID',
        Default = 1 otherwise

        The degree to be used for the polynomial part in the initial
        step of each local approximation.

        At this initial step the method will attempt to fit with a local
        approximation having polynomial part of degree 'Polynomial
        Starting Degree'.
        If 'Local Method' = 'POLYNOMIAL' and the approximation is deemed
        unreliable (according to 'Minimum Singular Value LPA'), the
        degree will be decremented by one and a new local approximation
        computed, ending with a constant approximation if no other is
        reliable.
        If 'Local Method' = 'HYBRID' and the approximation is deemed
        unreliable (according to 'Minimum Singular Value LHA'), a pure
        polynomial approximation of this degree will be tried instead.
        The method then proceeds as in the 'POLYNOMIAL' case.

        'Polynomial Starting Degree' is bounded from above by the
        maximum possible spline degree, which is 6 (when performing C^2
        global super-smoothing).
        Note that the best-case approximation error (see [Accuracy]) for
        C^2 smoothing with 'Supersmooth C2' = 'NO' is achieved for local
        polynomials of degree 5; that is, for this level of global
        smoothing no further benefit is gained by setting 'Polynomial
        Starting Degree' = 6.

        The default value gives a good compromise between efficiency and
        accuracy.
        In general the best approximation can be obtained by setting:

        - If 'Local Method' = 'POLYNOMIAL'

          - if 'Global Smoothing Level' = 1, 'Polynomial Starting
            Degree' = 3;

          - if 'Global Smoothing Level' = 2;

            - if 'Supersmooth C2' = 'NO', 'Polynomial Starting Degree' =
              5;

            - otherwise 'Polynomial Starting Degree' = 6.

        - If 'Local Method' = 'HYBRID', 'Polynomial Starting Degree' as
          small as possible.

        If 'Local Method' = 'RBF', this option setting is ignored.

    'Radial Basis Function' : str
        Default = 'MQ'

        'Radial Basis Function' selects the RBF to use in each local RBF
        approximation, while 'Scaling Coefficient RBF' selects the scale
        factor to use in its evaluation, as described below.

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate scale factor and RBF.

        If 'Local Method' = 'POLYNOMIAL', these option settings are
        ignored.

        If 'Local Method' = 'HYBRID' or 'RBF', the following
        (conditionally) positive definite functions may be chosen.

        Define R = sqrt(x^2+y^2) and rho = R/r.

        [table omitted]

        If 'Local Method' = 'HYBRID' the following conditionally
        positive definite functions may also be chosen.

        [table omitted]

    'Scaling Coefficient RBF' : float
        Default = 1.0

        'Radial Basis Function' selects the RBF to use in each local RBF
        approximation, while 'Scaling Coefficient RBF' selects the scale
        factor to use in its evaluation, as described below.

        A calibration procedure (experimenting with a small subset of
        the data to be fitted and validating the results) may be needed
        to choose the most appropriate scale factor and RBF.

        If 'Local Method' = 'POLYNOMIAL', these option settings are
        ignored.

        If 'Local Method' = 'HYBRID' or 'RBF', the following
        (conditionally) positive definite functions may be chosen.

        Define R = sqrt(x^2+y^2) and rho = R/r.

        [table omitted]

        If 'Local Method' = 'HYBRID' the following conditionally
        positive definite functions may also be chosen.

        [table omitted]

    'Separation LRBFA' : float
        Default = 16.0/'Scaling Coefficient RBF'

        A knot-separation parameter used to control the condition number
        of the matrix used in each local RBF approximation (LRBFA).
        A smaller value may mean greater numerical stability but fewer
        knots.

        If 'Local Method' = 'HYBRID' or 'POLYNOMIAL', this option
        setting is ignored.

    'Supersmooth C2' : str
        Default = 'NO'

        If 'Supersmooth C2' = 'YES', the C^2 spline is generated using
        additional smoothness constraints.
        This usually gives better results but at higher computational
        cost.

        If 'Global Smoothing Level' = 1 this option setting is ignored.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 4)
            On entry, `lsminp` = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `lsminp` <= `n`.

        (`errno` 5)
            On entry, `lsmaxp` = *<value>*.

            Constraint: `lsmaxp` >= 1.

        (`errno` 6)
            On entry, `nxcels` = *<value>*.

            Constraint: `nxcels` >= 1.

        (`errno` 7)
            On entry, `nycels` = *<value>*.

            Constraint: `nycels` >= 1.

        (`errno` 9)
            Option arrays are not initialized or are corrupted.

        (`errno` 11)
            An unexpected algorithmic failure was encountered. Please
            contact NAG.

        (`errno` 12)
            On entry, all elements of `x` or of `y` are equal.

        (`errno` 20)
            The selected radial basis function cannot be used with the
            RBF local method.

        (`errno` 21)
            The value of option 'Polynomial Starting Degree' was
            invalid.

    See Also
    --------
    :meth:`naginterfaces.library.examples.fit.dim2_spline_ts_sctr_ex.main`
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_ts_sctr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02jdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Input, f[n].
        utils._BYREF_INT, # Input, lsminp.
        utils._BYREF_INT, # Input, lsmaxp.
        utils._BYREF_INT, # Input, nxcels.
        utils._BYREF_INT, # Input, nycels.
        utils._BYREF_INT, # Input, lcoefs.
        utils._EngineFloat64ArrayType.c_type, # Output, coefs[lcoefs].
        utils._EngineIntArrayType.c_type, # Input/Output, iopts[liopts].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, opts[lopts].
        utils._BYREF_INT, # Input, liopts.
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lsminp = utils._EngineIntScalarType(
        lsminp, {'fun_name': fun_name, 'entity': 'lsminp'},
    ).to_ctypes()
    _lsmaxp = utils._EngineIntScalarType(
        lsmaxp, {'fun_name': fun_name, 'entity': 'lsmaxp'},
    ).to_ctypes()
    _nxcels = utils._EngineIntScalarType(
        nxcels, {'fun_name': fun_name, 'entity': 'nxcels'},
    ).to_ctypes()
    _nycels = utils._EngineIntScalarType(
        nycels, {'fun_name': fun_name, 'entity': 'nycels'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('coefs',),
        req_keys=('iopts', 'opts'),
    )
    _nxcels_py = utils._EngineIntScalarType.to_py(_nxcels)
    _nycels_py = utils._EngineIntScalarType.to_py(_nycels)
    _vl_liopts_py = utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (0,), validate=False)
    _vl_gsmoothness_py = utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (11,), validate=False)
    if _vl_liopts_py >= 12:
        if _vl_gsmoothness_py == 1:
            _coefs_dim1 = (((_nxcels_py+2)*(_nycels_py+2)+1)//2)*10+1
        elif _vl_gsmoothness_py == 2:
            _coefs_dim1 = 28*(_nxcels_py+2)*(_nycels_py+2)*4+1
        else:
            _coefs_dim1 = 0
    else:
        _coefs_dim1 = 0
    _coefs = utils._EngineFloat64ArrayType.empty_ctypes(_coefs_dim1)
    comm['coefs'] = _coefs
    _lcoefs_py = utils._EngineFloat64ArrayType.get_shape(comm['coefs'], {'fun_name': fun_name, 'entity': 'comm[\'coefs\']'}, exp_rank=1, validate=False)[0]
    _lcoefs = utils._EngineIntScalarType(
        _lcoefs_py, {'fun_name': fun_name, 'entity': '_lcoefs'},
    ).to_ctypes()
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    _iopts = comm['iopts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _opts = comm['opts']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _y,
        _f,
        _lsminp,
        _lsmaxp,
        _nxcels,
        _nycels,
        _lcoefs,
        _coefs,
        _iopts,
        _opts,
        _liopts,
        _lopts,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _lsminp_py = utils._EngineIntScalarType.to_py(_lsminp)
        _lsmaxp_py = utils._EngineIntScalarType.to_py(_lsmaxp)
        if _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_DATA_SIZE:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_MINP:
            _ierr_fl = 4
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lsminp = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT.format(_lsminp_py),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                    'Constraint: 1 <= lsminp <= n.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_MAXP:
            _ierr_fl = 5
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lsmaxp = {:s}.'.format(
                        utils._STR_FMT_INT.format(_lsmaxp_py),
                    ),
                    'Constraint: lsmaxp >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_GRIDX:
            _ierr_fl = 6
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxcels = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nxcels_py),
                    ),
                    'Constraint: nxcels >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_GRIDY:
            _ierr_fl = 7
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nycels = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nycels_py),
                    ),
                    'Constraint: nycels >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_INIT:
            _ierr_fl = 9
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Option arrays are not initialized or are corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_SVD and _ifmt == _nag_e_consts.IERR_TSFIT_SVD:
            _ierr_fl = 11
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected algorithmic failure was encountered. Please contact NAG.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_LSE:
            _ierr_fl = 11
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected algorithmic failure was encountered. Please contact NAG.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_DEGENERATE:
            _ierr_fl = 12
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all elements of x or of y are equal.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_OPTION and _ifmt == _nag_e_consts.IFMT_TSFIT_RBF:
            _ierr_fl = 20
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The selected radial basis function cannot be used with the RBF local method.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_OPTION and _ifmt == _nag_e_consts.IFMT_TSFIT_DEGREE:
            _ierr_fl = 21
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of option \'Polynomial Starting Degree\' was invalid.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def dim2_spline_ts_evalv(nevalv, xevalv, yevalv, comm, fevalv):
    u"""
    Evaluation at a vector of points of a spline computed by
    ``dim2_spline_ts_sctr``.

    For full information please refer to the NAG Library document for
    e02je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02jef.html

    Parameters
    ----------
    nevalv : int
        n, the number of values at which the spline is to be evaluated.

    xevalv : float, shape (`nevalv`)
        The (x_i) values at which the spline is to be evaluated.

    yevalv : float, shape (`nevalv`)
        The (y_i) values at which the spline is to be evaluated.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by prior calls to
        ``dim2_spline_ts_sctr`` and ``opt_set``.

    fevalv : float, shape (`nevalv`), modified in place
        On exit: if no exception or warning is raised on exit
        `fevalv`[i-1] contains the computed spline value at (x_i,y_i).

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `nevalv` = *<value>*.

            Constraint: `nevalv` >= 1.

        (`errno` 9)
            Option arrays are not initialized or are corrupted.

        (`errno` 10)
            The fitting routine has not been called, or the array of
            coefficients has been corrupted.

        (`errno` 13)
            On entry, `xevalv`[*<value>*] = *<value>* was outside the
            bounding box.

            Constraint: *<value>* <= `xevalv`[i-1] <= *<value>* for all
            i.

        (`errno` 14)
            On entry, `yevalv`[*<value>*] = *<value>* was outside the
            bounding box.

            Constraint: *<value>* <= `yevalv`[i-1] <= *<value>* for all
            i.

    See Also
    --------
    :meth:`naginterfaces.library.examples.fit.dim2_spline_ts_sctr_ex.main`
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_ts_evalv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02jeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nevalv.
        utils._EngineFloat64ArrayType.c_type, # Input, xevalv[nevalv].
        utils._EngineFloat64ArrayType.c_type, # Input, yevalv[nevalv].
        utils._BYREF_INT, # Input, lcoefs.
        utils._EngineFloat64ArrayType.c_type, # Input, coefs[lcoefs].
        utils._EngineFloat64ArrayType.c_type, # Output, fevalv[nevalv].
        utils._EngineIntArrayType.c_type, # Input, iopts[liopts].
        utils._EngineFloat64ArrayType.c_type, # Input, opts[lopts].
        utils._BYREF_INT, # Input, liopts.
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nevalv = utils._EngineIntScalarType(
        nevalv, {'fun_name': fun_name, 'entity': 'nevalv'},
    ).to_ctypes()
    _nevalv_py = utils._EngineIntScalarType.to_py(_nevalv)
    _xevalv = utils._EngineFloat64ArrayType(
        xevalv, {'fun_name': fun_name, 'entity': 'xevalv'},
        exp_shape=(_nevalv_py,),
    ).to_ctypes()
    _yevalv = utils._EngineFloat64ArrayType(
        yevalv, {'fun_name': fun_name, 'entity': 'yevalv'},
        exp_shape=(_nevalv_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('coefs', 'iopts', 'opts'),
    )
    _lcoefs_py = utils._EngineFloat64ArrayType.get_shape(comm['coefs'], {'fun_name': fun_name, 'entity': 'comm[\'coefs\']'}, exp_rank=1, validate=False)[0]
    _lcoefs = utils._EngineIntScalarType(
        _lcoefs_py, {'fun_name': fun_name, 'entity': '_lcoefs'},
    ).to_ctypes()
    _coefs = comm['coefs']
    _fevalv_cls = utils._EngineFloat64ArrayType(
        fevalv, {'fun_name': fun_name, 'entity': 'fevalv'},
        exp_shape=(_nevalv_py,),
        check_contents=False,
    )
    _fevalv = _fevalv_cls.to_ctypes(can_reference_contents=False)
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    _iopts = comm['iopts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _opts = comm['opts']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nevalv,
        _xevalv,
        _yevalv,
        _lcoefs,
        _coefs,
        _fevalv,
        _iopts,
        _opts,
        _liopts,
        _lopts,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_DATA_SIZE:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nevalv = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nevalv_py),
                    ),
                    'Constraint: nevalv >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_INIT:
            _ierr_fl = 9
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Option arrays are not initialized or are corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_COEF:
            _ierr_fl = 10
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The fitting routine has not been called, or the array of coefficients has',
                    'been corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_BOUNDS and _ifmt == _nag_e_consts.IFMT_TSFIT_X_BOUND:
            _ierr_fl = 13
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xevalv[{:s}] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'was outside the bounding box.',
                    'Constraint: {:s} <= xevalv[i-1] <= {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'for all i.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_BOUNDS and _ifmt == _nag_e_consts.IFMT_TSFIT_Y_BOUND:
            _ierr_fl = 14
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, yevalv[{:s}] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'was outside the bounding box.',
                    'Constraint: {:s} <= yevalv[i-1] <= {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'for all i.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fevalv_cls.refresh_py(_fevalv)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_spline_ts_evalm(nxeval, nyeval, xevalm, yevalm, comm, fevalm):
    u"""
    Evaluation at a mesh of points of a spline computed by
    ``dim2_spline_ts_sctr``.

    For full information please refer to the NAG Library document for
    e02jf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02jff.html

    Parameters
    ----------
    nxeval : int
        n_x, the number of values in the x direction forming the mesh on
        which the spline is to be evaluated.

    nyeval : int
        n_y, the number of values in the y direction forming the mesh on
        which the spline is to be evaluated.

    xevalm : float, shape (`nxeval`)
        The (x_i) values forming the mesh on which the spline is to be
        evaluated.

    yevalm : float, shape (`nyeval`)
        The (y_j) values forming the mesh on which the spline is to be
        evaluated.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by prior calls to
        ``dim2_spline_ts_sctr`` and ``opt_set``.

    fevalm : float, shape (`nxeval`, `nyeval`), modified in place
        On exit: if no exception or warning is raised on exit
        `fevalm`[i-1,j-1] contains the computed spline value at
        (x_i,y_j).

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `nxeval` = *<value>*.

            Constraint: `nxeval` >= 1.

        (`errno` 7)
            On entry, `nyeval` = *<value>*.

            Constraint: `nyeval` >= 1.

        (`errno` 9)
            Option arrays are not initialized or are corrupted.

        (`errno` 10)
            The fitting routine has not been called, or the array of
            coefficients has been corrupted.

        (`errno` 13)
            On entry, `xevalm`[*<value>*] = *<value>* was outside the
            bounding box.

            Constraint: *<value>* <= `xevalm`[i-1] <= *<value>* for all
            i.

        (`errno` 14)
            On entry, `yevalm`[*<value>*] = *<value>* was outside the
            bounding box.

            Constraint: *<value>* <= `yevalm`[j-1] <= *<value>* for all
            j.
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_ts_evalm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02jfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nxeval.
        utils._BYREF_INT, # Input, nyeval.
        utils._EngineFloat64ArrayType.c_type, # Input, xevalm[nxeval].
        utils._EngineFloat64ArrayType.c_type, # Input, yevalm[nyeval].
        utils._BYREF_INT, # Input, lcoefs.
        utils._EngineFloat64ArrayType.c_type, # Input, coefs[lcoefs].
        utils._EngineFloat64ArrayType.c_type, # Output, fevalm[nxeval, nyeval].
        utils._EngineIntArrayType.c_type, # Input, iopts[liopts].
        utils._EngineFloat64ArrayType.c_type, # Input, opts[lopts].
        utils._BYREF_INT, # Input, liopts.
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fevalm, 'fevalm'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nxeval = utils._EngineIntScalarType(
        nxeval, {'fun_name': fun_name, 'entity': 'nxeval'},
    ).to_ctypes()
    _nyeval = utils._EngineIntScalarType(
        nyeval, {'fun_name': fun_name, 'entity': 'nyeval'},
    ).to_ctypes()
    _nxeval_py = utils._EngineIntScalarType.to_py(_nxeval)
    _xevalm = utils._EngineFloat64ArrayType(
        xevalm, {'fun_name': fun_name, 'entity': 'xevalm'},
        exp_shape=(_nxeval_py,),
    ).to_ctypes()
    _nyeval_py = utils._EngineIntScalarType.to_py(_nyeval)
    _yevalm = utils._EngineFloat64ArrayType(
        yevalm, {'fun_name': fun_name, 'entity': 'yevalm'},
        exp_shape=(_nyeval_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('coefs', 'iopts', 'opts'),
    )
    _lcoefs_py = utils._EngineFloat64ArrayType.get_shape(comm['coefs'], {'fun_name': fun_name, 'entity': 'comm[\'coefs\']'}, exp_rank=1, validate=False)[0]
    _lcoefs = utils._EngineIntScalarType(
        _lcoefs_py, {'fun_name': fun_name, 'entity': '_lcoefs'},
    ).to_ctypes()
    _coefs = comm['coefs']
    _fevalm_cls = utils._EngineFloat64ArrayType(
        fevalm, {'fun_name': fun_name, 'entity': 'fevalm'},
        exp_shape=(_nxeval_py, _nyeval_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fevalm = _fevalm_cls.to_ctypes(can_reference_contents=False)
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    _iopts = comm['iopts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _opts = comm['opts']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nxeval,
        _nyeval,
        _xevalm,
        _yevalm,
        _lcoefs,
        _coefs,
        _fevalm,
        _iopts,
        _opts,
        _liopts,
        _lopts,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_GRIDX:
            _ierr_fl = 6
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nxeval = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nxeval_py),
                    ),
                    'Constraint: nxeval >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_CONSTRAINT and _ifmt == _nag_e_consts.IFMT_TSFIT_GRIDY:
            _ierr_fl = 7
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nyeval = {:s}.'.format(
                        utils._STR_FMT_INT.format(_nyeval_py),
                    ),
                    'Constraint: nyeval >= 1.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_INIT:
            _ierr_fl = 9
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Option arrays are not initialized or are corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_COEF:
            _ierr_fl = 10
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The fitting routine has not been called, or the array of coefficients has',
                    'been corrupted.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_BOUNDS and _ifmt == _nag_e_consts.IFMT_TSFIT_X_BOUND:
            _ierr_fl = 13
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xevalm[{:s}] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'was outside the bounding box.',
                    'Constraint: {:s} <= xevalm[i-1] <= {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'for all i.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_TSFIT_BOUNDS and _ifmt == _nag_e_consts.IFMT_TSFIT_Y_BOUND:
            _ierr_fl = 14
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, yevalm[{:s}] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'was outside the bounding box.',
                    'Constraint: {:s} <= yevalm[j-1] <= {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'for all j.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fevalm_cls.refresh_py(_fevalm)
        return_data = None
    _err_class.manage()
    return return_data

def pade_app(ia, ib, c, ic, a, b):
    u"""
    Padé approximants.

    For full information please refer to the NAG Library document for
    e02ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02raf.html

    Parameters
    ----------
    ia : int
        `ia` must specify l+1 and `ib` must specify m+1, where l and m
        are the degrees of the numerator and denominator of the
        approximant, respectively.

    ib : int
        `ia` must specify l+1 and `ib` must specify m+1, where l and m
        are the degrees of the numerator and denominator of the
        approximant, respectively.

    c : float, shape (`ic`)
        `c`[i-1] must specify, for i = 1,2,...,l+m+1, the coefficient of
        x^(i-1) in the given power series.

    ic : int
        The dimension of the array `c`.

    a : float, shape (`ia`), modified in place
        On exit: `a`[j], for j = 1,2,...,l+1, contains the coefficient
        a_j in the numerator of the approximant.

    b : float, shape (`ib`), modified in place
        On exit: `b`[k], for k = 1,2,...,m+1, contains the coefficient
        b_k in the denominator of the approximant.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ic` = *<value>*, `ia` = *<value>* and `ib` =
            *<value>*.

            Constraint: `ic` >= `ia`+`ib`-1.

        (`errno` 1)
            On entry, `ib` = *<value>*.

            Constraint: `ib` >= 1.

        (`errno` 1)
            On entry, `ia` = *<value>*.

            Constraint: `ia` >= 1.

        (`errno` 2)
            The Pade approximant is degenerate.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.pade_app'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, ia.
        utils._BYREF_INT, # Input, ib.
        utils._EngineFloat64ArrayType.c_type, # Input, c[ic].
        utils._BYREF_INT, # Input, ic.
        utils._EngineFloat64ArrayType.c_type, # Output, a[ia].
        utils._EngineFloat64ArrayType.c_type, # Output, b[ib].
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, jw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _ia = utils._EngineIntScalarType(
        ia, {'fun_name': fun_name, 'entity': 'ia'},
    ).to_ctypes()
    _ib = utils._EngineIntScalarType(
        ib, {'fun_name': fun_name, 'entity': 'ib'},
    ).to_ctypes()
    _ic = utils._EngineIntScalarType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
    ).to_ctypes()
    _ia_py = utils._EngineIntScalarType.to_py(_ia)
    _ib_py = utils._EngineIntScalarType.to_py(_ib)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_ia_py+_ib_py-1,),
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ia_py,),
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ib_py,),
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _jw = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _ia,
        _ib,
        _c,
        _ic,
        _a,
        _b,
        _w,
        _jw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ic = {:s}, ia = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ib = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ic >= ia+ib-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ib = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ib >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Pade approximant is degenerate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def pade_eval(a, ia, b, ib, x):
    u"""
    Evaluation of fitted rational function as computed by ``pade_app``.

    For full information please refer to the NAG Library document for
    e02rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02rbf.html

    Parameters
    ----------
    a : float, shape (`ia`)
        `a`[j], for j = 1,2,...,l+1, must contain the value of the
        coefficient a_j in the numerator of the rational function.

    ia : int
        The value of l+1, where l is the degree of the numerator.

    b : float, shape (`ib`)
        `b`[k], for k = 1,2,...,m+1, must contain the value of the
        coefficient b_k in the denominator of the rational function.

    ib : int
        The value of m+1, where m is the degree of the denominator.

    x : float
        The point x at which the rational function is to be evaluated.

    Returns
    -------
    ans : float
        The result of evaluating the rational function at the given
        point x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Evaluation at or near a pole.

        (`errno` 2)
            The first `ib` entries in `b` are zero: `ib` = *<value>*.

        (`errno` 2)
            On entry, `ib` = *<value>*.

            Constraint: `ib` >= 1.

        (`errno` 2)
            On entry, `ia` = *<value>*.

            Constraint: `ia` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.pade_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02rbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[ia].
        utils._BYREF_INT, # Input, ia.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ib].
        utils._BYREF_INT, # Input, ib.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, ans.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ia = utils._EngineIntScalarType(
        ia, {'fun_name': fun_name, 'entity': 'ia'},
    ).to_ctypes()
    _ia_py = utils._EngineIntScalarType.to_py(_ia)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ia_py,),
    ).to_ctypes()
    _ib = utils._EngineIntScalarType(
        ib, {'fun_name': fun_name, 'entity': 'ib'},
    ).to_ctypes()
    _ib_py = utils._EngineIntScalarType.to_py(_ib)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ib_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _ans = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _ia,
        _b,
        _ib,
        _x,
        _ans,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Evaluation at or near a pole.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The first ib entries in b are zero: ib = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ib = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ib >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ans_py = utils._EngineFloat64ScalarType.to_py(_ans)
        return_data = _ans_py
    _err_class.manage()
    return return_data

def dim2_spline_sort(px, py, lamda, mu, m, x, y, point, npoint):
    u"""
    Sort two-dimensional data into panels for fitting bicubic splines.

    For full information please refer to the NAG Library document for
    e02za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02zaf.html

    Parameters
    ----------
    px : int
        `px` and `py` must specify eight more than the number of
        intercepts on the x axis and y axis, respectively.

    py : int
        `px` and `py` must specify eight more than the number of
        intercepts on the x axis and y axis, respectively.

    lamda : float, shape (`px`)
        `lamda`[4] to `lamda`[`px`-5] must contain, in nondecreasing
        order, the intercepts on the x axis of the sides of the panels
        parallel to the y axis.

    mu : float, shape (`py`)
        `mu`[4] to `mu`[`py`-5] must contain, in nondecreasing order,
        the intercepts on the y axis of the sides of the panels parallel
        to the x axis.

    m : int
        The number m of data points.

    x : float, shape (`m`)
        The coordinates of the rth data point (x_r,y_r), for r =
        1,2,...,m.

    y : float, shape (`m`)
        The coordinates of the rth data point (x_r,y_r), for r =
        1,2,...,m.

    point : int, shape (`npoint`), modified in place
        On exit: for i = 1,2,...,`npoint`, `point`[m+i-1] = I1 is the
        index of the first point in panel i, `point`[I1-1] = I2 is the
        index of the second point in panel i and so on.

        `point`[In-1] = 0 indicates that `x`[In-1],`y`[In-1] was the
        last point in the panel.

        The coordinates of points in panel i can be accessed in turn by
        means of the following instructions:

        ::

          i_n = point[m+i-1]
          while i_n != 0:
              xi = x[in-1]
              yi = y[in-1]
              .
              .
              .
              i_n = point[i_n]
          ...

    npoint : int
        The dimension of the array `point`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, I = *<value>*, `mu`[I-1] = *<value>* and `mu`[I-2]
            = *<value>*.

            Constraint: `mu`[I-1] >= `mu`[I-2].

        (`errno` 1)
            On entry, I = *<value>*, `lamda`[I-1] = *<value>* and
            `lamda`[I-2] = *<value>*.

            Constraint: `lamda`[I-1] >= `lamda`[I-2].

        (`errno` 2)
            On entry, `npoint` = *<value>*, `m` = *<value>*, `px` =
            *<value>* and `py` = *<value>*.

            Constraint: `npoint` >= `m`+(`px`-7)*(`py`-7).

        (`errno` 2)
            On entry, `py` = *<value>*.

            Constraint: `py` >= 8.

        (`errno` 2)
            On entry, `px` = *<value>*.

            Constraint: `px` >= 8.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.fit.dim2_spline_sort'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, px.
        utils._BYREF_INT, # Input, py.
        utils._EngineFloat64ArrayType.c_type, # Input, lamda[px].
        utils._EngineFloat64ArrayType.c_type, # Input, mu[py].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineIntArrayType.c_type, # Output, point[npoint].
        utils._BYREF_INT, # Input, npoint.
        utils._EngineIntArrayType.c_type, # Workspace, adres.
        utils._BYREF_INT, # Input, nadres.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _px = utils._EngineIntScalarType(
        px, {'fun_name': fun_name, 'entity': 'px'},
    ).to_ctypes()
    _py = utils._EngineIntScalarType(
        py, {'fun_name': fun_name, 'entity': 'py'},
    ).to_ctypes()
    _px_py = utils._EngineIntScalarType.to_py(_px)
    _lamda = utils._EngineFloat64ArrayType(
        lamda, {'fun_name': fun_name, 'entity': 'lamda'},
        exp_shape=(_px_py,),
    ).to_ctypes()
    _py_py = utils._EngineIntScalarType.to_py(_py)
    _mu = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(_py_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _npoint = utils._EngineIntScalarType(
        npoint, {'fun_name': fun_name, 'entity': 'npoint'},
    ).to_ctypes()
    _point_cls = utils._EngineIntArrayType(
        point, {'fun_name': fun_name, 'entity': 'point'},
        exp_shape=(_m_py+(_px_py-7)*(_py_py-7),),
        check_contents=False,
    )
    _point = _point_cls.to_ctypes(can_reference_contents=False)
    _adres = utils._EngineIntArrayType.empty_ctypes(0)
    _nadres = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _px,
        _py,
        _lamda,
        _mu,
        _m,
        _x,
        _y,
        _point,
        _npoint,
        _adres,
        _nadres,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, mu[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'mu[I-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: mu[I-1] >= mu[I-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, lamda[I-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and lamda[I-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: lamda[I-1] >= lamda[I-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoint = {:s}, m = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'px = {:s} and py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: npoint >= m+(px-7)*(py-7).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, py = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: py >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, px = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: px >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _point_cls.refresh_py(_point)
        return_data = None
    _err_class.manage()
    return return_data

def opt_set(optstr, comm):
    u"""
    Option setting routine.

    For full information please refer to the NAG Library document for
    e02zk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02zkf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option to be set.

        Initialize = function name

          Initialize the option arrays `comm`['iopts'] and
          `comm`['opts'] for use with function function name, where
          function name is the short name of the problem solving
          function you wish to use.

        Defaults

          Resets all options to their default values.

        option = optval

          See [Optional Parameters] for details of valid values for
          option and optval. The equals sign (=) delimiter must be used
          to separate the option from its optval.

        The processing of `optstr` does not depend on its case.

        Each token in the option and optval component must be separated
        by at least one space.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 12)
            On entry, the expected delimiter '=' was not found in
            `optstr`: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to an
            integer: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to a real:
            `optstr` = *<value>*.

        (`errno` 14)
            On entry, attempting to initialize the option arrays but
            specified function name was not valid: name = *<value>*.

        (`errno` 15)
            On entry, the optval supplied for the integer option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 16)
            On entry, the optval supplied for the real option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 17)
            On entry, the optval supplied for the character option is
            not valid.

            `optstr` = *<value>*.

        (`errno` 21)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

    See Also
    --------
    :meth:`naginterfaces.library.examples.fit.dim2_spline_ts_sctr_ex.main`
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.opt_set'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02zkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, iopts[liopts].
        utils._BYREF_INT, # Input, liopts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, opts[lopts].
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iopts', 'opts'),
    )
    if _comm_needs_init:
        _iopts = utils._EngineIntArrayType.empty_ctypes(100)
        comm['iopts'] = _iopts
    else:
        _iopts = comm['iopts']
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    if _comm_needs_init:
        _opts = utils._EngineFloat64ArrayType.empty_ctypes(100)
        comm['opts'] = _opts
    else:
        _opts = comm['opts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _optstr,
        _iopts,
        _liopts,
        _opts,
        _lopts,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _optstr_py = utils._EngineCharScalarType.to_py(_optstr)
        if _ierr_py == _nag_e_consts.IERR_OPT_UNKNOWN and _ifmt == _nag_e_consts.IFMT_OPT_UNKNOWN:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the option in optstr was not recognized:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NO_EQUAL and _ifmt == _nag_e_consts.IFMT_OPT_NO_EQUAL:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the expected delimiter \'=\' was not found in optstr:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NON_NUMERIC and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, could not convert the specified optval to an integer:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NON_NUMERIC and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, could not convert the specified optval to a real:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_RNAME and _ifmt == _nag_e_consts.IFMT_OPT_RNAME:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, attempting to initialize the option arrays but',
                    'specified function name was not valid:',
                    'name = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_IVALUE and _ifmt == _nag_e_consts.IERR_OPT_IVALUE:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the integer option is not',
                    'valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_RVALUE and _ifmt == _nag_e_consts.IERR_OPT_RVALUE:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the real option is not valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_CVALUE and _ifmt == _nag_e_consts.IFMT_OPT_CVALUE_NOREC:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the character option is not',
                    'valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_NOT_INIT and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_INIT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_LINKED:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT and _ifmt == _nag_e_consts.IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def opt_get(optstr, comm):
    u"""
    Option getting routine.

    For full information please refer to the NAG Library document for
    e02zl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/e02/e02zlf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option whose current value is required.
        See [Optional Parameters] for information on valid options. In
        addition, the following is a valid option:

        Identify

          ``opt_get`` returns in `cvalue` the function name supplied to
          ``opt_set`` when the option arrays `comm`['iopts'] and
          `comm`['opts'] were initialized.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    Returns
    -------
    ivalue : int
        If the option supplied in `optstr` is an integer valued
        parameter, `ivalue` will hold its current value.

    rvalue : float
        If the option supplied in `optstr` is a real valued parameter,
        `rvalue` will hold its current value.

    cvalue : str
        If the option supplied in `optstr` is a character valued
        parameter, this argument will hold its current value, unless
        Identify is specified (see `optstr`).

    optype : int
        Indicates whether the option supplied in `optstr` is an integer,
        real or character valued parameter and hence which of `ivalue`,
        `rvalue` or `cvalue` holds the current value.

        `optype` = 1

          `optstr` is an integer valued option, its current value has
          been returned in `ivalue`.

        `optype` = 2

          `optstr` is a real valued option, its current value has been
          returned in `rvalue`.

        `optype` = 3

          `optstr` is a character valued option, its current value has
          been returned in `cvalue`.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 61)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.fit.opt_get'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('e02zlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._BYREF_INT, # Output, ivalue.
        utils._BYREF_FLOAT64, # Output, rvalue.
        utils._BYREF_STRING, # Output, cvalue(len=*).
        utils._BYREF_INT, # Output, optype.
        utils._EngineIntArrayType.c_type, # Input, iopts[liopts].
        utils._BYREF_INT, # Input, liopts.
        utils._EngineFloat64ArrayType.c_type, # Input, opts[lopts].
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cvalue.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    _ivalue = utils._EngineIntScalarType.empty_ctypes()
    _rvalue = utils._EngineFloat64ScalarType.empty_ctypes()
    _cvalue = utils._EngineCharScalarType.empty_ctypes(chrlen=39)
    _optype = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iopts', 'opts'),
    )
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    _iopts = comm['iopts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _opts = comm['opts']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _optstr,
        _ivalue,
        _rvalue,
        _cvalue,
        _optype,
        _iopts,
        _liopts,
        _opts,
        _lopts,
        _errbuf,
        _ierr,
        _optstr._length_,
        _cvalue._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _optstr_py = utils._EngineCharScalarType.to_py(_optstr)
        if _ierr_py == _nag_e_consts.IERR_OPT_UNKNOWN and _ifmt == _nag_e_consts.IFMT_OPT_UNKNOWN:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the option in optstr was not recognized:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ivalue_py = utils._EngineIntScalarType.to_py(_ivalue)
        _rvalue_py = utils._EngineFloat64ScalarType.to_py(_rvalue)
        _optype_py = utils._EngineIntScalarType.to_py(_optype)
        if _optype_py == _nag_e_consts.OPT_TYPE_IDENTIFY:
            _optype_py = _nag_e_consts.OPT_TYPE_CHAR
            if _ivalue_py == _nag_e_consts.TSFIT_SOLVER_ID:
                _cvalue_py = 'e02jd'
            else:
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        'The communication structure comm has either not been initialized,',
                        'has become corrupted, or is not compatible with this option',
                        'setting function.',
                    ],
                    errcodes=(_nag_e_consts.IERR_OPTS_CORRUPT_GETTER,),
                ).manage()
            _ivalue_py = 0
        else:
            _cvalue_py = utils._EngineCharScalarType.to_py(_cvalue)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ivalue', 'rvalue', 'cvalue', 'optype'],
        )(
            _ivalue_py,
            _rvalue_py,
            _cvalue_py,
            _optype_py,
        )
    _err_class.manage()
    return return_data
