# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
# pylint: disable=simplifiable-if-statement
u"""
Interfaces for the NAG Mark 27.0 `lapackeig` Chapter.

``lapackeig`` - Least Squares and Eigenvalue Problems (LAPACK)

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for the solution of linear least squares
problems, eigenvalue problems and singular value problems, as well as
associated computations.
It provides functions for:

- solution of linear least squares problems;

- solution of symmetric eigenvalue problems;

- solution of nonsymmetric eigenvalue problems;

- solution of singular value problems;

- solution of generalized linear least squares problems;

- solution of generalized symmetric-definite eigenvalue problems;

- solution of generalized nonsymmetric eigenvalue problems;

- solution of generalized singular value problems;

- matrix factorizations associated with the above problems;

- estimating condition numbers of eigenvalue and eigenvector problems;

- estimating the numerical rank of a matrix;

- solution of the Sylvester matrix equation.

Functions are provided for both real and complex data.

For a general introduction to the solution of linear least squares
problems, you should turn first to submodule ``linsys``.
The decision trees, at the end of submodule ``linsys``, direct you to
the most appropriate functions in submodule ``linsys`` or submodule
``lapackeig``. Submodule ``linsys`` and submodule ``lapackeig`` contain
Black Box (or driver) functions which enable standard linear least
squares problems to be solved by a call to a single function.

For a general introduction to eigenvalue and singular value problems,
you should turn first to submodule ``eigen``.
The decision trees, at the end of submodule ``eigen``, direct you to the
most appropriate functions in submodule ``eigen`` or submodule
``lapackeig``. Submodule ``eigen`` and submodule ``lapackeig`` contain
Black Box (or driver) functions which enable standard types of problem
to be solved by a call to a single function.
Often functions in submodule ``eigen`` call submodule ``lapackeig``
functions to perform the necessary computational tasks.

The functions in this module (submodule ``lapackeig``) handle only
dense, band, tridiagonal and Hessenberg matrices (not matrices with more
specialised structures, or general sparse matrices).
The tables in [Available Functions] and the decision trees in [Decision
Trees] direct you to the most appropriate functions in submodule
``lapackeig``.

The functions in this module have all been derived from the LAPACK
project.
They have been designed to be efficient on a wide range of
high-performance computers, without compromising efficiency on
conventional serial machines.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dgels(trans, m, n, nrhs, a, b):
    u"""
    Solves a real linear least squares problem of full rank.

    For full information please refer to the NAG Library document for
    f08aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08aaf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', the linear system involves A.

        If `trans` = 'T', the linear system involves A^T.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `m` >= `n`, `a` is overwritten by details of its QR
        factorization, as returned by ``dgeqrf``.

        If `m` < `n`, `a` is overwritten by details of its LQ
        factorization, as returned by ``dgelqf``.

    b : float, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the matrix B of right-hand side vectors, stored in
        columns; `b` is m by r if `trans` = 'N', or n by r if `trans` =
        'T'.

        On exit: `b` is overwritten by the solution vectors, x, stored
        in columns:

        - if `trans` = 'N' and m >= n, or `trans` = 'T' and m < n,
          elements 1 to min(m, n) in each column of `b` contain the
          least squares solution vectors; the residual sum of squares
          for the solution is given by the sum of squares of the modulus
          of elements (min(m, n)+1) to max(m, n) in that column;

        - otherwise, elements 1 to max(m, n) in each column of `b`
          contain the minimum norm solution vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the triangular factor of A is
            zero, so that A does not have full rank; the least squares
            solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgels'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _work,
        _lwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the triangular factor of A is'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'zero, so that A does not have full rank; the least squares solution could',
                    'not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgeqrt(m, n, nb, a, t):
    u"""
    Performs a QR factorization of real general rectangular matrix, with
    explicit blocking.

    For full information please refer to the NAG Library document for
    f08ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08abf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nb : int
        The explicitly chosen block size to be used in computing the QR
        factorization. See [Further Comments] for details.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the orthogonal matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m by n
        upper trapezoidal matrix R.

    t : float, shape (`nb`, min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix Q. The number
        of blocks is b = ⌈k/`nb`⌉, where k = min(m, n) and each block is
        of order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nb`.

            Constraint: 1 <= `nb` <= min(`m`, `n`).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, t[ldt, min(m, n)].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (t, 't')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if min(_m_py, _n_py) > 0:
        if not (_nb_py >= 1 and _nb_py <= min(_m_py, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= nb and nb <= min(m, n).'),
                ],
                errcodes=(-3,),
                entity='nb',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldt_py = utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, min(_m_py, _n_py)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _t = _t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nb,
        _a,
        _lda,
        _t,
        _ldt,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _t_cls.refresh_py(_t)
        return_data = None
    _err_class.manage()
    return return_data

def dgemqrt(side, trans, m, n, k, nb, v, t, c):
    u"""
    Applies the orthogonal transformation determined by ``dgeqrt``.

    For full information please refer to the NAG Library document for
    f08ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08acf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q. Usually `k` = min(m_A, n_A) where m_A, n_A are the
        dimensions of the matrix A supplied in a previous call to
        ``dgeqrt``.

    nb : int
        The block size used in the QR factorization performed in a
        previous call to ``dgeqrt``; this value must remain unchanged
        from that call.

    v : float, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqrt`` in the first k columns of its array
        argument a.

    t : float, shape (`nb`, `k`)
        Further details of the orthogonal matrix Q as returned by
        ``dgeqrt``. The number of blocks is b = ⌈k/`nb`⌉, where k =
        min(m, n) and each block is of order `nb` except for the last
        block, which is of order k-(b-1)*`nb`. For the b blocks the
        upper triangular block reflector factors T_1,T_2,...,T_b are
        stored in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -6)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `k`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgemqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, k].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input, t[ldt, k].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'), (t, 't'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-6,),
            entity='nb',
        ).manage()
    if _k_py > 0:
        if not _nb_py <= _k_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= k.'),
                ],
                errcodes=(-6,),
                entity='nb',
            ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _v_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 0
    exp_shape_v = (_v_dim1, _k_py)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _ldt_py = utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _nb,
        _v,
        _ldv,
        _t,
        _ldt,
        _c,
        _ldc,
        _work,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgeqrf(m, n, a, tau):
    u"""
    Performs a QR factorization of real general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08aef.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the orthogonal matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m by n
        upper trapezoidal matrix R.

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeqrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorgqr(m, n, k, a, tau):
    u"""
    Forms all or part of orthogonal Q from QR factorization determined
    by ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

    For full information please refer to the NAG Library document for
    f08af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08aff.html

    Parameters
    ----------
    m : int
        m, the order of the orthogonal matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

        On exit: the m by n matrix Q.

    tau : float, shape (max(1, `k`))
        Further details of the elementary reflectors, as returned by
        ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorgqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[max(1, k)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_n_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(max(1, _k_py),),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormqr(side, trans, m, n, k, a, tau, c):
    u"""
    Applies an orthogonal transformation determined by ``dgeqrf``,
    ``dgeqpf`` or ``dgeqp3``.

    For full information please refer to the NAG Library document for
    f08ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08agf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

    tau : float, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, k].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    exp_shape_a = (_a_dim1, _k_py)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgelqf(m, n, a, tau):
    u"""
    Performs a LQ factorization of real general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ahf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the elements above the diagonal are
        overwritten by details of the orthogonal matrix Q and the lower
        triangle is overwritten by the corresponding elements of the m
        by m lower triangular matrix L.

        If m > n, the strictly upper triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m by n
        lower trapezoidal matrix L.

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgelqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorglq(m, n, k, a, tau):
    u"""
    Forms all or part of orthogonal Q from LQ factorization determined
    by ``dgelqf``.

    For full information please refer to the NAG Library document for
    f08aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ajf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgelqf``.

        On exit: the m by n matrix Q.

    tau : float, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``dgelqf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorglq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_m_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormlq(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgelqf``.

    For full information please refer to the NAG Library document for
    f08ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08akf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgelqf``.

    tau : float, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``dgelqf``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormlq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08akft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgels(trans, m, n, nrhs, a, b):
    u"""
    Solves a complex linear least problem of full rank.

    For full information please refer to the NAG Library document for
    f08an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08anf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', the linear system involves A.

        If `trans` = 'C', the linear system involves A^H.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `m` >= `n`, `a` is overwritten by details of its QR
        factorization, as returned by ``zgeqrf``.

        If `m` < `n`, `a` is overwritten by details of its LQ
        factorization, as returned by ``zgelqf``.

    b : complex, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the matrix B of right-hand side vectors, stored in
        columns; `b` is m by r if `trans` = 'N', or n by r if `trans` =
        'C'.

        On exit: `b` is overwritten by the solution vectors, x, stored
        in columns:

        - if `trans` = 'N' and m >= n, or `trans` = 'C' and m < n,
          elements 1 to min(m, n) in each column of `b` contain the
          least squares solution vectors; the residual sum of squares
          for the solution is given by the sum of squares of the modulus
          of elements (min(m, n)+1) to max(m, n) in that column;

        - otherwise, elements 1 to max(m, n) in each column of `b`
          contain the minimum norm solution vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            Diagonal element *<value>* of the triangular factor of A is
            zero, so that A does not have full rank; the least squares
            solution could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgels'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-4,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _work,
        _lwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the triangular factor of A is'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'zero, so that A does not have full rank; the least squares solution could',
                    'not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def zgeqrt(m, n, nb, a, t):
    u"""
    Performs a QR factorization of complex general rectangular matrix
    using recursive algorithm.

    For full information please refer to the NAG Library document for
    f08ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08apf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nb : int
        The explicitly chosen block size to be used in computing the QR
        factorization. See [Further Comments] for details.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the unitary matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m by n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    t : complex, shape (`nb`, min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q. The number of
        blocks is b = ⌈k/`nb`⌉, where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nb`.

            Constraint: 1 <= `nb` <= min(`m`, `n`).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, t[ldt, min(m, n)].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (t, 't')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if min(_m_py, _n_py) > 0:
        if not (_nb_py >= 1 and _nb_py <= min(_m_py, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= nb and nb <= min(m, n).'),
                ],
                errcodes=(-3,),
                entity='nb',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldt_py = utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, min(_m_py, _n_py)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _t = _t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nb,
        _a,
        _lda,
        _t,
        _ldt,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _t_cls.refresh_py(_t)
        return_data = None
    _err_class.manage()
    return return_data

def zgemqrt(side, trans, m, n, k, nb, v, t, c):
    u"""
    Applies the unitary transformation determined by ``zgeqrt``.

    For full information please refer to the NAG Library document for
    f08aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08aqf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q. Usually `k` = min(m_A, n_A) where m_A, n_A are the
        dimensions of the matrix A supplied in a previous call to
        ``zgeqrt``.

    nb : int
        The block size used in the QR factorization performed in a
        previous call to ``zgeqrt``; this value must remain unchanged
        from that call.

    v : complex, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqrt`` in the first k columns of its array
        argument a.

    t : complex, shape (`nb`, `k`)
        Further details of the unitary matrix Q as returned by
        ``zgeqrt``. The number of blocks is b = ⌈k/`nb`⌉, where k =
        min(m, n) and each block is of order `nb` except for the last
        block, which is of order k-(b-1)*`nb`. For the b blocks the
        upper triangular block reflector factors T_1,T_2,...,T_b are
        stored in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -6)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `k`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgemqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, nb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, k].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input, t[ldt, k].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'), (t, 't'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-6,),
            entity='nb',
        ).manage()
    if _k_py > 0:
        if not _nb_py <= _k_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= k.'),
                ],
                errcodes=(-6,),
                entity='nb',
            ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _v_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 0
    exp_shape_v = (_v_dim1, _k_py)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _ldt_py = utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _nb,
        _v,
        _ldv,
        _t,
        _ldt,
        _c,
        _ldc,
        _work,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgeqrf(m, n, a, tau):
    u"""
    Performs a QR factorization of complex general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08asf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the unitary matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m by n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeqrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zungqr(m, n, k, a, tau):
    u"""
    Forms all or part of unitary Q from QR factorization determined by
    ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

    For full information please refer to the NAG Library document for
    f08at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08atf.html

    Parameters
    ----------
    m : int
        m, the order of the unitary matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

        On exit: the m by n matrix Q.

    tau : complex, shape (max(1, `k`))
        Further details of the elementary reflectors, as returned by
        ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zungqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[max(1, k)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_n_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(max(1, _k_py),),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmqr(side, trans, m, n, k, a, tau, c):
    u"""
    Applies a unitary transformation determined by ``zgeqrf``,
    ``zgeqpf`` or ``zgeqp3``.

    For full information please refer to the NAG Library document for
    f08au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08auf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

    tau : complex, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, k].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    exp_shape_a = (_a_dim1, _k_py)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgelqf(m, n, a, tau):
    u"""
    Performs a LQ factorization of complex general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08avf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the elements above the diagonal are
        overwritten by details of the unitary matrix Q and the lower
        triangle is overwritten by the corresponding elements of the m
        by m lower triangular matrix L.

        If m > n, the strictly upper triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m by n lower
        trapezoidal matrix L.

        The diagonal elements of L are real.

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgelqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08avft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zunglq(m, n, k, a, tau):
    u"""
    Forms all or part of unitary Q from LQ factorization determined by
    ``zgelqf``.

    For full information please refer to the NAG Library document for
    f08aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08awf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgelqf``.

        On exit: the m by n matrix Q.

    tau : complex, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``zgelqf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunglq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08awft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_m_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmlq(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgelqf``.

    For full information please refer to the NAG Library document for
    f08ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08axf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgelqf``.

    tau : complex, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``zgelqf``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmlq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08axft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgelsy(m, n, nrhs, a, b, jpvt, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem.

    For full information please refer to the NAG Library document for
    f08ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08baf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` has been overwritten by details of its complete
        orthogonal factorization.

    b : float, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[i-1] != 0, the ith column of A is permuted
        to the front of AP, otherwise column i is a free column.

        On exit: if `jpvt`[i-1] = k, the ith column of AP was the kth
        column of A.

    rcond : float
        Used to determine the effective rank of A, which is defined as
        the order of the largest leading triangular sub-matrix R_11 in
        the QR factorization of A, whose estimated condition number is <
        1/`rcond`.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the order of the sub-matrix R_11.
        This is the same as the order of the sub-matrix T_11 in the
        complete orthogonal factorization of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgelsy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _jpvt,
        _rcond,
        _rank,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _jpvt_cls.refresh_py(_jpvt)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def dtpqrt(m, n, l, nb, a, b, t):
    u"""
    QR factorization of real general triangular-pentagonal matrix.

    For full information please refer to the NAG Library document for
    f08bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bbf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrix B and the order of the
        upper triangular matrix A.

    l : int
        l, the number of rows of the trapezoidal part of B (i.e., B_2).

    nb : int
        The explicitly chosen block-size to be used in the algorithm for
        computing the QR factorization. See [Further Comments] for
        details.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix A.

        On exit: the upper triangle is overwritten by the corresponding
        elements of the n by n upper triangular matrix R.

    b : float, shape (`m`, `n`), modified in place
        On entry: the m by n pentagonal matrix B composed of an (m-l) by
        n rectangular matrix B_1 above an l by n upper trapezoidal
        matrix B_2.

        On exit: details of the orthogonal matrix Q.

    t : float, shape (`nb`, `n`), modified in place
        On exit: further details of the orthogonal matrix Q. The number
        of blocks is b = ⌈k/`nb`⌉, where k = min(m, n) and each block is
        of order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= min(`m`, `n`).

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtpqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (t, 't')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if not (_l_py >= 0 and _l_py <= min(_m_py, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= l and l <= min(m, n).'),
            ],
            errcodes=(-3,),
            entity='l',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-4,),
            entity='nb',
        ).manage()
    if _n_py > 0:
        if not _nb_py <= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= n.'),
                ],
                errcodes=(-4,),
                entity='nb',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldt_py = utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _t = _t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _l,
        _nb,
        _a,
        _lda,
        _b,
        _ldb,
        _t,
        _ldt,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _t_cls.refresh_py(_t)
        return_data = None
    _err_class.manage()
    return return_data

def dtpmqrt(side, trans, m, n, k, l, nb, v, t, c1, c2):
    u"""
    Applies the orthogonal transformation determined by ``dtpqrt``.

    For full information please refer to the NAG Library document for
    f08bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bcf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        The number of rows of the matrix C_2, that is,

        if `side` = 'L'

          then m_v, the number of rows of the matrix V;

        if `side` = 'R'

          then m_c, the number of rows of the matrix C.

    n : int
        The number of columns of the matrix C_2, that is,

        if `side` = 'L'

          then n_c, the number of columns of the matrix C;

        if `side` = 'R'

          then n_v, the number of columns of the matrix V.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    l : int
        l, the number of rows of the upper trapezoidal part of the
        pentagonal composite matrix V, passed (as b) in a previous call
        to ``dtpqrt``. This must be the same value used in the previous
        call to ``dtpqrt`` (see l in ``dtpqrt``).

    nb : int
        nb, the blocking factor used in a previous call to ``dtpqrt`` to
        compute the QR factorization of a triangular-pentagonal matrix
        containing composite matrices A and B.

    v : float, shape (:, `k`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        The m_v by n_v matrix V; this should remain unchanged from the
        array b returned by a previous call to ``dtpqrt``.

    t : float, shape (`nb`, `k`)
        This must remain unchanged from a previous call to ``dtpqrt``
        (see t in ``dtpqrt``).

    c1 : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `k`; if `side`='R': `m`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `n`; if `side`='R': `k`;
        otherwise: 0.

        On entry: C_1, the first part of the composite matrix C:

        if `side` = 'L'

          then `c1` contains the first k rows of C;

        if `side` = 'R'

          then `c1` contains the first k columns of C.

        On exit: `c1` is overwritten by the corresponding block of QC or
        Q^TC or CQ or CQ^T.

    c2 : float, shape (`m`, `n`), modified in place
        On entry: C_2, the second part of the composite matrix C.

        if `side` = 'L'

          then `c2` contains the remaining m_v rows of C;

        if `side` = 'R'

          then `c2` contains the remaining m_v columns of C;

        On exit: `c2` is overwritten by the corresponding block of QC or
        Q^TC or CQ or CQ^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= `k`.

        (`errno` -7)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -7)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `k`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtpmqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input, v[ldv, k].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input, t[ldt, k].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c1[ldc1, *].
        utils._BYREF_INT, # Input, ldc1.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c2[ldc2, n].
        utils._BYREF_INT, # Input, ldc2.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'), (t, 't'), (c1, 'c1'), (c2, 'c2')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-5,),
            entity='k',
        ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if not (_l_py >= 0 and _l_py <= _k_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= l and l <= k.'),
            ],
            errcodes=(-6,),
            entity='l',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-7,),
            entity='nb',
        ).manage()
    if _k_py > 0:
        if not _nb_py <= _k_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= k.'),
                ],
                errcodes=(-7,),
                entity='nb',
            ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _v_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 0
    exp_shape_v = (_v_dim1, _k_py)
    _v = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldt_py = utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc1 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c1, {'fun_name': fun_name, 'entity': 'c1'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc1'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _c1_dim1 = _k_py
    elif _side_py in ('R', 'r'):
        _c1_dim1 = _m_py
    else:
        _c1_dim1 = 0
    if _side_py in ('L', 'l'):
        _c1_dim2 = _n_py
    elif _side_py in ('R', 'r'):
        _c1_dim2 = _k_py
    else:
        _c1_dim2 = 0
    exp_shape_c1 = (_c1_dim1, _c1_dim2)
    _c1_cls = utils._EngineFloat64ArrayType(
        c1, {'fun_name': fun_name, 'entity': 'c1'},
        exp_shape=exp_shape_c1,
        mutual_sorder=_m_sorder,
    )
    _c1 = _c1_cls.to_ctypes()
    _ldc2 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc2'},
    ).to_ctypes()
    _c2_cls = utils._EngineFloat64ArrayType(
        c2, {'fun_name': fun_name, 'entity': 'c2'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c2 = _c2_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _l,
        _nb,
        _v,
        _ldv,
        _t,
        _ldt,
        _c1,
        _ldc1,
        _c2,
        _ldc2,
        _work,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c1_cls.refresh_py(_c1)
        _c2_cls.refresh_py(_c2)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dgeqpf',
    mkwdrn=None,
    advice=[
        'Please use dgeqp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgeqpf(m, n, a, jpvt, tau):
    u"""
    QR factorization, with column pivoting, of real general rectangular
    matrix.

    Note: ``dgeqpf`` is deprecated.
    Please use ``dgeqp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bef.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the orthogonal matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m by n
        upper trapezoidal matrix R.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[i-1] != 0, the i th column of A is moved to
        the beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the i th
        column of A is a free column.

        On exit: details of the permutation matrix P. More precisely, if
        `jpvt`[i-1] = k, the kth column of A is moved to become the i th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeqpf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _jpvt,
        _tau,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _jpvt_cls.refresh_py(_jpvt)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dgeqp3(m, n, a, jpvt, tau):
    u"""
    QR factorization, with column pivoting, using BLAS-3, of real
    general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bff.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the orthogonal matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m by n
        upper trapezoidal matrix R.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[j-1] != 0, the j th column of A is moved to
        the beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the j th
        column of A is a free column.

        On exit: details of the permutation matrix P. More precisely, if
        `jpvt`[j-1] = k, the kth column of A is moved to become the j th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeqp3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _jpvt,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _jpvt_cls.refresh_py(_jpvt)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dtzrzf(m, n, a, tau):
    u"""
    Reduces a real upper trapezoidal matrix to upper triangular form.

    For full information please refer to the NAG Library document for
    f08bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bhf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the leading m by n upper trapezoidal part of the array
        `a` must contain the matrix to be factorized.

        On exit: the leading m by m upper triangular part of `a`
        contains the upper triangular matrix R, and elements `m`+1 to
        `n` of the first m rows of `a`, with the array `tau`, represent
        the orthogonal matrix Z as a product of m elementary reflectors
        (see [the F08 Introduction]).

    tau : float, shape (`m`), modified in place
        On exit: the scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtzrzf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[m].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dormrz(side, trans, m, n, k, l, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dtzrzf``.

    For full information please refer to the NAG Library document for
    f08bk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bkf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Z or Z^T is to be applied to C.

        `side` = 'L'

          Z or Z^T is applied to C from the left.

        `side` = 'R'

          Z or Z^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Z or Z^T is to be applied to C.

        `trans` = 'N'

          Z is applied to C.

        `trans` = 'T'

          Z^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Z.

    l : int
        l, the number of columns of the matrix A containing the
        meaningful part of the Householder reflectors.

    a : float, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``dtzrzf``.

    tau : float, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dtzrzf``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by ZC or Z^TC or CZ or Z^TC as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: `m` >= `l` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: `n` >= `l` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormrz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _l_py and _l_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= l and l >= 0.'),
                ],
                errcodes=(-6,),
                entity='l',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _l_py and _l_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= l and l >= 0.'),
                ],
                errcodes=(-6,),
                entity='l',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _l,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgelsy(m, n, nrhs, a, b, jpvt, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem.

    For full information please refer to the NAG Library document for
    f08bn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bnf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` has been overwritten by details of its complete
        orthogonal factorization.

    b : complex, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: the n by r solution matrix X.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[i-1] != 0, the ith column of A is permuted
        to the front of AP, otherwise column i is a free column.

        On exit: if `jpvt`[i-1] = k, the ith column of AP was the kth
        column of A.

    rcond : float
        Used to determine the effective rank of A, which is defined as
        the order of the largest leading triangular sub-matrix R_11 in
        the QR factorization of A, whose estimated condition number is <
        1/`rcond`.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the order of the sub-matrix R_11.
        This is the same as the order of the sub-matrix T_11 in the
        complete orthogonal factorization of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgelsy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _jpvt,
        _rcond,
        _rank,
        _work,
        _lwork,
        _rwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _jpvt_cls.refresh_py(_jpvt)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def ztpqrt(m, n, l, nb, a, b, t):
    u"""
    QR factorization of complex triangular-pentagonal matrix.

    For full information please refer to the NAG Library document for
    f08bp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bpf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrix B and the order of the
        upper triangular matrix A.

    l : int
        l, the number of rows of the trapezoidal part of B (i.e., B_2).

    nb : int
        The explicitly chosen block-size to be used in the algorithm for
        computing the QR factorization. See [Further Comments] for
        details.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix A.

        On exit: the upper triangle is overwritten by the corresponding
        elements of the n by n upper triangular matrix R.

    b : complex, shape (`m`, `n`), modified in place
        On entry: the m by n pentagonal matrix B composed of an (m-l) by
        n rectangular matrix B_1 above an l by n upper trapezoidal
        matrix B_2.

        On exit: details of the unitary matrix Q.

    t : complex, shape (`nb`, `n`), modified in place
        On exit: further details of the unitary matrix Q. The number of
        blocks is b = ⌈k/`nb`⌉, where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb` by n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= min(`m`, `n`).

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztpqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, nb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (t, 't')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if not (_l_py >= 0 and _l_py <= min(_m_py, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= l and l <= min(m, n).'),
            ],
            errcodes=(-3,),
            entity='l',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-4,),
            entity='nb',
        ).manage()
    if _n_py > 0:
        if not _nb_py <= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= n.'),
                ],
                errcodes=(-4,),
                entity='nb',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldt_py = utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _t = _t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _l,
        _nb,
        _a,
        _lda,
        _b,
        _ldb,
        _t,
        _ldt,
        _work,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _t_cls.refresh_py(_t)
        return_data = None
    _err_class.manage()
    return return_data

def ztpmqrt(side, trans, m, n, k, l, nb, v, t, c1, c2):
    u"""
    Applies the unitary transformation determined by ``ztpqrt``.

    For full information please refer to the NAG Library document for
    f08bq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bqf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        The number of rows of the matrix C_2, that is,

        if `side` = 'L'

          then m_v, the number of rows of the matrix V;

        if `side` = 'R'

          then m_c, the number of rows of the matrix C.

    n : int
        The number of columns of the matrix C_2, that is,

        if `side` = 'L'

          then n_c, the number of columns of the matrix C;

        if `side` = 'R'

          then n_v, the number of columns of the matrix V.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    l : int
        l, the number of rows of the upper trapezoidal part of the
        pentagonal composite matrix V, passed (as b) in a previous call
        to ``ztpqrt``. This must be the same value used in the previous
        call to ``ztpqrt`` (see l in ``ztpqrt``).

    nb : int
        nb, the blocking factor used in a previous call to ``ztpqrt`` to
        compute the QR factorization of a triangular-pentagonal matrix
        containing composite matrices A and B.

    v : complex, shape (:, `k`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        The m_v by n_v matrix V; this should remain unchanged from the
        array b returned by a previous call to ``ztpqrt``.

    t : complex, shape (`nb`, `k`)
        This must remain unchanged from a previous call to ``ztpqrt``
        (see t in ``ztpqrt``).

    c1 : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `k`; if `side`='R': `m`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `n`; if `side`='R': `k`;
        otherwise: 0.

        On entry: C_1, the first part of the composite matrix C:

        if `side` = 'L'

          then `c1` contains the first k rows of C;

        if `side` = 'R'

          then `c1` contains the first k columns of C.

        On exit: `c1` is overwritten by the corresponding block of QC or
        Q^HC or CQ or CQ^H.

    c2 : complex, shape (`m`, `n`), modified in place
        On entry: C_2, the second part of the composite matrix C.

        if `side` = 'L'

          then `c2` contains the remaining m_v rows of C;

        if `side` = 'R'

          then `c2` contains the remaining m_v columns of C;

        On exit: `c2` is overwritten by the corresponding block of QC or
        Q^HC or CQ or CQ^H.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= `k`.

        (`errno` -7)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -7)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= `k`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztpmqrt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, nb.
        utils._EngineComplex128ArrayType.c_type, # Input, v[ldv, k].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input, t[ldt, k].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c1[ldc1, *].
        utils._BYREF_INT, # Input, ldc1.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c2[ldc2, n].
        utils._BYREF_INT, # Input, ldc2.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'), (t, 't'), (c1, 'c1'), (c2, 'c2')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-5,),
            entity='k',
        ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if not (_l_py >= 0 and _l_py <= _k_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= l and l <= k.'),
            ],
            errcodes=(-6,),
            entity='l',
        ).manage()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if not _nb_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nb >= 1.'),
            ],
            errcodes=(-7,),
            entity='nb',
        ).manage()
    if _k_py > 0:
        if not _nb_py <= _k_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('nb <= k.'),
                ],
                errcodes=(-7,),
                entity='nb',
            ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _v_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 0
    exp_shape_v = (_v_dim1, _k_py)
    _v = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldt_py = utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldt = utils._EngineIntScalarType(
        _ldt_py, {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_nb_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc1 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c1, {'fun_name': fun_name, 'entity': 'c1'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc1'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _c1_dim1 = _k_py
    elif _side_py in ('R', 'r'):
        _c1_dim1 = _m_py
    else:
        _c1_dim1 = 0
    if _side_py in ('L', 'l'):
        _c1_dim2 = _n_py
    elif _side_py in ('R', 'r'):
        _c1_dim2 = _k_py
    else:
        _c1_dim2 = 0
    exp_shape_c1 = (_c1_dim1, _c1_dim2)
    _c1_cls = utils._EngineComplex128ArrayType(
        c1, {'fun_name': fun_name, 'entity': 'c1'},
        exp_shape=exp_shape_c1,
        mutual_sorder=_m_sorder,
    )
    _c1 = _c1_cls.to_ctypes()
    _ldc2 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc2'},
    ).to_ctypes()
    _c2_cls = utils._EngineComplex128ArrayType(
        c2, {'fun_name': fun_name, 'entity': 'c2'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c2 = _c2_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _l,
        _nb,
        _v,
        _ldv,
        _t,
        _ldt,
        _c1,
        _ldc1,
        _c2,
        _ldc2,
        _work,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c1_cls.refresh_py(_c1)
        _c2_cls.refresh_py(_c2)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zgeqpf',
    mkwdrn=None,
    advice=[
        'Please use zgeqp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgeqpf(m, n, a, jpvt, tau):
    u"""
    QR factorization, with column pivoting, of complex general
    rectangular matrix.

    Note: ``zgeqpf`` is deprecated.
    Please use ``zgeqp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08bs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bsf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the unitary matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m by n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[i-1] != 0, the i th column of A is moved to
        the beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the i th
        column of A is a free column.

        On exit: details of the permutation matrix P. More precisely, if
        `jpvt`[i-1] = k, the kth column of A is moved to become the i th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeqpf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _jpvt,
        _tau,
        _work,
        _rwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _jpvt_cls.refresh_py(_jpvt)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zgeqp3(m, n, a, jpvt, tau):
    u"""
    QR factorization, with column pivoting, using BLAS-3, of complex
    general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08bt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08btf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the elements below the diagonal are
        overwritten by details of the unitary matrix Q and the upper
        triangle is overwritten by the corresponding elements of the n
        by n upper triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m by n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    jpvt : int, shape (`n`), modified in place
        On entry: if `jpvt`[j-1] != 0, the j th column of A is moved to
        the beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the j th
        column of A is a free column.

        On exit: details of the permutation matrix P. More precisely, if
        `jpvt`[j-1] = k, the kth column of A is moved to become the j th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeqp3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08btft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Input/Output, jpvt[n].
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _jpvt_cls = utils._EngineIntArrayType(
        jpvt, {'fun_name': fun_name, 'entity': 'jpvt'},
        exp_shape=(_n_py,),
    )
    _jpvt = _jpvt_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _jpvt,
        _tau,
        _work,
        _lwork,
        _rwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _jpvt_cls.refresh_py(_jpvt)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def ztzrzf(m, n, a, tau):
    u"""
    Reduces a complex upper trapezoidal matrix to upper triangular form.

    For full information please refer to the NAG Library document for
    f08bv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bvf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the leading m by n upper trapezoidal part of the array
        `a` must contain the matrix to be factorized.

        On exit: the leading m by m upper triangular part of `a`
        contains the upper triangular matrix R, and elements `m`+1 to
        `n` of the first m rows of `a`, with the array `tau`, represent
        the unitary matrix Z as a product of m elementary reflectors
        (see [the F08 Introduction]).

    tau : complex, shape (`m`), modified in place
        On exit: the scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztzrzf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[m].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zunmrz(side, trans, m, n, k, l, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``ztzrzf``.

    For full information please refer to the NAG Library document for
    f08bx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08bxf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Z or Z^H is to be applied to C.

        `side` = 'L'

          Z or Z^H is applied to C from the left.

        `side` = 'R'

          Z or Z^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Z or Z^H is to be applied to C.

        `trans` = 'N'

          Z is applied to C.

        `trans` = 'C'

          Z^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Z.

    l : int
        l, the number of columns of the matrix A containing the
        meaningful part of the Householder reflectors.

    a : complex, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``ztzrzf``.

    tau : complex, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``ztzrzf``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by ZC or Z^HC or CZ or Z^HC as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: `m` >= `l` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: `n` >= `l` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmrz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08bxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _l_py and _l_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= l and l >= 0.'),
                ],
                errcodes=(-6,),
                entity='l',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _l_py and _l_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= l and l >= 0.'),
                ],
                errcodes=(-6,),
                entity='l',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _l,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgeqlf(m, n, a, tau):
    u"""
    QL factorization of real general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cef.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the lower triangle of the subarray
        `a`[m-n:m,0:n] contains the n by n lower triangular matrix L.

        If m <= n, the elements on and below the (n-m)th superdiagonal
        contain the m by n lower trapezoidal matrix L.

        The remaining elements, with the array `tau`, represent the
        orthogonal matrix Q as a product of elementary reflectors (see
        [the F08 Introduction]).

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors (see
        [Further Comments]).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeqlf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ceft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorgql(m, n, k, a, tau):
    u"""
    Form all or part of orthogonal Q from QL factorization determined by
    ``dgeqlf``.

    For full information please refer to the NAG Library document for
    f08cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cff.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgeqlf``.

        On exit: the m by n matrix Q.

    tau : float, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``dgeqlf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorgql'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_n_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormql(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgeqlf``.

    For full information please refer to the NAG Library document for
    f08cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cgf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgeqlf``.

        On exit: is modified by ``dormql`` but restored on exit.

    tau : float, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``dgeqlf``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormql'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, k].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    exp_shape_a = (_a_dim1, _k_py)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgerqf(m, n, a, tau):
    u"""
    RQ factorization of real general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08chf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the upper triangle of the subarray
        `a`[0:m,n-m:n] contains the m by m upper triangular matrix R.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m by n upper trapezoidal matrix R; the remaining
        elements, with the array `tau`, represent the orthogonal matrix
        Q as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    tau : float, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgerqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08chft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorgrq(m, n, k, a, tau):
    u"""
    Form all or part of orthogonal Q from RQ factorization determined by
    ``dgerqf``.

    For full information please refer to the NAG Library document for
    f08cj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cjf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgerqf``.

        On exit: the m by n matrix Q.

    tau : float, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dgerqf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorgrq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_m_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormrq(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgerqf``.

    For full information please refer to the NAG Library document for
    f08ck

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ckf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : float, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        On entry: the ith row of `a` must contain the vector which
        defines the elementary reflector H_i, for i = 1,2,...,k, as
        returned by ``dgerqf``.

        On exit: is modified by ``dormrq`` but restored on exit.

    tau : float, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dgerqf``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormrq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ckft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[k].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgeqlf(m, n, a, tau):
    u"""
    QL factorization of complex general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08cs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08csf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the lower triangle of the subarray
        `a`[m-n:m,0:n] contains the n by n lower triangular matrix L.

        If m <= n, the elements on and below the (n-m)th superdiagonal
        contain the m by n lower trapezoidal matrix L.

        The remaining elements, with the array `tau`, represent the
        unitary matrix Q as a product of elementary reflectors (see [the
        F08 Introduction]).

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors (see
        [Further Comments]).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeqlf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08csft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zungql(m, n, k, a, tau):
    u"""
    Form all or part of unitary Q from QL factorization determined by
    ``zgeqlf``.

    For full information please refer to the NAG Library document for
    f08ct

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ctf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgeqlf``.

        On exit: the m by n matrix Q.

    tau : complex, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``zgeqlf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zungql'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ctft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_n_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmql(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgeqlf``.

    For full information please refer to the NAG Library document for
    f08cu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (:, `k`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgeqlf``.

        On exit: is modified by ``zunmql`` but restored on exit.

    tau : complex, shape (`k`)
        Further details of the elementary reflectors, as returned by
        ``zgeqlf``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmql'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, k].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    exp_shape_a = (_a_dim1, _k_py)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgerqf(m, n, a, tau):
    u"""
    RQ factorization of complex general rectangular matrix.

    For full information please refer to the NAG Library document for
    f08cv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cvf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the upper triangle of the subarray
        `a`[0:m,n-m:n] contains the m by m upper triangular matrix R.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m by n upper trapezoidal matrix R; the remaining
        elements, with the array `tau`, represent the unitary matrix Q
        as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    tau : complex, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgerqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zungrq(m, n, k, a, tau):
    u"""
    Form all or part of unitary Q from RQ factorization determined by
    ``zgerqf``.

    For full information please refer to the NAG Library document for
    f08cw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cwf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix Q.

    n : int
        n, the number of columns of the matrix Q.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgerqf``.

        On exit: the m by n matrix Q.

    tau : complex, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``zgerqf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m`.

        (`errno` -3)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zungrq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= _m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not (_m_py >= _k_py and _k_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= k and k >= 0.'),
            ],
            errcodes=(-3,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmrq(side, trans, m, n, k, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgerqf``.

    For full information please refer to the NAG Library document for
    f08cx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08cxf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        k, the number of elementary reflectors whose product defines the
        matrix Q.

    a : complex, shape (`k`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        On entry: the ith row of `a` must contain the vector which
        defines the elementary reflector H_i, for i = 1,2,...,k, as
        returned by ``zgerqf``.

        On exit: is modified by ``zunmrq`` but restored on exit.

    tau : complex, shape (`k`)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``zgerqf``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `m` >= `k` >= 0.

        (`errno` -5)
            On entry, error in parameter `k`.

            Constraint: `n` >= `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmrq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08cxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[k].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if _side_py in ('L', 'l'):
        if not (_m_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    elif _side_py in ('R', 'r'):
        if not (_n_py >= _k_py and _k_py >= 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= k and k >= 0.'),
                ],
                errcodes=(-5,),
                entity='k',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_k_py, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dsyev(jobz, uplo, n, a, w):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix.

    For full information please refer to the NAG Library document for
    f08fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08faf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if `jobz` = 'V', `a` contains the orthonormal
        eigenvectors of the matrix A.

        If `jobz` = 'N' then on exit the lower triangle (if `uplo` =
        'L') or the upper triangle (if `uplo` = 'U') of `a`, including
        the diagonal, is overwritten.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsyev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _w,
        _work,
        _lwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if _jobz_py in ('V', 'v'):
            _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def dsyevx(jobz, erange, uplo, n, a, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix.

    For full information please refer to the NAG Library document for
    f08fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsyevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-8,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-9,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-10,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dsyevd(job, uplo, n, a, w):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if `job` = 'V', `a` is overwritten by the orthogonal
        matrix Z which contains the eigenvectors of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsyevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _a,
        _lda,
        _w,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def dsyevr(jobz, erange, uplo, n, a, vl, vu, il, iu, abstol, w, z, isuppz):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix (Relatively Robust Representations).

    For full information please refer to the NAG Library document for
    f08fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fdf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

        For `erange` = 'V' or 'I' and `iu`-`il` < `n`-1, ``dstebz`` and
        ``dstein`` are called.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. See Demmel and Kahan.

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel for a discussion of which matrices can
        define their eigenvalues to high relative accuracy.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, est_m), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    isuppz : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `jobz`='V': 2*max(1, est_m); otherwise: 0.

        On exit: the support of the eigenvectors in `z`, i.e., the
        indices indicating the nonzero elements in `z`. The ith
        eigenvector is nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or 'I' and
        `iu`-`il` = `n`-1.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            ``dsyevr`` failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsyevr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Output, isuppz[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-8,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-9,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-10,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz_cls = utils._EngineIntArrayType(
        isuppz, {'fun_name': fun_name, 'entity': 'isuppz'},
        exp_shape=exp_shape_isuppz,
        check_contents=False,
    )
    _isuppz = _isuppz_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _isuppz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'dsyevr failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _isuppz_cls.refresh_py(_isuppz)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dsytrd(uplo, n, a, d, e, tau):
    u"""
    Orthogonal reduction of real symmetric matrix to symmetric
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: `a` is overwritten by the tridiagonal matrix T and
        details of the orthogonal matrix Q as specified by `uplo`.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    tau : float, shape (`n`-1), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsytrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, tau[n-1].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _d,
        _e,
        _tau,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorgtr(uplo, n, a, tau):
    u"""
    Generate orthogonal transformation matrix from reduction to
    tridiagonal form determined by ``dsytrd``.

    For full information please refer to the NAG Library document for
    f08ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fff.html

    Parameters
    ----------
    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``dsytrd``.

    n : int
        n, the order of the matrix Q.

    a : float, shape (`n`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dsytrd``.

        On exit: the n by n orthogonal matrix Q.

    tau : float, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``dsytrd``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorgtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[n-1].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormtr(side, uplo, trans, m, n, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dsytrd``.

    For full information please refer to the NAG Library document for
    f08fg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fgf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``dsytrd``.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    a : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dsytrd``.

    tau : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``dsytrd``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _uplo,
        _trans,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _uplo._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def ddisna(job, m, n, d, sep):
    u"""
    Computes the reciprocal condition numbers for the eigenvectors of a
    real symmetric or complex Hermitian matrix or for the left or right
    singular vectors of a general matrix.

    For full information please refer to the NAG Library document for
    f08fl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08flf.html

    Parameters
    ----------
    job : str, length 1
        Specifies for which problem the reciprocal condition number
        should be computed.

        `job` = 'E'

          The eigenvectors of a symmetric or Hermitian matrix.

        `job` = 'L'

          The left singular vectors of a general matrix.

        `job` = 'R'

          The right singular vectors of a general matrix.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix when `job` = 'L' or 'R'.

        If `job` = 'E', `n` is not referenced.

    d : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `job`='E': `m`; if `job` in ('L', 'R'): min(`m`,
        `n`); otherwise: 0.

        The eigenvalues if `job` = 'E', or singular values if `job` =
        'L' or 'R' of the matrix A.

    sep : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job`='E': `m`; if `job` in ('L', 'R'): min(`m`,
        `n`); otherwise: 0.

        On exit: the reciprocal condition numbers of the vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `d`.

            Constraint: the elements of the array must be in increasing
            or decreasing order.

        (`errno` -4)
            On entry, error in parameter `d`.

            Constraint: the elements of the array must not be negative.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ddisna'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08flft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[*].
        utils._EngineFloat64ArrayType.c_type, # Output, sep[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if _job_py in ('L', 'l', 'R', 'r'):
        if not _n_py >= 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= 0.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    if _job_py in ('E', 'e'):
        _d_dim1 = _m_py
    elif _job_py in ('L', 'l', 'R', 'r'):
        _d_dim1 = min(_m_py, _n_py)
    else:
        _d_dim1 = 0
    exp_shape_d = (_d_dim1,)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=exp_shape_d,
    )
    _d = _d_cls.to_ctypes()
    if _job_py in ('E', 'e'):
        if not _d_cls.is_monotonic(1-1, _m_py-1, 'either'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('d[0:m] must be monotonic.'),
                ],
                errcodes=(-4,),
                entity='d',
            ).manage()
    elif _job_py in ('L', 'l', 'R', 'r'):
        if not _d_cls.is_monotonic(1-1, min(_n_py, _m_py)-1, 'either'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('d[0:min(n, m)] must be monotonic.'),
                ],
                errcodes=(-4,),
                entity='d',
            ).manage()
    if _job_py not in ('E', 'e'):
        if not _d_cls.is_nonnegative(1-1, min(_n_py, _m_py)-1):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('d[0:min(n, m)] must not be negative.'),
                ],
                errcodes=(-4,),
                entity='d',
            ).manage()
    if _job_py in ('E', 'e'):
        _sep_dim1 = _m_py
    elif _job_py in ('L', 'l', 'R', 'r'):
        _sep_dim1 = min(_m_py, _n_py)
    else:
        _sep_dim1 = 0
    exp_shape_sep = (_sep_dim1,)
    _sep_cls = utils._EngineFloat64ArrayType(
        sep, {'fun_name': fun_name, 'entity': 'sep'},
        exp_shape=exp_shape_sep,
        check_contents=False,
    )
    _sep = _sep_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _m,
        _n,
        _d,
        _sep,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sep_cls.refresh_py(_sep)
        return_data = None
    _err_class.manage()
    return return_data

def zheev(jobz, uplo, n, a, w):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix.

    For full information please refer to the NAG Library document for
    f08fn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if `jobz` = 'V', `a` contains the orthonormal
        eigenvectors of the matrix A.

        If `jobz` = 'N' then on exit the lower triangle (if `uplo` =
        'L') or the upper triangle (if `uplo` = 'U') of `a`, including
        the diagonal, is overwritten.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zheev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _w,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        if _jobz_py in ('V', 'v'):
            _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zheevx(jobz, erange, uplo, n, a, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix.

    For full information please refer to the NAG Library document for
    f08fp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zheevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-8,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-9,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-10,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zheevd(job, uplo, n, a, w):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian matrix (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08fq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if `job` = 'V', `a` is overwritten by the unitary
        matrix Z which contains the eigenvectors of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zheevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _a,
        _lda,
        _w,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def zheevr(jobz, erange, uplo, n, a, vl, vu, il, iu, abstol, w, z, isuppz):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix (Relatively Robust Representations).

    For full information please refer to the NAG Library document for
    f08fr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08frf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

        For `erange` = 'V' or 'I' and `iu`-`il` < `n`-1, ``dstebz`` and
        ``zstein`` are called.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the real tridiagonal matrix obtained by reducing A to
        tridiagonal form. See Demmel and Kahan.

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel for a discussion of which matrices can
        define their eigenvalues to high relative accuracy.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : complex, shape (:, est_m), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    isuppz : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `jobz`='V': 2*max(1, est_m); otherwise: 0.

        On exit: the support of the eigenvectors in `z`, i.e., the
        indices indicating the nonzero elements in `z`. The ith
        eigenvector is nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or 'I' and
        `iu`-`il` = `n`-1.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            ``zheevr`` failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zheevr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08frft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Output, isuppz[*].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-8,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-9,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-10,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz_cls = utils._EngineIntArrayType(
        isuppz, {'fun_name': fun_name, 'entity': 'isuppz'},
        exp_shape=exp_shape_isuppz,
        check_contents=False,
    )
    _isuppz = _isuppz_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _isuppz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'zheevr failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _isuppz_cls.refresh_py(_isuppz)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def zhetrd(uplo, n, a, d, e, tau):
    u"""
    Unitary reduction of complex Hermitian matrix to real symmetric
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fsf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: `a` is overwritten by the tridiagonal matrix T and
        details of the unitary matrix Q as specified by `uplo`.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    tau : complex, shape (`n`-1), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhetrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Output, tau[n-1].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _d,
        _e,
        _tau,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zungtr(uplo, n, a, tau):
    u"""
    Generate unitary transformation matrix from reduction to tridiagonal
    form determined by ``zhetrd``.

    For full information please refer to the NAG Library document for
    f08ft

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ftf.html

    Parameters
    ----------
    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``zhetrd``.

    n : int
        n, the order of the matrix Q.

    a : complex, shape (`n`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zhetrd``.

        On exit: the n by n unitary matrix Q.

    tau : complex, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``zhetrd``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zungtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ftft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[n-1].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmtr(side, uplo, trans, m, n, a, tau, c):
    u"""
    Applies the unitary transformation matrix determined by ``zhetrd``.

    For full information please refer to the NAG Library document for
    f08fu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08fuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``zhetrd``.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    a : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zhetrd``.

    tau : complex, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``zhetrd``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08fuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _uplo,
        _trans,
        _m,
        _n,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _uplo._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dspev(jobz, uplo, n, ap, w, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix, packed storage.

    For full information please refer to the NAG Library document for
    f08ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the orthonormal
        eigenvectors of the matrix A, with the ith column of Z holding
        the eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ap,
        _w,
        _z,
        _ldz,
        _work,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dspevx(jobz, erange, uplo, n, ap, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix, packed storage.

    For full information please refer to the NAG Library document for
    f08gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the selected eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-8,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ap,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dspevd(job, uplo, n, ap, w, z):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix, packed storage (divide-and-conquer or
    Pal--Walker--Kahan variant of the QL or QR algorithm).

    For full information please refer to the NAG Library document for
    f08gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        On exit: if `job` = 'V', `z` is overwritten by the orthogonal
        matrix Z which contains the eigenvectors of A.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _ap,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dsptrd(uplo, n, ap, d, e, tau):
    u"""
    Orthogonal reduction of real symmetric matrix to symmetric
    tridiagonal form, packed storage.

    For full information please refer to the NAG Library document for
    f08ge

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the tridiagonal matrix T and
        details of the orthogonal matrix Q.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    tau : float, shape (`n`-1), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsptrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08geft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, tau[n-1].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _d,
        _e,
        _tau,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dopgtr(uplo, n, ap, tau, q):
    u"""
    Generate orthogonal transformation matrix from reduction to
    tridiagonal form determined by ``dsptrd``.

    For full information please refer to the NAG Library document for
    f08gf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gff.html

    Parameters
    ----------
    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``dsptrd``.

    n : int
        n, the order of the matrix Q.

    ap : float, shape (`n`*(`n`+1)/2)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dsptrd``.

    tau : float, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``dsptrd``.

    q : float, shape (`n`, `n`), modified in place
        On exit: the n by n orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dopgtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, tau[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, n].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((q, 'q'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _tau,
        _q,
        _ldq,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _q_cls.refresh_py(_q)
        return_data = None
    _err_class.manage()
    return return_data

def dopmtr(side, uplo, trans, m, n, ap, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dsptrd``.

    For full information please refer to the NAG Library document for
    f08gg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ggf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``dsptrd``.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    ap : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`*(`m`+1)/2; if `side`='R':
        `n`*(`n`+1)/2; otherwise: 0.

        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dsptrd``.

        On exit: is used as internal workspace prior to being restored
        and hence is unchanged.

    tau : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``dsptrd``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dopmtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ggft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[*].
        utils._EngineFloat64ArrayType.c_type, # Input, tau[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    if _side_py in ('L', 'l'):
        _ap_dim1 = _m_py*(_m_py+1)//2
    elif _side_py in ('R', 'r'):
        _ap_dim1 = _n_py*(_n_py+1)//2
    else:
        _ap_dim1 = 0
    exp_shape_ap = (_ap_dim1,)
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=exp_shape_ap,
    )
    _ap = _ap_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _uplo,
        _trans,
        _m,
        _n,
        _ap,
        _tau,
        _c,
        _ldc,
        _work,
        _info,
        _side._length_,
        _uplo._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zhpev(jobz, uplo, n, ap, w, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix, packed storage.

    For full information please refer to the NAG Library document for
    f08gn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the orthonormal
        eigenvectors of the matrix A, with the ith column of Z holding
        the eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ap,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhpevx(jobz, erange, uplo, n, ap, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix, packed storage.

    For full information please refer to the NAG Library document for
    f08gp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the selected eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-8,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ap,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def zhpevd(job, uplo, n, ap, w, z):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian matrix, packed storage (divide-and-conquer or
    Pal--Walker--Kahan variant of the QL or QR algorithm).

    For full information please refer to the NAG Library document for
    f08gq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the values generated during the
        reduction to tridiagonal form. The elements of the diagonal and
        the off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        On exit: if `job` = 'V', `z` is overwritten by the unitary
        matrix Z which contains the eigenvectors of A.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _ap,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhptrd(uplo, n, ap, d, e, tau):
    u"""
    Performs a unitary reduction of complex Hermitian matrix to real
    symmetric tridiagonal form, packed storage.

    For full information please refer to the NAG Library document for
    f08gs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gsf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: `ap` is overwritten by the tridiagonal matrix T and
        details of the unitary matrix Q.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    tau : complex, shape (`n`-1), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhptrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Output, tau[n-1].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _d,
        _e,
        _tau,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zupgtr(uplo, n, ap, tau, q):
    u"""
    Generates a unitary transformation matrix from reduction to
    tridiagonal form determined by ``zhptrd``.

    For full information please refer to the NAG Library document for
    f08gt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08gtf.html

    Parameters
    ----------
    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``zhptrd``.

    n : int
        n, the order of the matrix Q.

    ap : complex, shape (`n`*(`n`+1)/2)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zhptrd``.

    tau : complex, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``zhptrd``.

    q : complex, shape (`n`, `n`), modified in place
        On exit: the n by n unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zupgtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08gtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, tau[n-1].
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, n].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((q, 'q'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ap = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ap,
        _tau,
        _q,
        _ldq,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _q_cls.refresh_py(_q)
        return_data = None
    _err_class.manage()
    return return_data

def zupmtr(side, uplo, trans, m, n, ap, tau, c):
    u"""
    Applies the unitary transformation matrix determined by ``zhptrd``.

    For full information please refer to the NAG Library document for
    f08gu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08guf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    uplo : str, length 1
        This must be the same argument `uplo` as supplied to ``zhptrd``.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    ap : complex, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`*(`m`+1)/2; if `side`='R':
        `n`*(`n`+1)/2; otherwise: 0.

        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zhptrd``.

        On exit: is used as internal workspace prior to being restored
        and hence is unchanged.

    tau : complex, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``zhptrd``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zupmtr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08guft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[*].
        utils._EngineComplex128ArrayType.c_type, # Input, tau[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    if _side_py in ('L', 'l'):
        _ap_dim1 = _m_py*(_m_py+1)//2
    elif _side_py in ('R', 'r'):
        _ap_dim1 = _n_py*(_n_py+1)//2
    else:
        _ap_dim1 = 0
    exp_shape_ap = (_ap_dim1,)
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=exp_shape_ap,
    )
    _ap = _ap_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _uplo,
        _trans,
        _m,
        _n,
        _ap,
        _tau,
        _c,
        _ldc,
        _work,
        _info,
        _side._length_,
        _uplo._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dsbev(jobz, uplo, n, kd, ab, w, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric band matrix.

    For full information please refer to the NAG Library document for
    f08ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08haf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the orthonormal
        eigenvectors of the matrix A, with the ith column of Z holding
        the eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08haft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _w,
        _z,
        _ldz,
        _work,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dsbevx(jobz, erange, uplo, n, kd, ab, q, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric band matrix.

    For full information please refer to the NAG Library document for
    f08hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the n by n orthogonal matrix used in
        the reduction to tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-11,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-12,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-13,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _q,
        _ldq,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _q_cls.refresh_py(_q)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dsbevd(job, uplo, n, kd, ab, w, z):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric band matrix (divide-and-conquer or Pal--Walker--Kahan
    variant of the QL or QR algorithm).

    For full information please refer to the NAG Library document for
    f08hc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        On exit: if `job` = 'V', `z` is overwritten by the orthogonal
        matrix Z which contains the eigenvectors of A. The ith column of
        Z contains the eigenvector which corresponds to the eigenvalue
        `w`[i-1].

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dsbtrd(vect, uplo, n, kd, ab, d, e, q):
    u"""
    Performs an orthogonal reduction of real symmetric band matrix to
    symmetric tridiagonal form.

    For full information please refer to the NAG Library document for
    f08he

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q is to be returned.

        `vect` = 'V'

          Q is returned.

        `vect` = 'U'

          Q is updated (and the array `q` must contain a matrix on
          entry).

        `vect` = 'N'

          Q is not required.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    q : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('V', 'U'): max(1, `n`); if
        `vect`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('V', 'U'): `n`; if
        `vect`='N': 1; otherwise: 0.

        Optionally, on entry: if `vect` = 'U', `q` must contain the
        matrix formed in a previous stage of the reduction (for example,
        the reduction of a banded symmetric-definite generalized
        eigenproblem); otherwise `q` need not be set.

        On exit, if not None on entry: if `vect` = 'V' or 'U', the n by
        n matrix Q.

        If `vect` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'V', 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbtrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08heft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('V', 'v', 'U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'V\', \'v\', \'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _vect_py in ('U', 'u'):
        if q is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect in (\'U\', \'u\')',
            )
    if _vect_py not in ('N', 'n'):
        if q is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect not in (\'N\', \'n\')',
            )
    if q is None:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        _q = utils._EngineFloat64ArrayType.empty_ctypes((_q_dim1)*(_q_dim2))
    else:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        exp_shape_q = (_q_dim1, _q_dim2)
        _q_cls = utils._EngineFloat64ArrayType(
            q, {'fun_name': fun_name, 'entity': 'q'},
            exp_shape=exp_shape_q,
            mutual_sorder=_m_sorder,
            check_contents=_vect_py in ('U', 'u'),
            allow_none=True,
        )
        _q = _q_cls.to_ctypes(can_reference_contents=_vect_py in ('U', 'u'))
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _d,
        _e,
        _q,
        _ldq,
        _work,
        _info,
        _vect._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        if q is not None:
            _q_cls.refresh_py(_q)
        return_data = None
    _err_class.manage()
    return return_data

def zhbev(jobz, uplo, n, kd, ab, w, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian band matrix.

    For full information please refer to the NAG Library document for
    f08hn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the orthonormal
        eigenvectors of the matrix A, with the ith column of Z holding
        the eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhbevx(jobz, erange, uplo, n, kd, ab, q, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian band matrix.

    For full information please refer to the NAG Library document for
    f08hp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the n by n unitary matrix used in the
        reduction to tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-5,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-11,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-12,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-13,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _q,
        _ldq,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _q_cls.refresh_py(_q)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zhbevd(job, uplo, n, kd, ab, w, z):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian band matrix (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08hq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues of the matrix A in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        On exit: if `job` = 'V', `z` is overwritten by the unitary
        matrix Z which contains the eigenvectors of A. The ith column of
        Z contains the eigenvector which corresponds to the eigenvalue
        `w`[i-1].

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = {:s} and job = \'N\', the algorithm failed to'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'converge; {:s} elements of an intermediate tridiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero; if errno = {:s} and'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'job = \'V\', then the algorithm failed to compute an eigenvalue while working',
                    'on the submatrix lying in rows and column {:s}/(n+1)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'through mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhbtrd(vect, uplo, n, kd, ab, d, e, q):
    u"""
    Performs a unitary reduction of complex Hermitian band matrix to
    real symmetric tridiagonal form.

    For full information please refer to the NAG Library document for
    f08hs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08hsf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q is to be returned.

        `vect` = 'V'

          Q is returned.

        `vect` = 'U'

          Q is updated (and the array `q` must contain a matrix on
          entry).

        `vect` = 'N'

          Q is not required.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction to tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    d : float, shape (`n`), modified in place
        On exit: the diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1), modified in place
        On exit: the off-diagonal elements of the tridiagonal matrix T.

    q : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('V', 'U'): max(1, `n`); if
        `vect`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('V', 'U'): `n`; if
        `vect`='N': 1; otherwise: 0.

        Optionally, on entry: if `vect` = 'U', `q` must contain the
        matrix formed in a previous stage of the reduction (for example,
        the reduction of a banded Hermitian-definite generalized
        eigenproblem); otherwise `q` need not be set.

        On exit, if not None on entry: if `vect` = 'V' or 'U', the n by
        n matrix Q.

        If `vect` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'V', 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbtrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08hsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('V', 'v', 'U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'V\', \'v\', \'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    if not _kd_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kd >= 0.'),
            ],
            errcodes=(-4,),
            entity='kd',
        ).manage()
    _ldab = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _vect_py in ('U', 'u'):
        if q is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect in (\'U\', \'u\')',
            )
    if _vect_py not in ('N', 'n'):
        if q is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect not in (\'N\', \'n\')',
            )
    if q is None:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        _q = utils._EngineComplex128ArrayType.empty_ctypes((_q_dim1)*(_q_dim2))
    else:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        exp_shape_q = (_q_dim1, _q_dim2)
        _q_cls = utils._EngineComplex128ArrayType(
            q, {'fun_name': fun_name, 'entity': 'q'},
            exp_shape=exp_shape_q,
            mutual_sorder=_m_sorder,
            check_contents=_vect_py in ('U', 'u'),
            allow_none=True,
        )
        _q = _q_cls.to_ctypes(can_reference_contents=_vect_py in ('U', 'u'))
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _uplo,
        _n,
        _kd,
        _ab,
        _ldab,
        _d,
        _e,
        _q,
        _ldq,
        _work,
        _info,
        _vect._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        if q is not None:
            _q_cls.refresh_py(_q)
        return_data = None
    _err_class.manage()
    return return_data

def dstev(jobz, n, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f08ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    n : int
        n, the order of the matrix.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix A.

        On exit: if no exception or warning is raised, the eigenvalues
        in ascending order.

    e : float, shape (`n`-1), modified in place
        On entry: the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

        On exit: the contents of `e` are destroyed.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', then if no exception or warning is
        raised, `z` contains the orthonormal eigenvectors of the matrix
        A, with the ith column of Z holding the eigenvector associated
        with `d`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of `e` did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _info,
        _jobz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of e did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dstevx(jobz, erange, n, d, e, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f08jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    n : int
        n, the order of the matrix.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix A.

        On exit: may be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    e : float, shape (`n`-1), modified in place
        On entry: the (n-1) subdiagonal elements of the tridiagonal
        matrix A.

        On exit: may be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||A||_1 will be used in its place.
        Eigenvalues will be computed most accurately when `abstol` is
        set to twice the underflow threshold 2*machine.real_safe(), not
        zero. If this function returns with `errno` > 0, indicating that
        some eigenvectors did not converge, try setting `abstol` to
        2*machine.real_safe(). See Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1];

        - if an eigenvector fails to converge (`errno` > 0), then that
          column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` > 0, `jfail` contains the indices of the
          eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-8,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _n,
        _d,
        _e,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dstevd(job, n, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric tridiagonal matrix (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

          Only eigenvalues are computed.

        `job` = 'V'

          Eigenvalues and eigenvectors are computed.

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix T.

        On exit: the eigenvalues of the matrix T in ascending order.

    e : float, shape (`n`), modified in place
        On entry: the n-1 off-diagonal elements of the tridiagonal
        matrix T. The nth element of this array is used as workspace.

        On exit: `e` is overwritten with intermediate results.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job`='V': `n`; if `job`='N': 1;
        otherwise: 0.

        On exit: if `job` = 'V', `z` is overwritten by the orthogonal
        matrix Z which contains the eigenvectors of T.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of `e` did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstevd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py,),
    )
    _e = _e_cls.to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of e did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dstevr(jobz, erange, n, d, e, vl, vu, il, iu, abstol, w, z, isuppz):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric tridiagonal matrix (Relatively Robust
    Representations).

    For full information please refer to the NAG Library document for
    f08jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jdf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    n : int
        n, the order of the matrix.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix T.

        On exit: may be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    e : float, shape (`n`-1), modified in place
        On entry: the (n-1) subdiagonal elements of the tridiagonal
        matrix T.

        On exit: may be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place.
        See Demmel and Kahan.

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel for a discussion of which matrices can
        define their eigenvalues to high relative accuracy.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, est_m), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    isuppz : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `jobz`='V': 2*max(1, est_m); otherwise: 0.

        On exit: the support of the eigenvectors in `z`, i.e., the
        indices indicating the nonzero elements in `z`. The ith
        eigenvector is nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or 'I' and
        `iu`-`il` = `n`-1.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            An internal error has occurred in this function. Please
            refer to errno in ``dstebz``.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstevr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Output, isuppz[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-8,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz_cls = utils._EngineIntArrayType(
        isuppz, {'fun_name': fun_name, 'entity': 'isuppz'},
        exp_shape=exp_shape_isuppz,
        check_contents=False,
    )
    _isuppz = _isuppz_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _n,
        _d,
        _e,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _isuppz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred in this function. Please refer to errno in',
                    'dstebz.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _isuppz_cls.refresh_py(_isuppz)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dsteqr(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric
    tridiagonal matrix, reduced from real symmetric matrix using the
    implicit QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jef.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix T.

        On exit: the n eigenvalues in ascending order, unless `errno` >
        0 (in which case see [Errors]).

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the tridiagonal matrix T.

        On exit: `e` is overwritten.

    z : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; otherwise:
        0.

        Optionally, on entry: if `compz` = 'V', `z` must contain the
        orthogonal matrix Q from the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

        On exit, if not None on entry: if `compz` = 'V' or 'I', the n
        required orthonormal eigenvectors stored as columns of Z; the
        ith column corresponds to the ith eigenvalue, where i =
        1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*`n` iterations. In this case, `d` and `e`
            contain on exit the diagonal and off-diagonal elements,
            respectively, of a tridiagonal matrix orthogonally similar
            to T. *<value>* off-diagonal elements have not converged to
            zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsteqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v'):
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz in (\'V\', \'v\')',
            )
    if _compz_py not in ('N', 'n'):
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz not in (\'N\', \'n\')',
            )
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=_compz_py in ('V', 'v'),
        allow_none=True,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=_compz_py in ('V', 'v'))
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to find all the eigenvalues after a total',
                    'of 30*n iterations. In this case, d and e',
                    'contain on exit the diagonal and off-diagonal elements, respectively, of a',
                    'tridiagonal matrix orthogonally similar to T. {:s}'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'off-diagonal elements have not converged to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        if z is not None:
            _z_cls.refresh_py(_z)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dsterf(n, d, e):
    u"""
    Computes all eigenvalues of real symmetric tridiagonal matrix,
    root-free variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08jf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix T.

        On exit: the n eigenvalues in ascending order, unless `errno` >
        0 (in which case see [Errors]).

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the tridiagonal matrix T.

        On exit: `e` is overwritten.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*`n` iterations; *<value>* elements of `e` have
            not converged to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsterf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._BYREF_INT, # Output, info.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _d,
        _e,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to find all the eigenvalues after a total',
                    'of 30*n iterations; {:s} elements of e have'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converged to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dpteqr(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from real symmetric positive
    definite matrix.

    For full information please refer to the NAG Library document for
    f08jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jgf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix T.

        On exit: the n eigenvalues in descending order, unless `errno` >
        0, in which case `d` is overwritten.

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the tridiagonal matrix T.

        On exit: `e` is overwritten.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain the orthogonal
        matrix Q from the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

        On exit: if `compz` = 'V' or 'I', the n required orthonormal
        eigenvectors stored as columns of Z; the ith column corresponds
        to the ith eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i>0 and i<=`n`)
            The leading minor of order *<value>* is not positive
            definite and the Cholesky factorization of T could not be
            completed. Hence T itself is not positive definite.

        (`errno` i > `n`)
            The algorithm to compute the singular values of the Cholesky
            factor B failed to converge; *<value>* off-diagonal elements
            did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dpteqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the Cholesky factorization of T could not be',
                    'completed. Hence T itself is not positive definite.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm to compute the singular values of the Cholesky factor',
                    'B failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py-_n_py),
                    ),
                    'elements did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dstedc(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a matrix reduced to this form
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08jh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jhf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the symmetric tridiagonal matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix.

        On exit: if no exception or warning is raised, the eigenvalues
        in ascending order.

    e : float, shape (`n`-1), modified in place
        On entry: the subdiagonal elements of the tridiagonal matrix.

        On exit: `e` is overwritten.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; otherwise:
        1.

        On entry: if `compz` = 'V', `z` must contain the orthogonal
        matrix Q used in the reduction to tridiagonal form.

        On exit: if `compz` = 'V', `z` contains the orthonormal
        eigenvectors of the original symmetric matrix A, and if `compz`
        = 'I', `z` contains the orthonormal eigenvectors of the
        symmetric tridiagonal matrix T.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to compute an eigenvalue while working
            on the submatrix lying in rows and columns *<value>*/(`n`+1)
            through mod(*<value>*, `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstedc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to compute an eigenvalue while working on the submatrix',
                    'lying in rows and columns {:s}/(n+1) through'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dstebz(erange, order, n, vl, vu, il, iu, abstol, d, e, w, iblock, isplit):
    u"""
    Computes selected eigenvalues of real symmetric tridiagonal matrix
    by bisection.

    For full information please refer to the NAG Library document for
    f08jj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jjf.html

    Parameters
    ----------
    erange : str, length 1
        Indicates which eigenvalues are required.

        `erange` = 'A'

          All the eigenvalues are required.

        `erange` = 'V'

          All the eigenvalues in the half-open interval (`vl`,`vu`] are
          required.

        `erange` = 'I'

          Eigenvalues with indices `il` to `iu` are required.

    order : str, length 1
        Indicates the order in which the eigenvalues and their block
        numbers are to be stored.

        `order` = 'B'

          The eigenvalues are to be grouped by split-off block and
          ordered from smallest to largest within each block.

        `order` = 'E'

          The eigenvalues for the entire matrix are to be ordered from
          smallest to largest.

    n : int
        n, the order of the matrix T.

    vl : float
        If `erange` = 'V', the lower and upper bounds, respectively, of
        the half-open interval (`vl`,`vu`] within which the required
        eigenvalues lie.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds, respectively, of
        the half-open interval (`vl`,`vu`] within which the required
        eigenvalues lie.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', the indices of the first and last
        eigenvalues, respectively, to be computed (assuming that the
        eigenvalues are in ascending order).

        If `erange` = 'A' or 'V', `il` is not referenced.

    iu : int
        If `erange` = 'I', the indices of the first and last
        eigenvalues, respectively, to be computed (assuming that the
        eigenvalues are in ascending order).

        If `erange` = 'A' or 'V', `iu` is not referenced.

    abstol : float
        The absolute tolerance to which each eigenvalue is required. An
        eigenvalue (or cluster) is considered to have converged if it
        lies in an interval of width <= `abstol`. If `abstol` <= 0.0,
        the tolerance is taken as machine precision*||T||_1.

    d : float, shape (`n`)
        The diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1)
        The off-diagonal elements of the tridiagonal matrix T.

    w : float, shape (`n`), modified in place
        On exit: the required eigenvalues of the tridiagonal matrix T
        stored in `w`[0] to `w`[m-1].

    iblock : int, shape (`n`), modified in place
        On exit: at each row/column j where `e`[j-1] is zero or
        negligible, T is considered to split into a block diagonal
        matrix and `iblock`[i-1] contains the block number of the
        eigenvalue stored in `w`[i-1], for i = 1,2,...,m. Note that
        `iblock`[i-1] < 0 for some i whenever `errno` = 1 or 3 (see
        [Errors]) and `erange` = 'A' or 'V'.

    isplit : int, shape (`n`), modified in place
        On exit: the leading `nsplit` elements contain the points at
        which T splits up into sub-matrices as follows. The first
        sub-matrix consists of rows/columns 1 to `isplit`[0], the second
        sub-matrix consists of rows/columns `isplit`[0]+1 to
        `isplit`[1], ..., and the `nsplit`(th) sub-matrix consists of
        rows/columns `isplit`[`nsplit`-2]+1 to `isplit`[`nsplit`-1] (=
        n).

    Returns
    -------
    m : int
        m, the actual number of eigenvalues found.

    nsplit : int
        The number of diagonal blocks which constitute the tridiagonal
        matrix T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `order`.

            Constraint: `order` = 'B' or 'E'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -6)
            On entry, error in parameter `il`.

            Constraint: `il` >= 1 and `il` <= max(1, `n`).

        (`errno` -7)
            On entry, error in parameter `iu`.

            Constraint: `iu` <= `n` and `iu` >= min(`n`, `il`).

        (`errno` 2)
            If `erange` = 'I', the algorithm failed to compute some (or
            all) of the required eigenvalues. Try calling the function
            again with `erange` = 'A'.

        (`errno` 3)
            If `erange` = 'I', the algorithm failed to compute some (or
            all) of the required eigenvalues. Try calling the function
            again with `erange` = 'A'. If `erange` = 'A' or 'V', the
            algorithm failed to compute some (or all) of the required
            eigenvalues to the required accuracy. More precisely,
            `iblock`[*<value>*] < 0 indicates that eigenvalue *<value>*
            (stored in `w`[*<value>*]) failed to converge.

        (`errno` 4)
            No eigenvalues have been computed. The floating-point
            arithmetic on the computer is not behaving as expected.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            If `erange` = 'A' or 'V', the algorithm failed to compute
            some (or all) of the required eigenvalues to the required
            accuracy. More precisely, `iblock`[*<value>*] < 0 indicates
            that eigenvalue *<value>* (stored in `w`[*<value>*]) failed
            to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstebz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, order(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_INT, # Output, m.
        utils._BYREF_INT, # Output, nsplit.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineIntArrayType.c_type, # Output, iblock[n].
        utils._EngineIntArrayType.c_type, # Output, isplit[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of order.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _call_vendor = utils._EngineState._literal_true_cint
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='range',
        ).manage()
    _order = utils._EngineCharScalarType(
        order, {'fun_name': fun_name, 'entity': 'order'}, exp_chrlen=1,
    ).to_ctypes()
    _order_py = utils._EngineCharScalarType.to_py(_order)
    if _order_py not in ('B', 'b', 'E', 'e'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('order in (\'B\', \'b\', \'E\', \'e\').'),
            ],
            errcodes=(-2,),
            entity='order',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-5,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-6,),
                entity='il',
            ).manage()
        if not (_iu_py <= _n_py and _iu_py >= min(_n_py, _il_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu <= n.'),
                ],
                errcodes=(-7,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _nsplit = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _iblock_cls = utils._EngineIntArrayType(
        iblock, {'fun_name': fun_name, 'entity': 'iblock'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iblock = _iblock_cls.to_ctypes(can_reference_contents=False)
    _isplit_cls = utils._EngineIntArrayType(
        isplit, {'fun_name': fun_name, 'entity': 'isplit'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _isplit = _isplit_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _erange,
        _order,
        _n,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _d,
        _e,
        _m,
        _nsplit,
        _w,
        _iblock,
        _isplit,
        _work,
        _iwork,
        _info,
        _erange._length_,
        _order._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'If erange = \'A\' or \'V\',',
                    'the algorithm failed to compute some (or all) of the required eigenvalues',
                    'to the required accuracy. More precisely, iblock[{:s}] < 0'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'indicates that eigenvalue {:s} (stored in'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'w[{:s}]) failed to converge.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If erange = \'I\', the algorithm failed to compute some',
                    '(or all) of the required eigenvalues. Try calling the function again with',
                    'erange = \'A\'.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If erange = \'I\', the algorithm failed to compute some',
                    '(or all) of the required eigenvalues. Try calling the function again with',
                    'erange = \'A\'.',
                    'If erange = \'A\' or \'V\',',
                    'the algorithm failed to compute some (or all) of the required eigenvalues',
                    'to the required accuracy. More precisely, iblock[{:s}] < 0'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'indicates that eigenvalue {:s} (stored in'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'w[{:s}]) failed to converge.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No eigenvalues have been computed. The floating-point arithmetic on the',
                    'computer is not behaving as expected.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iblock_cls.refresh_py(_iblock)
        _isplit_cls.refresh_py(_isplit)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _nsplit_py = utils._EngineIntScalarType.to_py(_nsplit)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 'nsplit'],
        )(
            _m_py,
            _nsplit_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dstein(n, d, e, m, w, iblock, isplit, z, ifailv):
    u"""
    Computes selected eigenvectors of real symmetric tridiagonal matrix
    by inverse iteration, storing eigenvectors in real array.

    For full information please refer to the NAG Library document for
    f08jk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jkf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix T.

    d : float, shape (`n`)
        The diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1)
        The off-diagonal elements of the tridiagonal matrix T.

    m : int
        m, the number of eigenvectors to be returned.

    w : float, shape (`n`)
        The eigenvalues of the tridiagonal matrix T stored in `w`[0] to
        `w`[m-1], as returned by ``dstebz`` with order = 'B'.
        Eigenvalues associated with the first sub-matrix must be
        supplied first, in nondecreasing order; then those associated
        with the second sub-matrix, again in nondecreasing order; and so
        on.

    iblock : int, shape (`n`)
        The first m elements must contain the sub-matrix indices
        associated with the specified eigenvalues, as returned by
        ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `iblock`[i-1] to 1,
        for i = 1,2,...,m.

    isplit : int, shape (`n`)
        The points at which T breaks up into sub-matrices, as returned
        by ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `isplit`[0] to `n`.

    z : float, shape (`n`, `m`), modified in place
        On exit: the m eigenvectors, stored as columns of Z; the ith
        column corresponds to the ith specified eigenvalue, unless
        `errno` > 0 (in which case see [Errors]).

    ifailv : int, shape (`m`), modified in place
        On exit: if errno = i > 0, the first i elements of `ifailv`
        contain the indices of any eigenvectors which have failed to
        converge. The rest of the first `m` elements of `ifailv` are set
        to 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= `n`.

        (`errno` -5)
            On entry, error in parameter `w`.

            Constraint: `w`[i] <= `w`[i+1].

        (`errno` -6)
            On entry, error in parameter `iblock`.

            Constraint: `iblock`[i] <= `iblock`[i+1].

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by argument `ifailv`)
            each failed to converge in five iterations. The current
            iterate after five iterations is stored in the corresponding
            column of `z`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstein'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, w[n].
        utils._EngineIntArrayType.c_type, # Input, iblock[n].
        utils._EngineIntArrayType.c_type, # Input, isplit[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, m].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, ifailv[m].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not (_m_py >= 0 and _m_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= m and m <= n.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    for _vl_i_py in range(0, (_m_py-1)-1+1):
        if utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py,)) == utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py+1,)):
            if not utils._EngineFloat64ArrayType.get_element(w, {'fun_name': fun_name, 'entity': 'w'}, (_vl_i_py,)) <= utils._EngineFloat64ArrayType.get_element(w, {'fun_name': fun_name, 'entity': 'w'}, (_vl_i_py+1,)):
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        utils._lapack_constraint_err_text('w[i] <= w[i+1].'),
                    ],
                    errcodes=(-5,),
                    entity='w',
                ).manage()
    _iblock = utils._EngineIntArrayType(
        iblock, {'fun_name': fun_name, 'entity': 'iblock'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    for _vl_i_py in range(0, (_m_py-1)-1+1):
        if not utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py,)) <= utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py+1,)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iblock[i] <= iblock[i+1].'),
                ],
                errcodes=(-6,),
                entity='iblock',
            ).manage()
    _isplit = utils._EngineIntArrayType(
        isplit, {'fun_name': fun_name, 'entity': 'isplit'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _ifailv_cls = utils._EngineIntArrayType(
        ifailv, {'fun_name': fun_name, 'entity': 'ifailv'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _ifailv = _ifailv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _d,
        _e,
        _m,
        _w,
        _iblock,
        _isplit,
        _z,
        _ldz,
        _work,
        _iwork,
        _ifailv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} eigenvectors (as indicated by argument ifailv)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'each failed to converge in five iterations. The current iterate after five',
                    'iterations is stored in the corresponding column of z.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _z_cls.refresh_py(_z)
        _ifailv_cls.refresh_py(_ifailv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dstegr(jobz, erange, n, d, e, vl, vu, il, iu, w, z, isuppz):
    u"""
    Computes selected eigenvalues and, optionally, the corresponding
    eigenvectors of a real symmetric tridiagonal matrix or a symmetric
    matrix reduced to this form (Relatively Robust Representations).

    For full information please refer to the NAG Library document for
    f08jl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jlf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        Indicates which eigenvalues should be returned.

        `erange` = 'A'

          All eigenvalues will be found.

        `erange` = 'V'

          All eigenvalues in the half-open interval (`vl`,`vu`] will be
          found.

        `erange` = 'I'

          The `il`th through `iu`th eigenvectors will be found.

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix T.

        On exit: `d` is overwritten.

    e : float, shape (`n`), modified in place
        On entry: `e`[0] to `e`[`n`-2] are the subdiagonal elements of
        the tridiagonal matrix T. `e`[`n`-1] need not be set.

        On exit: `e` is overwritten.

    vl : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, est_m), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', then if no exception or warning is
        raised, the columns of `z` contain the orthonormal eigenvectors
        of the matrix T, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    isuppz : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `jobz`='V': 2*max(1, est_m); otherwise: 0.

        On exit: the support of the eigenvectors in Z, i.e., the indices
        indicating the nonzero elements in Z. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1].

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: 1 <= `il` <= `iu` <= `n`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: `il` = 1 and `iu` = 0.

        (`errno` i = 1)
            The dqds algorithm failed to converge.

        (`errno` i = 2)
            Inverse iteration failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dstegr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Output, isuppz[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py,),
    )
    _e = _e_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if (_erange_py in ('I', 'i')) and _n_py > 0:
        if not (_il_py >= 1 and _il_py <= _iu_py and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= il and il <= iu and iu <= n.'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    elif (_erange_py in ('I', 'i')) and _n_py == 0:
        if not (_il_py == 1 and _iu_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il = 1.'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType.c_type(0.0)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz_cls = utils._EngineIntArrayType(
        isuppz, {'fun_name': fun_name, 'entity': 'isuppz'},
        exp_shape=exp_shape_isuppz,
        check_contents=False,
    )
    _isuppz = _isuppz_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _n,
        _d,
        _e,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _isuppz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The dqds algorithm failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inverse iteration failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _isuppz_cls.refresh_py(_isuppz)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def zsteqr(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric
    tridiagonal matrix, reduced from complex Hermitian matrix, using the
    implicit QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08js

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jsf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix T.

        On exit: the n eigenvalues in ascending order, unless `errno` >
        0 (in which case see [Errors]).

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the tridiagonal matrix T.

        On exit: `e` is overwritten.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain the unitary matrix
        Q from the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

        On exit: if `compz` = 'V' or 'I', the n required orthonormal
        eigenvectors stored as columns of Z; the ith column corresponds
        to the ith eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*`n` iterations. In this case, `d` and `e`
            contain on exit the diagonal and off-diagonal elements,
            respectively, of a tridiagonal matrix unitarily similar to
            T. *<value>* off-diagonal elements have not converged to
            zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zsteqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to find all the eigenvalues after a total',
                    'of 30*n iterations. In this case, d and e',
                    'contain on exit the diagonal and off-diagonal elements, respectively, of a',
                    'tridiagonal matrix unitarily similar to T. {:s}'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'off-diagonal elements have not converged to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zpteqr(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from complex Hermitian positive
    definite matrix.

    For full information please refer to the NAG Library document for
    f08ju

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08juf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix T.

        On exit: the n eigenvalues in descending order, unless `errno` >
        0, in which case `d` is overwritten.

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the tridiagonal matrix T.

        On exit: `e` is overwritten.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain the unitary matrix
        Q from the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

        On exit: if `compz` = 'V' or 'I', the n required orthonormal
        eigenvectors stored as columns of Z; the ith column corresponds
        to the ith eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i>0 and i<=`n`)
            The leading minor of order *<value>* is not positive
            definite and the Cholesky factorization of T could not be
            completed. Hence T itself is not positive definite.

        (`errno` i > `n`)
            The algorithm to compute the singular values of the Cholesky
            factor B failed to converge; *<value>* off-diagonal elements
            did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zpteqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08juft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The leading minor of order {:s} is not positive definite'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and the Cholesky factorization of T could not be',
                    'completed. Hence T itself is not positive definite.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm to compute the singular values of the Cholesky factor',
                    'B failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py-_n_py),
                    ),
                    'elements did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zstedc(compz, n, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a complex Hermitian matrix reduced
    to this form (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08jv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jvf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

          Only the eigenvalues are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The eigenvalues and eigenvectors of A are computed (and the
          array `z` must contain the matrix Q on entry).

        `compz` = 'I'

          The eigenvalues and eigenvectors of T are computed (and the
          array `z` is initialized by the function).

    n : int
        n, the order of the symmetric tridiagonal matrix T.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the tridiagonal matrix.

        On exit: if no exception or warning is raised, the eigenvalues
        in ascending order.

    e : float, shape (`n`-1), modified in place
        On entry: the subdiagonal elements of the tridiagonal matrix.

        On exit: `e` is overwritten.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; otherwise:
        1.

        On entry: if `compz` = 'V', `z` must contain the unitary matrix
        Q used in the reduction to tridiagonal form.

        On exit: if `compz` = 'V', `z` contains the orthonormal
        eigenvectors of the original Hermitian matrix A, and if `compz`
        = 'I', `z` contains the orthonormal eigenvectors of the
        symmetric tridiagonal matrix T.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to compute an eigenvalue while working
            on the submatrix lying in rows and columns *<value>*/(`n`+1)
            through mod(*<value>*, `n`+1).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zstedc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-1,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compz,
        _n,
        _d,
        _e,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to compute an eigenvalue while working on the submatrix',
                    'lying in rows and columns {:s}/(n+1) through'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'mod({:s}, n+1).'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zstein(n, d, e, m, w, iblock, isplit, z, ifailv):
    u"""
    Computes selected eigenvectors of real symmetric tridiagonal matrix
    by inverse iteration, storing eigenvectors in complex array.

    For full information please refer to the NAG Library document for
    f08jx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jxf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix T.

    d : float, shape (`n`)
        The diagonal elements of the tridiagonal matrix T.

    e : float, shape (`n`-1)
        The off-diagonal elements of the tridiagonal matrix T.

    m : int
        m, the number of eigenvectors to be returned.

    w : float, shape (`n`)
        The eigenvalues of the tridiagonal matrix T stored in `w`[0] to
        `w`[m-1], as returned by ``dstebz`` with order = 'B'.
        Eigenvalues associated with the first sub-matrix must be
        supplied first, in nondecreasing order; then those associated
        with the second sub-matrix, again in nondecreasing order; and so
        on.

    iblock : int, shape (`n`)
        The first m elements must contain the sub-matrix indices
        associated with the specified eigenvalues, as returned by
        ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `iblock`[i-1] to 1,
        for i = 1,2,...,m.

    isplit : int, shape (`n`)
        The points at which T breaks up into sub-matrices, as returned
        by ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `isplit`[0] to `n`.

    z : complex, shape (`n`, `m`), modified in place
        On exit: the m eigenvectors, stored as columns of Z; the ith
        column corresponds to the ith specified eigenvalue, unless
        `errno` > 0 (in which case see [Errors]).

    ifailv : int, shape (`m`), modified in place
        On exit: if errno = i > 0, the first i elements of `ifailv`
        contain the indices of any eigenvectors which have failed to
        converge. The rest of the first `m` elements of `ifailv` are set
        to 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= `n`.

        (`errno` -5)
            On entry, error in parameter `w`.

            Constraint: `w`[i] <= `w`[i+1].

        (`errno` -6)
            On entry, error in parameter `iblock`.

            Constraint: `iblock`[i] <= `iblock`[i+1].

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by argument `ifailv`)
            each failed to converge in five iterations. The current
            iterate after five iterations is stored in the corresponding
            column of `z`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zstein'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, w[n].
        utils._EngineIntArrayType.c_type, # Input, iblock[n].
        utils._EngineIntArrayType.c_type, # Input, isplit[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, m].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, ifailv[m].
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not (_m_py >= 0 and _m_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= m and m <= n.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    for _vl_i_py in range(0, (_m_py-1)-1+1):
        if utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py,)) == utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py+1,)):
            if not utils._EngineFloat64ArrayType.get_element(w, {'fun_name': fun_name, 'entity': 'w'}, (_vl_i_py,)) <= utils._EngineFloat64ArrayType.get_element(w, {'fun_name': fun_name, 'entity': 'w'}, (_vl_i_py+1,)):
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        utils._lapack_constraint_err_text('w[i] <= w[i+1].'),
                    ],
                    errcodes=(-5,),
                    entity='w',
                ).manage()
    _iblock = utils._EngineIntArrayType(
        iblock, {'fun_name': fun_name, 'entity': 'iblock'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    for _vl_i_py in range(0, (_m_py-1)-1+1):
        if not utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py,)) <= utils._EngineIntArrayType.get_element(iblock, {'fun_name': fun_name, 'entity': 'iblock'}, (_vl_i_py+1,)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iblock[i] <= iblock[i+1].'),
                ],
                errcodes=(-6,),
                entity='iblock',
            ).manage()
    _isplit = utils._EngineIntArrayType(
        isplit, {'fun_name': fun_name, 'entity': 'isplit'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _ifailv_cls = utils._EngineIntArrayType(
        ifailv, {'fun_name': fun_name, 'entity': 'ifailv'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _ifailv = _ifailv_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _d,
        _e,
        _m,
        _w,
        _iblock,
        _isplit,
        _z,
        _ldz,
        _work,
        _iwork,
        _ifailv,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} eigenvectors (as indicated by argument ifailv)'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'each failed to converge in five iterations. The current iterate after five',
                    'iterations is stored in the corresponding column of z.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _z_cls.refresh_py(_z)
        _ifailv_cls.refresh_py(_ifailv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zstegr(jobz, erange, n, d, e, vl, vu, il, iu, w, z, isuppz):
    u"""
    Computes selected eigenvalues and, optionally, the corresponding
    eigenvectors of a real symmetric tridiagonal matrix or a complex
    Hermitian matrix reduced to this form (Relatively Robust
    Representations).

    For full information please refer to the NAG Library document for
    f08jy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08jyf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        Indicates which eigenvalues should be returned.

        `erange` = 'A'

          All eigenvalues will be found.

        `erange` = 'V'

          All eigenvalues in the half-open interval (`vl`,`vu`] will be
          found.

        `erange` = 'I'

          The `il`th through `iu`th eigenvectors will be found.

    n : int
        n, the order of the matrix T.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the tridiagonal matrix T.

        On exit: `d` is overwritten.

    e : float, shape (`n`), modified in place
        On entry: `e`[0] to `e`[`n`-2] are the subdiagonal elements of
        the tridiagonal matrix T. `e`[`n`-1] need not be set.

        On exit: `e` is overwritten.

    vl : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, est_m), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', then if no exception or warning is
        raised, the columns of `z` contain the orthonormal eigenvectors
        of the matrix T, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    isuppz : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `jobz`='V': 2*max(1, est_m); otherwise: 0.

        On exit: the support of the eigenvectors in Z, i.e., the indices
        indicating the nonzero elements in Z. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1].

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: 1 <= `il` <= `iu` <= `n`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: `il` = 1 and `iu` = 0.

        (`errno` (i > 0) and (i = 1))
            The dqds algorithm failed to converge.

        (`errno` (i > 0) and (i = 2))
            Inverse iteration failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zstegr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08jyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Output, isuppz[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py,),
    )
    _e = _e_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-7,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if (_erange_py in ('I', 'i')) and _n_py > 0:
        if not (_il_py >= 1 and _il_py <= _iu_py and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= il and il <= iu and iu <= n.'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    elif (_erange_py in ('I', 'i')) and _n_py == 0:
        if not (_il_py == 1 and _iu_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il = 1.'),
                ],
                errcodes=(-9,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType.c_type(0.0)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz_cls = utils._EngineIntArrayType(
        isuppz, {'fun_name': fun_name, 'entity': 'isuppz'},
        exp_shape=exp_shape_isuppz,
        check_contents=False,
    )
    _isuppz = _isuppz_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _n,
        _d,
        _e,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _isuppz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The dqds algorithm failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 0 and _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inverse iteration failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _isuppz_cls.refresh_py(_isuppz)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dgelss(m, n, nrhs, a, b, s, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition.

    For full information please refer to the NAG Library document for
    f08ka

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kaf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the first min(m, n) rows of A are overwritten with its
        right singular vectors, stored row-wise.

    b : float, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: `b` is overwritten by the n by r solution matrix X. If
        m >= n and `rank` = n, the residual sum of squares for the
        solution in the ith column is given by the sum of squares of
        elements n+1,...,m in that column.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A in decreasing order.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgelss'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _s,
        _rcond,
        _rank,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm for computing the SVD failed to converge;',
                    '{:s} off-diagonal elements of an intermediate bidiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _s_cls.refresh_py(_s)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def dgesvd(jobu, jobvt, m, n, a, s, u, vt, workb):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors.

    For full information please refer to the NAG Library document for
    f08kb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kbf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'A'

          All m columns of U are returned in array `u`.

        `jobu` = 'S'

          The first min(m, n) columns of U (the left singular vectors)
          are returned in the array `u`.

        `jobu` = 'O'

          The first min(m, n) columns of U (the left singular vectors)
          are overwritten on the array `a`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'A'

          All n rows of V^T are returned in the array `vt`.

        `jobvt` = 'S'

          The first min(m, n) rows of V^T (the right singular vectors)
          are returned in the array `vt`.

        `jobvt` = 'O'

          The first min(m, n) rows of V^T (the right singular vectors)
          are overwritten on the array `a`.

        `jobvt` = 'N'

          No rows of V^T (no right singular vectors) are computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobu` = 'O', `a` is overwritten with the first
        min(m, n) columns of U (the left singular vectors, stored
        column-wise).

        If `jobvt` = 'O', `a` is overwritten with the first min(m, n)
        rows of V^T (the right singular vectors, stored row-wise).

        If `jobu` != 'O' and `jobvt` != 'O', the contents of `a` are
        destroyed.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='A': `m`; if `jobu`='S': `m`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='A': `m`; if `jobu`='S':
        min(`m`, `n`); otherwise: 1.

        On exit: if `jobu` = 'A', `u` contains the m by m orthogonal
        matrix U.

        If `jobu` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobu` = 'N' or 'O', `u` is not referenced.

    vt : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvt`='A': `n`; if `jobvt`='S':
        min(`m`, `n`); otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvt`='A': `n`; if `jobvt`='S': `n`;
        otherwise: 1.

        On exit: if `jobvt` = 'A', `vt` contains the n by n orthogonal
        matrix V^T.

        If `jobvt` = 'S', `vt` contains the first min(m, n) rows of V^T
        (the right singular vectors, stored row-wise).

        If `jobvt` = 'N' or 'O', `vt` is not referenced.

    workb : float, shape (min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `workb`[1:min(`m`, `n`)] contains the
        unconverged superdiagonal elements of an upper bidiagonal matrix
        B whose diagonal is in `s` (not necessarily sorted). B
        satisfies A = UBV^T, so it has the same singular values as A,
        and singular vectors related by U and V^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` != 'O' or `jobu` != 'O'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If ``dgesvd`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgesvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobvt(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineFloat64ArrayType.c_type, # Output, work[min(m, n)].
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvt.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobvt = utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    if not (_jobvt_py not in ('O', 'o') or _jobu_py not in ('O', 'o')):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt not in (\'O\', \'o\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('A', 'a'):
        _u_dim1 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('A', 'a'):
        _u_dim2 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobvt_py in ('A', 'a'):
        _vt_dim1 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('A', 'a'):
        _vt_dim2 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineFloat64ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    utils._EngineFloat64ArrayType(
        workb, {'fun_name': fun_name, 'entity': 'workb'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _work = utils._EngineFloat64ArrayType.empty_ctypes(min(_m_py, _n_py))
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobvt,
        _m,
        _n,
        _a,
        _lda,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _info,
        _jobu._length_,
        _jobvt._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If dgesvd did not converge, errno specifies how many superdiagonals of an',
                    'intermediate bidiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        _u_cls.refresh_py(_u)
        _vt_cls.refresh_py(_vt)
        workb[:] = utils._EngineFloat64ArrayType.from_pointer(_work, (min(_m_py, _n_py),))
        utils._unlink_pointer(_work)
        return_data = None
    _err_class.manage()
    return return_data

def dgelsd(m, n, nrhs, a, b, s, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08kc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kcf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n coefficient matrix A.

        On exit: the contents of `a` are destroyed.

    b : float, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: `b` is overwritten by the n by r solution matrix X. If
        m >= n and `rank` = n, the residual sum of squares for the
        solution in the ith column is given by the sum of squares of
        elements n+1,...,m in that column.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A in decreasing order.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgelsd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _s,
        _rcond,
        _rank,
        _work,
        _lwork,
        _iwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm for computing the SVD failed to converge;',
                    '{:s} off-diagonal elements of an intermediate bidiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _s_cls.refresh_py(_s)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def dgesdd(jobz, m, n, a, s, u, vt):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08kd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kdf.html

    Parameters
    ----------
    jobz : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobz` = 'A'

          All m columns of U and all n rows of V^T are returned in the
          arrays `u` and `vt`.

        `jobz` = 'S'

          The first min(m, n) columns of U and the first min(m, n) rows
          of V^T are returned in the arrays `u` and `vt`.

        `jobz` = 'O'

          If `m` >= `n`, the first n columns of U are overwritten on the
          array `a` and all rows of V^T are returned in the array `vt`.
          Otherwise, all columns of U are returned in the array `u` and
          the first m rows of V^T are overwritten in the array `vt`.

        `jobz` = 'N'

          No columns of U or rows of V^T are computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobz` = 'O', `a` is overwritten with the first n
        columns of U (the left singular vectors, stored column-wise) if
        `m` >= `n`; `a` is overwritten with the first m rows of V^T (the
        right singular vectors, stored row-wise) otherwise.

        If `jobz` != 'O', the contents of `a` are destroyed.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`<`n`): `m`; if `jobz`='S': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`<`n`): `m`; if `jobz`='S': min(`m`, `n`); otherwise: 1.

        On exit:

        If `jobz` = 'A' or `jobz` = 'O' and `m` < `n`, `u` contains the
        m by m orthogonal matrix U.

        If `jobz` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobz` = 'O' and `m` >= `n`, or `jobz` = 'N', `u` is not
        referenced.

    vt : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`>=`n`): `n`; if `jobz`='S': min(`m`, `n`); otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`>=`n`): `n`; if `jobz`='S': `n`; otherwise: 1.

        On exit: if `jobz` = 'A' or `jobz` = 'O' and `m` >= `n`, `vt`
        contains the n by n orthogonal matrix V^T.

        If `jobz` = 'S', `vt` contains the first min(m, n) rows of V^T
        (the right singular vectors, stored row-wise).

        If `jobz` = 'O' and `m` < `n`, or `jobz` = 'N', `vt` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            ``dgesdd`` did not converge, the updating process failed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgesdd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim1 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim2 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim1 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim2 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineFloat64ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _m,
        _n,
        _a,
        _lda,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _iwork,
        _info,
        _jobz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'dgesdd did not converge, the updating process failed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        _u_cls.refresh_py(_u)
        _vt_cls.refresh_py(_vt)
        return_data = None
    _err_class.manage()
    return return_data

def dgebrd(m, n, a, d, e, tauq, taup):
    u"""
    Performs an orthogonal reduction of real general rectangular matrix
    to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ke

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kef.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the diagonal and first superdiagonal are
        overwritten by the upper bidiagonal matrix B, elements below the
        diagonal are overwritten by details of the orthogonal matrix Q
        and elements above the first superdiagonal are overwritten by
        details of the orthogonal matrix P.

        If m < n, the diagonal and first subdiagonal are overwritten by
        the lower bidiagonal matrix B, elements below the first
        subdiagonal are overwritten by details of the orthogonal matrix
        Q and elements above the diagonal are overwritten by details of
        the orthogonal matrix P.

    d : float, shape (min(`m`, `n`)), modified in place
        On exit: the diagonal elements of the bidiagonal matrix B.

    e : float, shape (min(`m`, `n`)-1), modified in place
        On exit: the off-diagonal elements of the bidiagonal matrix B.

    tauq : float, shape (min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix Q.

    taup : float, shape (min(`m`, `n`)), modified in place
        On exit: further details of the orthogonal matrix P.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgebrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08keft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, d[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, e[min(m, n)-1].
        utils._EngineFloat64ArrayType.c_type, # Output, tauq[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, taup[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_m_py, _n_py)-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tauq_cls = utils._EngineFloat64ArrayType(
        tauq, {'fun_name': fun_name, 'entity': 'tauq'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tauq = _tauq_cls.to_ctypes(can_reference_contents=False)
    _taup_cls = utils._EngineFloat64ArrayType(
        taup, {'fun_name': fun_name, 'entity': 'taup'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _taup = _taup_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _d,
        _e,
        _tauq,
        _taup,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tauq_cls.refresh_py(_tauq)
        _taup_cls.refresh_py(_taup)
        return_data = None
    _err_class.manage()
    return return_data

def dorgbr(vect, m, n, k, a, tau):
    u"""
    Generates an orthogonal transformation matrices from reduction to
    bidiagonal form determined by ``dgebrd``.

    For full information please refer to the NAG Library document for
    f08kf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kff.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the orthogonal matrix Q or P^T is generated.

        `vect` = 'Q'

          Q is generated.

        `vect` = 'P'

          P^T is generated.

    m : int
        m, the number of rows of the orthogonal matrix Q or P^T to be
        returned.

    n : int
        n, the number of columns of the orthogonal matrix Q or P^T to be
        returned.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgebrd``.

        On exit: the orthogonal matrix Q or P^T, or the leading rows or
        columns thereof, as specified by `vect`, `m` and `n`.

    tau : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `vect`='Q': max(1, min(`m`, `k`)); if `vect`='P':
        max(1, min(`n`, `k`)); otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``dgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= `k`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `m` = `n`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m` >= `k`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` = `m`.

        (`errno` -4)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorgbr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('Q', 'q', 'P', 'p'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'Q\', \'q\', \'P\', \'p\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-4,),
            entity='k',
        ).manage()
    if (_vect_py in ('Q', 'q')) and _m_py > _k_py:
        if not (_m_py >= _n_py and _n_py >= _k_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= n and n >= k.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('Q', 'q')) and _m_py <= _k_py:
        if not _m_py == _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m = n.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('P', 'p')) and _n_py > _k_py:
        if not (_n_py >= _m_py and _m_py >= _k_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= m and m >= k.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('P', 'p')) and _n_py <= _k_py:
        if not _n_py == _m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n = m.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _vect_py in ('Q', 'q'):
        _tau_dim1 = max(1, min(_m_py, _k_py))
    elif _vect_py in ('P', 'p'):
        _tau_dim1 = max(1, min(_n_py, _k_py))
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
        _vect._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormbr(vect, side, trans, m, n, k, a, tau, c):
    u"""
    Applies the orthogonal transformations from reduction to bidiagonal
    form determined by ``dgebrd``.

    For full information please refer to the NAG Library document for
    f08kg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kgf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q or Q^T or P or P^T is to be applied to C.

        `vect` = 'Q'

          Q or Q^T is applied to C.

        `vect` = 'P'

          P or P^T is applied to C.

    side : str, length 1
        Indicates how Q or Q^T or P or P^T is to be applied to C.

        `side` = 'L'

          Q or Q^T or P or P^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T or P or P^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or P or Q^T or P^T is to be applied to C.

        `trans` = 'N'

          Q or P is applied to C.

        `trans` = 'T'

          Q^T or P^T is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='Q': r; if `vect`='P': min(r,
        `k`); otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='Q': min(r, `k`); if
        `vect`='P': r; otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgebrd``.

    tau : float, shape (min(r, `k`))
        Further details of the elementary reflectors, as returned by
        ``dgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    c : float, shape (`m`, `n`), modified in place
        On entry: the matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or C^TQ or PC or
        P^TC or CP or C^TP as specified by `vect`, `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormbr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('Q', 'q', 'P', 'p'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'Q\', \'q\', \'P\', \'p\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-6,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _vl_r_py = _m_py
    elif _side_py in ('R', 'r'):
        _vl_r_py = _n_py
    else:
        _vl_r_py = 0
    if _vect_py in ('Q', 'q'):
        _a_dim1 = _vl_r_py
    elif _vect_py in ('P', 'p'):
        _a_dim1 = min(_vl_r_py, _k_py)
    else:
        _a_dim1 = 0
    if _vect_py in ('Q', 'q'):
        _a_dim2 = min(_vl_r_py, _k_py)
    elif _vect_py in ('P', 'p'):
        _a_dim2 = _vl_r_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_vl_r_py, _k_py),),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _vect._length_,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgejsv(joba, jobu, jobv, jobr, jobt, jobp, m, n, a, sva, u, v, workb, iworkb):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors
    (preconditioned Jacobi).

    For full information please refer to the NAG Library document for
    f08kh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08khf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the form of pivoting for the QR factorization stage;
        whether an estimate of the condition number of the scaled matrix
        is required; and the form of rank reduction that is performed.

        `joba` = 'C'

          The initial QR factorization of the input matrix is performed
          with column pivoting; no estimate of condition number is
          computed; and, the rank is reduced by only the underflowed
          part of the triangular factor R. This option works well (high
          relative accuracy) if A can be written in the form A = BD,
          with well-conditioned B and arbitrary diagonal matrix D. The
          accuracy cannot be spoiled by column scaling. The accuracy of
          the computed output depends on the condition of B, and the
          procedure attempts to achieve the best theoretical accuracy.

        `joba` = 'E'

          Computation as with `joba` = 'C' with an additional estimate
          of the condition number of B. It provides a realistic error
          bound.

        `joba` = 'F'

          The initial QR factorization of the input matrix is performed
          with full row and column pivoting; no estimate of condition
          number is computed; and, the rank is reduced by only the
          underflowed part of the triangular factor R. If A = D_1*C*D_2
          with ill-conditioned diagonal scalings D_1, D_2, and
          well-conditioned matrix C, this option gives higher accuracy
          than the `joba` = 'C' option. If the structure of the input
          matrix is not known, and relative accuracy is desirable, then
          this option is advisable.

        `joba` = 'G'

          Computation as with `joba` = 'F' with an additional estimate
          of the condition number of B, where A = DB (i.e., B = C*D_2).
          If A has heavily weighted rows, then using this condition
          number gives too pessimistic an error bound.

        `joba` = 'A'

          Computation as with `joba` = 'C' except in the treatment of
          rank reduction. In this case, small singular values are to be
          considered as noise and, if found, the matrix is treated as
          numerically rank deficient. The computed SVD, A = U Sigma V^T,
          is such that the relative residual norm (when comparing
          against A) is of the order O(m)*epsilon, where epsilon is
          machine precision. This gives the procedure licence to discard
          (set to zero) all singular values below `n`*epsilon*||A||.

        `joba` = 'R'

          Similar to `joba` = 'A'. The rank revealing property of the
          initial QR factorization is used to reveal (using the upper
          triangular factor) a gap, sigma_{r+1} < epsilonsigma_r, in
          which case the numerical rank is declared to be r. The SVD is
          computed with absolute error bounds, but more accurately than
          with `joba` = 'A'.

    jobu : str, length 1
        Specifies options for computing the left singular vectors U.

        `jobu` = 'U'

          The first n left singular vectors (columns of U) are computed
          and returned in the array `u`.

        `jobu` = 'F'

          All m left singular vectors are computed and returned in the
          array `u`.

        `jobu` = 'W'

          No left singular vectors are computed, but the array `u` (with
          ldu >= `m` and second dimension at least `n`) is available as
          workspace for computing right singular values. See the
          description of `u`.

        `jobu` = 'N'

          No left singular vectors are computed. `u` is not referenced
          when `jobv` = 'W' or 'N'.

    jobv : str, length 1
        Specifies options for computing the right singular vectors V.

        `jobv` = 'V'

          The n right singular vectors (columns of V) are computed and
          returned in the array `v`; Jacobi rotations are not explicitly
          accumulated.

        `jobv` = 'J'

          The n right singular vectors (columns of V) are computed and
          returned in the array `v`, but they are computed as the
          product of Jacobi rotations. This option is allowed only if
          `jobu` = 'U' or 'F', i.e., in computing the full SVD.

          This is equivalent to multiplying the input matrix, on the
          right, by the matrix V.

        `jobv` = 'W'

          No right singular values are computed, but the array `v` (with
          ldv >= `n` and second dimension at least `n`) is available as
          workspace for computing left singular values. See the
          description of `v`.

        `jobv` = 'N'

          No right singular vectors are computed. `v` is not referenced
          when `jobu` = 'W' or 'N' or `jobt` = 'N' or `m` != `n`.

    jobr : str, length 1
        Specifies the conditions under which columns of A are to be set
        to zero. This effectively specifies a lower limit on the range
        of singular values; any singular values below this limit are
        (through column zeroing) set to zero. If A != 0 is scaled so
        that the largest column (in the Euclidean norm) of cA is equal
        to the square root of the overflow threshold, then `jobr` allows
        the function to kill columns of A whose norm in cA is less than
        sqrt(sfmin) (for `jobr` = 'R'), or less than sfmin/epsilon
        (otherwise). sfmin is the safe range parameter, as returned by
        function ``machine.real_safe``.

        `jobr` = 'N'

          Only set to zero those columns of A for which the norm of
          corresponding column of cA < sfmin/epsilon, that is, those
          columns that are effectively zero (to machine precision)
          anyway. If the condition number of A is greater than the
          overflow threshold lambda, where lambda is the value returned
          by ``machine.real_largest``, you are recommended to use
          function ``dgesvj``.

        `jobr` = 'R'

          Set to zero those columns of A for which the norm of the
          corresponding column of cA < sqrt(sfmin). This approximately
          represents a restricted range for sigma(cA) of
          [sqrt(sfmin),sqrt(lambda)].

        For computing the singular values in the full range from the
        safe minimum up to the overflow threshold use ``dgesvj``.

    jobt : str, length 1
        Specifies, in the case n = m, whether the function is permitted
        to use the transpose of A for improved efficiency. If the matrix
        is square, then the procedure may use A^T if it seems to be
        better with respect to convergence. If the matrix is not square,
        `jobt` is ignored. The decision is based on two values of
        entropy over the adjoint orbit of A^TA. See the descriptions of
        work[5] and work[6].

        `jobt` = 'T'

          If n = m, perform an entropy test and, if the test indicates
          possibly faster convergence of the Jacobi process when using
          A^T, then form the transpose A^T. If A is replaced with A^T,
          then the row pivoting is included automatically.

        `jobt` = 'N'

          No entropy test and no transposition is performed.

        The option `jobt` = 'T' can be used to compute only the singular
        values, or the full SVD (U, Sigma and V).

        In the case where only one set of singular vectors (U or V) is
        required, the caller must still provide both `u` and `v`, as one
        of the matrices is used as workspace if the matrix A is
        transposed.

        See the descriptions of `u` and `v`.

    jobp : str, length 1
        Specifies whether the function should be allowed to introduce
        structured perturbations to drown denormalized numbers. For
        details see New fast and accurate Jacobi SVD Algorithm I and II.
        For the sake of simplicity, these perturbations are included
        only when the full SVD or only the singular values are
        requested.

        `jobp` = 'P'

          Introduce perturbation if A is found to be very badly scaled
          (introducing denormalized numbers).

        `jobp` = 'N'

          Do not perturb.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the contents of `a` are overwritten.

    sva : float, shape (`n`), modified in place
        On exit: the, possibly scaled, singular values of A.

        The singular values of A are sigma_i = alpha*`sva`[i-1], for i =
        1,2,...,n, where alpha = work[0]/work[1].

        Normally alpha = 1 and no scaling is required to obtain the
        singular values.

        However, if the largest singular value of A overflows or if
        small singular values have been saved from underflow by scaling
        the input matrix A, then alpha != 1.

        If `jobr` = 'R', then some of the singular values may be
        returned as exact zeros because they are below the numerical
        rank threshold or are denormalized numbers.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='F': `m`; if `jobu` in ('U',
        'W'): `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='F': `m`; if `jobu` in ('U',
        'W'): `n`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the m by n matrix of left
        singular vectors.

        If `jobu` = 'F', `u` contains the m by m matrix of left singular
        vectors, including an orthonormal basis of the orthogonal
        complement of Range(A).

        `u` is not referenced when `jobu` = 'W' or 'N' and one of the
        following is satisfied:

        - `jobv` = 'W' or 'N', or

        - `n` = 1, or

        - A is the zero matrix.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'J', 'W'): `n`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'J', 'W'): `n`;
        otherwise: 1.

        On exit: if `jobv` = 'V' or 'J', `v` contains the n by n matrix
        of right singular vectors.

        `v` is not referenced when `jobv` = 'W' or 'N' and one of the
        following is satisfied:

        - `jobu` = 'U' or 'F' and `jobt` = 'T', or

        - `n` = 1, or

        - A is the zero matrix.

    workb : float, shape (7), modified in place
        On exit: contains information about the completed job.

        `workb`[0]

          alpha = `workb`[0]/`workb`[1] is the scaling factor such that
          sigma_i = alpha*`sva`[i-1], for i = 1,2,...,n are the computed
          singular values of A. (See the description of `sva`.)

        `workb`[1]

          See the description of `workb`[0].

        `workb`[2]

          sconda, an estimate for the condition number of column
          equilibrated A (if `joba` = 'E' or 'G'). sconda is an estimate
          of sqrt((||(R^TR)^-1||_1)). It is computed using
          ``lapacklin.dpocon``. It satisfies n^(-1/4)*sconda <=
          ||R^-1||_2 <= n^(1/4)*sconda where R is the triangular factor
          from the QR factorization of A. However, if R is truncated and
          the numerical rank is determined to be strictly smaller than
          n, sconda is returned as -1, thus indicating that the smallest
          singular values might be lost.

        If full SVD is needed, and you are familiar with the details of
        the method, the following two condition numbers are useful for
        the analysis of the algorithm.

        `workb`[3]

          An estimate of the scaled condition number of the triangular
          factor in the first QR factorization.

        `workb`[4]

          An estimate of the scaled condition number of the triangular
          factor in the second QR factorization.

        The following two parameters are computed if `jobt` = 'T'.

        `workb`[5]

          The entropy of A^TA: this is the Shannon entropy of
          diag(A^TA)/trace(A^TA) taken as a point in the probability
          simplex.

        `workb`[6]

          The entropy of AA^T.

    iworkb : None or int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `n`>0 and `m`>0: 3; otherwise: 0.

        On exit, if not None on entry: contains information about the
        completed job.

        `iworkb`[0]

          The numerical rank of A determined after the initial QR
          factorization with pivoting. See the descriptions of `joba`
          and `jobr`.

        `iworkb`[1]

          The number of computed nonzero singular values.

        `iworkb`[2]

          If nonzero, a warning message. If `iworkb`[2] = 1, then some
          of the column norms of A were denormalized (tiny) numbers. The
          requested high accuracy is not warranted by the data.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'C', 'E', 'F', 'G', 'A' or 'R'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'F', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'J', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` != 'J'.

        (`errno` -4)
            On entry, error in parameter `jobr`.

            Constraint: `jobr` = 'N' or 'R'.

        (`errno` -5)
            On entry, error in parameter `jobt`.

            Constraint: `jobt` = 'T' or 'N'.

        (`errno` -6)
            On entry, error in parameter `jobp`.

            Constraint: `jobp` = 'P' or 'N'.

        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``dgejsv`` did not converge in the allowed number of
            iterations (30). The computed values might be inaccurate.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgejsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08khft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, joba(len=1).
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobr(len=1).
        utils._BYREF_STRING, # Input, jobt(len=1).
        utils._BYREF_STRING, # Input, jobp(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, sva[n].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, work[7].
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of joba.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobp.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (v, 'v')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _joba = utils._EngineCharScalarType(
        joba, {'fun_name': fun_name, 'entity': 'joba'}, exp_chrlen=1,
    ).to_ctypes()
    _joba_py = utils._EngineCharScalarType.to_py(_joba)
    if _joba_py not in ('C', 'c', 'E', 'e', 'F', 'f', 'G', 'g', 'A', 'a', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('joba in (\'C\', \'c\', \'E\', \'e\', \'F\', \'f\', \'G\', \'g\', \'A\', \'a\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='joba',
        ).manage()
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'F', 'f', 'W', 'w', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'F\', \'f\', \'W\', \'w\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'J', 'j', 'W', 'w', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'J\', \'j\', \'W\', \'w\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobv',
        ).manage()
    if _jobu_py in ('W', 'w', 'N', 'n'):
        if not _jobv_py not in ('J', 'j'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('jobv not in (\'J\', \'j\').'),
                ],
                errcodes=(-3,),
                entity='jobv',
            ).manage()
    _jobr = utils._EngineCharScalarType(
        jobr, {'fun_name': fun_name, 'entity': 'jobr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobr_py = utils._EngineCharScalarType.to_py(_jobr)
    if _jobr_py not in ('N', 'n', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobr in (\'N\', \'n\', \'R\', \'r\').'),
            ],
            errcodes=(-4,),
            entity='jobr',
        ).manage()
    _jobt = utils._EngineCharScalarType(
        jobt, {'fun_name': fun_name, 'entity': 'jobt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobt_py = utils._EngineCharScalarType.to_py(_jobt)
    if _jobt_py not in ('T', 't', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobt in (\'T\', \'t\', \'N\', \'n\').'),
            ],
            errcodes=(-5,),
            entity='jobt',
        ).manage()
    _jobp = utils._EngineCharScalarType(
        jobp, {'fun_name': fun_name, 'entity': 'jobp'}, exp_chrlen=1,
    ).to_ctypes()
    _jobp_py = utils._EngineCharScalarType.to_py(_jobp)
    if _jobp_py not in ('P', 'p', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobp in (\'P\', \'p\', \'N\', \'n\').'),
            ],
            errcodes=(-6,),
            entity='jobp',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-8,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sva_cls = utils._EngineFloat64ArrayType(
        sva, {'fun_name': fun_name, 'entity': 'sva'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sva = _sva_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('F', 'f'):
        _u_dim1 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('F', 'f'):
        _u_dim2 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim2 = _n_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    utils._EngineFloat64ArrayType(
        workb, {'fun_name': fun_name, 'entity': 'workb'},
        exp_shape=(7,),
        check_contents=False,
    )
    _work = utils._EngineFloat64ArrayType.empty_ctypes(7)
    _lwork = utils._EngineIntScalarType.c_type(0)
    if _n_py > 0 and _m_py > 0:
        _iworkb_dim1 = 3
    else:
        _iworkb_dim1 = 0
    exp_shape_iworkb = (_iworkb_dim1,)
    utils._EngineIntArrayType(
        iworkb, {'fun_name': fun_name, 'entity': 'iworkb'},
        exp_shape=exp_shape_iworkb,
        check_contents=False,
        allow_none=True,
    )
    if _n_py > 0 and _m_py > 0:
        _iwork_dim1 = 3
    else:
        _iwork_dim1 = 0
    _iwork = utils._EngineIntArrayType.empty_ctypes(_iwork_dim1)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _joba,
        _jobu,
        _jobv,
        _jobr,
        _jobt,
        _jobp,
        _m,
        _n,
        _a,
        _lda,
        _sva,
        _u,
        _ldu,
        _v,
        _ldv,
        _work,
        _lwork,
        _iwork,
        _info,
        _joba._length_,
        _jobu._length_,
        _jobv._length_,
        _jobr._length_,
        _jobt._length_,
        _jobp._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'dgejsv did not converge in the allowed number of iterations (30).',
                    'The computed values might be inaccurate.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _sva_cls.refresh_py(_sva)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        workb[:] = utils._EngineFloat64ArrayType.from_pointer(_work, (7,))
        utils._unlink_pointer(_work)
        if iworkb is not None:
            iworkb[:] = utils._EngineIntArrayType.from_pointer(_iwork, (_iworkb_dim1,))
            utils._unlink_pointer(_iwork)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgesvj(joba, jobu, jobv, m, n, a, sva, mv, v, ctol, work):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors (fast
    Jacobi).

    For full information please refer to the NAG Library document for
    f08kj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kjf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the structure of matrix A.

        `joba` = 'L'

          The input matrix A is lower triangular.

        `joba` = 'U'

          The input matrix A is upper triangular.

        `joba` = 'G'

          The input matrix A is a general m by n matrix, `m` >= `n`.

    jobu : str, length 1
        Specifies whether to compute the left singular vectors and if so
        whether you want to control their numerical orthogonality
        threshold.

        `jobu` = 'U'

          The left singular vectors corresponding to the nonzero
          singular values are computed and returned in the leading
          columns of `a`. See more details in the description of `a`.
          The numerical orthogonality threshold is set to approximately
          tol = ctol*epsilon, where epsilon is the machine precision and
          ctol = sqrt(m).

        `jobu` = 'C'

          Analogous to `jobu` = 'U', except that you can control the
          level of numerical orthogonality of the computed left singular
          vectors. The orthogonality threshold is set to tol =
          `ctol`*epsilon. The option `jobu` = 'C' can be used if
          m*epsilon is a satisfactory orthogonality of the computed left
          singular vectors, so `ctol` = `m` could save a few sweeps of
          Jacobi rotations. See the descriptions of `a` and `ctol`.

        `jobu` = 'N'

          The matrix U is not computed. However, see the description of
          `a`.

    jobv : str, length 1
        Specifies whether and how to compute the right singular vectors.

        `jobv` = 'V'

          The matrix V is computed and returned in the array `v`.

        `jobv` = 'A'

          The Jacobi rotations are applied to the leading m_v by n part
          of the array `v`. In other words, the right singular vector
          matrix V is not computed explicitly, instead it is applied to
          an m_v by n matrix initially stored in the first `mv` rows of
          `v`.

        `jobv` = 'N'

          The matrix V is not computed and the array `v` is not
          referenced.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the matrix U containing the left singular vectors of A.

        If `jobu` = 'U' or 'C'

          if errno = 0

            rank(A) orthonormal columns of U are returned in the leading
            rank(A) columns of the array `a`. Here rank(A) <= `n` is the
            number of computed singular values of A that are above the
            safe range parameter, as returned by ``machine.real_safe``.
            The singular vectors corresponding to underflowed or zero
            singular values are not computed. The value of rank(A) is
            returned by rounding `work`[1] to the nearest whole number.
            Also see the descriptions of `sva` and `work`. The computed
            columns of U are mutually numerically orthogonal up to
            approximately tol = sqrt(m)*epsilon; or tol = `ctol`*epsilon
            (`jobu` = 'C'), where epsilon is the machine precision, see
            the description of `jobu`.

          if errno > 0

            ``dgesvj`` did not converge in 30 iterations (sweeps). In
            this case, the computed columns of U may not be orthogonal
            up to tol. The output U (stored in `a`), Sigma (given by the
            computed singular values in `sva`) and V is still a
            decomposition of the input matrix A in the sense that the
            residual ||A-alpha*U*Sigma*V^T||_2/||A||_2 is small, where
            alpha is the value returned in `work`[0].

        If `jobu` = 'N'

          if errno = 0

            Note that the left singular vectors are 'for free' in the
            one-sided Jacobi SVD algorithm. However, if only the
            singular values are needed, the level of numerical
            orthogonality of U is not an issue and iterations are
            stopped when the columns of the iterated matrix are
            numerically orthogonal up to approximately m*epsilon. Thus,
            on exit, `a` contains the columns of U scaled with the
            corresponding singular values.

          if errno > 0

            ``dgesvj`` did not converge in 30 iterations (sweeps).

    sva : float, shape (`n`), modified in place
        On exit: the, possibly scaled, singular values of A.

        If errno = 0

          The singular values of A are sigma_i = alpha`sva`[i-1], for i
          = 1,2,...,n, where alpha is the scale factor stored in
          `work`[0]. Normally alpha = 1, however, if some of the
          singular values of A might underflow or overflow, then alpha
          != 1 and the scale factor needs to be applied to obtain the
          singular values.

        If errno > 0

          ``dgesvj`` did not converge in 30 iterations and alpha*`sva`
          may not be accurate.

    mv : int
        If `jobv` = 'A', the product of Jacobi rotations is applied to
        the first m_v rows of `v`.

        If `jobv` != 'A', `mv` is ignored.

        See the description of `jobv`.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `n`; if `jobv`='A': `mv`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'A'): `n`; otherwise:
        1.

        On entry: if `jobv` = 'A', `v` must contain an m_v by n matrix
        to be premultiplied by the matrix V of right singular vectors.

        On exit: the right singular vectors of A.

        If `jobv` = 'V', `v` contains the n by n matrix of the right
        singular vectors.

        If `jobv` = 'A', `v` contains the product of the computed right
        singular vector matrix and the initial matrix in the array `v`.

        If `jobv` = 'N', `v` is not referenced.

    ctol : float
        If `jobu` = 'C', `ctol` = ctol, the threshold for convergence.
        The process stops if all columns of A are mutually orthogonal up
        to ctol*epsilon. It is required that ctol >= 1, i.e., it is not
        possible to force the function to obtain orthogonality below
        epsilon. ctol greater than 1/epsilon is meaningless, where
        epsilon is the machine precision.

    work : float, shape (6), modified in place
        On exit: contains information about the completed job.

        `work`[0]

          The scaling factor, alpha, such that sigma_i =
          alpha`sva`[i-1], for i = 1,2,...,n are the computed singular
          values of A. (See the description of `sva`.)

        `work`[1]

          nint(`work`[1]) gives the number of the computed nonzero
          singular values.

        `work`[2]

          nint(`work`[2]) gives the number of the computed singular
          values that are larger than the underflow threshold.

        `work`[3]

          nint(`work`[3]) gives the number of iterations (sweeps of
          Jacobi rotations) needed for numerical convergence.

        `work`[4]

          max_{i != j}|cos(A(:,i), A(:,j))| in the last iteration
          (sweep). This is useful information in cases when ``dgesvj``
          did not converge, as it can be used to estimate whether the
          output is still useful and for subsequent analysis.

        `work`[5]

          The largest absolute value over all sines of the Jacobi
          rotation angles in the last sweep. It can be useful for
          subsequent analysis.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'L', 'U' or 'G'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'C' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'A' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `mv`.

            Constraint: `mv` >= 0.

        (`errno` -12)
            On entry, error in parameter `ctol`.

            Constraint: `ctol` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``dgesvj`` did not converge in the allowed number of
            iterations (30), but its output might still be useful.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgesvj'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, joba(len=1).
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, sva[n].
        utils._BYREF_INT, # Input, mv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[6].
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of joba.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (v, 'v')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _joba = utils._EngineCharScalarType(
        joba, {'fun_name': fun_name, 'entity': 'joba'}, exp_chrlen=1,
    ).to_ctypes()
    _joba_py = utils._EngineCharScalarType.to_py(_joba)
    if _joba_py not in ('L', 'l', 'U', 'u', 'G', 'g'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('joba in (\'L\', \'l\', \'U\', \'u\', \'G\', \'g\').'),
            ],
            errcodes=(-1,),
            entity='joba',
        ).manage()
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'C', 'c', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'C\', \'c\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'A', 'a', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'A\', \'a\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobv',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sva_cls = utils._EngineFloat64ArrayType(
        sva, {'fun_name': fun_name, 'entity': 'sva'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sva = _sva_cls.to_ctypes(can_reference_contents=False)
    _mv = utils._EngineIntScalarType(
        mv, {'fun_name': fun_name, 'entity': 'mv'},
    ).to_ctypes()
    _mv_py = utils._EngineIntScalarType.to_py(_mv)
    if not _mv_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('mv >= 0.'),
            ],
            errcodes=(-9,),
            entity='mv',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _n_py
    elif _jobv_py in ('A', 'a'):
        _v_dim1 = _mv_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'A', 'a'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _work_cls = utils._EngineFloat64ArrayType(
        work, {'fun_name': fun_name, 'entity': 'work'},
        exp_shape=(6,),
        check_contents=False,
    )
    _work = _work_cls.to_ctypes(can_reference_contents=False)
    _ctol = utils._EngineFloat64ScalarType(
        ctol, {'fun_name': fun_name, 'entity': 'ctol'},
    ).to_ctypes()
    _ctol_py = utils._EngineFloat64ScalarType.to_py(_ctol)
    if _jobu_py in ('C', 'c'):
        if not _ctol_py >= 1.0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ctol >= 1.0.'),
                ],
                errcodes=(-12,),
                entity='work',
            ).manage()
    _work[0] = _ctol
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _joba,
        _jobu,
        _jobv,
        _m,
        _n,
        _a,
        _lda,
        _sva,
        _mv,
        _v,
        _ldv,
        _work,
        _lwork,
        _info,
        _joba._length_,
        _jobu._length_,
        _jobv._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'dgesvj did not converge in the allowed number of iterations (30), but its',
                    'output might still be useful.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _sva_cls.refresh_py(_sva)
        _v_cls.refresh_py(_v)
        _work_cls.refresh_py(_work)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgesvdx(jobu, jobvt, erange, m, n, a, vl, vu, il, iu, s, u, vt, workb, jfail):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a real general matrix, optionally computing the
    corresponding left and right singular vectors.

    For full information please refer to the NAG Library document for
    f08km

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kmf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'V'

          The `ns` columns of U, as specified by `erange`, are returned
          in array `u`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'V'

          The `ns` rows of V^T, as specified by `erange`, are returned
          in the array `vt`.

        `jobvt` = 'N'

          No rows of V^T (no right singular vectors) are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

          All singular values will be found.

        `erange` = 'V'

          All singular values in the half-open interval (`vl`,`vu`] will
          be found.

        `erange` = 'I'

          The `il`th through `iu`th singular values will be found.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobu` != 'N' and `jobvt` != 'N', the contents of
        `a` are destroyed.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='V': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='V': nsmax; otherwise: 0.

        On exit, if not None on entry: if `jobu` = 'V', `u` contains the
        first `ns` columns of U (the left singular vectors, stored
        column-wise).

        If `jobu` = 'N', `u` is not referenced.

    vt : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvt`='V': min(`m`, `n`); otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvt`='V': `n`; otherwise: 0.

        On exit, if not None on entry: if `jobvt` = 'V', `vt` contains
        the first `ns` rows of V^T (the right singular vectors, stored
        row-wise).

        If `jobvt` = 'N', `vt` is not referenced.

    workb : float, shape (min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `workb`[1:min(`m`, `n`)] contains the
        unconverged superdiagonal elements of an upper bidiagonal matrix
        B whose diagonal is in `s` (not necessarily sorted). B satisfies
        A = UBV^T, so it has the same singular values as A, and singular
        vectors related by U and V^T.

    jfail : int, shape (2*min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `jfail` contains, in its first k
        nonzero elements, the indices of the k eigenvectors (associated
        with a left or right singular vector, see ``dbdsvdx``) that
        failed to converge.

    Returns
    -------
    ns : int
        The total number of singular values found. 0 <= `ns` <= min(`m`,
        `n`).

        If `erange` = 'A', `ns` = min(`m`, `n`).

        If `erange` = 'I', `ns` = `iu`-`il`+1.

        If `erange` = 'V' then the value of `ns` is not known in advance
        and so an upper limit should be used when specifying the
        dimensions of array `u`, e.g., min(`m`, `n`).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            If ``dgesvdx`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgesvdx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kmft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobvt(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_INT, # Output, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineFloat64ArrayType.c_type, # Output, work[min(m, n)].
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[2*min(m, n)].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobvt = utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    if _erange_py in ('V', 'v'):
        if not _vl_py >= 0.0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('0.0 <= vl.'),
                ],
                errcodes=(-8,),
                entity='vl',
            ).manage()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-9,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, min(_m_py, _n_py))):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= il and il <= max(1, min(m, n)).'),
                ],
                errcodes=(-10,),
                entity='il',
            ).manage()
        if not (min(_il_py, _m_py, _n_py) <= _iu_py and _iu_py <= min(_m_py, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('min(il, m, n) <= iu and iu <= min(m, n).'),
                ],
                errcodes=(-11,),
                entity='iu',
            ).manage()
    _ns = utils._EngineIntScalarType.empty_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _erange_py in ('I', 'i'):
        _vl_nsmax_py = _iu_py-_il_py+1
    elif _erange_py in ('A', 'a', 'V', 'v'):
        _vl_nsmax_py = min(_m_py, _n_py)
    else:
        _vl_nsmax_py = 0
    if _jobu_py in ('V', 'v'):
        if u is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u'},
                'must not be None when jobu in (\'V\', \'v\')',
            )
    if _jobu_py in ('V', 'v'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('V', 'v'):
        _u_dim2 = _vl_nsmax_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobvt_py in ('V', 'v'):
        if vt is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vt'},
                'must not be None when jobvt in (\'V\', \'v\')',
            )
    if _jobvt_py in ('V', 'v'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('V', 'v'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 0
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineFloat64ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    utils._EngineFloat64ArrayType(
        workb, {'fun_name': fun_name, 'entity': 'workb'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _work = utils._EngineFloat64ArrayType.empty_ctypes(min(_m_py, _n_py))
    _lwork = utils._EngineIntScalarType.c_type(0)
    utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(2*min(_m_py, _n_py),),
        check_contents=False,
    )
    _iwork = utils._EngineIntArrayType.empty_ctypes(2*min(_m_py, _n_py))
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobvt,
        _erange,
        _m,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _ns,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _iwork,
        _info,
        _jobu._length_,
        _jobvt._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If dgesvdx did not converge, errno specifies how many superdiagonals of an',
                    'intermediate bidiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        if u is not None:
            _u_cls.refresh_py(_u)
        if vt is not None:
            _vt_cls.refresh_py(_vt)
        workb[:] = utils._EngineFloat64ArrayType.from_pointer(_work, (min(_m_py, _n_py),))
        utils._unlink_pointer(_work)
        jfail[:] = utils._EngineIntArrayType.from_pointer(_iwork, (2*min(_m_py, _n_py),))
        utils._unlink_pointer(_iwork)
        _ns_py = utils._EngineIntScalarType.to_py(_ns)
        return_data = _ns_py
    _err_class.manage()
    return return_data

def zgelss(m, n, nrhs, a, b, s, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition.

    For full information please refer to the NAG Library document for
    f08kn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08knf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the first min(m, n) rows of A are overwritten with its
        right singular vectors, stored row-wise.

    b : complex, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: `b` is overwritten by the n by r solution matrix X. If
        m >= n and `rank` = n, the residual sum of squares for the
        solution in the ith column is given by the sum of squares of the
        modulus of elements n+1,...,m in that column.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A in decreasing order.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgelss'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08knft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _s,
        _rcond,
        _rank,
        _work,
        _lwork,
        _rwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm for computing the SVD failed to converge;',
                    '{:s} off-diagonal elements of an intermediate bidiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _s_cls.refresh_py(_s)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def zgesvd(jobu, jobvt, m, n, a, s, u, vt, rworkb):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors.

    For full information please refer to the NAG Library document for
    f08kp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kpf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'A'

          All m columns of U are returned in array `u`.

        `jobu` = 'S'

          The first min(m, n) columns of U (the left singular vectors)
          are returned in the array `u`.

        `jobu` = 'O'

          The first min(m, n) columns of U (the left singular vectors)
          are overwritten on the array `a`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^H.

        `jobvt` = 'A'

          All n rows of V^H are returned in the array `vt`.

        `jobvt` = 'S'

          The first min(m, n) rows of V^H (the right singular vectors)
          are returned in the array `vt`.

        `jobvt` = 'O'

          The first min(m, n) rows of V^H (the right singular vectors)
          are overwritten on the array `a`.

        `jobvt` = 'N'

          No rows of V^H (no right singular vectors) are computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobu` = 'O', `a` is overwritten with the first
        min(m, n) columns of U (the left singular vectors, stored
        column-wise).

        If `jobvt` = 'O', `a` is overwritten with the first min(m, n)
        rows of V^H (the right singular vectors, stored row-wise).

        If `jobu` != 'O' and `jobvt` != 'O', the contents of `a` are
        destroyed.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='A': `m`; if `jobu`='S': `m`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='A': `m`; if `jobu`='S':
        min(`m`, `n`); otherwise: 1.

        On exit: if `jobu` = 'A', `u` contains the m by m unitary matrix
        U.

        If `jobu` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobu` = 'N' or 'O', `u` is not referenced.

    vt : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvt`='A': `n`; if `jobvt`='S':
        min(`m`, `n`); otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvt` in ('A', 'S'): `n`; otherwise:
        1.

        On exit: if `jobvt` = 'A', `vt` contains the n by n unitary
        matrix V^H.

        If `jobvt` = 'S', `vt` contains the first min(m, n) rows of V^H
        (the right singular vectors, stored row-wise).

        If `jobvt` = 'N' or 'O', `vt` is not referenced.

    rworkb : float, shape (min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `rworkb`[0:min(`m`, `n`)-1] contains
        the unconverged superdiagonal elements of an upper bidiagonal
        matrix B whose diagonal is in S (not necessarily sorted). B
        satisfies A = UBV^T, so it has the same singular values as A,
        and singular vectors related by U and V^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` != 'O' or `jobu` != 'O'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If ``zgesvd`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgesvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobvt(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[min(m, n)].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvt.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobvt = utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    if not (_jobvt_py not in ('O', 'o') or _jobu_py not in ('O', 'o')):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt not in (\'O\', \'o\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('A', 'a'):
        _u_dim1 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('A', 'a'):
        _u_dim2 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineComplex128ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobvt_py in ('A', 'a'):
        _vt_dim1 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('A', 'a', 'S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineComplex128ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    utils._EngineFloat64ArrayType(
        rworkb, {'fun_name': fun_name, 'entity': 'rworkb'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(min(_m_py, _n_py))
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobvt,
        _m,
        _n,
        _a,
        _lda,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobu._length_,
        _jobvt._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If zgesvd did not converge, errno specifies how many superdiagonals of an',
                    'intermediate bidiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        _u_cls.refresh_py(_u)
        _vt_cls.refresh_py(_vt)
        rworkb[:] = utils._EngineFloat64ArrayType.from_pointer(_rwork, (min(_m_py, _n_py),))
        utils._unlink_pointer(_rwork)
        return_data = None
    _err_class.manage()
    return return_data

def zgelsd(m, n, nrhs, a, b, s, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08kq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kqf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    nrhs : int
        r, the number of right-hand sides, i.e., the number of columns
        of the matrices B and X.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n coefficient matrix A.

        On exit: the contents of `a` are destroyed.

    b : complex, shape (max(`m`, `n`), `nrhs`), modified in place
        On entry: the m by r right-hand side matrix B.

        On exit: `b` is overwritten by the n by r solution matrix X. If
        m >= n and `rank` = n, the residual sum of squares for the
        solution in the ith column is given by the sum of squares of the
        modulus of elements n+1,...,m in that column.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A in decreasing order.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `nrhs`.

            Constraint: `nrhs` >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgelsd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._BYREF_FLOAT64, # Input, rcond.
        utils._BYREF_INT, # Output, rank.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    if not _nrhs_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nrhs >= 0.'),
            ],
            errcodes=(-3,),
            entity='nrhs',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(_m_py, _n_py), _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _rcond = utils._EngineFloat64ScalarType(
        rcond, {'fun_name': fun_name, 'entity': 'rcond'},
    ).to_ctypes()
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _s,
        _rcond,
        _rank,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm for computing the SVD failed to converge;',
                    '{:s} off-diagonal elements of an intermediate bidiagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _s_cls.refresh_py(_s)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def zgesdd(jobz, m, n, a, s, u, vt):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08kr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08krf.html

    Parameters
    ----------
    jobz : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobz` = 'A'

          All m columns of U and all n rows of V^H are returned in the
          arrays `u` and `vt`.

        `jobz` = 'S'

          The first min(m, n) columns of U and the first min(m, n) rows
          of V^H are returned in the arrays `u` and `vt`.

        `jobz` = 'O'

          If `m` >= `n`, the first n columns of U are overwritten on the
          array `a` and all rows of V^H are returned in the array `vt`.
          Otherwise, all columns of U are returned in the array `u` and
          the first m rows of V^H are overwritten in the array `vt`.

        `jobz` = 'N'

          No columns of U or rows of V^H are computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobz` = 'O', `a` is overwritten with the first n
        columns of U (the left singular vectors, stored column-wise) if
        `m` >= `n`; `a` is overwritten with the first m rows of V^H (the
        right singular vectors, stored row-wise) otherwise.

        If `jobz` != 'O', the contents of `a` are destroyed.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`<`n`): `m`; if `jobz`='S': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`<`n`): `m`; if `jobz`='S': min(`m`, `n`); otherwise: 1.

        On exit:

        If `jobz` = 'A' or `jobz` = 'O' and `m` < `n`, `u` contains the
        m by m unitary matrix U.

        If `jobz` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobz` = 'O' and `m` >= `n`, or `jobz` = 'N', `u` is not
        referenced.

    vt : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`>=`n`): `n`; if `jobz`='S': min(`m`, `n`); otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='A' or (`jobz`='O' and
        `m`>=`n`): `n`; if `jobz`='S': `n`; otherwise: 1.

        On exit: if `jobz` = 'A' or `jobz` = 'O' and `m` >= `n`, `vt`
        contains the n by n unitary matrix V^H.

        If `jobz` = 'S', `vt` contains the first min(m, n) rows of V^H
        (the right singular vectors, stored row-wise).

        If `jobz` = 'O' and `m` < `n`, or `jobz` = 'N', `vt` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            ``zgesdd`` did not converge, the updating process failed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgesdd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08krft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('A', 'a', 'S', 's', 'O', 'o', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'A\', \'a\', \'S\', \'s\', \'O\', \'o\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim1 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim2 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineComplex128ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim1 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim2 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineComplex128ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _m,
        _n,
        _a,
        _lda,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _info,
        _jobz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'zgesdd did not converge, the updating process failed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        _u_cls.refresh_py(_u)
        _vt_cls.refresh_py(_vt)
        return_data = None
    _err_class.manage()
    return return_data

def zgebrd(m, n, a, d, e, tauq, taup):
    u"""
    Performs a unitary reduction of complex general rectangular matrix
    to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ks

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ksf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m >= n, the diagonal and first superdiagonal are
        overwritten by the upper bidiagonal matrix B, elements below the
        diagonal are overwritten by details of the unitary matrix Q and
        elements above the first superdiagonal are overwritten by
        details of the unitary matrix P.

        If m < n, the diagonal and first subdiagonal are overwritten by
        the lower bidiagonal matrix B, elements below the first
        subdiagonal are overwritten by details of the unitary matrix Q
        and elements above the diagonal are overwritten by details of
        the unitary matrix P.

    d : float, shape (min(`m`, `n`)), modified in place
        On exit: the diagonal elements of the bidiagonal matrix B.

    e : float, shape (min(`m`, `n`)-1), modified in place
        On exit: the off-diagonal elements of the bidiagonal matrix B.

    tauq : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix Q.

    taup : complex, shape (min(`m`, `n`)), modified in place
        On exit: further details of the unitary matrix P.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgebrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ksft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, d[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, e[min(m, n)-1].
        utils._EngineComplex128ArrayType.c_type, # Output, tauq[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Output, taup[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_m_py, _n_py)-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _tauq_cls = utils._EngineComplex128ArrayType(
        tauq, {'fun_name': fun_name, 'entity': 'tauq'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _tauq = _tauq_cls.to_ctypes(can_reference_contents=False)
    _taup_cls = utils._EngineComplex128ArrayType(
        taup, {'fun_name': fun_name, 'entity': 'taup'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _taup = _taup_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _d,
        _e,
        _tauq,
        _taup,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _tauq_cls.refresh_py(_tauq)
        _taup_cls.refresh_py(_taup)
        return_data = None
    _err_class.manage()
    return return_data

def zungbr(vect, m, n, k, a, tau):
    u"""
    Generates unitary transformation matrices from the reduction to
    bidiagonal form determined by ``zgebrd``.

    For full information please refer to the NAG Library document for
    f08kt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ktf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the unitary matrix Q or P^H is generated.

        `vect` = 'Q'

          Q is generated.

        `vect` = 'P'

          P^H is generated.

    m : int
        m, the number of rows of the unitary matrix Q or P^H to be
        returned.

    n : int
        n, the number of columns of the unitary matrix Q or P^H to be
        returned.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgebrd``.

        On exit: the unitary matrix Q or P^H, or the leading rows or
        columns thereof, as specified by `vect`, `m` and `n`.

    tau : complex, shape (:)
        Note: the required length for this argument is determined as
        follows: if `vect`='Q': min(`m`, `k`); if `vect`='P': min(`n`,
        `k`); otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``zgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= `k`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `m` = `n`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= `m` >= `k`.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` = `m`.

        (`errno` -4)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zungbr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ktft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[*].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('Q', 'q', 'P', 'p'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'Q\', \'q\', \'P\', \'p\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-4,),
            entity='k',
        ).manage()
    if (_vect_py in ('Q', 'q')) and _m_py > _k_py:
        if not (_m_py >= _n_py and _n_py >= _k_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m >= n and n >= k.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('Q', 'q')) and _m_py <= _k_py:
        if not _m_py == _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('m = n.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('P', 'p')) and _n_py > _k_py:
        if not (_n_py >= _m_py and _m_py >= _k_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n >= m and m >= k.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    elif (_vect_py in ('P', 'p')) and _n_py <= _k_py:
        if not _n_py == _m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('n = m.'),
                ],
                errcodes=(-3,),
                entity='n',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _vect_py in ('Q', 'q'):
        _tau_dim1 = min(_m_py, _k_py)
    elif _vect_py in ('P', 'p'):
        _tau_dim1 = min(_n_py, _k_py)
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
        _vect._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmbr(vect, side, trans, m, n, k, a, tau, c):
    u"""
    Applies the unitary transformations from reduction to bidiagonal
    form determined by ``zgebrd``.

    For full information please refer to the NAG Library document for
    f08ku

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kuf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q or Q^H or P or P^H is to be applied to C.

        `vect` = 'Q'

          Q or Q^H is applied to C.

        `vect` = 'P'

          P or P^H is applied to C.

    side : str, length 1
        Indicates how Q or Q^H or P or P^H is to be applied to C.

        `side` = 'L'

          Q or Q^H or P or P^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H or P or P^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or P or Q^H or P^H is to be applied to C.

        `trans` = 'N'

          Q or P is applied to C.

        `trans` = 'C'

          Q^H or P^H is applied to C.

    m : int
        m, the number of rows of the matrix C.

    n : int
        n, the number of columns of the matrix C.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='Q': r; if `vect`='P': min(r,
        `k`); otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='Q': min(r, `k`); if
        `vect`='P': r; otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgebrd``.

    tau : complex, shape (min(r, `k`))
        Further details of the elementary reflectors, as returned by
        ``zgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or C^HQ or PC or
        P^HC or CP or C^HP as specified by `vect`, `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmbr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('Q', 'q', 'P', 'p'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'Q\', \'q\', \'P\', \'p\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-3,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    if not _k_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('k >= 0.'),
            ],
            errcodes=(-6,),
            entity='k',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _vl_r_py = _m_py
    elif _side_py in ('R', 'r'):
        _vl_r_py = _n_py
    else:
        _vl_r_py = 0
    if _vect_py in ('Q', 'q'):
        _a_dim1 = _vl_r_py
    elif _vect_py in ('P', 'p'):
        _a_dim1 = min(_vl_r_py, _k_py)
    else:
        _a_dim1 = 0
    if _vect_py in ('Q', 'q'):
        _a_dim2 = min(_vl_r_py, _k_py)
    elif _vect_py in ('P', 'p'):
        _a_dim2 = _vl_r_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(min(_vl_r_py, _k_py),),
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _side,
        _trans,
        _m,
        _n,
        _k,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _vect._length_,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgejsv(joba, jobu, jobv, jobr, jobt, jobp, m, n, a, sva, u, v, rworkb, iworkb):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors
    (preconditioned Jacobi).

    For full information please refer to the NAG Library document for
    f08kv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kvf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the form of pivoting for the QR factorization stage;
        whether an estimate of the condition number of the scaled matrix
        is required; and the form of rank reduction that is performed.

        `joba` = 'C'

          The initial QR factorization of the input matrix is performed
          with column pivoting; no estimate of condition number is
          computed; and, the rank is reduced by only the underflowed
          part of the triangular factor R. This option works well (high
          relative accuracy) if A can be written in the form A = BD,
          with well-conditioned B and arbitrary diagonal matrix D. The
          accuracy cannot be spoiled by column scaling. The accuracy of
          the computed output depends on the condition of B, and the
          procedure attempts to achieve the best theoretical accuracy.

        `joba` = 'E'

          Computation as with `joba` = 'C' with an additional estimate
          of the condition number of B. It provides a realistic error
          bound.

        `joba` = 'F'

          The initial QR factorization of the input matrix is performed
          with full row and column pivoting; no estimate of condition
          number is computed; and, the rank is reduced by only the
          underflowed part of the triangular factor R. If A = D_1*C*D_2
          with ill-conditioned diagonal scalings D_1, D_2, and
          well-conditioned matrix C, this option gives higher accuracy
          than the `joba` = 'C' option. If the structure of the input
          matrix is not known, and relative accuracy is desirable, then
          this option is advisable.

        `joba` = 'G'

          Computation as with `joba` = 'F' with an additional estimate
          of the condition number of B, where A = DB (i.e., B = C*D_2).
          If A has heavily weighted rows, then using this condition
          number gives too pessimistic an error bound.

        `joba` = 'A'

          Computation as with `joba` = 'C' except in the treatment of
          rank reduction. In this case, small singular values are to be
          considered as noise and, if found, the matrix is treated as
          numerically rank deficient. The computed SVD, A = U Sigma V^H,
          is such that the relative residual norm (when comparing
          against A) is of the order O(m)*epsilon, where epsilon is
          machine precision. This gives the procedure licence to discard
          (set to zero) all singular values below `n`*epsilon*||A||.

        `joba` = 'R'

          Similar to `joba` = 'A'. The rank revealing property of the
          initial QR factorization is used to reveal (using the upper
          triangular factor) a gap, sigma_{r+1} < epsilonsigma_r, in
          which case the numerical rank is declared to be r. The SVD is
          computed with absolute error bounds, but more accurately than
          with `joba` = 'A'.

    jobu : str, length 1
        Specifies options for computing the left singular vectors U.

        `jobu` = 'U'

          The first n left singular vectors (columns of U) are computed
          and returned in the array `u`.

        `jobu` = 'F'

          All m left singular vectors are computed and returned in the
          array `u`.

        `jobu` = 'W'

          No left singular vectors are computed, but the array `u` (with
          ldu >= `m` and second dimension at least `n`) is available as
          workspace for computing right singular values. See the
          description of `u`.

        `jobu` = 'N'

          No left singular vectors are computed. `u` is not referenced
          when `jobv` = 'W' or 'N'.

    jobv : str, length 1
        Specifies options for computing the right singular vectors V.

        `jobv` = 'V'

          The n right singular vectors (columns of V) are computed and
          returned in the array `v`; Jacobi rotations are not explicitly
          accumulated.

        `jobv` = 'J'

          The n right singular vectors (columns of V) are computed and
          returned in the array `v`, but they are computed as the
          product of Jacobi rotations. This option is allowed only if
          `jobu` = 'U' or 'F', i.e., in computing the full SVD.

          This is equivalent to multiplying the input matrix, on the
          right, by the matrix V.

        `jobv` = 'W'

          No right singular values are computed, but the array `v` (with
          ldv >= `n` and second dimension at least `n`) is available as
          workspace for computing left singular values. See the
          description of `v`.

        `jobv` = 'N'

          No right singular vectors are computed. `v` is not referenced
          when `jobu` = 'W' or 'N' or `jobt` = 'N' or `m` != `n`.

    jobr : str, length 1
        Specifies the conditions under which columns of A are to be set
        to zero. This effectively specifies a lower limit on the range
        of singular values; any singular values below this limit are
        (through column zeroing) set to zero. If A != 0 is scaled so
        that the largest column (in the Euclidean norm) of cA is equal
        to the square root of the overflow threshold, then `jobr` allows
        the function to kill columns of A whose norm in cA is less than
        sqrt(sfmin) (for `jobr` = 'R'), or less than sfmin/epsilon
        (otherwise). sfmin is the safe range parameter, as returned by
        function ``machine.real_safe``.

        `jobr` = 'N'

          Only set to zero those columns of A for which the norm of
          corresponding column of cA < sfmin/epsilon, that is, those
          columns that are effectively zero (to machine precision)
          anyway. If the condition number of A is greater than the
          overflow threshold lambda, where lambda is the value returned
          by ``machine.real_largest``, you are recommended to use
          function ``zgesvj``.

        `jobr` = 'R'

          Set to zero those columns of A for which the norm of the
          corresponding column of cA < sqrt(sfmin). This approximately
          represents a restricted range for sigma(cA) of
          [sqrt(sfmin),sqrt(lambda)].

        For computing the singular values in the full range from the
        safe minimum up to the overflow threshold use ``zgesvj``.

    jobt : str, length 1
        Specifies, in the case n = m, whether the function is permitted
        to use the conjugate transpose of A for improved efficiency. If
        the matrix is square, then the procedure may use A^H if it seems
        to be better with respect to convergence. If the matrix is not
        square, `jobt` is ignored. The decision is based on two values
        of entropy over the adjoint orbit of A^HA. See the descriptions
        of `rworkb`[5] and `rworkb`[6].

        `jobt` = 'T'

          If n = m, perform an entropy test and, if the test indicates
          possibly faster convergence of the Jacobi process when using
          A^H, then form the conjugate transpose A^H. If A is replaced
          with A^H, then the row pivoting is included automatically.

        `jobt` = 'N'

          No entropy test and no transposition is performed.

        The option `jobt` = 'T' can be used to compute only the singular
        values, or the full SVD (U, Sigma and V).

        In the case where only one set of singular vectors (U or V) is
        required, the caller must still provide both `u` and `v`, as one
        of the matrices is used as workspace if the matrix A is
        transposed.

        See the descriptions of `u` and `v`.

    jobp : str, length 1
        Specifies whether the function should be allowed to introduce
        structured perturbations to drown denormalized numbers. For
        details see New fast and accurate Jacobi SVD Algorithm I and II.
        For the sake of simplicity, these perturbations are included
        only when the full SVD or only the singular values are
        requested.

        `jobp` = 'P'

          Introduce perturbation if A is found to be very badly scaled
          (introducing denormalized numbers).

        `jobp` = 'N'

          Do not perturb.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the contents of `a` are overwritten.

    sva : float, shape (`n`), modified in place
        On exit: the, possibly scaled, singular values of A.

        The singular values of A are sigma_i = alpha*`sva`[i-1], for i =
        1,2,...,n, where alpha = `rworkb`[0]/`rworkb`[1].

        Normally alpha = 1 and no scaling is required to obtain the
        singular values.

        However, if the largest singular value of A overflows or if
        small singular values have been saved from underflow by scaling
        the input matrix A, then alpha != 1.

        If `jobr` = 'R', then some of the singular values may be
        returned as exact zeros because they are below the numerical
        rank threshold or are denormalized numbers.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='F': `m`; if `jobu` in ('U',
        'W'): `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='F': `m`; if `jobu` in ('U',
        'W'): `n`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the m by n matrix of left
        singular vectors.

        If `jobu` = 'F', `u` contains the m by m matrix of left singular
        vectors, including an orthonormal basis of the orthogonal
        complement of Range(A).

        `u` is not referenced when `jobu` = 'W' or 'N' and one of the
        following is satisfied:

        - `jobv` = 'W' or 'N', or

        - `n` = 1, or

        - A is the zero matrix.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'J', 'W'): `n`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'J', 'W'): `n`;
        otherwise: 1.

        On exit: if `jobv` = 'V' or 'J', `v` contains the n by n matrix
        of right singular vectors.

        `v` is not referenced when `jobv` = 'W' or 'N' and one of the
        following is satisfied:

        - `jobu` = 'U' or 'F' and `jobt` = 'T', or

        - `n` = 1, or

        - A is the zero matrix.

    rworkb : float, shape (7), modified in place
        On exit: contains information about the completed job.

        `rworkb`[0]

          alpha = `rworkb`[0]/`rworkb`[1] is the scaling factor such
          that sigma_i = alpha*`sva`[i-1], for i = 1,2,...,n, are the
          computed singular values of A. See the description of `sva`.

        `rworkb`[1]

          See the description of `rworkb`[0].

        `rworkb`[2]

          sconda, an estimate for the condition number of column
          equilibrated A (if `joba` = 'E' or 'G'). sconda is an estimate
          of sqrt((||(R^TR)^-1||_1)). It is computed using
          ``lapacklin.dpocon``. It satisfies n^(-1/4)*sconda <=
          ||R^-1||_2 <= n^(1/4)*sconda where R is the triangular factor
          from the QR factorization of A. However, if R is truncated and
          the numerical rank is determined to be strictly smaller than
          n, sconda is returned as -1, thus indicating that the smallest
          singular values might be lost.

        If the full SVD is needed, and you are familiar with the details
        of the method, the following two condition numbers are useful
        for the analysis of the algorithm.

        `rworkb`[3]

          An estimate of the scaled condition number of the triangular
          factor in the first QR factorization.

        `rworkb`[4]

          An estimate of the scaled condition number of the triangular
          factor in the second QR factorization.

        The following two parameters are computed if `jobt` = 'T'.

        `rworkb`[5]

          The entropy of A^TA: this is the Shannon entropy of
          diag(A^TA)/trace(A^TA) taken as a point in the probability
          simplex.

        `rworkb`[6]

          The entropy of AA^T.

    iworkb : None or int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `n`>0 and `m`>0: 3; otherwise: 0.

        On exit, if not None on entry: contains information about the
        completed job.

        `iworkb`[0]

          The numerical rank of A determined after the initial QR
          factorization with pivoting. See the descriptions of `joba`
          and `jobr`.

        `iworkb`[1]

          The number of computed nonzero singular values.

        `iworkb`[2]

          If nonzero, a warning message. If `iworkb`[2] = 1, then some
          of the column norms of A were denormalized (tiny) numbers. The
          requested high accuracy is not warranted by the data.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'C', 'E', 'F', 'G', 'A' or 'R'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'F', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'J', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` != 'J'.

        (`errno` -4)
            On entry, error in parameter `jobr`.

            Constraint: `jobr` = 'N' or 'R'.

        (`errno` -5)
            On entry, error in parameter `jobt`.

            Constraint: `jobt` = 'T' or 'N'.

        (`errno` -6)
            On entry, error in parameter `jobp`.

            Constraint: `jobp` = 'P' or 'N'.

        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``zgejsv`` did not converge in the allowed number of
            iterations (30). The computed values might be inaccurate.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgejsv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, joba(len=1).
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobr(len=1).
        utils._BYREF_STRING, # Input, jobt(len=1).
        utils._BYREF_STRING, # Input, jobp(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, sva[n].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Workspace, cwork.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[7].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of joba.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobp.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (v, 'v')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _joba = utils._EngineCharScalarType(
        joba, {'fun_name': fun_name, 'entity': 'joba'}, exp_chrlen=1,
    ).to_ctypes()
    _joba_py = utils._EngineCharScalarType.to_py(_joba)
    if _joba_py not in ('C', 'c', 'E', 'e', 'F', 'f', 'G', 'g', 'A', 'a', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('joba in (\'C\', \'c\', \'E\', \'e\', \'F\', \'f\', \'G\', \'g\', \'A\', \'a\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='joba',
        ).manage()
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'F', 'f', 'W', 'w', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'F\', \'f\', \'W\', \'w\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'J', 'j', 'W', 'w', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'J\', \'j\', \'W\', \'w\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobv',
        ).manage()
    if _jobu_py in ('W', 'w', 'N', 'n'):
        if not _jobv_py not in ('J', 'j'):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('jobv not in (\'J\', \'j\').'),
                ],
                errcodes=(-3,),
                entity='jobv',
            ).manage()
    _jobr = utils._EngineCharScalarType(
        jobr, {'fun_name': fun_name, 'entity': 'jobr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobr_py = utils._EngineCharScalarType.to_py(_jobr)
    if _jobr_py not in ('N', 'n', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobr in (\'N\', \'n\', \'R\', \'r\').'),
            ],
            errcodes=(-4,),
            entity='jobr',
        ).manage()
    _jobt = utils._EngineCharScalarType(
        jobt, {'fun_name': fun_name, 'entity': 'jobt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobt_py = utils._EngineCharScalarType.to_py(_jobt)
    if _jobt_py not in ('T', 't', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobt in (\'T\', \'t\', \'N\', \'n\').'),
            ],
            errcodes=(-5,),
            entity='jobt',
        ).manage()
    _jobp = utils._EngineCharScalarType(
        jobp, {'fun_name': fun_name, 'entity': 'jobp'}, exp_chrlen=1,
    ).to_ctypes()
    _jobp_py = utils._EngineCharScalarType.to_py(_jobp)
    if _jobp_py not in ('P', 'p', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobp in (\'P\', \'p\', \'N\', \'n\').'),
            ],
            errcodes=(-6,),
            entity='jobp',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-8,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sva_cls = utils._EngineFloat64ArrayType(
        sva, {'fun_name': fun_name, 'entity': 'sva'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sva = _sva_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('F', 'f'):
        _u_dim1 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('F', 'f'):
        _u_dim2 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim2 = _n_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _cwork = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    utils._EngineFloat64ArrayType(
        rworkb, {'fun_name': fun_name, 'entity': 'rworkb'},
        exp_shape=(7,),
        check_contents=False,
    )
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(7)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    if _n_py > 0 and _m_py > 0:
        _iworkb_dim1 = 3
    else:
        _iworkb_dim1 = 0
    exp_shape_iworkb = (_iworkb_dim1,)
    utils._EngineIntArrayType(
        iworkb, {'fun_name': fun_name, 'entity': 'iworkb'},
        exp_shape=exp_shape_iworkb,
        check_contents=False,
        allow_none=True,
    )
    if _n_py > 0 and _m_py > 0:
        _iwork_dim1 = 3
    else:
        _iwork_dim1 = 0
    _iwork = utils._EngineIntArrayType.empty_ctypes(_iwork_dim1)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _joba,
        _jobu,
        _jobv,
        _jobr,
        _jobt,
        _jobp,
        _m,
        _n,
        _a,
        _lda,
        _sva,
        _u,
        _ldu,
        _v,
        _ldv,
        _cwork,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _info,
        _joba._length_,
        _jobu._length_,
        _jobv._length_,
        _jobr._length_,
        _jobt._length_,
        _jobp._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'zgejsv did not converge in the allowed number of iterations (30).',
                    'The computed values might be inaccurate.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _sva_cls.refresh_py(_sva)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        rworkb[:] = utils._EngineFloat64ArrayType.from_pointer(_rwork, (7,))
        utils._unlink_pointer(_rwork)
        if iworkb is not None:
            iworkb[:] = utils._EngineIntArrayType.from_pointer(_iwork, (_iworkb_dim1,))
            utils._unlink_pointer(_iwork)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgesvj(joba, jobu, jobv, m, n, a, sva, mv, v, ctol, rwork):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors (fast
    Jacobi).

    For full information please refer to the NAG Library document for
    f08kw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kwf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the structure of matrix A.

        `joba` = 'L'

          The input matrix A is lower triangular.

        `joba` = 'U'

          The input matrix A is upper triangular.

        `joba` = 'G'

          The input matrix A is a general m by n matrix, `m` >= `n`.

    jobu : str, length 1
        Specifies whether to compute the left singular vectors and if so
        whether you want to control their numerical orthogonality
        threshold.

        `jobu` = 'U'

          The left singular vectors corresponding to the nonzero
          singular values are computed and returned in the leading
          columns of `a`. See more details in the description of `a`.
          The numerical orthogonality threshold is set to approximately
          tol = sqrt(m)*epsilon, where epsilon is the machine precision.

        `jobu` = 'C'

          Analogous to `jobu` = 'U', except that you can control the
          level of numerical orthogonality of the computed left singular
          vectors. The orthogonality threshold is set to tol =
          `ctol`*epsilon. The option `jobu` = 'C' can be used if
          m*epsilon is a satisfactory orthogonality of the computed left
          singular vectors, so `ctol` = `m` could save a few sweeps of
          Jacobi rotations. See the descriptions of `a` and `ctol`.

        `jobu` = 'N'

          The matrix U is not computed. However, see the description of
          `a`.

    jobv : str, length 1
        Specifies whether and how to compute the right singular vectors.

        `jobv` = 'V'

          The matrix V is computed and returned in the array `v`.

        `jobv` = 'A'

          The Jacobi rotations are applied to the leading m_v by n part
          of the array `v`. In other words, the right singular vector
          matrix V is not computed explicitly, instead it is applied to
          an m_v by n matrix initially stored in the first `mv` rows of
          `v`.

        `jobv` = 'N'

          The matrix V is not computed and the array `v` is not
          referenced.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: the matrix U containing the left singular vectors of A.

        If `jobu` = 'U' or 'C'

          if errno = 0

            rank(A) unitary columns of U are returned in the leading
            rank(A) columns of the array `a`. Here rank(A) <= `n` is the
            number of computed singular values of A that are above the
            safe range parameter, as returned by ``machine.real_safe``.
            The singular vectors corresponding to underflowed or zero
            singular values are not computed. The value of rank(A) is
            returned by rounding `rwork`[1] to the nearest whole number.
            Also see the descriptions of `sva` and `rwork`. The computed
            columns of U are mutually numerically unitary up to
            approximately tol = sqrt(m)*epsilon; or tol = `ctol`*epsilon
            (`jobu` = 'C'), where epsilon is the machine precision, see
            the description of `jobu`.

          if errno > 0

            ``zgesvj`` did not converge in 30 iterations (sweeps). In
            this case, the computed columns of U may not be unitary up
            to tol. The output U (stored in `a`), Sigma (given by the
            computed singular values in `sva`) and V is still a
            decomposition of the input matrix A in the sense that the
            residual ||A-alpha*U*Sigma*V^H||_2/||A||_2 is small, where
            alpha is the value returned in `rwork`[0].

        If `jobu` = 'N'

          if errno = 0

            Note that the left singular vectors are 'for free' in the
            one-sided Jacobi SVD algorithm. However, if only the
            singular values are needed, the level of numerical
            orthogonality of U is not an issue and iterations are
            stopped when the columns of the iterated matrix are
            numerically unitary up to approximately m*epsilon. Thus, on
            exit, `a` contains the columns of U scaled with the
            corresponding singular values.

          if errno > 0

            ``zgesvj`` did not converge in 30 iterations (sweeps).

    sva : float, shape (`n`), modified in place
        On exit: the, possibly scaled, singular values of A.

        If errno = 0

          The singular values of A are sigma_i = alpha*`sva`[i-1], for i
          = 1,2,...,n, where alpha is the scale factor stored in
          `rwork`[0]. Normally alpha = 1, however, if some of the
          singular values of A might underflow or overflow, then alpha
          != 1 and the scale factor needs to be applied to obtain the
          singular values.

        If errno > 0

          ``zgesvj`` did not converge in 30 iterations and alpha*`sva`
          may not be accurate.

    mv : int
        If `jobv` = 'A', the product of Jacobi rotations is applied to
        the first m_v rows of `v`.

        If `jobv` != 'A', `mv` is ignored.

        See the description of `jobv`.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `n`; if `jobv`='A': `mv`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'A'): `n`; otherwise:
        1.

        On entry: if `jobv` = 'A', `v` must contain an m_v by n matrix
        to be premultiplied by the matrix V of right singular vectors.

        On exit: the right singular vectors of A.

        If `jobv` = 'V', `v` contains the n by n matrix of the right
        singular vectors.

        If `jobv` = 'A', `v` contains the product of the computed right
        singular vector matrix and the initial matrix in the array `v`.

        If `jobv` = 'N', `v` is not referenced.

    ctol : float
        If `jobu` = 'C', `ctol` = ctol, the threshold for convergence.
        The process stops if all columns of A are mutually orthogonal up
        to ctol*epsilon. It is required that ctol >= 1, i.e., it is not
        possible to force the function to obtain orthogonality below
        epsilon. ctol greater than 1/epsilon is meaningless, where
        epsilon is the machine precision.

    rwork : float, shape (6), modified in place
        On exit: contains information about the completed job.

        `rwork`[0]

          The scaling factor, alpha, such that sigma_i =
          alpha*`sva`[i-1], for i = 1,2,...,n, are the computed singular
          values of A. (See the description of `sva`.)

        `rwork`[1]

          nint(`rwork`[1]) gives the number of the computed nonzero
          singular values.

        `rwork`[2]

          nint(`rwork`[2]) gives the number of the computed singular
          values that are larger than the underflow threshold.

        `rwork`[3]

          nint(`rwork`[3]) gives the number of iterations (sweeps of
          Jacobi rotations) needed for numerical convergence.

        `rwork`[4]

          max_{i != j}|cos(A(:,i), A(:,j))| in the last iteration
          (sweep). This is useful information in cases when ``zgesvj``
          did not converge, as it can be used to estimate whether the
          output is still useful and for subsequent analysis.

        `rwork`[5]

          The largest absolute value over all sines of the Jacobi
          rotation angles in the last sweep. It can be useful for
          subsequent analysis.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'L', 'U' or 'G'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'C' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'A' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `m` >= `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `mv`.

            Constraint: `mv` >= 0.

        (`errno` -14)
            On entry, error in parameter `ctol`.

            Constraint: `ctol` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``zgesvj`` did not converge in the allowed number of
            iterations (30), but its output might still be useful.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgesvj'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, joba(len=1).
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, sva[n].
        utils._BYREF_INT, # Input, mv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Workspace, cwork.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[6].
        utils._BYREF_INT, # Input, lrwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of joba.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (v, 'v')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _joba = utils._EngineCharScalarType(
        joba, {'fun_name': fun_name, 'entity': 'joba'}, exp_chrlen=1,
    ).to_ctypes()
    _joba_py = utils._EngineCharScalarType.to_py(_joba)
    if _joba_py not in ('L', 'l', 'U', 'u', 'G', 'g'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('joba in (\'L\', \'l\', \'U\', \'u\', \'G\', \'g\').'),
            ],
            errcodes=(-1,),
            entity='joba',
        ).manage()
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'C', 'c', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'C\', \'c\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'A', 'a', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'A\', \'a\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobv',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_m_py >= _n_py and _n_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= n and n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sva_cls = utils._EngineFloat64ArrayType(
        sva, {'fun_name': fun_name, 'entity': 'sva'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sva = _sva_cls.to_ctypes(can_reference_contents=False)
    _mv = utils._EngineIntScalarType(
        mv, {'fun_name': fun_name, 'entity': 'mv'},
    ).to_ctypes()
    _mv_py = utils._EngineIntScalarType.to_py(_mv)
    if not _mv_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('mv >= 0.'),
            ],
            errcodes=(-9,),
            entity='mv',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _n_py
    elif _jobv_py in ('A', 'a'):
        _v_dim1 = _mv_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'A', 'a'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _cwork = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork_cls = utils._EngineFloat64ArrayType(
        rwork, {'fun_name': fun_name, 'entity': 'rwork'},
        exp_shape=(6,),
        check_contents=False,
    )
    _rwork = _rwork_cls.to_ctypes(can_reference_contents=False)
    _ctol = utils._EngineFloat64ScalarType(
        ctol, {'fun_name': fun_name, 'entity': 'ctol'},
    ).to_ctypes()
    _ctol_py = utils._EngineFloat64ScalarType.to_py(_ctol)
    if _jobu_py in ('C', 'c'):
        if not _ctol_py >= 1.0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ctol >= 1.0.'),
                ],
                errcodes=(-14,),
                entity='rwork',
            ).manage()
    _rwork[0] = _ctol
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _joba,
        _jobu,
        _jobv,
        _m,
        _n,
        _a,
        _lda,
        _sva,
        _mv,
        _v,
        _ldv,
        _cwork,
        _lwork,
        _rwork,
        _lrwork,
        _info,
        _joba._length_,
        _jobu._length_,
        _jobv._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'zgesvj did not converge in the allowed number of iterations (30), but its',
                    'output might still be useful.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _sva_cls.refresh_py(_sva)
        _v_cls.refresh_py(_v)
        _rwork_cls.refresh_py(_rwork)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgesvdx(jobu, jobvt, erange, m, n, a, vl, vu, il, iu, s, u, vt, rworkb, jfail):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a complex general matrix, optionally computing the
    corresponding left and right singular vectors.

    For full information please refer to the NAG Library document for
    f08kz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08kzf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'V'

          The `ns` columns of U, as specified by `erange`, are returned
          in array `u`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'V'

          The `ns` rows of V^T, as specified by `erange`, are returned
          in the array `vt`.

        `jobvt` = 'N'

          No rows of V^T (no right singular vectors) are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

          All singular values will be found.

        `erange` = 'V'

          All singular values in the half-open interval (`vl`,`vu`] will
          be found.

        `erange` = 'I'

          The `il`th through `iu`th singular values will be found.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `jobu` != 'N' and `jobvt` != 'N', the contents of
        `a` are destroyed.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    s : float, shape (min(`m`, `n`)), modified in place
        On exit: the singular values of A, sorted so that `s`[i-1] >=
        `s`[i].

    u : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='V': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='V': nsmax; otherwise: 0.

        On exit, if not None on entry: if `jobu` = 'V', `u` contains the
        first `ns` columns of U (the left singular vectors, stored
        column-wise).

        If `jobu` = 'N', `u` is not referenced.

    vt : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvt`='V': min(`m`, `n`); otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvt`='V': `n`; otherwise: 0.

        On exit, if not None on entry: if `jobvt` = 'V', `vt` contains
        the first `ns` rows of V^H (the right singular vectors, stored
        row-wise).

        If `jobvt` = 'N', `vt` is not referenced.

    rworkb : float, shape (min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `rworkb`[1:min(`m`, `n`)] contains the
        unconverged superdiagonal elements of an upper bidiagonal matrix
        B whose diagonal is in `s` (not necessarily sorted). B satisfies
        A = UBV^H, so it has the same singular values as A, and left and
        right singular vectors that are those of A pre-multiplied by U^H
        and V^H.

    jfail : int, shape (2*min(`m`, `n`)), modified in place
        On exit: if `errno` > 0, `jfail` contains, in its first k
        nonzero elements, the indices of the k eigenvectors (associated
        with a left or right singular vector, see ``dbdsvdx``) that
        failed to converge.

    Returns
    -------
    ns : int
        The total number of singular values found. 0 <= `ns` <= min(`m`,
        `n`).

        If `erange` = 'A', `ns` = min(`m`, `n`).

        If `erange` = 'I', `ns` = `iu`-`il`+1.

        If `erange` = 'V' then the value of `ns` is not known in advance
        and so an upper limit should be used when specifying the
        dimensions of array `u`, e.g., min(`m`, `n`).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -8)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            If ``zgesvdx`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgesvdx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08kzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobvt(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_INT, # Output, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[min(m, n)].
        utils._EngineIntArrayType.c_type, # Output, iwork[2*min(m, n)].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvt.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobvt = utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvt in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobvt',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    if _erange_py in ('V', 'v'):
        if not _vl_py >= 0.0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('0.0 <= vl.'),
                ],
                errcodes=(-8,),
                entity='vl',
            ).manage()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-9,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, min(_m_py, _n_py))):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= il and il <= max(1, min(m, n)).'),
                ],
                errcodes=(-10,),
                entity='il',
            ).manage()
        if not (min(_il_py, _m_py, _n_py) <= _iu_py and _iu_py <= min(_m_py, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('min(il, m, n) <= iu and iu <= min(m, n).'),
                ],
                errcodes=(-11,),
                entity='iu',
            ).manage()
    _ns = utils._EngineIntScalarType.empty_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _erange_py in ('I', 'i'):
        _vl_nsmax_py = _iu_py-_il_py+1
    elif _erange_py in ('A', 'a', 'V', 'v'):
        _vl_nsmax_py = min(_m_py, _n_py)
    else:
        _vl_nsmax_py = 0
    if _jobu_py in ('V', 'v'):
        if u is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u'},
                'must not be None when jobu in (\'V\', \'v\')',
            )
    if _jobu_py in ('V', 'v'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('V', 'v'):
        _u_dim2 = _vl_nsmax_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineComplex128ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _jobvt_py in ('V', 'v'):
        if vt is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vt'},
                'must not be None when jobvt in (\'V\', \'v\')',
            )
    if _jobvt_py in ('V', 'v'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('V', 'v'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 0
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt_cls = utils._EngineComplex128ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    utils._EngineFloat64ArrayType(
        rworkb, {'fun_name': fun_name, 'entity': 'rworkb'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(min(_m_py, _n_py))
    utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(2*min(_m_py, _n_py),),
        check_contents=False,
    )
    _iwork = utils._EngineIntArrayType.empty_ctypes(2*min(_m_py, _n_py))
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobvt,
        _erange,
        _m,
        _n,
        _a,
        _lda,
        _vl,
        _vu,
        _il,
        _iu,
        _ns,
        _s,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _info,
        _jobu._length_,
        _jobvt._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If zgesvdx did not converge, errno specifies how many superdiagonals of an',
                    'intermediate bidiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _s_cls.refresh_py(_s)
        if u is not None:
            _u_cls.refresh_py(_u)
        if vt is not None:
            _vt_cls.refresh_py(_vt)
        rworkb[:] = utils._EngineFloat64ArrayType.from_pointer(_rwork, (min(_m_py, _n_py),))
        utils._unlink_pointer(_rwork)
        jfail[:] = utils._EngineIntArrayType.from_pointer(_iwork, (2*min(_m_py, _n_py),))
        utils._unlink_pointer(_iwork)
        _ns_py = utils._EngineIntScalarType.to_py(_ns)
        return_data = _ns_py
    _err_class.manage()
    return return_data

def dgbbrd(vect, m, n, ncc, kl, ku, ab, d, e, q, pt, c):
    u"""
    Performs a reduction of real rectangular band matrix to upper
    bidiagonal form.

    For full information please refer to the NAG Library document for
    f08le

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08lef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the matrices Q and/or P^T are generated.

        `vect` = 'N'

          Neither Q nor P^T is generated.

        `vect` = 'Q'

          Q is generated.

        `vect` = 'P'

          P^T is generated.

        `vect` = 'B'

          Both Q and P^T are generated.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    ncc : int
        n_C, the number of columns of the matrix C.

    kl : int
        The number of subdiagonals, k_l, within the band of A.

    ku : int
        The number of superdiagonals, k_u, within the band of A.

    ab : float, shape (`kl`+`ku`+1, `n`), modified in place
        On entry: the original m by n band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction.

    d : float, shape (min(`m`, `n`)), modified in place
        On exit: the diagonal elements of the bidiagonal matrix B.

    e : float, shape (min(`m`, `n`)-1), modified in place
        On exit: the superdiagonal elements of the bidiagonal matrix B.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('Q', 'B'): `m`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('Q', 'B'): `m`; otherwise:
        1.

        On exit: if `vect` = 'Q' or 'B', contains the m by m orthogonal
        matrix Q.

        If `vect` = 'N' or 'P', `q` is not referenced.

    pt : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('P', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('P', 'B'): `n`; otherwise:
        1.

        On exit: the n by n orthogonal matrix P^T, if `vect` = 'P' or
        'B'. If `vect` = 'N' or 'Q', `pt` is not referenced.

    c : float, shape (:, `ncc`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncc`>0: `m`; if `ncc`=0: 1;
        otherwise: 0.

        On entry: an m by n_C matrix C.

        On exit: `c` is overwritten by Q^TC. If `ncc` = 0, `c` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N', 'Q', 'P' or 'B'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ncc`.

            Constraint: `ncc` >= 0.

        (`errno` -5)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -6)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgbbrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08left')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ncc.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, d[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, e[min(m, n)-1].
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Output, pt[ldpt, *].
        utils._BYREF_INT, # Input, ldpt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, ncc].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q'), (pt, 'pt'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('N', 'n', 'Q', 'q', 'P', 'p', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'N\', \'n\', \'Q\', \'q\', \'P\', \'p\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ncc = utils._EngineIntScalarType(
        ncc, {'fun_name': fun_name, 'entity': 'ncc'},
    ).to_ctypes()
    _ncc_py = utils._EngineIntScalarType.to_py(_ncc)
    if not _ncc_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncc >= 0.'),
            ],
            errcodes=(-4,),
            entity='ncc',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-5,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-6,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_m_py, _n_py)-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim1 = _m_py
    else:
        _q_dim1 = 1
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim2 = _m_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _ldpt_py = utils._EngineFloat64ArrayType.get_nd(pt, {'fun_name': fun_name, 'entity': 'pt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldpt = utils._EngineIntScalarType(
        _ldpt_py, {'fun_name': fun_name, 'entity': '_ldpt'},
    ).to_ctypes()
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim1 = _n_py
    else:
        _pt_dim1 = 1
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim2 = _n_py
    else:
        _pt_dim2 = 1
    exp_shape_pt = (_pt_dim1, _pt_dim2)
    _pt_cls = utils._EngineFloat64ArrayType(
        pt, {'fun_name': fun_name, 'entity': 'pt'},
        exp_shape=exp_shape_pt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _pt = _pt_cls.to_ctypes(can_reference_contents=False)
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    if _ncc_py > 0:
        _c_dim1 = _m_py
    elif _ncc_py == 0:
        _c_dim1 = 1
    else:
        _c_dim1 = 0
    exp_shape_c = (_c_dim1, _ncc_py)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=exp_shape_c,
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _m,
        _n,
        _ncc,
        _kl,
        _ku,
        _ab,
        _ldab,
        _d,
        _e,
        _q,
        _ldq,
        _pt,
        _ldpt,
        _c,
        _ldc,
        _work,
        _info,
        _vect._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _q_cls.refresh_py(_q)
        _pt_cls.refresh_py(_pt)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgbbrd(vect, m, n, ncc, kl, ku, ab, d, e, q, pt, c):
    u"""
    Reduction of complex rectangular band matrix to upper bidiagonal
    form.

    For full information please refer to the NAG Library document for
    f08ls

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08lsf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the matrices Q and/or P^H are generated.

        `vect` = 'N'

          Neither Q nor P^H is generated.

        `vect` = 'Q'

          Q is generated.

        `vect` = 'P'

          P^H is generated.

        `vect` = 'B'

          Both Q and P^H are generated.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    ncc : int
        n_C, the number of columns of the matrix C.

    kl : int
        The number of subdiagonals, k_l, within the band of A.

    ku : int
        The number of superdiagonals, k_u, within the band of A.

    ab : complex, shape (`kl`+`ku`+1, `n`), modified in place
        On entry: the original m by n band matrix A.

        On exit: `ab` is overwritten by values generated during the
        reduction.

    d : float, shape (min(`m`, `n`)), modified in place
        On exit: the diagonal elements of the bidiagonal matrix B.

    e : float, shape (min(`m`, `n`)-1), modified in place
        On exit: the superdiagonal elements of the bidiagonal matrix B.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('Q', 'B'): `m`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('Q', 'B'): `m`; otherwise:
        1.

        On exit: if `vect` = 'Q' or 'B', contains the m by m unitary
        matrix Q.

        If `vect` = 'N' or 'P', `q` is not referenced.

    pt : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('P', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('P', 'B'): `n`; otherwise:
        1.

        On exit: the n by n unitary matrix P^H, if `vect` = 'P' or 'B'.
        If `vect` = 'N' or 'Q', `pt` is not referenced.

    c : complex, shape (:, `ncc`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncc`>0: `m`; if `ncc`=0: 1;
        otherwise: 0.

        On entry: an m by n_C matrix C.

        On exit: `c` is overwritten by Q^HC. If `ncc` = 0, `c` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N', 'Q', 'P' or 'B'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ncc`.

            Constraint: `ncc` >= 0.

        (`errno` -5)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -6)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgbbrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08lsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ncc.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Output, d[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Output, e[min(m, n)-1].
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Output, pt[ldpt, *].
        utils._BYREF_INT, # Input, ldpt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, ncc].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (q, 'q'), (pt, 'pt'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('N', 'n', 'Q', 'q', 'P', 'p', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'N\', \'n\', \'Q\', \'q\', \'P\', \'p\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ncc = utils._EngineIntScalarType(
        ncc, {'fun_name': fun_name, 'entity': 'ncc'},
    ).to_ctypes()
    _ncc_py = utils._EngineIntScalarType.to_py(_ncc)
    if not _ncc_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncc >= 0.'),
            ],
            errcodes=(-4,),
            entity='ncc',
        ).manage()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    if not _kl_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kl >= 0.'),
            ],
            errcodes=(-5,),
            entity='kl',
        ).manage()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    if not _ku_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ku >= 0.'),
            ],
            errcodes=(-6,),
            entity='ku',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(min(_m_py, _n_py)-1,),
        check_contents=False,
    )
    _e = _e_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim1 = _m_py
    else:
        _q_dim1 = 1
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim2 = _m_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _ldpt_py = utils._EngineComplex128ArrayType.get_nd(pt, {'fun_name': fun_name, 'entity': 'pt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldpt = utils._EngineIntScalarType(
        _ldpt_py, {'fun_name': fun_name, 'entity': '_ldpt'},
    ).to_ctypes()
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim1 = _n_py
    else:
        _pt_dim1 = 1
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim2 = _n_py
    else:
        _pt_dim2 = 1
    exp_shape_pt = (_pt_dim1, _pt_dim2)
    _pt_cls = utils._EngineComplex128ArrayType(
        pt, {'fun_name': fun_name, 'entity': 'pt'},
        exp_shape=exp_shape_pt,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _pt = _pt_cls.to_ctypes(can_reference_contents=False)
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    if _ncc_py > 0:
        _c_dim1 = _m_py
    elif _ncc_py == 0:
        _c_dim1 = 1
    else:
        _c_dim1 = 0
    exp_shape_c = (_c_dim1, _ncc_py)
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=exp_shape_c,
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _m,
        _n,
        _ncc,
        _kl,
        _ku,
        _ab,
        _ldab,
        _d,
        _e,
        _q,
        _ldq,
        _pt,
        _ldpt,
        _c,
        _ldc,
        _work,
        _rwork,
        _info,
        _vect._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _q_cls.refresh_py(_q)
        _pt_cls.refresh_py(_pt)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dbdsvdx(uplo, jobz, erange, n, d, e, vl, vu, il, iu, s, z):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a real square bidiagonal matrix, optionally
    computing the corresponding left and right singular vectors.

    For full information please refer to the NAG Library document for
    f08mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08mbf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is upper or lower bidiagonal.

        `uplo` = 'U'

          B is upper bidiagonal.

        `uplo` = 'L'

          B is lower bidiagonal.

    jobz : str, length 1
        Indicates whether singular vectors are computed.

        `jobz` = 'N'

          Only singular values are computed.

        `jobz` = 'V'

          Singular values and singular vectors are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

          All singular values will be found.

        `erange` = 'V'

          All singular values in the half-open interval (`vl`,`vu`] will
          be found.

        `erange` = 'I'

          The `il`th through `iu`th singular values will be found.

    n : int
        n, the order of the bidiagonal matrix B.

    d : float, shape (`n`)
        The diagonal elements d of the bidiagonal matrix B.

    e : float, shape (`n`-1)
        The (n-1) off-diagonal elements e of the bidiagonal matrix B.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    s : float, shape (`n`), modified in place
        On exit: the first `ns` elements contain the selected singular
        values in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': max(2, `n`*2); otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+2; otherwise: `n`+1; otherwise: 1.

        If `jobz` = 'V', then if errno = 0 the first `ns` columns of `z`
        contain the singular vectors of the matrix B corresponding to
        the selected singular values, with U in rows 1 to n and V in
        rows n+1 to n*2.

    Returns
    -------
    ns : int
        The total number of singular values found. 0 <= `ns` <= `n`.

        If `erange` = 'A', `ns` = `n`.

        If `erange` = 'I', `ns` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors of
            the associated eigenproblem did not converge. Their indices
            are stored in array iwork.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dbdsvdx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08mbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n-1].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_INT, # Output, ns.
        utils._EngineFloat64ArrayType.c_type, # Output, s[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Output, iwork[ns].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    if _erange_py in ('V', 'v'):
        if not _vl_py >= 0.0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('0.0 <= vl.'),
                ],
                errcodes=(-7,),
                entity='vl',
            ).manage()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-8,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= il and il <= max(1, n).'),
                ],
                errcodes=(-9,),
                entity='il',
            ).manage()
        if not (min(_n_py, _il_py) <= _iu_py and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('min(n, il) <= iu and iu <= n.'),
                ],
                errcodes=(-10,),
                entity='iu',
            ).manage()
    _ns = utils._EngineIntScalarType.empty_ctypes()
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = max(2, _n_py*2)
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+2
        else:
            _z_dim2 = _n_py+1
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    _iwork = utils._EngineIntArrayType.empty_ctypes(_ns_py)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _jobz,
        _erange,
        _n,
        _d,
        _e,
        _vl,
        _vu,
        _il,
        _iu,
        _ns,
        _s,
        _z,
        _ldz,
        _work,
        _iwork,
        _info,
        _uplo._length_,
        _jobz._length_,
        _erange._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors of'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'the associated eigenproblem did not converge. Their indices are stored',
                    'in array iwork.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _s_cls.refresh_py(_s)
        _z_cls.refresh_py(_z)
        _ns_py = utils._EngineIntScalarType.to_py(_ns)
        return_data = _ns_py
    _err_class.manage()
    return return_data

def dbdsdc(uplo, compq, n, d, e, u, vt, q, iq):
    u"""
    Computes the singular value decomposition of a real bidiagonal
    matrix, optionally computing the singular vectors
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08md

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08mdf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is upper or lower bidiagonal.

        `uplo` = 'U'

          B is upper bidiagonal.

        `uplo` = 'L'

          B is lower bidiagonal.

    compq : str, length 1
        Specifies whether singular vectors are to be computed.

        `compq` = 'N'

          Compute singular values only.

        `compq` = 'P'

          Compute singular values and compute singular vectors in
          compact form.

        `compq` = 'I'

          Compute singular values and singular vectors.

    n : int
        n, the order of the matrix B.

    d : float, shape (`n`), modified in place
        On entry: the n diagonal elements of the bidiagonal matrix B.

        On exit: if no exception or warning is raised, the singular
        values of B.

    e : float, shape (`n`-1), modified in place
        On entry: the (n-1) off-diagonal elements of the bidiagonal
        matrix B.

        On exit: the contents of `e` are destroyed.

    u : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='I': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='I': `n`; otherwise: 0.

        On exit, if not None on entry: if `compq` = 'I', then if no
        exception or warning is raised, `u` contains the left singular
        vectors of the bidiagonal matrix B.

        If `compq` != 'I', `u` is not referenced.

    vt : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='I': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='I': `n`; otherwise: 0.

        On exit, if not None on entry: if `compq` = 'I', then if no
        exception or warning is raised, the rows of `vt` contain the
        right singular vectors of the bidiagonal matrix B.

        If `compq` != 'I', `vt` is not referenced.

    q : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `compq`='P': ldq; otherwise: 0.

        On exit, if not None on entry: if `compq` = 'P', then if no
        exception or warning is raised, `q` and `iq` contain the left
        and right singular vectors in a compact form, requiring
        O(`n`*log2(`n`)) space instead of 2*`n`^2. In particular, `q`
        contains all the real data in the first ldq =
        `n`*(11+2*smlsiz+8*int(log2(`n`/(smlsiz+1)))) elements of `q`,
        where smlsiz is equal to the maximum size of the subproblems at
        the bottom of the computation tree (usually about 25).

        If `compq` != 'P', `q` is not referenced.

    iq : None or int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `compq`='P': max(`n`, ldiq); otherwise: 0.

        On exit, if not None on entry: if `compq` = 'P', then if no
        exception or warning is raised, `q` and `iq` contain the left
        and right singular vectors in a compact form, requiring
        O(`n`*log2(`n`)) space instead of 2*`n`^2. In particular, `iq`
        contains all integer data in the first ldiq =
        `n`*(3+3*int(log2(`n`/(smlsiz+1)))) elements of `iq`, where
        smlsiz is equal to the maximum size of the subproblems at the
        bottom of the computation tree (usually about 25).

        If `compq` != 'P', `iq` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'P' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to compute a singular value. The update
            process of divide-and-conquer failed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dbdsdc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08mdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, vt[ldvt, *].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineFloat64ArrayType.c_type, # Output, q[*].
        utils._EngineIntArrayType.c_type, # Output, iq[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (vt, 'vt')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'P', 'p', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'P\', \'p\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compq',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _compq_py in ('I', 'i'):
        if u is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u'},
                'must not be None when compq in (\'I\', \'i\')',
            )
    if u is None:
        if _compq_py in ('I', 'i'):
            _u_dim1 = _n_py
        else:
            _u_dim1 = 1
        if _compq_py in ('I', 'i'):
            _u_dim2 = _n_py
        else:
            _u_dim2 = 0
        _u = utils._EngineFloat64ArrayType.empty_ctypes((_u_dim1)*(_u_dim2))
    else:
        if _compq_py in ('I', 'i'):
            _u_dim1 = _n_py
        else:
            _u_dim1 = 1
        if _compq_py in ('I', 'i'):
            _u_dim2 = _n_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u_cls = utils._EngineFloat64ArrayType(
            u, {'fun_name': fun_name, 'entity': 'u'},
            exp_shape=exp_shape_u,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldvt_py = utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvt = utils._EngineIntScalarType(
        _ldvt_py, {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _compq_py in ('I', 'i'):
        if vt is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vt'},
                'must not be None when compq in (\'I\', \'i\')',
            )
    if vt is None:
        if _compq_py in ('I', 'i'):
            _vt_dim1 = _n_py
        else:
            _vt_dim1 = 1
        if _compq_py in ('I', 'i'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 0
        _vt = utils._EngineFloat64ArrayType.empty_ctypes((_vt_dim1)*(_vt_dim2))
    else:
        if _compq_py in ('I', 'i'):
            _vt_dim1 = _n_py
        else:
            _vt_dim1 = 1
        if _compq_py in ('I', 'i'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 0
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt_cls = utils._EngineFloat64ArrayType(
            vt, {'fun_name': fun_name, 'entity': 'vt'},
            exp_shape=exp_shape_vt,
            mutual_sorder=_m_sorder,
            check_contents=False,
            allow_none=True,
        )
        _vt = _vt_cls.to_ctypes(can_reference_contents=False)
    _ldq, _ldiq = utils._f08mdan(True, _n)
    _ldq_py = utils._EngineIntScalarType.to_py(_ldq)
    _vl_ldq_py = _ldq_py
    if _compq_py in ('P', 'p'):
        if q is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when compq in (\'P\', \'p\')',
            )
    if q is None:
        if _compq_py in ('P', 'p'):
            _q_dim1 = _vl_ldq_py
        else:
            _q_dim1 = 0
        _q = utils._EngineFloat64ArrayType.empty_ctypes(_q_dim1)
    else:
        if _compq_py in ('P', 'p'):
            _q_dim1 = _vl_ldq_py
        else:
            _q_dim1 = 0
        exp_shape_q = (_q_dim1,)
        _q_cls = utils._EngineFloat64ArrayType(
            q, {'fun_name': fun_name, 'entity': 'q'},
            exp_shape=exp_shape_q,
            check_contents=False,
            allow_none=True,
        )
        _q = _q_cls.to_ctypes(can_reference_contents=False)
    _ldiq_py = utils._EngineIntScalarType.to_py(_ldiq)
    _vl_ldiq_py = _ldiq_py
    if _compq_py in ('P', 'p'):
        if iq is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'iq'},
                'must not be None when compq in (\'P\', \'p\')',
            )
    if iq is None:
        if _compq_py in ('P', 'p'):
            _iq_dim1 = max(_n_py, _vl_ldiq_py)
        else:
            _iq_dim1 = 0
        _iq = utils._EngineIntArrayType.empty_ctypes(_iq_dim1)
    else:
        if _compq_py in ('P', 'p'):
            _iq_dim1 = max(_n_py, _vl_ldiq_py)
        else:
            _iq_dim1 = 0
        exp_shape_iq = (_iq_dim1,)
        _iq_cls = utils._EngineIntArrayType(
            iq, {'fun_name': fun_name, 'entity': 'iq'},
            exp_shape=exp_shape_iq,
            check_contents=False,
            allow_none=True,
        )
        _iq = _iq_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _compq,
        _n,
        _d,
        _e,
        _u,
        _ldu,
        _vt,
        _ldvt,
        _q,
        _iq,
        _work,
        _iwork,
        _info,
        _uplo._length_,
        _compq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to compute a singular value. The update process of',
                    'divide-and-conquer failed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        if u is not None:
            _u_cls.refresh_py(_u)
        if vt is not None:
            _vt_cls.refresh_py(_vt)
        if q is not None:
            _q_cls.refresh_py(_q)
        if iq is not None:
            _iq_cls.refresh_py(_iq)
        return_data = None
    _err_class.manage()
    return return_data

def dbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, u, c):
    u"""
    Performs an SVD of real bidiagonal matrix reduced from real general
    matrix.

    For full information please refer to the NAG Library document for
    f08me

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08mef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is an upper or lower bidiagonal matrix.

        `uplo` = 'U'

          B is an upper bidiagonal matrix.

        `uplo` = 'L'

          B is a lower bidiagonal matrix.

    n : int
        n, the order of the matrix B.

    ncvt : int
        ncvt, the number of columns of the matrix V^T of right singular
        vectors. Set `ncvt` = 0 if no right singular vectors are
        required.

    nru : int
        nru, the number of rows of the matrix U of left singular
        vectors. Set `nru` = 0 if no left singular vectors are required.

    ncc : int
        ncc, the number of columns of the matrix C. Set `ncc` = 0 if no
        matrix C is supplied.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the bidiagonal matrix B.

        On exit: the singular values in decreasing order of magnitude,
        unless `errno` > 0 (in which case see [Errors]).

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the bidiagonal matrix B.

        On exit: `e` is overwritten, but if `errno` > 0 see [Errors].

    vt : float, shape (:, `ncvt`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncvt`>0: `n`; otherwise: 1.

        On entry: if `ncvt` > 0, `vt` must contain an n by ncvt matrix.
        If the right singular vectors of B are required, ncvt = n and
        `vt` must contain the unit matrix; if the right singular vectors
        of A are required, `vt` must contain the orthogonal matrix P^T
        returned by ``dorgbr`` with vect = 'P'.

        On exit: the n by ncvt matrix V^T or V^TP^T of right singular
        vectors, stored by rows.

        If `ncvt` = 0, `vt` is not referenced.

    u : float, shape (`nru`, `n`), modified in place
        On entry: if `nru` > 0, `u` must contain an nru by n matrix. If
        the left singular vectors of B are required, nru = n and `u`
        must contain the unit matrix; if the left singular vectors of A
        are required, `u` must contain the orthogonal matrix Q returned
        by ``dorgbr`` with vect = 'Q'.

        On exit: the nru by n matrix U or QU of left singular vectors,
        stored as columns of the matrix.

        If `nru` = 0, `u` is not referenced.

    c : float, shape (:, `ncc`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncc`>0: `n`; otherwise: 1.

        On entry: the n by ncc matrix C if `ncc` > 0.

        On exit: `c` is overwritten by the matrix U^TC. If `ncc` = 0,
        `c` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `ncvt`.

            Constraint: `ncvt` >= 0.

        (`errno` -4)
            On entry, error in parameter `nru`.

            Constraint: `nru` >= 0.

        (`errno` -5)
            On entry, error in parameter `ncc`.

            Constraint: `ncc` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* off-diagonals did not converge. The arrays `d` and
            `e` contain the diagonal and off-diagonal elements,
            respectively, of a bidiagonal matrix orthogonally equivalent
            to B.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dbdsqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08meft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ncvt.
        utils._BYREF_INT, # Input, nru.
        utils._BYREF_INT, # Input, ncc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vt[ldvt, ncvt].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[ldu, n].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, ncc].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((vt, 'vt'), (u, 'u'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ncvt = utils._EngineIntScalarType(
        ncvt, {'fun_name': fun_name, 'entity': 'ncvt'},
    ).to_ctypes()
    _ncvt_py = utils._EngineIntScalarType.to_py(_ncvt)
    if not _ncvt_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncvt >= 0.'),
            ],
            errcodes=(-3,),
            entity='ncvt',
        ).manage()
    _nru = utils._EngineIntScalarType(
        nru, {'fun_name': fun_name, 'entity': 'nru'},
    ).to_ctypes()
    _nru_py = utils._EngineIntScalarType.to_py(_nru)
    if not _nru_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nru >= 0.'),
            ],
            errcodes=(-4,),
            entity='nru',
        ).manage()
    _ncc = utils._EngineIntScalarType(
        ncc, {'fun_name': fun_name, 'entity': 'ncc'},
    ).to_ctypes()
    _ncc_py = utils._EngineIntScalarType.to_py(_ncc)
    if not _ncc_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncc >= 0.'),
            ],
            errcodes=(-5,),
            entity='ncc',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldvt = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _ncvt_py > 0:
        _vt_dim1 = _n_py
    else:
        _vt_dim1 = 1
    exp_shape_vt = (_vt_dim1, _ncvt_py)
    _vt_cls = utils._EngineFloat64ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
    )
    _vt = _vt_cls.to_ctypes()
    _ldu = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_nru_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    if _ncc_py > 0:
        _c_dim1 = _n_py
    else:
        _c_dim1 = 1
    exp_shape_c = (_c_dim1, _ncc_py)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=exp_shape_c,
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ncvt,
        _nru,
        _ncc,
        _d,
        _e,
        _vt,
        _ldvt,
        _u,
        _ldu,
        _c,
        _ldc,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} off-diagonals did not converge.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The arrays d and e contain the diagonal and',
                    'off-diagonal elements, respectively, of a bidiagonal matrix orthogonally',
                    'equivalent to B.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _vt_cls.refresh_py(_vt)
        _u_cls.refresh_py(_u)
        _c_cls.refresh_py(_c)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zbdsqr(uplo, n, ncvt, nru, ncc, d, e, vt, u, c):
    u"""
    Performs an SVD of real bidiagonal matrix reduced from complex
    general matrix.

    For full information please refer to the NAG Library document for
    f08ms

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08msf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is an upper or lower bidiagonal matrix.

        `uplo` = 'U'

          B is an upper bidiagonal matrix.

        `uplo` = 'L'

          B is a lower bidiagonal matrix.

    n : int
        n, the order of the matrix B.

    ncvt : int
        ncvt, the number of columns of the matrix V^H of right singular
        vectors. Set `ncvt` = 0 if no right singular vectors are
        required.

    nru : int
        nru, the number of rows of the matrix U of left singular
        vectors. Set `nru` = 0 if no left singular vectors are required.

    ncc : int
        ncc, the number of columns of the matrix C. Set `ncc` = 0 if no
        matrix C is supplied.

    d : float, shape (`n`), modified in place
        On entry: the diagonal elements of the bidiagonal matrix B.

        On exit: the singular values in decreasing order of magnitude,
        unless `errno` > 0 (in which case see [Errors]).

    e : float, shape (`n`-1), modified in place
        On entry: the off-diagonal elements of the bidiagonal matrix B.

        On exit: `e` is overwritten, but if `errno` > 0 see [Errors].

    vt : complex, shape (:, `ncvt`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncvt`>0: `n`; otherwise: 1.

        On entry: if `ncvt` > 0, `vt` must contain an n by ncvt matrix.
        If the right singular vectors of B are required, ncvt = n and
        `vt` must contain the unit matrix; if the right singular vectors
        of A are required, `vt` must contain the unitary matrix P^H
        returned by ``zungbr`` with vect = 'P'.

        On exit: the n by ncvt matrix V^H or V^HP^H of right singular
        vectors, stored by rows.

        If `ncvt` = 0, `vt` is not referenced.

    u : complex, shape (`nru`, `n`), modified in place
        On entry: if `nru` > 0, `u` must contain an nru by n matrix. If
        the left singular vectors of B are required, nru = n and `u`
        must contain the unit matrix; if the left singular vectors of A
        are required, `u` must contain the unitary matrix Q returned by
        ``zungbr`` with vect = 'Q'.

        On exit: the nru by n matrix U or QU of left singular vectors,
        stored as columns of the matrix.

        If `nru` = 0, `u` is not referenced.

    c : complex, shape (:, `ncc`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncc`>0: `n`; otherwise: 1.

        On entry: the n by ncc matrix C if `ncc` > 0.

        On exit: `c` is overwritten by the matrix U^HC. If `ncc` = 0,
        `c` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `ncvt`.

            Constraint: `ncvt` >= 0.

        (`errno` -4)
            On entry, error in parameter `nru`.

            Constraint: `nru` >= 0.

        (`errno` -5)
            On entry, error in parameter `ncc`.

            Constraint: `ncc` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* off-diagonals did not converge. The arrays `d` and
            `e` contain the diagonal and off-diagonal elements,
            respectively, of a bidiagonal matrix orthogonally equivalent
            to B.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zbdsqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08msft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ncvt.
        utils._BYREF_INT, # Input, nru.
        utils._BYREF_INT, # Input, ncc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vt[ldvt, ncvt].
        utils._BYREF_INT, # Input, ldvt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, u[ldu, n].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, ncc].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((vt, 'vt'), (u, 'u'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ncvt = utils._EngineIntScalarType(
        ncvt, {'fun_name': fun_name, 'entity': 'ncvt'},
    ).to_ctypes()
    _ncvt_py = utils._EngineIntScalarType.to_py(_ncvt)
    if not _ncvt_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncvt >= 0.'),
            ],
            errcodes=(-3,),
            entity='ncvt',
        ).manage()
    _nru = utils._EngineIntScalarType(
        nru, {'fun_name': fun_name, 'entity': 'nru'},
    ).to_ctypes()
    _nru_py = utils._EngineIntScalarType.to_py(_nru)
    if not _nru_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('nru >= 0.'),
            ],
            errcodes=(-4,),
            entity='nru',
        ).manage()
    _ncc = utils._EngineIntScalarType(
        ncc, {'fun_name': fun_name, 'entity': 'ncc'},
    ).to_ctypes()
    _ncc_py = utils._EngineIntScalarType.to_py(_ncc)
    if not _ncc_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ncc >= 0.'),
            ],
            errcodes=(-5,),
            entity='ncc',
        ).manage()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldvt = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldvt'},
    ).to_ctypes()
    if _ncvt_py > 0:
        _vt_dim1 = _n_py
    else:
        _vt_dim1 = 1
    exp_shape_vt = (_vt_dim1, _ncvt_py)
    _vt_cls = utils._EngineComplex128ArrayType(
        vt, {'fun_name': fun_name, 'entity': 'vt'},
        exp_shape=exp_shape_vt,
        mutual_sorder=_m_sorder,
    )
    _vt = _vt_cls.to_ctypes()
    _ldu = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_nru_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    if _ncc_py > 0:
        _c_dim1 = _n_py
    else:
        _c_dim1 = 1
    exp_shape_c = (_c_dim1, _ncc_py)
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=exp_shape_c,
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _ncvt,
        _nru,
        _ncc,
        _d,
        _e,
        _vt,
        _ldvt,
        _u,
        _ldu,
        _c,
        _ldc,
        _work,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} off-diagonals did not converge.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'The arrays d and e contain the diagonal and',
                    'off-diagonal elements, respectively, of a bidiagonal matrix orthogonally',
                    'equivalent to B.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _vt_cls.refresh_py(_vt)
        _u_cls.refresh_py(_u)
        _c_cls.refresh_py(_c)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgeev(jobvl, jobvr, n, a, wr, wi, vl, vr):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a real nonsymmetric matrix.

    For full information please refer to the NAG Library document for
    f08na

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08naf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` has been overwritten.

    wr : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    wi : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        their corresponding eigenvalues. If the jth eigenvalue is real,
        then u_j = `vl`[i-1,j-1], for i = 1,2,...,`n`. If the jth and
        (j+1)st eigenvalues form a complex conjugate pair, then u_j =
        `vl`[i-1,j-1]+i*`vl`[i-1,j] and u_{j+1} =
        `vl`[i-1,j-1]-i*`vl`[i-1,j], for i = 1,2,...,`n`.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        their corresponding eigenvalues. If the jth eigenvalue is real,
        then v_j = `vr`[i-1,j-1], for i = 1,2,...,`n`. If the jth and
        (j+1)st eigenvalues form a complex conjugate pair, then v_j =
        `vr`[i-1,j-1]+i*`vr`[i-1,j] and v_{j+1} =
        `vr`[i-1,j-1]-i*`vr`[i-1,j], for i = 1,2,...,`n`.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors have been computed; elements *<value>* to
            `n` of `wr` and `wi` contain eigenvalues which have
            converged.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08naft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _wr,
        _wi,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all',
                    'the eigenvalues, and no eigenvectors have been computed; elements',
                    '{:s} to n of wr'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'and wi contain eigenvalues which have converged.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgeevx(balanc, jobvl, jobvr, sense, n, a, wr, wi, vl, vr, scale, rconde, rcondv):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a real nonsymmetric matrix; also, optionally, the
    balancing transformation, the reciprocal condition numbers for the
    eigenvalues and for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nbf.html

    Parameters
    ----------
    balanc : str, length 1
        Indicates how the input matrix should be diagonally scaled
        and/or permuted to improve the conditioning of its eigenvalues.

        `balanc` = 'N'

          Do not diagonally scale or permute.

        `balanc` = 'P'

          Perform permutations to make the matrix more nearly upper
          triangular. Do not diagonally scale.

        `balanc` = 'S'

          Diagonally scale the matrix, i.e., replace A by DAD^-1, where
          D is a diagonal matrix chosen to make the rows and columns of
          A more equal in norm. Do not permute.

        `balanc` = 'B'

          Both diagonally scale and permute A.

        Computed reciprocal condition numbers will be for the matrix
        after balancing and/or permuting.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvl` must be set to `jobvl` = 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvr` must be set to `jobvr` = 'V'.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for eigenvalues only.

        `sense` = 'V'

          Computed for right eigenvectors only.

        `sense` = 'B'

          Computed for eigenvalues and right eigenvectors.

        If `sense` = 'E' or 'B', both left and right eigenvectors must
        also be computed (`jobvl` = 'V' and `jobvr` = 'V').

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` has been overwritten. If `jobvl` = 'V' or `jobvr` =
        'V', A contains the real Schur form of the balanced version of
        the input matrix A.

    wr : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    wi : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        their corresponding eigenvalues. If the jth eigenvalue is real,
        then u_j = `vl`[i-1,j-1], for i = 1,2,...,`n`. If the jth and
        (j+1)st eigenvalues form a complex conjugate pair, then u_j =
        `vl`[i-1,j-1]+i*`vl`[i-1,j] and u_{j+1} =
        `vl`[i-1,j-1]-i*`vl`[i-1,j], for i = 1,2,...,`n`.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        their corresponding eigenvalues. If the jth eigenvalue is real,
        then v_j = `vr`[i-1,j-1], for i = 1,2,...,`n`. If the jth and
        (j+1)st eigenvalues form a complex conjugate pair, then v_j =
        `vr`[i-1,j-1]+i*`vr`[i-1,j] and v_{j+1} =
        `vr`[i-1,j-1]-i*`vr`[i-1,j], for i = 1,2,...,`n`.

        If `jobvr` = 'N', `vr` is not referenced.

    scale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        when balancing A.

        If p_j is the index of the row and column interchanged with row
        and column j, and d_j is the scaling factor applied to row and
        column j, then

        - `scale`[j-1] = p_j, for j = 1,2,...,`ilo`-1;

        - `scale`[j-1] = d_j, for j = `ilo`,...,`ihi`;

        - `scale`[j-1] = p_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rconde : float, shape (`n`), modified in place
        On exit: `rconde`[j-1] is the reciprocal condition number of the
        jth eigenvalue.

    rcondv : float, shape (`n`), modified in place
        On exit: `rcondv`[j-1] is the reciprocal condition number of the
        jth right eigenvector.

    Returns
    -------
    ilo : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,`n`.

    ihi : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,`n`.

    abnrm : float
        The 1-norm of the balanced matrix (the maximum of the sum of
        absolute values of elements of any column).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors or condition numbers have been computed;
            elements 1 to `ilo`-1 and *<value>* to `n` of `wr` and `wi`
            contain eigenvalues which have converged.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, balanc(len=1).
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, scale[n].
        utils._BYREF_FLOAT64, # Output, abnrm.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of balanc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _balanc = utils._EngineCharScalarType(
        balanc, {'fun_name': fun_name, 'entity': 'balanc'}, exp_chrlen=1,
    ).to_ctypes()
    _balanc_py = utils._EngineCharScalarType.to_py(_balanc)
    if _balanc_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('balanc in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='balanc',
        ).manage()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-3,),
            entity='jobvr',
        ).manage()
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _scale_cls = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _scale = _scale_cls.to_ctypes(can_reference_contents=False)
    _abnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _balanc,
        _jobvl,
        _jobvr,
        _sense,
        _n,
        _a,
        _lda,
        _wr,
        _wi,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _ilo,
        _ihi,
        _scale,
        _abnrm,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _iwork,
        _info,
        _balanc._length_,
        _jobvl._length_,
        _jobvr._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all',
                    'the eigenvalues, and no eigenvectors or condition numbers have been',
                    'computed; elements 1 to ilo-1 and {:s} to n of'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'wr and wi contain eigenvalues which have converged.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        _scale_cls.refresh_py(_scale)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        _abnrm_py = utils._EngineFloat64ScalarType.to_py(_abnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi', 'abnrm'],
        )(
            _ilo_py,
            _ihi_py,
            _abnrm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dgehrd(n, ilo, ihi, a, tau):
    u"""
    Performs an orthogonal reduction of real general matrix to upper
    Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ne

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nef.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    ilo : int
        If A has been output by ``dgebal``, `ilo` and `ihi` must contain
        the values returned by that function. Otherwise, `ilo` must be
        set to 1 and `ihi` to `n`.

    ihi : int
        If A has been output by ``dgebal``, `ilo` and `ihi` must contain
        the values returned by that function. Otherwise, `ilo` must be
        set to 1 and `ihi` to `n`.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n general matrix A.

        On exit: `a` is overwritten by the upper Hessenberg matrix H and
        details of the orthogonal matrix Q.

    tau : float, shape (`n`-1), modified in place
        On exit: further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgehrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08neft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, tau[n-1].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-2,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-3,),
            entity='ihi',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def dorghr(n, ilo, ihi, a, tau):
    u"""
    Generates an orthogonal transformation matrix from reduction to
    Hessenberg form determined by ``dgehrd``.

    For full information please refer to the NAG Library document for
    f08nf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix Q.

    ilo : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``dgehrd``.

    ihi : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``dgehrd``.

    a : float, shape (`n`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``dgehrd``.

        On exit: the n by n orthogonal matrix Q.

    tau : float, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``dgehrd``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorghr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[n-1].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-2,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-3,),
            entity='ihi',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def dormhr(side, trans, m, n, ilo, ihi, a, tau, c):
    u"""
    Applies the orthogonal transformation matrix from reduction to
    Hessenberg form determined by ``dgehrd``.

    For full information please refer to the NAG Library document for
    f08ng

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ngf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

          Q or Q^T is applied to C from the left.

        `side` = 'R'

          Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'T'

          Q^T is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    ilo : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``dgehrd``.

    ihi : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``dgehrd``.

    a : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgehrd``.

    tau : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``dgehrd``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^TC or CQ or CQ^T as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `ilo`.

        (`errno` -6)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dormhr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ngft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, tau[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _side_py in ('R', 'r'):
        if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo >= 1.'),
                ],
                errcodes=(-5,),
                entity='ilo',
            ).manage()
        if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    elif _side_py in ('L', 'l'):
        if not (_ilo_py >= 1 and _ilo_py <= max(1, _m_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo >= 1.'),
                ],
                errcodes=(-5,),
                entity='ilo',
            ).manage()
        if not (_ihi_py >= min(_ilo_py, _m_py) and _ihi_py <= _m_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= min(ilo, m).'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineFloat64ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dgebal(job, n, a, scale):
    u"""
    Balances a real general matrix.

    For full information please refer to the NAG Library document for
    f08nh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nhf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether A is to be permuted and/or scaled (or
        neither).

        `job` = 'N'

          A is neither permuted nor scaled (but values are assigned to
          `ilo`, `ihi` and `scale`).

        `job` = 'P'

          A is permuted but not scaled.

        `job` = 'S'

          A is scaled but not permuted.

        `job` = 'B'

          A is both permuted and scaled.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by the balanced matrix. If `job` =
        'N', `a` is not referenced.

    scale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to A. More precisely, if p_j is the index of the row and column
        interchanged with row and column j and d_j is the scaling factor
        used to balance row and column j then

            `scale`[j-1] = {[table omitted]

        The order in which the interchanges are made is n to i_hi+1 then
        1 to i_lo-1.

    Returns
    -------
    ilo : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgebal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, scale[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _scale_cls = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _scale = _scale_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _n,
        _a,
        _lda,
        _ilo,
        _ihi,
        _scale,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _scale_cls.refresh_py(_scale)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi'],
        )(
            _ilo_py,
            _ihi_py,
        )
    _err_class.manage()
    return return_data

def dgebak(job, side, n, ilo, ihi, scale, m, v):
    u"""
    Transforms eigenvectors of real balanced matrix to those of original
    matrix supplied to ``dgebal``.

    For full information please refer to the NAG Library document for
    f08nj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08njf.html

    Parameters
    ----------
    job : str, length 1
        This must be the same argument `job` as supplied to ``dgebal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

          The left eigenvectors are transformed.

        `side` = 'R'

          The right eigenvectors are transformed.

    n : int
        n, the number of rows of the matrix of eigenvectors.

    ilo : int
        The values i_lo and i_hi, as returned by ``dgebal``.

    ihi : int
        The values i_lo and i_hi, as returned by ``dgebal``.

    scale : float, shape (`n`)
        Details of the permutations and/or the scaling factors used to
        balance the original real nonsymmetric matrix, as returned by
        ``dgebal``.

    m : int
        m, the number of columns of the matrix of eigenvectors.

    v : float, shape (`n`, `m`), modified in place
        On entry: the matrix of left or right eigenvectors to be
        transformed.

        On exit: the transformed eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgebak'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08njft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input, scale[n].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, m].
        utils._BYREF_INT, # Input, ldv.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-5,),
            entity='ihi',
        ).manage()
    _scale = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _side,
        _n,
        _ilo,
        _ihi,
        _scale,
        _m,
        _v,
        _ldv,
        _info,
        _job._length_,
        _side._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        return_data = None
    _err_class.manage()
    return return_data

def zgeev(jobvl, jobvr, n, a, w, vl, vr):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a complex nonsymmetric matrix.

    For full information please refer to the NAG Library document for
    f08nn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nnf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` has been overwritten.

    w : complex, shape (`n`), modified in place
        On exit: contains the computed eigenvalues.

    vl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        their corresponding eigenvalues; that is u_j = `vl`[i-1,j-1],
        for i = 1,2,...,`n`.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        their corresponding eigenvalues; that is v_j = `vr`[i-1,j-1],
        for i = 1,2,...,`n`.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors have been computed; elements *<value>* to
            `n` of `w` contain eigenvalues which have converged.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _w,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all',
                    'the eigenvalues, and no eigenvectors have been computed; elements',
                    '{:s} to n of w'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'contain eigenvalues which have converged.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgeevx(balanc, jobvl, jobvr, sense, n, a, w, vl, vr, scale, rconde, rcondv):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a complex nonsymmetric matrix; also, optionally, the
    balancing transformation, the reciprocal condition numbers for the
    eigenvalues and for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08np

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08npf.html

    Parameters
    ----------
    balanc : str, length 1
        Indicates how the input matrix should be diagonally scaled
        and/or permuted to improve the conditioning of its eigenvalues.

        `balanc` = 'N'

          Do not diagonally scale or permute.

        `balanc` = 'P'

          Perform permutations to make the matrix more nearly upper
          triangular. Do not diagonally scale.

        `balanc` = 'S'

          Diagonally scale the matrix, i.e., replace A by DAD^-1, where
          D is a diagonal matrix chosen to make the rows and columns of
          A more equal in norm. Do not permute.

        `balanc` = 'B'

          Both diagonally scale and permute A.

        Computed reciprocal condition numbers will be for the matrix
        after balancing and/or permuting.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvl` must be set to `jobvl` = 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvr` must be set to `jobvr` = 'V'.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for eigenvalues only.

        `sense` = 'V'

          Computed for right eigenvectors only.

        `sense` = 'B'

          Computed for eigenvalues and right eigenvectors.

        If `sense` = 'E' or 'B', both left and right eigenvectors must
        also be computed (`jobvl` = 'V' and `jobvr` = 'V').

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` has been overwritten. If `jobvl` = 'V' or `jobvr` =
        'V', A contains the Schur form of the balanced version of the
        matrix A.

    w : complex, shape (`n`), modified in place
        On exit: contains the computed eigenvalues.

    vl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        their corresponding eigenvalues; that is u_j = `vl`[i-1,j-1],
        for i = 1,2,...,`n`.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        their corresponding eigenvalues; that is v_j = `vr`[i-1,j-1],
        for i = 1,2,...,`n`.

        If `jobvr` = 'N', `vr` is not referenced.

    scale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        when balancing A.

        If p_j is the index of the row and column interchanged with row
        and column j, and d_j is the scaling factor applied to row and
        column j, then

        - `scale`[j-1] = p_j, for j = 1,2,...,`ilo`-1;

        - `scale`[j-1] = d_j, for j = `ilo`,...,`ihi`;

        - `scale`[j-1] = p_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rconde : float, shape (`n`), modified in place
        On exit: `rconde`[j-1] is the reciprocal condition number of the
        jth eigenvalue.

    rcondv : float, shape (`n`), modified in place
        On exit: `rcondv`[j-1] is the reciprocal condition number of the
        jth right eigenvector.

    Returns
    -------
    ilo : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,`n`.

    ihi : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,`n`.

    abnrm : float
        The 1-norm of the balanced matrix (the maximum of the sum of
        absolute values of elements of any column).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors or condition numbers have been computed;
            elements 1 to `ilo`-1 and *<value>* to `n` of `w` contain
            eigenvalues which have converged.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08npft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, balanc(len=1).
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, scale[n].
        utils._BYREF_FLOAT64, # Output, abnrm.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of balanc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _balanc = utils._EngineCharScalarType(
        balanc, {'fun_name': fun_name, 'entity': 'balanc'}, exp_chrlen=1,
    ).to_ctypes()
    _balanc_py = utils._EngineCharScalarType.to_py(_balanc)
    if _balanc_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('balanc in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='balanc',
        ).manage()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-3,),
            entity='jobvr',
        ).manage()
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _scale_cls = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _scale = _scale_cls.to_ctypes(can_reference_contents=False)
    _abnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _balanc,
        _jobvl,
        _jobvr,
        _sense,
        _n,
        _a,
        _lda,
        _w,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _ilo,
        _ihi,
        _scale,
        _abnrm,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _rwork,
        _info,
        _balanc._length_,
        _jobvl._length_,
        _jobvr._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all',
                    'the eigenvalues, and no eigenvectors or condition numbers have been',
                    'computed; elements 1 to ilo-1 and {:s} to n of'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'w contain eigenvalues which have converged.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        _scale_cls.refresh_py(_scale)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        _abnrm_py = utils._EngineFloat64ScalarType.to_py(_abnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi', 'abnrm'],
        )(
            _ilo_py,
            _ihi_py,
            _abnrm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zgehrd(n, ilo, ihi, a, tau):
    u"""
    Performs a unitary reduction of complex general matrix to upper
    Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ns

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nsf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    ilo : int
        If A has been output by ``zgebal``, `ilo` and `ihi` must contain
        the values returned by that function. Otherwise, `ilo` must be
        set to 1 and `ihi` to `n`.

    ihi : int
        If A has been output by ``zgebal``, `ilo` and `ihi` must contain
        the values returned by that function. Otherwise, `ilo` must be
        set to 1 and `ihi` to `n`.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n general matrix A.

        On exit: `a` is overwritten by the upper Hessenberg matrix H and
        details of the unitary matrix Q. The subdiagonal elements of H
        are real.

    tau : complex, shape (`n`-1), modified in place
        On exit: further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgehrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, tau[n-1].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-2,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-3,),
            entity='ihi',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau_cls = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
        check_contents=False,
    )
    _tau = _tau_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _tau_cls.refresh_py(_tau)
        return_data = None
    _err_class.manage()
    return return_data

def zunghr(n, ilo, ihi, a, tau):
    u"""
    Generates a unitary transformation matrix from reduction to
    Hessenberg form determined by ``zgehrd``.

    For full information please refer to the NAG Library document for
    f08nt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ntf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix Q.

    ilo : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``zgehrd``.

    ihi : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``zgehrd``.

    a : complex, shape (`n`, `n`), modified in place
        On entry: details of the vectors which define the elementary
        reflectors, as returned by ``zgehrd``.

        On exit: the n by n unitary matrix Q.

    tau : complex, shape (`n`-1)
        Further details of the elementary reflectors, as returned by
        ``zgehrd``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunghr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ntft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[n-1].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-2,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-3,),
            entity='ihi',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=(_n_py-1,),
    ).to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zunmhr(side, trans, m, n, ilo, ihi, a, tau, c):
    u"""
    Applies the unitary transformation matrix from reduction to
    Hessenberg form determined by ``zgehrd``.

    For full information please refer to the NAG Library document for
    f08nu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

          Q or Q^H is applied to C from the left.

        `side` = 'R'

          Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

          Q is applied to C.

        `trans` = 'C'

          Q^H is applied to C.

    m : int
        m, the number of rows of the matrix C; m is also the order of Q
        if `side` = 'L'.

    n : int
        n, the number of columns of the matrix C; n is also the order of
        Q if `side` = 'R'.

    ilo : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``zgehrd``.

    ihi : int
        These must be the same arguments `ilo` and `ihi`, respectively,
        as supplied to ``zgehrd``.

    a : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': `m`; if `side`='R': `n`;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgehrd``.

    tau : complex, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': `m`-1; if `side`='R': `n`-1; otherwise:
        0.

        Further details of the elementary reflectors, as returned by
        ``zgehrd``.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix C.

        On exit: `c` is overwritten by QC or Q^HC or CQ or CQ^H as
        specified by `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `ilo`.

        (`errno` -6)
            On entry, error in parameter `ihi`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zunmhr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, *].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, tau[*].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='trans',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _side_py in ('R', 'r'):
        if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo >= 1.'),
                ],
                errcodes=(-5,),
                entity='ilo',
            ).manage()
        if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    elif _side_py in ('L', 'l'):
        if not (_ilo_py >= 1 and _ilo_py <= max(1, _m_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo >= 1.'),
                ],
                errcodes=(-5,),
                entity='ilo',
            ).manage()
        if not (_ihi_py >= min(_ilo_py, _m_py) and _ihi_py <= _m_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= min(ilo, m).'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    if _side_py in ('L', 'l'):
        _a_dim1 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim1 = _n_py
    else:
        _a_dim1 = 0
    if _side_py in ('L', 'l'):
        _a_dim2 = _m_py
    elif _side_py in ('R', 'r'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    exp_shape_a = (_a_dim1, _a_dim2)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    if _side_py in ('L', 'l'):
        _tau_dim1 = _m_py-1
    elif _side_py in ('R', 'r'):
        _tau_dim1 = _n_py-1
    else:
        _tau_dim1 = 0
    exp_shape_tau = (_tau_dim1,)
    _tau = utils._EngineComplex128ArrayType(
        tau, {'fun_name': fun_name, 'entity': 'tau'},
        exp_shape=exp_shape_tau,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _trans,
        _m,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _tau,
        _c,
        _ldc,
        _work,
        _lwork,
        _info,
        _side._length_,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def zgebal(job, n, a, scale):
    u"""
    Balances a complex general matrix.

    For full information please refer to the NAG Library document for
    f08nv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nvf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether A is to be permuted and/or scaled (or
        neither).

        `job` = 'N'

          A is neither permuted nor scaled (but values are assigned to
          `ilo`, `ihi` and `scale`).

        `job` = 'P'

          A is permuted but not scaled.

        `job` = 'S'

          A is scaled but not permuted.

        `job` = 'B'

          A is both permuted and scaled.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by the balanced matrix. If `job` =
        'N', `a` is not referenced.

    scale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to A. More precisely, if p_j is the index of the row and column
        interchanged with row and column j and d_j is the scaling factor
        used to balance row and column j then

            `scale`[j-1] = {[table omitted]

        The order in which the interchanges are made is n to i_hi+1 then
        1 to i_lo-1.

    Returns
    -------
    ilo : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgebal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, scale[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _scale_cls = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _scale = _scale_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _n,
        _a,
        _lda,
        _ilo,
        _ihi,
        _scale,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _scale_cls.refresh_py(_scale)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi'],
        )(
            _ilo_py,
            _ihi_py,
        )
    _err_class.manage()
    return return_data

def zgebak(job, side, n, ilo, ihi, scale, m, v):
    u"""
    Transforms eigenvectors of complex balanced matrix to those of
    original matrix supplied to ``zgebal``.

    For full information please refer to the NAG Library document for
    f08nw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08nwf.html

    Parameters
    ----------
    job : str, length 1
        This must be the same argument `job` as supplied to ``zgebal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

          The left eigenvectors are transformed.

        `side` = 'R'

          The right eigenvectors are transformed.

    n : int
        n, the number of rows of the matrix of eigenvectors.

    ilo : int
        The values i_lo and i_hi, as returned by ``zgebal``.

    ihi : int
        The values i_lo and i_hi, as returned by ``zgebal``.

    scale : float, shape (`n`)
        Details of the permutations and/or the scaling factors used to
        balance the original complex general matrix, as returned by
        ``zgebal``.

    m : int
        m, the number of columns of the matrix of eigenvectors.

    v : complex, shape (`n`, `m`), modified in place
        On entry: the matrix of left or right eigenvectors to be
        transformed.

        On exit: the transformed eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgebak'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08nwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input, scale[n].
        utils._BYREF_INT, # Input, m.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, m].
        utils._BYREF_INT, # Input, ldv.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-5,),
            entity='ihi',
        ).manage()
    _scale = utils._EngineFloat64ArrayType(
        scale, {'fun_name': fun_name, 'entity': 'scale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _side,
        _n,
        _ilo,
        _ihi,
        _scale,
        _m,
        _v,
        _ldv,
        _info,
        _job._length_,
        _side._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        return_data = None
    _err_class.manage()
    return return_data

def dgees(jobvs, sort, select, n, a, wr, wi, vs, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    real Schur form, and, optionally, the matrix of Schur vectors.

    For full information please refer to the NAG Library document for
    f08pa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08paf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `select`).

    select : None or callable retval = select(wr, wi, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        Parameters
        ~~~~~~~~~~
        wr : float
            The real and imaginary parts of the eigenvalue.

        wi : float
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by its real Schur form T.

    wr : float, shape (`n`), modified in place
        On exit: see the description of `wi`.

    wi : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues in the same order that
        they appear on the diagonal of the output Schur form T. Complex
        conjugate pairs of eigenvalues will appear consecutively with
        the eigenvalue having the positive imaginary part first.

    vs : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        On exit: if `jobvs` = 'V', `vs` contains the orthogonal matrix Z
        of Schur vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `select` is True. (Complex conjugate pairs for which
        `select` is True for either eigenvalue count as 2.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgees'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_select_name = fun_name + ':select'

    selecth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, wr.
        utils._BYREF_FLOAT64, # Input, wi.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if select is None:
        select_auxil_ctypes = utils._EngineState._get_symbol('f08pazt')
        select_auxil_ctypes.restype = None
        select_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, wr.
            utils._BYREF_FLOAT64, # Input, wi.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selecth(_select, _selectsub, en_data, wr, wi, iretval, iuser, ruser):
        """Helper for select."""
        try:
            if select is None:
                select_auxil_ctypes(
                    en_data,
                    wr,
                    wi,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _wr_selecth = utils._EngineFloat64ScalarType.to_py(wr.contents)
            _wi_selecth = utils._EngineFloat64ScalarType.to_py(wi.contents)
            if data is not None:
                _retval_selecth = select(
                    _wr_selecth,
                    _wi_selecth,
                    data,
                )
            else:
                _retval_selecth = select(
                    _wr_selecth,
                    _wi_selecth,
                )
            _retval_selecth_ct = utils._EngineLogicalScalarType(
                _retval_selecth, {'fun_name': cb_select_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selecth_py = utils._EngineLogicalScalarType.to_py(_retval_selecth_ct)
            if _retval_selecth_py is False:
                _iretval_selecth_py = 0
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selecth_py is True:
                _iretval_selecth_py = 1
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_select_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08paft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvs(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selecth,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vs[ldvs, *].
        utils._BYREF_INT, # Input, ldvs.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vs, 'vs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvs = utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvs in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvs',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='sort',
        ).manage()
    utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = utils._UNUSED_CALLBACK
    _selectsub = utils._UNUSED_CALLBACK
    _selecth = utils._cast_callback(
        selecth, cb_selecth,
        {'fun_name': fun_name, 'entity': 'select'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldvs_py = utils._EngineFloat64ArrayType.get_nd(vs, {'fun_name': fun_name, 'entity': 'vs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvs = utils._EngineIntScalarType(
        _ldvs_py, {'fun_name': fun_name, 'entity': '_ldvs'},
    ).to_ctypes()
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs_cls = utils._EngineFloat64ArrayType(
        vs, {'fun_name': fun_name, 'entity': 'vs'},
        exp_shape=exp_shape_vs,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vs = _vs_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvs,
        _sort,
        _select,
        _selectsub,
        _selecth,
        _n,
        _a,
        _lda,
        _sdim,
        _wr,
        _wi,
        _vs,
        _ldvs,
        _work,
        _lwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvs._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all the eigenvalues.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the Schur form no longer satisfy',
                    'select = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _vs_cls.refresh_py(_vs)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def dgeesx(jobvs, sort, select, sense, n, a, wr, wi, vs, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    real Schur form, and, optionally, the matrix of Schur vectors; also,
    optionally, computes reciprocal condition numbers for selected
    eigenvalues.

    For full information please refer to the NAG Library document for
    f08pb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08pbf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `select`).

    select : None or callable retval = select(wr, wi, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        Parameters
        ~~~~~~~~~~
        wr : float
            The real and imaginary parts of the eigenvalue.

        wi : float
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for average of selected eigenvalues only.

        `sense` = 'V'

          Computed for selected right invariant subspace only.

        `sense` = 'B'

          Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by its real Schur form T.

    wr : float, shape (`n`), modified in place
        On exit: see the description of `wi`.

    wi : float, shape (`n`), modified in place
        On exit: `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues in the same order that
        they appear on the diagonal of the output Schur form T. Complex
        conjugate pairs of eigenvalues will appear consecutively with
        the eigenvalue having the positive imaginary part first.

    vs : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        On exit: if `jobvs` = 'V', `vs` contains the orthogonal matrix Z
        of Schur vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `select` is True. (Complex conjugate pairs for which
        `select` is True for either eigenvalue count as 2.)

    rconde : float
        If `sense` = 'E' or 'B', contains the reciprocal condition
        number for the average of the selected eigenvalues.

        If `sense` = 'N' or 'V', this argument is not referenced.

    rcondv : float
        If `sense` = 'V' or 'B', this argument contains the reciprocal
        condition number for the selected right invariant subspace.

        If `sense` = 'N' or 'E', this argument is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgeesx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_select_name = fun_name + ':select'

    selecth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, wr.
        utils._BYREF_FLOAT64, # Input, wi.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if select is None:
        select_auxil_ctypes = utils._EngineState._get_symbol('f08pazt')
        select_auxil_ctypes.restype = None
        select_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, wr.
            utils._BYREF_FLOAT64, # Input, wi.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selecth(_select, _selectsub, en_data, wr, wi, iretval, iuser, ruser):
        """Helper for select."""
        try:
            if select is None:
                select_auxil_ctypes(
                    en_data,
                    wr,
                    wi,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _wr_selecth = utils._EngineFloat64ScalarType.to_py(wr.contents)
            _wi_selecth = utils._EngineFloat64ScalarType.to_py(wi.contents)
            if data is not None:
                _retval_selecth = select(
                    _wr_selecth,
                    _wi_selecth,
                    data,
                )
            else:
                _retval_selecth = select(
                    _wr_selecth,
                    _wi_selecth,
                )
            _retval_selecth_ct = utils._EngineLogicalScalarType(
                _retval_selecth, {'fun_name': cb_select_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selecth_py = utils._EngineLogicalScalarType.to_py(_retval_selecth_ct)
            if _retval_selecth_py is False:
                _iretval_selecth_py = 0
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selecth_py is True:
                _iretval_selecth_py = 1
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_select_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08pbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvs(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selecth,
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vs[ldvs, *].
        utils._BYREF_INT, # Input, ldvs.
        utils._BYREF_FLOAT64, # Output, rconde.
        utils._BYREF_FLOAT64, # Output, rcondv.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vs, 'vs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvs = utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvs in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvs',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='sort',
        ).manage()
    utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = utils._UNUSED_CALLBACK
    _selectsub = utils._UNUSED_CALLBACK
    _selecth = utils._cast_callback(
        selecth, cb_selecth,
        {'fun_name': fun_name, 'entity': 'select'},
    )
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldvs_py = utils._EngineFloat64ArrayType.get_nd(vs, {'fun_name': fun_name, 'entity': 'vs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvs = utils._EngineIntScalarType(
        _ldvs_py, {'fun_name': fun_name, 'entity': '_ldvs'},
    ).to_ctypes()
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs_cls = utils._EngineFloat64ArrayType(
        vs, {'fun_name': fun_name, 'entity': 'vs'},
        exp_shape=exp_shape_vs,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vs = _vs_cls.to_ctypes(can_reference_contents=False)
    _rconde = utils._EngineFloat64ScalarType.empty_ctypes()
    _rcondv = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvs,
        _sort,
        _select,
        _selectsub,
        _selecth,
        _sense,
        _n,
        _a,
        _lda,
        _sdim,
        _wr,
        _wi,
        _vs,
        _ldvs,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvs._length_,
        _sort._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all the eigenvalues.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the Schur form no longer satisfy',
                    'select = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _vs_cls.refresh_py(_vs)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        _rconde_py = utils._EngineFloat64ScalarType.to_py(_rconde)
        _rcondv_py = utils._EngineFloat64ScalarType.to_py(_rcondv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['sdim', 'rconde', 'rcondv'],
        )(
            _sdim_py,
            _rconde_py,
            _rcondv_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dhseqr(job, compz, n, ilo, ihi, h, wr, wi, z):
    u"""
    Computes the eigenvalues and Schur factorization of real upper
    Hessenberg matrix reduced from real general matrix.

    For full information please refer to the NAG Library document for
    f08pe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08pef.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvalues only or the Schur form T is
        required.

        `job` = 'E'

          Eigenvalues only are required.

        `job` = 'S'

          The Schur form T is required.

    compz : str, length 1
        Indicates whether the Schur vectors are to be computed.

        `compz` = 'N'

          No Schur vectors are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The Schur vectors of A are computed (and the array `z` must
          contain the matrix Q on entry).

        `compz` = 'I'

          The Schur vectors of H are computed (and the array `z` is
          initialized by the function).

    n : int
        n, the order of the matrix H.

    ilo : int
        If the matrix A has been balanced by ``dgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to `n`.

    ihi : int
        If the matrix A has been balanced by ``dgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to `n`.

    h : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper Hessenberg matrix H, as returned by
        ``dgehrd``.

        On exit: if `job` = 'E', the array contains no useful
        information.

        If `job` = 'S', `h` is overwritten by the upper quasi-triangular
        matrix T from the Schur decomposition (the Schur form) unless
        `errno` > 0.

    wr : float, shape (`n`), modified in place
        On exit: the real and imaginary parts, respectively, of the
        computed eigenvalues, unless `errno` > 0 (in which case see
        [Errors]). Complex conjugate pairs of eigenvalues appear
        consecutively with the eigenvalue having positive imaginary part
        first. The eigenvalues are stored in the same order as on the
        diagonal of the Schur form T (if computed); see [Further
        Comments] for details.

    wi : float, shape (`n`), modified in place
        On exit: the real and imaginary parts, respectively, of the
        computed eigenvalues, unless `errno` > 0 (in which case see
        [Errors]). Complex conjugate pairs of eigenvalues appear
        consecutively with the eigenvalue having positive imaginary part
        first. The eigenvalues are stored in the same order as on the
        diagonal of the Schur form T (if computed); see [Further
        Comments] for details.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain the orthogonal
        matrix Q from the reduction to Hessenberg form.

        If `compz` = 'I', `z` need not be set.

        On exit: if `compz` = 'V' or 'I', `z` contains the orthogonal
        matrix of the required Schur vectors, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*(`ihi`-`ilo`+1) iterations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dhseqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08peft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((h, 'h'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'S\', \'s\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-5,),
            entity='ihi',
        ).manage()
    _ldh = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _h_cls = utils._EngineFloat64ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _h = _h_cls.to_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compz,
        _n,
        _ilo,
        _ihi,
        _h,
        _ldh,
        _wr,
        _wi,
        _z,
        _ldz,
        _work,
        _lwork,
        _info,
        _job._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to find all the eigenvalues after a total of',
                    '30*(ihi-ilo+1) iterations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _h_cls.refresh_py(_h)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _z_cls.refresh_py(_z)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dhsein(job, eigsrc, initv, select, n, h, wr, wi, vl, vr, mm, ifaill, ifailr):
    u"""
    Computes selected right and/or left eigenvectors of real upper
    Hessenberg matrix by inverse iteration.

    For full information please refer to the NAG Library document for
    f08pk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08pkf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

          Only right eigenvectors are computed.

        `job` = 'L'

          Only left eigenvectors are computed.

        `job` = 'B'

          Both left and right eigenvectors are computed.

    eigsrc : str, length 1
        Indicates whether the eigenvalues of H (stored in `wr` and `wi`)
        were found using ``dhseqr``.

        `eigsrc` = 'Q'

          The eigenvalues of H were found using ``dhseqr``; thus if H
          has any zero subdiagonal elements (and so is block
          triangular), then the jth eigenvalue can be assumed to be an
          eigenvalue of the block containing the jth row/column. This
          property allows the function to perform inverse iteration on
          just one diagonal block.

        `eigsrc` = 'N'

          No such assumption is made and the function performs inverse
          iteration using the whole matrix.

    initv : str, length 1
        Indicates whether you are supplying initial estimates for the
        selected eigenvectors.

        `initv` = 'N'

          No initial estimates are supplied.

        `initv` = 'U'

          Initial estimates are supplied in `vl` and/or `vr`.

    select : bool, shape (`n`), modified in place
        On entry: specifies which eigenvectors are to be computed. To
        obtain the real eigenvector corresponding to the real eigenvalue
        `wr`[j-1], `select`[j-1] must be set True. To select the complex
        eigenvector corresponding to the complex eigenvalue
        (`wr`[j-1],`wi`[j-1]) with complex conjugate (`wr`[j],`wi`[j]),
        `select`[j-1] and/or `select`[j] must be set True; the
        eigenvector corresponding to the first eigenvalue in the pair is
        computed.

        On exit: if a complex eigenvector was selected as specified
        above, `select`[j-1] is set to True and `select`[j] to False.

    n : int
        n, the order of the matrix H.

    h : float, shape (`n`, `n`)
        The n by n upper Hessenberg matrix H. If a NaN is detected in
        `h`, the function will return with `errno` = -6.

    wr : float, shape (`n`), modified in place
        On entry: the real and imaginary parts, respectively, of the
        eigenvalues of the matrix H. Complex conjugate pairs of values
        must be stored in consecutive elements of the arrays. If
        `eigsrc` = 'Q', the arrays must be exactly as returned by
        ``dhseqr``.

        On exit: some elements of `wr` may be modified, as close
        eigenvalues are perturbed slightly in searching for independent
        eigenvectors.

    wi : float, shape (`n`)
        On entry: the real and imaginary parts, respectively, of the
        eigenvalues of the matrix H. Complex conjugate pairs of values
        must be stored in consecutive elements of the arrays. If
        `eigsrc` = 'Q', the arrays must be exactly as returned by
        ``dhseqr``.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): `n`; if
        `job`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): `mm`; if
        `job`='R': 1; otherwise: 0.

        On entry: if `initv` = 'U' and `job` = 'L' or 'B', `vl` must
        contain starting vectors for inverse iteration for the left
        eigenvectors. Each starting vector must be stored in the same
        column or columns as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vl` need not be set.

        On exit: if `job` = 'L' or 'B', `vl` contains the computed left
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues. Corresponding to each selected real
        eigenvalue is a real eigenvector, occupying one column.
        Corresponding to each selected complex eigenvalue is a complex
        eigenvector, occupying two columns: the first column holds the
        real part and the second column holds the imaginary part.

        If `job` = 'R', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): `n`; if
        `job`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): `mm`; if
        `job`='L': 1; otherwise: 0.

        On entry: if `initv` = 'U' and `job` = 'R' or 'B', `vr` must
        contain starting vectors for inverse iteration for the right
        eigenvectors. Each starting vector must be stored in the same
        column or columns as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vr` need not be set.

        On exit: if `job` = 'R' or 'B', `vr` contains the computed right
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues. Corresponding to each selected real
        eigenvalue is a real eigenvector, occupying one column.
        Corresponding to each selected complex eigenvalue is a complex
        eigenvector, occupying two columns: the first column holds the
        real part and the second column holds the imaginary part.

        If `job` = 'L', `vr` is not referenced.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`. The actual
        number of columns required, m, is obtained by counting 1 for
        each selected real eigenvector and 2 for each selected complex
        eigenvector (see `select`); 0 <= m <= n.

    ifaill : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('L', 'B'): `mm`; if `job`='R': 1;
        otherwise: 0.

        On exit: if `job` = 'L' or 'B', then `ifaill`[i-1] = 0 if the
        selected left eigenvector converged and `ifaill`[i-1] = j > 0 if
        the eigenvector stored in the ith column of `vl` (corresponding
        to the jth eigenvalue as held in (`wr`[j-1],`wi`[j-1]) failed to
        converge. If the ith and (i+1)th columns of `vl` contain a
        selected complex eigenvector, then `ifaill`[i-1] and `ifaill`[i]
        are set to the same value.

        If `job` = 'R', `ifaill` is not referenced.

    ifailr : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('R', 'B'): `mm`; if `job`='L': 1;
        otherwise: 0.

        On exit: if `job` = 'R' or 'B', then `ifailr`[i-1] = 0 if the
        selected right eigenvector converged and `ifailr`[i-1] = j > 0
        if the eigenvector stored in the ith row or column of `vr`
        (corresponding to the jth eigenvalue as held in
        (`wr`[j-1],`wi`[j-1])) failed to converge. If the ith and
        (i+1)th rows or columns of `vr` contain a selected complex
        eigenvector, then `ifailr`[i-1] and `ifailr`[i] are set to the
        same value.

        If `job` = 'L', `ifailr` is not referenced.

    Returns
    -------
    m : int
        m, the number of columns of `vl` and/or `vr` required to store
        the selected eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `eigsrc`.

            Constraint: `eigsrc` = 'Q' or 'N'.

        (`errno` -3)
            On entry, error in parameter `initv`.

            Constraint: `initv` = 'N' or 'U'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `h`.

            Constraint: No element of this argument is equal to NaN.

        (`errno` -14)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by arguments `ifaill`
            and/or `ifailr`) failed to converge. The corresponding
            columns of `vl` and/or `vr` contain no useful information.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dhsein'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08pkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, eigsrc(len=1).
        utils._BYREF_STRING, # Input, initv(len=1).
        utils._EngineLogicalArrayType.c_type, # Input/Output, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Input, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Output, ifaill[*].
        utils._EngineIntArrayType.c_type, # Output, ifailr[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of eigsrc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of initv.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((h, 'h'), (vl, 'vl'), (vr, 'vr')),
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _vl_k_py = 0
    _vl_m_py = 0
    while _vl_k_py < _n_py:
        _vl_k_py = _vl_k_py+1
        if utils._EngineFloat64ArrayType.get_element(wi, {'fun_name': fun_name, 'entity': 'wi'}, (_vl_k_py-1,)) == 0:
            if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
        elif _vl_k_py < _n_py:
            if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                _vl_m_py = _vl_m_py+2
            _vl_k_py = _vl_k_py+1
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('R', 'r', 'L', 'l', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'R\', \'r\', \'L\', \'l\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _eigsrc = utils._EngineCharScalarType(
        eigsrc, {'fun_name': fun_name, 'entity': 'eigsrc'}, exp_chrlen=1,
    ).to_ctypes()
    _eigsrc_py = utils._EngineCharScalarType.to_py(_eigsrc)
    if _eigsrc_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('eigsrc in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='eigsrc',
        ).manage()
    _initv = utils._EngineCharScalarType(
        initv, {'fun_name': fun_name, 'entity': 'initv'}, exp_chrlen=1,
    ).to_ctypes()
    _initv_py = utils._EngineCharScalarType.to_py(_initv)
    if _initv_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('initv in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='initv',
        ).manage()
    _select_cls = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    )
    _select = _select_cls.to_ctypes()
    _ldh = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _h = utils._EngineFloat64ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
    )
    _wr = _wr_cls.to_ctypes()
    _wi = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if not _mm_py >= _vl_m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('mm >= m.'),
            ],
            errcodes=(-14,),
            entity='mm',
        ).manage()
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim1 = _n_py
    elif _job_py in ('R', 'r'):
        _vl_dim1 = 1
    else:
        _vl_dim1 = 0
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = _mm_py
    elif _job_py in ('R', 'r'):
        _vl_dim2 = 1
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
    )
    _vl = _vl_cls.to_ctypes()
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim1 = _n_py
    elif _job_py in ('L', 'l'):
        _vr_dim1 = 1
    else:
        _vr_dim1 = 0
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = _mm_py
    elif _job_py in ('L', 'l'):
        _vr_dim2 = 1
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
    )
    _vr = _vr_cls.to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    if _job_py in ('L', 'l', 'B', 'b'):
        _ifaill_dim1 = _mm_py
    elif _job_py in ('R', 'r'):
        _ifaill_dim1 = 1
    else:
        _ifaill_dim1 = 0
    exp_shape_ifaill = (_ifaill_dim1,)
    _ifaill_cls = utils._EngineIntArrayType(
        ifaill, {'fun_name': fun_name, 'entity': 'ifaill'},
        exp_shape=exp_shape_ifaill,
        check_contents=False,
    )
    _ifaill = _ifaill_cls.to_ctypes(can_reference_contents=False)
    if _job_py in ('R', 'r', 'B', 'b'):
        _ifailr_dim1 = _mm_py
    elif _job_py in ('L', 'l'):
        _ifailr_dim1 = 1
    else:
        _ifailr_dim1 = 0
    exp_shape_ifailr = (_ifailr_dim1,)
    _ifailr_cls = utils._EngineIntArrayType(
        ifailr, {'fun_name': fun_name, 'entity': 'ifailr'},
        exp_shape=exp_shape_ifailr,
        check_contents=False,
    )
    _ifailr = _ifailr_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _eigsrc,
        _initv,
        _select,
        _n,
        _h,
        _ldh,
        _wr,
        _wi,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _ifaill,
        _ifailr,
        _info,
        _job._length_,
        _eigsrc._length_,
        _initv._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} eigenvectors (as indicated by arguments'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'ifaill and/or ifailr) failed to converge.',
                    'The corresponding columns of vl and/or vr contain',
                    'no useful information.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _select_cls.refresh_py(_select)
        _wr_cls.refresh_py(_wr)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        _ifaill_cls.refresh_py(_ifaill)
        _ifailr_cls.refresh_py(_ifailr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zgees(jobvs, sort, select, n, a, w, vs, data=None):
    u"""
    Computes for complex square nonsymmetric matrix, the eigenvalues,
    the Schur form, and, optionally, the matrix of Schur vectors.

    For full information please refer to the NAG Library document for
    f08pn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08pnf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `select`).

    select : None or callable retval = select(w, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        Parameters
        ~~~~~~~~~~
        w : complex
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by its Schur form T.

    w : complex, shape (`n`), modified in place
        On exit: contains the computed eigenvalues, in the same order
        that they appear on the diagonal of the output Schur form T.

    vs : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        On exit: if `jobvs` = 'V', `vs` contains the unitary matrix Z of
        Schur vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues for which
        `select` is True.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgees'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_select_name = fun_name + ':select'

    selecth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, w.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if select is None:
        select_auxil_ctypes = utils._EngineState._get_symbol('f08pnzt')
        select_auxil_ctypes.restype = None
        select_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_COMPLEX128, # Input, w.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selecth(_select, _selectsub, en_data, w, iretval, iuser, ruser):
        """Helper for select."""
        try:
            if select is None:
                select_auxil_ctypes(
                    en_data,
                    w,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _w_selecth = utils._EngineComplex128ScalarType.to_py(w.contents)
            if data is not None:
                _retval_selecth = select(
                    _w_selecth,
                    data,
                )
            else:
                _retval_selecth = select(
                    _w_selecth,
                )
            _retval_selecth_ct = utils._EngineLogicalScalarType(
                _retval_selecth, {'fun_name': cb_select_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selecth_py = utils._EngineLogicalScalarType.to_py(_retval_selecth_ct)
            if _retval_selecth_py is False:
                _iretval_selecth_py = 0
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selecth_py is True:
                _iretval_selecth_py = 1
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_select_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08pnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvs(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selecth,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vs[ldvs, *].
        utils._BYREF_INT, # Input, ldvs.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vs, 'vs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvs = utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvs in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvs',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='sort',
        ).manage()
    utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = utils._UNUSED_CALLBACK
    _selectsub = utils._UNUSED_CALLBACK
    _selecth = utils._cast_callback(
        selecth, cb_selecth,
        {'fun_name': fun_name, 'entity': 'select'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldvs_py = utils._EngineComplex128ArrayType.get_nd(vs, {'fun_name': fun_name, 'entity': 'vs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvs = utils._EngineIntScalarType(
        _ldvs_py, {'fun_name': fun_name, 'entity': '_ldvs'},
    ).to_ctypes()
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs_cls = utils._EngineComplex128ArrayType(
        vs, {'fun_name': fun_name, 'entity': 'vs'},
        exp_shape=exp_shape_vs,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vs = _vs_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvs,
        _sort,
        _select,
        _selectsub,
        _selecth,
        _n,
        _a,
        _lda,
        _sdim,
        _w,
        _vs,
        _ldvs,
        _work,
        _lwork,
        _rwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvs._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all the eigenvalues.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the Schur form no longer satisfy',
                    'select = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _vs_cls.refresh_py(_vs)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def zgeesx(jobvs, sort, select, sense, n, a, w, vs, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    Schur form, and, optionally, the matrix of Schur vectors; also
    computes a reciprocal condition number for the average of the
    selected eigenvalues and for the right invariant subspace
    corresponding to these eigenvalues.

    For full information please refer to the NAG Library document for
    f08pp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ppf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `select`).

    select : None or callable retval = select(w, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        Parameters
        ~~~~~~~~~~
        w : complex
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for average of selected eigenvalues only.

        `sense` = 'V'

          Computed for selected right invariant subspace only.

        `sense` = 'B'

          Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by its Schur form T.

    w : complex, shape (`n`), modified in place
        On exit: contains the computed eigenvalues, in the same order
        that they appear on the diagonal of the output Schur form T.

    vs : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvs`='V': `n`; otherwise: 1.

        On exit: if `jobvs` = 'V', `vs` contains the unitary matrix Z of
        Schur vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues for which
        `select` is True.

    rconde : float
        If `sense` = 'E' or 'B', contains the reciprocal condition
        number for the average of the selected eigenvalues.

        If `sense` = 'N' or 'V', this argument is not referenced.

    rcondv : float
        If `sense` = 'V' or 'B', this argument contains the reciprocal
        condition number for the selected right invariant subspace.

        If `sense` = 'N' or 'E', this argument is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgeesx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_select_name = fun_name + ':select'

    selecth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, w.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if select is None:
        select_auxil_ctypes = utils._EngineState._get_symbol('f08pnzt')
        select_auxil_ctypes.restype = None
        select_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_COMPLEX128, # Input, w.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selecth(_select, _selectsub, en_data, w, iretval, iuser, ruser):
        """Helper for select."""
        try:
            if select is None:
                select_auxil_ctypes(
                    en_data,
                    w,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _w_selecth = utils._EngineComplex128ScalarType.to_py(w.contents)
            if data is not None:
                _retval_selecth = select(
                    _w_selecth,
                    data,
                )
            else:
                _retval_selecth = select(
                    _w_selecth,
                )
            _retval_selecth_ct = utils._EngineLogicalScalarType(
                _retval_selecth, {'fun_name': cb_select_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selecth_py = utils._EngineLogicalScalarType.to_py(_retval_selecth_ct)
            if _retval_selecth_py is False:
                _iretval_selecth_py = 0
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selecth_py is True:
                _iretval_selecth_py = 1
                utils._EngineIntScalarType(
                    _iretval_selecth_py, {'fun_name': cb_select_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_select_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08ppft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvs(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selecth,
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vs[ldvs, *].
        utils._BYREF_INT, # Input, ldvs.
        utils._BYREF_FLOAT64, # Output, rconde.
        utils._BYREF_FLOAT64, # Output, rcondv.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vs, 'vs')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvs = utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvs in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvs',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='sort',
        ).manage()
    utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = utils._UNUSED_CALLBACK
    _selectsub = utils._UNUSED_CALLBACK
    _selecth = utils._cast_callback(
        selecth, cb_selecth,
        {'fun_name': fun_name, 'entity': 'select'},
    )
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldvs_py = utils._EngineComplex128ArrayType.get_nd(vs, {'fun_name': fun_name, 'entity': 'vs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvs = utils._EngineIntScalarType(
        _ldvs_py, {'fun_name': fun_name, 'entity': '_ldvs'},
    ).to_ctypes()
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs_cls = utils._EngineComplex128ArrayType(
        vs, {'fun_name': fun_name, 'entity': 'vs'},
        exp_shape=exp_shape_vs,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vs = _vs_cls.to_ctypes(can_reference_contents=False)
    _rconde = utils._EngineFloat64ScalarType.empty_ctypes()
    _rcondv = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvs,
        _sort,
        _select,
        _selectsub,
        _selecth,
        _sense,
        _n,
        _a,
        _lda,
        _sdim,
        _w,
        _vs,
        _ldvs,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _rwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvs._length_,
        _sort._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to compute all the eigenvalues.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the Schur form no longer satisfy',
                    'select = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _vs_cls.refresh_py(_vs)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        _rconde_py = utils._EngineFloat64ScalarType.to_py(_rconde)
        _rcondv_py = utils._EngineFloat64ScalarType.to_py(_rcondv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['sdim', 'rconde', 'rcondv'],
        )(
            _sdim_py,
            _rconde_py,
            _rcondv_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zhseqr(job, compz, n, ilo, ihi, h, w, z):
    u"""
    Computes the eigenvalues and Schur factorization of complex upper
    Hessenberg matrix reduced from complex general matrix.

    For full information please refer to the NAG Library document for
    f08ps

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08psf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvalues only or the Schur form T is
        required.

        `job` = 'E'

          Eigenvalues only are required.

        `job` = 'S'

          The Schur form T is required.

    compz : str, length 1
        Indicates whether the Schur vectors are to be computed.

        `compz` = 'N'

          No Schur vectors are computed (and the array `z` is not
          referenced).

        `compz` = 'V'

          The Schur vectors of A are computed (and the array `z` must
          contain the matrix Q on entry).

        `compz` = 'I'

          The Schur vectors of H are computed (and the array `z` is
          initialized by the function).

    n : int
        n, the order of the matrix H.

    ilo : int
        If the matrix A has been balanced by ``zgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to `n`.

    ihi : int
        If the matrix A has been balanced by ``zgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to `n`.

    h : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper Hessenberg matrix H, as returned by
        ``zgehrd``.

        On exit: if `job` = 'E', the array contains no useful
        information.

        If `job` = 'S', `h` is overwritten by the upper triangular
        matrix T from the Schur decomposition (the Schur form) unless
        `errno` > 0.

    w : complex, shape (`n`), modified in place
        On exit: the computed eigenvalues, unless `errno` > 0 (in which
        case see [Errors]). The eigenvalues are stored in the same order
        as on the diagonal of the Schur form T (if computed).

    z : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        Optionally, on entry: if `compz` = 'V', `z` must contain the
        unitary matrix Q from the reduction to Hessenberg form.

        If `compz` = 'I', `z` need not be set.

        On exit, if not None on entry: if `compz` = 'V' or 'I', `z`
        contains the unitary matrix of the required Schur vectors,
        unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*(`ihi`-`ilo`+1) iterations.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhseqr'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08psft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((h, 'h'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'S\', \'s\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if not (_ihi_py >= min(_ilo_py, _n_py) and _ihi_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ihi >= min(ilo, n).'),
            ],
            errcodes=(-5,),
            entity='ihi',
        ).manage()
    _ldh = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _h_cls = utils._EngineComplex128ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _h = _h_cls.to_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v'):
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz in (\'V\', \'v\')',
            )
    if _compz_py not in ('N', 'n'):
        if z is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz not in (\'N\', \'n\')',
            )
    if z is None:
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim1 = max(1, _n_py)
        elif _compz_py in ('N', 'n'):
            _z_dim1 = 1
        else:
            _z_dim1 = 0
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim2 = _n_py
        elif _compz_py in ('N', 'n'):
            _z_dim2 = 1
        else:
            _z_dim2 = 0
        _z = utils._EngineComplex128ArrayType.empty_ctypes((_z_dim1)*(_z_dim2))
    else:
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim1 = max(1, _n_py)
        elif _compz_py in ('N', 'n'):
            _z_dim1 = 1
        else:
            _z_dim1 = 0
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim2 = _n_py
        elif _compz_py in ('N', 'n'):
            _z_dim2 = 1
        else:
            _z_dim2 = 0
        exp_shape_z = (_z_dim1, _z_dim2)
        _z_cls = utils._EngineComplex128ArrayType(
            z, {'fun_name': fun_name, 'entity': 'z'},
            exp_shape=exp_shape_z,
            mutual_sorder=_m_sorder,
            check_contents=_compz_py in ('V', 'v'),
            allow_none=True,
        )
        _z = _z_cls.to_ctypes(can_reference_contents=_compz_py in ('V', 'v'))
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compz,
        _n,
        _ilo,
        _ihi,
        _h,
        _ldh,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _info,
        _job._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == -999:
            _err_class = utils._EngineAllocationErrorExit(
                fun_name=fun_name,
            )
        elif _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to find all the eigenvalues after a total of',
                    '30*(ihi-ilo+1) iterations.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _h_cls.refresh_py(_h)
        _w_cls.refresh_py(_w)
        if z is not None:
            _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhsein(job, eigsrc, initv, select, n, h, w, vl, vr, mm, ifaill, ifailr):
    u"""
    Computes selected right and/or left eigenvectors of complex upper
    Hessenberg matrix by inverse iteration.

    For full information please refer to the NAG Library document for
    f08px

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08pxf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

          Only right eigenvectors are computed.

        `job` = 'L'

          Only left eigenvectors are computed.

        `job` = 'B'

          Both left and right eigenvectors are computed.

    eigsrc : str, length 1
        Indicates whether the eigenvalues of H (stored in `w`) were
        found using ``zhseqr``.

        `eigsrc` = 'Q'

          The eigenvalues of H were found using ``zhseqr``; thus if H
          has any zero subdiagonal elements (and so is block
          triangular), then the jth eigenvalue can be assumed to be an
          eigenvalue of the block containing the jth row/column. This
          property allows the function to perform inverse iteration on
          just one diagonal block.

        `eigsrc` = 'N'

          No such assumption is made and the function performs inverse
          iteration using the whole matrix.

    initv : str, length 1
        Indicates whether you are supplying initial estimates for the
        selected eigenvectors.

        `initv` = 'N'

          No initial estimates are supplied.

        `initv` = 'U'

          Initial estimates are supplied in `vl` and/or `vr`.

    select : bool, shape (`n`)
        Specifies which eigenvectors are to be computed. To select the
        eigenvector corresponding to the eigenvalue `w`[j-1],
        `select`[j-1] must be set to True.

    n : int
        n, the order of the matrix H.

    h : complex, shape (`n`, `n`)
        The n by n upper Hessenberg matrix H. If a NaN is detected in
        `h`, the function will return with `errno` = -6.

    w : complex, shape (`n`), modified in place
        On entry: the eigenvalues of the matrix H. If `eigsrc` = 'Q',
        the array must be exactly as returned by ``zhseqr``.

        On exit: the real parts of some elements of `w` may be modified,
        as close eigenvalues are perturbed slightly in searching for
        independent eigenvectors.

    vl : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): `n`; if
        `job`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): `mm`; if
        `job`='R': 1; otherwise: 0.

        Optionally, on entry: if `initv` = 'U' and `job` = 'L' or 'B',
        `vl` must contain starting vectors for inverse iteration for the
        left eigenvectors. Each starting vector must be stored in the
        same column as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vl` need not be set.

        On exit, if not None on entry: if `job` = 'L' or 'B', `vl`
        contains the computed left eigenvectors (as specified by
        `select`). The eigenvectors are stored consecutively in the
        columns of the array, in the same order as their eigenvalues.

        If `job` = 'R', `vl` is not referenced.

    vr : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): `n`; if
        `job`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): `mm`; if
        `job`='L': 1; otherwise: 0.

        Optionally, on entry: if `initv` = 'U' and `job` = 'R' or 'B',
        `vr` must contain starting vectors for inverse iteration for the
        right eigenvectors. Each starting vector must be stored in the
        same column as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vr` need not be set.

        On exit, if not None on entry: if `job` = 'R' or 'B', `vr`
        contains the computed right eigenvectors (as specified by
        `select`). The eigenvectors are stored consecutively in the
        columns of the array, in the same order as their eigenvalues.

        If `job` = 'L', `vr` is not referenced.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`. This must
        be an upper bound on the actual number of columns required, that
        is, the number of elements of `select`, in the first `n`, that
        are set to True.

    ifaill : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('L', 'B'): `mm`; if `job`='R': 1;
        otherwise: 0.

        On exit: if `job` = 'L' or 'B', then `ifaill`[i-1] = 0 if the
        selected left eigenvector converged and `ifaill`[i-1] = j > 0 if
        the eigenvector stored in the ith row or column of `vl`
        (corresponding to the jth eigenvalue) failed to converge.

        If `job` = 'R', `ifaill` is not referenced.

    ifailr : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('R', 'B'): `mm`; if `job`='L': 1;
        otherwise: 0.

        On exit: if `job` = 'R' or 'B', then `ifailr`[i-1] = 0 if the
        selected right eigenvector converged and `ifailr`[i-1] = j > 0
        if the eigenvector stored in the ith column of `vr`
        (corresponding to the jth eigenvalue) failed to converge.

        If `job` = 'L', `ifailr` is not referenced.

    Returns
    -------
    m : int
        m, the number of selected eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `eigsrc`.

            Constraint: `eigsrc` = 'Q' or 'N'.

        (`errno` -3)
            On entry, error in parameter `initv`.

            Constraint: `initv` = 'N' or 'U'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `h`.

            Constraint: No element of this argument is equal to NaN.

        (`errno` -13)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.

        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by arguments `ifaill`
            and/or `ifailr`) failed to converge. The corresponding
            columns of `vl` and/or `vr` contain no useful information.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhsein'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08pxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, eigsrc(len=1).
        utils._BYREF_STRING, # Input, initv(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, h[ldh, n].
        utils._BYREF_INT, # Input, ldh.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Output, ifaill[*].
        utils._EngineIntArrayType.c_type, # Output, ifailr[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of eigsrc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of initv.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((h, 'h'), (vl, 'vl'), (vr, 'vr')),
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _vl_m_py = 0
    for _vl_j_py in range(1, _n_py+1):
        if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
            _vl_m_py = _vl_m_py + 1
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('R', 'r', 'L', 'l', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'R\', \'r\', \'L\', \'l\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _eigsrc = utils._EngineCharScalarType(
        eigsrc, {'fun_name': fun_name, 'entity': 'eigsrc'}, exp_chrlen=1,
    ).to_ctypes()
    _eigsrc_py = utils._EngineCharScalarType.to_py(_eigsrc)
    if _eigsrc_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('eigsrc in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='eigsrc',
        ).manage()
    _initv = utils._EngineCharScalarType(
        initv, {'fun_name': fun_name, 'entity': 'initv'}, exp_chrlen=1,
    ).to_ctypes()
    _initv_py = utils._EngineCharScalarType.to_py(_initv)
    if _initv_py not in ('N', 'n', 'U', 'u'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('initv in (\'N\', \'n\', \'U\', \'u\').'),
            ],
            errcodes=(-3,),
            entity='initv',
        ).manage()
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldh = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldh'},
    ).to_ctypes()
    _h = utils._EngineComplex128ArrayType(
        h, {'fun_name': fun_name, 'entity': 'h'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
    )
    _w = _w_cls.to_ctypes()
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if not _mm_py >= _vl_m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('mm >= m.'),
            ],
            errcodes=(-13,),
            entity='mm',
        ).manage()
    if (_initv_py in ('U', 'u')) and _job_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (initv in (\'U\', \'u\')) and job in (\'L\', \'l\', \'B\', \'b\')',
            )
    if _job_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job in (\'L\', \'l\', \'B\', \'b\')',
            )
    if vl is None:
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _job_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        elif _job_py in ('R', 'r'):
            _vl_dim2 = 1
        else:
            _vl_dim2 = 0
        _vl = utils._EngineComplex128ArrayType.empty_ctypes((_vl_dim1)*(_vl_dim2))
    else:
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _job_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        elif _job_py in ('R', 'r'):
            _vl_dim2 = 1
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl_cls = utils._EngineComplex128ArrayType(
            vl, {'fun_name': fun_name, 'entity': 'vl'},
            exp_shape=exp_shape_vl,
            mutual_sorder=_m_sorder,
            check_contents=(_initv_py in ('U', 'u')) and _job_py in ('L', 'l', 'B', 'b'),
            allow_none=True,
        )
        _vl = _vl_cls.to_ctypes(can_reference_contents=(_initv_py in ('U', 'u')) and _job_py in ('L', 'l', 'B', 'b'))
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if (_initv_py in ('U', 'u')) and _job_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (initv in (\'U\', \'u\')) and job in (\'R\', \'r\', \'B\', \'b\')',
            )
    if _job_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job in (\'R\', \'r\', \'B\', \'b\')',
            )
    if vr is None:
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _job_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        elif _job_py in ('L', 'l'):
            _vr_dim2 = 1
        else:
            _vr_dim2 = 0
        _vr = utils._EngineComplex128ArrayType.empty_ctypes((_vr_dim1)*(_vr_dim2))
    else:
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _job_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        elif _job_py in ('L', 'l'):
            _vr_dim2 = 1
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr_cls = utils._EngineComplex128ArrayType(
            vr, {'fun_name': fun_name, 'entity': 'vr'},
            exp_shape=exp_shape_vr,
            mutual_sorder=_m_sorder,
            check_contents=(_initv_py in ('U', 'u')) and _job_py in ('R', 'r', 'B', 'b'),
            allow_none=True,
        )
        _vr = _vr_cls.to_ctypes(can_reference_contents=(_initv_py in ('U', 'u')) and _job_py in ('R', 'r', 'B', 'b'))
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    if _job_py in ('L', 'l', 'B', 'b'):
        _ifaill_dim1 = _mm_py
    elif _job_py in ('R', 'r'):
        _ifaill_dim1 = 1
    else:
        _ifaill_dim1 = 0
    exp_shape_ifaill = (_ifaill_dim1,)
    _ifaill_cls = utils._EngineIntArrayType(
        ifaill, {'fun_name': fun_name, 'entity': 'ifaill'},
        exp_shape=exp_shape_ifaill,
        check_contents=False,
    )
    _ifaill = _ifaill_cls.to_ctypes(can_reference_contents=False)
    if _job_py in ('R', 'r', 'B', 'b'):
        _ifailr_dim1 = _mm_py
    elif _job_py in ('L', 'l'):
        _ifailr_dim1 = 1
    else:
        _ifailr_dim1 = 0
    exp_shape_ifailr = (_ifailr_dim1,)
    _ifailr_cls = utils._EngineIntArrayType(
        ifailr, {'fun_name': fun_name, 'entity': 'ifailr'},
        exp_shape=exp_shape_ifailr,
        check_contents=False,
    )
    _ifailr = _ifailr_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _eigsrc,
        _initv,
        _select,
        _n,
        _h,
        _ldh,
        _w,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _rwork,
        _ifaill,
        _ifailr,
        _info,
        _job._length_,
        _eigsrc._length_,
        _initv._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    '{:s} eigenvectors (as indicated by arguments'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'ifaill and/or ifailr) failed to converge.',
                    'The corresponding columns of vl and/or vr contain',
                    'no useful information.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _w_cls.refresh_py(_w)
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        _ifaill_cls.refresh_py(_ifaill)
        _ifailr_cls.refresh_py(_ifailr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dtrexc(compq, n, t, q, ifst, ilst):
    u"""
    Reorders a Schur factorization of real matrix using orthogonal
    similarity transformation.

    For full information please refer to the NAG Library document for
    f08qf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qff.html

    Parameters
    ----------
    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

          The matrix Q of Schur vectors is updated.

        `compq` = 'N'

          No Schur vectors are updated.

    n : int
        n, the order of the matrix T.

    t : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper quasi-triangular matrix T in
        canonical Schur form, as returned by ``dhseqr``.

        On exit: `t` is overwritten by the updated matrix T~. See also
        [Further Comments].

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain the n by n
        orthogonal matrix Q of Schur vectors.

        On exit: if `compq` = 'V', `q` contains the updated matrix of
        Schur vectors.

        If `compq` = 'N', `q` is not referenced.

    ifst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements or blocks of T. The element or block with row index
        `ifst` is moved to row `ilst` by a sequence of exchanges between
        adjacent elements or blocks.

    ilst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements or blocks of T. The element or block with row index
        `ifst` is moved to row `ilst` by a sequence of exchanges between
        adjacent elements or blocks.

    Returns
    -------
    ifst : int
        If `ifst` pointed to the second row of a 2 by 2 block on entry,
        it is changed to point to the first row. `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +/-1).

    ilst : int
        If `ifst` pointed to the second row of a 2 by 2 block on entry,
        it is changed to point to the first row. `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +/-1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `ifst`.

        (`errno` -8)
            On entry, error in parameter `ilst`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Two adjacent diagonal elements or blocks could not be
            successfully exchanged. This error can only occur if the
            exchange involves at least one 2 by 2 block; it implies that
            the problem is very ill-conditioned, and that the
            eigenvalues of the two blocks are very close. On exit, T may
            have been partially reordered, and `ilst` points to the
            first row of the current position of the block being moved;
            Q (if requested) is updated consistently with T.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtrexc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_INT, # Input/Output, ifst.
        utils._BYREF_INT, # Input/Output, ilst.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ifst = utils._EngineIntScalarType(
        ifst, {'fun_name': fun_name, 'entity': 'ifst'},
    ).to_ctypes()
    _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
    if not (_ifst_py >= 1 and _ifst_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ifst and ifst <= n.'),
            ],
            errcodes=(-7,),
            entity='ifst',
        ).manage()
    _ilst = utils._EngineIntScalarType(
        ilst, {'fun_name': fun_name, 'entity': 'ilst'},
    ).to_ctypes()
    _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
    if not (_ilst_py >= 1 and _ilst_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ilst and ilst <= n.'),
            ],
            errcodes=(-8,),
            entity='ilst',
        ).manage()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _n,
        _t,
        _ldt,
        _q,
        _ldq,
        _ifst,
        _ilst,
        _work,
        _info,
        _compq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Two adjacent diagonal elements or blocks could not be successfully',
                    'exchanged. This error can only occur if the exchange involves at least',
                    'one 2 by 2 block; it implies that the problem is very ill-conditioned,',
                    'and that the eigenvalues of the two blocks are very close. On exit,',
                    'T may have been partially reordered, and ilst',
                    'points to the first row of the current position of the block being',
                    'moved; Q (if requested) is updated consistently with T.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_cls.refresh_py(_t)
        _q_cls.refresh_py(_q)
        _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
        _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ifst', 'ilst'],
        )(
            _ifst_py,
            _ilst_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtrsen(job, compq, select, n, t, q, wr, wi):
    u"""
    Reorders a Schur factorization of real matrix, form orthonormal
    basis of right invariant subspace for selected eigenvalues, with
    estimates of sensitivities.

    For full information please refer to the NAG Library document for
    f08qg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qgf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for the cluster
        of eigenvalues and/or the invariant subspace.

        `job` = 'N'

          No condition numbers are required.

        `job` = 'E'

          Only the condition number for the cluster of eigenvalues is
          computed.

        `job` = 'V'

          Only the condition number for the invariant subspace is
          computed.

        `job` = 'B'

          Condition numbers for both the cluster of eigenvalues and the
          invariant subspace are computed.

    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

          The matrix Q of Schur vectors is updated.

        `compq` = 'N'

          No Schur vectors are updated.

    select : bool, shape (`n`)
        The eigenvalues in the selected cluster. To select a real
        eigenvalue lambda_j, `select`[j-1] must be set True. To select a
        complex conjugate pair of eigenvalues lambda_j and lambda_{j+1}
        (corresponding to a 2 by 2 diagonal block), `select`[j-1] and/or
        `select`[j] must be set to True. A complex conjugate pair of
        eigenvalues must be either both included in the cluster or both
        excluded. See also [Further Comments].

    n : int
        n, the order of the matrix T.

    t : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper quasi-triangular matrix T in
        canonical Schur form, as returned by ``dhseqr``. See also
        [Further Comments].

        On exit: `t` is overwritten by the updated matrix T~.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain the n by n
        orthogonal matrix Q of Schur vectors, as returned by ``dhseqr``.

        On exit: if `compq` = 'V', `q` contains the updated matrix of
        Schur vectors; the first m columns of Q form an orthonormal
        basis for the specified invariant subspace.

        If `compq` = 'N', `q` is not referenced.

    wr : float, shape (`n`), modified in place
        On exit: the real and imaginary parts, respectively, of the
        reordered eigenvalues of T~. The eigenvalues are stored in the
        same order as on the diagonal of T~; see [Further Comments] for
        details. Note that if a complex eigenvalue is sufficiently
        ill-conditioned, then its value may differ significantly from
        its value before reordering.

    wi : float, shape (`n`), modified in place
        On exit: the real and imaginary parts, respectively, of the
        reordered eigenvalues of T~. The eigenvalues are stored in the
        same order as on the diagonal of T~; see [Further Comments] for
        details. Note that if a complex eigenvalue is sufficiently
        ill-conditioned, then its value may differ significantly from
        its value before reordering.

    Returns
    -------
    m : int
        m, the dimension of the specified invariant subspace. The value
        of m is obtained by counting 1 for each selected real eigenvalue
        and 2 for each selected complex conjugate pair of eigenvalues
        (see `select`); 0 <= m <= n.

    s : float
        If `job` = 'E' or 'B', `s` is a lower bound on the reciprocal
        condition number of the average of the selected cluster of
        eigenvalues. If `m` = 0 or `n`, `s` = 1; if `errno` = 1 (see
        [Errors]), `s` is set to zero.

        If `job` = 'N' or 'V', `s` is not referenced.

    sep : float
        If `job` = 'V' or 'B', `sep` is the estimated reciprocal
        condition number of the specified invariant subspace. If `m` = 0
        or `n`, `sep` = ||T||; if `errno` = 1 (see [Errors]), `sep` is
        set to zero.

        If `job` = 'N' or 'E', `sep` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The reordering of T failed because a selected eigenvalue was
            too close to an unselected eigenvalue.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtrsen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._BYREF_INT, # Output, m.
        utils._BYREF_FLOAT64, # Output, s.
        utils._BYREF_FLOAT64, # Output, sep.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='compq',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _s = utils._EngineFloat64ScalarType.empty_ctypes()
    _sep = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compq,
        _select,
        _n,
        _t,
        _ldt,
        _q,
        _ldq,
        _wr,
        _wi,
        _m,
        _s,
        _sep,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _job._length_,
        _compq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The reordering of T failed because a selected eigenvalue was too',
                    'close to an unselected eigenvalue.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_cls.refresh_py(_t)
        _q_cls.refresh_py(_q)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _s_py = utils._EngineFloat64ScalarType.to_py(_s)
        _sep_py = utils._EngineFloat64ScalarType.to_py(_sep)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 's', 'sep'],
        )(
            _m_py,
            _s_py,
            _sep_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dtrsyl(trana, tranb, isgn, m, n, a, b, c):
    u"""
    Solves the real Sylvester matrix equation AX+XB = C, A and B are
    upper quasi-triangular or transposes.

    For full information please refer to the NAG Library document for
    f08qh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qhf.html

    Parameters
    ----------
    trana : str, length 1
        Specifies the option op(A).

        `trana` = 'N'

          op(A) = A.

        `trana` = 'T' or 'C'

          op(A) = A^T.

    tranb : str, length 1
        Specifies the option op(B).

        `tranb` = 'N'

          op(B) = B.

        `tranb` = 'T' or 'C'

          op(B) = B^T.

    isgn : int
        Indicates the form of the Sylvester equation.

        `isgn` = +1

          The equation is of the form op(A)X+Xop(B) = alpha C.

        `isgn` = -1

          The equation is of the form op(A)X-Xop(B) = alpha C.

    m : int
        m, the order of the matrix A, and the number of rows in the
        matrices X and C.

    n : int
        n, the order of the matrix B, and the number of columns in the
        matrices X and C.

    a : float, shape (`m`, `m`)
        The m by m upper quasi-triangular matrix A in canonical Schur
        form, as returned by ``dhseqr``.

    b : float, shape (`n`, `n`)
        The n by n upper quasi-triangular matrix B in canonical Schur
        form, as returned by ``dhseqr``.

    c : float, shape (`m`, `n`), modified in place
        On entry: the m by n right-hand side matrix C.

        On exit: `c` is overwritten by the solution matrix X.

    Returns
    -------
    scale : float
        The value of the scale factor alpha.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trana`.

            Constraint: `trana` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `tranb`.

            Constraint: `tranb` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `isgn`.

            Constraint: `isgn` = +1 or -1.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A and B have common or close eigenvalues, perturbed values
            of which were used to solve the equation.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtrsyl'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trana(len=1).
        utils._BYREF_STRING, # Input, tranb(len=1).
        utils._BYREF_INT, # Input, isgn.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._BYREF_FLOAT64, # Output, scale.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trana.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tranb.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trana = utils._EngineCharScalarType(
        trana, {'fun_name': fun_name, 'entity': 'trana'}, exp_chrlen=1,
    ).to_ctypes()
    _trana_py = utils._EngineCharScalarType.to_py(_trana)
    if _trana_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trana in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trana',
        ).manage()
    _tranb = utils._EngineCharScalarType(
        tranb, {'fun_name': fun_name, 'entity': 'tranb'}, exp_chrlen=1,
    ).to_ctypes()
    _tranb_py = utils._EngineCharScalarType.to_py(_tranb)
    if _tranb_py not in ('N', 'n', 'T', 't', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('tranb in (\'N\', \'n\', \'T\', \'t\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='tranb',
        ).manage()
    _isgn = utils._EngineIntScalarType(
        isgn, {'fun_name': fun_name, 'entity': 'isgn'},
    ).to_ctypes()
    _isgn_py = utils._EngineIntScalarType.to_py(_isgn)
    if _isgn_py not in (+1, -1):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('isgn in (+1, -1).'),
            ],
            errcodes=(-3,),
            entity='isgn',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _scale = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trana,
        _tranb,
        _isgn,
        _m,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _scale,
        _info,
        _trana._length_,
        _tranb._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A and B have common or close eigenvalues, perturbed values of which were',
                    'used to solve the equation.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _c_cls.refresh_py(_c)
        _scale_py = utils._EngineFloat64ScalarType.to_py(_scale)
        return_data = _scale_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['scale'],
            )(
                _scale_py,
            )
    _err_class.manage()
    return return_data

def dtrevc(job, howmny, select, n, t, vl, vr, mm):
    u"""
    Computes left and right eigenvectors of real upper quasi-triangular
    matrix.

    For full information please refer to the NAG Library document for
    f08qk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qkf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

          Only right eigenvectors are computed.

        `job` = 'L'

          Only left eigenvectors are computed.

        `job` = 'B'

          Both left and right eigenvectors are computed.

    howmny : str, length 1
        Indicates how many eigenvectors are to be computed.

        `howmny` = 'A'

          All eigenvectors (as specified by `job`) are computed.

        `howmny` = 'B'

          All eigenvectors (as specified by `job`) are computed and then
          pre-multiplied by the matrix Q (which is overwritten).

        `howmny` = 'S'

          Selected eigenvectors (as specified by `job` and `select`) are
          computed.

    select : None or bool, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Optionally, on entry: specifies which eigenvectors are to be
        computed if `howmny` = 'S'. To obtain the real eigenvector
        corresponding to the real eigenvalue lambda_j, `select`[j-1]
        must be set True. To select the complex eigenvector
        corresponding to a complex conjugate pair of eigenvalues
        lambda_j and lambda_{j+1}, `select`[j-1] and/or `select`[j] must
        be set True; the eigenvector corresponding to the first
        eigenvalue in the pair is computed.

        On exit, if not None on entry: if a complex eigenvector was
        selected as specified above, `select`[j-1] is set to True and
        `select`[j] to False.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    n : int
        n, the order of the matrix T.

    t : float, shape (`n`, `n`)
        The n by n upper quasi-triangular matrix T in canonical Schur
        form, as returned by ``dhseqr``.

    vl : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): `n`; if
        `job`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): `mm`; otherwise:
        0.

        Optionally, on entry: if `howmny` = 'B' and `job` = 'L' or 'B',
        `vl` must contain an n by n matrix Q (usually the matrix of
        Schur vectors returned by ``dhseqr``).

        If `howmny` = 'A' or 'S', `vl` need not be set.

        On exit, if not None on entry: if `job` = 'L' or 'B', `vl`
        contains the computed left eigenvectors (as specified by
        `howmny` and `select`). The eigenvectors are stored
        consecutively in the columns of the array, in the same order as
        their eigenvalues. Corresponding to each real eigenvalue is a
        real eigenvector, occupying one column. Corresponding to each
        complex conjugate pair of eigenvalues, is a complex eigenvector
        occupying two columns; the first column holds the real part and
        the second column holds the imaginary part.

        If `job` = 'R', `vl` is not referenced.

    vr : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): `n`; if
        `job`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): `mm`; otherwise:
        0.

        Optionally, on entry: if `howmny` = 'B' and `job` = 'R' or 'B',
        `vr` must contain an n by n matrix Q (usually the matrix of
        Schur vectors returned by ``dhseqr``).

        If `howmny` = 'A' or 'S', `vr` need not be set.

        On exit, if not None on entry: if `job` = 'R' or 'B', `vr`
        contains the computed right eigenvectors (as specified by
        `howmny` and `select`). The eigenvectors are stored
        consecutively in the columns of the array, in the same order as
        their eigenvalues. Corresponding to each real eigenvalue is a
        real eigenvector, occupying one column. Corresponding to each
        complex conjugate pair of eigenvalues, is a complex eigenvector
        occupying two columns; the first column holds the real part and
        the second column holds the imaginary part.

        If `job` = 'L', `vr` is not referenced.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`. The
        precise number of columns required, m, is n if `howmny` = 'A' or
        'B'; if `howmny` = 'S', m is obtained by counting 1 for each
        selected real eigenvector and 2 for each selected complex
        eigenvector (see `select`), in which case 0 <= m <= n.

    Returns
    -------
    m : int
        m, the number of columns of `vl` and/or `vr` actually used to
        store the computed eigenvectors. If `howmny` = 'A' or 'B', `m`
        is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B', 'O' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -11)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= `n`.

        (`errno` -11)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtrevc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input/Output, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (vl, 'vl'), (vr, 'vr')),
    )
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'B', 'b', 'O', 'o', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'B\', \'b\', \'O\', \'o\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _vl_k_py = 0
    if (_howmny_py in ('S', 's')) and utils._EngineFloat64ArrayType.get_shape(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if utils._EngineFloat64ArrayType.get_element(t, {'fun_name': fun_name, 'entity': 't'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('R', 'r', 'L', 'l', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'R\', \'r\', \'L\', \'l\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    if _howmny_py in ('S', 's'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny in (\'S\', \'s\')',
            )
    if _howmny_py in ('S', 's'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny in (\'S\', \'s\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select_cls = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py in ('S', 's'),
        allow_none=True,
    )
    _select = _select_cls.to_ctypes(can_reference_contents=_howmny_py in ('S', 's'))
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if _howmny_py in ('A', 'a', 'B', 'b'):
        if not _mm_py >= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= n.'),
                ],
                errcodes=(-11,),
                entity='mm',
            ).manage()
    else:
        if not _mm_py >= _vl_m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= m.'),
                ],
                errcodes=(-11,),
                entity='mm',
            ).manage()
    if (_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'R\', \'r\')',
            )
    if _job_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job not in (\'R\', \'r\')',
            )
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim1 = _n_py
    elif _job_py in ('R', 'r'):
        _vl_dim1 = 1
    else:
        _vl_dim1 = 0
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'),
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'))
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if (_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'L\', \'l\')',
            )
    if _job_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job not in (\'L\', \'l\')',
            )
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim1 = _n_py
    elif _job_py in ('L', 'l'):
        _vr_dim1 = 1
    else:
        _vr_dim1 = 0
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'),
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'))
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _t,
        _ldt,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if select is not None:
            _select_cls.refresh_py(_select)
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dtrsna(job, howmny, select, n, t, vl, vr, s, sep, mm):
    u"""
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of real upper quasi-triangular matrix.

    For full information please refer to the NAG Library document for
    f08ql

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qlf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

          Condition numbers for eigenvalues only are computed.

        `job` = 'V'

          Condition numbers for eigenvectors only are computed.

        `job` = 'B'

          Condition numbers for both eigenvalues and eigenvectors are
          computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

          Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

          Condition numbers for selected eigenpairs (as specified by
          `select`) are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the real eigenvalue lambda_j,
        `select`[j-1] must be set True. To select condition numbers
        corresponding to a complex conjugate pair of eigenvalues
        lambda_j and lambda_{j+1}, `select`[j-1] and/or `select`[j] must
        be set to True.

        If `howmny` = 'A', `select` is not referenced.

    n : int
        n, the order of the matrix T.

    t : float, shape (`n`, `n`)
        The n by n upper quasi-triangular matrix T in canonical Schur
        form, as returned by ``dhseqr``.

    vl : None or float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; if
        `job`='V': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain the left eigenvectors
        of T (or of any matrix QTQ^T with Q orthogonal) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors must be stored in consecutive columns of `vl`, as
        returned by ``dtrevc`` or ``dhsein``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; if
        `job`='V': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain the right eigenvectors
        of T (or of any matrix QTQ^T with Q orthogonal) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors must be stored in consecutive columns of `vr`, as
        returned by ``dtrevc`` or ``dhsein``.

        If `job` = 'V', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('E', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: the reciprocal condition numbers
        of the selected eigenvalues if `job` = 'E' or 'B', stored in
        consecutive elements of the array. Thus `s`[j-1], `sep`[j-1] and
        the jth columns of `vl` and `vr` all correspond to the same
        eigenpair (but not in general the jth eigenpair unless all
        eigenpairs have been selected). For a complex conjugate pair of
        eigenvalues, two consecutive elements of `s` are set to the same
        value.

        If `job` = 'V', `s` is not referenced.

    sep : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('V', 'B'): `mm`; otherwise: 1.

        On exit, if not None on entry: the estimated reciprocal
        condition numbers of the selected right eigenvectors if `job` =
        'V' or 'B', stored in consecutive elements of the array. For a
        complex eigenvector, two consecutive elements of `sep` are set
        to the same value. If the eigenvalues cannot be reordered to
        compute `sep`[j], `sep`[j] is set to zero; this can only occur
        when the true value would be very small anyway.

        If `job` = 'E', `sep` is not referenced.

    mm : int
        The number of elements in the arrays `s` and `sep`, and the
        number of columns in the arrays `vl` and `vr` (if used). The
        precise number required, m, is n if `howmny` = 'A'; if `howmny`
        = 'S', m is obtained by counting 1 for each selected real
        eigenvalue, and 2 for each selected complex conjugate pair of
        eigenvalues (see `select`), in which case 0 <= m <= n.

    Returns
    -------
    m : int
        m, the number of elements of `s` and/or `sep` actually used to
        store the estimated condition numbers. If `howmny` = 'A', `m` is
        set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -13)
            On entry, error in parameter `mm`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtrsna'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineFloat64ArrayType.c_type, # Input, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Input, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, sep[*].
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, ldwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (vl, 'vl'), (vr, 'vr')),
    )
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _vl_k_py = 0
    if (_howmny_py in ('S', 's')) and utils._EngineFloat64ArrayType.get_shape(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if utils._EngineFloat64ArrayType.get_element(t, {'fun_name': fun_name, 'entity': 't'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    if _howmny_py not in ('A', 'a'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a'))
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if not _mm_py >= _vl_m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-13,),
            entity='mm',
        ).manage()
    if _job_py not in ('V', 'v'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim1 = _n_py
    elif _job_py in ('V', 'v'):
        _vl_dim1 = 1
    else:
        _vl_dim1 = 0
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _job_py not in ('V', 'v'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim1 = _n_py
    elif _job_py in ('V', 'v'):
        _vr_dim1 = 1
    else:
        _vr_dim1 = 0
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    if _job_py not in ('V', 'v'):
        if s is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 's'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _job_py not in ('E', 'e'):
        if sep is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'sep'},
                'must not be None when job not in (\'E\', \'e\')',
            )
    if _job_py in ('V', 'v', 'B', 'b'):
        _sep_dim1 = _mm_py
    else:
        _sep_dim1 = 1
    exp_shape_sep = (_sep_dim1,)
    _sep_cls = utils._EngineFloat64ArrayType(
        sep, {'fun_name': fun_name, 'entity': 'sep'},
        exp_shape=exp_shape_sep,
        check_contents=False,
        allow_none=True,
    )
    _sep = _sep_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    if _job_py in ('V', 'v', 'B', 'b'):
        _ldwork = utils._EngineIntScalarType(
            max(1, _n_py), {'fun_name': fun_name, 'entity': '_ldwork'},
        ).to_ctypes()
    elif _job_py in ('E', 'e'):
        _ldwork = utils._EngineIntScalarType.c_type(1)
    else:
        _ldwork = utils._EngineIntScalarType.c_type(0) # Added by code generator.
        _ldwork_py = 0 # Added by code generator.
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _t,
        _ldt,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _sep,
        _mm,
        _m,
        _work,
        _ldwork,
        _iwork,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if s is not None:
            _s_cls.refresh_py(_s)
        if sep is not None:
            _sep_cls.refresh_py(_sep)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def ztrexc(compq, n, t, q, ifst, ilst):
    u"""
    Reorders a Schur factorization of complex matrix using unitary
    similarity transformation.

    For full information please refer to the NAG Library document for
    f08qt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qtf.html

    Parameters
    ----------
    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

          The matrix Q of Schur vectors is updated.

        `compq` = 'N'

          No Schur vectors are updated.

    n : int
        n, the order of the matrix T.

    t : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix T, as returned by
        ``zhseqr``.

        On exit: `t` is overwritten by the updated matrix T~.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain the n by n unitary
        matrix Q of Schur vectors.

        On exit: if `compq` = 'V', `q` contains the updated matrix of
        Schur vectors.

        If `compq` = 'N', `q` is not referenced.

    ifst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements of T. The element with row index `ifst` is moved to row
        `ilst` by a sequence of exchanges between adjacent elements.

    ilst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements of T. The element with row index `ifst` is moved to row
        `ilst` by a sequence of exchanges between adjacent elements.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `ifst`.

            Constraint: 1 <= `ifst` <= `n`.

        (`errno` -8)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ilst` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztrexc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_INT, # Input, ifst.
        utils._BYREF_INT, # Input, ilst.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ifst = utils._EngineIntScalarType(
        ifst, {'fun_name': fun_name, 'entity': 'ifst'},
    ).to_ctypes()
    _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
    if not (_ifst_py >= 1 and _ifst_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ifst and ifst <= n.'),
            ],
            errcodes=(-7,),
            entity='ifst',
        ).manage()
    _ilst = utils._EngineIntScalarType(
        ilst, {'fun_name': fun_name, 'entity': 'ilst'},
    ).to_ctypes()
    _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
    if not (_ilst_py >= 1 and _ilst_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ilst and ilst <= n.'),
            ],
            errcodes=(-8,),
            entity='ilst',
        ).manage()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _n,
        _t,
        _ldt,
        _q,
        _ldq,
        _ifst,
        _ilst,
        _info,
        _compq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_cls.refresh_py(_t)
        _q_cls.refresh_py(_q)
        return_data = None
    _err_class.manage()
    return return_data

def ztrsen(job, compq, select, n, t, q, w):
    u"""
    Reorders a Schur factorization of complex matrix, form orthonormal
    basis of right invariant subspace for selected eigenvalues, with
    estimates of sensitivities.

    For full information please refer to the NAG Library document for
    f08qu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08quf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for the cluster
        of eigenvalues and/or the invariant subspace.

        `job` = 'N'

          No condition numbers are required.

        `job` = 'E'

          Only the condition number for the cluster of eigenvalues is
          computed.

        `job` = 'V'

          Only the condition number for the invariant subspace is
          computed.

        `job` = 'B'

          Condition numbers for both the cluster of eigenvalues and the
          invariant subspace are computed.

    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

          The matrix Q of Schur vectors is updated.

        `compq` = 'N'

          No Schur vectors are updated.

    select : bool, shape (`n`)
        Specifies the eigenvalues in the selected cluster. To select a
        complex eigenvalue lambda_j, `select`[j-1] must be set True.

    n : int
        n, the order of the matrix T.

    t : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix T, as returned by
        ``zhseqr``.

        On exit: `t` is overwritten by the updated matrix T~.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': `n`; if `compq`='N': 1;
        otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain the n by n unitary
        matrix Q of Schur vectors, as returned by ``zhseqr``.

        On exit: if `compq` = 'V', `q` contains the updated matrix of
        Schur vectors; the first m columns of Q form an orthonormal
        basis for the specified invariant subspace.

        If `compq` = 'N', `q` is not referenced.

    w : complex, shape (`n`), modified in place
        On exit: the reordered eigenvalues of T~. The eigenvalues are
        stored in the same order as on the diagonal of T~.

    Returns
    -------
    m : int
        m, the dimension of the specified invariant subspace, which is
        the same as the number of selected eigenvalues (see `select`); 0
        <= m <= n.

    s : float
        If `job` = 'E' or 'B', `s` is a lower bound on the reciprocal
        condition number of the average of the selected cluster of
        eigenvalues. If `m` = 0 or `n`, `s` = 1.

        If `job` = 'N' or 'V', `s` is not referenced.

    sep : float
        If `job` = 'V' or 'B', `sep` is the estimated reciprocal
        condition number of the specified invariant subspace. If `m` = 0
        or `n`, `sep` = ||T||.

        If `job` = 'N' or 'E', `sep` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztrsen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08quft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._BYREF_INT, # Output, m.
        utils._BYREF_FLOAT64, # Output, s.
        utils._BYREF_FLOAT64, # Output, sep.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='compq',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _s = utils._EngineFloat64ScalarType.empty_ctypes()
    _sep = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compq,
        _select,
        _n,
        _t,
        _ldt,
        _q,
        _ldq,
        _w,
        _m,
        _s,
        _sep,
        _work,
        _lwork,
        _info,
        _job._length_,
        _compq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_cls.refresh_py(_t)
        _q_cls.refresh_py(_q)
        _w_cls.refresh_py(_w)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _s_py = utils._EngineFloat64ScalarType.to_py(_s)
        _sep_py = utils._EngineFloat64ScalarType.to_py(_sep)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 's', 'sep'],
        )(
            _m_py,
            _s_py,
            _sep_py,
        )
    _err_class.manage()
    return return_data

def ztrsyl(trana, tranb, isgn, m, n, a, b, c):
    u"""
    Solves the complex Sylvester matrix equation AX+XB = C, A and B are
    upper triangular or conjugate-transposes.

    For full information please refer to the NAG Library document for
    f08qv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qvf.html

    Parameters
    ----------
    trana : str, length 1
        Specifies the option op(A).

        `trana` = 'N'

          op(A) = A.

        `trana` = 'C'

          op(A) = A^H.

    tranb : str, length 1
        Specifies the option op(B).

        `tranb` = 'N'

          op(B) = B.

        `tranb` = 'C'

          op(B) = B^H.

    isgn : int
        Indicates the form of the Sylvester equation.

        `isgn` = +1

          The equation is of the form op(A)X+Xop(B) = alpha C.

        `isgn` = -1

          The equation is of the form op(A)X-Xop(B) = alpha C.

    m : int
        m, the order of the matrix A, and the number of rows in the
        matrices X and C.

    n : int
        n, the order of the matrix B, and the number of columns in the
        matrices X and C.

    a : complex, shape (`m`, `m`)
        The m by m upper triangular matrix A.

    b : complex, shape (`n`, `n`)
        The n by n upper triangular matrix B.

    c : complex, shape (`m`, `n`), modified in place
        On entry: the m by n right-hand side matrix C.

        On exit: `c` is overwritten by the solution matrix X.

    Returns
    -------
    scal : float
        The value of the scale factor alpha.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trana`.

            Constraint: `trana` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `tranb`.

            Constraint: `tranb` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `isgn`.

            Constraint: `isgn` = +1 or -1.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A and B have common or close eigenvalues, perturbed values
            of which were used to solve the equation.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztrsyl'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trana(len=1).
        utils._BYREF_STRING, # Input, tranb(len=1).
        utils._BYREF_INT, # Input, isgn.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._BYREF_FLOAT64, # Output, scal.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trana.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tranb.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trana = utils._EngineCharScalarType(
        trana, {'fun_name': fun_name, 'entity': 'trana'}, exp_chrlen=1,
    ).to_ctypes()
    _trana_py = utils._EngineCharScalarType.to_py(_trana)
    if _trana_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trana in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trana',
        ).manage()
    _tranb = utils._EngineCharScalarType(
        tranb, {'fun_name': fun_name, 'entity': 'tranb'}, exp_chrlen=1,
    ).to_ctypes()
    _tranb_py = utils._EngineCharScalarType.to_py(_tranb)
    if _tranb_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('tranb in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-2,),
            entity='tranb',
        ).manage()
    _isgn = utils._EngineIntScalarType(
        isgn, {'fun_name': fun_name, 'entity': 'isgn'},
    ).to_ctypes()
    _isgn_py = utils._EngineIntScalarType.to_py(_isgn)
    if _isgn_py not in (+1, -1):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('isgn in (+1, -1).'),
            ],
            errcodes=(-3,),
            entity='isgn',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _scal = utils._EngineFloat64ScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trana,
        _tranb,
        _isgn,
        _m,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _scal,
        _info,
        _trana._length_,
        _tranb._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A and B have common or close eigenvalues, perturbed values of which were',
                    'used to solve the equation.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _c_cls.refresh_py(_c)
        _scal_py = utils._EngineFloat64ScalarType.to_py(_scal)
        return_data = _scal_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['scal'],
            )(
                _scal_py,
            )
    _err_class.manage()
    return return_data

def ztrevc(job, howmny, select, n, t, vl, vr, mm):
    u"""
    Computes left and right eigenvectors of complex upper triangular
    matrix.

    For full information please refer to the NAG Library document for
    f08qx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qxf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

          Only right eigenvectors are computed.

        `job` = 'L'

          Only left eigenvectors are computed.

        `job` = 'B'

          Both left and right eigenvectors are computed.

    howmny : str, length 1
        Indicates how many eigenvectors are to be computed.

        `howmny` = 'A'

          All eigenvectors (as specified by `job`) are computed.

        `howmny` = 'B'

          All eigenvectors (as specified by `job`) are computed and then
          pre-multiplied by the matrix Q (which is overwritten).

        `howmny` = 'S'

          Selected eigenvectors (as specified by `job` and `select`) are
          computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies which eigenvectors are to be computed if `howmny` =
        'S'. To obtain the eigenvector corresponding to the eigenvalue
        lambda_j, `select`[j-1] must be set True.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    n : int
        n, the order of the matrix T.

    t : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix T, as returned by
        ``zhseqr``.

        On exit: is used as internal workspace prior to being restored
        and hence is unchanged.

    vl : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): `n`; if
        `job`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): `mm`; otherwise:
        0.

        Optionally, on entry: if `howmny` = 'B' and `job` = 'L' or 'B',
        `vl` must contain an n by n matrix Q (usually the matrix of
        Schur vectors returned by ``zhseqr``).

        If `howmny` = 'A' or 'S', `vl` need not be set.

        On exit, if not None on entry: if `job` = 'L' or 'B', `vl`
        contains the computed left eigenvectors (as specified by
        `howmny` and `select`). The eigenvectors are stored
        consecutively in the columns of the array, in the same order as
        their eigenvalues.

        If `job` = 'R', `vl` is not referenced.

    vr : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): `n`; if
        `job`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): `mm`; otherwise:
        0.

        Optionally, on entry: if `howmny` = 'B' and `job` = 'R' or 'B',
        `vr` must contain an n by n matrix Q (usually the matrix of
        Schur vectors returned by ``zhseqr``).

        If `howmny` = 'A' or 'S', `vr` need not be set.

        On exit, if not None on entry: if `job` = 'R' or 'B', `vr`
        contains the computed right eigenvectors (as specified by
        `howmny` and `select`). The eigenvectors are stored
        consecutively in the columns of the array, in the same order as
        their eigenvalues.

        If `job` = 'L', `vr` is not referenced.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`. The
        precise number of columns required, m, is n if `howmny` = 'A' or
        'B'; if `howmny` = 'S', m is the number of selected eigenvectors
        (see `select`), in which case 0 <= m <= n.

    Returns
    -------
    m : int
        m, the number of selected eigenvectors. If `howmny` = 'A' or
        'B', `m` is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B', 'O' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -11)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= `n`.

        (`errno` -11)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztrevc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (vl, 'vl'), (vr, 'vr')),
    )
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'B', 'b', 'O', 'o', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'B\', \'b\', \'O\', \'o\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    if _howmny_py in ('S', 's'):
        _vl_m_py = 0
        for _vl_j_py in range(1, _n_py+1):
            if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
                _vl_m_py = _vl_m_py + 1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('R', 'r', 'L', 'l', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'R\', \'r\', \'L\', \'l\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    if _howmny_py in ('S', 's'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny in (\'S\', \'s\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py in ('S', 's'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py in ('S', 's'))
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t_cls = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _t = _t_cls.to_ctypes()
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if _howmny_py in ('A', 'a', 'B', 'b'):
        if not _mm_py >= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= n.'),
                ],
                errcodes=(-11,),
                entity='mm',
            ).manage()
    else:
        if not _mm_py >= _vl_m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= m.'),
                ],
                errcodes=(-11,),
                entity='mm',
            ).manage()
    if (_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'R\', \'r\')',
            )
    if _job_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job not in (\'R\', \'r\')',
            )
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim1 = _n_py
    elif _job_py in ('R', 'r'):
        _vl_dim1 = 1
    else:
        _vl_dim1 = 0
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'),
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'))
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if (_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'L\', \'l\')',
            )
    if _job_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job not in (\'L\', \'l\')',
            )
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim1 = _n_py
    elif _job_py in ('L', 'l'):
        _vr_dim1 = 1
    else:
        _vr_dim1 = 0
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'),
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'))
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _t,
        _ldt,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _rwork,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_cls.refresh_py(_t)
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def ztrsna(job, howmny, select, n, t, vl, vr, s, sep, mm):
    u"""
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of complex upper triangular matrix.

    For full information please refer to the NAG Library document for
    f08qy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08qyf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

          Condition numbers for eigenvalues only are computed.

        `job` = 'V'

          Condition numbers for eigenvectors only are computed.

        `job` = 'B'

          Condition numbers for both eigenvalues and eigenvectors are
          computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

          Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

          Condition numbers for selected eigenpairs (as specified by
          `select`) are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the eigenvalue lambda_j,
        `select`[j-1] must be set to True.

        If `howmny` = 'A', `select` is not referenced.

    n : int
        n, the order of the matrix T.

    t : complex, shape (`n`, `n`)
        The n by n upper triangular matrix T, as returned by ``zhseqr``.

    vl : None or complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; if
        `job`='V': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain the left eigenvectors
        of T (or of any matrix QTQ^H with Q unitary) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors must be stored in consecutive columns of `vl`, as
        returned by ``ztrevc`` or ``zhsein``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; if
        `job`='V': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain the right eigenvectors
        of T (or of any matrix QTQ^H with Q unitary) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors must be stored in consecutive columns of `vr`, as
        returned by ``ztrevc`` or ``zhsein``.

        If `job` = 'V', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('E', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: the reciprocal condition numbers
        of the selected eigenvalues if `job` = 'E' or 'B', stored in
        consecutive elements of the array. Thus `s`[j-1], `sep`[j-1] and
        the jth columns of `vl` and `vr` all correspond to the same
        eigenpair (but not in general the jth eigenpair unless all
        eigenpairs have been selected).

        If `job` = 'V', `s` is not referenced.

    sep : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('V', 'B'): `mm`; otherwise: 1.

        On exit, if not None on entry: the estimated reciprocal
        condition numbers of the selected right eigenvectors if `job` =
        'V' or 'B', stored in consecutive elements of the array.

        If `job` = 'E', `sep` is not referenced.

    mm : int
        The number of elements in the arrays `s` and `sep`, and the
        number of columns in the arrays `vl` and `vr` (if used). The
        precise number required, m, is n if `howmny` = 'A'; if `howmny`
        = 'S', m is the number of selected eigenpairs (see `select`), in
        which case 0 <= m <= n.

    Returns
    -------
    m : int
        m, the number of selected eigenpairs. If `howmny` = 'A', `m` is
        set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -13)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= `n`.

        (`errno` -13)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztrsna'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08qyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, t[ldt, n].
        utils._BYREF_INT, # Input, ldt.
        utils._EngineComplex128ArrayType.c_type, # Input, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Input, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, sep[*].
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, ldwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((t, 't'), (vl, 'vl'), (vr, 'vr')),
    )
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    if _howmny_py in ('S', 's'):
        _vl_m_py = 0
        for _vl_j_py in range(1, _n_py+1):
            if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
                _vl_m_py = _vl_m_py + 1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    if _howmny_py not in ('A', 'a'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a'))
    _ldt = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldt'},
    ).to_ctypes()
    _t = utils._EngineComplex128ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if _howmny_py in ('A', 'a'):
        if not _mm_py >= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= n.'),
                ],
                errcodes=(-13,),
                entity='mm',
            ).manage()
    else:
        if not _mm_py >= _vl_m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= m.'),
                ],
                errcodes=(-13,),
                entity='mm',
            ).manage()
    if _job_py not in ('V', 'v'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim1 = _n_py
    elif _job_py in ('V', 'v'):
        _vl_dim1 = 1
    else:
        _vl_dim1 = 0
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _job_py not in ('V', 'v'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim1 = _n_py
    elif _job_py in ('V', 'v'):
        _vr_dim1 = 1
    else:
        _vr_dim1 = 0
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    if _job_py not in ('V', 'v'):
        if s is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 's'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _job_py not in ('E', 'e'):
        if sep is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'sep'},
                'must not be None when job not in (\'E\', \'e\')',
            )
    if _job_py in ('V', 'v', 'B', 'b'):
        _sep_dim1 = _mm_py
    else:
        _sep_dim1 = 1
    exp_shape_sep = (_sep_dim1,)
    _sep_cls = utils._EngineFloat64ArrayType(
        sep, {'fun_name': fun_name, 'entity': 'sep'},
        exp_shape=exp_shape_sep,
        check_contents=False,
        allow_none=True,
    )
    _sep = _sep_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    if _job_py in ('V', 'v', 'B', 'b'):
        _ldwork = utils._EngineIntScalarType(
            max(1, _n_py), {'fun_name': fun_name, 'entity': '_ldwork'},
        ).to_ctypes()
    elif _job_py in ('E', 'e'):
        _ldwork = utils._EngineIntScalarType.c_type(1)
    else:
        _ldwork = utils._EngineIntScalarType.c_type(0) # Added by code generator.
        _ldwork_py = 0 # Added by code generator.
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _t,
        _ldt,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _sep,
        _mm,
        _m,
        _work,
        _ldwork,
        _rwork,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if s is not None:
            _s_cls.refresh_py(_s)
        if sep is not None:
            _sep_cls.refresh_py(_sep)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dorcsd(jobu1, jobu2, jobv1t, jobv2t, trans, signs, m, p, q, x11, x12, x21, x22, theta, u1, u2, v1t, v2t):
    u"""
    Computes the CS decomposition of an orthogonal matrix partitioned
    into four real submatrices.

    For full information please refer to the NAG Library document for
    f08ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08raf.html

    Parameters
    ----------
    jobu1 : str, length 1
        - If `jobu1` = 'Y', U_1 is computed;

        - otherwise, U_1 is not computed.

    jobu2 : str, length 1
        - If `jobu2` = 'Y', U_2 is computed;

        - otherwise, U_2 is not computed.

    jobv1t : str, length 1
        - If `jobv1t` = 'Y', V_1^T is computed;

        - otherwise, V_1^T is not computed.

    jobv2t : str, length 1
        - If `jobv2t` = 'Y', V_2^T is computed;

        - otherwise, V_2^T is not computed.

    trans : str, length 1
        - If `trans` = 'T', X, U_1, U_2, V_1^T and V_2^T are stored in
          row-major order;

        - otherwise, X, U_1, U_2, V_1^T and V_2^T are stored in
          column-major order.

    signs : str, length 1
        - If `signs` = 'O', the lower-left block is made nonpositive
          (the other convention);

        - otherwise, the upper-right block is made nonpositive (the
          default convention).

    m : int
        m, the number of rows and columns in the orthogonal matrix X.

    p : int
        p, the number of rows in X_{11} and X_{12}.

    q : int
        q, the number of columns in X_{11} and X_{21}.

    x11 : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `q`.

        On entry: the upper left partition of the orthogonal matrix X
        whose CSD is desired.

        On exit: contains details of the orthogonal matrix used in a
        simultaneous bidiagonalization process.

    x12 : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `m`-`q`.

        On entry: the upper right partition of the orthogonal matrix X
        whose CSD is desired.

        On exit: contains details of the orthogonal matrix used in a
        simultaneous bidiagonalization process.

    x21 : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise: `q`.

        On entry: the lower left partition of the orthogonal matrix X
        whose CSD is desired.

        On exit: contains details of the orthogonal matrix used in a
        simultaneous bidiagonalization process.

    x22 : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise:
        `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise:
        `m`-`q`.

        On entry: the lower right partition of the orthogonal matrix X
        CSD is desired.

        On exit: contains details of the orthogonal matrix used in a
        simultaneous bidiagonalization process.

    theta : float, shape (min(`p`, `m`-`p`, `q`, `m`-`q`)), modified in
    place
        On exit: the values theta_i for i = 1,2,...,r where r = min(p,
        m-p, q, m-q). The diagonal submatrices C and S of Sigma_p are
        constructed from these values as

        - C = diag(cos(`theta`[0]), ..., cos(`theta`[r-1])) and

        - S = diag(sin(`theta`[0]), ..., sin(`theta`[r-1])).

    u1 : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu1`='Y': `p`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu1`='Y': `p`; otherwise: 1.

        On exit, if not None on entry: if `jobu1` = 'Y', `u1` contains
        the p by p orthogonal matrix U_1.

    u2 : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu2`='Y': `m`-`p`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu2`='Y': `m`-`p`; otherwise: 1.

        On exit, if not None on entry: if `jobu2` = 'Y', `u2` contains
        the m-p by m-p orthogonal matrix U_2.

    v1t : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv1t`='Y': `q`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv1t`='Y': `q`; otherwise: 1.

        On exit, if not None on entry: if `jobv1t` = 'Y', `v1t` contains
        the q by q orthogonal matrix V_1^T.

    v2t : None or float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv2t`='Y': `m`-`q`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv2t`='Y': `m`-`q`; otherwise: 1.

        On exit, if not None on entry: if `jobv2t` = 'Y', `v2t` contains
        the m-q by m-q orthogonal matrix V_2^T.

    Raises
    ------
    NagValueError
        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `m`.

        (`errno` -9)
            On entry, error in parameter `q`.

            Constraint: 0 <= `q` <= `m`.

        (`errno` i > 0)
            The Jacobi-type procedure failed to converge during an
            internal reduction to bidiagonal-block form. The process
            requires convergence to min(`p`, `m`-`p`, `q`, `m`-`q`)
            values, the value of errno gives the number of converged
            values.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dorcsd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobu1(len=1).
        utils._BYREF_STRING, # Input, jobu2(len=1).
        utils._BYREF_STRING, # Input, jobv1t(len=1).
        utils._BYREF_STRING, # Input, jobv2t(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, signs(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x11[ldx11, *].
        utils._BYREF_INT, # Input, ldx11.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x12[ldx12, *].
        utils._BYREF_INT, # Input, ldx12.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x21[ldx21, *].
        utils._BYREF_INT, # Input, ldx21.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x22[ldx22, *].
        utils._BYREF_INT, # Input, ldx22.
        utils._EngineFloat64ArrayType.c_type, # Output, theta[min(p, m-p, q, m-q)].
        utils._EngineFloat64ArrayType.c_type, # Output, u1[ldu1, *].
        utils._BYREF_INT, # Input, ldu1.
        utils._EngineFloat64ArrayType.c_type, # Output, u2[ldu2, *].
        utils._BYREF_INT, # Input, ldu2.
        utils._EngineFloat64ArrayType.c_type, # Output, v1t[ldv1t, *].
        utils._BYREF_INT, # Input, ldv1t.
        utils._EngineFloat64ArrayType.c_type, # Output, v2t[ldv2t, *].
        utils._BYREF_INT, # Input, ldv2t.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu1.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu2.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv1t.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv2t.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of signs.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x11, 'x11'), (x12, 'x12'), (x21, 'x21'), (x22, 'x22'), (u1, 'u1'), (u2, 'u2'), (v1t, 'v1t'), (v2t, 'v2t')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineState._literal_true_cint
    _jobu1 = utils._EngineCharScalarType(
        jobu1, {'fun_name': fun_name, 'entity': 'jobu1'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu2 = utils._EngineCharScalarType(
        jobu2, {'fun_name': fun_name, 'entity': 'jobu2'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv1t = utils._EngineCharScalarType(
        jobv1t, {'fun_name': fun_name, 'entity': 'jobv1t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv2t = utils._EngineCharScalarType(
        jobv2t, {'fun_name': fun_name, 'entity': 'jobv2t'}, exp_chrlen=1,
    ).to_ctypes()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _signs = utils._EngineCharScalarType(
        signs, {'fun_name': fun_name, 'entity': 'signs'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not (_p_py >= 0 and _p_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= p and p <= m.'),
            ],
            errcodes=(-8,),
            entity='p',
        ).manage()
    _q = utils._EngineIntScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _q_py = utils._EngineIntScalarType.to_py(_q)
    if not (_q_py >= 0 and _q_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= q and q <= m.'),
            ],
            errcodes=(-9,),
            entity='q',
        ).manage()
    _ldx11 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x11, {'fun_name': fun_name, 'entity': 'x11'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx11'},
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py in ('T', 't'):
        _x11_dim1 = _q_py
    else:
        _x11_dim1 = _p_py
    if _trans_py in ('T', 't'):
        _x11_dim2 = _p_py
    else:
        _x11_dim2 = _q_py
    exp_shape_x11 = (_x11_dim1, _x11_dim2)
    _x11_cls = utils._EngineFloat64ArrayType(
        x11, {'fun_name': fun_name, 'entity': 'x11'},
        exp_shape=exp_shape_x11,
        mutual_sorder=_m_sorder,
    )
    _x11 = _x11_cls.to_ctypes()
    _ldx12 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x12, {'fun_name': fun_name, 'entity': 'x12'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx12'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x12_dim1 = _m_py-_q_py
    else:
        _x12_dim1 = _p_py
    if _trans_py in ('T', 't'):
        _x12_dim2 = _p_py
    else:
        _x12_dim2 = _m_py-_q_py
    exp_shape_x12 = (_x12_dim1, _x12_dim2)
    _x12_cls = utils._EngineFloat64ArrayType(
        x12, {'fun_name': fun_name, 'entity': 'x12'},
        exp_shape=exp_shape_x12,
        mutual_sorder=_m_sorder,
    )
    _x12 = _x12_cls.to_ctypes()
    _ldx21 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x21, {'fun_name': fun_name, 'entity': 'x21'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx21'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x21_dim1 = _q_py
    else:
        _x21_dim1 = _m_py-_p_py
    if _trans_py in ('T', 't'):
        _x21_dim2 = _m_py-_p_py
    else:
        _x21_dim2 = _q_py
    exp_shape_x21 = (_x21_dim1, _x21_dim2)
    _x21_cls = utils._EngineFloat64ArrayType(
        x21, {'fun_name': fun_name, 'entity': 'x21'},
        exp_shape=exp_shape_x21,
        mutual_sorder=_m_sorder,
    )
    _x21 = _x21_cls.to_ctypes()
    _ldx22 = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(x22, {'fun_name': fun_name, 'entity': 'x22'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx22'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x22_dim1 = _m_py-_q_py
    else:
        _x22_dim1 = _m_py-_p_py
    if _trans_py in ('T', 't'):
        _x22_dim2 = _m_py-_p_py
    else:
        _x22_dim2 = _m_py-_q_py
    exp_shape_x22 = (_x22_dim1, _x22_dim2)
    _x22_cls = utils._EngineFloat64ArrayType(
        x22, {'fun_name': fun_name, 'entity': 'x22'},
        exp_shape=exp_shape_x22,
        mutual_sorder=_m_sorder,
    )
    _x22 = _x22_cls.to_ctypes()
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(min(_p_py, _m_py-_p_py, _q_py, _m_py-_q_py),),
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _ldu1_py = utils._EngineFloat64ArrayType.get_nd(u1, {'fun_name': fun_name, 'entity': 'u1'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu1 = utils._EngineIntScalarType(
        _ldu1_py, {'fun_name': fun_name, 'entity': '_ldu1'},
    ).to_ctypes()
    _jobu1_py = utils._EngineCharScalarType.to_py(_jobu1)
    if _jobu1_py not in ('N', 'n'):
        if u1 is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u1'},
                'must not be None when jobu1 not in (\'N\', \'n\')',
            )
    if _jobu1_py in ('Y', 'y'):
        _u1_dim1 = _p_py
    else:
        _u1_dim1 = 0
    if _jobu1_py in ('Y', 'y'):
        _u1_dim2 = _p_py
    else:
        _u1_dim2 = 1
    exp_shape_u1 = (_u1_dim1, _u1_dim2)
    _u1_cls = utils._EngineFloat64ArrayType(
        u1, {'fun_name': fun_name, 'entity': 'u1'},
        exp_shape=exp_shape_u1,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u1 = _u1_cls.to_ctypes(can_reference_contents=False)
    _ldu2_py = utils._EngineFloat64ArrayType.get_nd(u2, {'fun_name': fun_name, 'entity': 'u2'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu2 = utils._EngineIntScalarType(
        _ldu2_py, {'fun_name': fun_name, 'entity': '_ldu2'},
    ).to_ctypes()
    _jobu2_py = utils._EngineCharScalarType.to_py(_jobu2)
    if _jobu2_py not in ('N', 'n'):
        if u2 is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u2'},
                'must not be None when jobu2 not in (\'N\', \'n\')',
            )
    if _jobu2_py in ('Y', 'y'):
        _u2_dim1 = _m_py-_p_py
    else:
        _u2_dim1 = 0
    if _jobu2_py in ('Y', 'y'):
        _u2_dim2 = _m_py-_p_py
    else:
        _u2_dim2 = 1
    exp_shape_u2 = (_u2_dim1, _u2_dim2)
    _u2_cls = utils._EngineFloat64ArrayType(
        u2, {'fun_name': fun_name, 'entity': 'u2'},
        exp_shape=exp_shape_u2,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u2 = _u2_cls.to_ctypes(can_reference_contents=False)
    _ldv1t_py = utils._EngineFloat64ArrayType.get_nd(v1t, {'fun_name': fun_name, 'entity': 'v1t'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldv1t = utils._EngineIntScalarType(
        _ldv1t_py, {'fun_name': fun_name, 'entity': '_ldv1t'},
    ).to_ctypes()
    _jobv1t_py = utils._EngineCharScalarType.to_py(_jobv1t)
    if _jobv1t_py not in ('N', 'n'):
        if v1t is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'v1t'},
                'must not be None when jobv1t not in (\'N\', \'n\')',
            )
    if _jobv1t_py in ('Y', 'y'):
        _v1t_dim1 = _q_py
    else:
        _v1t_dim1 = 0
    if _jobv1t_py in ('Y', 'y'):
        _v1t_dim2 = _q_py
    else:
        _v1t_dim2 = 1
    exp_shape_v1t = (_v1t_dim1, _v1t_dim2)
    _v1t_cls = utils._EngineFloat64ArrayType(
        v1t, {'fun_name': fun_name, 'entity': 'v1t'},
        exp_shape=exp_shape_v1t,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _v1t = _v1t_cls.to_ctypes(can_reference_contents=False)
    _ldv2t_py = utils._EngineFloat64ArrayType.get_nd(v2t, {'fun_name': fun_name, 'entity': 'v2t'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldv2t = utils._EngineIntScalarType(
        _ldv2t_py, {'fun_name': fun_name, 'entity': '_ldv2t'},
    ).to_ctypes()
    _jobv2t_py = utils._EngineCharScalarType.to_py(_jobv2t)
    if _jobv2t_py not in ('N', 'n'):
        if v2t is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'v2t'},
                'must not be None when jobv2t not in (\'N\', \'n\')',
            )
    if _jobv2t_py in ('Y', 'y'):
        _v2t_dim1 = _m_py-_q_py
    else:
        _v2t_dim1 = 0
    if _jobv2t_py in ('Y', 'y'):
        _v2t_dim2 = _m_py-_q_py
    else:
        _v2t_dim2 = 1
    exp_shape_v2t = (_v2t_dim1, _v2t_dim2)
    _v2t_cls = utils._EngineFloat64ArrayType(
        v2t, {'fun_name': fun_name, 'entity': 'v2t'},
        exp_shape=exp_shape_v2t,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _v2t = _v2t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobu1,
        _jobu2,
        _jobv1t,
        _jobv2t,
        _trans,
        _signs,
        _m,
        _p,
        _q,
        _x11,
        _ldx11,
        _x12,
        _ldx12,
        _x21,
        _ldx21,
        _x22,
        _ldx22,
        _theta,
        _u1,
        _ldu1,
        _u2,
        _ldu2,
        _v1t,
        _ldv1t,
        _v2t,
        _ldv2t,
        _work,
        _lwork,
        _iwork,
        _info,
        _jobu1._length_,
        _jobu2._length_,
        _jobv1t._length_,
        _jobv2t._length_,
        _trans._length_,
        _signs._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge during an internal reduction to',
                    'bidiagonal-block form. The process requires convergence to min(p, m-p, q, m-q)',
                    'values, the value of errno gives the number of converged values.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x11_cls.refresh_py(_x11)
        _x12_cls.refresh_py(_x12)
        _x21_cls.refresh_py(_x21)
        _x22_cls.refresh_py(_x22)
        _theta_cls.refresh_py(_theta)
        if u1 is not None:
            _u1_cls.refresh_py(_u1)
        if u2 is not None:
            _u2_cls.refresh_py(_u2)
        if v1t is not None:
            _v1t_cls.refresh_py(_v1t)
        if v2t is not None:
            _v2t_cls.refresh_py(_v2t)
        return_data = None
    _err_class.manage()
    return return_data

def zuncsd(jobu1, jobu2, jobv1t, jobv2t, trans, signs, m, p, q, x11, x12, x21, x22, theta, u1, u2, v1t, v2t):
    u"""
    Computes the CS decomposition of a unitary matrix partitioned into
    four complex submatrices.

    For full information please refer to the NAG Library document for
    f08rn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08rnf.html

    Parameters
    ----------
    jobu1 : str, length 1
        - If `jobu1` = 'Y', U_1 is computed;

        - otherwise, U_1 is not computed.

    jobu2 : str, length 1
        - If `jobu2` = 'Y', U_2 is computed;

        - otherwise, U_2 is not computed.

    jobv1t : str, length 1
        - If `jobv1t` = 'Y', V_1^T is computed;

        - otherwise, V_1^T is not computed.

    jobv2t : str, length 1
        - If `jobv2t` = 'Y', V_2^T is computed;

        - otherwise, V_2^T is not computed.

    trans : str, length 1
        - If `trans` = 'T', X, U_1, U_2, V_1^T and V_2^T are stored in
          row-major order;

        - otherwise, X, U_1, U_2, V_1^T and V_2^T are stored in
          column-major order.

    signs : str, length 1
        - If `signs` = 'O', the lower-left block is made nonpositive
          (the other convention);

        - otherwise, the upper-right block is made nonpositive (the
          default convention).

    m : int
        m, the number of rows and columns in the unitary matrix X.

    p : int
        p, the number of rows in X_{11} and X_{12}.

    q : int
        q, the number of columns in X_{11} and X_{21}.

    x11 : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `q`.

        On entry: the upper left partition of the unitary matrix X whose
        CSD is desired.

        On exit: contains details of the unitary matrix used in a
        simultaneous bidiagonalization process.

    x12 : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `m`-`q`.

        On entry: the upper right partition of the unitary matrix X
        whose CSD is desired.

        On exit: contains details of the unitary matrix used in a
        simultaneous bidiagonalization process.

    x21 : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise: `q`.

        On entry: the lower left partition of the unitary matrix X whose
        CSD is desired.

        On exit: contains details of the unitary matrix used in a
        simultaneous bidiagonalization process.

    x22 : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise:
        `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise:
        `m`-`q`.

        On entry: the lower right partition of the unitary matrix X CSD
        is desired.

        On exit: contains details of the unitary matrix used in a
        simultaneous bidiagonalization process.

    theta : float, shape (min(`p`, `m`-`p`, `q`, `m`-`q`)), modified in
    place
        On exit: the values theta_i for i = 1,2,...,r where r = min(p,
        m-p, q, m-q). The diagonal submatrices C and S of Sigma_p are
        constructed from these values as

        - C = diag(cos(`theta`[0]), ..., cos(`theta`[r-1])) and

        - S = diag(sin(`theta`[0]), ..., sin(`theta`[r-1])).

    u1 : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu1`='Y': `p`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu1`='Y': `p`; otherwise: 1.

        On exit, if not None on entry: if `jobu1` = 'Y', `u1` contains
        the p by p unitary matrix U_1.

    u2 : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu2`='Y': `m`-`p`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu2`='Y': `m`-`p`; otherwise: 1.

        On exit, if not None on entry: if `jobu2` = 'Y', `u2` contains
        the m-p by m-p unitary matrix U_2.

    v1t : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv1t`='Y': `q`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv1t`='Y': `q`; otherwise: 1.

        On exit, if not None on entry: if `jobv1t` = 'Y', `v1t` contains
        the q by q unitary matrix V_1^H.

    v2t : None or complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv2t`='Y': `m`-`q`; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv2t`='Y': `m`-`q`; otherwise: 1.

        On exit, if not None on entry: if `jobv2t` = 'Y', `v2t` contains
        the m-q by m-q unitary matrix V_2^H.

    Raises
    ------
    NagValueError
        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `m`.

        (`errno` -9)
            On entry, error in parameter `q`.

            Constraint: 0 <= `q` <= `m`.

        (`errno` i > 0)
            The Jacobi-type procedure failed to converge during an
            internal reduction to bidiagonal-block form. The process
            requires convergence to min(`p`, `m`-`p`, `q`, `m`-`q`)
            values, the value of errno gives the number of converged
            values.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zuncsd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08rnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobu1(len=1).
        utils._BYREF_STRING, # Input, jobu2(len=1).
        utils._BYREF_STRING, # Input, jobv1t(len=1).
        utils._BYREF_STRING, # Input, jobv2t(len=1).
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_STRING, # Input, signs(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, q.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x11[ldx11, *].
        utils._BYREF_INT, # Input, ldx11.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x12[ldx12, *].
        utils._BYREF_INT, # Input, ldx12.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x21[ldx21, *].
        utils._BYREF_INT, # Input, ldx21.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x22[ldx22, *].
        utils._BYREF_INT, # Input, ldx22.
        utils._EngineFloat64ArrayType.c_type, # Output, theta[min(p, m-p, q, m-q)].
        utils._EngineComplex128ArrayType.c_type, # Output, u1[ldu1, *].
        utils._BYREF_INT, # Input, ldu1.
        utils._EngineComplex128ArrayType.c_type, # Output, u2[ldu2, *].
        utils._BYREF_INT, # Input, ldu2.
        utils._EngineComplex128ArrayType.c_type, # Output, v1t[ldv1t, *].
        utils._BYREF_INT, # Input, ldv1t.
        utils._EngineComplex128ArrayType.c_type, # Output, v2t[ldv2t, *].
        utils._BYREF_INT, # Input, ldv2t.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu1.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu2.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv1t.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv2t.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of signs.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x11, 'x11'), (x12, 'x12'), (x21, 'x21'), (x22, 'x22'), (u1, 'u1'), (u2, 'u2'), (v1t, 'v1t'), (v2t, 'v2t')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineState._literal_true_cint
    _jobu1 = utils._EngineCharScalarType(
        jobu1, {'fun_name': fun_name, 'entity': 'jobu1'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu2 = utils._EngineCharScalarType(
        jobu2, {'fun_name': fun_name, 'entity': 'jobu2'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv1t = utils._EngineCharScalarType(
        jobv1t, {'fun_name': fun_name, 'entity': 'jobv1t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv2t = utils._EngineCharScalarType(
        jobv2t, {'fun_name': fun_name, 'entity': 'jobv2t'}, exp_chrlen=1,
    ).to_ctypes()
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _signs = utils._EngineCharScalarType(
        signs, {'fun_name': fun_name, 'entity': 'signs'}, exp_chrlen=1,
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-7,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not (_p_py >= 0 and _p_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= p and p <= m.'),
            ],
            errcodes=(-8,),
            entity='p',
        ).manage()
    _q = utils._EngineIntScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _q_py = utils._EngineIntScalarType.to_py(_q)
    if not (_q_py >= 0 and _q_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= q and q <= m.'),
            ],
            errcodes=(-9,),
            entity='q',
        ).manage()
    _ldx11 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x11, {'fun_name': fun_name, 'entity': 'x11'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx11'},
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py in ('T', 't'):
        _x11_dim1 = _q_py
    else:
        _x11_dim1 = _p_py
    if _trans_py in ('T', 't'):
        _x11_dim2 = _p_py
    else:
        _x11_dim2 = _q_py
    exp_shape_x11 = (_x11_dim1, _x11_dim2)
    _x11_cls = utils._EngineComplex128ArrayType(
        x11, {'fun_name': fun_name, 'entity': 'x11'},
        exp_shape=exp_shape_x11,
        mutual_sorder=_m_sorder,
    )
    _x11 = _x11_cls.to_ctypes()
    _ldx12 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x12, {'fun_name': fun_name, 'entity': 'x12'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx12'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x12_dim1 = _m_py-_q_py
    else:
        _x12_dim1 = _p_py
    if _trans_py in ('T', 't'):
        _x12_dim2 = _p_py
    else:
        _x12_dim2 = _m_py-_q_py
    exp_shape_x12 = (_x12_dim1, _x12_dim2)
    _x12_cls = utils._EngineComplex128ArrayType(
        x12, {'fun_name': fun_name, 'entity': 'x12'},
        exp_shape=exp_shape_x12,
        mutual_sorder=_m_sorder,
    )
    _x12 = _x12_cls.to_ctypes()
    _ldx21 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x21, {'fun_name': fun_name, 'entity': 'x21'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx21'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x21_dim1 = _q_py
    else:
        _x21_dim1 = _m_py-_p_py
    if _trans_py in ('T', 't'):
        _x21_dim2 = _m_py-_p_py
    else:
        _x21_dim2 = _q_py
    exp_shape_x21 = (_x21_dim1, _x21_dim2)
    _x21_cls = utils._EngineComplex128ArrayType(
        x21, {'fun_name': fun_name, 'entity': 'x21'},
        exp_shape=exp_shape_x21,
        mutual_sorder=_m_sorder,
    )
    _x21 = _x21_cls.to_ctypes()
    _ldx22 = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(x22, {'fun_name': fun_name, 'entity': 'x22'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldx22'},
    ).to_ctypes()
    if _trans_py in ('T', 't'):
        _x22_dim1 = _m_py-_q_py
    else:
        _x22_dim1 = _m_py-_p_py
    if _trans_py in ('T', 't'):
        _x22_dim2 = _m_py-_p_py
    else:
        _x22_dim2 = _m_py-_q_py
    exp_shape_x22 = (_x22_dim1, _x22_dim2)
    _x22_cls = utils._EngineComplex128ArrayType(
        x22, {'fun_name': fun_name, 'entity': 'x22'},
        exp_shape=exp_shape_x22,
        mutual_sorder=_m_sorder,
    )
    _x22 = _x22_cls.to_ctypes()
    _theta_cls = utils._EngineFloat64ArrayType(
        theta, {'fun_name': fun_name, 'entity': 'theta'},
        exp_shape=(min(_p_py, _m_py-_p_py, _q_py, _m_py-_q_py),),
        check_contents=False,
    )
    _theta = _theta_cls.to_ctypes(can_reference_contents=False)
    _ldu1_py = utils._EngineComplex128ArrayType.get_nd(u1, {'fun_name': fun_name, 'entity': 'u1'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu1 = utils._EngineIntScalarType(
        _ldu1_py, {'fun_name': fun_name, 'entity': '_ldu1'},
    ).to_ctypes()
    _jobu1_py = utils._EngineCharScalarType.to_py(_jobu1)
    if _jobu1_py not in ('N', 'n'):
        if u1 is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u1'},
                'must not be None when jobu1 not in (\'N\', \'n\')',
            )
    if _jobu1_py in ('Y', 'y'):
        _u1_dim1 = _p_py
    else:
        _u1_dim1 = 0
    if _jobu1_py in ('Y', 'y'):
        _u1_dim2 = _p_py
    else:
        _u1_dim2 = 1
    exp_shape_u1 = (_u1_dim1, _u1_dim2)
    _u1_cls = utils._EngineComplex128ArrayType(
        u1, {'fun_name': fun_name, 'entity': 'u1'},
        exp_shape=exp_shape_u1,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u1 = _u1_cls.to_ctypes(can_reference_contents=False)
    _ldu2_py = utils._EngineComplex128ArrayType.get_nd(u2, {'fun_name': fun_name, 'entity': 'u2'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldu2 = utils._EngineIntScalarType(
        _ldu2_py, {'fun_name': fun_name, 'entity': '_ldu2'},
    ).to_ctypes()
    _jobu2_py = utils._EngineCharScalarType.to_py(_jobu2)
    if _jobu2_py not in ('N', 'n'):
        if u2 is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'u2'},
                'must not be None when jobu2 not in (\'N\', \'n\')',
            )
    if _jobu2_py in ('Y', 'y'):
        _u2_dim1 = _m_py-_p_py
    else:
        _u2_dim1 = 0
    if _jobu2_py in ('Y', 'y'):
        _u2_dim2 = _m_py-_p_py
    else:
        _u2_dim2 = 1
    exp_shape_u2 = (_u2_dim1, _u2_dim2)
    _u2_cls = utils._EngineComplex128ArrayType(
        u2, {'fun_name': fun_name, 'entity': 'u2'},
        exp_shape=exp_shape_u2,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _u2 = _u2_cls.to_ctypes(can_reference_contents=False)
    _ldv1t_py = utils._EngineComplex128ArrayType.get_nd(v1t, {'fun_name': fun_name, 'entity': 'v1t'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldv1t = utils._EngineIntScalarType(
        _ldv1t_py, {'fun_name': fun_name, 'entity': '_ldv1t'},
    ).to_ctypes()
    _jobv1t_py = utils._EngineCharScalarType.to_py(_jobv1t)
    if _jobv1t_py not in ('N', 'n'):
        if v1t is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'v1t'},
                'must not be None when jobv1t not in (\'N\', \'n\')',
            )
    if _jobv1t_py in ('Y', 'y'):
        _v1t_dim1 = _q_py
    else:
        _v1t_dim1 = 0
    if _jobv1t_py in ('Y', 'y'):
        _v1t_dim2 = _q_py
    else:
        _v1t_dim2 = 1
    exp_shape_v1t = (_v1t_dim1, _v1t_dim2)
    _v1t_cls = utils._EngineComplex128ArrayType(
        v1t, {'fun_name': fun_name, 'entity': 'v1t'},
        exp_shape=exp_shape_v1t,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _v1t = _v1t_cls.to_ctypes(can_reference_contents=False)
    _ldv2t_py = utils._EngineComplex128ArrayType.get_nd(v2t, {'fun_name': fun_name, 'entity': 'v2t'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldv2t = utils._EngineIntScalarType(
        _ldv2t_py, {'fun_name': fun_name, 'entity': '_ldv2t'},
    ).to_ctypes()
    _jobv2t_py = utils._EngineCharScalarType.to_py(_jobv2t)
    if _jobv2t_py not in ('N', 'n'):
        if v2t is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'v2t'},
                'must not be None when jobv2t not in (\'N\', \'n\')',
            )
    if _jobv2t_py in ('Y', 'y'):
        _v2t_dim1 = _m_py-_q_py
    else:
        _v2t_dim1 = 0
    if _jobv2t_py in ('Y', 'y'):
        _v2t_dim2 = _m_py-_q_py
    else:
        _v2t_dim2 = 1
    exp_shape_v2t = (_v2t_dim1, _v2t_dim2)
    _v2t_cls = utils._EngineComplex128ArrayType(
        v2t, {'fun_name': fun_name, 'entity': 'v2t'},
        exp_shape=exp_shape_v2t,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _v2t = _v2t_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobu1,
        _jobu2,
        _jobv1t,
        _jobv2t,
        _trans,
        _signs,
        _m,
        _p,
        _q,
        _x11,
        _ldx11,
        _x12,
        _ldx12,
        _x21,
        _ldx21,
        _x22,
        _ldx22,
        _theta,
        _u1,
        _ldu1,
        _u2,
        _ldu2,
        _v1t,
        _ldv1t,
        _v2t,
        _ldv2t,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _info,
        _jobu1._length_,
        _jobu2._length_,
        _jobv1t._length_,
        _jobv2t._length_,
        _trans._length_,
        _signs._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge during an internal reduction to',
                    'bidiagonal-block form. The process requires convergence to min(p, m-p, q, m-q)',
                    'values, the value of errno gives the number of converged values.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x11_cls.refresh_py(_x11)
        _x12_cls.refresh_py(_x12)
        _x21_cls.refresh_py(_x21)
        _x22_cls.refresh_py(_x22)
        _theta_cls.refresh_py(_theta)
        if u1 is not None:
            _u1_cls.refresh_py(_u1)
        if u2 is not None:
            _u2_cls.refresh_py(_u2)
        if v1t is not None:
            _v1t_cls.refresh_py(_v1t)
        if v2t is not None:
            _v2t_cls.refresh_py(_v2t)
        return_data = None
    _err_class.manage()
    return return_data

def dsygv(itype, jobz, uplo, n, a, b, w):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08saf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if `jobz` = 'V', `a` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^TBZ = I;

        - if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric positive definite matrix B.

        On exit: if the function exits successfully or `errno` = 1 ...
        `n`, the part of `b` containing the matrix is overwritten by the
        triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsygv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08saft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _w,
        _work,
        _lwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def dsygvx(itype, jobz, erange, uplo, n, a, b, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08sbf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix B.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^TU or B = LL^T.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1]. The
          eigenvectors are normalized as follows:

          - if `itype` = 1 or 2, Z^TBZ = I;

          - if `itype` = 3, Z^TB^-1Z = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsygvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08sbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-4,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-11,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-12,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-13,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dsygvd(itype, jobz, uplo, n, a, b, w):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08sc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08scf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: if `jobz` = 'V', `a` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^TBZ = I;

        - if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix B.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^TU or B = LL^T.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsygvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08scft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _w,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def dsygst(itype, uplo, n, a, b):
    u"""
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, B factorized by ``lapacklin.dpotrf``.

    For full information please refer to the NAG Library document for
    f08se

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08sef.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

          - if `uplo` = 'U', C = U^invTHAU^-1;

          - if `uplo` = 'L', C = L^-1AL^invTH.

        `itype` = 2 or 3

          - if `uplo` = 'U', C = UAU^T;

          - if `uplo` = 'L', C = L^TAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and B = U^TU.

        `uplo` = 'L'

          The lower triangular part of A is stored and B = LL^T.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        On exit: the upper or lower triangle of `a` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`.

    b : float, shape (`n`, `n`)
        The Cholesky factor of B as specified by uplo and returned by
        ``lapacklin.dpotrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsygst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08seft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        return_data = None
    _err_class.manage()
    return return_data

def zhegv(itype, jobz, uplo, n, a, b, w):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08sn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08snf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if `jobz` = 'V', `a` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^HBZ = I;

        - if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian positive definite matrix B.

        On exit: if the function exits successfully or `errno` = 1 ...
        `n`, the part of `b` containing the matrix is overwritten by the
        triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhegv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08snft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _w,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def zhegvx(itype, jobz, erange, uplo, n, a, b, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08sp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08spf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: the lower triangle (if `uplo` = 'L') or the upper
        triangle (if `uplo` = 'U') of `a`, including the diagonal, is
        overwritten.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix B.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^HU or B = LL^H.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1]. The
          eigenvectors are normalized as follows:

          - if `itype` = 1 or 2, Z^HBZ = I;

          - if `itype` = 3, Z^HB^-1Z = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhegvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08spft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-4,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-11,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-12,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-13,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _erange,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zhegvd(itype, jobz, uplo, n, a, b, w):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08sq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08sqf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: if `jobz` = 'V', `a` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^HBZ = I;

        - if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix B.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^HU or B = LL^H.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhegvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08sqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _w,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def zhegst(itype, uplo, n, a, b):
    u"""
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, B factorized by ``lapacklin.zpotrf``.

    For full information please refer to the NAG Library document for
    f08ss

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ssf.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

          - if `uplo` = 'U', C = U^invTHAU^-1;

          - if `uplo` = 'L', C = L^-1AL^invTH.

        `itype` = 2 or 3

          - if `uplo` = 'U', C = UAU^H;

          - if `uplo` = 'L', C = L^HAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and B = U^HU.

        `uplo` = 'L'

          The lower triangular part of A is stored and B = LL^H.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        On exit: the upper or lower triangle of `a` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`.

    b : complex, shape (`n`, `n`), modified in place
        The Cholesky factor of B as specified by uplo and returned by
        ``lapacklin.zpotrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhegst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ssft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _uplo,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dspgv(itype, jobz, uplo, n, ap, bp, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage.

    For full information please refer to the NAG Library document for
    f08ta

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08taf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^TU or B = LL^T, in the same storage format
        as B.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^TBZ = I;

        - if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspgv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08taft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineFloat64ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
        _ldz,
        _work,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dspgvx(itype, jobz, erange, uplo, n, ap, bp, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage.

    For full information please refer to the NAG Library document for
    f08tb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tbf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^TU or B = LL^T, in the same storage format
        as B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1]. The
          eigenvectors are normalized as follows:

          - if `itype` = 1 or 2, Z^TBZ = I;

          - if `itype` = 3, Z^TB^-1Z = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspgvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-4,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineFloat64ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-9,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-10,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-11,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ap,
        _bp,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dspgvd(itype, jobz, uplo, n, ap, bp, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08tc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tcf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^TU or B = LL^T, in the same storage format
        as B.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^TBZ = I;

        - if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspgvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineFloat64ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dspgst(itype, uplo, n, ap, bp):
    u"""
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, packed storage, B factorized by ``lapacklin.dpptrf``.

    For full information please refer to the NAG Library document for
    f08te

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tef.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

          - if `uplo` = 'U', C = U^invTHAU^-1;

          - if `uplo` = 'L', C = L^-1AL^invTH.

        `itype` = 2 or 3

          - if `uplo` = 'U', C = UAU^T;

          - if `uplo` = 'L', C = L^TAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and B = U^TU.

        `uplo` = 'L'

          The lower triangular part of A is stored and B = LL^T.

    n : int
        n, the order of the matrices A and B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        matrix A, packed by columns.

        On exit: the upper or lower triangle of `ap` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`, using the same packed storage format as
        described above.

    bp : float, shape (`n`*(`n`+1)/2)
        The Cholesky factor of B as specified by `uplo` and returned by
        ``lapacklin.dpptrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dspgst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08teft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input, bp[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp = utils._EngineFloat64ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _uplo,
        _n,
        _ap,
        _bp,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def zhpgv(itype, jobz, uplo, n, ap, bp, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.

    For full information please refer to the NAG Library document for
    f08tn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tnf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^HU or B = LL^H, in the same storage format
        as B.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^HBZ = I;

        - if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpgv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineComplex128ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhpgvx(itype, jobz, erange, uplo, n, ap, bp, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.

    For full information please refer to the NAG Library document for
    f08tp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tpf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^HU or B = LL^H, in the same storage format
        as B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the first `m` elements contain the selected eigenvalues
        in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the orthonormal eigenvectors of the matrix A
          corresponding to the selected eigenvalues, with the ith column
          of Z holding the eigenvector associated with `w`[i-1]. The
          eigenvectors are normalized as follows:

          - if `itype` = 1 or 2, Z^HBZ = I;

          - if `itype` = 3, Z^HB^-1Z = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpgvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-4,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineComplex128ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-9,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-10,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-11,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ap,
        _bp,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zhpgvd(itype, jobz, uplo, n, ap, bp, w, z):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08tq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tqf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

          Az = lambda Bz.

        `itype` = 2

          ABz = lambda z.

        `itype` = 3

          BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: the contents of `ap` are destroyed.

    bp : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix B, packed by columns.

        On exit: the triangular factor U or L from the Cholesky
        factorization B = U^HU or B = LL^H, in the same storage format
        as B.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors. The eigenvectors are normalized as follows:

        - if `itype` = 1 or 2, Z^HBZ = I;

        - if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpgvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bp[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp_cls = utils._EngineComplex128ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _bp = _bp_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _jobz,
        _uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'then the leading minor of order {:s} of B is not'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'positive definite. The factorization of B could not be completed and no',
                    'eigenvalues or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        _bp_cls.refresh_py(_bp)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhpgst(itype, uplo, n, ap, bp):
    u"""
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, packed storage, B factorized by ``lapacklin.zpptrf``.

    For full information please refer to the NAG Library document for
    f08ts

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08tsf.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

          - if `uplo` = 'U', C = U^invTHAU^-1;

          - if `uplo` = 'L', C = L^-1AL^invTH.

        `itype` = 2 or 3

          - if `uplo` = 'U', C = UAU^H;

          - if `uplo` = 'L', C = L^HAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

          The upper triangular part of A is stored and B = U^HU.

        `uplo` = 'L'

          The lower triangular part of A is stored and B = LL^H.

    n : int
        n, the order of the matrices A and B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        matrix A, packed by columns.

        On exit: the upper or lower triangle of `ap` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`, using the same packed storage format as
        described above.

    bp : complex, shape (`n`*(`n`+1)/2)
        The Cholesky factor of B as specified by `uplo` and returned by
        ``lapacklin.zpptrf``.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhpgst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08tsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input, bp[n*(n+1)//2].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _itype_py = utils._EngineIntScalarType.to_py(_itype)
    if _itype_py not in (1, 2, 3):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('itype in (1, 2, 3).'),
            ],
            errcodes=(-1,),
            entity='itype',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _bp = utils._EngineComplex128ArrayType(
        bp, {'fun_name': fun_name, 'entity': 'bp'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    ).to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _itype,
        _uplo,
        _n,
        _ap,
        _bp,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ap_cls.refresh_py(_ap)
        return_data = None
    _err_class.manage()
    return return_data

def dsbgv(jobz, uplo, n, ka, kb, ab, bb, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08ua

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08uaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : float, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^TS, as returned by ``dpbstf``.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors, with the ith column of Z holding the eigenvector
        associated with `w`[i-1]. The eigenvectors are normalized so
        that Z^TBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``dpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbgv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08uaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _w,
        _z,
        _ldz,
        _work,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'dpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dsbgvx(jobz, erange, uplo, n, ka, kb, ab, bb, q, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ubf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : float, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        positive definite band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^TS, as returned by ``dpbstf``.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the n by n matrix, Q used in the
        reduction of the standard form, i.e., Cx = lambda x, from
        symmetric banded to tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the eigenvectors corresponding to the selected
          eigenvalues, with the ith column of Z holding the eigenvector
          associated with `w`[i-1]. The eigenvectors are normalized so
          that Z^TBZ = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -6)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` -14)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -15)
            On entry, error in parameter `il`.

        (`errno` -16)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``dpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbgvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ubft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-5,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-6,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-14,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-15,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-16,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _q,
        _ldq,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'dpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _q_cls.refresh_py(_q)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dsbgvd(jobz, uplo, n, ka, kb, ab, bb, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08uc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ucf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : float, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^TS, as returned by ``dpbstf``.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors, with the ith column of Z holding the eigenvector
        associated with `w`[i-1]. The eigenvectors are normalized so
        that Z^TBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``dpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbgvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ucft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'dpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dsbgst(vect, uplo, n, ka, kb, ab, bb, x):
    u"""
    Performs a reduction of real symmetric-definite banded generalized
    eigenproblem Ax = lambda Bx to standard form Cy = lambda y, such
    that C has the same bandwidth as A.

    For full information please refer to the NAG Library document for
    f08ue

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08uef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether X is to be returned.

        `vect` = 'N'

          X is not returned.

        `vect` = 'V'

          X is returned.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n symmetric
        band matrix A.

        On exit: the upper or lower triangle of `ab` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `uplo`.

    bb : float, shape (`kb`+1, `n`)
        The banded split Cholesky factor of B as specified by `uplo`,
        `n` and `kb` and returned by ``dpbstf``.

    x : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='V': `n`; if `vect`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='V': `n`; if `vect`='N': 1;
        otherwise: 0.

        On exit: the n by n matrix X = S^-1Q, if `vect` = 'V'.

        If `vect` = 'N', `x` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dsbgst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ueft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    if _vect_py in ('V', 'v'):
        _x_dim1 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim1 = 1
    else:
        _x_dim1 = 0
    if _vect_py in ('V', 'v'):
        _x_dim2 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim2 = 1
    else:
        _x_dim2 = 0
    exp_shape_x = (_x_dim1, _x_dim2)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _x,
        _ldx,
        _work,
        _info,
        _vect._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def dpbstf(uplo, n, kb, bb):
    u"""
    Computes a split Cholesky factorization of real symmetric positive
    definite band matrix A.

    For full information please refer to the NAG Library document for
    f08uf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08uff.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of B is
        stored.

        `uplo` = 'U'

          The upper triangular part of B is stored.

        `uplo` = 'L'

          The lower triangular part of B is stored.

    n : int
        n, the order of the matrix B.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    bb : float, shape (`kb`+1, `n`), modified in place
        On entry: the n by n symmetric positive definite band matrix B.

        On exit: B is overwritten by the elements of its split Cholesky
        factor S.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kb`.

            Constraint: `kb` >= 0.

        (`errno` i > 0)
            The factorization could not be completed, because the
            updated element b(*<value>*,*<value>*) would be the square
            root of a negative number. Hence B is not positive definite.
            This may indicate an error in forming the matrix B.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dpbstf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ufft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((bb, 'bb'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not _kb_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kb >= 0.'),
            ],
            errcodes=(-3,),
            entity='kb',
        ).manage()
    _ldbb_py = utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kb,
        _bb,
        _ldbb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The factorization could not be completed, because the updated element',
                    'b({:s},{:s}) would be the square'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'root of a negative number. Hence B is not positive definite.',
                    'This may indicate an error in forming the matrix B.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _bb_cls.refresh_py(_bb)
        return_data = None
    _err_class.manage()
    return return_data

def zhbgv(jobz, uplo, n, ka, kb, ab, bb, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08un

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08unf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : complex, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^HS, as returned by ``zpbstf``.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors, with the ith column of Z holding the eigenvector
        associated with `w`[i-1]. The eigenvectors are normalized so
        that Z^HBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbgv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08unft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineComplex128ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineComplex128ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'zpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhbgvx(jobz, erange, uplo, n, ka, kb, ab, bb, q, vl, vu, il, iu, abstol, w, z, jfail):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.

    For full information please refer to the NAG Library document for
    f08up

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08upf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : complex, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        positive definite band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^HS, as returned by ``zpbstf``.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', the n by n matrix, Q used in the
        reduction of the standard form, i.e., Cx = lambda x, from
        symmetric banded to tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': if `erange`='I':
        `iu`-`il`+1; otherwise: `n`; otherwise: 0.

        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` columns of
          Z contain the eigenvectors corresponding to the selected
          eigenvalues, with the ith column of Z holding the eigenvector
          associated with `w`[i-1]. The eigenvectors are normalized so
          that Z^HBZ = I;

        - if an eigenvector fails to converge (`errno` = 1 ... `n`),
          then that column of Z contains the latest approximation to the
          eigenvector, and the index of the eigenvector is returned in
          `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        Note: you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, shape (`n`), modified in place
        On exit: if `jobz` = 'V', then

        - if no exception or warning is raised, the first `m` elements
          of `jfail` are zero;

        - if `errno` = 1 ... `n`, the first errno elements of `jfail`
          contains the indices of the eigenvectors that failed to
          converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Returns
    -------
    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -6)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` -14)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -15)
            On entry, error in parameter `il`.

        (`errno` -16)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbgvx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08upft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, erange(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._BYREF_FLOAT64, # Input, vl.
        utils._BYREF_FLOAT64, # Input, vu.
        utils._BYREF_INT, # Input, il.
        utils._BYREF_INT, # Input, iu.
        utils._BYREF_FLOAT64, # Input, abstol.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineIntArrayType.c_type, # Output, jfail[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of erange.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _erange = utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = utils._EngineCharScalarType.to_py(_erange)
    if _erange_py not in ('A', 'a', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('erange in (\'A\', \'a\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='range',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-3,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-5,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-6,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineComplex128ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineComplex128ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _vl = utils._EngineFloat64ScalarType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
    ).to_ctypes()
    _vu = utils._EngineFloat64ScalarType(
        vu, {'fun_name': fun_name, 'entity': 'vu'},
    ).to_ctypes()
    _vl_py = utils._EngineFloat64ScalarType.to_py(_vl)
    _vu_py = utils._EngineFloat64ScalarType.to_py(_vu)
    if _erange_py in ('V', 'v'):
        if not _vl_py < _vu_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('vl < vu.'),
                ],
                errcodes=(-14,),
                entity='vu',
            ).manage()
    _il = utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = utils._EngineIntScalarType.to_py(_il)
    _iu = utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = utils._EngineIntScalarType.to_py(_iu)
    if _erange_py in ('I', 'i'):
        if not (_il_py >= 1 and _il_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('il >= 1.'),
                ],
                errcodes=(-15,),
                entity='il',
            ).manage()
        if not (_iu_py >= min(_n_py, _il_py) and _iu_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('iu >= min(n, il).'),
                ],
                errcodes=(-16,),
                entity='iu',
            ).manage()
    _abstol = utils._EngineFloat64ScalarType(
        abstol, {'fun_name': fun_name, 'entity': 'abstol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _jfail_cls = utils._EngineIntArrayType(
        jfail, {'fun_name': fun_name, 'entity': 'jfail'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _jfail = _jfail_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _erange,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _q,
        _ldq,
        _vl,
        _vu,
        _il,
        _iu,
        _abstol,
        _m,
        _w,
        _z,
        _ldz,
        _work,
        _rwork,
        _iwork,
        _jfail,
        _info,
        _jobz._length_,
        _erange._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} eigenvectors did'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not converge. Their indices are stored in array jfail.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'zpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _q_cls.refresh_py(_q)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        _jfail_cls.refresh_py(_jfail)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def zhbgvd(jobz, uplo, n, ka, kb, ab, bb, w, z):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem
    (divide-and-conquer).

    For full information please refer to the NAG Library document for
    f08uq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08uqf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

          Only eigenvalues are computed.

        `jobz` = 'V'

          Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: the contents of `ab` are overwritten.

    bb : complex, shape (`kb`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix B.

        On exit: the factor S from the split Cholesky factorization B =
        S^HS, as returned by ``zpbstf``.

    w : float, shape (`n`), modified in place
        On exit: the eigenvalues in ascending order.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobz`='V': `n`; otherwise: 1.

        On exit: if `jobz` = 'V', `z` contains the matrix Z of
        eigenvectors, with the ith column of Z holding the eigenvector
        associated with `w`[i-1]. The eigenvectors are normalized so
        that Z^HBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbgvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08uqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobz(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobz.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobz = utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobz in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobz',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineComplex128ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineComplex128ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _z = _z_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lrwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobz,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _w,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _info,
        _jobz._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py <= _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge; {:s} off-diagonal'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'elements of an intermediate tridiagonal form did not converge to zero.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'If errno = n+{:s}, for 1 <= {:s} <= n,'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'zpbstf returned errno = {:s}: B is not positive'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'definite. The factorization of B could not be completed and no eigenvalues',
                    'or eigenvectors were computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _bb_cls.refresh_py(_bb)
        _w_cls.refresh_py(_w)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zhbgst(vect, uplo, n, ka, kb, ab, bb, x):
    u"""
    Performs a reduction of complex Hermitian-definite banded
    generalized eigenproblem Ax = lambda Bx to standard form Cy = lambda
    y, such that C has the same bandwidth as A.

    For full information please refer to the NAG Library document for
    f08us

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08usf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether X is to be returned.

        `vect` = 'N'

          X is not returned.

        `vect` = 'V'

          X is returned.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

          The upper triangular part of A is stored.

        `uplo` = 'L'

          The lower triangular part of A is stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, shape (`ka`+1, `n`), modified in place
        On entry: the upper or lower triangle of the n by n Hermitian
        band matrix A.

        On exit: the upper or lower triangle of `ab` is overwritten by
        the corresponding upper or lower triangle of C as specified by
        `uplo`.

    bb : complex, shape (`kb`+1, `n`)
        The banded split Cholesky factor of B as specified by `uplo`,
        `n` and `kb` and returned by ``zpbstf``.

    x : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='V': `n`; if `vect`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='V': `n`; if `vect`='N': 1;
        otherwise: 0.

        On exit: the n by n matrix X = S^-1Q, if `vect` = 'V'.

        If `vect` = 'N', `x` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhbgst'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08usft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, vect(len=1).
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ka.
        utils._BYREF_INT, # Input, kb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineComplex128ArrayType.c_type, # Output, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vect.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (bb, 'bb'), (x, 'x')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _vect = utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = utils._EngineCharScalarType.to_py(_vect)
    if _vect_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('vect in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='vect',
        ).manage()
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-2,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ka = utils._EngineIntScalarType(
        ka, {'fun_name': fun_name, 'entity': 'ka'},
    ).to_ctypes()
    _ka_py = utils._EngineIntScalarType.to_py(_ka)
    if not _ka_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= 0.'),
            ],
            errcodes=(-4,),
            entity='ka',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not (_ka_py >= _kb_py and _kb_py >= 0):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ka >= kb and kb >= 0.'),
            ],
            errcodes=(-5,),
            entity='kb',
        ).manage()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_ka_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldbb_py = utils._EngineComplex128ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb = utils._EngineComplex128ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    if _vect_py in ('V', 'v'):
        _x_dim1 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim1 = 1
    else:
        _x_dim1 = 0
    if _vect_py in ('V', 'v'):
        _x_dim2 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim2 = 1
    else:
        _x_dim2 = 0
    exp_shape_x = (_x_dim1, _x_dim2)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _vect,
        _uplo,
        _n,
        _ka,
        _kb,
        _ab,
        _ldab,
        _bb,
        _ldbb,
        _x,
        _ldx,
        _work,
        _rwork,
        _info,
        _vect._length_,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ab_cls.refresh_py(_ab)
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def zpbstf(uplo, n, kb, bb):
    u"""
    Computes a split Cholesky factorization of complex Hermitian
    positive definite band matrix A.

    For full information please refer to the NAG Library document for
    f08ut

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08utf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of B is
        stored.

        `uplo` = 'U'

          The upper triangular part of B is stored.

        `uplo` = 'L'

          The lower triangular part of B is stored.

    n : int
        n, the order of the matrix B.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    bb : complex, shape (`kb`+1, `n`), modified in place
        On entry: the n by n Hermitian positive definite band matrix B.

        On exit: B is overwritten by the elements of its split Cholesky
        factor S.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `kb`.

            Constraint: `kb` >= 0.

        (`errno` i > 0)
            The factorization could not be completed, because the
            updated element b(*<value>*,*<value>*) would be the square
            root of a negative number. Hence B is not positive definite.
            This may indicate an error in forming the matrix B.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zpbstf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08utft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, bb[ldbb, n].
        utils._BYREF_INT, # Input, ldbb.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((bb, 'bb'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _uplo_py = utils._EngineCharScalarType.to_py(_uplo)
    if _uplo_py not in ('U', 'u', 'L', 'l'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('uplo in (\'U\', \'u\', \'L\', \'l\').'),
            ],
            errcodes=(-1,),
            entity='uplo',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _kb = utils._EngineIntScalarType(
        kb, {'fun_name': fun_name, 'entity': 'kb'},
    ).to_ctypes()
    _kb_py = utils._EngineIntScalarType.to_py(_kb)
    if not _kb_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('kb >= 0.'),
            ],
            errcodes=(-3,),
            entity='kb',
        ).manage()
    _ldbb_py = utils._EngineComplex128ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldbb = utils._EngineIntScalarType(
        _ldbb_py, {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineComplex128ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_kb_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _bb = _bb_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _uplo,
        _n,
        _kb,
        _bb,
        _ldbb,
        _info,
        _uplo._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The factorization could not be completed, because the updated element',
                    'b({:s},{:s}) would be the square'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'root of a negative number. Hence B is not positive definite.',
                    'This may indicate an error in forming the matrix B.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _bb_cls.refresh_py(_bb)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dggsvd',
    mkwdrn=None,
    advice=[
        'Please use dggsvd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggsvd(jobu, jobv, jobq, m, n, p, a, b, alpha, beta, u, v, q, iwork):
    u"""
    Computes the generalized singular value decomposition of a real
    matrix pair.

    Note: ``dggsvd`` is deprecated.
    Please use ``dggsvd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08va

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vaf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular matrix R, or part of R. See
        [Description] for details.

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix R if m-k-l < 0. See
        [Description] for details.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B, alpha_i and beta_i;

        - `alpha`[0:`k`] = 1,

        - `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

        - `alpha`[`k`:`k`+`l`] = C,

        - `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

        - `alpha`[`k`:`m`] = C,

        - `alpha`[`m`:`k`+`l`] = 0,

        - `beta`[`k`:`m`] = S,

        - `beta`[`m`:`k`+`l`] = 1, and

        - `alpha`[`k`+`l`:`n`] = 0,

        - `beta`[`k`+`l`:`n`] = 0.

        The notation `alpha`[`k`-1:`n`] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,`n`.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 0.

        On exit: if `jobu` = 'U', `u` contains the m by m orthogonal
        matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 0.

        On exit: if `jobv` = 'V', `v` contains the p by p orthogonal
        matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 0.

        On exit: if `jobq` = 'Q', `q` contains the n by n orthogonal
        matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, shape (`n`), modified in place
        On exit: stores the sorting information. More precisely, the
        following loop will sort `alpha`

        ::

          for i in range(k+1, min(m,k+l)+1):
            swap alpha[i-1] and alpha[iwork[i-1]-1]

        such that `alpha`[0] >= `alpha`[1] >= ... >= `alpha`[`n`-1].

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggsvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineIntArrayType.c_type, # Output, iwork[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-6,),
            entity='p',
        ).manage()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 0
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork_cls = utils._EngineIntArrayType(
        iwork, {'fun_name': fun_name, 'entity': 'iwork'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iwork = _iwork_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _n,
        _p,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _iwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _iwork_cls.refresh_py(_iwork)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

def dggsvd3(jobu, jobv, jobq, m, n, p, a, b, alpha, beta, u, v, q, iwork):
    u"""
    Computes, using BLAS-3, the generalized singular value decomposition
    of a real matrix pair.

    For full information please refer to the NAG Library document for
    f08vc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vcf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular matrix R, or part of R. See
        [Description] for details.

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix R if m-k-l < 0. See
        [Description] for details.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B, alpha_i and beta_i;

        - `alpha`[0:`k`] = 1,

        - `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

        - `alpha`[`k`:`k`+`l`] = C,

        - `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

        - `alpha`[`k`:`m`] = C,

        - `alpha`[`m`:`k`+`l`] = 0,

        - `beta`[`k`:`m`] = S,

        - `beta`[`m`:`k`+`l`] = 1, and

        - `alpha`[`k`+`l`:`n`] = 0,

        - `beta`[`k`+`l`:`n`] = 0.

        The notation `alpha`[`k`-1:`n`] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,`n`.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the m by m orthogonal
        matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the p by p orthogonal
        matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the n by n orthogonal
        matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, shape (`n`), modified in place
        On exit: stores the sorting information. More precisely, if I is
        the ordered set of indices of `alpha` containing C (denote as
        `alpha`[I], see `beta`), then the corresponding elements
        `iwork`[I]-1 contain the swap pivots, J, that sorts I such that
        `alpha`[I] is in descending numerical order.

        The following pseudocode sorts the set I:

        for i in I; j = J_i; swap I_i and I_j;end

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggsvd3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-6,),
            entity='p',
        ).manage()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork_cls = utils._EngineIntArrayType(
        iwork, {'fun_name': fun_name, 'entity': 'iwork'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iwork = _iwork_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _n,
        _p,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _lwork,
        _iwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _iwork_cls.refresh_py(_iwork)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dggsvp',
    mkwdrn=None,
    advice=[
        'Please use dggsvp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggsvp(jobu, jobv, jobq, m, p, n, a, b, tola, tolb, u, v, q):
    u"""
    Produces orthogonal matrices that simultaneously reduce the m by n
    matrix A and the p by n matrix B to upper triangular form.

    Note: ``dggsvp`` is deprecated.
    Please use ``dggsvp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08ve

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vef.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular (or trapezoidal) matrix
        described in [Description].

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix described in
        [Description].

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggsvp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08veft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, tau.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _tau = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _k,
        _l,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _iwork,
        _tau,
        _work,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

def dggsvp3(jobu, jobv, jobq, m, p, n, a, b, tola, tolb, u, v, q):
    u"""
    Produces orthogonal matrices, using BLAS-3, that simultaneously
    reduce the m by n matrix A and the p by n matrix B to upper
    triangular form.

    For full information please refer to the NAG Library document for
    f08vg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vgf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular (or trapezoidal) matrix
        described in [Description].

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix described in
        [Description].

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggsvp3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, tau.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _ldu_py = utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _tau = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _k,
        _l,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _iwork,
        _tau,
        _work,
        _lwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zggsvd',
    mkwdrn=None,
    advice=[
        'Please use zggsvd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggsvd(jobu, jobv, jobq, m, n, p, a, b, alpha, beta, u, v, q, iwork):
    u"""
    Computes the generalized singular value decomposition of a complex
    matrix pair.

    Note: ``zggsvd`` is deprecated.
    Please use ``zggsvd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08vn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vnf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular matrix R, or part of R. See
        [Description] for details.

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix R if m-k-l < 0. See
        [Description] for details.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B, alpha_i and beta_i;

        - `alpha`[0:`k`] = 1,

        - `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

        - `alpha`[`k`:`k`+`l`] = C,

        - `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

        - `alpha`[`k`:`m`] = C,

        - `alpha`[`m`:`k`+`l`] = 0,

        - `beta`[`k`:`m`] = S,

        - `beta`[`m`:`k`+`l`] = 1, and

        - `alpha`[`k`+`l`:`n`] = 0,

        - `beta`[`k`+`l`:`n`] = 0.

        The notation `alpha`[`k`-1:`n`] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,`n`.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 0.

        On exit: if `jobu` = 'U', `u` contains the m by m unitary matrix
        U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 0.

        On exit: if `jobv` = 'V', `v` contains the p by p unitary matrix
        V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 0.

        On exit: if `jobq` = 'Q', `q` contains the n by n unitary matrix
        Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, shape (`n`), modified in place
        On exit: stores the sorting information. More precisely, the
        following loop will sort `alpha`

        ::

          for i in range(k+1, min(m,k+l)+1):
            swap alpha[i-1] and alpha[iwork[i-1]-1]

        such that `alpha`[0] >= `alpha`[1] >= ... >= `alpha`[`n`-1].

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggsvd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-6,),
            entity='p',
        ).manage()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 0
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork_cls = utils._EngineIntArrayType(
        iwork, {'fun_name': fun_name, 'entity': 'iwork'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iwork = _iwork_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _n,
        _p,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _rwork,
        _iwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _iwork_cls.refresh_py(_iwork)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

def zggsvd3(jobu, jobv, jobq, m, n, p, a, b, alpha, beta, u, v, q, iwork):
    u"""
    Computes, using BLAS-3, the generalized singular value decomposition
    of a complex matrix pair.

    For full information please refer to the NAG Library document for
    f08vq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vqf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular matrix R, or part of R. See
        [Description] for details.

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix R if m-k-l < 0. See
        [Description] for details.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B, alpha_i and beta_i;

        - `alpha`[0:`k`] = 1,

        - `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

        - `alpha`[`k`:`k`+`l`] = C,

        - `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

        - `alpha`[`k`:`m`] = C,

        - `alpha`[`m`:`k`+`l`] = 0,

        - `beta`[`k`:`m`] = S,

        - `beta`[`m`:`k`+`l`] = 1, and

        - `alpha`[`k`+`l`:`n`] = 0,

        - `beta`[`k`+`l`:`n`] = 0.

        The notation `alpha`[`k`-1:`n`] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,`n`.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the m by m unitary matrix
        U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the p by p unitary matrix
        V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the n by n unitary matrix
        Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, shape (`n`), modified in place
        On exit: stores the sorting information. More precisely, if I is
        the ordered set of indices of `alpha` containing C (denote as
        `alpha`[I], see `beta`), then the corresponding elements
        `iwork`[I]-1 contain the swap pivots, J, that sorts I such that
        `alpha`[I] is in descending numerical order.

        The following pseudocode sorts the set I:

        for i in I; j = J_i; swap I_i and I_j;end

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (A;B).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggsvd3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Output, iwork[n].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-6,),
            entity='p',
        ).manage()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork_cls = utils._EngineIntArrayType(
        iwork, {'fun_name': fun_name, 'entity': 'iwork'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _iwork = _iwork_cls.to_ctypes(can_reference_contents=False)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _n,
        _p,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobi-type procedure failed to converge.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _iwork_cls.refresh_py(_iwork)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zggsvp',
    mkwdrn=None,
    advice=[
        'Please use zggsvp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggsvp(jobu, jobv, jobq, m, p, n, a, b, tola, tolb, u, v, q):
    u"""
    Produces unitary matrices that simultaneously reduce the complex, m
    by n, matrix A and the complex, p by n, matrix B to upper triangular
    form.

    Note: ``zggsvp`` is deprecated.
    Please use ``zggsvp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08vs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vsf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular (or trapezoidal) matrix
        described in [Description].

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix described in
        [Description].

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggsvp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineComplex128ArrayType.c_type, # Workspace, tau.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _tau = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _k,
        _l,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _iwork,
        _rwork,
        _tau,
        _work,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

def zggsvp3(jobu, jobv, jobq, m, p, n, a, b, tola, tolb, u, v, q):
    u"""
    Produces unitary matrices, using BLAS-3, that simultaneously reduce
    the complex, m by n, matrix A and the complex, p by n, matrix B to
    upper triangular form.

    For full information please refer to the NAG Library document for
    f08vu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08vuf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: contains the triangular (or trapezoidal) matrix
        described in [Description].

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: contains the triangular matrix described in
        [Description].

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu`='U': `m`; otherwise: 1.

        On exit: if `jobu` = 'U', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv`='V': `p`; otherwise: 1.

        On exit: if `jobv` = 'V', `v` contains the unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq`='Q': `n`; otherwise: 1.

        On exit: if `jobq` = 'Q', `q` contains the unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Description]; (k+l) is the effective numerical
        rank of (`a`^T `b`^T)^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggsvp3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08vuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._BYREF_INT, # Output, k.
        utils._BYREF_INT, # Output, l.
        utils._EngineComplex128ArrayType.c_type, # Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineComplex128ArrayType.c_type, # Workspace, tau.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType.empty_ctypes()
    _l = utils._EngineIntScalarType.empty_ctypes()
    _ldu_py = utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldu = utils._EngineIntScalarType(
        _ldu_py, {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _tau = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _k,
        _l,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _iwork,
        _rwork,
        _tau,
        _work,
        _lwork,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _k_py = utils._EngineIntScalarType.to_py(_k)
        _l_py = utils._EngineIntScalarType.to_py(_l)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['k', 'l'],
        )(
            _k_py,
            _l_py,
        )
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dggev',
    mkwdrn=None,
    advice=[
        'Please use dggev3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggev(jobvl, jobvr, n, a, b, alphar, alphai, beta, vl, vr):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.

    Note: ``dggev`` is deprecated.
    Please use ``dggev3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08wa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08waf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten.

    b : float, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alphar : float, shape (`n`), modified in place
        On exit: the element `alphar`[j-1] contains the real part of
        alpha_j.

    alphai : float, shape (`n`), modified in place
        On exit: the element `alphai`[j-1] contains the imaginary part
        of alpha_j.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        the corresponding eigenvalues.

        If the jth eigenvalue is real, then u_j = `vl`[:,j], the jth
        column of `vl`.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then u_j = `vl`[:,j]+i*`vl`[:,j+1] and u(j+1) =
        `vl`[:,j]-i*`vl`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        the corresponding eigenvalues.

        If the jth eigenvalue is real, then v_j = `vr`[:,j], the jth
        column of VR.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then v_j = `vr`[:,j]+i*`vr`[:,j+1] and v_{j+1} =
        `vr`[:,j]-i*`vr`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08waft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alphar,
        _alphai,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in dtgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dggevx(balanc, jobvl, jobvr, sense, n, a, b, alphar, alphai, beta, vl, vr, lscale, rscale, rconde, rcondv):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.

    For full information please refer to the NAG Library document for
    f08wb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wbf.html

    Parameters
    ----------
    balanc : str, length 1
        Specifies the balance option to be performed.

        `balanc` = 'N'

          Do not diagonally scale or permute.

        `balanc` = 'P'

          Permute only.

        `balanc` = 'S'

          Scale only.

        `balanc` = 'B'

          Both permute and scale.

        Computed reciprocal condition numbers will be for the matrices
        after permuting and/or balancing.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

        In the absence of other information, `balanc` = 'B' is
        recommended.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for eigenvalues only.

        `sense` = 'V'

          Computed for eigenvectors only.

        `sense` = 'B'

          Computed for eigenvalues and eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten. If `jobvl` = 'V' or `jobvr` =
        'V' or both, then A contains the first part of the real Schur
        form of the 'balanced' versions of the input A and B.

    b : float, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alphar : float, shape (`n`), modified in place
        On exit: the element `alphar`[j-1] contains the real part of
        alpha_j.

    alphai : float, shape (`n`), modified in place
        On exit: the element `alphai`[j-1] contains the imaginary part
        of alpha_j.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left generalized eigenvectors u_j
        are stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right generalized eigenvectors
        v_j are stored one after another in the columns of `vr`, in the
        same order as the corresponding eigenvalues. Each eigenvector
        will be scaled so the largest component will have |real
        part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    lscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the left side of A and B.

        If pl_j is the index of the row interchanged with row j, and
        dl_j is the scaling factor applied to row j, then:

        - `lscale`[j-1] = pl_j, for j = 1,2,...,`ilo`-1;

        - `lscale` = dl_j, for j = `ilo`,...,`ihi`;

        - `lscale` = pl_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the right side of A and B.

        If pr_j is the index of the column interchanged with column j,
        and dr_j is the scaling factor applied to column j, then:

        - `rscale`[j-1] = pr_j, for j = 1,2,...,`ilo`-1;

        - if `rscale` = dr_j, for j = `ilo`,...,`ihi`;

        - if `rscale` = pr_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rconde : float, shape (`n`), modified in place
        On exit: if `sense` = 'E' or 'B', the reciprocal condition
        numbers of the eigenvalues, stored in consecutive elements of
        the array. For a complex conjugate pair of eigenvalues two
        consecutive elements of `rconde` are set to the same value. Thus
        `rconde`[j-1], `rcondv`[j-1], and the jth columns of `vl` and
        `vr` all correspond to the jth eigenpair.

        If `sense` = 'V', `rconde` is not referenced.

    rcondv : float, shape (`n`), modified in place
        On exit: if `sense` = 'V' or 'B', the estimated reciprocal
        condition numbers of the eigenvectors, stored in consecutive
        elements of the array. For a complex eigenvector two consecutive
        elements of `rcondv` are set to the same value.

        If `sense` = 'E', `rcondv` is not referenced.

    Returns
    -------
    ilo : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,`n`.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = `n`.

    ihi : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,`n`.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = `n`.

    abnrm : float
        The 1-norm of the balanced matrix A.

    bbnrm : float
        The 1-norm of the balanced matrix B.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, balanc(len=1).
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rscale[n].
        utils._BYREF_FLOAT64, # Output, abnrm.
        utils._BYREF_FLOAT64, # Output, bbnrm.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of balanc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _balanc = utils._EngineCharScalarType(
        balanc, {'fun_name': fun_name, 'entity': 'balanc'}, exp_chrlen=1,
    ).to_ctypes()
    _balanc_py = utils._EngineCharScalarType.to_py(_balanc)
    if _balanc_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('balanc in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='balanc',
        ).manage()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-3,),
            entity='jobvr',
        ).manage()
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _lscale_cls = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _lscale = _lscale_cls.to_ctypes(can_reference_contents=False)
    _rscale_cls = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rscale = _rscale_cls.to_ctypes(can_reference_contents=False)
    _abnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _bbnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _balanc,
        _jobvl,
        _jobvr,
        _sense,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alphar,
        _alphai,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _abnrm,
        _bbnrm,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _iwork,
        _bwork,
        _info,
        _balanc._length_,
        _jobvl._length_,
        _jobvr._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in dtgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        _lscale_cls.refresh_py(_lscale)
        _rscale_cls.refresh_py(_rscale)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        _abnrm_py = utils._EngineFloat64ScalarType.to_py(_abnrm)
        _bbnrm_py = utils._EngineFloat64ScalarType.to_py(_bbnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi', 'abnrm', 'bbnrm'],
        )(
            _ilo_py,
            _ihi_py,
            _abnrm_py,
            _bbnrm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dggev3(jobvl, jobvr, n, a, b, alphar, alphai, beta, vl, vr):
    u"""
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    For full information please refer to the NAG Library document for
    f08wc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wcf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten.

    b : float, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alphar : float, shape (`n`), modified in place
        On exit: the element `alphar`[j-1] contains the real part of
        alpha_j.

    alphai : float, shape (`n`), modified in place
        On exit: the element `alphai`[j-1] contains the imaginary part
        of alpha_j.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left eigenvectors u_j are stored
        one after another in the columns of `vl`, in the same order as
        the corresponding eigenvalues.

        If the jth eigenvalue is real, then u_j = `vl`[:,j], the jth
        column of `vl`.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then u_j = `vl`[:,j]+i*`vl`[:,j+1] and u(j+1) =
        `vl`[:,j]-i*`vl`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right eigenvectors v_j are stored
        one after another in the columns of `vr`, in the same order as
        the corresponding eigenvalues.

        If the jth eigenvalue is real, then v_j = `vr`[:,j], the jth
        column of VR.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then v_j = `vr`[:,j]+i*`vr`[:,j+1] and v_{j+1} =
        `vr`[:,j]-i*`vr`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggev3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alphar,
        _alphai,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in dtgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dgghrd',
    mkwdrn=None,
    advice=[
        'Please use dgghd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgghrd(compq, compz, n, ilo, ihi, a, b, q, z):
    u"""
    Performs an orthogonal reduction of a pair of real general matrices
    to generalized upper Hessenberg form.

    Note: ``dgghrd`` is deprecated.
    Please use ``dgghd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08we

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wef.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed orthogonal matrix Q.

        `compq` = 'N'

          Do not compute Q.

        `compq` = 'I'

          The orthogonal matrix Q is returned.

        `compq` = 'V'

          `q` must contain an orthogonal matrix Q_1, and the product Q_1
          Q is returned.

    compz : str, length 1
        Specifies the form of the computed orthogonal matrix Z.

        `compz` = 'N'

          Do not compute Z.

        `compz` = 'I'

          The orthogonal matrix Z is returned.

        `compz` = 'V'

          `z` must contain an orthogonal matrix Z_1, and the product Z_1
          Z is returned.

    n : int
        n, the order of the matrices A and B.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix A of the matrix pair (A,B). Usually, this
        is the matrix A returned by ``dormqr``.

        On exit: `a` is overwritten by the upper Hessenberg matrix H.

    b : float, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix B of the matrix pair
        (A,B). Usually, this is the matrix B returned by the QR
        factorization function ``dgeqrf``.

        On exit: `b` is overwritten by the upper triangular matrix T.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain an orthogonal
        matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'I', `q` contains the orthogonal matrix Q.

        If `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain an orthogonal
        matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'I', `z` contains the orthogonal matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgghrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08weft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'I', 'i', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'I\', \'i\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not _ilo_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _info,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dgghd3(compq, compz, n, ilo, ihi, a, b, q, z):
    u"""
    Performs, using BLAS-3, an orthogonal reduction of a pair of real
    general matrices to generalized upper Hessenberg form.

    For full information please refer to the NAG Library document for
    f08wf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wff.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed orthogonal matrix Q.

        `compq` = 'N'

          Do not compute Q.

        `compq` = 'I'

          The orthogonal matrix Q is returned.

        `compq` = 'V'

          `q` must contain an orthogonal matrix Q_1, and the product Q_1
          Q is returned.

    compz : str, length 1
        Specifies the form of the computed orthogonal matrix Z.

        `compz` = 'N'

          Do not compute Z.

        `compz` = 'I'

          The orthogonal matrix Z is returned.

        `compz` = 'V'

          `z` must contain an orthogonal matrix Z_1, and the product Z_1
          Z is returned.

    n : int
        n, the order of the matrices A and B.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix A of the matrix pair (A,B). Usually, this
        is the matrix A returned by ``dormqr``.

        On exit: `a` is overwritten by the upper Hessenberg matrix H.

    b : float, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix B of the matrix pair
        (A,B). Usually, this is the matrix B returned by the QR
        factorization function ``dgeqrf``.

        On exit: `b` is overwritten by the upper triangular matrix T.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain an orthogonal
        matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'I', `q` contains the orthogonal matrix Q.

        If `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain an orthogonal
        matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'I', `z` contains the orthogonal matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgghd3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'I', 'i', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'I\', \'i\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not _ilo_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _work,
        _lwork,
        _info,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dggbal(job, n, a, b, lscale, rscale):
    u"""
    Balances a pair of real, square, matrices.

    For full information please refer to the NAG Library document for
    f08wh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08whf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on matrices A and B.

        `job` = 'N'

          No balancing is done. Initialize `ilo` = 1, `ihi` = `n`,
          `lscale`[i-1] = 1.0 and `rscale`[i-1] = 1.0, for i =
          1,2,...,n.

        `job` = 'P'

          Only permutations are used in balancing.

        `job` = 'S'

          Only scalings are are used in balancing.

        `job` = 'B'

          Both permutations and scalings are used in balancing.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by the balanced matrix. If `job` =
        'N', `a` is not referenced.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix B.

        On exit: `b` is overwritten by the balanced matrix. If `job` =
        'N', `b` is not referenced.

    lscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the left side of the matrices A and B. If P_i is the index of
        the row interchanged with row i and d_i is the scaling factor
        applied to row i, then

        - `lscale`[i-1] = P_i, for i = 1,2,...,i_lo-1;

        - `lscale`[i-1] = d_i, for i = i_lo,...,i_hi;

        - `lscale`[i-1] = P_i, for i = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    rscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the right side of the matrices A and B.

        If P_j is the index of the column interchanged with column j and
        d^_j is the scaling factor applied to column j, then

        - `rscale`[j-1] = P_j, for j = 1,2,...,i_lo-1;

        - `rscale`[j-1] = d^_j, for j = i_lo,...,i_hi;

        - `rscale`[j-1] = P_j, for j = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    Returns
    -------
    ilo : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggbal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08whft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rscale[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _lscale_cls = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _lscale = _lscale_cls.to_ctypes(can_reference_contents=False)
    _rscale_cls = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rscale = _rscale_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _work,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _lscale_cls.refresh_py(_lscale)
        _rscale_cls.refresh_py(_rscale)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi'],
        )(
            _ilo_py,
            _ihi_py,
        )
    _err_class.manage()
    return return_data

def dggbak(job, side, n, ilo, ihi, lscale, rscale, m, v):
    u"""
    Transforms eigenvectors of a pair of real balanced matrices to those
    of original matrix pair supplied to ``dggbal``.

    For full information please refer to the NAG Library document for
    f08wj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wjf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the backward transformation step required.

        `job` = 'N'

          No transformations are done.

        `job` = 'P'

          Only do backward transformations based on permutations.

        `job` = 'S'

          Only do backward transformations based on scaling.

        `job` = 'B'

          Do backward transformations for both permutations and scaling.

        Note: this must be the same argument `job` as supplied to
        ``dggbal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

          The left eigenvectors are transformed.

        `side` = 'R'

          The right eigenvectors are transformed.

    n : int
        n, the order of the matrices A and B of the generalized
        eigenvalue problem.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.

    lscale : float, shape (`n`)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B, as returned by a previous
        call to ``dggbal``.

    rscale : float, shape (`n`)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B, as returned by a previous
        call to ``dggbal``.

    m : int
        m, the required number of left or right eigenvectors.

    v : float, shape (`n`, `m`), modified in place
        On entry: the matrix of right or left eigenvectors, as returned
        by ``dggbal``.

        On exit: the transformed right or left eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -8)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggbak'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Input, rscale[n].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, m].
        utils._BYREF_INT, # Input, ldv.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ilo and ilo <= max(1, n).'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ihi_py >= _ilo_py and _ihi_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= ilo.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not _ilo_py == 1:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-4,),
                entity='ilo',
            ).manage()
        if not _ihi_py == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi = 0.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lscale = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _rscale = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not (_m_py >= 0 and _m_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= m and m <= n.'),
            ],
            errcodes=(-8,),
            entity='m',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _side,
        _n,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _m,
        _v,
        _ldv,
        _info,
        _job._length_,
        _side._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zggev',
    mkwdrn=None,
    advice=[
        'Please use zggev3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggev(jobvl, jobvr, n, a, b, alpha, beta, vl, vr):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.

    Note: ``zggev`` is deprecated.
    Please use ``zggev3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08wn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wnf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left generalized eigenvectors u_j
        are stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right generalized eigenvectors
        v_j are stored one after another in the columns of `vr`, in the
        same order as the corresponding eigenvalues. Each eigenvector
        will be scaled so the largest component will have |real
        part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggev'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alpha and',
                    'beta should be correct from element {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in ztgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zggevx(balanc, jobvl, jobvr, sense, n, a, b, alpha, beta, vl, vr, lscale, rscale, rconde, rcondv):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.

    For full information please refer to the NAG Library document for
    f08wp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wpf.html

    Parameters
    ----------
    balanc : str, length 1
        Specifies the balance option to be performed.

        `balanc` = 'N'

          Do not diagonally scale or permute.

        `balanc` = 'P'

          Permute only.

        `balanc` = 'S'

          Scale only.

        `balanc` = 'B'

          Both permute and scale.

        Computed reciprocal condition numbers will be for the matrices
        after permuting and/or balancing.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

        In the absence of other information, `balanc` = 'B' is
        recommended.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for eigenvalues only.

        `sense` = 'V'

          Computed for eigenvectors only.

        `sense` = 'B'

          Computed for eigenvalues and eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten. If `jobvl` = 'V' or `jobvr` =
        'V' or both, then A contains the first part of the Schur form of
        the 'balanced' versions of the input A and B.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left generalized eigenvectors u_j
        are stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right generalized eigenvectors
        v_j are stored one after another in the columns of `vr`, in the
        same order as the corresponding eigenvalues. Each eigenvector
        will be scaled so the largest component will have |real
        part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    lscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the left side of A and B.

        If pl_j is the index of the row interchanged with row j, and
        dl_j is the scaling factor applied to row j, then:

        - `lscale`[j-1] = pl_j, for j = 1,2,...,`ilo`-1;

        - `lscale` = dl_j, for j = `ilo`,...,`ihi`;

        - `lscale` = pl_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the right side of A and B.

        If pr_j is the index of the column interchanged with column j,
        and dr_j is the scaling factor applied to column j, then:

        - `rscale`[j-1] = pr_j, for j = 1,2,...,`ilo`-1;

        - if `rscale` = dr_j, for j = `ilo`,...,`ihi`;

        - if `rscale` = pr_j, for j = `ihi`+1,...,`n`.

        The order in which the interchanges are made is `n` to `ihi`+1,
        then 1 to `ilo`-1.

    rconde : float, shape (`n`), modified in place
        On exit: if `sense` = 'E' or 'B', the reciprocal condition
        numbers of the eigenvalues, stored in consecutive elements of
        the array.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, shape (`n`), modified in place
        On exit: if `sense` = 'V' or 'B', the estimated reciprocal
        condition numbers of the selected eigenvectors, stored in
        consecutive elements of the array.

        If `sense` = 'N' or 'E', `rcondv` is not referenced.

    Returns
    -------
    ilo : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,`n`.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = `n`.

    ihi : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,`n`.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = `n`.

    abnrm : float
        The 1-norm of the balanced matrix A.

    bbnrm : float
        The 1-norm of the balanced matrix B.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggevx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, balanc(len=1).
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rscale[n].
        utils._BYREF_FLOAT64, # Output, abnrm.
        utils._BYREF_FLOAT64, # Output, bbnrm.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of balanc.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _balanc = utils._EngineCharScalarType(
        balanc, {'fun_name': fun_name, 'entity': 'balanc'}, exp_chrlen=1,
    ).to_ctypes()
    _balanc_py = utils._EngineCharScalarType.to_py(_balanc)
    if _balanc_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('balanc in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='balanc',
        ).manage()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-3,),
            entity='jobvr',
        ).manage()
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-4,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _lscale_cls = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _lscale = _lscale_cls.to_ctypes(can_reference_contents=False)
    _rscale_cls = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rscale = _rscale_cls.to_ctypes(can_reference_contents=False)
    _abnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _bbnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _balanc,
        _jobvl,
        _jobvr,
        _sense,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _abnrm,
        _bbnrm,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _bwork,
        _info,
        _balanc._length_,
        _jobvl._length_,
        _jobvr._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alpha and',
                    'beta should be correct from element {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in ztgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        _lscale_cls.refresh_py(_lscale)
        _rscale_cls.refresh_py(_rscale)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        _abnrm_py = utils._EngineFloat64ScalarType.to_py(_abnrm)
        _bbnrm_py = utils._EngineFloat64ScalarType.to_py(_bbnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi', 'abnrm', 'bbnrm'],
        )(
            _ilo_py,
            _ihi_py,
            _abnrm_py,
            _bbnrm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def zggev3(jobvl, jobvr, n, a, b, alpha, beta, vl, vr):
    u"""
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    For full information please refer to the NAG Library document for
    f08wq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wqf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix A in the pair (A,B).

        On exit: `a` has been overwritten.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the matrix B in the pair (A,B).

        On exit: `b` has been overwritten.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': `n`; otherwise: 1.

        On exit: if `jobvl` = 'V', the left generalized eigenvectors u_j
        are stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': `n`; otherwise: 1.

        On exit: if `jobvr` = 'V', the right generalized eigenvectors
        v_j are stored one after another in the columns of `vr`, in the
        same order as the corresponding eigenvalues. Each eigenvector
        will be scaled so the largest component will have |real
        part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` `n`+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggev3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvl',
        ).manage()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvr',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobvl,
        _jobvr,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _work,
        _lwork,
        _rwork,
        _info,
        _jobvl._length_,
        _jobvr._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alpha and',
                    'beta should be correct from element {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A failure occurred in ztgevc while computing generalized eigenvectors.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vl_cls.refresh_py(_vl)
        _vr_cls.refresh_py(_vr)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zgghrd',
    mkwdrn=None,
    advice=[
        'Please use zgghd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgghrd(compq, compz, n, ilo, ihi, a, b, q, z):
    u"""
    Performs a unitary reduction of a pair of complex general matrices
    to generalized upper Hessenberg form.

    Note: ``zgghrd`` is deprecated.
    Please use ``zgghd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08ws

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wsf.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed unitary matrix Q.

        `compq` = 'N'

          Do not compute Q.

        `compq` = 'I'

          The unitary matrix Q is returned.

        `compq` = 'V'

          `q` must contain a unitary matrix Q_1, and the product Q_1 Q
          is returned.

    compz : str, length 1
        Specifies the form of the computed unitary matrix Z.

        `compz` = 'N'

          Do not compute Z.

        `compz` = 'V'

          `z` must contain a unitary matrix Z_1, and the product Z_1 Z
          is returned.

        `compz` = 'I'

          The unitary matrix Z is returned.

    n : int
        n, the order of the matrices A and B.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix A of the matrix pair (A,B). Usually, this
        is the matrix A returned by ``zunmqr``.

        On exit: `a` is overwritten by the upper Hessenberg matrix H.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix B of the matrix pair
        (A,B). Usually, this is the matrix B returned by the QR
        factorization function ``zgeqrf``.

        On exit: `b` is overwritten by the upper triangular matrix T.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain a unitary matrix
        Q_1.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'I', `q` contains the unitary matrix Q.

        Iif `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain a unitary matrix
        Z_1.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'I', `z` contains the unitary matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgghrd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'I', 'i', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'I\', \'i\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not _ilo_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _info,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zgghd3(compq, compz, n, ilo, ihi, a, b, q, z):
    u"""
    Performs, using BLAS-3, a unitary reduction of a pair of complex
    general matrices to generalized upper Hessenberg form.

    For full information please refer to the NAG Library document for
    f08wt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wtf.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed unitary matrix Q.

        `compq` = 'N'

          Do not compute Q.

        `compq` = 'I'

          The unitary matrix Q is returned.

        `compq` = 'V'

          `q` must contain a unitary matrix Q_1, and the product Q_1 Q
          is returned.

    compz : str, length 1
        Specifies the form of the computed unitary matrix Z.

        `compz` = 'N'

          Do not compute Z.

        `compz` = 'V'

          `z` must contain a unitary matrix Z_1, and the product Z_1 Z
          is returned.

        `compz` = 'I'

          The unitary matrix Z is returned.

    n : int
        n, the order of the matrices A and B.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix A of the matrix pair (A,B). Usually, this
        is the matrix A returned by ``zunmqr``.

        On exit: `a` is overwritten by the upper Hessenberg matrix H.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the upper triangular matrix B of the matrix pair
        (A,B). Usually, this is the matrix B returned by the QR
        factorization function ``zgeqrf``.

        On exit: `b` is overwritten by the upper triangular matrix T.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', `q` must contain a unitary matrix
        Q_1.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'I', `q` contains the unitary matrix Q.

        Iif `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, `n`); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', `z` must contain a unitary matrix
        Z_1.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'I', `z` contains the unitary matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgghd3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'I', 'i', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'I\', \'i\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not _ilo_py >= 1:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ilo >= 1.'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim1 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim1 = 1
    else:
        _q_dim1 = 0
    if _compq_py in ('I', 'i', 'V', 'v'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim1 = max(1, _n_py)
    elif _compz_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _work,
        _lwork,
        _info,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def zggbal(job, n, a, b, lscale, rscale):
    u"""
    Balances a pair of complex, square, matrices.

    For full information please refer to the NAG Library document for
    f08wv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wvf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on matrices A and B.

        `job` = 'N'

          No balancing is done. Initialize `ilo` = 1, `ihi` = `n`,
          `lscale`[i-1] = 1.0 and `rscale`[i-1] = 1.0, for i =
          1,2,...,n.

        `job` = 'P'

          Only permutations are used in balancing.

        `job` = 'S'

          Only scalings are are used in balancing.

        `job` = 'B'

          Both permutations and scalings are used in balancing.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is overwritten by the balanced matrix. If `job` =
        'N', `a` is not referenced.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix B.

        On exit: `b` is overwritten by the balanced matrix. If `job` =
        'N', `b` is not referenced.

    lscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the left side of the matrices A and B. If P_i is the index of
        the row interchanged with row i and d_i is the scaling factor
        applied to row i, then

        - `lscale`[i-1] = P_i, for i = 1,2,...,i_lo-1;

        - `lscale`[i-1] = d_i, for i = i_lo,...,i_hi;

        - `lscale`[i-1] = P_i, for i = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    rscale : float, shape (`n`), modified in place
        On exit: details of the permutations and scaling factors applied
        to the right side of the matrices A and B.

        If P_j is the index of the column interchanged with column j and
        d^_j is the scaling factor applied to column j, then

        - `rscale`[j-1] = P_j, for j = 1,2,...,i_lo-1;

        - `rscale`[j-1] = d^_j, for j = i_lo,...,i_hi;

        - `rscale`[j-1] = P_j, for j = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    Returns
    -------
    ilo : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggbal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, ilo.
        utils._BYREF_INT, # Output, ihi.
        utils._EngineFloat64ArrayType.c_type, # Output, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Output, rscale[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ilo = utils._EngineIntScalarType.empty_ctypes()
    _ihi = utils._EngineIntScalarType.empty_ctypes()
    _lscale_cls = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _lscale = _lscale_cls.to_ctypes(can_reference_contents=False)
    _rscale_cls = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rscale = _rscale_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _work,
        _info,
        _job._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _lscale_cls.refresh_py(_lscale)
        _rscale_cls.refresh_py(_rscale)
        _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
        _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ilo', 'ihi'],
        )(
            _ilo_py,
            _ihi_py,
        )
    _err_class.manage()
    return return_data

def zggbak(job, side, n, ilo, ihi, lscale, rscale, m, v):
    u"""
    Transforms eigenvectors of a pair of complex balanced matrices to
    those of original matrix pair supplied to ``zggbal``.

    For full information please refer to the NAG Library document for
    f08ww

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08wwf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the backtransformation step required.

        `job` = 'N'

          No transformations are done.

        `job` = 'P'

          Only do backward transformations based on permutations.

        `job` = 'S'

          Only do backward transformations based on scaling.

        `job` = 'B'

          Do backward transformations for both permutations and scaling.

        Note: this must be identical to the argument `job` as supplied
        to ``zggbal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

          The left eigenvectors are transformed.

        `side` = 'R'

          The right eigenvectors are transformed.

    n : int
        n, the order of the matrices A and B of the generalized
        eigenvalue problem.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.

    lscale : float, shape (`n`)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B, as returned by a previous
        call to ``zggbal``.

    rscale : float, shape (`n`)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B, as returned by a previous
        call to ``zggbal``.

    m : int
        m, the required number of left or right eigenvectors.

    v : complex, shape (`n`, `m`), modified in place
        On entry: the matrix of right or left eigenvectors, as returned
        by ``zggbal``.

        On exit: the transformed right or left eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -8)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggbak'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08wwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input, lscale[n].
        utils._EngineFloat64ArrayType.c_type, # Input, rscale[n].
        utils._BYREF_INT, # Input, m.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, m].
        utils._BYREF_INT, # Input, ldv.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('N', 'n', 'P', 'p', 'S', 's', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'N\', \'n\', \'P\', \'p\', \'S\', \'s\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-2,),
            entity='side',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    if not (_ilo_py >= 1 and _ilo_py <= max(1, _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ilo and ilo <= max(1, n).'),
            ],
            errcodes=(-4,),
            entity='ilo',
        ).manage()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ihi_py >= _ilo_py and _ihi_py <= max(1, _n_py)):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi >= ilo.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not _ilo_py == 1:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-4,),
                entity='ilo',
            ).manage()
        if not _ihi_py == 0:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ihi = 0.'),
                ],
                errcodes=(-5,),
                entity='ihi',
            ).manage()
    _lscale = utils._EngineFloat64ArrayType(
        lscale, {'fun_name': fun_name, 'entity': 'lscale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _rscale = utils._EngineFloat64ArrayType(
        rscale, {'fun_name': fun_name, 'entity': 'rscale'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not (_m_py >= 0 and _m_py <= _n_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= m and m <= n.'),
            ],
            errcodes=(-8,),
            entity='m',
        ).manage()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _side,
        _n,
        _ilo,
        _ihi,
        _lscale,
        _rscale,
        _m,
        _v,
        _ldv,
        _info,
        _job._length_,
        _side._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _v_cls.refresh_py(_v)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.dgges',
    mkwdrn=None,
    advice=[
        'Please use dgges3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgges(jobvsl, jobvsr, sort, selctg, n, a, b, alphar, alphai, beta, vsl, vsr, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the
    left and/or right matrices of Schur vectors.

    Note: ``dgges`` is deprecated.
    Please use ``dgges3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08xa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xaf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(ar, ai, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : float, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alphar : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    alphai : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,`n`, are the diagonals of the complex Schur form (S,T)
        that would result if the 2 by 2 diagonal blocks of the real
        Schur form of (A,B) were further reduced to triangular form
        using 2 by 2 complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgges'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ar.
        utils._BYREF_FLOAT64, # Input, ai.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xazt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, ar.
            utils._BYREF_FLOAT64, # Input, ai.
            utils._BYREF_FLOAT64, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, ar, ai, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    ar,
                    ai,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _ar_selctgh = utils._EngineFloat64ScalarType.to_py(ar.contents)
            _ai_selctgh = utils._EngineFloat64ScalarType.to_py(ai.contents)
            _b_selctgh = utils._EngineFloat64ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineFloat64ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineFloat64ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineFloat64ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineFloat64ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineFloat64ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alphar,
        _alphai,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _work,
        _lwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def dggesx(jobvsl, jobvsr, sort, selctg, sense, n, a, b, alphar, alphai, beta, vsl, vsr, rconde, rcondv, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the
    left and/or right matrices of Schur vectors; also, optionally,
    computes reciprocal condition numbers for selected eigenvalues.

    For full information please refer to the NAG Library document for
    f08xb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xbf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(ar, ai, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for average of selected eigenvalues only.

        `sense` = 'V'

          Computed for selected deflating subspaces only.

        `sense` = 'B'

          Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : float, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alphar : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    alphai : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,`n`, are the diagonals of the complex Schur form (S,T)
        that would result if the 2 by 2 diagonal blocks of the real
        Schur form of (A,B) were further reduced to triangular form
        using 2 by 2 complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 0.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 0.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    rconde : float, shape (2), modified in place
        On exit: if `sense` = 'E' or 'B', `rconde`[0] and `rconde`[1]
        contain the reciprocal condition numbers for the average of the
        selected eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, shape (2), modified in place
        On exit: if `sense` = 'V' or 'B', `rcondv`[0] and `rcondv`[1]
        contain the reciprocal condition numbers for the selected
        deflating subspaces.

        if `sense` = 'N' or 'E', `rcondv` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggesx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ar.
        utils._BYREF_FLOAT64, # Input, ai.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xazt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, ar.
            utils._BYREF_FLOAT64, # Input, ai.
            utils._BYREF_FLOAT64, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, ar, ai, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    ar,
                    ai,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _ar_selctgh = utils._EngineFloat64ScalarType.to_py(ar.contents)
            _ai_selctgh = utils._EngineFloat64ScalarType.to_py(ai.contents)
            _b_selctgh = utils._EngineFloat64ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineFloat64ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[2].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[2].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-5,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineFloat64ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 0
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineFloat64ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineFloat64ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 0
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineFloat64ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(2,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(2,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _sense,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alphar,
        _alphai,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def dgges3(jobvsl, jobvsr, sort, selctg, n, a, b, alphar, alphai, beta, vsl, vsr, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, the generalized real Schur form and,
    optionally, the left and/or right matrices of Schur vectors.

    For full information please refer to the NAG Library document for
    f08xc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xcf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(ar, ai, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : float, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alphar : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    alphai : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j =
        1,2,...,`n`, will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,`n`, are the diagonals of the complex Schur form (S,T)
        that would result if the 2 by 2 diagonal blocks of the real
        Schur form of (A,B) were further reduced to triangular form
        using 2 by 2 complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        Note: the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgges3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ar.
        utils._BYREF_FLOAT64, # Input, ai.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xazt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, ar.
            utils._BYREF_FLOAT64, # Input, ai.
            utils._BYREF_FLOAT64, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, ar, ai, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    ar,
                    ai,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _ar_selctgh = utils._EngineFloat64ScalarType.to_py(ar.contents)
            _ai_selctgh = utils._EngineFloat64ScalarType.to_py(ai.contents)
            _b_selctgh = utils._EngineFloat64ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _ar_selctgh,
                    _ai_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineFloat64ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineFloat64ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineFloat64ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineFloat64ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineFloat64ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alphar,
        _alphai,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _work,
        _lwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. No eigenvectors have been calculated but alphar[j],',
                    'alphai[j] and beta[j] should be correct from element',
                    '{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def dhgeqz(job, compq, compz, n, ilo, ihi, a, b, alphar, alphai, beta, q, z):
    u"""
    Computes eigenvalues and generalized Schur factorization of real
    generalized upper Hessenberg form reduced from a pair of real
    general matrices.

    For full information please refer to the NAG Library document for
    f08xe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xef.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on (A,B).

        `job` = 'E'

          The matrix pair (A,B) on exit might not be in the generalized
          Schur form.

        `job` = 'S'

          The matrix pair (A,B) on exit will be in the generalized Schur
          form.

    compq : str, length 1
        Specifies the operations to be performed on Q:

        `compq` = 'N'

          The array `q` is unchanged.

        `compq` = 'V'

          The left transformation Q is accumulated on the array `q`.

        `compq` = 'I'

          The array `q` is initialized to the identity matrix before the
          left transformation Q is accumulated in `q`.

    compz : str, length 1
        Specifies the operations to be performed on Z.

        `compz` = 'N'

          The array `z` is unchanged.

        `compz` = 'V'

          The right transformation Z is accumulated on the array `z`.

        `compz` = 'I'

          The array `z` is initialized to the identity matrix before the
          right transformation Z is accumulated in `z`.

    n : int
        n, the order of the matrices A, B, Q and Z.

    ilo : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``dggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = `n`.

    ihi : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``dggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = `n`.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper Hessenberg matrix A. The elements
        below the first subdiagonal must be set to zero.

        On exit: if `job` = 'S', the matrix pair (A,B) will be
        simultaneously reduced to generalized Schur form.

        If `job` = 'E', the 1 by 1 and 2 by 2 diagonal blocks of the
        matrix pair (A,B) will give generalized eigenvalues but the
        remaining elements will be irrelevant.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix B. The elements
        below the diagonal must be zero.

        On exit: if `job` = 'S', the matrix pair (A,B) will be
        simultaneously reduced to generalized Schur form.

        If `job` = 'E', the 1 by 1 and 2 by 2 diagonal blocks of the
        matrix pair (A,B) will give generalized eigenvalues but the
        remaining elements will be irrelevant.

    alphar : float, shape (`n`), modified in place
        On exit: the real parts of alpha_j, for j = 1,2,...,n.

    alphai : float, shape (`n`), modified in place
        On exit: the imaginary parts of alpha_j, for j = 1,2,...,n.

    beta : float, shape (`n`), modified in place
        On exit: beta_j, for j = 1,2,...,n.

    q : float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('V', 'I'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', the matrix Q_0. The matrix Q_0 is
        usually the matrix Q returned by ``dgghrd``.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'V', `q` contains the matrix product QQ_0.

        If `compq` = 'I', `q` contains the transformation matrix Q.

    z : float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', the matrix Z_0. The matrix Z_0 is
        usually the matrix Z returned by ``dgghrd``.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'V', `z` contains the matrix product ZZ_0.

        If `compz` = 'I', `z` contains the transformation matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i>`n` and i<=2*`n`)
            The computation of shifts failed and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i > 2*`n`)
            An unexpected Library error has occurred.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dhgeqz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08xeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'S\', \'s\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v', 'I', 'i'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_n_py, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_n_py, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _alphar,
        _alphai,
        _beta,
        _q,
        _ldq,
        _z,
        _ldz,
        _work,
        _lwork,
        _info,
        _job._length_,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py < _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration did not converge and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py and _info_py <= 2*_n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of shifts failed and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py-_n_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 2*_n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected Library error has occurred.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.lapackeig.zgges',
    mkwdrn=None,
    advice=[
        'Please use zgges3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgges(jobvsl, jobvsr, sort, selctg, n, a, b, alpha, beta, vsl, vsr, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors.

    Note: ``zgges`` is deprecated.
    Please use ``zgges3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08xn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xnf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(a, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues. `alpha`[j-1], for j = 1,2,...,`n`
        and `beta`[j-1], for j = 1,2,...,`n`, are the diagonals of the
        complex Schur form (A,B) output by ``zgges``. The `beta`[j-1]
        will be non-negative real.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||A||_2 in magnitude, and `beta` will always be less than
        and usually comparable with ||B||_2.

    vsl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgges'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, a.
        utils._BYREF_COMPLEX128, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xnzt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_COMPLEX128, # Input, a.
            utils._BYREF_COMPLEX128, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, a, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    a,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _a_selctgh = utils._EngineComplex128ScalarType.to_py(a.contents)
            _b_selctgh = utils._EngineComplex128ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineComplex128ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineComplex128ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineComplex128ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineComplex128ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineComplex128ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alpha,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _work,
        _lwork,
        _rwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py < _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration did not converge and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def zggesx(jobvsl, jobvsr, sort, selctg, sense, n, a, b, alpha, beta, vsl, vsr, rconde, rcondv, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors; also, optionally,
    computes reciprocal condition numbers for selected eigenvalues.

    For full information please refer to the NAG Library document for
    f08xp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xpf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(a, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

          None are computed.

        `sense` = 'E'

          Computed for average of selected eigenvalues only.

        `sense` = 'V'

          Computed for selected deflating subspaces only.

        `sense` = 'B'

          Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues. `alpha`[j-1] and `beta`[j-1],j =
        1,2,...,`n` are the diagonals of the complex Schur form (S,T).
        `beta`[j-1] will be non-negative real.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||`a`|| in magnitude, and `beta` will always be less than
        and usually comparable with ||`b`||.

    vsl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    rconde : float, shape (2), modified in place
        On exit: if `sense` = 'E' or 'B', `rconde`[0] and `rconde`[1]
        contain the reciprocal condition numbers for the average of the
        selected eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, shape (2), modified in place
        On exit: if `sense` = 'V' or 'B', `rcondv`[0] and `rcondv`[1]
        contain the reciprocal condition numbers for the selected
        deflating subspaces.

        if `sense` = 'N' or 'E', `rcondv` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The QZ iteration failed. (A,B) are not in Schur form, but
            `alpha`[j] and `beta`[j] should be correct from element
            *<value>*.

        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggesx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, a.
        utils._BYREF_COMPLEX128, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xnzt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_COMPLEX128, # Input, a.
            utils._BYREF_COMPLEX128, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, a, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    a,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _a_selctgh = utils._EngineComplex128ScalarType.to_py(a.contents)
            _b_selctgh = utils._EngineComplex128ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_STRING, # Input, sense(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineComplex128ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineFloat64ArrayType.c_type, # Output, rconde[2].
        utils._EngineFloat64ArrayType.c_type, # Output, rcondv[2].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sense.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _sense = utils._EngineCharScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'}, exp_chrlen=1,
    ).to_ctypes()
    _sense_py = utils._EngineCharScalarType.to_py(_sense)
    if _sense_py not in ('N', 'n', 'E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sense in (\'N\', \'n\', \'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-5,),
            entity='sense',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineComplex128ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineComplex128ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineComplex128ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineComplex128ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _rconde_cls = utils._EngineFloat64ArrayType(
        rconde, {'fun_name': fun_name, 'entity': 'rconde'},
        exp_shape=(2,),
        check_contents=False,
    )
    _rconde = _rconde_cls.to_ctypes(can_reference_contents=False)
    _rcondv_cls = utils._EngineFloat64ArrayType(
        rcondv, {'fun_name': fun_name, 'entity': 'rcondv'},
        exp_shape=(2,),
        check_contents=False,
    )
    _rcondv = _rcondv_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _sense,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alpha,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _rconde,
        _rcondv,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _liwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
        _sense._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0 and _info_py <= _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed. (A,B) are not in Schur form, but alpha[j] and',
                    'beta[j] should be correct from element {:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _rconde_cls.refresh_py(_rconde)
        _rcondv_cls.refresh_py(_rcondv)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def zgges3(jobvsl, jobvsr, sort, selctg, n, a, b, alpha, beta, vsl, vsr, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, the generalized complex Schur form and,
    optionally, the left and/or right matrices of Schur vectors.

    For full information please refer to the NAG Library document for
    f08xq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xqf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

          Eigenvalues are not ordered.

        `sort` = 'S'

          Eigenvalues are ordered (see `selctg`).

    selctg : None or callable retval = selctg(a, b, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the first of the pair of matrices, A.

        On exit: `a` has been overwritten by its generalized Schur form
        S.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the second of the pair of matrices, B.

        On exit: `b` has been overwritten by its generalized Schur form
        T.

    alpha : complex, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be
        the generalized eigenvalues. `alpha`[j-1], for j = 1,2,...,`n`
        and `beta`[j-1], for j = 1,2,...,`n`, are the diagonals of the
        complex Schur form (A,B) output by ``zgges3``. The `beta`[j-1]
        will be non-negative real.

        Note: the quotients `alpha`[j-1]/`beta`[j-1] may easily overflow
        or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||A||_2 in magnitude, and `beta` will always be less than
        and usually comparable with ||B||_2.

    vsl : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsl`='V': `n`; otherwise: 1.

        On exit: if `jobvsl` = 'V', `vsl` will contain the left Schur
        vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvsr`='V': `n`; otherwise: 1.

        On exit: if `jobvsr` = 'V', `vsr` will contain the right Schur
        vectors, Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgges3'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_selctg_name = fun_name + ':selctg'

    selctgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_COMPLEX128, # Input, a.
        utils._BYREF_COMPLEX128, # Input, b.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
    )

    if selctg is None:
        selctg_auxil_ctypes = utils._EngineState._get_symbol('f08xnzt')
        selctg_auxil_ctypes.restype = None
        selctg_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_COMPLEX128, # Input, a.
            utils._BYREF_COMPLEX128, # Input, b.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input, ruser[*].
        )

    def cb_selctgh(_selctg, _selctgsub, en_data, a, b, iretval, iuser, ruser):
        """Helper for selctg."""
        try:
            if selctg is None:
                selctg_auxil_ctypes(
                    en_data,
                    a,
                    b,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _a_selctgh = utils._EngineComplex128ScalarType.to_py(a.contents)
            _b_selctgh = utils._EngineComplex128ScalarType.to_py(b.contents)
            if data is not None:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                    data,
                )
            else:
                _retval_selctgh = selctg(
                    _a_selctgh,
                    _b_selctgh,
                )
            _retval_selctgh_ct = utils._EngineLogicalScalarType(
                _retval_selctgh, {'fun_name': cb_selctg_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_selctgh_py = utils._EngineLogicalScalarType.to_py(_retval_selctgh_ct)
            if _retval_selctgh_py is False:
                _iretval_selctgh_py = 0
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_selctgh_py is True:
                _iretval_selctgh_py = 1
                utils._EngineIntScalarType(
                    _iretval_selctgh_py, {'fun_name': cb_selctg_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_selctg_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f08xqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_STRING, # Input, jobvsl(len=1).
        utils._BYREF_STRING, # Input, jobvsr(len=1).
        utils._BYREF_STRING, # Input, sort(len=1).
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        selctgh,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Output, sdim.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Output, vsl[ldvsl, *].
        utils._BYREF_INT, # Input, ldvsl.
        utils._EngineComplex128ArrayType.c_type, # Output, vsr[ldvsr, *].
        utils._BYREF_INT, # Input, ldvsr.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvsr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of sort.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vsl, 'vsl'), (vsr, 'vsr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _call_vendor = utils._EngineIntScalarType.c_type(0)
    _jobvsl = utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsl in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-1,),
            entity='jobvsl',
        ).manage()
    _jobvsr = utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py not in ('N', 'n', 'V', 'v'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobvsr in (\'N\', \'n\', \'V\', \'v\').'),
            ],
            errcodes=(-2,),
            entity='jobvsr',
        ).manage()
    _sort = utils._EngineCharScalarType(
        sort, {'fun_name': fun_name, 'entity': 'sort'}, exp_chrlen=1,
    ).to_ctypes()
    _sort_py = utils._EngineCharScalarType.to_py(_sort)
    if _sort_py not in ('N', 'n', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('sort in (\'N\', \'n\', \'S\', \'s\').'),
            ],
            errcodes=(-3,),
            entity='sort',
        ).manage()
    utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = utils._UNUSED_CALLBACK
    _selctgsub = utils._UNUSED_CALLBACK
    _selctgh = utils._cast_callback(
        selctgh, cb_selctgh,
        {'fun_name': fun_name, 'entity': 'selctg'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _sdim = utils._EngineIntScalarType.empty_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvsl_py = utils._EngineComplex128ArrayType.get_nd(vsl, {'fun_name': fun_name, 'entity': 'vsl'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsl = utils._EngineIntScalarType(
        _ldvsl_py, {'fun_name': fun_name, 'entity': '_ldvsl'},
    ).to_ctypes()
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl_cls = utils._EngineComplex128ArrayType(
        vsl, {'fun_name': fun_name, 'entity': 'vsl'},
        exp_shape=exp_shape_vsl,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsl = _vsl_cls.to_ctypes(can_reference_contents=False)
    _ldvsr_py = utils._EngineComplex128ArrayType.get_nd(vsr, {'fun_name': fun_name, 'entity': 'vsr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvsr = utils._EngineIntScalarType(
        _ldvsr_py, {'fun_name': fun_name, 'entity': '_ldvsr'},
    ).to_ctypes()
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr_cls = utils._EngineComplex128ArrayType(
        vsr, {'fun_name': fun_name, 'entity': 'vsr'},
        exp_shape=exp_shape_vsr,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vsr = _vsr_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _call_vendor,
        _jobvsl,
        _jobvsr,
        _sort,
        _selctg,
        _selctgsub,
        _selctgh,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _sdim,
        _alpha,
        _beta,
        _vsl,
        _ldvsl,
        _vsr,
        _ldvsr,
        _work,
        _lwork,
        _rwork,
        _bwork,
        _iuser,
        _ruser,
        _info,
        _jobvsl._length_,
        _jobvsr._length_,
        _sort._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == _n_py+1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed with an unexpected error, please contact NAG.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After reordering, roundoff changed values of some complex eigenvalues so',
                    'that leading eigenvalues in the generalized Schur form no longer satisfy',
                    'selctg = True. This could also be caused by underflow due to scaling.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == _n_py+3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The eigenvalues could not be reordered because some eigenvalues were too',
                    'close to separate (the problem is very ill-conditioned).',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py < _n_py:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration did not converge and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _vsl_cls.refresh_py(_vsl)
        _vsr_cls.refresh_py(_vsr)
        _sdim_py = utils._EngineIntScalarType.to_py(_sdim)
        return_data = _sdim_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['sdim'],
            )(
                _sdim_py,
            )
    _err_class.manage()
    return return_data

def zhgeqz(job, compq, compz, n, ilo, ihi, a, b, alpha, beta, q, z):
    u"""
    Eigenvalues and generalized Schur factorization of complex
    generalized upper Hessenberg form reduced from a pair of complex,
    square, matrices.

    For full information please refer to the NAG Library document for
    f08xs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08xsf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on (A,B).

        `job` = 'E'

          The matrix pair (A,B) on exit might not be in the generalized
          Schur form.

        `job` = 'S'

          The matrix pair (A,B) on exit will be in the generalized Schur
          form.

    compq : str, length 1
        Specifies the operations to be performed on Q:

        `compq` = 'N'

          The array `q` is unchanged.

        `compq` = 'V'

          The left transformation Q is accumulated on the array `q`.

        `compq` = 'I'

          The array `q` is initialized to the identity matrix before the
          left transformation Q is accumulated in `q`.

    compz : str, length 1
        Specifies the operations to be performed on Z.

        `compz` = 'N'

          The array `z` is unchanged.

        `compz` = 'V'

          The right transformation Z is accumulated on the array `z`.

        `compz` = 'I'

          The array `z` is initialized to the identity matrix before the
          right transformation Z is accumulated in `z`.

    n : int
        n, the order of the matrices A, B, Q and Z.

    ilo : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``zggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = `n`.

    ihi : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``zggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = `n`.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper Hessenberg matrix A. The elements
        below the first subdiagonal must be set to zero.

        On exit: if `job` = 'S', the matrix pair (A,B) will be
        simultaneously reduced to generalized Schur form.

        If `job` = 'E', the 1 by 1 and 2 by 2 diagonal blocks of the
        matrix pair (A,B) will give generalized eigenvalues but the
        remaining elements will be irrelevant.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n upper triangular matrix B. The elements
        below the diagonal must be zero.

        On exit: if `job` = 'S', the matrix pair (A,B) will be
        simultaneously reduced to generalized Schur form.

        If `job` = 'E', the 1 by 1 and 2 by 2 diagonal blocks of the
        matrix pair (A,B) will give generalized eigenvalues but the
        remaining elements will be irrelevant.

    alpha : complex, shape (`n`), modified in place
        On exit: alpha_j, for j = 1,2,...,n.

    beta : complex, shape (`n`), modified in place
        On exit: beta_j, for j = 1,2,...,n.

    q : complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('V', 'I'): `n`; if
        `compq`='N': 1; otherwise: 0.

        On entry: if `compq` = 'V', the matrix Q_0. The matrix Q_0 is
        usually the matrix Q returned by ``zgghrd``.

        If `compq` = 'N', `q` is not referenced.

        On exit: if `compq` = 'V', `q` contains the matrix product QQ_0.

        If `compq` = 'I', `q` contains the transformation matrix Q.

    z : complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): `n`; if
        `compz`='N': 1; otherwise: 0.

        On entry: if `compz` = 'V', the matrix Z_0. The matrix Z_0 is
        usually the matrix Z returned by ``zgghrd``.

        If `compz` = 'N', `z` is not referenced.

        On exit: if `compz` = 'V', `z` contains the matrix product ZZ_0.

        If `compz` = 'I', `z` contains the transformation matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= `n`.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i>`n` and i<=2*`n`)
            The computation of shifts failed and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i > 2*`n`)
            An unexpected Library error has occurred.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zhgeqz'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08xsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, compq(len=1).
        utils._BYREF_STRING, # Input, compz(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ilo.
        utils._BYREF_INT, # Input, ihi.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of compz.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'S\', \'s\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _compq = utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = utils._EngineCharScalarType.to_py(_compq)
    if _compq_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compq in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-2,),
            entity='compq',
        ).manage()
    _compz = utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = utils._EngineCharScalarType.to_py(_compz)
    if _compz_py not in ('N', 'n', 'V', 'v', 'I', 'i'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('compz in (\'N\', \'n\', \'V\', \'v\', \'I\', \'i\').'),
            ],
            errcodes=(-3,),
            entity='compz',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _ilo = utils._EngineIntScalarType(
        ilo, {'fun_name': fun_name, 'entity': 'ilo'},
    ).to_ctypes()
    _ihi = utils._EngineIntScalarType(
        ihi, {'fun_name': fun_name, 'entity': 'ihi'},
    ).to_ctypes()
    _ilo_py = utils._EngineIntScalarType.to_py(_ilo)
    _ihi_py = utils._EngineIntScalarType.to_py(_ihi)
    if _n_py > 0:
        if not (_ilo_py >= 1 and _ilo_py <= _ihi_py and _ihi_py <= _n_py):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('1 <= ilo and ilo <= ihi and ihi <= n.'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    elif _n_py == 0:
        if not (_ilo_py == 1 and _ihi_py == 0):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ilo = 1.'),
                ],
                errcodes=(-6,),
                entity='ihi',
            ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _compq_py in ('V', 'v', 'I', 'i'):
        _q_dim2 = _n_py
    elif _compq_py in ('N', 'n'):
        _q_dim2 = 1
    else:
        _q_dim2 = 0
    exp_shape_q = (_n_py, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz_py = utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    if _compz_py in ('V', 'v', 'I', 'i'):
        _z_dim2 = _n_py
    elif _compz_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_n_py, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _compq,
        _compz,
        _n,
        _ilo,
        _ihi,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _q,
        _ldq,
        _z,
        _ldz,
        _work,
        _lwork,
        _rwork,
        _info,
        _job._length_,
        _compq._length_,
        _compz._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py < _n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration did not converge and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > _n_py and _info_py <= 2*_n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation of shifts failed and the matrix pair (A,B) is not in the',
                    'generalized Schur form. The computed alpha_i and beta_i should be correct',
                    'for i = {:s},...,{:s}.'.format(
                        utils._STR_FMT_INT.format(_info_py-_n_py+1),
                        utils._STR_FMT_INT.format(_n_py),
                    ),
                ],
                errcodes=(_info_py,),
            )
        elif _info_py > 2*_n_py:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected Library error has occurred.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        return_data = None
    _err_class.manage()
    return return_data

def dtgsja(jobu, jobv, jobq, m, p, n, k, l, a, b, tola, tolb, alpha, beta, u, v, q):
    u"""
    Computes the generalized singular value decomposition of a real
    upper triangular (or trapezoidal) matrix pair.

    For full information please refer to the NAG Library document for
    f08ye

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yef.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', `u` must contain an orthogonal matrix U_1 on
        entry, and the product U_1 U is returned.

        If `jobu` = 'I', `u` is initialized to the unit matrix, and the
        orthogonal matrix U is returned.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', `v` must contain an orthogonal matrix V_1 on
        entry, and the product V_1 V is returned.

        If `jobv` = 'I', `v` is initialized to the unit matrix, and the
        orthogonal matrix V is returned.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', `q` must contain an orthogonal matrix Q_1 on
        entry, and the product Q_1 Q is returned.

        If `jobq` = 'I', `q` is initialized to the unit matrix, and the
        orthogonal matrix Q is returned.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    k : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``dtgsja``.

    l : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``dtgsja``.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m-k-l >= 0, `a`[0:k+l,n-k-l:n] contains the (k+l) by
        (k+l) upper triangular matrix R.

        If m-k-l < 0, `a`[0:m,n-k-l:n] contains the first m rows of the
        (k+l) by (k+l) upper triangular matrix R, and the submatrix R_33
        is returned in `b`[m-k:l,n+m-k-l:n].

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: if m-k-l < 0, `b`[m-k:l,n+m-k-l:n] contains the
        submatrix R_33 of R.

    tola : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``dggsvp`` or ``dggsvp3``, say

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

    tolb : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``dggsvp`` or ``dggsvp3``, say

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B;

        - `alpha`[i] = 1, `beta`[i] = 0, for i = 0,...,k-1, and

        - if m-k-l >= 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for i
          = k,...,k+l-1, or

        - if m-k-l < 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for i
          = k,...,m-1 and `alpha`[i] = 0, `beta`[i] = 1, for i =
          m,...,k+l-1.

        Furthermore, if k+l < n, `alpha`[i] = `beta`[i] = 0, for i =
        k+l,...,n-1.

    u : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu` in ('U', 'I'): `m`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu` in ('U', 'I'): `m`; otherwise:
        1.

        On entry: if `jobu` = 'U', `u` must contain an m by m matrix U_1
        (usually the orthogonal matrix returned by ``dggsvp`` or
        ``dggsvp3``).

        On exit: if `jobu` = 'U', `u` contains the product U_1 U.

        If `jobu` = 'I', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'I'): `p`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'I'): `p`; otherwise:
        1.

        On entry: if `jobv` = 'V', `v` must contain an p by p matrix V_1
        (usually the orthogonal matrix returned by ``dggsvp`` or
        ``dggsvp3``).

        On exit: if `jobv` = 'I', `v` contains the orthogonal matrix V.

        If `jobv` = 'V', `v` contains the product V_1 V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq` in ('Q', 'I'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq` in ('Q', 'I'): `n`; otherwise:
        1.

        On entry: if `jobq` = 'Q', `q` must contain an n by n matrix Q_1
        (usually the orthogonal matrix returned by ``dggsvp`` or
        ``dggsvp3``).

        On exit: if `jobq` = 'I', `q` contains the orthogonal matrix Q.

        If `jobq` = 'Q', `q` contains the product Q_1 Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    ncycle : int
        The number of cycles required for convergence.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'I' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'I' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q', 'I' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` 1)
            The procedure does not converge after 40 cycles.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgsja'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, ncycle.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u', 'I', 'i'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u', 'I', 'i'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v', 'I', 'i'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'I', 'i'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q', 'I', 'i'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q', 'I', 'i'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ncycle = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _ncycle,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The procedure does not converge after 40 cycles.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _ncycle_py = utils._EngineIntScalarType.to_py(_ncycle)
        return_data = _ncycle_py
    _err_class.manage()
    return return_data

def dtgexc(wantq, wantz, n, a, b, q, z, ifst, ilst):
    u"""
    Reorders the generalized real Schur decomposition of a real matrix
    pair using an orthogonal equivalence transformation.

    For full information please refer to the NAG Library document for
    f08yf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yff.html

    Parameters
    ----------
    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    n : int
        n, the order of the matrices S and T.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix S in the pair (S,T).

        On exit: the updated matrix S^.

    b : float, shape (`n`, `n`), modified in place
        On entry: the matrix T, in the pair (S,T).

        On exit: the updated matrix T^

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On entry: if `wantq` = True, the orthogonal matrix Q.

        On exit: if `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        On entry: if `wantz` = True, the orthogonal matrix Z.

        On exit: if `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    ifst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal blocks of (S,T). The block with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        blocks.

    ilst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal blocks of (S,T). The block with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        blocks.

    Returns
    -------
    ifst : int
        If `ifst` pointed on entry to the second row of a 2 by 2 block,
        it is changed to point to the first row; `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +1 or -1).

    ilst : int
        If `ifst` pointed on entry to the second row of a 2 by 2 block,
        it is changed to point to the first row; `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +1 or -1).

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -13)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ifst` <= `n` and 1 <= `ilst` <= `n`.

        (`errno` 1)
            The transformed matrix pair would be too far from
            generalized Schur form; the problem is ill-conditioned.
            (S,T) may have been partially reordered, and `ilst` points
            to the first row of the current position of the block being
            moved.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgexc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._BYREF_LOGICAL, # Input, wantz.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Input/Output, ifst.
        utils._BYREF_INT, # Input/Output, ilst.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _wantz = utils._EngineLogicalScalarType(
        wantz, {'fun_name': fun_name, 'entity': 'wantz'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _wantz_py = utils._EngineLogicalScalarType.to_py(_wantz)
    if _wantz_py:
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _wantz_py:
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _ifst = utils._EngineIntScalarType(
        ifst, {'fun_name': fun_name, 'entity': 'ifst'},
    ).to_ctypes()
    _ilst = utils._EngineIntScalarType(
        ilst, {'fun_name': fun_name, 'entity': 'ilst'},
    ).to_ctypes()
    _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
    _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
    if not ((_ifst_py >= 1 and _ifst_py <= _n_py) and (_ilst_py >= 1 and _ilst_py <= _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ifst and ifst <= n.'),
            ],
            errcodes=(-13,),
            entity='ilst',
        ).manage()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _wantq,
        _wantz,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _ifst,
        _ilst,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The transformed matrix pair would be too far from generalized Schur form;',
                    'the problem is ill-conditioned. (S,T) may have been partially reordered, and',
                    'ilst points to the first row of the current position of the block being',
                    'moved.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
        _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ifst', 'ilst'],
        )(
            _ifst_py,
            _ilst_py,
        )
    _err_class.manage()
    return return_data

def dtgsen(ijob, wantq, wantz, select, n, a, b, alphar, alphai, beta, q, z, dif):
    u"""
    Reorders the generalized real Schur decomposition of a real matrix
    pair using an orthogonal equivalence transformation, computes the
    generalized eigenvalues of the reordered pair and, optionally,
    computes the estimates of reciprocal condition numbers for
    eigenvalues and eigenspaces.

    For full information please refer to the NAG Library document for
    f08yg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ygf.html

    Parameters
    ----------
    ijob : int
        Specifies whether condition numbers are required for the cluster
        of eigenvalues (p and q) or the deflating subspaces (Dif_u and
        Dif_l).

        `ijob` = 0

          Only reorder with respect to `select`. No extras.

        `ijob` = 1

          Reciprocal of norms of 'projections' onto left and right
          eigenspaces with respect to the selected cluster (p and q).

        `ijob` = 2

          The upper bounds on Dif_u and Dif_l. F-norm-based estimate
          (`dif`[0:2]).

        `ijob` = 3

          Estimate of Dif_u and Dif_l. 1-norm-based estimate
          (`dif`[0:2]). About five times as expensive as `ijob` = 2.

        `ijob` = 4

          Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 2.
          Economic version to get it all.

        `ijob` = 5

          Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 3.

    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    select : bool, shape (`n`)
        Specifies the eigenvalues in the selected cluster. To select a
        real eigenvalue lambda_j, `select`[j-1] must be set to True.

        To select a complex conjugate pair of eigenvalues lambda_j and
        lambda_{j+1}, corresponding to a 2 by 2 diagonal block, either
        `select`[j-1] or `select`[j] or both must be set to True; a
        complex conjugate pair of eigenvalues must be either both
        included in the cluster or both excluded.

    n : int
        n, the order of the matrices S and T.

    a : float, shape (`n`, `n`), modified in place
        On entry: the matrix S in the pair (S,T).

        On exit: the updated matrix S^.

    b : float, shape (`n`, `n`), modified in place
        On entry: the matrix T, in the pair (S,T).

        On exit: the updated matrix T^

    alphar : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    alphai : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alphar`[j-1]/`beta`[j-1] and `alphai`[j-1]/`beta`[j-1]
        are the real and imaginary parts respectively of the jth
        eigenvalue, for j = 1,2,...,`n`.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive then `alphai`[j] is negative, and the jth and (j+1)st
        eigenvalues are a complex conjugate pair.

        Conjugate pairs of eigenvalues correspond to the 2 by 2 diagonal
        blocks of S^.

        These 2 by 2 blocks can be reduced by applying complex unitary
        transformations to (S^,T^) to obtain the complex Schur form
        (S~,T~), where S~ is triangular (and complex).

        In this form `alphar`+i`alphai` and `beta` are the diagonals of
        S~ and T~ respectively.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On entry: if `wantq` = True, the n by n matrix Q.

        On exit: if `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        On entry: if `wantz` = True, the n by n matrix Z.

        On exit: if `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    dif : float, shape (2), modified in place
        On exit: if `ijob` >= 2, `dif`[0:2] store the estimates of Dif_u
        and Dif_l.

        If `ijob` = 2 or 4, `dif`[0:2] are F-norm-based upper bounds on
        Dif_u and Dif_l.

        If `ijob` = 3 or 5, `dif`[0:2] are 1-norm-based estimates of
        Dif_u and Dif_l.

        If `m` = 0 or n, `dif`[0:2] = ||(A,B)||_F.

        If `ijob` = 0 or 1, `dif` is not referenced.

    Returns
    -------
    m : int
        The dimension of the specified pair of left and right
        eigenspaces (deflating subspaces).

    pl : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspaces with respect to the selected cluster. 0 <
        `pl`, `pr` <= 1.

        If `m` = 0 or `m` = `n`, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    pr : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspaces with respect to the selected cluster. 0 <
        `pl`, `pr` <= 1.

        If `m` = 0 or `m` = `n`, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 5.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` 1)
            Reordering of (S,T) failed because the transformed matrix
            pair would be too far from generalized Schur form; the
            problem is very ill-conditioned. (S,T) may have been
            partially reordered. If requested, 0 is returned in `dif`[0]
            and `dif`[1], `pl` and `pr`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgsen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ygft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ijob.
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._BYREF_LOGICAL, # Input, wantz.
        utils._EngineLogicalArrayType.c_type, # Input, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Output, m.
        utils._BYREF_FLOAT64, # Output, pl.
        utils._BYREF_FLOAT64, # Output, pr.
        utils._EngineFloat64ArrayType.c_type, # Output, dif[2].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ijob = utils._EngineIntScalarType(
        ijob, {'fun_name': fun_name, 'entity': 'ijob'},
    ).to_ctypes()
    _ijob_py = utils._EngineIntScalarType.to_py(_ijob)
    if not (_ijob_py >= 0 and _ijob_py <= 5):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ijob >= 0 and ijob <= 5.'),
            ],
            errcodes=(-1,),
            entity='ijob',
        ).manage()
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _wantz = utils._EngineLogicalScalarType(
        wantz, {'fun_name': fun_name, 'entity': 'wantz'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _wantz_py = utils._EngineLogicalScalarType.to_py(_wantz)
    if _wantz_py:
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _wantz_py:
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _pl = utils._EngineFloat64ScalarType.empty_ctypes()
    _pr = utils._EngineFloat64ScalarType.empty_ctypes()
    _dif_cls = utils._EngineFloat64ArrayType(
        dif, {'fun_name': fun_name, 'entity': 'dif'},
        exp_shape=(2,),
        check_contents=False,
    )
    _dif = _dif_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _ijob,
        _wantq,
        _wantz,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alphar,
        _alphai,
        _beta,
        _q,
        _ldq,
        _z,
        _ldz,
        _m,
        _pl,
        _pr,
        _dif,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Reordering of (S,T) failed because the transformed matrix pair would be too',
                    'far from generalized Schur form; the problem is very ill-conditioned.',
                    '(S,T) may have been partially reordered. If requested, 0 is returned in',
                    'dif[0] and dif[1], pl and pr.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        _dif_cls.refresh_py(_dif)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _pl_py = utils._EngineFloat64ScalarType.to_py(_pl)
        _pr_py = utils._EngineFloat64ScalarType.to_py(_pr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 'pl', 'pr'],
        )(
            _m_py,
            _pl_py,
            _pr_py,
        )
    _err_class.manage()
    return return_data

def dtgsyl(trans, ijob, m, n, a, b, c, d, e, f):
    u"""
    Solves the real-valued, generalized, quasi-trangular, Sylvester
    equation.

    For full information please refer to the NAG Library document for
    f08yh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yhf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', solve the generalized Sylvester equation (1).

        If `trans` = 'T', solve the 'transposed' system (2).

    ijob : int
        Specifies what kind of functionality is to be performed when
        `trans` = 'N'.

        `ijob` = 0

          Solve (1) only.

        `ijob` = 1

          The functionality of `ijob` = 0 or 3.

        `ijob` = 2

          The functionality of `ijob` = 0 or 4.

        `ijob` = 3

          Only an estimate of Dif[(A,D),(B,E)] is computed based on the
          Frobenius norm.

        `ijob` = 4

          Only an estimate of Dif[(A,D),(B,E)] is computed based on the
          1-norm.

        If `trans` = 'T', `ijob` is not referenced.

    m : int
        m, the order of the matrices A and D, and the row dimension of
        the matrices C, F, R and L.

    n : int
        n, the order of the matrices B and E, and the column dimension
        of the matrices C, F, R and L.

    a : float, shape (`m`, `m`)
        The upper quasi-triangular matrix A.

    b : float, shape (`n`, `n`)
        The upper quasi-triangular matrix B.

    c : float, shape (`m`, `n`), modified in place
        On entry: contains the right-hand-side matrix C.

        On exit: if `ijob` = 0, 1 or 2, `c` is overwritten by the
        solution matrix R.

        If `trans` = 'N' and `ijob` = 3 or 4, `c` holds R, the solution
        achieved during the computation of the Dif estimate.

    d : float, shape (`m`, `m`)
        The upper triangular matrix D.

    e : float, shape (`n`, `n`)
        The upper triangular matrix E.

    f : float, shape (`m`, `n`), modified in place
        On entry: contains the right-hand side matrix F.

        On exit: if `ijob` = 0, 1 or 2, `f` is overwritten by the
        solution matrix L.

        If `trans` = 'N' and `ijob` = 3 or 4, `f` holds L, the solution
        achieved during the computation of the Dif estimate.

    Returns
    -------
    scale : float
        alpha, the scaling factor in (1) and (2).

        If 0 < `scale` < 1, `c` and `f` hold the solutions R and L,
        respectively, to a slightly perturbed system but the input
        arrays `a`, `b`, `d` and `e` have not been changed.

        If `scale` = 0, `c` and `f` hold the solutions R and L,
        respectively, to the homogeneous system with C = F = 0.

        In this case `dif` is not referenced.

        Normally, `scale` = 1.

    dif : float
        The estimate of Dif. If `ijob` = 0, this argument is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 4.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` > 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` > 0.

        (`errno` i > 0)
            (A,D) and (B,E) have common or close eigenvalues and so no
            solution could be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgsyl'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, ijob.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Input, d[ldd, m].
        utils._BYREF_INT, # Input, ldd.
        utils._EngineFloat64ArrayType.c_type, # Input, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, f[ldf, n].
        utils._BYREF_INT, # Input, ldf.
        utils._BYREF_FLOAT64, # Output, scale.
        utils._BYREF_FLOAT64, # Output, dif.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (f, 'f')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'T', 't'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'T\', \'t\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _ijob = utils._EngineIntScalarType(
        ijob, {'fun_name': fun_name, 'entity': 'ijob'},
    ).to_ctypes()
    _ijob_py = utils._EngineIntScalarType.to_py(_ijob)
    if _trans_py in ('N', 'n'):
        if not (_ijob_py >= 0 and _ijob_py <= 4):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ijob >= 0 and ijob <= 4.'),
                ],
                errcodes=(-2,),
                entity='ijob',
            ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py > 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m > 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py > 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n > 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _ldd = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lde = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldf'},
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _f = _f_cls.to_ctypes()
    _scale = utils._EngineFloat64ScalarType.empty_ctypes()
    _dif = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _ijob,
        _m,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _d,
        _ldd,
        _e,
        _lde,
        _f,
        _ldf,
        _scale,
        _dif,
        _work,
        _lwork,
        _iwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    '(A,D) and (B,E) have common or close eigenvalues and so no solution could',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _f_cls.refresh_py(_f)
        _scale_py = utils._EngineFloat64ScalarType.to_py(_scale)
        _dif_py = utils._EngineFloat64ScalarType.to_py(_dif)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scale', 'dif'],
        )(
            _scale_py,
            _dif_py,
        )
    _err_class.manage()
    return return_data

def dtgevc(side, howmny, select, n, a, b, vl, vr, mm):
    u"""
    Computes right and left generalized eigenvectors of the matrix pair
    (A,B) which is assumed to be in generalized upper Schur form.

    For full information please refer to the NAG Library document for
    f08yk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ykf.html

    Parameters
    ----------
    side : str, length 1
        Specifies the required sets of generalized eigenvectors.

        `side` = 'R'

          Only right eigenvectors are computed.

        `side` = 'L'

          Only left eigenvectors are computed.

        `side` = 'B'

          Both left and right eigenvectors are computed.

    howmny : str, length 1
        Specifies further details of the required generalized
        eigenvectors.

        `howmny` = 'A'

          All right and/or left eigenvectors are computed.

        `howmny` = 'B'

          All right and/or left eigenvectors are computed; they are
          backtransformed using the input matrices supplied in arrays
          `vr` and/or `vl`.

        `howmny` = 'S'

          Selected right and/or left eigenvectors, defined by the array
          `select`, are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenvectors to be computed if `howmny` = 'S'. To
        select the generalized eigenvector corresponding to the jth
        generalized eigenvalue, the jth element of `select` should be
        set to True; if the eigenvalue corresponds to a complex
        conjugate pair, then real and imaginary parts of eigenvectors
        corresponding to the complex conjugate eigenvalue pair will be
        computed.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    n : int
        n, the order of the matrices A and B.

    a : float, shape (`n`, `n`)
        The matrix pair (A,B) must be in the generalized Schur form.
        Usually, this is the matrix A returned by ``dhgeqz``.

    b : float, shape (`n`, `n`)
        The matrix pair (A,B) must be in the generalized Schur form. If
        A has a 2 by 2 diagonal block then the corresponding 2 by 2
        block of B must be diagonal with positive elements. Usually,
        this is the matrix B returned by ``dhgeqz``.

    vl : None or float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('L', 'B'): `mm`; if
        `side`='R': 1; otherwise: 0.

        Optionally, on entry: if `howmny` = 'B' and `side` = 'L' or 'B',
        `vl` must be initialized to an n by n matrix Q. Usually, this is
        the orthogonal matrix Q of left Schur vectors returned by
        ``dhgeqz``.

        On exit, if not None on entry: if `side` = 'L' or 'B', `vl`
        contains:

        - if `howmny` = 'A', the matrix Y of left eigenvectors of (A,B);

        - if `howmny` = 'B', the matrix QY;

        - if `howmny` = 'S', the left eigenvectors of (A,B) specified by
          `select`, stored consecutively in the columns of the array
          `vl`, in the same order as their corresponding eigenvalues.

        A complex eigenvector corresponding to a complex eigenvalue is
        stored in two consecutive columns, the first holding the real
        part, and the second the imaginary part.

        If `side` = 'R', `vl` is not referenced.

    vr : None or float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('R', 'B'): `mm`; if
        `side`='L': 1; otherwise: 0.

        Optionally, on entry: if `howmny` = 'B' and `side` = 'R' or 'B',
        `vr` must be initialized to an n by n matrix Z. Usually, this is
        the orthogonal matrix Z of right Schur vectors returned by
        ``dhgeqz``.

        On exit, if not None on entry: if `side` = 'R' or 'B', `vr`
        contains:

        - if `howmny` = 'A', the matrix X of right eigenvectors of
          (A,B);

        - if `howmny` = 'B', the matrix ZX;

        - if `howmny` = 'S', the right eigenvectors of (A,B) specified
          by `select`, stored consecutively in the columns of the array
          `vr`, in the same order as their corresponding eigenvalues.

        A complex eigenvector corresponding to a complex eigenvalue is
        stored in two consecutive columns, the first holding the real
        part, and the second the imaginary part.

        If `side` = 'L', `vr` is not referenced.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`.

    Returns
    -------
    m : int
        The number of columns in the arrays `vl` and/or `vr` actually
        used to store the eigenvectors. If `howmny` = 'A' or 'B', `m` is
        set to `n`. Each selected real eigenvector occupies one column
        and each selected complex eigenvector occupies two columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'B', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B' or 'S'.

        (`errno` -3)
            On entry, error in parameter `select`.

            Constraint: `select`[j] = True or False, for j = 0,...,n-1.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `a`.

        (`errno` -7)
            On entry, error in parameter `b`.

        (`errno` -13)
            On entry, error in parameter `mm`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The 2 by 2 block (*<value>*:*<value>*+1) does not have
            complex eigenvalues.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgevc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ykft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _vl_ilabad_py = False
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    if utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py and utils._EngineFloat64ArrayType.get_shape(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0] >= _n_py:
        _vl_ilbbad_py = False
        for _vl_j_py in range(1, _n_py-1+1):
            if utils._EngineFloat64ArrayType.get_element(a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_j_py+1)-1, _vl_j_py-1)) != 0:
                if _vl_j_py < _n_py-1:
                    if utils._EngineFloat64ArrayType.get_element(a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_j_py+2)-1, (_vl_j_py+1)-1)) != 0:
                        _vl_ilabad_py = True
                        break
                if utils._EngineFloat64ArrayType.get_element(b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, _vl_j_py-1)) == 0 or utils._EngineFloat64ArrayType.get_element(b, {'fun_name': fun_name, 'entity': 'b'}, ((_vl_j_py+1)-1, (_vl_j_py+1)-1)) == 0 or utils._EngineFloat64ArrayType.get_element(b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, (_vl_j_py+1)-1)) != 0:
                    _vl_ilbbad_py = True
                    break
        _vl_ilbbad = utils._EngineLogicalScalarType(
            _vl_ilbbad_py, {'fun_name': fun_name, 'entity': '_vl_ilbbad_py'},
        ).to_ctypes()
    else:
        _vl_ilbbad_py = False
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'B', 'b', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'B\', \'b\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _vl_k_py = 0
    if (_howmny_py in ('S', 's')) and utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if utils._EngineFloat64ArrayType.get_element(a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('B', 'b', 'L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'B\', \'b\', \'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    if _howmny_py not in ('A', 'a', 'B', 'b'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\', \'B\', \'b\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a', 'B', 'b'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a', 'B', 'b'))
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    if not not _vl_ilabad_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-5,),
            entity='a',
        ).manage()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    if not not _vl_ilbbad_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-7,),
            entity='b',
        ).manage()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if not _mm_py >= _vl_m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-13,),
            entity='mm',
        ).manage()
    if (_howmny_py in ('B', 'b')) and _side_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and side in (\'L\', \'l\', \'B\', \'b\')',
            )
    if _side_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when side in (\'L\', \'l\', \'B\', \'b\')',
            )
    if _side_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = _mm_py
    elif _side_py in ('R', 'r'):
        _vl_dim2 = 1
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_n_py, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _side_py in ('L', 'l', 'B', 'b'),
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _side_py in ('L', 'l', 'B', 'b'))
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if (_howmny_py in ('B', 'b')) and _side_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and side in (\'R\', \'r\', \'B\', \'b\')',
            )
    if _side_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when side in (\'R\', \'r\', \'B\', \'b\')',
            )
    if _side_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = _mm_py
    elif _side_py in ('L', 'l'):
        _vr_dim2 = 1
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_n_py, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _side_py in ('R', 'r', 'B', 'b'),
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _side_py in ('R', 'r', 'B', 'b'))
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _howmny,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _info,
        _side._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The 2 by 2 block ({:s}:{:s}+1) does'.format(
                        utils._STR_FMT_INT.format(_info_py),
                        utils._STR_FMT_INT.format(_info_py),
                    ),
                    'not have complex eigenvalues.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def dtgsna(job, howmny, select, n, a, b, vl, vr, s, dif, mm):
    u"""
    Estimates reciprocal condition numbers for specified eigenvalues
    and/or eigenvectors of a real matrix pair in generalized real Schur
    canonical form.

    For full information please refer to the NAG Library document for
    f08yl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ylf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

          Condition numbers for eigenvalues only are computed.

        `job` = 'V'

          Condition numbers for eigenvectors only are computed.

        `job` = 'B'

          Condition numbers for both eigenvalues and eigenvectors are
          computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

          Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

          Condition numbers for selected eigenpairs (as specified by
          `select`) are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the real eigenvalue lambda_j,
        `select`[j-1] must be set True. To select condition numbers
        corresponding to a complex conjugate pair of eigenvalues
        lambda_j and lambda_{j+1}, `select`[j-1] and/or `select`[j] must
        be set to True.

        If `howmny` = 'A', `select` is not referenced.

    n : int
        n, the order of the matrix pair (S,T).

    a : float, shape (`n`, `n`)
        The upper quasi-triangular matrix S.

    b : float, shape (`n`, `n`)
        The upper triangular matrix T.

    vl : None or float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain left eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vl`, as returned by ``dggev`` or ``dtgevc``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or float, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain right eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vr`, as returned by ``dggev`` or ``dtgevc``.

        If `job` = 'V', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('E', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: if `job` = 'E' or 'B', the
        reciprocal condition numbers of the selected eigenvalues, stored
        in consecutive elements of the array. For a complex conjugate
        pair of eigenvalues two consecutive elements of `s` are set to
        the same value. Thus `s`[j-1], `dif`[j-1], and the jth columns
        of VL and VR all correspond to the same eigenpair (but not in
        general the jth eigenpair, unless all eigenpairs are selected).

        If `job` = 'V', `s` is not referenced.

    dif : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('V', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: if `job` = 'V' or 'B', the
        estimated reciprocal condition numbers of the selected
        eigenvectors, stored in consecutive elements of the array. For a
        complex eigenvector two consecutive elements of `dif` are set to
        the same value. If the eigenvalues cannot be reordered to
        compute `dif`[j-1], `dif`[j-1] is set to 0; this can only occur
        when the true value would be very small anyway.

        If `job` = 'E', `dif` is not referenced.

    mm : int
        The number of elements in the arrays `s` and `dif`.

    Returns
    -------
    m : int
        M, the number of elements of the arrays `s` and `dif` used to
        store the specified condition numbers; for each selected real
        eigenvalue one element is used, and for each selected complex
        conjugate pair of eigenvalues, two elements are used. If
        `howmny` = 'A', `m` is set to `n`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= `n`.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dtgsna'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ylft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Input, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, dif[*].
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _vl_k_py = 0
    if (_howmny_py in ('S', 's')) and utils._EngineFloat64ArrayType.get_shape(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if utils._EngineFloat64ArrayType.get_element(a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    if _howmny_py not in ('A', 'a'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a'))
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if _howmny_py in ('A', 'a'):
        if not _mm_py >= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= n.'),
                ],
                errcodes=(-15,),
                entity='mm',
            ).manage()
    else:
        if not _mm_py >= _vl_m_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= m.'),
                ],
                errcodes=(-15,),
                entity='mm',
            ).manage()
    if _job_py not in ('V', 'v'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _job_py not in ('V', 'v'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when job not in (\'V\', \'v\')',
            )
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=_job_py not in ('V', 'v'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_job_py not in ('V', 'v'))
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _job_py in ('V', 'v', 'B', 'b'):
        _dif_dim1 = _mm_py
    else:
        _dif_dim1 = 0
    exp_shape_dif = (_dif_dim1,)
    _dif_cls = utils._EngineFloat64ArrayType(
        dif, {'fun_name': fun_name, 'entity': 'dif'},
        exp_shape=exp_shape_dif,
        check_contents=False,
        allow_none=True,
    )
    _dif = _dif_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _dif,
        _mm,
        _m,
        _work,
        _lwork,
        _iwork,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if s is not None:
            _s_cls.refresh_py(_s)
        if dif is not None:
            _dif_cls.refresh_py(_dif)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def ztgsja(jobu, jobv, jobq, m, p, n, k, l, a, b, tola, tolb, alpha, beta, u, v, q):
    u"""
    Computes the generalized singular value decomposition of a complex
    upper triangular (or trapezoidal) matrix pair.

    For full information please refer to the NAG Library document for
    f08ys

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ysf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', `u` must contain a unitary matrix U_1 on entry,
        and the product U_1 U is returned.

        If `jobu` = 'I', `u` is initialized to the unit matrix, and the
        unitary matrix U is returned.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', `v` must contain a unitary matrix V_1 on entry,
        and the product V_1 V is returned.

        If `jobv` = 'I', `v` is initialized to the unit matrix, and the
        unitary matrix V is returned.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', `q` must contain a unitary matrix Q_1 on entry,
        and the product Q_1 Q is returned.

        If `jobq` = 'I', `q` is initialized to the unit matrix, and the
        unitary matrix Q is returned.

        If `jobq` = 'N', Q is not computed.

    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    k : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``ztgsja``.

    l : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``ztgsja``.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m-k-l >= 0, `a`[0:k+l,n-k-l:n] contains the (k+l) by
        (k+l) upper triangular matrix R.

        If m-k-l < 0, `a`[0:m,n-k-l:n] contains the first m rows of the
        (k+l) by (k+l) upper triangular matrix R, and the submatrix R_33
        is returned in `b`[m-k:l,n+m-k-l:n].

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: if m-k-l < 0, `b`[m-k:l,n+m-k-l:n] contains the
        submatrix R_33 of R.

    tola : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``zggsvp`` or ``zggsvp3``, say

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

    tolb : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``zggsvp`` or ``zggsvp3``, say

            `tola` = max(`m`, `n`)||A||epsilon,;`tolb` = max(`p`,
            `n`)||B||epsilon,

        where epsilon is the machine precision.

    alpha : float, shape (`n`), modified in place
        On exit: see the description of `beta`.

    beta : float, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain the generalized singular
        value pairs of A and B;

        - `alpha`[i] = 1, `beta`[i] = 0, for i = 0,...,k-1, and

        - if m-k-l >= 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for i
          = k,...,k+l-1, or

        - if m-k-l < 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for i
          = k,...,m-1 and `alpha`[i] = 0, `beta`[i] = 1, for i =
          m,...,k+l-1.

        Furthermore, if k+l < n, `alpha`[i] = `beta`[i] = 0, for i =
        k+l,...,n-1.

    u : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu` in ('U', 'I'): `m`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu` in ('U', 'I'): `m`; otherwise:
        1.

        On entry: if `jobu` = 'U', `u` must contain an m by m matrix U_1
        (usually the unitary matrix returned by ``zggsvp`` or
        ``zggsvp3``).

        On exit: if `jobu` = 'U', `u` contains the product U_1 U.

        If `jobu` = 'I', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'I'): `p`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'I'): `p`; otherwise:
        1.

        On entry: if `jobv` = 'V', `v` must contain an p by p matrix V_1
        (usually the unitary matrix returned by ``zggsvp`` or
        ``zggsvp3``).

        On exit: if `jobv` = 'I', `v` contains the unitary matrix V.

        If `jobv` = 'V', `v` contains the product V_1 V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq` in ('Q', 'I'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq` in ('Q', 'I'): `n`; otherwise:
        1.

        On entry: if `jobq` = 'Q', `q` must contain an n by n matrix Q_1
        (usually the unitary matrix returned by ``zggsvp`` or
        ``zggsvp3``).

        On exit: if `jobq` = 'I', `q` contains the unitary matrix Q.

        If `jobq` = 'Q', `q` contains the product Q_1 Q.

        If `jobq` = 'N', `q` is not referenced.

    Returns
    -------
    ncycle : int
        The number of cycles required for convergence.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'I' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'I' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q', 'I' or 'N'.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -5)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -6)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` 1)
            The procedure does not converge after 40 cycles.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgsja'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ysft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, jobu(len=1).
        utils._BYREF_STRING, # Input, jobv(len=1).
        utils._BYREF_STRING, # Input, jobq(len=1).
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, l.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Input, tola.
        utils._BYREF_FLOAT64, # Input, tolb.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, u[ldu, *].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, v[ldv, *].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Output, ncycle.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobu.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobv.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobq.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (u, 'u'), (v, 'v'), (q, 'q')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _jobu = utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py not in ('U', 'u', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobu in (\'U\', \'u\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-1,),
            entity='jobu',
        ).manage()
    _jobv = utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py not in ('V', 'v', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobv in (\'V\', \'v\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-2,),
            entity='jobv',
        ).manage()
    _jobq = utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py not in ('Q', 'q', 'I', 'i', 'N', 'n'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('jobq in (\'Q\', \'q\', \'I\', \'i\', \'N\', \'n\').'),
            ],
            errcodes=(-3,),
            entity='jobq',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-4,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-5,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-6,),
            entity='n',
        ).manage()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _tola = utils._EngineFloat64ScalarType(
        tola, {'fun_name': fun_name, 'entity': 'tola'},
    ).to_ctypes()
    _tolb = utils._EngineFloat64ScalarType(
        tolb, {'fun_name': fun_name, 'entity': 'tolb'},
    ).to_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldu = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    if _jobu_py in ('U', 'u', 'I', 'i'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u', 'I', 'i'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u_cls = utils._EngineComplex128ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=exp_shape_u,
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    if _jobv_py in ('V', 'v', 'I', 'i'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'I', 'i'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        mutual_sorder=_m_sorder,
    )
    _v = _v_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    if _jobq_py in ('Q', 'q', 'I', 'i'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q', 'I', 'i'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _ncycle = utils._EngineIntScalarType.empty_ctypes()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _jobu,
        _jobv,
        _jobq,
        _m,
        _p,
        _n,
        _k,
        _l,
        _a,
        _lda,
        _b,
        _ldb,
        _tola,
        _tolb,
        _alpha,
        _beta,
        _u,
        _ldu,
        _v,
        _ldv,
        _q,
        _ldq,
        _work,
        _ncycle,
        _info,
        _jobu._length_,
        _jobv._length_,
        _jobq._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The procedure does not converge after 40 cycles.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _q_cls.refresh_py(_q)
        _ncycle_py = utils._EngineIntScalarType.to_py(_ncycle)
        return_data = _ncycle_py
    _err_class.manage()
    return return_data

def ztgexc(wantq, wantz, n, a, b, q, z, ifst, ilst):
    u"""
    Reorders the generalized Schur decomposition of a complex matrix
    pair using an unitary equivalence transformation.

    For full information please refer to the NAG Library document for
    f08yt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ytf.html

    Parameters
    ----------
    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    n : int
        n, the order of the matrices S and T.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix S in the pair (S,T).

        On exit: the updated matrix S^.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the matrix T, in the pair (S,T).

        On exit: the updated matrix T^

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On entry: if `wantq` = True, the unitary matrix Q.

        On exit: if `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        On entry: if `wantz` = True, the unitary matrix Z.

        On exit: if `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    ifst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal elements of (S,T). The element with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        diagonal elements.

    ilst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal elements of (S,T). The element with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        diagonal elements.

    Returns
    -------
    ilst : int
        `ilst` points to the row in its final position.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -13)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ifst` <= `n` and 1 <= `ilst` <= `n`.

        (`errno` 1)
            The transformed matrix pair would be too far from
            generalized Schur form; the problem is ill-conditioned.
            (S,T) may have been partially reordered, and `ilst` points
            to the first row of the current position of the block being
            moved.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgexc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ytft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._BYREF_LOGICAL, # Input, wantz.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Input, ifst.
        utils._BYREF_INT, # Input/Output, ilst.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _wantz = utils._EngineLogicalScalarType(
        wantz, {'fun_name': fun_name, 'entity': 'wantz'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _wantz_py = utils._EngineLogicalScalarType.to_py(_wantz)
    if _wantz_py:
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _wantz_py:
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _ifst = utils._EngineIntScalarType(
        ifst, {'fun_name': fun_name, 'entity': 'ifst'},
    ).to_ctypes()
    _ilst = utils._EngineIntScalarType(
        ilst, {'fun_name': fun_name, 'entity': 'ilst'},
    ).to_ctypes()
    _ifst_py = utils._EngineIntScalarType.to_py(_ifst)
    _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
    if not ((_ifst_py >= 1 and _ifst_py <= _n_py) and (_ilst_py >= 1 and _ilst_py <= _n_py)):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('1 <= ifst and ifst <= n.'),
            ],
            errcodes=(-13,),
            entity='ilst',
        ).manage()
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _wantq,
        _wantz,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _q,
        _ldq,
        _z,
        _ldz,
        _ifst,
        _ilst,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The transformed matrix pair would be too far from generalized Schur form;',
                    'the problem is ill-conditioned. (S,T) may have been partially reordered, and',
                    'ilst points to the first row of the current position of the block being',
                    'moved.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        _ilst_py = utils._EngineIntScalarType.to_py(_ilst)
        return_data = _ilst_py
    _err_class.manage()
    return return_data

def ztgsen(ijob, wantq, wantz, select, n, a, b, alpha, beta, q, z, dif):
    u"""
    Reorders the generalized Schur decomposition of a complex matrix
    pair using an unitary equivalence transformation, computes the
    generalized eigenvalues of the reordered pair and, optionally,
    computes the estimates of reciprocal condition numbers for
    eigenvalues and eigenspaces.

    For full information please refer to the NAG Library document for
    f08yu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yuf.html

    Parameters
    ----------
    ijob : int
        Specifies whether condition numbers are required for the cluster
        of eigenvalues (p and q) or the deflating subspaces (Dif_u and
        Dif_l).

        `ijob` = 0

          Only reorder with respect to `select`. No extras.

        `ijob` = 1

          Reciprocal of norms of 'projections' onto left and right
          eigenspaces with respect to the selected cluster (p and q).

        `ijob` = 2

          The upper bounds on Dif_u and Dif_l. F-norm-based estimate
          (`dif`[0:2]).

        `ijob` = 3

          Estimate of Dif_u and Dif_l. 1-norm-based estimate
          (`dif`[0:2]). About five times as expensive as `ijob` = 2.

        `ijob` = 4

          Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 2.
          Economic version to get it all.

        `ijob` = 5

          Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 3.

    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    select : bool, shape (`n`)
        Specifies the eigenvalues in the selected cluster. To select an
        eigenvalue lambda_j, `select`[j-1] must be set to True.

    n : int
        n, the order of the matrices S and T.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the matrix S in the pair (S,T).

        On exit: the updated matrix S^.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the matrix T, in the pair (S,T).

        On exit: the updated matrix T^

    alpha : complex, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain diagonal elements of S^ and
        T^, respectively, when the pair (S,T) has been reduced to
        generalized Schur form. `alpha`[i-1]/`beta`[i-1], for i =
        1,2,...,`n`, are the eigenvalues.

    beta : complex, shape (`n`), modified in place
        On exit: `alpha` and `beta` contain diagonal elements of S^ and
        T^, respectively, when the pair (S,T) has been reduced to
        generalized Schur form. `alpha`[i-1]/`beta`[i-1], for i =
        1,2,...,`n`, are the eigenvalues.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On entry: if `wantq` = True, the n by n matrix Q.

        On exit: if `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: `n`; otherwise: 1.

        On entry: if `wantz` = True, the n by n matrix Z.

        On exit: if `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    dif : float, shape (2), modified in place
        On exit: if `ijob` >= 2, `dif`[0:2] store the estimates of Dif_u
        and Dif_l.

        If `ijob` = 2 or 4, `dif`[0:2] are F-norm-based upper bounds on
        Dif_u and Dif_l.

        If `ijob` = 3 or 5, `dif`[0:2] are 1-norm-based estimates of
        Dif_u and Dif_l.

        If `m` = 0 or n, `dif`[0:2] = ||(A,B)||_F.

        If `ijob` = 0 or 1, `dif` is not referenced.

    Returns
    -------
    m : int
        The dimension of the specified pair of left and right
        eigenspaces (deflating subspaces), where `m` will be in the
        range 0 <= `m` <= `n`.

    pl : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspace with respect to the selected cluster. 0 < `pl`,
        `pr` <= 1.

        If `m` = 0 or `m` = `n`, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    pr : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspace with respect to the selected cluster. 0 < `pl`,
        `pr` <= 1.

        If `m` = 0 or `m` = `n`, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 5.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` 1)
            Reordering of (S,T) failed because the transformed matrix
            pair would be too far from generalized Schur form; the
            problem is very ill-conditioned. (S,T) may have been
            partially reordered. If requested, 0 is returned in `dif`[0]
            and `dif`[1], `pl` and `pr`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgsen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ijob.
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._BYREF_LOGICAL, # Input, wantz.
        utils._EngineLogicalArrayType.c_type, # Input, select[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, z[ldz, *].
        utils._BYREF_INT, # Input, ldz.
        utils._BYREF_INT, # Output, m.
        utils._BYREF_FLOAT64, # Output, pl.
        utils._BYREF_FLOAT64, # Output, pr.
        utils._EngineFloat64ArrayType.c_type, # Output, dif[2].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q'), (z, 'z')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ijob = utils._EngineIntScalarType(
        ijob, {'fun_name': fun_name, 'entity': 'ijob'},
    ).to_ctypes()
    _ijob_py = utils._EngineIntScalarType.to_py(_ijob)
    if not (_ijob_py >= 0 and _ijob_py <= 5):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('ijob >= 0 and ijob <= 5.'),
            ],
            errcodes=(-1,),
            entity='ijob',
        ).manage()
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _wantz = utils._EngineLogicalScalarType(
        wantz, {'fun_name': fun_name, 'entity': 'wantz'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-5,),
            entity='n',
        ).manage()
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldq = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
    )
    _q = _q_cls.to_ctypes()
    _ldz = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _wantz_py = utils._EngineLogicalScalarType.to_py(_wantz)
    if _wantz_py:
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _wantz_py:
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z_cls = utils._EngineComplex128ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=exp_shape_z,
        mutual_sorder=_m_sorder,
    )
    _z = _z_cls.to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _pl = utils._EngineFloat64ScalarType.empty_ctypes()
    _pr = utils._EngineFloat64ScalarType.empty_ctypes()
    _dif_cls = utils._EngineFloat64ArrayType(
        dif, {'fun_name': fun_name, 'entity': 'dif'},
        exp_shape=(2,),
        check_contents=False,
    )
    _dif = _dif_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _ijob,
        _wantq,
        _wantz,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _alpha,
        _beta,
        _q,
        _ldq,
        _z,
        _ldz,
        _m,
        _pl,
        _pr,
        _dif,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Reordering of (S,T) failed because the transformed matrix pair would be too',
                    'far from generalized Schur form; the problem is very ill-conditioned.',
                    '(S,T) may have been partially reordered. If requested, 0 is returned in',
                    'dif[0] and dif[1], pl and pr.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        _q_cls.refresh_py(_q)
        _z_cls.refresh_py(_z)
        _dif_cls.refresh_py(_dif)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _pl_py = utils._EngineFloat64ScalarType.to_py(_pl)
        _pr_py = utils._EngineFloat64ScalarType.to_py(_pr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 'pl', 'pr'],
        )(
            _m_py,
            _pl_py,
            _pr_py,
        )
    _err_class.manage()
    return return_data

def ztgsyl(trans, ijob, m, n, a, b, c, d, e, f):
    u"""
    Solves the complex generalized Sylvester equation.

    For full information please refer to the NAG Library document for
    f08yv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yvf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', solve the generalized Sylvester equation (1).

        If `trans` = 'C', solve the 'conjugate transposed' system (2).

    ijob : int
        Specifies what kind of functionality is to be performed when
        `trans` = 'N'.

        `ijob` = 0

          Solve (1) only.

        `ijob` = 1

          The functionality of `ijob` = 0 or 3.

        `ijob` = 2

          The functionality of `ijob` = 0 or 4.

        `ijob` = 3

          Only an estimate of Dif[(A,D),(B,E)] is computed based on the
          Frobenius norm.

        `ijob` = 4

          Only an estimate of Dif[(A,D),(B,E)] is computed based on the
          1-norm.

        If `trans` = 'C', `ijob` is not referenced.

    m : int
        m, the order of the matrices A and D, and the row dimension of
        the matrices C, F, R and L.

    n : int
        n, the order of the matrices B and E, and the column dimension
        of the matrices C, F, R and L.

    a : complex, shape (`m`, `m`)
        The upper triangular matrix A.

    b : complex, shape (`n`, `n`)
        The upper triangular matrix B.

    c : complex, shape (`m`, `n`), modified in place
        On entry: contains the right-hand-side matrix C.

        On exit: if `ijob` = 0, 1 or 2, `c` is overwritten by the
        solution matrix R.

        If `trans` = 'N' and `ijob` = 3 or 4, `c` holds R, the solution
        achieved during the computation of the Dif estimate.

    d : complex, shape (`m`, `m`)
        The upper triangular matrix D.

    e : complex, shape (`n`, `n`)
        The upper triangular matrix E.

    f : complex, shape (`m`, `n`), modified in place
        On entry: contains the right-hand side matrix F.

        On exit: if `ijob` = 0, 1 or 2, `f` is overwritten by the
        solution matrix L.

        If `trans` = 'N' and `ijob` = 3 or 4, `f` holds L, the solution
        achieved during the computation of the Dif estimate.

    Returns
    -------
    scale : float
        alpha, the scaling factor in (1) and (2).

        If 0 < `scale` < 1, `c` and `f` hold the solutions R and L,
        respectively, to a slightly perturbed system but the input
        arrays `a`, `b`, `d` and `e` have not been changed.

        If `scale` = 0, `c` and `f` hold the solutions R and L,
        respectively, to the homogeneous system with C = F = 0.

        In this case `dif` is not referenced.

        Normally, `scale` = 1.

    dif : float
        The estimate of Dif. If `ijob` = 0, this argument is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 4.

        (`errno` -3)
            On entry, error in parameter `m`.

            Constraint: `m` > 0.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` > 0.

        (`errno` i > 0)
            (A,D) and (B,E) have common or close eigenvalues and so no
            solution could be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgsyl'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, ijob.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Input, d[ldd, m].
        utils._BYREF_INT, # Input, ldd.
        utils._EngineComplex128ArrayType.c_type, # Input, e[lde, n].
        utils._BYREF_INT, # Input, lde.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, f[ldf, n].
        utils._BYREF_INT, # Input, ldf.
        utils._BYREF_FLOAT64, # Output, scale.
        utils._BYREF_FLOAT64, # Output, dif.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (d, 'd'), (e, 'e'), (f, 'f')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = utils._EngineCharScalarType.to_py(_trans)
    if _trans_py not in ('N', 'n', 'C', 'c'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('trans in (\'N\', \'n\', \'C\', \'c\').'),
            ],
            errcodes=(-1,),
            entity='trans',
        ).manage()
    _ijob = utils._EngineIntScalarType(
        ijob, {'fun_name': fun_name, 'entity': 'ijob'},
    ).to_ctypes()
    _ijob_py = utils._EngineIntScalarType.to_py(_ijob)
    if _trans_py in ('N', 'n'):
        if not (_ijob_py >= 0 and _ijob_py <= 4):
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('ijob >= 0 and ijob <= 4.'),
                ],
                errcodes=(-2,),
                entity='ijob',
            ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py > 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m > 0.'),
            ],
            errcodes=(-3,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py > 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n > 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _ldd = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    _d = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_m_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lde = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(e, {'fun_name': fun_name, 'entity': 'e'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lde'},
    ).to_ctypes()
    _e = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(f, {'fun_name': fun_name, 'entity': 'f'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldf'},
    ).to_ctypes()
    _f_cls = utils._EngineComplex128ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _f = _f_cls.to_ctypes()
    _scale = utils._EngineFloat64ScalarType.empty_ctypes()
    _dif = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _trans,
        _ijob,
        _m,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _d,
        _ldd,
        _e,
        _lde,
        _f,
        _ldf,
        _scale,
        _dif,
        _work,
        _lwork,
        _iwork,
        _info,
        _trans._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py > 0:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    '(A,D) and (B,E) have common or close eigenvalues and so no solution could',
                    'be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _f_cls.refresh_py(_f)
        _scale_py = utils._EngineFloat64ScalarType.to_py(_scale)
        _dif_py = utils._EngineFloat64ScalarType.to_py(_dif)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['scale', 'dif'],
        )(
            _scale_py,
            _dif_py,
        )
    _err_class.manage()
    return return_data

def ztgevc(side, howmny, select, n, a, b, vl, vr, mm):
    u"""
    Computes left and right eigenvectors of a pair of complex upper
    triangular matrices.

    For full information please refer to the NAG Library document for
    f08yx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yxf.html

    Parameters
    ----------
    side : str, length 1
        Specifies the required sets of generalized eigenvectors.

        `side` = 'R'

          Only right eigenvectors are computed.

        `side` = 'L'

          Only left eigenvectors are computed.

        `side` = 'B'

          Both left and right eigenvectors are computed.

    howmny : str, length 1
        Specifies further details of the required generalized
        eigenvectors.

        `howmny` = 'A'

          All right and/or left eigenvectors are computed.

        `howmny` = 'B'

          All right and/or left eigenvectors are computed; they are
          backtransformed using the input matrices supplied in arrays
          `vr` and/or `vl`.

        `howmny` = 'S'

          Selected right and/or left eigenvectors, defined by the array
          `select`, are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenvectors to be computed if `howmny` = 'S'. To
        select the generalized eigenvector corresponding to the jth
        generalized eigenvalue, the jth element of `select` should be
        set to True.

    n : int
        n, the order of the matrices A and B.

    a : complex, shape (`n`, `n`)
        The matrix A must be in upper triangular form. Usually, this is
        the matrix A returned by ``zhgeqz``.

    b : complex, shape (`n`, `n`)
        The matrix B must be in upper triangular form with non-negative
        real diagonal elements. Usually, this is the matrix B returned
        by ``zhgeqz``.

    vl : None or complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('L', 'B'): `mm`; if
        `side`='R': 1; otherwise: 0.

        Optionally, on entry: if `howmny` = 'B' and `side` = 'L' or 'B',
        `vl` must be initialized to an n by n matrix Q. Usually, this is
        the unitary matrix Q of left Schur vectors returned by
        ``zhgeqz``.

        On exit, if not None on entry: if `side` = 'L' or 'B', `vl`
        contains:

        - if `howmny` = 'A', the matrix Y of left eigenvectors of (A,B);

        - if `howmny` = 'B', the matrix QY;

        - if `howmny` = 'S', the left eigenvectors of (A,B) specified by
          `select`, stored consecutively in the columns of the array
          `vl`, in the same order as their corresponding eigenvalues.

    vr : None or complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('R', 'B'): `mm`; if
        `side`='L': 1; otherwise: 0.

        Optionally, on entry: if `howmny` = 'B' and `side` = 'R' or 'B',
        `vr` must be initialized to an n by n matrix Z. Usually, this is
        the unitary matrix Z of right Schur vectors returned by
        ``dhgeqz``.

        On exit, if not None on entry: if `side` = 'R' or 'B', `vr`
        contains:

        - if `howmny` = 'A', the matrix X of right eigenvectors of
          (A,B);

        - if `howmny` = 'B', the matrix ZX;

        - if `howmny` = 'S', the right eigenvectors of (A,B) specified
          by `select`, stored consecutively in the columns of the array
          `vr`, in the same order as their corresponding eigenvalues.

    mm : int
        The number of columns in the arrays `vl` and/or `vr`.

    Returns
    -------
    m : int
        The number of columns in the arrays `vl` and/or `vr` actually
        used to store the eigenvectors. If `howmny` = 'A' or 'B', `m` is
        set to `n`. Each selected eigenvector occupies one column.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'B', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B' or 'S'.

        (`errno` -3)
            On entry, error in parameter `select`.

            Constraint: `select`[j] = True or False, for j = 0,...,n-1.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `b`.

        (`errno` -13)
            On entry, error in parameter `mm`.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgevc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, side(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of side.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    _vl_m_py = 0
    for _vl_j_py in range(1, _n_py+1):
        if utils._EngineLogicalArrayType.get_element(select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
            _vl_m_py = _vl_m_py + 1
    if utils._EngineComplex128ArrayType.get_shape(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0] >= _n_py:
        _vl_ilbbad_py = False
        for _vl_j_py in range(1, _n_py+1):
            if utils._EngineComplex128ArrayType.get_element(b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, _vl_j_py-1)).imag != 0:
                _vl_ilbbad_py = True
                break
        _vl_ilbbad = utils._EngineLogicalScalarType(
            _vl_ilbbad_py, {'fun_name': fun_name, 'entity': '_vl_ilbbad_py'},
        ).to_ctypes()
    else:
        _vl_ilbbad_py = False
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _side = utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = utils._EngineCharScalarType.to_py(_side)
    if _side_py not in ('B', 'b', 'L', 'l', 'R', 'r'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('side in (\'B\', \'b\', \'L\', \'l\', \'R\', \'r\').'),
            ],
            errcodes=(-1,),
            entity='side',
        ).manage()
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'B', 'b', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'B\', \'b\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    if _howmny_py not in ('A', 'a', 'B', 'b'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\', \'B\', \'b\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a', 'B', 'b'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a', 'B', 'b'))
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    if not not _vl_ilbbad_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-7,),
            entity='b',
        ).manage()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if not _mm_py >= _vl_m_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Argument fails constraint check.'
            ],
            errcodes=(-13,),
            entity='mm',
        ).manage()
    if (_howmny_py in ('B', 'b')) and _side_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and side not in (\'R\', \'r\')',
            )
    if _side_py not in ('R', 'r'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when side not in (\'R\', \'r\')',
            )
    if _side_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = _mm_py
    elif _side_py in ('R', 'r'):
        _vl_dim2 = 1
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_n_py, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _side_py not in ('R', 'r'),
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _side_py not in ('R', 'r'))
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if (_howmny_py in ('B', 'b')) and _side_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and side not in (\'L\', \'l\')',
            )
    if _side_py not in ('L', 'l'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when side not in (\'L\', \'l\')',
            )
    if _side_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = _mm_py
    elif _side_py in ('L', 'l'):
        _vr_dim2 = 1
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_n_py, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=(_howmny_py in ('B', 'b')) and _side_py not in ('L', 'l'),
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=(_howmny_py in ('B', 'b')) and _side_py not in ('L', 'l'))
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _side,
        _howmny,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _mm,
        _m,
        _work,
        _rwork,
        _info,
        _side._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def ztgsna(job, howmny, select, n, a, b, vl, vr, s, dif, mm):
    u"""
    Estimates reciprocal condition numbers for specified eigenvalues
    and/or eigenvectors of a complex matrix pair in generalized Schur
    canonical form.

    For full information please refer to the NAG Library document for
    f08yy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08yyf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

          Condition numbers for eigenvalues only are computed.

        `job` = 'V'

          Condition numbers for eigenvectors only are computed.

        `job` = 'B'

          Condition numbers for both eigenvalues and eigenvectors are
          computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

          Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

          Condition numbers for selected eigenpairs (as specified by
          `select`) are computed.

    select : None or bool, shape (:)
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': `n`; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the eigenvalue lambda_j,
        `select`[j-1] must be set to True.

        If `howmny` = 'A', `select` is not referenced.

    n : int
        n, the order of the matrix pair (S,T).

    a : complex, shape (`n`, `n`)
        The upper triangular matrix S.

    b : complex, shape (`n`, `n`)
        The upper triangular matrix T.

    vl : None or complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain left eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vl`, as returned by ``zggev`` or ``ztgevc``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or complex, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): `n`; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain right eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vr`, as returned by ``zggev`` or ``ztgevc``.

        If `job` = 'V', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('E', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: if `job` = 'E' or 'B', the
        reciprocal condition numbers of the selected eigenvalues, stored
        in consecutive elements of the array.

        If `job` = 'V', `s` is not referenced.

    dif : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `job` in ('V', 'B'): `mm`; otherwise: 0.

        On exit, if not None on entry: if `job` = 'V' or 'B', the
        estimated reciprocal condition numbers of the selected
        eigenvectors, stored in consecutive elements of the array. If
        the eigenvalues cannot be reordered to compute `dif`[j-1],
        `dif`[j-1] is set to 0; this can only occur when the true value
        would be very small anyway.

        If `job` = 'E', `dif` is not referenced.

    mm : int
        The number of elements in the arrays `s` and `dif`.

    Returns
    -------
    m : int
        The number of elements of the arrays `s` and `dif` used to store
        the specified condition numbers; for each selected eigenvalue
        one element is used.

        If `howmny` = 'A', `m` is set to `n`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= `n`.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= the number of selected eigenvalues.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.ztgsna'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08yyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, job(len=1).
        utils._BYREF_STRING, # Input, howmny(len=1).
        utils._EngineLogicalArrayType.c_type, # Input, select[*].
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Input, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, dif[*].
        utils._BYREF_INT, # Input, mm.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Output, info.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of job.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of howmny.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (vl, 'vl'), (vr, 'vr')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('E', 'e', 'V', 'v', 'B', 'b'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('job in (\'E\', \'e\', \'V\', \'v\', \'B\', \'b\').'),
            ],
            errcodes=(-1,),
            entity='job',
        ).manage()
    _howmny = utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = utils._EngineCharScalarType.to_py(_howmny)
    if _howmny_py not in ('A', 'a', 'S', 's'):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('howmny in (\'A\', \'a\', \'S\', \'s\').'),
            ],
            errcodes=(-2,),
            entity='howmny',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-4,),
            entity='n',
        ).manage()
    if _howmny_py not in ('A', 'a'):
        if select is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny not in (\'A\', \'a\')',
            )
    if _howmny_py in ('S', 's'):
        _select_dim1 = _n_py
    else:
        _select_dim1 = 1
    exp_shape_select = (_select_dim1,)
    _select = utils._EngineLogicalArrayType(
        select, {'fun_name': fun_name, 'entity': 'select'},
        exp_shape=exp_shape_select,
        check_contents=_howmny_py not in ('A', 'a'),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_howmny_py not in ('A', 'a'))
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb_py = utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldvl = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _mm = utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = utils._EngineIntScalarType.to_py(_mm)
    if _howmny_py in ('A', 'a'):
        if not _mm_py >= _n_py:
            utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    utils._lapack_constraint_err_text('mm >= n.'),
                ],
                errcodes=(-15,),
                entity='mm',
            ).manage()
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _job_py in ('E', 'e', 'B', 'b'):
        _vl_dim2 = _mm_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        allow_none=True,
    ).to_ctypes()
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _job_py in ('E', 'e', 'B', 'b'):
        _vr_dim2 = _mm_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        allow_none=True,
    ).to_ctypes()
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _job_py in ('V', 'v', 'B', 'b'):
        _dif_dim1 = _mm_py
    else:
        _dif_dim1 = 0
    exp_shape_dif = (_dif_dim1,)
    _dif_cls = utils._EngineFloat64ArrayType(
        dif, {'fun_name': fun_name, 'entity': 'dif'},
        exp_shape=exp_shape_dif,
        check_contents=False,
        allow_none=True,
    )
    _dif = _dif_cls.to_ctypes(can_reference_contents=False)
    _m = utils._EngineIntScalarType.empty_ctypes()
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _job,
        _howmny,
        _select,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _dif,
        _mm,
        _m,
        _work,
        _lwork,
        _iwork,
        _info,
        _job._length_,
        _howmny._length_,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        if s is not None:
            _s_cls.refresh_py(_s)
        if dif is not None:
            _dif_cls.refresh_py(_dif)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
    _err_class.manage()
    return return_data

def dgglse(m, n, p, a, b, c, d, x):
    u"""
    Solves the real linear equality-constrained least squares (LSE)
    problem.

    For full information please refer to the NAG Library document for
    f08za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zaf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` is overwritten.

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: `b` is overwritten.

    c : float, shape (`m`), modified in place
        On entry: the right-hand side vector c for the least squares
        part of the LSE problem.

        On exit: the residual sum of squares for the solution vector x
        is given by the sum of squares of elements
        `c`[`n`-`p`],`c`[`n`-`p`+1],...,`c`[`m`-1]; the remaining
        elements are overwritten.

    d : float, shape (`p`), modified in place
        On entry: the right-hand side vector d for the equality
        constraints.

        On exit: `d` is overwritten.

    x : float, shape (`n`), modified in place
        On exit: the solution vector x of the LSE problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `n` <= `m`+`p`.

        (`errno` 1)
            The upper triangular factor R associated with B in the
            generalized RQ factorization of the pair (B,A) is singular,
            so that rank(B) < p; the least squares solution could not be
            computed.

        (`errno` 2)
            The (N-P) by (N-P) part of the upper trapezoidal factor T
            associated with A in the generalized RQ factorization of the
            pair (B,A) is singular, so that the rank of the matrix (E)
            comprising the rows of A and B is less than n; the least
            squares solutions could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dgglse'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[p].
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not (_p_py >= 0 and _p_py <= _n_py and _n_py <= _m_py+_p_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= p and p <= n and n <= m+p.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py,),
    )
    _c = _c_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_p_py,),
    )
    _d = _d_cls.to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _p,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _d,
        _x,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The upper triangular factor R associated with B in the generalized RQ',
                    'factorization of the pair (B,A) is singular, so that rank(B) < p; the',
                    'least squares solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The (N-P) by (N-P) part of the upper trapezoidal factor T associated with',
                    'A in the generalized RQ factorization of the pair (B,A) is singular, so that',
                    'the rank of the matrix (E) comprising the rows of A and B is less than n;',
                    'the least squares solutions could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _d_cls.refresh_py(_d)
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def dggglm(m, n, p, a, b, d, x, y):
    u"""
    Solves a real general Gauss--Markov linear model (GLM) problem.

    For full information please refer to the NAG Library document for
    f08zb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zbf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrices A and B.

    n : int
        n, the number of columns of the matrix A.

    p : int
        p, the number of columns of the matrix B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` is overwritten.

    b : float, shape (`m`, `p`), modified in place
        On entry: the m by p matrix B.

        On exit: `b` is overwritten.

    d : float, shape (`m`), modified in place
        On entry: the left-hand side vector d of the GLM equation.

        On exit: `d` is overwritten.

    x : float, shape (`n`), modified in place
        On exit: the solution vector x of the GLM problem.

    y : float, shape (`p`), modified in place
        On exit: the solution vector y of the GLM problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: 0 <= `n` <= `m`.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= `m`-`n`.

        (`errno` 1)
            The upper triangular factor R associated with A in the
            generalized RQ factorization of the pair (A,B) is singular,
            so that rank(A) < n; the least squares solution could not be
            computed.

        (`errno` 2)
            The bottom (m-n) by (m-n) part of the upper trapezoidal
            factor T associated with B in the generalized QR
            factorization of the pair (A,B) is singular, so that rank(A
            B) < n; the least squares solutions could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggglm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, p].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[m].
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, y[p].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_n_py >= 0 and _n_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= n and n <= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= _m_py-_n_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= m-n.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _p_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_m_py,),
    )
    _d = _d_cls.to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_p_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _p,
        _a,
        _lda,
        _b,
        _ldb,
        _d,
        _x,
        _y,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The upper triangular factor R associated with A in the generalized RQ',
                    'factorization of the pair (A,B) is singular, so that rank(A) < n;',
                    'the least squares solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The bottom (m-n) by (m-n) part of the upper trapezoidal factor T associated',
                    'with B in the generalized QR factorization of the pair (A,B) is singular, so',
                    'that rank(A B) < n; the least squares solutions could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _d_cls.refresh_py(_d)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def dggqrf(n, m, p, a, taua, b, taub):
    u"""
    Computes a generalized QR factorization of a real matrix pair.

    For full information please refer to the NAG Library document for
    f08ze

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zef.html

    Parameters
    ----------
    n : int
        n, the number of rows of the matrices A and B.

    m : int
        m, the number of columns of the matrix A.

    p : int
        p, the number of columns of the matrix B.

    a : float, shape (`n`, `m`), modified in place
        On entry: the n by m matrix A.

        On exit: the elements on and above the diagonal of the array
        contain the min(n, m) by m upper trapezoidal matrix R (R is
        upper triangular if n >= m); the elements below the diagonal,
        with the array `taua`, represent the orthogonal matrix Q as a
        product of min(n, m) elementary reflectors (see [the F08
        Introduction]).

    taua : float, shape (min(`n`, `m`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the orthogonal matrix Q.

    b : float, shape (`n`, `p`), modified in place
        On entry: the n by p matrix B.

        On exit: if n <= p, the upper triangle of the subarray
        `b`[0:n,p-n:p] contains the n by n upper triangular matrix T_12.

        If n > p, the elements on and above the (n-p)th subdiagonal
        contain the n by p upper trapezoidal matrix T; the remaining
        elements, with the array `taub`, represent the orthogonal matrix
        Z as a product of elementary reflectors (see [the F08
        Introduction]).

    taub : float, shape (min(`n`, `p`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the orthogonal matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggqrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, taua[min(n, m)].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, p].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, taub[min(n, p)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _taua_cls = utils._EngineFloat64ArrayType(
        taua, {'fun_name': fun_name, 'entity': 'taua'},
        exp_shape=(min(_n_py, _m_py),),
        check_contents=False,
    )
    _taua = _taua_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _p_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _taub_cls = utils._EngineFloat64ArrayType(
        taub, {'fun_name': fun_name, 'entity': 'taub'},
        exp_shape=(min(_n_py, _p_py),),
        check_contents=False,
    )
    _taub = _taub_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _m,
        _p,
        _a,
        _lda,
        _taua,
        _b,
        _ldb,
        _taub,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _taua_cls.refresh_py(_taua)
        _b_cls.refresh_py(_b)
        _taub_cls.refresh_py(_taub)
        return_data = None
    _err_class.manage()
    return return_data

def dggrqf(m, p, n, a, taua, b, taub):
    u"""
    Computes a generalized RQ factorization of a real matrix pair.

    For full information please refer to the NAG Library document for
    f08zf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zff.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the upper triangle of the subarray
        `a`[0:m,n-m:n] contains the m by m upper triangular matrix R_12.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m by n upper trapezoidal matrix R; the remaining
        elements, with the array `taua`, represent the orthogonal matrix
        Q as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    taua : float, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the orthogonal matrix Q.

    b : float, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: the elements on and above the diagonal of the array
        contain the min(p, n) by n upper trapezoidal matrix T (T is
        upper triangular if p >= n); the elements below the diagonal,
        with the array `taub`, represent the orthogonal matrix Z as a
        product of elementary reflectors (see [the F08 Introduction]).

    taub : float, shape (min(`p`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the orthogonal matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.dggrqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, taua[min(m, n)].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Output, taub[min(p, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-2,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _taua_cls = utils._EngineFloat64ArrayType(
        taua, {'fun_name': fun_name, 'entity': 'taua'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _taua = _taua_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _taub_cls = utils._EngineFloat64ArrayType(
        taub, {'fun_name': fun_name, 'entity': 'taub'},
        exp_shape=(min(_p_py, _n_py),),
        check_contents=False,
    )
    _taub = _taub_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _taua,
        _b,
        _ldb,
        _taub,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _taua_cls.refresh_py(_taua)
        _b_cls.refresh_py(_b)
        _taub_cls.refresh_py(_taub)
        return_data = None
    _err_class.manage()
    return return_data

def zgglse(m, n, p, a, b, c, d, x):
    u"""
    Solves the complex linear equality-constrained least squares (LSE)
    problem.

    For full information please refer to the NAG Library document for
    f08zn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08znf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrices A and B.

    p : int
        p, the number of rows of the matrix B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` is overwritten.

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: `b` is overwritten.

    c : complex, shape (`m`), modified in place
        On entry: the right-hand side vector c for the least squares
        part of the LSE problem.

        On exit: the residual sum of squares for the solution vector x
        is given by the sum of squares of elements
        `c`[`n`-`p`],`c`[`n`-`p`+1],...,`c`[`m`-1]; the remaining
        elements are overwritten.

    d : complex, shape (`p`), modified in place
        On entry: the right-hand side vector d for the equality
        constraints.

        On exit: `d` is overwritten.

    x : complex, shape (`n`), modified in place
        On exit: the solution vector x of the LSE problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `n` <= `m`+`p`.

        (`errno` 1)
            The upper triangular factor R associated with B in the
            generalized RQ factorization of the pair (B,A) is singular,
            so that rank(B) < p; the least squares solution could not be
            computed.

        (`errno` 2)
            The (N-P) by (N-P) part of the upper trapezoidal factor T
            associated with A in the generalized RQ factorization of the
            pair (B,A) is singular, so that the rank of the matrix (E)
            comprising the rows of A and B is less than n; the least
            squares solutions could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zgglse'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08znft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[m].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[p].
        utils._EngineComplex128ArrayType.c_type, # Output, x[n].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not (_p_py >= 0 and _p_py <= _n_py and _n_py <= _m_py+_p_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= p and p <= n and n <= m+p.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m_py,),
    )
    _c = _c_cls.to_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_p_py,),
    )
    _d = _d_cls.to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _p,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _d,
        _x,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The upper triangular factor R associated with B in the generalized RQ',
                    'factorization of the pair (B,A) is singular, so that rank(B) < p; the',
                    'least squares solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The (N-P) by (N-P) part of the upper trapezoidal factor T associated with',
                    'A in the generalized RQ factorization of the pair (B,A) is singular, so that',
                    'the rank of the matrix (E) comprising the rows of A and B is less than n;',
                    'the least squares solutions could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _d_cls.refresh_py(_d)
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def zggglm(m, n, p, a, b, d, x, y):
    u"""
    Solves a complex general Gauss--Markov linear model (GLM) problem.

    For full information please refer to the NAG Library document for
    f08zp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zpf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrices A and B.

    n : int
        n, the number of columns of the matrix A.

    p : int
        p, the number of columns of the matrix B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: `a` is overwritten.

    b : complex, shape (`m`, `p`), modified in place
        On entry: the m by p matrix B.

        On exit: `b` is overwritten.

    d : complex, shape (`m`), modified in place
        On entry: the left-hand side vector d of the GLM equation.

        On exit: `d` is overwritten.

    x : complex, shape (`n`), modified in place
        On exit: the solution vector x of the GLM problem.

    y : complex, shape (`p`), modified in place
        On exit: the solution vector y of the GLM problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: 0 <= `n` <= `m`.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= `m`-`n`.

        (`errno` 1)
            The upper triangular factor R associated with A in the
            generalized RQ factorization of the pair (A,B) is singular,
            so that rank(A) < n; the least squares solution could not be
            computed.

        (`errno` 2)
            The bottom (m-n) by (m-n) part of the upper trapezoidal
            factor T associated with B in the generalized QR
            factorization of the pair (A,B) is singular, so that rank(A
            B) < n; the least squares solutions could not be computed.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggglm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zpft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, p.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, p].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[m].
        utils._EngineComplex128ArrayType.c_type, # Output, x[n].
        utils._EngineComplex128ArrayType.c_type, # Output, y[p].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not (_n_py >= 0 and _n_py <= _m_py):
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('0 <= n and n <= m.'),
            ],
            errcodes=(-2,),
            entity='n',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= _m_py-_n_py:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= m-n.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _p_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_m_py,),
    )
    _d = _d_cls.to_ctypes()
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_p_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _n,
        _p,
        _a,
        _lda,
        _b,
        _ldb,
        _d,
        _x,
        _y,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        if _info_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The upper triangular factor R associated with A in the generalized RQ',
                    'factorization of the pair (A,B) is singular, so that rank(A) < n;',
                    'the least squares solution could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        elif _info_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The bottom (m-n) by (m-n) part of the upper trapezoidal factor T associated',
                    'with B in the generalized QR factorization of the pair (A,B) is singular, so',
                    'that rank(A B) < n; the least squares solutions could not be computed.',
                ],
                errcodes=(_info_py,),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _info_py),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _d_cls.refresh_py(_d)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def zggqrf(n, m, p, a, taua, b, taub):
    u"""
    Computes a generalized QR factorization of a complex matrix pair.

    For full information please refer to the NAG Library document for
    f08zs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08zsf.html

    Parameters
    ----------
    n : int
        n, the number of rows of the matrices A and B.

    m : int
        m, the number of columns of the matrix A.

    p : int
        p, the number of columns of the matrix B.

    a : complex, shape (`n`, `m`), modified in place
        On entry: the n by m matrix A.

        On exit: the elements on and above the diagonal of the array
        contain the min(n, m) by m upper trapezoidal matrix R (R is
        upper triangular if n >= m); the elements below the diagonal,
        with the array `taua`, represent the unitary matrix Q as a
        product of min(n, m) elementary reflectors (see [the F08
        Introduction]).

    taua : complex, shape (min(`n`, `m`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the unitary matrix Q.

    b : complex, shape (`n`, `p`), modified in place
        On entry: the n by p matrix B.

        On exit: if n <= p, the upper triangle of the subarray
        `b`[0:n,p-n:p] contains the n by n upper triangular matrix T_12.

        If n > p, the elements on and above the (n-p)th subdiagonal
        contain the n by p upper trapezoidal matrix T; the remaining
        elements, with the array `taub`, represent the unitary matrix Z
        as a product of elementary reflectors (see [the F08
        Introduction]).

    taub : complex, shape (min(`n`, `p`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the unitary matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggqrf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08zsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, m].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, taua[min(n, m)].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, p].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, taub[min(n, p)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-1,),
            entity='n',
        ).manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-2,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-3,),
            entity='p',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _taua_cls = utils._EngineComplex128ArrayType(
        taua, {'fun_name': fun_name, 'entity': 'taua'},
        exp_shape=(min(_n_py, _m_py),),
        check_contents=False,
    )
    _taua = _taua_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _p_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _taub_cls = utils._EngineComplex128ArrayType(
        taub, {'fun_name': fun_name, 'entity': 'taub'},
        exp_shape=(min(_n_py, _p_py),),
        check_contents=False,
    )
    _taub = _taub_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _n,
        _m,
        _p,
        _a,
        _lda,
        _taua,
        _b,
        _ldb,
        _taub,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _taua_cls.refresh_py(_taua)
        _b_cls.refresh_py(_b)
        _taub_cls.refresh_py(_taub)
        return_data = None
    _err_class.manage()
    return return_data

def zggrqf(m, p, n, a, taua, b, taub):
    u"""
    Computes a generalized RQ factorization of a complex matrix pair.

    For full information please refer to the NAG Library document for
    f08zt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f08/f08ztf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    p : int
        p, the number of rows of the matrix B.

    n : int
        n, the number of columns of the matrices A and B.

    a : complex, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if m <= n, the upper triangle of the subarray
        `a`[0:m,n-m:n] contains the m by m upper triangular matrix R_12.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m by n upper trapezoidal matrix R; the remaining
        elements, with the array `taua`, represent the unitary matrix Q
        as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    taua : complex, shape (min(`m`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the unitary matrix Q.

    b : complex, shape (`p`, `n`), modified in place
        On entry: the p by n matrix B.

        On exit: the elements on and above the diagonal of the array
        contain the min(p, n) by n upper trapezoidal matrix T (T is
        upper triangular if p >= n); the elements below the diagonal,
        with the array `taub`, represent the unitary matrix Z as a
        product of elementary reflectors (see [the F08 Introduction]).

    taub : complex, shape (min(`p`, `n`)), modified in place
        On exit: the scalar factors of the elementary reflectors which
        represent the unitary matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -2)
            On entry, error in parameter `p`.

            Constraint: `p` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.lapackeig.zggrqf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f08ztft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_LOGICAL, # Input, call_vendor.
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Output, taua[min(m, n)].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Output, taub[min(p, n)].
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._BYREF_INT, # Output, info.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _call_vendor = utils._EngineState._literal_true_cint
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if not _m_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('m >= 0.'),
            ],
            errcodes=(-1,),
            entity='m',
        ).manage()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = utils._EngineIntScalarType.to_py(_p)
    if not _p_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('p >= 0.'),
            ],
            errcodes=(-2,),
            entity='p',
        ).manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if not _n_py >= 0:
        utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                utils._lapack_constraint_err_text('n >= 0.'),
            ],
            errcodes=(-3,),
            entity='n',
        ).manage()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _taua_cls = utils._EngineComplex128ArrayType(
        taua, {'fun_name': fun_name, 'entity': 'taua'},
        exp_shape=(min(_m_py, _n_py),),
        check_contents=False,
    )
    _taua = _taua_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_p_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _taub_cls = utils._EngineComplex128ArrayType(
        taub, {'fun_name': fun_name, 'entity': 'taub'},
        exp_shape=(min(_p_py, _n_py),),
        check_contents=False,
    )
    _taub = _taub_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _info = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _call_vendor,
        _en_data,
        _m,
        _p,
        _n,
        _a,
        _lda,
        _taua,
        _b,
        _ldb,
        _taub,
        _work,
        _lwork,
        _info,
    )
    utils._handle_cb_exception(excs)
    _info_py = utils._EngineIntScalarType.to_py(_info)
    if _info_py == 0:
        _err_class = utils._EngineNoErrorExit()
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            errcodes=(utils._IERR_UNEXPECTED, _info_py),
        )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _taua_cls.refresh_py(_taua)
        _b_cls.refresh_py(_b)
        _taub_cls.refresh_py(_taub)
        return_data = None
    _err_class.manage()
    return return_data
