# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `linsys` Chapter.

``linsys`` - Simultaneous Linear Equations

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the solution of the matrix equation AX =
B, where B may be a single vector or a matrix of multiple right-hand
sides.
The matrix A may be real, complex, symmetric, Hermitian, positive
definite, positive definite Toeplitz or banded.
It may also be rectangular, in which case a least squares solution is
obtained.

Much of the functionality of this module has been superseded by
functions from submodule ``lapacklin`` and submodule ``lapackeig``
(LAPACK routines) as those modules have grown and have included driver
and expert driver functions.

For a general introduction to sparse systems of equations, see [the F11
Introduction], which provides functions for large sparse systems. Some
functions for sparse problems are also included in this module; they are
described in [Sparse Matrix Functions].

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

@utils._withdrawal(
    'naginterfaces.base.linsys.withdraw_real_posdef_solve_ref',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dposvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_posdef_solve_ref(a, b, n, m, c, bb):
    u"""
    Solution of real symmetric positive definite simultaneous linear
    equations with multiple right-hand sides using iterative refinement
    (Black Box).

    Note: ``withdraw_real_posdef_solve_ref`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dposvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04abf.html

    Parameters
    ----------
    a : float, shape (`n`, `n`), modified in place
        On entry: the upper triangle of the n by n positive definite
        symmetric matrix A. The elements of the array below the diagonal
        need not be set.

        On exit: the elements of the array below the diagonal are
        overwritten; the upper triangle of A is unchanged.

    b : float, shape (`n`, `m`)
        The n by m right-hand side matrix B.

    n : int
        n, the order of the matrix A.

    m : int
        m, the number of right-hand sides.

    c : float, shape (`n`, `m`), modified in place
        On exit: the n by m solution matrix X.

    bb : float, shape (`n`, `m`), modified in place
        On exit: the final n by m residual matrix R = B-AX.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is not positive definite.

        (`errno` 2)
            Matrix A is too ill-conditioned; iterative refinement fails
            to improve the solution.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.withdraw_real_posdef_solve_ref'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, m].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wkspce.
        utils._EngineFloat64ArrayType.c_type, # Output, bb[ldbb, m].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (bb, 'bb')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _wkspce = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ldbb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _bb = _bb_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _b,
        _ldb,
        _n,
        _m,
        _c,
        _ldc,
        _wkspce,
        _bb,
        _ldbb,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is too ill-conditioned;',
                    'iterative refinement fails to improve the solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        _bb_cls.refresh_py(_bb)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.linsys.withdraw_real_square_solve_ref',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dgesvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_square_solve_ref(a, b, n, m, c, aa, bb):
    u"""
    Solution of real simultaneous linear equations with multiple
    right-hand sides using iterative refinement (Black Box).

    Note: ``withdraw_real_square_solve_ref`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dgesvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04aef.html

    Parameters
    ----------
    a : float, shape (`n`, `n`)
        The n by n matrix A.

    b : float, shape (`n`, `m`)
        The n by m right-hand side matrix B.

    n : int
        n, the order of the matrix A.

    m : int
        m, the number of right-hand sides.

    c : float, shape (`n`, `m`), modified in place
        On exit: the n by m solution matrix X.

    aa : float, shape (`n`, `n`), modified in place
        On exit: the triangular factors L and U, except that the unit
        diagonal elements of U are not stored.

    bb : float, shape (`n`, `m`), modified in place
        On exit: the final n by m residual matrix R = B-AX.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is approximately singular.

        (`errno` 2)
            The matrix A is too ill-conditioned.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.withdraw_real_square_solve_ref'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, m].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, m].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wkspce.
        utils._EngineFloat64ArrayType.c_type, # Output, aa[ldaa, n].
        utils._BYREF_INT, # Input, ldaa.
        utils._EngineFloat64ArrayType.c_type, # Output, bb[ldbb, m].
        utils._BYREF_INT, # Input, ldbb.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (aa, 'aa'), (bb, 'bb')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _wkspce = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ldaa = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(aa, {'fun_name': fun_name, 'entity': 'aa'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaa'},
    ).to_ctypes()
    _aa_cls = utils._EngineFloat64ArrayType(
        aa, {'fun_name': fun_name, 'entity': 'aa'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _aa = _aa_cls.to_ctypes(can_reference_contents=False)
    _ldbb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldbb'},
    ).to_ctypes()
    _bb_cls = utils._EngineFloat64ArrayType(
        bb, {'fun_name': fun_name, 'entity': 'bb'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _bb = _bb_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _b,
        _ldb,
        _n,
        _m,
        _c,
        _ldc,
        _wkspce,
        _aa,
        _ldaa,
        _bb,
        _ldbb,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is approximately singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _aa_cls.refresh_py(_aa)
        _bb_cls.refresh_py(_bb)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_lsqsol(a, x, b, m, n, ir, eps, qr, alpha, ipiv):
    u"""
    Least squares solution of m real equations in n unknowns, rank = n,
    m >= n using iterative refinement (Black Box).

    For full information please refer to the NAG Library document for
    f04am

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04amf.html

    Parameters
    ----------
    a : float, shape (`m`, `n`)
        The m by n matrix A.

    x : float, shape (`n`, `ir`), modified in place
        On exit: the n by r solution matrix X.

    b : float, shape (`m`, `ir`)
        The m by r right-hand side matrix B.

    m : int
        m, the number of rows of the matrix A, i.e., the number of
        equations.

    n : int
        n, the number of columns of the matrix A, i.e., the number of
        unknowns.

    ir : int
        r, the number of right-hand sides.

    eps : float
        Must be set to the value of the machine precision.

    qr : float, shape (`m`, `n`), modified in place
        On exit: details of the QR factorization.

    alpha : float, shape (`n`), modified in place
        On exit: the diagonal elements of the upper triangular matrix R.

    ipiv : int, shape (`n`), modified in place
        On exit: details of the column interchanges.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The rank of `a` is less than `n`. The problem does not have
            a unique solution.

        (`errno` 2)
            The iterative refinement fails to converge. The matrix A is
            too ill-conditioned.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_gen_lsqsol'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04amft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, ir].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, ir].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ir.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Output, qr[ldqr, n].
        utils._BYREF_INT, # Input, ldqr.
        utils._EngineFloat64ArrayType.c_type, # Output, alpha[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, e.
        utils._EngineFloat64ArrayType.c_type, # Workspace, y.
        utils._EngineFloat64ArrayType.c_type, # Workspace, z.
        utils._EngineFloat64ArrayType.c_type, # Workspace, r.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (x, 'x'), (b, 'b'), (qr, 'qr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _ir_py = utils._EngineIntScalarType.to_py(_ir)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _ir_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _ir_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _ldqr_py = utils._EngineFloat64ArrayType.get_nd(qr, {'fun_name': fun_name, 'entity': 'qr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldqr = utils._EngineIntScalarType(
        _ldqr_py, {'fun_name': fun_name, 'entity': '_ldqr'},
    ).to_ctypes()
    _qr_cls = utils._EngineFloat64ArrayType(
        qr, {'fun_name': fun_name, 'entity': 'qr'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _qr = _qr_cls.to_ctypes(can_reference_contents=False)
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _e = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _y = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _z = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _r = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _x,
        _ldx,
        _b,
        _ldb,
        _m,
        _n,
        _ir,
        _eps,
        _qr,
        _ldqr,
        _alpha,
        _e,
        _y,
        _z,
        _r,
        _ipiv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The rank of a is less than n.',
                    'The problem does not have a unique solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative refinement fails to converge.',
                    'The matrix A is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _qr_cls.refresh_py(_qr)
        _alpha_cls.refresh_py(_alpha)
        _ipiv_cls.refresh_py(_ipiv)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.linsys.withdraw_real_posdef_solve_1rhs',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dposvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_posdef_solve_1rhs(a, b, n, c):
    u"""
    Solution of real symmetric positive definite simultaneous linear
    equations, one right-hand side using iterative refinement (Black
    Box).

    Note: ``withdraw_real_posdef_solve_1rhs`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dposvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04asf.html

    Parameters
    ----------
    a : float, shape (`n`, `n`), modified in place
        On entry: the upper triangle of the n by n positive definite
        symmetric matrix A. The elements of the array below the diagonal
        need not be set.

        On exit: the elements of the array below the diagonal are
        overwritten; the upper triangle of `a` is unchanged.

    b : float, shape (max(1, `n`))
        The right-hand side vector b.

    n : int
        n, the order of the matrix A.

    c : float, shape (max(1, `n`)), modified in place
        On exit: the solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is not positive definite.

        (`errno` 2)
            Matrix A is too ill-conditioned; iterative refinement fails
            to improve the solution.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.withdraw_real_posdef_solve_1rhs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[max(1, n)].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, c[max(1, n)].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk2.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(max(1, _n_py),),
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(max(1, _n_py),),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _wk1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wk2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _b,
        _n,
        _c,
        _wk1,
        _wk2,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is too ill-conditioned;',
                    'iterative refinement fails to improve the solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.linsys.withdraw_real_square_solve_1rhs',
    mkwdrn='28',
    advice=[
        'Please use lapacklin.dgesvx instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_real_square_solve_1rhs(a, b, n, c, aa):
    u"""
    Solution of real simultaneous linear equations, one right-hand side
    using iterative refinement (Black Box).

    Note: ``withdraw_real_square_solve_1rhs`` will be removed in
    naginterfaces 28.0.0.
    Please use ``lapacklin.dgesvx`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f04at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04atf.html

    Parameters
    ----------
    a : float, shape (`n`, `n`)
        The n by n matrix A.

    b : float, shape (`n`)
        The right-hand side vector b.

    n : int
        n, the order of the matrix A.

    c : float, shape (`n`), modified in place
        On exit: the solution vector x.

    aa : float, shape (`n`, `n`), modified in place
        On exit: the triangular factors L and U, except that the unit
        diagonal elements of U are not stored.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Matrix A is approximately singular.

        (`errno` 2)
            The matrix A is too ill-conditioned.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.withdraw_real_square_solve_1rhs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._EngineFloat64ArrayType.c_type, # Output, aa[ldaa, n].
        utils._BYREF_INT, # Input, ldaa.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wks1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wks2.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (aa, 'aa')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _ldaa = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(aa, {'fun_name': fun_name, 'entity': 'aa'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldaa'},
    ).to_ctypes()
    _aa_cls = utils._EngineFloat64ArrayType(
        aa, {'fun_name': fun_name, 'entity': 'aa'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _aa = _aa_cls.to_ctypes(can_reference_contents=False)
    _wks1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wks2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _lda,
        _b,
        _n,
        _c,
        _aa,
        _ldaa,
        _wks1,
        _wks2,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix A is approximately singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix A is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _aa_cls.refresh_py(_aa)
        return_data = None
    _err_class.manage()
    return return_data

def real_sparse_fac_solve(n, a, licn, comm, rhs, mtype):
    u"""
    Solution of real sparse simultaneous linear equations (coefficient
    matrix already factorized).

    For full information please refer to the NAG Library document for
    f04ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04axf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, shape (`licn`)
        The nonzero elements in the factorization of the matrix A, as
        returned by ``matop.real_gen_sparse_lu`` or
        ``matop.real_gen_sparse_lu_reuse``.

    licn : int
        The dimension of the arrays `a` and `comm`['icn'].

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``matop.real_gen_sparse_lu``.

    rhs : float, shape (`n`), modified in place
        On entry: the right-hand side vector b.

        On exit: `rhs` is overwritten by the solution vector x.

    mtype : int
        `mtype` specifies the task to be performed.

        `mtype` = 1

          Solve Ax = b.

        `mtype` != 1

          Solve A^Tx = b.

    Returns
    -------
    resid : float
        The value of the maximum residual, max(|b_i-sum_j^a_{ij}x_j|),
        over all the unsatisfied equations, in case
        ``matop.real_gen_sparse_lu`` or
        ``matop.real_gen_sparse_lu_reuse`` has been used to factorize a
        singular or rectangular matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_sparse_fac_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04axft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[licn].
        utils._BYREF_INT, # Input, licn.
        utils._EngineIntArrayType.c_type, # Input, icn[licn].
        utils._EngineIntArrayType.c_type, # Input/Output, ikeep[5*n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rhs[n].
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, mtype.
        utils._EngineIntArrayType.c_type, # Input, idisp[2].
        utils._BYREF_FLOAT64, # Output, resid.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _licn = utils._EngineIntScalarType(
        licn, {'fun_name': fun_name, 'entity': 'licn'},
    ).to_ctypes()
    _licn_py = utils._EngineIntScalarType.to_py(_licn)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_licn_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icn', 'ikeep', 'idisp'),
    )
    _icn = comm['icn']
    _ikeep = comm['ikeep']
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _rhs_cls = utils._EngineFloat64ArrayType(
        rhs, {'fun_name': fun_name, 'entity': 'rhs'},
        exp_shape=(_n_py,),
    )
    _rhs = _rhs_cls.to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _mtype = utils._EngineIntScalarType(
        mtype, {'fun_name': fun_name, 'entity': 'mtype'},
    ).to_ctypes()
    _idisp = comm['idisp']
    _resid = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _licn,
        _icn,
        _ikeep,
        _rhs,
        _w,
        _mtype,
        _idisp,
        _resid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rhs_cls.refresh_py(_rhs)
        _resid_py = utils._EngineFloat64ScalarType.to_py(_resid)
        return_data = _resid_py
    _err_class.manage()
    return return_data

def real_square_solve(n, nrhs, a, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real system of linear equations.

    For full information please refer to the NAG Library document for
    f04ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04baf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: if no exception is raised, the factors L and U from the
        factorization A = PLU. The unit diagonal elements of L are not
        stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_square_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_band_solve(n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real banded system of linear equations.

    For full information please refer to the NAG Library document for
    f04bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bbf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kl : int
        The number of subdiagonals k_l, within the band of A.

    ku : int
        The number of superdiagonals k_u, within the band of A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ab : float, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n matrix A.

        See [Further Comments] for further details.

        On exit: if no exception is raised, `ab` is overwritten by
        details of the factorization.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -6)
            On entry, ldab = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` -4)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` -2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_band_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -6 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'kl = {:s} and ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 8:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_tridiag_solve(n, nrhs, dl, d, du, du2, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f04bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bcf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    dl : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: if no exception is raised, `dl` is overwritten by the
        (n-1) multipliers that define the matrix L from the LU
        factorization of A.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: if no exception is raised, `d` is overwritten by the n
        diagonal elements of the upper triangular matrix U from the LU
        factorization of A.

    du : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A

        On exit: if no exception is raised, `du` is overwritten by the
        (n-1) elements of the first superdiagonal of U.

    du2 : float, shape (`n`-2), modified in place
        On exit: if no exception is raised, `du2` returns the (n-2)
        elements of the second superdiagonal of U.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will
        always be either i or (i+1); `ipiv`[i-1] = i indicates a row
        interchange was not required.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_tridiag_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineFloat64ArrayType.c_type, # Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _dl_cls = utils._EngineFloat64ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineFloat64ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _du2_cls = utils._EngineFloat64ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
        check_contents=False,
    )
    _du2 = _du2_cls.to_ctypes(can_reference_contents=False)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py == _n_py and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_posdef_solve(uplo, n, nrhs, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite system of linear equations.

    For full information please refer to the NAG Library document for
    f04bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bdf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        `a` contains the upper triangular part of the matrix A, and the
        strictly lower triangular part of `a` is not referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        `a` contains the lower triangular part of the matrix A, and the
        strictly upper triangular part of `a` is not referenced.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^TU or
        A = LL^T.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_posdef_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_posdef_packed_solve(uplo, n, nrhs, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite system of linear equations,
    packed storage.

    For full information please refer to the NAG Library document for
    f04be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bef.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A. The upper or lower
        triangular part of the symmetric matrix is packed column-wise in
        a linear array. The jth column of A is stored in the array `ap`
        as follows:

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^TU or
        A = LL^T, in the same storage format as A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_posdef_packed_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04beft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_posdef_band_solve(uplo, n, kd, nrhs, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite banded system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kd : int
        The number of superdiagonals k (and the number of subdiagonals)
        of the band matrix A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ab : float, shape (`kd`+1, `n`), modified in place
        On entry: the n by n symmetric band matrix A. The upper or lower
        triangular part of the symmetric matrix is stored in the first
        `kd`+1 rows of the array. The jth column of A is stored in the
        jth column of the array `ab` as follows:

        See [Further Comments] below for further details.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^TU or
        A = LL^T, in the same storage format as A.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -4)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -3)
            On entry, `kd` = *<value>*.

            Constraint: `kd` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_posdef_band_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _ldab_py = utils._EngineFloat64ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ab_cls = utils._EngineFloat64ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kd >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 8:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_posdef_tridiag_solve(n, nrhs, d, e, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric positive definite tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bgf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the
        tridiagonal matrix A.

        On exit: if the function exits successfully or `errno` = `n` +
        1, `d` is overwritten by the n diagonal elements of the diagonal
        matrix D from the LDL^T factorization of A.

    e : float, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        tridiagonal matrix A.

        On exit: if the function exits successfully or `errno` = `n` +
        1, `e` is overwritten by the (n-1) subdiagonal elements of the
        unit lower bidiagonal matrix L from the LDL^T factorization of
        A. (`e` can also be regarded as the superdiagonal of the unit
        upper bidiagonal factor U from the U^TDU factorization of A.)

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_posdef_tridiag_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_solve(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f04bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bhf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n symmetric matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.dsytrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, details of the interchanges
        and the block structure of D, as determined by
        ``lapacklin.dsytrf``.

        `ipiv`[k-1] > 0

          Rows and columns k and `ipiv`[k-1] were interchanged, and
          d_{kk} is a 1 by 1 diagonal block.

        `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0

          Rows and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block.

        `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0

          Rows and columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_symm_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_packed_solve(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a real symmetric system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04bjf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ap : float, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed column-wise in a
        linear array. The jth column of the matrix A is stored in the
        array `ap` as follows:

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.dsptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, details of the interchanges
        and the block structure of D, as determined by
        ``lapacklin.dsptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : float, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_symm_packed_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04bjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ap_cls = utils._EngineFloat64ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_square_solve(n, nrhs, a, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex system of linear equations.

    For full information please refer to the NAG Library document for
    f04ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04caf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n coefficient matrix A.

        On exit: if no exception is raised, the factors L and U from the
        factorization A = PLU. The unit diagonal elements of L are not
        stored.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_square_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_band_solve(n, kl, ku, nrhs, ab, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex banded system of linear equations.

    For full information please refer to the NAG Library document for
    f04cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cbf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kl : int
        The number of subdiagonals k_l, within the band of A.

    ku : int
        The number of superdiagonals k_u, within the band of A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ab : complex, shape (2*`kl`+`ku`+1, `n`), modified in place
        On entry: the n by n matrix A.

        See [Further Comments] for further details.

        On exit: if no exception is raised, `ab` is overwritten by
        details of the factorization.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] = i
        indicates a row interchange was not required.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no exception is raised, an estimate of the reciprocal of the
        condition number of the matrix A, computed as `rcond` =
        (||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -6)
            On entry, ldab = *<value>*, `kl` = *<value>* and `ku` =
            *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` -4)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -3)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` -2)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_band_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kl.
        utils._BYREF_INT, # Input, ku.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kl = utils._EngineIntScalarType(
        kl, {'fun_name': fun_name, 'entity': 'kl'},
    ).to_ctypes()
    _ku = utils._EngineIntScalarType(
        ku, {'fun_name': fun_name, 'entity': 'ku'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _kl_py = utils._EngineIntScalarType.to_py(_kl)
    _ku_py = utils._EngineIntScalarType.to_py(_ku)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(2*_kl_py+_ku_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _kl,
        _ku,
        _nrhs,
        _ab,
        _ldab,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -6 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldab = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'kl = {:s} and ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: ldab >= 2*kl+ku+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ku = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ku >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kl >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 8:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_tridiag_solve(n, nrhs, dl, d, du, du2, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex tridiagonal system of linear equations.

    For full information please refer to the NAG Library document for
    f04cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04ccf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    dl : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        matrix A.

        On exit: if no exception is raised, `dl` is overwritten by the
        (n-1) multipliers that define the matrix L from the LU
        factorization of A.

    d : complex, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the matrix A.

        On exit: if no exception is raised, `d` is overwritten by the n
        diagonal elements of the upper triangular matrix U from the LU
        factorization of A.

    du : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) superdiagonal elements of the
        matrix A

        On exit: if no exception is raised, `du` is overwritten by the
        (n-1) elements of the first superdiagonal of U.

    du2 : complex, shape (`n`-2), modified in place
        On exit: if no exception is raised, `du2` returns the (n-2)
        elements of the second superdiagonal of U.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, the pivot indices that
        define the permutation matrix P; at the ith step row i of the
        matrix was interchanged with row `ipiv`[i-1]. `ipiv`[i-1] will
        always be either i or (i+1); `ipiv`[i-1] = i indicates a row
        interchange was not required.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal element *<value>* of the upper triangular factor is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_tridiag_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, dl[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, du[n-1].
        utils._EngineComplex128ArrayType.c_type, # Output, du2[n-2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _dl_cls = utils._EngineComplex128ArrayType(
        dl, {'fun_name': fun_name, 'entity': 'dl'},
        exp_shape=(_n_py-1,),
    )
    _dl = _dl_cls.to_ctypes()
    _d_cls = utils._EngineComplex128ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _du_cls = utils._EngineComplex128ArrayType(
        du, {'fun_name': fun_name, 'entity': 'du'},
        exp_shape=(_n_py-1,),
    )
    _du = _du_cls.to_ctypes()
    _du2_cls = utils._EngineComplex128ArrayType(
        du2, {'fun_name': fun_name, 'entity': 'du2'},
        exp_shape=(_n_py-2,),
        check_contents=False,
    )
    _du2 = _du2_cls.to_ctypes(can_reference_contents=False)
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _dl,
        _d,
        _du,
        _du2,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal element {:s} of the upper triangular factor is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'zero. The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dl_cls.refresh_py(_dl)
        _d_cls.refresh_py(_d)
        _du_cls.refresh_py(_du)
        _du2_cls.refresh_py(_du2)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_posdef_solve(uplo, n, nrhs, a, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite system of linear equations.

    For full information please refer to the NAG Library document for
    f04cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cdf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        `a` contains the upper triangular part of the matrix A, and the
        strictly lower triangular part of `a` is not referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        `a` contains the lower triangular part of the matrix A, and the
        strictly upper triangular part of `a` is not referenced.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^HU or
        A = LL^H.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_posdef_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_posdef_packed_solve(uplo, n, nrhs, ap, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite system of linear equations,
    packed storage.

    For full information please refer to the NAG Library document for
    f04ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cef.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A. The upper or lower
        triangular part of the Hermitian matrix is packed column-wise in
        a linear array. The jth column of A is stored in the array `ap`
        as follows:

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^HU or
        A = LL^H, in the same storage format as A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_posdef_packed_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04ceft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_posdef_band_solve(uplo, n, kd, nrhs, ab, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite banded system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cff.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    kd : int
        The number of superdiagonals k (and the number of subdiagonals)
        of the band matrix A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ab : complex, shape (`kd`+1, `n`), modified in place
        On entry: the n by n Hermitian band matrix A. The upper or lower
        triangular part of the Hermitian matrix is stored in the first
        `kd`+1 rows of the array. The jth column of A is stored in the
        jth column of the array `ab` as follows:

        See [Further Comments] below for further details.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the factor U or L from the Cholesky factorization A = U^HU or
        A = LL^H, in the same storage format as A.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -4)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -3)
            On entry, `kd` = *<value>*.

            Constraint: `kd` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_posdef_band_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04cfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, kd.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ab[ldab, n].
        utils._BYREF_INT, # Input, ldab.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ab, 'ab'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _kd = utils._EngineIntScalarType(
        kd, {'fun_name': fun_name, 'entity': 'kd'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _ldab_py = utils._EngineComplex128ArrayType.get_nd(ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldab = utils._EngineIntScalarType(
        _ldab_py, {'fun_name': fun_name, 'entity': '_ldab'},
    ).to_ctypes()
    _kd_py = utils._EngineIntScalarType.to_py(_kd)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ab_cls = utils._EngineComplex128ArrayType(
        ab, {'fun_name': fun_name, 'entity': 'ab'},
        exp_shape=(_kd_py+1, _n_py),
        mutual_sorder=_m_sorder,
    )
    _ab = _ab_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _kd,
        _nrhs,
        _ab,
        _ldab,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: kd >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 8:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ab_cls.refresh_py(_ab)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_posdef_tridiag_solve(n, nrhs, d, e, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian positive definite tridiagonal system of linear
    equations.

    For full information please refer to the NAG Library document for
    f04cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cgf.html

    Parameters
    ----------
    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    d : float, shape (`n`), modified in place
        On entry: must contain the n diagonal elements of the
        tridiagonal matrix A.

        On exit: if the function exits successfully or `errno` = `n` +
        1, `d` is overwritten by the n diagonal elements of the diagonal
        matrix D from the LDL^H factorization of A.

    e : complex, shape (`n`-1), modified in place
        On entry: must contain the (n-1) subdiagonal elements of the
        tridiagonal matrix A.

        On exit: if the function exits successfully or `errno` = `n` +
        1, `e` is overwritten by the (n-1) subdiagonal elements of the
        unit lower bidiagonal matrix L from the LDL^H factorization of
        A. (`e` can also be regarded as the conjugate of the
        superdiagonal of the unit upper bidiagonal factor U from the
        U^HDU factorization of A.)

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the reciprocal of the condition number of the matrix
        A, computed as `rcond` = 1/(||A||_1,||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -2)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` (i > 0) and (i<=`n`))
            The principal minor of order *<value>* of the matrix A is
            not positive definite. The factorization has not been
            completed and the solution could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_posdef_tridiag_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04cgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, e[n-1].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    )
    _d = _d_cls.to_ctypes()
    _e_cls = utils._EngineComplex128ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py-1,),
    )
    _e = _e_cls.to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nrhs,
        _d,
        _e,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The principal minor of order {:s} of the matrix A is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'positive definite. The factorization has not been completed and the solution',
                    'could not be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _d_cls.refresh_py(_d)
        _e_cls.refresh_py(_e)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_herm_solve(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution and error-bound to a complex Hermitian system
    of linear equations.

    For full information please refer to the NAG Library document for
    f04ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04chf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n Hermitian matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zhetrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, details of the interchanges
        and the block structure of D, as determined by
        ``lapacklin.zhetrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_herm_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04chft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_herm_packed_solve(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex Hermitian system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04cj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04cjf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n Hermitian matrix A, packed column-wise in a
        linear array. The jth column of the matrix A is stored in the
        array `ap` as follows:

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zhptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, details of the interchanges
        and the block structure of D, as determined by
        ``lapacklin.zhptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_herm_packed_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04cjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_symm_solve(uplo, n, nrhs, a, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex symmetric system of linear equations.

    For full information please refer to the NAG Library document for
    f04dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04dhf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n complex symmetric matrix A.

        If `uplo` = 'U', the leading `n` by `n` upper triangular part of
        the array `a` contains the upper triangular part of the matrix
        A, and the strictly lower triangular part of `a` is not
        referenced.

        If `uplo` = 'L', the leading `n` by `n` lower triangular part of
        the array `a` contains the lower triangular part of the matrix
        A, and the strictly upper triangular part of `a` is not
        referenced.

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^T or A = LDL^T as computed by
        ``lapacklin.zsytrf``.

    ipiv : int, shape (`n`), modified in place
        On exit: if no exception is raised, details of the interchanges
        and the block structure of D, as determined by
        ``lapacklin.zsytrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` not one of 'U' or 'u' or 'L' or 'l': `uplo`
            = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_symm_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04dhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _a,
        _lda,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo not one of \'U\' or \'u\' or \'L\' or \'l\':',
                    'uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 7:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_symm_packed_solve(uplo, n, nrhs, ap, ipiv, b):
    u"""
    Computes the solution, estimated condition number and error-bound to
    a complex symmetric system of linear equations, packed storage.

    For full information please refer to the NAG Library document for
    f04dj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04djf.html

    Parameters
    ----------
    uplo : str, length 1
        If `uplo` = 'U', the upper triangle of the matrix A is stored.

        If `uplo` = 'L', the lower triangle of the matrix A is stored.

    n : int
        The number of linear equations n, i.e., the order of the matrix
        A.

    nrhs : int
        The number of right-hand sides r, i.e., the number of columns of
        the matrix B.

    ap : complex, shape (`n`*(`n`+1)/2), modified in place
        On entry: the n by n symmetric matrix A, packed column-wise in a
        linear array. The jth column of the matrix A is stored in the
        array `ap` as follows:

        On exit: if no exception is raised, the block diagonal matrix D
        and the multipliers used to obtain the factor U or L from the
        factorization A = UDU^H or A = LDL^H as computed by
        ``lapacklin.zsptrf``, stored as a packed triangular matrix in
        the same storage format as A.

    ipiv : int, shape (`n`), modified in place
        On exit: if no constraints are violated, details of the
        interchanges and the block structure of D, as determined by
        ``lapacklin.zsptrf``.

        - If `ipiv`[k-1] > 0, then rows and columns k and `ipiv`[k-1]
          were interchanged, and d_{kk} is a 1 by 1 diagonal block;

        - if `uplo` = 'U' and `ipiv`[k-1] = `ipiv`[k-2] < 0, then rows
          and columns k-1 and -`ipiv`[k-1] were interchanged and
          d_{k-1:k,k-1:k} is a 2 by 2 diagonal block;

        - if `uplo` = 'L' and `ipiv`[k-1] = `ipiv`[k] < 0, then rows and
          columns k+1 and -`ipiv`[k-1] were interchanged and
          d_{k:k+1,k:k+1} is a 2 by 2 diagonal block.

    b : complex, shape (`n`, `nrhs`), modified in place
        On entry: the n by r matrix of right-hand sides B.

        On exit: if the function exits successfully or `errno` = `n` +
        1, the n by r solution matrix X.

    Returns
    -------
    rcond : float
        If no constraints are violated, an estimate of the reciprocal of
        the condition number of the matrix A, computed as `rcond` =
        1/(||A||_1||A^-1||_1).

    errbnd : float
        If the function exits successfully or `errno` = `n` + 1, an
        estimate of the forward error bound for a computed solution x^,
        such that ||x^-x||_1/||x||_1 <= `errbnd`, where x^ is a column
        of the computed solution returned in the array `b` and x is the
        corresponding column of the exact solution X. If `rcond` is less
        than machine precision, `errbnd` is returned as unity.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` -2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -1)
            On entry, `uplo` != 'U' or 'L': `uplo` = *<value>*.

        (`errno` (i > 0) and (i<=`n`))
            Diagonal block *<value>* of the block diagonal matrix is
            zero. The factorization has been completed, but the solution
            could not be computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+1)
            A solution has been computed, but `rcond` is less than
            machine precision so that the matrix A is numerically
            singular.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_symm_packed_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04djft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, uplo(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nrhs.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, ap[n*(n+1)//2].
        utils._EngineIntArrayType.c_type, # Output, ipiv[n].
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, nrhs].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_FLOAT64, # Output, rcond.
        utils._BYREF_FLOAT64, # Output, errbnd.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of uplo.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _uplo = utils._EngineCharScalarType(
        uplo, {'fun_name': fun_name, 'entity': 'uplo'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nrhs = utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ap_cls = utils._EngineComplex128ArrayType(
        ap, {'fun_name': fun_name, 'entity': 'ap'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _ap = _ap_cls.to_ctypes()
    _ipiv_cls = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ipiv = _ipiv_cls.to_ctypes(can_reference_contents=False)
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _nrhs_py = utils._EngineIntScalarType.to_py(_nrhs)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _nrhs_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _rcond = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbnd = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _uplo,
        _n,
        _nrhs,
        _ap,
        _ipiv,
        _b,
        _ldb,
        _rcond,
        _errbnd,
        _errbuf,
        _ierr,
        _uplo._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrhs = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrhs >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, uplo != \'U\' or \'L\': uplo = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ierr_py <= _n_py and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Diagonal block {:s} of the block diagonal matrix is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The factorization has been completed, but the solution could not',
                    'be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _n_py+1 and _ifmt == 6:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution has been computed, but rcond is less than machine precision',
                    'so that the matrix A is numerically singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ap_cls.refresh_py(_ap)
        _ipiv_cls.refresh_py(_ipiv)
        _b_cls.refresh_py(_b)
        _rcond_py = utils._EngineFloat64ScalarType.to_py(_rcond)
        _errbnd_py = utils._EngineFloat64ScalarType.to_py(_errbnd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['rcond', 'errbnd'],
        )(
            _rcond_py,
            _errbnd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_toeplitz_yule(n, t, x, wantp, p, wantv, v):
    u"""
    Solution of the Yule--Walker equations for real symmetric positive
    definite Toeplitz matrix, one right-hand side.

    For full information please refer to the NAG Library document for
    f04fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04fef.html

    Parameters
    ----------
    n : int
        The order of the Toeplitz matrix T.

    t : float, shape (`n`+1)
        `t`[0] must contain the value tau_0 of the diagonal elements of
        T, and the remaining `n` elements of `t` must contain the
        elements of the vector t.

    x : float, shape (`n`), modified in place
        On exit: the solution vector x.

    wantp : bool
        Must be set to True if the partial (auto)correlation
        coefficients are required, and must be set to False otherwise.

    p : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `wantp`= True: `n`; otherwise: 1.

        On exit: with `wantp` as True, the ith element of `p` contains
        the partial (auto)correlation coefficient, or reflection
        coefficient, p_i for the ith step. (See [Further Comments] and
        submodule ``tsa``.) If `wantp` is False, `p` is not referenced.
        Note that in any case, x_n = p_n.

    wantv : bool
        Must be set to True if the mean square prediction errors are
        required, and must be set to False otherwise.

    v : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `wantv`= True: `n`; otherwise: 1.

        On exit: with `wantv` as True, the ith element of `v` contains
        the mean square prediction error, or predictor error variance
        ratio, v_i, for the ith step. (See [Further Comments] and
        submodule ``tsa``.) If `wantv` is False, `v` is not referenced.

    Returns
    -------
    vlast : float
        The value of v_n, the mean square prediction error for the final
        step.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Principal minor *<value>* is not positive definite. Value of
            the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_toeplitz_yule'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._BYREF_LOGICAL, # Input, wantp.
        utils._EngineFloat64ArrayType.c_type, # Output, p[*].
        utils._BYREF_LOGICAL, # Input, wantv.
        utils._EngineFloat64ArrayType.c_type, # Output, v[*].
        utils._BYREF_FLOAT64, # Output, vlast.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _wantp = utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = utils._EngineLogicalScalarType.to_py(_wantp)
    if _wantp_py:
        _p_dim1 = _n_py
    else:
        _p_dim1 = 1
    exp_shape_p = (_p_dim1,)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=exp_shape_p,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _wantv = utils._EngineLogicalScalarType(
        wantv, {'fun_name': fun_name, 'entity': 'wantv'},
    ).to_ctypes()
    _wantv_py = utils._EngineLogicalScalarType.to_py(_wantv)
    if _wantv_py:
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    exp_shape_v = (_v_dim1,)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=exp_shape_v,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _vlast = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _x,
        _wantp,
        _p,
        _wantv,
        _v,
        _vlast,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Principal minor {:s} is not positive definite.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Value of the reflection coefficient is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _p_cls.refresh_py(_p)
        _v_cls.refresh_py(_v)
        _vlast_py = utils._EngineFloat64ScalarType.to_py(_vlast)
        return_data = _vlast_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['vlast'],
            )(
                _vlast_py,
            )
    _err_class.manage()
    return return_data

def real_toeplitz_solve(n, t, b, x, wantp, p):
    u"""
    Solution of real symmetric positive definite Toeplitz system, one
    right-hand side.

    For full information please refer to the NAG Library document for
    f04ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04fff.html

    Parameters
    ----------
    n : int
        The order of the Toeplitz matrix T.

    t : float, shape (`n`)
        `t`[i] must contain the value tau_i, for i = 0,1,...,`n`-1.

    b : float, shape (`n`)
        The right-hand side vector b.

    x : float, shape (`n`), modified in place
        On exit: the solution vector x.

    wantp : bool
        Must be set to True if the reflection coefficients are required,
        and must be set to False otherwise.

    p : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `wantp`= True: `n`-1; otherwise: 1.

        On exit: with `wantp` as True, the ith element of `p` contains
        the reflection coefficient, p_i, for the ith step, for i =
        1,2,...,`n`-1. (See [Further Comments].) If `wantp` is False,
        `p` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Principal minor *<value>* is not positive definite. Value of
            the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_toeplitz_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._BYREF_LOGICAL, # Input, wantp.
        utils._EngineFloat64ArrayType.c_type, # Output, p[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _wantp = utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = utils._EngineLogicalScalarType.to_py(_wantp)
    if _wantp_py:
        _p_dim1 = _n_py-1
    else:
        _p_dim1 = 1
    exp_shape_p = (_p_dim1,)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=exp_shape_p,
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _b,
        _x,
        _wantp,
        _p,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py > 0 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Principal minor {:s} is not positive definite.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Value of the reflection coefficient is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _p_cls.refresh_py(_p)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_solve(m, n, a, b, tol, work, lwork):
    u"""
    Least squares (if rank = n) or minimal least squares (if rank < n)
    solution of m real equations in n unknowns, m >= n.

    For full information please refer to the NAG Library document for
    f04jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04jgf.html

    Parameters
    ----------
    m : int
        m, the number of rows of `a`.

    n : int
        n, the number of columns of `a`.

    a : float, shape (`m`, `n`), modified in place
        On entry: the m by n matrix A.

        On exit: if `svd` is returned as False, `a` is overwritten by
        details of the QR factorization of A.

        If `svd` is returned as True, the first n rows of `a` are
        overwritten by the right-hand singular vectors, stored by rows;
        and the remaining rows of the array are used as workspace.

    b : float, shape (`m`), modified in place
        On entry: the right-hand side vector b.

        On exit: the first n elements of `b` contain the minimal least
        squares solution vector x. The remaining m-n elements are used
        for workspace.

    tol : float
        A relative tolerance to be used to determine the rank of A.
        `tol` should be chosen as approximately the largest relative
        error in the elements of A. For example, if the elements of A
        are correct to about 4 significant figures then `tol` should be
        set to about 5*10^(-4). See [Further Comments] for a description
        of how `tol` is used to determine rank. If `tol` is outside the
        range (epsilon,1.0), where epsilon is the machine precision, the
        value epsilon is used in place of `tol`. For most problems this
        is unreasonably small.

    work : float, shape (`lwork`), modified in place
        On exit: if `svd` is returned as False, then the first n
        elements of `work` contain information on the QR factorization
        of A (see argument `a` above), and `work`[n] contains the
        condition number ||R||_E||R^-1||_E of the upper triangular
        matrix R.

        If `svd` is returned as True, then the first n elements of
        `work` contain the singular values of A arranged in descending
        order and `work`[n] contains the total number of iterations
        taken by the QR algorithm.

        The rest of `work` is used as workspace.

    lwork : int
        The dimension of the array `work`.

    Returns
    -------
    svd : bool
        Is returned as False if the least squares solution has been
        obtained from the QR factorization of A. In this case A is of
        full rank. `svd` is returned as True if the least squares
        solution has been obtained from the singular value decomposition
        of A.

    sigma : float
        The standard error, i.e., the value sqrt(r^Tr/(m-k)) when m > k,
        and the value zero when m = k. Here r is the residual vector
        b-Ax and k is the rank of A.

    irank : int
        k, the rank of the matrix A. It should be noted that it is
        possible for `irank` to be returned as n and `svd` to be
        returned as True. This means that the matrix R only just failed
        the test for nonsingularity.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lwork` is too small. Minimum size required:
            *<value>*.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` >= `n`.

        (`errno` 2)
            The QR algorithm has failed to converge to the singular
            values in 50*`n` iterations. This failure can only happen
            when the singular value decomposition is employed, but even
            then it is not likely to occur.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_gen_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04jgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[m].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_LOGICAL, # Output, svd.
        utils._BYREF_FLOAT64, # Output, sigma.
        utils._BYREF_INT, # Output, irank.
        utils._EngineFloat64ArrayType.c_type, # Output, work[lwork].
        utils._BYREF_INT, # Input, lwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py,),
    )
    _b = _b_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _svd = utils._EngineLogicalScalarType.empty_ctypes()
    _sigma = utils._EngineFloat64ScalarType.empty_ctypes()
    _irank = utils._EngineIntScalarType.empty_ctypes()
    _lwork = utils._EngineIntScalarType(
        lwork, {'fun_name': fun_name, 'entity': 'lwork'},
    ).to_ctypes()
    _lwork_py = utils._EngineIntScalarType.to_py(_lwork)
    _work_cls = utils._EngineFloat64ArrayType(
        work, {'fun_name': fun_name, 'entity': 'work'},
        exp_shape=(_lwork_py,),
        check_contents=False,
    )
    _work = _work_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _b,
        _tol,
        _svd,
        _sigma,
        _irank,
        _work,
        _lwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwork is too small. Minimum size required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm has failed to converge to the singular values in 50*n',
                    'iterations. This failure can only happen when the singular value',
                    'decomposition is employed, but even then it is not likely to occur.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _work_cls.refresh_py(_work)
        _svd_py = utils._EngineLogicalScalarType.to_py(_svd)
        _sigma_py = utils._EngineFloat64ScalarType.to_py(_sigma)
        _irank_py = utils._EngineIntScalarType.to_py(_irank)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['svd', 'sigma', 'irank'],
        )(
            _svd_py,
            _sigma_py,
            _irank_py,
        )
    _err_class.manage()
    return return_data

def real_tridiag_fac_solve(job, n, a, b, c, d, ipiv, y, tol):
    u"""
    Solution of real tridiagonal simultaneous linear equations
    (coefficient matrix already factorized by
    ``matop.real_gen_tridiag_lu``).

    For full information please refer to the NAG Library document for
    f04le

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04lef.html

    Parameters
    ----------
    job : int
        Must specify the equations to be solved.

        `job` = 1

          The equations (T-lambda I)x = y are to be solved, but diagonal
          elements of U are not to be perturbed.

        `job` = -1

          The equations (T-lambda I)x = y are to be solved and, if
          overflow would otherwise occur, diagonal elements of U are to
          be perturbed. See argument `tol`.

        `job` = 2

          The equations (T-lambda I)^Tx = y are to be solved, but
          diagonal elements of U are not to be perturbed.

        `job` = -2

          The equations (T-lambda I)^Tx = y are to be solved and, if
          overflow would otherwise occur, diagonal elements of U are to
          be perturbed. See argument `tol`.

        `job` = 3

          The equations Ux = y are to be solved, but diagonal elements
          of U are not to be perturbed.

        `job` = -3

          The equations Ux = y are to be solved and, if overflow would
          otherwise occur, diagonal elements of U are to be perturbed.
          See argument `tol`.

    n : int
        n, the order of the matrix T.

    a : float, shape (`n`)
        The diagonal elements of U as returned by
        ``real_tridiag_fac_solve``.

    b : float, shape (`n`)
        The elements of the first superdiagonal of U as returned by
        ``real_tridiag_fac_solve``.

    c : float, shape (`n`)
        The subdiagonal elements of L as returned by
        ``real_tridiag_fac_solve``.

    d : float, shape (`n`)
        The elements of the second superdiagonal of U as returned by
        ``real_tridiag_fac_solve``.

    ipiv : int, shape (`n`)
        Details of the matrix P as returned by
        ``matop.real_gen_tridiag_lu``.

    y : float, shape (`n`), modified in place
        On entry: the right-hand side vector y.

        On exit: `y` is overwritten by the solution vector x.

    tol : float
        The minimum perturbation to be made to very small diagonal
        elements of U. `tol` is only referenced when `job` is negative.
        `tol` should normally be chosen as about epsilon||U||, where
        epsilon is the machine precision, but if `tol` is supplied as
        non-positive, it is reset to epsilon max(|u_{ij}|).

    Returns
    -------
    tol : float
        If on entry `tol` is non-positive, it is reset as just
        described. Otherwise `tol` is unchanged.

    Raises
    ------
    NagValueError
        (`errno` i > 1)
            Overflow would occur computing the element I in the
            solution. I = *<value>*.

        (`errno` 1)
            On entry, `job` = *<value>*.

            Constraint: `job` != 0 and `job` >= -3 and `job` <= 3.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_tridiag_fac_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04left')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, job.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input, c[n].
        utils._EngineFloat64ArrayType.c_type, # Input, d[n].
        utils._EngineIntArrayType.c_type, # Input, ipiv[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._BYREF_FLOAT64, # Input/Output, tol.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _job = utils._EngineIntScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ipiv = utils._EngineIntArrayType(
        ipiv, {'fun_name': fun_name, 'entity': 'ipiv'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _n,
        _a,
        _b,
        _c,
        _d,
        _ipiv,
        _y,
        _tol,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py > 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow would occur computing the element I in the solution.',
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: job != 0 and job >= -3 and job <= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        _tol_py = utils._EngineFloat64ScalarType.to_py(_tol)
        return_data = _tol_py
    _err_class.manage()
    return return_data

def real_blkdiag_fac_solve(trans, n, nbloks, blkstr, a, lena, pivot, b, ir):
    u"""
    Solution of real almost block diagonal simultaneous linear equations
    (coefficient matrix already factorized by
    ``matop.real_gen_blkdiag_lu``).

    For full information please refer to the NAG Library document for
    f04lh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04lhf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies the equations to be solved.

        `trans` = 'N'

          Solve AX = B.

        `trans` = 'T'

          Solve A^TX = B.

    n : int
        n, the order of the matrix A.

    nbloks : int
        The total number of blocks of the matrix A, as supplied to
        ``real_blkdiag_fac_solve``.

    blkstr : int, shape (3, `nbloks`)
        Information which describes the block structure of A, as
        supplied to ``real_blkdiag_fac_solve``.

    a : float, shape (`lena`)
        The elements in the factorization of A, as returned by
        ``real_blkdiag_fac_solve``.

    lena : int
        The dimension of the array `a`.

    pivot : int, shape (`n`)
        Details of the interchanges in the factorization, as returned by
        ``real_blkdiag_fac_solve``.

    b : float, shape (`n`, `ir`), modified in place
        On entry: the n by r right-hand side matrix B.

        On exit: `b` is overwritten by the n by r solution matrix X.

    ir : int
        r, the number of right-hand sides.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lena` is too small. `lena` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, the following equality does not hold:
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k =
            2,`nbloks`) = `n`.

        (`errno` 1)
            On entry, the following equality does not hold:
            sum(`blkstr`[0,k-1]:k = 1,`nbloks`) = `n`.

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[2,K-1] = *<value>*,
            `blkstr`[2,K-2] = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1]+`blkstr`[2,K-2] >=
            `blkstr`[1,K-1].

        (`errno` 1)
            On entry, the following inequality was not satisfied for: J
            = *<value>*. sum(`blkstr`[1,k-1]-`blkstr`[2,k-1]:k = 1,J) <=
            sum(`blkstr`[0,k-1]:k = 1,J) <=
            `blkstr`[1,0]+sum(`blkstr`[1,k-1]-`blkstr`[2,k-2]:k = 2,J).

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>*,
            `blkstr`[2,K-1] = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1]-`blkstr`[2,K-1] <=
            `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>*, `blkstr`[1,K-1] = *<value>* and
            `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= `blkstr`[0,K-1].

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[2,K-1] = *<value>*.

            Constraint: `blkstr`[2,K-1] >= 0.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[1,K-1] = *<value>*.

            Constraint: `blkstr`[1,K-1] >= 1.

        (`errno` 1)
            On entry, K = *<value>* and `blkstr`[0,K-1] = *<value>*.

            Constraint: `blkstr`[0,K-1] >= 1.

        (`errno` 1)
            On entry, `n` = *<value>* and `nbloks` = *<value>*.

            Constraint: `n` >= `nbloks`.

        (`errno` 1)
            On entry, `nbloks` = *<value>*.

            Constraint: `nbloks` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `ir` = *<value>*.

            Constraint: `ir` >= 1.

        (`errno` 1)
            On entry, `trans` != 'N' or 'T': `trans` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_blkdiag_fac_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04lhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, trans(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nbloks.
        utils._EngineIntArrayType.c_type, # Input, blkstr[3, nbloks].
        utils._EngineFloat64ArrayType.c_type, # Input, a[lena].
        utils._BYREF_INT, # Input, lena.
        utils._EngineIntArrayType.c_type, # Input, pivot[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, ir].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, ir.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of trans.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((blkstr, 'blkstr'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _trans = utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nbloks = utils._EngineIntScalarType(
        nbloks, {'fun_name': fun_name, 'entity': 'nbloks'},
    ).to_ctypes()
    _nbloks_py = utils._EngineIntScalarType.to_py(_nbloks)
    _blkstr = utils._EngineIntArrayType(
        blkstr, {'fun_name': fun_name, 'entity': 'blkstr'},
        exp_shape=(3, _nbloks_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lena = utils._EngineIntScalarType(
        lena, {'fun_name': fun_name, 'entity': 'lena'},
    ).to_ctypes()
    _lena_py = utils._EngineIntScalarType.to_py(_lena)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_lena_py,),
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _pivot = utils._EngineIntArrayType(
        pivot, {'fun_name': fun_name, 'entity': 'pivot'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _ir_py = utils._EngineIntScalarType.to_py(_ir)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _ir_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _trans,
        _n,
        _nbloks,
        _blkstr,
        _a,
        _lena,
        _pivot,
        _b,
        _ldb,
        _ir,
        _errbuf,
        _ierr,
        _trans._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lena is too small.',
                    'lena = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following equality does not hold:',
                    'blkstr[1,0]+sum(blkstr[1,k-1]-blkstr[2,k-2]:k = 2,nbloks) = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following equality does not hold:',
                    'sum(blkstr[0,k-1]:k = 1,nbloks) = n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s}, blkstr[2,K-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'blkstr[2,K-2] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'blkstr[1,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: blkstr[2,K-1]+blkstr[2,K-2] >= blkstr[1,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the following inequality was not satisfied for:',
                    'J = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'sum(blkstr[1,k-1]-blkstr[2,k-1]:k = 1,J) <=',
                    'sum(blkstr[0,k-1]:k = 1,J) <=',
                    'blkstr[1,0]+sum(blkstr[1,k-1]-blkstr[2,k-2]:k = 2,J).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s}, blkstr[1,K-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'blkstr[2,K-1] = {:s} and blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: blkstr[1,K-1]-blkstr[2,K-1] <= blkstr[0,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s}, blkstr[1,K-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: blkstr[1,K-1] >= blkstr[0,K-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[2,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[2,K-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[1,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[1,K-1] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, K = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'blkstr[0,K-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: blkstr[0,K-1] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and nbloks = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= nbloks.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nbloks = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nbloks >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, trans != \'N\' or \'T\': trans = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def real_posdef_vband_solve(n, al, lal, d, nrow, ir, b, iselct, x):
    u"""
    Solution of real symmetric positive definite variable-bandwidth
    simultaneous linear equations (coefficient matrix already factorized
    by ``matop.real_vband_posdef_fac``).

    For full information please refer to the NAG Library document for
    f04mc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mcf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix L.

    al : float, shape (`lal`)
        The elements within the envelope of the lower triangular matrix
        L, taken in row by row order, as returned by
        ``matop.real_vband_posdef_fac``. The unit diagonal elements of L
        must be stored explicitly.

    lal : int
        The dimension of the array `al`.

    d : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `iselct`>=4: 1; otherwise: `n`.

        The diagonal elements of the diagonal matrix D. `d` is not
        referenced if `iselct` >= 4.

    nrow : int, shape (`n`)
        `nrow`[i-1] must contain the width of row i of L, i.e., the
        number of elements between the first (leftmost) nonzero element
        and the element on the diagonal, inclusive.

    ir : int
        r, the number of right-hand sides.

    b : float, shape (`n`, `ir`)
        The n by r right-hand side matrix B. See also [Further
        Comments].

    iselct : int
        Must specify the type of system to be solved, as follows:

        `iselct` = 1

          Solve LDL^TX = B.

        `iselct` = 2

          Solve LDX = B.

        `iselct` = 3

          Solve DL^TX = B.

        `iselct` = 4

          Solve LL^TX = B.

        `iselct` = 5

          Solve LX = B.

        `iselct` = 6

          Solve L^TX = B.

    x : float, shape (`n`, `ir`), modified in place
        On exit: the n by r solution matrix X. See also [Further
        Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lal` = *<value>* and `nrow`[0]+...+`nrow`[`n`-1]
            = *<value>*.

            Constraint: `lal` >= `nrow`[0]+...+`nrow`[`n`-1].

        (`errno` 1)
            On entry, I = *<value>* and `nrow`[I-1] = *<value>*.

            Constraint: `nrow`[I-1] >= 1 and `nrow`[I-1] <= I.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `ir` = *<value>*.

            Constraint: `ir` >= 1.

        (`errno` 3)
            On entry, `iselct` = *<value>*.

            Constraint: `iselct` >= 1 and `iselct` <= 6.

        (`errno` 4)
            On entry, I = *<value>*.

            Constraint: `d`[I-1] != 0.0.

        (`errno` 5)
            At least one diagonal entry of `al` is not unit.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_posdef_vband_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04mcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, al[lal].
        utils._BYREF_INT, # Input, lal.
        utils._EngineFloat64ArrayType.c_type, # Input, d[*].
        utils._EngineIntArrayType.c_type, # Input, nrow[n].
        utils._BYREF_INT, # Input, ir.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, ir].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, iselct.
        utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, ir].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'), (x, 'x')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lal = utils._EngineIntScalarType(
        lal, {'fun_name': fun_name, 'entity': 'lal'},
    ).to_ctypes()
    _lal_py = utils._EngineIntScalarType.to_py(_lal)
    _al = utils._EngineFloat64ArrayType(
        al, {'fun_name': fun_name, 'entity': 'al'},
        exp_shape=(_lal_py,),
    ).to_ctypes()
    _iselct = utils._EngineIntScalarType(
        iselct, {'fun_name': fun_name, 'entity': 'iselct'},
    ).to_ctypes()
    _iselct_py = utils._EngineIntScalarType.to_py(_iselct)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if _iselct_py >= 4:
        _d_dim1 = 1
    else:
        _d_dim1 = _n_py
    exp_shape_d = (_d_dim1,)
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=exp_shape_d,
    ).to_ctypes()
    _nrow = utils._EngineIntArrayType(
        nrow, {'fun_name': fun_name, 'entity': 'nrow'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _ir_py = utils._EngineIntScalarType.to_py(_ir)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _ir_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _ir_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _al,
        _lal,
        _d,
        _nrow,
        _ir,
        _b,
        _ldb,
        _iselct,
        _x,
        _ldx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lal = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nrow[0]+...+nrow[n-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lal >= nrow[0]+...+nrow[n-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s} and nrow[I-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nrow[I-1] >= 1 and nrow[I-1] <= I.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iselct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iselct >= 1 and iselct <= 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: d[I-1] != 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one diagonal entry of al is not unit.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def real_toeplitz_yule_update(n, t, x, v):
    u"""
    Update solution of the Yule--Walker equations for real symmetric
    positive definite Toeplitz matrix.

    For full information please refer to the NAG Library document for
    f04me

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mef.html

    Parameters
    ----------
    n : int
        The order of the Toeplitz matrix T.

    t : float, shape (`n`+1)
        `t`[0] must contain the value tau_0 of the diagonal elements of
        T, and the remaining `n` elements of `t` must contain the
        elements of the vector t_n.

    x : float, shape (`n`), modified in place
        On entry: with `n` > 1 the (n-1) elements of the solution vector
        x_{n-1} as returned by a previous call to
        ``real_toeplitz_yule_update``. The element `x`[`n`-1] need not
        be specified.

        On exit: the solution vector x_n. The element `x`[`n`-1] returns
        the partial (auto)correlation coefficient, or reflection
        coefficient, for the nth step. If |`x`[`n`-1]| >= 1.0, the
        matrix T_{n+1} will not be positive definite to working
        accuracy.

    v : float
        With `n` > 1 the mean square prediction error for the (n-1)th
        step, as returned by a previous call to
        ``real_toeplitz_yule_update``.

    Returns
    -------
    v : float
        The mean square prediction error, or predictor error variance
        ratio, nu_n, for the nth step. (See [Further Comments] and [the
        G13 Introduction].)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `n` = *<value>* and `x`[`n`-2] = *<value>*.

            Constraint: if `n` > 1, abs(`x`[`n`-2]) < 1.0.

        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Matrix of order *<value>* would not be positive definite.
            Value of the reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_toeplitz_yule_update'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04meft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n+1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._BYREF_FLOAT64, # Input/Output, v.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _v = utils._EngineFloat64ScalarType(
        v, {'fun_name': fun_name, 'entity': 'v'},
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _x,
        _v,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and x[n-2] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if n > 1, abs(x[n-2]) < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Matrix of order {:s} would not be positive definite.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Value of the reflection coefficient is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _v_py = utils._EngineFloat64ScalarType.to_py(_v)
        return_data = _v_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['v'],
            )(
                _v_py,
            )
    _err_class.manage()
    return return_data

def real_toeplitz_update(n, t, b, x, work):
    u"""
    Update solution of real symmetric positive definite Toeplitz system.

    For full information please refer to the NAG Library document for
    f04mf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04mff.html

    Parameters
    ----------
    n : int
        The order of the Toeplitz matrix T.

    t : float, shape (`n`)
        `t`[i] must contain the value tau_i, for i = 0,1,...,`n`-1.

    b : float, shape (`n`)
        The right-hand side vector b_n.

    x : float, shape (`n`), modified in place
        On entry: with `n` > 1 the (n-1) elements of the solution vector
        x_{n-1} as returned by a previous call to
        ``real_toeplitz_update``. The element `x`[`n`-1] need not be
        specified.

        On exit: the solution vector x_n.

    work : float, shape (2*`n`-1), modified in place
        On entry: with `n` > 2 the elements of `work` should be as
        returned from a previous call to ``real_toeplitz_update`` with
        (`n`-1) as the argument `n`.

        On exit: the first (`n`-1) elements of `work` contain the
        solution to the Yule--Walker equations

            T_{n-1}y_{n-1} = -t_{n-1},

        where t_{n-1} = (tau_1tau_2...tau_{n-1})^`t`.

    Returns
    -------
    p : float
        The reflection coefficient p_{n-1}. (See [Further Comments].)

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, `t`[0] = *<value>*.

            Constraint: `t`[0] > 0.0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The Toeplitz Matrix is not positive definite Value of the
            reflection coefficient is *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_toeplitz_update'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04mfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[2*n-1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _work_cls = utils._EngineFloat64ArrayType(
        work, {'fun_name': fun_name, 'entity': 'work'},
        exp_shape=(2*_n_py-1,),
    )
    _work = _work_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _b,
        _x,
        _p,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: t[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Toeplitz Matrix is not positive definite',
                    'Value of the reflection coefficient is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _work_cls.refresh_py(_work)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def real_gen_sparse_lsqsol(m, n, b, x, se, aprod, damp, atol, btol, conlim, itnlim, msglvl, data=None, io_manager=None):
    u"""
    Sparse linear least squares problem, m real equations in n unknowns.

    For full information please refer to the NAG Library document for
    f04qa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04qaf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    b : float, shape (`m`), modified in place
        On entry: the right-hand side vector b.

        On exit: `b` is overwritten.

    x : float, shape (`n`), modified in place
        On exit: the solution vector x.

    se : float, shape (`n`), modified in place
        On exit: the estimates of the standard errors of the components
        of x. Thus `se`[i-1] contains an estimate of sqrt(nu_{ii}),
        where nu_{ii} is the ith diagonal element of the estimated
        variance-covariance matrix V. The estimates returned in `se`
        will be the lower bounds on the actual estimated standard
        errors, but will usually be correct to at least one significant
        figure.

    aprod : callable mode = aprod(mode, m, n, x, y, data=None)
        `aprod` must perform the operations y := y+Ax and x := x+A^Ty
        for given vectors x and y.

        Parameters
        ~~~~~~~~~~
        mode : int
            Specifies which operation is to be performed.

            `mode` = 1

              `aprod` must compute y+Ax.

            `mode` = 2

              `aprod` must compute x+A^Ty.

        m : int
            m, the number of rows of A.

        n : int
            n, the number of columns of A.

        x : float, ndarray, shape (`n`), to be modified in place
            On entry: the vector x.

            On exit: if `mode` = 1, `x` must be unchanged.

            If `mode` = 2, `x` must contain x+A^Ty.

        y : float, ndarray, shape (`m`), to be modified in place
            On entry: the vector y.

            On exit: if `mode` = 1, `y` must contain y+Ax.

            If `mode` = 2, `y` must be unchanged.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        mode : int
            May be used as a flag to indicate a failure in the
            computation of y+Ax or x+A^Ty. If `mode` is negative on exit
            from `aprod`, ``real_gen_sparse_lsqsol`` will exit
            immediately with errno set to `mode`.

    damp : float
        The value lambda. If either problem (1) or problem (2) is to be
        solved, `damp` must be supplied as zero.

    atol : float
        The tolerance, tol_1, of the convergence criteria (6) and (7);
        it should be an estimate of the largest relative error in the
        elements of A. For example, if the elements of A are correct to
        about 4 significant figures, then `atol` should be set to about
        5*10^(-4). If `atol` is supplied as less than epsilon, where
        epsilon is the machine precision, the value epsilon is used
        instead of `atol`.

    btol : float
        The tolerance, tol_2, of the convergence criterion (6); it
        should be an estimate of the largest relative error in the
        elements of B. For example, if the elements of B are correct to
        about 4 significant figures, then `btol` should be set to about
        5*10^(-4). If `btol` is supplied as less than epsilon, the value
        epsilon is used instead of `btol`.

    conlim : float
        The value c_lim of equation (8); it should be an upper limit on
        the condition number of A_. `conlim` should not normally be
        chosen much larger than 1.0/`atol`. If `conlim` is supplied as
        zero, the value 1.0/epsilon is used instead of `conlim`.

    itnlim : int
        An upper limit on the number of iterations. If `itnlim` <= 0,
        the value `n` is used in place of `itnlim`, but for
        ill-conditioned problems a higher value of `itnlim` is likely to
        be necessary.

    msglvl : int
        The level of printing from ``real_gen_sparse_lsqsol``. If
        `msglvl` <= 0, then no printing occurs, but otherwise messages
        will be output on the file object associated with the advisory
        I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`). A
        description of the printed output is given in [Further
        Comments]. The level of printing is determined as follows:

        `msglvl` <= 0

          No printing.

        `msglvl` = 1

          A brief summary is printed just prior to return from
          ``real_gen_sparse_lsqsol``.

        `msglvl` >= 2

          A summary line is printed periodically to monitor the progress
          of ``real_gen_sparse_lsqsol``, together with a brief summary
          just prior to return from ``real_gen_sparse_lsqsol``.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    itnlim : int
        Unchanged unless `itnlim` <= 0 on entry, in which case it is set
        to `n`.

    itn : int
        The number of iterations performed.

    anorm : float
        An estimate of ||A_|| for the matrix A_ of (4).

    acond : float
        An estimate of cond(A_) which is a lower bound.

    rnorm : float
        An estimate of ||r_|| for the residual, r_, of (5) corresponding
        to the solution x returned in `x`. Note that ||r_|| is the
        function being minimized.

    arnorm : float
        An estimate of the ||A_^Tr_|| corresponding to the solution x
        returned in `x`.

    xnorm : float
        An estimate of ||x|| for the solution x returned in `x`.

    inform : int
        The reason for termination of ``real_gen_sparse_lsqsol``.

        `inform` = 0

          The exact solution is x = 0. No iterations are performed in
          this case.

        `inform` = 1

          The termination criterion of (6) has been satisfied with tol_1
          and tol_2 as the values supplied in `atol` and `btol`
          respectively.

        `inform` = 2

          The termination criterion of (7) has been satisfied with tol_1
          as the value supplied in `atol`.

        `inform` = 3

          The termination criterion of (6) has been satisfied with tol_1
          and/or tol_2 as the value epsilon, where epsilon is the
          machine precision. One or both of the values supplied in
          `atol` and `btol` must have been less than epsilon and was too
          small for this machine.

        `inform` = 4

          The termination criterion of (7) has been satisfied with tol_1
          as the value epsilon. The value supplied in `atol` must have
          been less than epsilon and was too small for this machine.

        The values `inform` = 5, 6 or 7 correspond to failure with
        `errno` = 2, 3 and 4 respectively (see [Errors]) and when errno
        is negative `inform` will be set to the same negative value.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            Termination criteria not satisfied, but the condition number
            bound supplied in `conlim` has been met.

        (`errno` 3)
            Termination criteria not satisfied, but the condition number
            is bounded by 1/``machine.precision``.

        (`errno` 4)
            Maximum number of iterations reached.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i < 0)
            `mode` in `aprod` has been set to: *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_gen_sparse_lsqsol'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_aprod_name = fun_name + ':aprod'

    aprodh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, mode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[m].
        utils._BYREF_INT, # Input, lruser.
        utils._BYREF_INT, # Input, liuser.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_aprodh(_aprod, en_data, mode, m, n, x, y, lruser, liuser, _iuser, _ruser):
        """Helper for aprod."""
        try:
            _mode_aprodh = utils._EngineIntScalarType.to_py(mode.contents)
            _m_aprodh = utils._EngineIntScalarType.to_py(m.contents)
            _n_aprodh = utils._EngineIntScalarType.to_py(n.contents)
            _x_aprodh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_aprodh,),
            )
            _y_aprodh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_m_aprodh,),
            )
            if data is not None:
                _mode_aprodh = aprod(
                    _mode_aprodh,
                    _m_aprodh,
                    _n_aprodh,
                    _x_aprodh,
                    _y_aprodh,
                    data,
                )
            else:
                _mode_aprodh = aprod(
                    _mode_aprodh,
                    _m_aprodh,
                    _n_aprodh,
                    _x_aprodh,
                    _y_aprodh,
                )
            utils._EngineIntScalarType(
                _mode_aprodh, {'fun_name': cb_aprod_name, 'returned': 'mode'},
            ).refresh_ctypes_buffer(mode)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _x_aprodh, {'fun_name': cb_aprod_name, 'returned': 'x'}, x,
            )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_aprodh, {'fun_name': cb_aprod_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f04qaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[m].
        utils._EngineFloat64ArrayType.c_type, # Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, se[n].
        utils._UNUSED_FUNCTYPE,
        aprodh,
        utils._BYREF_FLOAT64, # Input, damp.
        utils._BYREF_FLOAT64, # Input, atol.
        utils._BYREF_FLOAT64, # Input, btol.
        utils._BYREF_FLOAT64, # Input, conlim.
        utils._BYREF_INT, # Input/Output, itnlim.
        utils._BYREF_INT, # Input, msglvl.
        utils._BYREF_INT, # Input, nadv.
        utils._BYREF_INT, # Output, itn.
        utils._BYREF_FLOAT64, # Output, anorm.
        utils._BYREF_FLOAT64, # Output, acond.
        utils._BYREF_FLOAT64, # Output, rnorm.
        utils._BYREF_FLOAT64, # Output, arnorm.
        utils._BYREF_FLOAT64, # Output, xnorm.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lruser.
        utils._BYREF_INT, # Input, liuser.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Output, inform.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py,),
    )
    _b = _b_cls.to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _se_cls = utils._EngineFloat64ArrayType(
        se, {'fun_name': fun_name, 'entity': 'se'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _se = _se_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(aprod, {'fun_name': fun_name, 'entity': 'aprod'})
    _aprod = utils._UNUSED_CALLBACK
    _aprodh = utils._cast_callback(
        aprodh, cb_aprodh,
        {'fun_name': fun_name, 'entity': 'aprod'},
    )
    _damp = utils._EngineFloat64ScalarType(
        damp, {'fun_name': fun_name, 'entity': 'damp'},
    ).to_ctypes()
    _atol = utils._EngineFloat64ScalarType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
    ).to_ctypes()
    _btol = utils._EngineFloat64ScalarType(
        btol, {'fun_name': fun_name, 'entity': 'btol'},
    ).to_ctypes()
    _conlim = utils._EngineFloat64ScalarType(
        conlim, {'fun_name': fun_name, 'entity': 'conlim'},
    ).to_ctypes()
    _itnlim = utils._EngineIntScalarType(
        itnlim, {'fun_name': fun_name, 'entity': 'itnlim'},
    ).to_ctypes()
    _msglvl = utils._EngineIntScalarType(
        msglvl, {'fun_name': fun_name, 'entity': 'msglvl'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _itn = utils._EngineIntScalarType.empty_ctypes()
    _anorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _acond = utils._EngineFloat64ScalarType.empty_ctypes()
    _rnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _arnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _xnorm = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lruser = utils._EngineIntScalarType.c_type(0)
    _liuser = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _inform = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _m,
        _n,
        _b,
        _x,
        _se,
        _aprod,
        _aprodh,
        _damp,
        _atol,
        _btol,
        _conlim,
        _itnlim,
        _msglvl,
        _nadv,
        _itn,
        _anorm,
        _acond,
        _rnorm,
        _arnorm,
        _xnorm,
        _work,
        _lruser,
        _liuser,
        _iuser,
        _ruser,
        _inform,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py < 0 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'mode in aprod has been set to: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination criteria not satisfied, but the condition number',
                    'bound supplied in conlim has been met.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination criteria not satisfied, but the condition number',
                    'is bounded by 1/machine.precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Maximum number of iterations reached.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        _x_cls.refresh_py(_x)
        _se_cls.refresh_py(_se)
        _itnlim_py = utils._EngineIntScalarType.to_py(_itnlim)
        _itn_py = utils._EngineIntScalarType.to_py(_itn)
        _anorm_py = utils._EngineFloat64ScalarType.to_py(_anorm)
        _acond_py = utils._EngineFloat64ScalarType.to_py(_acond)
        _rnorm_py = utils._EngineFloat64ScalarType.to_py(_rnorm)
        _arnorm_py = utils._EngineFloat64ScalarType.to_py(_arnorm)
        _xnorm_py = utils._EngineFloat64ScalarType.to_py(_xnorm)
        _inform_py = utils._EngineIntScalarType.to_py(_inform)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['itnlim', 'itn', 'anorm', 'acond', 'rnorm', 'arnorm', 'xnorm', 'inform'],
        )(
            _itnlim_py,
            _itn_py,
            _anorm_py,
            _acond_py,
            _rnorm_py,
            _arnorm_py,
            _xnorm_py,
            _inform_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_gen_lsq_covmat(job, p, sigma, a, svd, irank, sv, cj):
    u"""
    Covariance matrix for linear least squares problems, m real
    equations in n unknowns.

    For full information please refer to the NAG Library document for
    f04ya

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04yaf.html

    Parameters
    ----------
    job : int
        Specifies which elements of C are required.

        `job` = -1

          The upper triangular part of C is required.

        `job` = 0

          The diagonal elements of C are required.

        `job` > 0

          The elements of column `job` of C are required.

    p : int
        p, the order of the variance-covariance matrix C.

    sigma : float
        The standard error of the residual vector.

    a : float, shape (:, `p`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if not((`svd`) or `job`=-1: `p`; if
        `svd`= True and `job`>=0: `irank`; otherwise: 0.

        On entry: if `svd` = False, `a` must contain the upper
        triangular matrix U of the QU factorization of X, or of the
        Cholesky factorization of X^TX; elements of the array below the
        diagonal need not be set.

        If `svd` = True, A must contain the first k rows of the matrix
        V^T, where k is the rank of X and V is the right-hand orthogonal
        matrix of the singular value decomposition of X.

        Thus the ith row must contain the ith right-hand singular vector
        of X.

        On exit: if `job` >= 0, A is unchanged.

        If `job` = -1, `a` contains the upper triangle of the symmetric
        matrix C.

        If `svd` = True, elements of the array below the diagonal are
        used as workspace.

        If `svd` = False, they are unchanged.

    svd : bool
        Must be True if the least squares solution was obtained from a
        singular value decomposition of X. `svd` must be False if the
        least squares solution was obtained from either a QU
        factorization of X or a Cholesky factorization of X^TX. In the
        latter case the rank of X is assumed to be p and so is
        applicable only to full rank problems with n >= p.

    irank : int
        If `svd` = True, `irank` must specify the rank k of the matrix
        X.

        If `svd` = False, `irank` is not referenced and the rank of X is
        assumed to be p.

    sv : float, shape (`p`)
        If `svd` = True, `sv` must contain the first `irank` singular
        values of X.

        If `svd` = False, `sv` is not referenced.

    cj : float, shape (`p`), modified in place
        On exit: if `job` = 0, `cj` returns the diagonal elements of C.

        If `job` = j > 0, `cj` returns the jth column of C.

        If `job` = -1, `cj` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lda = *<value>* and `p` = *<value>*.

            Constraint: if `svd` = False, lda >= `p`.

        (`errno` 1)
            On entry, lda = *<value>* and `p` = *<value>*.

            Constraint: if `svd` = True and `job` = -1, lda >= `p`.

        (`errno` 1)
            On entry, `job` = *<value>*, lda = *<value>* and `irank` =
            *<value>*.

            Constraint: if `svd` = True and `job` >= 0, lda >= max(1,
            `irank`).

        (`errno` 1)
            On entry, `irank` = *<value>* and `p` = *<value>*.

            Constraint: if `svd` = True, `irank` >= 0 and `irank` <=
            `p`.

        (`errno` 1)
            On entry, `job` = *<value>* and `p` = *<value>*.

            Constraint: `job` >= -1 and `job` <= `p`.

        (`errno` 1)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` >= 0.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 1.

        (`errno` 2)
            On entry, `irank` = *<value>*.

            Constraint: if `svd` = True, `irank` > 0.

        (`errno` 3)
            On entry, `svd` = False and overflow would occur in
            calculating an element of C. The upper triangular matrix U
            must be very nearly singular.

        (`errno` 4)
            On entry, `svd` = False and one of the first `irank`
            singular values is zero. Either the first `irank` singular
            values or `irank` must be incorrect: `irank` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_gen_lsq_covmat'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04yaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, job.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, p].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_LOGICAL, # Input, svd.
        utils._BYREF_INT, # Input, irank.
        utils._EngineFloat64ArrayType.c_type, # Input, sv[p].
        utils._EngineFloat64ArrayType.c_type, # Output, cj[p].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _job = utils._EngineIntScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'},
    ).to_ctypes()
    _p = utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _svd = utils._EngineLogicalScalarType(
        svd, {'fun_name': fun_name, 'entity': 'svd'},
    ).to_ctypes()
    _svd_py = utils._EngineLogicalScalarType.to_py(_svd)
    _job_py = utils._EngineIntScalarType.to_py(_job)
    _p_py = utils._EngineIntScalarType.to_py(_p)
    _irank = utils._EngineIntScalarType(
        irank, {'fun_name': fun_name, 'entity': 'irank'},
    ).to_ctypes()
    _irank_py = utils._EngineIntScalarType.to_py(_irank)
    if not _svd_py or _job_py == -1:
        _a_dim1 = _p_py
    elif _svd_py and _job_py >= 0:
        _a_dim1 = _irank_py
    else:
        _a_dim1 = 0
    exp_shape_a = (_a_dim1, _p_py)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=exp_shape_a,
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _sv = utils._EngineFloat64ArrayType(
        sv, {'fun_name': fun_name, 'entity': 'sv'},
        exp_shape=(_p_py,),
    ).to_ctypes()
    _cj_cls = utils._EngineFloat64ArrayType(
        cj, {'fun_name': fun_name, 'entity': 'cj'},
        exp_shape=(_p_py,),
        check_contents=False,
    )
    _cj = _cj_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _job,
        _p,
        _sigma,
        _a,
        _lda,
        _svd,
        _irank,
        _sv,
        _cj,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if svd = False, lda >= p.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lda = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if svd = True and job = -1, lda >= p.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job = {:s}, lda = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'irank = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if svd = True and job >= 0, lda >= max(1, irank).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irank = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if svd = True, irank >= 0 and irank <= p.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, job = {:s} and p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: job >= -1 and job <= p.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sigma >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irank = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if svd = True, irank > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, svd = False and overflow would occur in calculating',
                    'an element of C. The upper triangular matrix U must be very',
                    'nearly singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, svd = False and one of the first irank singular',
                    'values is zero. Either the first irank singular values or',
                    'irank must be incorrect:',
                    'irank = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _cj_cls.refresh_py(_cj)
        return_data = None
    _err_class.manage()
    return return_data

def real_gen_norm_rcomm(irevcm, m, n, x, y, estnrm, t, seed, comm):
    u"""
    Norm estimation (for use in condition estimation), real rectangular
    matrix.

    For full information please refer to the NAG Library document for
    f04yd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04ydf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

        On intermediate entry: `irevcm` must be unchanged.

    m : int
        The number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    x : float, shape (`n`, `t`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 1, contains the current
        matrix X.

        On intermediate entry: if `irevcm` = 2, must contain A^TY.

        On final exit: the array is undefined.

    y : float, shape (`m`, `t`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current
        matrix Y.

        On intermediate entry: if `irevcm` = 1, must contain AX.

        On final exit: the array is undefined.

    estnrm : float
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    t : int
        The number of columns t of the matrices X and Y. This is an
        argument that can be used to control the accuracy and
        reliability of the estimate and corresponds roughly to the
        number of columns of A that are visited during each iteration of
        the algorithm.

        If `t` >= 2 then a partly random starting matrix is used in the
        algorithm.

    seed : int
        The seed used for random number generation.

        If `t` = 1, `seed` is not used.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1 or 2, and `x` contains the
        n*t matrix X and `y` contains the m*t matrix Y. The calling
        program must

        - if `irevcm` = 1, evaluate AX and store the result in `y`

          or

          if `irevcm` = 2, evaluate A^TY and store the result in `x`,

        - call ``real_gen_norm_rcomm`` once again, with all the other
          arguments unchanged.

        On final exit: `irevcm` = 0.

    estnrm : float
        On final exit: an estimate (a lower bound) for ||A||_1.

    Raises
    ------
    NagValueError
        (`errno` -10)
            On entry, `t` = *<value>* and `seed` = *<value>*.

            Constraint: if `t` > 1, `seed` >= 1.

        (`errno` -9)
            On entry, `m` = *<value>* and `t` = *<value>*.

            Constraint: 1 <= `t` <= `m`.

        (`errno` -3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            Internal error; please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.real_gen_norm_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04ydft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, t].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[ldy, t].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_FLOAT64, # Input/Output, estnrm.
        utils._BYREF_INT, # Input, t.
        utils._BYREF_INT, # Input, seed.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[m*t].
        utils._EngineIntArrayType.c_type, # Input/Output, iwork[2*n+5*t+20].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _t = utils._EngineIntScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t_py = utils._EngineIntScalarType.to_py(_t)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _t_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineFloat64ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py, _t_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _estnrm = utils._EngineFloat64ScalarType(
        estnrm, {'fun_name': fun_name, 'entity': 'estnrm'},
    ).to_ctypes()
    _seed = utils._EngineIntScalarType(
        seed, {'fun_name': fun_name, 'entity': 'seed'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('work', 'iwork'),
    )
    if _comm_needs_init:
        _work = utils._EngineFloat64ArrayType.empty_ctypes(_m_py*_t_py)
        comm['work'] = _work
    else:
        _work = comm['work']
    if _comm_needs_init:
        _iwork = utils._EngineIntArrayType.empty_ctypes(2*_n_py+5*_t_py+20)
        comm['iwork'] = _iwork
    else:
        _iwork = comm['iwork']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _m,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _estnrm,
        _t,
        _seed,
        _work,
        _iwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s} and seed = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if t > 1, seed >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -9 and _ifmt == 1239:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= t <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 1236:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error; please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _estnrm_py = utils._EngineFloat64ScalarType.to_py(_estnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'estnrm'],
        )(
            _irevcm_py,
            _estnrm_py,
        )
    _err_class.manage()
    return return_data

def complex_gen_norm_rcomm(irevcm, m, n, x, y, estnrm, t, seed, comm):
    u"""
    Norm estimation (for use in condition estimation), complex
    rectangular matrix.

    For full information please refer to the NAG Library document for
    f04zd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f04/f04zdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must be set to 0.

        On intermediate entry: `irevcm` must be unchanged.

    m : int
        The number of rows of the matrix A.

    n : int
        On initial entry: n, the number of columns of the matrix A.

    x : complex, shape (`n`, `t`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 1, contains the current
        matrix X.

        On intermediate entry: if `irevcm` = 2, must contain A^HY.

        On final exit: the array is undefined.

    y : complex, shape (`m`, `t`), modified in place
        On initial entry: need not be set.

        On intermediate exit: if `irevcm` = 2, contains the current
        matrix Y.

        On intermediate entry: if `irevcm` = 1, must contain AX.

        On final exit: the array is undefined.

    estnrm : float
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    t : int
        The number of columns t of the matrices X and Y. This is an
        argument that can be used to control the accuracy and
        reliability of the estimate and corresponds roughly to the
        number of columns of A that are visited during each iteration of
        the algorithm.

        If `t` >= 2 then a partly random starting matrix is used in the
        algorithm.

    seed : int
        The seed used for random number generation.

        If `t` = 1, `seed` is not used.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 1 or 2, and `x` contains the
        n*t matrix X and `y` contains the m*t matrix Y. The calling
        program must

        - if `irevcm` = 1, evaluate AX and store the result in `y`

          or

          if `irevcm` = 2, evaluate A^HY and store the result in `x`,
          where A^H is the complex conjugate transpose;

        - call ``complex_gen_norm_rcomm`` once again, with all the
          arguments unchanged.

        On final exit: `irevcm` = 0.

    estnrm : float
        On final exit: an estimate (a lower bound) for ||A||_1.

    Raises
    ------
    NagValueError
        (`errno` -10)
            On entry, `t` = *<value>* and `seed` = *<value>*.

            Constraint: if `t` > 1, `seed` >= 1.

        (`errno` -9)
            On entry, `m` = *<value>* and `t` = *<value>*.

            Constraint: 1 <= `t` <= `m`.

        (`errno` -3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` -1)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            Internal error; please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.linsys.complex_gen_norm_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f04zdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, x[ldx, t].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, y[ldy, t].
        utils._BYREF_INT, # Input, ldy.
        utils._BYREF_FLOAT64, # Input/Output, estnrm.
        utils._BYREF_INT, # Input, t.
        utils._BYREF_INT, # Input, seed.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, work[m*t].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[2*n].
        utils._EngineIntArrayType.c_type, # Input/Output, iwork[2*n+5*t+20].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineComplex128ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _t = utils._EngineIntScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t_py = utils._EngineIntScalarType.to_py(_t)
    _x_cls = utils._EngineComplex128ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _t_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineComplex128ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _y_cls = utils._EngineComplex128ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py, _t_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _estnrm = utils._EngineFloat64ScalarType(
        estnrm, {'fun_name': fun_name, 'entity': 'estnrm'},
    ).to_ctypes()
    _seed = utils._EngineIntScalarType(
        seed, {'fun_name': fun_name, 'entity': 'seed'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('work', 'rwork', 'iwork'),
    )
    if _comm_needs_init:
        _work = utils._EngineComplex128ArrayType.empty_ctypes(_m_py*_t_py)
        comm['work'] = _work
    else:
        _work = comm['work']
    if _comm_needs_init:
        _rwork = utils._EngineFloat64ArrayType.empty_ctypes(2*_n_py)
        comm['rwork'] = _rwork
    else:
        _rwork = comm['rwork']
    if _comm_needs_init:
        _iwork = utils._EngineIntArrayType.empty_ctypes(2*_n_py+5*_t_py+20)
        comm['iwork'] = _iwork
    else:
        _iwork = comm['iwork']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _m,
        _n,
        _x,
        _ldx,
        _y,
        _ldy,
        _estnrm,
        _t,
        _seed,
        _work,
        _rwork,
        _iwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s} and seed = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if t > 1, seed >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -9 and _ifmt == 1239:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= t <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 1236:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 1235:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1234:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error; please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _estnrm_py = utils._EngineFloat64ScalarType.to_py(_estnrm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'estnrm'],
        )(
            _irevcm_py,
            _estnrm_py,
        )
    _err_class.manage()
    return return_data
