# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `ode` Chapter.

``ode`` - Ordinary Differential Equations

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the numerical solution of ordinary
differential equations.
There are two main types of problem: those in which all boundary
conditions are specified at one point (initial value problems), and
those in which the boundary conditions are distributed between two or
more points (boundary value problems and eigenvalue problems). Functions
are available for initial value problems, two-point boundary value
problems and Sturm--Liouville eigenvalue problems.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def bvp_shoot_genpar_intern(h, e, parerr, param, c, n, n1, m1, aux, bcaux, raaux, prsol, data=None):
    u"""
    Ordinary differential equations, boundary value problem, shooting
    and matching technique, allowing interior matching point, general
    parameters to be determined.

    For full information please refer to the NAG Library document for
    d02ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02agf.html

    Parameters
    ----------
    h : float
        `h` must be set to an estimate of the step size, h, needed for
        integration.

    e : float, shape (`n`)
        `e`[i-1] must be set to a small quantity to control the ith
        solution component. The element `e`[i-1] is used:

        - in the bound on the local error in the ith component of the
          solution y_i during integration,

        - in the convergence test on the ith component of the solution
          y_i at the matching point in the Newton iteration.

        The elements `e`[i-1] should not be chosen too small.

        They should usually be several orders of magnitude larger than
        machine precision.

    parerr : float, shape (`n1`)
        `parerr`[i-1] must be set to a small quantity to control the ith
        parameter component. The element `parerr`[i-1] is used:

        - in the convergence test on the ith parameter in the Newton
          iteration,

        - in perturbing the ith parameter when approximating the
          derivatives of the components of the solution with respect to
          the ith parameter, for use in the Newton iteration.

        The elements `parerr`[i-1] should not be chosen too small.

        They should usually be several orders of magnitude larger than
        machine precision.

    param : float, shape (`n1`), modified in place
        On entry: `param`[i-1] must be set to an estimate for the ith
        parameter, p_i, for i = 1,2,...,`n1`.

        On exit: the corrected value for the ith parameter, unless an
        error has occurred, when it contains the last calculated value
        of the parameter (possibly perturbed by
        `parerr`[i-1]*(1+|`param`[i-1]|) if the error occurred when
        calculating the approximate derivatives).

    c : float, shape (`m1`, `n`), modified in place
        On exit: the solution when `m1` > 1 (see `m1`).

        If `m1` = 1, the elements of `c` are not used.

    n : int
        n, the total number of differential equations.

    n1 : int
        n_1, the number of parameters.

        If `n1` < `n`, the last `n`-`n1` differential equations (in
        `aux`) are driving equations (see [Description]).

    m1 : int
        Determines whether or not the final solution is computed as well
        as the parameter values.

        `m1` = 1

          The final solution is not calculated;

        `m1` > 1

          The final values of the solution at interval (length of
          range)/(`m1`-1) are calculated and stored sequentially in the
          array `c` starting with the values of y_i evaluated at the
          first end point (see `raaux`) stored in `c`[0,i-1].

    aux : callable aux(n, n1, f, y, x, param, data=None)
        `aux` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments, x,y_1,...,y_n,
        p_1,...,p_n_1.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        n1 : int
            n_1, the number of parameters.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        x : float
            x, the value of the argument.

        param : float, ndarray, shape (`n1`)
            p_i, for i = 1,2,...,n_1, the value of the parameters.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    bcaux : callable bcaux(n, n1, g0, g1, param, data=None)
        `bcaux` must evaluate the values of y_i at the end points of the
        range given the values of p_1,...,p_n_1.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        n1 : int
            n_1, the number of parameters.

        g0 : float, ndarray, shape (`n`), to be modified in place
            On exit: the values y_i, for i = 1,2,...,n, at the boundary
            point x_0 (see `raaux`).

        g1 : float, ndarray, shape (`n`), to be modified in place
            On exit: the values y_i, for i = 1,2,...,n, at the boundary
            point x_1 (see `raaux`).

        param : float, ndarray, shape (`n1`)
            p_i, for i = 1,2,...,n, the value of the parameters.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    raaux : callable (x0, x1, r) = raaux(n1, param, data=None)
        `raaux` must evaluate the end points, x_0 and x_1, of the range
        and the matching point, r, given the values p_1,p_2,...,p_n_1.

        Parameters
        ~~~~~~~~~~
        n1 : int
            n_1, the number of parameters.

        param : float, ndarray, shape (`n1`)
            p_i, for i = 1,2,...,n_1, the value of the parameters.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        x0 : float
            Must contain the left-hand end of the range, x_0.

        x1 : float
            Must contain the right-hand end of the range x_1.

        r : float
            Must contain the matching point, r.

    prsol : callable prsol(param, res, n1, err, data=None)
        `prsol` is called at each iteration of the Newton method and can
        be used to print the current values of the parameters p_i, for i
        = 1,2,...,n_1, their errors, e_i, and the sum of squares of the
        errors at the matching point, r.

        Parameters
        ~~~~~~~~~~
        param : float, ndarray, shape (`n1`)
            p_i, for i = 1,2,...,n_1, the current value of the
            parameters.

        res : float
            The sum of squares of the errors in the arguments, sum_i =
            1^n_1e_i^2.

        n1 : int
            n_1, the number of parameters.

        err : float, ndarray, shape (`n1`)
            The errors in the parameters, e_i, for i = 1,2,...,n_1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    h : float
        The last step length used.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>* and `n` = *<value>*.

            Constraint: `n1` <= `n`.

        (`errno` 2)
            No further progress can be made when stepping to obtain a
            Jacobian update for the current parameter values. Step
            length `h` = *<value>*.

        (`errno` 3)
            Currently the matching point r does not lie in range
            [x_0,x_1].

            If x_0, x_1 or r depend on the parameters then this may
            occur when care is not taken to avoid it.

            r = *<value>*, x_0 = *<value>* and x_1 = *<value>*.

        (`errno` 4)
            No further progress can be made when stepping to a solution
            corresponding to the current parameter values.

            Step length `h` = *<value>*.

        (`errno` 5)
            The Jacobian for parameter corrections is singular.

        (`errno` 6)
            The Newton method failed to converge while updating
            parameter values.

        (`errno` 7)
            The Newton method has not converged after 12 iterations
            while updating parameter values.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_shoot_genpar_intern'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_aux_name = fun_name + ':aux'

    auxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, param[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_auxh(_aux, en_data, n, n1, f, y, x, param, _iuser, _ruser):
        """Helper for aux."""
        try:
            _n_auxh = utils._EngineIntScalarType.to_py(n.contents)
            _n1_auxh = utils._EngineIntScalarType.to_py(n1.contents)
            _f_auxh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_auxh,),
                can_reference_contents=False,
            )
            _y_auxh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_auxh,),
            )
            _x_auxh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _param_auxh = utils._EngineFloat64ArrayType.from_pointer(
                param,
                (_n1_auxh,),
            )
            if data is not None:
                aux(
                    _n_auxh,
                    _n1_auxh,
                    _f_auxh,
                    _y_auxh,
                    _x_auxh,
                    _param_auxh,
                    data,
                )
            else:
                aux(
                    _n_auxh,
                    _n1_auxh,
                    _f_auxh,
                    _y_auxh,
                    _x_auxh,
                    _param_auxh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_auxh, {'fun_name': cb_aux_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._unlink_pointer(y)
            utils._unlink_pointer(param)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bcaux_name = fun_name + ':bcaux'

    bcauxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Output, g0[n].
        utils._EngineFloat64ArrayType.c_type, # Output, g1[n].
        utils._EngineFloat64ArrayType.c_type, # Input, param[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bcauxh(_bcaux, en_data, n, n1, g0, g1, param, _iuser, _ruser):
        """Helper for bcaux."""
        try:
            _n_bcauxh = utils._EngineIntScalarType.to_py(n.contents)
            _n1_bcauxh = utils._EngineIntScalarType.to_py(n1.contents)
            _g0_bcauxh = utils._EngineFloat64ArrayType.from_pointer(
                g0,
                (_n_bcauxh,),
                can_reference_contents=False,
            )
            _g1_bcauxh = utils._EngineFloat64ArrayType.from_pointer(
                g1,
                (_n_bcauxh,),
                can_reference_contents=False,
            )
            _param_bcauxh = utils._EngineFloat64ArrayType.from_pointer(
                param,
                (_n1_bcauxh,),
            )
            if data is not None:
                bcaux(
                    _n_bcauxh,
                    _n1_bcauxh,
                    _g0_bcauxh,
                    _g1_bcauxh,
                    _param_bcauxh,
                    data,
                )
            else:
                bcaux(
                    _n_bcauxh,
                    _n1_bcauxh,
                    _g0_bcauxh,
                    _g1_bcauxh,
                    _param_bcauxh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g0_bcauxh, {'fun_name': cb_bcaux_name, 'returned': 'g0'}, g0,
            )
            utils._unlink_pointer(g0)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g1_bcauxh, {'fun_name': cb_bcaux_name, 'returned': 'g1'}, g1,
            )
            utils._unlink_pointer(g1)
            utils._unlink_pointer(param)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_raaux_name = fun_name + ':raaux'

    raauxh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_FLOAT64, # Output, x0.
        utils._BYREF_FLOAT64, # Output, x1.
        utils._BYREF_FLOAT64, # Output, r.
        utils._EngineFloat64ArrayType.c_type, # Input, param[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_raauxh(_raaux, en_data, n1, x0, x1, r, param, _iuser, _ruser):
        """Helper for raaux."""
        try:
            _n1_raauxh = utils._EngineIntScalarType.to_py(n1.contents)
            _param_raauxh = utils._EngineFloat64ArrayType.from_pointer(
                param,
                (_n1_raauxh,),
            )
            if data is not None:
                _x0_raauxh, _x1_raauxh, _r_raauxh = raaux(
                    _n1_raauxh,
                    _param_raauxh,
                    data,
                )
            else:
                _x0_raauxh, _x1_raauxh, _r_raauxh = raaux(
                    _n1_raauxh,
                    _param_raauxh,
                )
            utils._EngineFloat64ScalarType(
                _x0_raauxh, {'fun_name': cb_raaux_name, 'returned': 'x0'},
            ).refresh_ctypes_buffer(x0)
            utils._EngineFloat64ScalarType(
                _x1_raauxh, {'fun_name': cb_raaux_name, 'returned': 'x1'},
            ).refresh_ctypes_buffer(x1)
            utils._EngineFloat64ScalarType(
                _r_raauxh, {'fun_name': cb_raaux_name, 'returned': 'r'},
            ).refresh_ctypes_buffer(r)
            utils._unlink_pointer(param)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    prsolh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, param[n1].
        utils._BYREF_FLOAT64, # Input, res.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Input, err[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_prsolh(_prsol, en_data, param, res, n1, err, _iuser, _ruser):
        """Helper for prsol."""
        try:
            _n1_prsolh = utils._EngineIntScalarType.to_py(n1.contents)
            _param_prsolh = utils._EngineFloat64ArrayType.from_pointer(
                param,
                (_n1_prsolh,),
            )
            _res_prsolh = utils._EngineFloat64ScalarType.to_py(res.contents)
            _err_prsolh = utils._EngineFloat64ArrayType.from_pointer(
                err,
                (_n1_prsolh,),
            )
            if data is not None:
                prsol(
                    _param_prsolh,
                    _res_prsolh,
                    _n1_prsolh,
                    _err_prsolh,
                    data,
                )
            else:
                prsol(
                    _param_prsolh,
                    _res_prsolh,
                    _n1_prsolh,
                    _err_prsolh,
                )
            utils._unlink_pointer(param)
            utils._unlink_pointer(err)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, h.
        utils._EngineFloat64ArrayType.c_type, # Input, e[n].
        utils._EngineFloat64ArrayType.c_type, # Input, parerr[n1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, param[n1].
        utils._EngineFloat64ArrayType.c_type, # Output, c[m1, n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, m1.
        utils._UNUSED_FUNCTYPE,
        auxh,
        utils._UNUSED_FUNCTYPE,
        bcauxh,
        utils._UNUSED_FUNCTYPE,
        raauxh,
        utils._UNUSED_FUNCTYPE,
        prsolh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, mat.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wspace.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wspac1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wspac2.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _parerr = utils._EngineFloat64ArrayType(
        parerr, {'fun_name': fun_name, 'entity': 'parerr'},
        exp_shape=(_n1_py,),
    ).to_ctypes()
    _param_cls = utils._EngineFloat64ArrayType(
        param, {'fun_name': fun_name, 'entity': 'param'},
        exp_shape=(_n1_py,),
    )
    _param = _param_cls.to_ctypes()
    _m1 = utils._EngineIntScalarType(
        m1, {'fun_name': fun_name, 'entity': 'm1'},
    ).to_ctypes()
    _m1_py = utils._EngineIntScalarType.to_py(_m1)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_m1_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(aux, {'fun_name': fun_name, 'entity': 'aux'})
    _aux = utils._UNUSED_CALLBACK
    _auxh = utils._cast_callback(
        auxh, cb_auxh,
        {'fun_name': fun_name, 'entity': 'aux'},
    )
    utils._check_callable(bcaux, {'fun_name': fun_name, 'entity': 'bcaux'})
    _bcaux = utils._UNUSED_CALLBACK
    _bcauxh = utils._cast_callback(
        bcauxh, cb_bcauxh,
        {'fun_name': fun_name, 'entity': 'bcaux'},
    )
    utils._check_callable(raaux, {'fun_name': fun_name, 'entity': 'raaux'})
    _raaux = utils._UNUSED_CALLBACK
    _raauxh = utils._cast_callback(
        raauxh, cb_raauxh,
        {'fun_name': fun_name, 'entity': 'raaux'},
    )
    utils._check_callable(prsol, {'fun_name': fun_name, 'entity': 'prsol'})
    _prsol = utils._UNUSED_CALLBACK
    _prsolh = utils._cast_callback(
        prsolh, cb_prsolh,
        {'fun_name': fun_name, 'entity': 'prsol'},
    )
    _mat = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wspace = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wspac1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _wspac2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _h,
        _e,
        _parerr,
        _param,
        _c,
        _n,
        _n1,
        _m1,
        _aux,
        _auxh,
        _bcaux,
        _bcauxh,
        _raaux,
        _raauxh,
        _prsol,
        _prsolh,
        _mat,
        _wspace,
        _wspac1,
        _wspac2,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n1 <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further progress can be made when stepping to obtain a Jacobian update',
                    'for the current parameter values. Step length h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Currently the matching point r does not lie in range [x_0,x_1].',
                    'If x_0, x_1 or r depend on the parameters then this may occur when care',
                    'is not taken to avoid it.',
                    'r = {:s}, x_0 = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x_1 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further progress can be made when stepping to a solution corresponding',
                    'to the current parameter values.',
                    'Step length h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobian for parameter corrections is singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton method failed to converge while updating parameter values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton method has not converged after 12 iterations while updating',
                    'parameter values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _h_py = utils._EngineFloat64ScalarType.to_py(_h)
        _param_cls.refresh_py(_param)
        _c_cls.refresh_py(_c)
        return_data = _h_py
    _err_class.manage()
    return return_data

def ivp_rkm_val_simple(x, xend, n, y, tol, hmax, m, val, fcn, data=None):
    u"""
    Ordinary differential equations, initial value problem,
    Runge--Kutta--Merson method, until a component attains given value
    (simple driver).

    For full information please refer to the NAG Library document for
    d02bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02bgf.html

    Parameters
    ----------
    x : float
        Must be set to the initial value of the independent variable x.

    xend : float
        The final value of the independent variable x.

        If `xend` < `x` on entry integration will proceed in the
        negative direction.

    n : int
        n, the number of differential equations.

    y : float, shape (`n`), modified in place
        On entry: the initial values of the solution y_1,y_2,...,y_n.

        On exit: the computed values of the solution at a point near the
        solution `x`, unless an error has occurred when they contain the
        computed values at the final value of `x`.

    tol : float
        Must be set to a positive tolerance for controlling the error in
        the integration and in the determination of the position where
        y_m = alpha.

        ``ivp_rkm_val_simple`` has been designed so that, for most
        problems, a reduction in `tol` leads to an approximately
        proportional reduction in the error in the solution obtained in
        the integration.

        The relation between changes in `tol` and the error in the
        determination of the position where y_m = alpha is less clear,
        but for `tol` small enough the error should be approximately
        proportional to `tol`.

        However, the actual relation between `tol` and the accuracy
        cannot be guaranteed.

        You are strongly recommended to call ``ivp_rkm_val_simple`` with
        more than one value for `tol` and to compare the results
        obtained to estimate their accuracy.

        In the absence of any prior knowledge you might compare results
        obtained by calling ``ivp_rkm_val_simple`` with `tol` =
        10.0^(-p) and `tol` = 10.0^(-p-1) if p correct decimal digits in
        the solution are required.

    hmax : float
        Controls how the sign of y_m-alpha is checked.

        `hmax` = 0.0

          y_m-alpha is checked at every internal integration step.

        `hmax` != 0.0

          The computed solution is checked for a change in sign of
          y_m-alpha at steps of not greater than |`hmax`|. This facility
          should be used if there is any chance of 'missing' the change
          in sign by checking too infrequently. For example, if two
          changes of sign of y_m-alpha are expected within a distance h,
          say, of each other then a suitable value for `hmax` might be
          `hmax` = h/2. If only one change of sign in y_m-alpha is
          expected on the range `x` to `xend` then `hmax` = 0.0 is most
          appropriate.

    m : int
        The index m of the component of the solution whose value is to
        be checked.

    val : float
        The value of alpha in the equation y_m = alpha to be solved for
        `x`.

    fcn : callable fcn(n, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments x,y_1,...,y_n.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of differential equations.

        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        The point where the component y_m attains the value alpha unless
        an error has occurred, when it contains the value of x at the
        error. In particular, if y_m != alpha anywhere on the range x =
        `x` to x = `xend`, it will contain `xend` on exit.

    tol : float
        Normally unchanged. However if the range from `x` to the
        position where y_m = alpha (or to the final value of `x` if an
        error occurs) is so short that a small change in `tol` is
        unlikely to make any change in the computed solution then, on
        return, `tol` has its sign changed. To check results returned
        with `tol` < 0.0, ``ivp_rkm_val_simple`` should be called again
        with a positive value of `tol` whose magnitude is considerably
        smaller than that of the previous call.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` <= `n`.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 2)
            The value of `tol`, *<value>*, is too small for the function
            to make any further progress across the integration range.
            Current value of `x` = *<value>*.

        (`errno` 3)
            The value of `tol`, *<value>*, is too small for the function
            to take an initial step.

        (`errno` 5)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

        (`errno` 6)
            A serious error occurred in a call to the internal
            integrator.

            The error code internally was *<value>*. Please contact NAG.

        (`errno` 7)
            Unexpected internal error in call to interpolation routine.

            The interpolation routine returned error flag *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            No change in sign of the function y_m-alpha was detected in
            the integration range.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkm_val_simple'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02bgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, xend.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._BYREF_FLOAT64, # Input/Output, tol.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input, val.
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xend = utils._EngineFloat64ScalarType(
        xend, {'fun_name': fun_name, 'entity': 'xend'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _val = utils._EngineFloat64ScalarType(
        val, {'fun_name': fun_name, 'entity': 'val'},
    ).to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _xend,
        _n,
        _y,
        _tol,
        _hmax,
        _m,
        _val,
        _fcn,
        _fcnh,
        _w,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tol, {:s}, is too small for the function to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'make any further progress across the integration range.',
                    'Current value of x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tol, {:s}, is too small for the function to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'take an initial step.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No change in sign of the function y_m-alpha was detected in the',
                    'integration range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred in this function.',
                    'Check the function call and any array sizes.',
                    'If the call is correct then please contact NAG for assistance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in a call to the internal integrator.',
                    'The error code internally was {:s}. Please contact NAG.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal error in call to interpolation routine.',
                    'The interpolation routine returned error flag {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_cls.refresh_py(_y)
        _tol_py = utils._EngineFloat64ScalarType.to_py(_tol)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'tol'],
        )(
            _x_py,
            _tol_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_rkm_zero_simple(x, xend, n, y, tol, irelab, hmax, fcn, g, data=None):
    u"""
    Ordinary differential equations, initial value problem,
    Runge--Kutta--Merson method, until function of solution is zero
    (simple driver).

    For full information please refer to the NAG Library document for
    d02bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02bhf.html

    Parameters
    ----------
    x : float
        Must be set to the initial value of the independent variable x.

    xend : float
        The final value of the independent variable x.

        If `xend` < `x` on entry, integration proceeds in a negative
        direction.

    n : int
        n, the number of differential equations.

    y : float, shape (`n`), modified in place
        On entry: the initial values of the solution y_1,y_2,...,y_n.

        On exit: the computed values of the solution at the final point
        x = `x`.

    tol : float
        Must be set to a positive tolerance for controlling the error in
        the integration and in the determination of the position where
        g(x,y) = 0.0.

        ``ivp_rkm_zero_simple`` has been designed so that, for most
        problems, a reduction in `tol` leads to an approximately
        proportional reduction in the error in the solution obtained in
        the integration.

        The relation between changes in `tol` and the error in the
        determination of the position where g(x,y) = 0.0 is less clear,
        but for `tol` small enough the error should be approximately
        proportional to `tol`.

        However, the actual relation between `tol` and the accuracy
        cannot be guaranteed.

        You are strongly recommended to call ``ivp_rkm_zero_simple``
        with more than one value for `tol` and to compare the results
        obtained to estimate their accuracy.

        In the absence of any prior knowledge you might compare results
        obtained by calling ``ivp_rkm_zero_simple`` with `tol` =
        10.0^(-p) and `tol` = 10.0^(-p-1) if p correct decimal digits in
        the solution are required.

    irelab : int
        Determines the type of error control. At each step in the
        numerical solution an estimate of the local error, est, is made.
        For the current step to be accepted the following condition must
        be satisfied:

        `irelab` = 0

          est <= `tol`*max{1.0,|y_1|,|y_2|,...,|y_n|};

        `irelab` = 1

          est <= `tol`;

        `irelab` = 2

          est <= `tol`*max{epsilon,|y_1|,|y_2|,...,|y_n|}, where epsilon
          is machine precision.

        If the appropriate condition is not satisfied, the step size is
        reduced and the solution recomputed on the current step.

        If you wish to measure the error in the computed solution in
        terms of the number of correct decimal places, set `irelab` = 1
        on entry, whereas if the error requirement is in terms of the
        number of correct significant digits, set `irelab` = 2.

        Where there is no preference in the choice of error test,
        `irelab` = 0 will result in a mixed error test.

        It should be borne in mind that the computed solution will be
        used in evaluating g(x,y).

    hmax : float
        If `hmax` = 0.0, no special action is taken.

        If `hmax` != 0.0, a check is made for a change in sign of g(x,y)
        at steps not greater than |`hmax`|.

        This facility should be used if there is any chance of 'missing'
        the change in sign by checking too infrequently.

        For example, if two changes of sign of g(x,y) are expected
        within a distance h, say, of each other, a suitable value for
        `hmax` might be `hmax` = h/2.

        If only one change of sign in g(x,y) is expected on the range
        `x` to `xend`, the choice `hmax` = 0.0 is most appropriate.

    fcn : callable fcn(n, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments x,y_1,...,y_n.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of differential equations.

        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    g : callable retval = g(n, x, y, data=None)
        `g` must evaluate the function g(x,y) at a specified point.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of differential equations.

        x : float
            x, the value of the independent variable.

        y : float, ndarray, shape (`n`)
            The value of y_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x,y) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        The point where g(x,y) = 0.0 unless an error has occurred, when
        it contains the value of x at the error. In particular, if
        g(x,y) != 0.0 anywhere on the range `x` to `xend`, it will
        contain `xend` on exit.

    tol : float
        Normally unchanged. However if the range from x = `x` to the
        position where g(x,y) = 0.0 (or to the final value of x if an
        error occurs) is so short that a small change in `tol` is
        unlikely to make any change in the computed solution, `tol` is
        returned with its sign changed. To check results returned with
        `tol` < 0.0, ``ivp_rkm_zero_simple`` should be called again with
        a positive value of `tol` whose magnitude is considerably
        smaller than that of the previous call.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irelab` = *<value>*.

            Constraint: 0 <= `irelab` <= 2.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 3)
            The value of `tol`, *<value>*, is too small for the function
            to take an initial step.

        (`errno` 5)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

        (`errno` 6)
            A serious error occurred in a call to the internal
            integrator.

            The error code internally was *<value>*. Please contact NAG.

        (`errno` 7)
            Unexpected internal error in call to interpolation routine.

            The interpolation routine returned error flag *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The value of `tol`, *<value>*, is too small for the function
            to make any further progress across the integration range.
            Current value of x = *<value>*.

        (`errno` 4)
            No change in sign of the function g(x,y) was detected in the
            integration range.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkm_zero_simple'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, _gsub, en_data, n, x, y, retval, _iuser, _ruser):
        """Helper for g."""
        try:
            _n_gh = utils._EngineIntScalarType.to_py(n.contents)
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_gh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_gh,),
            )
            if data is not None:
                _retval_gh = g(
                    _n_gh,
                    _x_gh,
                    _y_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _n_gh,
                    _x_gh,
                    _y_gh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02bhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, xend.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._BYREF_FLOAT64, # Input/Output, tol.
        utils._BYREF_INT, # Input, irelab.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xend = utils._EngineFloat64ScalarType(
        xend, {'fun_name': fun_name, 'entity': 'xend'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _irelab = utils._EngineIntScalarType(
        irelab, {'fun_name': fun_name, 'entity': 'irelab'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _xend,
        _n,
        _y,
        _tol,
        _irelab,
        _hmax,
        _fcn,
        _fcnh,
        _g,
        _gsub,
        _gh,
        _w,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irelab = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= irelab <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tol, {:s}, is too small for the function to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'make any further progress across the integration range.',
                    'Current value of x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tol, {:s}, is too small for the function to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'take an initial step.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No change in sign of the function g(x,y) was detected in the integration',
                    'range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred in this function.',
                    'Check the function call and any array sizes.',
                    'If the call is correct then please contact NAG for assistance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in a call to the internal integrator.',
                    'The error code internally was {:s}. Please contact NAG.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal error in call to interpolation routine.',
                    'The interpolation routine returned error flag {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_cls.refresh_py(_y)
        _tol_py = utils._EngineFloat64ScalarType.to_py(_tol)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'tol'],
        )(
            _x_py,
            _tol_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_rk_zero_simple(x, xend, n, y, fcn, tol, relabs, output, g, data=None):
    u"""
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, until function of solution is zero, integration over range
    with intermediate output (simple driver).

    For full information please refer to the NAG Library document for
    d02bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02bjf.html

    Parameters
    ----------
    x : float
        The initial value of the independent variable x.

    xend : float
        The final value of the independent variable. If `xend` < `x`,
        integration will proceed in the negative direction.

    n : int
        n, the number of equations.

    y : float, shape (`n`), modified in place
        On entry: the initial values of the solution y_1,y_2,...,y_n at
        x = `x`.

        On exit: the computed values of the solution at the final point
        x = `x`.

    fcn : callable fcn(x, n, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments x,y_1,...,y_n.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        n : int
            n, the number of equations.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    tol : float
        A positive tolerance for controlling the error in the
        integration. Hence `tol` affects the determination of the
        position where g(x,y) = 0, if g is supplied.

        ``ivp_rk_zero_simple`` has been designed so that, for most
        problems, a reduction in `tol` leads to an approximately
        proportional reduction in the error in the solution.

        However, the actual relation between `tol` and the accuracy
        achieved cannot be guaranteed.

        You are strongly recommended to call ``ivp_rk_zero_simple`` with
        more than one value for `tol` and to compare the results
        obtained to estimate their accuracy.

        In the absence of any prior knowledge, you might compare the
        results obtained by calling ``ivp_rk_zero_simple`` with `relabs`
        = 'D' and with each of `tol` = 10.0^(-p) and `tol` = 10.0^(-p-1)
        where p correct significant digits are required in the solution,
        y.

        The accuracy of the value x such that g(x,y) = 0 is indirectly
        controlled by varying `tol`.

        You should experiment to determine this accuracy.

    relabs : str, length 1
        The type of error control. At each step in the numerical
        solution an estimate of the local error, est, is made. For the
        current step to be accepted the following condition must be
        satisfied:

            est = max(e_i/(tau_r*max(|y_i|,tau_a))) <= 1.0

        where tau_r and tau_a are defined by

        [table omitted]

        where epsilon_r and epsilon_a are small machine-dependent
        numbers and e_i is an estimate of the local error at y_i,
        computed internally. If the condition is not satisfied, the step
        size is reduced and the solution is recomputed on the current
        step. If you wish to measure the error in the computed solution
        in terms of the number of correct decimal places, `relabs`
        should be set to 'A' on entry, whereas if the error requirement
        is in terms of the number of correct significant digits,
        `relabs` should be set to 'R'. If you prefer a mixed error test,
        `relabs` should be set to 'M', otherwise if you have no
        preference, `relabs` should be set to the default 'D'. Note that
        in this case 'D' is taken to be 'R'.

    output : None or callable xsol = output(xsol, n, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `output` permits access to intermediate values of the computed
        solution (for example to print or plot them), at successive
        user-specified points.

        It is initially called by ``ivp_rk_zero_simple`` with `xsol` =
        `x` (the initial value of x).

        You must reset `xsol` to the next point (between the current
        `xsol` and `xend`) where `output` is to be called, and so on at
        each call to `output`.

        If, after a call to `output`, the reset point `xsol` is beyond
        `xend`, ``ivp_rk_zero_simple`` will integrate to `xend` with no
        further calls to `output`; if a call to `output` is required at
        the point `xsol` = `xend`, `xsol` must be given precisely the
        value `xend`.

        Parameters
        ~~~~~~~~~~
        xsol : float
            The output value of the independent variable x.

        n : int
            n, the number of equations.

        y : float, ndarray, shape (`n`)
            The computed solution at the point `xsol`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        xsol : float
            You must set `xsol` to the next value of x at which `output`
            is to be called.

    g : None or callable retval = g(x, n, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `g` must evaluate the function g(x,y) for specified values x,y.

        It specifies the function g for which the first position x where
        g(x,y) = 0 is to be found.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        n : int
            n, the number of equations.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x,y) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If g is supplied by you, it contains the point where g(x,y) = 0,
        unless g(x,y) != 0 anywhere on the range `x` to `xend`, in which
        case, `x` will contain `xend` (and the error indicator `errno` =
        6 is set); if g is not supplied by you it contains `xend`.
        However, if an error has occurred, it contains the value of x at
        which the error occurred.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = `xend` = *<value>*.

            Constraint: `x` != `xend`.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: *<value>* <= `tol` < 0.01.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `relabs` = *<value>*.

            Constraint: `relabs` = 'M', 'A', 'R' or 'D'.

        (`errno` 2)
            With the given value of `tol`, no further progress can be
            made across the integration range from the current point x =
            `x`.

            The solution is returned at this point, `x` = *<value>*.

        (`errno` 3)
            With the given value of `tol`, no initial progress can be
            made from the starting point of integration provided.

        (`errno` 4)
            On the first call to `output`, `xsol` was returned as
            *<value>*, which is inconsistent with (`x`,`xend`) ---
            (*<value>*,*<value>*).

        (`errno` 4)
            On the first call to `output`, `xsol` remained unchanged.

        (`errno` 5)
            On a call to `output`, `xsol` was returned as *<value>*,
            which is inconsistent with previous `xsol` and `xend` ---
            (*<value>*,*<value>*).

        (`errno` 5)
            On a call to `output`, `xsol` remained unchanged.

        (`errno` 6)
            No change in sign of the function g(x,y) was detected in the
            integration range.

        (`errno` 7)
            Unexpected internal error in call to zero-finding routine.

            The zero-finding routine returned error flag *<value>*.

        (`errno` 7)
            Unexpected internal error in call to interpolation routine.

            The interpolation routine returned error flag *<value>*.

        (`errno` 8)
            Unexpected internal error in call to step integrator.

            The step integrator returned error flag *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rk_zero_simple'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, x, n, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _x_fcnh,
                    _n_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _x_fcnh,
                    _n_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_output_name = fun_name + ':output'

    outputh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, xsol.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if output is None:
        output_auxil_ctypes = utils._EngineState._get_symbol('d02bjxt')
        output_auxil_ctypes.restype = None
        output_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input/Output, xsol.
            utils._BYREF_INT, # Input, n.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_outputh(_output, en_data, xsol, n, y, iuser, ruser):
        """Helper for output."""
        try:
            if output is None:
                output_auxil_ctypes(
                    en_data,
                    xsol,
                    n,
                    y,
                    iuser,
                    ruser,
                )
                return
            _xsol_outputh = utils._EngineFloat64ScalarType.to_py(xsol.contents)
            _n_outputh = utils._EngineIntScalarType.to_py(n.contents)
            _y_outputh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_outputh,),
            )
            if data is not None:
                _xsol_outputh = output(
                    _xsol_outputh,
                    _n_outputh,
                    _y_outputh,
                    data,
                )
            else:
                _xsol_outputh = output(
                    _xsol_outputh,
                    _n_outputh,
                    _y_outputh,
                )
            utils._EngineFloat64ScalarType(
                _xsol_outputh, {'fun_name': cb_output_name, 'returned': 'xsol'},
            ).refresh_ctypes_buffer(xsol)
            utils._unlink_pointer(y)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if g is None:
        g_auxil_ctypes = utils._EngineState._get_symbol('d02bjwt')
        g_auxil_ctypes.restype = None
        g_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, x.
            utils._BYREF_INT, # Input, n.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._BYREF_FLOAT64, # Output, retval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_gh(_g, _gsub, en_data, x, n, y, retval, iuser, ruser):
        """Helper for g."""
        try:
            if g is None:
                g_auxil_ctypes(
                    en_data,
                    x,
                    n,
                    y,
                    retval,
                    iuser,
                    ruser,
                )
                return
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _n_gh = utils._EngineIntScalarType.to_py(n.contents)
            _y_gh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_gh,),
            )
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    _n_gh,
                    _y_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                    _n_gh,
                    _y_gh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02bjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, xend.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_STRING, # Input, relabs(len=1).
        utils._UNUSED_FUNCTYPE,
        outputh,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of relabs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xend = utils._EngineFloat64ScalarType(
        xend, {'fun_name': fun_name, 'entity': 'xend'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _relabs = utils._EngineCharScalarType(
        relabs, {'fun_name': fun_name, 'entity': 'relabs'}, exp_chrlen=1,
    ).to_ctypes()
    utils._check_callable(output, {'fun_name': fun_name, 'entity': 'output'}, allow_none=True)
    _output = utils._UNUSED_CALLBACK
    _outputh = utils._cast_callback(
        outputh, cb_outputh,
        {'fun_name': fun_name, 'entity': 'output'},
    )
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'}, allow_none=True)
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _xend,
        _n,
        _y,
        _fcn,
        _fcnh,
        _tol,
        _relabs,
        _output,
        _outputh,
        _g,
        _gsub,
        _gh,
        _w,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _relabs._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = xend = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x != xend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: {:s} <= tol < 0.01.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, relabs = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: relabs = \'M\', \'A\', \'R\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'With the given value of tol, no further progress can be made across',
                    'the integration range from the current point x = x.',
                    'The solution is returned at this point, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'With the given value of tol, no initial progress can be made from',
                    'the starting point of integration provided.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On the first call to output, xsol was returned as {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'which is inconsistent with (x,xend) ---',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On the first call to output, xsol remained unchanged.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On a call to output, xsol was returned as {:s}, which is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'inconsistent with previous xsol and xend ---',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On a call to output, xsol remained unchanged.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No change in sign of the function g(x,y) was detected in the integration',
                    'range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal error in call to zero-finding routine.',
                    'The zero-finding routine returned error flag {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal error in call to interpolation routine.',
                    'The interpolation routine returned error flag {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected internal error in call to step integrator.',
                    'The step integrator returned error flag {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_cls.refresh_py(_y)
        return_data = _x_py
    _err_class.manage()
    return return_data

def ivp_adams_zero_simple(x, xend, n, y, fcn, tol, relabs, output, g, data=None):
    u"""
    Ordinary differential equations, initial value problem, Adams'
    method, until function of solution is zero, intermediate output
    (simple driver).

    For full information please refer to the NAG Library document for
    d02cj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02cjf.html

    Parameters
    ----------
    x : float
        The initial value of the independent variable x.

    xend : float
        The final value of the independent variable. If `xend` < `x`,
        integration will proceed in the negative direction.

    n : int
        n, the number of differential equations.

    y : float, shape (`n`), modified in place
        On entry: the initial values of the solution y_1,y_2,...,y_n at
        x = `x`.

        On exit: the computed values of the solution at the final point
        x = `x`.

    fcn : callable fcn(x, n, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments x,y_1,...,y_n.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        n : int
            n, the total number of differential equations.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    tol : float
        A positive tolerance for controlling the error in the
        integration. Hence `tol` affects the determination of the
        position where g(x,y) = 0.0, if g is supplied.

        ``ivp_adams_zero_simple`` has been designed so that, for most
        problems, a reduction in `tol` leads to an approximately
        proportional reduction in the error in the solution.

        However, the actual relation between `tol` and the accuracy
        achieved cannot be guaranteed.

        You are strongly recommended to call ``ivp_adams_zero_simple``
        with more than one value for `tol` and to compare the results
        obtained to estimate their accuracy.

        In the absence of any prior knowledge, you might compare the
        results obtained by calling ``ivp_adams_zero_simple`` with `tol`
        = 10.0^(-p) and `tol` = 10.0^(-p-1) where p correct decimal
        digits are required in the solution.

    relabs : str, length 1
        The type of error control. At each step in the numerical
        solution an estimate of the local error, est, is made. For the
        current step to be accepted the following condition must be
        satisfied:

            est = sqrt(sum_i = 1^n(e_i/(tau_r*|y_i|+tau_a))^2) <= 1.0

        where tau_r and tau_a are defined by

        [table omitted]

        where epsilon is a small machine-dependent number and e_i is an
        estimate of the local error at y_i, computed internally. If the
        appropriate condition is not satisfied, the step size is reduced
        and the solution is recomputed on the current step. If you wish
        to measure the error in the computed solution in terms of the
        number of correct decimal places, `relabs` should be set to 'A'
        on entry, whereas if the error requirement is in terms of the
        number of correct significant digits, `relabs` should be set to
        'R'. If you prefer a mixed error test, `relabs` should be set to
        'M', otherwise if you have no preference, `relabs` should be set
        to the default 'D'. Note that in this case 'D' is taken to be
        'M'.

    output : None or callable xsol = output(xsol, n, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `output` permits access to intermediate values of the computed
        solution (for example to print or plot them), at successive
        user-specified points.

        It is initially called by ``ivp_adams_zero_simple`` with `xsol`
        = `x` (the initial value of x).

        You must reset `xsol` to the next point (between the current
        `xsol` and `xend`) where `output` is to be called, and so on at
        each call to `output`.

        If, after a call to `output`, the reset point `xsol` is beyond
        `xend`, ``ivp_adams_zero_simple`` will integrate to `xend` with
        no further calls to `output`; if a call to `output` is required
        at the point `xsol` = `xend`, `xsol` must be given precisely the
        value `xend`.

        Parameters
        ~~~~~~~~~~
        xsol : float
            The output value of the independent variable x.

        n : int
            n, the total number of differential equations.

        y : float, ndarray, shape (`n`)
            The computed solution at the point `xsol`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        xsol : float
            You must set `xsol` to the next value of x at which `output`
            is to be called.

    g : None or callable retval = g(x, n, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `g` must evaluate the function g(x,y) for specified values x,y.

        It specifies the function g for which the first position x where
        g(x,y) = 0 is to be found.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        n : int
            n, the total number of differential equations.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x,y) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If g is supplied by you, it contains the point where g(x,y) =
        0.0, unless g(x,y) != 0.0 anywhere on the range `x` to `xend`,
        in which case, `x` will contain `xend`. If g is not supplied by
        you it contains `xend`, unless an error has occurred, when it
        contains the value of x at the error.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `relabs` = *<value>*.

            Constraint: `relabs` = 'M', 'A', 'R' or 'D'.

        (`errno` 1)
            On entry, `x` = *<value>* and `xend` = *<value>*.

            Constraint: `x` != `xend`.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            No integration steps have been taken. Progress not possible
            with the input value of `tol` = *<value>*.

        (`errno` 4)
            On the first call to `output`, `xsol` was returned as
            *<value>*, which is inconsistent with (`x`,`xend`) ---
            (*<value>*,*<value>*).

        (`errno` 4)
            On the first call to `output`, `xsol` remained unchanged.

        (`errno` 5)
            On a call to `output`, `xsol` remained unchanged.

            `xsol` = *<value>*.

        (`errno` 7)
            Impossible error --- internal variable IDID = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Integration successful as far as x = *<value>*, but further
            progress not possible with the input value of `tol` =
            *<value>*.

        (`errno` 5)
            On a call to `output`, `xsol` was returned as *<value>*,
            which is inconsistent with previous `xsol` and `xend` ---
            (*<value>*,*<value>*).

        (`errno` 6)
            No change in sign of the function g(x,y) was detected in the
            integration range.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_zero_simple'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, x, n, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _x_fcnh,
                    _n_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _x_fcnh,
                    _n_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_output_name = fun_name + ':output'

    outputh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, xsol.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if output is None:
        output_auxil_ctypes = utils._EngineState._get_symbol('d02cjxt')
        output_auxil_ctypes.restype = None
        output_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input/Output, xsol.
            utils._BYREF_INT, # Input, n.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_outputh(_output, en_data, xsol, n, y, iuser, ruser):
        """Helper for output."""
        try:
            if output is None:
                output_auxil_ctypes(
                    en_data,
                    xsol,
                    n,
                    y,
                    iuser,
                    ruser,
                )
                return
            _xsol_outputh = utils._EngineFloat64ScalarType.to_py(xsol.contents)
            _n_outputh = utils._EngineIntScalarType.to_py(n.contents)
            _y_outputh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_outputh,),
            )
            if data is not None:
                _xsol_outputh = output(
                    _xsol_outputh,
                    _n_outputh,
                    _y_outputh,
                    data,
                )
            else:
                _xsol_outputh = output(
                    _xsol_outputh,
                    _n_outputh,
                    _y_outputh,
                )
            utils._EngineFloat64ScalarType(
                _xsol_outputh, {'fun_name': cb_output_name, 'returned': 'xsol'},
            ).refresh_ctypes_buffer(xsol)
            utils._unlink_pointer(y)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if g is None:
        g_auxil_ctypes = utils._EngineState._get_symbol('d02cjwt')
        g_auxil_ctypes.restype = None
        g_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_FLOAT64, # Input, x.
            utils._BYREF_INT, # Input, n.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._BYREF_FLOAT64, # Output, retval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_gh(_g, _gsub, en_data, x, n, y, retval, iuser, ruser):
        """Helper for g."""
        try:
            if g is None:
                g_auxil_ctypes(
                    en_data,
                    x,
                    n,
                    y,
                    retval,
                    iuser,
                    ruser,
                )
                return
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _n_gh = utils._EngineIntScalarType.to_py(n.contents)
            _y_gh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_gh,),
            )
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    _n_gh,
                    _y_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                    _n_gh,
                    _y_gh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02cjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, xend.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_STRING, # Input, relabs(len=1).
        utils._UNUSED_FUNCTYPE,
        outputh,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of relabs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xend = utils._EngineFloat64ScalarType(
        xend, {'fun_name': fun_name, 'entity': 'xend'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _relabs = utils._EngineCharScalarType(
        relabs, {'fun_name': fun_name, 'entity': 'relabs'}, exp_chrlen=1,
    ).to_ctypes()
    utils._check_callable(output, {'fun_name': fun_name, 'entity': 'output'}, allow_none=True)
    _output = utils._UNUSED_CALLBACK
    _outputh = utils._cast_callback(
        outputh, cb_outputh,
        {'fun_name': fun_name, 'entity': 'output'},
    )
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'}, allow_none=True)
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _xend,
        _n,
        _y,
        _fcn,
        _fcnh,
        _tol,
        _relabs,
        _output,
        _outputh,
        _g,
        _gsub,
        _gh,
        _w,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _relabs._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, relabs = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: relabs = \'M\', \'A\', \'R\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and xend = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x != xend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration successful as far as x = {:s}, but further'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'progress not possible with the input value of',
                    'tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No integration steps have been taken. Progress not possible with the',
                    'input value of tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On the first call to output, xsol was returned as {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'which is inconsistent with (x,xend) ---',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On the first call to output, xsol remained unchanged.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On a call to output, xsol was returned as {:s}, which is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'inconsistent with previous xsol and xend ---',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On a call to output, xsol remained unchanged.',
                    'xsol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No change in sign of the function g(x,y) was detected in the integration',
                    'range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Impossible error --- internal variable IDID = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_cls.refresh_py(_y)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def ivp_bdf_zero_simple(x, xend, n, y, fcn, pederv, tol, relabs, output, g, data=None, io_manager=None):
    u"""
    Ordinary differential equations, stiff initial value problem,
    backward differentiation formulae method, until function of solution
    is zero, intermediate output (simple driver).

    For full information please refer to the NAG Library document for
    d02ej

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ejf.html

    Parameters
    ----------
    x : float
        The initial value of the independent variable x.

    xend : float
        The final value of the independent variable. If `xend` < `x`,
        integration will proceed in the negative direction.

    n : int
        n, the number of differential equations.

    y : float, shape (`n`), modified in place
        On entry: the initial values of the solution y_1,y_2,...,y_n at
        x = `x`.

        On exit: the computed values of the solution at the final point
        x = `x`.

    fcn : callable fcn(n, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) for given values of its arguments x,y_1,...,y_n.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        x : float
            x, the value of the independent variable.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    pederv : None or callable pederv(n, x, y, pw, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `pederv` must evaluate the Jacobian of the system (that is, the
        partial derivatives df_i/dy_j) for given values of the variables
        x,y_1,y_2,...,y_n.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        x : float
            x, the value of the independent variable.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        pw : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `pw`[i-1,j-1] must contain the value of df_i/dy_j,
            for j = 1,2,...,n, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    tol : float
        Must be set to a positive tolerance for controlling the error in
        the integration. Hence `tol` affects the determination of the
        position where g(x,y) = 0.0, if `g` is supplied.

        ``ivp_bdf_zero_simple`` has been designed so that, for most
        problems, a reduction in `tol` leads to an approximately
        proportional reduction in the error in the solution.

        However, the actual relation between `tol` and the accuracy
        achieved cannot be guaranteed.

        You are strongly recommended to call ``ivp_bdf_zero_simple``
        with more than one value for `tol` and to compare the results
        obtained to estimate their accuracy.

        In the absence of any prior knowledge, you might compare the
        results obtained by calling ``ivp_bdf_zero_simple`` with `tol` =
        10^(-p) and `tol` = 10^(-p-1) if p correct decimal digits are
        required in the solution.

    relabs : str, length 1
        The type of error control. At each step in the numerical
        solution an estimate of the local error, est, is made. For the
        current step to be accepted the following condition must be
        satisfied:

            est = sqrt(1/(n)sum_i = 1^n(e_i/(tau_r*|y_i|+tau_a))^2) <=
            1.0

        where tau_r and tau_a are defined by

        [table omitted]

        where epsilon is a small machine-dependent number and e_i is an
        estimate of the local error at y_i, computed internally. If the
        appropriate condition is not satisfied, the step size is reduced
        and the solution is recomputed on the current step. If you wish
        to measure the error in the computed solution in terms of the
        number of correct decimal places, `relabs` should be set to 'A'
        on entry, whereas if the error requirement is in terms of the
        number of correct significant digits, `relabs` should be set to
        'R'. If you prefer a mixed error test, `relabs` should be set to
        'M', otherwise if you have no preference, `relabs` should be set
        to the default 'D'. Note that in this case 'D' is taken to be
        'R'.

    output : None or callable xsol = output(n, xsol, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `output` permits access to intermediate values of the computed
        solution (for example to print or plot them), at successive
        user-specified points.

        It is initially called by ``ivp_bdf_zero_simple`` with `xsol` =
        `x` (the initial value of x).

        You must reset `xsol` to the next point (between the current
        `xsol` and `xend`) where `output` is to be called, and so on at
        each call to `output`.

        If, after a call to `output`, the reset point `xsol` is beyond
        `xend`, ``ivp_bdf_zero_simple`` will integrate to `xend` with no
        further calls to `output`; if a call to `output` is required at
        the point `xsol` = `xend`, `xsol` must be given precisely the
        value `xend`.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        xsol : float
            x, the value of the independent variable.

        y : float, ndarray, shape (`n`)
            The computed solution at the point `xsol`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        xsol : float
            You must set `xsol` to the next value of x at which `output`
            is to be called.

    g : None or callable retval = g(n, x, y, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `g` must evaluate the function g(x,y) for specified values x,y.

        It specifies the function g for which the first position x where
        g(x,y) = 0 is to be found.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the total number of differential equations.

        x : float
            x, the value of the independent variable.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the variable.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g at the specified points.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    x : float
        If `g` is supplied by you, `x` contains the point where g(x,y) =
        0.0, unless g(x,y) != 0.0 anywhere on the range `x` to `xend`,
        in which case, `x` will contain `xend`. If `g` is not supplied
        `x` contains `xend`, unless an error has occurred, when it
        contains the value of x at the error.

    tol : float
        Normally unchanged. However if the range `x` to `xend` is so
        short that a small change in `tol` is unlikely to make any
        change in the computed solution, then, on return, `tol` has its
        sign changed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `relabs` = *<value>*.

            Constraint: `relabs` = 'M', 'A', 'R' or 'D'.

        (`errno` 1)
            On entry, `x` = *<value>* and `xend` = *<value>*.

            Constraint: `x` != `xend`.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            No integration steps have been taken. Progress not possible
            with the input value of `tol` = *<value>*.

        (`errno` 4)
            No integration steps have been taken. `xsol` has been set
            illegally.

        (`errno` 7)
            Integration successful as far as x = *<value>*, but an
            internal error has occurred during rootfinding.

        (`errno` 9)
            Impossible error --- internal variable IERR = *<value>*.

        (`errno` 9)
            Impossible error --- internal variable IREVCM = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Integration successful as far as x = *<value>*, but further
            progress not possible with the input value of `tol` =
            *<value>*.

        (`errno` 5)
            Integration successful as far as x = *<value>*, but `xsol`
            has been reset illegally.

        (`errno` 6)
            No change in sign of the function g(x,y) was detected in the
            integration range.

        (`errno` 8)
            Integration successful as far as x = *<value>*, but an
            internal error has occurred during interpolation.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.ode.ivp_bdf_zero_simple_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_bdf_zero_simple'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_pederv_name = fun_name + ':pederv'

    pedervh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, pw[n, n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if pederv is None:
        pederv_auxil_ctypes = utils._EngineState._get_symbol('d02ejyt')
        pederv_auxil_ctypes.restype = None
        pederv_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, n.
            utils._BYREF_FLOAT64, # Input, x.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._EngineFloat64ArrayType.c_type, # Output, pw[n, n].
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_pedervh(_pederv, en_data, n, x, y, pw, iuser, ruser):
        """Helper for pederv."""
        try:
            if pederv is None:
                pederv_auxil_ctypes(
                    en_data,
                    n,
                    x,
                    y,
                    pw,
                    iuser,
                    ruser,
                )
                return
            _n_pedervh = utils._EngineIntScalarType.to_py(n.contents)
            _x_pedervh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_pedervh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_pedervh,),
            )
            _pw_pedervh = utils._EngineFloat64ArrayType.from_pointer(
                pw,
                (_n_pedervh, _n_pedervh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                pederv(
                    _n_pedervh,
                    _x_pedervh,
                    _y_pedervh,
                    _pw_pedervh,
                    data,
                )
            else:
                pederv(
                    _n_pedervh,
                    _x_pedervh,
                    _y_pedervh,
                    _pw_pedervh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _pw_pedervh, {'fun_name': cb_pederv_name, 'returned': 'pw'}, pw,
            )
            utils._unlink_pointer(pw)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_output_name = fun_name + ':output'

    outputh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input/Output, xsol.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if output is None:
        output_auxil_ctypes = utils._EngineState._get_symbol('d02ejxt')
        output_auxil_ctypes.restype = None
        output_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, n.
            utils._BYREF_FLOAT64, # Input/Output, xsol.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_outputh(_output, en_data, n, xsol, y, iuser, ruser):
        """Helper for output."""
        try:
            if output is None:
                output_auxil_ctypes(
                    en_data,
                    n,
                    xsol,
                    y,
                    iuser,
                    ruser,
                )
                return
            _n_outputh = utils._EngineIntScalarType.to_py(n.contents)
            _xsol_outputh = utils._EngineFloat64ScalarType.to_py(xsol.contents)
            _y_outputh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_outputh,),
            )
            if data is not None:
                _xsol_outputh = output(
                    _n_outputh,
                    _xsol_outputh,
                    _y_outputh,
                    data,
                )
            else:
                _xsol_outputh = output(
                    _n_outputh,
                    _xsol_outputh,
                    _y_outputh,
                )
            utils._EngineFloat64ScalarType(
                _xsol_outputh, {'fun_name': cb_output_name, 'returned': 'xsol'},
            ).refresh_ctypes_buffer(xsol)
            utils._unlink_pointer(y)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if g is None:
        g_auxil_ctypes = utils._EngineState._get_symbol('d02ejwt')
        g_auxil_ctypes.restype = None
        g_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, n.
            utils._BYREF_FLOAT64, # Input, x.
            utils._EngineFloat64ArrayType.c_type, # Input, y[n].
            utils._BYREF_FLOAT64, # Output, retval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_gh(_g, _gsub, en_data, n, x, y, retval, iuser, ruser):
        """Helper for g."""
        try:
            if g is None:
                g_auxil_ctypes(
                    en_data,
                    n,
                    x,
                    y,
                    retval,
                    iuser,
                    ruser,
                )
                return
            _n_gh = utils._EngineIntScalarType.to_py(n.contents)
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_gh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_gh,),
            )
            if data is not None:
                _retval_gh = g(
                    _n_gh,
                    _x_gh,
                    _y_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _n_gh,
                    _x_gh,
                    _y_gh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02ejft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, xend.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._UNUSED_FUNCTYPE,
        pedervh,
        utils._BYREF_FLOAT64, # Input/Output, tol.
        utils._BYREF_STRING, # Input, relabs(len=1).
        utils._UNUSED_FUNCTYPE,
        outputh,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, iw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of relabs.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xend = utils._EngineFloat64ScalarType(
        xend, {'fun_name': fun_name, 'entity': 'xend'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    utils._check_callable(pederv, {'fun_name': fun_name, 'entity': 'pederv'}, allow_none=True)
    _pederv = utils._UNUSED_CALLBACK
    _pedervh = utils._cast_callback(
        pedervh, cb_pedervh,
        {'fun_name': fun_name, 'entity': 'pederv'},
    )
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _relabs = utils._EngineCharScalarType(
        relabs, {'fun_name': fun_name, 'entity': 'relabs'}, exp_chrlen=1,
    ).to_ctypes()
    utils._check_callable(output, {'fun_name': fun_name, 'entity': 'output'}, allow_none=True)
    _output = utils._UNUSED_CALLBACK
    _outputh = utils._cast_callback(
        outputh, cb_outputh,
        {'fun_name': fun_name, 'entity': 'output'},
    )
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'}, allow_none=True)
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _x,
        _xend,
        _n,
        _y,
        _fcn,
        _fcnh,
        _pederv,
        _pedervh,
        _tol,
        _relabs,
        _output,
        _outputh,
        _g,
        _gsub,
        _gh,
        _w,
        _iw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _relabs._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, relabs = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: relabs = \'M\', \'A\', \'R\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and xend = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x != xend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration successful as far as x = {:s}, but further'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'progress not possible with the input value of',
                    'tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No integration steps have been taken. Progress not possible with the',
                    'input value of tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No integration steps have been taken. xsol has been set illegally.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration successful as far as x = {:s}, but xsol has'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'been reset illegally.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No change in sign of the function g(x,y) was detected in the integration',
                    'range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration successful as far as x = {:s}, but an'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'internal error has occurred during rootfinding.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integration successful as far as x = {:s}, but an'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'internal error has occurred during interpolation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Impossible error --- internal variable IERR = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Impossible error --- internal variable IREVCM = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_cls.refresh_py(_y)
        _tol_py = utils._EngineFloat64ScalarType.to_py(_tol)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'tol'],
        )(
            _x_py,
            _tol_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_fd_nonlin_fixedbc(u, v, n, a, b, tol, fcn, mnp, x, y, np, itrace, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, boundary value problem, finite
    difference technique with deferred correction, simple nonlinear
    problem.

    For full information please refer to the NAG Library document for
    d02ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02gaf.html

    Parameters
    ----------
    u : float, shape (`n`, 2)
        `u`[i-1,0] must be set to the known or estimated value of y_i at
        a and `u`[i-1,1] must be set to the known or estimated value of
        y_i at b, for i = 1,2,...,n.

    v : float, shape (`n`, 2)
        `v`[i-1,j-1] must be set to 0.0 if `u`[i-1,j-1] is a known value
        and to 1.0 if `u`[i-1,j-1] is an estimated value, for j =
        1,2,...,2, for i = 1,2,...,n.

    n : int
        n, the number of equations.

    a : float
        a, the left-hand boundary point.

    b : float
        b, the right-hand boundary point.

    tol : float
        A positive absolute error tolerance. If

            a = x_1 < x_2 < ... < x_`np` = b

        is the final mesh, z_j(x_i) is the jth component of the
        approximate solution at x_i, and y_j(x) is the jth component of
        the true solution of equation (1) (see [Description]) and the
        boundary conditions, then, except in extreme cases, it is
        expected that

            |z_j(x_i)-y_j(x_i)| <= `tol`, i = 1,2,...,`np` and j =
            1,2,...,n.

    fcn : callable fcn(n, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i), for i = 1,2,...,n, at a general point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    mnp : int
        The maximum permitted number of mesh points.

    x : float, shape (`mnp`), modified in place
        On entry: if `np` >= 4 (see `np`), the first `np` elements must
        define an initial mesh. Otherwise the elements of `x` need not
        be set.

        On exit: `x`[0],`x`[1],...,`x`[`np`-1] define the final mesh
        (with the returned value of `np`) satisfying the relation (3).

    y : float, shape (`n`, `mnp`), modified in place
        On exit: the approximate solution z_j(x_i) satisfying (2), on
        the final mesh, that is

            `y`[j-1,i-1] = z_j(x_i), i = 1,2,...,`np` and j = 1,2,...,n,

        where `np` is the number of points in the final mesh.

        The remaining columns of `y` are not used.

    np : int
        Determines whether a default or user-supplied mesh is used.

        `np` = 0

          A default value of 4 for `np` and a corresponding equispaced
          mesh `x`[0],`x`[1],...,`x`[`np`-1] are used.

        `np` >= 4

          You must define an initial mesh using the array `x` as
          described.

    itrace : int
        If `itrace` = 0 warning messages be suppressed, otherwise
        warning messages will be printed (see [Errors]).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    np : int
        The number of points in the final (returned) mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The sequence `x` is not strictly increasing. For i =
            *<value>*, `x`[i-1] = *<value>* and `x`[i] = *<value>*.

        (`errno` 1)
            On entry: `b` = *<value>* and `x`[`np`-1] = *<value>*.

            Constraint: `a` = `x`[0] < `x`[1] < ... < `x`[`np`-1] = `b`,
            `np` >= 4.

        (`errno` 1)
            On entry: `a` = *<value>* and `x`[0] = *<value>*.

            Constraint: `a` = `x`[0] < `x`[1] < ... < `x`[`np`-1] = `b`,
            `np` >= 4.

        (`errno` 1)
            The sum of known left and right boundary values must equal
            the number of equations: the number of known left boundary
            values = *<value>*, the number of known right boundary
            values = *<value>*, the number of equations = *<value>*.

        (`errno` 1)
            The number of known right boundary values must be less than
            the number of equations: the number of known right boundary
            values = *<value>*, the number of equations = *<value>*.

        (`errno` 1)
            The number of known left boundary values must be less than
            the number of equations: the number of known left boundary
            values = *<value>*, the number of equations = *<value>*.

        (`errno` 1)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > `a`.

        (`errno` 1)
            On entry, `mnp` = *<value>*.

            Constraint: `mnp` >= 32.

        (`errno` 1)
            On entry, `np` = *<value>* and `mnp` = *<value>*.

            Constraint: `np` <= `mnp`.

        (`errno` 1)
            On entry, `np` = *<value>*.

            Constraint: `np` = 0 or `np` >= 4.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            The Newton iteration has failed to converge.

            This could be due to there being too few points in the
            initial mesh or to the initial approximate solution being
            too inaccurate. If this latter reason is suspected or you
            cannot make changes to prevent this error, you should use
            the routine with a continuation facility instead.

        (`errno` 5)
            A serious error occurred in a call to the internal
            integrator.

            The error code internally was *<value>*. Please contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Newton iteration has reached round-off level.

            If desired accuracy has not been reached, `tol` is too small
            for this problem and this machine precision.

        (`errno` 4)
            A finer mesh is required for the accuracy requested; that
            is, `mnp` = *<value>* is not large enough.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_fd_nonlin_fixedbc'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._EngineFloat64ArrayType.c_type, # Input, u[n, 2].
        utils._EngineFloat64ArrayType.c_type, # Input, v[n, 2].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, mnp.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[mnp].
        utils._EngineFloat64ArrayType.c_type, # Output, y[n, mnp].
        utils._BYREF_INT, # Input/Output, np.
        utils._BYREF_INT, # Input, itrace.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (v, 'v'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _u = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _v = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _mnp = utils._EngineIntScalarType(
        mnp, {'fun_name': fun_name, 'entity': 'mnp'},
    ).to_ctypes()
    _mnp_py = utils._EngineIntScalarType.to_py(_mnp)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mnp_py,),
    )
    _x = _x_cls.to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _mnp_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _np = utils._EngineIntScalarType(
        np, {'fun_name': fun_name, 'entity': 'np'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType.c_type(0)
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _liw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(10)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _u,
        _v,
        _n,
        _a,
        _b,
        _tol,
        _fcn,
        _fcnh,
        _mnp,
        _x,
        _y,
        _np,
        _itrace,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The sequence x is not strictly increasing. For i = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'x[i-1] = {:s} and x[i] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry: b = {:s} and x[np-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a = x[0] < x[1] < ... < x[np-1] = b, np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry: a = {:s} and x[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a = x[0] < x[1] < ... < x[np-1] = b, np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The sum of known left and right boundary values must equal the number of',
                    'equations:',
                    'the number of known left boundary values = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the number of known right boundary values = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'the number of equations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of known right boundary values must be less than the number of',
                    'equations:',
                    'the number of known right boundary values = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the number of equations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of known left boundary values must be less than the number of',
                    'equations:',
                    'the number of known left boundary values = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the number of equations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b > a.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mnp >= 32.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s} and mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: np <= mnp.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np = 0 or np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge.',
                    'This could be due to there being too few points in the initial mesh or to',
                    'the initial approximate solution being too inaccurate. If this latter reason',
                    'is suspected or you cannot make changes to prevent this error, you should',
                    'use the routine with a continuation facility instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Newton iteration has reached round-off level.',
                    'If desired accuracy has not been reached, tol is too',
                    'small for this problem and this machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A finer mesh is required for the accuracy requested;',
                    'that is, mnp = {:s} is not large enough.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in a call to the internal integrator.',
                    'The error code internally was {:s}. Please contact NAG.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _np_py = utils._EngineIntScalarType.to_py(_np)
        return_data = _np_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['np'],
            )(
                _np_py,
            )
    _err_class.manage()
    return return_data

def bvp_fd_lin_gen(a, b, n, tol, fcnf, fcng, c, d, gam, mnp, x, y, np, itrace, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, boundary value problem, finite
    difference technique with deferred correction, general linear
    problem.

    For full information please refer to the NAG Library document for
    d02gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02gbf.html

    Parameters
    ----------
    a : float
        a, the left-hand boundary point.

    b : float
        b, the right-hand boundary point.

    n : int
        The number of equations; that is n is the order of system (1).

    tol : float
        A positive absolute error tolerance. If

            a = x_1 < x_2 < ... < x_`np` = b

        is the final mesh, z(x) is the approximate solution from
        ``bvp_fd_lin_gen`` and y(x) is the true solution of equations
        (1) and (2) then, except in extreme cases, it is expected that

            ||z-y|| <= `tol`

        where

            ||u|| = max_{1 <= i <= `n`}max_{1 <= j <= `np`}|u_i(x_j)|.

    fcnf : callable fcnf(n, x, f, data=None)
        `fcnf` must evaluate the matrix F(x) in (1) at a general point
        x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float
            x, the value of the independent variable.

        f : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `f`[j-1,i-1] must contain the (i,j)th element of
            the matrix F(x), for j = 1,2,...,n, for i = 1,2,...,n. (See
            [Example] for an example.)

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcng : callable fcng(n, x, g, data=None)
        `fcng` must evaluate the vector g(x) in (1) at a general point
        x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float
            x, the value of the independent variable.

        g : float, ndarray, shape (`n`), to be modified in place
            On exit: the ith element of the vector g(x), for i =
            1,2,...,n. (See [Example] for an example.)

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    c : float, shape (`n`, `n`), modified in place
        On entry: the arrays `c` and `d` must be set to the matrices C
        and D in (2)). `gam` must be set to the vector gamma in (2).

        On exit: the rows of `c` and `d` and the components of `gam` are
        reordered so that the boundary conditions are in the order:

        - conditions on y(a) only;

        - condition involving y(a) and y(b); and

        - conditions on y(b) only.

        The function will be slightly more efficient if the arrays `c`,
        `d` and `gam` are ordered in this way before entry, and in this
        event they will be unchanged on exit.

        Note that the problems (1) and (2) must be of boundary value
        type, that is neither C nor D may be identically zero.

        Note also that the rank of the matrix [C,D] must be n for the
        problem to be properly posed.

        Any violation of these conditions will lead to an error exit.

    d : float, shape (`n`, `n`), modified in place
        On entry: the arrays `c` and `d` must be set to the matrices C
        and D in (2)). `gam` must be set to the vector gamma in (2).

        On exit: the rows of `c` and `d` and the components of `gam` are
        reordered so that the boundary conditions are in the order:

        - conditions on y(a) only;

        - condition involving y(a) and y(b); and

        - conditions on y(b) only.

        The function will be slightly more efficient if the arrays `c`,
        `d` and `gam` are ordered in this way before entry, and in this
        event they will be unchanged on exit.

        Note that the problems (1) and (2) must be of boundary value
        type, that is neither C nor D may be identically zero.

        Note also that the rank of the matrix [C,D] must be n for the
        problem to be properly posed.

        Any violation of these conditions will lead to an error exit.

    gam : float, shape (`n`), modified in place
        On entry: the arrays `c` and `d` must be set to the matrices C
        and D in (2)). `gam` must be set to the vector gamma in (2).

        On exit: the rows of `c` and `d` and the components of `gam` are
        reordered so that the boundary conditions are in the order:

        - conditions on y(a) only;

        - condition involving y(a) and y(b); and

        - conditions on y(b) only.

        The function will be slightly more efficient if the arrays `c`,
        `d` and `gam` are ordered in this way before entry, and in this
        event they will be unchanged on exit.

        Note that the problems (1) and (2) must be of boundary value
        type, that is neither C nor D may be identically zero.

        Note also that the rank of the matrix [C,D] must be n for the
        problem to be properly posed.

        Any violation of these conditions will lead to an error exit.

    mnp : int
        The maximum permitted number of mesh points.

    x : float, shape (`mnp`), modified in place
        On entry: if `np` >= 4 (see `np`), the first `np` elements must
        define an initial mesh. Otherwise the elements of x need not be
        set.

        On exit: `x`[0],`x`[1],...,`x`[`np`-1] define the final mesh
        (with the returned value of `np`) satisfying the relation
        [equation].

    y : float, shape (`n`, `mnp`), modified in place
        On exit: the approximate solution z(x) satisfying [equation], on
        the final mesh, that is

            `y`[j-1,i-1] = z_j(x_i), i = 1,2,...,`np` and j = 1,2,...,n

        where `np` is the number of points in the final mesh.

        The remaining columns of `y` are not used.

    np : int
        Determines whether a default mesh or user-supplied mesh is used.

        `np` = 0

          A default value of 4 for `np` and a corresponding equispaced
          mesh `x`[0],`x`[1],...,`x`[`np`-1] are used.

        `np` >= 4

          You must define an initial mesh `x` as in [equation].

    itrace : int
        If `itrace` = 0 warning messages be suppressed, otherwise
        warning messages will be printed (see [Errors]).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    np : int
        The number of points in the final (returned) mesh.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The sequence `x` is not strictly increasing. For i =
            *<value>*, `x`[i-1] = *<value>* and `x`[i] = *<value>*.

        (`errno` 1)
            On entry: `b` = *<value>* and `x`[`np`-1] = *<value>*.

            Constraint: `a` = `x`[0] < `x`[1] < ... < `x`[`np`-1] = `b`,
            `np` >= 4.

        (`errno` 1)
            On entry: `a` = *<value>* and `x`[0] = *<value>*.

            Constraint: `a` = `x`[0] < `x`[1] < ... < `x`[`np`-1] = `b`,
            `np` >= 4.

        (`errno` 1)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > `a`.

        (`errno` 1)
            On entry, `mnp` = *<value>*.

            Constraint: `mnp` >= 32.

        (`errno` 1)
            On entry, `np` = *<value>* and `mnp` = *<value>*.

            Constraint: `np` <= `mnp`.

        (`errno` 1)
            On entry, `np` = *<value>*.

            Constraint: `np` = 0 or `np` >= 4.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            More than `n` columns of the `n` by 2*`n` matrix [C,D] are
            identically zero, i.e., the boundary conditions are rank
            deficient. The number of non-identically zero columns is
            *<value>*.

        (`errno` 2)
            Row *<value>* of the array `c` and the corresponding row of
            array `d` are identically zero, i.e., the boundary
            conditions are rank deficient.

        (`errno` 2)
            `c` is identically zero; `n` conditions are set in `d`.

            At least one condition must be on the left. `n` = *<value>*.

        (`errno` 2)
            `d` is identically zero; `n` conditions are set in `c`.

            At least one condition must be on the right. `n` =
            *<value>*.

        (`errno` 4)
            A serious error occurred in a call to the internal
            integrator.

            The error code internally was *<value>*. Please contact NAG.

        (`errno` 5)
            At least one row of the `n` by 2*`n` matrix [C,D] is a
            linear combination of the other rows determined up to a
            numerical tolerance, i.e., the boundary conditions are rank
            deficient. The index of first such row is *<value>*.

        (`errno` 5)
            At least one row of the `n` by 2*`n` matrix [C,D] is a
            linear combination of the other rows, i.e., the boundary
            conditions are rank deficient. The index of the first such
            row is *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Newton iteration has reached round-off level.

            If desired accuracy has not been reached, `tol` is too small
            for this problem and this machine precision.

        (`errno` 3)
            The Newton iteration has failed to converge.

            This could be due to there being too few points in the
            initial mesh or to the initial approximate solution being
            too inaccurate. If this latter reason is suspected or you
            cannot make changes to prevent this error, you should use
            the routine with a continuation facility instead.

        (`errno` 3)
            A finer mesh is required for the accuracy requested; that
            is, `mnp` is not large enough.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_fd_lin_gen'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcnf_name = fun_name + ':fcnf'

    fcnfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Output, f[n, n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnfh(_fcnf, en_data, n, x, f, _iuser, _ruser):
        """Helper for fcnf."""
        try:
            _n_fcnfh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnfh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _f_fcnfh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnfh, _n_fcnfh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                fcnf(
                    _n_fcnfh,
                    _x_fcnfh,
                    _f_fcnfh,
                    data,
                )
            else:
                fcnf(
                    _n_fcnfh,
                    _x_fcnfh,
                    _f_fcnfh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnfh, {'fun_name': cb_fcnf_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_fcng_name = fun_name + ':fcng'

    fcngh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Output, g[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcngh(_fcng, en_data, n, x, g, _iuser, _ruser):
        """Helper for fcng."""
        try:
            _n_fcngh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcngh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _g_fcngh = utils._EngineFloat64ArrayType.from_pointer(
                g,
                (_n_fcngh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcng(
                    _n_fcngh,
                    _x_fcngh,
                    _g_fcngh,
                    data,
                )
            else:
                fcng(
                    _n_fcngh,
                    _x_fcngh,
                    _g_fcngh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g_fcngh, {'fun_name': cb_fcng_name, 'returned': 'g'}, g,
            )
            utils._unlink_pointer(g)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._UNUSED_FUNCTYPE,
        fcnfh,
        utils._UNUSED_FUNCTYPE,
        fcngh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n, n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, d[n, n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, gam[n].
        utils._BYREF_INT, # Input, mnp.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[mnp].
        utils._EngineFloat64ArrayType.c_type, # Output, y[n, mnp].
        utils._BYREF_INT, # Input/Output, np.
        utils._BYREF_INT, # Input, itrace.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'), (d, 'd'), (y, 'y')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    utils._check_callable(fcnf, {'fun_name': fun_name, 'entity': 'fcnf'})
    _fcnf = utils._UNUSED_CALLBACK
    _fcnfh = utils._cast_callback(
        fcnfh, cb_fcnfh,
        {'fun_name': fun_name, 'entity': 'fcnf'},
    )
    utils._check_callable(fcng, {'fun_name': fun_name, 'entity': 'fcng'})
    _fcng = utils._UNUSED_CALLBACK
    _fcngh = utils._cast_callback(
        fcngh, cb_fcngh,
        {'fun_name': fun_name, 'entity': 'fcng'},
    )
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _d = _d_cls.to_ctypes()
    _gam_cls = utils._EngineFloat64ArrayType(
        gam, {'fun_name': fun_name, 'entity': 'gam'},
        exp_shape=(_n_py,),
    )
    _gam = _gam_cls.to_ctypes()
    _mnp = utils._EngineIntScalarType(
        mnp, {'fun_name': fun_name, 'entity': 'mnp'},
    ).to_ctypes()
    _mnp_py = utils._EngineIntScalarType.to_py(_mnp)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mnp_py,),
    )
    _x = _x_cls.to_ctypes()
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _mnp_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _np = utils._EngineIntScalarType(
        np, {'fun_name': fun_name, 'entity': 'np'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType.c_type(0)
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _liw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(10)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _a,
        _b,
        _n,
        _tol,
        _fcnf,
        _fcnfh,
        _fcng,
        _fcngh,
        _c,
        _d,
        _gam,
        _mnp,
        _x,
        _y,
        _np,
        _itrace,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The sequence x is not strictly increasing. For i = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'x[i-1] = {:s} and x[i] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry: b = {:s} and x[np-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a = x[0] < x[1] < ... < x[np-1] = b, np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry: a = {:s} and x[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a = x[0] < x[1] < ... < x[np-1] = b, np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b > a.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mnp >= 32.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s} and mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: np <= mnp.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np = 0 or np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than n columns of the n by 2*n matrix [C,D] are identically zero,',
                    'i.e., the boundary conditions are rank deficient.',
                    'The number of non-identically zero columns is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Row {:s} of the array c and the corresponding row of array'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'd are identically zero, i.e., the boundary conditions are rank deficient.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'c is identically zero; n conditions are set in d.',
                    'At least one condition must be on the left. n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'd is identically zero; n conditions are set in c.',
                    'At least one condition must be on the right. n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Newton iteration has reached round-off level.',
                    'If desired accuracy has not been reached, tol is too',
                    'small for this problem and this machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge.',
                    'This could be due to there being too few points in the initial mesh or to',
                    'the initial approximate solution being too inaccurate. If this latter reason',
                    'is suspected or you cannot make changes to prevent this error, you should',
                    'use the routine with a continuation facility instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A finer mesh is required for the accuracy requested; that is,',
                    'mnp is not large enough.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in a call to the internal integrator.',
                    'The error code internally was {:s}. Please contact NAG.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one row of the n by 2*n matrix [C,D] is a linear combination',
                    'of the other rows determined up to a numerical tolerance, i.e., the',
                    'boundary conditions are rank deficient. The index of first such row is',
                    '{:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one row of the n by 2*n matrix [C,D] is a linear combination',
                    'of the other rows, i.e., the boundary conditions are rank deficient.',
                    'The index of the first such row is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _c_cls.refresh_py(_c)
        _d_cls.refresh_py(_d)
        _gam_cls.refresh_py(_gam)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _np_py = utils._EngineIntScalarType.to_py(_np)
        return_data = _np_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['np'],
            )(
                _np_py,
            )
    _err_class.manage()
    return return_data

def bvp_shoot_bval(u, v, n, a, b, tol, fcn, soln, m1, monlev, w, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, boundary value problem, shooting
    and matching, boundary values to be determined.

    For full information please refer to the NAG Library document for
    d02ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02haf.html

    Parameters
    ----------
    u : float, shape (`n`, 2), modified in place
        On entry: `u`[i-1,0] must be set to the known or estimated value
        of y_i at a and `u`[i-1,1] must be set to the known or estimated
        value of y_i at b, for i = 1,2,...,n.

        On exit: the known values unaltered, and corrected values of the
        estimates, unless an error has occurred. If an error has
        occurred, `u` contains the known values and the latest values of
        the estimates.

    v : float, shape (`n`, 2)
        `v`[i-1,j-1] must be set to 0.0 if `u`[i-1,j-1] is a known value
        and to 1.0 if `u`[i-1,j-1] is an estimated value, for j =
        1,2,...,2, for i = 1,2,...,n.

    n : int
        n, the number of equations.

    a : float
        a, the initial point of the interval of integration.

    b : float
        b, the final point of the interval of integration.

    tol : float
        Must be set to a small quantity suitable for:

        - testing the local error in y_i during integration,

        - testing for the convergence of y_i at b,

        - calculating the perturbation in estimated boundary values for
          y_i, which are used to obtain the approximate derivatives of
          the residuals for use in the Newton iteration.

        You are advised to check your results by varying `tol`.

    fcn : callable fcn(n, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i), for i = 1,2,...,n, at a general point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of f_i(x), for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    soln : float, shape (`n`, `m1`), modified in place
        On exit: the solution when `m1` > 1.

    m1 : int
        A value which controls output.

        `m1` = 1

          The final solution is not evaluated.

        `m1` > 1

          The final values of y_i at interval (b-a)/(`m1`-1) are
          calculated and stored in the array `soln` by columns, starting
          with values y_i at a stored in `soln`[i-1,0], for i =
          1,2,...,n.

    monlev : int
        Setting `monlev` = 0 disables monitoring of the pseudo-Newton
        iteration. Setting `monlev` = 1 enables this monitoring.

    w : float, shape (`n`, 3*`n`+17+max(11, `n`)), modified in place
        On exit: if `errno` = 2, 3, 4 or 5, `w`[i-1,0], for i =
        1,2,...,n, contains the solution at the point where the
        integration fails and the point of failure is returned in
        `w`[0,1].

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `monlev` = *<value>*.

            Constraint: `monlev` = 0 or 1.

        (`errno` 1)
            On entry, incorrect number of boundary values were flagged
            as known.

            Number flagged as known: *<value>*, but number should be
            *<value>*.

        (`errno` 1)
            On entry all left-hand boundary values were flagged as
            known.

        (`errno` 1)
            On entry no left-hand boundary values were flagged as known.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            In the integration with initial or final parameters, the
            step size was reduced too far for the integration to
            proceed. Either this function is not a suitable method for
            solving the problem, or the initial choice of parameters is
            very poor.

        (`errno` 3)
            In the integration with initial or final parameters, a
            suitable initial step could not be found. Either this
            function is not suitable for solving the problem, or the
            initial choice of parameters is very poor.

        (`errno` 4)
            An initial step-length could be found for integration to
            proceed with the current parameters.

        (`errno` 5)
            The step-length required to calculate the Jacobian to
            sufficient accuracy is too small

        (`errno` 6)
            The Jacobian has an insignificant column. Make sure that the
            solution vector depends on all the parameters.

        (`errno` 7)
            An internal singular value decomposition has failed.

            This error can be avoided by changing the initial parameter
            estimates.

        (`errno` 8)
            The Newton iteration has failed to converge.

            This can indicate a poor initial choice of parameters or a
            very difficult problem.

            Consider varying elements of the parameter convergence
            control if the residuals are small; otherwise vary initial
            parameter estimates.

        (`errno` 10)
            Internal error in calculating residual. Please contact NAG.

        (`errno` 11)
            Internal error in calculating Jacobian. Please contact NAG.

        (`errno` 12)
            Internal error in Newton method. Please contact NAG.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_shoot_bval'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02haft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, u[n, 2].
        utils._EngineFloat64ArrayType.c_type, # Input, v[n, 2].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._EngineFloat64ArrayType.c_type, # Output, soln[n, m1].
        utils._BYREF_INT, # Input, m1.
        utils._BYREF_INT, # Input, monlev.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n, sdw].
        utils._BYREF_INT, # Input, sdw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[15].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[15].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (v, 'v'), (soln, 'soln'), (w, 'w')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    )
    _u = _u_cls.to_ctypes()
    _v = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, 2),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _m1 = utils._EngineIntScalarType(
        m1, {'fun_name': fun_name, 'entity': 'm1'},
    ).to_ctypes()
    _m1_py = utils._EngineIntScalarType.to_py(_m1)
    _soln_cls = utils._EngineFloat64ArrayType(
        soln, {'fun_name': fun_name, 'entity': 'soln'},
        exp_shape=(_n_py, _m1_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _soln = _soln_cls.to_ctypes(can_reference_contents=False)
    _monlev = utils._EngineIntScalarType(
        monlev, {'fun_name': fun_name, 'entity': 'monlev'},
    ).to_ctypes()
    _sdw_py = utils._EngineFloat64ArrayType.get_shape(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2)[1]
    _sdw = utils._EngineIntScalarType(
        _sdw_py, {'fun_name': fun_name, 'entity': '_sdw'},
    ).to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py, 3*_n_py+17+max(11, _n_py)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(15)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(15)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _u,
        _v,
        _n,
        _a,
        _b,
        _tol,
        _fcn,
        _fcnh,
        _soln,
        _m1,
        _monlev,
        _w,
        _sdw,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, monlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: monlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, incorrect number of boundary values were flagged as known.',
                    'Number flagged as known: {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'but number should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry all left-hand boundary values were flagged as known.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry no left-hand boundary values were flagged as known.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration with initial or final parameters, the step size',
                    'was reduced too far for the integration to proceed. Either this function',
                    'is not a suitable method for solving the problem, or the initial choice',
                    'of parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration with initial or final parameters, a suitable initial',
                    'step could not be found. Either this function is not suitable for',
                    'solving the problem, or the initial choice of parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An initial step-length could be found for integration to proceed',
                    'with the current parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The step-length required to calculate the Jacobian to sufficient accuracy',
                    'is too small',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobian has an insignificant column. Make sure',
                    'that the solution vector depends on all the parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal singular value decomposition has failed.',
                    'This error can be avoided by changing the initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge.',
                    'This can indicate a poor initial choice of parameters or a very difficult',
                    'problem.',
                    'Consider varying elements of the parameter convergence control if',
                    'the residuals are small; otherwise vary initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating residual. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating Jacobian. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in Newton method. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _u_cls.refresh_py(_u)
        _soln_cls.refresh_py(_soln)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_shoot_genpar(p, n1, pe, e, n, soln, m1, monlev, fcn, bc, ebndry, w, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, boundary value problem, shooting
    and matching, general parameters to be determined.

    For full information please refer to the NAG Library document for
    d02hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02hbf.html

    Parameters
    ----------
    p : float, shape (`n1`), modified in place
        On entry: an estimate for the ith argument, p_i, for i =
        1,2,...,n_1.

        On exit: the corrected value for the ith argument, unless an
        error has occurred, when it contains the last calculated value
        of the argument.

    n1 : int
        n_1, the number of arguments.

    pe : float, shape (`n1`)
        The elements of `pe` must be given small positive values. The
        element `pe`[i-1] is used

        - in the convergence test on the ith argument in the Newton
          iteration, and

        - in perturbing the ith argument when approximating the
          derivatives of the components of the solution with respect to
          this argument for use in the Newton iteration.

        The elements `pe`[i-1] should not be chosen too small.

        They should usually be several orders of magnitude larger than
        machine precision.

    e : float, shape (`n`)
        The elements of `e` must be given positive values. The element
        `e`[i-1] is used in the bound on the local error in the ith
        component of the solution y_i during integration.

        The elements `e`[i-1] should not be chosen too small.

        They should usually be several orders of magnitude larger than
        machine precision.

    n : int
        n, the total number of differential equations.

    soln : float, shape (`n`, `m1`), modified in place
        On exit: the solution when `m1` > 1.

    m1 : int
        A value which controls exit values.

        `m1` = 1

          The final solution is not calculated.

        `m1` > 1

          The final values of the solution at interval (length of
          range)/(`m1`-1) are calculated and stored sequentially in the
          array `soln` starting with the values of the solutions
          evaluated at the first end point (see `ebndry`) stored in the
          first column of `soln`.

    monlev : int
        Setting `monlev` = 0 disables monitoring of the pseudo-Newton
        iteration. Setting `monlev` = 1 enables this monitoring.

    fcn : callable fcn(n, n1, x, y, f, p, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i), for i = 1,2,...,n, at a general point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        n1 : int
            n_1, the number of parameters.

        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,n. The f_i may
            depend upon the parameters p_j, for j = 1,2,...,n_1. If
            there are any driving equations (see [Description]) then
            these must be numbered first in the ordering of the
            components of `f` in `fcn`.

        p : float, ndarray, shape (`n1`)
            The current estimate of the argument p_i, for i =
            1,2,...,n_1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    bc : callable bc(n, n1, g1, g2, p, data=None)
        `bc` must place in `g1` and `g2` the boundary conditions at a
        and b respectively (see `ebndry`).

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        n1 : int
            n_1, the number of parameters.

        g1 : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of y_i(a), (where this may be a known
            value or a function of the parameters p_j, for j =
            1,2,...,n_1, for i = 1,2,...,n).

        g2 : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of y_i(b), for i = 1,2,...,n, (where
            these may be known values or functions of the parameters
            p_j, for j = 1,2,...,n_1). If n > n_1, so that there are
            some driving equations, the first n-n_1 values of `g2` need
            not be set since they are never used.

        p : float, ndarray, shape (`n1`)
            An estimate of the argument p_i, for i = 1,2,...,n_1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    ebndry : callable (a, b) = ebndry(n1, p, data=None)
        `ebndry` must evaluate the boundary points a and b, each of
        which may depend on the arguments p_1,p_2,...,p_n_1.

        The integrations in the shooting method are always from a to b.

        Parameters
        ~~~~~~~~~~
        n1 : int
            n_1, the number of parameters.

        p : float, ndarray, shape (`n1`)
            The current estimate of the ith argument, p_i, for i =
            1,2,...,n_1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        a : float
            a, one of the boundary points.

        b : float
            The second boundary point, b. Note that `b` > `a` forces the
            direction of integration to be that of increasing x. If `a`
            and `b` are interchanged the direction of integration is
            reversed.

    w : float, shape (`n`, 3*`n`+14+max(11, `n`)), modified in place
        On exit: with `errno` = 2, 3, 4 or 5 (see [Errors]), `w`[i-1,0],
        for i = 1,2,...,n, contains the solution at the point x when the
        error occurred. `w`[0,1] contains x.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `monlev` = *<value>*.

            Constraint: `monlev` = 0 or 1.

        (`errno` 1)
            On entry a negative or zero local error tolerance has been
            set.

        (`errno` 1)
            On entry a negative or zero convergence test tolerance has
            been set.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>* and `n1` = *<value>*.

            Constraint: `n` >= `n1`.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            The step length for the integration became too short to
            proceed when calculating the residual.

        (`errno` 2)
            In the integration with initial or final parameters, the
            step size was reduced too far for the integration to
            proceed. Either this function is not a suitable method for
            solving the problem, or the initial choice of parameters is
            very poor.

        (`errno` 3)
            An initial step-length could be found for integration to
            proceed with the current parameters.

        (`errno` 3)
            In the integration with initial or final parameters, a
            suitable initial step could not be found. Either this
            function is not suitable for solving the problem, or the
            initial choice of parameters is very poor.

        (`errno` 4)
            The step-length required to calculate the Jacobian to
            sufficient accuracy is too small

        (`errno` 5)
            An initial step-length could be found for Jacobian
            calculation to proceed with the current parameters.

        (`errno` 6)
            The Jacobian has an insignificant column. Make sure that the
            solution vector depends on all the parameters.

        (`errno` 7)
            An internal singular value decomposition has failed.

            This error can be avoided by changing the initial parameter
            estimates.

        (`errno` 8)
            The Newton iteration has failed to converge.

            This can indicate a poor initial choice of parameters or a
            very difficult problem.

            Consider varying elements of the parameter convergence
            control if the residuals are small; otherwise vary initial
            parameter estimates.

        (`errno` 9)
            Internal error in Newton method. Please contact NAG.

        (`errno` 10)
            Internal error in calculating Jacobian. Please contact NAG.

        (`errno` 11)
            Internal error in calculating residual. Please contact NAG.

        (`errno` 12)
            Internal error in calculating residual. Please contact NAG.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_shoot_genpar'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._EngineFloat64ArrayType.c_type, # Input, p[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, n, n1, x, y, f, p, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _n1_fcnh = utils._EngineIntScalarType.to_py(n1.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            _p_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_n1_fcnh,),
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _n1_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _p_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _n1_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _p_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bc_name = fun_name + ':bc'

    bch = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Output, g1[n].
        utils._EngineFloat64ArrayType.c_type, # Output, g2[n].
        utils._EngineFloat64ArrayType.c_type, # Input, p[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bch(_bc, en_data, n, n1, g1, g2, p, _iuser, _ruser):
        """Helper for bc."""
        try:
            _n_bch = utils._EngineIntScalarType.to_py(n.contents)
            _n1_bch = utils._EngineIntScalarType.to_py(n1.contents)
            _g1_bch = utils._EngineFloat64ArrayType.from_pointer(
                g1,
                (_n_bch,),
                can_reference_contents=False,
            )
            _g2_bch = utils._EngineFloat64ArrayType.from_pointer(
                g2,
                (_n_bch,),
                can_reference_contents=False,
            )
            _p_bch = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_n1_bch,),
            )
            if data is not None:
                bc(
                    _n_bch,
                    _n1_bch,
                    _g1_bch,
                    _g2_bch,
                    _p_bch,
                    data,
                )
            else:
                bc(
                    _n_bch,
                    _n1_bch,
                    _g1_bch,
                    _g2_bch,
                    _p_bch,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g1_bch, {'fun_name': cb_bc_name, 'returned': 'g1'}, g1,
            )
            utils._unlink_pointer(g1)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g2_bch, {'fun_name': cb_bc_name, 'returned': 'g2'}, g2,
            )
            utils._unlink_pointer(g2)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_range_name = fun_name + ':range'

    rangeh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_FLOAT64, # Output, a.
        utils._BYREF_FLOAT64, # Output, b.
        utils._EngineFloat64ArrayType.c_type, # Input, p[n1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_rangeh(_range, en_data, n1, a, b, p, _iuser, _ruser):
        """Helper for range."""
        try:
            _n1_rangeh = utils._EngineIntScalarType.to_py(n1.contents)
            _p_rangeh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_n1_rangeh,),
            )
            if data is not None:
                _a_rangeh, _b_rangeh = ebndry(
                    _n1_rangeh,
                    _p_rangeh,
                    data,
                )
            else:
                _a_rangeh, _b_rangeh = ebndry(
                    _n1_rangeh,
                    _p_rangeh,
                )
            utils._EngineFloat64ScalarType(
                _a_rangeh, {'fun_name': cb_range_name, 'returned': 'a'},
            ).refresh_ctypes_buffer(a)
            utils._EngineFloat64ScalarType(
                _b_rangeh, {'fun_name': cb_range_name, 'returned': 'b'},
            ).refresh_ctypes_buffer(b)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02hbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[n1].
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Input, pe[n1].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, soln[n, m1].
        utils._BYREF_INT, # Input, m1.
        utils._BYREF_INT, # Input, monlev.
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._UNUSED_FUNCTYPE,
        bch,
        utils._UNUSED_FUNCTYPE,
        rangeh,
        utils._EngineFloat64ArrayType.c_type, # Output, w[n, sdw].
        utils._BYREF_INT, # Input, sdw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[15].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[15].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((soln, 'soln'), (w, 'w')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_n1_py,),
    )
    _p = _p_cls.to_ctypes()
    _pe = utils._EngineFloat64ArrayType(
        pe, {'fun_name': fun_name, 'entity': 'pe'},
        exp_shape=(_n1_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _e = utils._EngineFloat64ArrayType(
        e, {'fun_name': fun_name, 'entity': 'e'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m1 = utils._EngineIntScalarType(
        m1, {'fun_name': fun_name, 'entity': 'm1'},
    ).to_ctypes()
    _m1_py = utils._EngineIntScalarType.to_py(_m1)
    _soln_cls = utils._EngineFloat64ArrayType(
        soln, {'fun_name': fun_name, 'entity': 'soln'},
        exp_shape=(_n_py, _m1_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _soln = _soln_cls.to_ctypes(can_reference_contents=False)
    _monlev = utils._EngineIntScalarType(
        monlev, {'fun_name': fun_name, 'entity': 'monlev'},
    ).to_ctypes()
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    utils._check_callable(bc, {'fun_name': fun_name, 'entity': 'bc'})
    _bc = utils._UNUSED_CALLBACK
    _bch = utils._cast_callback(
        bch, cb_bch,
        {'fun_name': fun_name, 'entity': 'bc'},
    )
    utils._check_callable(ebndry, {'fun_name': fun_name, 'entity': 'ebndry'})
    _ebndry = utils._UNUSED_CALLBACK
    _rangeh = utils._cast_callback(
        rangeh, cb_rangeh,
        {'fun_name': fun_name, 'entity': 'range'},
    )
    _sdw_py = utils._EngineFloat64ArrayType.get_shape(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2)[1]
    _sdw = utils._EngineIntScalarType(
        _sdw_py, {'fun_name': fun_name, 'entity': '_sdw'},
    ).to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py, 3*_n_py+14+max(11, _n_py)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(15)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(15)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _p,
        _n1,
        _pe,
        _e,
        _n,
        _soln,
        _m1,
        _monlev,
        _fcn,
        _fcnh,
        _bc,
        _bch,
        _ebndry,
        _rangeh,
        _w,
        _sdw,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, monlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: monlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry a negative or zero local error tolerance has been set.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry a negative or zero convergence test tolerance has been set.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= n1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The step length for the integration became too short to proceed',
                    'when calculating the residual.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration with initial or final parameters, the step size',
                    'was reduced too far for the integration to proceed. Either this function',
                    'is not a suitable method for solving the problem, or the initial choice',
                    'of parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An initial step-length could be found for integration to proceed',
                    'with the current parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration with initial or final parameters, a suitable initial',
                    'step could not be found. Either this function is not suitable for',
                    'solving the problem, or the initial choice of parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The step-length required to calculate the Jacobian to sufficient accuracy',
                    'is too small',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An initial step-length could be found for Jacobian calculation to proceed',
                    'with the current parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobian has an insignificant column. Make sure',
                    'that the solution vector depends on all the parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal singular value decomposition has failed.',
                    'This error can be avoided by changing the initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge.',
                    'This can indicate a poor initial choice of parameters or a very difficult',
                    'problem.',
                    'Consider varying elements of the parameter convergence control if',
                    'the residuals are small; otherwise vary initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in Newton method. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating Jacobian. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating residual. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating residual. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _soln_cls.refresh_py(_soln)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_coll_nth(n, cf, bc, x0, x1, k1, kp, c, comm, data=None):
    u"""
    Ordinary differential equations, boundary value problem, collocation
    and least squares, single nth-order linear equation.

    For full information please refer to the NAG Library document for
    d02ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02jaf.html

    Parameters
    ----------
    n : int
        n, the order of the differential equation.

    cf : callable retval = cf(j, x, data=None)
        `cf` defines the differential equation (see [Description]).

        It must return the value of a function f_j(x) at a given point
        x, where, for 1 <= j <= n+1, f_j(x) is the coefficient of
        y^(j-1)(x) in the equation, and f_0(x) is the right-hand side.

        Parameters
        ~~~~~~~~~~
        j : int
            The index of the function f_j to be evaluated.

        x : float
            The point at which f_j is to be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f_j(x) at the given point x.

    bc : callable (j, rhs) = bc(i, data=None)
        `bc` defines the boundary conditions, each of which has the form
        y^(k-1)(x_1) = s_k or y^(k-1)(x_0) = s_k.

        The boundary conditions may be specified in any order.

        Parameters
        ~~~~~~~~~~
        i : int
            The index of the boundary condition to be defined.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        j : int
            Must be set to -k if the boundary condition is y^(k-1)(x_0)
            = s_k, and to +k if it is y^(k-1)(x_1) = s_k.

            `j` must not be set to the same value k for two different
            values of `i`.

        rhs : float
            Must be set to the value s_k.

    x0 : float
        The left- and right-hand boundaries, x_0 and x_1, respectively.

    x1 : float
        The left- and right-hand boundaries, x_0 and x_1, respectively.

    k1 : int
        The number of coefficients to be returned in the Chebyshev
        series representation of the solution (hence the degree of the
        polynomial approximation is `k1`-1).

    kp : int
        The number of collocation points to be used.

    c : float, shape (`k1`), modified in place
        On exit: the computed Chebyshev coefficients; that is, the
        computed solution is:

            ⎲_i = 1^`k1``c`[i-1]T_{i-1}(x)

        where T_i(x) is the ith Chebyshev polynomial of the first kind,
        and ⎲ denotes that the first coefficient, `c`[0], is halved.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kp` = *<value>*, `n` = *<value>* and `k1` =
            *<value>*.

            Constraint: `kp`+`n` >= `k1`.

        (`errno` 1)
            On entry, `k1` = *<value>* and `n` = *<value>*.

            Constraint: `k1` >= `n`+1.

        (`errno` 1)
            On entry, `x1` = *<value>* and `x0` = *<value>*.

            Constraint: `x1` > `x0`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            Either the boundary conditions are not linearly independent,
            or the coefficient matrix is rank deficient. Increasing the
            number of collocation points may overcome this latter
            problem.

        (`errno` 4)
            Iterative refinement in the least squares solution has
            failed to converge. The coefficient matrix is too
            ill-conditioned.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nth'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_cf_name = fun_name + ':cf'

    cfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, j.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_cfh(_cf, _cfsub, en_data, j, x, retval, _iuser, _ruser):
        """Helper for cf."""
        try:
            _j_cfh = utils._EngineIntScalarType.to_py(j.contents)
            _x_cfh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_cfh = cf(
                    _j_cfh,
                    _x_cfh,
                    data,
                )
            else:
                _retval_cfh = cf(
                    _j_cfh,
                    _x_cfh,
                )
            utils._EngineFloat64ScalarType(
                _retval_cfh, {'fun_name': cb_cf_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bc_name = fun_name + ':bc'

    bch = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, i.
        utils._BYREF_INT, # Output, j.
        utils._BYREF_FLOAT64, # Output, rhs.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bch(_bc, en_data, i, j, rhs, _iuser, _ruser):
        """Helper for bc."""
        try:
            _i_bch = utils._EngineIntScalarType.to_py(i.contents)
            if data is not None:
                _j_bch, _rhs_bch = bc(
                    _i_bch,
                    data,
                )
            else:
                _j_bch, _rhs_bch = bc(
                    _i_bch,
                )
            utils._EngineIntScalarType(
                _j_bch, {'fun_name': cb_bc_name, 'returned': 'j'},
            ).refresh_ctypes_buffer(j)
            utils._EngineFloat64ScalarType(
                _rhs_bch, {'fun_name': cb_bc_name, 'returned': 'rhs'},
            ).refresh_ctypes_buffer(rhs)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        cfh,
        utils._UNUSED_FUNCTYPE,
        bch,
        utils._BYREF_FLOAT64, # Input, x0.
        utils._BYREF_FLOAT64, # Input, x1.
        utils._BYREF_INT, # Input, k1.
        utils._BYREF_INT, # Input, kp.
        utils._EngineFloat64ArrayType.c_type, # Output, c[k1].
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[5].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    utils._check_callable(cf, {'fun_name': fun_name, 'entity': 'cf'})
    _cf = utils._UNUSED_CALLBACK
    _cfsub = utils._UNUSED_CALLBACK
    _cfh = utils._cast_callback(
        cfh, cb_cfh,
        {'fun_name': fun_name, 'entity': 'cf'},
    )
    utils._check_callable(bc, {'fun_name': fun_name, 'entity': 'bc'})
    _bc = utils._UNUSED_CALLBACK
    _bch = utils._cast_callback(
        bch, cb_bch,
        {'fun_name': fun_name, 'entity': 'bc'},
    )
    _x0 = utils._EngineFloat64ScalarType(
        x0, {'fun_name': fun_name, 'entity': 'x0'},
    ).to_ctypes()
    _x1 = utils._EngineFloat64ScalarType(
        x1, {'fun_name': fun_name, 'entity': 'x1'},
    ).to_ctypes()
    _k1 = utils._EngineIntScalarType(
        k1, {'fun_name': fun_name, 'entity': 'k1'},
    ).to_ctypes()
    _kp = utils._EngineIntScalarType(
        kp, {'fun_name': fun_name, 'entity': 'kp'},
    ).to_ctypes()
    _k1_py = utils._EngineIntScalarType.to_py(_k1)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_k1_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType.c_type(0)
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(5)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(5)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _cf,
        _cfsub,
        _cfh,
        _bc,
        _bch,
        _x0,
        _x1,
        _k1,
        _kp,
        _c,
        _w,
        _lw,
        _iw,
        _iuser,
        _ruser,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kp = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'k1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: kp+n >= k1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k1 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k1 >= n+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x1 = {:s} and x0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x1 > x0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the boundary conditions are not linearly independent, or the',
                    'coefficient matrix is rank deficient. Increasing the number of',
                    'collocation points may overcome this latter problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Iterative refinement in the least squares solution has failed to',
                    'converge. The coefficient matrix is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_coll_sys(n, cf, bc, x0, x1, k1, kp, c, comm, data=None):
    u"""
    Ordinary differential equations, boundary value problem, collocation
    and least squares, system of first-order linear equations.

    For full information please refer to the NAG Library document for
    d02jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02jbf.html

    Parameters
    ----------
    n : int
        n, the order of the system of differential equations.

    cf : callable retval = cf(i, j, x, data=None)
        `cf` defines the system of differential equations (see
        [Description]).

        It must return the value of a coefficient function a_{i,j}(x),
        of A, at a given point x, or of a right-hand side function
        r_i(x) if `j` = 0.

        Parameters
        ~~~~~~~~~~
        i : int
            Indicate the function to be evaluated, namely a_{i,j}(x) if
            1 <= `j` <= n, or r_i(x) if `j` = 0.

            1 <= `i` <= n, 0 <= `j` <= n.

        j : int
            Indicate the function to be evaluated, namely a_{i,j}(x) if
            1 <= `j` <= n, or r_i(x) if `j` = 0.

            1 <= `i` <= n, 0 <= `j` <= n.

        x : float
            The point at which the function is to be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of a coefficient function a_{i,j}(x), of A, at a
            given point x, or of a right-hand side function r_i(x) if
            `j` = 0.

    bc : callable (j, rhs) = bc(i, data=None)
        `bc` defines the n boundary conditions, which have the form
        y_k(x_0) = s or y_k(x_1) = s.

        The boundary conditions may be specified in any order.

        Parameters
        ~~~~~~~~~~
        i : int
            The index of the boundary condition to be defined.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        j : int
            Must be set to -k if the ith boundary condition is y_k(x_0)
            = s, or to +k if it is y_k(x_1) = s.

            `j` must not be set to the same value k for two different
            values of `i`.

        rhs : float
            The value s.

    x0 : float
        The left- and right-hand boundaries, x_0 and x_1, respectively.

    x1 : float
        The left- and right-hand boundaries, x_0 and x_1, respectively.

    k1 : int
        The number of coefficients to be returned in the Chebyshev
        series representation of the components of the solution (hence
        the degree of the polynomial approximation is `k1`-1).

    kp : int
        The number of collocation points to be used.

    c : float, shape (`k1`, `n`), modified in place
        On exit: the computed Chebyshev coefficients of the kth
        component of the solution, y_k; that is, the computed solution
        is:

            y_k = ⎲_i = 1^`k1``c`[i-1,k-1]T_{i-1}(x), 1 <= k <= n

        where T_i(x) is the ith Chebyshev polynomial of the first kind,
        and ⎲ denotes that the first coefficient, `c`[0,k-1], is halved.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kp` = *<value>* and `k1` = *<value>*.

            Constraint: `kp`+1 >= `k1`.

        (`errno` 1)
            On entry, `k1` = *<value>*.

            Constraint: `k1` >= 2.

        (`errno` 1)
            On entry, `x1` = *<value>* and `x0` = *<value>*.

            Constraint: `x1` > `x0`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            Either the boundary conditions are not linearly independent,
            or the coefficient matrix is rank deficient. Increasing the
            number of collocation points may overcome this latter
            problem.

        (`errno` 4)
            Iterative refinement in the least squares solution has
            failed to converge. The coefficient matrix is too
            ill-conditioned.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_sys'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_cf_name = fun_name + ':cf'

    cfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, i.
        utils._BYREF_INT, # Input, j.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_cfh(_cf, _cfsub, en_data, i, j, x, retval, _iuser, _ruser):
        """Helper for cf."""
        try:
            _i_cfh = utils._EngineIntScalarType.to_py(i.contents)
            _j_cfh = utils._EngineIntScalarType.to_py(j.contents)
            _x_cfh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_cfh = cf(
                    _i_cfh,
                    _j_cfh,
                    _x_cfh,
                    data,
                )
            else:
                _retval_cfh = cf(
                    _i_cfh,
                    _j_cfh,
                    _x_cfh,
                )
            utils._EngineFloat64ScalarType(
                _retval_cfh, {'fun_name': cb_cf_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bc_name = fun_name + ':bc'

    bch = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, i.
        utils._BYREF_INT, # Output, j.
        utils._BYREF_FLOAT64, # Output, rhs.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bch(_bc, en_data, i, j, rhs, _iuser, _ruser):
        """Helper for bc."""
        try:
            _i_bch = utils._EngineIntScalarType.to_py(i.contents)
            if data is not None:
                _j_bch, _rhs_bch = bc(
                    _i_bch,
                    data,
                )
            else:
                _j_bch, _rhs_bch = bc(
                    _i_bch,
                )
            utils._EngineIntScalarType(
                _j_bch, {'fun_name': cb_bc_name, 'returned': 'j'},
            ).refresh_ctypes_buffer(j)
            utils._EngineFloat64ScalarType(
                _rhs_bch, {'fun_name': cb_bc_name, 'returned': 'rhs'},
            ).refresh_ctypes_buffer(rhs)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02jbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        cfh,
        utils._UNUSED_FUNCTYPE,
        bch,
        utils._BYREF_FLOAT64, # Input, x0.
        utils._BYREF_FLOAT64, # Input, x1.
        utils._BYREF_INT, # Input, k1.
        utils._BYREF_INT, # Input, kp.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[5].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    utils._check_callable(cf, {'fun_name': fun_name, 'entity': 'cf'})
    _cf = utils._UNUSED_CALLBACK
    _cfsub = utils._UNUSED_CALLBACK
    _cfh = utils._cast_callback(
        cfh, cb_cfh,
        {'fun_name': fun_name, 'entity': 'cf'},
    )
    utils._check_callable(bc, {'fun_name': fun_name, 'entity': 'bc'})
    _bc = utils._UNUSED_CALLBACK
    _bch = utils._cast_callback(
        bch, cb_bch,
        {'fun_name': fun_name, 'entity': 'bc'},
    )
    _x0 = utils._EngineFloat64ScalarType(
        x0, {'fun_name': fun_name, 'entity': 'x0'},
    ).to_ctypes()
    _x1 = utils._EngineFloat64ScalarType(
        x1, {'fun_name': fun_name, 'entity': 'x1'},
    ).to_ctypes()
    _k1 = utils._EngineIntScalarType(
        k1, {'fun_name': fun_name, 'entity': 'k1'},
    ).to_ctypes()
    _kp = utils._EngineIntScalarType(
        kp, {'fun_name': fun_name, 'entity': 'kp'},
    ).to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _k1_py = utils._EngineIntScalarType.to_py(_k1)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_k1_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType.c_type(0)
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _liw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(5)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(5)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _cf,
        _cfsub,
        _cfh,
        _bc,
        _bch,
        _x0,
        _x1,
        _k1,
        _kp,
        _c,
        _ldc,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kp = {:s} and k1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: kp+1 >= k1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k1 >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x1 = {:s} and x0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x1 > x0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the boundary conditions are not linearly independent, or the',
                    'coefficient matrix is rank deficient. Increasing the number of',
                    'collocation points may overcome this latter problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Iterative refinement in the least squares solution has failed to',
                    'converge. The coefficient matrix is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def sl2_reg_finite(xl, xr, coeffn, bcond, k, tol, elam, delam, monit, data=None):
    u"""
    Second-order Sturm--Liouville problem, regular system, finite range,
    eigenvalue only.

    For full information please refer to the NAG Library document for
    d02ka

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02kaf.html

    Parameters
    ----------
    xl : float
        The left- and right-hand end points a and b respectively, of the
        interval of definition of the problem.

    xr : float
        The left- and right-hand end points a and b respectively, of the
        interval of definition of the problem.

    coeffn : callable (p, q, dqdl) = coeffn(x, elam, jint, data=None)
        `coeffn` must compute the values of the coefficient functions
        p(x) and q(x;lambda) for given values of x and lambda.
        [Description] states the conditions which p and q must satisfy.

        Parameters
        ~~~~~~~~~~
        x : float
            The current value of x.

        elam : float
            The current trial value of the eigenvalue argument lambda.

        jint : int
            Need not be set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        p : float
            The value of p(x) for the current value of x.

        q : float
            The value of q(x;lambda) for the current value of x and the
            current trial value of lambda.

        dqdl : float
            The value of dq/d lambda(x;lambda) for the current value of
            x and the current trial value of lambda. However `dqdl` is
            only used in error estimation and, in the rare cases where
            it may be difficult to evaluate, an approximation (say to
            within 20%) will suffice.

    bcond : float, shape (3, 2), modified in place
        On entry: `bcond`[0,0] and `bcond`[1,0] must contain the numbers
        a_1, a_2 specifying the left-hand boundary condition in the form

            a_2 y(a) = a_1 p(a)y'(a)

        where |a_2|+|a_1 p(a)| != 0.

        `bcond`[0,1] and `bcond`[1,1] must contain b_1, b_2 such that

            b_2 y(b) = b_1 p(b)y'(b)

        where |b_2|+|b_1 p(b)| != 0.

        Note the occurrence of p(a), p(b) in these formulae.

        On exit: `bcond`[2,0] and `bcond`[2,1] hold values
        sigma_l,sigma_r estimating the sensitivity of the computed
        eigenvalue to changes in the boundary conditions. These values
        should only be of interest if the boundary conditions are, in
        some sense, an approximation to some 'true' boundary conditions.
        For example, if the range [`xl`, `xr`] should really be [0,
        infinity ] but instead `xr` has been given a large value and the
        boundary conditions at infinity applied at `xr`, the sensitivity
        argument sigma_r may be of interest. Refer to [Further
        Comments], for the actual meaning of sigma_r and sigma_l.

    k : int
        k, the index of the required eigenvalue when the eigenvalues are
        ordered

            lambda_0 < lambda_1 < lambda_2 < ... < lambda_k < ....

    tol : float
        The tolerance argument which determines the accuracy of the
        computed eigenvalue. The error estimate held in `delam` on exit
        satisfies the mixed absolute/relative error test

            `delam` <= `tol`*max(1.0, |`elam`|),

        where `elam` is the final estimate of the eigenvalue. `delam` is
        usually somewhat smaller than the right-hand side of (1) but not
        several orders of magnitude smaller.

    elam : float
        An initial estimate of the eigenvalue lambda~.

    delam : float
        An indication of the scale of the problem in the
        lambda-direction. `delam` holds the initial 'search step'
        (positive or negative). Its value is not critical, but the first
        two trial evaluations are made at `elam` and `elam`+`delam`, so
        the function will work most efficiently if the eigenvalue lies
        between these values. A reasonable choice (if a closer bound is
        not known) is about half the distance between adjacent
        eigenvalues in the neighbourhood of the one sought. In practice,
        there will often be a problem, similar to the one in hand but
        with known eigenvalues, which will help one to choose initial
        values for `elam` and `delam`.

        If `delam` = 0.0 on entry, it is given the default value of
        0.25*max(1.0, |`elam`|).

    monit : None or callable monit(nit, iflag, elam, finfo, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monit` is called by ``sl2_reg_finite`` at the end of each
        iteration for lambda and allows you to monitor the course of the
        computation by printing out the arguments (see [Example] for an
        example).

        Parameters
        ~~~~~~~~~~
        nit : int
            15 minus the number of iterations used so far in the search
            for lambda~. (Up to 15 iterations are permitted.)

        iflag : int
            Describes what phase the computation is in.

            `iflag` < 0

              An error occurred in the computation at this iteration; an
              error exit from ``sl2_reg_finite`` will follow.

            `iflag` = 1

              The function is trying to bracket the eigenvalue lambda~.

            `iflag` = 2

              The function is converging to the eigenvalue lambda~
              (having already bracketed it).

            Normally, the iteration will terminate after a sequence of
            iterates with `iflag` = 2, but occasionally the bracket on
            lambda~ thus determined will not be sufficiently small and
            the iteration will be repeated with tighter accuracy
            control.

        elam : float
            The current trial value of lambda~.

        finfo : float, ndarray, shape (15)
            Information about the behaviour of the shooting method, and
            diagnostic information in the case of errors. It should not
            normally be printed in full if no error has occurred (that
            is, if `iflag` >= 0), though the first few components may be
            of interest to you. In case of an error (`iflag` < 0) all
            the components of `finfo` should be printed.

            The contents of `finfo` are as follows:

            `finfo`[0]

              The current value of the 'miss-distance' or 'residual'
              function f(lambda) on which the shooting method is based.
              f(lambda~) = 0 in theory. This is set to zero if `iflag` <
              0.

            `finfo`[1]

              An estimate of the quantity d lambda defined as follows.
              Consider the perturbation in the miss-distance f(lambda)
              that would result if the local error in the solution of
              the differential equation were always positive and equal
              to its maximum permitted value. Then d lambda is the
              perturbation in lambda that would have the same effect on
              f(lambda). Thus, at the zero of f(lambda),|d lambda| is an
              approximate bound on the perturbation of the zero (that is
              the eigenvalue) caused by errors in numerical solution. If
              d lambda is very large then it is possible that there has
              been a programming error in `coeffn` such that q is
              independent of lambda. If this is the case, an error exit
              with `errno` = 5 should follow. `finfo`[1] is set to zero
              if `iflag` < 0.

            `finfo`[2]

              The number of internal iterations, using the same value of
              lambda and tighter accuracy tolerances, needed to bring
              the accuracy (that is, the value of d lambda) to an
              acceptable value. Its value should normally be 1.0, and
              should almost never exceed 2.0.

            `finfo`[3]

              The number of calls to `coeffn` at this iteration.

            `finfo`[4]

              The number of successful steps taken by the internal
              differential equation solver at this iteration. A step is
              successful if it is used to advance the integration.

            `finfo`[5]

              The number of unsuccessful steps used by the internal
              integrator at this iteration.

            `finfo`[6]

              The number of successful steps at the maximum step size
              taken by the internal integrator at this iteration.

            `finfo`[7]

              Not used.

            `finfo`[8] to `finfo`[14]

              Set to zero, unless `iflag` < 0 in which case they hold
              the following values describing the point of failure:

              `finfo`[8]

                1 or 2 depending on whether integration was in a forward
                or backward direction at the time of failure.

              `finfo`[9]

                The value of the independent variable, x, the point at
                which the error occurred.

              `finfo`[10], `finfo`[11], `finfo`[12]

                The current values of the Pruefer dependent variables
                beta, phi and rho respectively. See [Description] for a
                description of these variables.

              `finfo`[13]

                The local-error tolerance being used by the internal
                integrator at the point of failure.

              `finfo`[14]

                The last integration mesh point.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    elam : float
        The final computed estimate, whether or not an error occurred.

    delam : float
        If no exception or warning is raised, `delam` holds an estimate
        of the absolute error in the computed eigenvalue, that is
        |lambda~-`elam`| ~= `delam`, where lambda~ is the true
        eigenvalue.

        If an exception is raised, `delam` may hold an estimate of the
        error, or its initial value, depending on the value of errno.

        See [Errors] for further details.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            The constraint on the left- or right-hand boundary condition
            has been violated.

        (`errno` 3)
            The function p(x) became zero or changed sign in the
            interval [a,b].

        (`errno` 4)
            After 15 iterations the eigenvalue had not been found to the
            required accuracy.

        (`errno` 5)
            The bracketing phase failed to bracket the eigenvalue within
            ten iterations. This may be due to an error in formulating
            the problem (for example, q is independent of lambda), or by
            very poor initial estimates for the eigenvalue and search
            step.

        (`errno` 6)
            Too small a step size was required at the start of a
            sub-interval to obtain the desired accuracy.

        (`errno` 7)
            Too small a step size was required during solution in a
            sub-interval to obtain the desired accuracy.

        (`errno` 9)
            An internal error has occurred corresponding to a pole of
            the matching function. Try solving the problem again with a
            smaller tolerance.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The tolerance set is too small for the problem being solved
            and the machine precision being used. The local eigenvalue
            estimate returned is likely to be a very good approximation.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.sl2_reg_finite'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_coeffn_name = fun_name + ':coeffn'

    coeffnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, q.
        utils._BYREF_FLOAT64, # Output, dqdl.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._BYREF_INT, # Input, jint.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_coeffnh(_coeffn, en_data, p, q, dqdl, x, elam, jint, _iuser, _ruser):
        """Helper for coeffn."""
        try:
            _x_coeffnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _elam_coeffnh = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _jint_coeffnh = utils._EngineIntScalarType.to_py(jint.contents)
            if data is not None:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                    data,
                )
            else:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                )
            utils._EngineFloat64ScalarType(
                _p_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'p'},
            ).refresh_ctypes_buffer(p)
            utils._EngineFloat64ScalarType(
                _q_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'q'},
            ).refresh_ctypes_buffer(q)
            utils._EngineFloat64ScalarType(
                _dqdl_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'dqdl'},
            ).refresh_ctypes_buffer(dqdl)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nit.
        utils._BYREF_INT, # Input, iflag.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._EngineFloat64ArrayType.c_type, # Input, finfo[15].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, nit, iflag, elam, finfo, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _nit_monith = utils._EngineIntScalarType.to_py(nit.contents)
            _iflag_monith = utils._EngineIntScalarType.to_py(iflag.contents)
            _elam_monith = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _finfo_monith = utils._EngineFloat64ArrayType.from_pointer(
                finfo,
                (15,),
            )
            if data is not None:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                    data,
                )
            else:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                )
            utils._unlink_pointer(finfo)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02kaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xl.
        utils._BYREF_FLOAT64, # Input, xr.
        utils._UNUSED_FUNCTYPE,
        coeffnh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bcond[3, 2].
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Input/Output, elam.
        utils._BYREF_FLOAT64, # Input/Output, delam.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((bcond, 'bcond'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _xl = utils._EngineFloat64ScalarType(
        xl, {'fun_name': fun_name, 'entity': 'xl'},
    ).to_ctypes()
    _xr = utils._EngineFloat64ScalarType(
        xr, {'fun_name': fun_name, 'entity': 'xr'},
    ).to_ctypes()
    utils._check_callable(coeffn, {'fun_name': fun_name, 'entity': 'coeffn'})
    _coeffn = utils._UNUSED_CALLBACK
    _coeffnh = utils._cast_callback(
        coeffnh, cb_coeffnh,
        {'fun_name': fun_name, 'entity': 'coeffn'},
    )
    _bcond_cls = utils._EngineFloat64ArrayType(
        bcond, {'fun_name': fun_name, 'entity': 'bcond'},
        exp_shape=(3, 2),
        mutual_sorder=_m_sorder,
    )
    _bcond = _bcond_cls.to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _elam = utils._EngineFloat64ScalarType(
        elam, {'fun_name': fun_name, 'entity': 'elam'},
    ).to_ctypes()
    _delam = utils._EngineFloat64ScalarType(
        delam, {'fun_name': fun_name, 'entity': 'delam'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xl,
        _xr,
        _coeffn,
        _coeffnh,
        _bcond,
        _k,
        _tol,
        _elam,
        _delam,
        _monit,
        _monith,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The constraint on the left- or right-hand boundary condition has been',
                    'violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function p(x) became zero or changed sign in the interval [a,b].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'After 15 iterations the eigenvalue had not been found to the required',
                    'accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The bracketing phase failed to bracket the eigenvalue within ten',
                    'iterations. This may be due to an error in formulating the problem',
                    '(for example, q is independent of lambda), or by very poor initial',
                    'estimates for the eigenvalue and search step.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required at the start of a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required during solution in a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The tolerance set is too small for the problem being solved and the',
                    'machine precision being used. The local eigenvalue estimate returned is',
                    'likely to be a very good approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred corresponding to a pole of the matching',
                    'function. Try solving the problem again with a smaller tolerance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _bcond_cls.refresh_py(_bcond)
        _elam_py = utils._EngineFloat64ScalarType.to_py(_elam)
        _delam_py = utils._EngineFloat64ScalarType.to_py(_delam)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['elam', 'delam'],
        )(
            _elam_py,
            _delam_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sl2_breaks_vals(xpoint, m, coeffn, bdyval, k, tol, elam, delam, hmax, maxit, maxfun, monit, data=None):
    u"""
    Second-order Sturm--Liouville problem, regular/singular system,
    finite/infinite range, eigenvalue only, user-specified break-points.

    For full information please refer to the NAG Library document for
    d02kd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02kdf.html

    Parameters
    ----------
    xpoint : float, shape (`m`)
        The points where the boundary conditions computed by `bdyval`
        are to be imposed, and also any break-points, i.e., `xpoint`[0]
        to `xpoint`[m-1] must contain values x_1,...,x_m such that

            x_1 <= x_2 < x_3 < ... < x_{m-1} <= x_m

        with the following meanings:

        - x_1 and x_m are the left- and right-hand end points, a and b,
          of the domain of definition of the Sturm--Liouville system if
          these are finite. If either a or b is infinite, the
          corresponding value x_1 or x_m may be a more-or-less
          arbitrarily 'large' number of appropriate sign.

        - x_2 and x_{m-1} are the Boundary Matching Points (BMPs), that
          is the points at which the left and right boundary conditions
          computed in `bdyval` are imposed.

          If the left-hand end point is a regular point then you should
          set x_2 = x_1 (= a), while if it is a singular point you must
          set x_2 > x_1.

          Similarly x_{m-1} = x_m (= b) if the right-hand end point is
          regular, and x_{m-1} < x_m if it is singular.

        - The remaining m-4 points x_3,...,x_{m-2}, if any, define
          'break-points' which divide the interval [x_2,x_{m-1}] into
          m-3 sub-intervals

              i_1 = [x_2,x_3],...,i_{m-3} = [x_{m-2},x_{m-1}].

          Numerical integration of the differential equation is stopped
          and restarted at each break-point. In simple cases no
          break-points are needed. However, if p(x) or q(x;lambda) are
          given by different formulae in different parts of the
          interval, integration is more efficient if the range is broken
          up by break-points in the appropriate way. Similarly points
          where any jumps occur in p(x) or q(x;lambda), or in their
          derivatives up to the fifth-order, should appear as
          break-points.

          Examples are given in [Further Comments]. `xpoint` determines
          the position of the Shooting Matching Point (SMP), as
          explained in [Further Comments].

    m : int
        The number of points in the array `xpoint`.

    coeffn : callable (p, q, dqdl) = coeffn(x, elam, jint, data=None)
        `coeffn` must compute the values of the coefficient functions
        p(x) and q(x;lambda) for given values of x and lambda.
        [Description] states the conditions which p and q must satisfy.

        See [Further Comments] for examples.

        Parameters
        ~~~~~~~~~~
        x : float
            The current value of x.

        elam : float
            The current trial value of the eigenvalue argument lambda.

        jint : int
            The index j of the sub-interval i_j (see specification of
            `xpoint`) in which x lies.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        p : float
            The value of p(x) for the current value of x.

        q : float
            The value of q(x;lambda) for the current value of x and the
            current trial value of lambda.

        dqdl : float
            The value of dq/d lambda(x;lambda) for the current value of
            x and the current trial value of lambda. However `dqdl` is
            only used in error estimation and, in the rare cases where
            it may be difficult to evaluate, an approximation (say to
            within 20%) will suffice.

    bdyval : callable bdyval(xl, xr, elam, yl, yr, data=None)
        `bdyval` must define the boundary conditions.

        For each end point, `bdyval` must return (in `yl` or `yr`)
        values of y(x) and p(x)y'(x) which are consistent with the
        boundary conditions at the end points; only the ratio of the
        values matters.

        Here x is a given point (`xl` or `xr`) equal to, or close to,
        the end point.

        Parameters
        ~~~~~~~~~~
        xl : float
            If a is a regular end point of the system (so that a = x_1 =
            x_2), `xl` contains a. If a is a singular point (so that a
            <= x_1 < x_2), `xl` contains a point x such that x_1 < x <=
            x_2.

        xr : float
            If b is a regular end point of the system (so that x_{m-1} =
            x_m = b), `xr` contains b. If b is a singular point (so that
            x_{m-1} < x_m <= b), `xr` contains a point x such that
            x_{m-1} <= x < x_m.

        elam : float
            The current trial value of lambda.

        yl : float, ndarray, shape (3), to be modified in place
            On exit: `yl`[0] and `yl`[1] should contain values of y(x)
            and p(x)y'(x) respectively (not both zero) which are
            consistent with the boundary condition at the left-hand end
            point, given by x = `xl`. `yl`[2] should not be set.

        yr : float, ndarray, shape (3), to be modified in place
            On exit: `yr`[0] and `yr`[1] should contain values of y(x)
            and p(x)y'(x) respectively (not both zero) which are
            consistent with the boundary condition at the right-hand end
            point, given by x = `xr`. `yr`[2] should not be set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    k : int
        k, the index of the required eigenvalue when the eigenvalues are
        ordered

            lambda_0 < lambda_1 < lambda_2 < ... < lambda_k < ....

    tol : float
        The tolerance argument which determines the accuracy of the
        computed eigenvalue. The error estimate held in `delam` on exit
        satisfies the mixed absolute/relative error test

            `delam` <= `tol`*max(1.0, |`elam`|),

        where `elam` is the final estimate of the eigenvalue. `delam` is
        usually somewhat smaller than the right-hand side of (1) but not
        several orders of magnitude smaller.

    elam : float
        An initial estimate of the eigenvalue lambda~.

    delam : float
        An indication of the scale of the problem in the
        lambda-direction. `delam` holds the initial 'search step'
        (positive or negative). Its value is not critical, but the first
        two trial evaluations are made at `elam` and `elam`+`delam`, so
        the function will work most efficiently if the eigenvalue lies
        between these values. A reasonable choice (if a closer bound is
        not known) is half the distance between adjacent eigenvalues in
        the neighbourhood of the one sought. In practice, there will
        often be a problem, similar to the one in hand but with known
        eigenvalues, which will help one to choose initial values for
        `elam` and `delam`.

        If `delam` = 0.0 on entry, it is given the default value of
        0.25*max(1.0, |`elam`|).

    hmax : float, shape (2, `m`), modified in place
        On entry: `hmax`[0,j-1] should contain a maximum step size to be
        used by the differential equation code in the jth sub-interval
        i_j (as described in the specification of argument `xpoint`),
        for j = 1,2,...,m-3. If it is zero the function generates a
        maximum step size internally.

        It is recommended that `hmax`[0,j-1] be set to zero unless the
        coefficient functions p and q have features (such as a narrow
        peak) within the jth sub-interval that could be 'missed' if a
        long step were taken.

        In such a case `hmax`[0,j-1] should be set to about half the
        distance over which the feature should be observed.

        Too small a value will increase the computing time for the
        function.

        See [Further Comments] for further suggestions.

        The rest of the array is used as workspace.

        On exit: `hmax`[0,m-2] and `hmax`[0,m-1] contain the sensitivity
        coefficients sigma_l,sigma_r, described in [Further Comments].
        Other entries contain diagnostic output in the case of an error
        exit (see [Errors]).

    maxit : int
        A bound on n_r, the number of root-finding iterations allowed,
        that is the number of trial values of lambda that are used. If
        `maxit` <= 0, no such bound is assumed. (See also `maxfun`.)

    maxfun : int
        A bound on n_f, the number of calls to `coeffn` made in any one
        root-finding iteration. If `maxfun` <= 0, no such bound is
        assumed.

    monit : None or callable monit(nit, iflag, elam, finfo, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monit` is called by ``sl2_breaks_vals`` at the end of each
        root-finding iteration and allows you to monitor the course of
        the computation by printing out the arguments (see [Example] for
        an example).

        Parameters
        ~~~~~~~~~~
        nit : int
            The current value of the argument `maxit` of
            ``sl2_breaks_vals``, this is decreased by one at each
            iteration.

        iflag : int
            Describes what phase the computation is in.

            `iflag` < 0

              An error occurred in the computation at this iteration; an
              error exit from ``sl2_breaks_vals`` with errno = -`iflag`
              will follow.

            `iflag` = 1

              The function is trying to bracket the eigenvalue lambda~.

            `iflag` = 2

              The function is converging to the eigenvalue lambda~
              (having already bracketed it).

        elam : float
            The current trial value of lambda.

        finfo : float, ndarray, shape (15)
            Information about the behaviour of the shooting method, and
            diagnostic information in the case of errors. It should not
            normally be printed in full if no error has occurred (that
            is, if `iflag` > 0), though the first few components may be
            of interest to you. In case of an error (`iflag` < 0) all
            the components of `finfo` should be printed.

            The contents of `finfo` are as follows:

            `finfo`[0]

              The current value of the 'miss-distance' or 'residual'
              function f(lambda) on which the shooting method is based.
              (See [Further Comments] for further information.)
              `finfo`[0] is set to zero if `iflag` < 0.

            `finfo`[1]

              An estimate of the quantity d lambda defined as follows.
              Consider the perturbation in the miss-distance f(lambda)
              that would result if the local error in the solution of
              the differential equation were always positive and equal
              to its maximum permitted value. Then d lambda is the
              perturbation in lambda that would have the same effect on
              f(lambda). Thus, at the zero of f(lambda),|d lambda| is an
              approximate bound on the perturbation of the zero (that is
              the eigenvalue) caused by errors in numerical solution. If
              d lambda is very large then it is possible that there has
              been a programming error in `coeffn` such that q is
              independent of lambda. If this is the case, an error exit
              with `errno` = 5 should follow. `finfo`[1] is set to zero
              if `iflag` < 0.

            `finfo`[2]

              The number of internal iterations, using the same value of
              lambda and tighter accuracy tolerances, needed to bring
              the accuracy (that is, the value of d lambda) to an
              acceptable value. Its value should normally be 1.0, and
              should almost never exceed 2.0.

            `finfo`[3]

              The number of calls to `coeffn` at this iteration.

            `finfo`[4]

              The number of successful steps taken by the internal
              differential equation solver at this iteration. A step is
              successful if it is used to advance the integration.

            `finfo`[5]

              The number of unsuccessful steps used by the internal
              integrator at this iteration.

            `finfo`[6]

              The number of successful steps at the maximum step size
              taken by the internal integrator at this iteration.

            `finfo`[7]

              Not used.

            `finfo`[8] to `finfo`[14]

              Set to zero, unless `iflag` < 0 in which case they hold
              the following values describing the point of failure:

              `finfo`[8]

                The index of the sub-interval where failure occurred, in
                the range 1 to m-3. In case of an error in `bdyval`, it
                is set to 0 or m-2 depending on whether the left or
                right boundary condition caused the error.

              `finfo`[9]

                The value of the independent variable, x, the point at
                which the error occurred. In case of an error in
                `bdyval`, it is set to the value of `xl` or `xr` as
                appropriate (see the specification of `bdyval`).

              `finfo`[10], `finfo`[11], `finfo`[12]

                The current values of the Pruefer dependent variables
                beta, phi and rho respectively. These are set to zero in
                case of an error in `bdyval`. (See ``sl2_breaks_funs``
                for a description of these variables.)

              `finfo`[13]

                The local-error tolerance being used by the internal
                integrator at the point of failure. This is set to zero
                in the case of an error in `bdyval`.

              `finfo`[14]

                The last integration mesh point. This is set to zero in
                the case of an error in `bdyval`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    elam : float
        The final computed estimate, whether or not an error occurred.

    delam : float
        If no exception or warning is raised, `delam` holds an estimate
        of the absolute error in the computed eigenvalue, that is
        |lambda~-`elam`| ~= `delam`. (In [Further Comments] we discuss
        the assumptions under which this is true.) The true error is
        rarely more than twice, or less than a tenth, of the estimated
        error.

        If an exception is raised, `delam` may hold an estimate of the
        error, or its initial value, depending on the value of errno.

        See [Errors] for further details.

    maxit : int
        Will have been decreased by the number of iterations actually
        performed, whether or not it was positive on entry.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, the sequence of boundary and break points is not
            monotonic.

            For i = *<value>*, point i-1 = *<value>* and point i =
            *<value>*.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 4.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            The constraint on the specification of left- or right-hand
            boundary condition has been violated.

        (`errno` 3)
            The function p(x) became zero or changed sign in the
            interval [a,b].

        (`errno` 6)
            The last iteration was terminated because more than
            *<value>* calls to evaluate p, q and its derivative were
            performed.

        (`errno` 7)
            Too small a step size was required at the start of a
            sub-interval to obtain the desired accuracy.

        (`errno` 8)
            Too small a step size was required during solution in a
            sub-interval to obtain the desired accuracy.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            After *<value>* iterations the eigenvalue had not been found
            to the required accuracy.

        (`errno` 5)
            The bracketing phase failed to bracket the eigenvalue within
            ten iterations. This may be due to an error in formulating
            the problem (for example, q is independent of lambda), or by
            very poor initial estimates for the eigenvalue and search
            step.

        (`errno` 9)
            The tolerance set is too small for the problem being solved
            and the machine precision being used. The local eigenvalue
            estimate returned is likely to be a very good approximation.

        (`errno` 10)
            An internal error has occurred corresponding to a pole of
            the matching function. Try solving the problem again with a
            smaller tolerance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.sl2_breaks_vals'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_coeffn_name = fun_name + ':coeffn'

    coeffnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, q.
        utils._BYREF_FLOAT64, # Output, dqdl.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._BYREF_INT, # Input, jint.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_coeffnh(_coeffn, en_data, p, q, dqdl, x, elam, jint, _iuser, _ruser):
        """Helper for coeffn."""
        try:
            _x_coeffnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _elam_coeffnh = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _jint_coeffnh = utils._EngineIntScalarType.to_py(jint.contents)
            if data is not None:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                    data,
                )
            else:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                )
            utils._EngineFloat64ScalarType(
                _p_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'p'},
            ).refresh_ctypes_buffer(p)
            utils._EngineFloat64ScalarType(
                _q_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'q'},
            ).refresh_ctypes_buffer(q)
            utils._EngineFloat64ScalarType(
                _dqdl_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'dqdl'},
            ).refresh_ctypes_buffer(dqdl)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bdyval_name = fun_name + ':bdyval'

    bdyvalh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xl.
        utils._BYREF_FLOAT64, # Input, xr.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._EngineFloat64ArrayType.c_type, # Output, yl[3].
        utils._EngineFloat64ArrayType.c_type, # Output, yr[3].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bdyvalh(_bdyval, en_data, xl, xr, elam, yl, yr, work, _iuser, _ruser):
        """Helper for bdyval."""
        try:
            _xl_bdyvalh = utils._EngineFloat64ScalarType.to_py(xl.contents)
            _xr_bdyvalh = utils._EngineFloat64ScalarType.to_py(xr.contents)
            _elam_bdyvalh = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _yl_bdyvalh = utils._EngineFloat64ArrayType.from_pointer(
                yl,
                (3,),
                can_reference_contents=False,
            )
            _yr_bdyvalh = utils._EngineFloat64ArrayType.from_pointer(
                yr,
                (3,),
                can_reference_contents=False,
            )
            if data is not None:
                bdyval(
                    _xl_bdyvalh,
                    _xr_bdyvalh,
                    _elam_bdyvalh,
                    _yl_bdyvalh,
                    _yr_bdyvalh,
                    data,
                )
            else:
                bdyval(
                    _xl_bdyvalh,
                    _xr_bdyvalh,
                    _elam_bdyvalh,
                    _yl_bdyvalh,
                    _yr_bdyvalh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yl_bdyvalh, {'fun_name': cb_bdyval_name, 'returned': 'yl'}, yl,
            )
            utils._unlink_pointer(yl)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yr_bdyvalh, {'fun_name': cb_bdyval_name, 'returned': 'yr'}, yr,
            )
            utils._unlink_pointer(yr)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nit.
        utils._BYREF_INT, # Input, iflag.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._EngineFloat64ArrayType.c_type, # Input, finfo[15].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, nit, iflag, elam, finfo, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _nit_monith = utils._EngineIntScalarType.to_py(nit.contents)
            _iflag_monith = utils._EngineIntScalarType.to_py(iflag.contents)
            _elam_monith = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _finfo_monith = utils._EngineFloat64ArrayType.from_pointer(
                finfo,
                (15,),
            )
            if data is not None:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                    data,
                )
            else:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                )
            utils._unlink_pointer(finfo)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02kdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, xpoint[m].
        utils._BYREF_INT, # Input, m.
        utils._UNUSED_FUNCTYPE,
        coeffnh,
        utils._UNUSED_FUNCTYPE,
        bdyvalh,
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Input/Output, elam.
        utils._BYREF_FLOAT64, # Input/Output, delam.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, hmax[2, m].
        utils._BYREF_INT, # Input/Output, maxit.
        utils._BYREF_INT, # Input, maxfun.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((hmax, 'hmax'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _xpoint = utils._EngineFloat64ArrayType(
        xpoint, {'fun_name': fun_name, 'entity': 'xpoint'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    utils._check_callable(coeffn, {'fun_name': fun_name, 'entity': 'coeffn'})
    _coeffn = utils._UNUSED_CALLBACK
    _coeffnh = utils._cast_callback(
        coeffnh, cb_coeffnh,
        {'fun_name': fun_name, 'entity': 'coeffn'},
    )
    utils._check_callable(bdyval, {'fun_name': fun_name, 'entity': 'bdyval'})
    _bdyval = utils._UNUSED_CALLBACK
    _bdyvalh = utils._cast_callback(
        bdyvalh, cb_bdyvalh,
        {'fun_name': fun_name, 'entity': 'bdyval'},
    )
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _elam = utils._EngineFloat64ScalarType(
        elam, {'fun_name': fun_name, 'entity': 'elam'},
    ).to_ctypes()
    _delam = utils._EngineFloat64ScalarType(
        delam, {'fun_name': fun_name, 'entity': 'delam'},
    ).to_ctypes()
    _hmax_cls = utils._EngineFloat64ArrayType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
        exp_shape=(2, _m_py),
        mutual_sorder=_m_sorder,
    )
    _hmax = _hmax_cls.to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _maxfun = utils._EngineIntScalarType(
        maxfun, {'fun_name': fun_name, 'entity': 'maxfun'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    _work = utils._EngineFloat64ArrayType.empty_ctypes(1)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xpoint,
        _m,
        _coeffn,
        _coeffnh,
        _bdyval,
        _bdyvalh,
        _k,
        _tol,
        _elam,
        _delam,
        _hmax,
        _maxit,
        _maxfun,
        _monit,
        _monith,
        _work,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the sequence of boundary and break points is not monotonic.',
                    'For i = {:s}, point i-1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'point i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The constraint on the specification of left- or right-hand boundary',
                    'condition has been violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function p(x) became zero or changed sign in the interval [a,b].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'After {:s} iterations the eigenvalue had not been found to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the required accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The bracketing phase failed to bracket the eigenvalue within ten',
                    'iterations. This may be due to an error in formulating the problem',
                    '(for example, q is independent of lambda), or by very poor initial',
                    'estimates for the eigenvalue and search step.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The last iteration was terminated because more than {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'calls to evaluate p, q and its derivative were performed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required at the start of a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required during solution in a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The tolerance set is too small for the problem being solved and the',
                    'machine precision being used. The local eigenvalue estimate returned is',
                    'likely to be a very good approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred corresponding to a pole of the matching',
                    'function. Try solving the problem again with a smaller tolerance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _elam_py = utils._EngineFloat64ScalarType.to_py(_elam)
        _delam_py = utils._EngineFloat64ScalarType.to_py(_delam)
        _hmax_cls.refresh_py(_hmax)
        _maxit_py = utils._EngineIntScalarType.to_py(_maxit)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['elam', 'delam', 'maxit'],
        )(
            _elam_py,
            _delam_py,
            _maxit_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sl2_breaks_funs(xpoint, m, match, coeffn, bdyval, k, tol, elam, delam, hmax, maxit, maxfun, monit, report, data=None):
    u"""
    Second-order Sturm--Liouville problem, regular/singular system,
    finite/infinite range, eigenvalue and eigenfunction, user-specified
    break-points.

    For full information please refer to the NAG Library document for
    d02ke

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02kef.html

    Parameters
    ----------
    xpoint : float, shape (`m`)
        The points where the boundary conditions computed by `bdyval`
        are to be imposed, and also any break-points, i.e., `xpoint`[0]
        to `xpoint`[m-1] must contain values x_1,...,x_m such that

            x_1 <= x_2 < x_3 < ... < x_{m-1} <= x_m

        with the following meanings:

        - x_1 and x_m are the left- and right-hand end points, a and b,
          of the domain of definition of the Sturm--Liouville system if
          these are finite. If either a or b is infinite, the
          corresponding value x_1 or x_m may be a more-or-less
          arbitrarily 'large' number of appropriate sign.

        - x_2 and x_{m-1} are the Boundary Matching Points (BMPs), that
          is the points at which the left and right boundary conditions
          computed in `bdyval` are imposed.

          If the left-hand end point is a regular point then you should
          set x_2 = x_1 (= a), while if it is a singular point you must
          set x_2 > x_1.

          Similarly x_{m-1} = x_m (= b) if the right-hand end point is
          regular, and x_{m-1} < x_m if it is singular.

        - The remaining m-4 points x_3,...,x_{m-2}, if any, define
          'break-points' which divide the interval [x_2,x_{m-1}] into
          m-3 sub-intervals

              i_1 = [x_2,x_3],...,i_{m-3} = [x_{m-2},x_{m-1}].

          Numerical integration of the differential equation is stopped
          and restarted at each break-point. In simple cases no
          break-points are needed. However, if p(x) or q(x;lambda) are
          given by different formulae in different parts of the
          interval, integration is more efficient if the range is broken
          up by break-points in the appropriate way. Similarly points
          where any jumps occur in p(x) or q(x;lambda), or in their
          derivatives up to the fifth-order, should appear as
          break-points.

          Examples are given in [Further Comments]. `xpoint` determines
          the position of the Shooting Matching Point (SMP), as
          explained in [Further Comments].

    m : int
        The number of points in the array `xpoint`.

    match : int
        Must be set to the index of the 'break-point' to be used as the
        matching point (see [Further Comments]). If `match` is set to a
        value outside the range [2,m-1] then a default value is taken,
        corresponding to the break-point nearest the centre of the
        interval [`xpoint`[1],`xpoint`[m-1]].

    coeffn : callable (p, q, dqdl) = coeffn(x, elam, jint, data=None)
        `coeffn` must compute the values of the coefficient functions
        p(x) and q(x;lambda) for given values of x and lambda.
        [Description] states the conditions which p and q must satisfy.

        See [Further Comments] for examples.

        Parameters
        ~~~~~~~~~~
        x : float
            The current value of x.

        elam : float
            The current trial value of the eigenvalue argument lambda.

        jint : int
            The index j of the sub-interval i_j (see specification of
            `xpoint`) in which x lies.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        p : float
            The value of p(x) for the current value of x.

        q : float
            The value of q(x;lambda) for the current value of x and the
            current trial value of lambda.

        dqdl : float
            The value of dq/d lambda(x;lambda) for the current value of
            x and the current trial value of lambda. However `dqdl` is
            only used in error estimation and, in the rare cases where
            it may be difficult to evaluate, an approximation (say to
            within 20%) will suffice.

    bdyval : callable bdyval(xl, xr, elam, yl, yr, data=None)
        `bdyval` must define the boundary conditions.

        For each end point, `bdyval` must return (in `yl` or `yr`)
        values of y(x) and p(x)y'(x) which are consistent with the
        boundary conditions at the end points; only the ratio of the
        values matters.

        Here x is a given point (`xl` or `xr`) equal to, or close to,
        the end point.

        Parameters
        ~~~~~~~~~~
        xl : float
            If a is a regular end point of the system (so that a = x_1 =
            x_2), `xl` contains a. If a is a singular point (so that a
            <= x_1 < x_2), `xl` contains a point x such that x_1 < x <=
            x_2.

        xr : float
            If b is a regular end point of the system (so that x_{m-1} =
            x_m = b), `xr` contains b. If b is a singular point (so that
            x_{m-1} < x_m <= b), `xr` contains a point x such that
            x_{m-1} <= x < x_m.

        elam : float
            The current trial value of lambda.

        yl : float, ndarray, shape (3), to be modified in place
            On exit: `yl`[0] and `yl`[1] should contain values of y(x)
            and p(x)y'(x) respectively (not both zero) which are
            consistent with the boundary condition at the left-hand end
            point, given by x = `xl`. `yl`[2] should not be set.

        yr : float, ndarray, shape (3), to be modified in place
            On exit: `yr`[0] and `yr`[1] should contain values of y(x)
            and p(x)y'(x) respectively (not both zero) which are
            consistent with the boundary condition at the right-hand end
            point, given by x = `xr`. `yr`[2] should not be set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    k : int
        k, the index of the required eigenvalue when the eigenvalues are
        ordered

            lambda_0 < lambda_1 < lambda_2 < ... < lambda_k < ....

    tol : float
        The tolerance argument which determines the accuracy of the
        computed eigenvalue. The error estimate held in `delam` on exit
        satisfies the mixed absolute/relative error test

            `delam` <= `tol`*max(1.0, |`elam`|),

        where `elam` is the final estimate of the eigenvalue. `delam` is
        usually somewhat smaller than the right-hand side of (1) but not
        several orders of magnitude smaller.

    elam : float
        An initial estimate of the eigenvalue lambda~.

    delam : float
        An indication of the scale of the problem in the
        lambda-direction. `delam` holds the initial 'search step'
        (positive or negative). Its value is not critical, but the first
        two trial evaluations are made at `elam` and `elam`+`delam`, so
        the function will work most efficiently if the eigenvalue lies
        between these values. A reasonable choice (if a closer bound is
        not known) is half the distance between adjacent eigenvalues in
        the neighbourhood of the one sought. In practice, there will
        often be a problem, similar to the one in hand but with known
        eigenvalues, which will help one to choose initial values for
        `elam` and `delam`.

        If `delam` = 0.0 on entry, it is given the default value of
        0.25*max(1.0, |`elam`|).

    hmax : float, shape (2, `m`), modified in place
        On entry: `hmax`[0,j-1] should contain a maximum step size to be
        used by the differential equation code in the jth sub-interval
        i_j (as described in the specification of argument `xpoint`),
        for j = 1,2,...,m-3. If it is zero the function generates a
        maximum step size internally.

        It is recommended that `hmax`[0,j-1] be set to zero unless the
        coefficient functions p and q have features (such as a narrow
        peak) within the jth sub-interval that could be 'missed' if a
        long step were taken.

        In such a case `hmax`[0,j-1] should be set to about half the
        distance over which the feature should be observed.

        Too small a value will increase the computing time for the
        function.

        See [Further Comments] for further suggestions.

        The rest of the array is used as workspace.

        On exit: `hmax`[0,m-2] and `hmax`[0,m-1] contain the sensitivity
        coefficients sigma_l,sigma_r, described in [Further Comments].
        Other entries contain diagnostic output in the case of an error
        exit (see [Errors]).

    maxit : int
        A bound on n_r, the number of root-finding iterations allowed,
        that is the number of trial values of lambda that are used. If
        `maxit` <= 0, no such bound is assumed. (See also `maxfun`.)

    maxfun : int
        A bound on n_f, the number of calls to `coeffn` made in any one
        root-finding iteration. If `maxfun` <= 0, no such bound is
        assumed.

    monit : None or callable monit(nit, iflag, elam, finfo, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monit` is called by ``sl2_breaks_funs`` at the end of each
        root-finding iteration and allows you to monitor the course of
        the computation by printing out the arguments (see [Example] for
        an example).

        Parameters
        ~~~~~~~~~~
        nit : int
            The current value of the argument `maxit` of
            ``sl2_breaks_funs``, this is decreased by one at each
            iteration.

        iflag : int
            Describes what phase the computation is in.

            `iflag` < 0

              An error occurred in the computation at this iteration; an
              error exit from ``sl2_breaks_funs`` with errno = -`iflag`
              will follow.

            `iflag` = 1

              The function is trying to bracket the eigenvalue lambda~.

            `iflag` = 2

              The function is converging to the eigenvalue lambda~
              (having already bracketed it).

        elam : float
            The current trial value of lambda.

        finfo : float, ndarray, shape (15)
            Information about the behaviour of the shooting method, and
            diagnostic information in the case of errors. It should not
            normally be printed in full if no error has occurred (that
            is, if `iflag` > 0), though the first few components may be
            of interest to you. In case of an error (`iflag` < 0) all
            the components of `finfo` should be printed.

            The contents of `finfo` are as follows:

            `finfo`[0]

              The current value of the 'miss-distance' or 'residual'
              function f(lambda) on which the shooting method is based.
              (See [Further Comments] for further information.)
              `finfo`[0] is set to zero if `iflag` < 0.

            `finfo`[1]

              An estimate of the quantity d lambda defined as follows.
              Consider the perturbation in the miss-distance f(lambda)
              that would result if the local error in the solution of
              the differential equation were always positive and equal
              to its maximum permitted value. Then d lambda is the
              perturbation in lambda that would have the same effect on
              f(lambda). Thus, at the zero of f(lambda),|d lambda| is an
              approximate bound on the perturbation of the zero (that is
              the eigenvalue) caused by errors in numerical solution. If
              d lambda is very large then it is possible that there has
              been a programming error in `coeffn` such that q is
              independent of lambda. If this is the case, an error exit
              with `errno` = 5 should follow. `finfo`[1] is set to zero
              if `iflag` < 0.

            `finfo`[2]

              The number of internal iterations, using the same value of
              lambda and tighter accuracy tolerances, needed to bring
              the accuracy (that is, the value of d lambda) to an
              acceptable value. Its value should normally be 1.0, and
              should almost never exceed 2.0.

            `finfo`[3]

              The number of calls to `coeffn` at this iteration.

            `finfo`[4]

              The number of successful steps taken by the internal
              differential equation solver at this iteration. A step is
              successful if it is used to advance the integration.

            `finfo`[5]

              The number of unsuccessful steps used by the internal
              integrator at this iteration.

            `finfo`[6]

              The number of successful steps at the maximum step size
              taken by the internal integrator at this iteration.

            `finfo`[7]

              Not used.

            `finfo`[8] to `finfo`[14]

              Set to zero, unless `iflag` < 0 in which case they hold
              the following values describing the point of failure:

              `finfo`[8]

                The index of the sub-interval where failure occurred, in
                the range 1 to m-3. In case of an error in `bdyval`, it
                is set to 0 or m-2 depending on whether the left or
                right boundary condition caused the error.

              `finfo`[9]

                The value of the independent variable, x, the point at
                which the error occurred. In case of an error in
                `bdyval`, it is set to the value of `xl` or `xr` as
                appropriate (see the specification of `bdyval`).

              `finfo`[10], `finfo`[11], `finfo`[12]

                The current values of the Pruefer dependent variables
                beta, phi and rho respectively. These are set to zero in
                case of an error in `bdyval`.

              `finfo`[13]

                The local-error tolerance being used by the internal
                integrator at the point of failure. This is set to zero
                in the case of an error in `bdyval`.

              `finfo`[14]

                The last integration mesh point. This is set to zero in
                the case of an error in `bdyval`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    report : callable report(x, v, jint, data=None)
        `report` provides the means by which you may compute the
        eigenfunction y(x) and its derivative at each integration mesh
        point x. (See [Further Comments] for an example.)

        Parameters
        ~~~~~~~~~~
        x : float
            The current value of the independent variable x. See
            [Further Comments] for the order in which values of x are
            supplied.

        v : float, ndarray, shape (3)
            `v`[0], `v`[1], `v`[2] hold the current values of the
            Pruefer variables beta, phi, rho respectively.

        jint : int
            Indicates the sub-interval between break-points in which `x`
            lies exactly as for `coeffn`, except that at the extreme
            left-hand end point (when x = `xpoint`[1]) `jint` is set to
            0 and at the extreme right-hand end point (when x = x_r =
            `xpoint`[m-1]) `jint` is set to m-2.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    match : int
        The index of the break-point actually used as the matching
        point.

    elam : float
        The final computed estimate, whether or not an error occurred.

    delam : float
        If no exception or warning is raised, `delam` holds an estimate
        of the absolute error in the computed eigenvalue, that is
        |lambda~-`elam`| ~= `delam`. (In [Further Comments] we discuss
        the assumptions under which this is true.) The true error is
        rarely more than twice, or less than a tenth, of the estimated
        error.

        If an exception is raised, `delam` may hold an estimate of the
        error, or its initial value, depending on the value of errno.

        See [Errors] for further details.

    maxit : int
        Will have been decreased by the number of iterations actually
        performed, whether or not it was positive on entry.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 4.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            On entry, the sequence of boundary and break points is not
            monotonic.

            For i = *<value>*, point i-1 = *<value>* and point i =
            *<value>*.

        (`errno` 2)
            The constraint on the specification of left- or right-hand
            boundary condition has been violated.

        (`errno` 3)
            The function p(x) became zero or changed sign in the
            interval [a,b].

        (`errno` 4)
            After *<value>* iterations the eigenvalue had not been found
            to the required accuracy.

        (`errno` 5)
            The bracketing phase failed to bracket the eigenvalue within
            ten iterations. This may be due to an error in formulating
            the problem (for example, q is independent of lambda), or by
            very poor initial estimates for the eigenvalue and search
            step.

        (`errno` 6)
            The last iteration was terminated because more than
            *<value>* calls to evaluate p, q and its derivative were
            performed.

        (`errno` 7)
            Too small a step size was required at the start of a
            sub-interval to obtain the desired accuracy.

        (`errno` 8)
            Too small a step size was required during solution in a
            sub-interval to obtain the desired accuracy.

        (`errno` 10)
            An internal error has occurred corresponding to a pole of
            the matching function. Try solving the problem again with a
            smaller tolerance.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 9)
            The tolerance set is too small for the problem being solved
            and the machine precision being used. The local eigenvalue
            estimate returned is likely to be a very good approximation.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.sl2_breaks_funs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_coeffn_name = fun_name + ':coeffn'

    coeffnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, q.
        utils._BYREF_FLOAT64, # Output, dqdl.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._BYREF_INT, # Input, jint.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_coeffnh(_coeffn, en_data, p, q, dqdl, x, elam, jint, _iuser, _ruser):
        """Helper for coeffn."""
        try:
            _x_coeffnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _elam_coeffnh = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _jint_coeffnh = utils._EngineIntScalarType.to_py(jint.contents)
            if data is not None:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                    data,
                )
            else:
                _p_coeffnh, _q_coeffnh, _dqdl_coeffnh = coeffn(
                    _x_coeffnh,
                    _elam_coeffnh,
                    _jint_coeffnh,
                )
            utils._EngineFloat64ScalarType(
                _p_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'p'},
            ).refresh_ctypes_buffer(p)
            utils._EngineFloat64ScalarType(
                _q_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'q'},
            ).refresh_ctypes_buffer(q)
            utils._EngineFloat64ScalarType(
                _dqdl_coeffnh, {'fun_name': cb_coeffn_name, 'returned': 'dqdl'},
            ).refresh_ctypes_buffer(dqdl)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bdyval_name = fun_name + ':bdyval'

    bdyvalh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xl.
        utils._BYREF_FLOAT64, # Input, xr.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._EngineFloat64ArrayType.c_type, # Output, yl[3].
        utils._EngineFloat64ArrayType.c_type, # Output, yr[3].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bdyvalh(_bdyval, en_data, xl, xr, elam, yl, yr, work, _iuser, _ruser):
        """Helper for bdyval."""
        try:
            _xl_bdyvalh = utils._EngineFloat64ScalarType.to_py(xl.contents)
            _xr_bdyvalh = utils._EngineFloat64ScalarType.to_py(xr.contents)
            _elam_bdyvalh = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _yl_bdyvalh = utils._EngineFloat64ArrayType.from_pointer(
                yl,
                (3,),
                can_reference_contents=False,
            )
            _yr_bdyvalh = utils._EngineFloat64ArrayType.from_pointer(
                yr,
                (3,),
                can_reference_contents=False,
            )
            if data is not None:
                bdyval(
                    _xl_bdyvalh,
                    _xr_bdyvalh,
                    _elam_bdyvalh,
                    _yl_bdyvalh,
                    _yr_bdyvalh,
                    data,
                )
            else:
                bdyval(
                    _xl_bdyvalh,
                    _xr_bdyvalh,
                    _elam_bdyvalh,
                    _yl_bdyvalh,
                    _yr_bdyvalh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yl_bdyvalh, {'fun_name': cb_bdyval_name, 'returned': 'yl'}, yl,
            )
            utils._unlink_pointer(yl)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yr_bdyvalh, {'fun_name': cb_bdyval_name, 'returned': 'yr'}, yr,
            )
            utils._unlink_pointer(yr)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nit.
        utils._BYREF_INT, # Input, iflag.
        utils._BYREF_FLOAT64, # Input, elam.
        utils._EngineFloat64ArrayType.c_type, # Input, finfo[15].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, nit, iflag, elam, finfo, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _nit_monith = utils._EngineIntScalarType.to_py(nit.contents)
            _iflag_monith = utils._EngineIntScalarType.to_py(iflag.contents)
            _elam_monith = utils._EngineFloat64ScalarType.to_py(elam.contents)
            _finfo_monith = utils._EngineFloat64ArrayType.from_pointer(
                finfo,
                (15,),
            )
            if data is not None:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                    data,
                )
            else:
                monit(
                    _nit_monith,
                    _iflag_monith,
                    _elam_monith,
                    _finfo_monith,
                )
            utils._unlink_pointer(finfo)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    reporth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, v[3].
        utils._BYREF_INT, # Input, jint.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_reporth(_report, en_data, x, v, jint, _iuser, _ruser):
        """Helper for report."""
        try:
            _x_reporth = utils._EngineFloat64ScalarType.to_py(x.contents)
            _v_reporth = utils._EngineFloat64ArrayType.from_pointer(
                v,
                (3,),
            )
            _jint_reporth = utils._EngineIntScalarType.to_py(jint.contents)
            if data is not None:
                report(
                    _x_reporth,
                    _v_reporth,
                    _jint_reporth,
                    data,
                )
            else:
                report(
                    _x_reporth,
                    _v_reporth,
                    _jint_reporth,
                )
            utils._unlink_pointer(v)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02keft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, xpoint[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input/Output, match.
        utils._UNUSED_FUNCTYPE,
        coeffnh,
        utils._UNUSED_FUNCTYPE,
        bdyvalh,
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Input/Output, elam.
        utils._BYREF_FLOAT64, # Input/Output, delam.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, hmax[2, m].
        utils._BYREF_INT, # Input/Output, maxit.
        utils._BYREF_INT, # Input, maxfun.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._UNUSED_FUNCTYPE,
        reporth,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, work[1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((hmax, 'hmax'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _xpoint = utils._EngineFloat64ArrayType(
        xpoint, {'fun_name': fun_name, 'entity': 'xpoint'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _match = utils._EngineIntScalarType(
        match, {'fun_name': fun_name, 'entity': 'match'},
    ).to_ctypes()
    utils._check_callable(coeffn, {'fun_name': fun_name, 'entity': 'coeffn'})
    _coeffn = utils._UNUSED_CALLBACK
    _coeffnh = utils._cast_callback(
        coeffnh, cb_coeffnh,
        {'fun_name': fun_name, 'entity': 'coeffn'},
    )
    utils._check_callable(bdyval, {'fun_name': fun_name, 'entity': 'bdyval'})
    _bdyval = utils._UNUSED_CALLBACK
    _bdyvalh = utils._cast_callback(
        bdyvalh, cb_bdyvalh,
        {'fun_name': fun_name, 'entity': 'bdyval'},
    )
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _elam = utils._EngineFloat64ScalarType(
        elam, {'fun_name': fun_name, 'entity': 'elam'},
    ).to_ctypes()
    _delam = utils._EngineFloat64ScalarType(
        delam, {'fun_name': fun_name, 'entity': 'delam'},
    ).to_ctypes()
    _hmax_cls = utils._EngineFloat64ArrayType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
        exp_shape=(2, _m_py),
        mutual_sorder=_m_sorder,
    )
    _hmax = _hmax_cls.to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _maxfun = utils._EngineIntScalarType(
        maxfun, {'fun_name': fun_name, 'entity': 'maxfun'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    utils._check_callable(report, {'fun_name': fun_name, 'entity': 'report'})
    _report = utils._UNUSED_CALLBACK
    _reporth = utils._cast_callback(
        reporth, cb_reporth,
        {'fun_name': fun_name, 'entity': 'report'},
    )
    _work = utils._EngineFloat64ArrayType.empty_ctypes(1)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xpoint,
        _m,
        _match,
        _coeffn,
        _coeffnh,
        _bdyval,
        _bdyvalh,
        _k,
        _tol,
        _elam,
        _delam,
        _hmax,
        _maxit,
        _maxfun,
        _monit,
        _monith,
        _report,
        _reporth,
        _work,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the sequence of boundary and break points is not monotonic.',
                    'For i = {:s}, point i-1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'point i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The constraint on the specification of left- or right-hand boundary',
                    'condition has been violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function p(x) became zero or changed sign in the interval [a,b].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'After {:s} iterations the eigenvalue had not been found to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the required accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The bracketing phase failed to bracket the eigenvalue within ten',
                    'iterations. This may be due to an error in formulating the problem',
                    '(for example, q is independent of lambda), or by very poor initial',
                    'estimates for the eigenvalue and search step.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The last iteration was terminated because more than {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'calls to evaluate p, q and its derivative were performed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required at the start of a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too small a step size was required during solution in a sub-interval to',
                    'obtain the desired accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The tolerance set is too small for the problem being solved and the',
                    'machine precision being used. The local eigenvalue estimate returned is',
                    'likely to be a very good approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred corresponding to a pole of the matching',
                    'function. Try solving the problem again with a smaller tolerance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _match_py = utils._EngineIntScalarType.to_py(_match)
        _elam_py = utils._EngineFloat64ScalarType.to_py(_elam)
        _delam_py = utils._EngineFloat64ScalarType.to_py(_delam)
        _hmax_cls.refresh_py(_hmax)
        _maxit_py = utils._EngineIntScalarType.to_py(_maxit)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['match', 'elam', 'delam', 'maxit'],
        )(
            _match_py,
            _elam_py,
            _delam_py,
            _maxit_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_2nd_rkn(fcn, neq, t, tend, y, yp, ydp, comm, data=None):
    u"""
    Second-order ordinary differential equations, initial value problem,
    Runge--Kutta--Nystrom method.

    For full information please refer to the NAG Library document for
    d02la

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02laf.html

    Parameters
    ----------
    fcn : callable fcn(neq, t, y, f, data=None)
        `fcn` must evaluate the functions f_i (that is the second
        derivatives y_i^'') for given values of its arguments x,
        y_1,y_2,...,y_`neq`.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations.

        t : float
            x, the value of the argument.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the value of the argument.

        f : float, ndarray, shape (`neq`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,`neq`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    neq : int
        The number of second-order ordinary differential equations to be
        solved by ``ivp_2nd_rkn``. It must contain the same value as the
        argument `neq` used in a prior call to ``ivp_2nd_rkn_setup``.

    t : float
        The initial value of the independent variable x.

    tend : float
        The end point of the range of integration. If `tend` < `t` on
        initial entry, integration will proceed in the negative
        direction. `tend` may be reset, in the direction of integration,
        before any continuation call.

    y : float, shape (`neq`), modified in place
        On entry: the initial values of the solution
        y_1,y_2,...,y_`neq`.

        On exit: the computed values of the solution at the exit value
        of `t`. If the integration is to be continued, possibly with a
        new value for `tend`, these values must not be changed.

    yp : float, shape (`neq`), modified in place
        On entry: the initial values of the derivatives
        y'_1,y'_2,...,y'_`neq`.

        On exit: the computed values of the derivatives at the exit
        value of `t`. If the integration is to be continued, possibly
        with a new value for `tend`, these values must not be changed.

    ydp : float, shape (`neq`), modified in place
        On entry: must be unchanged from a previous call to
        ``ivp_2nd_rkn``.

        On exit: the computed values of the second derivative at the
        exit value of `t`, unless illegal input is detected, in which
        case the elements of `ydp` may not have been initialized. If the
        integration is to be continued, possibly with a new value for
        `tend`, these values must not be changed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_2nd_rkn_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    t : float
        The value of the independent variable, which is usually `tend`,
        unless an error has occurred or the code is operating in
        one-step mode. If the integration is to be continued, possibly
        with a new value for `tend`, `t` must not be changed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*, but in the previous call to the
            setup function neq = *<value>*.

        (`errno` 1)
            The previous call to the setup function resulted in the
            error exit *<value>*.

        (`errno` 1)
            The setup function ``ivp_2nd_rkn_setup`` has not been
            called.

        (`errno` 1)
            `tend` has been reset such that the direction of integration
            is reversed.

        (`errno` 3)
            To satisfy the accuracy requirements the step size,
            *<value>*, at `t` = *<value>*, is too small for the machine
            precision.

        (`errno` 4)
            Two successive errors detected at the current value of `t` =
            *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `t` = *<value>* and `tend` = *<value>*.

            Constraint: `t` != `tend`.

        (`errno` 2)
            The maximum number of steps, *<value>*, has been attempted.

        (`errno` 5)
            Inefficiency detected in integrating exactly to values of
            `tend`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_2nd_rkn'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, f[neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, neq, t, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _neq_fcnh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_fcnh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neq_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02laft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tend.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, yp[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydp[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[10].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[60].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[400].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tend = utils._EngineFloat64ScalarType(
        tend, {'fun_name': fun_name, 'entity': 'tend'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _yp_cls = utils._EngineFloat64ArrayType(
        yp, {'fun_name': fun_name, 'entity': 'yp'},
        exp_shape=(_neq_py,),
    )
    _yp = _yp_cls.to_ctypes()
    _ydp_cls = utils._EngineFloat64ArrayType(
        ydp, {'fun_name': fun_name, 'entity': 'ydp'},
        exp_shape=(_neq_py,),
    )
    _ydp = _ydp_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwsav',),
        req_keys=('rwork', 'lwsav', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _iuser = None
    _ruser = None
    _lwsav = comm['lwsav']
    _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(400)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _neq,
        _t,
        _tend,
        _y,
        _yp,
        _ydp,
        _rwork,
        _lrwork,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _tend_py = utils._EngineFloat64ScalarType.to_py(_tend)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}, but in the previous call'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The previous call to the setup function resulted in the',
                    'error exit {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The setup function ivp_2nd_rkn_setup has not been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tend has been reset such that the direction of integration is reversed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s} and tend = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT.format(_tend_py),
                    ),
                    'Constraint: t != tend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of steps, {:s}, has been attempted.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'To satisfy the accuracy requirements the step size,',
                    '{:s}, at t = {:s}, is too small'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'for the machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Two successive errors detected at the current value of',
                    't = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inefficiency detected in integrating exactly to values of tend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _yp_cls.refresh_py(_yp)
        _ydp_cls.refresh_py(_ydp)
        return_data = _t_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['t'],
            )(
                _t_py,
            )
    _err_class.manage()
    return return_data

def ivp_2nd_rkn_setup(neq, h, tol, thres, thresp, maxstp, start, onestp, high, comm):
    u"""
    Second-order ordinary differential equations, initial value problem,
    setup for ``ivp_2nd_rkn``.

    For full information please refer to the NAG Library document for
    d02lx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02lxf.html

    Parameters
    ----------
    neq : int
        The number of second-order ordinary differential equations to be
        solved by ``ivp_2nd_rkn``.

    h : float
        If `start` = True, `h` may specify an initial step size to be
        attempted in ``ivp_2nd_rkn``.

        If `start` = False, `h` may specify a step size to override the
        choice of next step attempted made internally to
        ``ivp_2nd_rkn``.

        The sign of `h` is not important, as the absolute value of `h`
        is chosen and the appropriate sign is selected by
        ``ivp_2nd_rkn``.

        If this option is not required then you must set `h` = 0.0.

    tol : float
        Must be set to a relative tolerance for controlling the error in
        the integration by ``ivp_2nd_rkn``. ``ivp_2nd_rkn`` has been
        designed so that, for most problems, a reduction in `tol` leads
        to an approximately proportional reduction in the error in the
        solution. However the actual relation between `tol` and the
        accuracy of the solution cannot be guaranteed. You are strongly
        recommended to repeat the integration with a smaller value of
        `tol` and compare the results. See the description of `thres`
        and `thresp` for further details of how `tol` is used.

    thres : float, shape (`neq`)
        `thres` and `thresp` may be set to thresholds for use in the
        error control of ``ivp_2nd_rkn``. At each step in the numerical
        integration estimates of the local errors E1(i) and E2(i) in the
        solution, y_i, and its derivative, y'_i, respectively are
        computed, for i = 1,2,...,`neq`. For the step to be accepted
        conditions of the following type must be satisfied:

            max_{1 <= i <= `neq`} (E1(i)/max(`thres`[i-1], |y_i|)) <=
            `tol`,;;;max_{1 <= i <= `neq`} (E2(i)/max(`thresp`[i-1],
            |y'_i|)) <= `tol`.

        If one or both of these is not satisfied then the step size is
        reduced and the solution is recomputed.

        If `thres`[0] <= 0.0 on entry, a value of 50.0*epsilon is used
        for `thres`[i-1], for i = 1,2,...,`neq`, where epsilon is
        machine precision.

        Similarly for `thresp`.

    thresp : float, shape (`neq`)
        `thresp` and `thresp` may be set to thresholds for use in the
        error control of ``ivp_2nd_rkn``. At each step in the numerical
        integration estimates of the local errors E1(i) and E2(i) in the
        solution, y_i, and its derivative, y'_i, respectively are
        computed, for i = 1,2,...,`neq`. For the step to be accepted
        conditions of the following type must be satisfied:

            max_{1 <= i <= `neq`} (E1(i)/max(`thres`[i-1], |y_i|)) <=
            `tol`,;;;max_{1 <= i <= `neq`} (E2(i)/max(`thresp`[i-1],
            |y'_i|)) <= `tol`.

        If one or both of these is not satisfied then the step size is
        reduced and the solution is recomputed.

        If `thres`[0] <= 0.0 on entry, a value of 50.0*epsilon is used
        for `thres`[i-1], for i = 1,2,...,`neq`, where epsilon is
        machine precision.

        Similarly for `thresp`.

    maxstp : int
        A bound on the number of steps attempted in any one call of
        ``ivp_2nd_rkn``.

        If `maxstp` <= 0 on entry, a value of 1000 is used.

    start : bool
        Specifies whether or not the call of ``ivp_2nd_rkn`` is for a
        new problem. `start` = True indicates that a new problem is to
        be solved. `start` = False indicates the call of
        ``ivp_2nd_rkn_setup`` is prior to a continuation call of
        ``ivp_2nd_rkn``.

    onestp : bool
        The mode of operation for ``ivp_2nd_rkn``.

        `onestp` = True

          ``ivp_2nd_rkn`` will operate in one-step mode, that is it will
          return after each successful step.

        `onestp` = False

          ``ivp_2nd_rkn`` will operate in interval mode, that is it will
          return at the end of the integration interval.

    high : bool
        If `high` = True, a high-order method will be used, whereas if
        `high` = False, a low-order method will be used. (See the
        specification of ``ivp_2nd_rkn`` for further details.)

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    start : bool
        `start` = False.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `thresp`[*<value>*] = *<value>* is not positive.

        (`errno` 1)
            On entry, `thres`[*<value>*] = *<value>* is not positive.

        (`errno` 2)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 2)
            On entry, `high` = False and lrwork = *<value>*.

            Constraint: if `high` = False then lrwork >= *<value>*.

        (`errno` 2)
            On entry, `high` = True and lrwork = *<value>*.

            Constraint: if `high` = True then lrwork >= *<value>*.

        (`errno` 3)
            On entry, `tol` = *<value>*.

            Constraint: *<value>* < `tol` < *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_2nd_rkn_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02lxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Input, thres[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, thresp[neq].
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_LOGICAL, # Input/Output, start.
        utils._BYREF_LOGICAL, # Input, onestp.
        utils._BYREF_LOGICAL, # Input, high.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[10].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _thres = utils._EngineFloat64ArrayType(
        thres, {'fun_name': fun_name, 'entity': 'thres'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _thresp = utils._EngineFloat64ArrayType(
        thresp, {'fun_name': fun_name, 'entity': 'thresp'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _start = utils._EngineLogicalScalarType(
        start, {'fun_name': fun_name, 'entity': 'start'},
    ).to_ctypes()
    _onestp = utils._EngineLogicalScalarType(
        onestp, {'fun_name': fun_name, 'entity': 'onestp'},
    ).to_ctypes()
    _high = utils._EngineLogicalScalarType(
        high, {'fun_name': fun_name, 'entity': 'high'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwork', 'lwsav', 'iwsav'),
    )
    _high_py = utils._EngineLogicalScalarType.to_py(_high)
    if _comm_needs_init:
        if _high_py:
            _rwork_dim1 = 16+20*_neq_py
        elif not _high_py:
            _rwork_dim1 = 16+11*_neq_py
        else:
            _rwork_dim1 = 0
        _rwork = utils._EngineFloat64ArrayType.empty_ctypes(_rwork_dim1)
        comm['rwork'] = _rwork
    else:
        _rwork = comm['rwork']
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(10)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(60)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _h,
        _tol,
        _thres,
        _thresp,
        _maxstp,
        _start,
        _onestp,
        _high,
        _rwork,
        _lrwork,
        _lwsav,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, thresp[{:s}] = {:s} is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'positive.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, thres[{:s}] = {:s} is not'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'positive.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, high = False and lrwork = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if high = False then lrwork >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, high = True and lrwork = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if high = True then lrwork >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: {:s} < tol < {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _start_py = utils._EngineLogicalScalarType.to_py(_start)
        return_data = _start_py
    _err_class.manage()
    return return_data

def ivp_2nd_rkn_diag(neq, thres, thresp, comm):
    u"""
    Second-order ordinary differential equations, initial value problem,
    diagnostics for ``ivp_2nd_rkn``.

    For full information please refer to the NAG Library document for
    d02ly

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02lyf.html

    Parameters
    ----------
    neq : int
        The number of second-order ordinary differential equations
        solved by ``ivp_2nd_rkn``. It must be the same as the argument
        `neq` supplied to ``ivp_2nd_rkn_setup`` and ``ivp_2nd_rkn``.

    thres : float, shape (`neq`), modified in place
        On exit: the ith solution threshold value used in the error
        control strategy. (See ``ivp_2nd_rkn_setup``.)

    thresp : float, shape (`neq`), modified in place
        On exit: the ith derivative threshold value used in the error
        control strategy. (See ``ivp_2nd_rkn_setup``.)

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_2nd_rkn_setup``.

    Returns
    -------
    hnext : float
        The next step size which ``ivp_2nd_rkn``, if called, would
        attempt.

    hused : float
        The last successful step size used by ``ivp_2nd_rkn``.

    hstart : float
        The initial step size used on the current integration problem by
        ``ivp_2nd_rkn``.

    nsucc : int
        The number of steps attempted by ``ivp_2nd_rkn`` that have been
        successful since the start of the current problem.

    nfail : int
        The number of steps attempted by ``ivp_2nd_rkn`` that have
        failed since the start of the current problem.

    natt : int
        The number of steps attempted before the initial step was
        successful. Over a large number of problems the cost of an
        attempted step of this type is approximately half that of a
        normal attempted step.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*, but in the previous call to the
            setup function neq = *<value>*.

        (`errno` 1)
            The integrator function ``ivp_2nd_rkn`` has not been called.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_2nd_rkn_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02lyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Output, hnext.
        utils._BYREF_FLOAT64, # Output, hused.
        utils._BYREF_FLOAT64, # Output, hstart.
        utils._BYREF_INT, # Output, nsucc.
        utils._BYREF_INT, # Output, nfail.
        utils._BYREF_INT, # Output, natt.
        utils._EngineFloat64ArrayType.c_type, # Output, thres[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, thresp[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input, iwsav[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _hnext = utils._EngineFloat64ScalarType.empty_ctypes()
    _hused = utils._EngineFloat64ScalarType.empty_ctypes()
    _hstart = utils._EngineFloat64ScalarType.empty_ctypes()
    _nsucc = utils._EngineIntScalarType.empty_ctypes()
    _nfail = utils._EngineIntScalarType.empty_ctypes()
    _natt = utils._EngineIntScalarType.empty_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _thres_cls = utils._EngineFloat64ArrayType(
        thres, {'fun_name': fun_name, 'entity': 'thres'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _thres = _thres_cls.to_ctypes(can_reference_contents=False)
    _thresp_cls = utils._EngineFloat64ArrayType(
        thresp, {'fun_name': fun_name, 'entity': 'thresp'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _thresp = _thresp_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _hnext,
        _hused,
        _hstart,
        _nsucc,
        _nfail,
        _natt,
        _thres,
        _thresp,
        _rwork,
        _lrwork,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}, but in the previous call'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integrator function ivp_2nd_rkn has not been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _thres_cls.refresh_py(_thres)
        _thresp_cls.refresh_py(_thresp)
        _hnext_py = utils._EngineFloat64ScalarType.to_py(_hnext)
        _hused_py = utils._EngineFloat64ScalarType.to_py(_hused)
        _hstart_py = utils._EngineFloat64ScalarType.to_py(_hstart)
        _nsucc_py = utils._EngineIntScalarType.to_py(_nsucc)
        _nfail_py = utils._EngineIntScalarType.to_py(_nfail)
        _natt_py = utils._EngineIntScalarType.to_py(_natt)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['hnext', 'hused', 'hstart', 'nsucc', 'nfail', 'natt'],
        )(
            _hnext_py,
            _hused_py,
            _hstart_py,
            _nsucc_py,
            _nfail_py,
            _natt_py,
        )
    _err_class.manage()
    return return_data

def ivp_2nd_rkn_interp(neq, t, y, yp, nwant, twant, ywant, ypwant, comm):
    u"""
    Second-order ordinary differential equations, initial value problem,
    interpolation for ``ivp_2nd_rkn``.

    For full information please refer to the NAG Library document for
    d02lz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02lzf.html

    Parameters
    ----------
    neq : int
        The number of second-order ordinary differential equations being
        solved by ``ivp_2nd_rkn``. It must contain the same value as the
        argument `neq` in a prior call to ``ivp_2nd_rkn``.

    t : float
        t, the current value at which the solution and its derivative
        have been computed (as returned in argument `t` on output from
        ``ivp_2nd_rkn``).

    y : float, shape (`neq`)
        The ith component of the solution at t, for i = 1,2,...,`neq`,
        as returned from ``ivp_2nd_rkn``.

    yp : float, shape (`neq`)
        The ith component of the derivative at t, for i = 1,2,...,`neq`,
        as returned from ``ivp_2nd_rkn``.

    nwant : int
        The number of components of the solution and derivative whose
        values at `twant` are required. The first `nwant` components are
        evaluated.

    twant : float
        The point at which components of the solution and derivative are
        to be evaluated. `twant` should not normally be an extrapolation
        point, that is `twant` should satisfy

            told <= `twant` <= `t`,

        or if integration is proceeding in the negative direction

            told >= `twant` >= `t`,

        where told is the previous integration point which is held in an
        element of the array `comm`['rwork'] and is, to within rounding,
        `t`-hused. (hused is given by ``ivp_2nd_rkn_diag``.)
        Extrapolation is permitted but not recommended, and `errno` = 2
        is returned whenever extrapolation is attempted.

    ywant : float, shape (`nwant`), modified in place
        On exit: the calculated value of the ith component of the
        solution at t = `twant`, for i = 1,2,...,`nwant`.

    ypwant : float, shape (`nwant`), modified in place
        On exit: the calculated value of the ith component of the
        derivative at t = `twant`, for i = 1,2,...,`nwant`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_2nd_rkn_setup``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*, but in a previous call to the
            setup function neq = *<value>*.

        (`errno` 1)
            The integrator function ``ivp_2nd_rkn`` has not been called.

        (`errno` 1)
            No successful steps, interpolation impossible at `twant` =
            *<value>*.

        (`errno` 1)
            On entry, `nwant` = *<value>*.

            Constraint: `nwant` >= 1.

        (`errno` 1)
            On entry, `nwant` = *<value>* and `neq` = *<value>*.

            Constraint: `nwant` <= `neq`.

        (`errno` 3)
            Interpolation is not permitted on data from the high order
            formulas.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Extrapolation at `twant` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_2nd_rkn_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02lzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, yp[neq].
        utils._BYREF_INT, # Input, nwant.
        utils._BYREF_FLOAT64, # Input, twant.
        utils._EngineFloat64ArrayType.c_type, # Output, ywant[nwant].
        utils._EngineFloat64ArrayType.c_type, # Output, ypwant[nwant].
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[60].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _yp = utils._EngineFloat64ArrayType(
        yp, {'fun_name': fun_name, 'entity': 'yp'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _nwant = utils._EngineIntScalarType(
        nwant, {'fun_name': fun_name, 'entity': 'nwant'},
    ).to_ctypes()
    _twant = utils._EngineFloat64ScalarType(
        twant, {'fun_name': fun_name, 'entity': 'twant'},
    ).to_ctypes()
    _nwant_py = utils._EngineIntScalarType.to_py(_nwant)
    _ywant_cls = utils._EngineFloat64ArrayType(
        ywant, {'fun_name': fun_name, 'entity': 'ywant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ywant = _ywant_cls.to_ctypes(can_reference_contents=False)
    _ypwant_cls = utils._EngineFloat64ArrayType(
        ypwant, {'fun_name': fun_name, 'entity': 'ypwant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ypwant = _ypwant_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _t,
        _y,
        _yp,
        _nwant,
        _twant,
        _ywant,
        _ypwant,
        _rwork,
        _lrwork,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}, but in a previous call to the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'setup function neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integrator function ivp_2nd_rkn has not been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No successful steps, interpolation impossible at',
                    'twant = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwant >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwant <= neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extrapolation at twant = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Interpolation is not permitted on data from the high order formulas.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ywant_cls.refresh_py(_ywant)
        _ypwant_cls.refresh_py(_ypwant)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dae_dassl_cont(comm):
    u"""
    Implicit ordinary differential equations/DAEs, initial value
    problem, DASSL method continuation for ``dae_dassl_gen``.

    For full information please refer to the NAG Library document for
    d02mc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02mcf.html

    Parameters
    ----------
    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dae_dassl_gen``.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.dae_dassl_cont'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02mcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineIntArrayType.c_type, # Input/Output, icom[15].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icom',),
    )
    _icom = comm['icom']
    engine_ctypes(
        _en_data,
        _icom,
    )
    utils._handle_cb_exception(excs)
    return_data = None
    return return_data

def ivp_stiff_dassl(neqmax, sdysav, maxord, con, tcrit, hmin, hmax, h0, maxstp, mxhnil, norm):
    u"""
    Ordinary differential equations, initial value problem, DASSL
    method, setup for d02m--n functions.

    For full information please refer to the NAG Library document for
    d02mv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02mvf.html

    Parameters
    ----------
    neqmax : int
        A bound on the maximum number of differential equations to be
        solved.

    sdysav : int
        The second dimension of the array ysav that will be supplied to
        the integrator, as declared in the (sub)program from which the
        integrator is called (e.g., see ``ivp_stiff_exp_fulljac``).

    maxord : int
        The maximum order to be used for the BDF method. If `maxord` = 0
        or `maxord` > 5, `maxord` = 5 is assumed.

    con : float, shape (3), modified in place
        On entry: values to be used to control step size choice during
        integration. If any `con`[i] = 0.0 on entry, it is replaced by
        its default value described below. In most cases this is the
        recommended setting.

        `con`[0], `con`[1], and `con`[2] are factors used to bound step
        size changes.

        If the current step size h fails, the modulus of the next step
        size is bounded by `con`[0]*|h|.

        The default value of `con`[0] is 2.0.

        Note that the new step size may be used with a method of
        different order to the failed step.

        If the initial step size is h, the modulus of the step size on
        the second step is bounded by `con`[2]*|h|.

        At any other stage in the integration, if the current step size
        is h, the modulus of the next step size is bounded by
        `con`[1]*|h|.

        The default values are 10.0 for `con`[1] and 1000.0 for
        `con`[2].

        On exit: the values actually to be used by the integration
        function.

    tcrit : float
        A point beyond which integration must not be attempted. The use
        of `tcrit` is described under the argument itask in the
        specification for the integrator (e.g., see
        ``ivp_stiff_exp_fulljac``). A value, 0.0 say, must be specified
        even if itask subsequently specifies that `tcrit` will not be
        used.

    hmin : float
        The minimum absolute step size to be allowed. Set `hmin` = 0.0
        if this option is not required.

    hmax : float
        The maximum absolute step size to be allowed. Set `hmax` = 0.0
        if this option is not required.

    h0 : float
        The step size to be attempted on the first step. Set `h0` = 0.0
        if the initial step size is calculated internally.

    maxstp : int
        The maximum number of steps to be attempted during one call to
        the integrator after which it will return with `errno` = 2
        (e.g., see ``ivp_stiff_exp_fulljac``). Set `maxstp` = 0 if no
        limit is to be imposed.

    mxhnil : int
        The maximum number of warnings printed (if itrace >= 0, e.g.,
        see ``ivp_stiff_exp_fulljac``) per problem when t+h = t on a
        step (h = current step size). If `mxhnil` <= 0, a default value
        of 10 is assumed.

    norm : str, length 1
        Indicates the type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L2 norm.

        `norm` = 'D'

          Is the same as 'A'.

        If vnorm denotes the norm of the vector v of length n, for the
        averaged L2 norm

            vnormB = sqrt(1/nsum_i = 1^n((v_i)/(w_i))^2),

        while for the maximum norm

            vnorm = max_{1 <= i <= n}|(v_i)/(w_i)|.

        If you wish to weight the maximum norm or the L2 norm, rtol and
        atol should be scaled appropriately on input to the integrator
        (see under itol in the specification of the integrator for the
        formulation of the weight vector w_i from rtol and atol, e.g.,
        see ``ivp_stiff_exp_fulljac``).

        Only the first character to the actual argument `norm` is passed
        to ``ivp_stiff_dassl``; hence it is permissible for the actual
        argument to be more descriptive, e.g., 'Maximum', 'Average L2'
        or 'Default' in a call to ``ivp_stiff_dassl``.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The sequence `con` is not strictly increasing: `con`[0] <
            `con`[1] < `con`[2] is required.

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>*.

            Constraint: `con`[*<value>*] >= 1.0.

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>*.

            Constraint: `con`[*<value>*] >= 0.0.

        (`errno` 1)
            On entry, `norm` was not valid: `norm` = *<value>*.

            Constraint: `norm` = 'M', 'A' or 'D'.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `sdysav` = *<value>* and `maxord`+3 = *<value>*.

            Constraint: `sdysav` >= `maxord`+3.

        (`errno` 1)
            On entry, `maxord` = *<value>*.

            Constraint: `maxord` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, `maxord` > 5. `maxord` = 5 assumed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_dassl'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02mvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_INT, # Input, maxord.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, con[3].
        utils._BYREF_FLOAT64, # Input, tcrit.
        utils._BYREF_FLOAT64, # Input, hmin.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_FLOAT64, # Input, h0.
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_INT, # Input, mxhnil.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[50+4*neqmax].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _maxord = utils._EngineIntScalarType(
        maxord, {'fun_name': fun_name, 'entity': 'maxord'},
    ).to_ctypes()
    _con_cls = utils._EngineFloat64ArrayType(
        con, {'fun_name': fun_name, 'entity': 'con'},
        exp_shape=(3,),
    )
    _con = _con_cls.to_ctypes()
    _tcrit = utils._EngineFloat64ScalarType(
        tcrit, {'fun_name': fun_name, 'entity': 'tcrit'},
    ).to_ctypes()
    _hmin = utils._EngineFloat64ScalarType(
        hmin, {'fun_name': fun_name, 'entity': 'hmin'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _h0 = utils._EngineFloat64ScalarType(
        h0, {'fun_name': fun_name, 'entity': 'h0'},
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _mxhnil = utils._EngineIntScalarType(
        mxhnil, {'fun_name': fun_name, 'entity': 'mxhnil'},
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _neqmax_py = utils._EngineIntScalarType.to_py(_neqmax)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(50+4*_neqmax_py)
    _comm_py = {}
    _comm_py['rwork'] = _rwork
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqmax,
        _sdysav,
        _maxord,
        _con,
        _tcrit,
        _hmin,
        _hmax,
        _h0,
        _maxstp,
        _mxhnil,
        _norm,
        _rwork,
        _errbuf,
        _ierr,
        _norm._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The sequence con is not strictly increasing: con[0] < con[1] < con[2] is',
                    'required.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: con[{:s}] >= 1.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: con[{:s}] >= 0.0.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm was not valid: norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'M\', \'A\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sdysav = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'maxord+3 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdysav >= maxord+3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord > 5. maxord = 5 assumed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxord >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _con_cls.refresh_py(_con)
        return_data = _comm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['comm'],
            )(
                _comm_py,
            )
    _err_class.manage()
    return return_data

def dae_dassl_setup(neq, maxord, jceval, hmax, h0, itol):
    u"""
    Implicit ordinary differential equations/DAEs, initial value
    problem, setup for ``dae_dassl_gen``.

    For full information please refer to the NAG Library document for
    d02mw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02mwf.html

    Parameters
    ----------
    neq : int
        The number of differential-algebraic equations to be solved.

    maxord : int
        The maximum order to be used for the BDF method. Orders up to
        5th order are available; setting `maxord` > 5 means that the
        maximum order used will be 5.

    jceval : str, length 1
        Specifies the technique to be used to compute the Jacobian.

        `jceval` = 'N'

          The Jacobian is to be evaluated numerically by the integrator.

        `jceval` = 'A'

          You must supply a function to evaluate the Jacobian on a call
          to the integrator.

        Only the first character of the actual paramater `jceval` is
        passed to ``dae_dassl_setup``; hence it is permissible for the
        actual argument to be more descriptive, e.g., 'Numerical' or
        'Analytical', on a call to ``dae_dassl_setup``.

    hmax : float
        The maximum absolute step size to be allowed. Set `hmax` = 0.0
        if this option is not required.

    h0 : float
        The step size to be attempted on the first step. Set `h0` = 0.0
        if the initial step size is calculated internally.

    itol : int
        A value to indicate the form of the local error test.

        `itol` = 0

          rtol and atol are single element vectors.

        `itol` = 1

          rtol and atol are vectors. This should be chosen if you want
          to apply different tolerances to each equation in the system.

        See ``dae_dassl_gen``.

        Note: the tolerances must either both be single element vectors
        or both be vectors of length `neq`.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 2)
            On entry, `maxord` = *<value>*.

            Constraint: `maxord` >= 1.

        (`errno` 3)
            On entry, `jceval` has an illegal value: `jceval` =
            *<value>*.

            Constraint: `jceval` = 'N' or 'A'.

        (`errno` 4)
            On entry, `hmax` = *<value>*.

            Constraint: `hmax` >= 0.0.

        (`errno` 6)
            On entry, `itol` = *<value>*.

            Constraint: `itol` = 0 or 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.dae_dassl_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02mwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, maxord.
        utils._BYREF_STRING, # Input, jceval(len=1).
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_FLOAT64, # Input, h0.
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Output, icom[licom].
        utils._BYREF_INT, # Input, licom.
        utils._EngineFloat64ArrayType.c_type, # Output, com[lcom].
        utils._BYREF_INT, # Input, lcom.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jceval.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _maxord = utils._EngineIntScalarType(
        maxord, {'fun_name': fun_name, 'entity': 'maxord'},
    ).to_ctypes()
    _jceval = utils._EngineCharScalarType(
        jceval, {'fun_name': fun_name, 'entity': 'jceval'}, exp_chrlen=1,
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _h0 = utils._EngineFloat64ScalarType(
        h0, {'fun_name': fun_name, 'entity': 'h0'},
    ).to_ctypes()
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _licom = utils._EngineIntScalarType(
        _neq_py+50, {'fun_name': fun_name, 'entity': '_licom'},
    ).to_ctypes()
    _icom = utils._EngineIntArrayType.empty_ctypes(_neq_py+50)
    _comm_py = {}
    _comm_py['icom'] = _icom
    _maxord_py = utils._EngineIntScalarType.to_py(_maxord)
    _lcom = utils._EngineIntScalarType(
        40+(_maxord_py+4)*_neq_py+(2*(_neq_py-1)+(_neq_py-1)+1)*_neq_py+2*(_neq_py//((_neq_py-1)+(_neq_py-1)+1)+1), {'fun_name': fun_name, 'entity': '_lcom'},
    ).to_ctypes()
    _com = utils._EngineFloat64ArrayType.empty_ctypes(40+(_maxord_py+4)*_neq_py+(2*(_neq_py-1)+(_neq_py-1)+1)*_neq_py+2*(_neq_py//((_neq_py-1)+(_neq_py-1)+1)+1))
    _comm_py['com'] = _com
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _maxord,
        _jceval,
        _hmax,
        _h0,
        _itol,
        _icom,
        _licom,
        _com,
        _lcom,
        _errbuf,
        _ierr,
        _jceval._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxord >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jceval has an illegal value: jceval = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jceval = \'N\' or \'A\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1013:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: hmax >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1015:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itol = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def ivp_stiff_interp(tsol, sol, m, ldysav, neq, comm, sdysav):
    u"""
    Ordinary differential equations, initial value problem,
    interpolation for d02m--n routines (all integration methods),
    natural interpolant.

    For full information please refer to the NAG Library document for
    d02mz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02mzf.html

    Parameters
    ----------
    tsol : float
        The point at which the first `m` components of the solution are
        to be evaluated. `tsol` should not normally be an extrapolation
        point. Extrapolation is permitted but not recommended.

    sol : float, shape (`m`), modified in place
        On exit: the calculated value of the solution at `tsol`.

    m : int
        The number of components of the solution whose values are
        required.

    ldysav : int
        The value used for the argument `ldysav` when calling the
        integrator.

    neq : int
        The value used for the argument `neq` when calling the
        integrator.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_exp_fulljac``, ``ivp_stiff_exp_bandjac``,
        ``ivp_stiff_exp_sparjac``, ``ivp_stiff_imp_fulljac``,
        ``ivp_stiff_imp_bandjac``, ``ivp_stiff_imp_sparjac``,
        ``ivp_stiff_exp_revcom`` or ``ivp_stiff_imp_revcom``.

    sdysav : int
        The value used for the argument `sdysav` when calling the
        integrator.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `m` = *<value>* and `neq` = *<value>*.

            Constraint: `m` <= `neq`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            Either the integrator function has not been called prior to
            the first call of this function or the array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, `tsol` is an extrapolation point.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02mzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, tsol.
        utils._EngineFloat64ArrayType.c_type, # Output, sol[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, neq.
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[50+4*neq].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tsol = utils._EngineFloat64ScalarType(
        tsol, {'fun_name': fun_name, 'entity': 'tsol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _sol_cls = utils._EngineFloat64ArrayType(
        sol, {'fun_name': fun_name, 'entity': 'sol'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _sol = _sol_cls.to_ctypes(can_reference_contents=False)
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('ysav', 'rwork', 'iwsav', 'rwsav'),
    )
    _ysav = comm['ysav']
    _rwork = comm['rwork']
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tsol,
        _sol,
        _m,
        _ldysav,
        _neq,
        _ysav,
        _sdysav,
        _rwork,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator function has not been called prior to the first call',
                    'of this function or the array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tsol is an extrapolation point.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _sol_cls.refresh_py(_sol)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_exp_fulljac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, fcn, sdysav, jac, monitr, itask, itrace, data=None, io_manager=None):
    u"""
    Explicit ordinary differential equations, stiff initial value
    problem, full Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nbf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_fulljac_setup`` and one of ``ivp_stiff_bdf`` or
        ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_exp_fulljac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    fcn : callable ires = fcn(neq, t, y, f, ires, data=None)
        `fcn` must evaluate the derivative vector for the explicit
        ordinary differential equation system, defined by y' = g(t,y).

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        f : float, ndarray, shape (`neq`), to be modified in place
            On exit: the value y'_i, given by y'_i = g_i(t,y), for i =
            1,2,...,`neq`.

        ires : int
            `ires` = 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            You may set `ires` as follows to indicate certain conditions
            in `fcn` to the integrator:

            `ires` = 1

              Indicates a normal return from `fcn`, that is `ires` has
              not been altered by you and integration continues.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specification of the integrator setup functions
        ``ivp_stiff_bdf`` and ``ivp_stiff_blend``. This value must be
        the same as that supplied to the integrator setup function.

    jac : None or callable jac(neq, t, y, h, d, p, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the full linear algebra setup
        function ``ivp_stiff_fulljac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        p : float, ndarray, shape (`neq`, `neq`), to be modified in
        place
            On entry: is set to zero.

            On exit: `p`[i-1,j-1] must contain dr_i/dy_j, for j =
            1,2,...,`neq`, for i = 1,2,...,`neq`.

            Only the nonzero elements of this array need be set, since
            it is preset to zero before the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        ldysav : int
            An upper bound on the number of differential equations to be
            solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_exp_fulljac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector, y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in `fcn`)
              caused an early termination (this facility could be used
              to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_exp_fulljac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` is set != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit (e.g., see
          ``ivp_stiff_dassl``). tcrit must be specified as an option in
          one of the integrator setup functions before the first call to
          the integrator, or specified in the optional input function
          before a continuation call. tcrit may be equal to or beyond
          `tout`, but not before it, in the direction of integration.

        `itask` = 5

          Take one step only and return, without passing tcrit (e.g.,
          see ``ivp_stiff_dassl``). tcrit must be specified as under
          `itask` = 4.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    Raises
    ------
    NagValueError
        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. `hmin` =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. `hmax` =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 7)
            The derivative evaluation procedure set the error flag
            `ires` = 3 repeatedly despite attempts by the integrator to
            avoid this.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            The linear algebra setup routine for full Jacobian was not
            called first.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 6)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 11)
            `fcn` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 12)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.

    See Also
    --------
    :meth:`naginterfaces.library.examples.ode.ivp_stiff_exp_fulljac_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_exp_fulljac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, f[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, neq, t, y, f, ires, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _neq_fcnh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_fcnh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neq_fcnh,),
                can_reference_contents=False,
            )
            _ires_fcnh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                    data,
                )
            else:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_fcnh, {'fun_name': cb_fcn_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[neq, neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, h, d, p, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _p_jach = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_neq_jach, _neq_jach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _p_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _p_jach,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _p_jach, {'fun_name': cb_jac_name, 'returned': 'p'}, p,
            )
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02nbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _ydot = _ydot_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _fcn,
        _fcnh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _monitr,
        _monitrh,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'tcrit = {:s} and the current time is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99953:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99952:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99955:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The derivative evaluation procedure set the error flag ires = 3 repeatedly',
                    'despite attempts by the integrator to avoid this.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99949:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'fcn set ires = 2, which signals that the integration should terminate.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99970:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99958:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear algebra setup routine for full Jacobian was not called first.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        return_data = _t_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['t'],
            )(
                _t_py,
            )
    _err_class.manage()
    return return_data

def ivp_stiff_exp_bandjac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, fcn, sdysav, jac, monitr, itask, itrace, data=None, io_manager=None):
    u"""
    Explicit ordinary differential equations, stiff initial value
    problem, banded Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02nc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ncf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_bandjac_setup`` and one of ``ivp_stiff_bdf`` or
        ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_exp_bandjac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    fcn : callable ires = fcn(neq, t, y, f, ires, data=None)
        `fcn` must evaluate the derivative vector for the explicit
        ordinary differential equation system, defined by y' = g(t,y).

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        f : float, ndarray, shape (`neq`), to be modified in place
            On exit: the value y'_i, given by y'_i = g_i(t,y), for i =
            1,2,...,`neq`.

        ires : int
            `ires` = 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            You may set `ires` as follows to indicate certain conditions
            in `fcn` to the integrator:

            `ires` = 1

              Indicates a normal return from `fcn`, that is `ires` has
              not been altered by you and integration continues.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specification of the integrator setup functions
        ``ivp_stiff_bdf`` and ``ivp_stiff_blend``. This value must be
        the same as that supplied to the integrator setup function.

    jac : None or callable jac(neq, t, y, h, d, ml, mu, p, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the banded linear algebra setup
        function ``ivp_stiff_bandjac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        ml : int
            The number of subdiagonals and superdiagonals respectively
            in the band.

        mu : int
            The number of subdiagonals and superdiagonals respectively
            in the band.

        p : float, ndarray, shape (`ml`+`mu`+1, `neq`), to be modified
        in place
            On entry: is set to zero.

            On exit: elements of the Jacobian matrix dr/dy stored as
            specified by the following pseudocode:

            ::

              for i in range(1, neq+1):
                j1 = max(i-ml, 1)
                j2 = min(i+mu, neq)
                for j in range(j1, j2+1):
                  k = min(ml+1-i, 0) + j
                  p[k-1, i-1] = J[i-1, j-1]

            See also ``lapacklin.dgbtrf``.

            Only nonzero elements of this array need be set, since it is
            preset to zero before the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        ldysav : int
            An upper bound on the number of differential equations to be
            solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_exp_bandjac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector, y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in `fcn`)
              caused an early termination (this facility could be used
              to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_exp_bandjac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` is set != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit may be equal
          to or beyond `tout`, but not before it, in the direction of
          integration.

        `itask` = 5

          Take one step only and return, without passing tcrit. tcrit
          must be specified as under `itask` = 4.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    Raises
    ------
    NagValueError
        (`errno` 1)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. `hmin` =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. `hmax` =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 7)
            The derivative evaluation procedure set the error flag
            `ires` = 3 repeatedly despite attempts by the integrator to
            avoid this.

        (`errno` 10)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            The linear algebra setup routine for banded Jacobian was not
            called first.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 6)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 11)
            `fcn` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_exp_bandjac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, f[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, neq, t, y, f, ires, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _neq_fcnh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_fcnh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neq_fcnh,),
                can_reference_contents=False,
            )
            _ires_fcnh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                    data,
                )
            else:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_fcnh, {'fun_name': cb_fcn_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[ml+mu+1, neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, h, d, ml, mu, p, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _ml_jach = utils._EngineIntScalarType.to_py(ml.contents)
            _mu_jach = utils._EngineIntScalarType.to_py(mu.contents)
            _p_jach = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_ml_jach+_mu_jach+1, _neq_jach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _ml_jach,
                    _mu_jach,
                    _p_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _ml_jach,
                    _mu_jach,
                    _p_jach,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _p_jach, {'fun_name': cb_jac_name, 'returned': 'p'}, p,
            )
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02ncft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _ydot = _ydot_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'jacpvt', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    if _comm_needs_init:
        _jacpvt = utils._EngineIntArrayType.empty_ctypes(_ldysav_py)
        comm['jacpvt'] = _jacpvt
    else:
        _jacpvt = comm['jacpvt']
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _fcn,
        _fcnh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _monitr,
        _monitrh,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'tcrit = {:s} and the current time is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99953:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99952:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99955:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The derivative evaluation procedure set the error flag ires = 3 repeatedly',
                    'despite attempts by the integrator to avoid this.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99949:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'fcn set ires = 2, which signals that the integration should terminate.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99958:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear algebra setup routine for banded Jacobian was not called first.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        return_data = _t_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['t'],
            )(
                _t_py,
            )
    _err_class.manage()
    return return_data

def ivp_stiff_exp_sparjac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, fcn, sdysav, jac, monitr, itask, itrace, data=None, io_manager=None):
    u"""
    Explicit ordinary differential equations, stiff initial value
    problem, sparse Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02nd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ndf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_sparjac_setup`` and one of ``ivp_stiff_bdf`` or
        ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_exp_sparjac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    fcn : callable ires = fcn(neq, t, y, f, ires, data=None)
        `fcn` must evaluate the derivative vector for the explicit
        ordinary differential equation system, defined by y' = g(t,y).

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        f : float, ndarray, shape (`neq`), to be modified in place
            On exit: the value y'_i, given by y'_i = g_i(t,y), for i =
            1,2,...,`neq`.

        ires : int
            `ires` = 1.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            You may set `ires` as follows to indicate certain conditions
            in `fcn` to the integrator:

            `ires` = 1

              Indicates a normal return from `fcn`, that is `ires` has
              not been altered by you and integration continues.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specification of the integrator setup functions
        ``ivp_stiff_bdf`` and ``ivp_stiff_blend``. This value must be
        the same as that supplied to the integrator setup function.

    jac : None or callable jac(neq, t, y, h, d, j, pdj, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the sparse linear algebra setup
        function ``ivp_stiff_sparjac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        j : int
            The column of the Jacobian that `jac` must return in the
            array `pdj`.

        pdj : float, ndarray, shape (`neq`), to be modified in place
            On entry: is set to zero.

            On exit: `pdj`[i-1] should be set to the (i,j)th element of
            the Jacobian, where j is given by `j`. Only nonzero elements
            of this array need be set, since it is preset to zero before
            the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential equations being solved.

        ldysav : int
            An upper bound on the number of differential equations to be
            solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_exp_sparjac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector, y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in `fcn`)
              caused an early termination (this facility could be used
              to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_exp_sparjac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` is set != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit (e.g., see
          ``ivp_stiff_dassl``). tcrit must be specified as an option in
          one of the integrator setup functions before the first call to
          the integrator, or specified in the optional input function
          before a continuation call. tcrit may be equal to or beyond
          `tout`, but not before it, in the direction of integration.

        `itask` = 5

          Take one step only and return, without passing tcrit (e.g.,
          see ``ivp_stiff_dassl``). tcrit must be specified as under
          `itask` = 4.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    Raises
    ------
    NagValueError
        (`errno` 1)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. `hmin` =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. `hmax` =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 7)
            The derivative evaluation procedure set the error flag
            `ires` = 3 repeatedly despite attempts by the integrator to
            avoid this.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            The linear algebra setup function for sparse Jacobian was
            not called first.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 6)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 11)
            `fcn` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_exp_sparjac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, f[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, neq, t, y, f, ires, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _neq_fcnh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_fcnh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neq_fcnh,),
                can_reference_contents=False,
            )
            _ires_fcnh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                    data,
                )
            else:
                _ires_fcnh = fcn(
                    _neq_fcnh,
                    _t_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _ires_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._EngineIntScalarType(
                _ires_fcnh, {'fun_name': cb_fcn_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_INT, # Input, j.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, pdj[neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, h, d, j, pdj, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _j_jach = utils._EngineIntScalarType.to_py(j.contents)
            _pdj_jach = utils._EngineFloat64ArrayType.from_pointer(
                pdj,
                (_neq_jach,),
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _j_jach,
                    _pdj_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _h_jach,
                    _d_jach,
                    _j_jach,
                    _pdj_jach,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _pdj_jach, {'fun_name': cb_jac_name, 'returned': 'pdj'}, pdj,
            )
            utils._unlink_pointer(pdj)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02ndft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _ydot = _ydot_cls.to_ctypes(can_reference_contents=False)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'jacpvt', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    _jacpvt = comm['jacpvt']
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _fcn,
        _fcnh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _monitr,
        _monitrh,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'tcrit = {:s} and the current time is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99953:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99952:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99976:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99955:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The derivative evaluation procedure set the error flag ires = 3 repeatedly',
                    'despite attempts by the integrator to avoid this.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99949:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'fcn set ires = 2, which signals that the integration should terminate.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99958:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The linear algebra setup function for sparse Jacobian was not called first.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        return_data = _t_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['t'],
            )(
                _t_py,
            )
    _err_class.manage()
    return return_data

def dae_dassl_gen(neq, t, tout, y, ydot, rtol, atol, itask, res, jac, comm, data=None):
    u"""
    Implicit ordinary differential equations/DAEs, initial value
    problem, DASSL method integrator.

    For full information please refer to the NAG Library document for
    d02ne

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nef.html

    Parameters
    ----------
    neq : int
        The number of differential-algebraic equations to be solved.

    t : float
        On initial entry: the initial value of the independent variable,
        t.

    tout : float
        The next value of t at which a computed solution is desired.

        On initial entry: `tout` is used to determine the direction of
        integration. Integration is permitted in either direction (see
        also `itask`).

    y : float, shape (`neq`), modified in place
        On initial entry: the vector of initial values of the dependent
        variables y.

        On intermediate exit: the computed solution vector, y, evaluated
        at t = T.

        On final exit: the computed solution vector, evaluated at t
        (usually t = `tout`).

    ydot : float, shape (`neq`), modified in place
        On initial entry: `ydot` must contain approximations to the time
        derivatives y' of the vector y evaluated at the initial value of
        the independent variable.

        On exit: the time derivatives y' of the vector y at the last
        integration point.

    rtol : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icom'][1]=1: `neq`; if `comm`['icom'][1]=0:
        1; otherwise: 0.

        On entry: the relative local error tolerance.

        On exit: `rtol` remains unchanged unless ``dae_dassl_gen`` exits
        with `errno` = 16 in which case the values may have been
        increased to values estimated to be appropriate for continuing
        the integration.

    atol : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icom'][1]=1: `neq`; if `comm`['icom'][1]=0:
        1; otherwise: 0.

        On entry: the absolute local error tolerance.

        On exit: `atol` remains unchanged unless ``dae_dassl_gen`` exits
        with `errno` = 16 in which case the values may have been
        increased to values estimated to be appropriate for continuing
        the integration.

    itask : int
        On initial entry: need not be set.

    res : callable ires = res(neq, t, y, ydot, r, ires, data=None)
        `res` must evaluate the residual

            R = F(t,y,y').

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential-algebraic equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        ydot : float, ndarray, shape (`neq`)
            The derivative of the solution at the current point t.

        r : float, ndarray, shape (`neq`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of R, for i
            = 1,2,...,`neq` where

                R = F(`t`,`y`,`ydot`).

        ires : int
            Is always equal to zero.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            `ires` should normally be left unchanged. However, if an
            illegal value of `y` is encountered, `ires` should be set to
            -1; ``dae_dassl_gen`` will then attempt to resolve the
            problem so that illegal values of `y` are not encountered.
            `ires` should be set to -2 if you wish to return control to
            the calling function; this will cause ``dae_dassl_gen`` to
            exit with `errno` = 23.

    jac : None or callable jac(neq, t, y, ydot, pd, lenpd, cj,
        data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        Evaluates the matrix of partial derivatives, J, where

            J_{ij} = dF_i/dy_j+`cj`*dF_i/dy'_j, i,j = 1,2,...,`neq`.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of differential-algebraic equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        ydot : float, ndarray, shape (`neq`)
            The derivative of the solution at the current point t.

        pd : float, ndarray, shape (`lenpd`), to be modified in place
            On entry: this argument is preset to zero before the call to
            `jac`.

            On exit: if the Jacobian is full then `pd`[(j-1)*`neq`+i-1]
            = J_{ij}, for j = 1,2,...,`neq`, for i = 1,2,...,`neq`; if
            the Jacobian is banded then `pd`[(j-1)*(2ml+mu+1)+ml+mu+i-j]
            = J_{ij}, for max(1, j-mu) <= i <= min(n, j+ml); (see also
            in ``lapacklin.dgbtrf``).

        lenpd : int
            The dimension of the array `pd`.

        cj : float
            `cj` is a scalar constant which will be defined in
            ``dae_dassl_gen``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dae_dassl_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    t : float
        On intermediate exit: t, the current value of the independent
        variable.

        On final exit: the value of the independent variable at which
        the computed solution y is returned (usually at `tout`).

    itask : int
        The task performed by the integrator on successful completion or
        an indicator that a problem occurred during integration.

        `itask` = 2

          The integration to `tout` was successfully completed (`t` =
          `tout`) by stepping exactly to `tout`.

        `itask` = 3

          The integration to `tout` was successfully completed (`t` =
          `tout`) by stepping past `tout`. `y` and `ydot` are obtained
          by interpolation.

        `itask` < 0

          Different negative values of `itask` returned correspond to
          different failure exits. errno should always be checked in
          such cases and the corrective action taken where appropriate.

        this argument must remain unchanged between calls to
        ``dae_dassl_gen``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 3)
            On entry, `t` = *<value>*.

            Constraint: `tout` != `t`.

        (`errno` 3)
            `tout` is too close to `t` to start integration: `tout`-`t`
            = *<value>*: hmin = *<value>*.

        (`errno` 3)
            `tout` is behind `t` in the direction of h: `tout`-`t` =
            *<value>*, h = *<value>*.

        (`errno` 6)
            Some element of `rtol` is less than zero.

        (`errno` 7)
            Some element of `atol` is less than zero.

        (`errno` 8)
            A previous call to this function returned with `itask` =
            *<value>* and no appropriate action was taken.

        (`errno` 11)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 12)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 13)
            `comm`['com'] array is of insufficient length; length
            required = *<value>*; actual length lcom = *<value>*.

        (`errno` 14)
            All elements of `rtol` and `atol` are zero.

        (`errno` 15)
            Maximum number of steps taken on this call before reaching
            `tout`: `t` = *<value>*, maximum number of steps =
            *<value>*.

        (`errno` 16)
            Too much accuracy requested for precision of machine. `rtol`
            and `atol` were increased by scale factor R. Try running
            again with these scaled tolerances. `t` = *<value>*, R =
            *<value>*.

        (`errno` 17)
            A solution component has become zero when a purely relative
            tolerance (zero absolute tolerance) was selected for that
            component. `t` = *<value>*, `y`[I-1] = *<value>* for
            component I = *<value>*.

        (`errno` 18)
            The error test failed repeatedly with abs(h) = hmin. `t` =
            *<value>*. Stepsize h = *<value>*.

        (`errno` 19)
            The corrector repeatedly failed to converge with abs(h) =
            hmin. `t` = *<value>*. Stepsize h = *<value>*.

        (`errno` 20)
            The iteration matrix is singular. `t` = *<value>*. Stepsize
            h = *<value>*.

        (`errno` 21)
            The corrector could not converge and the error test failed
            repeatedly. `t` = *<value>*. Stepsize h = *<value>*.

        (`errno` 22)
            `ires` was set to -1 during a call to `res` and could not be
            resolved. `t` = *<value>*. Stepsize h = *<value>*.

        (`errno` 23)
            `ires` was set to -2 during a call to `res`. `t` =
            *<value>*. Stepsize = *<value>*.

        (`errno` 24)
            The initial `ydot` could not be computed. `t` = *<value>*.
            Stepsize h = *<value>*.

        (`errno` 25)
            Repeated occurrences of input constraint violations have
            been detected. This could result in a potential infinite
            loop: `itask` = *<value>*. Current violation corresponds to
            exit with errno = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.dae_dassl_gen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_res_name = fun_name + ':res'

    resh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, r[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_resh(_res, en_data, neq, t, y, ydot, r, ires, _iuser, _ruser):
        """Helper for res."""
        try:
            _neq_resh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_resh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_resh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_resh,),
            )
            _ydot_resh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_resh,),
            )
            _r_resh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_resh,),
                can_reference_contents=False,
            )
            _ires_resh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_resh = res(
                    _neq_resh,
                    _t_resh,
                    _y_resh,
                    _ydot_resh,
                    _r_resh,
                    _ires_resh,
                    data,
                )
            else:
                _ires_resh = res(
                    _neq_resh,
                    _t_resh,
                    _y_resh,
                    _ydot_resh,
                    _r_resh,
                    _ires_resh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_resh, {'fun_name': cb_res_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_resh, {'fun_name': cb_res_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, pd[lenpd].
        utils._BYREF_INT, # Input, lenpd.
        utils._BYREF_FLOAT64, # Input, cj.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, ydot, pd, lenpd, cj, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _ydot_jach = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_jach,),
            )
            _lenpd_jach = utils._EngineIntScalarType.to_py(lenpd.contents)
            _pd_jach = utils._EngineFloat64ArrayType.from_pointer(
                pd,
                (_lenpd_jach,),
            )
            _cj_jach = utils._EngineFloat64ScalarType.to_py(cj.contents)
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _pd_jach,
                    _lenpd_jach,
                    _cj_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _pd_jach,
                    _lenpd_jach,
                    _cj_jach,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _pd_jach, {'fun_name': cb_jac_name, 'returned': 'pd'}, pd,
            )
            utils._unlink_pointer(pd)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02neft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, atol[*].
        utils._BYREF_INT, # Input/Output, itask.
        utils._UNUSED_FUNCTYPE,
        resh,
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineIntArrayType.c_type, # Input/Output, icom[50+neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, com[lcom].
        utils._BYREF_INT, # Input, lcom.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icom', 'com'),
    )
    _icom = comm['icom']
    if utils._EngineIntArrayType.get_element(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, (1,), validate=False) == 1:
        _rtol_dim1 = _neq_py
    elif utils._EngineIntArrayType.get_element(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, (1,), validate=False) == 0:
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = 0
    exp_shape_rtol = (_rtol_dim1,)
    _rtol_cls = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    )
    _rtol = _rtol_cls.to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, (1,), validate=False) == 1:
        _atol_dim1 = _neq_py
    elif utils._EngineIntArrayType.get_element(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, (1,), validate=False) == 0:
        _atol_dim1 = 1
    else:
        _atol_dim1 = 0
    exp_shape_atol = (_atol_dim1,)
    _atol_cls = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    )
    _atol = _atol_cls.to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    utils._check_callable(res, {'fun_name': fun_name, 'entity': 'res'})
    _res = utils._UNUSED_CALLBACK
    _resh = utils._cast_callback(
        resh, cb_resh,
        {'fun_name': fun_name, 'entity': 'res'},
    )
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    _lcom_py = utils._EngineFloat64ArrayType.get_shape(comm['com'], {'fun_name': fun_name, 'entity': 'comm[\'com\']'}, exp_rank=1, validate=False)[0]
    _lcom = utils._EngineIntScalarType(
        _lcom_py, {'fun_name': fun_name, 'entity': '_lcom'},
    ).to_ctypes()
    _com = comm['com']
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _t,
        _tout,
        _y,
        _ydot,
        _rtol,
        _atol,
        _itask,
        _res,
        _resh,
        _jac,
        _jach,
        _icom,
        _com,
        _lcom,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tout != t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1031:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tout is too close to t to start integration:',
                    'tout-t = {:s}: hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1032:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tout is behind t in the direction of h: tout-t = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1060:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Some element of rtol is less than zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 1070:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Some element of atol is less than zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 1080:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A previous call to this function returned with',
                    'itask = {:s} and no appropriate action was taken.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 2010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior to the',
                    'first call of this function or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 2010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior to the',
                    'first call of this function or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1130:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'comm[\'com\'] array is of insufficient length;',
                    'length required = {:s};'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'actual length lcom = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 1061:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All elements of rtol and atol are zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 2150:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Maximum number of steps taken on this call before reaching tout:',
                    't = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 2160:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of machine.',
                    'rtol and atol were increased by scale factor R.',
                    'Try running again with these scaled tolerances.',
                    't = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'R = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 2170:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A solution component has become zero when a purely relative tolerance (zero',
                    'absolute tolerance) was selected for that component.',
                    't = {:s}, y[I-1] = {:s} for component'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'I = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 2180:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The error test failed repeatedly with abs(h) = hmin.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 2190:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The corrector repeatedly failed to converge with abs(h) = hmin.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 2200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration matrix is singular.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 2210:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The corrector could not converge and the error test failed repeatedly.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 2220:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires was set to -1 during a call to res and could not be resolved.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 2230:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires was set to -2 during a call to res.',
                    't = {:s}. Stepsize = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 24 and _ifmt == 2240:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial ydot could not be computed.',
                    't = {:s}. Stepsize h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 25 and _ifmt == 2250:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Repeated occurrences of input constraint violations have been detected.',
                    'This could result in a potential infinite loop:',
                    'itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Current violation corresponds to exit with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _rtol_cls.refresh_py(_rtol)
        _atol_cls.refresh_py(_atol)
        _itask_py = utils._EngineIntScalarType.to_py(_itask)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'itask'],
        )(
            _t_py,
            _itask_py,
        )
    _err_class.manage()
    return return_data

def ivp_stiff_imp_fulljac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, resid, sdysav, jac, monitr, lderiv, itask, itrace, data=None, io_manager=None):
    u"""
    Implicit/algebraic ordinary differential equations, stiff initial
    value problem, full Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02ng

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ngf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of equations to be solved during
        the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On entry: if `lderiv`[0] = True, `ydot` must contain
        approximations to the time derivatives y' of the vector y.

        If `lderiv`[0] = False, `ydot` need not be set on entry.

        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_fulljac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_imp_fulljac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    resid : callable ires = resid(neq, t, y, ydot, r, ires, data=None)
        `resid` must evaluate the residual

            r = g(t,y)-A(t,y)y'

        in one case and

            r = -A(t,y)y'

        in another.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        ydot : float, ndarray, shape (`neq`)
            The value of y'_i, for i = 1,2,...,`neq`, at t.

        r : float, ndarray, shape (`neq`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of r, for i
            = 1,2,...,`neq`, where

                r = g(t,y)-A(t,y)y'

            or

                r = -A(t,y)y'

            and where the definition of r is determined by the input
            value of `ires`.

        ires : int
            The form of the residual that must be returned in array `r`.

            `ires` = -1

              The residual defined in equation (2) must be returned.

            `ires` = 1

              The residual defined in equation (1) must be returned.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should be unchanged unless one of the following actions is
            required of the integrator, in which case `ires` should be
            set accordingly.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible, the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specifications of the integrator setup functions
        ``ivp_stiff_dassl``, ``ivp_stiff_bdf`` and ``ivp_stiff_blend``.
        This value must be the same as that supplied to the integrator
        setup function.

    jac : None or callable jac(neq, t, y, ydot, h, d, p, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the full linear algebra setup
        function ``ivp_stiff_fulljac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        ydot : float, ndarray, shape (`neq`)
            The derivative of the solution at the current point t.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        p : float, ndarray, shape (`neq`, `neq`), to be modified in
        place
            On entry: is set to zero.

            On exit: `p`[i-1,j-1] must contain dr_i/dy_j, for j =
            1,2,...,`neq`, for i = 1,2,...,`neq`.

            Only the nonzero elements of this array need be set, since
            it is preset to zero before the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        ldysav : int
            An upper bound on the number of equations to be solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_imp_fulljac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector A(t,y)y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in
              `resid`) caused an early termination (this facility could
              be used to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_imp_fulljac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    lderiv : bool, shape (2), modified in place
        On entry: `lderiv`[0] must be set to True if you have supplied
        both an initial y and an initial y'. `lderiv`[0] must be set to
        False if only the initial y has been supplied.

        `lderiv`[1] must be set to True if the integrator is to use a
        modified Newton method to evaluate the initial y and y'.

        Note that y and y', if supplied, are used as initial estimates.

        This method involves taking a small step at the start of the
        integration, and if `itask` = 6 on entry, `t` and `tout` will be
        set to the result of taking this small step. `lderiv`[1] must be
        set to False if the integrator is to use functional iteration to
        evaluate the initial y and y', and if this fails a modified
        Newton method will then be attempted. `lderiv`[1] = True is
        recommended if there are implicit equations or the initial y and
        y' are zero.

        On exit: `lderiv`[0] is normally unchanged. However if `itask` =
        6 and internal initialization was successful then `lderiv`[0] =
        True.

        `lderiv`[1] = True, if implicit equations were detected.

        Otherwise `lderiv`[1] = False.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit may be equal
          to or beyond `tout`, but not before it, in the direction of
          integration.

        `itask` = 5

          Take one step only and return, without passing tcrit. tcrit
          must be specified as under `itask` = 4.

        `itask` = 6

          The integrator will solve for the initial values of y and y'
          only and then return to the calling (sub)program without doing
          the integration. This option can be used to check the initial
          values of y and y'. Functional iteration or a 'small' backward
          Euler method used in conjunction with a damped Newton
          iteration is used to calculate these values (see `lderiv`).
          Note that if a backward Euler step is used then the value of t
          will have been advanced a short distance from the initial
          point.

        Note: if ``ivp_stiff_imp_fulljac`` is recalled with a different
        value of `itask` (and `tout` altered), the initialization
        procedure is repeated, possibly leading to different initial
        conditions.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    tout : float
        Normally unchanged. However, when `itask` = 6, `tout` contains
        the value of `t` at which initial values have been computed
        without performing any integration. See descriptions of `itask`
        and `lderiv`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ires` = *<value>* and dy/dt = 0.0 for all
            elements.

            Check the evaluation of the residual for this value of
            `ires`.

        (`errno` 1)
            `ires` was set to an illegal value during initialization.

            `ires` = *<value>* at time *<value>*.

        (`errno` 1)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. `hmin` =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. `hmax` =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 7)
            `resid` set `ires` = 3, which signals that an error
            condition has occurred in the solution vector, its time
            derivative or in the value of t. It was not possible to
            remove this condition.

            `ires` = *<value>* at t = *<value>*.

        (`errno` 8)
            Attempt was made to reduce the step size to a value less
            than the minimum step size during the calculation of initial
            values.

            Minimum stepsize: *<value>*.

        (`errno` 8)
            The user problem has one or more inconsistencies between the
            `ires` = 1 and `ires` = -1 parts. Integration will not be
            attempted.

        (`errno` 8)
            The residual function returned an error when calculating the
            initial values of the solution and its time derivative.

        (`errno` 8)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough integer store provided for internal storage
            pointers.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            Either the full matrix linear algebra setup function was not
            called first or a communication array has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            Nonlinear solver failed to converge using a damped Newton
            method to solve for initial values.

            Damping factor: *<value>*; convergence rate: *<value>*.

        (`errno` 11)
            `resid` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_imp_fulljac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_resid_name = fun_name + ':resid'

    residh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, r[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_residh(_resid, en_data, neq, t, y, ydot, r, ires, _iuser, _ruser):
        """Helper for resid."""
        try:
            _neq_residh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_residh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_residh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_residh,),
            )
            _ydot_residh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_residh,),
            )
            _r_residh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_residh,),
                can_reference_contents=False,
            )
            _ires_residh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                    data,
                )
            else:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_residh, {'fun_name': cb_resid_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_residh, {'fun_name': cb_resid_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[neq, neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, ydot, h, d, p, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _ydot_jach = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _p_jach = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_neq_jach, _neq_jach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _p_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _p_jach,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _p_jach, {'fun_name': cb_jac_name, 'returned': 'p'}, p,
            )
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02ngft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input/Output, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        residh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._EngineLogicalArrayType.c_type, # Input/Output, lderiv[2].
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(resid, {'fun_name': fun_name, 'entity': 'resid'})
    _resid = utils._UNUSED_CALLBACK
    _residh = utils._cast_callback(
        residh, cb_residh,
        {'fun_name': fun_name, 'entity': 'resid'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _lderiv_cls = utils._EngineLogicalArrayType(
        lderiv, {'fun_name': fun_name, 'entity': 'lderiv'},
        exp_shape=(2,),
    )
    _lderiv = _lderiv_cls.to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _resid,
        _residh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _monitr,
        _monitrh,
        _lderiv,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99937:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ires = {:s} and dy/dt = 0.0 for all elements.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Check the evaluation of the residual for this value of ires.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99939:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires was set to an illegal value during initialization.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and',
                    'tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99931:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99931:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99940:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 3, which signals that an error condition',
                    'has occurred in the solution vector, its time derivative or in the value',
                    'of t. It was not possible to remove this condition.',
                    'ires = {:s} at t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonlinear solver failed to converge using a damped Newton',
                    'method to solve for initial values.',
                    'Damping factor: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'convergence rate: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99932:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempt was made to reduce the step size to a value less',
                    'than the minimum step size during the calculation of initial values.',
                    'Minimum stepsize: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99933:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The user problem has one or more inconsistencies between',
                    'the ires = 1 and ires = -1 parts. Integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99934:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual function returned an error when calculating',
                    'the initial values of the solution and its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99935:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99941:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check the problem',
                    'formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for internal storage pointers.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 2, which signals that the integration',
                    'should terminate. ires = {:s} at time'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99930:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the full matrix linear algebra setup function was not called',
                    'first or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _tout_py = utils._EngineFloat64ScalarType.to_py(_tout)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _lderiv_cls.refresh_py(_lderiv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'tout'],
        )(
            _t_py,
            _tout_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_imp_bandjac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, resid, sdysav, jac, monitr, lderiv, itask, itrace, data=None, io_manager=None):
    u"""
    Implicit/algebraic ordinary differential equations, stiff initial
    value problem, banded Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02nh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nhf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of equations to be solved during
        the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On entry: if `lderiv`[0] = True, `ydot` must contain
        approximations to the time derivatives y' of the vector y.

        If `lderiv`[0] = False, `ydot` need not be set on entry.

        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_bandjac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_imp_bandjac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    resid : callable ires = resid(neq, t, y, ydot, r, ires, data=None)
        `resid` must evaluate the residual

            r = g(t,y)-A(t,y)y'

        in one case and

            r = -A(t,y)y'

        in another.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        ydot : float, ndarray, shape (`neq`)
            The value of y'_i, for i = 1,2,...,`neq`, at t.

        r : float, ndarray, shape (`neq`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of r, for i
            = 1,2,...,`neq`, where

                r = g(t,y)-A(t,y)y'

            or

                r = -A(t,y)y'

            and where the definition of r is determined by the input
            value of `ires`.

        ires : int
            The form of the residual that must be returned in array `r`.

            `ires` = -1

              The residual defined in equation (2) must be returned.

            `ires` = 1

              The residual defined in equation (1) must be returned.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should be unchanged unless one of the following actions is
            required of the integrator, in which case `ires` should be
            set accordingly.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible, the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specifications of the integrator setup functions
        ``ivp_stiff_dassl``, ``ivp_stiff_bdf`` and ``ivp_stiff_blend``.
        This value must be the same as that supplied to the integrator
        setup function.

    jac : None or callable jac(neq, t, y, ydot, h, d, ml, mu, p,
        data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the banded linear algebra setup
        function ``ivp_stiff_bandjac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        ydot : float, ndarray, shape (`neq`)
            The derivative of the solution at the current point t.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        ml : int
            The number of subdiagonals and superdiagonals respectively
            in the band.

        mu : int
            The number of subdiagonals and superdiagonals respectively
            in the band.

        p : float, ndarray, shape (`ml`+`mu`+1, `neq`), to be modified
        in place
            On entry: is set to zero.

            On exit: elements of the Jacobian matrix dr/dy stored as
            specified by the following pseudocode

            ::

              for i in range(1, neq+1):
                j1 = max(i-ml, 1)
                j2 = min(i+mu, neq)
                for j in range(j1, j2+1):
                  k = min(ml+1-i, 0) + j
                  p[k-1, i-1] = J[i-1, j-1]

            See also the function document for ``lapacklin.dgbtrf``.

            Only nonzero elements of this array need be set, since it is
            preset to zero before the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        ldysav : int
            An upper bound on the number of equations to be solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_imp_bandjac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector A(t,y)y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in
              `resid`) caused an early termination (this facility could
              be used to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_imp_bandjac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    lderiv : bool, shape (2), modified in place
        On entry: `lderiv`[0] must be set to True if you have supplied
        both an initial y and an initial y'. `lderiv`[0] must be set to
        False if only the initial y has been supplied.

        `lderiv`[1] must be set to True if the integrator is to use a
        modified Newton method to evaluate the initial y and y'.

        Note that y and y', if supplied, are used as initial estimates.

        This method involves taking a small step at the start of the
        integration, and if `itask` = 6 on entry, `t` and `tout` will be
        set to the result of taking this small step. `lderiv`[1] must be
        set to False if the integrator is to use functional iteration to
        evaluate the initial y and y', and if this fails a modified
        Newton method will then be attempted. `lderiv`[1] = True is
        recommended if there are implicit equations or the initial y and
        y' are zero.

        On exit: `lderiv`[0] is normally unchanged. However if `itask` =
        6 and internal initialization was successful then `lderiv`[0] =
        True.

        `lderiv`[1] = True, if implicit equations were detected.

        Otherwise `lderiv`[1] = False.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit may be equal
          to or beyond `tout`, but not before it, in the direction of
          integration.

        `itask` = 5

          Take one step only and return, without passing tcrit. tcrit
          must be specified as under `itask` = 4.

        `itask` = 6

          The integrator will solve for the initial values of y and y'
          only and then return to the calling (sub)program without doing
          the integration. This option can be used to check the initial
          values of y and y'. Functional iteration or a 'small' backward
          Euler method used in conjunction with a damped Newton
          iteration is used to calculate these values (see `lderiv`).
          Note that if a backward Euler step is used then the value of t
          will have been advanced a short distance from the initial
          point.

        Note: if ``ivp_stiff_imp_bandjac`` is recalled with a different
        value of `itask` (and `tout` altered), the initialization
        procedure is repeated, possibly leading to different initial
        conditions.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    tout : float
        Normally unchanged. However, when `itask` = 6, `tout` contains
        the value of `t` at which initial values have been computed
        without performing any integration. See descriptions of `itask`
        and `lderiv`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ires` = *<value>* and dy/dt = 0.0 for all
            elements.

            Check the evaluation of the residual for this value of
            `ires`.

        (`errno` 1)
            `ires` was set to an illegal value during initialization.

            `ires` = *<value>* at time *<value>*.

        (`errno` 1)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. `hmin` =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. `hmax` =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 7)
            `resid` set `ires` = 3, which signals that an error
            condition has occurred in the solution vector, its time
            derivative or in the value of t. It was not possible to
            remove this condition.

            `ires` = *<value>* at t = *<value>*.

        (`errno` 8)
            Attempt was made to reduce the step size to a value less
            than the minimum step size during the calculation of initial
            values.

            Minimum stepsize: *<value>*.

        (`errno` 8)
            The residual function returned an error when calculating the
            initial values of the solution and its time derivative.

        (`errno` 8)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough integer store provided for internal storage
            pointers.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            Either the banded matrix linear algebra setup function was
            not called first or a communication array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 8)
            Nonlinear solver failed to converge using a damped Newton
            method to solve for initial values.

            Damping factor: *<value>*; convergence rate: *<value>*.

        (`errno` 8)
            The user problem has one or more inconsistencies between the
            `ires` = 1 and `ires` = -1 parts. Integration will not be
            attempted.

        (`errno` 11)
            `resid` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_imp_bandjac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_resid_name = fun_name + ':resid'

    residh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, r[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_residh(_resid, en_data, neq, t, y, ydot, r, ires, _iuser, _ruser):
        """Helper for resid."""
        try:
            _neq_residh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_residh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_residh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_residh,),
            )
            _ydot_residh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_residh,),
            )
            _r_residh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_residh,),
                can_reference_contents=False,
            )
            _ires_residh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                    data,
                )
            else:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_residh, {'fun_name': cb_resid_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_residh, {'fun_name': cb_resid_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[ml+mu+1, neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, ydot, h, d, ml, mu, p, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _ydot_jach = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _ml_jach = utils._EngineIntScalarType.to_py(ml.contents)
            _mu_jach = utils._EngineIntScalarType.to_py(mu.contents)
            _p_jach = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_ml_jach+_mu_jach+1, _neq_jach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _ml_jach,
                    _mu_jach,
                    _p_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _ml_jach,
                    _mu_jach,
                    _p_jach,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _p_jach, {'fun_name': cb_jac_name, 'returned': 'p'}, p,
            )
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02nhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input/Output, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        residh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._EngineLogicalArrayType.c_type, # Input/Output, lderiv[2].
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'jacpvt', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(resid, {'fun_name': fun_name, 'entity': 'resid'})
    _resid = utils._UNUSED_CALLBACK
    _residh = utils._cast_callback(
        residh, cb_residh,
        {'fun_name': fun_name, 'entity': 'resid'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    if _comm_needs_init:
        _jacpvt = utils._EngineIntArrayType.empty_ctypes(_neq_py)
        comm['jacpvt'] = _jacpvt
    else:
        _jacpvt = comm['jacpvt']
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _lderiv_cls = utils._EngineLogicalArrayType(
        lderiv, {'fun_name': fun_name, 'entity': 'lderiv'},
        exp_shape=(2,),
    )
    _lderiv = _lderiv_cls.to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _resid,
        _residh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _monitr,
        _monitrh,
        _lderiv,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99937:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ires = {:s} and dy/dt = 0.0 for all elements.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Check the evaluation of the residual for this value of ires.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99939:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires was set to an illegal value during initialization.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and',
                    'tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99940:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 3, which signals that an error condition',
                    'has occurred in the solution vector, its time derivative or in the value',
                    'of t. It was not possible to remove this condition.',
                    'ires = {:s} at t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonlinear solver failed to converge using a damped Newton',
                    'method to solve for initial values.',
                    'Damping factor: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'convergence rate: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99932:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempt was made to reduce the step size to a value less',
                    'than the minimum step size during the calculation of initial values.',
                    'Minimum stepsize: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99933:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The user problem has one or more inconsistencies between',
                    'the ires = 1 and ires = -1 parts. Integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99934:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual function returned an error when calculating',
                    'the initial values of the solution and its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99935:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99941:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for internal storage pointers.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 2, which signals that the integration',
                    'should terminate. ires = {:s} at time'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99930:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the banded matrix linear algebra setup function was not called',
                    'first or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _tout_py = utils._EngineFloat64ScalarType.to_py(_tout)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _lderiv_cls.refresh_py(_lderiv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'tout'],
        )(
            _t_py,
            _tout_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_imp_sparjac(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, resid, sdysav, jac, monitr, lderiv, itask, itrace, data=None, io_manager=None):
    u"""
    Implicit/algebraic ordinary differential equations, stiff initial
    value problem, sparse Jacobian (comprehensive).

    For full information please refer to the NAG Library document for
    d02nj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02njf.html

    Parameters
    ----------
    neq : int
        The number of differential equations to be solved.

    ldysav : int
        A bound on the maximum number of equations to be solved during
        the integration.

    t : float
        t, the value of the independent variable. The input value of `t`
        is used only on the first call as the initial point of the
        integration.

    tout : float
        The next value of t at which a computed solution is desired. For
        the initial t, the input value of `tout` is used to determine
        the direction of integration. Integration is permitted in either
        direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On entry: the values of the dependent variables (solution). On
        the first call the first `neq` elements of `y` must contain the
        vector of initial values.

        On exit: the computed solution vector, evaluated at `t` (usually
        `t` = `tout`).

    ydot : float, shape (`neq`), modified in place
        On entry: if `lderiv`[0] = True, `ydot` must contain
        approximations to the time derivatives y' of the vector y.

        If `lderiv`[0] = False, `ydot` need not be set on entry.

        On exit: the time derivatives y' of the vector y at the last
        integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_sparjac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        The relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        The absolute local error tolerance.

    itol : int
        A value to indicate the form of the local error test. `itol`
        indicates to ``ivp_stiff_imp_sparjac`` whether to interpret
        either or both of `rtol` or `atol` as a vector or a scalar. The
        error test to be satisfied is ||e_i/w_i|| < 1.0, where w_i is
        defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    resid : callable ires = resid(neq, t, y, ydot, r, ires, data=None)
        `resid` must evaluate the residual

            r = g(t,y)-A(t,y)y'

        in one case and

            r = -A(t,y)y'

        in another.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            The value of y_i, for i = 1,2,...,`neq`.

        ydot : float, ndarray, shape (`neq`)
            The value of y'_i, for i = 1,2,...,`neq`, at t.

        r : float, ndarray, shape (`neq`), to be modified in place
            On exit: `r`[i-1] must contain the ith component of r, for i
            = 1,2,...,`neq`, where

                r = g(t,y)-A(t,y)y'

            or

                r = -A(t,y)y'

            and where the definition of r is determined by the input
            value of `ires`.

        ires : int
            The form of the residual that must be returned in array `r`.

            `ires` = -1

              The residual defined in equation (2) must be returned.

            `ires` = 1

              The residual defined in equation (1) must be returned.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        ires : int
            Should be unchanged unless one of the following actions is
            required of the integrator, in which case `ires` should be
            set accordingly.

            `ires` = 2

              Indicates to the integrator that control should be passed
              back immediately to the calling (sub)program with the
              error indicator set to `errno` = 11.

            `ires` = 3

              Indicates to the integrator that an error condition has
              occurred in the solution vector, its time derivative or in
              the value of t. The integrator will use a smaller time
              step to try to avoid this condition. If this is not
              possible, the integrator returns to the calling
              (sub)program with the error indicator set to `errno` = 7.

            `ires` = 4

              Indicates to the integrator to stop its current operation
              and to enter `monitr` immediately with argument `imon` =
              -2.

    sdysav : int
        An appropriate value for `sdysav` is described in the
        specifications of the integrator setup functions
        ``ivp_stiff_dassl``, ``ivp_stiff_bdf`` and ``ivp_stiff_blend``.
        This value must be the same as that supplied to the integrator
        setup function.

    jac : None or callable jac(neq, t, y, ydot, h, d, j, pdj, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jac` must evaluate the Jacobian of the system.

        If this option is not required, the actual argument for `jac`
        must be None. You must indicate to the integrator whether this
        option is to be used by setting the argument jceval
        appropriately in a call to the sparse linear algebra setup
        function ``ivp_stiff_sparjac_setup``.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        t : float
            t, the current value of the independent variable.

        y : float, ndarray, shape (`neq`)
            y_i, for i = 1,2,...,`neq`, the current solution component.

        ydot : float, ndarray, shape (`neq`)
            The derivative of the solution at the current point t.

        h : float
            The current step size.

        d : float
            The argument d which depends on the integration method.

        j : int
            The column of the Jacobian that `jac` must return in the
            array `pdj`.

        pdj : float, ndarray, shape (`neq`), to be modified in place
            On entry: is set to zero.

            On exit: `pdj`[i-1] should be set to the (i,j)th element of
            the Jacobian, where j is given by `j`. Only nonzero elements
            of this array need be set, since it is preset to zero before
            the call to `jac`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monitr : None or callable (hnext, imon, inln, hmin, hmax) =
        monitr(neq, ldysav, sdysav, t, hlast, hnext, y, ydot, comm, r,
        acor, imon, hmin, hmax, nqu, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monitr` performs tasks requested by you.

        If this option is not required, the actual argument for `monitr`
        must be None.

        Parameters
        ~~~~~~~~~~
        neq : int
            The number of equations being solved.

        ldysav : int
            An upper bound on the number of equations to be solved.

        sdysav : int
            This argument is the same value as `sdysav` in the call of
            ``ivp_stiff_imp_sparjac``.

        t : float
            The current value of the independent variable.

        hlast : float
            The last step size successfully used by the integrator.

        hnext : float
            The step size that the integrator proposes to take on the
            next step.

        y : float, ndarray, shape (`neq`), to be modified in place
            On entry: y, the values of the dependent variables evaluated
            at t.

            On exit: these values must not be changed unless `imon` is
            set to 2.

        ydot : float, ndarray, shape (`neq`)
            The time derivatives y' of the vector y.

        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument may be passed as argument comm in a call to
            ``ivp_stiff_nat_interp`` or comm in a call to
            ``ivp_stiff_c1_interp`` to enable you to carry out
            interpolation using either of those functions.

        r : float, ndarray, shape (`neq`)
            If `imon` = 0 and `inln` = 3, the first `neq` elements
            contain the residual vector A(t,y)y'-g(t,y).

        acor : float, ndarray, shape (`neq`, 2)
            With `imon` = 1, `acor`[i-1,0] contains the weight used for
            the ith equation when the norm is evaluated, and
            `acor`[i-1,1] contains the estimated local error for the ith
            equation. The scaled local error at the end of a timestep
            may be obtained by calling ``ivp_stiff_errest``.

        imon : int
            A flag indicating under what circumstances `monitr` was
            called:

            `imon` = -2

              Entry from the integrator after `ires` = 4 (set in
              `resid`) caused an early termination (this facility could
              be used to locate discontinuities).

            `imon` = -1

              The current step failed repeatedly.

            `imon` = 0

              Entry after a call to the internal nonlinear equation
              solver (see `inln`).

            `imon` = 1

              The current step was successful.

        hmin : float
            The minimum step size to be taken on the next step.

        hmax : float
            The maximum step size to be taken on the next step.

        nqu : int
            The order of the integrator used on the last step. This is
            supplied to enable you to carry out interpolation using
            either of the functions ``ivp_stiff_nat_interp`` or
            ``ivp_stiff_c1_interp``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        hnext : float
            The next step size to be used. If this is different from the
            input value, `imon` must be set to 4.

        imon : int
            May be reset to determine subsequent action in
            ``ivp_stiff_imp_sparjac``.

            `imon` = -2

              Integration is to be halted. A return will be made from
              the integrator to the calling (sub)program with `errno` =
              12.

            `imon` = -1

              Allow the integrator to continue with its own internal
              strategy. The integrator will try up to three restarts
              unless `imon` != -1 on exit.

            `imon` = 0

              Return to the internal nonlinear equation solver, where
              the action taken is determined by the value of `inln` (see
              `inln`).

            `imon` = 1

              Normal exit to the integrator to continue integration.

            `imon` = 2

              Restart the integration at the current time point. The
              integrator will restart from order 1 when this option is
              used. The solution `y`, provided by `monitr`, will be used
              for the initial conditions.

            `imon` = 3

              Try to continue with the same step size and order as was
              to be used before the call to `monitr`. `hmin` and `hmax`
              may be altered if desired.

            `imon` = 4

              Continue the integration but using a new value of `hnext`
              and possibly new values of `hmin` and `hmax`.

        inln : int
            The action to be taken by the internal nonlinear equation
            solver when `monitr` is exited with `imon` = 0. By setting
            `inln` = 3 and returning to the integrator, the residual
            vector is evaluated and placed in the array `r`, and then
            `monitr` is called again. At present this is the only option
            available: `inln` must not be set to any other value.

        hmin : float
            The minimum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4.

        hmax : float
            The maximum step size to be used. If this is different from
            the input value, `imon` must be set to 3 or 4. If `hmax` is
            set to zero, no limit is assumed.

    lderiv : bool, shape (2), modified in place
        On entry: `lderiv`[0] must be set to True if you have supplied
        both an initial y and an initial y'. `lderiv`[0] must be set to
        False if only the initial y has been supplied.

        `lderiv`[1] must be set to True if the integrator is to use a
        modified Newton method to evaluate the initial y and y'.

        Note that y and y', if supplied, are used as initial estimates.

        This method involves taking a small step at the start of the
        integration, and if `itask` = 6 on entry, `t` and `tout` will be
        set to the result of taking this small step. `lderiv`[1] must be
        set to False if the integrator is to use functional iteration to
        evaluate the initial y and y', and if this fails a modified
        Newton method will then be attempted. `lderiv`[1] = True is
        recommended if there are implicit equations or the initial y and
        y' are zero.

        On exit: `lderiv`[0] is normally unchanged. However if `itask` =
        6 and internal initialization was successful then `lderiv`[0] =
        True.

        `lderiv`[1] = True, if implicit equations were detected.

        Otherwise `lderiv`[1] = False.

    itask : int
        The task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit may be equal
          to or beyond `tout`, but not before it, in the direction of
          integration.

        `itask` = 5

          Take one step only and return, without passing tcrit. tcrit
          must be specified as under `itask` = 4.

        `itask` = 6

          The integrator will solve for the initial values of y and y'
          only and then return to the calling (sub)program without doing
          the integration. This option can be used to check the initial
          values of y and y'. Functional iteration or a 'small' backward
          Euler method used in conjunction with a damped Newton
          iteration is used to calculate these values (see `lderiv`).
          Note that if a backward Euler step is used then the value of t
          will have been advanced a short distance from the initial
          point.

        Note: if ``ivp_stiff_imp_sparjac`` is recalled with a different
        value of `itask` (and `tout` altered), the initialization
        procedure is repeated, possibly leading to different initial
        conditions.

    itrace : int
        The level of output that is printed by the integrator. `itrace`
        may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        The value at which the computed solution y is returned (usually
        at `tout`).

    tout : float
        Normally unchanged. However, when `itask` = 6, `tout` contains
        the value of `t` at which initial values have been computed
        without performing any integration. See descriptions of `itask`
        and `lderiv`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ires` = *<value>* and dy/dt = 0.0 for all
            elements.

            Check the evaluation of the residual for this value of
            `ires`.

        (`errno` 1)
            `ires` was set to an illegal value during initialization.

            `ires` = *<value>* at time *<value>*.

        (`errno` 1)
            `monitr` set `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, h = *<value>*, is too small.

        (`errno` 1)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. hmin =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. hmax =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            `monitr` set `inln` = *<value>*.

            Constraint: `inln` = 3.

        (`errno` 1)
            `monitr` appears to have overwritten the solution vector.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

            See ``ivp_stiff_dassl`` for details of maxstp.

        (`errno` 7)
            `resid` set `ires` = 3, which signals that an error
            condition has occurred in the solution vector, its time
            derivative or in the value of t. It was not possible to
            remove this condition.

            `ires` = *<value>* at t = *<value>*.

        (`errno` 8)
            Attempt was made to reduce the step size to a value less
            than the minimum step size during the calculation of initial
            values.

            Minimum stepsize: *<value>*.

        (`errno` 8)
            The residual function returned an error when calculating the
            initial values of the solution and its time derivative.

        (`errno` 8)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough integer store provided for internal storage
            pointers.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

        (`errno` 15)
            Either the sparse matrix linear algebra setup function was
            not called first or a communication array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 8)
            Nonlinear solver failed to converge using a damped Newton
            method to solve for initial values.

            Damping factor: *<value>*; convergence rate: *<value>*.

        (`errno` 8)
            The user problem has one or more inconsistencies between the
            `ires` = 1 and `ires` = -1 parts. Integration will not be
            attempted.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 11)
            `resid` set `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_imp_sparjac'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_resid_name = fun_name + ':resid'

    residh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Output, r[neq].
        utils._BYREF_INT, # Input/Output, ires.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_residh(_resid, en_data, neq, t, y, ydot, r, ires, _iuser, _ruser):
        """Helper for resid."""
        try:
            _neq_residh = utils._EngineIntScalarType.to_py(neq.contents)
            _t_residh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_residh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_residh,),
            )
            _ydot_residh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_residh,),
            )
            _r_residh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_residh,),
                can_reference_contents=False,
            )
            _ires_residh = utils._EngineIntScalarType.to_py(ires.contents)
            if data is not None:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                    data,
                )
            else:
                _ires_residh = resid(
                    _neq_residh,
                    _t_residh,
                    _y_residh,
                    _ydot_residh,
                    _r_residh,
                    _ires_residh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _r_residh, {'fun_name': cb_resid_name, 'returned': 'r'}, r,
            )
            utils._unlink_pointer(r)
            utils._EngineIntScalarType(
                _ires_residh, {'fun_name': cb_resid_name, 'returned': 'ires'},
            ).refresh_ctypes_buffer(ires)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jac_name = fun_name + ':jac'

    jach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, t.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_INT, # Input, j.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, pdj[neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jach(_jac, en_data, neq, t, y, ydot, h, d, j, pdj, _iuser, _ruser):
        """Helper for jac."""
        if jac is None:
            return
        try:
            _neq_jach = utils._EngineIntScalarType.to_py(neq.contents)
            _t_jach = utils._EngineFloat64ScalarType.to_py(t.contents)
            _y_jach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_jach,),
            )
            _ydot_jach = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_jach,),
            )
            _h_jach = utils._EngineFloat64ScalarType.to_py(h.contents)
            _d_jach = utils._EngineFloat64ScalarType.to_py(d.contents)
            _j_jach = utils._EngineIntScalarType.to_py(j.contents)
            _pdj_jach = utils._EngineFloat64ArrayType.from_pointer(
                pdj,
                (_neq_jach,),
            )
            if data is not None:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _j_jach,
                    _pdj_jach,
                    data,
                )
            else:
                jac(
                    _neq_jach,
                    _t_jach,
                    _y_jach,
                    _ydot_jach,
                    _h_jach,
                    _d_jach,
                    _j_jach,
                    _pdj_jach,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _pdj_jach, {'fun_name': cb_jac_name, 'returned': 'pdj'}, pdj,
            )
            utils._unlink_pointer(pdj)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_monitr_name = fun_name + ':monitr'

    monitrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, hlast.
        utils._BYREF_FLOAT64, # Input/Output, hnext.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._EngineFloat64ArrayType.c_type, # Input, r[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq, 2].
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Output, inln.
        utils._BYREF_FLOAT64, # Input/Output, hmin.
        utils._BYREF_FLOAT64, # Input/Output, hmax.
        utils._BYREF_INT, # Input, nqu.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
    )

    def cb_monitrh(_monitr, en_data, neq, ldysav, sdysav, t, hlast, hnext, y, ydot, ysav, r, acor, imon, inln, hmin, hmax, nqu, _iuser, _ruser, cwsav, lwsav, iwsav, rwsav, cwsav_chrlen):
        """Helper for monitr."""
        if monitr is None:
            return
        try:
            _neq_monitrh = utils._EngineIntScalarType.to_py(neq.contents)
            _ldysav_monitrh = utils._EngineIntScalarType.to_py(ldysav.contents)
            _sdysav_monitrh = utils._EngineIntScalarType.to_py(sdysav.contents)
            _t_monitrh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _hlast_monitrh = utils._EngineFloat64ScalarType.to_py(hlast.contents)
            _hnext_monitrh = utils._EngineFloat64ScalarType.to_py(hnext.contents)
            _y_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_monitrh,),
            )
            _ydot_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                ydot,
                (_neq_monitrh,),
            )
            _comm_monitrh = {'ysav': ysav, 'cwsav': cwsav, 'lwsav': lwsav, 'iwsav': iwsav, 'rwsav': rwsav}
            _r_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                r,
                (_neq_monitrh,),
            )
            _acor_monitrh = utils._EngineFloat64ArrayType.from_pointer(
                acor,
                (_neq_monitrh, 2),
                sorder=utils._EngineData.default_sorder,
            )
            _imon_monitrh = utils._EngineIntScalarType.to_py(imon.contents)
            _hmin_monitrh = utils._EngineFloat64ScalarType.to_py(hmin.contents)
            _hmax_monitrh = utils._EngineFloat64ScalarType.to_py(hmax.contents)
            _nqu_monitrh = utils._EngineIntScalarType.to_py(nqu.contents)
            if data is not None:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                    data,
                )
            else:
                _hnext_monitrh, _imon_monitrh, _inln_monitrh, _hmin_monitrh, _hmax_monitrh = monitr(
                    _neq_monitrh,
                    _ldysav_monitrh,
                    _sdysav_monitrh,
                    _t_monitrh,
                    _hlast_monitrh,
                    _hnext_monitrh,
                    _y_monitrh,
                    _ydot_monitrh,
                    _comm_monitrh,
                    _r_monitrh,
                    _acor_monitrh,
                    _imon_monitrh,
                    _hmin_monitrh,
                    _hmax_monitrh,
                    _nqu_monitrh,
                )
            utils._EngineFloat64ScalarType(
                _hnext_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hnext'},
            ).refresh_ctypes_buffer(hnext)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_monitrh, {'fun_name': cb_monitr_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._unlink_pointer(ydot)
            utils._unlink_pointer(r)
            utils._unlink_pointer(acor)
            utils._EngineIntScalarType(
                _imon_monitrh, {'fun_name': cb_monitr_name, 'returned': 'imon'},
            ).refresh_ctypes_buffer(imon)
            utils._EngineIntScalarType(
                _inln_monitrh, {'fun_name': cb_monitr_name, 'returned': 'inln'},
            ).refresh_ctypes_buffer(inln)
            utils._EngineFloat64ScalarType(
                _hmin_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmin'},
            ).refresh_ctypes_buffer(hmin)
            utils._EngineFloat64ScalarType(
                _hmax_monitrh, {'fun_name': cb_monitr_name, 'returned': 'hmax'},
            ).refresh_ctypes_buffer(hmax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02njft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input/Output, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._UNUSED_FUNCTYPE,
        residh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._UNUSED_FUNCTYPE,
        jach,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._UNUSED_FUNCTYPE,
        monitrh,
        utils._EngineLogicalArrayType.c_type, # Input/Output, lderiv[2].
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'wkjac', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'jacpvt', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    utils._check_callable(resid, {'fun_name': fun_name, 'entity': 'resid'})
    _resid = utils._UNUSED_CALLBACK
    _residh = utils._cast_callback(
        residh, cb_residh,
        {'fun_name': fun_name, 'entity': 'resid'},
    )
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    utils._check_callable(jac, {'fun_name': fun_name, 'entity': 'jac'}, allow_none=True)
    _jac = utils._UNUSED_CALLBACK
    _jach = utils._cast_callback(
        jach, cb_jach,
        {'fun_name': fun_name, 'entity': 'jac'},
    )
    if _comm_needs_init:
        _wkjac = utils._EngineFloat64ArrayType.empty_ctypes(int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (45,), validate=False)))
        comm['wkjac'] = _wkjac
    else:
        _wkjac = comm['wkjac']
    _nwkjac_py = utils._EngineFloat64ArrayType.get_shape(comm['wkjac'], {'fun_name': fun_name, 'entity': 'comm[\'wkjac\']'}, exp_rank=1, validate=False)[0]
    _nwkjac = utils._EngineIntScalarType(
        _nwkjac_py, {'fun_name': fun_name, 'entity': '_nwkjac'},
    ).to_ctypes()
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    _jacpvt = comm['jacpvt']
    utils._check_callable(monitr, {'fun_name': fun_name, 'entity': 'monitr'}, allow_none=True)
    _monitr = utils._UNUSED_CALLBACK
    _monitrh = utils._cast_callback(
        monitrh, cb_monitrh,
        {'fun_name': fun_name, 'entity': 'monitr'},
    )
    _lderiv_cls = utils._EngineLogicalArrayType(
        lderiv, {'fun_name': fun_name, 'entity': 'lderiv'},
        exp_shape=(2,),
    )
    _lderiv = _lderiv_cls.to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _iuser = None
    _ruser = None
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _resid,
        _residh,
        _ysav,
        _sdysav,
        _jac,
        _jach,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _monitr,
        _monitrh,
        _lderiv,
        _itask,
        _itrace,
        _nadv,
        _iuser,
        _ruser,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99937:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ires = {:s} and dy/dt = 0.0 for all elements.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Check the evaluation of the residual for this value of ires.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99939:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ires was set to an illegal value during initialization.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current',
                    'time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, h = {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and',
                    'tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr set inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'monitr appears to have overwritten the solution vector.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'See ivp_stiff_dassl for details of maxstp.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99940:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 3, which signals that an error condition',
                    'has occurred in the solution vector, its time derivative or in the value',
                    'of t. It was not possible to remove this condition.',
                    'ires = {:s} at t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonlinear solver failed to converge using a damped Newton',
                    'method to solve for initial values.',
                    'Damping factor: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'convergence rate: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99932:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempt was made to reduce the step size to a value less',
                    'than the minimum step size during the calculation of initial values.',
                    'Minimum stepsize: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99933:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The user problem has one or more inconsistencies between',
                    'the ires = 1 and ires = -1 parts. Integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99934:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual function returned an error when calculating',
                    'the initial values of the solution and its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99935:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99941:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for internal storage pointers.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'resid set ires = 2, which signals that the integration',
                    'should terminate. ires = {:s} at time'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99930:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the sparse matrix linear algebra setup function was not called',
                    'first or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _tout_py = utils._EngineFloat64ScalarType.to_py(_tout)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _lderiv_cls.refresh_py(_lderiv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'tout'],
        )(
            _t_py,
            _tout_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_exp_revcom(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, sdysav, wkjac, nwkjac, imon, inln, ires, irevcm, itask, itrace, io_manager=None):
    u"""
    Explicit ordinary differential equations, stiff initial value
    problem (reverse communication, comprehensive).

    For full information please refer to the NAG Library document for
    d02nm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nmf.html

    Parameters
    ----------
    neq : int
        The dimension of the arrays `y` and `ydot`.

        On initial entry: the number of differential equations to be
        solved.

    ldysav : int
        On initial entry: an upper bound on the maximum number of
        differential equations to be solved during the integration.

    t : float
        On initial entry: t, the value of the independent variable. The
        input value of `t` is used only on the first call as the initial
        point of the integration.

    tout : float
        On initial entry: the next value of t at which a computed
        solution is desired. For the initial t, the input value of
        `tout` is used to determine the direction of integration.
        Integration is permitted in either direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On initial entry: the values of the dependent variables
        (solution). On the first call the first `neq` elements of y must
        contain the vector of initial values.

        On final exit: the computed solution vector evaluated at `t`
        (usually t = `tout`).

    ydot : float, shape (`neq`), modified in place
        On intermediate entry: must be set to the derivatives as defined
        under the description of `irevcm`.

        On final exit: the time derivatives y' of the vector y at the
        last integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to one
        of ``ivp_stiff_fulljac_setup``, ``ivp_stiff_bandjac_setup`` or
        ``ivp_stiff_sparjac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        On initial entry: the relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        On initial entry: the absolute local error tolerance.

    itol : int
        On initial entry: a value to indicate the form of the local
        error test. `itol` indicates to ``ivp_stiff_exp_revcom`` whether
        to interpret either or both of `rtol` or `atol` as a vector or a
        scalar. The error test to be satisfied is ||e_i/w_i|| < 1.0,
        where w_i is defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    sdysav : int
        On initial entry: an appropriate value for `sdysav` is described
        in the specifications of the integrator setup functions
        ``ivp_stiff_bdf`` and ``ivp_stiff_blend``. This value must be
        the same as that supplied to the integrator setup function.

    wkjac : float, shape (`nwkjac`), modified in place
        On intermediate entry: elements of the Jacobian as defined under
        the description of `irevcm`. If a numerical Jacobian was
        requested then `wkjac` is used for workspace.

        On intermediate exit: the Jacobian is overwritten.

    nwkjac : int
        The dimension of the array `wkjac`.

        On initial entry: the actual size depends on the linear algebra
        method used. An appropriate value for `nwkjac` is described in
        the specifications of the linear algebra setup functions
        ``ivp_stiff_fulljac_setup``, ``ivp_stiff_bandjac_setup`` and
        ``ivp_stiff_sparjac_setup`` for full, banded and sparse matrix
        linear algebra respectively. This value must be the same as that
        supplied to the linear algebra setup function.

    imon : int
        On intermediate entry: may be reset to determine subsequent
        action in ``ivp_stiff_exp_revcom``.

        `imon` = -2

          Integration is to be halted. A return will be made from
          ``ivp_stiff_exp_revcom`` to the calling (sub)program with
          `errno` = 12.

        `imon` = -1

          Allow ``ivp_stiff_exp_revcom`` to continue with its own
          internal strategy. The integrator will try up to three
          restarts unless `imon` != -1.

        `imon` = 0

          Return to the internal nonlinear equation solver, where the
          action taken is determined by the value of `inln`.

        `imon` = 1

          Normal exit to ``ivp_stiff_exp_revcom`` to continue
          integration.

        `imon` = 2

          Restart the integration at the current time point. The
          integrator will restart from order 1 when this option is used.
          The solution `y`, provided by the `monitr` operation (see
          [Description]), will be used for the initial conditions.

        `imon` = 3

          Try to continue with the same step size and order as was to be
          used before entering the `monitr` operation (see
          [Description]). `hmin` and `hmax` may be altered if desired.

        `imon` = 4

          Continue the integration but using a new value of `hnext` and
          possibly new values of `hmin` and `hmax`.

    inln : int
        On intermediate entry: with `imon` = 0 and `irevcm` = 9, `inln`
        specifies the action to be taken by the internal nonlinear
        equation solver. By setting `inln` = 3 and returning to
        ``ivp_stiff_exp_revcom``, the residual vector is evaluated and
        placed in `comm`['rwork'][50+2*`neq`+i-1], for i = 1,2,...,`neq`
        and then the `monitr` operation (see [Description]) is invoked
        again. At present this is the only option available: `inln` must
        not be set to any other value.

    ires : int
        On intermediate entry: should be unchanged unless one of the
        following actions is required of ``ivp_stiff_exp_revcom`` in
        which case `ires` should be set accordingly.

        `ires` = 2

          Indicates to ``ivp_stiff_exp_revcom`` that control should be
          passed back immediately to the calling (sub)program with the
          error indicator set to `errno` = 11.

        `ires` = 3

          Indicates to ``ivp_stiff_exp_revcom`` that an error condition
          has occurred in the solution vector, its time derivative or in
          the value of t. The integrator will use a smaller time step to
          try to avoid this condition. If this is not possible
          ``ivp_stiff_exp_revcom`` returns to the calling (sub)program
          with the error indicator set to `errno` = 7.

        `ires` = 4

          Indicates to ``ivp_stiff_exp_revcom`` to stop its current
          operation and to enter the `monitr` operation (see
          [Description]) immediately.

    irevcm : int
        On initial entry: must contain 0.

        On intermediate entry: should remain unchanged.

    itask : int
        On initial entry: the task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit (e.g., see
          ``ivp_stiff_bdf``) may be equal to or beyond `tout`, but not
          before it in the direction of integration.

        `itask` = 5

          Take one step only and return, without passing tcrit (e.g.,
          see ``ivp_stiff_bdf``). tcrit must be specified under `itask`
          = 4.

    itrace : int
        On initial entry: the level of output that is printed by the
        integrator. `itrace` may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        On final exit: the value at which the computed solution y is
        returned (usually at `tout`).

    imon : int
        On intermediate exit: used to pass information between
        ``ivp_stiff_exp_revcom`` and the MONITR operation (see
        [Description]). With `irevcm` = 9, `imon` contains a flag
        indicating under what circumstances the return from
        ``ivp_stiff_exp_revcom`` occurred:

        `imon` = -2

          Exit from ``ivp_stiff_exp_revcom`` after `ires` = 4 caused an
          early termination (this facility could be used to locate
          discontinuities).

        `imon` = -1

          The current step failed repeatedly.

        `imon` = 0

          Exit from ``ivp_stiff_exp_revcom`` after a call to the
          internal nonlinear equation solver.

        `imon` = 1

          The current step was successful.

    inln : int
        On intermediate exit: contains a flag indicating the action to
        be taken, if any, by the internal nonlinear equation solver.

    ires : int
        On intermediate exit: with `irevcm` = 1, 2, 3, 4 or 5, `ires`
        contains the value 1.

    irevcm : int
        On intermediate exit: indicates what action you must take before
        re-entering. The possible exit values of `irevcm` are 1, 3, 4,
        5, 8, 9 or 10, which should be interpreted as follows:

        `irevcm` = 1, 3, 4 or 5

          Indicates that an FCN operation (see [Description]) is
          required: y' = g(t,y) must be supplied, where `y`[i-1] is
          located in y_i, for i = 1,2,...,`neq`.

          For `irevcm` = 1 or 3, y'_i should be placed in location
          `comm`['rwork'][50+2*`neq`+i-1], for i = 1,2,...,`neq`.

          For `irevcm` = 4, y'_i should be placed in location
          `comm`['rwork'][50+`neq`+i-1], for i = 1,2,...,`neq`.

          For `irevcm` = 5, y'_i should be placed in location
          `ydot`[i-1], for i = 1,2,...,`neq`.

        `irevcm` = 8

          Indicates that a JAC operation (see [Description]) is
          required: the Jacobian matrix must be supplied.

          If full matrix linear algebra is being used, the (i,j)th
          element of the Jacobian must be stored in
          `wkjac`[(j-1)*`neq`+i-1].

          If banded matrix linear algebra is being used then the (i,j)th
          element of the Jacobian must be stored in
          `wkjac`[(i-1)*m_B+k-1], where m_B = m_L+m_U+1 and k =
          min(m_L-i+1, 0)+j; here m_L and m_U are the number of
          subdiagonals and superdiagonals, respectively, in the band.

          If sparse matrix linear algebra is being used then
          ``ivp_stiff_sparjac_enq`` must be called to determine which
          column of the Jacobian is required and where it should be
          stored.

          will return in j the number of the column of the Jacobian that
          is required and will set iplace = 1 or 2.

          If iplace = 1, the (i,j)th element of the Jacobian must be
          stored in `comm`['rwork'][50+2*`neq`+i-1]; otherwise it must
          be stored in `comm`['rwork'][50+`neq`+i-1].

        `irevcm` = 9

          Indicates that a MONITR operation (see [Description]) can be
          performed.

        `irevcm` = 10

          Indicates that the current step was not successful, due to
          error test failure or convergence test failure. The only
          information supplied to you on this return is the current
          value of the independent variable t, located in
          `comm`['rwork'][18]. No values must be changed before
          re-entering ``ivp_stiff_exp_revcom``; this facility enables
          you to determine the number of unsuccessful steps.

        On final exit: `irevcm` = 0 indicated the user-specified task
        has been completed or an error has been encountered (see the
        descriptions for `itask` and errno).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On re-entry, `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, *<value>*, is too small.

        (`errno` 1)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `nwkjac` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `nwkjac` = *<value>*, nwkjac (setup) = *<value>*.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. hmin =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. hmax =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On re-entry, `inln` != 3 for the case `irevcm` = 9 and
            `imon` = 0.

            `inln` = *<value>*.

        (`errno` 1)
            On entry, `irevcm` is invalid: `irevcm` = *<value>*.

        (`errno` 1)
            On re-entry, the solution vector appears to have been
            overwritten.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 7)
            The derivative evaluation procedure set the error flag
            `ires` = 3 repeatedly despite attempts by the integrator to
            avoid this.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            There were repeated convergence-test failures on an
            attempted step, before completing the requested task, but
            the integration was successful as far as `t`. This may be
            caused by an inaccurate Jacobian matrix or one which is
            incorrectly computed.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 11)
            On re-entry, `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_exp_revcom'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nmft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Input/Output, inln.
        utils._BYREF_INT, # Input/Output, ires.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'jacpvt', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    _nwkjac = utils._EngineIntScalarType(
        nwkjac, {'fun_name': fun_name, 'entity': 'nwkjac'},
    ).to_ctypes()
    _nwkjac_py = utils._EngineIntScalarType.to_py(_nwkjac)
    _wkjac_cls = utils._EngineFloat64ArrayType(
        wkjac, {'fun_name': fun_name, 'entity': 'wkjac'},
        exp_shape=(_nwkjac_py,),
    )
    _wkjac = _wkjac_cls.to_ctypes()
    if _comm_needs_init:
        _jacpvt = utils._EngineIntArrayType.empty_ctypes(_neq_py)
        comm['jacpvt'] = _jacpvt
    else:
        _jacpvt = comm['jacpvt']
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    _imon = utils._EngineIntScalarType(
        imon, {'fun_name': fun_name, 'entity': 'imon'},
    ).to_ctypes()
    _inln = utils._EngineIntScalarType(
        inln, {'fun_name': fun_name, 'entity': 'inln'},
    ).to_ctypes()
    _ires = utils._EngineIntScalarType(
        ires, {'fun_name': fun_name, 'entity': 'ires'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _ysav,
        _sdysav,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _imon,
        _inln,
        _ires,
        _irevcm,
        _itask,
        _itrace,
        _nadv,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and',
                    'tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of nwkjac is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'nwkjac = {:s}, nwkjac (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, inln != 3 for the case irevcm = 9 and imon = 0.',
                    'inln = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm is invalid: irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, the solution vector appears to have been overwritten.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99953:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99952:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated convergence-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful as far',
                    'as t. This may be caused by an inaccurate Jacobian matrix or one which is',
                    'incorrectly computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99955:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The derivative evaluation procedure set the error flag ires = 3 repeatedly',
                    'despite attempts by the integrator to avoid this.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99949:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check',
                    'the problem formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, ires = 2, which signals that the integration',
                    'should terminate. ires = {:s} at time'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _wkjac_cls.refresh_py(_wkjac)
        _imon_py = utils._EngineIntScalarType.to_py(_imon)
        _inln_py = utils._EngineIntScalarType.to_py(_inln)
        _ires_py = utils._EngineIntScalarType.to_py(_ires)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'imon', 'inln', 'ires', 'irevcm'],
        )(
            _t_py,
            _imon_py,
            _inln_py,
            _ires_py,
            _irevcm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_imp_revcom(neq, ldysav, t, tout, y, ydot, comm, rtol, atol, itol, sdysav, wkjac, nwkjac, imon, inln, ires, irevcm, lderiv, itask, itrace, io_manager=None):
    u"""
    Implicit/algebraic ordinary differential equations, stiff initial
    value problem (reverse communication, comprehensive).

    For full information please refer to the NAG Library document for
    d02nn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nnf.html

    Parameters
    ----------
    neq : int
        The dimension of the arrays `y` and `ydot`.

        On initial entry: the number of equations to be solved.

    ldysav : int
        On initial entry: a bound on the maximum number of equations to
        be solved during the integration.

    t : float
        On initial entry: t, the value of the independent variable. The
        input value of `t` is used only on the first call as the initial
        point of the integration.

    tout : float
        On initial entry: the next value of t at which a computed
        solution is desired. For the initial t, the input value of
        `tout` is used to determine the direction of integration.
        Integration is permitted in either direction (see also `itask`).

    y : float, shape (`neq`), modified in place
        On initial entry: the values of the dependent variables
        (solution). On the first call the first `neq` elements of y must
        contain the vector of initial values.

        On final exit: the computed solution vector evaluated at `t`
        (usually t = `tout`).

    ydot : float, shape (`neq`), modified in place
        On initial entry: if `lderiv`[0] = True, `ydot` must contain
        approximations to the time derivatives y' of the vector y. If
        `lderiv`[0] = False, `ydot` need not be set on entry.

        On final exit: contains the time derivatives y' of the vector y
        at the last integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to one
        of ``ivp_stiff_fulljac_setup``, ``ivp_stiff_bandjac_setup`` or
        ``ivp_stiff_sparjac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    rtol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 2): 1; otherwise: `neq`.

        On initial entry: the relative local error tolerance.

    atol : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `itol` in (1, 3): 1; otherwise: `neq`.

        On initial entry: the absolute local error tolerance.

    itol : int
        On initial entry: a value to indicate the form of the local
        error test. `itol` indicates to ``ivp_stiff_imp_revcom`` whether
        to interpret either or both of `rtol` or `atol` as a vector or a
        scalar. The error test to be satisfied is ||e_i/w_i|| < 1.0,
        where w_i is defined as follows:

        [table omitted]

        e_i is an estimate of the local error in y_i, computed
        internally, and the choice of norm to be used is defined by a
        previous call to an integrator setup function.

    sdysav : int
        On initial entry: an appropriate value for `sdysav` is described
        in the specifications of the integrator setup functions
        ``ivp_stiff_dassl``, ``ivp_stiff_bdf`` and ``ivp_stiff_blend``.
        This value must be the same as that supplied to the integrator
        setup function.

    wkjac : float, shape (`nwkjac`), modified in place
        On intermediate entry: elements of the Jacobian as defined under
        the description of `irevcm`. If a numerical Jacobian was
        requested then `wkjac` is used for workspace.

        On intermediate exit: the Jacobian is overwritten.

    nwkjac : int
        The dimension of the array `wkjac`.

        On initial entry: the actual size depends on the linear algebra
        method used. An appropriate value for `nwkjac` is described in
        the specifications of the linear algebra setup functions
        ``ivp_stiff_fulljac_setup``, ``ivp_stiff_bandjac_setup`` and
        ``ivp_stiff_sparjac_setup`` for full, banded and sparse matrix
        linear algebra respectively. This value must be the same as that
        supplied to the linear algebra setup function.

    imon : int
        On intermediate entry: may be reset to determine subsequent
        action in ``ivp_stiff_imp_revcom``.

        `imon` = -2

          Integration is to be halted. A return will be made from
          ``ivp_stiff_imp_revcom`` to the calling (sub)program with
          `errno` = 12.

        `imon` = -1

          Allow ``ivp_stiff_imp_revcom`` to continue with its own
          internal strategy. The integrator will try up to three
          restarts unless `imon` != -1.

        `imon` = 0

          Return to the internal nonlinear equation solver, where the
          action taken is determined by the value of `inln`.

        `imon` = 1

          Normal exit to ``ivp_stiff_imp_revcom`` to continue
          integration.

        `imon` = 2

          Restart the integration at the current time point. The
          integrator will restart from order 1 when this option is used.
          The internal initialization module solves for new values of y
          and y' by using the values supplied in `y` and `ydot` by the
          `monitr` operation (see [Description]) as initial estimates.

        `imon` = 3

          Try to continue with the same step size and order as was to be
          used before entering the `monitr` operation (see
          [Description]). `hmin` and `hmax` may be altered if desired.

        `imon` = 4

          Continue the integration but using a new value of `hnext` and
          possibly new values of `hmin` and `hmax`.

    inln : int
        On intermediate entry: with `imon` = 0 and `irevcm` = 9, `inln`
        specifies the action to be taken by the internal nonlinear
        equation solver. By setting `inln` = 3 and returning to
        ``ivp_stiff_imp_revcom``, the residual vector is evaluated and
        placed in `comm`['rwork'][50+2*`neq`+i-1], for i = 1,2,...,`neq`
        and then the `monitr` operation (see [Description]) is invoked
        again. At present this is the only option available: `inln` must
        not be set to any other value.

    ires : int
        On intermediate entry: should be unchanged unless one of the
        following actions is required of ``ivp_stiff_imp_revcom`` in
        which case `ires` should be set accordingly.

        `ires` = 2

          Indicates to ``ivp_stiff_imp_revcom`` that control should be
          passed back immediately to the calling (sub)program with the
          error indicator set to `errno` = 11.

        `ires` = 3

          Indicates to ``ivp_stiff_imp_revcom`` that an error condition
          has occurred in the solution vector, its time derivative or in
          the value of t. The integrator will use a smaller time step to
          try to avoid this condition. If this is not possible
          ``ivp_stiff_imp_revcom`` returns to the calling (sub)program
          with the error indicator set to `errno` = 7.

        `ires` = 4

          Indicates to ``ivp_stiff_imp_revcom`` to stop its current
          operation and to enter the `monitr` operation (see
          [Description]) immediately.

    irevcm : int
        On initial entry: must contain 0.

        On intermediate entry: should remain unchanged.

    lderiv : bool, shape (2), modified in place
        On initial entry: `lderiv`[0] must be set to True if you have
        supplied both an initial y and an initial y'. `lderiv`[0] must
        be set to False if only the initial y has been supplied.

        `lderiv`[1] must be set to True if the integrator is to use a
        modified Newton method to evaluate the initial y and y'.

        Note that y and y', if supplied, are used as initial estimates.

        This method involves taking a small step at the start of the
        integration, and if `itask` = 6 on entry, `t` and `tout` will be
        set to the result of taking this small step. `lderiv`[1] must be
        set to False if the integrator is to use functional iteration to
        evaluate the initial y and y', and if this fails a modified
        Newton method will then be attempted. `lderiv`[1] = True is
        recommended if there are implicit equations or the initial y and
        y' are zero.

        On final exit: `lderiv`[0] is normally unchanged. However if
        `itask` = 6 and internal initialization was successful then
        `lderiv`[0] = True.

        `lderiv`[1] = True, if implicit equations were detected.

        Otherwise `lderiv`[1] = False.

    itask : int
        On initial entry: the task to be performed by the integrator.

        `itask` = 1

          Normal computation of output values of y(t) at t = `tout` (by
          overshooting and interpolating).

        `itask` = 2

          Take one step only and return.

        `itask` = 3

          Stop at the first internal integration point at or beyond t =
          `tout` and return.

        `itask` = 4

          Normal computation of output values of y(t) at t = `tout` but
          without overshooting t = tcrit. tcrit must be specified as an
          option in one of the integrator setup functions before the
          first call to the integrator, or specified in the optional
          input function before a continuation call. tcrit (e.g., see
          ``ivp_stiff_bdf``) may be equal to or beyond `tout`, but not
          before it in the direction of integration.

        `itask` = 5

          Take one step only and return, without passing tcrit (e.g.,
          see ``ivp_stiff_bdf``). tcrit must be specified under `itask`
          = 4.

        `itask` = 6

          The integrator will solve for the initial values of y and y'
          only and then return to the calling (sub)program without doing
          the integration. This option can be used to check the initial
          values of y and y'. Functional iteration or a 'small' backward
          Euler method used in conjunction with a damped Newton
          iteration is used to calculate these values (see `lderiv`).
          Note that if a backward Euler step is used then the value of t
          will have been advanced a short distance from the initial
          point.

        Note: if ``ivp_stiff_imp_revcom`` is recalled with a different
        value of `itask` (and `tout` altered) then the initialization
        procedure is repeated, possibly leading to different initial
        conditions.

    itrace : int
        On initial entry: the level of output that is printed by the
        integrator. `itrace` may take the value -1, 0, 1, 2 or 3.

        `itrace` < -1

          -1 is assumed and similarly if `itrace` > 3, 3 is assumed.

        `itrace` = -1

          No output is generated.

        `itrace` = 0

          Only warning messages are printed on the current error message
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

        `itrace` > 0

          Warning messages are printed as above, and on the file object
          associated with the advisory I/O unit (see
          :class:`~naginterfaces.base.utils.FileObjManager`) output is
          generated which details Jacobian entries, the nonlinear
          iteration and the time integration. The advisory messages are
          given in greater detail the larger the value of `itrace`.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    t : float
        On final exit: the value at which the computed solution y is
        returned (usually at `tout`).

    tout : float
        Is unaltered unless `itask` = 6 and `lderiv`[1] = True on entry
        (see also `itask` and `lderiv`) in which case `tout` will be set
        to the result of taking a small step at the start of the
        integration.

    imon : int
        On intermediate exit: used to pass information between
        ``ivp_stiff_imp_revcom`` and the `monitr` operation (see
        [Description]). With `irevcm` = 9, `imon` contains a flag
        indicating under what circumstances the return from
        ``ivp_stiff_imp_revcom`` occurred:

        `imon` = -2

          Exit from ``ivp_stiff_imp_revcom`` after `ires` = 4 (set in
          the `resid` operation (see [Description]) caused an early
          termination (this facility could be used to locate
          discontinuities).

        `imon` = -1

          The current step failed repeatedly.

        `imon` = 0

          Exit from ``ivp_stiff_imp_revcom`` after a call to the
          internal nonlinear equation solver.

        `imon` = 1

          The current step was successful.

    inln : int
        On intermediate exit: contains a flag indicating the action to
        be taken, if any, by the internal nonlinear equation solver.

    ires : int
        On intermediate exit: with `irevcm` = 1, 2, 3, 4, 5, 6, 7 or 11,
        `ires` specifies the form of the residual to be returned by the
        `resid` operation (see [Description]).

        If `ires` = 1, -r = g(t,y)-A(t,y)y' must be returned.

        If `ires` = -1, -r^= -A(t,y)y' must be returned.

    irevcm : int
        On intermediate exit: indicates what action you must take before
        re-entering ``ivp_stiff_imp_revcom``. The possible exit values
        of `irevcm` are 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 or 11 which should
        be interpreted as follows:

        `irevcm` = 1, 2, 3, 4, 5, 6, 7 or 11

          Indicates that a `resid` operation (see [Description]) is
          required: you must supply the residual of the system. For each
          of these values of `irevcm`, y_i is located in `y`[i-1], for i
          = 1,2,...,`neq`.

          For `irevcm` = 1, 3, 6 or 11, y'_i is located in `ydot`[i-1]
          and r_i should be stored in `comm`['rwork'][50+2*`neq`+i-1],
          for i = 1,2,...,`neq`.

          For `irevcm` = 2, y'_i is located in
          `comm`['rwork'][50+`neq`+i-1] and r_i should be stored in
          `comm`['rwork'][50+2*`neq`+i-1], for i = 1,2,...,`neq`.

          For `irevcm` = 4 or 7, y'_i is located in `ydot`[i-1] and r_i
          should be stored in `comm`['rwork'][50+`neq`+i-1], for i =
          1,2,...,`neq`.

          For `irevcm` = 5, y'_i is located in
          `comm`['rwork'][50+2*`neq`+i-1] and r_i should be stored in
          `ydot`[i-1], for i = 1,2,...,`neq`.

        `irevcm` = 8

          Indicates that a `jac` operation (see [Description]) is
          required: you must supply the Jacobian matrix.

          If full matrix linear algebra is being used, the (i,j)th
          element of the Jacobian must be stored in
          `wkjac`[(j-1)*`neq`+i-1].

          If banded matrix linear algebra is being used, the (i,j)th
          element of the Jacobian must be stored in
          `wkjac`[(i-1)*m_B+k-1], where m_B = m_L+m_U+1 and k =
          min(m_L-i+1, 0)+j; here m_L and m_U are the number of
          subdiagonals and superdiagonals, respectively, in the band.

          If sparse matrix linear algebra is being used,
          ``ivp_stiff_sparjac_enq`` must be called to determine which
          column of the Jacobian is required and where it should be
          stored.

          will return in j the number of the column of the Jacobian that
          is required and will set iplace = 1 or 2 (see
          ``ivp_stiff_sparjac_enq``).

          If iplace = 1, you must store the nonzero element (i,j) of the
          Jacobian in `comm`['rwork'][50+2*`neq`+i-1]; otherwise it must
          be stored in `comm`['rwork'][50+`neq`+i-1].

        `irevcm` = 9

          Indicates that a `monitr` operation (see [Description]) can be
          performed.

        `irevcm` = 10

          Indicates that the current step was not successful, due to
          error test failure or convergence test failure. The only
          information supplied to you on this return is the current
          value of the variable t, located in `comm`['rwork'][18]. No
          values must be changed before re-entering
          ``ivp_stiff_imp_revcom``; this facility enables you to
          determine the number of unsuccessful steps.

        On final exit: `irevcm` = 0 indicating that the user-specified
        task has been completed or an error has been encountered (see
        the descriptions for `itask` and errno).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ires` = *<value>* and dy/dt = 0.0 for all
            elements.

            Check the evaluation of the residual for this value of
            `ires`.

        (`errno` 1)
            On re-entry, `ires` has been set to an illegal value during
            initialization.

            `ires` = *<value>* at time *<value>*.

        (`errno` 1)
            On re-entry, `imon` = *<value>*.

            Constraint: -2 <= `imon` <= 4.

        (`errno` 1)
            Failure during internal time interpolation. `tout` and the
            current time are too close.

            `itask` = *<value>* and `tout` = *<value>* and the current
            time is *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before the current
            time in the direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and the current time
            is *<value>*.

        (`errno` 1)
            `itask` = 3 and `tout` is more than an integration step
            behind the current time.

            `tout` = *<value>*, current time minus step size: *<value>*.

        (`errno` 1)
            The initial stepsize, h = *<value>*, is too small.

        (`errno` 1)
            Weight number i = *<value>* used in the local error test is
            too small. Check the values of `rtol` and `atol`.

            `atol`[i-1] and `y`[i-1] may both be zero.

            Weight i = *<value>*.

        (`errno` 1)
            On entry, `itask` = 4 or 5 and tcrit is before `tout` in the
            direction of integration.

            `itask` = *<value>*, tcrit = *<value>* and `tout` =
            *<value>*.

        (`errno` 1)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 1)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 1)
            Either the value of `nwkjac` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `nwkjac` = *<value>*, nwkjac (setup) = *<value>*.

        (`errno` 1)
            Either the value of `sdysav` is not the same as the value
            supplied to the setup function or a communication array has
            become corrupted.

            `sdysav` = *<value>*, sdysav (setup) = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) minimum stepsize was
            provided in a prior call to a setup function. hmin =
            *<value>*.

        (`errno` 1)
            On entry, `tout` is less than `t` with respect to the
            direction of integration given by the sign of h0 in a prior
            call to a setup function.

            `tout` = *<value>*, `t` = *<value>* and h0 = *<value>*.

        (`errno` 1)
            On entry, an illegal (negative) maximum number of steps was
            provided in a prior call to a setup function. maxstp =
            *<value>*.

        (`errno` 1)
            On entry, `itol` = *<value>*.

            Constraint: 1 <= `itol` <= 4.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            Either the function was entered on a continuation call
            without a prior call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, `tout` is too close to `t` to start integration.

            `tout` = *<value>* and `t` = *<value>*.

        (`errno` 1)
            On entry, `itask` = *<value>*.

            Constraint: 1 <= `itask` <= 5.

        (`errno` 1)
            Either the integrator setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On entry, an illegal (negative) maximum stepsize was
            provided in a prior call to a setup function. hmax =
            *<value>*.

        (`errno` 1)
            Either the linear algebra setup function has not been called
            prior to the first call of this function, or a communication
            array has become corrupted.

        (`errno` 1)
            On re-entry, `inln` = *<value>* for the case `irevcm` = 9
            and `imon` = 0.

            Constraint: `inln` = 3.

        (`errno` 1)
            On entry, `irevcm` = *<value>*.

            Constraint: 0 <= `irevcm` <= 11.

        (`errno` 1)
            On re-entry, the solution vector appears to have been
            overwritten.

            Further integration will not be attempted.

        (`errno` 2)
            At time *<value>* the maximum number of allowed steps on
            this call was taken before reaching the next output point
            `tout` = *<value>*.

            Maximum number of steps = *<value>*.

        (`errno` 7)
            On re-entry, `ires` = 3 which signals that an error
            condition has occurred in the solution vector, its time
            derivative or in the value of t. It was not possible to
            remove this condition.

            `ires` = *<value>* at t = *<value>*.

        (`errno` 8)
            Attempt was made to reduce the step size to a value less
            than the minimum step size during the calculation of initial
            values.

            Minimum stepsize: *<value>*.

        (`errno` 8)
            The user problem has one or more inconsistencies between the
            `ires` = 1 and `ires` = -1 parts. Integration will not be
            attempted.

        (`errno` 8)
            The residual function returned an error when calculating the
            initial values of the solution and its time derivative.

        (`errno` 8)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 9)
            A singular Jacobian has been encountered. You should check
            the problem formulation and Jacobian calculation.

        (`errno` 10)
            Workspace error occurred when trying to form the Jacobian
            matrix in calculating the initial values of the solution and
            its time derivative.

        (`errno` 10)
            Not enough integer store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 10)
            Larger integer workspace required.

            Provided: *<value>*; required: *<value>*.

        (`errno` 10)
            Not enough real store provided for sparse matrix solver.

            Units of store needed: *<value>*. Amount provided:
            *<value>*.

        (`errno` 14)
            On entry, too much accuracy requested for precision of the
            machine at the start of problem. The tolerances should be
            checked; the requested accuracy should be reduced by a
            factor of at least *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Too much accuracy requested for precision of the machine at
            time *<value>*.

            The tolerances should be checked; the requested accuracy
            should be reduced by a factor of at least *<value>*.

        (`errno` 4)
            There were repeated error-test failures on an attempted
            step, before completing the requested task, but the
            integration was successful as far as `t`. The problem may
            have a singularity, or the local error requirements may be
            inappropriate.

        (`errno` 5)
            Nonlinear solver failed to converge using a damped Newton
            method to solve for initial values.

            Damping factor: *<value>*; convergence rate: *<value>*.

        (`errno` 6)
            At time *<value>*, error weight *<value>* became zero. Check
            the values of `atol`, `rtol` and `itol` supplied.

        (`errno` 11)
            On re-entry, `ires` = 2, which signals that the integration
            should terminate. `ires` = *<value>* at time *<value>*.

        (`errno` 12)
            A return was forced by setting `imon` = -2, but the
            integration was successful as far as `t`.

        (`errno` 13)
            The requested task has been completed, but it is estimated
            that a small change in `rtol` and `atol` is unlikely to
            produce any change in the computed solution. (This ONLY
            applies when you are NOT operating in one step mode; that
            is, when `itask` != 2 or 5.)

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_imp_revcom'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nnft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._BYREF_FLOAT64, # Input/Output, tout.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ydot[neq].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neq].
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[*].
        utils._EngineFloat64ArrayType.c_type, # Input, atol[*].
        utils._BYREF_INT, # Input, itol.
        utils._EngineIntArrayType.c_type, # Input/Output, inform[23].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, sdysav.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wkjac[nwkjac].
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input/Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._BYREF_INT, # Input/Output, imon.
        utils._BYREF_INT, # Input/Output, inln.
        utils._BYREF_INT, # Input/Output, ires.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._EngineLogicalArrayType.c_type, # Input/Output, lderiv[2].
        utils._BYREF_INT, # Input, itask.
        utils._BYREF_INT, # Input, itrace.
        utils._BYREF_INT, # Input, nadv.
        utils._EngineCharArrayType.c_type, # Input/Output, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py,),
    )
    _y = _y_cls.to_ctypes()
    _ydot_cls = utils._EngineFloat64ArrayType(
        ydot, {'fun_name': fun_name, 'entity': 'ydot'},
        exp_shape=(_neq_py,),
    )
    _ydot = _ydot_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('inform', 'ysav', 'jacpvt', 'lwsav', 'iwsav', 'rwsav'),
        req_keys=('rwork', 'cwsav'),
    )
    _rwork = comm['rwork']
    _itol = utils._EngineIntScalarType(
        itol, {'fun_name': fun_name, 'entity': 'itol'},
    ).to_ctypes()
    _itol_py = utils._EngineIntScalarType.to_py(_itol)
    if _itol_py in (1, 2):
        _rtol_dim1 = 1
    else:
        _rtol_dim1 = _neq_py
    exp_shape_rtol = (_rtol_dim1,)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=exp_shape_rtol,
    ).to_ctypes()
    if _itol_py in (1, 3):
        _atol_dim1 = 1
    else:
        _atol_dim1 = _neq_py
    exp_shape_atol = (_atol_dim1,)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=exp_shape_atol,
    ).to_ctypes()
    if _comm_needs_init:
        _inform = utils._EngineIntArrayType.empty_ctypes(23)
        comm['inform'] = _inform
    else:
        _inform = comm['inform']
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _ldysav_py = utils._EngineIntScalarType.to_py(_ldysav)
    _sdysav_py = utils._EngineIntScalarType.to_py(_sdysav)
    if _comm_needs_init:
        _ysav = utils._EngineFloat64ArrayType.empty_ctypes(_ldysav_py*_sdysav_py)
        comm['ysav'] = _ysav
    else:
        _ysav = comm['ysav']
    _nwkjac = utils._EngineIntScalarType(
        nwkjac, {'fun_name': fun_name, 'entity': 'nwkjac'},
    ).to_ctypes()
    _nwkjac_py = utils._EngineIntScalarType.to_py(_nwkjac)
    _wkjac_cls = utils._EngineFloat64ArrayType(
        wkjac, {'fun_name': fun_name, 'entity': 'wkjac'},
        exp_shape=(_nwkjac_py,),
    )
    _wkjac = _wkjac_cls.to_ctypes()
    if _comm_needs_init:
        if int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (33,), validate=False)) == 2:
            _jacpvt_dim1 = _neq_py
        elif int(utils._EngineFloat64ArrayType.get_element(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, (33,), validate=False)) == 3:
            _jacpvt_dim1 = 20*_neq_py
        else:
            _jacpvt_dim1 = 1
        _jacpvt = utils._EngineIntArrayType.empty_ctypes(_jacpvt_dim1)
        comm['jacpvt'] = _jacpvt
    else:
        _jacpvt = comm['jacpvt']
    _njcpvt_py = utils._EngineIntArrayType.get_shape(comm['jacpvt'], {'fun_name': fun_name, 'entity': 'comm[\'jacpvt\']'}, exp_rank=1, validate=False)[0]
    _njcpvt = utils._EngineIntScalarType(
        _njcpvt_py, {'fun_name': fun_name, 'entity': '_njcpvt'},
    ).to_ctypes()
    _imon = utils._EngineIntScalarType(
        imon, {'fun_name': fun_name, 'entity': 'imon'},
    ).to_ctypes()
    _inln = utils._EngineIntScalarType(
        inln, {'fun_name': fun_name, 'entity': 'inln'},
    ).to_ctypes()
    _ires = utils._EngineIntScalarType(
        ires, {'fun_name': fun_name, 'entity': 'ires'},
    ).to_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _lderiv_cls = utils._EngineLogicalArrayType(
        lderiv, {'fun_name': fun_name, 'entity': 'lderiv'},
        exp_shape=(2,),
    )
    _lderiv = _lderiv_cls.to_ctypes()
    _itask = utils._EngineIntScalarType(
        itask, {'fun_name': fun_name, 'entity': 'itask'},
    ).to_ctypes()
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _nadv = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _cwsav = comm['cwsav']
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(100)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(505)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(1100)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _neq,
        _ldysav,
        _t,
        _tout,
        _y,
        _ydot,
        _rwork,
        _rtol,
        _atol,
        _itol,
        _inform,
        _ysav,
        _sdysav,
        _wkjac,
        _nwkjac,
        _jacpvt,
        _njcpvt,
        _imon,
        _inln,
        _ires,
        _irevcm,
        _lderiv,
        _itask,
        _itrace,
        _nadv,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99937:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ires = {:s} and dy/dt = 0.0 for all elements.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Check the evaluation of the residual for this value of ires.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99939:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, ires has been set to an illegal value during initialization.',
                    'ires = {:s} at time {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, imon = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -2 <= imon <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure during internal time interpolation. tout and the',
                    'current time are too close.',
                    'itask = {:s} and tout = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and tcrit is before',
                    'the current time in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'current time is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'itask = 3 and tout is more than an integration step behind the current time.',
                    'tout = {:s}, current time minus step size:'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial stepsize, h = {:s}, is too small.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Weight number i = {:s} used in the local error test is too'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'small. Check the values of rtol and atol.',
                    'atol[i-1] and y[i-1] may both be zero.',
                    'Weight i = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = 4 or 5 and',
                    'tcrit is before',
                    'tout in the direction of integration.',
                    'itask = {:s}, tcrit = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of nwkjac is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'nwkjac = {:s}, nwkjac (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the value of sdysav is not the same as the value supplied to the',
                    'setup function or a communication array has become corrupted.',
                    'sdysav = {:s}, sdysav (setup) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) minimum stepsize was provided in a prior',
                    'call to a setup function. hmin = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is less than t with respect to the direction of',
                    'integration given by the sign of h0 in a prior',
                    'call to a setup function.',
                    'tout = {:s}, t = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and h0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum number of steps was provided in a',
                    'prior call to a setup function. maxstp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itol <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the function was entered on a continuation call without a prior',
                    'call of this function, or a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout is too close to t to start integration.',
                    'tout = {:s} and t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itask = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= itask <= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the integrator setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an illegal (negative) maximum stepsize was provided in a prior',
                    'call to a setup function. hmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the linear algebra setup function has not been called prior',
                    'to the first call of this function, or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, inln = {:s} for the case irevcm = 9 and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'imon = 0.',
                    'Constraint: inln = 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= irevcm <= 11.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, the solution vector appears to have been overwritten.',
                    'Further integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s} the maximum number of allowed'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'steps on this call was taken before reaching the next',
                    'output point tout = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Maximum number of steps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99966:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too much accuracy requested for precision of the machine',
                    'at time {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The tolerances should be checked; the requested accuracy should be',
                    'reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There were repeated error-test failures on an attempted step, before',
                    'completing the requested task, but the integration was successful',
                    'as far as t. The problem may have a singularity, or the local error',
                    'requirements may be inappropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99931:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Nonlinear solver failed to converge using a damped Newton',
                    'method to solve for initial values.',
                    'Damping factor: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'convergence rate: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99967:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At time {:s}, error weight {:s} became'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'zero. Check the values of atol, rtol and itol supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99940:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, ires = 3 which signals that an error condition',
                    'has occurred in the solution vector, its time derivative or in the value',
                    'of t. It was not possible to remove this condition.',
                    'ires = {:s} at t = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99932:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Attempt was made to reduce the step size to a value less',
                    'than the minimum step size during the calculation of initial values.',
                    'Minimum stepsize: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99933:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The user problem has one or more inconsistencies between',
                    'the ires = 1 and ires = -1 parts. Integration will not be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99934:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The residual function returned an error when calculating',
                    'the initial values of the solution and its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99935:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99941:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A singular Jacobian has been encountered. You should check the problem',
                    'formulation and Jacobian calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99960:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Workspace error occurred when trying to form the Jacobian',
                    'matrix in calculating the initial values of the solution and',
                    'its time derivative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99961:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough integer store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99962:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Larger integer workspace required.',
                    'Provided: {:s}; required: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99963:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not enough real store provided for sparse matrix solver.',
                    'Units of store needed: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Amount provided: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99971:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On re-entry, ires = 2, which signals that the integration',
                    'should terminate. ires = {:s} at time'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99951:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A return was forced by setting imon = -2, but the integration was',
                    'successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99950:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested task has been completed, but it is estimated that a small',
                    'change in rtol and atol is unlikely to produce any change in the computed',
                    'solution. (This ONLY applies when you are NOT operating in one step mode;',
                    'that is, when itask != 2 or 5.)',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much accuracy requested for precision of the machine at the',
                    'start of problem. The tolerances should be checked; the requested accuracy',
                    'should be reduced by a factor of at least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _tout_py = utils._EngineFloat64ScalarType.to_py(_tout)
        _y_cls.refresh_py(_y)
        _ydot_cls.refresh_py(_ydot)
        _wkjac_cls.refresh_py(_wkjac)
        _imon_py = utils._EngineIntScalarType.to_py(_imon)
        _inln_py = utils._EngineIntScalarType.to_py(_inln)
        _ires_py = utils._EngineIntScalarType.to_py(_ires)
        _lderiv_cls.refresh_py(_lderiv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'tout', 'imon', 'inln', 'ires', 'irevcm'],
        )(
            _t_py,
            _tout_py,
            _imon_py,
            _inln_py,
            _ires_py,
            _irevcm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dae_dassl_linalg(neq, ml, mu, comm, licom):
    u"""
    Implicit ordinary differential equations/DAEs, initial value problem
    linear algebra setup function for ``dae_dassl_gen``.

    For full information please refer to the NAG Library document for
    d02np

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02npf.html

    Parameters
    ----------
    neq : int
        The number of differential-algebraic equations to be solved.

    ml : int
        m_L, the number of subdiagonals in the band.

    mu : int
        m_U, the number of superdiagonals in the band.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dae_dassl_setup``.

    licom : int
        The dimension of the array `comm`['icom'].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 2)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 2)
            On entry, `ml` = *<value>* and `neq` = *<value>*.

            Constraint: `ml` <= `neq`-1.

        (`errno` 3)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

        (`errno` 3)
            On entry, `mu` = *<value>* and `neq` = *<value>*.

            Constraint: `mu` <= `neq`-1.

        (`errno` 4)
            Either the initialization function has not been called prior
            to the first call of this function or the communication
            array has become corrupted.

        (`errno` 5)
            On entry, `licom` is too small: `licom` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.dae_dassl_linalg'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02npft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._EngineIntArrayType.c_type, # Input/Output, icom[licom].
        utils._BYREF_INT, # Input, licom.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _ml = utils._EngineIntScalarType(
        ml, {'fun_name': fun_name, 'entity': 'ml'},
    ).to_ctypes()
    _mu = utils._EngineIntScalarType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
    ).to_ctypes()
    _licom = utils._EngineIntScalarType(
        licom, {'fun_name': fun_name, 'entity': 'licom'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icom',),
    )
    _icom = comm['icom']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _ml,
        _mu,
        _icom,
        _licom,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1020:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ml >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1021:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ml <= neq-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mu >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1031:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mu <= neq-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 2040:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior to the',
                    'first call of this function or the communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1050:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, licom is too small: licom = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_stiff_sparjac_enq(comm):
    u"""
    Ordinary differential equations, initial value problem, for use with
    d02m--n functions, sparse Jacobian, enquiry function.

    For full information please refer to the NAG Library document for
    d02nr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nrf.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_exp_revcom`` or ``ivp_stiff_imp_revcom``.

    Returns
    -------
    j : int
        The index j of the column of the Jacobian which is required.

    iplace : int
        Indicates which locations in the array rwork to fill with the
        jth column.

        If `iplace` = 1, the (i,j)th element of the Jacobian must be
        placed in rwork[50+2*ldysav+i-1], otherwise the (i,j)th element
        must be placed in rwork[50+ldysav+i-1].

        If jceval = 'F', in the previous call to
        ``ivp_stiff_sparjac_setup``, `iplace` = 2 always, hence the jth
        column of the Jacobian must be placed in rwork[50+ldysav+i-1],
        for i = 1,2,...,neq.

        rwork, neq and ldysav are arguments of ``ivp_stiff_exp_revcom``
        and ``ivp_stiff_imp_revcom``.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_sparjac_enq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nrft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, j.
        utils._BYREF_INT, # Output, iplace.
        utils._EngineIntArrayType.c_type, # Input, inform[23].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _j = utils._EngineIntScalarType.empty_ctypes()
    _iplace = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('inform',),
    )
    _inform = comm['inform']
    engine_ctypes(
        _en_data,
        _j,
        _iplace,
        _inform,
    )
    utils._handle_cb_exception(excs)
    _j_py = utils._EngineIntScalarType.to_py(_j)
    _iplace_py = utils._EngineIntScalarType.to_py(_iplace)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['j', 'iplace'],
    )(
        _j_py,
        _iplace_py,
    )
    return return_data

def ivp_stiff_fulljac_setup(neq, neqmax, jceval, nwkjac, comm):
    u"""
    Ordinary differential equations, initial value problem, for use with
    d02m--n functions, full Jacobian, linear algebra set up.

    For full information please refer to the NAG Library document for
    d02ns

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nsf.html

    Parameters
    ----------
    neq : int
        The number of differential equations.

    neqmax : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    jceval : str, length 1
        Specifies the technique to be used to compute the Jacobian.

        `jceval` = 'N'

          The Jacobian is to be evaluated numerically by the integrator.
          If this option is used, the actual argument corresponding to
          jac in the call to ``ivp_stiff_exp_fulljac`` or
          ``ivp_stiff_imp_fulljac`` must be specified as None.

        `jceval` = 'A'

          You must supply a (sub)program to evaluate the Jacobian on a
          call to the integrator.

        `jceval` = 'D'

          The default choice is to be made. In this case 'D' is
          interpreted as 'N'.

        Only the first character of the actual argument `jceval` is
        passed to ``ivp_stiff_fulljac_setup``; hence it is permissible
        for the actual argument to be more descriptive 'Numerical',
        'Analytical' or 'Default' on a call to
        ``ivp_stiff_fulljac_setup``.

    nwkjac : int
        The size of the workspace array wkjac, which you are supplying
        to the integrator, as declared in the (sub)program from which
        ``ivp_stiff_fulljac_setup`` is called.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `jceval` = *<value>*.

            Constraint: `jceval` = 'A', 'N' or 'D'.

        (`errno` 1)
            On entry, `nwkjac` = *<value>* and `neqmax` = *<value>*.

            Constraint: `nwkjac` >= (`neqmax`+1)*`neqmax`.

        (`errno` 1)
            On entry, `neq` = *<value>* and `neqmax` = *<value>*.

            Constraint: `neq` <= `neqmax`.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.

    See Also
    --------
    :meth:`naginterfaces.library.examples.ode.ivp_stiff_exp_fulljac_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_fulljac_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_STRING, # Input, jceval(len=1).
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neqmax].
        utils._EngineCharArrayType.c_type, # Output, cwsav(len=80)[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jceval.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _jceval = utils._EngineCharScalarType(
        jceval, {'fun_name': fun_name, 'entity': 'jceval'}, exp_chrlen=1,
    ).to_ctypes()
    _nwkjac = utils._EngineIntScalarType(
        nwkjac, {'fun_name': fun_name, 'entity': 'nwkjac'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('cwsav',),
        req_keys=('rwork',),
    )
    _rwork = comm['rwork']
    _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
    _cwsav_chrlen = 80
    comm['cwsav'] = _cwsav
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _neqmax,
        _jceval,
        _nwkjac,
        _rwork,
        _cwsav,
        _errbuf,
        _ierr,
        _jceval._length_,
        _cwsav_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jceval = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jceval = \'A\', \'N\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwkjac = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwkjac >= (neqmax+1)*neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_stiff_bandjac_setup(neq, neqmax, jceval, ml, mu, nwkjac, njcpvt, comm):
    u"""
    Ordinary differential equations, initial value problem, for use with
    d02m--n functions, banded Jacobian, linear algebra set up.

    For full information please refer to the NAG Library document for
    d02nt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ntf.html

    Parameters
    ----------
    neq : int
        The number of differential equations.

    neqmax : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    jceval : str, length 1
        Specifies the technique to be used to compute the Jacobian as
        follows:

        `jceval` = 'N'

          The Jacobian is to be evaluated numerically by the integrator.
          If this option is used, the actual argument corresponding to
          jac in the call to ``ivp_stiff_exp_bandjac`` or
          ``ivp_stiff_imp_bandjac`` must be specified as None.

        `jceval` = 'A'

          You must supply a (sub)program to evaluate the Jacobian on a
          call to the integrator.

        `jceval` = 'D'

          The default choice is to be made. In this case 'D' is
          interpreted as 'N'.

        Only the first character of the actual argument `jceval` is
        passed to ``ivp_stiff_bandjac_setup``; hence it is permissible
        for the actual argument to be more descriptive, e.g.,
        'Numerical', 'Analytical' or 'Default', on a call to
        ``ivp_stiff_bandjac_setup``.

    ml : int
        m_L, the number of subdiagonals in the band.

    mu : int
        m_U, the number of superdiagonals in the band.

    nwkjac : int
        The size of the workspace array wkjac, which you are supplying
        to the integrator, as declared in the (sub)program from which
        ``ivp_stiff_bandjac_setup`` is called.

    njcpvt : int
        The size of the workspace array jacpvt, which you are supplying
        to the integrator, as declared in the (sub)program from which
        ``ivp_stiff_bandjac_setup`` is called.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `jceval` = *<value>*.

            Constraint: `jceval` = 'A', 'N' or 'D'.

        (`errno` 1)
            On entry, `mu` = *<value>* and `neq` = *<value>*.

            Constraint: `mu` <= `neq`-1.

        (`errno` 1)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

        (`errno` 1)
            On entry, `ml` = *<value>* and `neq` = *<value>*.

            Constraint: `ml` <= `neq`-1.

        (`errno` 1)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 1)
            On entry, `nwkjac` = *<value>*, `ml` = *<value>*, `mu` =
            *<value>* and `neqmax` = *<value>*.

            Constraint: `nwkjac` >= (2*`ml`+`mu`+1)*`neqmax`.

        (`errno` 1)
            On entry, `njcpvt` = *<value>* and `neqmax` = *<value>*.

            Constraint: `njcpvt` >= `neqmax`.

        (`errno` 1)
            On entry, `neq` = *<value>* and `neqmax` = *<value>*.

            Constraint: `neq` <= `neqmax`.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_bandjac_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02ntft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_STRING, # Input, jceval(len=1).
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._BYREF_INT, # Input, nwkjac.
        utils._BYREF_INT, # Input, njcpvt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neqmax].
        utils._EngineCharArrayType.c_type, # Output, cwsav(len=80)[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jceval.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _jceval = utils._EngineCharScalarType(
        jceval, {'fun_name': fun_name, 'entity': 'jceval'}, exp_chrlen=1,
    ).to_ctypes()
    _ml = utils._EngineIntScalarType(
        ml, {'fun_name': fun_name, 'entity': 'ml'},
    ).to_ctypes()
    _mu = utils._EngineIntScalarType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
    ).to_ctypes()
    _nwkjac = utils._EngineIntScalarType(
        nwkjac, {'fun_name': fun_name, 'entity': 'nwkjac'},
    ).to_ctypes()
    _njcpvt = utils._EngineIntScalarType(
        njcpvt, {'fun_name': fun_name, 'entity': 'njcpvt'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('cwsav',),
        req_keys=('rwork',),
    )
    _rwork = comm['rwork']
    _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
    _cwsav_chrlen = 80
    comm['cwsav'] = _cwsav
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _neqmax,
        _jceval,
        _ml,
        _mu,
        _nwkjac,
        _njcpvt,
        _rwork,
        _cwsav,
        _errbuf,
        _ierr,
        _jceval._length_,
        _cwsav_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jceval = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jceval = \'A\', \'N\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mu <= neq-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mu >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ml <= neq-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ml >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwkjac = {:s}, ml = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'mu = {:s} and neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: nwkjac >= (2*ml+mu+1)*neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, njcpvt = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: njcpvt >= neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_stiff_sparjac_setup(neq, neqmax, jceval, nwkjac, ia, nia, ja, nja, comm, njcpvt, sens, u, eta, lblock, isplit):
    u"""
    Ordinary differential equations, initial value problem, for use with
    d02m--n functions, sparse Jacobian, linear algebra set up.

    For full information please refer to the NAG Library document for
    d02nu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nuf.html

    Parameters
    ----------
    neq : int
        The number of differential equations.

    neqmax : int
        A bound on the maximum number of differential equations to be
        solved during the integration.

    jceval : str, length 1
        Specifies the technique to be used to compute the Jacobian.

        `jceval` = 'N'

          The sparsity structure and the value of the Jacobian are to be
          determined numerically by the integrator.

        `jceval` = 'S'

          The sparsity structure of the Jacobian is supplied in the
          arrays `ia` and `ja` but its value is to be determined
          numerically. This is the recommended mode of operation unless
          it is a simple matter to supply the Jacobian.

        `jceval` = 'A'

          The Jacobian will be evaluated by calls to jac. The sparsity
          structure will be estimated by calls to jac; that is, no
          explicit sparsity structure need be supplied in the arrays
          `ia` and `ja`.

        `jceval` = 'F'

          The sparsity structure of the Jacobian is supplied in `ia` and
          `ja`, and its value will be determined by calls to jac. This
          is the recommended mode of operation if the jac is simple to
          form.

        `jceval` = 'D'

          The default choice is to be made. In this case 'D' is
          interpreted as 'S'.

        If the sparsity structure is supplied in arrays `ia` and `ja`,
        any evidence from the numerical or analytical formation of the
        Jacobian that this structure is not correct, is ignored.

        Only the first character of the actual argument `jceval` is
        passed to ``ivp_stiff_sparjac_setup``; hence it is permissible
        for the actual argument to be more descriptive, e.g.,
        'Numerical', 'Structural', 'Analytical', 'Full information' or
        'Default' in a call to ``ivp_stiff_sparjac_setup``.

        If the option `jceval` = 'N', 'S' or 'D' is used then the actual
        argument corresponding to jac in the call to
        ``ivp_stiff_exp_sparjac`` or ``ivp_stiff_imp_sparjac`` must be
        specified as None.

        If integration is to be performed by reverse communication
        (``ivp_stiff_exp_revcom`` or ``ivp_stiff_imp_revcom``) then
        `jceval` should be set to either 'N' or 'A'.

        In this case `ia` and `ja` are not used and their lengths may be
        set to 1.

    nwkjac : int
        The size of the array wkjac, which you are supplying to the
        integrator, as declared in the (sub)program from which
        ``ivp_stiff_sparjac_setup`` is called.

    ia : int, shape (`nia`)
        If `jceval` = 'S', 'F' or 'D', `ia` must contain details of the
        sparsity pattern to be used for the Jacobian. See `ja`.

        `ia` is not used if `jceval` = 'N' or 'A'.

    nia : int
        The dimension of the array `ia`.

    ja : int, shape (`nja`)
        If `jceval` = 'S', 'F' or 'D', `ja` must contain details of the
        sparsity pattern to be used for the Jacobian. `ja` contains the
        row indices where nonzero elements occur, reading in column-wise
        order, and `ia` contains the starting locations in `ja` of the
        descriptions of columns 1,2,...,`neq` in that order, with
        `ia`[0] = 1. Thus for each column index j = 1,2,...,`neq`, the
        values of the row index i in column j where a nonzero element
        may occur are given by

            i = `ja`[k]

        where `ia`[j-1] <= k < `ia`[j].

        Thus the total number of nonzeros, nelement, must be
        `ia`[`neq`]-1.

        For example, for the following matrix

            [table omitted]

        where x represents nonzero elements (13 in all) the arrays `ia`
        and `ja` should be

            [table omitted]

        `ja` is not used if `jceval` = 'N' or 'A'.

    nja : int
        The dimension of the array `ja`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    njcpvt : int
        The length of the array `comm`['jacpvt'], which you are
        supplying to the integrator, as dimensioned in the sub(program)
        from which ``ivp_stiff_sparjac_setup`` is called.

    sens : float
        A threshold argument used to determine whether or not a matrix
        element is zero; when `sens` is set to 0.0 on entry, the
        function will use `sens` = 100.0*machine precision. Otherwise
        the absolute value of `sens` is used.

    u : float
        Should have a value between 0.0 and 0.9999. Otherwise a default
        value of 0.1 is used. When the sparsity pattern has been
        evaluated, the first Jacobian computed is decomposed with `u`
        governing the choice of pivots; subsequent Jacobian
        decompositions use the same pattern of decomposition until the
        sparsity pattern is re-evaluated. When searching a row for a
        pivot, any element is excluded from the search which is less
        than `u` times the largest of those elements in the row
        available as pivots. Thus decreasing `u` biases the algorithm
        towards maintaining sparsity at the expense of numerical
        stability.

    eta : float
        A relative pivot threshold, below which on subsequent
        decompositions (as described under `u`), an internal error is
        provoked.

        `eta` > 1.0

          No check on pivot size is made.

        `eta` <= 0.0

          The default value `eta` = 1.0e-4 is used.

    lblock : bool
        Indicates if preordering is used before decomposition.

        If `lblock` = True, on entry, the Jacobian matrix is preordered
        to block lower triangular form before a decomposition is
        performed (this is the recommended mode).

        If you know the structure of the Jacobian to be irreducible,
        that is not permutable to block lower triangular form, you
        should set `lblock` = False.

        For example, a Jacobian arising from using the method of lines
        for parabolic partial differential equations would normally be
        irreducible. (See the specification of
        ``ivp_stiff_sparjac_diag`` for optional output concerning
        `lblock`.)

    isplit : int
        This argument is used for splitting the integer workspace
        `comm`['jacpvt'] to effect an efficient decomposition. It must
        satisfy 1 <= `isplit` <= 99. If `isplit` lies outside this range
        on entry, a default value of 73 is used. An appropriate value
        for `isplit` for subsequent runs on similar problems is
        available via the optional output ``ivp_stiff_sparjac_diag``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwkjac` = *<value>* and `ia`[`neq`]-1+2*`neq` =
            *<value>*.

            Constraint: `nwkjac` >= `ia`[`neq`]-1+2*`neq`.

        (`errno` 1)
            On entry, `ja` defines duplicate elements in row *<value>*
            and column *<value>*.

        (`errno` 1)
            On entry, i = *<value>*, `ja`[i] = *<value>* and `neq` =
            *<value>*.

            Constraint: 1 <= `ja`[i] <= `neq` for all i.

        (`errno` 1)
            On entry, i = *<value>*, `ia`[i] = *<value>*, i-1 =
            *<value>*, `ia`[i-1] = *<value>*, `neq` = *<value>*.

            Constraint: `ia`[*<value>*]-`ia`[*<value>*] <= `neq`.

        (`errno` 1)
            On entry, `ia`[*<value>*] = *<value>* and `ia`[*<value>*] =
            *<value>*.

            Constraint: `ia`[*<value>*] >= `ia`[*<value>*].

        (`errno` 1)
            On entry, `ia`[0] = *<value>*.

            Constraint: `ia`[0] = 1.

        (`errno` 1)
            On entry, `njcpvt` = *<value>* and
            3*(`ia`[`neq`]-1)+14*`neq`+1 = *<value>*.

            Constraint: `njcpvt` >= 3*(`ia`[`neq`]-1)+14*`neq`+1.

        (`errno` 1)
            On entry, `nja` = *<value>* and `ia`[`neq`]-1 = *<value>*.

            Constraint: `nja` >= `ia`[`neq`]-1.

        (`errno` 1)
            On entry, `ia`[`neq`]-1 = *<value>* and `neq` = *<value>*.

            Constraint: `ia`[`neq`]-1 >= `neq`.

        (`errno` 1)
            On entry, `nia` = *<value>* and `neq`+1 = *<value>*.

            Constraint: `nia` >= `neq`+1.

        (`errno` 1)
            On entry, `jceval` = *<value>*.

            Constraint: `jceval` = 'A', 'N', 'S', 'F' or 'D'.

        (`errno` 1)
            On entry, `neq` = *<value>* and `neqmax` = *<value>*.

            Constraint: `neq` <= `neqmax`.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_sparjac_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_STRING, # Input, jceval(len=1).
        utils._BYREF_INT, # Input, nwkjac.
        utils._EngineIntArrayType.c_type, # Input, ia[nia].
        utils._BYREF_INT, # Input, nia.
        utils._EngineIntArrayType.c_type, # Input, ja[nja].
        utils._BYREF_INT, # Input, nja.
        utils._EngineIntArrayType.c_type, # Output, jacpvt[njcpvt].
        utils._BYREF_INT, # Input, njcpvt.
        utils._BYREF_FLOAT64, # Input, sens.
        utils._BYREF_FLOAT64, # Input, u.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._BYREF_LOGICAL, # Input, lblock.
        utils._BYREF_INT, # Input, isplit.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neqmax].
        utils._EngineCharArrayType.c_type, # Output, cwsav(len=80)[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jceval.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _jceval = utils._EngineCharScalarType(
        jceval, {'fun_name': fun_name, 'entity': 'jceval'}, exp_chrlen=1,
    ).to_ctypes()
    _nwkjac = utils._EngineIntScalarType(
        nwkjac, {'fun_name': fun_name, 'entity': 'nwkjac'},
    ).to_ctypes()
    _nia = utils._EngineIntScalarType(
        nia, {'fun_name': fun_name, 'entity': 'nia'},
    ).to_ctypes()
    _nia_py = utils._EngineIntScalarType.to_py(_nia)
    _ia = utils._EngineIntArrayType(
        ia, {'fun_name': fun_name, 'entity': 'ia'},
        exp_shape=(_nia_py,),
    ).to_ctypes()
    _nja = utils._EngineIntScalarType(
        nja, {'fun_name': fun_name, 'entity': 'nja'},
    ).to_ctypes()
    _nja_py = utils._EngineIntScalarType.to_py(_nja)
    _ja = utils._EngineIntArrayType(
        ja, {'fun_name': fun_name, 'entity': 'ja'},
        exp_shape=(_nja_py,),
    ).to_ctypes()
    _njcpvt = utils._EngineIntScalarType(
        njcpvt, {'fun_name': fun_name, 'entity': 'njcpvt'},
    ).to_ctypes()
    _njcpvt_py = utils._EngineIntScalarType.to_py(_njcpvt)
    _jacpvt = utils._EngineIntArrayType.empty_ctypes(_njcpvt_py)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('jacpvt', 'cwsav'),
        req_keys=('rwork',),
    )
    comm['jacpvt'] = _jacpvt
    _sens = utils._EngineFloat64ScalarType(
        sens, {'fun_name': fun_name, 'entity': 'sens'},
    ).to_ctypes()
    _u = utils._EngineFloat64ScalarType(
        u, {'fun_name': fun_name, 'entity': 'u'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    _lblock = utils._EngineLogicalScalarType(
        lblock, {'fun_name': fun_name, 'entity': 'lblock'},
    ).to_ctypes()
    _isplit = utils._EngineIntScalarType(
        isplit, {'fun_name': fun_name, 'entity': 'isplit'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _cwsav = utils._EngineCharArrayType.empty_ctypes(10, chrlen=80)
    _cwsav_chrlen = 80
    comm['cwsav'] = _cwsav
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _neqmax,
        _jceval,
        _nwkjac,
        _ia,
        _nia,
        _ja,
        _nja,
        _jacpvt,
        _njcpvt,
        _sens,
        _u,
        _eta,
        _lblock,
        _isplit,
        _rwork,
        _cwsav,
        _errbuf,
        _ierr,
        _jceval._length_,
        _cwsav_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99882:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwkjac = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ia[neq]-1+2*neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwkjac >= ia[neq]-1+2*neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99883:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ja defines duplicate elements in row {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'column {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99885:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, ja[i] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= ja[i] <= neq for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99887:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, ia[i] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'i-1 = {:s}, ia[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[6]),
                    ),
                    'neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[7]),
                    ),
                    'Constraint: ia[{:s}]-ia[{:s}] <= neq.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99889:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia[{:s}] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'ia[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[6]),
                    ),
                    'Constraint: ia[{:s}] >= ia[{:s}].'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99890:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ia[0] = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99891:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, njcpvt = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '3*(ia[neq]-1)+14*neq+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: njcpvt >= 3*(ia[neq]-1)+14*neq+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99892:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nja = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ia[neq]-1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nja >= ia[neq]-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99893:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ia[neq]-1 = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ia[neq]-1 >= neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99894:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nia = {:s} and neq+1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nia >= neq+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jceval = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jceval = \'A\', \'N\', \'S\', \'F\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_stiff_bdf(neqmax, sdysav, maxord, method, petzld, con, tcrit, hmin, hmax, h0, maxstp, mxhnil, norm):
    u"""
    Ordinary differential equations, initial value problem, backward
    differentiation formulae method, setup for d02m--n functions.

    For full information please refer to the NAG Library document for
    d02nv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nvf.html

    Parameters
    ----------
    neqmax : int
        A bound on the maximum number of differential equations to be
        solved.

    sdysav : int
        The second dimension of the array ysav that will be supplied to
        the integrator, as declared in the (sub)program from which the
        integrator is called.

    maxord : int
        The maximum order to be used for the BDF method.

    method : str, length 1
        Specifies the method to be used to solve the system of nonlinear
        equations arising on each step of the BDF code.

        `method` = 'N'

          A modified Newton iteration is used.

        `method` = 'F'

          Functional iteration is used.

        `method` = 'D'

          A modified Newton iteration is used.

        Note: a linear algebra setup function must be called even when
        using functional iteration, since if difficulty is encountered a
        switch is made to a modified Newton method.

        Only the first character of the actual argument `method` is
        passed to ``ivp_stiff_bdf``; hence it is permissible for the
        actual argument to be more descriptive e.g., 'Newton',
        'Functional iteration' or 'Default' in a call to
        ``ivp_stiff_bdf``.

    petzld : bool
        Specifies whether the Petzold local error test is to be used. If
        `petzld` is set to True on entry, the Petzold local error test
        is used, otherwise a conventional test is used. The Petzold test
        results in extra overhead cost but is more stable and reliable
        for differential/algebraic equations.

    con : float, shape (6), modified in place
        On entry: values to be used to control step size choice during
        integration. If any `con`[i-1] = 0.0 on entry, it is replaced by
        its default value described below. In most cases this is the
        recommended setting.

        `con`[0], `con`[1], and `con`[2] are factors used to bound step
        size changes.

        If the current step size h fails, the modulus of the next step
        size is bounded by `con`[0]*|h|.

        The default value of `con`[0] is 2.0.

        Note that the new step size may be used with a method of
        different order to the failed step.

        If the initial step size is h, the modulus of the step size on
        the second step is bounded by `con`[2]*|h|.

        At any other stage in the integration, if the current step size
        is h, the modulus of the next step size is bounded by
        `con`[1]*|h|.

        The default values are 10.0 for `con`[1] and 1000.0 for
        `con`[2].

        `con`[3], `con`[4] and `con`[5] are 'tuning' constants used in
        determining the next order and step size.

        They are used to scale the error estimates used in determining
        whether to keep the same order of the BDF method, decrease the
        order or increase the order respectively.

        The larger the value of `con`[i-1], for i = 4,5,...,6, the less
        likely the choice of the corresponding order.

        The default values are: `con`[3] = 1.2, `con`[4] = 1.3, `con`[5]
        = 1.4.

        On exit: the values actually used by ``ivp_stiff_bdf``.

    tcrit : float
        A point beyond which integration must not be attempted. The use
        of `tcrit` is described under the argument itask in the
        specification for the integrator (e.g., see
        ``ivp_stiff_exp_fulljac``). A value, 0.0 say, must be specified
        even if itask subsequently specifies that `tcrit` will not be
        used.

    hmin : float
        The minimum absolute step size to be allowed. Set `hmin` = 0.0
        if this option is not required.

    hmax : float
        The maximum absolute step size to be allowed. Set `hmax` = 0.0
        if this option is not required.

    h0 : float
        The step size to be attempted on the first step. Set `h0` = 0.0
        if the initial step size is calculated internally.

    maxstp : int
        The maximum number of steps to be attempted during one call to
        the integrator after which it will return with `errno` = 2. Set
        `maxstp` = 0 if no limit is to be imposed.

    mxhnil : int
        The maximum number of warnings printed (if itrace >= 0) per
        problem when t+h = t on a step (h = current step size). If
        `mxhnil` <= 0, a default value of 10 is assumed.

    norm : str, length 1
        Indicates the type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L2 norm.

        `norm` = 'D'

          Is the same as `norm` = 'A'.

        If vnorm denotes the norm of the vector v of length n, for the
        averaged L2 norm

            vnorm = sqrt(1/nsum_i = 1^n(v_i/w_i)^2),

        while for the maximum norm

            vnorm = max_i|v_i/w_i|.

        If you wish to weight the maximum norm or the L2 norm, rtol and
        atol should be scaled appropriately on input to the integrator
        (see under itol in the specification of the integrator for the
        formulation of the weight vector w_i from rtol and atol, e.g.,
        see ``ivp_stiff_exp_fulljac``).

        Only the first character to the actual argument `norm` is passed
        to ``ivp_stiff_bdf``; hence it is permissible for the actual
        argument to be more descriptive e.g., 'Maximum', 'Average L2' or
        'Default' in a call to ``ivp_stiff_bdf``.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `con`[1] = *<value>* and `con`[2] = *<value>*.

            Constraint: `con`[1] <= `con`[2].

        (`errno` 1)
            On entry, `con`[0] = *<value>* and `con`[1] = *<value>*.

            Constraint: `con`[0] <= `con`[1].

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>* was less than 1.0.

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>* was less than 0.0.

        (`errno` 1)
            On entry, `method` = *<value>* and `maxord` = *<value>*.

            Constraint: `maxord` <= *<value>*, the maximum allowed order
            for the method define by `method`.

        (`errno` 1)
            On entry, `maxord` = *<value>*.

            Constraint: `maxord` >= 1.

        (`errno` 1)
            On entry, `sdysav` = *<value>* and `maxord` = *<value>*.

            Constraint: `sdysav` > `maxord`.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'M', 'A' or 'D'.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'N', 'F' or 'D'.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.

    See Also
    --------
    :meth:`naginterfaces.library.examples.ode.ivp_stiff_exp_fulljac_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_bdf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_INT, # Input, maxord.
        utils._BYREF_STRING, # Input, method(len=1).
        utils._BYREF_LOGICAL, # Input, petzld.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, con[6].
        utils._BYREF_FLOAT64, # Input, tcrit.
        utils._BYREF_FLOAT64, # Input, hmin.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_FLOAT64, # Input, h0.
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_INT, # Input, mxhnil.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[50+4*neqmax].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of method.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _maxord = utils._EngineIntScalarType(
        maxord, {'fun_name': fun_name, 'entity': 'maxord'},
    ).to_ctypes()
    _method = utils._EngineCharScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'}, exp_chrlen=1,
    ).to_ctypes()
    _petzld = utils._EngineLogicalScalarType(
        petzld, {'fun_name': fun_name, 'entity': 'petzld'},
    ).to_ctypes()
    _con_cls = utils._EngineFloat64ArrayType(
        con, {'fun_name': fun_name, 'entity': 'con'},
        exp_shape=(6,),
    )
    _con = _con_cls.to_ctypes()
    _tcrit = utils._EngineFloat64ScalarType(
        tcrit, {'fun_name': fun_name, 'entity': 'tcrit'},
    ).to_ctypes()
    _hmin = utils._EngineFloat64ScalarType(
        hmin, {'fun_name': fun_name, 'entity': 'hmin'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _h0 = utils._EngineFloat64ScalarType(
        h0, {'fun_name': fun_name, 'entity': 'h0'},
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _mxhnil = utils._EngineIntScalarType(
        mxhnil, {'fun_name': fun_name, 'entity': 'mxhnil'},
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _neqmax_py = utils._EngineIntScalarType.to_py(_neqmax)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(50+4*_neqmax_py)
    _comm_py = {}
    _comm_py['rwork'] = _rwork
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqmax,
        _sdysav,
        _maxord,
        _method,
        _petzld,
        _con,
        _tcrit,
        _hmin,
        _hmax,
        _h0,
        _maxstp,
        _mxhnil,
        _norm,
        _rwork,
        _errbuf,
        _ierr,
        _method._length_,
        _norm._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'con[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con[1] <= con[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'con[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con[0] <= con[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s} was less'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'than 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s} was less'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'than 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = \'{:s}\' and'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxord <= {:s}, the maximum allowed order for'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'the method define by method.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxord >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sdysav = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdysav > maxord.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'M\', \'A\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = \'N\', \'F\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _con_cls.refresh_py(_con)
        return_data = _comm_py
    _err_class.manage()
    return return_data

def ivp_stiff_blend(neqmax, sdysav, maxord, con, tcrit, hmin, hmax, h0, maxstp, mxhnil, norm):
    u"""
    Ordinary differential equations, initial value problem, Blend
    method, setup for d02m--n functions.

    For full information please refer to the NAG Library document for
    d02nw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nwf.html

    Parameters
    ----------
    neqmax : int
        A bound on the maximum number of differential equations to be
        solved.

    sdysav : int
        The second dimension of the array ysav that will be supplied to
        the integrator, as declared in the (sub)program from which the
        integrator is called (e.g., see ``ivp_stiff_exp_fulljac``).

    maxord : int
        The maximum order to be used for the BLEND method.

    con : float, shape (6), modified in place
        On entry: values to be used to control step size choice during
        integration. If any `con`[i-1] = 0.0 on entry, it is replaced by
        its default value described below. In most cases this is the
        recommended setting.

        `con`[0], `con`[1], and `con`[2] are factors used to bound step
        size changes.

        If the current step size h fails, the modulus of the next step
        size is bounded by `con`[0]*|h|.

        The default value of `con`[0] is 2.0.

        Note that the new step size may be used with a method of
        different order to the failed step.

        If the initial step size is h, the modulus of the step size on
        the second step is bounded by `con`[2]*|h|.

        At any other stage in the integration, if the current step size
        is h, the modulus of the next step size is bounded by
        `con`[1]*|h|.

        The default values are 10.0 for `con`[1] and 1000.0 for
        `con`[2].

        `con`[3], `con`[4] and `con`[5] are 'tuning' constants used in
        determining the next order and step size.

        They are used to scale the error estimates used in determining
        whether to keep the same order of the BLEND method, decrease the
        order or increase the order respectively.

        The larger the value of `con`[i-1], for i = 4,5,...,6, the less
        likely the choice of the corresponding order.

        The default values are: `con`[3] = 1.2, `con`[4] = 1.3, `con`[5]
        = 1.4.

        On exit: the values actually used by ``ivp_stiff_blend``.

    tcrit : float
        A point beyond which integration must not be attempted. The use
        of `tcrit` is described under the argument itask in the
        specification for the integrator (e.g., see
        ``ivp_stiff_exp_fulljac``). A value, 0.0 say, must be specified
        even if itask subsequently specifies that `tcrit` will not be
        used.

    hmin : float
        The minimum absolute step size to be allowed. Set `hmin` = 0.0
        if this option is not required.

    hmax : float
        The maximum absolute step size to be allowed. Set `hmax` = 0.0
        if this option is not required.

    h0 : float
        The step size to be attempted on the first step. Set `h0` = 0.0
        if the initial step size is calculated internally.

    maxstp : int
        The maximum number of steps to be attempted during one call to
        the integrator after which it will return with `errno` = 2. Set
        `maxstp` = 0 if no limit is to be imposed.

    mxhnil : int
        The maximum number of warnings printed (if itrace >= 0, e.g.,
        see ``ivp_stiff_exp_fulljac``) per problem when t+h = t on a
        step (h = current step size). If `mxhnil` <= 0, a default value
        of 10 is assumed.

    norm : str, length 1
        Indicates the type of norm to be used.

        `norm` = 'M'

          Maximum norm.

        `norm` = 'A'

          Averaged L2 norm.

        `norm` = 'D'

          Is the same as 'A'.

        If vnorm denotes the norm of the vector v of length n, for the
        averaged L2 norm

            vnorm = sqrt(1/nsum_i = 1^n(v_i/w_i)^2),

        while for the maximum norm

            vnorm = max_i|v_i/w_i|.

        If you wish to weight the maximum norm or the L2 norm, rtol and
        atol should be scaled appropriately on input to the integrator
        (see under itol in the specification of the integrator for the
        formulation of the weight vector w_i from rtol and atol, e.g.,
        ``ivp_stiff_exp_fulljac``).

        Only the first character of the actual argument `norm` is passed
        to ``ivp_stiff_blend``; hence it is permissible for the actual
        argument to be more descriptive e.g., 'Maximum', 'Average L2' or
        'Default' in a call to ``ivp_stiff_blend``.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `con`[1] = *<value>* and `con`[2] = *<value>*.

            Constraint: `con`[1] <= `con`[2].

        (`errno` 1)
            On entry, `maxord` = *<value>* and `sdysav` = *<value>*.

            Constraint: `sdysav` >= `maxord`+3.

        (`errno` 1)
            On entry, `maxord` = *<value>*.

            Constraint: 0 < `maxord` <= 11.

        (`errno` 1)
            On entry, `con`[0] = *<value>* and `con`[1] = *<value>*.

            Constraint: `con`[0] <= `con`[1].

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>* was less than 1.0.

        (`errno` 1)
            On entry, `con`[*<value>*] = *<value>* was less than 0.0.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = 'M', 'A' or 'D'.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_blend'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_INT, # Input, maxord.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, con[6].
        utils._BYREF_FLOAT64, # Input, tcrit.
        utils._BYREF_FLOAT64, # Input, hmin.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_FLOAT64, # Input, h0.
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_INT, # Input, mxhnil.
        utils._BYREF_STRING, # Input, norm(len=1).
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[50+4*neqmax].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of norm.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    _maxord = utils._EngineIntScalarType(
        maxord, {'fun_name': fun_name, 'entity': 'maxord'},
    ).to_ctypes()
    _con_cls = utils._EngineFloat64ArrayType(
        con, {'fun_name': fun_name, 'entity': 'con'},
        exp_shape=(6,),
    )
    _con = _con_cls.to_ctypes()
    _tcrit = utils._EngineFloat64ScalarType(
        tcrit, {'fun_name': fun_name, 'entity': 'tcrit'},
    ).to_ctypes()
    _hmin = utils._EngineFloat64ScalarType(
        hmin, {'fun_name': fun_name, 'entity': 'hmin'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _h0 = utils._EngineFloat64ScalarType(
        h0, {'fun_name': fun_name, 'entity': 'h0'},
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _mxhnil = utils._EngineIntScalarType(
        mxhnil, {'fun_name': fun_name, 'entity': 'mxhnil'},
    ).to_ctypes()
    _norm = utils._EngineCharScalarType(
        norm, {'fun_name': fun_name, 'entity': 'norm'}, exp_chrlen=1,
    ).to_ctypes()
    _neqmax_py = utils._EngineIntScalarType.to_py(_neqmax)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(50+4*_neqmax_py)
    _comm_py = {}
    _comm_py['rwork'] = _rwork
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqmax,
        _sdysav,
        _maxord,
        _con,
        _tcrit,
        _hmin,
        _hmax,
        _h0,
        _maxstp,
        _mxhnil,
        _norm,
        _rwork,
        _errbuf,
        _ierr,
        _norm._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[1] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'con[2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con[1] <= con[2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'sdysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdysav >= maxord+3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 < maxord <= 11.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'con[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: con[0] <= con[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s} was less'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'than 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, con[{:s}] = {:s} was less'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'than 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, norm = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: norm = \'M\', \'A\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _con_cls.refresh_py(_con)
        return_data = _comm_py
    _err_class.manage()
    return return_data

def ivp_stiff_sparjac_diag(icall, lblock, comm):
    u"""
    Ordinary differential equations, initial value problem, sparse
    Jacobian, linear algebra diagnostics, for use with d02m--n
    functions.

    For full information please refer to the NAG Library document for
    d02nx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nxf.html

    Parameters
    ----------
    icall : int
        Indicates whether or not all output arguments have been set
        during the call to the integrator. If so, that is, if the
        integrator returned with the function exits successfully or
        `errno` = 12, then `icall` must be set to 0. Otherwise `icall`
        must be set to 1, indicating that integration did not take place
        due to lack of space in arrays wkjac and jacpvt, and only
        `liwreq`, `liwusd`, `lrwreq`, `lrwusd` have been set.

    lblock : bool
        The value used for the argument `lblock` when calling
        ``ivp_stiff_sparjac_setup``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_exp_sparjac``, ``ivp_stiff_imp_sparjac`` or
        ``ivp_stiff_imp_revcom``.

    Returns
    -------
    liwreq : int
        The length of the int workspace jacpvt reserved for the sparse
        matrix functions.

    liwusd : int
        The length of the int workspace jacpvt actually used by the
        sparse matrix functions.

    lrwreq : int
        The length of the double workspace wkjac reserved for the sparse
        matrix functions.

    lrwusd : int
        The length of the double workspace wkjac actually used by the
        sparse matrix functions.

    nlu : int
        The number of LU decompositions done during the integration.

    nnz : int
        The number of nonzeros in the Jacobian.

    ngp : int
        The number of fcn or resid calls needed to form the Jacobian.

    isplit : int
        An appropriate value for the argument `isplit` when calling
        ``ivp_stiff_sparjac_setup`` for subsequent runs of similar
        problems.

    igrow : int
        An estimate of the growth of the elements encountered during the
        last LU decomposition performed. If the actual estimate exceeds
        the largest possible integer value for the machine being used
        (see ``machine.integer_max``) `igrow` is set to the value
        returned by ``machine.integer_max``.

    nblock : int
        If `lblock` = True, `nblock` contains the number of diagonal
        blocks in the Jacobian matrix permuted to block lower triangular
        form. If `nblock` = 1 then on subsequent runs of a similar
        problem `lblock` should be set to False in the call to
        ``ivp_stiff_sparjac_setup``.

        If `lblock` = False, `nblock` = 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_sparjac_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, icall.
        utils._BYREF_INT, # Output, liwreq.
        utils._BYREF_INT, # Output, liwusd.
        utils._BYREF_INT, # Output, lrwreq.
        utils._BYREF_INT, # Output, lrwusd.
        utils._BYREF_INT, # Output, nlu.
        utils._BYREF_INT, # Output, nnz.
        utils._BYREF_INT, # Output, ngp.
        utils._BYREF_INT, # Output, isplit.
        utils._BYREF_INT, # Output, igrow.
        utils._BYREF_LOGICAL, # Input, lblock.
        utils._BYREF_INT, # Output, nblock.
        utils._EngineIntArrayType.c_type, # Input, inform[23].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _icall = utils._EngineIntScalarType(
        icall, {'fun_name': fun_name, 'entity': 'icall'},
    ).to_ctypes()
    _liwreq = utils._EngineIntScalarType.empty_ctypes()
    _liwusd = utils._EngineIntScalarType.empty_ctypes()
    _lrwreq = utils._EngineIntScalarType.empty_ctypes()
    _lrwusd = utils._EngineIntScalarType.empty_ctypes()
    _nlu = utils._EngineIntScalarType.empty_ctypes()
    _nnz = utils._EngineIntScalarType.empty_ctypes()
    _ngp = utils._EngineIntScalarType.empty_ctypes()
    _isplit = utils._EngineIntScalarType.empty_ctypes()
    _igrow = utils._EngineIntScalarType.empty_ctypes()
    _lblock = utils._EngineLogicalScalarType(
        lblock, {'fun_name': fun_name, 'entity': 'lblock'},
    ).to_ctypes()
    _nblock = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('inform',),
    )
    _inform = comm['inform']
    engine_ctypes(
        _en_data,
        _icall,
        _liwreq,
        _liwusd,
        _lrwreq,
        _lrwusd,
        _nlu,
        _nnz,
        _ngp,
        _isplit,
        _igrow,
        _lblock,
        _nblock,
        _inform,
    )
    utils._handle_cb_exception(excs)
    _liwreq_py = utils._EngineIntScalarType.to_py(_liwreq)
    _liwusd_py = utils._EngineIntScalarType.to_py(_liwusd)
    _lrwreq_py = utils._EngineIntScalarType.to_py(_lrwreq)
    _lrwusd_py = utils._EngineIntScalarType.to_py(_lrwusd)
    _nlu_py = utils._EngineIntScalarType.to_py(_nlu)
    _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
    _ngp_py = utils._EngineIntScalarType.to_py(_ngp)
    _isplit_py = utils._EngineIntScalarType.to_py(_isplit)
    _igrow_py = utils._EngineIntScalarType.to_py(_igrow)
    _nblock_py = utils._EngineIntScalarType.to_py(_nblock)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['liwreq', 'liwusd', 'lrwreq', 'lrwusd', 'nlu', 'nnz', 'ngp', 'isplit', 'igrow', 'nblock'],
    )(
        _liwreq_py,
        _liwusd_py,
        _lrwreq_py,
        _lrwusd_py,
        _nlu_py,
        _nnz_py,
        _ngp_py,
        _isplit_py,
        _igrow_py,
        _nblock_py,
    )
    return return_data

def ivp_stiff_integ_diag(neq, neqmax, comm, algequ):
    u"""
    Ordinary differential equations, initial value problem, integrator
    diagnostics, for use with d02m--n functions.

    For full information please refer to the NAG Library document for
    d02ny

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nyf.html

    Parameters
    ----------
    neq : int
        The value used for the argument `neq` when calling the
        integrator.

    neqmax : int
        The value used for the argument `neqmax` when calling the
        integrator.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_exp_fulljac``, ``ivp_stiff_exp_bandjac``,
        ``ivp_stiff_exp_sparjac``, ``ivp_stiff_imp_fulljac``,
        ``ivp_stiff_imp_bandjac``, ``ivp_stiff_imp_sparjac``,
        ``ivp_stiff_exp_revcom`` or ``ivp_stiff_imp_revcom``.

    algequ : bool, shape (`neq`), modified in place
        On exit: `algequ`[i-1] = True if the ith equation integrated was
        detected to be algebraic, otherwise `algequ`[i-1] = False. Note
        that when the integrators for explicit equations are being used,
        then `algequ`[i-1] = False, for i = 1,2,...,`neq`.

    Returns
    -------
    hu : float
        The last successful step size.

    h : float
        The proposed next step size for continuing the integration.

    tcur : float
        t, the value of the independent variable which the integrator
        has actually reached. `tcur` will always be at least as far as
        the output value of the argument t in the direction of
        integration, but may be further (if overshooting and
        interpolation at tout was specified, e.g., see
        ``ivp_stiff_exp_fulljac``).

    tolsf : float
        A tolerance scale factor, `tolsf` >= 1.0, which is computed when
        a request for too much accuracy is detected by the integrator
        (indicated by a return with `errno` = 3 or 14). If itol is left
        unaltered but rtol and atol are uniformly scaled up by a factor
        of this argument the next call to the integrator is deemed
        likely to succeed.

    nst : int
        The number of steps taken in the integration so far.

    nre : int
        The number of function or residual evaluations (fcn (e.g., see
        ``ivp_stiff_exp_fulljac``) or resid (e.g., see
        ``ivp_stiff_imp_fulljac``) calls) used in the integration so
        far.

    nje : int
        The number of Jacobian evaluations used in the integration so
        far. This equals the number of matrix LU decompositions.

    nqu : int
        The order of the method last used (successfully) in the
        integration.

    nq : int
        The proposed order of the method for continuing the integration.

    niter : int
        The number of iterations performed in the integration so far by
        the nonlinear equation solver.

    imxer : int
        The index of the component of largest magnitude in the weighted
        local error vector (e_i/w_i), for i = 1,2,...,`neq`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neq` = *<value>* and `neqmax` = *<value>*.

            Constraint: `neq` <= `neqmax`.

        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_integ_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_FLOAT64, # Output, hu.
        utils._BYREF_FLOAT64, # Output, h.
        utils._BYREF_FLOAT64, # Output, tcur.
        utils._BYREF_FLOAT64, # Output, tolsf.
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[50+4*neq].
        utils._BYREF_INT, # Output, nst.
        utils._BYREF_INT, # Output, nre.
        utils._BYREF_INT, # Output, nje.
        utils._BYREF_INT, # Output, nqu.
        utils._BYREF_INT, # Output, nq.
        utils._BYREF_INT, # Output, niter.
        utils._BYREF_INT, # Output, imxer.
        utils._EngineLogicalArrayType.c_type, # Output, algequ[neq].
        utils._EngineIntArrayType.c_type, # Input, inform[23].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _hu = utils._EngineFloat64ScalarType.empty_ctypes()
    _h = utils._EngineFloat64ScalarType.empty_ctypes()
    _tcur = utils._EngineFloat64ScalarType.empty_ctypes()
    _tolsf = utils._EngineFloat64ScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'inform'),
    )
    _rwork = comm['rwork']
    _nst = utils._EngineIntScalarType.empty_ctypes()
    _nre = utils._EngineIntScalarType.empty_ctypes()
    _nje = utils._EngineIntScalarType.empty_ctypes()
    _nqu = utils._EngineIntScalarType.empty_ctypes()
    _nq = utils._EngineIntScalarType.empty_ctypes()
    _niter = utils._EngineIntScalarType.empty_ctypes()
    _imxer = utils._EngineIntScalarType.empty_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _algequ_cls = utils._EngineLogicalArrayType(
        algequ, {'fun_name': fun_name, 'entity': 'algequ'},
        exp_shape=(_neq_py,),
        check_contents=False,
    )
    _algequ = _algequ_cls.to_ctypes(can_reference_contents=False)
    _inform = comm['inform']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _neqmax,
        _hu,
        _h,
        _tcur,
        _tolsf,
        _rwork,
        _nst,
        _nre,
        _nje,
        _nqu,
        _nq,
        _niter,
        _imxer,
        _algequ,
        _inform,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= neqmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _algequ_cls.refresh_py(_algequ)
        _hu_py = utils._EngineFloat64ScalarType.to_py(_hu)
        _h_py = utils._EngineFloat64ScalarType.to_py(_h)
        _tcur_py = utils._EngineFloat64ScalarType.to_py(_tcur)
        _tolsf_py = utils._EngineFloat64ScalarType.to_py(_tolsf)
        _nst_py = utils._EngineIntScalarType.to_py(_nst)
        _nre_py = utils._EngineIntScalarType.to_py(_nre)
        _nje_py = utils._EngineIntScalarType.to_py(_nje)
        _nqu_py = utils._EngineIntScalarType.to_py(_nqu)
        _nq_py = utils._EngineIntScalarType.to_py(_nq)
        _niter_py = utils._EngineIntScalarType.to_py(_niter)
        _imxer_py = utils._EngineIntScalarType.to_py(_imxer)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['hu', 'h', 'tcur', 'tolsf', 'nst', 'nre', 'nje', 'nqu', 'nq', 'niter', 'imxer'],
        )(
            _hu_py,
            _h_py,
            _tcur_py,
            _tolsf_py,
            _nst_py,
            _nre_py,
            _nje_py,
            _nqu_py,
            _nq_py,
            _niter_py,
            _imxer_py,
        )
    _err_class.manage()
    return return_data

def ivp_stiff_contin(neqmax, tcrit, h, hmin, hmax, maxstp, mxhnil, comm):
    u"""
    Ordinary differential equations, initial value problem, setup for
    continuation calls to integrator, for use with d02m--n functions.

    For full information please refer to the NAG Library document for
    d02nz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02nzf.html

    Parameters
    ----------
    neqmax : int
        The value used for the argument `neqmax` when calling the
        integrator.

    tcrit : float
        A point beyond which integration must not be attempted. The use
        of `tcrit` is described under the argument itask in the
        specification for the integrator (e.g., see
        ``ivp_stiff_exp_fulljac``). A value, 0.0 say, must be specified
        even if itask subsequently specifies that `tcrit` will not be
        used.

    h : float
        The next step size to be attempted. Set `h` = 0.0 if the current
        value of `h` is not to be changed.

    hmin : float
        The minimum absolute step size to be allowed. Set `hmin` = 0.0
        if this option is not required. Set `hmin` < 0.0 if the current
        value of `hmin` is not to be changed.

    hmax : float
        The maximum absolute step size to be allowed. Set `hmax` = 0.0
        if this option is not required. Set `hmax` < 0.0 if the current
        value of `hmax` is not to be changed.

    maxstp : int
        The maximum number of steps to be attempted during one call to
        the integrator after which it will return with `errno` = 2 (see
        ``ivp_stiff_exp_bandjac``). Set `maxstp` = 0 if this option is
        not required. Set `maxstp` < 0 if the current value of `maxstp`
        is not to be changed.

    mxhnil : int
        The maximum number of warnings printed (if itrace >= 0, e.g.,
        see ``ivp_stiff_exp_fulljac``) per problem when t+h = t on a
        step (h = current step size). If `mxhnil` <= 0, a default value
        of 10 is assumed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_stiff_dassl``, ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neqmax` = *<value>*.

            Constraint: `neqmax` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_contin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02nzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqmax.
        utils._BYREF_FLOAT64, # Input, tcrit.
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, hmin.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_INT, # Input, mxhnil.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[50+4*neqmax].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqmax = utils._EngineIntScalarType(
        neqmax, {'fun_name': fun_name, 'entity': 'neqmax'},
    ).to_ctypes()
    _tcrit = utils._EngineFloat64ScalarType(
        tcrit, {'fun_name': fun_name, 'entity': 'tcrit'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _hmin = utils._EngineFloat64ScalarType(
        hmin, {'fun_name': fun_name, 'entity': 'hmin'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _mxhnil = utils._EngineIntScalarType(
        mxhnil, {'fun_name': fun_name, 'entity': 'mxhnil'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork',),
    )
    _rwork = comm['rwork']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqmax,
        _tcrit,
        _h,
        _hmin,
        _hmax,
        _maxstp,
        _mxhnil,
        _rwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_rkts_range(f, n, twant, ygot, ypgot, ymax, comm, data=None):
    u"""
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration over range with output.

    For full information please refer to the NAG Library document for
    d02pe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02pef.html

    Parameters
    ----------
    f : callable f(t, n, y, yp, data=None)
        `f` must evaluate the functions f_i (that is the first
        derivatives y'_i) for given values of the arguments t, y_i.

        Parameters
        ~~~~~~~~~~
        t : float
            t, the current value of the independent variable.

        n : int
            n, the number of ordinary differential equations in the
            system to be solved.

        y : float, ndarray, shape (`n`)
            The current values of the dependent variables, y_i, for i =
            1,2,...,n.

        yp : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of ordinary differential equations in the system
        to be solved.

    twant : float
        t, the next value of the independent variable where a solution
        is desired.

    ygot : float, shape (`n`), modified in place
        On entry: on the first call to ``ivp_rkts_range``, `ygot` need
        not be set. On all subsequent calls `ygot` must remain
        unchanged.

        On exit: an approximation to the true solution at the value of
        `tgot`. At each step of the integration to `tgot`, the local
        error has been controlled as specified in ``ivp_rkts_setup``.
        The local error has still been controlled even when `tgot` !=
        `twant`, that is after a return with `errno` > 1.

    ypgot : float, shape (`n`), modified in place
        On exit: an approximation to the first derivative of the true
        solution at `tgot`.

    ymax : float, shape (`n`), modified in place
        On entry: on the first call to ``ivp_rkts_range``, `ymax` need
        not be set. On all subsequent calls `ymax` must remain
        unchanged.

        On exit: `ymax`[i-1] contains the largest value of |y_i|
        computed at any step in the integration so far.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    tgot : float
        t, the value of the independent variable at which a solution has
        been computed. On successful exit with no exception or warning
        is raised, `tgot` will equal `twant`. On exit with `errno` > 1,
        a solution has still been computed at the value of `tgot` but in
        general `tgot` will not equal `twant`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            On entry, the communication arrays have become corrupted, or
            a catastrophic error has already been detected elsewhere.
            You cannot continue integrating the problem.

        (`errno` 1)
            `twant` is too close to the last value of `tgot` (tstart on
            setup).

            When using the method of order 8 at setup, these must differ
            by at least *<value>*. Their absolute difference is
            *<value>*.

        (`errno` 1)
            `twant` lies beyond tend (setup) in the direction of
            integration.

            `twant` = *<value>* and tend = *<value>*.

        (`errno` 1)
            `twant` lies beyond tend (setup) in the direction of
            integration, but is very close to tend.

            You may have intended `twant` = tend.

            abs(`twant`-tend) = *<value>*.

        (`errno` 1)
            `twant` does not lie in the direction of integration.
            `twant` = *<value>*.

        (`errno` 1)
            tend (setup) had already been reached in a previous call.

            To start a new problem, you will need to call the setup
            function.

        (`errno` 1)
            You cannot call this function after it has returned an
            error.

            You must call the setup function to start another problem.

        (`errno` 1)
            You cannot call this function when you have specified, in
            the setup function, that the step integrator will be used.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            This function is being used inefficiently because the step
            size has been reduced drastically many times to obtain
            answers at many points. Using the order 4 and 5 pair method
            at setup is more appropriate here. You can continue
            integrating this problem.

        (`errno` 3)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. However, you can
            continue integrating the problem.

        (`errno` 4)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. Your problem has been
            diagnosed as stiff. If the situation persists, it will cost
            roughly *<value>* times as much to reach tend (setup) as it
            has cost to reach the current time. You should probably call
            functions intended for stiff problems. However, you can
            continue integrating the problem.

        (`errno` 4)
            Your problem has been diagnosed as stiff. If the situation
            persists, it will cost roughly *<value>* times as much to
            reach tend (setup) as it has cost to reach the current time.
            You should probably call functions intended for stiff
            problems. However, you can continue integrating the problem.

        (`errno` 5)
            In order to satisfy your error requirements the solver has
            to use a step size of *<value>* at the current time,
            *<value>*. This step size is too small for the machine
            precision, and is smaller than *<value>*.

        (`errno` 6)
            The global error assessment algorithm failed at start of
            integration.

            The integration is being terminated.

        (`errno` 6)
            The global error assessment may not be reliable for times
            beyond *<value>*.

            The integration is being terminated.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_range'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, yp[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, t, n, y, yp, _iuser, _ruser):
        """Helper for f."""
        try:
            _t_fh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fh,),
            )
            _yp_fh = utils._EngineFloat64ArrayType.from_pointer(
                yp,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                    data,
                )
            else:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yp_fh, {'fun_name': cb_f_name, 'returned': 'yp'}, yp,
            )
            utils._unlink_pointer(yp)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02peft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, twant.
        utils._BYREF_FLOAT64, # Output, tgot.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ygot[n].
        utils._EngineFloat64ArrayType.c_type, # Output, ypgot[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ymax[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _twant = utils._EngineFloat64ScalarType(
        twant, {'fun_name': fun_name, 'entity': 'twant'},
    ).to_ctypes()
    _tgot = utils._EngineFloat64ScalarType.empty_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ygot_cls = utils._EngineFloat64ArrayType(
        ygot, {'fun_name': fun_name, 'entity': 'ygot'},
        exp_shape=(_n_py,),
    )
    _ygot = _ygot_cls.to_ctypes()
    _ypgot_cls = utils._EngineFloat64ArrayType(
        ypgot, {'fun_name': fun_name, 'entity': 'ypgot'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ypgot = _ypgot_cls.to_ctypes(can_reference_contents=False)
    _ymax_cls = utils._EngineFloat64ArrayType(
        ymax, {'fun_name': fun_name, 'entity': 'ymax'},
        exp_shape=(_n_py,),
    )
    _ymax = _ymax_cls.to_ctypes()
    _iuser = None
    _ruser = None
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fh,
        _n,
        _twant,
        _tgot,
        _ygot,
        _ypgot,
        _ymax,
        _iuser,
        _ruser,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'twant is too close to the last value of tgot (tstart on setup).',
                    'When using the method of order 8 at setup, these must differ by at least',
                    '{:s}. Their absolute difference is'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'twant lies beyond tend (setup) in the direction of integration.',
                    'twant = {:s} and tend = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'twant lies beyond tend (setup) in the direction of integration, but is',
                    'very close to tend.',
                    'You may have intended twant = tend.',
                    'abs(twant-tend) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'twant does not lie in the direction of integration.',
                    'twant = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tend (setup) had already been reached in a previous call.',
                    'To start a new problem, you will need to call the setup function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function after it has returned an error.',
                    'You must call the setup function to start another problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function when you have specified, in the setup',
                    'function, that the step integrator will be used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'This function is being used inefficiently because the step size has been',
                    'reduced drastically many times to obtain answers at many points.',
                    'Using the order 4 and 5 pair method at setup is more appropriate here.',
                    'You can continue integrating this problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed. However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed. Your problem has been diagnosed as stiff. If the situation',
                    'persists, it will cost roughly {:s} times as much to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'reach tend (setup) as it has cost to reach the current time.',
                    'You should probably call functions intended for stiff problems.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Your problem has been diagnosed as stiff. If the situation persists,',
                    'it will cost roughly {:s} times as much to reach tend'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '(setup) as it has cost to reach the current time. You should probably call',
                    'functions intended for stiff problems. However, you can continue',
                    'integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In order to satisfy your error requirements the solver has to',
                    'use a step size of {:s} at the current time,'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. This step size is too small for the'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'machine precision, and is smaller than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99982:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment algorithm failed at start of integration.',
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment may not be reliable for times beyond',
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ygot_cls.refresh_py(_ygot)
        _ypgot_cls.refresh_py(_ypgot)
        _ymax_cls.refresh_py(_ymax)
        _tgot_py = utils._EngineFloat64ScalarType.to_py(_tgot)
        return_data = _tgot_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['tgot'],
            )(
                _tgot_py,
            )
    _err_class.manage()
    return return_data

def ivp_rkts_onestep(f, n, ynow, ypnow, comm, data=None):
    u"""
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration over one step.

    For full information please refer to the NAG Library document for
    d02pf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02pff.html

    Parameters
    ----------
    f : callable f(t, n, y, yp, data=None)
        `f` must evaluate the functions f_i (that is the first
        derivatives y'_i) for given values of the arguments t, y_i.

        Parameters
        ~~~~~~~~~~
        t : float
            t, the current value of the independent variable.

        n : int
            n, the number of ordinary differential equations in the
            system to be solved.

        y : float, ndarray, shape (`n`)
            The current values of the dependent variables, y_i, for i =
            1,2,...,n.

        yp : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of ordinary differential equations in the system
        to be solved.

    ynow : float, shape (`n`), modified in place
        On exit: an approximation to the solution at `tnow`. The local
        error of the step to `tnow` was no greater than permitted by the
        specified tolerances (see ``ivp_rkts_setup``).

    ypnow : float, shape (`n`), modified in place
        On exit: an approximation to the first derivative of the
        solution at `tnow`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    tnow : float
        t, the value of the independent variable at which a solution has
        been computed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            tend, as specified in the setup function, has already been
            reached. To start a new problem, you will need to call the
            setup function. To continue integration beyond tend then
            ``ivp_rkts_reset_tend`` must first be called to reset tend
            to a new end value.

        (`errno` 1)
            A call to this function cannot be made after it has returned
            an error.

            The setup function must be called to start another problem.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            On entry, the communication arrays have become corrupted, or
            a catastrophic error has already been detected elsewhere.
            You cannot continue integrating the problem.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            More than 100 output points have been obtained by
            integrating to tend (as specified in the setup function).
            They have been so clustered that it would probably be (much)
            more efficient to use the interpolation function (if
            abs(method) = 3, switch to abs(method) = 2 at setup).
            However, you can continue integrating the problem.

        (`errno` 3)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. However, you can
            continue integrating the problem.

        (`errno` 4)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. Your problem has been
            diagnosed as stiff. If the situation persists, it will cost
            roughly *<value>* times as much to reach tend (setup) as it
            has cost to reach the current time. You should probably call
            functions intended for stiff problems. However, you can
            continue integrating the problem.

        (`errno` 4)
            Your problem has been diagnosed as stiff. If the situation
            persists, it will cost roughly *<value>* times as much to
            reach tend (setup) as it has cost to reach the current time.
            You should probably call functions intended for stiff
            problems. However, you can continue integrating the problem.

        (`errno` 5)
            In order to satisfy your error requirements the solver has
            to use a step size of *<value>* at the current time,
            *<value>*. This step size is too small for the machine
            precision, and is smaller than *<value>*.

        (`errno` 6)
            The global error assessment may not be reliable for times
            beyond *<value>*.

            The integration is being terminated.

        (`errno` 6)
            The global error assessment algorithm failed at start of
            integration.

            The integration is being terminated.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_onestep'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, yp[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, t, n, y, yp, _iuser, _ruser):
        """Helper for f."""
        try:
            _t_fh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fh,),
            )
            _yp_fh = utils._EngineFloat64ArrayType.from_pointer(
                yp,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                    data,
                )
            else:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yp_fh, {'fun_name': cb_f_name, 'returned': 'yp'}, yp,
            )
            utils._unlink_pointer(yp)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02pfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Output, tnow.
        utils._EngineFloat64ArrayType.c_type, # Output, ynow[n].
        utils._EngineFloat64ArrayType.c_type, # Output, ypnow[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _tnow = utils._EngineFloat64ScalarType.empty_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _ynow_cls = utils._EngineFloat64ArrayType(
        ynow, {'fun_name': fun_name, 'entity': 'ynow'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ynow = _ynow_cls.to_ctypes(can_reference_contents=False)
    _ypnow_cls = utils._EngineFloat64ArrayType(
        ypnow, {'fun_name': fun_name, 'entity': 'ypnow'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _ypnow = _ypnow_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fh,
        _n,
        _tnow,
        _ynow,
        _ypnow,
        _iuser,
        _ruser,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tend, as specified in the setup function, has already been reached.',
                    'To start a new problem, you will need to call the setup function.',
                    'To continue integration beyond tend then',
                    'ivp_rkts_reset_tend must first be called to',
                    'reset tend to a new end value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A call to this function cannot be made after it has returned an error.',
                    'The setup function must be called to start another problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than 100 output points have been obtained by integrating to',
                    'tend (as specified in the setup function).',
                    'They have been so clustered that it would probably be (much) more efficient',
                    'to use the interpolation function (if abs(method) = 3,',
                    'switch to abs(method) = 2 at setup).',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 301:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed. Your problem has been diagnosed as stiff. If the',
                    'situation persists, it will cost roughly {:s} times'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'as much to reach tend (setup) as it has cost to reach the current time.',
                    'You should probably call functions intended for stiff problems.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 302:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Your problem has been diagnosed as stiff. If the situation persists,',
                    'it will cost roughly {:s} times as much to reach tend'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '(setup) as it has cost to reach the current time. You should probably call',
                    'functions intended for stiff problems.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In order to satisfy your error requirements the solver has to',
                    'use a step size of {:s} at the current time,'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. This step size is too small for the'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'machine precision, and is smaller than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 601:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment may not be reliable for times beyond',
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 602:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment algorithm failed at start of integration.',
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ynow_cls.refresh_py(_ynow)
        _ypnow_cls.refresh_py(_ypnow)
        _tnow_py = utils._EngineFloat64ScalarType.to_py(_tnow)
        return_data = _tnow_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['tnow'],
            )(
                _tnow_py,
            )
    _err_class.manage()
    return return_data

def ivp_rk_step_revcomm(irevcm, n, y, yp, comm):
    u"""
    Ordinary differential equations, initial value problem, Runge--Kutta
    method, integration by reverse communication.

    For full information please refer to the NAG Library document for
    d02pg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02pgf.html

    Parameters
    ----------
    irevcm : int
        On initial entry `irevcm` must be set to zero, otherwise should
        be unchanged from the previous call.

    n : int
        n, the number of ordinary differential equations in the system
        to be solved.

    y : float, shape (`n`), modified in place
        On intermediate exit: `y` contains the value of the solution y
        at which the derivatives y' are to be evaluated.

        On final exit: the approximation to the solution computed
        following a successful step.

    yp : float, shape (`n`)
        On initial entry: `yp` need not be set.

        On intermediate entry: `yp` must contain the value of the
        derivatives y' = f(t,y) where t is supplied in `t` and y is
        supplied in the array `y`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit `irevcm` returns a value > 0 to indicate
        that a function evaluation is required prior to re-entry.

        On final exit returns -1 or -2 denoting success or failure,
        respectively.

    t : float
        On intermediate exit: this argument contains the value of the
        independent variable t at which the derivatives y' are to be
        evaluated.

        On final exit: the value of t at which a solution has been
        computed following a successful step.

    Raises
    ------
    NagValueError
        (`errno` 1)
            tend, as specified in the setup function, has already been
            reached. To start a new problem, you will need to call the
            setup function. To continue integration beyond tend then
            ``ivp_rkts_reset_tend`` must first be called to reset tend
            to a new end value.

        (`errno` 1)
            A call to this function cannot be made after it has returned
            an error.

            The setup function must be called to start another problem.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted.

        (`errno` 1)
            `irevcm` < 0 on entry.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            On entry, the communication arrays have become corrupted, or
            a catastrophic error has already been detected elsewhere.
            You cannot continue integrating the problem.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            More than 100 output points have been obtained by
            integrating to tend (as specified in the setup function).
            They have been so clustered that it would probably be (much)
            more efficient to use the interpolation function. However,
            you can continue integrating the problem.

        (`errno` 3)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. However, you can
            continue integrating the problem.

        (`errno` 4)
            Approximately *<value>* function evaluations have been used
            to compute the solution since the integration started or
            since this message was last printed. Your problem has been
            diagnosed as stiff. If the situation persists, it will cost
            roughly *<value>* times as much to reach tend (setup) as it
            has cost to reach the current time. You should probably call
            functions intended for stiff problems. However, you can
            continue integrating the problem.

        (`errno` 4)
            Your problem has been diagnosed as stiff. If the situation
            persists, it will cost roughly *<value>* times as much to
            reach tend (setup) as it has cost to reach the current time.
            You should probably call functions intended for stiff
            problems. However, you can continue integrating the problem.

        (`errno` 5)
            In order to satisfy your error requirements the solver has
            to use a step size of *<value>* at the current time,
            *<value>*. This step size is too small for the machine
            precision, and is smaller than *<value>*.

        (`errno` 6)
            The global error assessment may not be reliable for times
            beyond *<value>*.

            The integration is being terminated.

        (`errno` 6)
            The global error assessment algorithm failed at start of
            integration.

            The integration is being terminated.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rk_step_revcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02pgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Output, t.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineFloat64ArrayType.c_type, # Input, yp[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType.empty_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _yp = utils._EngineFloat64ArrayType(
        yp, {'fun_name': fun_name, 'entity': 'yp'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _t,
        _y,
        _yp,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py not in (-1, -2) or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'tend, as specified in the setup function, has already been reached.',
                    'To start a new problem, you will need to call the setup function.',
                    'To continue integration beyond tend then',
                    'ivp_rkts_reset_tend must first be called to',
                    'reset tend to a new end value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A call to this function cannot be made after it has returned an error.',
                    'The setup function must be called to start another problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'irevcm < 0 on entry.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than 100 output points have been obtained by integrating to',
                    'tend (as specified in the setup function).',
                    'They have been so clustered that it would probably be (much) more efficient',
                    'to use the interpolation function.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 301:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Approximately {:s} function evaluations have been used to'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'compute the solution since the integration started or since this message',
                    'was last printed. Your problem has been diagnosed as stiff. If the',
                    'situation persists, it will cost roughly {:s} times'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'as much to reach tend (setup) as it has cost to reach the current time.',
                    'You should probably call functions intended for stiff problems.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 302:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Your problem has been diagnosed as stiff. If the situation persists,',
                    'it will cost roughly {:s} times as much to reach tend'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '(setup) as it has cost to reach the current time. You should probably call',
                    'functions intended for stiff problems.',
                    'However, you can continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'In order to satisfy your error requirements the solver has to',
                    'use a step size of {:s} at the current time,'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}. This step size is too small for the'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'machine precision, and is smaller than {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 601:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment may not be reliable for times beyond',
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 602:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The global error assessment algorithm failed at start of integration.',
                    'The integration is being terminated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _y_cls.refresh_py(_y)
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 't'],
        )(
            _irevcm_py,
            _t_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_rk_interp_setup(irevcm, n, nwant, y, yp, comm):
    u"""
    Set up interpolant by reverse communication for solution and
    derivative evaluations at points within the range of the last
    integration step taken by ``ivp_rk_step_revcomm``.

    For full information please refer to the NAG Library document for
    d02ph

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02phf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` must be set to zero to indicate that
        the interpolant for a new step is being taken.

        On intermediate entry: `irevcm` should remain unchanged.

    n : int
        n, the number of ordinary differential equations in the system
        to be solved by the integration function.

    nwant : int
        The number of components of the solution to be computed. The
        first `nwant` components are evaluated.

    y : float, shape (`n`), modified in place
        On intermediate exit: `y` contains the value of the solution y
        at which the derivatives y' are to be evaluated.

        On final exit: contains no useful information.

    yp : float, shape (`n`)
        On initial entry: need not be set.

        On intermediate entry: `yp` must contain the values of the
        derivatives y'_i for the given values of the parameters t, y_i.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit: this argument returns a value 1 to
        indicate that a function evaluation is required prior to
        re-entry; the value of the derivatives must be returned in `yp`
        where the value of t is supplied in `t` and the values y(t) are
        supplied in the array `y`.

        On final exit:

        `irevcm` = -1

          Successful exit; `comm`['rwsav'] and `comm`['wcomm'] contain
          details of the interpolant.

        `irevcm` = -2

          Error exit; errno should be interrogated to determine the
          nature of the error.

    t : float
        On intermediate exit: `t` contains the value of the independent
        variable t at which the derivatives y' are to be evaluated.

        On final exit: contains no useful information.

    Raises
    ------
    NagValueError
        (`errno` 1)
            You cannot call this function after the integrator has
            returned an error.

        (`errno` 1)
            You cannot call this function before you have called the
            step integrator.

        (`errno` 1)
            You cannot call this function after the range integrator has
            been called.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            On entry, `nwant` = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `nwant` <= `n`.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere.

            You cannot continue integrating the problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rk_interp_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02phft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nwant.
        utils._BYREF_FLOAT64, # Output, t.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineFloat64ArrayType.c_type, # Input, yp[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wcomm[lwcomm].
        utils._BYREF_INT, # Input, lwcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nwant = utils._EngineIntScalarType(
        nwant, {'fun_name': fun_name, 'entity': 'nwant'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType.empty_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    _yp = utils._EngineFloat64ArrayType(
        yp, {'fun_name': fun_name, 'entity': 'yp'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wcomm',),
        req_keys=('iwsav', 'rwsav'),
    )
    _nwant_py = utils._EngineIntScalarType.to_py(_nwant)
    _vl_methd_py = utils._EngineIntArrayType.get_element(comm['iwsav'], {'fun_name': fun_name, 'entity': 'comm[\'iwsav\']'}, (41,), validate=False)
    if _comm_needs_init:
        if _vl_methd_py == 1:
            _wcomm_dim1 = 1
        elif _vl_methd_py == 2:
            _wcomm_dim1 = _n_py+max(_n_py, 5*_nwant_py)
        elif _vl_methd_py == 3:
            _wcomm_dim1 = 8*_nwant_py
        else:
            _wcomm_dim1 = 0
        _wcomm = utils._EngineFloat64ArrayType.empty_ctypes(_wcomm_dim1)
        comm['wcomm'] = _wcomm
    else:
        _wcomm = comm['wcomm']
    _lwcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['wcomm'], {'fun_name': fun_name, 'entity': 'comm[\'wcomm\']'}, exp_rank=1, validate=False)[0]
    _lwcomm = utils._EngineIntScalarType(
        _lwcomm_py, {'fun_name': fun_name, 'entity': '_lwcomm'},
    ).to_ctypes()
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _nwant,
        _t,
        _y,
        _yp,
        _wcomm,
        _lwcomm,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py not in (-1, -2) or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 96:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function after the integrator has returned an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 97:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the step integrator.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function after the range integrator has been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= nwant <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 112:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 't'],
        )(
            _irevcm_py,
            _t_py,
        )
    _err_class.manage()
    return return_data

def ivp_rk_interp_eval(icheck, n, nwant, t, ideriv, sol, comm, lwcomm):
    u"""
    Evaluate interpolant, set up using ``ivp_rkts_setup``, to
    approximate solution and/or solution derivatives at a point within
    the range of the last integration step taken by
    ``ivp_rk_step_revcomm``.

    For full information please refer to the NAG Library document for
    d02pj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02pjf.html

    Parameters
    ----------
    icheck : int
        Indicates whether consistency checks on input arguments should
        be performed

        `icheck` != 1

          Don't perform checks on input arguments.

        `icheck` = 1

          Perform consistency checks on input arguments.

        It is recommended to use `icheck` = 1 on the first call
        following a call to ``ivp_rk_interp_setup`` and to set `icheck`
        != 1 on subsequent calls within the last step to avoid the
        overhead of argument checking.

    n : int
        n, the dimension of the system of ODEs being integrated.

    nwant : int
        Only the first `nwant` system components to be computed. This
        should be the same value as passed to ``ivp_rk_interp_setup``
        when computing the interpolant.

    t : float
        t, the value of the independent variable where a solution is
        desired. Although any value of t can be supplied, accurate
        solutions can only be obtained for values in the range of the
        last time-step taken by ``ivp_rk_step_revcomm``.

    ideriv : int
        `ideriv` = 0

          Compute approximations to the first `nwant` components of the
          solution y(t).

        `ideriv` = 1

          Compute approximations to the first `nwant` components of the
          first derivatives of the solution y'(t).

    sol : float, shape (`nwant`), modified in place
        On exit:

        `ideriv` = 0

          The first `nwant` components of the solution y(t).

        `ideriv` = 1

          The first `nwant` components of the first derivatives of the
          solution y'(t).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    lwcomm : int
        Length of `comm`['wcomm']. This should be the same value as
        supplied in a previous call to ``ivp_rk_interp_setup``.

        If in a previous call to ``ivp_rkts_setup``:

        - method = 1 or -1, `lwcomm` must be at least 1.

        - method = 2 or -2, `lwcomm` must be at least `n`+max(`n`,
          5*`nwant`).

        - method = 3 or -3, `lwcomm` >= 8*`nwant`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The previous call to the interpolation setup function
            returned an error.

        (`errno` 1)
            You cannot call this function before you have called the
            interpolation setup.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            routine was `n` = *<value>*.

        (`errno` 1)
            On entry, `ideriv` = *<value>*.

            Constraint: `ideriv` = 0 or 1.

        (`errno` 1)
            On entry, `nwant` = *<value>*, but on interpolation setup
            `nwant` = *<value>*.

            Constraint: `nwant` must be unchanged from setup.

        (`errno` 1)
            On entry, `lwcomm` = *<value>* and `nwant` = *<value>*.

            Constraint: for method = -3 or 3, `lwcomm` >= 8*`nwant`.

        (`errno` 1)
            On entry, `lwcomm` = *<value>*, `n` = *<value>* and `nwant`
            = *<value>*.

            Constraint: for method = -2 or 2, `lwcomm` >= `n`+max(`n`,
            5*`nwant`).

        (`errno` 1)
            On entry, `lwcomm` = *<value>*.

            Constraint: for method = -1 or 1, `lwcomm` >= 1.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere.

            You cannot continue integrating the problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rk_interp_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02pjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, icheck.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nwant.
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, ideriv.
        utils._EngineFloat64ArrayType.c_type, # Output, sol[nwant].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wcomm[lwcomm].
        utils._BYREF_INT, # Input, lwcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _icheck = utils._EngineIntScalarType(
        icheck, {'fun_name': fun_name, 'entity': 'icheck'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nwant = utils._EngineIntScalarType(
        nwant, {'fun_name': fun_name, 'entity': 'nwant'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _ideriv = utils._EngineIntScalarType(
        ideriv, {'fun_name': fun_name, 'entity': 'ideriv'},
    ).to_ctypes()
    _nwant_py = utils._EngineIntScalarType.to_py(_nwant)
    _sol_cls = utils._EngineFloat64ArrayType(
        sol, {'fun_name': fun_name, 'entity': 'sol'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _sol = _sol_cls.to_ctypes(can_reference_contents=False)
    _lwcomm = utils._EngineIntScalarType(
        lwcomm, {'fun_name': fun_name, 'entity': 'lwcomm'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('wcomm', 'iwsav', 'rwsav'),
    )
    _wcomm = comm['wcomm']
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _icheck,
        _n,
        _nwant,
        _t,
        _ideriv,
        _sol,
        _wcomm,
        _lwcomm,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 96:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The previous call to the interpolation setup function returned an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 97:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the interpolation',
                    'setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup routine was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ideriv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ideriv = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s}, but'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'on interpolation setup nwant = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwant must be unchanged from setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwcomm = {:s} and nwant = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for method = -3 or 3, lwcomm >= 8*nwant.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 109:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwcomm = {:s}, n = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and nwant = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: for method = -2 or 2, lwcomm >= n+max(n, 5*nwant).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 110:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwcomm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for method = -1 or 1, lwcomm >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 112:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sol_cls.refresh_py(_sol)
        return_data = None
    _err_class.manage()
    return return_data

def ivp_rkts_setup(n, tstart, tend, yinit, tol, thresh, method, hstart):
    u"""
    Ordinary differential equations, initial value problem, setup for
    ``ivp_rkts_range`` and ``ivp_rkts_onestep``.

    For full information please refer to the NAG Library document for
    d02pq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02pqf.html

    Parameters
    ----------
    n : int
        n, the number of ordinary differential equations in the system
        to be solved by the integration function.

    tstart : float
        The initial value of the independent variable, t_0.

    tend : float
        The final value of the independent variable, t_f, at which the
        solution is required. `tstart` and `tend` together determine the
        direction of integration.

    yinit : float, shape (`n`)
        y_0, the initial values of the solution, y_i, for i = 1,2,...,n.

    tol : float
        A relative error tolerance. The actual tolerance used is
        max(10*machine precision, min(`tol`, 0.01)); that is, the
        minimum tolerance is set at 10 times machine precision and the
        maximum tolerance is set at 0.01.

    thresh : float, shape (`n`)
        A vector of thresholds. For the ith component, the actual
        threshold used is max(sqrt(safe range), `thresh`[i-1]), where
        safe range is the value returned by ``machine.real_safe``.

    method : int
        The Runge--Kutta method to be used.

        `method` = 1 or -1

          A 2(3) pair is used.

        `method` = 2 or -2

          A 4(5) pair is used.

        `method` = 3 or -3

          A 7(8) pair is used.

    hstart : float
        A value for the size of the first step in the integration to be
        attempted. The absolute value of `hstart` is used with the
        direction being determined by `tstart` and `tend`. The actual
        first step taken by the integrator may be different to `hstart`
        if the underlying algorithm determines that `hstart` is
        unsuitable. If `hstart` = 0.0 then the size of the first step is
        computed automatically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `tstart` = *<value>*.

            Constraint: `tstart` != `tend`.

        (`errno` 1)
            On entry, `tstart` is too close to `tend`.

            abs(`tstart`-`tend`) = *<value>*, but this quantity should
            be at least *<value>*.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = -3, -2, -1, 1, 2 or 3.

        (`errno` 1)
            On entry, too much workspace required.

            Workspace provided was *<value>*, workspace required is
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02pqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, tstart.
        utils._BYREF_FLOAT64, # Input, tend.
        utils._EngineFloat64ArrayType.c_type, # Input, yinit[n].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Input, thresh[n].
        utils._BYREF_INT, # Input, method.
        utils._BYREF_FLOAT64, # Input, hstart.
        utils._EngineIntArrayType.c_type, # Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _tstart = utils._EngineFloat64ScalarType(
        tstart, {'fun_name': fun_name, 'entity': 'tstart'},
    ).to_ctypes()
    _tend = utils._EngineFloat64ScalarType(
        tend, {'fun_name': fun_name, 'entity': 'tend'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _yinit = utils._EngineFloat64ArrayType(
        yinit, {'fun_name': fun_name, 'entity': 'yinit'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _thresh = utils._EngineFloat64ArrayType(
        thresh, {'fun_name': fun_name, 'entity': 'thresh'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _method = utils._EngineIntScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'},
    ).to_ctypes()
    _hstart = utils._EngineFloat64ScalarType(
        hstart, {'fun_name': fun_name, 'entity': 'hstart'},
    ).to_ctypes()
    _iwsav = utils._EngineIntArrayType.empty_ctypes(130)
    _comm_py = {}
    _comm_py['iwsav'] = _iwsav
    _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(32*_n_py+350)
    _comm_py['rwsav'] = _rwsav
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _tstart,
        _tend,
        _yinit,
        _tol,
        _thresh,
        _method,
        _hstart,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tstart = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tstart != tend.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tstart is too close to tend.',
                    'abs(tstart-tend) = {:s}, but this quantity should be at'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 107:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = -3, -2, -1, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 109:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, too much workspace required.',
                    'Workspace provided was {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'workspace required is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def ivp_rkts_reset_tend(tendnu, comm):
    u"""
    Ordinary differential equations, initial value problem, resets end
    of range for ``ivp_rkts_onestep``.

    For full information please refer to the NAG Library document for
    d02pr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02prf.html

    Parameters
    ----------
    tendnu : float
        The new value for t_f.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            You cannot call this function before you have called the
            setup function.

        (`errno` 1)
            You cannot call this function after the integrator has
            returned an error.

        (`errno` 1)
            You cannot call this function before you have called the
            step integrator.

        (`errno` 1)
            You cannot call this function when the range integrator has
            been used.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere. You
            cannot continue integrating the problem.

        (`errno` 1)
            On entry, `tendnu` is not beyond tnow (step integrator) in
            the direction of integration.

            The direction is positive, `tendnu` = *<value>* and tnow =
            *<value>*.

        (`errno` 1)
            On entry, `tendnu` is not beyond tnow (step integrator) in
            the direction of integration.

            The direction is negative, `tendnu` = *<value>* and tnow =
            *<value>*.

        (`errno` 1)
            On entry, `tendnu` is too close to tnow (step integrator).
            Their difference is *<value>*, but this quantity must be at
            least *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_reset_tend'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02prft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, tendnu.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tendnu = utils._EngineFloat64ScalarType(
        tendnu, {'fun_name': fun_name, 'entity': 'tendnu'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tendnu,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 92:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the setup function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 96:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function after the integrator has returned an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 97:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the step integrator.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function when the range integrator has been used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tendnu is not beyond tnow (step integrator) in the direction of',
                    'integration.',
                    'The direction is positive, tendnu = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'tnow = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tendnu is not beyond tnow (step integrator) in the direction of',
                    'integration.',
                    'The direction is negative, tendnu = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'tnow = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tendnu is too close to tnow (step integrator).',
                    'Their difference is {:s}, but this quantity must be at'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'least {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def ivp_rkts_interp(n, twant, ideriv, nwant, ywant, ypwant, f, comm, data=None):
    u"""
    Ordinary differential equations, initial value problem,
    interpolation for ``ivp_rkts_onestep``.

    For full information please refer to the NAG Library document for
    d02ps

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02psf.html

    Parameters
    ----------
    n : int
        n, the number of ordinary differential equations in the system
        to be solved by the integration function.

    twant : float
        t, the value of the independent variable where a solution is
        desired.

    ideriv : int
        Determines whether the solution and/or its first derivative are
        to be computed

        `ideriv` = 0

          compute approximate solution.

        `ideriv` = 1

          compute approximate first derivative.

        `ideriv` = 2

          compute approximate solution and first derivative.

    nwant : int
        The number of components of the solution to be computed. The
        first `nwant` components are evaluated.

    ywant : float, shape (`nwant`), modified in place
        On exit: an approximation to the first `nwant` components of the
        solution at `twant` if `ideriv` = 0 or 2. Otherwise `ywant` is
        not defined.

    ypwant : float, shape (`nwant`), modified in place
        On exit: an approximation to the first `nwant` components of the
        first derivative at `twant` if `ideriv` = 1 or 2. Otherwise
        `ypwant` is not defined.

    f : callable f(t, n, y, yp, data=None)
        `f` must evaluate the functions f_i (that is the first
        derivatives y'_i) for given values of the arguments t,y_i.

        It must be the same procedure as supplied to
        ``ivp_rkts_onestep``.

        Parameters
        ~~~~~~~~~~
        t : float
            t, the current value of the independent variable.

        n : int
            n, the number of ordinary differential equations in the
            system to be solved.

        y : float, ndarray, shape (`n`)
            The current values of the dependent variables, y_i, for i =
            1,2,...,n.

        yp : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of f_i, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            You cannot call this function after the integrator has
            returned an error.

        (`errno` 1)
            You cannot call this function before you have called the
            step integrator.

        (`errno` 1)
            You cannot call this function when you have specified, in
            the setup function, that the range integrator will be used.

        (`errno` 1)
            method = -3 or 3 in setup, but interpolation is not
            available for this method. Either use method = -2 or 2 in
            setup or use reset function to force the integrator to step
            to particular points.

        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            On entry, `ideriv` = *<value>*.

            Constraint: `ideriv` = 0, 1 or 2.

        (`errno` 1)
            On entry, `nwant` = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `nwant` <= `n`.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere.

            You cannot continue integrating the problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, yp[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, t, n, y, yp, _iuser, _ruser):
        """Helper for f."""
        try:
            _t_fh = utils._EngineFloat64ScalarType.to_py(t.contents)
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fh,),
            )
            _yp_fh = utils._EngineFloat64ArrayType.from_pointer(
                yp,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                    data,
                )
            else:
                f(
                    _t_fh,
                    _n_fh,
                    _y_fh,
                    _yp_fh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _yp_fh, {'fun_name': cb_f_name, 'returned': 'yp'}, yp,
            )
            utils._unlink_pointer(yp)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02psft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, twant.
        utils._BYREF_INT, # Input, ideriv.
        utils._BYREF_INT, # Input, nwant.
        utils._EngineFloat64ArrayType.c_type, # Output, ywant[nwant].
        utils._EngineFloat64ArrayType.c_type, # Output, ypwant[nwant].
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wcomm[lwcomm].
        utils._BYREF_INT, # Input, lwcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _twant = utils._EngineFloat64ScalarType(
        twant, {'fun_name': fun_name, 'entity': 'twant'},
    ).to_ctypes()
    _ideriv = utils._EngineIntScalarType(
        ideriv, {'fun_name': fun_name, 'entity': 'ideriv'},
    ).to_ctypes()
    _nwant = utils._EngineIntScalarType(
        nwant, {'fun_name': fun_name, 'entity': 'nwant'},
    ).to_ctypes()
    _nwant_py = utils._EngineIntScalarType.to_py(_nwant)
    _ywant_cls = utils._EngineFloat64ArrayType(
        ywant, {'fun_name': fun_name, 'entity': 'ywant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ywant = _ywant_cls.to_ctypes(can_reference_contents=False)
    _ypwant_cls = utils._EngineFloat64ArrayType(
        ypwant, {'fun_name': fun_name, 'entity': 'ypwant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ypwant = _ypwant_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wcomm',),
        req_keys=('iwsav', 'rwsav'),
    )
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _vl_methd_py = utils._EngineIntArrayType.get_element(comm['iwsav'], {'fun_name': fun_name, 'entity': 'comm[\'iwsav\']'}, (41,), validate=False)
    if _comm_needs_init:
        if _vl_methd_py == 1:
            _wcomm_dim1 = 1
        elif _vl_methd_py == 2:
            _wcomm_dim1 = _n_py+max(_n_py, 5*_nwant_py)
        else:
            _wcomm_dim1 = 0
        _wcomm = utils._EngineFloat64ArrayType.empty_ctypes(_wcomm_dim1)
        comm['wcomm'] = _wcomm
    else:
        _wcomm = comm['wcomm']
    _lwcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['wcomm'], {'fun_name': fun_name, 'entity': 'comm[\'wcomm\']'}, exp_rank=1, validate=False)[0]
    _lwcomm = utils._EngineIntScalarType(
        _lwcomm_py, {'fun_name': fun_name, 'entity': '_lwcomm'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _twant,
        _ideriv,
        _nwant,
        _ywant,
        _ypwant,
        _f,
        _fh,
        _wcomm,
        _lwcomm,
        _iuser,
        _ruser,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 96:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function after the integrator has returned an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 97:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the step integrator.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function when you have specified, in the setup',
                    'function, that the range integrator will be used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'method = -3 or 3 in setup, but interpolation is not available',
                    'for this method. Either use method = -2 or 2 in setup or use reset',
                    'function to force the integrator to step to particular points.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ideriv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ideriv = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= nwant <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 112:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ywant_cls.refresh_py(_ywant)
        _ypwant_cls.refresh_py(_ypwant)
        return_data = None
    _err_class.manage()
    return return_data

def ivp_rkts_diag(comm):
    u"""
    Ordinary differential equations, initial value problem, integration
    diagnostics for ``ivp_rkts_range`` and ``ivp_rkts_onestep``.

    For full information please refer to the NAG Library document for
    d02pt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ptf.html

    Parameters
    ----------
    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_rkts_setup`` and one of ``ivp_rkts_range``,
        ``ivp_rkts_onestep`` or ``ivp_rk_step_revcomm``.

    Returns
    -------
    fevals : int
        The total number of evaluations of f used in the integration so
        far; this includes evaluations of f required for the secondary
        integration necessary if ``ivp_rkts_setup`` had previously been
        called with method > 0.

    stepcost : int
        The cost in terms of number of evaluations of f of a typical
        step with the method being used for the integration. The method
        is specified by the argument method in a prior call to
        ``ivp_rkts_setup``.

    waste : float
        The number of attempted steps that failed to meet the local
        error requirement divided by the total number of steps attempted
        so far in the integration. A 'large' fraction indicates that the
        integrator is having trouble with the problem being solved. This
        can happen when the problem is 'stiff' and also when the
        solution has discontinuities in a low-order derivative.

    stepsok : int
        The number of accepted steps.

    hnext : float
        The step size the integrator will attempt to use for the next
        step.

    Raises
    ------
    NagValueError
        (`errno` 1)
            You cannot call this function before you have called the
            integrator.

        (`errno` 1)
            You have already made one call to this function after the
            integrator could not achieve specified accuracy.

            You cannot call this function again.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere.

            You cannot continue integrating the problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02ptft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Output, fevals.
        utils._BYREF_INT, # Output, stepcost.
        utils._BYREF_FLOAT64, # Output, waste.
        utils._BYREF_INT, # Output, stepsok.
        utils._BYREF_FLOAT64, # Output, hnext.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input, rwsav[350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _fevals = utils._EngineIntScalarType.empty_ctypes()
    _stepcost = utils._EngineIntScalarType.empty_ctypes()
    _waste = utils._EngineFloat64ScalarType.empty_ctypes()
    _stepsok = utils._EngineIntScalarType.empty_ctypes()
    _hnext = utils._EngineFloat64ScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fevals,
        _stepcost,
        _waste,
        _stepsok,
        _hnext,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the integrator.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You have already made one call to this function after the',
                    'integrator could not achieve specified accuracy.',
                    'You cannot call this function again.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fevals_py = utils._EngineIntScalarType.to_py(_fevals)
        _stepcost_py = utils._EngineIntScalarType.to_py(_stepcost)
        _waste_py = utils._EngineFloat64ScalarType.to_py(_waste)
        _stepsok_py = utils._EngineIntScalarType.to_py(_stepsok)
        _hnext_py = utils._EngineFloat64ScalarType.to_py(_hnext)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['fevals', 'stepcost', 'waste', 'stepsok', 'hnext'],
        )(
            _fevals_py,
            _stepcost_py,
            _waste_py,
            _stepsok_py,
            _hnext_py,
        )
    _err_class.manage()
    return return_data

def ivp_rkts_errass(n, rmserr, comm):
    u"""
    Ordinary differential equations, initial value problem, error
    assessment diagnostics for ``ivp_rkts_range`` and
    ``ivp_rkts_onestep``.

    For full information please refer to the NAG Library document for
    d02pu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02puf.html

    Parameters
    ----------
    n : int
        n, the number of ordinary differential equations in the system
        to be solved by the integration function.

    rmserr : float, shape (`n`), modified in place
        On exit: `rmserr`[i-1] approximates the RMS average of the true
        error of the numerical solution for the ith solution component,
        for i = 1,2,...,n. The average is taken over all steps from the
        beginning of the integration to the current integration point.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_rkts_setup``.

    Returns
    -------
    errmax : float
        The maximum weighted approximate true error taken over all
        solution components and all steps.

    terrmx : float
        The first value of the independent variable where an approximate
        true error attains the maximum value, `errmax`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*, but the value passed to the setup
            function was n = *<value>*.

        (`errno` 1)
            No error assessment is available since the integrator has
            not actually taken any successful steps.

        (`errno` 1)
            No error assessment is available since you did not ask for
            it in your call to the setup function.

        (`errno` 1)
            You cannot call this function before you have called the
            integrator.

        (`errno` 1)
            You have already made one call to this function after the
            integrator could not achieve specified accuracy.

            You cannot call this function again.

        (`errno` 1)
            On entry, a previous call to the setup function has not been
            made or the communication arrays have become corrupted, or a
            catastrophic error has already been detected elsewhere.

            You cannot continue integrating the problem.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_rkts_errass'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02puft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, rmserr[n].
        utils._BYREF_FLOAT64, # Output, errmax.
        utils._BYREF_FLOAT64, # Output, terrmx.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[130].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[32*n+350].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _rmserr_cls = utils._EngineFloat64ArrayType(
        rmserr, {'fun_name': fun_name, 'entity': 'rmserr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _rmserr = _rmserr_cls.to_ctypes(can_reference_contents=False)
    _errmax = utils._EngineFloat64ScalarType.empty_ctypes()
    _terrmx = utils._EngineFloat64ScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _rmserr,
        _errmax,
        _terrmx,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}, but the value passed'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'to the setup function was n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 95:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No error assessment is available since the integrator',
                    'has not actually taken any successful steps.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 96:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No error assessment is available since you did not',
                    'ask for it in your call to the setup function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 97:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You cannot call this function before you have called the integrator.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 98:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You have already made one call to this function after the',
                    'integrator could not achieve specified accuracy.',
                    'You cannot call this function again.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a previous call to the setup function has not been made or',
                    'the communication arrays have become corrupted, or a catastrophic',
                    'error has already been detected elsewhere.',
                    'You cannot continue integrating the problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rmserr_cls.refresh_py(_rmserr)
        _errmax_py = utils._EngineFloat64ScalarType.to_py(_errmax)
        _terrmx_py = utils._EngineFloat64ScalarType.to_py(_terrmx)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['errmax', 'terrmx'],
        )(
            _errmax_py,
            _terrmx_py,
        )
    _err_class.manage()
    return return_data

def ivp_adams_roots(fcn, neqf, t, y, tout, g, neqg, comm, data=None):
    u"""
    Ordinary differential equations, initial value problem, Adams'
    method with root-finding (direct communication, comprehensive).

    For full information please refer to the NAG Library document for
    d02qf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qff.html

    Parameters
    ----------
    fcn : callable fcn(neqf, x, y, f, data=None)
        `fcn` must evaluate the functions f_i (that is the first
        derivatives y'_i) for given values of its arguments x,
        y_1,y_2,...,y_`neqf`.

        Parameters
        ~~~~~~~~~~
        neqf : int
            The number of differential equations.

        x : float
            The current value of the argument x.

        y : float, ndarray, shape (`neqf`)
            y_i, for i = 1,2,...,`neqf`, the current value of the
            argument.

        f : float, ndarray, shape (`neqf`), to be modified in place
            On exit: the value of f_i, for i = 1,2,...,`neqf`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    neqf : int
        The number of first-order ordinary differential equations to be
        solved by ``ivp_adams_roots``. It must contain the same value as
        the argument `neqf` used in a prior call to ``ivp_adams_setup``.

    t : float
        After a call to ``ivp_adams_setup`` with statef = 'S', `t` must
        be set to the initial value of the independent variable x.

    y : float, shape (`neqf`), modified in place
        On entry: the initial values of the solution
        y_1,y_2,...,y_`neqf`.

        On exit: the computed values of the solution at the exit value
        of `t`. If the integration is to be continued, possibly with a
        new value for `tout`, these values must not be changed.

    tout : float
        The next value of x at which a computed solution is required.
        For the initial `t`, the input value of `tout` is used to
        determine the direction of integration. Integration is permitted
        in either direction. If `tout` = `t` on exit, `tout` must be
        reset beyond `t` in the direction of integration, before any
        continuation call.

    g : None or callable retval = g(neqf, x, y, yp, k, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `g` must evaluate a given component of g(x,y,y') at a specified
        point.

        Parameters
        ~~~~~~~~~~
        neqf : int
            The number of differential equations being solved.

        x : float
            The current value of the independent variable.

        y : float, ndarray, shape (`neqf`)
            The current values of the dependent variables.

        yp : float, ndarray, shape (`neqf`)
            The current values of the derivatives of the dependent
            variables.

        k : int
            The component of g which must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the component of g(x,y,y') at the specified
            point.

    neqg : int
        The number of event functions which you are defining for
        root-finding. If root-finding is not required the value for
        `neqg` must be <= 0. Otherwise it must be the same argument
        `neqg` used in the prior call to ``ivp_adams_setup``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_adams_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    t : float
        The value of x at which y has been computed. This may be an
        intermediate output point, a root, `tout` or a point at which an
        error has occurred. If the integration is to be continued,
        possibly with a new value for `tout`, `t` must not be changed.

    root : bool
        If root-finding was required (`neqg` > 0 on entry), `root`
        specifies whether or not the output value of the argument `t` is
        a root of one of the event functions. If `root` = False, no root
        was detected, whereas `root` = True indicates a root and you
        should make a call to ``ivp_adams_rootdiag`` for further
        information.

        If root-finding was not required (`neqg` = 0 on entry), then on
        exit `root` = False.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The value of `tout`, *<value>*, indicates a change in the
            integration direction. This is not permitted on a
            continuation call.

        (`errno` 1)
            The value of `t` has been changed from *<value>* to
            *<value>*.

            This is not permitted on a continuation call.

        (`errno` 1)
            On entry, `tout` = *<value>* but crit = True in
            ``ivp_adams_setup``.

            Integration cannot be attempted beyond tcrit = *<value>*.

        (`errno` 1)
            On entry, `tout` = `t` = *<value>*.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = 0 in
            ``ivp_adams_setup``.

            Constraint: if `neqg` <= 0 then `neqg` = neqg in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = 0 in
            ``ivp_adams_setup``.

            Constraint: if `neqg` > 0 then neqg > 0 in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = *<value>* in
            ``ivp_adams_setup``.

            Constraint: if `neqg` <= 0 then neqg <= 0 in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqf` = *<value>* and neqf = *<value>* in
            ``ivp_adams_setup``.

            Constraint: `neqf` = neqf in ``ivp_adams_setup``.

        (`errno` 1)
            The call to setup function ``ivp_adams_setup`` produced an
            error.

        (`errno` 1)
            The setup function ``ivp_adams_setup`` has not been called.

        (`errno` 3)
            The error tolerances are too stringent.

        (`errno` 7)
            Two successive errors detected at the current value of `t`,
            *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The maximum number of steps has been attempted.

            If integration is to be continued then the function may be
            called again and a further maxstp in ``ivp_adams_setup``
            steps will be attempted.

        (`errno` 4)
            Error weight i has become zero during the integration.

            atol[i] = 0.0 in ``ivp_adams_setup`` but `y`[i] is now 0.0.
            Integration successful as far as `t`.

        (`errno` 5)
            The problem appears to be stiff.

        (`errno` 6)
            A change in sign of an event function has been detected but
            the root-finding process appears to have converged to a
            singular point of `t` rather than a root. Integration may be
            continued by calling the function again.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_roots'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neqf].
        utils._EngineFloat64ArrayType.c_type, # Output, f[neqf].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, neqf, x, y, f, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _neqf_fcnh = utils._EngineIntScalarType.to_py(neqf.contents)
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neqf_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neqf_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _neqf_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    data,
                )
            else:
                fcn(
                    _neqf_fcnh,
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neqf].
        utils._EngineFloat64ArrayType.c_type, # Input, yp[neqf].
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if g is None:
        g_auxil_ctypes = utils._EngineState._get_symbol('d02qfzt')
        g_auxil_ctypes.restype = None
        g_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._BYREF_INT, # Input, neqf.
            utils._BYREF_FLOAT64, # Input, x.
            utils._EngineFloat64ArrayType.c_type, # Input, y[neqf].
            utils._EngineFloat64ArrayType.c_type, # Input, yp[neqf].
            utils._BYREF_INT, # Input, k.
            utils._BYREF_FLOAT64, # Output, retval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_gh(_g, _gsub, en_data, neqf, x, y, yp, k, retval, iuser, ruser):
        """Helper for g."""
        try:
            if g is None:
                g_auxil_ctypes(
                    en_data,
                    neqf,
                    x,
                    y,
                    yp,
                    k,
                    retval,
                    iuser,
                    ruser,
                )
                return
            _neqf_gh = utils._EngineIntScalarType.to_py(neqf.contents)
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_gh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neqf_gh,),
            )
            _yp_gh = utils._EngineFloat64ArrayType.from_pointer(
                yp,
                (_neqf_gh,),
            )
            _k_gh = utils._EngineIntScalarType.to_py(k.contents)
            if data is not None:
                _retval_gh = g(
                    _neqf_gh,
                    _x_gh,
                    _y_gh,
                    _yp_gh,
                    _k_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _neqf_gh,
                    _x_gh,
                    _y_gh,
                    _yp_gh,
                    _k_gh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(yp)
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02qfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neqf].
        utils._BYREF_FLOAT64, # Input, tout.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_INT, # Input, neqg.
        utils._BYREF_LOGICAL, # Output, root.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[50].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[180].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[380].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _neqf = utils._EngineIntScalarType(
        neqf, {'fun_name': fun_name, 'entity': 'neqf'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _neqf_py = utils._EngineIntScalarType.to_py(_neqf)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neqf_py,),
    )
    _y = _y_cls.to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'}, allow_none=True)
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _neqg = utils._EngineIntScalarType(
        neqg, {'fun_name': fun_name, 'entity': 'neqg'},
    ).to_ctypes()
    _root = utils._EngineLogicalScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwsav',),
        req_keys=('rwork', 'iwork', 'lwsav', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    _iwork = comm['iwork']
    _iuser = None
    _ruser = None
    _lwsav = comm['lwsav']
    _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(380)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _neqf,
        _t,
        _y,
        _tout,
        _g,
        _gsub,
        _gh,
        _neqg,
        _root,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tout, {:s}, indicates a change in the'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'integration direction. This is not permitted on a continuation call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of t has been changed from {:s} to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'This is not permitted on a continuation call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} but crit = True in ivp_adams_setup.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Integration cannot be attempted beyond tcrit = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and neqg = 0 in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if neqg <= 0 then neqg = neqg in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and neqg = 0 in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if neqg > 0 then neqg > 0 in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqg = {:s} in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if neqg <= 0 then neqg <= 0 in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqf = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqf = {:s} in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neqf = neqf in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The call to setup function ivp_adams_setup produced an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The setup function ivp_adams_setup has not been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of steps has been attempted.',
                    'If integration is to be continued then the function may be called again',
                    'and a further maxstp in ivp_adams_setup steps will be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The error tolerances are too stringent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error weight i has become zero during the integration.',
                    'atol[i] = 0.0 in ivp_adams_setup but y[i]',
                    'is now 0.0. Integration successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The problem appears to be stiff.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A change in sign of an event function has been detected but the root-finding',
                    'process appears to have converged to a singular point of t rather than',
                    'a root. Integration may be continued by calling the function again.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Two successive errors detected at the current value of t,',
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _root_py = utils._EngineLogicalScalarType.to_py(_root)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'root'],
        )(
            _t_py,
            _root_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_adams_roots_revcom(neqf, t, y, tout, neqg, irevcm, grvcm, kgrvcm, comm):
    u"""
    Ordinary differential equations, initial value problem, Adams'
    method with root-finding (reverse communication, comprehensive).

    For full information please refer to the NAG Library document for
    d02qg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qgf.html

    Parameters
    ----------
    neqf : int
        The dimension of the array `y`.

        On initial entry: the number of first-order ordinary
        differential equations to be solved by
        ``ivp_adams_roots_revcom``. It must contain the same value as
        the argument `neqf` used in the prior call to
        ``ivp_adams_setup``.

    t : float
        On initial entry: that is after a call to ``ivp_adams_setup``
        with statef = 'S', `t` must be set to the initial value of the
        independent variable x.

    y : float, shape (`neqf`), modified in place
        On initial entry: the initial values of the solution
        y_1,y_2,...,y_`neqf`.

        On final exit: the computed values of the solution at the exit
        value of `t`. If the integration is to be continued, possibly
        with a new value for `tout`, these values must not be changed.

    tout : float
        On initial entry: the next value of x at which a computed
        solution is required. For the initial `t`, the input value of
        `tout` is used to determine the direction of integration.
        Integration is permitted in either direction. If `tout` = `t` on
        exit, `tout` must be reset beyond `t` in the direction of
        integration, before any continuation call.

    neqg : int
        On initial entry: the number of event functions which you are
        defining for root-finding. If root-finding is not required the
        value for `neqg` must be <= 0. Otherwise it must be the same
        value as the argument `neqg` used in the prior call to
        ``ivp_adams_setup``.

    irevcm : int
        On initial entry: must have the value 0.

    grvcm : float
        On initial entry: need not be set.

        On intermediate entry: with `irevcm` = 9, 10, 11 or 12, `grvcm`
        must contain the value of g_k(x,y,y'), where k is given by
        `kgrvcm`.

    kgrvcm : int
        On intermediate entry: with `irevcm` = 9, 10, 11 or 12, `kgrvcm`
        must remain unchanged from a previous call to
        ``ivp_adams_roots_revcom``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``ivp_adams_setup``.

    Returns
    -------
    t : float
        On final exit: the value of x at which y has been computed. This
        may be an intermediate output point, a root, `tout` or a point
        at which an error has occurred. If the integration is to be
        continued, possibly with a new value for `tout`, `t` must not be
        changed.

    root : bool
        On final exit: if root-finding was required (`neqg` > 0 on
        entry), `root` specifies whether or not the output value of the
        argument `t` is a root of one of the event functions. If `root`
        = False, no root was detected, whereas `root` = True indicates a
        root and you should make a call to ``ivp_adams_rootdiag`` for
        further information.

        If root-finding was not required (`neqg` = 0 on entry), `root` =
        False.

    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``ivp_adams_roots_revcom`` with `irevcm` unchanged.

        `irevcm` = 1, 2, 3, 4, 5, 6 or 7

          Indicates that you must supply y' = f(x,y), where x is given
          by `trvcm` and y_i is returned in `y`[i-1], for i =
          1,2,...,`neqf` when `yrvcm` = 0 and
          `comm`['rwork'][`yrvcm`+i-2], for i = 1,2,...,`neqf` when
          `yrvcm` != 0. y'_i should be placed in location
          `comm`['rwork'][`yprvcm`+i-2], for i = 1,2,...,`neqf`.

        `irevcm` = 8

          Indicates that the current step was not successful due to
          error test failure. The only information supplied to you on
          this return is the current value of the independent variable
          `t`, as given by `trvcm`. No values must be changed before
          re-entering ``ivp_adams_roots_revcom``. This facility enables
          you to determine the number of unsuccessful steps.

        `irevcm` = 9, 10, 11 or 12

          Indicates that you must supply g_k(x,y,y'), where k is given
          by `kgrvcm`, x is given by `trvcm`, y_i is given by `y`[i-1]
          and y'_i is given by `comm`['rwork'][`yprvcm`-1+i-1]. The
          result g_k should be placed in the variable `grvcm`.

        On final exit: has the value 0, which indicates that an output
        point or root has been reached or an error has occurred (see
        errno).

    trvcm : float
        On intermediate exit: the current value of the independent
        variable.

    yrvcm : int
        On intermediate exit: with `irevcm` = 1, 2, 3, 4, 5, 6, 7, 9,
        10, 11 or 12, `yrvcm` specifies the locations of the dependent
        variables y for use in evaluating the differential system or the
        event functions.

        `yrvcm` = 0

          y_i is given by `y`[i], for i = 1,2,...,`neqf`.

        `yrvcm` != 0

          y_i is given by `comm`['rwork'][`yrvcm`+i-2], for i =
          1,2,...,`neqf`.

    yprvcm : int
        On intermediate exit: with `irevcm` = 1, 2, 3, 4, 5, 6 or 7,
        `yprvcm` specifies the positions in `comm`['rwork'] at which you
        should place the derivatives y'. y'_i should be placed in
        location `comm`['rwork'][`yprvcm`+i-2], for i = 1,2,...,`neqf`.

        With `irevcm` = 9, 10, 11 or 12, `yprvcm` specifies the
        locations of the derivatives y' for use in evaluating the event
        functions. y'_i is given by `comm`['rwork'][`yprvcm`+i-2], for i
        = 1,2,...,`neqf`.

    kgrvcm : int
        On intermediate exit: with `irevcm` = 9, 10, 11 or 12, `kgrvcm`
        specifies which event function g_k(x,y,y') you must evaluate.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irevcm` had an illegal value, *<value>*.

        (`errno` 1)
            The value of `tout`, *<value>*, indicates a change in the
            integration direction. This is not permitted on a
            continuation call.

        (`errno` 1)
            The value of `t` has been changed from *<value>* to
            *<value>*.

            This is not permitted on a continuation call.

        (`errno` 1)
            On entry, `tout` = *<value>* but crit = True in
            ``ivp_adams_setup``.

            Integration cannot be attempted beyond tcrit = *<value>*.

        (`errno` 1)
            On entry, `tout` = `t` = *<value>*.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = 0 in
            ``ivp_adams_setup``.

            Constraint: if `neqg` <= 0 then `neqg` = neqg in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = 0 in
            ``ivp_adams_setup``.

            Constraint: if `neqg` > 0 then neqg > 0 in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = *<value>* in
            ``ivp_adams_setup``.

            Constraint: if `neqg` <= 0 then neqg <= 0 in
            ``ivp_adams_setup``.

        (`errno` 1)
            On entry, `neqf` = *<value>* and neqf = *<value>* in
            ``ivp_adams_setup``.

            Constraint: `neqf` = neqf in ``ivp_adams_setup``.

        (`errno` 1)
            The call to setup function ``ivp_adams_setup`` produced an
            error.

        (`errno` 1)
            The setup function ``ivp_adams_setup`` has not been called.

        (`errno` 3)
            The error tolerances are too stringent.

        (`errno` 7)
            Two successive errors detected at the current value of `t`,
            *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The maximum number of steps has been attempted.

            If integration is to be continued then the function may be
            called again and a further maxstp in ``ivp_adams_setup``
            steps will be attempted.

        (`errno` 4)
            Error weight i has become zero during the integration.

            atol[i] = 0.0 in ``ivp_adams_setup`` but `y`[i] is now 0.0.
            Integration successful as far as `t`.

        (`errno` 5)
            The problem appears to be stiff.

        (`errno` 6)
            A change in sign of an event function has been detected but
            the root-finding process appears to have converged to a
            singular point of `t` rather than a root. Integration may be
            continued by calling the function again.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_roots_revcom'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02qgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_FLOAT64, # Input/Output, t.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[neqf].
        utils._BYREF_FLOAT64, # Input, tout.
        utils._BYREF_INT, # Input, neqg.
        utils._BYREF_LOGICAL, # Output, root.
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_FLOAT64, # Output, trvcm.
        utils._BYREF_INT, # Output, yrvcm.
        utils._BYREF_INT, # Output, yprvcm.
        utils._BYREF_FLOAT64, # Input, grvcm.
        utils._BYREF_INT, # Input/Output, kgrvcm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[50].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[180].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[380].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqf = utils._EngineIntScalarType(
        neqf, {'fun_name': fun_name, 'entity': 'neqf'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _neqf_py = utils._EngineIntScalarType.to_py(_neqf)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neqf_py,),
    )
    _y = _y_cls.to_ctypes()
    _tout = utils._EngineFloat64ScalarType(
        tout, {'fun_name': fun_name, 'entity': 'tout'},
    ).to_ctypes()
    _neqg = utils._EngineIntScalarType(
        neqg, {'fun_name': fun_name, 'entity': 'neqg'},
    ).to_ctypes()
    _root = utils._EngineLogicalScalarType.empty_ctypes()
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _trvcm = utils._EngineFloat64ScalarType.empty_ctypes()
    _yrvcm = utils._EngineIntScalarType.empty_ctypes()
    _yprvcm = utils._EngineIntScalarType.empty_ctypes()
    _grvcm = utils._EngineFloat64ScalarType(
        grvcm, {'fun_name': fun_name, 'entity': 'grvcm'},
    ).to_ctypes()
    _kgrvcm = utils._EngineIntScalarType(
        kgrvcm, {'fun_name': fun_name, 'entity': 'kgrvcm'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwsav',),
        req_keys=('rwork', 'iwork', 'lwsav', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    _iwork = comm['iwork']
    _lwsav = comm['lwsav']
    _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(380)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqf,
        _t,
        _y,
        _tout,
        _neqg,
        _root,
        _irevcm,
        _trvcm,
        _yrvcm,
        _yprvcm,
        _grvcm,
        _kgrvcm,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm had an illegal value, {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of tout, {:s}, indicates a change in the'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'integration direction. This is not permitted on a continuation call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of t has been changed from {:s} to'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'This is not permitted on a continuation call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = {:s} but crit = True in ivp_adams_setup.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Integration cannot be attempted beyond tcrit = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tout = t = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and neqg = 0 in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if neqg <= 0 then neqg = neqg in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and neqg = 0 in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if neqg > 0 then neqg > 0 in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqg = {:s} in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if neqg <= 0 then neqg <= 0 in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqf = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqf = {:s} in ivp_adams_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neqf = neqf in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The call to setup function ivp_adams_setup produced an error.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The setup function ivp_adams_setup has not been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of steps has been attempted.',
                    'If integration is to be continued then the function may be called again',
                    'and a further maxstp in ivp_adams_setup steps will be attempted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The error tolerances are too stringent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error weight i has become zero during the integration.',
                    'atol[i] = 0.0 in ivp_adams_setup but y[i]',
                    'is now 0.0. Integration successful as far as t.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The problem appears to be stiff.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A change in sign of an event function has been detected but the root-finding',
                    'process appears to have converged to a singular point of t rather than',
                    'a root. Integration may be continued by calling the function again.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Two successive errors detected at the current value of t,',
                    '{:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        _y_cls.refresh_py(_y)
        _kgrvcm_py = utils._EngineIntScalarType.to_py(_kgrvcm)
        _root_py = utils._EngineLogicalScalarType.to_py(_root)
        _trvcm_py = utils._EngineFloat64ScalarType.to_py(_trvcm)
        _yrvcm_py = utils._EngineIntScalarType.to_py(_yrvcm)
        _yprvcm_py = utils._EngineIntScalarType.to_py(_yprvcm)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['t', 'root', 'irevcm', 'trvcm', 'yrvcm', 'yprvcm', 'kgrvcm'],
        )(
            _t_py,
            _root_py,
            _irevcm_py,
            _trvcm_py,
            _yrvcm_py,
            _yprvcm_py,
            _kgrvcm_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_adams_setup(statef, neqf, vectol, atol, latol, rtol, lrtol, onestp, crit, tcrit, hmax, maxstp, neqg, alterg, sophst, comm):
    u"""
    Ordinary differential equations, initial value problem, setup for
    ``ivp_adams_roots`` and ``ivp_adams_roots_revcom``.

    For full information please refer to the NAG Library document for
    d02qw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qwf.html

    Parameters
    ----------
    statef : str, length 1
        Specifies whether the integration function (``ivp_adams_roots``
        or ``ivp_adams_roots_revcom``) is to start a new system of
        ordinary differential equations, restart a system or continue
        with a system.

        `statef` = 'S'

          Start integration with a new differential system.

        `statef` = 'R'

          Restart integration with the current differential system.

        `statef` = 'C'

          Continue integration with the current differential system.

    neqf : int
        The number of ordinary differential equations to be solved by
        the integration function. `neqf` must remain unchanged on
        subsequent calls to ``ivp_adams_setup`` with `statef` = 'C' or
        'R'.

    vectol : bool
        Specifies whether vector or scalar error control is to be
        employed for the local error test in the integration.

        If `vectol` = True, vector error control will be used and you
        must specify values of `rtol`[i-1] and `atol`[i-1], for i =
        1,2,...,`neqf`.

        Otherwise scalar error control will be used and you must specify
        values of just `rtol`[0] and `atol`[0].

        The error test to be satisfied is of the form

            sqrt(sum_i = 1^`neqf`((e_i)/(w_i))^2) <= 1.0.

        where w_i is defined as follows:

        [table omitted]

        and e_i is an estimate of the local error in y_i, computed
        internally. `vectol` must remain unchanged on subsequent calls
        to ``ivp_adams_setup`` with `statef` = 'C' or 'R'.

    atol : float, shape (`latol`)
        The absolute local error tolerance (see `vectol`).

    latol : int
        The dimension of the array `atol`.

    rtol : float, shape (`lrtol`)
        The relative local error tolerance (see `vectol`).

    lrtol : int
        The dimension of the array `rtol`.

    onestp : bool
        The mode of operation of the integration function. If `onestp` =
        True, the integration function will operate in one-step mode,
        that is it will return after each successful step. Otherwise the
        integration function will operate in interval mode, that is it
        will return at the end of the integration interval.

    crit : bool
        Specifies whether or not there is a value for the independent
        variable beyond which integration is not to be attempted.
        Setting `crit` = True indicates that there is such a point,
        whereas `crit` = False indicates that there is no such
        restriction.

    tcrit : float
        With `crit` = True, `tcrit` must be set to a value of the
        independent variable beyond which integration is not to be
        attempted. Otherwise `tcrit` is not referenced.

    hmax : float
        If `hmax` != 0.0, a bound on the absolute step size during the
        integration is taken to be |`hmax`|.

        If `hmax` = 0.0, no bound is assumed on the step size during the
        integration.

        A bound may be required if there are features of the solution on
        very short ranges of integration which may be missed.

        You should try `hmax` = 0.0 first.

        Note: this argument only affects the step size if the option
        `crit` = True is being used.

    maxstp : int
        A bound on the number of attempted steps in any one call to the
        integration function. If `maxstp` <= 0 on entry, a value of 1000
        is used.

    neqg : int
        Specifies whether or not root-finding is required in
        ``ivp_adams_roots`` or ``ivp_adams_roots_revcom``.

        `neqg` <= 0

          No root-finding is attempted.

        `neqg` > 0

          Root-finding is required and `neqg` event functions will be
          specified for the integration function.

    alterg : bool
        Specifies whether or not the event functions have been
        redefined. `alterg` need not be set if `statef` = 'S'. On
        subsequent calls to ``ivp_adams_setup``, if `neqg` has been set
        positive, `alterg` = False specifies that the event functions
        remain unchanged, whereas `alterg` = True specifies that the
        event functions have changed. Because of the expense in
        reinitializing the root searching procedure, `alterg` should be
        set to True only if the event functions really have been
        altered. `alterg` need not be set if the root-finding option is
        not used.

    sophst : bool
        The type of search technique to be used in the root-finding. If
        `sophst` = True then a sophisticated and reliable but expensive
        technique will be used, whereas for `sophst` = False a simple
        but less reliable technique will be used. If `neqg` <= 0,
        `sophst` is not referenced.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    statef : str, length 1
        Is set to 'C', except that if an error is detected, `statef` is
        unchanged.

    alterg : bool
        Is set to False.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `statef` = *<value>*.

            Constraint: `statef` = 'S', 'R' or 'C'.

        (`errno` 1)
            On entry, for i = *<value>*, `atol`[i] = 0.0, `rtol`[i] =
            *<value>* and 4*eps = *<value>*.

            Constraint: if `atol`[i] = 0.0 then `rtol`[i] >= 4*eps for
            all i.

        (`errno` 1)
            On entry, `rtol`[*<value>*] = *<value>*.

            Constraint: `rtol`[i] >= 0.0 for all i.

        (`errno` 1)
            On entry, `atol`[*<value>*] = *<value>*.

            Constraint: `atol`[i] >= 0.0 for all i.

        (`errno` 1)
            On entry, `statef` = 'R' or 'C', `alterg` = False and `neqg`
            = *<value>*.

            Constraint: if `statef` = 'R' or 'C' and `alterg` = False
            then `neqg` must retain its value from the prior call with
            `statef` = 'S'.

        (`errno` 1)
            On entry, `statef` = 'R' or 'C' and `vectol` = False.

            Constraint: if `statef` = 'R' or 'C' then `vectol` must
            retain its value from the prior call with `statef` = 'S'.

        (`errno` 1)
            On entry, `statef` = 'R' or 'C' and `vectol` = True.

            Constraint: if `statef` = 'R' or 'C' then `vectol` must
            retain its value from the prior call with `statef` = 'S'.

        (`errno` 1)
            On entry, `statef` = 'R' or 'C' and `neqf` = *<value>*.

            Constraint: if `statef` = 'R' or 'C' then `neqf` must retain
            its value, *<value>*, from the prior call with `statef` =
            'S'.

        (`errno` 1)
            On entry, `latol` = *<value>*.

            Constraint: `latol` >= 1.

        (`errno` 1)
            On entry, `lrtol` = *<value>*.

            Constraint: `lrtol` >= 1.

        (`errno` 1)
            On entry, `vectol` = True, `latol` = *<value>* and `neqf` =
            *<value>*.

            Constraint: if `vectol` = True then `latol` >= `neqf`.

        (`errno` 1)
            On entry, `vectol` = True, `lrtol` = *<value>* and `neqf` =
            *<value>*.

            Constraint: if `vectol` = True then `lrtol` >= `neqf`.

        (`errno` 1)
            On entry, `neqf` = *<value>*.

            Constraint: `neqf` >= 1.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02qwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input/Output, statef(len=1).
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_LOGICAL, # Input, vectol.
        utils._EngineFloat64ArrayType.c_type, # Input, atol[latol].
        utils._BYREF_INT, # Input, latol.
        utils._EngineFloat64ArrayType.c_type, # Input, rtol[lrtol].
        utils._BYREF_INT, # Input, lrtol.
        utils._BYREF_LOGICAL, # Input, onestp.
        utils._BYREF_LOGICAL, # Input, crit.
        utils._BYREF_FLOAT64, # Input, tcrit.
        utils._BYREF_FLOAT64, # Input, hmax.
        utils._BYREF_INT, # Input, maxstp.
        utils._BYREF_INT, # Input, neqg.
        utils._BYREF_LOGICAL, # Input/Output, alterg.
        utils._BYREF_LOGICAL, # Input, sophst.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[50].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[180].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of statef.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _statef = utils._EngineCharScalarType(
        statef, {'fun_name': fun_name, 'entity': 'statef'}, exp_chrlen=1,
    ).to_ctypes()
    _neqf = utils._EngineIntScalarType(
        neqf, {'fun_name': fun_name, 'entity': 'neqf'},
    ).to_ctypes()
    _vectol = utils._EngineLogicalScalarType(
        vectol, {'fun_name': fun_name, 'entity': 'vectol'},
    ).to_ctypes()
    _latol = utils._EngineIntScalarType(
        latol, {'fun_name': fun_name, 'entity': 'latol'},
    ).to_ctypes()
    _latol_py = utils._EngineIntScalarType.to_py(_latol)
    _atol = utils._EngineFloat64ArrayType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
        exp_shape=(_latol_py,),
    ).to_ctypes()
    _lrtol = utils._EngineIntScalarType(
        lrtol, {'fun_name': fun_name, 'entity': 'lrtol'},
    ).to_ctypes()
    _lrtol_py = utils._EngineIntScalarType.to_py(_lrtol)
    _rtol = utils._EngineFloat64ArrayType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
        exp_shape=(_lrtol_py,),
    ).to_ctypes()
    _onestp = utils._EngineLogicalScalarType(
        onestp, {'fun_name': fun_name, 'entity': 'onestp'},
    ).to_ctypes()
    _crit = utils._EngineLogicalScalarType(
        crit, {'fun_name': fun_name, 'entity': 'crit'},
    ).to_ctypes()
    _tcrit = utils._EngineFloat64ScalarType(
        tcrit, {'fun_name': fun_name, 'entity': 'tcrit'},
    ).to_ctypes()
    _hmax = utils._EngineFloat64ScalarType(
        hmax, {'fun_name': fun_name, 'entity': 'hmax'},
    ).to_ctypes()
    _maxstp = utils._EngineIntScalarType(
        maxstp, {'fun_name': fun_name, 'entity': 'maxstp'},
    ).to_ctypes()
    _neqg = utils._EngineIntScalarType(
        neqg, {'fun_name': fun_name, 'entity': 'neqg'},
    ).to_ctypes()
    _alterg = utils._EngineLogicalScalarType(
        alterg, {'fun_name': fun_name, 'entity': 'alterg'},
    ).to_ctypes()
    _sophst = utils._EngineLogicalScalarType(
        sophst, {'fun_name': fun_name, 'entity': 'sophst'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rwork', 'iwork', 'lwsav', 'iwsav'),
    )
    _neqf_py = utils._EngineIntScalarType.to_py(_neqf)
    _neqg_py = utils._EngineIntScalarType.to_py(_neqg)
    _vectol_py = utils._EngineLogicalScalarType.to_py(_vectol)
    if _vectol_py:
        _vu_j_py = _neqf_py
    else:
        _vu_j_py = 1
    _sophst_py = utils._EngineLogicalScalarType.to_py(_sophst)
    if _sophst_py:
        _vu_k_py = 14
    else:
        _vu_k_py = 5
    if _comm_needs_init:
        _rwork = utils._EngineFloat64ArrayType.empty_ctypes(21*(1+_neqf_py)+2*_vu_j_py+_vu_k_py*_neqg_py+2)
        comm['rwork'] = _rwork
    else:
        _rwork = comm['rwork']
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    if _comm_needs_init:
        if _sophst_py:
            _iwork_dim1 = 21+4*_neqg_py
        elif not _sophst_py:
            _iwork_dim1 = 21+_neqg_py
        else:
            _iwork_dim1 = 0
        _iwork = utils._EngineIntArrayType.empty_ctypes(_iwork_dim1)
        comm['iwork'] = _iwork
    else:
        _iwork = comm['iwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(50)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(180)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _statef,
        _neqf,
        _vectol,
        _atol,
        _latol,
        _rtol,
        _lrtol,
        _onestp,
        _crit,
        _tcrit,
        _hmax,
        _maxstp,
        _neqg,
        _alterg,
        _sophst,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _lwsav,
        _iwsav,
        _errbuf,
        _ierr,
        _statef._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statef = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: statef = \'S\', \'R\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s}, atol[i] = 0.0,'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'rtol[i] = {:s} and 4*eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if atol[i] = 0.0 then rtol[i] >= 4*eps for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rtol[i] >= 0.0 for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: atol[i] >= 0.0 for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statef = \'R\' or \'C\', alterg = False and',
                    'neqg = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if statef = \'R\' or \'C\' and alterg = False then neqg must',
                    'retain its value from the prior call with statef = \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statef = \'R\' or \'C\' and vectol = False.',
                    'Constraint: if statef = \'R\' or \'C\' then vectol must retain its value',
                    'from the prior call with statef = \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statef = \'R\' or \'C\' and vectol = True.',
                    'Constraint: if statef = \'R\' or \'C\' then vectol must retain its value',
                    'from the prior call with statef = \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, statef = \'R\' or \'C\' and neqf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if statef = \'R\' or \'C\' then neqf must retain its value,',
                    '{:s}, from the prior call with statef = \'S\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, latol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: latol >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lrtol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lrtol >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vectol = True, latol = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if vectol = True then latol >= neqf.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vectol = True, lrtol = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'neqf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if vectol = True then lrtol >= neqf.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neqf >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _statef_py = utils._EngineCharScalarType.to_py(_statef)
        _alterg_py = utils._EngineLogicalScalarType.to_py(_alterg)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['statef', 'alterg'],
        )(
            _statef_py,
            _alterg_py,
        )
    _err_class.manage()
    return return_data

def ivp_adams_diag(neqf, yp, comm):
    u"""
    Ordinary differential equations, initial value problem, diagnostics
    for ``ivp_adams_roots`` and ``ivp_adams_roots_revcom``.

    For full information please refer to the NAG Library document for
    d02qx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qxf.html

    Parameters
    ----------
    neqf : int
        The number of first-order ordinary differential equations solved
        by the integration function. It must be the same argument `neqf`
        supplied to the setup function ``ivp_adams_setup`` and the
        integration functions ``ivp_adams_roots`` or
        ``ivp_adams_roots_revcom``.

    yp : float, shape (`neqf`), modified in place
        On exit: the approximate derivative of the solution component
        y_i, as supplied in y_i on output from the integration function
        at the output value of t. These values are obtained by the
        evaluation of y' = f(x,y) except when the output value of the
        argument t in the call to the integration function is tout and
        `tcurr` != tout, in which case they are obtained by
        interpolation.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_adams_setup`` and one of ``ivp_adams_roots`` or
        ``ivp_adams_roots_revcom``.

    Returns
    -------
    tcurr : float
        The value of the independent variable which the integrator has
        actually reached. `tcurr` will always be at least as far as the
        output value of the argument t (from the integration function)
        in the direction of integration, but may be further.

    hlast : float
        The last successful step size used by the integrator.

    hnext : float
        The next step size which the integration function would attempt.

    odlast : int
        The order of the method last used (successfully) by the
        integration function.

    odnext : int
        The order of the method which the integration function would
        attempt on the next step.

    nsucc : int
        The number of steps attempted by the integration function that
        have been successful since the start of the current problem.

    nfail : int
        The number of steps attempted by the integration function that
        have failed since the start of the current problem.

    tolfac : float
        A tolerance scale factor, `tolfac` >= 1.0, returned when the
        integration function exits with `errno` = 3. If rtol and atol
        are uniformly scaled up by a factor of this argument and
        ``ivp_adams_setup`` is called, the next call to the integration
        function is deemed likely to succeed.

    badcmp : int
        If the integration function returned with `errno` = 4, `badcmp`
        specifies the index of the component which forced the error
        exit. Otherwise `badcmp` is 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neqf` = *<value>* and neqf = *<value>* in
            ``ivp_adams_setup``.

            Constraint: `neqf` = neqf in ``ivp_adams_setup``.

        (`errno` 1)
            Neither of the appropriate two integrator functions has been
            called.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02qxft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqf.
        utils._EngineFloat64ArrayType.c_type, # Output, yp[neqf].
        utils._BYREF_FLOAT64, # Output, tcurr.
        utils._BYREF_FLOAT64, # Output, hlast.
        utils._BYREF_FLOAT64, # Output, hnext.
        utils._BYREF_INT, # Output, odlast.
        utils._BYREF_INT, # Output, odnext.
        utils._BYREF_INT, # Output, nsucc.
        utils._BYREF_INT, # Output, nfail.
        utils._BYREF_FLOAT64, # Output, tolfac.
        utils._BYREF_INT, # Output, badcmp.
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineIntArrayType.c_type, # Input, iwsav[180].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqf = utils._EngineIntScalarType(
        neqf, {'fun_name': fun_name, 'entity': 'neqf'},
    ).to_ctypes()
    _neqf_py = utils._EngineIntScalarType.to_py(_neqf)
    _yp_cls = utils._EngineFloat64ArrayType(
        yp, {'fun_name': fun_name, 'entity': 'yp'},
        exp_shape=(_neqf_py,),
        check_contents=False,
    )
    _yp = _yp_cls.to_ctypes(can_reference_contents=False)
    _tcurr = utils._EngineFloat64ScalarType.empty_ctypes()
    _hlast = utils._EngineFloat64ScalarType.empty_ctypes()
    _hnext = utils._EngineFloat64ScalarType.empty_ctypes()
    _odlast = utils._EngineIntScalarType.empty_ctypes()
    _odnext = utils._EngineIntScalarType.empty_ctypes()
    _nsucc = utils._EngineIntScalarType.empty_ctypes()
    _nfail = utils._EngineIntScalarType.empty_ctypes()
    _tolfac = utils._EngineFloat64ScalarType.empty_ctypes()
    _badcmp = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'iwork', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    _iwork = comm['iwork']
    _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqf,
        _yp,
        _tcurr,
        _hlast,
        _hnext,
        _odlast,
        _odnext,
        _nsucc,
        _nfail,
        _tolfac,
        _badcmp,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqf = {:s} and neqf = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ivp_adams_setup.',
                    'Constraint: neqf = neqf in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Neither of the appropriate two integrator functions has been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _yp_cls.refresh_py(_yp)
        _tcurr_py = utils._EngineFloat64ScalarType.to_py(_tcurr)
        _hlast_py = utils._EngineFloat64ScalarType.to_py(_hlast)
        _hnext_py = utils._EngineFloat64ScalarType.to_py(_hnext)
        _odlast_py = utils._EngineIntScalarType.to_py(_odlast)
        _odnext_py = utils._EngineIntScalarType.to_py(_odnext)
        _nsucc_py = utils._EngineIntScalarType.to_py(_nsucc)
        _nfail_py = utils._EngineIntScalarType.to_py(_nfail)
        _tolfac_py = utils._EngineFloat64ScalarType.to_py(_tolfac)
        _badcmp_py = utils._EngineIntScalarType.to_py(_badcmp)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['tcurr', 'hlast', 'hnext', 'odlast', 'odnext', 'nsucc', 'nfail', 'tolfac', 'badcmp'],
        )(
            _tcurr_py,
            _hlast_py,
            _hnext_py,
            _odlast_py,
            _odnext_py,
            _nsucc_py,
            _nfail_py,
            _tolfac_py,
            _badcmp_py,
        )
    _err_class.manage()
    return return_data

def ivp_adams_rootdiag(neqg, events, resids, comm):
    u"""
    Ordinary differential equations, initial value problem, root-finding
    diagnostics for ``ivp_adams_roots`` and ``ivp_adams_roots_revcom``.

    For full information please refer to the NAG Library document for
    d02qy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qyf.html

    Parameters
    ----------
    neqg : int
        The number of event functions defined for the integration
        function. It must be the same argument `neqg` supplied to the
        setup function ``ivp_adams_setup`` and to the integration
        function (``ivp_adams_roots`` or ``ivp_adams_roots_revcom``).

    events : int, shape (`neqg`), modified in place
        On exit: information about the kth event function on a very
        small interval containing the root, t (see ``ivp_adams_roots``
        and ``ivp_adams_roots_revcom``), as output from the integration
        function. All roots lying in this interval are considered
        indistinguishable numerically and therefore should be regarded
        as defining a root at t. The possible values of `events`[k-1]
        with their interpretations are as follows:

        `events`[k-1] = 0

          The kth event function did not have a root.

        `events`[k-1] = -1

          The kth event function changed sign from positive to negative
          about a root, in the direction of integration.

        `events`[k-1] = 1

          The kth event function changed sign from negative to positive
          about a root, in the direction of integration.

        `events`[k-1] = 2

          A root was identified, but no change in sign was observed.

    resids : float, shape (`neqg`), modified in place
        On exit: the value of the kth event function computed at the
        root, t (see ``ivp_adams_roots`` and
        ``ivp_adams_roots_revcom``).

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_adams_setup`` and one of ``ivp_adams_roots`` or
        ``ivp_adams_roots_revcom``.

    Returns
    -------
    index : int
        The index k of the event equation g_k(x,y,y') = 0 for which the
        root has been detected.

    itype : int
        Information about the root detected for the event equation
        defined by `index`. The possible values of `itype` with their
        interpretations are as follows:

        `itype` = 1

          A simple root, or lack of distinguishing information
          available.

        `itype` = 2

          A root of even multiplicity is believed to have been detected,
          that is no change in sign of the event function was found.

        `itype` = 3

          A high-order root of odd multiplicity.

        `itype` = 4

          A possible root, but due to high multiplicity or a clustering
          of roots accurate evaluation of the event function was
          prohibited by round-off error and/or cancellation.

        In general, the accuracy of the root is less reliable for values
        of `itype` > 1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `neqg` = *<value>* and neqg = *<value>* in
            ``ivp_adams_setup``.

            Constraint: `neqg` = neqg in ``ivp_adams_setup``.

        (`errno` 1)
            The integrator did not end at an event.

        (`errno` 1)
            Neither of the appropriate two integrator functions has been
            called.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_rootdiag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02qyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqg.
        utils._BYREF_INT, # Output, index.
        utils._BYREF_INT, # Output, itype.
        utils._EngineIntArrayType.c_type, # Output, events[neqg].
        utils._EngineFloat64ArrayType.c_type, # Output, resids[neqg].
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineIntArrayType.c_type, # Input, iwsav[180].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqg = utils._EngineIntScalarType(
        neqg, {'fun_name': fun_name, 'entity': 'neqg'},
    ).to_ctypes()
    _index = utils._EngineIntScalarType.empty_ctypes()
    _itype = utils._EngineIntScalarType.empty_ctypes()
    _neqg_py = utils._EngineIntScalarType.to_py(_neqg)
    _events_cls = utils._EngineIntArrayType(
        events, {'fun_name': fun_name, 'entity': 'events'},
        exp_shape=(_neqg_py,),
        check_contents=False,
    )
    _events = _events_cls.to_ctypes(can_reference_contents=False)
    _resids_cls = utils._EngineFloat64ArrayType(
        resids, {'fun_name': fun_name, 'entity': 'resids'},
        exp_shape=(_neqg_py,),
        check_contents=False,
    )
    _resids = _resids_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'iwork', 'iwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    _iwork = comm['iwork']
    _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqg,
        _index,
        _itype,
        _events,
        _resids,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqg = {:s} and neqg = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ivp_adams_setup.',
                    'Constraint: neqg = neqg in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integrator did not end at an event.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Neither of the appropriate two integrator functions has been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _events_cls.refresh_py(_events)
        _resids_cls.refresh_py(_resids)
        _index_py = utils._EngineIntScalarType.to_py(_index)
        _itype_py = utils._EngineIntScalarType.to_py(_itype)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['index', 'itype'],
        )(
            _index_py,
            _itype_py,
        )
    _err_class.manage()
    return return_data

def ivp_adams_interp(neqf, twant, nwant, ywant, ypwant, comm):
    u"""
    Ordinary differential equations, initial value problem,
    interpolation for ``ivp_adams_roots`` or ``ivp_adams_roots_revcom``.

    For full information please refer to the NAG Library document for
    d02qz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02qzf.html

    Parameters
    ----------
    neqf : int
        The number of first-order ordinary differential equations being
        solved by the integration function. It must contain the same
        value as the argument `neqf` in a prior call to the setup
        function ``ivp_adams_setup``.

    twant : float
        The point at which components of the solution and derivative are
        to be evaluated. `twant` should not normally be an extrapolation
        point, that is `twant` should satisfy

        - told <= `twant` <= T,

        or if integration is proceeding in the negative direction

        - told >= `twant` >= T,

        where told is the previous integration point and is, to within
        rounding, tcurr -- hlast (see ``ivp_adams_diag``). Extrapolation
        is permitted but not recommended and `errno` = 2 is returned
        whenever extrapolation is attempted.

    nwant : int
        The number of components of the solution and derivative whose
        values at `twant` are required. The first `nwant` components are
        evaluated.

    ywant : float, shape (`nwant`), modified in place
        On exit: the calculated value of the ith component of the
        solution at `twant`, for i = 1,2,...,`nwant`.

    ypwant : float, shape (`nwant`), modified in place
        On exit: the calculated value of the ith component of the
        derivative at `twant`, for i = 1,2,...,`nwant`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_adams_setup`` and one of ``ivp_adams_roots`` or
        ``ivp_adams_roots_revcom``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwant` = *<value>*.

            Constraint: `nwant` >= 1.

        (`errno` 1)
            On entry, `nwant` = *<value>* and `neqf` = *<value>*.

            Constraint: `nwant` <= `neqf`.

        (`errno` 1)
            On entry, `neqf` = *<value>* and neqf = *<value>* in
            ``ivp_adams_setup``.

            Constraint: `neqf` = neqf in ``ivp_adams_setup``.

        (`errno` 1)
            Neither of the appropriate two integrator functions has been
            called.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            No successful steps had been taken, so interpolation is
            impossible at `twant` = *<value>*.

        (`errno` 2)
            Extrapolation performed at `twant` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_adams_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02qzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neqf.
        utils._BYREF_FLOAT64, # Input, twant.
        utils._BYREF_INT, # Input, nwant.
        utils._EngineFloat64ArrayType.c_type, # Output, ywant[nwant].
        utils._EngineFloat64ArrayType.c_type, # Output, ypwant[nwant].
        utils._EngineFloat64ArrayType.c_type, # Input, rwork[lrwork].
        utils._BYREF_INT, # Input, lrwork.
        utils._EngineIntArrayType.c_type, # Input, iwork[liwork].
        utils._BYREF_INT, # Input, liwork.
        utils._EngineLogicalArrayType.c_type, # Input, lwsav[50].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[180].
        utils._EngineFloat64ArrayType.c_type, # Input, rwsav[380].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neqf = utils._EngineIntScalarType(
        neqf, {'fun_name': fun_name, 'entity': 'neqf'},
    ).to_ctypes()
    _twant = utils._EngineFloat64ScalarType(
        twant, {'fun_name': fun_name, 'entity': 'twant'},
    ).to_ctypes()
    _nwant = utils._EngineIntScalarType(
        nwant, {'fun_name': fun_name, 'entity': 'nwant'},
    ).to_ctypes()
    _nwant_py = utils._EngineIntScalarType.to_py(_nwant)
    _ywant_cls = utils._EngineFloat64ArrayType(
        ywant, {'fun_name': fun_name, 'entity': 'ywant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ywant = _ywant_cls.to_ctypes(can_reference_contents=False)
    _ypwant_cls = utils._EngineFloat64ArrayType(
        ypwant, {'fun_name': fun_name, 'entity': 'ypwant'},
        exp_shape=(_nwant_py,),
        check_contents=False,
    )
    _ypwant = _ypwant_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rwork', 'iwork', 'lwsav', 'iwsav', 'rwsav'),
    )
    _lrwork_py = utils._EngineFloat64ArrayType.get_shape(comm['rwork'], {'fun_name': fun_name, 'entity': 'comm[\'rwork\']'}, exp_rank=1, validate=False)[0]
    _lrwork = utils._EngineIntScalarType(
        _lrwork_py, {'fun_name': fun_name, 'entity': '_lrwork'},
    ).to_ctypes()
    _rwork = comm['rwork']
    _liwork_py = utils._EngineIntArrayType.get_shape(comm['iwork'], {'fun_name': fun_name, 'entity': 'comm[\'iwork\']'}, exp_rank=1, validate=False)[0]
    _liwork = utils._EngineIntScalarType(
        _liwork_py, {'fun_name': fun_name, 'entity': '_liwork'},
    ).to_ctypes()
    _iwork = comm['iwork']
    _lwsav = comm['lwsav']
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neqf,
        _twant,
        _nwant,
        _ywant,
        _ypwant,
        _rwork,
        _lrwork,
        _iwork,
        _liwork,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwant >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwant = {:s} and neqf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwant <= neqf.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No successful steps had been taken, so interpolation is impossible at',
                    'twant = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neqf = {:s} and neqf = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ivp_adams_setup.',
                    'Constraint: neqf = neqf in ivp_adams_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Neither of the appropriate two integrator functions has been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extrapolation performed at twant = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ywant_cls.refresh_py(_ywant)
        _ypwant_cls.refresh_py(_ypwant)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_fd_nonlin_gen(n, mnp, np, numbeg, nummix, tol, init, x, y, abt, fcn, g, ijac, jacobf, jacobg, deleps, jaceps, jacgep, itrace, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, finite difference technique with deferred correction,
    continuation facility.

    For full information please refer to the NAG Library document for
    d02ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02raf.html

    Parameters
    ----------
    n : int
        n, the number of differential equations.

    mnp : int
        `mnp` must be set to the maximum permitted number of points in
        the finite difference mesh. If lwork or liwork are too small
        then internally `mnp` will be replaced by the maximum permitted
        by these values.

    np : int
        Must be set to the number of points to be used in the initial
        mesh.

    numbeg : int
        The number of left-hand boundary conditions (that is the number
        involving y(a) only).

    nummix : int
        The number of coupled boundary conditions (that is the number
        involving both y(a) and y(b)).

    tol : float
        A positive absolute error tolerance. If

            a = x_1 < x_2 < ... < x_`np` = b

        is the final mesh, z_j(x_i) is the jth component of the
        approximate solution at x_i, and y_j(x) is the jth component of
        the true solution of (1) and (2), then, except in extreme
        circumstances, it is expected that

            |z_j(x_i)-y_j(x_i)| <= `tol`, i = 1,2,...,`np` and j =
            1,2,...,n.

    init : int
        Indicates whether you wish to supply an initial mesh and
        approximate solution (`init` = 1) or whether default values are
        to be used, (`init` = 0).

    x : float, shape (`mnp`), modified in place
        On entry: you must set `x`[0] = a and `x`[`np`-1] = b. If `init`
        = 0 on entry a default equispaced mesh will be used, otherwise
        you must specify a mesh by setting `x`[i-1] = x_i, for i =
        2,3,...,`np`-1.

        On exit: `x`[0],`x`[1],...,`x`[`np`-1] define the final mesh
        (with the returned value of `np`) and `x`[0] = a and `x`[`np`-1]
        = b.

    y : float, shape (`n`, `mnp`), modified in place
        On entry: if `init` = 0, `y` need not be set.

        If `init` = 1, the array `y` must contain an initial
        approximation to the solution such that `y`[j-1,i-1] contains an
        approximation to

            y_j(x_i), i = 1,2,...,`np` and j = 1,2,...,n.

        On exit: the approximate solution z_j(x_i) satisfying (5) on the
        final mesh, that is

            `y`[j-1,i-1] = z_j(x_i), i = 1,2,...,`np` and j = 1,2,...,n,

        where `np` is the number of points in the final mesh. If an
        error has occurred then `y` contains the latest approximation to
        the solution. The remaining columns of `y` are not used.

    abt : float, shape (`n`), modified in place
        On exit: `abt`[i-1], for i = 1,2,...,n, holds the largest
        estimated error (in magnitude) of the ith component of the
        solution over all mesh points.

    fcn : callable fcn(x, eps, y, f, n, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i) at a general point x for a given value of epsilon, the
        continuation parameter (see [Description]).

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        eps : float
            epsilon, the value of the continuation parameter. This is 1
            if continuation is not being used.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the values of the dependent
            variables at x.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the values of the derivatives f_i evaluated at x
            given epsilon, for i = 1,2,...,n.

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    g : callable g(eps, ya, yb, bc, n, data=None)
        `g` must evaluate the boundary conditions in equation (3) and
        place them in the array `bc`.

        Parameters
        ~~~~~~~~~~
        eps : float
            epsilon, the value of the continuation parameter. This is 1
            if continuation is not being used.

        ya : float, ndarray, shape (`n`)
            The value y_i(a), for i = 1,2,...,n.

        yb : float, ndarray, shape (`n`)
            The value y_i(b), for i = 1,2,...,n.

        bc : float, ndarray, shape (`n`), to be modified in place
            On exit: the values g_i(y(a),y(b),epsilon), for i =
            1,2,...,n. These must be ordered as follows:

            - first, the conditions involving only y(a) (see `numbeg`);

            - next, the `nummix` coupled conditions involving both y(a)
              and y(b) (see `nummix`); and,

            - finally, the conditions involving only y(b)
              (`n`-`numbeg`-`nummix`).

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    ijac : int
        Indicates whether or not you are supplying Jacobian evaluation
        functions.

        `ijac` != 0

          You must supply `jacobf` and `jacobg` and also, when
          continuation is used, `jaceps` and `jacgep`.

        `ijac` = 0

          Numerical differentiation is used to calculate the Jacobian
          and None may be used for the respective callbacks.

    jacobf : None or callable jacobf(x, eps, y, f, n, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jacobf` evaluates the Jacobian (df_i/dy_j), for j = 1,2,...,n,
        for i = 1,2,...,n, given x and y_j, for j = 1,2,...,n.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        eps : float
            epsilon, the value of the continuation parameter. This is 1
            if continuation is not being used.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the values of the dependent
            variables at x.

        f : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `f`[j-1,i-1] must be set to the value of df_i/dy_j,
            evaluated at the point (x,y), for j = 1,2,...,n, for i =
            1,2,...,n.

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    jacobg : None or callable jacobg(eps, ya, yb, aj, bj, n, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jacobg` evaluates the Jacobians (dg_i/(dy_j(a))) and
        (dg_i/(dy_j(b))).

        The ordering of the rows of `aj` and `bj` must correspond to the
        ordering of the boundary conditions described in the
        specification of `g`.

        Parameters
        ~~~~~~~~~~
        eps : float
            epsilon, the value of the continuation parameter. This is 1
            if continuation is not being used.

        ya : float, ndarray, shape (`n`)
            The value y_i(a), for i = 1,2,...,n.

        yb : float, ndarray, shape (`n`)
            The value y_i(b), for i = 1,2,...,n.

        aj : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `aj`[i-1,j-1] must be set to the value
            dg_i/(dy_j(a)), for j = 1,2,...,n, for i = 1,2,...,n.

        bj : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `bj`[i-1,j-1] must be set to the value
            dg_i/(dy_j(b)), for j = 1,2,...,n, for i = 1,2,...,n.

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    deleps : float
        Must be given a value which specifies whether continuation is
        required. If `deleps` <= 0.0 or `deleps` >= 1.0 then it is
        assumed that continuation is not required. If 0.0 < `deleps` <
        1.0 then it is assumed that continuation is required unless
        `deleps` < sqrt(machine precision) when an error exit is taken.
        `deleps` is used as the increment epsilon_2-epsilon_1 (see (4))
        and the choice `deleps` = 0.1 is recommended.

    jaceps : None or callable jaceps(x, eps, y, f, n, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jaceps` evaluates the derivative df_i/d epsilon given x and y
        if continuation is being used.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the independent variable.

        eps : float
            epsilon, the value of the continuation parameter.

        y : float, ndarray, shape (`n`)
            The solution values y_i, for i = 1,2,...,n, at the point x.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: `f`[i-1] must contain the value df_i/d epsilon at
            the point (x,y), for i = 1,2,...,n.

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    jacgep : None or callable jacgep(eps, ya, yb, bcep, n, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `jacgep` evaluates the derivatives dg_i/d epsilon if
        continuation is being used.

        Parameters
        ~~~~~~~~~~
        eps : float
            epsilon, the value of the continuation parameter.

        ya : float, ndarray, shape (`n`)
            The value of y_i(a), for i = 1,2,...,n.

        yb : float, ndarray, shape (`n`)
            The value of y_i(b), for i = 1,2,...,n.

        bcep : float, ndarray, shape (`n`), to be modified in place
            On exit: `bcep`[i-1] must contain the value of dg_i/d
            epsilon, for i = 1,2,...,n.

        n : int
            n, the number of equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    itrace : int
        If `itrace` = 0 warning messages be suppressed, otherwise
        warning messages will be printed (see [Errors]).

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    np : int
        The number of points in the final mesh.

    deleps : float
        An overestimate of the increment epsilon_p-epsilon_{p-1} (in
        fact the value of the increment which would have been tried if
        the restriction epsilon_p = 1 had not been imposed). If
        continuation was not requested then `deleps` = 0.0.

        If continuation is not requested then `jaceps` and `jacgep` may
        each be replaced by None.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry the mesh points are not in strictly ascending
            order.

            For i = *<value>*, mesh point i = *<value>*, but mesh point
            i+1 = *<value>*.

        (`errno` 1)
            On entry, `numbeg` = *<value>*, `nummix` = *<value>* and `n`
            = *<value>*.

            Constraint: `numbeg`+`nummix` <= `n`.

        (`errno` 1)
            On entry, `nummix` = *<value>*.

            Constraint: `nummix` >= 0.

        (`errno` 1)
            On entry, `numbeg` = *<value>* and `n` = *<value>*.

            Constraint: `numbeg` < `n`.

        (`errno` 1)
            On entry, `numbeg` = *<value>*.

            Constraint: `numbeg` >= 0.

        (`errno` 1)
            On entry, `x`[0] = *<value>* and `x`[`np`-1] = *<value>*.

            Constraint: `x`[0] < `x`[`np`-1].

        (`errno` 1)
            On entry, `mnp` = *<value>*.

            Constraint: `mnp` >= 32.

        (`errno` 1)
            On entry, `np` = *<value>* and `mnp` = *<value>*.

            Constraint: `np` <= `mnp`.

        (`errno` 1)
            On entry, `np` = *<value>*.

            Constraint: `np` >= 4.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            A finer mesh is required for the accuracy requested; that
            is, `mnp` = *<value>* is not large enough.

        (`errno` 3)
            The Newton iteration has failed to converge.

            This could be due to there being too few points in the
            initial mesh or to the initial approximate solution being
            too inaccurate.

            If this latter reason is suspected or you cannot make
            changes to prevent this error, you should use the function
            with a continuation facility instead.

        (`errno` 5)
            The Jacobian for the boundary conditions is singular.

            This may occur due to faulty coding of the Jacobian or, in
            some circumstances, to a zero initial choice of approximate
            solution.

        (`errno` 6)
            There is no dependence on the continuation parameter when
            continuation is being used. This can be due to faulty coding
            of derivatives with respect to the continuation parameter or
            to a zero initial choice of approximate solution.

        (`errno` 7)
            The continuation step is required to be less than machine
            precision for continuation to proceed. It is likely that
            either the problem has no solution for some value of the
            continuation parameter near the current value or that the
            problem is so difficult that even with continuation it is
            unlikely to be solved using this function. In the latter
            case using more mesh points initially may help.

        (`errno` 8)
            A serious error occurred in a call to the internal
            integrator.

            The error code internally was *<value>*.

            Please contact NAG.

        (`errno` 9)
            A continuation error occurred, but continuation is not being
            used.

            Please contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Newton iteration has reached round-off level.

            If desired accuracy has not been reached, `tol` is too small
            for this problem and this machine precision.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_fd_nonlin_gen'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, x, eps, y, f, n, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _eps_fcnh = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _x_fcnh,
                    _eps_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _n_fcnh,
                    data,
                )
            else:
                fcn(
                    _x_fcnh,
                    _eps_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _n_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, ya[n].
        utils._EngineFloat64ArrayType.c_type, # Input, yb[n].
        utils._EngineFloat64ArrayType.c_type, # Output, bc[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, en_data, eps, ya, yb, bc, n, _iuser, _ruser):
        """Helper for g."""
        try:
            _eps_gh = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_gh = utils._EngineIntScalarType.to_py(n.contents)
            _ya_gh = utils._EngineFloat64ArrayType.from_pointer(
                ya,
                (_n_gh,),
            )
            _yb_gh = utils._EngineFloat64ArrayType.from_pointer(
                yb,
                (_n_gh,),
            )
            _bc_gh = utils._EngineFloat64ArrayType.from_pointer(
                bc,
                (_n_gh,),
                can_reference_contents=False,
            )
            if data is not None:
                g(
                    _eps_gh,
                    _ya_gh,
                    _yb_gh,
                    _bc_gh,
                    _n_gh,
                    data,
                )
            else:
                g(
                    _eps_gh,
                    _ya_gh,
                    _yb_gh,
                    _bc_gh,
                    _n_gh,
                )
            utils._unlink_pointer(ya)
            utils._unlink_pointer(yb)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _bc_gh, {'fun_name': cb_g_name, 'returned': 'bc'}, bc,
            )
            utils._unlink_pointer(bc)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jacobf_name = fun_name + ':jacobf'

    jacobfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n, n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jacobfh(_jacobf, en_data, x, eps, y, f, n, _iuser, _ruser):
        """Helper for jacobf."""
        if jacobf is None:
            return
        try:
            _x_jacobfh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _eps_jacobfh = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_jacobfh = utils._EngineIntScalarType.to_py(n.contents)
            _y_jacobfh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_jacobfh,),
            )
            _f_jacobfh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_jacobfh, _n_jacobfh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                jacobf(
                    _x_jacobfh,
                    _eps_jacobfh,
                    _y_jacobfh,
                    _f_jacobfh,
                    _n_jacobfh,
                    data,
                )
            else:
                jacobf(
                    _x_jacobfh,
                    _eps_jacobfh,
                    _y_jacobfh,
                    _f_jacobfh,
                    _n_jacobfh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_jacobfh, {'fun_name': cb_jacobf_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jacobg_name = fun_name + ':jacobg'

    jacobgh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, ya[n].
        utils._EngineFloat64ArrayType.c_type, # Input, yb[n].
        utils._EngineFloat64ArrayType.c_type, # Output, aj[n, n].
        utils._EngineFloat64ArrayType.c_type, # Output, bj[n, n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jacobgh(_jacobg, en_data, eps, ya, yb, aj, bj, n, _iuser, _ruser):
        """Helper for jacobg."""
        if jacobg is None:
            return
        try:
            _eps_jacobgh = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_jacobgh = utils._EngineIntScalarType.to_py(n.contents)
            _ya_jacobgh = utils._EngineFloat64ArrayType.from_pointer(
                ya,
                (_n_jacobgh,),
            )
            _yb_jacobgh = utils._EngineFloat64ArrayType.from_pointer(
                yb,
                (_n_jacobgh,),
            )
            _aj_jacobgh = utils._EngineFloat64ArrayType.from_pointer(
                aj,
                (_n_jacobgh, _n_jacobgh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            _bj_jacobgh = utils._EngineFloat64ArrayType.from_pointer(
                bj,
                (_n_jacobgh, _n_jacobgh),
                can_reference_contents=False,
                sorder=_m_sorder,
            )
            if data is not None:
                jacobg(
                    _eps_jacobgh,
                    _ya_jacobgh,
                    _yb_jacobgh,
                    _aj_jacobgh,
                    _bj_jacobgh,
                    _n_jacobgh,
                    data,
                )
            else:
                jacobg(
                    _eps_jacobgh,
                    _ya_jacobgh,
                    _yb_jacobgh,
                    _aj_jacobgh,
                    _bj_jacobgh,
                    _n_jacobgh,
                )
            utils._unlink_pointer(ya)
            utils._unlink_pointer(yb)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _aj_jacobgh, {'fun_name': cb_jacobg_name, 'returned': 'aj'}, aj,
            )
            utils._unlink_pointer(aj)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _bj_jacobgh, {'fun_name': cb_jacobg_name, 'returned': 'bj'}, bj,
            )
            utils._unlink_pointer(bj)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jaceps_name = fun_name + ':jaceps'

    jacepsh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jacepsh(_jaceps, en_data, x, eps, y, f, n, _iuser, _ruser):
        """Helper for jaceps."""
        if jaceps is None:
            return
        try:
            _x_jacepsh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _eps_jacepsh = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_jacepsh = utils._EngineIntScalarType.to_py(n.contents)
            _y_jacepsh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_jacepsh,),
            )
            _f_jacepsh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_jacepsh,),
                can_reference_contents=False,
            )
            if data is not None:
                jaceps(
                    _x_jacepsh,
                    _eps_jacepsh,
                    _y_jacepsh,
                    _f_jacepsh,
                    _n_jacepsh,
                    data,
                )
            else:
                jaceps(
                    _x_jacepsh,
                    _eps_jacepsh,
                    _y_jacepsh,
                    _f_jacepsh,
                    _n_jacepsh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_jacepsh, {'fun_name': cb_jaceps_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_jacgep_name = fun_name + ':jacgep'

    jacgeph = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, ya[n].
        utils._EngineFloat64ArrayType.c_type, # Input, yb[n].
        utils._EngineFloat64ArrayType.c_type, # Output, bcep[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_jacgeph(_jacgep, en_data, eps, ya, yb, bcep, n, _iuser, _ruser):
        """Helper for jacgep."""
        if jacgep is None:
            return
        try:
            _eps_jacgeph = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _n_jacgeph = utils._EngineIntScalarType.to_py(n.contents)
            _ya_jacgeph = utils._EngineFloat64ArrayType.from_pointer(
                ya,
                (_n_jacgeph,),
            )
            _yb_jacgeph = utils._EngineFloat64ArrayType.from_pointer(
                yb,
                (_n_jacgeph,),
            )
            _bcep_jacgeph = utils._EngineFloat64ArrayType.from_pointer(
                bcep,
                (_n_jacgeph,),
                can_reference_contents=False,
            )
            if data is not None:
                jacgep(
                    _eps_jacgeph,
                    _ya_jacgeph,
                    _yb_jacgeph,
                    _bcep_jacgeph,
                    _n_jacgeph,
                    data,
                )
            else:
                jacgep(
                    _eps_jacgeph,
                    _ya_jacgeph,
                    _yb_jacgeph,
                    _bcep_jacgeph,
                    _n_jacgeph,
                )
            utils._unlink_pointer(ya)
            utils._unlink_pointer(yb)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _bcep_jacgeph, {'fun_name': cb_jacgep_name, 'returned': 'bcep'}, bcep,
            )
            utils._unlink_pointer(bcep)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, mnp.
        utils._BYREF_INT, # Input/Output, np.
        utils._BYREF_INT, # Input, numbeg.
        utils._BYREF_INT, # Input, nummix.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, init.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[mnp].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[ldy, mnp].
        utils._BYREF_INT, # Input, ldy.
        utils._EngineFloat64ArrayType.c_type, # Output, abt[n].
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_INT, # Input, ijac.
        utils._UNUSED_FUNCTYPE,
        jacobfh,
        utils._UNUSED_FUNCTYPE,
        jacobgh,
        utils._BYREF_FLOAT64, # Input/Output, deleps.
        utils._UNUSED_FUNCTYPE,
        jacepsh,
        utils._UNUSED_FUNCTYPE,
        jacgeph,
        utils._BYREF_INT, # Input, itrace.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._BYREF_INT, # Input, liwork.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((y, 'y'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _mnp = utils._EngineIntScalarType(
        mnp, {'fun_name': fun_name, 'entity': 'mnp'},
    ).to_ctypes()
    _np = utils._EngineIntScalarType(
        np, {'fun_name': fun_name, 'entity': 'np'},
    ).to_ctypes()
    _numbeg = utils._EngineIntScalarType(
        numbeg, {'fun_name': fun_name, 'entity': 'numbeg'},
    ).to_ctypes()
    _nummix = utils._EngineIntScalarType(
        nummix, {'fun_name': fun_name, 'entity': 'nummix'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _init = utils._EngineIntScalarType(
        init, {'fun_name': fun_name, 'entity': 'init'},
    ).to_ctypes()
    _mnp_py = utils._EngineIntScalarType.to_py(_mnp)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mnp_py,),
    )
    _x = _x_cls.to_ctypes()
    _ldy_py = utils._EngineFloat64ArrayType.get_nd(y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldy = utils._EngineIntScalarType(
        _ldy_py, {'fun_name': fun_name, 'entity': '_ldy'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py, _mnp_py),
        mutual_sorder=_m_sorder,
    )
    _y = _y_cls.to_ctypes()
    _abt_cls = utils._EngineFloat64ArrayType(
        abt, {'fun_name': fun_name, 'entity': 'abt'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _abt = _abt_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _ijac = utils._EngineIntScalarType(
        ijac, {'fun_name': fun_name, 'entity': 'ijac'},
    ).to_ctypes()
    utils._check_callable(jacobf, {'fun_name': fun_name, 'entity': 'jacobf'}, allow_none=True)
    _jacobf = utils._UNUSED_CALLBACK
    _jacobfh = utils._cast_callback(
        jacobfh, cb_jacobfh,
        {'fun_name': fun_name, 'entity': 'jacobf'},
    )
    utils._check_callable(jacobg, {'fun_name': fun_name, 'entity': 'jacobg'}, allow_none=True)
    _jacobg = utils._UNUSED_CALLBACK
    _jacobgh = utils._cast_callback(
        jacobgh, cb_jacobgh,
        {'fun_name': fun_name, 'entity': 'jacobg'},
    )
    _deleps = utils._EngineFloat64ScalarType(
        deleps, {'fun_name': fun_name, 'entity': 'deleps'},
    ).to_ctypes()
    utils._check_callable(jaceps, {'fun_name': fun_name, 'entity': 'jaceps'}, allow_none=True)
    _jaceps = utils._UNUSED_CALLBACK
    _jacepsh = utils._cast_callback(
        jacepsh, cb_jacepsh,
        {'fun_name': fun_name, 'entity': 'jaceps'},
    )
    utils._check_callable(jacgep, {'fun_name': fun_name, 'entity': 'jacgep'}, allow_none=True)
    _jacgep = utils._UNUSED_CALLBACK
    _jacgeph = utils._cast_callback(
        jacgeph, cb_jacgeph,
        {'fun_name': fun_name, 'entity': 'jacgep'},
    )
    _itrace = utils._EngineIntScalarType(
        itrace, {'fun_name': fun_name, 'entity': 'itrace'},
    ).to_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _liwork = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(10)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _n,
        _mnp,
        _np,
        _numbeg,
        _nummix,
        _tol,
        _init,
        _x,
        _y,
        _ldy,
        _abt,
        _fcn,
        _fcnh,
        _g,
        _gh,
        _ijac,
        _jacobf,
        _jacobfh,
        _jacobg,
        _jacobgh,
        _deleps,
        _jaceps,
        _jacepsh,
        _jacgep,
        _jacgeph,
        _itrace,
        _work,
        _lwork,
        _iwork,
        _liwork,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry the mesh points are not in strictly ascending order.',
                    'For i = {:s}, mesh point i = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'but mesh point i+1 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numbeg = {:s}, nummix = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: numbeg+nummix <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nummix = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nummix >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numbeg = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: numbeg < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numbeg = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: numbeg >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[0] = {:s} and x[np-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x[0] < x[np-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mnp >= 32.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s} and mnp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: np <= mnp.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A finer mesh is required for the accuracy requested;',
                    'that is, mnp = {:s} is not large enough.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge.',
                    'This could be due to there being too few points in the initial mesh or to',
                    'the initial approximate solution being too inaccurate.',
                    'If this latter reason is suspected or you cannot make changes to prevent',
                    'this error, you should use the function with a continuation facility',
                    'instead.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Newton iteration has reached round-off level.',
                    'If desired accuracy has not been reached, tol is too',
                    'small for this problem and this machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobian for the boundary conditions is singular.',
                    'This may occur due to faulty coding of the Jacobian or, in some',
                    'circumstances, to a zero initial choice of approximate solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is no dependence on the continuation parameter when continuation',
                    'is being used. This can be due to faulty coding of derivatives with',
                    'respect to the continuation parameter or to a zero initial choice of',
                    'approximate solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The continuation step is required to be less than machine precision for',
                    'continuation to proceed. It is likely that either the problem has no',
                    'solution for some value of the continuation parameter near the current',
                    'value or that the problem is so difficult that even with continuation',
                    'it is unlikely to be solved using this function. In the latter case',
                    'using more mesh points initially may help.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in a call to the internal integrator.',
                    'The error code internally was {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A continuation error occurred, but continuation is not being used.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _np_py = utils._EngineIntScalarType.to_py(_np)
        _x_cls.refresh_py(_x)
        _y_cls.refresh_py(_y)
        _abt_cls.refresh_py(_abt)
        _deleps_py = utils._EngineFloat64ScalarType.to_py(_deleps)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['np', 'deleps'],
        )(
            _np_py,
            _deleps_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_shoot_genpar_algeq(p, m, n, n1, pe, pf, e, dp, npoint, swp, icount, brkpts, bc, fcn, eqn, constr, ymax, monlev, monit, prsol, w, comm, data=None, io_manager=None):
    u"""
    Ordinary differential equations, boundary value problem, shooting
    and matching technique, subject to extra algebraic equations,
    general parameters to be determined.

    For full information please refer to the NAG Library document for
    d02sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02saf.html

    Parameters
    ----------
    p : float, shape (`m`), modified in place
        On entry: `p`[i] must be set to an estimate of the ith argument,
        p_i, for i = 1,2,...,m.

        On exit: the corrected value for the ith argument, unless an
        error has occurred, when it contains the last calculated value
        of the argument.

    m : int
        m, the number of arguments.

    n : int
        n, the total number of differential equations.

    n1 : int
        n_1, the number of differential equations active in the matching
        process. The active equations must be placed last in the
        numbering in `fcn` and `bc`. The first `n`-`n1` equations are
        used as the driving equations.

    pe : float, shape (`m`)
        `pe`[i-1], for i = 1,2,...,m, must be set to a positive value
        for use in the convergence test in the ith argument p_i. See the
        description of `pf` for further details.

    pf : float, shape (`m`), modified in place
        On entry: `pf`[i-1], for i = 1,2,...,m, should be set to a
        'floor' value in the convergence test on the ith argument p_i.
        If `pf`[i-1] <= 0.0 on entry then it is set to the small
        positive value sqrt(epsilon) (where epsilon may in most cases be
        considered to be machine precision); otherwise it is used
        unchanged.

        The Newton iteration is presumed to have converged if a full
        Newton step is taken (`istate` = 1 in the specification of
        `monit`), the singular values of the Jacobian are not being
        significantly perturbed (also see `monit`) and if the Newton
        correction C_i satisfies

            |C_i| <= `pe`[i-1]*max(|p_i|, `pf`[i-1]), i = 1,2,...,m,

        where p_i is the current value of the ith argument.

        The values `pf`[i-1] are also used in determining the Newton
        iterates as discussed in [Description], see equation (6).

        On exit: the values actually used.

    e : None or float, shape (`n`)
        Note: if this argument is None then a default value will be
        used, determined as follows: 10^-5.

        Values for use in controlling the local error in the integration
        of the differential equations. If err_i is an estimate of the
        local error in y_i, for i = 1,2,...,n,

            |err_i| <= `e`[i-1]*max{sqrt(epsilon),|y_i|},

        where epsilon may in most cases be considered to be machine
        precision.

    dp : float, shape (`m`), modified in place
        On entry: a value to be used in perturbing the argument p_i in
        the numerical differentiation to estimate the Jacobian used in
        Newton's method. If `dp`[i] = 0.0 on entry, an estimate is made
        internally by setting

            `dp`[i] = sqrt(epsilon)*max(`pf`[i], |p_i|),

        where p_i is the initial value of the argument supplied by you
        and epsilon may in most cases be considered to be machine
        precision. The estimate of the Jacobian, J, is made using
        forward differences, that is for each i, for i = 1,2,...,m, p_i
        is perturbed to p_i+`dp`[i] and the ith column of J is estimated
        as

            (r(p_i+`dp`[i])-r(p_i))/`dp`[i]

        where the other components of p are unchanged (see (3) for the
        notation used). If this fails to produce a Jacobian with
        significant columns, backward differences are tried by
        perturbing p_i to p_i-`dp`[i] and if this also fails then
        central differences are used with p_i perturbed to
        p_i+10.0*`dp`[i]. If this also fails then the calculation of the
        Jacobian is abandoned. If the Jacobian has not previously been
        calculated then an error exit is taken. If an earlier estimate
        of the Jacobian is available then the current argument set, p_i,
        for i = 1,2,...,m, is abandoned in favour of the last argument
        set from which useful progress was made and the singular values
        of the Jacobian used at the point are modified before proceeding
        with the Newton iteration. You are recommended to use the
        default value `dp`[i] = 0.0 unless you have prior knowledge of a
        better choice. If any of the perturbations described are likely
        to lead to an unfortunate set of argument values then you should
        use `constr` to prevent such perturbations (all changes of
        arguments are checked by a call to `constr`).

        On exit: the values actually used.

    npoint : int
        2 plus the number of break-points in the range of definition of
        the system of differential equations (1).

    swp : float, shape (`npoint`, 6), modified in place
        On entry: `swp`[i-1,0] must contain an estimate for an initial
        step size for integration across the ith sub-interval
        [`x`[i-1],`x`[i]], for i = 1,2,...,`npoint`-1, (see `brkpts`).
        `swp`[i-1,0] should have the same sign as `x`[i]-`x`[i-1] if it
        is nonzero. If `swp`[i-1,0] = 0.0, on entry, a default value for
        the initial step size is calculated internally. This is the
        recommended mode of entry.

        `swp`[i-1,2] must contain a lower bound for the modulus of the
        step size on the ith sub-interval [`x`[ i -1],`x`[ i + 1 -1]],
        for i = 1,2,...,`npoint`-1.

        If `swp`[i-1,2] = 0.0 on entry, a very small default value is
        used.

        By setting `swp`[i-1,2] > 0.0 but smaller than the expected step
        sizes (assuming you have some insight into the likely step
        sizes) expensive integrations with arguments p far from the
        solution can be avoided.

        `swp`[i-1,1] must contain an upper bound on the modulus of the
        step size to be used in the integration on [`x`[ i -1],`x`[ i +
        1 -1]], for i = 1,2,...,`npoint`-1.

        If `swp`[i-1,1] = 0.0 on entry no bound is assumed.

        This is the recommended mode of entry unless the solution is
        expected to have important features which might be 'missed' in
        the integration if the step size were permitted to be chosen
        freely.

        On exit: `swp`[i-1,0] contains the initial step size used on the
        last integration on [`x`[ i -1],`x`[ i + 1 -1]], for i =
        1,2,...,`npoint`-1, (excluding integrations during the
        calculation of the Jacobian).

        `swp`[i-1,1], for i = 1,2,...,`npoint`-1, is usually unchanged.

        If the maximum step size `swp`[i-1,1] is so small or the length
        of the range [`x`[ i -1],`x`[ i + 1 -1]] is so short that on the
        last integration the step size was not controlled in the main by
        the size of the error tolerances `e`[i-1] but by these other
        factors, `swp`[`npoint`-1,1] is set to the floating-point value
        of i if the problem last occurred in [`x`[ i -1],`x`[ i + 1
        -1]].

        Any results obtained when this value is returned as nonzero
        should be viewed with caution.

        `swp`[i-1,2], for i = 1,2,...,`npoint`-1, are unchanged.

        If an error exit with `errno` = 4, 5 or 6 (see [Errors]) occurs
        on the integration made from `x`[i-1] to `x`[i] the
        floating-point value of i is returned in `swp`[`npoint`-1,0].

        The actual point x in [`x`[ i -1],`x`[ i + 1 -1]] where the
        error occurred is returned in `swp`[0,4] (see also the
        specification of `w`).

        The floating-point value of `npoint` is returned in
        `swp`[`npoint`-1,0] if the error exit is caused by a call to
        `bc`.

        If an error exit occurs when estimating the Jacobian matrix
        (`errno` = 7, 8, 9, 10, 11 or 12, see [Errors]) and if argument
        p_i was the cause of the failure then on exit
        `swp`[`npoint`-1,0] contains the floating-point value of i.

        `swp`[i-1,3] contains the point `x`[i-1], for i =
        1,2,...,`npoint`, used at the solution p or at the final values
        of p if an error occurred.

        `swp` is also partly used as workspace.

    icount : int
        An upper bound on the number of Newton iterations. If `icount` =
        0 on entry, no check on the number of iterations is made (this
        is the recommended mode of entry).

    brkpts : callable brkpts(x, npoint, p, m, data=None)
        `brkpts` must specify the break-points x_i, for i =
        1,2,...,`npoint`, which may depend on the arguments p_j, for j =
        1,2,...,m.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`npoint`), to be modified in place
            On exit: the ith break-point, for i = 1,2,...,`npoint`. The
            sequence (`x`[i]) must be strictly monotonic, that is either

                a = `x`[0] < `x`[1] < ... < `x`[`npoint`-1] = b

            or

                a = `x`[0] > `x`[1] > ... > `x`[`npoint`-1] = b.

        npoint : int
            2 plus the number of break-points in (a,b).

        p : float, ndarray, shape (`m`)
            The current estimate of the ith argument, for i = 1,2,...,m.

        m : int
            m, the number of arguments.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    bc : callable bc(g1, g2, p, m, n, data=None)
        `bc` must place in `g1` and `g2` the boundary conditions at a
        and b respectively.

        Parameters
        ~~~~~~~~~~
        g1 : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of y_i(a), for i = 1,2,...,n, (where this
            may be a known value or a function of the parameters p_j,
            for j = 1,2,...,m).

        g2 : float, ndarray, shape (`n`), to be modified in place
            On exit: the value of y_i(b), for i = 1,2,...,n, (where
            these may be known values or functions of the parameters
            p_j, for j = 1,2,...,m). If n > n_1, so that there are some
            driving equations, the first n-n_1 values of `g2` need not
            be set since they are never used.

        p : float, ndarray, shape (`m`)
            An estimate of the ith argument, p_i, for i = 1,2,...,m.

        m : int
            m, the number of arguments.

        n : int
            n, the number of differential equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcn : callable fcn(x, y, f, n, p, m, i, data=None)
        `fcn` must evaluate the functions f_i (i.e., the derivatives
        y'_i), for i = 1,2,...,n.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the value of the argument.

        y : float, ndarray, shape (`n`)
            y_i, for i = 1,2,...,n, the value of the argument.

        f : float, ndarray, shape (`n`), to be modified in place
            On exit: the derivative of y_i, for i = 1,2,...,n, evaluated
            at x. `f`[i] may depend upon the parameters p_j, for j =
            1,2,...,m. If there are any driving equations (see
            [Description]) then these must be numbered first in the
            ordering of the components of `f`.

        n : int
            n, the number of equations.

        p : float, ndarray, shape (`m`)
            The current estimate of the ith argument p_i, for i =
            1,2,...,m.

        m : int
            m, the number of arguments.

        i : int
            Specifies the sub-interval [x_i,x_{i+1}] on which the
            derivatives are to be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    eqn : None or callable eqn(e, q, p, m, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `eqn` is used to describe the additional algebraic equations to
        be solved in the determination of the parameters, p_i, for i =
        1,2,...,m.

        If there are no additional algebraic equations (i.e., m = n_1)
        then `eqn` is never called and may be None.

        Parameters
        ~~~~~~~~~~
        e : float, ndarray, shape (`q`), to be modified in place
            On exit: the vector of residuals, r_2(p), that is the amount
            by which the current estimates of the arguments fail to
            satisfy the algebraic equations.

        q : int
            The number of algebraic equations, m-n_1.

        p : float, ndarray, shape (`m`)
            The current estimate of the ith argument p_i, for i =
            1,2,...,m.

        m : int
            m, the number of arguments.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    constr : None or callable retval = constr(p, m, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `constr` is used to prevent the pseudo-Newton iteration running
        into difficulty. `constr` should return the value True if the
        constraints are satisfied by the parameters p_1,p_2,...,p_m.

        Otherwise `constr` should return the value False. If None is
        supplied the function will act as if `constr` returns the value
        True at all times, and in the first instance this is recommended
        as the actual argument.

        Parameters
        ~~~~~~~~~~
        p : float, ndarray, shape (`m`)
            An estimate of the ith argument, p_i, for i = 1,2,...,m.

        m : int
            m, the number of arguments.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the constraints are satisfied by the
            parameters `p`.

    ymax : float
        A non-negative value which is used as a bound on all values
        ||y(x)||_infinity where y(x) is the solution at any point x
        between `x`[0] and `x`[`npoint`-1] for the current arguments
        p_1,p_2,...,p_m. If this bound is exceeded the integration is
        terminated and the current arguments are rejected. Such a
        rejection will result in an error exit if it prevents the
        initial residual or Jacobian, or the final solution, being
        calculated. If `ymax` = 0 on entry, no bound on the solution y
        is used; that is the integrations proceed without any checking
        on the size of ||y||_infinity.

    monlev : int
        Setting `monlev` = 0 disables monitoring of the pseudo-Newton
        iteration. Setting `monlev` = 1 enables this monitoring.

    monit : None or callable monit(istate, iflag, ifail1, p, m, f,
        pnorm, pnorm1, eps, d, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monit` enables you to monitor the values of various quantities
        during the calculation. If `monlev` = 0 on entry no monitoring
        is performed and this argument is never referenced. Otherwise,
        if `monlev` = 1 on entry, you may supply None, to use
        NAG-defined default monitoring, or you may supply your own
        custom monitoring function. The function is called by
        ``bvp_shoot_genpar_algeq`` after every calculation of the norm
        ||`d`^(-1)J~^+r||_2 which determines the strategy of the Newton
        method, every time there is an internal error exit leading to a
        change of strategy, and before an error exit when calculating
        the initial Jacobian.

        Parameters
        ~~~~~~~~~~
        istate : int
            The state of the Newton iteration.

            `istate` = 0

              The calculation of the residual, Jacobian and
              ||`d`^(-1)J~^+r||_2 are taking place.

            `istate` = 1 to 5

              During the Newton iteration a factor of 2^(-`istate`+1) of
              the Newton step is being used to try to reduce the norm.

            `istate` = 6

              The current Newton step has been rejected and the Jacobian
              is being re-calculated.

            `istate` = -6 to -1

              An internal error exit has caused the rejection of the
              current set of argument values, p. -`istate` is the value
              which `istate` would have taken if the error had not
              occurred.

            `istate` = -7

              An internal error exit has occurred when calculating the
              initial Jacobian.

        iflag : int
            Whether or not the Jacobian being used has been calculated
            at the beginning of the current iteration. If the Jacobian
            has been updated then `iflag` = 1; otherwise `iflag` = 2.
            The Jacobian is only calculated when convergence to the
            current argument values has been slow.

        ifail1 : int
            If -6 <= `istate` <= -1, `ifail1` specifies the errno error
            number that would be produced were control returned to you.
            `ifail1` is unspecified for values of `istate` outside this
            range.

        p : float, ndarray, shape (`m`)
            The current estimate of the ith argument p_i, for i =
            1,2,...,m.

        m : int
            m, the number of arguments.

        f : float, ndarray, shape (`m`)
            r, the residual corresponding to the current argument
            values, provided 1 <= `istate` <= 5 or `istate` = -7. `f` is
            unspecified for other values of `istate`.

        pnorm : float
            A quantity against which all reductions in norm are
            currently measured.

        pnorm1 : float
            p, the norm of the current arguments. It is set for 1 <=
            `istate` <= 5 and is undefined for other values of `istate`.

        eps : float
            Gives some indication of the convergence rate. It is the
            current singular value modification factor. It is zero
            initially and whenever convergence is proceeding steadily.
            `eps` is epsilon^(3/8) or greater (where epsilon may in most
            cases be considered machine precision) when the singular
            values of J are approximately zero or when convergence is
            not being achieved. The larger the value of `eps` the worse
            the convergence rate. When `eps` becomes too large the
            Newton iteration is terminated.

        d : float, ndarray, shape (`m`)
            J, the singular values of the current modified Jacobian
            matrix. If `d`[m-1] is small relative to `d`[0] for a number
            of Jacobians corresponding to different argument values then
            the computed results should be viewed with suspicion. It
            could be that the matching equations do not depend
            significantly on some argument (which could be due to a
            programming error in `fcn`, `bc`, `brkpts` or `eqn`).
            Alternatively, the system of differential equations may be
            very ill-conditioned when viewed as an initial value
            problem, in which case ``bvp_shoot_genpar_algeq`` is
            unsuitable. This may also be indicated by some singular
            values being very large. These values of `d`[i-1], for i =
            1,2,...,m, should not be changed.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    prsol : None or callable z = prsol(z, y, n, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `prsol` can be used to obtain values of the solution y at a
        selected point z by integration across the final range
        [`x`[0],`x`[ `npoint` -1]].

        If no output is required None can be used as the actual
        argument.

        Parameters
        ~~~~~~~~~~
        z : float
            Contains x_1 on the first call. On subsequent calls `z`
            contains its previous output value.

        y : float, ndarray, shape (`n`)
            The solution value y_i, for i = 1,2,...,n, at z.

        n : int
            n, the total number of differential equations.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        z : float
            The next point at which output is required. The new point
            must be nearer `x`[`npoint`-1] than the old.

            If `z` is set to a point outside [`x`[0],`x`[ `npoint` -1]]
            the process stops and control returns from
            ``bvp_shoot_genpar_algeq`` to the (sub)program from which
            ``bvp_shoot_genpar_algeq`` is called.

            Otherwise the next call to `prsol` is made by
            ``bvp_shoot_genpar_algeq`` at the point `z`, with solution
            values y_1,y_2,...,y_n at `z` contained in `y`.

            If `z` is set to `x`[`npoint`-1] exactly, the final call to
            `prsol` is made with y_1,y_2,...,y_n as values of the
            solution at `x`[`npoint`-1] produced by the integration.

            In general the solution values obtained at `x`[`npoint`-1]
            from `prsol` will differ from the values obtained at this
            point by a call to `bc`.

            The difference between the two solutions is the residual r.

            You are reminded that the points
            `x`[0],`x`[1],...,`x`[`npoint`-1] are available in the
            locations `swp`[0,3],`swp`[1,3],...,`swp`[`npoint`-1,3] at
            all times.

    w : float, shape (max(`n`, `m`), 3*`m`+12+max(11, `m`)), modified in
    place
        On exit: in the case of an error exit of the type where the
        point of failure is returned in `swp`[0,4], the solution at this
        point of failure is returned in `w`[i-1,0], for i = 1,2,...,n.

        Otherwise `w` is used for workspace.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `monlev` = *<value>*.

            Constraint: `monlev` = 0 or 1.

        (`errno` 1)
            On entry, an element of the parameter convergence control
            array is zero or negative.

        (`errno` 1)
            On entry an element of the local error control array is zero
            or negative.

        (`errno` 1)
            On entry, `ymax` = *<value>*.

            Constraint: `ymax` >= 0.0.

        (`errno` 1)
            On entry, `npoint` = *<value>*.

            Constraint: `npoint` >= 2.

        (`errno` 1)
            On entry, `icount` = *<value>*.

            Constraint: `icount` >= 0.

        (`errno` 1)
            On entry, `m` = *<value>* and `n1` = *<value>*.

            Constraint: `m` >= `n1`.

        (`errno` 1)
            On entry, `n` = *<value>* and `n1` = *<value>*.

            Constraint: `n` >= `n1`.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            The constraints have been violated by the initial
            parameters.

        (`errno` 3)
            The sequence of break-points is not strictly monotonic in
            their initial specification.

        (`errno` 4)
            In the integration from first to last break-point with
            initial or final parameters, the step size was reduced too
            far for the integration to proceed. Consider reversing the
            order of break-points or relaxing the local error control.
            If this error exit still results, either this function is
            not a suitable method for solving the problem, or the
            initial choice of parameters is very poor.

        (`errno` 5)
            In the integration from first to last break-point with
            initial or final parameters, a suitable initial step could
            not be found on one of the intervals. Consider reversing the
            order of break-points or relaxing the local error control.
            If this error exit still results, either this function is
            not suitable for solving the problem, or the initial choice
            of parameters is very poor.

        (`errno` 6)
            During integration the solution exceeded `ymax` in
            magnitude, where, on entry, `ymax` = *<value>*.

        (`errno` 7)
            On calculating the initial approximation to the Jacobian,
            the constraints were violated.

        (`errno` 8)
            On perturbing the parameters when calculating the initial
            approximation to the Jacobian, the monotonicity condition on
            break-points is violated.

        (`errno` 9)
            An initial step-length could be found for integration to
            proceed with the current parameters.

        (`errno` 10)
            The step-length required to calculate the Jacobian to
            sufficient accuracy is too small.

        (`errno` 11)
            On calculating the initial approximation to the Jacobian,
            the solution to the system of differential equations
            exceeded `ymax` in magnitude, where, on entry, `ymax` =
            *<value>*.

        (`errno` 12)
            The Jacobian has an insignificant column. Make sure that the
            solution vector depends on all the parameters.

        (`errno` 13)
            An internal singular value decomposition has failed. This
            error can be avoided by changing the initial parameter
            estimates.

        (`errno` 14)
            The Newton iteration has failed to converge. This can
            indicate a poor initial choice of parameters or a very
            difficult problem. Consider varying elements of the
            parameter convergence control if the residuals are small;
            otherwise vary initial parameter estimates.

        (`errno` 15)
            The number of iterations permitted by `icount` has been
            exceeded where this was set to a positive value on entry.
            `icount` = *<value>*.

        (`errno` 16)
            Internal error in calculating Jacobian. Please contact NAG.

        (`errno` 17)
            Internal error in calculating residual. Please contact NAG.

        (`errno` 18)
            Internal error in integration. Please contact NAG.

        (`errno` 19)
            Internal error in Newton method. Please contact NAG.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_shoot_genpar_algeq'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_range_name = fun_name + ':range'

    rangeh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Output, x[npoint].
        utils._BYREF_INT, # Input, npoint.
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_rangeh(_range, en_data, x, npoint, p, m, _iuser, _ruser):
        """Helper for range."""
        try:
            _npoint_rangeh = utils._EngineIntScalarType.to_py(npoint.contents)
            _x_rangeh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_npoint_rangeh,),
                can_reference_contents=False,
            )
            _m_rangeh = utils._EngineIntScalarType.to_py(m.contents)
            _p_rangeh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_rangeh,),
            )
            if data is not None:
                brkpts(
                    _x_rangeh,
                    _npoint_rangeh,
                    _p_rangeh,
                    _m_rangeh,
                    data,
                )
            else:
                brkpts(
                    _x_rangeh,
                    _npoint_rangeh,
                    _p_rangeh,
                    _m_rangeh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _x_rangeh, {'fun_name': cb_range_name, 'returned': 'x'}, x,
            )
            utils._unlink_pointer(x)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bc_name = fun_name + ':bc'

    bch = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Output, g1[n].
        utils._EngineFloat64ArrayType.c_type, # Output, g2[n].
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bch(_bc, en_data, g1, g2, p, m, n, _iuser, _ruser):
        """Helper for bc."""
        try:
            _n_bch = utils._EngineIntScalarType.to_py(n.contents)
            _g1_bch = utils._EngineFloat64ArrayType.from_pointer(
                g1,
                (_n_bch,),
                can_reference_contents=False,
            )
            _g2_bch = utils._EngineFloat64ArrayType.from_pointer(
                g2,
                (_n_bch,),
                can_reference_contents=False,
            )
            _m_bch = utils._EngineIntScalarType.to_py(m.contents)
            _p_bch = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_bch,),
            )
            if data is not None:
                bc(
                    _g1_bch,
                    _g2_bch,
                    _p_bch,
                    _m_bch,
                    _n_bch,
                    data,
                )
            else:
                bc(
                    _g1_bch,
                    _g2_bch,
                    _p_bch,
                    _m_bch,
                    _n_bch,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g1_bch, {'fun_name': cb_bc_name, 'returned': 'g1'}, g1,
            )
            utils._unlink_pointer(g1)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _g2_bch, {'fun_name': cb_bc_name, 'returned': 'g2'}, g2,
            )
            utils._unlink_pointer(g2)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, i.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fcnh(_fcn, en_data, x, y, f, n, p, m, i, _iuser, _ruser):
        """Helper for fcn."""
        try:
            _x_fcnh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _y_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_fcnh,),
            )
            _f_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            _m_fcnh = utils._EngineIntScalarType.to_py(m.contents)
            _p_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_fcnh,),
            )
            _i_fcnh = utils._EngineIntScalarType.to_py(i.contents)
            if data is not None:
                fcn(
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _n_fcnh,
                    _p_fcnh,
                    _m_fcnh,
                    _i_fcnh,
                    data,
                )
            else:
                fcn(
                    _x_fcnh,
                    _y_fcnh,
                    _f_fcnh,
                    _n_fcnh,
                    _p_fcnh,
                    _m_fcnh,
                    _i_fcnh,
                )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_fcnh, {'fun_name': cb_fcn_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_eqn_name = fun_name + ':eqn'

    eqnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Output, e[q].
        utils._BYREF_INT, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_eqnh(_eqn, en_data, e, q, p, m, _iuser, _ruser):
        """Helper for eqn."""
        if eqn is None:
            return
        try:
            _q_eqnh = utils._EngineIntScalarType.to_py(q.contents)
            _e_eqnh = utils._EngineFloat64ArrayType.from_pointer(
                e,
                (_q_eqnh,),
                can_reference_contents=False,
            )
            _m_eqnh = utils._EngineIntScalarType.to_py(m.contents)
            _p_eqnh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_eqnh,),
            )
            if data is not None:
                eqn(
                    _e_eqnh,
                    _q_eqnh,
                    _p_eqnh,
                    _m_eqnh,
                    data,
                )
            else:
                eqn(
                    _e_eqnh,
                    _q_eqnh,
                    _p_eqnh,
                    _m_eqnh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _e_eqnh, {'fun_name': cb_eqn_name, 'returned': 'e'}, e,
            )
            utils._unlink_pointer(e)
            utils._unlink_pointer(p)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_constr_name = fun_name + ':constr'

    constrh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Output, iretval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    if constr is None:
        constr_auxil_ctypes = utils._EngineState._get_symbol('d02hbyt')
        constr_auxil_ctypes.restype = None
        constr_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._EngineFloat64ArrayType.c_type, # Input, p[m].
            utils._BYREF_INT, # Input, m.
            utils._BYREF_INT, # Output, iretval.
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        )

    def cb_constrh(_constr, _constrsub, en_data, p, m, iretval, iuser, ruser):
        """Helper for constr."""
        try:
            if constr is None:
                constr_auxil_ctypes(
                    en_data,
                    p,
                    m,
                    iretval,
                    iuser,
                    ruser,
                )
                return
            _m_constrh = utils._EngineIntScalarType.to_py(m.contents)
            _p_constrh = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_constrh,),
            )
            if data is not None:
                _retval_constrh = constr(
                    _p_constrh,
                    _m_constrh,
                    data,
                )
            else:
                _retval_constrh = constr(
                    _p_constrh,
                    _m_constrh,
                )
            utils._unlink_pointer(p)
            _retval_constrh_ct = utils._EngineLogicalScalarType(
                _retval_constrh, {'fun_name': cb_constr_name, 'returned': 'retval'},
            ).to_ctypes()
            _retval_constrh_py = utils._EngineLogicalScalarType.to_py(_retval_constrh_ct)
            if _retval_constrh_py is False:
                _iretval_constrh_py = 0
                utils._EngineIntScalarType(
                    _iretval_constrh_py, {'fun_name': cb_constr_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            elif _retval_constrh_py is True:
                _iretval_constrh_py = 1
                utils._EngineIntScalarType(
                    _iretval_constrh_py, {'fun_name': cb_constr_name, 'returned': '_iretval'},
                ).refresh_ctypes_buffer(iretval)
            else:
                raise utils.NagValueError(
                    {'fun_name': cb_constr_name, 'returned': 'retval'},
                    'Unexpected value for retval.',
                )
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, istate.
        utils._BYREF_INT, # Input, iflag.
        utils._BYREF_INT, # Input, ifail1.
        utils._EngineFloat64ArrayType.c_type, # Input, p[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, f[m].
        utils._BYREF_FLOAT64, # Input, pnorm.
        utils._BYREF_FLOAT64, # Input, pnorm1.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Input, d[m].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[15].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[15].
    )

    if monit is None:
        monit_auxil_ctypes = utils._EngineState._get_symbol('d02hbxt')
        monit_auxil_ctypes.restype = None
        monit_auxil_ctypes.argtypes = (
            utils._BYREF_ENGINEDATA,
            utils._PRINT_REC_FUNCTYPE,
            utils._PRINT_RECH_FUNCTYPE,
            utils._BYREF_INT, # Input, istate.
            utils._BYREF_INT, # Input, iflag.
            utils._BYREF_INT, # Input, ifail1.
            utils._EngineFloat64ArrayType.c_type, # Input, p[m].
            utils._BYREF_INT, # Input, m.
            utils._EngineFloat64ArrayType.c_type, # Input, f[m].
            utils._BYREF_FLOAT64, # Input, pnorm.
            utils._BYREF_FLOAT64, # Input, pnorm1.
            utils._BYREF_FLOAT64, # Input, eps.
            utils._EngineFloat64ArrayType.c_type, # Input, d[m].
            utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
            utils._EngineIntArrayType.c_type, # Input/Output, iwsav[15].
            utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[15].
        )

    def cb_monith(_monit, en_data, print_rec, print_rech, istate, iflag, ifail1, p, m, f, pnorm, pnorm1, eps, d, iuser, ruser, iwsav, rwsav):
        """Helper for monit."""
        try:
            if monit is None:
                monit_auxil_ctypes(
                    en_data,
                    print_rec,
                    print_rech,
                    istate,
                    iflag,
                    ifail1,
                    p,
                    m,
                    f,
                    pnorm,
                    pnorm1,
                    eps,
                    d,
                    iuser,
                    ruser,
                    iwsav,
                    rwsav,
                )
                return
            _istate_monith = utils._EngineIntScalarType.to_py(istate.contents)
            _iflag_monith = utils._EngineIntScalarType.to_py(iflag.contents)
            _ifail1_monith = utils._EngineIntScalarType.to_py(ifail1.contents)
            _m_monith = utils._EngineIntScalarType.to_py(m.contents)
            _p_monith = utils._EngineFloat64ArrayType.from_pointer(
                p,
                (_m_monith,),
            )
            _f_monith = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_m_monith,),
            )
            _pnorm_monith = utils._EngineFloat64ScalarType.to_py(pnorm.contents)
            _pnorm1_monith = utils._EngineFloat64ScalarType.to_py(pnorm1.contents)
            _eps_monith = utils._EngineFloat64ScalarType.to_py(eps.contents)
            _d_monith = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_m_monith,),
            )
            if data is not None:
                monit(
                    _istate_monith,
                    _iflag_monith,
                    _ifail1_monith,
                    _p_monith,
                    _m_monith,
                    _f_monith,
                    _pnorm_monith,
                    _pnorm1_monith,
                    _eps_monith,
                    _d_monith,
                    data,
                )
            else:
                monit(
                    _istate_monith,
                    _iflag_monith,
                    _ifail1_monith,
                    _p_monith,
                    _m_monith,
                    _f_monith,
                    _pnorm_monith,
                    _pnorm1_monith,
                    _eps_monith,
                    _d_monith,
                )
            utils._unlink_pointer(p)
            utils._unlink_pointer(f)
            utils._unlink_pointer(d)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_prsol_name = fun_name + ':prsol'

    prsolh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, z.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_prsolh(_prsol, en_data, z, y, n, _iuser, _ruser):
        """Helper for prsol."""
        if prsol is None:
            return
        try:
            _z_prsolh = utils._EngineFloat64ScalarType.to_py(z.contents)
            _n_prsolh = utils._EngineIntScalarType.to_py(n.contents)
            _y_prsolh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_n_prsolh,),
            )
            if data is not None:
                _z_prsolh = prsol(
                    _z_prsolh,
                    _y_prsolh,
                    _n_prsolh,
                    data,
                )
            else:
                _z_prsolh = prsol(
                    _z_prsolh,
                    _y_prsolh,
                    _n_prsolh,
                )
            utils._EngineFloat64ScalarType(
                _z_prsolh, {'fun_name': cb_prsol_name, 'returned': 'z'},
            ).refresh_ctypes_buffer(z)
            utils._unlink_pointer(y)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02saft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._EngineFloat64ArrayType.c_type, # Input/Output, p[m].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Input, pe[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, pf[m].
        utils._EngineFloat64ArrayType.c_type, # Input, e[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dp[m].
        utils._BYREF_INT, # Input, npoint.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, swp[ldswp, 6].
        utils._BYREF_INT, # Input, ldswp.
        utils._BYREF_INT, # Input, icount.
        utils._UNUSED_FUNCTYPE,
        rangeh,
        utils._UNUSED_FUNCTYPE,
        bch,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._UNUSED_FUNCTYPE,
        eqnh,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        constrh,
        utils._BYREF_FLOAT64, # Input, ymax.
        utils._BYREF_INT, # Input, monlev.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._UNUSED_FUNCTYPE,
        prsolh,
        utils._EngineFloat64ArrayType.c_type, # Output, w[ldw, sdw].
        utils._BYREF_INT, # Input, ldw.
        utils._BYREF_INT, # Input, sdw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineLogicalArrayType.c_type, # Input/Output, lwsav[5].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[15].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[15].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((swp, 'swp'), (w, 'w')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_m_py,),
    )
    _p = _p_cls.to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _pe = utils._EngineFloat64ArrayType(
        pe, {'fun_name': fun_name, 'entity': 'pe'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _pf_cls = utils._EngineFloat64ArrayType(
        pf, {'fun_name': fun_name, 'entity': 'pf'},
        exp_shape=(_m_py,),
    )
    _pf = _pf_cls.to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if e is None:
        _e = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
        _e[:] = [10**-5]*_n_py
    else:
        _e = utils._EngineFloat64ArrayType(
            e, {'fun_name': fun_name, 'entity': 'e'},
            exp_shape=(_n_py,),
            allow_none=True,
        ).to_ctypes()
    _dp_cls = utils._EngineFloat64ArrayType(
        dp, {'fun_name': fun_name, 'entity': 'dp'},
        exp_shape=(_m_py,),
    )
    _dp = _dp_cls.to_ctypes()
    _npoint = utils._EngineIntScalarType(
        npoint, {'fun_name': fun_name, 'entity': 'npoint'},
    ).to_ctypes()
    _ldswp_py = utils._EngineFloat64ArrayType.get_nd(swp, {'fun_name': fun_name, 'entity': 'swp'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldswp = utils._EngineIntScalarType(
        _ldswp_py, {'fun_name': fun_name, 'entity': '_ldswp'},
    ).to_ctypes()
    _npoint_py = utils._EngineIntScalarType.to_py(_npoint)
    _swp_cls = utils._EngineFloat64ArrayType(
        swp, {'fun_name': fun_name, 'entity': 'swp'},
        exp_shape=(_npoint_py, 6),
        mutual_sorder=_m_sorder,
    )
    _swp = _swp_cls.to_ctypes()
    _icount = utils._EngineIntScalarType(
        icount, {'fun_name': fun_name, 'entity': 'icount'},
    ).to_ctypes()
    utils._check_callable(brkpts, {'fun_name': fun_name, 'entity': 'brkpts'})
    _brkpts = utils._UNUSED_CALLBACK
    _rangeh = utils._cast_callback(
        rangeh, cb_rangeh,
        {'fun_name': fun_name, 'entity': 'range'},
    )
    utils._check_callable(bc, {'fun_name': fun_name, 'entity': 'bc'})
    _bc = utils._UNUSED_CALLBACK
    _bch = utils._cast_callback(
        bch, cb_bch,
        {'fun_name': fun_name, 'entity': 'bc'},
    )
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    utils._check_callable(eqn, {'fun_name': fun_name, 'entity': 'eqn'}, allow_none=True)
    _eqn = utils._UNUSED_CALLBACK
    _eqnh = utils._cast_callback(
        eqnh, cb_eqnh,
        {'fun_name': fun_name, 'entity': 'eqn'},
    )
    utils._check_callable(constr, {'fun_name': fun_name, 'entity': 'constr'}, allow_none=True)
    _constr = utils._UNUSED_CALLBACK
    _constrsub = utils._UNUSED_CALLBACK
    _constrh = utils._cast_callback(
        constrh, cb_constrh,
        {'fun_name': fun_name, 'entity': 'constr'},
    )
    _ymax = utils._EngineFloat64ScalarType(
        ymax, {'fun_name': fun_name, 'entity': 'ymax'},
    ).to_ctypes()
    _monlev = utils._EngineIntScalarType(
        monlev, {'fun_name': fun_name, 'entity': 'monlev'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    utils._check_callable(prsol, {'fun_name': fun_name, 'entity': 'prsol'}, allow_none=True)
    _prsol = utils._UNUSED_CALLBACK
    _prsolh = utils._cast_callback(
        prsolh, cb_prsolh,
        {'fun_name': fun_name, 'entity': 'prsol'},
    )
    _ldw_py = utils._EngineFloat64ArrayType.get_nd(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldw = utils._EngineIntScalarType(
        _ldw_py, {'fun_name': fun_name, 'entity': '_ldw'},
    ).to_ctypes()
    _sdw_py = utils._EngineFloat64ArrayType.get_nd(w, {'fun_name': fun_name, 'entity': 'w'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _sdw = utils._EngineIntScalarType(
        _sdw_py, {'fun_name': fun_name, 'entity': '_sdw'},
    ).to_ctypes()
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(max(_n_py, _m_py), 3*_m_py+12+max(11, _m_py)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('lwsav', 'iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _lwsav = utils._EngineLogicalArrayType.empty_ctypes(5)
        comm['lwsav'] = _lwsav
    else:
        _lwsav = comm['lwsav']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(15)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(15)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _p,
        _m,
        _n,
        _n1,
        _pe,
        _pf,
        _e,
        _dp,
        _npoint,
        _swp,
        _ldswp,
        _icount,
        _brkpts,
        _rangeh,
        _bc,
        _bch,
        _fcn,
        _fcnh,
        _eqn,
        _eqnh,
        _constr,
        _constrsub,
        _constrh,
        _ymax,
        _monlev,
        _monit,
        _monith,
        _prsol,
        _prsolh,
        _w,
        _ldw,
        _sdw,
        _iuser,
        _ruser,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, monlev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: monlev = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an element of the parameter convergence control array',
                    'is zero or negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry an element of the local error control array is zero or negative.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ymax >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npoint = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npoint >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, icount = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: icount >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m >= n1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n >= n1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The constraints have been violated by the initial parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The sequence of break-points is not strictly monotonic in their initial',
                    'specification.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration from first to last break-point with initial or final',
                    'parameters, the step size was reduced too far for the integration to',
                    'proceed. Consider reversing the order of break-points or relaxing the',
                    'local error control. If this error exit still results, either this',
                    'function is not a suitable method for solving the problem, or the initial',
                    'choice of parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In the integration from first to last break-point with initial or final',
                    'parameters, a suitable initial step could not be found on one of the',
                    'intervals. Consider reversing the order of break-points or relaxing the',
                    'local error control. If this error exit still results, either this',
                    'function is not suitable for solving the problem, or the initial choice of',
                    'parameters is very poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During integration the solution exceeded ymax in magnitude, where,',
                    'on entry, ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On calculating the initial approximation to the Jacobian, the constraints',
                    'were violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On perturbing the parameters when calculating the initial approximation',
                    'to the Jacobian, the monotonicity condition on break-points is violated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99978:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An initial step-length could be found for integration to proceed',
                    'with the current parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The step-length required to calculate the Jacobian to sufficient accuracy',
                    'is too small.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On calculating the initial approximation to the Jacobian, the solution',
                    'to the system of differential equations exceeded ymax in magnitude,',
                    'where, on entry, ymax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Jacobian has an insignificant column. Make sure',
                    'that the solution vector depends on all the parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal singular value decomposition has failed. This error',
                    'can be avoided by changing the initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99973:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Newton iteration has failed to converge. This can indicate a poor',
                    'initial choice of parameters or a very difficult problem. Consider',
                    'varying elements of the parameter convergence control if the residuals',
                    'are small; otherwise vary initial parameter estimates.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of iterations permitted by icount has been exceeded where this',
                    'was set to a positive value on entry. icount = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99970:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating Jacobian. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in calculating residual. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in integration. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 19 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error in Newton method. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_cls.refresh_py(_p)
        _pf_cls.refresh_py(_pf)
        _dp_cls.refresh_py(_dp)
        _swp_cls.refresh_py(_swp)
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_coll_nth_comp(n, m, l, x0, x1, k1, kp, c, coeff, bdyc, data=None):
    u"""
    nth-order linear ordinary differential equations, boundary value
    problem, collocation and least squares.

    For full information please refer to the NAG Library document for
    d02tg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02tgf.html

    Parameters
    ----------
    n : int
        n, the number of differential equations in the system.

    m : int, shape (`n`)
        `m`[i] must be set to the highest order derivative occurring in
        the ith equation, for i = 1,2,...,n.

    l : int, shape (`n`)
        `l`[i] must be set to the number of boundary conditions
        associated with the ith equation, for i = 1,2,...,n.

    x0 : float
        The left-hand boundary, x_0.

    x1 : float
        The right-hand boundary, x_1.

    k1 : int
        The number of coefficients, k_1, to be returned in the Chebyshev
        series representation of the solution (hence, the degree of the
        polynomial approximation is `k1`-1).

    kp : int
        The number of collocation points to be used, k_p.

    c : float, shape (`k1`, `n`), modified in place
        On exit: the kth column of `c` contains the computed Chebyshev
        coefficients of the kth component of the solution, y_k; that is,
        the computed solution is:

            y_k = ⎲_i = 1^k_1`c`[i-1,k-1]T_{i-1}(x), 1 <= k <= n,

        where T_i(x) is the Chebyshev polynomial of the first kind and ⎲
        denotes that the first coefficient, `c`[0,k-1], is halved.

    coeff : callable rhs = coeff(x, i, a, ia, ia1, rhs, data=None)
        `coeff` defines the system of differential equations (see
        [Description]).

        It must evaluate the coefficient functions f_kj^i(x) and the
        right-hand side function r^i(x) of the ith equation at a given
        point.

        Only nonzero entries of the array `a` and `rhs` need be
        specifically assigned, since all elements are set to zero by
        ``bvp_coll_nth_comp`` before calling `coeff`.

        Parameters
        ~~~~~~~~~~
        x : float
            x, the point at which the functions must be evaluated.

        i : int
            The equation for which the coefficients and right-hand side
            are to be evaluated.

        a : float, ndarray, shape (`ia`, `ia1`), to be modified in place
            On entry: all elements of `a` are set to zero.

            On exit: `a`[k-1,j-1] must contain the value f_kj^i(x), for
            1 <= k <= n, 1 <= j <= m_i+1.

        ia : int
            The first dimension of the array `a`.

            The second dimension of the array `a`.

        ia1 : int
            The first dimension of the array `a`.

            The second dimension of the array `a`.

        rhs : float
            Is set to zero.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        rhs : float
            It must contain the value r^i(x).

    bdyc : callable (x, rhs) = bdyc(i, j, a, ia, ia1, rhs, data=None)
        `bdyc` defines the boundary conditions (see [Description]).

        It must evaluate the coefficient functions f_kj^ij and
        right-hand side function r^(ij) in the jth boundary condition
        associated with the ith equation, at the point x^(ij) at which
        the boundary condition is applied.

        Only nonzero entries of the array `a` and `rhs` need be
        specifically assigned, since all elements are set to zero by
        ``bvp_coll_nth_comp`` before calling `bdyc`.

        Parameters
        ~~~~~~~~~~
        i : int
            The differential equation with which the condition is
            associated.

        j : int
            The boundary condition for which the coefficients and
            right-hand side are to be evaluated.

        a : float, ndarray, shape (`ia`, `ia1`), to be modified in place
            On entry: all elements of `a` are set to zero.

            On exit: the value f_kj^ij(x^(ij)), for 1 <= k <= n, 1 <= j
            <= m_i+1.

        ia : int
            The first dimension of the array `a`.

            The second dimension of the array `a`.

        ia1 : int
            The first dimension of the array `a`.

            The second dimension of the array `a`.

        rhs : float
            Is set to zero.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        x : float
            x^(ij), the value at which the boundary condition is
            applied.

        rhs : float
            The value r^(ij)(x^(ij)).

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kp` = *<value>* and `k1` = *<value>*.

            Constraint: `kp`+1 >= `k1`.

        (`errno` 1)
            On entry, `k1` = *<value>* and max(`m`[i-1]) = *<value>*.

            Constraint: `k1` >= 1+max(`m`[i-1]).

        (`errno` 1)
            On entry, one of the equations is of order less than 1.

        (`errno` 1)
            On entry, `x1` = *<value>* and `x0` = *<value>*.

            Constraint: `x1` > `x0`.

        (`errno` 1)
            On entry, a negative number of boundary conditions was set
            for one of the system equations.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            Either the boundary conditions are not linearly independent,
            or the coefficient matrix is rank deficient. Increasing the
            number of collocation points may overcome this latter
            problem.

        (`errno` 4)
            Iterative refinement in the least squares solution has
            failed to converge. The coefficient matrix is too
            ill-conditioned.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nth_comp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_coeff_name = fun_name + ':coeff'

    coeffh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, i.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[ia, ia1].
        utils._BYREF_INT, # Input, ia.
        utils._BYREF_INT, # Input, ia1.
        utils._BYREF_FLOAT64, # Input/Output, rhs.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_coeffh(_coeff, en_data, x, i, a, ia, ia1, rhs, _iuser, _ruser):
        """Helper for coeff."""
        try:
            _x_coeffh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _i_coeffh = utils._EngineIntScalarType.to_py(i.contents)
            _ia_coeffh = utils._EngineIntScalarType.to_py(ia.contents)
            _ia1_coeffh = utils._EngineIntScalarType.to_py(ia1.contents)
            _a_coeffh = utils._EngineFloat64ArrayType.from_pointer(
                a,
                (_ia_coeffh, _ia1_coeffh),
                sorder=_m_sorder,
            )
            _rhs_coeffh = utils._EngineFloat64ScalarType.to_py(rhs.contents)
            if data is not None:
                _rhs_coeffh = coeff(
                    _x_coeffh,
                    _i_coeffh,
                    _a_coeffh,
                    _ia_coeffh,
                    _ia1_coeffh,
                    _rhs_coeffh,
                    data,
                )
            else:
                _rhs_coeffh = coeff(
                    _x_coeffh,
                    _i_coeffh,
                    _a_coeffh,
                    _ia_coeffh,
                    _ia1_coeffh,
                    _rhs_coeffh,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _a_coeffh, {'fun_name': cb_coeff_name, 'returned': 'a'}, a,
            )
            utils._unlink_pointer(a)
            utils._EngineFloat64ScalarType(
                _rhs_coeffh, {'fun_name': cb_coeff_name, 'returned': 'rhs'},
            ).refresh_ctypes_buffer(rhs)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_bdyc_name = fun_name + ':bdyc'

    bdych = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Output, x.
        utils._BYREF_INT, # Input, i.
        utils._BYREF_INT, # Input, j.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[ia, ia1].
        utils._BYREF_INT, # Input, ia.
        utils._BYREF_INT, # Input, ia1.
        utils._BYREF_FLOAT64, # Input/Output, rhs.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_bdych(_bdyc, en_data, x, i, j, a, ia, ia1, rhs, _iuser, _ruser):
        """Helper for bdyc."""
        try:
            _i_bdych = utils._EngineIntScalarType.to_py(i.contents)
            _j_bdych = utils._EngineIntScalarType.to_py(j.contents)
            _ia_bdych = utils._EngineIntScalarType.to_py(ia.contents)
            _ia1_bdych = utils._EngineIntScalarType.to_py(ia1.contents)
            _a_bdych = utils._EngineFloat64ArrayType.from_pointer(
                a,
                (_ia_bdych, _ia1_bdych),
                sorder=_m_sorder,
            )
            _rhs_bdych = utils._EngineFloat64ScalarType.to_py(rhs.contents)
            if data is not None:
                _x_bdych, _rhs_bdych = bdyc(
                    _i_bdych,
                    _j_bdych,
                    _a_bdych,
                    _ia_bdych,
                    _ia1_bdych,
                    _rhs_bdych,
                    data,
                )
            else:
                _x_bdych, _rhs_bdych = bdyc(
                    _i_bdych,
                    _j_bdych,
                    _a_bdych,
                    _ia_bdych,
                    _ia1_bdych,
                    _rhs_bdych,
                )
            utils._EngineFloat64ScalarType(
                _x_bdych, {'fun_name': cb_bdyc_name, 'returned': 'x'},
            ).refresh_ctypes_buffer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _a_bdych, {'fun_name': cb_bdyc_name, 'returned': 'a'}, a,
            )
            utils._unlink_pointer(a)
            utils._EngineFloat64ScalarType(
                _rhs_bdych, {'fun_name': cb_bdyc_name, 'returned': 'rhs'},
            ).refresh_ctypes_buffer(rhs)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02tgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input, m[n].
        utils._EngineIntArrayType.c_type, # Input, l[n].
        utils._BYREF_FLOAT64, # Input, x0.
        utils._BYREF_FLOAT64, # Input, x1.
        utils._BYREF_INT, # Input, k1.
        utils._BYREF_INT, # Input, kp.
        utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._UNUSED_FUNCTYPE,
        coeffh,
        utils._UNUSED_FUNCTYPE,
        bdych,
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Workspace, iw.
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m = utils._EngineIntArrayType(
        m, {'fun_name': fun_name, 'entity': 'm'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _l = utils._EngineIntArrayType(
        l, {'fun_name': fun_name, 'entity': 'l'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x0 = utils._EngineFloat64ScalarType(
        x0, {'fun_name': fun_name, 'entity': 'x0'},
    ).to_ctypes()
    _x1 = utils._EngineFloat64ScalarType(
        x1, {'fun_name': fun_name, 'entity': 'x1'},
    ).to_ctypes()
    _k1 = utils._EngineIntScalarType(
        k1, {'fun_name': fun_name, 'entity': 'k1'},
    ).to_ctypes()
    _kp = utils._EngineIntScalarType(
        kp, {'fun_name': fun_name, 'entity': 'kp'},
    ).to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _k1_py = utils._EngineIntScalarType.to_py(_k1)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_k1_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(coeff, {'fun_name': fun_name, 'entity': 'coeff'})
    _coeff = utils._UNUSED_CALLBACK
    _coeffh = utils._cast_callback(
        coeffh, cb_coeffh,
        {'fun_name': fun_name, 'entity': 'coeff'},
    )
    utils._check_callable(bdyc, {'fun_name': fun_name, 'entity': 'bdyc'})
    _bdyc = utils._UNUSED_CALLBACK
    _bdych = utils._cast_callback(
        bdych, cb_bdych,
        {'fun_name': fun_name, 'entity': 'bdyc'},
    )
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType.c_type(0)
    _iw = utils._EngineIntArrayType.empty_ctypes(0)
    _liw = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _l,
        _x0,
        _x1,
        _k1,
        _kp,
        _c,
        _ldc,
        _coeff,
        _coeffh,
        _bdyc,
        _bdych,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, kp = {:s} and k1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: kp+1 >= k1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k1 = {:s} and max(m[i-1]) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k1 >= 1+max(m[i-1]).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one of the equations is of order less than 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x1 = {:s} and x0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x1 > x0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a negative number of boundary conditions was set for one of the',
                    'system equations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the boundary conditions are not linearly independent, or the',
                    'coefficient matrix is rank deficient. Increasing the number of',
                    'collocation points may overcome this latter problem.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Iterative refinement in the least squares solution has failed to',
                    'converge. The coefficient matrix is too ill-conditioned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_coll_nlin_solve(ffun, fjac, gafun, gbfun, gajac, gbjac, guess, comm, data=None):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, collocation technique (thread safe).

    For full information please refer to the NAG Library document for
    d02tl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02tlf.html

    Parameters
    ----------
    ffun : callable ffun(x, y, neq, m, mmax, f, data=None)
        `ffun` must evaluate the functions f_i for given values
        x,z(y(x)).

        Parameters
        ~~~~~~~~~~
        x : float
            x, the independent variable.

        y : float, ndarray, shape (`neq`, `mmax`)
            `y`[i-1,j] contains y_i^(j)(x), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(x) = y_i(x).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        f : float, ndarray, shape (`neq`), to be modified in place
            On exit: `f`[i-1] must contain f_i, for i = 1,2,...,`neq`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fjac : callable fjac(x, y, neq, m, mmax, dfdy, data=None)
        `fjac` must evaluate the partial derivatives of f_i with respect
        to the elements of

        z(y(x)) =
        (y_1(x),y_1^1(x),...,y_1^(m_1-1)(x),y_2(x),...,y_n^(m_n-1)(x)).

        Parameters
        ~~~~~~~~~~
        x : float
            x, the independent variable.

        y : float, ndarray, shape (`neq`, `mmax`)
            `y`[i-1,j] contains y_i^(j)(x), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(x) = y_i(x).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        dfdy : float, ndarray, shape (`neq`, `neq`, `mmax`), to be
        modified in place
            On entry: set to zero.

            On exit: `dfdy`[i-1,j-1,k] must contain the partial
            derivative of f_i with respect to y_j^(k), for k =
            0,1,...,`m`[j-1]-1, for j = 1,2,...,`neq`, for i =
            1,2,...,`neq`. Only nonzero partial derivatives need be set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    gafun : callable gafun(ya, neq, m, mmax, nlbc, ga, data=None)
        `gafun` must evaluate the boundary conditions at the left-hand
        end of the range, that is functions g_i(z(y(a))) for given
        values of z(y(a)).

        Parameters
        ~~~~~~~~~~
        ya : float, ndarray, shape (`neq`, `mmax`)
            `ya`[i-1,j] contains y_i^(j)(a), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(a) = y_i(a).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        nlbc : int
            The number of boundary conditions at a.

        ga : float, ndarray, shape (`nlbc`), to be modified in place
            On exit: `ga`[i-1] must contain g_i(z(y(a))), for i =
            1,2,...,`nlbc`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    gbfun : callable gbfun(yb, neq, m, mmax, nrbc, gb, data=None)
        `gbfun` must evaluate the boundary conditions at the right-hand
        end of the range, that is functions g__i(z(y(b))) for given
        values of z(y(b)).

        Parameters
        ~~~~~~~~~~
        yb : float, ndarray, shape (`neq`, `mmax`)
            `yb`[i-1,j] contains y_i^(j)(b), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(b) = y_i(b).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        nrbc : int
            The number of boundary conditions at b.

        gb : float, ndarray, shape (`nrbc`), to be modified in place
            On exit: `gb`[i-1] must contain g__i(z(y(b))), for i =
            1,2,...,`nrbc`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    gajac : callable gajac(ya, neq, m, mmax, nlbc, dgady, data=None)
        `gajac` must evaluate the partial derivatives of g_i(z(y(a)))
        with respect to the elements of z(y(a)) =
        (y_1(a),y_1^1(a),...,y_1^(m_1-1)(a),y_2(a),...,y_n^(m_n-1)(a)).

        Parameters
        ~~~~~~~~~~
        ya : float, ndarray, shape (`neq`, `mmax`)
            `ya`[i-1,j] contains y_i^(j)(a), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(a) = y_i(a).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        nlbc : int
            The number of boundary conditions at a.

        dgady : float, ndarray, shape (`nlbc`, `neq`, `mmax`), to be
        modified in place
            On entry: set to zero.

            On exit: `dgady`[i-1,j-1,k] must contain the partial
            derivative of g_i(z(y(a))) with respect to y_j^(k)(a), for k
            = 0,1,...,`m`[j-1]-1, for j = 1,2,...,`neq`, for i =
            1,2,...,`nlbc`. Only nonzero partial derivatives need be
            set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    gbjac : callable gbjac(yb, neq, m, mmax, nrbc, dgbdy, data=None)
        `gbjac` must evaluate the partial derivatives of g__i(z(y(b)))
        with respect to the elements of z(y(b)) =
        (y_1(b),y_1^1(b),...,y_1^(m_1-1)(b),y_2(b),...,y_n^(m_n-1)(b)).

        Parameters
        ~~~~~~~~~~
        yb : float, ndarray, shape (`neq`, `mmax`)
            `yb`[i-1,j] contains y_i^(j)(b), for j = 0,1,...,`m`[i-1]-1,
            for i = 1,2,...,`neq`.

            Note:  y_i^(0)(b) = y_i(b).

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        nrbc : int
            The number of boundary conditions at b.

        dgbdy : float, ndarray, shape (`nrbc`, `neq`, `mmax`), to be
        modified in place
            On entry: set to zero.

            On exit: `dgbdy`[i-1,j-1,k] must contain the partial
            derivative of g__i(z(y(b))) with respect to y_j^(k)(b), for
            k = 0,1,...,`m`[j-1]-1, for j = 1,2,...,`neq`, for i =
            1,2,...,`nrbc`. Only nonzero partial derivatives need be
            set.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    guess : callable guess(x, neq, m, mmax, y, dym, data=None)
        `guess` must return initial approximations for the solution
        components y_i^(j) and the derivatives y_i^(m_i), for j =
        0,1,...,`m`[i-1]-1, for i = 1,2,...,`neq`.

        Try to compute each derivative y_i^(m_i) such that it
        corresponds to your approximations to y_i^(j), for j =
        0,1,...,`m`[i-1]-1.

        You should not call `ffun` to compute y_i^(m_i).

        Parameters
        ~~~~~~~~~~
        x : float
            x, the independent variable; x in [a,b].

        neq : int
            The number of differential equations.

        m : int, ndarray, shape (`neq`)
            `m`[i-1] contains m_i, the order of the ith differential
            equation, for i = 1,2,...,`neq`.

        mmax : int
            max(`m`[i]).

        y : float, ndarray, shape (`neq`, `mmax`), to be modified in
        place
            On exit: `y`[i-1,j] must contain y_i^(j)(x), for j =
            0,1,...,`m`[i-1]-1, for i = 1,2,...,`neq`.

            Note:  y_i^(0)(x) = y_i(x).

        dym : float, ndarray, shape (`neq`), to be modified in place
            On exit: `dym`[i-1] must contain y_i^(m_i)(x), for i =
            1,2,...,`neq`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``bvp_coll_nlin_setup``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the setup function has not been called or the
            communication arrays have become corrupted. No solution will
            be computed.

        (`errno` 2)
            Numerical singularity has been detected in the Jacobian used
            in the Newton iteration.

            No results have been generated. Check the coding of the
            functions for calculating the Jacobians of system and
            boundary conditions.

        (`errno` 3)
            All Newton iterations that have been attempted have failed
            to converge.

            No results have been generated. Check the coding of the
            functions for calculating the Jacobians of system and
            boundary conditions.

            Try to provide a better initial solution approximation.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            A Newton iteration has failed to converge. The computation
            has not succeeded but results have been returned for an
            intermediate mesh on which convergence was achieved.

            These results should be treated with extreme caution.

        (`errno` 5)
            The expected number of sub-intervals required to continue
            the computation exceeds the maximum specified: *<value>*.

            Results have been generated which may be useful.

            Try increasing this number or relaxing the error
            requirements.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nlin_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_ffun_name = fun_name + ':ffun'

    ffunh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._EngineFloat64ArrayType.c_type, # Output, f[neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_ffunh(_ffun, en_data, x, y, neq, m, mmax, f, _iuser, _ruser):
        """Helper for ffun."""
        try:
            _x_ffunh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _neq_ffunh = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_ffunh = utils._EngineIntScalarType.to_py(mmax.contents)
            _y_ffunh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_ffunh, _mmax_ffunh),
                sorder=utils._EngineData.default_sorder,
            )
            _m_ffunh = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_ffunh,),
            )
            _f_ffunh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_neq_ffunh,),
                can_reference_contents=False,
            )
            if data is not None:
                ffun(
                    _x_ffunh,
                    _y_ffunh,
                    _neq_ffunh,
                    _m_ffunh,
                    _mmax_ffunh,
                    _f_ffunh,
                    data,
                )
            else:
                ffun(
                    _x_ffunh,
                    _y_ffunh,
                    _neq_ffunh,
                    _m_ffunh,
                    _mmax_ffunh,
                    _f_ffunh,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_ffunh, {'fun_name': cb_ffun_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_fjac_name = fun_name + ':fjac'

    fjach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, y[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dfdy[neq, neq, mmax].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fjach(_fjac, en_data, x, y, neq, m, mmax, dfdy, _iuser, _ruser):
        """Helper for fjac."""
        try:
            _x_fjach = utils._EngineFloat64ScalarType.to_py(x.contents)
            _neq_fjach = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_fjach = utils._EngineIntScalarType.to_py(mmax.contents)
            _y_fjach = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_fjach, _mmax_fjach),
                sorder=utils._EngineData.default_sorder,
            )
            _m_fjach = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_fjach,),
            )
            _dfdy_fjach = utils._EngineFloat64ArrayType.from_pointer(
                dfdy,
                (_neq_fjach, _neq_fjach, _mmax_fjach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                fjac(
                    _x_fjach,
                    _y_fjach,
                    _neq_fjach,
                    _m_fjach,
                    _mmax_fjach,
                    _dfdy_fjach,
                    data,
                )
            else:
                fjac(
                    _x_fjach,
                    _y_fjach,
                    _neq_fjach,
                    _m_fjach,
                    _mmax_fjach,
                    _dfdy_fjach,
                )
            utils._unlink_pointer(y)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _dfdy_fjach, {'fun_name': cb_fjac_name, 'returned': 'dfdy'}, dfdy,
            )
            utils._unlink_pointer(dfdy)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_gafun_name = fun_name + ':gafun'

    gafunh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, ya[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._BYREF_INT, # Input, nlbc.
        utils._EngineFloat64ArrayType.c_type, # Output, ga[nlbc].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gafunh(_gafun, en_data, ya, neq, m, mmax, nlbc, ga, _iuser, _ruser):
        """Helper for gafun."""
        try:
            _neq_gafunh = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_gafunh = utils._EngineIntScalarType.to_py(mmax.contents)
            _ya_gafunh = utils._EngineFloat64ArrayType.from_pointer(
                ya,
                (_neq_gafunh, _mmax_gafunh),
                sorder=utils._EngineData.default_sorder,
            )
            _m_gafunh = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_gafunh,),
            )
            _nlbc_gafunh = utils._EngineIntScalarType.to_py(nlbc.contents)
            _ga_gafunh = utils._EngineFloat64ArrayType.from_pointer(
                ga,
                (_nlbc_gafunh,),
                can_reference_contents=False,
            )
            if data is not None:
                gafun(
                    _ya_gafunh,
                    _neq_gafunh,
                    _m_gafunh,
                    _mmax_gafunh,
                    _nlbc_gafunh,
                    _ga_gafunh,
                    data,
                )
            else:
                gafun(
                    _ya_gafunh,
                    _neq_gafunh,
                    _m_gafunh,
                    _mmax_gafunh,
                    _nlbc_gafunh,
                    _ga_gafunh,
                )
            utils._unlink_pointer(ya)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _ga_gafunh, {'fun_name': cb_gafun_name, 'returned': 'ga'}, ga,
            )
            utils._unlink_pointer(ga)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_gbfun_name = fun_name + ':gbfun'

    gbfunh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, yb[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._BYREF_INT, # Input, nrbc.
        utils._EngineFloat64ArrayType.c_type, # Output, gb[nrbc].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gbfunh(_gbfun, en_data, yb, neq, m, mmax, nrbc, gb, _iuser, _ruser):
        """Helper for gbfun."""
        try:
            _neq_gbfunh = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_gbfunh = utils._EngineIntScalarType.to_py(mmax.contents)
            _yb_gbfunh = utils._EngineFloat64ArrayType.from_pointer(
                yb,
                (_neq_gbfunh, _mmax_gbfunh),
                sorder=utils._EngineData.default_sorder,
            )
            _m_gbfunh = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_gbfunh,),
            )
            _nrbc_gbfunh = utils._EngineIntScalarType.to_py(nrbc.contents)
            _gb_gbfunh = utils._EngineFloat64ArrayType.from_pointer(
                gb,
                (_nrbc_gbfunh,),
                can_reference_contents=False,
            )
            if data is not None:
                gbfun(
                    _yb_gbfunh,
                    _neq_gbfunh,
                    _m_gbfunh,
                    _mmax_gbfunh,
                    _nrbc_gbfunh,
                    _gb_gbfunh,
                    data,
                )
            else:
                gbfun(
                    _yb_gbfunh,
                    _neq_gbfunh,
                    _m_gbfunh,
                    _mmax_gbfunh,
                    _nrbc_gbfunh,
                    _gb_gbfunh,
                )
            utils._unlink_pointer(yb)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _gb_gbfunh, {'fun_name': cb_gbfun_name, 'returned': 'gb'}, gb,
            )
            utils._unlink_pointer(gb)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_gajac_name = fun_name + ':gajac'

    gajach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, ya[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._BYREF_INT, # Input, nlbc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dgady[nlbc, neq, mmax].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gajach(_gajac, en_data, ya, neq, m, mmax, nlbc, dgady, _iuser, _ruser):
        """Helper for gajac."""
        try:
            _neq_gajach = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_gajach = utils._EngineIntScalarType.to_py(mmax.contents)
            _ya_gajach = utils._EngineFloat64ArrayType.from_pointer(
                ya,
                (_neq_gajach, _mmax_gajach),
                sorder=utils._EngineData.default_sorder,
            )
            _m_gajach = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_gajach,),
            )
            _nlbc_gajach = utils._EngineIntScalarType.to_py(nlbc.contents)
            _dgady_gajach = utils._EngineFloat64ArrayType.from_pointer(
                dgady,
                (_nlbc_gajach, _neq_gajach, _mmax_gajach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                gajac(
                    _ya_gajach,
                    _neq_gajach,
                    _m_gajach,
                    _mmax_gajach,
                    _nlbc_gajach,
                    _dgady_gajach,
                    data,
                )
            else:
                gajac(
                    _ya_gajach,
                    _neq_gajach,
                    _m_gajach,
                    _mmax_gajach,
                    _nlbc_gajach,
                    _dgady_gajach,
                )
            utils._unlink_pointer(ya)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _dgady_gajach, {'fun_name': cb_gajac_name, 'returned': 'dgady'}, dgady,
            )
            utils._unlink_pointer(dgady)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_gbjac_name = fun_name + ':gbjac'

    gbjach = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, yb[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._BYREF_INT, # Input, nrbc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dgbdy[nrbc, neq, mmax].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gbjach(_gbjac, en_data, yb, neq, m, mmax, nrbc, dgbdy, _iuser, _ruser):
        """Helper for gbjac."""
        try:
            _neq_gbjach = utils._EngineIntScalarType.to_py(neq.contents)
            _mmax_gbjach = utils._EngineIntScalarType.to_py(mmax.contents)
            _yb_gbjach = utils._EngineFloat64ArrayType.from_pointer(
                yb,
                (_neq_gbjach, _mmax_gbjach),
                sorder=utils._EngineData.default_sorder,
            )
            _m_gbjach = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_gbjach,),
            )
            _nrbc_gbjach = utils._EngineIntScalarType.to_py(nrbc.contents)
            _dgbdy_gbjach = utils._EngineFloat64ArrayType.from_pointer(
                dgbdy,
                (_nrbc_gbjach, _neq_gbjach, _mmax_gbjach),
                sorder=utils._EngineData.default_sorder,
            )
            if data is not None:
                gbjac(
                    _yb_gbjach,
                    _neq_gbjach,
                    _m_gbjach,
                    _mmax_gbjach,
                    _nrbc_gbjach,
                    _dgbdy_gbjach,
                    data,
                )
            else:
                gbjac(
                    _yb_gbjach,
                    _neq_gbjach,
                    _m_gbjach,
                    _mmax_gbjach,
                    _nrbc_gbjach,
                    _dgbdy_gbjach,
                )
            utils._unlink_pointer(yb)
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _dgbdy_gbjach, {'fun_name': cb_gbjac_name, 'returned': 'dgbdy'}, dgbdy,
            )
            utils._unlink_pointer(dgbdy)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_guess_name = fun_name + ':guess'

    guessh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, mmax.
        utils._EngineFloat64ArrayType.c_type, # Output, y[neq, mmax].
        utils._EngineFloat64ArrayType.c_type, # Output, dym[neq].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_guessh(_guess, en_data, x, neq, m, mmax, y, dym, _iuser, _ruser):
        """Helper for guess."""
        try:
            _x_guessh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _neq_guessh = utils._EngineIntScalarType.to_py(neq.contents)
            _m_guessh = utils._EngineIntArrayType.from_pointer(
                m,
                (_neq_guessh,),
            )
            _mmax_guessh = utils._EngineIntScalarType.to_py(mmax.contents)
            _y_guessh = utils._EngineFloat64ArrayType.from_pointer(
                y,
                (_neq_guessh, _mmax_guessh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _dym_guessh = utils._EngineFloat64ArrayType.from_pointer(
                dym,
                (_neq_guessh,),
                can_reference_contents=False,
            )
            if data is not None:
                guess(
                    _x_guessh,
                    _neq_guessh,
                    _m_guessh,
                    _mmax_guessh,
                    _y_guessh,
                    _dym_guessh,
                    data,
                )
            else:
                guess(
                    _x_guessh,
                    _neq_guessh,
                    _m_guessh,
                    _mmax_guessh,
                    _y_guessh,
                    _dym_guessh,
                )
            utils._unlink_pointer(m)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _y_guessh, {'fun_name': cb_guess_name, 'returned': 'y'}, y,
            )
            utils._unlink_pointer(y)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _dym_guessh, {'fun_name': cb_guess_name, 'returned': 'dym'}, dym,
            )
            utils._unlink_pointer(dym)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d02tlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        ffunh,
        utils._UNUSED_FUNCTYPE,
        fjach,
        utils._UNUSED_FUNCTYPE,
        gafunh,
        utils._UNUSED_FUNCTYPE,
        gbfunh,
        utils._UNUSED_FUNCTYPE,
        gajach,
        utils._UNUSED_FUNCTYPE,
        gbjach,
        utils._UNUSED_FUNCTYPE,
        guessh,
        utils._BYREF_INT, # Input, mmax.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(ffun, {'fun_name': fun_name, 'entity': 'ffun'})
    _ffun = utils._UNUSED_CALLBACK
    _ffunh = utils._cast_callback(
        ffunh, cb_ffunh,
        {'fun_name': fun_name, 'entity': 'ffun'},
    )
    utils._check_callable(fjac, {'fun_name': fun_name, 'entity': 'fjac'})
    _fjac = utils._UNUSED_CALLBACK
    _fjach = utils._cast_callback(
        fjach, cb_fjach,
        {'fun_name': fun_name, 'entity': 'fjac'},
    )
    utils._check_callable(gafun, {'fun_name': fun_name, 'entity': 'gafun'})
    _gafun = utils._UNUSED_CALLBACK
    _gafunh = utils._cast_callback(
        gafunh, cb_gafunh,
        {'fun_name': fun_name, 'entity': 'gafun'},
    )
    utils._check_callable(gbfun, {'fun_name': fun_name, 'entity': 'gbfun'})
    _gbfun = utils._UNUSED_CALLBACK
    _gbfunh = utils._cast_callback(
        gbfunh, cb_gbfunh,
        {'fun_name': fun_name, 'entity': 'gbfun'},
    )
    utils._check_callable(gajac, {'fun_name': fun_name, 'entity': 'gajac'})
    _gajac = utils._UNUSED_CALLBACK
    _gajach = utils._cast_callback(
        gajach, cb_gajach,
        {'fun_name': fun_name, 'entity': 'gajac'},
    )
    utils._check_callable(gbjac, {'fun_name': fun_name, 'entity': 'gbjac'})
    _gbjac = utils._UNUSED_CALLBACK
    _gbjach = utils._cast_callback(
        gbjach, cb_gbjach,
        {'fun_name': fun_name, 'entity': 'gbjac'},
    )
    utils._check_callable(guess, {'fun_name': fun_name, 'entity': 'guess'})
    _guess = utils._UNUSED_CALLBACK
    _guessh = utils._cast_callback(
        guessh, cb_guessh,
        {'fun_name': fun_name, 'entity': 'guess'},
    )
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _vl_mmax_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _mmax = utils._EngineIntScalarType(
        _vl_mmax_py, {'fun_name': fun_name, 'entity': '_mmax'},
    ).to_ctypes()
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['rcomm'], {'fun_name': fun_name, 'entity': 'comm[\'rcomm\']'}, exp_rank=1, validate=False)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm = comm['rcomm']
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ffun,
        _ffunh,
        _fjac,
        _fjach,
        _gafun,
        _gafunh,
        _gbfun,
        _gbfunh,
        _gajac,
        _gajach,
        _gbjac,
        _gbjach,
        _guess,
        _guessh,
        _mmax,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the setup function has not been called or the communication arrays',
                    'have become corrupted. No solution will be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Numerical singularity has been detected in the Jacobian used in the',
                    'Newton iteration.',
                    'No results have been generated. Check the coding of the',
                    'functions for calculating the Jacobians of system and boundary conditions.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All Newton iterations that have been attempted have failed to converge.',
                    'No results have been generated. Check the coding of the functions for',
                    'calculating the Jacobians of system and boundary conditions.',
                    'Try to provide a better initial solution approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'A Newton iteration has failed to converge. The computation has not',
                    'succeeded but results have been returned for an intermediate mesh on',
                    'which convergence was achieved.',
                    'These results should be treated with extreme caution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The expected number of sub-intervals required to continue the computation',
                    'exceeds the maximum specified: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Results have been generated which may be useful.',
                    'Try increasing this number or relaxing the error requirements.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_coll_nlin_setup(neq, m, nlbc, nrbc, ncol, tols, mxmesh, nmesh, mesh, ipmesh):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, setup for ``bvp_coll_nlin_solve``.

    For full information please refer to the NAG Library document for
    d02tv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02tvf.html

    Parameters
    ----------
    neq : int
        n, the number of ordinary differential equations to be solved.

    m : int, shape (`neq`)
        `m`[i-1] must contain m_i, the order of the ith differential
        equation, for i = 1,2,...,n.

    nlbc : int
        p, the number of left boundary conditions defined at the
        left-hand end, a (= `mesh`[0]).

    nrbc : int
        q, the number of right boundary conditions defined at the
        right-hand end, b (= `mesh`[`nmesh`-1]).

    ncol : int
        The number of collocation points to be used in each mesh
        sub-interval.

    tols : float, shape (`neq`)
        `tols`[i-1] must contain the error requirement for the ith
        solution component.

    mxmesh : int
        The maximum number of mesh points to be used during the solution
        process.

    nmesh : int
        The number of points to be used in the initial mesh of the
        solution process.

    mesh : float, shape (`mxmesh`)
        The positions of the initial `nmesh` mesh points. The remaining
        elements of `mesh` need not be set. You should try to place the
        mesh points in areas where you expect the solution to vary most
        rapidly. In the absence of any other information the points
        should be equally distributed on [a,b].

        `mesh`[0] must contain the left boundary point, a, and
        `mesh`[`nmesh`-1] must contain the right boundary point, b.

    ipmesh : int, shape (`mxmesh`)
        `ipmesh`[i-1] specifies whether or not the initial mesh point
        defined in `mesh`[i-1], for i = 1,2,...,`nmesh`, should be a
        fixed point in all meshes computed during the solution process.
        The remaining elements of `ipmesh` need not be set.

        `ipmesh`[i-1] = 1

          Indicates that `mesh`[i-1] should be a fixed point in all
          meshes.

        `ipmesh`[i-1] = 2

          Indicates that `mesh`[i-1] is not a fixed point.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ipmesh`[i] != 1 or 2 for some i.

        (`errno` 1)
            On entry, `ipmesh`[0] or `ipmesh`[`nmesh`-1] does not equal
            1.

        (`errno` 1)
            On entry, the elements of `mesh` are not strictly
            increasing.

        (`errno` 1)
            On entry, `nlbc` = *<value>*, `nrbc` = *<value>* and
            sum(`m`[i]) = *<value>*.

            Constraint: `nlbc`+`nrbc` = sum(`m`[i]).

        (`errno` 1)
            On entry, `nlbc` = *<value>* and `nrbc` = *<value>*.

            Constraint: `nlbc` >= 1 and `nrbc` >= 1.

        (`errno` 1)
            On entry, `ncol` = *<value>* and max(`m`[i]) = *<value>*.

            Constraint: max(`m`[i]) <= `ncol` <= 7.

        (`errno` 1)
            On entry, `tols`[*<value>*] = *<value>*.

            Constraint: `tols`[i-1] > *<value>* for all i.

        (`errno` 1)
            On entry, `m`[*<value>*] = *<value>*.

            Constraint: 1 <= `m`[i-1] <= 4 for all i.

        (`errno` 1)
            On entry, `mxmesh` = *<value>* and `nmesh` = *<value>*.

            Constraint: `mxmesh` >= 2*`nmesh`-1.

        (`errno` 1)
            On entry, `nmesh` = *<value>*.

            Constraint: `nmesh` >= 6.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nlin_setup'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02tvft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._EngineIntArrayType.c_type, # Input, m[neq].
        utils._BYREF_INT, # Input, nlbc.
        utils._BYREF_INT, # Input, nrbc.
        utils._BYREF_INT, # Input, ncol.
        utils._EngineFloat64ArrayType.c_type, # Input, tols[neq].
        utils._BYREF_INT, # Input, mxmesh.
        utils._BYREF_INT, # Input, nmesh.
        utils._EngineFloat64ArrayType.c_type, # Input, mesh[mxmesh].
        utils._EngineIntArrayType.c_type, # Input, ipmesh[mxmesh].
        utils._EngineFloat64ArrayType.c_type, # Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _m = utils._EngineIntArrayType(
        m, {'fun_name': fun_name, 'entity': 'm'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _nlbc = utils._EngineIntScalarType(
        nlbc, {'fun_name': fun_name, 'entity': 'nlbc'},
    ).to_ctypes()
    _nrbc = utils._EngineIntScalarType(
        nrbc, {'fun_name': fun_name, 'entity': 'nrbc'},
    ).to_ctypes()
    _ncol = utils._EngineIntScalarType(
        ncol, {'fun_name': fun_name, 'entity': 'ncol'},
    ).to_ctypes()
    _tols = utils._EngineFloat64ArrayType(
        tols, {'fun_name': fun_name, 'entity': 'tols'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _mxmesh = utils._EngineIntScalarType(
        mxmesh, {'fun_name': fun_name, 'entity': 'mxmesh'},
    ).to_ctypes()
    _nmesh = utils._EngineIntScalarType(
        nmesh, {'fun_name': fun_name, 'entity': 'nmesh'},
    ).to_ctypes()
    _mxmesh_py = utils._EngineIntScalarType.to_py(_mxmesh)
    _mesh = utils._EngineFloat64ArrayType(
        mesh, {'fun_name': fun_name, 'entity': 'mesh'},
        exp_shape=(_mxmesh_py,),
    ).to_ctypes()
    _ipmesh = utils._EngineIntArrayType(
        ipmesh, {'fun_name': fun_name, 'entity': 'ipmesh'},
        exp_shape=(_mxmesh_py,),
    ).to_ctypes()
    _lrcomm = utils._EngineIntScalarType.c_type(0)
    _lrcomm_py = utils._EngineIntScalarType.to_py(_lrcomm)
    _rcomm = utils._EngineFloat64ArrayType.empty_ctypes(_lrcomm_py)
    _comm_py = {}
    _licomm = utils._EngineIntScalarType.c_type(2)
    _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
    _icomm = utils._EngineIntArrayType.empty_ctypes(_licomm_py)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _m,
        _nlbc,
        _nrbc,
        _ncol,
        _tols,
        _mxmesh,
        _nmesh,
        _mesh,
        _ipmesh,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    if _ierr.value == utils._IERR_SUCCESS:
        _lrcomm = utils._EngineIntScalarType.c_type(_icomm[0])
        _lrcomm_py = utils._EngineIntScalarType.to_py(_lrcomm)
        _rcomm = utils._EngineFloat64ArrayType.empty_ctypes(_lrcomm_py)
        _comm_py['rcomm'] = _rcomm
        _licomm = utils._EngineIntScalarType.c_type(_icomm[1])
        _licomm_py = utils._EngineIntScalarType.to_py(_licomm)
        _icomm = utils._EngineIntArrayType.empty_ctypes(_licomm_py)
        _comm_py['icomm'] = _icomm
        engine_ctypes(
            _en_data,
            _neq,
            _m,
            _nlbc,
            _nrbc,
            _ncol,
            _tols,
            _mxmesh,
            _nmesh,
            _mesh,
            _ipmesh,
            _rcomm,
            _lrcomm,
            _icomm,
            _licomm,
            _errbuf,
            _ierr,
            _errbuf._length_,
        )
        utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipmesh[i] != 1 or 2 for some i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipmesh[0] or ipmesh[nmesh-1] does not equal 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the elements of mesh are not strictly increasing.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nlbc = {:s}, nrbc = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'sum(m[i]) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: nlbc+nrbc = sum(m[i]).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nlbc = {:s} and nrbc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nlbc >= 1 and nrbc >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncol = {:s} and max(m[i]) = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: max(m[i]) <= ncol <= 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tols[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: tols[i-1] > {:s} for all i.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= m[i-1] <= 4 for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mxmesh = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nmesh = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mxmesh >= 2*nmesh-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nmesh = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nmesh >= 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = _comm_py
    _err_class.manage()
    return return_data

def bvp_coll_nlin_contin(mxmesh, nmesh, mesh, ipmesh, comm):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, continuation facility for ``bvp_coll_nlin_solve``.

    For full information please refer to the NAG Library document for
    d02tx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02txf.html

    Parameters
    ----------
    mxmesh : int
        The maximum number of points allowed in the mesh.

    nmesh : int
        The number of points to be used in the new initial mesh. It is
        strongly recommended that if this function is called that the
        suggested value for `nmesh` is used. In this case the arrays
        mesh and ipmesh returned by ``bvp_coll_nlin_diag`` can be passed
        to this function without any modification.

    mesh : float, shape (`mxmesh`)
        The `nmesh` points to be used in the new initial mesh as
        specified by `ipmesh`.

    ipmesh : int, shape (`mxmesh`)
        Specifies the points in `mesh` to be used as the new initial
        mesh. Let {i_j:j = 1,2,...,`nmesh`} be the set of array indices
        of `ipmesh` such that `ipmesh`[i_j-1] = 1 or 2 and 1 = i_1 < i_2
        < ... < i_`nmesh`. Then `mesh`[i_j-1] will be included in the
        new initial mesh.

        If `ipmesh`[i_j-1] = 1, `mesh`[i_j-1] will be a fixed point in
        the new initial mesh.

        If `ipmesh`[k-1] = 3 for any k, `mesh`[k-1] will not be included
        in the new mesh.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``bvp_coll_nlin_setup``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The entries in `mesh` are not strictly increasing.

        (`errno` 1)
            The last point of the new mesh does not coincide with the
            right hand end of the range previously specified.

            Last point of the new mesh: *<value>*; right-hand end of the
            range: *<value>*.

        (`errno` 1)
            You have set the element of `ipmesh` corresponding to the
            last element of `mesh` to be included in the new mesh as
            *<value>*, which is not 1.

        (`errno` 1)
            Expected *<value>* elements of `ipmesh` to be 1 or 2, but
            *<value>* such elements found.

        (`errno` 1)
            An element of `ipmesh` was set to -1 before `nmesh` elements
            containing 1 or 2 were detected.

        (`errno` 1)
            `ipmesh`[i] != -1, 1, 2 or 3 for some i.

        (`errno` 1)
            On entry, `ipmesh`[0] = *<value>*.

            Constraint: `ipmesh`[0] = 1.

        (`errno` 1)
            The first element of array `mesh` does not coincide with the
            left-hand end of the range previously specified.

            First element of `mesh`: *<value>*; left-hand of the range:
            *<value>*.

        (`errno` 1)
            On entry, `nmesh` = *<value>* and `mxmesh` = *<value>*.

            Constraint: `nmesh` <= (`mxmesh`+1)/2.

        (`errno` 1)
            On entry, `nmesh` = *<value>*.

            Constraint: `nmesh` >= 6.

        (`errno` 1)
            On entry, `mxmesh` = *<value>* and mxmesh = *<value>* in
            ``bvp_coll_nlin_setup``.

            Constraint: `mxmesh` = mxmesh in ``bvp_coll_nlin_setup``.

        (`errno` 1)
            The solver function did not produce any results suitable for
            remeshing.

        (`errno` 1)
            The solver function does not appear to have been called.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nlin_contin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02txft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mxmesh.
        utils._BYREF_INT, # Input, nmesh.
        utils._EngineFloat64ArrayType.c_type, # Input, mesh[mxmesh].
        utils._EngineIntArrayType.c_type, # Input, ipmesh[mxmesh].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _mxmesh = utils._EngineIntScalarType(
        mxmesh, {'fun_name': fun_name, 'entity': 'mxmesh'},
    ).to_ctypes()
    _nmesh = utils._EngineIntScalarType(
        nmesh, {'fun_name': fun_name, 'entity': 'nmesh'},
    ).to_ctypes()
    _mxmesh_py = utils._EngineIntScalarType.to_py(_mxmesh)
    _mesh = utils._EngineFloat64ArrayType(
        mesh, {'fun_name': fun_name, 'entity': 'mesh'},
        exp_shape=(_mxmesh_py,),
    ).to_ctypes()
    _ipmesh = utils._EngineIntArrayType(
        ipmesh, {'fun_name': fun_name, 'entity': 'ipmesh'},
        exp_shape=(_mxmesh_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['rcomm'], {'fun_name': fun_name, 'entity': 'comm[\'rcomm\']'}, exp_rank=1, validate=False)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm = comm['rcomm']
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mxmesh,
        _nmesh,
        _mesh,
        _ipmesh,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The entries in mesh are not strictly increasing.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The last point of the new mesh does not coincide with the right',
                    'hand end of the range previously specified.',
                    'Last point of the new mesh: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'right-hand end of the range: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'You have set the element of ipmesh corresponding to the last element',
                    'of mesh to be included in the new mesh as {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'which is not 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Expected {:s} elements of ipmesh to be 1 or 2, but'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} such elements found.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An element of ipmesh was set to -1 before nmesh elements',
                    'containing 1 or 2 were detected.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'ipmesh[i] != -1, 1, 2 or 3 for some i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ipmesh[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ipmesh[0] = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The first element of array mesh does not coincide with the left-hand end',
                    'of the range previously specified.',
                    'First element of mesh: {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'left-hand of the range: {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nmesh = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mxmesh = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nmesh <= (mxmesh+1)/2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nmesh = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nmesh >= 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mxmesh = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mxmesh = {:s} in bvp_coll_nlin_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mxmesh = mxmesh in bvp_coll_nlin_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not produce any results suitable for remeshing.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function does not appear to have been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        return_data = None
    _err_class.manage()
    return return_data

def bvp_coll_nlin_interp(x, y, neq, mmax, comm):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, interpolation for ``bvp_coll_nlin_solve``.

    For full information please refer to the NAG Library document for
    d02ty

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02tyf.html

    Parameters
    ----------
    x : float
        x, the independent variable.

    y : float, shape (`neq`, `mmax`), modified in place
        On exit: `y`[i-1,j] contains an approximation to y_i^(j)(x), for
        j = 0,1,...,m_i-1, for i = 1,2,...,`neq`. The remaining elements
        of `y` (where m_i < `mmax`) are initialized to 0.0.

    neq : int
        The number of differential equations.

    mmax : int
        The maximal order of the differential equations, max(m_i), for i
        = 1,2,...,`neq`.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``bvp_coll_nlin_setup``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= *<value>*.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= *<value>*.

        (`errno` 1)
            On entry, `mmax` = *<value>* and max(m[i]) = *<value>* in
            ``bvp_coll_nlin_setup``.

            Constraint: `mmax` = max(m[i]) in ``bvp_coll_nlin_setup``.

        (`errno` 1)
            The solver function did not produce any results suitable for
            interpolation.

        (`errno` 1)
            The solver function does not appear to have been called.

        (`errno` 1)
            On entry, `neq` = *<value>* and neq = *<value>* in
            ``bvp_coll_nlin_setup``.

            Constraint: `neq` = neq in ``bvp_coll_nlin_setup``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The solver function did not satisfy the error requirements.

            Interpolated values should be treated with caution.

        (`errno` 2)
            The solver function did not converge to a suitable solution.

            A converged intermediate solution has been used.

            Interpolated values should be treated with caution.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nlin_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02tyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Output, y[neq, mmax].
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_INT, # Input, mmax.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((y, 'y'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _mmax = utils._EngineIntScalarType(
        mmax, {'fun_name': fun_name, 'entity': 'mmax'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _mmax_py = utils._EngineIntScalarType.to_py(_mmax)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_neq_py, _mmax_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['rcomm'], {'fun_name': fun_name, 'entity': 'comm[\'rcomm\']'}, exp_rank=1, validate=False)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm = comm['rcomm']
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _neq,
        _mmax,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mmax = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'max(m[i]) = {:s} in bvp_coll_nlin_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mmax = max(m[i]) in bvp_coll_nlin_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not produce any results suitable for',
                    'interpolation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function does not appear to have been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and neq = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'bvp_coll_nlin_setup.',
                    'Constraint: neq = neq in bvp_coll_nlin_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not satisfy the error requirements.',
                    'Interpolated values should be treated with caution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not converge to a suitable solution.',
                    'A converged intermediate solution has been used.',
                    'Interpolated values should be treated with caution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _y_cls.refresh_py(_y)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_coll_nlin_diag(mxmesh, mesh, ipmesh, comm):
    u"""
    Ordinary differential equations, general nonlinear boundary value
    problem, diagnostics for ``bvp_coll_nlin_solve``.

    For full information please refer to the NAG Library document for
    d02tz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02tzf.html

    Parameters
    ----------
    mxmesh : int
        The maximum number of points allowed in the mesh.

    mesh : float, shape (`mxmesh`), modified in place
        On exit: `mesh`[i-1] contains the ith point of the mesh last
        used by ``bvp_coll_nlin_solve``, for i = 1,2,...,`nmesh`.
        `mesh`[0] will contain a and `mesh`[`nmesh`-1] will contain b.
        The remaining elements of `mesh` are not initialized.

    ipmesh : int, shape (`mxmesh`), modified in place
        On exit: `ipmesh`[i-1] specifies the nature of the point
        `mesh`[i-1], for i = 1,2,...,`nmesh`, in the final mesh computed
        by ``bvp_coll_nlin_solve``.

        `ipmesh`[i-1] = 1

          Indicates that the ith point is a fixed point and was used by
          the solver before an extrapolation-like error test.

        `ipmesh`[i-1] = 2

          Indicates that the ith point was used by the solver before an
          extrapolation-like error test.

        `ipmesh`[i-1] = 3

          Indicates that the ith point was used by the solver only as
          part of an extrapolation-like error test.

        The remaining elements of `ipmesh` are initialized to -1.

        See [Further Comments] for advice on how these values may be
        used in conjunction with a continuation process.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``bvp_coll_nlin_setup``.

    Returns
    -------
    nmesh : int
        The number of points in the mesh last used by
        ``bvp_coll_nlin_solve``.

    ermx : float
        An estimate of the maximum error in the solution computed by
        ``bvp_coll_nlin_solve``, that is

            `ermx` = max(||y_i-v_i||/((1.0+||v_i||)))

        where v_i is the approximate solution for the ith solution
        component. If ``bvp_coll_nlin_solve`` returned successfully with
        no exception or warning is raised, `ermx` will be less than
        tols[`ijermx`-1] in ``bvp_coll_nlin_setup`` where tols contains
        the error requirements as specified in [Description] and
        [Parameters].

        If ``bvp_coll_nlin_solve`` returned with `errno` = 5, `ermx`
        will be greater than tols[`ijermx`-1] in
        ``bvp_coll_nlin_setup``.

        If ``bvp_coll_nlin_solve`` returned any other value for errno
        then an error estimate is not available and `ermx` is
        initialized to 0.0.

    iermx : int
        Indicates the mesh sub-interval where the value of `ermx` has
        been computed, that is [`mesh`[ `iermx` -1],`mesh`[ `iermx` + 1
        -1]].

        If an estimate of the error is not available then `iermx` is
        initialized to 0.

    ijermx : int
        Indicates the component i (= `ijermx`) of the solution for which
        `ermx` has been computed, that is the approximation of y_i on
        [`mesh`[`iermx`-1],`mesh`[`iermx`]] is estimated to have the
        largest error of all components y_i over mesh sub-intervals
        defined by `mesh`.

        If an estimate of the error is not available then `ijermx` is
        initialized to 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The solver function did not produce any results suitable for
            interpolation.

        (`errno` 1)
            The solver function does not appear to have been called.

        (`errno` 1)
            On entry, `mxmesh` = *<value>* and mxmesh = *<value>* in
            ``bvp_coll_nlin_setup``.

            Constraint: `mxmesh` = mxmesh in ``bvp_coll_nlin_setup``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The solver function did not satisfy the error requirements.

            Information has been supplied on the last mesh used.

        (`errno` 2)
            The solver function did not converge to a suitable solution.

            A converged intermediate solution has been used.

            Error estimate information is not available.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_coll_nlin_diag'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02tzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mxmesh.
        utils._BYREF_INT, # Output, nmesh.
        utils._EngineFloat64ArrayType.c_type, # Output, mesh[mxmesh].
        utils._EngineIntArrayType.c_type, # Output, ipmesh[mxmesh].
        utils._BYREF_FLOAT64, # Output, ermx.
        utils._BYREF_INT, # Output, iermx.
        utils._BYREF_INT, # Output, ijermx.
        utils._EngineFloat64ArrayType.c_type, # Input, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _mxmesh = utils._EngineIntScalarType(
        mxmesh, {'fun_name': fun_name, 'entity': 'mxmesh'},
    ).to_ctypes()
    _nmesh = utils._EngineIntScalarType.empty_ctypes()
    _mxmesh_py = utils._EngineIntScalarType.to_py(_mxmesh)
    _mesh_cls = utils._EngineFloat64ArrayType(
        mesh, {'fun_name': fun_name, 'entity': 'mesh'},
        exp_shape=(_mxmesh_py,),
        check_contents=False,
    )
    _mesh = _mesh_cls.to_ctypes(can_reference_contents=False)
    _ipmesh_cls = utils._EngineIntArrayType(
        ipmesh, {'fun_name': fun_name, 'entity': 'ipmesh'},
        exp_shape=(_mxmesh_py,),
        check_contents=False,
    )
    _ipmesh = _ipmesh_cls.to_ctypes(can_reference_contents=False)
    _ermx = utils._EngineFloat64ScalarType.empty_ctypes()
    _iermx = utils._EngineIntScalarType.empty_ctypes()
    _ijermx = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(comm['rcomm'], {'fun_name': fun_name, 'entity': 'comm[\'rcomm\']'}, exp_rank=1, validate=False)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm = comm['rcomm']
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mxmesh,
        _nmesh,
        _mesh,
        _ipmesh,
        _ermx,
        _iermx,
        _ijermx,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not produce any results suitable for',
                    'interpolation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function does not appear to have been called.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mxmesh = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mxmesh = {:s} in bvp_coll_nlin_setup.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mxmesh = mxmesh in bvp_coll_nlin_setup.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not satisfy the error requirements.',
                    'Information has been supplied on the last mesh used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solver function did not converge to a suitable solution.',
                    'A converged intermediate solution has been used.',
                    'Error estimate information is not available.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _mesh_cls.refresh_py(_mesh)
        _ipmesh_cls.refresh_py(_ipmesh)
        _nmesh_py = utils._EngineIntScalarType.to_py(_nmesh)
        _ermx_py = utils._EngineFloat64ScalarType.to_py(_ermx)
        _iermx_py = utils._EngineIntScalarType.to_py(_iermx)
        _ijermx_py = utils._EngineIntScalarType.to_py(_ijermx)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nmesh', 'ermx', 'iermx', 'ijermx'],
        )(
            _nmesh_py,
            _ermx_py,
            _iermx_py,
            _ijermx_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def bvp_ps_lin_coeffs(n, f, c):
    u"""
    Coefficients of Chebyshev interpolating polynomial from function
    values on Chebyshev grid.

    For full information please refer to the NAG Library document for
    d02ua

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02uaf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1. This is also the
        largest order of Chebyshev polynomial in the Chebyshev series to
        be computed.

    f : float, shape (`n`+1)
        The function values f(x_r), for r = 1,2,...,n+1.

    c : float, shape (`n`+1), modified in place
        On exit: the Chebyshev coefficients, c_j, for j = 1,2,...,n+1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_coeffs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02uaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, f[n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py+1,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _f,
        _c,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_cgl_vals(n, a, b, q, c, f):
    u"""
    Function or low-order-derivative values on Chebyshev grid from
    coefficients of Chebyshev interpolating polynomial.

    For full information please refer to the NAG Library document for
    d02ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ubf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1. This is also the
        largest order of Chebyshev polynomial in the Chebyshev series to
        be computed.

    a : float
        a, the lower bound of domain [a,b].

    b : float
        b, the upper bound of domain [a,b].

    q : int
        The order, q, of the derivative to evaluate.

    c : float, shape (`n`+1)
        The Chebyshev coefficients, c_i, for i = 1,2,...,n+1.

    f : float, shape (`n`+1), modified in place
        On exit: the derivatives S^(q)x_i, for i = 1,2,...,n+1, of the
        Chebyshev series, S.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` < `b`.

        (`errno` 3)
            On entry, `q` = *<value>*.

            Constraint: 0 <= `q` <= 4.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_cgl_vals'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02ubft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, q.
        utils._EngineFloat64ArrayType.c_type, # Input, c[n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, f[n+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _q = utils._EngineIntScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py+1,),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _q,
        _c,
        _f,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a < b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= q <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _f_cls.refresh_py(_f)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_cgl_grid(n, a, b, x):
    u"""
    Chebyshev Gauss--Lobatto grid generation.

    For full information please refer to the NAG Library document for
    d02uc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02ucf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1. This is also the
        largest order of Chebyshev polynomial in the Chebyshev series to
        be computed.

    a : float
        a, the lower bound of domain [a,b].

    b : float
        b, the upper bound of domain [a,b].

    x : float, shape (`n`+1), modified in place
        On exit: the Chebyshev Gauss--Lobatto grid points, x_i, for i =
        1,2,...,n+1, on [a,b].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` < `b`.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_cgl_grid'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02ucft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._EngineFloat64ArrayType.c_type, # Output, x[n+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py+1,),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a < b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_cgl_deriv(n, f, fd):
    u"""
    Differentiate a function by the FFT using function values on
    Chebyshev grid.

    For full information please refer to the NAG Library document for
    d02ud

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02udf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1.

    f : float, shape (`n`+1)
        The function values f(x_i), for i = 1,2,...,n+1

    fd : float, shape (`n`+1), modified in place
        On exit: the approximations to the derivatives of the function
        evaluated at the Chebyshev Gauss--Lobatto points. For functions
        defined on [a,b], the returned derivative values (corresponding
        to the domain [-1,1]) must be multiplied by the factor 2/(b-a)
        to obtain the correct values on [a,b].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_cgl_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02udft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, f[n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, fd[n+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _fd_cls = utils._EngineFloat64ArrayType(
        fd, {'fun_name': fun_name, 'entity': 'fd'},
        exp_shape=(_n_py+1,),
        check_contents=False,
    )
    _fd = _fd_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _f,
        _fd,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fd_cls.refresh_py(_fd)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_solve(n, a, b, m, c, bmat, y, bvec, f, uc):
    u"""
    Solve linear constant coefficient boundary value problem on
    Chebyshev grid, Integral formulation.

    For full information please refer to the NAG Library document for
    d02ue

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02uef.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1.

    a : float
        a, the lower bound of domain [a,b].

    b : float
        b, the upper bound of domain [a,b].

    m : int
        The order, m, of the boundary value problem to be solved.

    c : float, shape (`n`+1)
        The Chebyshev coefficients c_j, j = 0,1,...,n, for the
        right-hand side of the boundary value problem. Usually these are
        obtained by a previous call of ``bvp_ps_lin_coeffs``.

    bmat : float, shape (`m`, `m`+1), modified in place
        On entry: `bmat`[i-1,j] must contain the coefficients B_{i,j+1},
        for j = 0,1,...,m, for i = 1,2,...,m, in the problem formulation
        of [Description].

        On exit: the coefficients have been scaled to form an equivalent
        problem defined on the domain [-1,1].

    y : float, shape (`m`)
        The points, y_i, for i = 1,2,...,m, where the boundary
        conditions are discretized.

    bvec : float, shape (`m`)
        The values, beta_i, for i = 1,2,...,m, in the formulation of the
        boundary conditions given in [Description].

    f : float, shape (`m`+1), modified in place
        On entry: the coefficients, f_j, for j = 1,2,...,m+1, in the
        formulation of the linear boundary value problem given in
        [Description]. The highest order term, `f`[`m`], needs to be
        nonzero to have a well posed problem.

        On exit: the coefficients have been scaled to form an equivalent
        problem defined on the domain [-1,1].

    uc : float, shape (`n`+1, `m`+1), modified in place
        On exit: the Chebyshev coefficients in the Chebyshev series
        representations of the solution and derivatives of the solution
        to the boundary value problem. The n+1 elements `uc`[0:`n`+1,0]
        contain the coefficients representing the solution U(x_i), for i
        = 0,1,...,n. `uc`[0:`n`+1,j] contains the coefficients
        representing the jth derivative of U, for j = 1,2,...,m.

    Returns
    -------
    resid : float
        The maximum residual resulting from substituting the solution
        vectors returned in `uc` into the equations representing the
        linear boundary value problem and associated boundary
        conditions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 8.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` < `b`.

        (`errno` 3)
            On entry, `f`[`m`] = 0.0.

        (`errno` 6)
            On entry, `m` = *<value>*.

            Constraint: 1 <= `m` <= 4.

        (`errno` 7)
            Internal error while unpacking matrix during iterative
            refinement.

            Please contact NAG.

        (`errno` 8)
            Singular matrix encountered during iterative refinement.

            Please check that your system is well posed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 9)
            During iterative refinement, the maximum number of
            iterations was reached.

            Number of iterations = *<value>* and residual achieved =
            *<value>*.

        (`errno` 10)
            During iterative refinement, convergence was achieved, but
            the residual is more than 100*machine precision. Residual
            achieved on convergence = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_solve'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02ueft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, c[n+1].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, bmat[m, m+1].
        utils._EngineFloat64ArrayType.c_type, # Input, y[m].
        utils._EngineFloat64ArrayType.c_type, # Input, bvec[m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, f[m+1].
        utils._EngineFloat64ArrayType.c_type, # Output, uc[n+1, m+1].
        utils._BYREF_FLOAT64, # Output, resid.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((bmat, 'bmat'), (uc, 'uc')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _bmat_cls = utils._EngineFloat64ArrayType(
        bmat, {'fun_name': fun_name, 'entity': 'bmat'},
        exp_shape=(_m_py, _m_py+1),
        mutual_sorder=_m_sorder,
    )
    _bmat = _bmat_cls.to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _bvec = utils._EngineFloat64ArrayType(
        bvec, {'fun_name': fun_name, 'entity': 'bvec'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_m_py+1,),
    )
    _f = _f_cls.to_ctypes()
    _uc_cls = utils._EngineFloat64ArrayType(
        uc, {'fun_name': fun_name, 'entity': 'uc'},
        exp_shape=(_n_py+1, _m_py+1),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _uc = _uc_cls.to_ctypes(can_reference_contents=False)
    _resid = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _m,
        _c,
        _bmat,
        _y,
        _bvec,
        _f,
        _uc,
        _resid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a < b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, f[m] = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= m <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= m <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal error while unpacking matrix during iterative refinement.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Singular matrix encountered during iterative refinement.',
                    'Please check that your system is well posed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 1004:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'During iterative refinement, the maximum number of iterations was reached.',
                    'Number of iterations = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'residual achieved = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 1005:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'During iterative refinement, convergence was achieved,',
                    'but the residual is more than 100*machine precision.',
                    'Residual achieved on convergence = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _bmat_cls.refresh_py(_bmat)
        _f_cls.refresh_py(_f)
        _uc_cls.refresh_py(_uc)
        _resid_py = utils._EngineFloat64ScalarType.to_py(_resid)
        return_data = _resid_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['resid'],
            )(
                _resid_py,
            )
    _err_class.manage()
    return return_data

def bvp_ps_lin_grid_vals(n, nip, x, f, xip, fip):
    u"""
    Interpolate a function from Chebyshev grid to uniform grid using
    barycentric Lagrange interpolation.

    For full information please refer to the NAG Library document for
    d02uw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02uwf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points for the input data is n+1.

    nip : int
        The number, m, of grid points in the uniform mesh x^ onto which
        function values are interpolated. If `nip` = 1 then on
        successful exit from ``bvp_ps_lin_grid_vals``, `fip`[0] will
        contain the value f(x_n).

    x : float, shape (`n`+1)
        The grid points, x_i, for i = 0,1,...,n, at which the function
        is specified.

        Usually this should be the array of Chebyshev Gauss--Lobatto
        points returned in ``bvp_ps_lin_cgl_grid``.

    f : float, shape (`n`+1)
        The function values, f(x_i), for i = 0,1,...,n.

    xip : float, shape (`nip`), modified in place
        On exit: the evenly-spaced grid points, x^_j, for j = 1,2,...,m.

    fip : float, shape (`nip`), modified in place
        On exit: the set of interpolated values f^(x^_j), for j =
        1,2,...,m. Here f^(x^_j)≈f(x = x^_j).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.

        (`errno` 2)
            On entry, `nip` = *<value>*.

            Constraint: `nip` > 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_grid_vals'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02uwft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nip.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n+1].
        utils._EngineFloat64ArrayType.c_type, # Input, f[n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, xip[nip].
        utils._EngineFloat64ArrayType.c_type, # Output, fip[nip].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nip = utils._EngineIntScalarType(
        nip, {'fun_name': fun_name, 'entity': 'nip'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _nip_py = utils._EngineIntScalarType.to_py(_nip)
    _xip_cls = utils._EngineFloat64ArrayType(
        xip, {'fun_name': fun_name, 'entity': 'xip'},
        exp_shape=(_nip_py,),
        check_contents=False,
    )
    _xip = _xip_cls.to_ctypes(can_reference_contents=False)
    _fip_cls = utils._EngineFloat64ArrayType(
        fip, {'fun_name': fun_name, 'entity': 'fip'},
        exp_shape=(_nip_py,),
        check_contents=False,
    )
    _fip = _fip_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _nip,
        _x,
        _f,
        _xip,
        _fip,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nip > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _xip_cls.refresh_py(_xip)
        _fip_cls.refresh_py(_fip)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_quad_weights(n, w):
    u"""
    Clenshaw--Curtis quadrature weights for integration using computed
    Chebyshev coefficients.

    For full information please refer to the NAG Library document for
    d02uy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02uyf.html

    Parameters
    ----------
    n : int
        n, where the number of grid points is n+1.

    w : float, shape (`n`+1), modified in place
        On exit: the Clenshaw--Curtis quadrature weights, w_i, for i =
        0,1,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` is even.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_quad_weights'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02uyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, w[n+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py+1,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _w,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n is even.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _w_cls.refresh_py(_w)
        return_data = None
    _err_class.manage()
    return return_data

def bvp_ps_lin_cheb_eval(k, x):
    u"""
    Chebyshev polynomial evaluation, T_k(x).

    For full information please refer to the NAG Library document for
    d02uz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02uzf.html

    Parameters
    ----------
    k : int
        The order of the Chebyshev polynomial.

    x : float
        The point at which to evaluate the polynomial.

    Returns
    -------
    t : float
        The value, T, of the Chebyshev polynomial order k evaluated at
        x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: -1.0 <= `x` <= 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.bvp_ps_lin_cheb_eval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02uzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, t.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _k,
        _x,
        _t,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -1.0 <= x <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _t_py = utils._EngineFloat64ScalarType.to_py(_t)
        return_data = _t_py
    _err_class.manage()
    return return_data

def ivp_stiff_nat_interp(xsol, sol, m, comm, ldysav, sdysav, neq, x, nqu, hu, h):
    u"""
    Ordinary differential equations, initial value problem,
    interpolation for d02m--n routines (BLEND and BDF methods only),
    natural interpolant.

    For full information please refer to the NAG Library document for
    d02xj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02xjf.html

    Parameters
    ----------
    xsol : float
        The point at which the first m components of the solution are to
        be evaluated. `xsol` should not be an extrapolation point, that
        is `xsol` should satisfy (`xsol`-`x`)*`hu` <= 0.0. Extrapolation
        is permitted but not recommended.

    sol : float, shape (`m`), modified in place
        On exit: the calculated value of the ith component of the
        solution at `xsol`, for i = 1,2,...,m.

    m : int
        m, the number of components of the solution whose values at
        `xsol` are required. The first `m` components are evaluated.

    comm : dict, communication object
        Communication structure populated by a previous call to the
        integrator.

    ldysav : int
        The value used for the argument `ldysav` when calling the
        integrator.

    sdysav : int
        The value used for the argument `sdysav` when calling the
        integrator.

    neq : int
        The value used for the argument `neq` when calling the
        integrator.

    x : float
        The latest value at which the solution has been computed, as
        provided in the argument tcur on return from the optional output
        ``ivp_stiff_integ_diag``.

    nqu : int
        The order of the method used up to the latest value at which the
        solution has been computed, as provided in the argument nqu on
        return from the optional output ``ivp_stiff_integ_diag``.

    hu : float
        The last successful step used, that is the step used in the
        integration to get to `x`, as provided in the argument hu on
        return from the optional output ``ivp_stiff_integ_diag``.

    h : float
        The next step size to be attempted in the integration, as
        provided in the argument h on return from the optional output
        ``ivp_stiff_integ_diag``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `sdysav` = *<value>* and `nqu` = *<value>*.

            Constraint: `sdysav` >= `nqu`+1.

        (`errno` 1)
            On entry, `nqu` = *<value>*.

            Constraint: `nqu` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `m` = *<value>* and `neq` = *<value>*.

            Constraint: `m` <= `neq`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            On entry, `ldysav` = *<value>*.

            Constraint: `ldysav` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `hu` = 0.0.

        (`errno` 2)
            On entry, `h` = 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            On entry, `xsol` = *<value>*, `x` = *<value>* and `h` =
            *<value>*.

            Constraint: `xsol` should not be an extrapolation point.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_nat_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02xjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xsol.
        utils._EngineFloat64ArrayType.c_type, # Output, sol[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, nqu.
        utils._BYREF_FLOAT64, # Input, hu.
        utils._BYREF_FLOAT64, # Input, h.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _xsol = utils._EngineFloat64ScalarType(
        xsol, {'fun_name': fun_name, 'entity': 'xsol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _sol_cls = utils._EngineFloat64ArrayType(
        sol, {'fun_name': fun_name, 'entity': 'sol'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _sol = _sol_cls.to_ctypes(can_reference_contents=False)
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('ysav',),
    )
    _ysav = comm['ysav']
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _nqu = utils._EngineIntScalarType(
        nqu, {'fun_name': fun_name, 'entity': 'nqu'},
    ).to_ctypes()
    _hu = utils._EngineFloat64ScalarType(
        hu, {'fun_name': fun_name, 'entity': 'hu'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xsol,
        _sol,
        _m,
        _ysav,
        _ldysav,
        _sdysav,
        _neq,
        _x,
        _nqu,
        _hu,
        _h,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sdysav = {:s} and nqu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdysav >= nqu+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nqu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nqu >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldysav >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, hu = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, h = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xsol = {:s}, x = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: xsol should not be an extrapolation point.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _sol_cls.refresh_py(_sol)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_c1_interp(xsol, sol, m, comm, ldysav, sdysav, acor, neq, x, nqu, hu, h):
    u"""
    Ordinary differential equations, initial value problem,
    interpolation for d02m--n functions, C^1 interpolant.

    For full information please refer to the NAG Library document for
    d02xk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02xkf.html

    Parameters
    ----------
    xsol : float
        The point at which the first m components of the solution are to
        be evaluated. `xsol` should not be an extrapolation point, that
        is `xsol` should satisfy (`xsol`-`x`)*`hu` <= 0.0. Extrapolation
        is permitted but not recommended.

    sol : float, shape (`m`), modified in place
        On exit: the calculated value of the ith component of the
        solution at `xsol`, for i = 1,2,...,m.

    m : int
        The number of components of the solution whose values at `xsol`
        are required. The first m components are evaluated.

    comm : dict, communication object
        Communication structure populated by a previous call to the
        integrator.

    ldysav : int
        The value used for the argument `ldysav` when calling the
        integrator.

    sdysav : int
        The value used for the argument `sdysav` when calling the
        integrator.

    acor : float, shape (`neq`)
        The value returned in position (`ldysav`+50+i), for i =
        1,2,...,`neq`, of the argument rwork returned by the integrator.
        If one of the direct communication D02N functions is being
        employed and ``ivp_stiff_c1_interp`` is to be used in monitr,
        acor[i-1] must contain the value given in position (i,2) of the
        monitr argument acor, for i = 1,2,...,`neq` (e.g., see
        ``ivp_stiff_exp_fulljac``).

    neq : int
        The value used for the argument `neq` when calling the
        integrator.

    x : float
        The latest value at which the solution has been computed, as
        provided in the argument tcur on return from the optional output
        ``ivp_stiff_integ_diag``.

    nqu : int
        The order of the method used up to the latest value at which the
        solution has been computed, as provided in the argument nqu on
        return from the optional output ``ivp_stiff_integ_diag``.

    hu : float
        The last successful step used, that is the step used in the
        integration to get to `x`, as provided in the argument hu on
        return from the optional output ``ivp_stiff_integ_diag``.

    h : float
        The next step size to be attempted in the integration, as
        provided in the argument h on return from the optional output
        ``ivp_stiff_integ_diag``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The Petzold error test was selected.

        (`errno` 1)
            The BDF integration method is not being used.

        (`errno` 1)
            On entry, `sdysav` = *<value>* and `nqu` = *<value>*.

            Constraint: `sdysav` >= `nqu`+1.

        (`errno` 1)
            On entry, `nqu` = *<value>*.

            Constraint: `nqu` >= 1.

        (`errno` 1)
            On entry, `neq` = *<value>* and `ldysav` = *<value>*.

            Constraint: `neq` <= `ldysav`.

        (`errno` 1)
            On entry, `m` = *<value>* and `neq` = *<value>*.

            Constraint: `m` <= `neq`.

        (`errno` 1)
            On entry, `neq` = *<value>*.

            Constraint: `neq` >= 1.

        (`errno` 1)
            On entry, `ldysav` = *<value>*.

            Constraint: `ldysav` >= 1.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `hu` = 0.0.

        (`errno` 2)
            On entry, `h` = 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Extrapolation has been performed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_c1_interp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02xkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xsol.
        utils._EngineFloat64ArrayType.c_type, # Output, sol[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineFloat64ArrayType.c_type, # Input, ysav[ldysav*sdysav].
        utils._BYREF_INT, # Input, ldysav.
        utils._BYREF_INT, # Input, sdysav.
        utils._EngineFloat64ArrayType.c_type, # Input, acor[neq].
        utils._BYREF_INT, # Input, neq.
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, nqu.
        utils._BYREF_FLOAT64, # Input, hu.
        utils._BYREF_FLOAT64, # Input, h.
        utils._EngineCharArrayType.c_type, # Input, cwsav(len=80)[10].
        utils._EngineLogicalArrayType.c_type, # Input, lwsav[100].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input, rwsav[1100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cwsav.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _xsol = utils._EngineFloat64ScalarType(
        xsol, {'fun_name': fun_name, 'entity': 'xsol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _sol_cls = utils._EngineFloat64ArrayType(
        sol, {'fun_name': fun_name, 'entity': 'sol'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _sol = _sol_cls.to_ctypes(can_reference_contents=False)
    _ldysav = utils._EngineIntScalarType(
        ldysav, {'fun_name': fun_name, 'entity': 'ldysav'},
    ).to_ctypes()
    _sdysav = utils._EngineIntScalarType(
        sdysav, {'fun_name': fun_name, 'entity': 'sdysav'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('ysav', 'cwsav', 'lwsav', 'iwsav', 'rwsav'),
    )
    _ysav = comm['ysav']
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _acor = utils._EngineFloat64ArrayType(
        acor, {'fun_name': fun_name, 'entity': 'acor'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _nqu = utils._EngineIntScalarType(
        nqu, {'fun_name': fun_name, 'entity': 'nqu'},
    ).to_ctypes()
    _hu = utils._EngineFloat64ScalarType(
        hu, {'fun_name': fun_name, 'entity': 'hu'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _cwsav = comm['cwsav']
    _lwsav = comm['lwsav']
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xsol,
        _sol,
        _m,
        _ysav,
        _ldysav,
        _sdysav,
        _acor,
        _neq,
        _x,
        _nqu,
        _hu,
        _h,
        _cwsav,
        _lwsav,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        80,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Petzold error test was selected.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The BDF integration method is not being used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sdysav = {:s} and nqu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sdysav >= nqu+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nqu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nqu >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s} and ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: neq <= ldysav.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= neq.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, neq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: neq >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldysav = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldysav >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, hu = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, h = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extrapolation has been performed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _sol_cls.refresh_py(_sol)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def ivp_stiff_errest(neq, v, w, comm):
    u"""
    Ordinary differential equations, initial value problem, weighted
    norm of local error estimate for d02m--n functions.

    For full information please refer to the NAG Library document for
    d02za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d02/d02zaf.html

    Parameters
    ----------
    neq : int
        The number of differential equations, as defined for the
        integrator being used.

    v : float, shape (`neq`)
        The vector, the weighted norm of which is to be evaluated by
        ``ivp_stiff_errest``. `v` is calculated internally by the
        integrator being used.

    w : float, shape (`neq`)
        The weights, calculated internally by the integrator, to be used
        in the norm evaluation.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by prior calls to
        ``ivp_stiff_fulljac_setup`` and one of ``ivp_stiff_dassl``,
        ``ivp_stiff_bdf`` or ``ivp_stiff_blend``.

    Returns
    -------
    errnrm : float
        ||v||, the norm of the scaled local error estimate.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The value of the norm would either overflow or is close to
            overflowing.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.ode.ivp_stiff_errest'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d02zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, neq.
        utils._EngineFloat64ArrayType.c_type, # Input, v[neq].
        utils._EngineFloat64ArrayType.c_type, # Input, w[neq].
        utils._EngineIntArrayType.c_type, # Input, iwsav[505].
        utils._EngineFloat64ArrayType.c_type, # Input, rwsav[1100].
        utils._BYREF_FLOAT64, # Output, errnrm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _neq = utils._EngineIntScalarType(
        neq, {'fun_name': fun_name, 'entity': 'neq'},
    ).to_ctypes()
    _neq_py = utils._EngineIntScalarType.to_py(_neq)
    _v = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_neq_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iwsav', 'rwsav'),
    )
    _iwsav = comm['iwsav']
    _rwsav = comm['rwsav']
    _errnrm = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _neq,
        _v,
        _w,
        _iwsav,
        _rwsav,
        _errnrm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of the norm would either overflow or is close to overflowing.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _errnrm_py = utils._EngineFloat64ScalarType.to_py(_errnrm)
        return_data = _errnrm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['errnrm'],
            )(
                _errnrm_py,
            )
    _err_class.manage()
    return return_data
