# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `quad` Chapter.

``quad`` - Quadrature

This module provides functions for the numerical evaluation of definite
integrals in one or more dimensions and for evaluating weights and
abscissae of integration rules.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01intro.html

See Also
--------
``naginterfaces.library.examples.quad`` :
    This subpackage contains examples for the ``quad`` module.
    See also the :ref:`library_quad_ex` subsection.
"""

# NAG Copyright 2017-2019.

from __future__ import absolute_import

import collections as _collections

from math import ceil

from ..base import utils as _b_utils
from ..library import _utils

def dim1_fin_well(a, b, epsr, f, nlimit, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Patterson, suitable for well-behaved integrands.

    For full information please refer to the NAG Library document for
    d01ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ahf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsr : float
        The relative accuracy required.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    nlimit : int
        A limit to the number of function evaluations. If `nlimit` <= 0,
        the function uses a default limit of 10000.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    din : float
        The value of the definite interval.

    npts : int
        The number of function evaluations used in the calculation of
        the integral.

    relerr : float
        A rough estimate of the relative error achieved.

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, `epsr` = *<value>*.

            Constraint: `epsr` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Integral has failed to converge to requested accuracy.

        (`errno` 2)
            Too many unsuccessful levels of subdivision.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import dim1_fin_well as b_dim1_fin_well

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_fin_well'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_dim1_fin_well(
            a,
            b,
            epsr,
            _f,
            nlimit,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _din_py = _return_scalars.din
    _npts_py = _return_scalars.npts
    _relerr_py = _return_scalars.relerr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['din', 'npts', 'relerr'],
    )(
        _din_py,
        _npts_py,
        _relerr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_bad(f, a, b, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Piessens and de Doncker, allowing for badly behaved integrands.

    For full information please refer to the NAG Library document for
    d01aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ajf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_bad as b_dim1_fin_bad

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_fin_bad'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_bad(
            _f,
            a,
            b,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_osc(f, a, b, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, method
    suitable for oscillating functions.

    For full information please refer to the NAG Library document for
    d01ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01akf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 4)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_osc as b_dim1_fin_osc

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_fin_osc'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_osc(
            _f,
            a,
            b,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_sing(f, a, b, points, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, allowing for
    singularities at user-specified break-points.

    For full information please refer to the NAG Library document for
    d01al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01alf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    points : float, array-like, shape (npts)
        The user-specified break-points.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed (`lw`-2*npts-4)/4. The more
        difficult the integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/2.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed (`liw`-npts-2)/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, LIMIT = *<value>* and npts = *<value>*.

            Constraint: LIMIT > npts.

        (`errno` 6)
            On entry, break-points are specified outside (`a`,`b`): `a`
            = *<value>* and `b` = *<value>*.

        (`errno` 6)
            On entry, npts = *<value>*.

            Constraint: npts >= 0.

        (`errno` 7)
            On entry, `liw` is too small. `liw` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 7)
            On entry, `lw` is too small. `lw` = *<value>*. Minimum
            possible dimension: *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>* `lw` = *<value>* `liw` = *<value>* npts =
            *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_sing as b_dim1_fin_sing

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_fin_sing'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _points = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'points'},
        points,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _npts_py = _b_utils._EngineFloat64ArrayType.get_shape(_points, {'fun_name': fun_name, 'entity': 'points'}, exp_rank=1)[0]
    _npts = _b_utils._EngineIntScalarType(
        _npts_py, {'fun_name': fun_name, 'entity': '_npts'},
    ).to_ctypes()
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//2 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_sing(
            _f,
            a,
            b,
            _npts,
            _points,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_inf(f, bound, inf, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, infinite or semi-infinite
    interval.

    For full information please refer to the NAG Library document for
    d01am

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01amf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    bound : float
        The finite limit of the integration range (if present). `bound`
        is not used if the interval is doubly infinite.

    inf : int
        Indicates the kind of integration range.

        `inf` = 1

          The range is [`bound`, + infinity).

        `inf` = -1

          The range is (- infinity,`bound`].

        `inf` = 2

          The range is (- infinity, + infinity).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `inf` = *<value>*.

            Constraint: `inf` = -1, 1 or 2.

        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around one of the
            sub-intervals (*<value>*,*<value>*) or
            (*<value>*,*<value>*).

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_inf as b_dim1_inf

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_inf'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_inf(
            _f,
            bound,
            inf,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_wtrig(g, a, b, omega, key, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function cos(omega x) or sin(omega x).

    For full information please refer to the NAG Library document for
    d01an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01anf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    omega : float
        The argument omega in the weight function of the transform.

    key : int
        Indicates which integral is to be computed.

        `key` = 1

          w(x) = cos(omega x).

        `key` = 2

          w(x) = sin(omega x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/2.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1 and `key` <= 2.

        (`errno` 7)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 7)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_wtrig as b_dim1_fin_wtrig

    def cb_g(x, data=None):
        """Wrapping user's g. Called as base g."""
        if data is not None:
            _retval_g = g(
                x,
                data,
            )
        else:
            _retval_g = g(
                x,
            )
        return _retval_g

    fun_name = 'naginterfaces.library.quad.dim1_fin_wtrig'

    _b_utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = cb_g
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//2 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_wtrig(
            _g,
            a,
            b,
            omega,
            key,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_wsing(g, a, b, alfa, beta, key, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function with end-point singularities of algebraico-logarithmic
    type.

    For full information please refer to the NAG Library document for
    d01ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01apf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration.

    alfa : float
        The argument alpha in the weight function.

    beta : float
        The argument beta in the weight function.

    key : int
        Indicates which weight function is to be used.

        `key` = 1

          w(x) = (x-a)^alpha(b-x)^beta.

        `key` = 2

          w(x) = (x-a)^alpha(b-x)^beta ln(x-a).

        `key` = 3

          w(x) = (x-a)^alpha(b-x)^beta ln(b-x).

        `key` = 4

          w(x) = (x-a)^alpha(b-x)^beta ln(x-a)ln(b-x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` <= 4.

        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1.

        (`errno` 4)
            On entry, `beta` = *<value>*.

            Constraint: `beta` > -1.0.

        (`errno` 4)
            On entry, `alfa` = *<value>*.

            Constraint: `alfa` > -1.0.

        (`errno` 4)
            On entry, `b` = *<value>* and `a` = *<value>*.

            Constraint: `b` > `a`.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 8.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_wsing as b_dim1_fin_wsing

    def cb_g(x, data=None):
        """Wrapping user's g. Called as base g."""
        if data is not None:
            _retval_g = g(
                x,
                data,
            )
        else:
            _retval_g = g(
                x,
            )
        return _retval_g

    fun_name = 'naginterfaces.library.quad.dim1_fin_wsing'

    _b_utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = cb_g
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_wsing(
            _g,
            a,
            b,
            alfa,
            beta,
            key,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_wcauchy(g, a, b, c, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function 1/(x-c), Cauchy principal value (Hilbert transform).

    For full information please refer to the NAG Library document for
    d01aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01aqf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    c : float
        The argument c in the weight function.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `a` = *<value>*, `b` = *<value>* and `c` =
            *<value>*.

            Constraint: `c` != `a` and `c` != `b`.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_fin_wcauchy as b_dim1_fin_wcauchy

    def cb_g(x, data=None):
        """Wrapping user's g. Called as base g."""
        if data is not None:
            _retval_g = g(
                x,
                data,
            )
        else:
            _retval_g = g(
                x,
            )
        return _retval_g

    fun_name = 'naginterfaces.library.quad.dim1_fin_wcauchy'

    _b_utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = cb_g
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_wcauchy(
            _g,
            a,
            b,
            c,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_indef(a, b, f, relacc, absacc, maxrul, iparm, alpha, data=None):
    u"""
    One-dimensional quadrature, non-adaptive, finite interval with
    provision for indefinite integrals.

    For full information please refer to the NAG Library document for
    d01ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01arf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point in [a,b] at which the integrand f must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    relacc : float
        The relative accuracy required. If convergence according to
        absolute accuracy is required, `relacc` should be set to zero
        (but see also [Accuracy]). If `relacc` < 0.0, its absolute value
        is used.

        If `iparm` = 2, `relacc` is not used.

    absacc : float
        The absolute accuracy required. If convergence according to
        relative accuracy is required, `absacc` should be set to zero
        (but see also [Accuracy]). If `absacc` < 0.0, its absolute value
        is used.

        If `iparm` = 2, `absacc` is not used.

    maxrul : int
        The maximum number of successive rules that may be used.

    iparm : int
        Indicates the task to be performed by the function.

        `iparm` = 0

          Only the definite integral over [a,b] is evaluated.

        `iparm` = 1

          As well as the definite integral, the expansion of the
          integrand in Legendre polynomials over [a,b] is calculated,
          using the same values of the integrand as used to compute the
          integral. The expansion coefficients, and some other
          quantities, are returned in `alpha` for later use in computing
          indefinite integrals.

        `iparm` = 2

          f(t) is integrated analytically over [a,b] using the
          previously computed expansion, stored in `alpha`. No further
          evaluations of the integrand are required. The function must
          previously have been called with `iparm` = 1 and the interval
          [a,b] must lie within that specified for the previous call. In
          this case only the arguments `a`, `b`, `iparm`, `ans` and
          `alpha` are used.

    alpha : float, array-like, shape (390)
        If `iparm` = 2, `alpha` must contain the coefficients of the
        Legendre expansions of the integrand, as returned by a previous
        call of ``dim1_indef`` with `iparm` = 1 and a range containing
        the present range.

        If `iparm` = 0 or 1, `alpha` need not be set on entry.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    acc : float
        If `iparm` = 0 or 1, `acc` contains the absolute value of the
        difference between the last two successive estimates of the
        integral. This may be used as a measure of the accuracy actually
        achieved.

        If `iparm` = 2, `acc` is not used.

    ans : float
        The estimated value of the integral.

    n : int
        When `iparm` = 0 or 1, `n` contains the number of integrand
        evaluations used in the calculation of the integral.

        If `iparm` = 2, `n` is not used.

    alpha : float, ndarray, shape (390)
        If `iparm` = 1, the first m elements of `alpha` hold the
        coefficients of the Legendre expansion of the integrand, and the
        value of m is stored in `alpha`[389]. `alpha` must not be
        changed between a call with `iparm` = 1 and subsequent calls
        with `iparm` = 2.

        If `iparm` = 2, the first m elements of `alpha` are unchanged on
        exit.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `iparm` = *<value>*.

            Constraint: `iparm` = 0, 1 or 2.

        (`errno` 3)
            The function was called with `iparm` = 2, but a previous
            call with `iparm` = 1 has been omitted, or made with zero
            integration interval.

        (`errno` 4)
            The function was called with `iparm` = 2, but the interval
            for indefinite integration is not contained within the
            interval specified in the previous call with `iparm` = 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            All `maxrul` rules have been used but the integral has not
            converged to the accuracy requested. `maxrul` = *<value>*.

        (`errno` 1)
            Failure in convergence during the run with `iparm` = 1 in
            which the Legendre expansion was created.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import dim1_indef as b_dim1_indef

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_indef'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _alpha = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'alpha'},
        alpha,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_dim1_indef(
            a,
            b,
            _f,
            relacc,
            absacc,
            maxrul,
            iparm,
            _alpha,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _alpha_py = _alpha
    _acc_py = _return_scalars.acc
    _ans_py = _return_scalars.ans
    _n_py = _return_scalars.n
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['acc', 'ans', 'n', 'alpha'],
    )(
        _acc_py,
        _ans_py,
        _n_py,
        _alpha_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_inf_wtrig(g, a, omega, key, epsabs, limlst=50, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, semi-infinite interval, weight
    function cos(omega x) or sin(omega x).

    For full information please refer to the NAG Library document for
    d01as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01asf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    omega : float
        The argument omega in the weight function of the transform.

    key : int
        Indicates which integral is to be computed.

        `key` = 1

          w(x) = cos(omega x).

        `key` = 2

          w(x) = sin(omega x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    limlst : int, optional
        An upper bound on the number of intervals C_k needed for the
        integration.

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which each interval C_k may
        be divided by the function. The number of sub-intervals cannot
        exceed `lw`/4. The more difficult the integrand, the larger `lw`
        should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/2.

        The number of sub-intervals into which each interval C_k may be
        divided cannot exceed `liw`/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    lst : int
        The number of intervals C_k actually used for the integration.

    erlst : float, ndarray, shape (`limlst`)
        `erlst`[k-1] contains the error estimate corresponding to the
        integral contribution over the interval C_k, for k =
        1,2,...,`lst`.

    rslst : float, ndarray, shape (`limlst`)
        `rslst`[k-1] contains the integral contribution over the
        interval C_k, for k = 1,2,...,`lst`.

    ierlst : int, ndarray, shape (`limlst`)
        `ierlst`[k-1] contains the error flag corresponding to
        `rslst`[k-1], for k = 1,2,...,`lst`.

        In the cases `errno` = 7, 8 or 9, additional information about
        the cause of the error can be obtained from the array `ierlst`,
        as follows:

        `ierlst`[k-1] = 1

          The maximum number of subdivisions = min(`lw`/4,`liw`/2) has
          been achieved on the kth interval.

        `ierlst`[k-1] = 2

          Occurrence of round-off error is detected and prevents the
          tolerance imposed on the kth interval from being achieved.

        `ierlst`[k-1] = 3

          Extremely bad integrand behaviour occurs at some points of the
          kth interval.

        `ierlst`[k-1] = 4

          The integration procedure over the kth interval does not
          converge (to within the required accuracy) due to round-off in
          the extrapolation procedure invoked on this interval. It is
          assumed that the result on this interval is the best which can
          be obtained.

        `ierlst`[k-1] = 5

          The integral over the kth interval is probably divergent or
          slowly convergent. It must be noted that divergence can occur
          with any other value of `ierlst`[k-1].

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the maximum number of sub-intervals actually
        used for integrating over any of the intervals C_k. The rest of
        the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `limlst` = *<value>*.

            Constraint: `limlst` >= 3.

        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` <= 2.

        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1.

        (`errno` 10)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 10)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

        (`errno` 7)
            Bad integration behaviour occurs within interval K: K =
            *<value>*: (*<value>*,*<value>*). Inspect `ierlst` for more
            details.

        (`errno` 8)
            The maximum number of intervals (`limlst`) has been reached:
            `limlst` = *<value>*. Inspect `ierlst` for more details.

        (`errno` 9)
            Extrapolation does not converge to the requested accuracy.
            Inspect `ierlst` for more details.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import dim1_inf_wtrig as b_dim1_inf_wtrig

    def cb_g(x, data=None):
        """Wrapping user's g. Called as base g."""
        if data is not None:
            _retval_g = g(
                x,
                data,
            )
        else:
            _retval_g = g(
                x,
            )
        return _retval_g

    fun_name = 'naginterfaces.library.quad.dim1_inf_wtrig'

    _b_utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = cb_g
    _limlst = _b_utils._EngineIntScalarType(
        limlst, {'fun_name': fun_name, 'entity': 'limlst'},
    ).to_ctypes()
    _limlst_py = _b_utils._EngineIntScalarType.to_py(_limlst)
    _erlst = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_limlst_py,),
    )
    _rslst = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_limlst_py,),
    )
    _ierlst = _b_utils._EngineIntArrayType.empty_ndarray(
        (_limlst_py,),
    )
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _liw = _lw_py//2 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_inf_wtrig(
            _g,
            a,
            omega,
            key,
            epsabs,
            _limlst,
            _erlst,
            _rslst,
            _ierlst,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _lst_py = _return_scalars.lst
    _erlst_py = _erlst
    _rslst_py = _rslst
    _ierlst_py = _ierlst
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'lst', 'erlst', 'rslst', 'ierlst', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _lst_py,
        _erlst_py,
        _rslst_py,
        _ierlst_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_bad_vec(f, a, b, epsabs, epsrel, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, variant of
    ``dim1_fin_bad`` efficient on vector machines.

    For full information please refer to the NAG Library document for
    d01at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01atf.html

    Parameters
    ----------
    f : callable fv = f(x, data=None)
        `f` must return the values of the integrand f at a set of
        points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The points at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (n)
            `fv`[j-1] must contain the value of f at the point `x`[j-1],
            for j = 1,2,...,n.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import dim1_fin_bad_vec as b_dim1_fin_bad_vec

    fun_name = 'naginterfaces.library.quad.dim1_fin_bad_vec'

    cb_f_name = fun_name + ':f'

    def cb_f(x, fv, n, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fv_f = f(
                x,
                data,
            )
        else:
            _fv_f = f(
                x,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fv'},
            _fv_f,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_bad_vec(
            _f,
            a,
            b,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_osc_vec(f, a, b, epsabs, epsrel, key=6, lw=800, liw=None, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, variant of
    ``dim1_fin_osc`` efficient on vector machines.

    For full information please refer to the NAG Library document for
    d01au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01auf.html

    Parameters
    ----------
    f : callable fv = f(x, data=None)
        `f` must return the values of the integrand f at a set of
        points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The points at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (n)
            `fv`[j-1] must contain the value of f at the point `x`[j-1],
            for j = 1,2,...,n.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    key : int, optional
        Indicates which integration rule is to be used.

        `key` = 1

          For the Gauss 7-point and Kronrod 15-point rule.

        `key` = 2

          For the Gauss 10-point and Kronrod 21-point rule.

        `key` = 3

          For the Gauss 15-point and Kronrod 31-point rule.

        `key` = 4

          For the Gauss 20-point and Kronrod 41-point rule.

        `key` = 5

          For the Gauss 25-point and Kronrod 51-point rule.

        `key` = 6

          For the Gauss 30-point and Kronrod 61-point rule.

    lw : int, optional
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    liw : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `lw`/4.

        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    w : float, ndarray, shape (`lw`)
        Details of the computation see [Further Comments] for more
        information.

    iw : int, ndarray, shape (`liw`)
        `iw`[0] contains the actual number of sub-intervals used. The
        rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1 and `key` <= 6.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import dim1_fin_osc_vec as b_dim1_fin_osc_vec

    fun_name = 'naginterfaces.library.quad.dim1_fin_osc_vec'

    cb_f_name = fun_name + ':f'

    def cb_f(x, fv, n, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fv_f = f(
                x,
                data,
            )
        else:
            _fv_f = f(
                x,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fv'},
            _fv_f,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _lw = _b_utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = _b_utils._EngineIntScalarType.to_py(_lw)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lw_py,),
    )
    _liw = _lw_py//4 if liw is None else liw
    _liw = _b_utils._EngineIntScalarType(
        _liw, {'fun_name': fun_name, 'entity': '_liw'},
    ).to_ctypes()
    _liw_py = _b_utils._EngineIntScalarType.to_py(_liw)
    _iw = _b_utils._EngineIntArrayType.empty_ndarray(
        (_liw_py,),
    )
    try:
        _return_scalars = b_dim1_fin_osc_vec(
            _f,
            a,
            b,
            key,
            epsabs,
            epsrel,
            _w,
            _lw,
            _iw,
            _liw,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _w_py = _w
    _iw_py = _iw
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr', 'w', 'iw'],
    )(
        _result_py,
        _abserr_py,
        _w_py,
        _iw_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_gauss_wgen(itype, a, b, c, d, n):
    u"""
    Calculation of weights and abscissae for Gaussian quadrature rules,
    general choice of rule.

    For full information please refer to the NAG Library document for
    d01bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01bcf.html

    Parameters
    ----------
    itype : int
        Indicates the type of quadrature rule.

        `itype` = 0

          Gauss--Legendre, with normal weights.

        `itype` = 1

          Gauss--Jacobi, with normal weights.

        `itype` = -1

          Gauss--Jacobi, with adjusted weights.

        `itype` = 2

          Exponential Gauss, with normal weights.

        `itype` = -2

          Exponential Gauss, with adjusted weights.

        `itype` = 3

          Gauss--Laguerre, with normal weights.

        `itype` = -3

          Gauss--Laguerre, with adjusted weights.

        `itype` = 4

          Gauss--Hermite, with normal weights.

        `itype` = -4

          Gauss--Hermite, with adjusted weights.

        `itype` = 5

          Rational Gauss, with normal weights.

        `itype` = -5

          Rational Gauss, with adjusted weights.

    a : float
        The parameter `a` which occurs in the quadrature formula

    b : float
        The parameter `b` which occurs in the quadrature formula

    c : float
        The parameter `c` which occurs in the quadrature formula

    d : float
        The parameter `d` which occurs in the quadrature formula

    n : int
        n, the number of weights and abscissae to be returned. If
        `itype` = -2 or -4 and `c` != 0.0, an odd value of `n` may raise
        problems (see `errno` = 6).

    Returns
    -------
    weight : float, ndarray, shape (`n`)
        The `n` weights.

    abscis : float, ndarray, shape (`n`)
        The `n` abscissae.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `itype` = *<value>*.

            Constraint: abs(`itype`) < 6.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 3)
            On entry, `a`, `b`, `c`, or `d` is not in the allowed range:
            `a` = *<value>*, `b` = *<value>* `c` = *<value>*, `d` =
            *<value>* and `itype` = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The algorithm for computing eigenvalues of a tridiagonal
            matrix has failed to converge.

        (`errno` 4)
            One or more of the weights are larger than rmax, the largest
            floating point number on this computer (see
            ``machine.real_largest``): rmax = *<value>*.

            Possible solutions are to use a smaller value of n; or, if
            using adjusted weights to change to normal weights.

        (`errno` 5)
            One or more of the weights are too small to be distinguished
            from zero on this machine.

            The underflowing weights are returned as zero, which may be
            a usable approximation.

            Possible solutions are to use a smaller value of n; or, if
            using normal weights, to change to adjusted weights.

        (`errno` 6)
            The contribution of the central abscissa to the summation is
            indeterminate.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import dim1_gauss_wgen as b_dim1_gauss_wgen

    fun_name = 'naginterfaces.library.quad.dim1_gauss_wgen'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _weight = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _abscis = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_dim1_gauss_wgen(
            itype,
            a,
            b,
            c,
            d,
            _n,
            _weight,
            _abscis,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _weight_py = _weight
    _abscis_py = _abscis
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['weight', 'abscis'],
    )(
        _weight_py,
        _abscis_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_fin_smooth(f, a, b, epsabs, epsrel, data=None):
    u"""
    One-dimensional quadrature, non-adaptive, finite interval.

    For full information please refer to the NAG Library document for
    d01bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01bdf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    See Also
    --------
    :meth:`naginterfaces.library.examples.quad.dim1_fin_smooth_ex.main`
    """

    from ..base.quad import dim1_fin_smooth as b_dim1_fin_smooth

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim1_fin_smooth'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _return_scalars = b_dim1_fin_smooth(
        _f,
        a,
        b,
        epsabs,
        epsrel,
        data,
    )
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr'],
    )(
        _result_py,
        _abserr_py,
    )
    return _return_data

def dim2_fin(ya, yb, phi1, phi2, f, absacc, data=None):
    u"""
    Two-dimensional quadrature, finite region.

    For full information please refer to the NAG Library document for
    d01da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01daf.html

    Parameters
    ----------
    ya : float
        a, the lower limit of the integral.

    yb : float
        b, the upper limit of the integral. It is not necessary that a <
        b.

    phi1 : callable retval = phi1(y, data=None)
        `phi1` must return the lower limit of the inner integral for a
        given value of y.

        Parameters
        ~~~~~~~~~~
        y : float
            The value of y for which the lower limit must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The lower limit of the inner integral for the given value of
            y.

    phi2 : callable retval = phi2(y, data=None)
        `phi2` must return the upper limit of the inner integral for a
        given value of y.

        Parameters
        ~~~~~~~~~~
        y : float
            The value of y for which the upper limit must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The upper limit of the inner integral for the given value of
            y.

    f : callable retval = f(x, y, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The coordinates of the point (x,y) at which the integrand f
            must be evaluated.

        y : float
            The coordinates of the point (x,y) at which the integrand f
            must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    absacc : float
        The absolute accuracy requested.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    ans : float
        The estimated value of the integral.

    npts : int
        The total number of function evaluations.

    Raises
    ------
    NagValueError
        (`errno` i mod 10!=0)
            The outer integral has not converged, and n of the inner
            integrals have not converged with 255 points: n = *<value>*.
            `ans` may still contain an approximate estimate of the
            integral, but its reliability will decrease as n increases.

        (`errno` i mod 10 = 0)
            The outer integral has converged, but n of the inner
            integrals have not converged with 255 points: n = *<value>*.
            `ans` may still contain an approximate estimate of the
            integral, but its reliability will decrease as n increases.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The outer integral has not converged with 255 points.
            However, all the inner integrals have converged, and `ans`
            may still contain an approximate estimate of the integral.
    """

    from ..base.quad import dim2_fin as b_dim2_fin

    def cb_phi1(y, data=None):
        """Wrapping user's phi1. Called as base phi1."""
        if data is not None:
            _retval_phi1 = phi1(
                y,
                data,
            )
        else:
            _retval_phi1 = phi1(
                y,
            )
        return _retval_phi1

    def cb_phi2(y, data=None):
        """Wrapping user's phi2. Called as base phi2."""
        if data is not None:
            _retval_phi2 = phi2(
                y,
                data,
            )
        else:
            _retval_phi2 = phi2(
                y,
            )
        return _retval_phi2

    def cb_f(x, y, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                y,
                data,
            )
        else:
            _retval_f = f(
                x,
                y,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.dim2_fin'

    _b_utils._check_callable(phi1, {'fun_name': fun_name, 'entity': 'phi1'})
    _phi1 = cb_phi1
    _b_utils._check_callable(phi2, {'fun_name': fun_name, 'entity': 'phi2'})
    _phi2 = cb_phi2
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_dim2_fin(
            ya,
            yb,
            _phi1,
            _phi2,
            _f,
            absacc,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ans_py = _return_scalars.ans
    _npts_py = _return_scalars.npts
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ans', 'npts'],
    )(
        _ans_py,
        _npts_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_adapt_multi(a, b, mincls, maxcls, nfun, funsub, absreq, relreq, wrkstr, data=None):
    u"""
    Multidimensional adaptive quadrature over hyper-rectangle, multiple
    integrands.

    For full information please refer to the NAG Library document for
    d01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01eaf.html

    Parameters
    ----------
    a : float, array-like, shape (ndim)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, array-like, shape (ndim)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    mincls : int
        Must be set either to the minimum number of `funsub` calls to be
        allowed, in which case `mincls` >= 0 or to a negative value. In
        this case, the function continues the calculation started in a
        previous call with the same integrands and integration limits:
        no arguments other than `mincls`, `maxcls`, `absreq` or `relreq`
        must be changed between the calls.

    maxcls : int
        The maximum number of `funsub` calls to be allowed. In the
        continuation case this is the number of new `funsub` calls to be
        allowed.

    nfun : int
        m, the number of integrands.

    funsub : callable f = funsub(z, nfun, data=None)
        `funsub` must evaluate the integrands f_i at a given point.

        Parameters
        ~~~~~~~~~~
        z : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrands must be
            evaluated.

        nfun : int
            m, the number of integrands.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        f : float, array-like, shape (`nfun`)
            The value of the ith integrand at the given point.

    absreq : float
        The absolute accuracy required by you.

    relreq : float
        The relative accuracy required by you.

    wrkstr : float, array-like, shape (lenwrk)
        If `mincls` < 0, `wrkstr` must be unchanged from the previous
        call of ``md_adapt_multi``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mincls : int
        Gives the number of `funsub` calls actually used by
        ``md_adapt_multi``. For the continuation case (`mincls` < 0 on
        entry) this is the number of new `funsub` calls on the current
        call to ``md_adapt_multi``.

    wrkstr : float, ndarray, shape (lenwrk)
        Contains information about the current subdivision which could
        be used in a continuation call.

    finest : float, ndarray, shape (`nfun`)
        `finest`[i-1] specifies the best estimate obtained from the ith
        integral, for i = 1,2,...,m.

    absest : float, ndarray, shape (`nfun`)
        `absest`[i-1] specifies the estimated absolute accuracy of
        `finest`[i-1], for i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `maxcls` = *<value>* and R = *<value>*.

            Constraint: `maxcls` >= R = 1+ndim*(4*ndim^2-6*ndim+14)/3.

        (`errno` 4)
            On entry, `maxcls` = *<value>* and R = *<value>*.

            Constraint: `maxcls` >= R = 2^ndim+2*ndim^2+2*ndim+1.

        (`errno` 4)
            On entry, lenwrk is too small. lenwrk = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 4)
            On entry, `maxcls` = *<value>* and `mincls` = *<value>*.

            Constraint: `maxcls` >= `mincls`.

        (`errno` 4)
            On entry, `nfun` = *<value>*.

            Constraint: `nfun` >= 1.

        (`errno` 4)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            `maxcls` too small to obtain required accuracy. `maxcls` =
            *<value>*.

        (`errno` 2)
            lenwrk too small for the function to continue. lenwrk =
            *<value>*.

        (`errno` 3)
            `maxcls` too small to make any progress. `maxcls` =
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import md_adapt_multi as b_md_adapt_multi

    fun_name = 'naginterfaces.library.quad.md_adapt_multi'

    cb_funsub_name = fun_name + ':funsub'

    def cb_funsub(ndim, z, nfun, f, data=None):
        """Wrapping user's funsub. Called as base funsub."""
        if data is not None:
            _f_funsub = funsub(
                z,
                nfun,
                data,
            )
        else:
            _f_funsub = funsub(
                z,
                nfun,
            )
        f[:] = _utils._to_ndarray(
            {'fun_name': cb_funsub_name, 'returned': 'f'},
            _f_funsub,
            copy=False,
            exp_shape=(nfun,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ndim_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_utils._check_callable(funsub, {'fun_name': fun_name, 'entity': 'funsub'})
    _funsub = cb_funsub
    _wrkstr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wrkstr'},
        wrkstr,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lenwrk_py = _b_utils._EngineFloat64ArrayType.get_shape(_wrkstr, {'fun_name': fun_name, 'entity': 'wrkstr'}, exp_rank=1)[0]
    _lenwrk = _b_utils._EngineIntScalarType(
        _lenwrk_py, {'fun_name': fun_name, 'entity': '_lenwrk'},
    ).to_ctypes()
    _nfun = _b_utils._EngineIntScalarType(
        nfun, {'fun_name': fun_name, 'entity': 'nfun'},
    ).to_ctypes()
    _nfun_py = _b_utils._EngineIntScalarType.to_py(_nfun)
    _finest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nfun_py,),
    )
    _absest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nfun_py,),
    )
    try:
        _return_scalar = b_md_adapt_multi(
            _ndim,
            _a,
            _b,
            mincls,
            maxcls,
            _nfun,
            _funsub,
            absreq,
            relreq,
            _lenwrk,
            _wrkstr,
            _finest,
            _absest,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _mincls_py = _return_scalar
    _wrkstr_py = _wrkstr
    _finest_py = _finest
    _absest_py = _absest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['mincls', 'wrkstr', 'finest', 'absest'],
    )(
        _mincls_py,
        _wrkstr_py,
        _finest_py,
        _absest_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_sgq_multi_vec(ni, f, maxdlv, comm, data=None, spiked_sorder='C'):
    u"""
    Multi-dimensional quadrature using sparse grids.

    Note: this function uses optional algorithmic parameters, see also:
    ``opt_set``, ``opt_get``.

    For full information please refer to the NAG Library document for
    d01es

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01esf.html

    Parameters
    ----------
    ni : int
        n_i, the number of integrands.

    f : callable (fm, iflag) = f(ni, ndim, xtr, icolzp, irowix, xs, qs,
        iflag, data=None)
        `f` must return the value of the integrands f_j at a set of n_x
        d-dimensional points x_i, implicitly supplied as columns of a
        matrix X(d,n_x).

        If X was supplied explicitly you would find that most of the
        elements attain the same value, x_tr; the larger the number of
        dimensions, the greater the proportion of elements of X would be
        equal to x_tr.

        So, X is effectively a sparse matrix, except that the 'zero'
        elements are replaced by elements that are all equal to the
        value x_tr.

        For this reason X is supplied, as though it were a sparse
        matrix, in compressed column storage (CCS) format (see [the F11
        Introduction]).

        Parameters
        ~~~~~~~~~~
        ni : int
            n_i, the number of integrands.

        ndim : int
            d, the number of dimensions.

        xtr : float
            x_tr, the value of the trivial elements of X.

        icolzp : int, ndarray, shape (nx+1)
            The set {`icolzp`[i-1],...,`icolzp`[i]-1} contains the
            indices of `irowix` and `xs` corresponding to the
            non-trivial elements of column i of X and hence of the point
            x_i, for i = 1,2,...,n_x.

        irowix : int, ndarray, shape (nntr)
            The row indices corresponding to the non-trivial elements of
            X.

        xs : float, ndarray, shape (nntr)
            x_{k,i} != x_tr, the non-trivial entries of X.

        qs : int, ndarray, shape (nntr)
            q_{k,i} != 1, the indices of the underlying quadrature rules
            corresponding to x_{k,i} != x_tr.

        iflag : int
            If `iflag` = 0, this is the first call to `f`. n_x = 1, and
            the entire point x_1 will satisfy x_{k,1} = x_tr, for k =
            1,2,...,d. In addition, nntr contains the total number of
            abscissae from the underlying one-dimensional quadrature;
            `xs` contains the complete set of abscissae and `qs`
            contains the corresponding quadrature indices, with `xs`[0]
            = x_tr and `qs`[0] = 1. This will always be called in
            serial.

            In subsequent calls to `f`, `iflag` = 1.

            Subsequent calls may be made from within an OpenMP parallel
            region.

            See [Parallelism and Performance] for details.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fm : float, array-like, shape (`ni`, nx)
            `fm`[p-1,i-1] = f_p(x_i), for p = 1,2,...,n_i, for i =
            1,2,...,n_x.

        iflag : int
            Set `iflag` < 0 if you wish to force an immediate exit from
            ``md_sgq_multi_vec`` with `errno` = -1.

    maxdlv : int, array-like, shape (ndim)
        m, the array of maximum levels for each dimension.
        `maxdlv`[j-1], for j = 1,2,...,d, contains m_j, the maximum
        level of quadrature rule dimension j will support.

        The default value, min(m_q, L) will be used if either
        `maxdlv`[j-1] <= 0 or `maxdlv`[j-1] >= min(m_q,L) (for details
        on the default values for m_q and L and on how to change these
        values see the options 'Maximum Level', 'Maximum Quadrature
        Level' and 'Quadrature Rule').

        If `maxdlv`[j-1] = 1 for all j, only one evaluation will be
        performed, and as such no error estimation will be possible.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    data : arbitrary, optional
        User-communication data for callback functions.

    spiked_sorder : str, optional
        If `fm` in `f` is spiked (i.e., has unit extent in all but one
        dimension, or has size 1) and is treated as contiguous in both
        the Fortran and the C sense by NumPy, `spiked_sorder` selects
        the storage order to associate with it in the NAG Engine:

        spiked_sorder = 'C'
          row-major storage will be used;

        spiked_sorder = 'F'
          column-major storage will be used.

    Returns
    -------
    dinest : float, ndarray, shape (`ni`)
        `dinest`[p-1] contains the final estimate F^_p of the definite
        integral F_p, for p = 1,2,...,n_i.

    errest : float, ndarray, shape (`ni`)
        `errest`[p-1] contains the final error estimate E_p of the
        definite integral F_p, for p = 1,2,...,n_i.

    ivalid : int, ndarray, shape (`ni`)
        `ivalid`[p-1] indicates the final state of integral p, for p =
        1,2,...,n_i.

        `ivalid`[p-1] = 0

          The error estimate for integral p was below the requested
          tolerance.

        `ivalid`[p-1] = 1

          The error estimate for integral p was below the requested
          tolerance. The final level used was non-isotropic.

        `ivalid`[p-1] = 2

          The error estimate for integral p was above the requested
          tolerance.

        `ivalid`[p-1] = 3

          The error estimate for integral p was above
          max(0.1|F^_p|,0.01).

        `ivalid`[p-1] < 0

          You aborted the evaluation before an error estimate could be
          made.

    Other Parameters
    ----------------
    'Absolute Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_a, the absolute tolerance required.

    'Index Level' : int
        Default = 4

        The maximum level at which function values are stored for later
        use.
        Larger values use increasingly more memory, and require more
        time to access specific elements.
        Lower levels result in more repeated computation.
        The 'Maximum Quadrature Level', m_q is the effective upper limit
        on 'Index Level'.
        If i >= m_q, ``md_sgq_multi_vec`` will use m_q as the value of
        'Index Level'.

        Constraint: i >= 1.

    'Maximum Level' : int
        Default = 5

        i = L, the maximum number of levels to evaluate.

        Constraint: 1 < i <= 20.

        Note: the maximum allowable level in any single dimension, m_q,
        is governed by the 'Quadrature Rule' selected. If a value
        greater than m_q is set, only a subset of subspaces from higher
        levels will be used. Should this subset be empty for a given
        level, computation will consider the preceding level to be the
        maximum level and will terminate.

    'Maximum Nx' : int
        Default = 128

        i = maxn_x, the maximum number of points to evaluate in a single
        call to `f`.

        Constraint: 1 <= i <= 16384.

    'Maximum Quadrature Level' : int
        Queriable only

        i = m_q, the maximum level of the underlying one-dimensional
        quadrature rule (see 'Quadrature Rule').

    'Minimum Level' : int
        Default = 2

        The minimum number of levels which must be evaluated before an
        error estimate is used to determine convergence.

        Constraint: i > 1.

        Note: if the minimum level is greater than the maximum
        computable level, the maximum level will be used.

    'Quadrature Rule' : str
        Default = 'Gauss--Patterson'

        The underlying one-dimensional quadrature rule to be used in the
        construction.
        Open rules do not require evaluations at boundaries.

        'Quadrature Rule' = 'Gauss--Patterson' or 'GP'
          The interlacing Gauss--Patterson rules. Level l uses 2^l -1
          abscissae. All levels are open. These rules provide high order
          accuracy. m_q = 9.

        'Quadrature Rule' = 'Clenshaw--Curtis' or 'CC'
          The interlacing Clenshaw--Curtis rules. Level l uses 2^(l
          -1)+1 abscissae. All levels above level 1 are closed. m_q =
          12.

    'Relative Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_a, the relative tolerance required.

    'Summation Precision' : str
        Default = 'HIGHER'

        Determines whether ``md_sgq_multi_vec`` uses working precision
        or higher-than-working precision to accumulate the actions over
        subspaces.

        'Summation Precision' = 'HIGHER' or 'H'
          Higher-than-working precision is used to accumulate the action
          over a subspace, and for the accumulation of all such actions.
          This is more expensive computationally, although this is
          probably negligible in comparison to the cost of evaluating
          the integrands and the overall runtime. This significantly
          reduces variation in the result when changing the number of
          threads.

        'Summation Precision' = 'WORKING' or 'W'
          Working precision is used to accumulate the actions over
          subspaces. This may provide some speedup, particularly if n_i
          or n_t is large. The results of parallel simulations will
          however be more prone to variation.

    'Serial Levels' : int
        Default = 1

        i = s_l, the number of levels to be evaluated in serial before
        initializing parallelization.
        For relatively trivial integrands, this may need to be set
        greater than the default to reduce parallel overhead.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `ni` = *<value>*.

            Constraint: `ni` >= 1.

        (`errno` 21)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 1001)
            Either the option arrays `comm`['iopts'] and `comm`['opts']
            have not been initialized for ``md_sgq_multi_vec``, or they
            have become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The requested accuracy was not achieved for at least one
            integral.

        (`errno` 2)
            No accuracy was achieved for at least one integral.
    """

    from ..base.quad import md_sgq_multi_vec as b_md_sgq_multi_vec

    fun_name = 'naginterfaces.library.quad.md_sgq_multi_vec'

    cb_f_name = fun_name + ':f'

    def cb_f(ni, ndim, nx, xtr, nntr, icolzp, irowix, xs, qs, fm, iflag, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fm_f, _iflag_f = f(
                ni,
                ndim,
                xtr,
                icolzp,
                irowix,
                xs,
                qs,
                iflag,
                data,
            )
        else:
            _fm_f, _iflag_f = f(
                ni,
                ndim,
                xtr,
                icolzp,
                irowix,
                xs,
                qs,
                iflag,
            )
        _fm_f_np = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fm'},
            _fm_f,
            copy=False,
            exp_shape=(ni, nx),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _iflag_f_base = _iflag_f
        _utils._check_callback_sorder(
            {'fun_name': cb_f_name},
            ((_fm_f_np, 'fm'),),
            wrapper_sorder=_m_sorder,
            spiked_sorder=spiked_sorder,
        )
        fm[:, :] = _fm_f_np
        return _iflag_f_base

    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        (),
        spiked_sorder=spiked_sorder,
    )
    _maxdlv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'maxdlv'},
        maxdlv,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_maxdlv, {'fun_name': fun_name, 'entity': 'maxdlv'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _ni = _b_utils._EngineIntScalarType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
    ).to_ctypes()
    _ni_py = _b_utils._EngineIntScalarType.to_py(_ni)
    _dinest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ni_py,),
    )
    _errest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ni_py,),
    )
    _ivalid = _b_utils._EngineIntArrayType.empty_ndarray(
        (_ni_py,),
    )
    try:
        b_md_sgq_multi_vec(
            _ni,
            _ndim,
            _f,
            _maxdlv,
            _dinest,
            _errest,
            _ivalid,
            comm,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _dinest_py = _dinest
    _errest_py = _errest
    _ivalid_py = _ivalid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dinest', 'errest', 'ivalid'],
    )(
        _dinest_py,
        _errest_py,
        _ivalid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_gauss(nptvec, weight, abscis, f, data=None):
    u"""
    Multidimensional Gaussian quadrature over hyper-rectangle.

    For full information please refer to the NAG Library document for
    d01fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fbf.html

    Parameters
    ----------
    nptvec : int, array-like, shape (ndim)
        `nptvec`[j-1] must specify the number of points in the jth
        dimension of the summation, for j = 1,2,...,n.

    weight : float, array-like, shape (lwa)
        Must contain in succession the weights for the various
        dimensions, i.e., `weight`[k-1] contains the ith weight for the
        jth dimension, with

            k = `nptvec`[0]+`nptvec`[1]+...+`nptvec`[j-2]+i.

    abscis : float, array-like, shape (lwa)
        Must contain in succession the abscissae for the various
        dimensions, i.e., `abscis`[k-1] contains the ith abscissa for
        the jth dimension, with

            k = `nptvec`[0]+`nptvec`[1]+...+`nptvec`[j-2]+i.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mdint : float
        The estimate of the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lwa is too small. lwa = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim <= 20.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.
    """

    from ..base.quad import md_gauss as b_md_gauss

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.md_gauss'

    _nptvec = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nptvec'},
        nptvec,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_nptvec, {'fun_name': fun_name, 'entity': 'nptvec'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _weight = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'weight'},
        weight,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lwa_py = _b_utils._EngineFloat64ArrayType.get_shape(_weight, {'fun_name': fun_name, 'entity': 'weight'}, exp_rank=1)[0]
    _lwa = _b_utils._EngineIntScalarType(
        _lwa_py, {'fun_name': fun_name, 'entity': '_lwa'},
    ).to_ctypes()
    _abscis = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'abscis'},
        abscis,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _return_scalar = b_md_gauss(
        _ndim,
        _nptvec,
        _lwa,
        _weight,
        _abscis,
        _f,
        data,
    )
    return _return_scalar

def md_adapt(a, b, minpts, maxpts, f, eps, data=None):
    u"""
    Multidimensional adaptive quadrature over hyper-rectangle.

    For full information please refer to the NAG Library document for
    d01fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fcf.html

    Parameters
    ----------
    a : float, array-like, shape (ndim)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, array-like, shape (ndim)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    minpts : int
        Must be set to the minimum number of integrand evaluations to be
        allowed.

    maxpts : int
        The maximum number of integrand evaluations to be allowed.

    f : callable retval = f(z, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        z : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    eps : float
        The relative error acceptable to you. When the solution is zero
        or very small relative accuracy may not be achievable but you
        may still set `eps` to a reasonable value and check for the
        error exit `errno` = 2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    minpts : int
        Contains the actual number of integrand evaluations used by
        ``md_adapt``.

    acc : float
        The estimated relative error in `finval`.

    finval : float
        The best estimate obtained for the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxpts` is too small. `maxpts` = *<value>*.
            Minimum possible dimension: *<value>*.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `minpts` = *<value>* and `maxpts` = *<value>*.

            Constraint: `minpts` <= `maxpts`.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim <= 15.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            `maxpts` too small to obtain requested accuracy `eps`:
            `maxpts` = *<value>* and `eps` = *<value>*.

        (`errno` 3)
            lenwrk was too small to complete computation. `finval` and
            `acc` contain estimates of integral and relative error, but
            `acc` is greater than `eps`.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import md_adapt as b_md_adapt

    def cb_f(ndim, z, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                z,
                data,
            )
        else:
            _retval_f = f(
                z,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.md_adapt'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ndim_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_md_adapt(
            _ndim,
            _a,
            _b,
            minpts,
            maxpts,
            _f,
            eps,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _minpts_py = _return_scalars.minpts
    _acc_py = _return_scalars.acc
    _finval_py = _return_scalars.finval
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['minpts', 'acc', 'finval'],
    )(
        _minpts_py,
        _acc_py,
        _finval_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_sphere(ndim, f, sigma, limit, region=None, r0=0.8, u=1.5, data=None):
    u"""
    Multidimensional quadrature, Sag--Szekeres method, general product
    region or n-sphere.

    For full information please refer to the NAG Library document for
    d01fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fdf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    sigma : float
        Indicates the region of integration.

        `sigma` >= 0.0

          The integration is carried out over the n-sphere of radius
          `sigma`, centred at the origin.

        `sigma` < 0.0

          The integration is carried out over the product region
          described by `region`.

    limit : int
        The approximate maximum number of integrand evaluations to be
        used.

    region : None or callable (c, d) = region(x, j, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sigma` < 0.0, `region` must evaluate the limits of
        integration in any dimension.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            `x`[0],...,`x`[j-2] contain the current values of the first
            (j-1) variables, which may be used if necessary in
            calculating c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        c : float
            The lower limit c_j of the range of x_j.

        d : float
            The upper limit d_j of the range of x_j.

    r0 : float, optional
        The cut-off radius on the unit n-sphere, which may be regarded
        as an adjustable parameter of the method.

    u : float, optional
        Must specify an adjustable parameter of the transformation to
        the unit n-sphere.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    ncalls : int
        The actual number of integrand evaluations used. (See also
        [Further Comments].)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 30.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `limit` = *<value>*.

            Constraint: `limit` >= 100.

        (`errno` 3)
            On entry, `r0` = *<value>*.

            Constraint: `r0` < 1.0.

        (`errno` 3)
            On entry, `r0` = *<value>*.

            Constraint: `r0` > 0.0.

        (`errno` 4)
            On entry, `u` = *<value>*.

            Constraint: `u` > 0.0.
    """

    from ..base.quad import md_sphere as b_md_sphere

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    def cb_region(ndim, x, j, data=None):
        """Wrapping user's region. Called as base region."""
        if data is not None:
            _c_region, _d_region = region(
                x,
                j,
                data,
            )
        else:
            _c_region, _d_region = region(
                x,
                j,
            )
        _c_region_base = _c_region
        _d_region_base = _d_region
        return (
            _c_region_base,
            _d_region_base,
        )

    fun_name = 'naginterfaces.library.quad.md_sphere'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _b_utils._check_callable(region, {'fun_name': fun_name, 'entity': 'region'}, allow_none=True)
    _region = None if region is None else cb_region
    _return_scalars = b_md_sphere(
        ndim,
        _f,
        sigma,
        _region,
        limit,
        r0,
        u,
        data,
    )
    _result_py = _return_scalars.result
    _ncalls_py = _return_scalars.ncalls
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'ncalls'],
    )(
        _result_py,
        _ncalls_py,
    )
    return _return_data

def dim1_data(x, y):
    u"""
    One-dimensional quadrature, integration of function defined by data
    values, Gill--Miller method.

    For full information please refer to the NAG Library document for
    d01ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gaf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The values of the independent variable, i.e., the
        x_1,x_2,...,x_n.

    y : float, array-like, shape (n)
        The values of the dependent variable y_i at the points x_i, for
        i = 1,2,...,n.

    Returns
    -------
    ans : float
        The estimated value of the integral.

    er : float
        An estimate of the uncertainty in `ans`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 4.

        (`errno` 2)
            On entry, `x`[0] = *<value>* and `x`[1] = *<value>*,
            `x`[I-2] = *<value>*, `x`[I-1] = *<value>* and I =
            *<value>*.

            Constraint: either `x`[0] < `x`[1] < ... < `x`[n-1] or
            `x`[0] > `x`[1] > ... > `x`[n-1].

        (`errno` 3)
            On entry, I = *<value>*, `x`[I-1] = *<value>* and `x`[I-2] =
            *<value>*.

            Constraint: `x`[I-1] != `x`[I-2].
    """

    from ..base.quad import dim1_data as b_dim1_data

    fun_name = 'naginterfaces.library.quad.dim1_data'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = b_dim1_data(
        _x,
        _y,
        _n,
    )
    _ans_py = _return_scalars.ans
    _er_py = _return_scalars.er
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ans', 'er'],
    )(
        _ans_py,
        _er_py,
    )
    return _return_data

def md_mcarlo(a, b, mincls, maxcls, f, eps, wrkstr, finest, data=None):
    u"""
    Multidimensional quadrature over hyper-rectangle, Monte Carlo
    method.

    For full information please refer to the NAG Library document for
    d01gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gbf.html

    Parameters
    ----------
    a : float, array-like, shape (ndim)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, array-like, shape (ndim)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    mincls : int
        Must be set

        - either to the minimum number of integrand evaluations to be
          allowed, in which case `mincls` >= 0;

        - or to a negative value. In this case, the function assumes
          that a previous call had been made with the same arguments
          ndim, `a` and `b` and with either the same integrand (in which
          case ``md_mcarlo`` continues calculation) or a similar
          integrand (in which case ``md_mcarlo`` begins the calculation
          with the subdivision used in the last iteration of the
          previous call). See also `wrkstr`.

    maxcls : int
        The maximum number of integrand evaluations to be allowed. In
        the continuation case this is the number of new integrand
        evaluations to be allowed. These counts do not include zero
        integrand values.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    eps : float
        The relative accuracy required.

    wrkstr : float, array-like, shape (lenwrk)
        If `mincls` < 0, `wrkstr` must be unchanged from the previous
        call of ``md_mcarlo`` -- except that for a new integrand
        `wrkstr`[lenwrk-1] must be set to 0.0. See also `mincls`.

    finest : float
        Must be unchanged from a previous call to ``md_mcarlo``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mincls : int
        Contains the number of integrand evaluations actually used by
        ``md_mcarlo``.

    acc : float
        The estimated relative accuracy of `finest`.

    wrkstr : float, ndarray, shape (lenwrk)
        Contains information about the current sub-interval structure
        which could be used in later calls of ``md_mcarlo``. In
        particular, `wrkstr`[j-1] gives the number of sub-intervals used
        along the jth coordinate axis.

    finest : float
        The best estimate obtained for the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `maxcls` = *<value>* and ndim = *<value>*.

            Constraint: `maxcls` >= 4*(ndim+1).

        (`errno` 1)
            On entry, `maxcls` = *<value>* and `mincls` = *<value>*.

            Constraint: `maxcls` > `mincls`.

        (`errno` 1)
            On entry, lenwrk is too small. lenwrk = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            `maxcls` too small to obtain required accuracy `eps`.
            `maxcls` = *<value>* and `eps` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.quad import md_mcarlo as b_md_mcarlo

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.md_mcarlo'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ndim_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _wrkstr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wrkstr'},
        wrkstr,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lenwrk_py = _b_utils._EngineFloat64ArrayType.get_shape(_wrkstr, {'fun_name': fun_name, 'entity': 'wrkstr'}, exp_rank=1)[0]
    _lenwrk = _b_utils._EngineIntScalarType(
        _lenwrk_py, {'fun_name': fun_name, 'entity': '_lenwrk'},
    ).to_ctypes()
    try:
        _return_scalars = b_md_mcarlo(
            _ndim,
            _a,
            _b,
            mincls,
            maxcls,
            _f,
            eps,
            _lenwrk,
            _wrkstr,
            finest,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _mincls_py = _return_scalars.mincls
    _wrkstr_py = _wrkstr
    _finest_py = _return_scalars.finest
    _acc_py = _return_scalars.acc
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['mincls', 'acc', 'wrkstr', 'finest'],
    )(
        _mincls_py,
        _acc_py,
        _wrkstr_py,
        _finest_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_numth(f, region, npts, vk, nrand, itrans=0, data=None):
    u"""
    Multidimensional quadrature, general product region,
    number-theoretic method.

    For full information please refer to the NAG Library document for
    d01gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gcf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    region : callable (c, d) = region(x, j, data=None)
        `region` must evaluate the limits of integration in any
        dimension.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            `x`[0],...,`x`[j-2] contain the current values of the first
            (j-1) variables, which may be used if necessary in
            calculating c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        c : float
            The lower limit c_j of the range of x_j.

        d : float
            The upper limit d_j of the range of x_j.

    npts : int
        The Korobov rule to be used. There are two alternatives
        depending on the value of `npts`.

        - 1 <= `npts` <= 6.

          In this case one of six preset rules is chosen using 2129,
          5003, 10007, 20011, 40009 or 80021 points depending on the
          respective value of `npts` being 1, 2, 3, 4, 5 or 6.

        - `npts` > 6.

          `npts` is the number of actual points to be used with
          corresponding optimal coefficients supplied in the array `vk`.

    vk : float, array-like, shape (ndim)
        If `npts` > 6, `vk` must contain the n optimal coefficients
        (which may be calculated using ``md_numth_coeff_prime`` or
        ``md_numth_coeff_2prime``).

        If `npts` <= 6, `vk` need not be set.

    nrand : int
        The number of random samples to be generated in the error
        estimation (generally a small value, say 3 to 5, is sufficient).
        The total number of integrand evaluations will be
        `nrand`*`npts`.

    itrans : int, optional
        Indicates whether the periodising transformation is to be used.

        `itrans` = '0'

          The transformation is to be used.

        `itrans` != '0'

          The transformation is to be suppressed (to cover cases where
          the integrand may already be periodic or where you want to
          specify a particular transformation in the definition of `f`).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    vk : float, ndarray, shape (ndim)
        If `npts` > 6, `vk` is unchanged.

        If `npts` <= 6, `vk` contains the n optimal coefficients used by
        the preset rule.

    res : float
        The approximation to the integral I.

    err : float
        The standard error as computed from `nrand` sample values. If
        `nrand` = 1, `err` contains zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim <= 20.

        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 1.

        (`errno` 3)
            On entry, `nrand` = *<value>*.

            Constraint: `nrand` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import md_numth as b_md_numth

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    def cb_region(ndim, x, j, data=None):
        """Wrapping user's region. Called as base region."""
        if data is not None:
            _c_region, _d_region = region(
                x,
                j,
                data,
            )
        else:
            _c_region, _d_region = region(
                x,
                j,
            )
        _c_region_base = _c_region
        _d_region_base = _d_region
        return (
            _c_region_base,
            _d_region_base,
        )

    fun_name = 'naginterfaces.library.quad.md_numth'

    _vk = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vk'},
        vk,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ndim_py = _b_utils._EngineFloat64ArrayType.get_shape(_vk, {'fun_name': fun_name, 'entity': 'vk'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _b_utils._check_callable(region, {'fun_name': fun_name, 'entity': 'region'})
    _region = cb_region
    _return_scalars = b_md_numth(
        _ndim,
        _f,
        _region,
        npts,
        _vk,
        nrand,
        itrans,
        data,
    )
    _vk_py = _vk
    _res_py = _return_scalars.res
    _err_py = _return_scalars.err
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['vk', 'res', 'err'],
    )(
        _vk_py,
        _res_py,
        _err_py,
    )
    return _return_data

def md_numth_vec(vecfun, vecreg, npts, vk, nrand, itrans=0, data=None):
    u"""
    Multidimensional quadrature, general product region,
    number-theoretic method, variant of ``md_numth`` efficient on vector
    machines.

    For full information please refer to the NAG Library document for
    d01gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gdf.html

    Parameters
    ----------
    vecfun : callable fv = vecfun(x, data=None)
        `vecfun` must evaluate the integrand at a specified set of
        points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (m, ndim)
            The coordinates of the m points at which the integrand must
            be evaluated. `x`[i-1,j-1] contains the jth coordinate of
            the ith point.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (m)
            `fv`[i-1] must contain the value of the integrand of the ith
            point, i.e., `fv`[i-1] =
            f(`x`[i-1,0],`x`[i-1,1],...,`x`[i-1,ndim-1]), for i =
            1,2,...,m.

    vecreg : callable (c, d) = vecreg(x, j, data=None)
        `vecreg` must evaluate the limits of integration in any
        dimension for a set of points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (m, ndim)
            For i = 1,2,...,m, `x`[i-1,0], `x`[i-1,1],...,`x`[i-1,j-2]
            contain the current values of the first (j-1) coordinates of
            the ith point, which may be used if necessary in calculating
            the m values of c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        c : float, array-like, shape (m)
            `c`[i-1] must be set to the lower limit of the range for
            `x`[i-1,j-1], for i = 1,2,...,m.

        d : float, array-like, shape (m)
            `d`[i-1] must be set to the upper limit of the range for
            `x`[i-1,j-1], for i = 1,2,...,m.

    npts : int
        The Korobov rule to be used. There are two alternatives
        depending on the value of `npts`.

        - 1 <= `npts` <= 6.

          In this case one of six preset rules is chosen using 2129,
          5003, 10007, 20011, 40009 or 80021 points depending on the
          respective value of `npts` being 1, 2, 3, 4, 5 or 6.

        - `npts` > 6.

          `npts` is the number of actual points to be used with
          corresponding optimal coefficients supplied in the array `vk`.

    vk : float, array-like, shape (ndim)
        If `npts` > 6, `vk` must contain the n optimal coefficients
        (which may be calculated using ``md_numth_coeff_prime`` or
        ``md_numth_coeff_2prime``).

        If `npts` <= 6, `vk` need not be set.

    nrand : int
        The number of random samples to be generated (generally a small
        value, say 3 to 5, is sufficient). The estimate, `res`, of the
        value of the integral returned by the function is then the
        average of `nrand` calculations with different random origin
        shifts. If `npts` > 6, the total number of integrand evaluations
        will be `nrand`*`npts`. If 1 <= `npts` <= 6, the number of
        integrand evaluations will be `nrand`*p, where p is the number
        of points corresponding to the six preset rules. For reasons of
        efficiency, these values are calculated a number at a time in
        `vecfun`.

    itrans : int, optional
        Indicates whether the periodising transformation is to be used.

        `itrans` = 0

          The transformation is to be used.

        `itrans` = 1

          The transformation is to be suppressed (to cover cases where
          the integrand may already be periodic or where you want to
          specify a particular transformation in the definition of
          `vecfun`).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    vk : float, ndarray, shape (ndim)
        If `npts` > 6, `vk` is unchanged.

        If `npts` <= 6, `vk` contains the n optimal coefficients used by
        the preset rule.

    res : float
        The approximation to the integral I.

    err : float
        The standard error as computed from `nrand` sample values. If
        `nrand` = 1, `err` contains zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: 1 <= ndim <= 20.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 1.

        (`errno` 3)
            On entry, `nrand` = *<value>*.

            Constraint: `nrand` >= 1.
    """

    from ..base.quad import md_numth_vec as b_md_numth_vec

    fun_name = 'naginterfaces.library.quad.md_numth_vec'

    cb_vecfun_name = fun_name + ':vecfun'

    def cb_vecfun(ndim, x, fv, m, data=None):
        """Wrapping user's vecfun. Called as base vecfun."""
        if data is not None:
            _fv_vecfun = vecfun(
                x,
                data,
            )
        else:
            _fv_vecfun = vecfun(
                x,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_vecfun_name, 'returned': 'fv'},
            _fv_vecfun,
            copy=False,
            exp_shape=(m,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    cb_vecreg_name = fun_name + ':vecreg'

    def cb_vecreg(ndim, x, j, c, d, m, data=None):
        """Wrapping user's vecreg. Called as base vecreg."""
        if data is not None:
            _c_vecreg, _d_vecreg = vecreg(
                x,
                j,
                data,
            )
        else:
            _c_vecreg, _d_vecreg = vecreg(
                x,
                j,
            )
        c[:] = _utils._to_ndarray(
            {'fun_name': cb_vecreg_name, 'returned': 'c'},
            _c_vecreg,
            copy=False,
            exp_shape=(m,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        d[:] = _utils._to_ndarray(
            {'fun_name': cb_vecreg_name, 'returned': 'd'},
            _d_vecreg,
            copy=False,
            exp_shape=(m,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _vk = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vk'},
        vk,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ndim_py = _b_utils._EngineFloat64ArrayType.get_shape(_vk, {'fun_name': fun_name, 'entity': 'vk'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _b_utils._check_callable(vecfun, {'fun_name': fun_name, 'entity': 'vecfun'})
    _vecfun = cb_vecfun
    _b_utils._check_callable(vecreg, {'fun_name': fun_name, 'entity': 'vecreg'})
    _vecreg = cb_vecreg
    _return_scalars = b_md_numth_vec(
        _ndim,
        _vecfun,
        _vecreg,
        npts,
        _vk,
        nrand,
        itrans,
        data,
    )
    _vk_py = _vk
    _res_py = _return_scalars.res
    _err_py = _return_scalars.err
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['vk', 'res', 'err'],
    )(
        _vk_py,
        _res_py,
        _err_py,
    )
    return _return_data

def md_numth_coeff_prime(ndim, npts):
    u"""
    Korobov optimal coefficients for use in ``md_numth`` or
    ``md_numth_vec``, when number of points is prime.

    For full information please refer to the NAG Library document for
    d01gy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gyf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    npts : int
        p, the number of points to be used.

    Returns
    -------
    vk : float, ndarray, shape (`ndim`)
        The n optimal coefficients.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 5.

        (`errno` 3)
            On entry, `npts` = *<value>*.

            Constraint: `npts` must be a prime number.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The machine precision is insufficient to perform the
            computation exactly. Try reducing `npts`: `npts` =
            *<value>*.
    """

    from ..base.quad import md_numth_coeff_prime as b_md_numth_coeff_prime

    fun_name = 'naginterfaces.library.quad.md_numth_coeff_prime'

    _ndim = _b_utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = _b_utils._EngineIntScalarType.to_py(_ndim)
    _vk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ndim_py,),
    )
    try:
        b_md_numth_coeff_prime(
            _ndim,
            npts,
            _vk,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _vk_py = _vk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['vk'],
    )(
        _vk_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _vk_py

def md_numth_coeff_2prime(ndim, np1, np2):
    u"""
    Korobov optimal coefficients for use in ``md_numth`` or
    ``md_numth_vec``, when number of points is product of two primes.

    For full information please refer to the NAG Library document for
    d01gz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gzf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    np1 : int
        The larger prime factor p_1 of the number of points in the
        integration rule.

    np2 : int
        The smaller prime factor p_2 of the number of points in the
        integration rule. For maximum efficiency, p_2^2 should be close
        to p_1.

    Returns
    -------
    vk : float, ndarray, shape (`ndim`)
        The n optimal coefficients.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `np1` = *<value>* and `np2` = *<value>*.

            Constraint: `np1` > `np2`.

        (`errno` 2)
            On entry, `np2` = *<value>*.

            Constraint: `np2` >= 2.

        (`errno` 2)
            On entry, `np1` = *<value>*.

            Constraint: `np1` >= 5.

        (`errno` 3)
            On entry, `np1`*`np2` exceeds largest machine integer. `np1`
            = *<value>* and `np2` = *<value>*.

        (`errno` 4)
            On entry, `np1` = *<value>*.

            Constraint: `np1` must be a prime number.

        (`errno` 5)
            On entry, `np2` = *<value>*.

            Constraint: `np2` must be a prime number.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The machine precision is insufficient to perform the
            computation exactly. Try reducing `np1` or `np2`: `np1` =
            *<value>* and `np2` = *<value>*.
    """

    from ..base.quad import md_numth_coeff_2prime as b_md_numth_coeff_2prime

    fun_name = 'naginterfaces.library.quad.md_numth_coeff_2prime'

    _ndim = _b_utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = _b_utils._EngineIntScalarType.to_py(_ndim)
    _vk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ndim_py,),
    )
    try:
        b_md_numth_coeff_2prime(
            _ndim,
            np1,
            np2,
            _vk,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _vk_py = _vk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['vk'],
    )(
        _vk_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _vk_py

def md_sphere_bad(f, ndim, radius, epsa, epsr, icoord, method=0, data=None):
    u"""
    Multidimensional quadrature over an n-sphere, allowing for badly
    behaved integrands.

    For full information please refer to the NAG Library document for
    d01ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01jaf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated. These coordinates are given in Cartesian or
            spherical polar form according to the value of `icoord`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    ndim : int
        n, the dimension of the sphere.

    radius : float
        alpha, the radius of the sphere.

    epsa : float
        The requested absolute tolerance. If `epsa` < 0.0, its absolute
        value is used. See [Accuracy].

    epsr : float
        The requested relative tolerance.

        `epsr` < 0.0

          Its absolute value is used.

        `epsr` < 10*(machine precision)

          The latter value is used as `epsr` by the function. See
          [Accuracy].

    icoord : int
        Must specify which kind of coordinates are used in `f`.

        `icoord` = 0

          Cartesian coordinates x_i, for i = 1,2,...,n.

        `icoord` = 1

          Spherical coordinates (see [Further Comments]): `x`[0] = rho;
          `x`[i-1] = theta_{i-1}, for i = 2,3,...,n.

        `icoord` = 2,

          Special spherical polar coordinates (see [Further Comments]),
          with the additional transformation rho = alpha-lambda: `x`[0]
          = lambda = alpha-rho; `x`[i-1] = theta_{i-1}, for i =
          2,3,...,n.

    method : int, optional
        Must specify the transformation to be used by the function. The
        choice depends on the behaviour of the integrand and on the
        required accuracy.

        For well-behaved functions and functions with mild singularities
        on the surface of the sphere only:

        `method` = 1

          Low accuracy required.

        `method` = 2

          High accuracy required.

        For functions with severe singularities on the surface of the
        sphere only:

        `method` = 3

          Low accuracy required.

        `method` = 4

          High accuracy required.

          (in this case `icoord` must be set to `icoord` = 2, and the
          function defined in special spherical coordinates).

        For functions with a singularity at the centre of the sphere
        (and possibly with singularities on the surface as well):

        `method` = 5

          Low accuracy required.

        `method` = 6

          High accuracy required.

        `method` = 0 can be used as a default value and is equivalent
        to:

        - `method` = 1 if `epsr` > 10^(-6), and

        - `method` = 2 if `epsr` <= 10^(-6).

        The distinction between low and high required accuracies, as
        mentioned above, depends also on the behaviour of the function.

        Roughly one may assume the critical value of `epsa` and `epsr`
        to be 10^(-6), but the critical value will be smaller for a
        well-behaved integrand and larger for an integrand with severe
        singularities.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    esterr : float
        An estimate of the modulus of the absolute error.

    nevals : int
        The number of function evaluations used.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `radius` = *<value>*.

            Constraint: `radius` >= 0.0.

        (`errno` 4)
            On entry, `method` = 4 and `icoord` = *<value>*.

            Constraint: when `method` = 4, `icoord` = 2.

        (`errno` 4)
            On entry, `method` = 3 and `icoord` = *<value>*.

            Constraint: when `method` = 3, `icoord` = 2.

        (`errno` 4)
            On entry, `icoord` = 2 and `method` = *<value>*.

            Constraint: when `icoord` = 2, `method` = 3 or 4.

        (`errno` 4)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 2.

        (`errno` 4)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The required accuracy could not be achieved.

        (`errno` 2)
            The required accuracy could not be achieved.

        (`errno` 3)
            The required accuracy could not be achieved.

            If `method` = 0, 1 or 2, setting `method` = 3 or 4 may help.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import md_sphere_bad as b_md_sphere_bad

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.md_sphere_bad'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_md_sphere_bad(
            _f,
            ndim,
            radius,
            epsa,
            epsr,
            method,
            icoord,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _result_py = _return_scalars.result
    _esterr_py = _return_scalars.esterr
    _nevals_py = _return_scalars.nevals
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'esterr', 'nevals'],
    )(
        _result_py,
        _esterr_py,
        _nevals_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def md_simplex(ndim, vert, f, minord, finvls, data=None):
    u"""
    Multidimensional quadrature over an n-simplex.

    For full information please refer to the NAG Library document for
    d01pa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01paf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    vert : float, array-like, shape (`ndim`+1, 2*(`ndim`+1))
        `vert`[i-1,j-1] must be set to the jth component of the ith
        vertex for the simplex integration region, for j = 1,2,...,n,
        for i = 1,2,...,n+1. If `minord` > 0, `vert` must be unchanged
        since the previous call of ``md_simplex``.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (ndim)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    minord : int
        Must specify the highest order of the approximations currently
        available in the array `finvls`.

        `minord` = 0

          Indicates an initial call.

        `minord` > 0

          Indicates that
          `finvls`[0],`finvls`[1],...,`finvls`[`minord`-1] have already
          been computed in a previous call of ``md_simplex``.

    finvls : float, array-like, shape (maxord)
        If `minord` > 0,
        `finvls`[0],`finvls`[1],...,`finvls`[`minord`-1] must contain
        approximations to the integral previously computed by
        ``md_simplex``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    vert : float, ndarray, shape (`ndim`+1, 2*(`ndim`+1))
        These values are unchanged. The rest of the array `vert` is used
        for workspace and contains information to be used if another
        call of ``md_simplex`` is made with `minord` > 0. In particular
        `vert`[n,2n+1] contains the volume of the simplex.

    minord : int
        `minord` = maxord.

    finvls : float, ndarray, shape (maxord)
        Contains these values unchanged, and the newly computed values
        `finvls`[`minord`],`finvls`[`minord`+1],...,`finvls`[maxord-1].
        `finvls`[j-1] is an approximation to the integral of polynomial
        degree 2j-1.

    esterr : float
        An absolute error estimate for `finvls`[maxord-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, maxord = *<value>* and `minord` = *<value>*.

            Constraint: maxord > `minord`.

        (`errno` 1)
            On entry, `minord` = *<value>*.

            Constraint: `minord` >= 0.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 2.

        (`errno` 2)
            The volume of the simplex integration region is too large or
            too small to be represented on the machine.
    """

    from ..base.quad import md_simplex as b_md_simplex

    def cb_f(ndim, x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.quad.md_simplex'

    _vert = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vert'},
        vert,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _finvls = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'finvls'},
        finvls,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _maxord_py = _b_utils._EngineFloat64ArrayType.get_shape(_finvls, {'fun_name': fun_name, 'entity': 'finvls'}, exp_rank=1)[0]
    _maxord = _b_utils._EngineIntScalarType(
        _maxord_py, {'fun_name': fun_name, 'entity': '_maxord'},
    ).to_ctypes()
    _return_scalars = b_md_simplex(
        ndim,
        _vert,
        _f,
        minord,
        _maxord,
        _finvls,
        data,
    )
    _vert_py = _vert
    _minord_py = _return_scalars.minord
    _finvls_py = _finvls
    _esterr_py = _return_scalars.esterr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['vert', 'minord', 'finvls', 'esterr'],
    )(
        _vert_py,
        _minord_py,
        _finvls_py,
        _esterr_py,
    )
    return _return_data

def dim1_gen_vec_multi_rcomm(irevcm, a, b, needi, x, nx, fm, dinest, errest, comm, nagcal):
    u"""
    One-dimensional quadrature, adaptive, finite interval, multiple
    integrands, vectorized abscissae, reverse communication.

    For full information please refer to the NAG Library document for
    d01ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01raf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 1.

        `irevcm` = 1

          Sets up data structures in `comm`['icom'] and `comm`['com']
          and starts a new integration.

        On intermediate entry: `irevcm` should normally be left
        unchanged. However, if `irevcm` is set to a negative value,
        ``dim1_gen_vec_multi_rcomm`` will terminate, (see `irevcm` = 11
        and `irevcm` = 12 above).

    a : float
        a, the lower bound of the domain.

    b : float
        b, the upper bound of the domain.

        If |b-a| < 10 epsilon, where epsilon is the machine precision
        (see ``machine.precision``), ``dim1_gen_vec_multi_rcomm`` will
        return `dinest`[j-1] = `errest`[j-1] = 0.0, for j = 1,2,...,n_i.

    needi : int, ndarray, shape (ni), modified in place
        On initial entry: need not be set.

        On intermediate exit: `needi`[j-1] indicates what action must be
        taken for integral j = 1,2,...n_i (see `irevcm`).

        `needi`[j-1] = 0

          Do not provide f_j(x_i). Any provided values will be ignored.

        `needi`[j-1] = 1

          The values f_j(x_i) must be provided in `fm`[j-1,i-1], for i =
          1,2,...,n_x.

        `needi`[j-1] = 2

          The values f_j(x_i) are not required, however the error
          estimate for integral j is still above the requested
          tolerance. If you wish to provide values for the evaluation of
          integral j, set `needi`[j-1] = 1, and supply f_j(x_i) in
          `fm`[j-1,i-1], for i = 1,2,...,n_x.

        `needi`[j-1] = 3

          The error estimate for integral j cannot be improved to below
          the requested tolerance directly, either because no more new
          splits may be performed due to exhaustion, or due to the
          detection of extremely bad integrand behaviour. However,
          providing the values f_j(x_i) may still lead to some
          improvement, and may lead to an acceptable error estimate
          indirectly using Wynn's epsilon algorithm. If you wish to
          provide values for the evaluation of integral j, set
          `needi`[j-1] = 1, and supply f_j(x_i) in `fm`[j-1,i-1], for i
          = 1,2,...,n_x.

        `needi`[j-1] = 4

          The error estimate of integral j is below the requested
          tolerance. If you believe this to be false, if for example the
          result in `dinest`[j-1] is greatly different to what you may
          expect, you may force the algorithm to re-evaluate this
          conclusion by including the evaluations of integrand j at x_i,
          for i = 1,2,...,n_x, and setting `needi`[j-1] = 1. Integral
          and error estimation will be performed again during the next
          iteration.

        On intermediate entry: `needi`[j-1] may be used to indicate what
        action you have taken for integral j.

        `needi`[j-1] = 1

          You have provided the values f_j(x_i) in `fm`[j-1,i-1], for i
          = 1,2,...,n_x.

        `needi`[j-1] < 0

          You are abandoning the evaluation of integral j. The current
          values of `dinest`[j-1] and `errest`[j-1] will be returned on
          final completion.

        Otherwise you have not provided the value f_j(x_i).

        On final exit: `needi`[j-1] indicates the final state of
        integral j.

        `needi`[j-1] = 0

          The error estimate for F_j is below the requested tolerance.

        `needi`[j-1] = 1

          The error estimate for F_j is below the requested tolerance
          after extrapolation.

        `needi`[j-1] = 2

          The error estimate for F_j is above the requested tolerance.

        `needi`[j-1] = 3

          The error estimate for F_j is above the requested tolerance,
          and extremely bad behaviour of integral j has been detected.

        `needi`[j-1] < 0

          You prohibited further evaluation of integral j.

    x : float, ndarray, shape (lenx), modified in place
        On initial entry: if 'Primary Division Mode' = 'AUTOMATIC', `x`
        need not be set. This is the default behaviour.

        If 'Primary Division Mode' = 'MANUAL', `x` is used to supply a
        set of initial 'break-points' inside the domain of integration.

        Specifically, `x`[i-1] must contain a break-point x_i^0, for i =
        1,2,...,(s_pri-1), where s_pri is the number of 'Primary
        Divisions'.

        On intermediate exit: `x`[i-1] is the abscissa x_i, for i =
        1,2,...,n_x, at which the appropriate integrals must be
        evaluated.

    nx : int
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    fm : float, array-like, shape (ldfmrq, sdfmrq)
        On initial entry: need not be set.

        On intermediate entry: if indicated by `needi`[j-1] you must
        supply the values f_j(x_i) in `fm`[j-1,i-1], for j =
        1,2,...,n_i, for i = 1,2,...,n_x.

    dinest : float, ndarray, shape (ni), modified in place
        On initial entry: need not be set.

        On intermediate entry: must not be altered.

        On exit: contains the current estimates of the ni integrals. If
        `irevcm` = 0, this will be the final solution.

    errest : float, ndarray, shape (ni), modified in place
        On initial entry: need not be set.

        On intermediate entry: must not be altered.

        On exit: contains the current error estimates for the ni
        integrals. If `irevcm` = 0, `errest` contains the final error
        estimates of the ni integrals.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    nagcal : int
        Indicates what level of printing is required.

        itrace = 0

          No printing will be displayed.

        itrace = 1

          Upon completion, any intervals where bad behaviour was
          detected will be printed, as well as the estimate of the
          definite integral and error for any integrals which behaved
          badly over this interval.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 11 or 12.

        `irevcm` requests the integrands f_j(x_i) be evaluated for all
        required j in 1,...,n_i as indicated by `needi`, and at all the
        points x_i, for i = 1,2,...,n_x.

        Abscissae x_i are provided in `x`[i-1] and f_j(x_i) must be
        returned in `fm`[j-1,i-1].

        During the initial solve phase:

        `irevcm` = 11

          Function values are required to construct the initial
          estimates of the definite integrals.

          If `needi`[j-1] = 1, f_j(x_i) must be supplied in
          `fm`[j-1,i-1].

          This will be the case unless you have abandoned the evaluation
          of specific integrals on a previous call.

          If `needi`[j-1] = 0, you have previously abandoned the
          evaluation of integral j, and hence should not supply the
          value of f_j(x_i).

          `dinest` and `errest` contain incomplete information during
          this phase.

          As such you should not abandon the evaluation of any integrals
          during this phase unless you do not require their estimate.

          If `irevcm` is set to a negative value during this phase,
          `needi`[j-1], for j = 1,2,...,n_i, will be set to this
          negative value and `errno` = -1 will be returned.

        During the adaptive solve phase:

        `irevcm` = 12

          Function values are required to improve the estimates of the
          definite integrals.

          If `needi`[j-1] = 0, any evaluation of f_j(x_i) will be
          discarded, so there is no need to provide them.

          If `needi`[j-1] = 1, f_j(x_i) must be provided in
          `fm`[j-1,i-1].

          If `needi`[j-1] = 2, 3 or 4, the current error estimate of
          integral j does not require integrand j to be evaluated and
          provided in `fm`[j-1,i-1].

          Should you choose to, integrand j can be evaluated in which
          case `needi`[j-1] must be set to 1.

          `dinest` and `errest` contain complete information during this
          phase.

          If `irevcm` is set to a negative value during this phase
          `errno` = 1, 2 or 3 will be returned and the elements of
          `needi` will reflect the current state of the adaptive
          process.

        On final exit: `irevcm` = 0.

        `irevcm` = 0

          Indicates the algorithm has completed.

    sid : int
        For advanced users.

        On intermediate exit: this argument identifies a specific set of
        abscissae, x, returned during the integration process. When a
        new set of abscissae are generated the value of this argument is
        incremented by 1. Advanced users may store calculations required
        for an identified set x, and reuse them should
        ``dim1_gen_vec_multi_rcomm`` return the same value of this
        argument, i.e., the same set of abscissae was used.

    nx : int
        On intermediate exit: n_x, the number of abscissae at which
        integrands are required.

    Other Parameters
    ----------------
    'Absolute Interval Minimum' : float
        Default = 128.0 epsilon

        r = delta_a, the absolute lower limit for a segment to be
        considered for subdivision.
        See also 'Relative Interval Minimum' and [Further Comments].

        Constraint: r >= 128 epsilon.

    'Absolute Tolerance' : float
        Default = 1024 epsilon

        r = epsilon_a, the absolute tolerance required.
        See also 'Relative Tolerance' and [Description].

        Constraint: r >= 0.0.

    'Extrapolation' : str
        Default = ON

        Activate or deactivate the use of the epsilon algorithm (Wynn).
        'Extrapolation' often reduces the number of iterations required
        to achieve the desired solution, but it can occasionally lead to
        premature convergence towards an incorrect answer.

        ON
          Use extrapolation.

        OFF
          Disable extrapolation.

    'Extrapolation Safeguard' : float
        Default = 1.0e-12

        r = epsilon_safe.
        If epsilon_q is the estimated error from the quadrature
        evaluation alone, and epsilon_ex is the error estimate
        determined using extrapolation, then the extrapolated solution
        will only be accepted if epsilon_safeepsilon_q <= epsilon_ex.

    'Maximum Subdivisions' : int
        Default = 50

        i = max_sdiv, the maximum number of subdivisions the algorithm
        may use in the adaptive phase, forming at most an additional
        (2*max_sdiv) segments.

    'Primary Divisions' : int
        Default = 1

        i = s_pri, the number of initial segments of the domain [a,b].
        By default the initial segment is the entire domain.

        Constraint: 0 < i < 1000000.

    'Primary Division Mode' : str
        Default = 'AUTOMATIC'

        Determines how the initial set of s_pri segments will be
        generated.

        AUTOMATIC
          ``dim1_gen_vec_multi_rcomm`` will automatically generate s_pri
          segments of equal size covering the interval [a,b].

        MANUAL
          ``dim1_gen_vec_multi_rcomm`` will use the break-points x_i^0,
          for i = 1,2,...,s_pri-1, supplied in `x` on initial entry to
          generate the initial segments covering [a,b]. These may be
          supplied in any order, however it will be more efficient to
          supply them in ascending (or descending if a > b) order.
          Repeated break-points are allowed, although this will generate
          fewer initial segments.

        Note: an absolute bound on the size of an initial segment of
        10.0 epsilon is automatically applied in all cases, and will
        result in fewer initial subdivisions being generated if
        automatically generated or supplied break-points result in
        segments smaller than this.

    'Prioritize Error' : str
        Default = 'LEVEL'

        Indicates how new subdivisions of segments sustaining
        unacceptable local errors for integrals should be prioritized.

        LEVEL
          Segments with lower level with unsatisfactory error estimates
          will be chosen over segments with greater error on higher
          levels. This will probably lead to more integrals being
          improved in earlier iterations of the algorithm, and hence
          will probably lead to fewer repeated returns (see argument
          `sid`), and to more integrals being satisfactorily estimated
          if computational exhaustion occurs.

        MAXERR
          The segment with the worst overall error will be split,
          regardless of level. This will more rapidly improve the worst
          integral estimates, although it will probably result in the
          fewest integrals being improved in earlier iterations, and may
          hence lead to more repeated returns (see argument `sid`), and
          potentially fewer integrals satisfying the requested
          tolerances if computational exhaustion occurs.

    'Quadrature Rule' : str
        Default = GK15

        The basic quadrature rule to be used during the integration.
        Currently 6 Gauss--Kronrod rules are available, all identifiable
        by the letters GK followed by the number of points required by
        the Kronrod rule.
        Higher order rules generally provide higher accuracy with fewer
        subdivisons.
        However, for integrands with sharp singularities, lower order
        rules may be more efficient, particularly if the integrand away
        from the singularity is well behaved.
        With higher order rules, you may need to increase the 'Absolute
        Interval Minimum' and the 'Relative Interval Minimum' to
        maintain numerical difference between the abscissae and the
        segment bounds.

        GK15
          The Gauss--Kronrod rule based on 7 Gauss points and 15 Kronrod
          points.

        GK21
          The Gauss--Kronrod rule based on 10 Gauss points and 21
          Kronrod points. This is the rule used by ``dim1_fin_bad_vec``

        GK31
          The Gauss--Kronrod rule based on 15 Gauss points and 31
          Kronrod points.

        GK41
          The Gauss--Kronrod rule based on 20 Gauss points and 41
          Kronrod points.

        GK51
          The Gauss--Kronrod rule based on 25 Gauss points and 51
          Kronrod points.

        GK61
          The Gauss--Kronrod rule based on 30 Gauss points and 61
          Kronrod points. This is the highest order rule, most suitable
          for highly oscilliatory integrals.

    'Relative Interval Minimum' : float
        Default = 1.0e-6

        r = delta_r, the relative factor in the lower limit,
        delta_r|b-a|, for a segment to be considered for subdivision.
        See also 'Absolute Interval Minimum' and [Further Comments].

        Constraint: r >= 0.0.

    'Relative Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_r, the required relative tolerance.
        See also 'Absolute Tolerance' and [Description].

        Constraint: r >= 0.0.

        Note: setting both epsilon_r = epsilon_a = 0.0 is possible,
        although it will most likely result in an excessive amount of
        computational effort.

    'Index LDI' : int
        query only

        I_ldi, the index of `comm`['icom'] required for obtaining ldi.
        See [Details of the Computation].

    'Index LDR' : int
        query only

        I_ldr, the index of `comm`['icom'] required for obtaining ldr.
        See [Details of the Computation].

    'Index NSDIV' : int
        query only

        I_nsdiv, the index of `comm`['icom'] required for obtaining
        nsdiv.
        See [Details of the Computation].

    'Index NSEG' : int
        query only

        I_nseg, the index of `comm`['icom'] required for obtaining nseg.
        See [Details of the Computation].

    'Index FCP' : int
        query only

        I_fcp, the index of `comm`['icom'] required for obtaining fcp.
        See [Details of the Computation].

    'Index EVALSP' : int
        query only

        I_evalsp, the index of `comm`['icom'] required for obtaining
        evalsp.
        See [Details of the Computation].

    'Index DSP' : int
        query only

        I_dsp, the index of `comm`['icom'] required for obtaining dsp.
        See [Details of the Computation].

    'Index ESP' : int
        query only

        I_esp, the index of `comm`['icom'] required for obtaining esp.
        See [Details of the Computation].

    'Index SINFOIP' : int
        query only

        I_sinfoip, the index of `comm`['icom'] required for obtaining
        sinfoip.
        See [Details of the Computation].

    'Index SINFORP' : int
        query only

        I_sinforp, the index of `comm`['icom'] required for obtaining
        sinforp.
        See [Details of the Computation].

    Raises
    ------
    NagValueError
        (`errno` 11)
            `irevcm` had an illegal value.

            On entry, `irevcm` = *<value>*.

        (`errno` 21)
            On entry, ni = *<value>*.

            Constraint: ni >= 1.

        (`errno` 71)
            On entry, 'Primary Division Mode' = 'MANUAL' and at least
            one supplied break-point in `x` is outside of the domain of
            integration.

        (`errno` 81)
            lenx is insufficient for the chosen options.

            On entry, lenx = *<value>*.

            Constraint: lenx >= *<value>*.

        (`errno` 1001)
            Either the option arrays `comm`['iopts'] and `comm`['opts']
            have not been initialized for ``dim1_gen_vec_multi_rcomm``,
            or they have become corrupted.

        (`errno` 1101)
            On entry, one of `comm`['icom'] and `comm`['com'] has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Evaluation of all integrals has been stopped during the
            initial phase.

        (`errno` 1)
            At least one error estimate exceeded the requested
            tolerances.

        (`errno` 2)
            Extremely bad behaviour was detected for at least one
            integral.

        (`errno` 3)
            Extremely bad behaviour was detected for at least one
            integral. At least one other integral error estimate was
            above the requested tolerance.
    """

    from ..base import _nag_d_consts_engine
    from ..base.quad import dim1_gen_vec_multi_rcomm as b_dim1_gen_vec_multi_rcomm

    fun_name = 'naginterfaces.library.quad.dim1_gen_vec_multi_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'needi'},
        needi,
    )
    _ni_py = _b_utils._EngineIntArrayType.get_shape(needi, {'fun_name': fun_name, 'entity': 'needi'}, exp_rank=1)[0]
    _ni = _b_utils._EngineIntScalarType(
        _ni_py, {'fun_name': fun_name, 'entity': '_ni'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _lenx_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _lenx = _b_utils._EngineIntScalarType(
        _lenx_py, {'fun_name': fun_name, 'entity': '_lenx'},
    ).to_ctypes()
    _comm_needs_init = _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('icom', 'com'),
        req_keys=('iopts', 'opts'),
    )
    if _b_utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (19,), validate=False) == _nag_d_consts_engine.D01RA_FLEAD_I:
        _vl_ldfmrq_py = _ni_py
    else:
        _vl_ldfmrq_py = _b_utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (38,), validate=False)
    _fm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fm'},
        fm,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'dinest'},
        dinest,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'errest'},
        errest,
    )
    try:
        _return_scalars = b_dim1_gen_vec_multi_rcomm(
            irevcm,
            _ni,
            a,
            b,
            needi,
            x,
            _lenx,
            nx,
            _fm,
            dinest,
            errest,
            comm,
            nagcal,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _irevcm_py = _return_scalars.irevcm
    _nx_py = _return_scalars.nx
    _sid_py = _return_scalars.sid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'sid', 'nx'],
    )(
        _irevcm_py,
        _sid_py,
        _nx_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.quad.withdraw_1d_gen_vec_multi_diagnostic',
    mkwdrn='28',
)
def withdraw_1d_gen_vec_multi_diagnostic(dinest, errest, comm, monit=None, data=None):
    u"""
    Diagnostic function for ``dim1_gen_vec_multi_rcomm``.

    Note: ``withdraw_1d_gen_vec_multi_diagnostic`` will be removed in
    naginterfaces 28.0.0.
    There is no replacement for this routine.

    For full information please refer to the NAG Library document for
    d01rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rbf.html

    Parameters
    ----------
    dinest : float, array-like, shape (ni)
        `dinest`[j-1] contains the current estimate of the definite
        integral of integrand j, for j = 1,2,...,n_i.

    errest : float, array-like, shape (ni)
        `errest`[j-1] contains the current error estimate associated
        with integral j, for j = 1,2,...,n_i.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_gen_vec_multi_rcomm``.

    monit : None or callable (dinest, errest) = monit(ns, dinest,
        errest, fcount, sinfoi, evals, sinfor, fs, es, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `monit` allows you to examine details of the adaptive process
        after a call of ``dim1_gen_vec_multi_rcomm`` returning irevcm =
        11 or 12.

        Additionally, after a call of ``dim1_gen_vec_multi_rcomm``
        returning irevcm = 0, `monit` may enhance the computed solution.

        Parameters
        ~~~~~~~~~~
        ns : int
            n_s, the number of segments formed during the adaptive
            procedure that can currently be examined. n_s =
            2n_sdiv+s_pri, where n_sdiv is the number of segments that
            have been subdivided (the number of subdivisions).

        dinest : float, ndarray, shape (ni)
            `dinest`[j-1] contains the current estimate of the definite
            integral of integrand j, for j = 1,2,...,n_i.

        errest : float, ndarray, shape (ni)
            `errest`[j-1] contains the current error estimate associated
            with integral j, for j = 1,2,...,n_i.

        fcount : int, ndarray, shape (ni)
            `fcount`[j-1] contains the number of different
            approximations of integral j calculated so far, for j =
            1,2,...,n_i.

        sinfoi : int, ndarray, shape ((`ns`-1)*ldi+5)
            `sinfoi` contains information about the hierarchy of
            splitting each segment k, for k = 1,2,...,n_s.

            `sinfoi`[ldi*(k-1)]

              Contains a unique identifier, the sid, related to segment
              k.

            `sinfoi`[ldi*(k-1)+1]

              Contains the parent segment number of segment k, the
              reference to the segment that was split to create segment
              k.

            `sinfoi`[ldi*(k-1)+2] and `sinfoi`[ldi*(k-1)+3]

              Contain the segment numbers of the two child segments
              formed from segment k, if segment k has been split. If
              segment k has not been split, these will be negative.

            `sinfoi`[ldi*(k-1)+4]

              Contains the level at which the segment exists,
              corresponding to n_a+1, where n_a is the number of
              ancestor segments of segment k.

              If `sinfoi`[ldi*(k-1)+4] < 0, this segment is considered
              too small for further splitting, and its level is
              |`sinfoi`[ldi*(k-1)+4]|.

        evals : int, ndarray, shape ((`ns`-1)*ldi+ni)
            Contains information to indicate whether an estimate of the
            integral j has been obtained over segment k, and if so
            whether this evaluation still contributes to the
            approximation in `dinest`[j-1], for k = 1,2,...,n_s, for j =
            1,2,...,n_i.

            `evals`[ldi*(k-1)+j-1] = 0

              Indicates that integral j has not been evaluated over
              segment k.

            `evals`[ldi*(k-1)+j-1] = 1

              Indicates that integral j has been evaluated over segment
              k, and that this evaluation contributes to the total
              approximation `dinest`[j-1].

            `evals`[ldi*(k-1)+j-1] = 2

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation `dinest`[j-1], and that you have requested
              no further evaluation of this integral at this segment by
              setting needi[j-1] < 0 in ``dim1_gen_vec_multi_rcomm``.

            `evals`[ldi*(k-1)+j-1] = 3

              Indicates that integral j has been evaluated over segment
              k, and this evaluation no longer contributes to
              `dinest`[j-1].

            `evals`[ldi*(k-1)+j-1] = 4

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation `dinest`[j-1], and that this segment is too
              small for any further splitting to be performed. Integral
              j also has a local error estimate over this segment above
              the requested tolerance. Such segments cause
              ``dim1_gen_vec_multi_rcomm`` to return `errno` = 2 or 3,
              indicating extremely bad behaviour.

            `evals`[ldi*(k-1)+j-1] = 5

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation of `dinest`[j-1], and that this segment is
              too small for any further splitting to be performed. The
              local error estimate is however below the requested
              tolerance.

        sinfor : float, ndarray, shape ((`ns`-1)*ldr+2)
            `sinfor` contains the bounds of each segment k, for k =
            1,2,...,n_s.

            `sinfor`[ldr*(k-1)]

              Contains the lower bound of segment k.

            `sinfor`[ldr*(k-1)+1]

              Contains the upper bound of segment k.

        fs : float, ndarray, shape ((`ns`-1)*ldr+ni)
            `fs`[ldr*(k-1)+j-1] contains the definite integral estimate
            of the jth integral over the kth segment, for k =
            1,2,...,n_s, for j = 1,2,...,n_i.

        es : float, ndarray, shape ((`ns`-1)*ldr+ni)
            `es`[ldr*(k-1)+j-1] contains the definite integral error
            estimate of the jth integral over the kth segment, for k =
            1,2,...,n_s, for j = 1,2,...,n_i.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        dinest : float, array-like, shape (ni)
            You may refine the estimates in `dinest`. This should only
            be done after ``dim1_gen_vec_multi_rcomm`` has returned
            irevcm = 0.

        errest : float, array-like, shape (ni)
            You may refine the estimates in `errest`. This should only
            be done after ``dim1_gen_vec_multi_rcomm`` has returned
            irevcm = 0.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    dinest : float, ndarray, shape (ni)
        This argument is not altered by
        ``withdraw_1d_gen_vec_multi_diagnostic`` directly. It may be
        changed by `monit`.

    errest : float, ndarray, shape (ni)
        This argument is not altered by
        ``withdraw_1d_gen_vec_multi_diagnostic`` directly. It may be
        changed by `monit`.

    licusd : int
        The number of elements of the array `comm`['icom'], passed to
        ``dim1_gen_vec_multi_rcomm``, actually used.

    lcusd : int
        The number of elements of `comm`['com'] used by
        ``dim1_gen_vec_multi_rcomm``.

    Raises
    ------
    NagValueError
        (`errno` 21)
            On entry, `ni` is not consistent with that used to construct
            the communication arrays.

            On entry, `ni` = *<value>*.

            Constraint: ni = *<value>*.

        (`errno` 1101)
            Either the communication arrays have not been created by
            ``dim1_gen_vec_multi_rcomm``, or they have become corrupted.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.quad import withdraw_1d_gen_vec_multi_diagnostic as b_withdraw_1d_gen_vec_multi_diagnostic

    fun_name = 'naginterfaces.library.quad.withdraw_1d_gen_vec_multi_diagnostic'

    cb_monit_name = fun_name + ':monit'

    def cb_monit(ni, ns, dinest, errest, fcount, sinfoi, evals, ldi, sinfor, fs, es, ldr, data=None):
        """Wrapping user's monit. Called as base monit."""
        if data is not None:
            _dinest_monit, _errest_monit = monit(
                ns,
                dinest,
                errest,
                fcount,
                sinfoi,
                evals,
                sinfor,
                fs,
                es,
                data,
            )
        else:
            _dinest_monit, _errest_monit = monit(
                ns,
                dinest,
                errest,
                fcount,
                sinfoi,
                evals,
                sinfor,
                fs,
                es,
            )
        dinest[:] = _utils._to_ndarray(
            {'fun_name': cb_monit_name, 'returned': 'dinest'},
            _dinest_monit,
            copy=False,
            exp_shape=(ni,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        errest[:] = _utils._to_ndarray(
            {'fun_name': cb_monit_name, 'returned': 'errest'},
            _errest_monit,
            copy=False,
            exp_shape=(ni,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _b_utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = None if monit is None else cb_monit
    _dinest = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dinest'},
        dinest,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ni_py = _b_utils._EngineFloat64ArrayType.get_shape(_dinest, {'fun_name': fun_name, 'entity': 'dinest'}, exp_rank=1)[0]
    _ni = _b_utils._EngineIntScalarType(
        _ni_py, {'fun_name': fun_name, 'entity': '_ni'},
    ).to_ctypes()
    _errest = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'errest'},
        errest,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = b_withdraw_1d_gen_vec_multi_diagnostic(
        _monit,
        _ni,
        _dinest,
        _errest,
        comm,
        data,
    )
    _dinest_py = _dinest
    _errest_py = _errest
    _licusd_py = _return_scalars.licusd
    _lcusd_py = _return_scalars.lcusd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dinest', 'errest', 'licusd', 'lcusd'],
    )(
        _dinest_py,
        _errest_py,
        _licusd_py,
        _lcusd_py,
    )
    return _return_data

def dim1_gen_vec_multi_dimreq(ni, iopts, opts):
    u"""
    Determine required array dimensions for
    ``dim1_gen_vec_multi_rcomm``.

    For full information please refer to the NAG Library document for
    d01rc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rcf.html

    Parameters
    ----------
    ni : int
        n_i, the number of integrals which will be approximated in the
        subsequent call to ``dim1_gen_vec_multi_rcomm``.

    iopts : int, array-like, shape (100)
        The integer option array as returned by ``opt_set``.

    opts : float, array-like, shape (100)
        The real option array as returned by ``opt_set``.

    Returns
    -------
    lenxrq : int
        lenxrq, the minimum dimension of the array x that can be used in
        a subsequent call to ``dim1_gen_vec_multi_rcomm``.

    ldfmrq : int
        ldfmrq, the minimum leading dimension of the array fm that can
        be used in a subsequent call to ``dim1_gen_vec_multi_rcomm``.

    sdfmrq : int
        sdfmrq, the minimum second dimension of the array fm that can be
        used in a subsequent call to ``dim1_gen_vec_multi_rcomm``.

        Note: the minimum dimension of the array fm is ldfmrq*sdfmrq.

    licmin : int
        licmin, the minimum dimension of the array icom that must be
        passed to ``dim1_gen_vec_multi_rcomm`` to enable it to calculate
        a single approximation to all the n_i integrals over the
        interval [a,b] with s_pri initial segments.

    licmax : int
        licmax the dimension of the array icom that must be passed to
        ``dim1_gen_vec_multi_rcomm`` to enable it to exhaust the
        adaptive process controlled by the currently set options for the
        n_i integrals over the interval [a,b] with s_pri initial
        segments.

    lcmin : int
        lcmin, the minimum dimension of the array com that must be
        passed to ``dim1_gen_vec_multi_rcomm`` to enable it to calculate
        a single approximation to all the n_i integrals over the
        interval [a,b] with s_pri initial segments.

    lcmax : int
        lcmax, the dimension of the array com that must be passed to
        ``dim1_gen_vec_multi_rcomm`` to enable it to exhaust the
        adaptive process controlled by the currently set options for the
        n_i integrals over the interval [a,b] with s_pri initial
        segments.

    Raises
    ------
    NagValueError
        (`errno` 21)
            On entry, `ni` = *<value>*.

            Constraint: `ni` > 0.

        (`errno` 1001)
            One of the option arrays `iopts` or `opts` has become
            corrupted. Re-initialize the arrays using ``opt_set``.
    """

    from ..base.quad import dim1_gen_vec_multi_dimreq as b_dim1_gen_vec_multi_dimreq

    fun_name = 'naginterfaces.library.quad.dim1_gen_vec_multi_dimreq'

    _iopts = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iopts'},
        iopts,
        copy=False,
    )
    _opts = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'opts'},
        opts,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = b_dim1_gen_vec_multi_dimreq(
        ni,
        _iopts,
        _opts,
    )
    _lenxrq_py = _return_scalars.lenxrq
    _ldfmrq_py = _return_scalars.ldfmrq
    _sdfmrq_py = _return_scalars.sdfmrq
    _licmin_py = _return_scalars.licmin
    _licmax_py = _return_scalars.licmax
    _lcmin_py = _return_scalars.lcmin
    _lcmax_py = _return_scalars.lcmax
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lenxrq', 'ldfmrq', 'sdfmrq', 'licmin', 'licmax', 'lcmin', 'lcmax'],
    )(
        _lenxrq_py,
        _ldfmrq_py,
        _sdfmrq_py,
        _licmin_py,
        _licmax_py,
        _lcmin_py,
        _lcmax_py,
    )
    return _return_data

def dim1_fin_gonnet_vec(a, b, f, epsabs, epsrel, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Gonnet, allowing for badly behaved integrands.

    For full information please refer to the NAG Library document for
    d01rg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rgf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

        Note: if `a` = `b`, the function will immediately return
        `dinest` = 0.0, `errest` = 0.0 and `nevals` = 0.

    f : callable (fv, iflag) = f(x, iflag, data=None)
        `f` must return the value of the integrand f at a set of points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (nx)
            The abscissae, x_i, for i = 1,2,...,nx, at which function
            values are required.

        iflag : int
            `iflag` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (nx)
            `fv` must contain the values of the integrand f. `fv`[i-1] =
            f(x_i) for all i = 1,2,...,nx.

        iflag : int
            Set `iflag` < 0 to force an immediate exit with `errno` =
            -1.

    epsabs : float
        The absolute accuracy required.

        If `epsabs` is negative, |`epsabs`| is used.

        See [Accuracy].

        If `epsabs` = 0.0, only the relative error will be used.

    epsrel : float
        The relative accuracy required.

        If `epsrel` is negative, |`epsrel`| is used.

        See [Accuracy].

        If `epsrel` = 0.0, only the absolute error will be used
        otherwise the actual value of `epsrel` used by
        ``dim1_fin_gonnet_vec`` is max(machine precision, |`epsrel`|).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    dinest : float
        The estimate of the definite integral `f`.

    errest : float
        The error estimate of the definite integral `f`.

    nevals : int
        The total number of points at which the integrand, f, has been
        evaluated.

    Raises
    ------
    NagValueError
        (`errno` 14)
            Both `epsabs` = 0.0 and `epsrel` = 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The requested accuracy was not achieved. Consider using
            larger values of `epsabs` and `epsrel`.

        (`errno` 2)
            The integral is probably divergent or slowly convergent.
    """

    from ..base.quad import dim1_fin_gonnet_vec as b_dim1_fin_gonnet_vec

    fun_name = 'naginterfaces.library.quad.dim1_fin_gonnet_vec'

    cb_f_name = fun_name + ':f'

    def cb_f(x, nx, fv, iflag, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fv_f, _iflag_f = f(
                x,
                iflag,
                data,
            )
        else:
            _fv_f, _iflag_f = f(
                x,
                iflag,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fv'},
            _fv_f,
            copy=False,
            exp_shape=(nx,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _iflag_f_base = _iflag_f
        return _iflag_f_base

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_dim1_fin_gonnet_vec(
            a,
            b,
            _f,
            epsabs,
            epsrel,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _dinest_py = _return_scalars.dinest
    _errest_py = _return_scalars.errest
    _nevals_py = _return_scalars.nevals
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dinest', 'errest', 'nevals'],
    )(
        _dinest_py,
        _errest_py,
        _nevals_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_gauss_wres(key, a, b, n):
    u"""
    Pre-computed weights and abscissae for Gaussian quadrature rules,
    restricted choice of rule.

    For full information please refer to the NAG Library document for
    d01tb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tbf.html

    Parameters
    ----------
    key : int
        Indicates the quadrature formula.

        `key` = 0

          Gauss--Legendre quadrature on a finite interval, using normal
          weights.

        `key` = 3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          normal weights.

        `key` = -3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          adjusted weights.

        `key` = 4

          Gauss--Hermite quadrature on an infinite interval, using
          normal weights.

        `key` = -4

          Gauss--Hermite quadrature on an infinite interval, using
          adjusted weights.

        `key` = -5

          Rational Gauss quadrature on a semi-infinite interval, using
          adjusted weights.

    a : float
        The quantities a and b as described in the appropriate
        sub-section of [Description].

    b : float
        The quantities a and b as described in the appropriate
        sub-section of [Description].

    n : int
        n, the number of weights and abscissae to be returned.

    Returns
    -------
    weight : float, ndarray, shape (`n`)
        The `n` weights.

    abscis : float, ndarray, shape (`n`)
        The `n` abscissae.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `key` = *<value>*.

            Constraint: `key` = 0, 3, -3, 4, -4 or -5.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for Gauss-Laguerre
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`b`) > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for Gauss-Hermite
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for rational Gauss
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`a`+`b`) > 0.0.

        (`errno` 14)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The `n`-point rule is not among those stored.

            On entry: `n` = *<value>*.

            `n`-rule used: `n` = *<value>*.

        (`errno` 2)
            Underflow occurred in calculation of normal weights.

            Reduce `n` or use adjusted weights: `n` = *<value>*.

        (`errno` 3)
            No nonzero weights were generated for the provided
            parameters.
    """

    from ..base.quad import dim1_gauss_wres as b_dim1_gauss_wres

    fun_name = 'naginterfaces.library.quad.dim1_gauss_wres'

    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _weight = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _abscis = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_dim1_gauss_wres(
            key,
            a,
            b,
            _n,
            _weight,
            _abscis,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _weight_py = _weight
    _abscis_py = _abscis
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['weight', 'abscis'],
    )(
        _weight_py,
        _abscis_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def dim1_gauss_wrec(a, b, c, muzero):
    u"""
    Calculation of weights and abscissae for Gaussian quadrature rules,
    method of Golub and Welsch.

    For full information please refer to the NAG Library document for
    d01td

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tdf.html

    Parameters
    ----------
    a : float, array-like, shape (n)
        `a` contains the coefficients a(j).

    b : float, array-like, shape (n)
        `b` contains the coefficients b(j).

    c : float, array-like, shape (n)
        `c` contains the coefficients c(j).

    muzero : float
        `muzero` contains the definite integral of the weight function
        for the interval of interest.

    Returns
    -------
    b : float, ndarray, shape (n)
        Elements of this argument are altered to make the underlying
        eigenvalue problem symmetric.

    c : float, ndarray, shape (n)
        Elements of this argument are altered to make the underlying
        eigenvalue problem symmetric.

    weight : float, ndarray, shape (n)
        `weight`[j-1] contains the weight corresponding to the jth
        abscissa.

    abscis : float, ndarray, shape (n)
        `abscis`[j-1] the jth abscissa.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The number of weights and abscissae requested (n) is less
            than 1: n = *<value>*.

        (`errno` 4)
            Unexpected failure in eigenvalue computation. Please contact
            NAG.

        (`errno` 5)
            The algorithm failed to converge. The ith diagonal was not
            zero: i = *<value>*.
    """

    from ..base.quad import dim1_gauss_wrec as b_dim1_gauss_wrec

    fun_name = 'naginterfaces.library.quad.dim1_gauss_wrec'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _weight = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _abscis = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_gauss_wrec(
        _n,
        _a,
        _b,
        _c,
        muzero,
        _weight,
        _abscis,
    )
    _b_py = _b
    _c_py = _c
    _weight_py = _weight
    _abscis_py = _abscis
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b', 'c', 'weight', 'abscis'],
    )(
        _b_py,
        _c_py,
        _weight_py,
        _abscis_py,
    )
    return _return_data

def dim1_gauss_recm(n, mu):
    u"""
    Generates recursion coefficients needed by ``dim1_gauss_wrec`` to
    calculate a Gaussian quadrature rule.

    For full information please refer to the NAG Library document for
    d01te

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tef.html

    Parameters
    ----------
    n : int
        n, the number of weights and abscissae required.

    mu : float, array-like, shape (2*`n`+1)
        `mu`[i] must contain the value of the moment with respect to x^i
        i.e., intw(x)x^idx, for i = 0,1,...,2n.

    Returns
    -------
    a : float, ndarray, shape (`n`)
        Values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    b : float, ndarray, shape (`n`)
        Values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    c : float, ndarray, shape (`n`)
        Values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The number of weights and abscissae requested (`n`) is less
            than 1: `n` = *<value>*.

        (`errno` 2)
            The problem is too ill conditioned, it breaks down at row
            *<value>*.
    """

    from ..base.quad import dim1_gauss_recm as b_dim1_gauss_recm

    fun_name = 'naginterfaces.library.quad.dim1_gauss_recm'

    _mu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'mu'},
        mu,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _a = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_dim1_gauss_recm(
        _n,
        _mu,
        _a,
        _b,
        _c,
    )
    _a_py = _a
    _b_py = _b
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'b', 'c'],
    )(
        _a_py,
        _b_py,
        _c_py,
    )
    return _return_data

def dim1_gauss_vec(key, a, b, n, f, data=None):
    u"""
    One-dimensional Gaussian quadrature, choice of weight functions
    (vectorized).

    For full information please refer to the NAG Library document for
    d01ua

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01uaf.html

    Parameters
    ----------
    key : int
        Indicates the quadrature formula.

        `key` = 0

          Gauss--Legendre quadrature on a finite interval, using normal
          weights.

        `key` = 3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          normal weights.

        `key` = -3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          adjusted weights.

        `key` = 4

          Gauss--Hermite quadrature on an infinite interval, using
          normal weights.

        `key` = -4

          Gauss--Hermite quadrature on an infinite interval, using
          adjusted weights.

        `key` = -5

          Rational Gauss quadrature on a semi-infinite interval, using
          adjusted weights.

    a : float
        The quantities a and b as described in the appropriate
        subsection of [Description].

    b : float
        The quantities a and b as described in the appropriate
        subsection of [Description].

    n : int
        n, the number of abscissae to be used.

    f : callable (fv, iflag) = f(x, iflag, data=None)
        `f` must return the value of the integrand f, or the normalized
        integrand g, at a specified point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (nx)
            The abscissae, x_i, for i = 1,2,...,n_x at which function
            values are required.

        iflag : int
            `iflag` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (nx)
            If adjusted weights are used, the values of the integrand f.
            `fv`[i-1] = f(x_i), for i = 1,2,...,n_x.

            Otherwise the values of the normalized integrand g.
            `fv`[i-1] = g(x_i), for i = 1,2,...,n_x.

        iflag : int
            Set `iflag` < 0 if you wish to force an immediate exit from
            ``dim1_gauss_vec`` with `errno` = -1.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    dinest : float
        The estimate of the definite integral.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `key` = *<value>*.

            Constraint: `key` = 0, 3, -3, 4, -4 or -5.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`b`) > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`a`+`b`) > 0.0.

        (`errno` 14)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The n-point rule is not among those stored.

            On entry: `n` = *<value>*.

            n-point rule used: `n` = *<value>*.

        (`errno` 2)
            Underflow occurred in calculation of normal weights.

            Reduce `n` or use adjusted weights: `n` = *<value>*.

        (`errno` 3)
            No nonzero weights were generated for the provided
            parameters.
    """

    from ..base.quad import dim1_gauss_vec as b_dim1_gauss_vec

    fun_name = 'naginterfaces.library.quad.dim1_gauss_vec'

    cb_f_name = fun_name + ':f'

    def cb_f(x, nx, fv, iflag, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fv_f, _iflag_f = f(
                x,
                iflag,
                data,
            )
        else:
            _fv_f, _iflag_f = f(
                x,
                iflag,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fv'},
            _fv_f,
            copy=False,
            exp_shape=(nx,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _iflag_f_base = _iflag_f
        return _iflag_f_base

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalar = b_dim1_gauss_vec(
            key,
            a,
            b,
            n,
            _f,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _dinest_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dinest'],
    )(
        _dinest_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _dinest_py

def dim1_inf_exp_wt(f, n, data=None):
    u"""
    Non-automatic function to evaluate int_0^infinity exp(-x^2)f(x)dx.

    For full information please refer to the NAG Library document for
    d01ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ubf.html

    Parameters
    ----------
    f : callable (fv, istop) = f(x, istop, data=None)
        `f` must return the integrand function values f(x_i) for the
        given x_i, for i = 1,2,...,`n`.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The points at which the integrand function f must be
            evaluated.

        istop : int
            `istop` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fv : float, array-like, shape (n)
            `fv`[i-1] must contain the value of the integrand f(x_i)
            evaluated at the point `x`[i-1], for i = 1,2,...,`n`.

        istop : int
            You may set `istop` to a negative number if at any time it
            is impossible to evaluate the function f(x). In this case
            ``dim1_inf_exp_wt`` halts with errno set to the value of
            `istop` and the value returned in `ans` will be that of a
            non-signalling NaN.

    n : int
        `n` specifies the number of weights and abscissae to be used.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    ans : float
        If no exception or warning is raised, this argument contains an
        approximation to the integral. Otherwise, this argument will be
        a non-signalling NaN.

    Raises
    ------
    NagValueError
        (`errno` i < 0)
            The user has halted the calculation.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: 1 <= `n` <= 25.

        (`errno` 2)
            On entry, `n` = *<value>*.

            `n` is not one of the allowed values.
    """

    from ..base.quad import dim1_inf_exp_wt as b_dim1_inf_exp_wt

    fun_name = 'naginterfaces.library.quad.dim1_inf_exp_wt'

    cb_f_name = fun_name + ':f'

    def cb_f(x, fv, n, istop, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fv_f, _istop_f = f(
                x,
                istop,
                data,
            )
        else:
            _fv_f, _istop_f = f(
                x,
                istop,
            )
        fv[:] = _utils._to_ndarray(
            {'fun_name': cb_f_name, 'returned': 'fv'},
            _fv_f,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _istop_f_base = _istop_f
        return _istop_f_base

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _return_scalar = b_dim1_inf_exp_wt(
        _f,
        n,
        data,
    )
    _ans_py = _return_scalar
    return _ans_py

def opt_set(optstr, comm):
    u"""
    Option setting function.

    For full information please refer to the NAG Library document for
    d01zk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01zkf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option to be set.

        Initialize = function name

          Initialize the option arrays `comm`['iopts'] and
          `comm`['opts'] for use with function function name, where
          function name is the short name associated with the function
          of interest.

        Defaults

          Resets all options to their default values.

        option = optval

          See [Optional Parameters] for details of valid values for
          option and optval. The equals sign (=) delimiter must be used
          to separate the option from its optval value.

        `optstr` is case insensitive.

        Each token in the option and optval component must be separated
        by at least one space.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 14)
            On entry, attempting to initialize the option arrays but
            specified function name was not valid: name = *<value>*.

        (`errno` 21)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 12)
            On entry, the expected delimiter '=' was not found in
            `optstr`: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to an
            integer: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to a real:
            `optstr` = *<value>*.

        (`errno` 15)
            On entry, the optval supplied for the integer option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 16)
            On entry, the optval supplied for the real option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 17)
            On entry, the optval supplied for the character option is
            not valid.

            `optstr` = *<value>*.
    """

    from ..base.quad import opt_set as b_opt_set

    b_opt_set(
        optstr,
        comm,
    )

def opt_get(optstr, comm):
    u"""
    Option getting function.

    For full information please refer to the NAG Library document for
    d01zl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01zlf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option whose current value is required.
        See [Optional Parameters] for information on valid options. In
        addition, the following is a valid option:

        Identify

          In which case ``opt_get`` returns in cvalue the 6 character
          function name supplied to ``opt_set`` when the option arrays
          `comm`['iopts'] and `comm`['opts'] were initialized.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    Returns
    -------
    optvalue : dict
        The option-value ``dict``, with the following keys:

        ``'value'`` : float, int or str
            The value of the requested option.

        ``'annotation'`` : None or str
            Possible additional information about the option value.

    Raises
    ------
    NagValueError
        (`errno` 61)
            The arrays `comm`['iopts'] and `comm`['opts'] have either
            not been initialized, have become corrupted, or are not
            compatible with this option setting function.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 41)
            On entry, `optstr` indicates a character option, but cvalue
            is too short to hold the stored value. The returned value
            will be truncated.
    """

    from ..base import _nag_d_consts
    from ..base.quad import opt_get as b_opt_get

    try:
        _return_scalars = b_opt_get(
            optstr,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    fun_name = 'naginterfaces.library.quad.opt_get'

    _optype_py = _return_scalars.optype
    _ivalue_py = _return_scalars.ivalue
    _rvalue_py = _return_scalars.rvalue
    _cvalue_py = _return_scalars.cvalue
    if _optype_py == 1:
        _optvalue_py = {}
        _optvalue_py['value'] = _ivalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 2:
        _optvalue_py = {}
        _optvalue_py['value'] = _rvalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 3:
        _optvalue_py = {}
        _optvalue_py['value'] = _cvalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 4:
        _optvalue_py = {}
        _optvalue_py['value'] = _ivalue_py
        _optvalue_py['annotation'] = _cvalue_py
    elif _optype_py == 5:
        _optvalue_py = {}
        _optvalue_py['value'] = _rvalue_py
        _optvalue_py['annotation'] = _cvalue_py
    elif _optype_py == 6:
        _optvalue_py = {}
        if _ivalue_py == _nag_d_consts.D01ES_SOLVER_ID:
            _optvalue_py = {}
            _optvalue_py['value'] = 'md_sgq_multi_vec'
        elif _ivalue_py == _nag_d_consts.D01RA_SOLVER_ID:
            _optvalue_py = {}
            _optvalue_py['value'] = 'dim1_gen_vec_multi_rcomm'
        else:
            raise _b_utils.NagException(
                {'fun_name': fun_name},
                'inconceivable option configuration',
            )
        _optvalue_py['annotation'] = None
    else:
        raise _b_utils.NagException(
            {'fun_name': fun_name},
            'inconceivable option configuration',
        )
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['optvalue'],
    )(
        _optvalue_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _optvalue_py
