# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `sparse` Chapter.

``sparse`` - Large Scale Linear Systems

This module provides functions for the solution of large sparse systems
of simultaneous linear equations.
These include iterative methods for real nonsymmetric and symmetric,
complex non-Hermitian and Hermitian linear systems and direct methods
for general real linear systems.
Further direct methods are currently available in submodule ``matop``
and submodule ``linsys``.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def real_gen_basic_setup(method, precon, n, m, tol, maxitn, anorm, sigmax, monit, norm=None, weight='N', iterm=1):
    u"""
    Real sparse nonsymmetric linear systems, setup for
    ``real_gen_basic_solver``.

    For full information please refer to the NAG Library document for
    f11bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11bdf.html

    Parameters
    ----------
    method : str
        The iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    precon : str, length 1
        Determines whether preconditioning is used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'P'

          Preconditioning.

    n : int
        n, the order of the matrix A.

    m : int
        If `method` = 'RGMRES', `m` is the dimension m of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The tolerance tau for the termination criterion. If `tol` <=
        0.0, tau = max(sqrt(epsilon), sqrt(n)*epsilon) is used, where
        epsilon is the machine precision. Otherwise tau = max(`tol`,
        10*epsilon, sqrt(n)*epsilon) is used.

    maxitn : int
        The maximum number of iterations.

    anorm : float
        If `anorm` > 0.0, the value of ||A||_p to be used in the
        termination criterion (2) (`iterm` = 1).

        If `anorm` <= 0.0, `iterm` = 1 and `norm` = '1' or 'I', then
        ||A||_1 = ||A||_infinity is estimated internally by
        ``real_gen_basic_solver``.

        If `iterm` = 2, `anorm` is not referenced.

    sigmax : float
        If `iterm` = 2, the largest singular value sigma_1 of the
        preconditioned iteration matrix; otherwise, `sigmax` is not
        referenced.

        If `sigmax` <= 0.0, `iterm` = 2 and `method` = 'RGMRES', the
        value of sigma_1 will be estimated internally.

    monit : int
        If `monit` > 0, the frequency at which a monitoring step is
        executed by ``real_gen_basic_solver``: if `method` = 'CGS' or
        'TFQMR', a monitoring step is executed every `monit` iterations;
        otherwise, a monitoring step is executed every `monit`
        super-iterations (groups of up to m or l iterations for RGMRES
        or BI-CGSTAB(l), respectively).

        There are some additional computational costs involved in
        monitoring the solution and residual vectors when the
        BI-CGSTAB(l) method is used with l > 1.

    norm : None or str, length 1, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `iterm` = 1: 'I'; otherwise:
        '2'.

        Defines the matrix and vector norm to be used in the termination
        criteria.

        `norm` = '1'

          l_1 norm.

        `norm` = 'I'

          l_infinity norm.

        `norm` = '2'

          l_2 norm.

    weight : str, length 1, optional
        Specifies whether a vector w of user-supplied weights is to be
        used in the computation of the vector norms required in
        termination criterion (2) (`iterm` = 1): ||v||_p^(w) =
        ||v^(w)||_p, where v_i^(w) = w_i v_i, for i = 1,2,...,n. The
        suffix p = 1,2, infinity denotes the vector norm used, as
        specified by the argument `norm`. Note that weights cannot be
        used when `iterm` = 2, i.e., when criterion (3) is used.

        `weight` = 'W'

          User-supplied weights are to be used and must be supplied on
          initial entry to ``real_gen_basic_solver``.

        `weight` = 'N'

          All weights are implicitly set equal to one. Weights do not
          need to be supplied on initial entry to
          ``real_gen_basic_solver``.

    iterm : int, optional
        Defines the termination criterion to be used.

        `iterm` = 1

          Use the termination criterion defined in (2).

        `iterm` = 2

          Use the termination criterion defined in (3).

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` -12)
            On entry, `monit` = *<value>* and `maxitn` = *<value>*.

            Constraint: `monit` <= `maxitn`.

        (`errno` -11)
            On entry, `iterm` = 2, `method` = *<value>* and `sigmax` =
            *<value>*.

            Constraint: if `iterm` = 2 and `method` = 'CGS' or
            'BICGSTAB', `sigmax` > 0.0.

        (`errno` -10)
            On entry, `iterm` = 1, `norm` = '2' and `anorm` = *<value>*.

            Constraint: if `iterm` = 1 and `norm` = '2', `anorm` > 0.0.

        (`errno` -9)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` > 0.

        (`errno` -8)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` -7)
            On entry, `m` = *<value>* and `method` = *<value>*.

            Constraint: if `method` = 'RGMRES' or 'BICGSTAB', `m` > 0.

        (`errno` -7)
            On entry, `m` = *<value>*, `n` = *<value>* and `method` =
            *<value>*.

            Constraint: if `method` = 'RGMRES', `m` <= min(`n`, 50). If
            `method` = 'BICGSTAB', `m` <= min(`n`, 10).

        (`errno` -6)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` -5)
            On entry, `iterm` = *<value>*.

            Constraint: `iterm` = 1 or 2.

        (`errno` -5)
            On entry, `iterm` = 2 and `method` = 'TFQMR'.

            Constraint: if `iterm` = 2, `method` != 'TFQMR'.

        (`errno` -5)
            On entry, `iterm` = 2 and `norm` = *<value>*.

            Constraint: if `iterm` = 2, `norm` = '2'.

        (`errno` -5)
            On entry, `iterm` = 2 and `weight` = *<value>*.

            Constraint: if `iterm` = 2, `weight` = 'N'.

        (`errno` -4)
            On entry, `weight` = *<value>*.

            Constraint: `weight` = 'N' or 'W'.

        (`errno` -3)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'I' or '2'.

        (`errno` -2)
            On entry, `precon` = *<value>*.

            Constraint: `precon` = 'N' or 'P'.

        (`errno` -1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS', 'BICGSTAB' or
            'TFQMR'.

        (`errno` 1)
            ``real_gen_basic_setup`` has been called out of sequence:
            either ``real_gen_basic_setup`` has been called twice or
            ``real_gen_basic_solver`` has not terminated its current
            task.
    """

    from ..base.sparse import real_gen_basic_setup as b_real_gen_basic_setup

    fun_name = 'naginterfaces.library.sparse.real_gen_basic_setup'

    _iterm = _b_utils._EngineIntScalarType(
        iterm, {'fun_name': fun_name, 'entity': 'iterm'},
    ).to_ctypes()
    _iterm_py = _b_utils._EngineIntScalarType.to_py(_iterm)
    if _iterm_py == 1:
        _norm = 'I'
    else:
        _norm = '2'
    _norm = _norm if norm is None else norm
    _return_scalar = b_real_gen_basic_setup(
        method,
        precon,
        _norm,
        weight,
        _iterm,
        n,
        m,
        tol,
        maxitn,
        anorm,
        sigmax,
        monit,
    )
    _comm_py = _return_scalar
    return _comm_py

def real_gen_basic_solver(irevcm, u, v, wgt, comm):
    u"""
    Real sparse nonsymmetric linear systems, preconditioned RGMRES, CGS,
    Bi-CGSTAB or TFQMR method.

    For full information please refer to the NAG Library document for
    f11be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11bef.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must either be unchanged from its
        previous exit value, or can have one of the following values.

        `irevcm` = 5

          Tidy termination: the computation will terminate at the end of
          the current iteration. Further reverse communication exits may
          occur depending on when the termination request is issued.
          ``real_gen_basic_solver`` will then return with the
          termination code `irevcm` = 4. Note that before calling
          ``real_gen_basic_solver`` with `irevcm` = 5 the calling
          program must have performed the tasks required by the value of
          `irevcm` returned by the previous call to
          ``real_gen_basic_solver``, otherwise subsequently returned
          values may be invalid.

        `irevcm` = 6

          Immediate termination: ``real_gen_basic_solver`` will return
          immediately with termination code `irevcm` = 4 and with any
          useful information available. This includes the last iterate
          of the solution. The residual vector is generally not
          available.

          Immediate termination may be useful, for example, when errors
          are detected during matrix-vector multiplication or during the
          solution of the preconditioning equation.

        Changing `irevcm` to any other value between calls will result
        in an error.

    u : float, ndarray, shape (n), modified in place
        On initial entry: an initial estimate, x_0, of the solution of
        the system of equations Ax = b.

        On intermediate entry: must remain unchanged.

        On intermediate exit: the returned value of `irevcm` determines
        the contents of `u` in the following way:

        - if `irevcm` = -1, 1 or 2, `u` holds the vector u on which the
          operation specified by `irevcm` is to be carried out;

        - if `irevcm` = 3, `u` holds the current iterate of the solution
          vector.

        On final exit: if `errno` = 3 or `errno` < 0, the array `u` is
        unchanged from the initial entry to ``real_gen_basic_solver``.

        If `errno` = 1, the array `u` is unchanged from the last entry
        to ``real_gen_basic_solver``.

        Otherwise, `u` holds the last available iterate of the solution
        of the system of equations, for all returned values of errno.

    v : float, ndarray, shape (n), modified in place
        On initial entry: the right-hand side b of the system of
        equations Ax = b.

        On intermediate entry: the returned value of `irevcm` determines
        the contents of `v` in the following way:

        - if `irevcm` = -1, 1 or 2, `v` must store the vector v, the
          result of the operation specified by the value of `irevcm`
          returned by the previous call to ``real_gen_basic_solver``;

        - if `irevcm` = 3, `v` must remain unchanged.

        On intermediate exit: if `irevcm` = 3, `v` holds the current
        iterate of the residual vector. Note that this is an
        approximation to the true residual vector. Otherwise, it does
        not contain any useful information.

        On final exit: if `errno` = 3 or `errno` < 0, the array `v` is
        unchanged from the initial entry to ``real_gen_basic_solver``.

        If `errno` = 1, the array `v` is unchanged from the last entry
        to ``real_gen_basic_solver``.

        If the function exits successfully or `errno` = 2, the array `v`
        contains the true residual vector of the system of equations
        (see also [Errors]).

        Otherwise, `v` stores the last available iterate of the residual
        vector unless `errno` = 8 is returned on last entry, in which
        case `v` is set to 0.0.

    wgt : float, array-like, shape (n)
        The user-supplied weights, if these are to be used in the
        computation of the vector norms in the termination criterion
        (see [Description] and [Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_gen_basic_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product v =
          A^Tu, where u and v are stored in `u` and `v`, respectively;
          RGMRES, CGS and BI-CGSTAB(l) methods return `irevcm` = -1 only
          if the matrix norm ||A||_1 or ||A||_infinity is estimated
          internally using Higham's method. This can only happen if
          iterm = 1 in ``real_gen_basic_setup``.

        `irevcm` = 1

          The calling program must compute the matrix-vector product v =
          Au, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 2

          The calling program must solve the preconditioning equation Mv
          = u, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 3

          Monitoring step: the solution and residual at the current
          iteration are returned in the arrays `u` and `v`,
          respectively. No action by the calling program is required.
          ``real_gen_basic_diag`` can be called at this step to return
          additional information.

        On final exit: `irevcm` = 4: ``real_gen_basic_solver`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, errors have been
        detected or the calling program has requested termination.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: either `irevcm` must be unchanged from its
            previous exit value or `irevcm` = 5 or 6.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 3)
            Either ``real_gen_basic_setup`` was not called before
            calling ``real_gen_basic_solver`` or it has returned an
            error.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            ``real_gen_basic_solver`` has already completed its tasks.
            You need to set a new problem.

        (`errno` 2)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 2)
            User-requested tidy termination. The required accuracy has
            not been achieved. However, a reasonable accuracy may have
            been achieved.

        (`errno` 4)
            User-requested tidy termination. The solution has not
            converged after *<value>* iterations.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            Algorithm breakdown at iteration no. *<value>*.

        (`errno` 8)
            User-requested immediate termination.

        (`errno` 10)
            The weights in array `wgt` are all zero.
    """

    from ..base.sparse import real_gen_basic_solver as b_real_gen_basic_solver

    fun_name = 'naginterfaces.library.sparse.real_gen_basic_solver'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_real_gen_basic_solver(
            irevcm,
            u,
            v,
            _wgt,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def real_gen_basic_diag(comm):
    u"""
    Real sparse nonsymmetric linear systems, diagnostic for
    ``real_gen_basic_solver``.

    For full information please refer to the NAG Library document for
    f11bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11bff.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_gen_basic_setup``.

    Returns
    -------
    itn : int
        The number of iterations carried out by
        ``real_gen_basic_solver``.

    stplhs : float
        The current value of the left-hand side of the termination
        criterion used by ``real_gen_basic_solver``.

    stprhs : float
        The current value of the right-hand side of the termination
        criterion used by ``real_gen_basic_solver``.

    anorm : float
        If iterm = 1 in the previous call to ``real_gen_basic_setup``,
        then this argument contains ||A||_p, where p = 1, 2 or infinity,
        either supplied or, in the case of 1 or infinity, estimated by
        ``real_gen_basic_solver``; otherwise `anorm` = 0.0.

    sigmax : float
        If iterm = 2 in the previous call to ``real_gen_basic_setup``,
        the current estimate of the largest singular value sigma_1(A_)
        of the preconditioned iteration matrix, either when it has been
        supplied to ``real_gen_basic_setup`` or it has been estimated by
        ``real_gen_basic_solver`` (see also [Description] and
        [Parameters]); otherwise, `sigmax` = 0.0 is returned.

    Raises
    ------
    NagValueError
        (`errno` 1)
            ``real_gen_basic_diag`` has been called out of sequence.
    """

    from ..base.sparse import real_gen_basic_diag as b_real_gen_basic_diag

    _return_scalars = b_real_gen_basic_diag(
        comm,
    )
    _itn_py = _return_scalars.itn
    _stplhs_py = _return_scalars.stplhs
    _stprhs_py = _return_scalars.stprhs
    _anorm_py = _return_scalars.anorm
    _sigmax_py = _return_scalars.sigmax
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['itn', 'stplhs', 'stprhs', 'anorm', 'sigmax'],
    )(
        _itn_py,
        _stplhs_py,
        _stprhs_py,
        _anorm_py,
        _sigmax_py,
    )
    return _return_data

def complex_gen_basic_setup(method, precon, n, m, tol, maxitn, anorm, sigmax, monit, lwork, norm=None, weight='N', iterm=1):
    u"""
    Complex sparse non-Hermitian linear systems, setup for
    ``complex_gen_basic_solver``.

    For full information please refer to the NAG Library document for
    f11br

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11brf.html

    Parameters
    ----------
    method : str
        The iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    precon : str, length 1
        Determines whether preconditioning is used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'P'

          Preconditioning.

    n : int
        n, the order of the matrix A.

    m : int
        If `method` = 'RGMRES', `m` is the dimension m of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The tolerance tau for the termination criterion. If `tol` <=
        0.0, tau = max(sqrt(epsilon), sqrt(n)*epsilon) is used, where
        epsilon is the machine precision. Otherwise tau = max(`tol`,
        10*epsilon, sqrt(n)*epsilon) is used.

    maxitn : int
        The maximum number of iterations.

    anorm : float
        If `anorm` > 0.0, the value of ||A||_p to be used in the
        termination criterion (2) (`iterm` = 1).

        If `anorm` <= 0.0, `iterm` = 1 and `norm` = '1' or 'I', then
        ||A||_1 = ||A||_infinity is estimated internally by
        ``complex_gen_basic_solver``.

        If `iterm` = 2, `anorm` is not referenced.

    sigmax : float
        If `iterm` = 2, the largest singular value sigma_1 of the
        preconditioned iteration matrix; otherwise, `sigmax` is not
        referenced.

        If `sigmax` <= 0.0, `iterm` = 2 and `method` = 'RGMRES', the
        value of sigma_1 will be estimated internally.

    monit : int
        If `monit` > 0, the frequency at which a monitoring step is
        executed by ``complex_gen_basic_solver``: if `method` = 'CGS' or
        'TFQMR', a monitoring step is executed every `monit` iterations;
        otherwise, a monitoring step is executed every `monit`
        super-iterations (groups of up to m or l iterations for RGMRES
        or BI-CGSTAB(l), respectively).

        There are some additional computational costs involved in
        monitoring the solution and residual vectors when the
        BI-CGSTAB(l) method is used with l > 1.

    lwork : int
        The dimension of the array `comm`['work'].

    norm : None or str, length 1, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `iterm` = 1: 'I'; otherwise:
        '2'.

        Defines the matrix and vector norm to be used in the termination
        criteria.

        `norm` = '1'

          l_1 norm.

        `norm` = 'I'

          l_infinity norm.

        `norm` = '2'

          l_2 norm.

    weight : str, length 1, optional
        Specifies whether a vector w of user-supplied weights is to be
        used in the computation of the vector norms required in
        termination criterion (2) (`iterm` = 1): ||v||_p^(w) =
        ||v^(w)||_p, where v_i^(w) = w_i v_i, for i = 1,2,...,n. The
        suffix p = 1,2, infinity denotes the vector norm used, as
        specified by the argument `norm`. Note that weights cannot be
        used when `iterm` = 2, i.e., when criterion (3) is used.

        `weight` = 'W'

          User-supplied weights are to be used and must be supplied on
          initial entry to ``complex_gen_basic_solver``.

        `weight` = 'N'

          All weights are implicitly set equal to one. Weights do not
          need to be supplied on initial entry to
          ``complex_gen_basic_solver``.

    iterm : int, optional
        Defines the termination criterion to be used.

        `iterm` = 1

          Use the termination criterion defined in (2).

        `iterm` = 2

          Use the termination criterion defined in (3).

    Returns
    -------
    lwreq : int
        The minimum amount of workspace required by
        ``complex_gen_basic_solver``. (See also [Parameters].)

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` -15)
            On entry, `lwork` = *<value>*.

            Constraint: `lwork` >= 120.

        (`errno` -12)
            On entry, `monit` = *<value>* and `maxitn` = *<value>*.

            Constraint: `monit` <= `maxitn`.

        (`errno` -11)
            On entry, `iterm` = 2, `method` = *<value>* and `sigmax` =
            *<value>*.

            Constraint: if `iterm` = 2 and `method` = 'CGS' or
            'BICGSTAB', `sigmax` > 0.0.

        (`errno` -10)
            On entry, `iterm` = 1, `norm` = '2' and `anorm` = *<value>*.

            Constraint: if `iterm` = 1 and `norm` = '2', `anorm` > 0.0.

        (`errno` -9)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` > 0.

        (`errno` -8)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` -7)
            On entry, `m` = *<value>*, `n` = *<value>* and `method` =
            *<value>*.

            Constraint: if `method` = 'RGMRES', `m` <= min(`n`, 50). If
            `method` = 'BICGSTAB', `m` <= min(`n`, 10).

        (`errno` -7)
            On entry, `m` = *<value>* and `method` = *<value>*.

            Constraint: if `method` = 'RGMRES' or 'BICGSTAB', `m` > 0.

        (`errno` -6)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` -5)
            On entry, `iterm` = 2 and `method` = 'TFQMR'.

            Constraint: if `iterm` = 2, `method` != 'TFQMR'.

        (`errno` -5)
            On entry, `iterm` = 2 and `norm` = *<value>*.

            Constraint: if `iterm` = 2, `norm` = '2'.

        (`errno` -5)
            On entry, `iterm` = 2 and `weight` = *<value>*.

            Constraint: if `iterm` = 2, `weight` = 'N'.

        (`errno` -5)
            On entry, `iterm` = *<value>*.

            Constraint: `iterm` = 1 or 2.

        (`errno` -4)
            On entry, `weight` = *<value>*.

            Constraint: `weight` = 'N' or 'W'.

        (`errno` -3)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'I' or '2'.

        (`errno` -2)
            On entry, `precon` = *<value>*.

            Constraint: `precon` = 'N' or 'P'.

        (`errno` -1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS', 'BICGSTAB' or
            'TFQMR'.

        (`errno` 1)
            ``complex_gen_basic_setup`` has been called out of sequence:
            either ``complex_gen_basic_setup`` has been called twice or
            ``complex_gen_basic_solver`` has not terminated its current
            task.
    """

    from ..base.sparse import complex_gen_basic_setup as b_complex_gen_basic_setup

    fun_name = 'naginterfaces.library.sparse.complex_gen_basic_setup'

    _iterm = _b_utils._EngineIntScalarType(
        iterm, {'fun_name': fun_name, 'entity': 'iterm'},
    ).to_ctypes()
    _iterm_py = _b_utils._EngineIntScalarType.to_py(_iterm)
    if _iterm_py == 1:
        _norm = 'I'
    else:
        _norm = '2'
    _norm = _norm if norm is None else norm
    _return_scalars = b_complex_gen_basic_setup(
        method,
        precon,
        _norm,
        weight,
        _iterm,
        n,
        m,
        tol,
        maxitn,
        anorm,
        sigmax,
        monit,
        lwork,
    )
    _lwreq_py = _return_scalars.lwreq
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lwreq', 'comm'],
    )(
        _lwreq_py,
        _comm_py,
    )
    return _return_data

def complex_gen_basic_solver(irevcm, u, v, wgt, comm):
    u"""
    Complex sparse non-Hermitian linear systems, preconditioned RGMRES,
    CGS, Bi-CGSTAB or TFQMR method.

    For full information please refer to the NAG Library document for
    f11bs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11bsf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must either be unchanged from its
        previous exit value, or can have one of the following values.

        `irevcm` = 5

          Tidy termination: the computation will terminate at the end of
          the current iteration. Further reverse communication exits may
          occur depending on when the termination request is issued.
          ``complex_gen_basic_solver`` will then return with the
          termination code `irevcm` = 4. Note that before calling
          ``complex_gen_basic_solver`` with `irevcm` = 5 the calling
          program must have performed the tasks required by the value of
          `irevcm` returned by the previous call to
          ``complex_gen_basic_solver``, otherwise subsequently returned
          values may be invalid.

        `irevcm` = 6

          Immediate termination: ``complex_gen_basic_solver`` will
          return immediately with termination code `irevcm` = 4 and with
          any useful information available. This includes the last
          iterate of the solution.

          Immediate termination may be useful, for example, when errors
          are detected during matrix-vector multiplication or during the
          solution of the preconditioning equation.

        Changing `irevcm` to any other value between calls will result
        in an error.

    u : complex, ndarray, shape (n), modified in place
        On initial entry: an initial estimate, x_0, of the solution of
        the system of equations Ax = b.

        On intermediate entry: must remain unchanged.

        On intermediate exit: the returned value of `irevcm` determines
        the contents of `u` as follows.

        If `irevcm` = -1, 1 or 2, `u` holds the vector u on which the
        operation specified by `irevcm` is to be carried out.

        If `irevcm` = 3, `u` holds the current iterate of the solution
        vector.

        On final exit: if `errno` = 3 or `errno` < 0, the array `u` is
        unchanged from the initial entry to
        ``complex_gen_basic_solver``.

        If `errno` = 1, the array `u` is unchanged from the last entry
        to ``complex_gen_basic_solver``.

        Otherwise, `u` holds the last available iterate of the solution
        of the system of equations, for all returned values of errno.

    v : complex, ndarray, shape (n), modified in place
        On initial entry: the right-hand side b of the system of
        equations Ax = b.

        On intermediate entry: the returned value of `irevcm` determines
        the contents of `v` as follows.

        If `irevcm` = -1, 1 or 2, `v` must store the vector v, the
        result of the operation specified by the value of `irevcm`
        returned by the previous call to ``complex_gen_basic_solver``.

        If `irevcm` = 3, `v` must remain unchanged.

        On intermediate exit: if `irevcm` = 3, `v` holds the current
        iterate of the residual vector. Note that this is an
        approximation to the true residual vector. Otherwise, it does
        not contain any useful information.

        On final exit: if `errno` = 3 or `errno` < 0, the array `v` is
        unchanged from the initial entry to
        ``complex_gen_basic_solver``.

        If `errno` = 1, the array `v` is unchanged from the last entry
        to ``complex_gen_basic_solver``.

        If the function exits successfully or `errno` = 2, the array `v`
        contains the true residual vector of the system of equations
        (see also [Errors]).

        Otherwise, `v` stores the last available iterate of the residual
        vector unless `errno` = 8 is returned on last entry, in which
        case `v` is set to 0.0.

    wgt : float, array-like, shape (n)
        The user-supplied weights, if these are to be used in the
        computation of the vector norms in the termination criterion
        (see [Description] and [Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_gen_basic_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product v =
          A^Hu, where u and v are stored in `u` and `v`, respectively;
          RGMRES, CGS and BI-CGSTAB(l) methods return `irevcm` = -1 only
          if the matrix norm ||A||_1 or ||A||_infinity is estimated
          internally using Higham's method. This can only happen if
          iterm = 1 in ``complex_gen_basic_setup``.

        `irevcm` = 1

          The calling program must compute the matrix-vector product v =
          Au, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 2

          The calling program must solve the preconditioning equation Mv
          = u, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 3

          Monitoring step: the solution and residual at the current
          iteration are returned in the arrays `u` and `v`,
          respectively. No action by the calling program is required.
          ``complex_gen_basic_diag`` can be called at this step to
          return additional information.

        On final exit: `irevcm` = 4: ``complex_gen_basic_solver`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, errors have been
        detected or the calling program has requested termination.

    Raises
    ------
    NagValueError
        (`errno` -6)
            On entry, lwork = *<value>*.

            Constraint: lwork >= lwreq, where lwreq is returned by
            ``complex_gen_basic_setup``.

        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: either `irevcm` must be unchanged from its
            previous exit value or `irevcm` = 5 or 6.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 3)
            Either ``complex_gen_basic_setup`` was not called before
            calling ``complex_gen_basic_solver`` or it has returned an
            error.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            ``complex_gen_basic_solver`` has already completed its
            tasks. You need to set a new problem.

        (`errno` 2)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 2)
            User-requested termination: the required accuracy could not
            be obtained. However, a reasonable accuracy may have been
            achieved.

        (`errno` 4)
            User-requested tidy termination. The solution has not
            converged after *<value>* iterations.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            Algorithm breakdown at iteration no. *<value>*.

        (`errno` 8)
            User-requested immediate termination.

        (`errno` 10)
            The weights in array `wgt` are all zero.
    """

    from ..base.sparse import complex_gen_basic_solver as b_complex_gen_basic_solver

    fun_name = 'naginterfaces.library.sparse.complex_gen_basic_solver'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_complex_gen_basic_solver(
            irevcm,
            u,
            v,
            _wgt,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def complex_gen_basic_diag(comm):
    u"""
    Complex sparse non-Hermitian linear systems, diagnostic for
    ``complex_gen_basic_solver``.

    For full information please refer to the NAG Library document for
    f11bt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11btf.html

    Parameters
    ----------
    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_gen_basic_setup``.

    Returns
    -------
    itn : int
        The number of iterations carried out by
        ``complex_gen_basic_solver``.

    stplhs : float
        The current value of the left-hand side of the termination
        criterion used by ``complex_gen_basic_solver``.

    stprhs : float
        The current value of the right-hand side of the termination
        criterion used by ``complex_gen_basic_solver``.

    anorm : float
        If iterm = 1 in the previous call to
        ``complex_gen_basic_setup``, then this argument contains
        ||A||_p, where p = 1, 2 or infinity, either supplied or, in the
        case of 1 or infinity, estimated by
        ``complex_gen_basic_solver``; otherwise `anorm` = 0.0.

    sigmax : float
        If iterm = 2 in the previous call to
        ``complex_gen_basic_setup``, the current estimate of the largest
        singular value sigma_1(A_) of the preconditioned iteration
        matrix when it is used by the termination criterion in
        ``complex_gen_basic_solver``, either when it has been supplied
        to ``complex_gen_basic_setup`` or it has been estimated by
        ``complex_gen_basic_solver`` (see also [Description] and
        [Parameters]); otherwise, `sigmax` = 0.0 is returned.

    Raises
    ------
    NagValueError
        (`errno` 1)
            ``complex_gen_basic_diag`` has been called out of sequence.
    """

    from ..base.sparse import complex_gen_basic_diag as b_complex_gen_basic_diag

    _return_scalars = b_complex_gen_basic_diag(
        comm,
    )
    _itn_py = _return_scalars.itn
    _stplhs_py = _return_scalars.stplhs
    _stprhs_py = _return_scalars.stprhs
    _anorm_py = _return_scalars.anorm
    _sigmax_py = _return_scalars.sigmax
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['itn', 'stplhs', 'stprhs', 'anorm', 'sigmax'],
    )(
        _itn_py,
        _stplhs_py,
        _stprhs_py,
        _anorm_py,
        _sigmax_py,
    )
    return _return_data

def real_gen_precon_ilu(nnz, a, irow, icol, lfill, dtol, milu, ipivp, ipivq, pstrat='C'):
    u"""
    Real sparse nonsymmetric linear systems, incomplete LU
    factorization.

    For full information please refer to the NAG Library document for
    f11da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11daf.html

    Parameters
    ----------
    nnz : int
        The number of nonzero elements in the matrix A.

    a : float, array-like, shape (la)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``real_gen_sort`` may be used to order the elements
        in this way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    lfill : int
        If `lfill` >= 0 its value is the maximum level of fill allowed
        in the decomposition (see [Further Comments]). A negative value
        of `lfill` indicates that `dtol` will be used to control the
        fill instead.

    dtol : float
        If `lfill` < 0, `dtol` is used as a drop tolerance to control
        the fill-in (see [Further Comments]); otherwise `dtol` is not
        referenced.

    milu : str, length 1
        Indicates whether or not the factorization should be modified to
        preserve row-sums (see [Further Comments]).

        `milu` = 'M'

          The factorization is modified.

        `milu` = 'N'

          The factorization is not modified.

    ipivp : int, array-like, shape (n)
        If `pstrat` = 'U', `ipivp`[k-1] and `ipivq`[k-1] must specify
        the row and column indices of the element used as a pivot at
        elimination stage k. Otherwise `ipivp` and `ipivq` need not be
        initialized.

    ipivq : int, array-like, shape (n)
        If `pstrat` = 'U', `ipivp`[k-1] and `ipivq`[k-1] must specify
        the row and column indices of the element used as a pivot at
        elimination stage k. Otherwise `ipivp` and `ipivq` need not be
        initialized.

    pstrat : str, length 1, optional
        Specifies the pivoting strategy to be adopted.

        `pstrat` = 'N'

          No pivoting is carried out.

        `pstrat` = 'U'

          Pivoting is carried out according to the user-defined input
          values of `ipivp` and `ipivq`.

        `pstrat` = 'P'

          Partial pivoting by columns for stability is carried out.

        `pstrat` = 'C'

          Complete pivoting by rows for sparsity, and by columns for
          stability, is carried out.

    Returns
    -------
    a : float, ndarray, shape (la)
        The first `nnz` entries of `a` contain the nonzero elements of A
        and the next `nnzc` entries contain the elements of the matrix
        C. Matrix elements are ordered by increasing row index, and by
        increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipivp : int, ndarray, shape (n)
        The pivot indices. If `ipivp`[k-1] = i and `ipivq`[k-1] = j then
        the element in row i and column j was used as the pivot at
        elimination stage k.

    ipivq : int, ndarray, shape (n)
        The pivot indices. If `ipivp`[k-1] = i and `ipivq`[k-1] = j then
        the element in row i and column j was used as the pivot at
        elimination stage k.

    istr : int, ndarray, shape (n+1)
        `istr`[i-1]-1, for i = 1,2,...,n, is the starting address in the
        arrays `a`, `irow` and `icol` of row i of the matrix C.
        `istr`[n]-1 is the address of the last nonzero element in C plus
        one.

    idiag : int, ndarray, shape (n)
        `idiag`[i-1], for i = 1,2,...,n, holds the index of arrays `a`,
        `irow` and `icol` which holds the diagonal element in row i of
        the matrix C.

    nnzc : int
        The number of nonzero elements in the matrix C.

    npivm : int
        If `npivm` > 0 it gives the number of pivots which were modified
        during the factorization to ensure that M exists.

        If `npivm` = -1 no pivot modifications were required, but a
        local restart occurred (see [Algorithmic Details]).

        The quality of the preconditioner will generally depend on the
        returned value of `npivm`.

        If `npivm` is large the preconditioner may not be satisfactory.

        In this case it may be advantageous to call
        ``real_gen_precon_ilu`` again with an increased value of
        `lfill`, a reduced value of `dtol`, or set `pstrat` = 'C'.

        See also [Further Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `milu` = *<value>*.

            Constraint: `milu` = 'M' or 'N'.

        (`errno` 1)
            On entry, `pstrat` = *<value>*.

            Constraint: `pstrat` = 'N', 'U', 'P' or 'C'.

        (`errno` 1)
            On entry, `dtol` = *<value>*.

            Constraint: `dtol` >= 0.0.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 2)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, a user-supplied value of `ipivq` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipivq` lies outside the
            range [1,n].

        (`errno` 3)
            On entry, a user-supplied value of `ipivp` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipivp` lies outside the
            range [1,n].

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large. The decomposition has been terminated before
            completion. Either increase la, or reduce the fill by
            reducing `lfill`, or increasing `dtol`.

        (`errno` 5)
            A serious error has occurred in an internal call to an
            auxiliary function. Check all function calls and array
            sizes. Seek expert help.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_gen_precon_ilu as b_real_gen_precon_ilu

    fun_name = 'naginterfaces.library.sparse.real_gen_precon_ilu'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _idiag = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_real_gen_precon_ilu(
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        lfill,
        dtol,
        pstrat,
        milu,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipivp_py = _ipivp
    _ipivq_py = _ipivq
    _istr_py = _istr
    _idiag_py = _idiag
    _nnzc_py = _return_scalars.nnzc
    _npivm_py = _return_scalars.npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipivp', 'ipivq', 'istr', 'idiag', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipivp_py,
        _ipivq_py,
        _istr_py,
        _idiag_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def real_gen_precon_ilu_solve(trans, a, irow, icol, ipivp, ipivq, istr, idiag, y, check='N'):
    u"""
    Solution of linear system involving incomplete LU preconditioning
    matrix generated by ``real_gen_precon_ilu``.

    For full information please refer to the NAG Library document for
    f11db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dbf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix M is transposed.

        `trans` = 'N'

          Mx = y is solved.

        `trans` = 'T'

          M^Tx = y is solved.

    a : float, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``real_gen_precon_ilu``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    ipivp : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    ipivq : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    idiag : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

    y : float, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix M
        should be checked.

        `check` = 'C'

          Checks are carried on the values of n, `irow`, `icol`,
          `ipivp`, `ipivq`, `istr` and `idiag`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `ipivq`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipivq`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipivq`[i-1] >= 1 and `ipivq`[i-1] <= n.

        (`errno` 3)
            On entry, `ipivp`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipivp`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipivp`[i-1] >= 1 and `ipivp`[i-1] <= n.

        (`errno` 3)
            On entry, `istr`[i-1] is inconsistent with `irow`: i =
            *<value>*.

        (`errno` 3)
            On entry, `idiag`[i-1] appears to be incorrect: i =
            *<value>*.

        (`errno` 3)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>*, and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, `istr` appears to be invalid.
    """

    from ..base.sparse import real_gen_precon_ilu_solve as b_real_gen_precon_ilu_solve

    fun_name = 'naginterfaces.library.sparse.real_gen_precon_ilu_solve'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_gen_precon_ilu_solve(
        trans,
        _n,
        _a,
        _la,
        _irow,
        _icol,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def real_gen_solve_ilu(method, nnz, a, irow, icol, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x):
    u"""
    Solution of real sparse nonsymmetric linear system, RGMRES, CGS,
    Bi-CGSTAB or TFQMR method, preconditioner computed by
    ``real_gen_precon_ilu``.

    For full information please refer to the NAG Library document for
    f11dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dcf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    nnz : int
        The number of nonzero elements in the matrix A. This must be the
        same value as was supplied in the preceding call to
        ``real_gen_precon_ilu``.

    a : float, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``real_gen_precon_ilu``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    ipivp : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    ipivq : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    idiag : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method; otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : float, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : float, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: `m` >= 1 and `m` <= min(n, *<value>*).

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The CS representation of the preconditioner is invalid.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.

        (`errno` 7)
            A serious error, code *<value>*, has occurred in an internal
            call. Check all function calls and array sizes. Seek expert
            help.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_gen_solve_ilu as b_real_gen_solve_ilu

    fun_name = 'naginterfaces.library.sparse.real_gen_solve_ilu'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_gen_solve_ilu(
            method,
            _n,
            nnz,
            _a,
            _la,
            _irow,
            _icol,
            _ipivp,
            _ipivq,
            _istr,
            _idiag,
            _b,
            m,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_gen_precon_ssor_solve(trans, a, irow, icol, rdiag, omega, y, check='N'):
    u"""
    Solution of linear system involving preconditioning matrix generated
    by applying SSOR to real sparse nonsymmetric matrix.

    For full information please refer to the NAG Library document for
    f11dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11ddf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix M is transposed.

        `trans` = 'N'

          Mx = y is solved.

        `trans` = 'T'

          M^Tx = y is solved.

    a : float, array-like, shape (nnz)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``real_gen_sort`` may be used to order the elements
        in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    rdiag : float, array-like, shape (n)
        The elements of the diagonal matrix D^-1, where D is the
        diagonal part of A.

    omega : float
        The relaxation parameter omega.

    y : float, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix M
        should be checked.

        `check` = 'C'

          Checks are carried on the values of n, nnz, `irow`, `icol` and
          `omega`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 2)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: 1 <= nnz <= n^2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: 1 <= nnz <= n^2.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 4)
            The matrix A has no diagonal entry in row *<value>*.
    """

    from ..base.sparse import real_gen_precon_ssor_solve as b_real_gen_precon_ssor_solve

    fun_name = 'naginterfaces.library.sparse.real_gen_precon_ssor_solve'

    _rdiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rdiag'},
        rdiag,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_rdiag, {'fun_name': fun_name, 'entity': 'rdiag'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_gen_precon_ssor_solve(
        trans,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _rdiag,
        omega,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def real_gen_solve_jacssor(method, precon, a, irow, icol, omega, b, m, tol, maxitn, x):
    u"""
    Solution of real sparse nonsymmetric linear system, RGMRES, CGS,
    Bi-CGSTAB, or TFQMR method, Jacobi or SSOR preconditioner (Black
    Box).

    For full information please refer to the NAG Library document for
    f11de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11def.html

    Parameters
    ----------
    method : str
        The iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    precon : str, length 1
        Specifies the type of preconditioning to be used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'J'

          Jacobi.

        `precon` = 'S'

          Symmetric successive-over-relaxation.

    a : float, array-like, shape (nnz)
        The nonzero elements of the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``real_gen_sort`` may be used to order the elements
        in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in `a`.

    omega : float
        If `precon` = 'S', `omega` is the relaxation parameter omega to
        be used in the SSOR method. Otherwise `omega` need not be
        initialized and is not referenced.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : float, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : float, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: `m` >= 1 and `m` <= MIN(n,*<value>*).

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, `omega` = *<value>*.

            Constraint: `omega` > 0.0 and `omega` < 2.0.

        (`errno` 1)
            On entry, `precon` != 'N', 'J' or 'S': `precon` = *<value>*.

        (`errno` 1)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The matrix A has a zero diagonal entry in row *<value>*.

        (`errno` 3)
            The matrix A has no diagonal entry in row *<value>*.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.

        (`errno` 7)
            A serious error has occurred in an internal call: IREVCM =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

        (`errno` 7)
            A serious error has occurred in an internal call: errno =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_gen_solve_jacssor as b_real_gen_solve_jacssor

    fun_name = 'naginterfaces.library.sparse.real_gen_solve_jacssor'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_gen_solve_jacssor(
            method,
            precon,
            _n,
            _nnz,
            _a,
            _irow,
            _icol,
            omega,
            _b,
            m,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_gen_precon_bdilu(n, nnz, a, irow, icol, istb, indb, lfill, dtol, milu, ipivp, ipivq, pstrat=None):
    u"""
    Real sparse nonsymmetric linear system, incomplete LU factorization
    of local or overlapping diagonal blocks.

    For full information please refer to the NAG Library document for
    f11df

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dff.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nnz : int
        The number of nonzero elements in the matrix A.

    a : float, array-like, shape (la)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``real_gen_sort`` may be used to order the elements
        in this way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    istb : int, array-like, shape (nb+1)
        `istb`[b-1], for b = 1,2,...,nb, holds the indices in arrays
        `indb`, `ipivp`, `ipivq` and `idiag` that, on successful exit
        from this function, define block b. `istb`[nb] holds the sum of
        the number of rows in all blocks plus `istb`[0].

    indb : int, array-like, shape (lindb)
        `indb` must hold the row indices appearing in each diagonal
        block, stored consecutively. Thus the elements
        `indb`[`istb`[b-1]-1] to `indb`[`istb`[b]-2] are the row indices
        in the bth block, for b = 1,2,...,nb.

    lfill : int, array-like, shape (nb)
        If `lfill`[b-1] >= 0 its value is the maximum level of fill
        allowed in the decomposition of the block (see [Further
        Comments]). A negative value of `lfill`[b-1] indicates that
        `dtol`[b-1] will be used to control the fill in the block
        instead.

    dtol : float, array-like, shape (nb)
        If `lfill`[b-1] < 0 then `dtol`[b-1] is used as a drop tolerance
        in the block to control the fill-in (see [Further Comments]);
        otherwise `dtol`[b-1] is not referenced.

    milu : str, length 1, array-like, shape (nb)
        `milu`[b-1], for b = 1,2,...,nb, indicates whether or not the
        factorization in the block should be modified to preserve
        row-sums (see [Further Comments]).

        `milu`[b-1] = 'M'

          The factorization is modified.

        `milu`[b-1] = 'N'

          The factorization is not modified.

    ipivp : int, array-like, shape (lindb)
        If `pstrat`[b-1] = 'U', `ipivp`[`istb`[b-1]+k-2] and
        `ipivq`[`istb`[b-1]+k-2] must specify the row and column indices
        of the element used as a pivot at elimination stage k of the
        factorization of the block. Otherwise `ipivp` and `ipivq` need
        not be initialized.

    ipivq : int, array-like, shape (lindb)
        If `pstrat`[b-1] = 'U', `ipivp`[`istb`[b-1]+k-2] and
        `ipivq`[`istb`[b-1]+k-2] must specify the row and column indices
        of the element used as a pivot at elimination stage k of the
        factorization of the block. Otherwise `ipivp` and `ipivq` need
        not be initialized.

    pstrat : None or str, length 1, array-like, shape (nb), optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 'C'.

        `pstrat`[b-1], for b = 1,2,...,nb, specifies the pivoting
        strategy to be adopted in the block as follows:

        `pstrat`[b-1] = 'N'

          No pivoting is carried out.

        `pstrat`[b-1] = 'U'

          Pivoting is carried out according to the user-defined input
          values of `ipivp` and `ipivq`.

        `pstrat`[b-1] = 'P'

          Partial pivoting by columns for stability is carried out.

        `pstrat`[b-1] = 'C'

          Complete pivoting by rows for sparsity, and by columns for
          stability, is carried out.

    Returns
    -------
    a : float, ndarray, shape (la)
        The first `nnz` entries of `a` contain the nonzero elements of A
        and the next `nnzc` entries contain the elements of the matrices
        C_b, for b = 1,2,...,nb stored consecutively. Within each block
        the matrix elements are ordered by increasing row index, and by
        increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipivp : int, ndarray, shape (lindb)
        The row and column indices of the pivot elements, arranged
        consecutively for each block, as for `indb`. If
        `ipivp`[`istb`[b-1]+k-2] = i and `ipivq`[`istb`[b-1]+k-2] = j,
        the element in row i and column j of A_b was used as the pivot
        at elimination stage k.

    ipivq : int, ndarray, shape (lindb)
        The row and column indices of the pivot elements, arranged
        consecutively for each block, as for `indb`. If
        `ipivp`[`istb`[b-1]+k-2] = i and `ipivq`[`istb`[b-1]+k-2] = j,
        the element in row i and column j of A_b was used as the pivot
        at elimination stage k.

    istr : int, ndarray, shape (lindb+1)
        `istr`[`istb`[b-1]+k-2], gives the index in the arrays `a`,
        `irow` and `icol` of row k of the matrix C_b, for k =
        1,2,...,`istb`[b]-`istb`[b-1], for b = 1,2,...,nb.

        `istr`[`istb`[nb]-1] contains `nnz`+`nnzc`+1.

    idiag : int, ndarray, shape (lindb)
        `idiag`[`istb`[b-1]+k-2], gives the index in the arrays `a`,
        `irow` and `icol` of the diagonal element in row k of the matrix
        C_b, for k = 1,2,...,`istb`[b]-`istb`[b-1], for b = 1,2,...,nb.

    nnzc : int
        The sum total number of nonzero elements in the matrices C_b,
        for b = 1,2,...,nb.

    npivm : int, ndarray, shape (nb)
        If `npivm`[b-1] > 0 it gives the number of pivots which were
        modified during the factorization to ensure that M_b exists.

        If `npivm`[b-1] = -1 no pivot modifications were required, but a
        local restart occurred (see [Further Comments f11daf]). The
        quality of the preconditioner will generally depend on the
        returned values of `npivm`[b-1], for b = 1,2,...,nb.

        If `npivm`[b-1] is large, for some block, the preconditioner may
        not be satisfactory.

        In this case it may be advantageous to call
        ``real_gen_precon_bdilu`` again with an increased value of
        `lfill`[b-1], a reduced value of `dtol`[b-1], or `pstrat`[b-1] =
        'C'.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, nb = *<value>* and `n` = *<value>*.

            Constraint: 1 <= nb <= `n`.

        (`errno` 1)
            On entry, `istb`[0] = *<value>*.

            Constraint: `istb`[0] >= 1.

        (`errno` 1)
            On entry, for b = *<value>*, `istb`[b] = *<value>* and
            `istb`[b-1] = *<value>*.

            Constraint: `istb`[b] > `istb`[b-1], for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `indb`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `indb`[m-1] <= `n`, for m =
            1,2,...,`istb`[nb]-1

        (`errno` 1)
            On entry, lindb = *<value>*, `istb`[nb]-1 = *<value>* and nb
            = *<value>*.

            Constraint: lindb >= `istb`[nb]-1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>* and `n` = *<value>*.

            Constraint: `nnz` <= `n`^2.

        (`errno` 1)
            On entry, `dtol`[*<value>*] = *<value>*.

            Constraint: `dtol`[b-1] >= 0.0, for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `pstrat`[*<value>*] = *<value>*.

            Constraint: `pstrat`[b-1] = 'N', 'U', 'P' or 'C' for all b.

        (`errno` 1)
            On entry, `milu`[*<value>*] = *<value>*.

            Constraint: `milu`[b-1] = 'M' or 'N' for all b.

        (`errno` 2)
            On entry, `irow`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `irow`[i-1] <= `n`, for i = 1,2,...,`nnz`.

        (`errno` 2)
            On entry, `icol`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `icol`[j-1] <= `n`, for j = 1,2,...,`nnz`.

        (`errno` 2)
            On entry, element *<value>* of `a` was out of order.

        (`errno` 2)
            On entry, location *<value>* of (`irow`,`icol`) was a
            duplicate.

        (`errno` 3)
            On entry, the user-supplied value of `ipivp` for block
            *<value>* lies outside its range.

        (`errno` 3)
            On entry, the user-supplied value of `ipivp` for block
            *<value>* was repeated.

        (`errno` 3)
            On entry, the user-supplied value of `ipivq` for block
            *<value>* lies outside its range.

        (`errno` 3)
            On entry, the user-supplied value of `ipivq` for block
            *<value>* was repeated.

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large.

            The decomposition has been terminated before completion.

            Either increase la, or reduce the fill by reducing `lfill`,
            or increasing `dtol`.
    """

    from ..base.sparse import real_gen_precon_bdilu as b_real_gen_precon_bdilu

    fun_name = 'naginterfaces.library.sparse.real_gen_precon_bdilu'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _istb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istb'},
        istb,
        copy=False,
    )
    _nb = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_istb, {'fun_name': fun_name, 'entity': 'istb'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _indb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'indb'},
        indb,
        copy=False,
    )
    _lindb_py = _b_utils._EngineIntArrayType.get_shape(_indb, {'fun_name': fun_name, 'entity': 'indb'}, exp_rank=1)[0]
    _lindb = _b_utils._EngineIntScalarType(
        _lindb_py, {'fun_name': fun_name, 'entity': '_lindb'},
    ).to_ctypes()
    _lfill = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lfill'},
        lfill,
        copy=False,
    )
    _dtol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dtol'},
        dtol,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    if pstrat is None:
        _pstrat = _b_utils._EngineCharArrayType.empty_ctypes(_nb_py, chrlen=1)
        _pstrat[:] = ['C']*_nb_py
    else:
        _pstrat = pstrat
    _milu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'milu'},
        milu,
        copy=False,
    )
    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_lindb_py+1,),
    )
    _idiag = _b_utils._EngineIntArrayType.empty_ndarray(
        (_lindb_py,),
    )
    _npivm = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nb_py,),
    )
    _return_scalar = b_real_gen_precon_bdilu(
        n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        _nb,
        _istb,
        _indb,
        _lindb,
        _lfill,
        _dtol,
        _pstrat,
        _milu,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        _npivm,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipivp_py = _ipivp
    _ipivq_py = _ipivq
    _istr_py = _istr
    _idiag_py = _idiag
    _nnzc_py = _return_scalar
    _npivm_py = _npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipivp', 'ipivq', 'istr', 'idiag', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipivp_py,
        _ipivq_py,
        _istr_py,
        _idiag_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def real_gen_solve_bdilu(method, nnz, a, irow, icol, istb, indb, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x):
    u"""
    Solution of real sparse nonsymmetric linear system, RGMRES, CGS,
    Bi-CGSTAB or TFQMR method, incomplete LU block diagonal
    preconditioner computed by ``real_gen_precon_bdilu``.

    For full information please refer to the NAG Library document for
    f11dg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dgf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    nnz : int
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    a : float, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    istb : int, array-like, shape (nb+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    indb : int, array-like, shape (lindb)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    ipivp : int, array-like, shape (lindb)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    ipivq : int, array-like, shape (lindb)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    istr : int, array-like, shape (lindb+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    idiag : int, array-like, shape (lindb)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``real_gen_precon_bdilu``.

        The arrays `istb`, `indb` and `a` together with the the scalars
        n, `nnz`, la, nb and lindb must be the same values that were
        supplied in the preceding call to ``real_gen_precon_bdilu``.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),sqrt(n)epsilon) is
        used, where epsilon is the machine precision. Otherwise tau =
        max(`tol`,10 epsilon, sqrt(n)epsilon) is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : float, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : float, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, nb = *<value>* and n = *<value>*.

            Constraint: 1 <= nb <= n.

        (`errno` 1)
            On entry, `istb`[0] = *<value>*.

            Constraint: `istb`[0] >= 1.

        (`errno` 1)
            On entry, for b = *<value>*, `istb`[b] = *<value>* and
            `istb`[b-1] = *<value>*.

            Constraint: `istb`[b] > `istb`[b-1], for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `indb`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `indb`[m-1] <= n, for m =
            1,2,...,`istb`[nb]-1

        (`errno` 1)
            On entry, lindb = *<value>*, `istb`[nb]-1 = *<value>* and nb
            = *<value>*.

            Constraint: lindb >= `istb`[nb]-1.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: if `method` = 'RGMRES', 1 <= `m` <= min(n,
            *<value>*).

            If `method` = 'BICGSTAB', 1 <= `m` <= min(n, *<value>*).

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 2)
            On entry, `irow`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `irow`[i-1] <= n, for i = 1,2,...,`nnz`.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``real_gen_precon_bdilu`` and ``real_gen_solve_bdilu``.

        (`errno` 2)
            On entry, `icol`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `icol`[i-1] <= n, for i = 1,2,...,`nnz`.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``real_gen_precon_bdilu`` and ``real_gen_solve_bdilu``.

        (`errno` 2)
            On entry, element *<value>* of `a` was out of order.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``real_gen_precon_bdilu`` and ``real_gen_solve_bdilu``.

        (`errno` 2)
            On entry, location *<value>* of (`irow`,`icol`) was a
            duplicate.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``real_gen_precon_bdilu`` and ``real_gen_solve_bdilu``.

        (`errno` 3)
            The CS representation of the preconditioner is invalid.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``real_gen_precon_bdilu`` and ``real_gen_solve_bdilu``.

        (`errno` 4)
            The required accuracy could not be obtained. However a
            reasonable accuracy may have been achieved. You should check
            the output value of `rnorm` for acceptability. This error
            code usually implies that your problem has been fully and
            satisfactorily solved to within or close to the accuracy
            available on your system. Further iterations are unlikely to
            improve on this situation.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.
    """

    from ..base.sparse import real_gen_solve_bdilu as b_real_gen_solve_bdilu

    fun_name = 'naginterfaces.library.sparse.real_gen_solve_bdilu'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istb'},
        istb,
        copy=False,
    )
    _nb = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_istb, {'fun_name': fun_name, 'entity': 'istb'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _indb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'indb'},
        indb,
        copy=False,
    )
    _lindb_py = _b_utils._EngineIntArrayType.get_shape(_indb, {'fun_name': fun_name, 'entity': 'indb'}, exp_rank=1)[0]
    _lindb = _b_utils._EngineIntScalarType(
        _lindb_py, {'fun_name': fun_name, 'entity': '_lindb'},
    ).to_ctypes()
    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = b_real_gen_solve_bdilu(
        method,
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        _nb,
        _istb,
        _indb,
        _lindb,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        _b,
        m,
        tol,
        maxitn,
        _x,
    )
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    return _return_data

def real_gen_precon_jacobi(store, trans, init, niter, a, irow, icol, b, diag, check='N'):
    u"""
    Real, sparse, symmetric or nonsymmetric, linear systems, line Jacobi
    preconditioner.

    For full information please refer to the NAG Library document for
    f11dk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dkf.html

    Parameters
    ----------
    store : str, length 1
        Specifies whether the matrix A is stored using symmetric
        coordinate storage (SCS) (applicable only to a symmetric matrix
        A) or coordinate storage (CS) (applicable to both symmetric and
        non-symmetric matrices).

        `store` = 'N'

          The complete matrix A is stored in CS format.

        `store` = 'S'

          The lower triangle of the symmetric matrix A is stored in SCS
          format.

    trans : str, length 1
        If `store` = 'N', specifies whether the approximate solution of
        Ax = b or of A^Tx = b is required.

        `trans` = 'N'

          The approximate solution of Ax = b is calculated.

        `trans` = 'T'

          The approximate solution of A^Tx = b is calculated.

    init : str, length 1
        On first entry, `init` should be set to 'I', unless the diagonal
        elements of A are already stored in the array `diag`. If `diag`
        already contains the diagonal of A, it must be set to 'N'.

        `init` = 'N'

          `diag` must contain the diagonal of A.

        `init` = 'I'

          `diag` will store the diagonal of A on exit.

    niter : int
        The number of Jacobi iterations requested.

    a : float, array-like, shape (nnz)
        If `store` = 'N', the nonzero elements in the matrix A (CS
        format).

        If `store` = 'S', the nonzero elements in the lower triangle of
        the matrix A (SCS format).

        In both cases, the elements of either A or of its lower triangle
        must be ordered by increasing row index and by increasing column
        index within each row.

        Multiple entries for the same row and columns indices are not
        permitted.

        The function ``real_gen_sort`` or ``real_symm_sort`` may be used
        to reorder the elements in this way for CS and SCS storage,
        respectively.

    irow : int, array-like, shape (nnz)
        If `store` = 'N', the row indices of the nonzero elements
        supplied in `a`.

        If `store` = 'S', the row and column indices of the nonzero
        elements of the lower triangle of the matrix A supplied in `a`.

    icol : int, array-like, shape (nnz)
        If `store` = 'N', the column indices of the nonzero elements
        supplied in `a`.

        If `store` = 'S', the row and column indices of the nonzero
        elements of the lower triangle of the matrix A supplied in `a`.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    diag : float, array-like, shape (n)
        If `init` = 'N', the diagonal elements of A.

    check : str, length 1, optional
        Specifies whether or not the CS or SCS representation of the
        matrix A should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow`,
          `icol`; if `init` = 'N', `diag` is also checked.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : float, ndarray, shape (n)
        The approximate solution vector x_`niter`.

    diag : float, ndarray, shape (n)
        If `init` = 'N', unchanged on exit.

        If `init` = 'I', the diagonal elements of A.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `niter` = *<value>*.

            Constraint: `niter` >= 1.

        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'N' or 'C'.

        (`errno` 1)
            On entry, `init` = *<value>*.

            Constraint: `init` = 'N' or 'I'.

        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 1)
            On entry, `store` = *<value>*.

            Constraint: `store` = 'N' or 'S'.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= n.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, I = *<value>*, `irow`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[I-1] >= 1 and `irow`[I-1] <= n.

        (`errno` 4)
            On entry, the element `diag`[I-1] is zero: I = *<value>*.

        (`errno` 5)
            On entry, the diagonal element of the Ith row is zero or
            missing: I = *<value>*.
    """

    from ..base.sparse import real_gen_precon_jacobi as b_real_gen_precon_jacobi

    fun_name = 'naginterfaces.library.sparse.real_gen_precon_jacobi'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _diag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_real_gen_precon_jacobi(
        store,
        trans,
        init,
        niter,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _b,
        _x,
        _diag,
    )
    _diag_py = _diag
    _x_py = _x
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'diag'],
    )(
        _x_py,
        _diag_py,
    )
    return _return_data

def complex_gen_precon_ilu(nnz, a, irow, icol, lfill, dtol, milu, ipivp, ipivq, pstrat='C'):
    u"""
    Complex sparse non-Hermitian linear systems, incomplete LU
    factorization.

    For full information please refer to the NAG Library document for
    f11dn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dnf.html

    Parameters
    ----------
    nnz : int
        The number of nonzero elements in the matrix A.

    a : complex, array-like, shape (la)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``complex_gen_sort`` may be used to order the
        elements in this way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    lfill : int
        If `lfill` >= 0 its value is the maximum level of fill allowed
        in the decomposition (see [Further Comments]). A negative value
        of `lfill` indicates that `dtol` will be used to control the
        fill instead.

    dtol : float
        If `lfill` < 0, `dtol` is used as a drop tolerance to control
        the fill-in (see [Further Comments]); otherwise `dtol` is not
        referenced.

    milu : str, length 1
        Indicates whether or not the factorization should be modified to
        preserve row-sums (see [Further Comments]).

        `milu` = 'M'

          The factorization is modified.

        `milu` = 'N'

          The factorization is not modified.

    ipivp : int, array-like, shape (n)
        If `pstrat` = 'U', `ipivp`[k-1] and `ipivq`[k-1] must specify
        the row and column indices of the element used as a pivot at
        elimination stage k. Otherwise `ipivp` and `ipivq` need not be
        initialized.

    ipivq : int, array-like, shape (n)
        If `pstrat` = 'U', `ipivp`[k-1] and `ipivq`[k-1] must specify
        the row and column indices of the element used as a pivot at
        elimination stage k. Otherwise `ipivp` and `ipivq` need not be
        initialized.

    pstrat : str, length 1, optional
        Specifies the pivoting strategy to be adopted.

        `pstrat` = 'N'

          No pivoting is carried out.

        `pstrat` = 'U'

          Pivoting is carried out according to the user-defined input
          values of `ipivp` and `ipivq`.

        `pstrat` = 'P'

          Partial pivoting by columns for stability is carried out.

        `pstrat` = 'C'

          Complete pivoting by rows for sparsity, and by columns for
          stability, is carried out.

    Returns
    -------
    a : complex, ndarray, shape (la)
        The first `nnz` entries of `a` contain the nonzero elements of A
        and the next `nnzc` entries contain the elements of the matrix
        C. Matrix elements are ordered by increasing row index, and by
        increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipivp : int, ndarray, shape (n)
        The pivot indices. If `ipivp`[k-1] = i and `ipivq`[k-1] = j then
        the element in row i and column j was used as the pivot at
        elimination stage k.

    ipivq : int, ndarray, shape (n)
        The pivot indices. If `ipivp`[k-1] = i and `ipivq`[k-1] = j then
        the element in row i and column j was used as the pivot at
        elimination stage k.

    istr : int, ndarray, shape (n+1)
        `istr`[i-1]-1, for i = 1,2,...,n, is the starting address in the
        arrays `a`, `irow` and `icol` of row i of the matrix C.
        `istr`[n]-1 is the address of the last nonzero element in C plus
        one.

    idiag : int, ndarray, shape (n)
        `idiag`[i-1], for i = 1,2,...,n, holds the index of arrays `a`,
        `irow` and `icol` which holds the diagonal element in row i of
        the matrix C.

    nnzc : int
        The number of nonzero elements in the matrix C.

    npivm : int
        If `npivm` > 0 it gives the number of pivots which were modified
        during the factorization to ensure that M exists.

        If `npivm` = -1 no pivot modifications were required, but a
        local restart occurred (see [Algorithmic Details]).

        The quality of the preconditioner will generally depend on the
        returned value of `npivm`.

        If `npivm` is large the preconditioner may not be satisfactory.

        In this case it may be advantageous to call
        ``complex_gen_precon_ilu`` again with an increased value of
        `lfill`, a reduced value of `dtol`, or set `pstrat` = 'C'.

        See also [Further Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `milu` != 'M' or 'N': `milu` = *<value>*.

        (`errno` 1)
            On entry, `pstrat` != 'N', 'U', 'P' or 'C': `pstrat` =
            *<value>*.

        (`errno` 1)
            On entry, `dtol` = *<value>*.

            Constraint: `dtol` >= 0.0

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: 1 <= `icol`[i-1] <= n, for i = 1,2,...,`nnz`

        (`errno` 2)
            On entry, I = *<value>*, `irow`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: 1 <= `irow`[i-1] <= n, for i = 1,2,...,`nnz`

        (`errno` 3)
            On entry, a user-supplied value of `ipivq` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipivq` lies outside the
            range [1,n].

        (`errno` 3)
            On entry, a user-supplied value of `ipivp` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipivp` lies outside the
            range [1,n].

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large. The decomposition has been terminated before
            completion. Either increase la, or reduce the fill by
            reducing `lfill`, or increasing `dtol`.

        (`errno` 5)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sparse import complex_gen_precon_ilu as b_complex_gen_precon_ilu

    fun_name = 'naginterfaces.library.sparse.complex_gen_precon_ilu'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _idiag = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_complex_gen_precon_ilu(
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        lfill,
        dtol,
        pstrat,
        milu,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipivp_py = _ipivp
    _ipivq_py = _ipivq
    _istr_py = _istr
    _idiag_py = _idiag
    _nnzc_py = _return_scalars.nnzc
    _npivm_py = _return_scalars.npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipivp', 'ipivq', 'istr', 'idiag', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipivp_py,
        _ipivq_py,
        _istr_py,
        _idiag_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def complex_gen_precon_ilu_solve(trans, a, irow, icol, ipivp, ipivq, istr, idiag, y, check='N'):
    u"""
    Solution of complex linear system involving incomplete LU
    preconditioning matrix generated by ``complex_gen_precon_ilu``.

    For full information please refer to the NAG Library document for
    f11dp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dpf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix M is transposed.

        `trans` = 'N'

          Mx = y is solved.

        `trans` = 'T'

          M^Tx = y is solved.

    a : complex, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``complex_gen_precon_ilu``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    ipivp : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    ipivq : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    idiag : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

    y : complex, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix M
        should be checked.

        `check` = 'C'

          Checks are carried on the values of n, `irow`, `icol`,
          `ipivp`, `ipivq`, `istr` and `idiag`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : complex, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `ipivq`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipivq`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipivq`[i-1] >= 1 and `ipivq`[i-1] <= n.

        (`errno` 3)
            On entry, `ipivp`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipivp`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipivp`[i-1] >= 1 and `ipivp`[i-1] <= n.

        (`errno` 3)
            On entry, `istr`[i-1] is inconsistent with `irow`: i =
            *<value>*.

        (`errno` 3)
            On entry, `idiag`[i-1] appears to be incorrect: i =
            *<value>*.

        (`errno` 3)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>*, and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, `istr` appears to be invalid.
    """

    from ..base.sparse import complex_gen_precon_ilu_solve as b_complex_gen_precon_ilu_solve

    fun_name = 'naginterfaces.library.sparse.complex_gen_precon_ilu_solve'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_gen_precon_ilu_solve(
        trans,
        _n,
        _a,
        _la,
        _irow,
        _icol,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def complex_gen_solve_ilu(method, nnz, a, irow, icol, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x):
    u"""
    Solution of complex sparse non-Hermitian linear system, RGMRES, CGS,
    Bi-CGSTAB or TFQMR method, preconditioner computed by
    ``complex_gen_precon_ilu`` (Black Box).

    For full information please refer to the NAG Library document for
    f11dq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dqf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    nnz : int
        The number of nonzero elements in the matrix A. This must be the
        same value as was supplied in the preceding call to
        ``complex_gen_precon_ilu``.

    a : complex, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``complex_gen_precon_ilu``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    ipivp : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    ipivq : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    idiag : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipivp`, `ipivq`,
        `istr` and `idiag` by a previous call to
        ``complex_gen_precon_ilu``.

        `ipivp` and `ipivq` are restored on exit.

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : complex, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : complex, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: `m` >= 1 and `m` <= min(n, *<value>*).

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>*, and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The CS representation of the preconditioner is invalid.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.

        (`errno` 7)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.
    """

    from ..base.sparse import complex_gen_solve_ilu as b_complex_gen_solve_ilu

    fun_name = 'naginterfaces.library.sparse.complex_gen_solve_ilu'

    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipivp, {'fun_name': fun_name, 'entity': 'ipivp'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    try:
        _return_scalars = b_complex_gen_solve_ilu(
            method,
            _n,
            nnz,
            _a,
            _la,
            _irow,
            _icol,
            _ipivp,
            _ipivq,
            _istr,
            _idiag,
            _b,
            m,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_gen_precon_ssor_solve(trans, a, irow, icol, rdiag, omega, y, check='N'):
    u"""
    Solution of linear system involving preconditioning matrix generated
    by applying SSOR to complex sparse non-Hermitian matrix.

    For full information please refer to the NAG Library document for
    f11dr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11drf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix M is transposed.

        `trans` = 'N'

          Mx = y is solved.

        `trans` = 'T'

          M^Hx = y is solved.

    a : complex, array-like, shape (nnz)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``complex_gen_sort`` may be used to order the
        elements in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in `a`.

    rdiag : complex, array-like, shape (n)
        The elements of the diagonal matrix D^-1, where D is the
        diagonal part of A.

    omega : float
        The relaxation parameter omega.

    y : complex, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix M
        should be checked.

        `check` = 'C'

          Checks are carried on the values of n, nnz, `irow`, `icol` and
          `omega`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : complex, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` != 'C' or 'N': `check` = *<value>*.

        (`errno` 1)
            On entry, `trans` != 'N' or 'T': `trans` = *<value>*.

        (`errno` 2)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 4)
            The matrix A has no diagonal entry in row *<value>*.
    """

    from ..base.sparse import complex_gen_precon_ssor_solve as b_complex_gen_precon_ssor_solve

    fun_name = 'naginterfaces.library.sparse.complex_gen_precon_ssor_solve'

    _rdiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rdiag'},
        rdiag,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_rdiag, {'fun_name': fun_name, 'entity': 'rdiag'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_gen_precon_ssor_solve(
        trans,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _rdiag,
        omega,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def complex_gen_solve_jacssor(method, precon, a, irow, icol, omega, b, m, tol, maxitn, x):
    u"""
    Solution of complex sparse non-Hermitian linear system, RGMRES, CGS,
    Bi-CGSTAB or TFQMR method, Jacobi or SSOR preconditioner Black Box.

    For full information please refer to the NAG Library document for
    f11ds

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dsf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    precon : str, length 1
        Specifies the type of preconditioning to be used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'J'

          Jacobi.

        `precon` = 'S'

          Symmetric successive-over-relaxation (SSOR).

    a : complex, array-like, shape (nnz)
        The nonzero elements of the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``complex_gen_sort`` may be used to order the
        elements in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in `a`.

    omega : float
        If `precon` = 'S', `omega` is the relaxation parameter omega to
        be used in the SSOR method. Otherwise `omega` need not be
        initialized and is not referenced.

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : complex, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : complex, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: 0 < `m` <= min(n, *<value>*).

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0

        (`errno` 1)
            On entry, `precon` = *<value>*.

            Constraint: `precon` = 'N', 'J' or 'S'.

        (`errno` 1)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: 1 <= nnz <= n^2.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The matrix A has a zero diagonal entry in row *<value>*.

        (`errno` 3)
            The matrix A has no diagonal entry in row *<value>*.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.

        (`errno` 7)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.
    """

    from ..base.sparse import complex_gen_solve_jacssor as b_complex_gen_solve_jacssor

    fun_name = 'naginterfaces.library.sparse.complex_gen_solve_jacssor'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    try:
        _return_scalars = b_complex_gen_solve_jacssor(
            method,
            precon,
            _n,
            _nnz,
            _a,
            _irow,
            _icol,
            omega,
            _b,
            m,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_gen_precon_bdilu(n, nnz, a, irow, icol, istb, indb, lfill, dtol, milu, ipivp, ipivq, pstrat=None):
    u"""
    Complex, sparse, non-Hermitian linear system, incomplete LU
    factorization of local or overlapping diagonal blocks.

    For full information please refer to the NAG Library document for
    f11dt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dtf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nnz : int
        The number of nonzero elements in the matrix A.

    a : complex, array-like, shape (la)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``complex_gen_sort`` may be used to order the
        elements in this way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    istb : int, array-like, shape (nb+1)
        `istb`[b-1], for b = 1,2,...,nb, holds the indices in arrays
        `indb`, `ipivp`, `ipivq` and `idiag` that, on successful exit
        from this function, define block b. `istb`[nb] holds the sum of
        the number of rows in all blocks plus `istb`[0].

    indb : int, array-like, shape (lindb)
        `indb` must hold the row indices appearing in each diagonal
        block, stored consecutively. Thus the elements
        `indb`[`istb`[b-1]-1] to `indb`[`istb`[b]-2] are the row indices
        in the bth block, for b = 1,2,...,nb.

    lfill : int, array-like, shape (nb)
        If `lfill`[b-1] >= 0 its value is the maximum level of fill
        allowed in the decomposition of the block b (see [Further
        Comments]). A negative value of `lfill`[b-1] indicates that
        `dtol`[b-1] will be used to control the fill in block b instead.

    dtol : float, array-like, shape (nb)
        If `lfill`[b-1] < 0 then `dtol`[b-1] is used as a drop tolerance
        in block b to control the fill-in (see [Further Comments]);
        otherwise `dtol`[b-1] is not referenced.

    milu : str, length 1, array-like, shape (nb)
        `milu`[b-1], for b = 1,2,...,nb, indicates whether or not the
        factorization in block b should be modified to preserve row-sums
        (see [Further Comments]).

        `milu`[b-1] = 'M'

          The factorization is modified.

        `milu`[b-1] = 'N'

          The factorization is not modified.

    ipivp : int, array-like, shape (lindb)
        If `pstrat`[b-1] = 'U', `ipivp`[`istb`[b-1]+k-2] and
        `ipivq`[`istb`[b-1]+k-2] must specify the row and column indices
        of the element used as a pivot at elimination stage k of the
        factorization of the block. Otherwise `ipivp` and `ipivq` need
        not be initialized.

    ipivq : int, array-like, shape (lindb)
        If `pstrat`[b-1] = 'U', `ipivp`[`istb`[b-1]+k-2] and
        `ipivq`[`istb`[b-1]+k-2] must specify the row and column indices
        of the element used as a pivot at elimination stage k of the
        factorization of the block. Otherwise `ipivp` and `ipivq` need
        not be initialized.

    pstrat : None or str, length 1, array-like, shape (nb), optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 'C'.

        `pstrat`[b-1], for b = 1,2,...,nb, specifies the pivoting
        strategy to be adopted in the block as follows:

        `pstrat`[b-1] = 'N'

          No pivoting is carried out.

        `pstrat`[b-1] = 'U'

          Pivoting is carried out according to the user-defined input
          values of `ipivp` and `ipivq`.

        `pstrat`[b-1] = 'P'

          Partial pivoting by columns for stability is carried out.

        `pstrat`[b-1] = 'C'

          Complete pivoting by rows for sparsity, and by columns for
          stability, is carried out.

    Returns
    -------
    a : complex, ndarray, shape (la)
        The first `nnz` entries of `a` contain the nonzero elements of A
        and the next `nnzc` entries contain the elements of the matrices
        C_b, for b = 1,2,...,nb stored consecutively. Within each block
        the matrix elements are ordered by increasing row index, and by
        increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipivp : int, ndarray, shape (lindb)
        The row and column indices of the pivot elements, arranged
        consecutively for each block, as for `indb`. If
        `ipivp`[`istb`[b-1]+k-2] = i and `ipivq`[`istb`[b-1]+k-2] = j,
        the element in row i and column j of A_b was used as the pivot
        at elimination stage k.

    ipivq : int, ndarray, shape (lindb)
        The row and column indices of the pivot elements, arranged
        consecutively for each block, as for `indb`. If
        `ipivp`[`istb`[b-1]+k-2] = i and `ipivq`[`istb`[b-1]+k-2] = j,
        the element in row i and column j of A_b was used as the pivot
        at elimination stage k.

    istr : int, ndarray, shape (lindb+1)
        `istr`[`istb`[b-1]+k-2], gives the index in the arrays `a`,
        `irow` and `icol` of row k of the matrix C_b, for k =
        1,2,...,`istb`[b]-`istb`[b-1], for b = 1,2,...,nb.

        `istr`[`istb`[nb]-1] contains `nnz`+`nnzc`+1.

    idiag : int, ndarray, shape (lindb)
        `idiag`[`istb`[b-1]+k-2], gives the index in the arrays `a`,
        `irow` and `icol` of the diagonal element in row k of the matrix
        C_b, for k = 1,2,...,`istb`[b]-`istb`[b-1], for b = 1,2,...,nb.

    nnzc : int
        The sum total number of nonzero elements in the matrices C_b,
        for b = 1,2,...,nb.

    npivm : int, ndarray, shape (nb)
        If `npivm`[b-1] > 0 it gives the number of pivots which were
        modified during the factorization to ensure that M_b exists.

        If `npivm`[b-1] = -1 no pivot modifications were required, but a
        local restart occurred (see [Further Comments f11dnf]). The
        quality of the preconditioner will generally depend on the
        returned values of `npivm`[b-1], for b = 1,2,...,nb.

        If `npivm`[b-1] is large, for some b, the preconditioner may not
        be satisfactory.

        In this case it may be advantageous to call
        ``complex_gen_precon_bdilu`` again with an increased value of
        `lfill`[b-1], a reduced value of `dtol`[b-1], or `pstrat`[b-1] =
        'C'.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, nb = *<value>* and `n` = *<value>*.

            Constraint: 1 <= nb <= `n`.

        (`errno` 1)
            On entry, `istb`[0] = *<value>*.

            Constraint: `istb`[0] >= 1.

        (`errno` 1)
            On entry, for b = *<value>*, `istb`[b] = *<value>* and
            `istb`[b-1] = *<value>*.

            Constraint: `istb`[b] > `istb`[b-1], for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `indb`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `indb`[m-1] <= `n`, for m =
            1,2,...,`istb`[nb]-1

        (`errno` 1)
            On entry, lindb = *<value>*, `istb`[nb]-1 = *<value>* and nb
            = *<value>*.

            Constraint: lindb >= `istb`[nb]-1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>* and `n` = *<value>*.

            Constraint: `nnz` <= `n`^2.

        (`errno` 1)
            On entry, `dtol`[*<value>*] = *<value>*.

            Constraint: `dtol`[b-1] >= 0.0, for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `pstrat`[*<value>*] = *<value>*.

            Constraint: `pstrat`[b-1] = 'N', 'U', 'P' or 'C' for all b.

        (`errno` 1)
            On entry, `milu`[*<value>*] = *<value>*.

            Constraint: `milu`[b-1] = 'M' or 'N' for all b.

        (`errno` 2)
            On entry, `irow`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `irow`[i-1] <= `n`, for i = 1,2,...,`nnz`.

        (`errno` 2)
            On entry, `icol`[*<value>*] = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `icol`[j-1] <= `n`, for j = 1,2,...,`nnz`.

        (`errno` 2)
            On entry, element *<value>* of `a` was out of order.

        (`errno` 2)
            On entry, location *<value>* of (`irow`,`icol`) was a
            duplicate.

        (`errno` 3)
            On entry, the user-supplied value of `ipivp` for block
            *<value>* lies outside its range.

        (`errno` 3)
            On entry, the user-supplied value of `ipivp` for block
            *<value>* was repeated.

        (`errno` 3)
            On entry, the user-supplied value of `ipivq` for block
            *<value>* lies outside its range.

        (`errno` 3)
            On entry, the user-supplied value of `ipivq` for block
            *<value>* was repeated.

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large.

            The decomposition has been terminated before completion.

            Either increase la, or reduce the fill by reducing `lfill`,
            or increasing `dtol`.
    """

    from ..base.sparse import complex_gen_precon_bdilu as b_complex_gen_precon_bdilu

    fun_name = 'naginterfaces.library.sparse.complex_gen_precon_bdilu'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _istb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istb'},
        istb,
        copy=False,
    )
    _nb = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_istb, {'fun_name': fun_name, 'entity': 'istb'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _indb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'indb'},
        indb,
        copy=False,
    )
    _lindb_py = _b_utils._EngineIntArrayType.get_shape(_indb, {'fun_name': fun_name, 'entity': 'indb'}, exp_rank=1)[0]
    _lindb = _b_utils._EngineIntScalarType(
        _lindb_py, {'fun_name': fun_name, 'entity': '_lindb'},
    ).to_ctypes()
    _lfill = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lfill'},
        lfill,
        copy=False,
    )
    _dtol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dtol'},
        dtol,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    if pstrat is None:
        _pstrat = _b_utils._EngineCharArrayType.empty_ctypes(_nb_py, chrlen=1)
        _pstrat[:] = ['C']*_nb_py
    else:
        _pstrat = pstrat
    _milu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'milu'},
        milu,
        copy=False,
    )
    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_lindb_py+1,),
    )
    _idiag = _b_utils._EngineIntArrayType.empty_ndarray(
        (_lindb_py,),
    )
    _npivm = _b_utils._EngineIntArrayType.empty_ndarray(
        (_nb_py,),
    )
    _return_scalar = b_complex_gen_precon_bdilu(
        n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        _nb,
        _istb,
        _indb,
        _lindb,
        _lfill,
        _dtol,
        _pstrat,
        _milu,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        _npivm,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipivp_py = _ipivp
    _ipivq_py = _ipivq
    _istr_py = _istr
    _idiag_py = _idiag
    _nnzc_py = _return_scalar
    _npivm_py = _npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipivp', 'ipivq', 'istr', 'idiag', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipivp_py,
        _ipivq_py,
        _istr_py,
        _idiag_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def complex_gen_solve_bdilu(method, nnz, a, irow, icol, istb, indb, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x):
    u"""
    Solution of complex, sparse, non-Hermitian linear system, RGMRES,
    CGS, Bi-CGSTAB or TFQMR method, incomplete LU block diagonal
    preconditioner computed by ``complex_gen_precon_bdilu``.

    For full information please refer to the NAG Library document for
    f11du

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11duf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'RGMRES'

          Restarted generalized minimum residual method.

        `method` = 'CGS'

          Conjugate gradient squared method.

        `method` = 'BICGSTAB'

          Bi-conjugate gradient stabilized (l) method.

        `method` = 'TFQMR'

          Transpose-free quasi-minimal residual method.

    nnz : int
        The value of `nnz` as supplied to a previous call to
        ``complex_gen_precon_bdilu``

    a : complex, array-like, shape (la)
        The value of `a` returned by a previous call to
        ``complex_gen_precon_bdilu``

    irow : int, array-like, shape (la)
        The value of `irow` returned by a previous call to
        ``complex_gen_precon_bdilu``

    icol : int, array-like, shape (la)
        The value of `icol` returned by a previous call to
        ``complex_gen_precon_bdilu``

    istb : int, array-like, shape (nb+1)
        The value of `istb` as supplied to a previous call to
        ``complex_gen_precon_bdilu``

    indb : int, array-like, shape (lindb)
        The value of `indb` as supplied to a previous call to
        ``complex_gen_precon_bdilu``

    ipivp : int, array-like, shape (lindb)
        The value of `ipivp` returned by a previous call to
        ``complex_gen_precon_bdilu``

    ipivq : int, array-like, shape (lindb)
        The value of `ipivq` returned by a previous call to
        ``complex_gen_precon_bdilu``

    istr : int, array-like, shape (lindb+1)
        The value of `istr` returned by a previous call to
        ``complex_gen_precon_bdilu``

    idiag : int, array-like, shape (lindb)
        The value of `idiag` returned by a previous call to
        ``complex_gen_precon_bdilu``

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    m : int
        If `method` = 'RGMRES', `m` is the dimension of the restart
        subspace.

        If `method` = 'BICGSTAB', `m` is the order l of the polynomial
        BI-CGSTAB method.

        Otherwise, `m` is not referenced.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),sqrt(n)epsilon) is
        used, where epsilon is the machine precision. Otherwise tau =
        max(`tol`,10 epsilon, sqrt(n)epsilon) is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : complex, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : complex, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n^2.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, nb = *<value>* and n = *<value>*.

            Constraint: 1 <= nb <= n.

        (`errno` 1)
            On entry, `istb`[0] = *<value>*.

            Constraint: `istb`[0] >= 1.

        (`errno` 1)
            On entry, for b = *<value>*, `istb`[b] = *<value>* and
            `istb`[b-1] = *<value>*.

            Constraint: `istb`[b] > `istb`[b-1], for b = 1,2,...,nb.

        (`errno` 1)
            On entry, `indb`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `indb`[m-1] <= n, for m =
            1,2,...,`istb`[nb]-1

        (`errno` 1)
            On entry, lindb = *<value>*, `istb`[nb]-1 = *<value>* and nb
            = *<value>*.

            Constraint: lindb >= `istb`[nb]-1.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'RGMRES', 'CGS' or 'BICGSTAB'.

        (`errno` 1)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: if `method` = 'RGMRES', 1 <= `m` <= min(n,
            *<value>*).

            If `method` = 'BICGSTAB', 1 <= `m` <= min(n, *<value>*).

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 2)
            On entry, `irow`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `irow`[i-1] <= n, for i = 1,2,...,`nnz`.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``complex_gen_precon_bdilu`` and
            ``complex_gen_solve_bdilu``.

        (`errno` 2)
            On entry, `icol`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `icol`[i-1] <= n, for i = 1,2,...,`nnz`.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``complex_gen_precon_bdilu`` and
            ``complex_gen_solve_bdilu``.

        (`errno` 2)
            On entry, element *<value>* of `a` was out of order.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``complex_gen_precon_bdilu`` and
            ``complex_gen_solve_bdilu``.

        (`errno` 2)
            On entry, location *<value>* of (`irow`,`icol`) was a
            duplicate.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``complex_gen_precon_bdilu`` and
            ``complex_gen_solve_bdilu``.

        (`errno` 3)
            The CS representation of the preconditioner is invalid.

            Check that `a`, `irow`, `icol`, `ipivp`, `ipivq`, `istr` and
            `idiag` have not been corrupted between calls to
            ``complex_gen_precon_bdilu`` and
            ``complex_gen_solve_bdilu``.

        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            Algorithmic breakdown. A solution is returned, although it
            is possible that it is completely inaccurate.
    """

    from ..base.sparse import complex_gen_solve_bdilu as b_complex_gen_solve_bdilu

    fun_name = 'naginterfaces.library.sparse.complex_gen_solve_bdilu'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istb'},
        istb,
        copy=False,
    )
    _nb = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_istb, {'fun_name': fun_name, 'entity': 'istb'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _indb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'indb'},
        indb,
        copy=False,
    )
    _lindb_py = _b_utils._EngineIntArrayType.get_shape(_indb, {'fun_name': fun_name, 'entity': 'indb'}, exp_rank=1)[0]
    _lindb = _b_utils._EngineIntScalarType(
        _lindb_py, {'fun_name': fun_name, 'entity': '_lindb'},
    ).to_ctypes()
    _ipivp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivp'},
        ipivp,
        copy=True,
    )
    _ipivq = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipivq'},
        ipivq,
        copy=True,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _idiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'idiag'},
        idiag,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _return_scalars = b_complex_gen_solve_bdilu(
        method,
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        _nb,
        _istb,
        _indb,
        _lindb,
        _ipivp,
        _ipivq,
        _istr,
        _idiag,
        _b,
        m,
        tol,
        maxitn,
        _x,
    )
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    return _return_data

def complex_gen_precon_jacobi(store, trans, init, niter, a, irow, icol, b, diag, check='N'):
    u"""
    Complex, sparse, Hermitian or non-Hermitian, linear systems, line
    Jacobi preconditioner.

    For full information please refer to the NAG Library document for
    f11dx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11dxf.html

    Parameters
    ----------
    store : str, length 1
        Specifies whether the matrix A is stored using symmetric
        coordinate storage (SCS) (applicable only to a Hermitian matrix
        A) or coordinate storage (CS) (applicable to both Hermitian and
        non-Hermitian matrices).

        `store` = 'N'

          The complete matrix A is stored in CS format.

        `store` = 'S'

          The lower triangle of the Hermitian matrix A is stored in SCS
          format.

    trans : str, length 1
        If `store` = 'N', specifies whether the approximate solution of
        Ax = b or of A^Hx = b is required.

        `trans` = 'N'

          The approximate solution of Ax = b is calculated.

        `trans` = 'T'

          The approximate solution of A^Hx = b is calculated.

    init : str, length 1
        On first entry, `init` should be set to 'I', unless the diagonal
        elements of A are already stored in the array `diag`. If `diag`
        already contains the diagonal of A, it must be set to 'N'.

        `init` = 'N'

          `diag` must contain the diagonal of A.

        `init` = 'I'

          `diag` will store the diagonal of A on exit.

    niter : int
        The number of Jacobi iterations requested.

    a : complex, array-like, shape (nnz)
        If `store` = 'N', the nonzero elements in the matrix A (CS
        format).

        If `store` = 'S', the nonzero elements in the lower triangle of
        the matrix A (SCS format).

        In both cases, the elements of either A or of its lower triangle
        must be ordered by increasing row index and by increasing column
        index within each row.

        Multiple entries for the same row and columns indices are not
        permitted.

        The function ``complex_gen_sort`` or ``complex_herm_sort`` may
        be used to reorder the elements in this way for CS and SCS
        storage, respectively.

    irow : int, array-like, shape (nnz)
        If `store` = 'N', the row indices of the nonzero elements
        supplied in `a`.

        If `store` = 'S', the row and column indices of the nonzero
        elements of the lower triangle of the matrix A supplied in `a`.

    icol : int, array-like, shape (nnz)
        If `store` = 'N', the column indices of the nonzero elements
        supplied in `a`.

        If `store` = 'S', the row and column indices of the nonzero
        elements of the lower triangle of the matrix A supplied in `a`.

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    diag : complex, array-like, shape (n)
        If `init` = 'N', the diagonal elements of A.

    check : str, length 1, optional
        Specifies whether or not the CS or SCS representation of the
        matrix A should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow`,
          `icol`; if `init` = 'N', `diag` is also checked.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : complex, ndarray, shape (n)
        The approximate solution vector x_`niter`.

    diag : complex, ndarray, shape (n)
        If `init` = 'N', unchanged on exit.

        If `init` = 'I', the diagonal elements of A.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `niter` = *<value>*.

            Constraint: `niter` >= 1.

        (`errno` 1)
            On entry, `check` != 'N' or 'C': `check` = *<value>*.

        (`errno` 1)
            On entry, `init` != 'N' or 'I': `init` = *<value>*.

        (`errno` 1)
            On entry, `trans` != 'N' or 'T': `trans` = *<value>*.

        (`errno` 1)
            On entry, `store` != 'N' or 'S': `store` = *<value>*.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, I = *<value>*, `irow`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[I-1] >= 1 and `irow`[I-1] <= n.

        (`errno` 4)
            On entry, the element `diag`[I-1] is zero: I = *<value>*.

        (`errno` 5)
            On entry, the diagonal element of the Ith row is zero or
            missing: I = *<value>*.
    """

    from ..base.sparse import complex_gen_precon_jacobi as b_complex_gen_precon_jacobi

    fun_name = 'naginterfaces.library.sparse.complex_gen_precon_jacobi'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _diag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
        copy=True,
    )
    b_complex_gen_precon_jacobi(
        store,
        trans,
        init,
        niter,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _b,
        _x,
        _diag,
    )
    _diag_py = _diag
    _x_py = _x
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'diag'],
    )(
        _x_py,
        _diag_py,
    )
    return _return_data

def real_symm_basic_setup(method, precon, n, tol, maxitn, anorm, sigmax, maxits, monit, sigcmp='N', norm=None, weight='N', iterm=1, sigtol=0.01, lwork=None):
    u"""
    Real sparse symmetric linear systems, setup for
    ``real_symm_basic_solver``.

    For full information please refer to the NAG Library document for
    f11gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11gdf.html

    Parameters
    ----------
    method : str
        The iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method (CG).

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

        `method` = 'MINRES'

          Minimum residual method (MINRES).

    precon : str, length 1
        Determines whether preconditioning is used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'P'

          Preconditioning.

    n : int
        n, the order of the matrix A.

    tol : float
        The tolerance tau for the termination criterion.

        If `tol` <= 0.0, tau = max(sqrt(epsilon), sqrt(n)epsilon) is
        used, where epsilon is the machine precision.

        Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon) is used.

    maxitn : int
        The maximum number of iterations.

    anorm : float
        If `anorm` > 0.0, the value of ||A||_p to be used in the
        termination criterion (2) (`iterm` = 1).

        If `anorm` <= 0.0, `iterm` = 1 and `norm` = '1' or 'I', ||A||_1
        = ||A||_infinity is estimated internally by
        ``real_symm_basic_solver``.

        If `iterm` = 2, `anorm` is not referenced.

        It has no effect if `method` = 'MINRES'.

    sigmax : float
        If `sigmax` > 0.0, the value of sigma_1(A_) =
        ||E^-1AE^invTH||_2.

        If `sigmax` <= 0.0, sigma_1(A_) is estimated by
        ``real_symm_basic_solver`` when either `sigcmp` = 'S' or
        termination criterion (3) (`iterm` = 2) is employed, though it
        will be used only in the latter case.

        Otherwise, or if `method` = 'MINRES', `sigmax` is not
        referenced.

    maxits : int
        The maximum iteration number k = `maxits` for which sigma_1(T_k)
        is computed by bisection (see also [Description]). If `sigcmp` =
        'N' or `sigmax` > 0.0, or if `method` = 'MINRES', `maxits` is
        not referenced.

    monit : int
        If `monit` > 0, the frequency at which a monitoring step is
        executed by ``real_symm_basic_solver``: the current solution and
        residual iterates will be returned by ``real_symm_basic_solver``
        and a call to ``real_symm_basic_diag`` made possible every
        `monit` iterations, starting from the (`monit`)th. Otherwise, no
        monitoring takes place.

        There are some additional computational costs involved in
        monitoring the solution and residual vectors when the Lanczos
        method (SYMMLQ) is used.

    sigcmp : str, length 1, optional
        Determines whether an estimate of sigma_1(A_) =
        ||E^-1AE^invTH||_2, the largest singular value of the
        preconditioned matrix of the coefficients, is to be computed
        using the bisection method on the sequence of tridiagonal
        matrices {T_k} generated during the iteration. Note that A_= A
        when a preconditioner is not used.

        If `sigmax` > 0.0 , i.e., when sigma_1(A_) is supplied, the
        value of `sigcmp` is ignored.

        `sigcmp` = 'S'

          sigma_1(A_) is to be computed using the bisection method.

        `sigcmp` = 'N'

          The bisection method is not used.

          If the termination criterion (3) is used, requiring
          sigma_1(A_), an inexpensive estimate is computed and used (see
          [Description]).

        It is not used if `method` = 'MINRES'.

    norm : None or str, length 1, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `iterm` = 1: 'I'; otherwise:
        '2'.

        If `method` = 'CG' or 'SYMMLQ', `norm` defines the matrix and
        vector norm to be used in the termination criteria.

        `norm` = '1'

          Use the l_1 norm.

        `norm` = 'I'

          Use the l_infinity norm.

        `norm` = '2'

          Use the l_2 norm.

        It has no effect if `method` = 'MINRES'.

    weight : str, length 1, optional
        Specifies whether a vector w of user-supplied weights is to be
        used in the vector norms used in the computation of termination
        criterion (2) (`iterm` = 1): ||v||_p^(w) = ||v^(w)||_p, where
        v_i^(w) = w_i v_i, for i = 1,2,...,n. The suffix p = 1,2,
        infinity denotes the vector norm used, as specified by the
        argument `norm`. Note that weights cannot be used when `iterm` =
        2, i.e., when criterion (3) is used.

        `weight` = 'W'

          User-supplied weights are to be used and must be supplied on
          initial entry to ``real_symm_basic_solver``.

        `weight` = 'N'

          All weights are implicitly set equal to one. Weights do not
          need to be supplied on initial entry to
          ``real_symm_basic_solver``.

        It has no effect if `method` = 'MINRES'.

    iterm : int, optional
        Defines the termination criterion to be used.

        `iterm` = 1

          Use the termination criterion defined in (2) (both conjugate
          gradient and Lanczos (SYMMLQ) methods).

        `iterm` = 2

          Use the termination criterion defined in (3) (Lanczos method
          (SYMMLQ) only).

        It has no effect if `method` = 'MINRES'.

    sigtol : float, optional
        The tolerance used in assessing the convergence of the estimate
        of sigma_1(A_) = ||A_||_2 when the bisection method is used.

        If `sigtol` <= 0.0, the default value `sigtol` = 0.01 is used.

        The actual value used is max(`sigtol`, epsilon).

        If `sigcmp` = 'N' or `sigmax` > 0.0, `sigtol` is not referenced.

        It has no effect if `method` = 'MINRES'.

    lwork : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 120+9*`n`+2*(`maxits`+1).

        The dimension of the array `comm`['work'].

    Returns
    -------
    lwreq : int
        The minimum amount of workspace required by
        ``real_symm_basic_solver``. (See also [Parameters].)

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` -14)
            On entry, `monit` = *<value>* and `maxitn` = *<value>*.

            Constraint: `monit` <= `maxitn`.

        (`errno` -13)
            On entry, `sigcmp` = 'S', `sigmax` <= 0.0, `maxits` =
            *<value>* and `maxitn` = *<value>*.

            Constraint: if `sigcmp` = 'S' and `sigmax` <= 0.0, `maxits`
            <= `maxitn`.

        (`errno` -13)
            On entry, `sigcmp` = 'S', `sigmax` <= 0.0 and `maxits` =
            *<value>*.

            Constraint: if `sigcmp` = 'S' and `sigmax` <= 0.0, `maxits`
            >= 1.

        (`errno` -12)
            On entry, `sigcmp` = 'S', `sigmax` <= 0.0 and `sigtol` =
            *<value>*.

            Constraint: if `sigcmp` = 'S' and `sigmax` <= 0.0, `sigtol`
            < 1.0.

        (`errno` -10)
            On entry, `iterm` = 1, `norm` = '2' and `anorm` = *<value>*.

            Constraint: if `iterm` = 1 and `norm` = '2', `anorm` > 0.0.

        (`errno` -9)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` > 0.

        (`errno` -8)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` -7)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` -6)
            On entry, `iterm` = 2 and `method` = 'CG'.

            Constraint: if `iterm` = 2, `method` != 'CG'.

        (`errno` -6)
            On entry, `iterm` = 2 and `norm` = *<value>*.

            Constraint: if `iterm` = 2, `norm` = '2'.

        (`errno` -6)
            On entry, `iterm` = 2 and `weight` = *<value>*.

            Constraint: if `iterm` = 2, `weight` = 'N'.

        (`errno` -6)
            On entry, `iterm` = *<value>*.

            Constraint: `iterm` = 1 or 2.

        (`errno` -5)
            On entry, `weight` = *<value>*.

            Constraint: `weight` = 'N' or 'W'.

        (`errno` -4)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'I' or '2'.

        (`errno` -3)
            On entry, `sigcmp` = *<value>*.

            Constraint: `sigcmp` = 'S' or 'N'.

        (`errno` -2)
            On entry, `precon` = *<value>*.

            Constraint: `precon` = 'N' or 'P'.

        (`errno` -1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'CG', 'SYMMLQ' or 'MINRES'.

        (`errno` 1)
            ``real_symm_basic_setup`` has been called out of sequence:
            either ``real_symm_basic_setup`` has been called twice or
            ``real_symm_basic_solver`` has not terminated its current
            task.
    """

    from ..base.sparse import real_symm_basic_setup as b_real_symm_basic_setup

    fun_name = 'naginterfaces.library.sparse.real_symm_basic_setup'

    _iterm = _b_utils._EngineIntScalarType(
        iterm, {'fun_name': fun_name, 'entity': 'iterm'},
    ).to_ctypes()
    _iterm_py = _b_utils._EngineIntScalarType.to_py(_iterm)
    if _iterm_py == 1:
        _norm = 'I'
    else:
        _norm = '2'
    _norm = _norm if norm is None else norm
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _maxits = _b_utils._EngineIntScalarType(
        maxits, {'fun_name': fun_name, 'entity': 'maxits'},
    ).to_ctypes()
    _maxits_py = _b_utils._EngineIntScalarType.to_py(_maxits)
    _lwork = 120+9*_n_py+2*(_maxits_py+1) if lwork is None else lwork
    _return_scalars = b_real_symm_basic_setup(
        method,
        precon,
        sigcmp,
        _norm,
        weight,
        _iterm,
        _n,
        tol,
        maxitn,
        anorm,
        sigmax,
        sigtol,
        _maxits,
        monit,
        _lwork,
    )
    _lwreq_py = _return_scalars.lwreq
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lwreq', 'comm'],
    )(
        _lwreq_py,
        _comm_py,
    )
    return _return_data

def real_symm_basic_solver(irevcm, u, v, wgt, comm):
    u"""
    Real sparse symmetric linear systems, preconditioned conjugate
    gradient or Lanczos method or the MINRES algorithm.

    For full information please refer to the NAG Library document for
    f11ge

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11gef.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must either be unchanged from its
        previous exit value, or can have one of the following values.

        `irevcm` = 5

          Tidy termination: the computation will terminate at the end of
          the current iteration. Further reverse communication exits may
          occur depending on when the termination request is issued.
          ``real_symm_basic_solver`` will then return with the
          termination code `irevcm` = 4. Note that before calling
          ``real_symm_basic_solver`` with `irevcm` = 5 the calling
          program must have performed the tasks required by the value of
          `irevcm` returned by the previous call to
          ``real_symm_basic_solver``, otherwise subsequently returned
          values may be invalid.

        `irevcm` = 6

          Immediate termination: ``real_symm_basic_solver`` will return
          immediately with termination code `irevcm` = 4 and with any
          useful information available. This includes the last iterate
          of the solution and, for conjugate gradient only, the last
          iterate of the residual vector. The residual vector is
          generally not available when the Lanczos method (SYMMLQ) is
          used. ``real_symm_basic_solver`` will then return with the
          termination code `irevcm` = 4.

          Immediate termination may be useful, for example, when errors
          are detected during matrix-vector multiplication or during the
          solution of the preconditioning equation.

        Changing `irevcm` to any other value between calls will result
        in an error.

    u : float, ndarray, shape (n), modified in place
        On initial entry: an initial estimate, x_0, of the solution of
        the system of equations Ax = b.

        On intermediate entry: must remain unchanged.

        On intermediate exit: the returned value of `irevcm` determines
        the contents of `u` in the following way.

        If `irevcm` = 1 or 2, `u` holds the vector u on which the
        operation specified by `irevcm` is to be carried out.

        If `irevcm` = 3, `u` holds the current iterate of the solution
        vector.

        On final exit: if `errno` = 3 or -i, the array `u` is unchanged
        from the initial entry to ``real_symm_basic_solver``. If `errno`
        = 1, the array `u` is unchanged from the last entry to
        ``real_symm_basic_solver``. Otherwise, `u` holds the last
        iterate of the solution of the system of equations, for all
        returned values of errno.

    v : float, ndarray, shape (n), modified in place
        On initial entry: the right-hand side b of the system of
        equations Ax = b.

        On intermediate entry: the returned value of `irevcm` determines
        the contents of `v` in the following way.

        If `irevcm` = 1 or 2, `v` must store the vector v, the result of
        the operation specified by the value of `irevcm` returned by the
        previous call to ``real_symm_basic_solver``.

        If `irevcm` = 3, `v` must remain unchanged.

        On intermediate exit: if `irevcm` = 3, `v` holds the current
        iterate of the residual vector. Note that this is an
        approximation to the true residual vector. Otherwise, it does
        not contain any useful information.

        On final exit: if `errno` = 3 or -i, the array `v` is unchanged
        from the last entry to ``real_symm_basic_solver``. If `errno` =
        1, the array `v` is unchanged from the initial entry to
        ``real_symm_basic_solver``. If the function exits successfully
        or `errno` = 2, the array `v` contains the true residual vector
        of the system of equations (see also [Errors]). Otherwise, `v`
        stores the last iterate of the residual vector unless the
        Lanczos method (SYMMLQ) was used and `errno` >= 5, in which case
        `v` is set to 0.0.

    wgt : float, array-like, shape (n)
        The user-supplied weights, if these are to be used in the
        computation of the vector norms in the termination criterion
        (see [Description] and [Parameters]).

        Weights are NOT used in the MINRES algorithm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_basic_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = 1

          The calling program must compute the matrix-vector product v =
          Au, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 2

          The calling program must solve the preconditioning equation Mv
          = u, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 3

          Monitoring step: the solution and residual at the current
          iteration are returned in the arrays `u` and `v`,
          respectively. No action by the calling program is required. To
          return additional information ``real_symm_basic_diag`` can be
          called at this step.

        On final exit: if `irevcm` = 4, ``real_symm_basic_solver`` has
        completed its tasks. The value of errno determines whether the
        iteration has been successfully completed, errors have been
        detected or the calling program has requested termination.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: either `irevcm` must be unchanged from its
            previous exit value or `irevcm` = 5 or 6.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 3)
            Either ``real_symm_basic_setup`` was not called before
            calling this function or it has returned an error.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 9)
            The matrix of the coefficients A appears to be singular. The
            computation cannot continue.

        (`errno` 10)
            The weights in array `wgt` are all zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            ``real_symm_basic_solver`` has already completed its tasks.
            You need to set a new problem.

        (`errno` 2)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 2)
            User-requested termination: the required accuracy could not
            be obtained. However, a reasonable accuracy may have been
            achieved.

        (`errno` 4)
            User-requested tidy termination. The solution has not
            converged after *<value>* iterations.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 7)
            The matrix of the coefficients A appears not to be positive
            definite. The computation cannot continue.

        (`errno` 8)
            User-requested immediate termination.
    """

    from ..base.sparse import real_symm_basic_solver as b_real_symm_basic_solver

    fun_name = 'naginterfaces.library.sparse.real_symm_basic_solver'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_real_symm_basic_solver(
            irevcm,
            u,
            v,
            _wgt,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def real_symm_basic_diag(comm):
    u"""
    Real sparse symmetric linear systems, diagnostic for
    ``real_symm_basic_solver``.

    For full information please refer to the NAG Library document for
    f11gf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11gff.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_basic_setup``.

    Returns
    -------
    itn : int
        The number of iterations carried out by
        ``real_symm_basic_solver``.

    stplhs : float
        The current value of the left-hand side of the termination
        criterion used by ``real_symm_basic_solver``.

    stprhs : float
        The current value of the right-hand side of the termination
        criterion used by ``real_symm_basic_solver``.

    anorm : float
        For CG and SYMMLQ methods, the norm ||A||_1 = ||A||_infinity
        when either it has been supplied to ``real_symm_basic_setup`` or
        it has been estimated by ``real_symm_basic_solver`` (see also
        [Description] and [Parameters]). Otherwise, `anorm` = 0.0 is
        returned.

        For MINRES method, an estimate of the infinity norm of the
        preconditioned matrix operator.

    sigmax : float
        For CG and SYMMLQ methods, the current estimate of the largest
        singular value sigma_1(A_) of the preconditioned iteration
        matrix A_= E^-1AE^invT, when either it has been supplied to
        ``real_symm_basic_setup`` or it has been estimated by
        ``real_symm_basic_solver`` (see also [Description] and
        [Parameters]). Note that if `its` < `itn` then `sigmax` contains
        the final estimate. If, on final exit from
        ``real_symm_basic_solver``, `its` = `itn`, the estimation of
        sigma_1(A_) may have not converged; in this case you should look
        at the value returned in `sigerr`. Otherwise, `sigmax` = 0.0 is
        returned.

        For MINRES method, an estimate of the final transformed
        residual.

    its : int
        For CG and SYMMLQ methods, the number of iterations employed so
        far in the computation of the estimate of sigma_1(A_), the
        largest singular value of the preconditioned matrix A_=
        E^-1AE^invTH, when sigma_1(A_) has been estimated by
        ``real_symm_basic_solver`` using the bisection method (see also
        [Description], [Parameters] and [Further Comments]). Otherwise,
        `its` = 0 is returned.

    sigerr : float
        For CG and SYMMLQ methods, if sigma_1(A_) has been estimated by
        ``real_symm_basic_solver`` using bisection,

            `sigerr` = max(|sigma_1^(k)-sigma_1^(k-1)|/sigma_1^(k),
            |sigma_1^(k)-sigma_1^(k-2)|/sigma_1^(k)),

        where k = `its` denotes the iteration number. The estimation has
        converged if `sigerr` <= sigtol where sigtol is an input
        argument to ``real_symm_basic_setup``. Otherwise, `sigerr` = 0.0
        is returned.

        For MINRES method, an estimate of the condition number of the
        preconditioned matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            ``real_symm_basic_diag`` has been called out of sequence.
    """

    from ..base.sparse import real_symm_basic_diag as b_real_symm_basic_diag

    _return_scalars = b_real_symm_basic_diag(
        comm,
    )
    _itn_py = _return_scalars.itn
    _stplhs_py = _return_scalars.stplhs
    _stprhs_py = _return_scalars.stprhs
    _anorm_py = _return_scalars.anorm
    _sigmax_py = _return_scalars.sigmax
    _its_py = _return_scalars.its
    _sigerr_py = _return_scalars.sigerr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['itn', 'stplhs', 'stprhs', 'anorm', 'sigmax', 'its', 'sigerr'],
    )(
        _itn_py,
        _stplhs_py,
        _stprhs_py,
        _anorm_py,
        _sigmax_py,
        _its_py,
        _sigerr_py,
    )
    return _return_data

def complex_herm_basic_setup(method, precon, n, tol, maxitn, anorm, sigmax, maxits, monit, sigcmp='N', norm=None, weight='N', iterm=1, sigtol=0.01, lwork=None):
    u"""
    Complex sparse Hermitian linear systems, setup for
    ``complex_herm_basic_solver``.

    For full information please refer to the NAG Library document for
    f11gr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11grf.html

    Parameters
    ----------
    method : str
        The iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method.

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

    precon : str, length 1
        Determines whether preconditioning is used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'P'

          Preconditioning.

    n : int
        n, the order of the matrix A.

    tol : float
        The tolerance tau for the termination criterion.

        If `tol` <= 0.0, tau = max(sqrt(epsilon), sqrt(n)epsilon) is
        used, where epsilon is the machine precision.

        Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon) is used.

    maxitn : int
        The maximum number of iterations.

    anorm : float
        If `anorm` > 0.0, the value of ||A||_p to be used in the
        termination criterion (2) (`iterm` = 1).

        If `anorm` <= 0.0, `iterm` = 1 and `norm` = '1' or 'I', ||A||_1
        = ||A||_infinity is estimated internally by
        ``complex_herm_basic_solver``.

        If `iterm` = 2, `anorm` is not referenced.

    sigmax : float
        If `sigmax` > 0.0, the value of sigma_1(A_) =
        ||E^-1AE^invTH||_2.

        If `sigmax` <= 0.0, sigma_1(A_) is estimated by
        ``complex_herm_basic_solver`` when either `sigcmp` = 'S' or
        termination criterion (3) (`iterm` = 2) is employed, though it
        will be used only in the latter case.

        Otherwise, `sigmax` is not referenced.

    maxits : int
        The maximum iteration number k = `maxits` for which sigma_1(T_k)
        is computed by bisection (see also [Description]). If `sigcmp` =
        'N' or `sigmax` > 0.0, `maxits` is not referenced.

    monit : int
        If `monit` > 0, the frequency at which a monitoring step is
        executed by ``complex_herm_basic_solver``: the current solution
        and residual iterates will be returned by
        ``complex_herm_basic_solver`` and a call to
        ``complex_herm_basic_diag`` made possible every `monit`
        iterations, starting from iteration number `monit`. Otherwise,
        no monitoring takes place. There are some additional
        computational costs involved in monitoring the solution and
        residual vectors when the Lanczos method (SYMMLQ) is used.

    sigcmp : str, length 1, optional
        Determines whether an estimate of sigma_1(A_) =
        ||E^-1AE^invTH||_2, the largest singular value of the
        preconditioned matrix of the coefficients, is to be computed
        using the bisection method on the sequence of tridiagonal
        matrices {T_k} generated during the iteration. Note that A_= A
        when a preconditioner is not used.

        If `sigmax` > 0.0 , i.e., when sigma_1(A_) is supplied, the
        value of `sigcmp` is ignored.

        `sigcmp` = 'S'

          sigma_1(A_) is to be computed using the bisection method.

        `sigcmp` = 'N'

          The bisection method is not used.

          If the termination criterion (3) is used, requiring
          sigma_1(A_), an inexpensive estimate is computed and used (see
          [Description]).

    norm : None or str, length 1, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `iterm` = 1: 'I'; otherwise:
        '2'.

        Defines the matrix and vector norm to be used in the termination
        criteria.

        `norm` = '1'

          Use the l_1 norm.

        `norm` = 'I'

          Use the l_infinity norm.

        `norm` = '2'

          Use the l_2 norm.

    weight : str, length 1, optional
        Specifies whether a vector w of user-supplied weights is to be
        used in the vector norms used in the computation of termination
        criterion (2) (`iterm` = 1): ||v||_p^(w) = ||v^(w)||_p, where
        v_i^(w) = w_i v_i, for i = 1,2,...,n. The suffix p = 1,2,
        infinity denotes the vector norm used, as specified by the
        argument `norm`. Note that weights cannot be used when `iterm` =
        2, i.e., when criterion (3) is used.

        `weight` = 'W'

          User-supplied weights are to be used and must be supplied on
          initial entry to ``complex_herm_basic_solver``.

        `weight` = 'N'

          All weights are implicitly set equal to one. Weights do not
          need to be supplied on initial entry to
          ``complex_herm_basic_solver``.

    iterm : int, optional
        Defines the termination criterion to be used.

        `iterm` = 1

          Use the termination criterion defined in (2) (both conjugate
          gradient and Lanczos (SYMMLQ) methods).

        `iterm` = 2

          Use the termination criterion defined in (3) (Lanczos method
          (SYMMLQ) only).

    sigtol : float, optional
        The tolerance used in assessing the convergence of the estimate
        of sigma_1(A_) = ||A_||_2 when the bisection method is used.

        If `sigtol` <= 0.0, the default value `sigtol` = 0.01 is used.

        The actual value used is max(`sigtol`, epsilon).

        If `sigcmp` = 'N' or `sigmax` > 0.0, `sigtol` is not referenced.

    lwork : None or int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 120+6*`n`+2*(`maxits`+1).

        The dimension of the array `comm`['work'].

    Returns
    -------
    lwreq : int
        The minimum amount of workspace required by
        ``complex_herm_basic_solver``. (See also [Parameters].)

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` -14)
            On entry, `monit` = *<value>* and `maxitn` = *<value>*.

            Constraint: `monit` <= `maxitn`.

        (`errno` -13)
            On entry, `maxits` = *<value>*, `maxitn` = *<value>*,
            `sigcmp` = *<value>*, and `sigmax` = *<value>*.

            Constraint: if `sigcmp` = 'S' and `sigmax` <= 0.0, 1 <=
            `maxits` <= `maxitn`.

        (`errno` -12)
            On entry, `sigcmp` = *<value>*, `sigtol` = *<value>*, and
            `sigmax` = *<value>*.

            Constraint: if `sigcmp` = 'S' and `sigmax` <= 0.0, `sigtol`
            < 1.0.

        (`errno` -10)
            On entry, `norm` = *<value>*, `iterm` = *<value>*, and
            `anorm` = *<value>*.

            Constraint: if `iterm` = 1 and `norm` = '2' or 'I', `anorm`
            > 0.0.

        (`errno` -9)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` > 0.

        (`errno` -8)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` -7)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` -6)
            On entry, `iterm` = *<value>*.

            Constraint: `iterm` = 1 or 2.

        (`errno` -6)
            On entry, `iterm` = *<value>* and `method` = *<value>*.

            Constraint: if `iterm` = 2, `method` = 'SYMMLQ'.

        (`errno` -6)
            On entry, `iterm` = *<value>* and `norm` = *<value>*.

            Constraint: if `iterm` = 2, `norm` = '2' or 'I'.

        (`errno` -5)
            On entry, `weight` = *<value>*.

            Constraint: `weight` = 'N' or 'W'.

        (`errno` -5)
            On entry, `iterm` = *<value>* and `weight` = *<value>*.

            Constraint: if `iterm` = 2, `weight` = 'N'.

        (`errno` -4)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'I' or '2'.

        (`errno` -3)
            On entry, `sigcmp` = *<value>*.

            Constraint: `sigcmp` = 'N' or 'S'.

        (`errno` -2)
            On entry, `precon` = *<value>*.

            Constraint: `precon` = 'N' or 'P'.

        (`errno` -1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'CG' or 'SYMMLQ'.

        (`errno` 1)
            ``complex_herm_basic_setup`` has been called out of
            sequence: either ``complex_herm_basic_setup`` has been
            called twice or ``complex_herm_basic_solver`` has not
            terminated its current task.
    """

    from ..base.sparse import complex_herm_basic_setup as b_complex_herm_basic_setup

    fun_name = 'naginterfaces.library.sparse.complex_herm_basic_setup'

    _iterm = _b_utils._EngineIntScalarType(
        iterm, {'fun_name': fun_name, 'entity': 'iterm'},
    ).to_ctypes()
    _iterm_py = _b_utils._EngineIntScalarType.to_py(_iterm)
    if _iterm_py == 1:
        _norm = 'I'
    else:
        _norm = '2'
    _norm = _norm if norm is None else norm
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _maxits = _b_utils._EngineIntScalarType(
        maxits, {'fun_name': fun_name, 'entity': 'maxits'},
    ).to_ctypes()
    _maxits_py = _b_utils._EngineIntScalarType.to_py(_maxits)
    _lwork = 120+6*_n_py+2*(_maxits_py+1) if lwork is None else lwork
    _return_scalars = b_complex_herm_basic_setup(
        method,
        precon,
        sigcmp,
        _norm,
        weight,
        _iterm,
        _n,
        tol,
        maxitn,
        anorm,
        sigmax,
        sigtol,
        _maxits,
        monit,
        _lwork,
    )
    _lwreq_py = _return_scalars.lwreq
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lwreq', 'comm'],
    )(
        _lwreq_py,
        _comm_py,
    )
    return _return_data

def complex_herm_basic_solver(irevcm, u, v, wgt, comm):
    u"""
    Complex sparse Hermitian linear systems, preconditioned conjugate
    gradient or Lanczos.

    For full information please refer to the NAG Library document for
    f11gs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11gsf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: `irevcm` must either be unchanged from
        its previous exit value, or can have one of the following
        values.

        `irevcm` = 5

          Tidy termination: the computation will terminate at the end of
          the current iteration. Further reverse communication exits may
          occur depending on when the termination request is issued.
          ``complex_herm_basic_solver`` will then return with the
          termination code `irevcm` = 4. Note that before calling
          ``complex_herm_basic_solver`` with `irevcm` = 5 the calling
          program must have performed the tasks required by the value of
          `irevcm` returned by the previous call to
          ``complex_herm_basic_solver``, otherwise subsequently returned
          values may be invalid.

        `irevcm` = 6

          Immediate termination: ``complex_herm_basic_solver`` will
          return immediately with termination code `irevcm` = 4 and with
          any useful information available. This includes the last
          iterate of the solution and, for conjugate gradient only, the
          last iterate of the residual vector. The residual vector is
          generally not available when the Lanczos method (SYMMLQ) is
          used. ``complex_herm_basic_solver`` will then return with the
          termination code `irevcm` = 4.

          Immediate termination may be useful, for example, when errors
          are detected during matrix-vector multiplication or during the
          solution of the preconditioning equation.

        Changing `irevcm` to any other value between calls will result
        in an error.

    u : complex, ndarray, shape (n), modified in place
        On initial entry: an initial estimate, x_0, of the solution of
        the system of equations Ax = b.

        On intermediate entry: must remain unchanged.

        On intermediate exit: the returned value of `irevcm` determines
        the contents of `u` in the following way.

        If `irevcm` = 1 or 2, `u` holds the vector u on which the
        operation specified by `irevcm` is to be carried out.

        If `irevcm` = 3, `u` holds the current iterate of the solution
        vector.

        On final exit: if `errno` = 3 or -i, the array `u` is unchanged
        from the initial entry to ``complex_herm_basic_solver``. If
        `errno` = 1, the array `u` is unchanged from the last entry to
        ``complex_herm_basic_solver``. Otherwise, `u` holds the last
        iterate of the solution of the system of equations, for all
        returned values of errno.

    v : complex, ndarray, shape (n), modified in place
        On initial entry: the right-hand side b of the system of
        equations Ax = b.

        On intermediate entry: the returned value of `irevcm` determines
        the contents of `v` in the following way.

        If `irevcm` = 1 or 2, `v` must store the vector v, the result of
        the operation specified by the value of `irevcm` returned by the
        previous call to ``complex_herm_basic_solver``

        If `irevcm` = 3, `v` must remain unchanged.

        On intermediate exit: if `irevcm` = 3, `v` holds the current
        iterate of the residual vector. Note that this is an
        approximation to the true residual vector. Otherwise, it does
        not contain any useful information.

        On final exit: if `errno` = 3 or 0, the array `v` is unchanged
        from the last entry to ``complex_herm_basic_solver``. If `errno`
        = 1, the array `v` is unchanged from the initial entry to
        ``complex_herm_basic_solver``. If the function exits
        successfully or `errno` = 2, the array `v` contains the true
        residual vector of the system of equations (see also [Errors]).
        Otherwise, `v` stores the last iterate of the residual vector
        unless the Lanczos method (SYMMLQ) was used and `errno` >= 5, in
        which case `v` is set to 0.0.

    wgt : float, array-like, shape (n)
        The user-supplied weights, if these are to be used in the
        computation of the vector norms in the termination criterion
        (see [Description] and [Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_herm_basic_setup``.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = 1

          The calling program must compute the matrix-vector product v =
          Au, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 2

          The calling program must solve the preconditioning equation Mv
          = u, where u and v are stored in `u` and `v`, respectively.

        `irevcm` = 3

          Monitoring step: the solution and residual at the current
          iteration are returned in the arrays `u` and `v`,
          respectively. No action by the calling program is required. To
          return additional information ``complex_herm_basic_diag`` can
          be called at this step.

        On final exit: if `irevcm` = 4, ``complex_herm_basic_solver``
        has completed its tasks. The value of errno determines whether
        the iteration has been successfully completed, errors have been
        detected or the calling program has requested termination.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On intermediate re-entry, `irevcm` = *<value>*.

            Constraint: either `irevcm` must be unchanged from its
            previous exit value or `irevcm` = 5 or 6.

        (`errno` -1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 3)
            Either ``complex_herm_basic_setup`` was not called before
            calling ``complex_herm_basic_solver`` or it has returned an
            error.

        (`errno` 9)
            The matrix of the coefficients A appears to be singular. The
            computation cannot continue.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            ``complex_herm_basic_solver`` has already completed its
            tasks. You need to set a new problem.

        (`errno` 2)
            The required accuracy could not be obtained. However, a
            reasonable accuracy may have been achieved.

        (`errno` 2)
            User-requested termination: the required accuracy could not
            be obtained. However, a reasonable accuracy may have been
            achieved.

        (`errno` 4)
            User-requested tidy termination. The solution has not
            converged after *<value>* iterations.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 7)
            The matrix of the coefficients A appears not to be positive
            definite. The computation cannot continue.

        (`errno` 8)
            User-requested immediate termination.

        (`errno` 10)
            The weights in array `wgt` are all zero.
    """

    from ..base.sparse import complex_herm_basic_solver as b_complex_herm_basic_solver

    fun_name = 'naginterfaces.library.sparse.complex_herm_basic_solver'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_complex_herm_basic_solver(
            irevcm,
            u,
            v,
            _wgt,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def complex_herm_basic_diag(comm):
    u"""
    Complex sparse Hermitian linear systems, diagnostic for
    ``complex_herm_basic_solver``.

    For full information please refer to the NAG Library document for
    f11gt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11gtf.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_herm_basic_solver``.

    Returns
    -------
    itn : int
        The number of iterations carried out by
        ``complex_herm_basic_solver``.

    stplhs : float
        The current value of the left-hand side of the termination
        criterion used by ``complex_herm_basic_solver``.

    stprhs : float
        The current value of the right-hand side of the termination
        criterion used by ``complex_herm_basic_solver``.

    anorm : float
        The norm ||A||_1 = ||A||_infinity when either it has been
        supplied to ``complex_herm_basic_setup`` or it has been
        estimated by ``complex_herm_basic_solver`` (see also
        [Description] and [Parameters]).

        Otherwise, `anorm` = 0.0 is returned.

    sigmax : float
        The current estimate of the largest singular value sigma_1(A_)
        of the preconditioned iteration matrix A_= E^-1AE^(-H), when
        either it has been supplied to ``complex_herm_basic_setup`` or
        it has been estimated by ``complex_herm_basic_solver`` (see also
        [Description] and [Parameters]). Note that if `its` < `itn` then
        `sigmax` contains the final estimate. If, on final exit from
        ``complex_herm_basic_solver``, `its` = `itn`, the estimation of
        sigma_1(A_) may have not converged: in this case you should look
        at the value returned in `sigerr`. Otherwise, `sigmax` = 0.0 is
        returned.

    its : int
        The number of iterations employed so far in the computation of
        the estimate of sigma_1(A_), the largest singular value of the
        preconditioned matrix A_= E^-1AE^(-H), when sigma_1(A_) has been
        estimated by ``complex_herm_basic_solver`` using the bisection
        method (see also [Description], [Parameters] and [Further
        Comments]). Otherwise, `its` = 0 is returned.

    sigerr : float
        If sigma_1(A_) has been estimated by
        ``complex_herm_basic_solver`` using bisection,

            `sigerr` = max(|sigma_1^(k)-sigma_1^(k-1)|/sigma_1^(k),
            |sigma_1^(k)-sigma_1^(k-2)|/sigma_1^(k)),

        where k = `its` denotes the iteration number. The estimation has
        converged if `sigerr` <= sigtol where sigtol is an input
        argument to ``complex_herm_basic_setup``.

        Otherwise, `sigerr` = 0.0 is returned.

    Raises
    ------
    NagValueError
        (`errno` 1)
            ``complex_herm_basic_diag`` has been called out of sequence.
    """

    from ..base.sparse import complex_herm_basic_diag as b_complex_herm_basic_diag

    _return_scalars = b_complex_herm_basic_diag(
        comm,
    )
    _itn_py = _return_scalars.itn
    _stplhs_py = _return_scalars.stplhs
    _stprhs_py = _return_scalars.stprhs
    _anorm_py = _return_scalars.anorm
    _sigmax_py = _return_scalars.sigmax
    _its_py = _return_scalars.its
    _sigerr_py = _return_scalars.sigerr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['itn', 'stplhs', 'stprhs', 'anorm', 'sigmax', 'its', 'sigerr'],
    )(
        _itn_py,
        _stplhs_py,
        _stprhs_py,
        _anorm_py,
        _sigmax_py,
        _its_py,
        _sigerr_py,
    )
    return _return_data

def real_symm_precon_ichol(nnz, a, irow, icol, lfill, dtol, mic, dscale, ipiv, pstrat='M'):
    u"""
    Real sparse symmetric matrix, incomplete Cholesky factorization.

    For full information please refer to the NAG Library document for
    f11ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jaf.html

    Parameters
    ----------
    nnz : int
        The number of nonzero elements in the lower triangular part of
        the matrix A.

    a : float, array-like, shape (la)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``real_symm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    lfill : int
        If `lfill` >= 0 its value is the maximum level of fill allowed
        in the decomposition (see [Further Comments]). A negative value
        of `lfill` indicates that `dtol` will be used to control the
        fill instead.

    dtol : float
        If `lfill` < 0, `dtol` is used as a drop tolerance to control
        the fill-in (see [Further Comments]); otherwise `dtol` is not
        referenced.

    mic : str, length 1
        Indicates whether or not the factorization should be modified to
        preserve row sums (see [Further Comments]).

        `mic` = 'M'

          The factorization is modified.

        `mic` = 'N'

          The factorization is not modified.

    dscale : float
        The diagonal scaling parameter. All diagonal elements are
        multiplied by the factor (1+`dscale`) at the start of the
        factorization. This can be used to ensure that the
        preconditioner is positive definite. See [Further Comments].

    ipiv : int, array-like, shape (n)
        If `pstrat` = 'U', `ipiv`[i-1] must specify the row index of the
        diagonal element used as a pivot at elimination stage i.
        Otherwise `ipiv` need not be initialized.

    pstrat : str, length 1, optional
        Specifies the pivoting strategy to be adopted.

        `pstrat` = 'N'

          No pivoting is carried out.

        `pstrat` = 'M'

          Diagonal pivoting aimed at minimizing fill-in is carried out,
          using the Markowitz strategy.

        `pstrat` = 'U'

          Diagonal pivoting is carried out according to the user-defined
          input value of `ipiv`.

    Returns
    -------
    a : float, ndarray, shape (la)
        The first `nnz` elements of `a` contain the nonzero elements of
        A and the next `nnzc` elements contain the elements of the lower
        triangular matrix C. Matrix elements are ordered by increasing
        row index, and by increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipiv : int, ndarray, shape (n)
        The pivot indices. If `ipiv`[i-1] = j then the diagonal element
        in row j was used as the pivot at elimination stage i.

    istr : int, ndarray, shape (n+1)
        `istr`[i-1]-1, for i = 1,2,...,n, is the starting address in the
        arrays `a`, `irow` and `icol` of row i of the matrix C.
        `istr`[n]-1 is the address of the last nonzero element in C plus
        one.

    nnzc : int
        The number of nonzero elements in the lower triangular matrix C.

    npivm : int
        The number of pivots which were modified during the
        factorization to ensure that M was positive definite. The
        quality of the preconditioner will generally depend on the
        returned value of `npivm`. If `npivm` is large the
        preconditioner may not be satisfactory. In this case it may be
        advantageous to call ``real_symm_precon_ichol`` again with an
        increased value of either `lfill` or `dscale`. See also [Further
        Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n*(n+1)/2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `pstrat` != 'N', 'U' or 'M': `pstrat` = *<value>*.

        (`errno` 1)
            On entry, `mic` != 'M' or 'N': `mic` = *<value>*.

        (`errno` 1)
            On entry, `dtol` = *<value>*.

            Constraint: `dtol` >= 0.0.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, a user-supplied value of `ipiv` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipiv` lies outside the
            range [1,n].

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large. The decomposition has been terminated before
            completion. Either increase la, or reduce the fill by
            setting `pstrat` = 'M', reducing `lfill`, or increasing
            `dtol`.

        (`errno` 5)
            A serious error has occurred in an internal call. Check all
            function calls and array sizes. Seek expert help.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_symm_precon_ichol as b_real_symm_precon_ichol

    fun_name = 'naginterfaces.library.sparse.real_symm_precon_ichol'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalars = b_real_symm_precon_ichol(
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        lfill,
        dtol,
        mic,
        dscale,
        pstrat,
        _ipiv,
        _istr,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipiv_py = _ipiv
    _istr_py = _istr
    _nnzc_py = _return_scalars.nnzc
    _npivm_py = _return_scalars.npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipiv', 'istr', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipiv_py,
        _istr_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def real_symm_precon_ichol_solve(a, irow, icol, ipiv, istr, y, check='N'):
    u"""
    Solution of linear system involving incomplete Cholesky
    preconditioning matrix generated by ``real_symm_precon_ichol``.

    For full information please refer to the NAG Library document for
    f11jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jbf.html

    Parameters
    ----------
    a : float, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``real_symm_precon_ichol``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    ipiv : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    y : float, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the input data should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, `irow`, `icol`,
          `ipiv` and `istr`.

        `check` = 'N'

          No checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `ipiv`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipiv`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipiv`[i-1] >= 1 and `ipiv`[i-1] <= n.

        (`errno` 3)
            On entry, `istr`[i-1] is inconsistent with `irow`: i =
            *<value>*.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, `istr` appears to be invalid.
    """

    from ..base.sparse import real_symm_precon_ichol_solve as b_real_symm_precon_ichol_solve

    fun_name = 'naginterfaces.library.sparse.real_symm_precon_ichol_solve'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_symm_precon_ichol_solve(
        _n,
        _a,
        _la,
        _irow,
        _icol,
        _ipiv,
        _istr,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def real_symm_solve_ichol(method, nnz, a, irow, icol, ipiv, istr, b, tol, maxitn, x):
    u"""
    Solution of real sparse symmetric linear system, conjugate
    gradient/Lanczos method, preconditioner computed by
    ``real_symm_precon_ichol`` (Black Box).

    For full information please refer to the NAG Library document for
    f11jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jcf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method.

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

    nnz : int
        The number of nonzero elements in the lower triangular part of
        the matrix A. This must be the same value as was supplied in the
        preceding call to ``real_symm_precon_ichol``.

    a : float, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``real_symm_precon_ichol``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    ipiv : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``real_symm_precon_ichol``.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : float, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : float, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'CG' or 'SYMMLQ'.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n*(n+1)/2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>*,
            `irow`[i-1] = *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= `irow`[i-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The SCS representation of the preconditioner is invalid.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 7)
            The matrix of the coefficients `a` appears not to be
            positive definite. The computation cannot continue.

        (`errno` 8)
            A serious error has occurred in an internal call: IREVCM =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

        (`errno` 8)
            A serious error has occurred in an internal call: errno =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However a
            reasonable accuracy has been achieved.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_symm_solve_ichol as b_real_symm_solve_ichol

    fun_name = 'naginterfaces.library.sparse.real_symm_solve_ichol'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _la_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = b_real_symm_solve_ichol(
            method,
            _n,
            nnz,
            _a,
            _la,
            _irow,
            _icol,
            _ipiv,
            _istr,
            _b,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_symm_precon_ssor_solve(a, irow, icol, rdiag, omega, y, check='N'):
    u"""
    Solution of linear system involving preconditioning matrix generated
    by applying SSOR to real sparse symmetric matrix.

    For full information please refer to the NAG Library document for
    f11jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jdf.html

    Parameters
    ----------
    a : float, array-like, shape (nnz)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``real_symm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    rdiag : float, array-like, shape (n)
        The elements of the diagonal matrix D^-1, where D is the
        diagonal part of A.

    omega : float
        The relaxation parameter omega.

    y : float, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the input data should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow`, `icol`
          and `omega`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    x : float, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` != 'C' or 'N': `check` = *<value>*.

        (`errno` 2)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 4)
            The matrix A has no diagonal entry in row *<value>*.
    """

    from ..base.sparse import real_symm_precon_ssor_solve as b_real_symm_precon_ssor_solve

    fun_name = 'naginterfaces.library.sparse.real_symm_precon_ssor_solve'

    _rdiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rdiag'},
        rdiag,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_rdiag, {'fun_name': fun_name, 'entity': 'rdiag'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_symm_precon_ssor_solve(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _rdiag,
        omega,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def real_symm_solve_jacssor(method, precon, a, irow, icol, omega, b, tol, maxitn, x):
    u"""
    Solution of real sparse symmetric linear system, conjugate
    gradient/Lanczos method, Jacobi or SSOR preconditioner (Black Box).

    For full information please refer to the NAG Library document for
    f11je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jef.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method.

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

    precon : str, length 1
        Specifies the type of preconditioning to be used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'J'

          Jacobi.

        `precon` = 'S'

          Symmetric successive-over-relaxation (SSOR).

    a : float, array-like, shape (nnz)
        The nonzero elements of the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``real_symm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    omega : float
        If `precon` = 'S', `omega` is the relaxation parameter omega to
        be used in the SSOR method. Otherwise `omega` need not be
        initialized.

    b : float, array-like, shape (n)
        The right-hand side vector b.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : float, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : float, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'CG' or 'SYMMLQ'.

        (`errno` 1)
            On entry, `omega` = *<value>*.

            Constraint: `omega` > 0.0 and `omega` < 2.0.

        (`errno` 1)
            On entry, `precon` != 'N', 'J' or 'S': `precon` = *<value>*.

        (`errno` 1)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The matrix A has a zero diagonal entry in row *<value>*.

        (`errno` 3)
            The matrix A has no diagonal entry in row *<value>*.

        (`errno` 4)
            The required accuracy could not be obtained. However a
            reasonable accuracy has been achieved.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 7)
            The matrix of the coefficients `a` appears not to be
            positive definite. The computation cannot continue.

        (`errno` 8)
            A serious error has occurred in an internal call: IREVCM =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

        (`errno` 8)
            A serious error has occurred in an internal call: errno =
            *<value>*. Check all function calls and array sizes. Seek
            expert help.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sparse import real_symm_solve_jacssor as b_real_symm_solve_jacssor

    fun_name = 'naginterfaces.library.sparse.real_symm_solve_jacssor'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = b_real_symm_solve_jacssor(
        method,
        precon,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        omega,
        _b,
        tol,
        maxitn,
        _x,
    )
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    return _return_data

def complex_herm_precon_ichol(nnz, a, irow, icol, lfill, dtol, mic, dscale, ipiv, pstrat='M'):
    u"""
    Complex sparse Hermitian matrix, incomplete Cholesky factorization.

    For full information please refer to the NAG Library document for
    f11jn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jnf.html

    Parameters
    ----------
    nnz : int
        The number of nonzero elements in the lower triangular part of
        the matrix A.

    a : complex, array-like, shape (la)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``complex_herm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (la)
        The row indices of the nonzero elements supplied in `a`.

    icol : int, array-like, shape (la)
        The column indices of the nonzero elements supplied in `a`.

    lfill : int
        If `lfill` >= 0 its value is the maximum level of fill allowed
        in the decomposition (see [Further Comments]). A negative value
        of `lfill` indicates that `dtol` will be used to control the
        fill instead.

    dtol : float
        If `lfill` < 0, `dtol` is used as a drop tolerance to control
        the fill-in (see [Further Comments]); otherwise `dtol` is not
        referenced.

    mic : str, length 1
        Indicates whether or not the factorization should be modified to
        preserve row sums (see [Further Comments]).

        `mic` = 'M'

          The factorization is modified.

        `mic` = 'N'

          The factorization is not modified.

    dscale : float
        The diagonal scaling parameter. All diagonal elements are
        multiplied by the factor (1.0+`dscale`) at the start of the
        factorization. This can be used to ensure that the
        preconditioner is positive definite. See also [Further
        Comments].

    ipiv : int, array-like, shape (n)
        If `pstrat` = 'U', `ipiv`[i-1] must specify the row index of the
        diagonal element to be used as a pivot at elimination stage i.
        Otherwise `ipiv` need not be initialized.

    pstrat : str, length 1, optional
        Specifies the pivoting strategy to be adopted.

        `pstrat` = 'N'

          No pivoting is carried out.

        `pstrat` = 'M'

          Diagonal pivoting aimed at minimizing fill-in is carried out,
          using the Markowitz strategy.

        `pstrat` = 'U'

          Diagonal pivoting is carried out according to the user-defined
          input array `ipiv`.

    Returns
    -------
    a : complex, ndarray, shape (la)
        The first `nnz` elements of `a` contain the nonzero elements of
        A and the next `nnzc` elements contain the elements of the lower
        triangular matrix C. Matrix elements are ordered by increasing
        row index, and by increasing column index within each row.

    irow : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    icol : int, ndarray, shape (la)
        The row and column indices of the nonzero elements returned in
        `a`.

    ipiv : int, ndarray, shape (n)
        The pivot indices. If `ipiv`[i-1] = j, the diagonal element in
        row j was used as the pivot at elimination stage i.

    istr : int, ndarray, shape (n+1)
        `istr`[i-1]-1, for i = 1,2,...,n, is the starting address in the
        arrays `a`, `irow` and `icol` of row i of the matrix C.
        `istr`[n]-1 is the address of the last nonzero element in C plus
        one.

    nnzc : int
        The number of nonzero elements in the lower triangular matrix C.

    npivm : int
        The number of pivots which were modified during the
        factorization to ensure that M was positive definite. The
        quality of the preconditioner will generally depend on the
        returned value of `npivm`. If `npivm` is large the
        preconditioner may not be satisfactory. In this case it may be
        advantageous to call ``complex_herm_precon_ichol`` again with an
        increased value of either `lfill` or `dscale`. See also [Further
        Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n*(n+1)/2

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `pstrat` != 'N', 'U' or 'M': `pstrat` = *<value>*.

        (`errno` 1)
            On entry, `mic` != 'M' or 'N': `mic` = *<value>*.

        (`errno` 1)
            On entry, `dtol` = *<value>*.

            Constraint: `dtol` >= 0.0

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 2)
            On entry, I = *<value>*, `irow`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[I-1] >= 1 and `irow`[I-1] <= n.

        (`errno` 3)
            On entry, a user-supplied value of `ipiv` is repeated.

        (`errno` 3)
            On entry, a user-supplied value of `ipiv` lies outside the
            range [1,n].

        (`errno` 4)
            The number of nonzero entries in the decomposition is too
            large. The decomposition has been terminated before
            completion. Either increase la, or reduce the fill by
            setting `pstrat` = 'M', reducing `lfill`, or increasing
            `dtol`.

        (`errno` 5)
            A serious error has occurred in an internal call. Check all
            function calls and array sizes. Seek expert help.
    """

    from ..base.sparse import complex_herm_precon_ichol as b_complex_herm_precon_ichol

    fun_name = 'naginterfaces.library.sparse.complex_herm_precon_ichol'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalars = b_complex_herm_precon_ichol(
        _n,
        nnz,
        _a,
        _la,
        _irow,
        _icol,
        lfill,
        dtol,
        mic,
        dscale,
        pstrat,
        _ipiv,
        _istr,
    )
    _a_py = _a
    _irow_py = _irow
    _icol_py = _icol
    _ipiv_py = _ipiv
    _istr_py = _istr
    _nnzc_py = _return_scalars.nnzc
    _npivm_py = _return_scalars.npivm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'ipiv', 'istr', 'nnzc', 'npivm'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _ipiv_py,
        _istr_py,
        _nnzc_py,
        _npivm_py,
    )
    return _return_data

def complex_herm_precon_ilu_solve(a, irow, icol, ipiv, istr, y, check='N'):
    u"""
    Solution of complex linear system involving incomplete Cholesky
    preconditioning matrix generated by ``complex_herm_precon_ichol``.

    For full information please refer to the NAG Library document for
    f11jp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jpf.html

    Parameters
    ----------
    a : complex, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``complex_herm_precon_ichol``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    ipiv : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    y : complex, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the input data should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, `irow`, `icol`,
          `ipiv` and `istr`.

        `check` = 'N'

          None of these checks are carried out.

    Returns
    -------
    x : complex, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `ipiv`[i-1] is a repeated value: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `ipiv`[i-1] = *<value>*, n =
            *<value>*.

            Constraint: `ipiv`[i-1] >= 1 and `ipiv`[i-1] <= n.

        (`errno` 3)
            On entry, `istr`[i-1] is inconsistent with `irow`: i =
            *<value>*.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, `istr` appears to be invalid.
    """

    from ..base.sparse import complex_herm_precon_ilu_solve as b_complex_herm_precon_ilu_solve

    fun_name = 'naginterfaces.library.sparse.complex_herm_precon_ilu_solve'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_herm_precon_ilu_solve(
        _n,
        _a,
        _la,
        _irow,
        _icol,
        _ipiv,
        _istr,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def complex_herm_solve_ilu(method, nnz, a, irow, icol, ipiv, istr, b, tol, maxitn, x):
    u"""
    Solution of complex sparse Hermitian linear system, conjugate
    gradient/Lanczos method, preconditioner computed by
    ``complex_herm_precon_ichol`` (Black Box).

    For full information please refer to the NAG Library document for
    f11jq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jqf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method.

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

    nnz : int
        The number of nonzero elements in the lower triangular part of
        the matrix A. This must be the same value as was supplied in the
        preceding call to ``complex_herm_precon_ichol``.

    a : complex, array-like, shape (la)
        The values returned in the array `a` by a previous call to
        ``complex_herm_precon_ichol``.

    irow : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    icol : int, array-like, shape (la)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    ipiv : int, array-like, shape (n)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    istr : int, array-like, shape (n+1)
        The values returned in arrays `irow`, `icol`, `ipiv` and `istr`
        by a previous call to ``complex_herm_precon_ichol``.

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : complex, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : complex, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||_infinity, where k
        is the output value of `itn`.

    itn : int
        The number of iterations carried out.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'CG' or 'SYMMLQ'.

        (`errno` 1)
            On entry, la = *<value>* and `nnz` = *<value>*.

            Constraint: la >= 2*`nnz`.

        (`errno` 1)
            On entry, `nnz` = *<value>* and n = *<value>*.

            Constraint: `nnz` <= n*(n+1)/2.

        (`errno` 1)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[i-1],`icol`[i-1]) is a
            duplicate: i = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>*,
            `irow`[i-1] = *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= `irow`[i-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            On entry, `istr` appears to be invalid.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 7)
            The matrix of the coefficients `a` appears not to be
            positive definite. The computation cannot continue.

        (`errno` 8)
            A serious error, code *<value>*, has occurred in an internal
            call. Check all function calls and array sizes. Seek expert
            help.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy could not be obtained. However a
            reasonable accuracy has been achieved.
    """

    from ..base.sparse import complex_herm_solve_ilu as b_complex_herm_solve_ilu

    fun_name = 'naginterfaces.library.sparse.complex_herm_solve_ilu'

    _ipiv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ipiv'},
        ipiv,
        copy=True,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_ipiv, {'fun_name': fun_name, 'entity': 'ipiv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _la_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _la = _b_utils._EngineIntScalarType(
        _la_py, {'fun_name': fun_name, 'entity': '_la'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _istr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istr'},
        istr,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    try:
        _return_scalars = b_complex_herm_solve_ilu(
            method,
            _n,
            nnz,
            _a,
            _la,
            _irow,
            _icol,
            _ipiv,
            _istr,
            _b,
            tol,
            maxitn,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_herm_precon_ssor_solve(a, irow, icol, rdiag, omega, y, check='N'):
    u"""
    Solution of linear system involving preconditioning matrix generated
    by applying SSOR to complex sparse Hermitian matrix.

    For full information please refer to the NAG Library document for
    f11jr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jrf.html

    Parameters
    ----------
    a : complex, array-like, shape (nnz)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``complex_herm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    rdiag : float, array-like, shape (n)
        The elements of the diagonal matrix D^-1, where D is the
        diagonal part of A. Note that since A is Hermitian the elements
        of D^-1 are necessarily real.

    omega : float
        The relaxation parameter omega.

    y : complex, array-like, shape (n)
        The right-hand side vector y.

    check : str, length 1, optional
        Specifies whether or not the input data should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow`, `icol`
          and `omega`.

        `check` = 'N'

          None of these checks are carried out.

    Returns
    -------
    x : complex, ndarray, shape (n)
        The solution vector x.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>*,
            `irow`[I-1] = *<value>*.

            Constraint: 1 <= `icol`[i-1] <= `irow`[i-1].

        (`errno` 3)
            On entry, I = *<value>*, `irow`[I-1] = *<value>* and n =
            *<value>*.

            Constraint: 1 <= `irow`[i-1] <= n.

        (`errno` 4)
            The matrix A has no diagonal entry in row *<value>*.
    """

    from ..base.sparse import complex_herm_precon_ssor_solve as b_complex_herm_precon_ssor_solve

    fun_name = 'naginterfaces.library.sparse.complex_herm_precon_ssor_solve'

    _rdiag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rdiag'},
        rdiag,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_rdiag, {'fun_name': fun_name, 'entity': 'rdiag'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_herm_precon_ssor_solve(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _rdiag,
        omega,
        check,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def complex_herm_solve_jacssor(method, precon, a, irow, icol, omega, b, tol, maxitn, x):
    u"""
    Solution of complex sparse Hermitian linear system, conjugate
    gradient/Lanczos method, Jacobi or SSOR preconditioner (Black Box).

    For full information please refer to the NAG Library document for
    f11js

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11jsf.html

    Parameters
    ----------
    method : str
        Specifies the iterative method to be used.

        `method` = 'CG'

          Conjugate gradient method.

        `method` = 'SYMMLQ'

          Lanczos method (SYMMLQ).

    precon : str, length 1
        Specifies the type of preconditioning to be used.

        `precon` = 'N'

          No preconditioning.

        `precon` = 'J'

          Jacobi.

        `precon` = 'S'

          Symmetric successive-over-relaxation (SSOR).

    a : complex, array-like, shape (nnz)
        The nonzero elements of the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``complex_herm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    omega : float
        If `precon` = 'S', `omega` is the relaxation parameter omega to
        be used in the SSOR method. Otherwise `omega` need not be
        initialized.

    b : complex, array-like, shape (n)
        The right-hand side vector b.

    tol : float
        The required tolerance. Let x_k denote the approximate solution
        at iteration k, and r_k the corresponding residual. The
        algorithm is considered to have converged at iteration k if

            ||r_k||_infinity <=
            tau*(||b||_infinity+||A||_infinity||x_k||_infinity).

        If `tol` <= 0.0, tau = max(sqrt(epsilon),10
        epsilon,sqrt(n)epsilon) is used, where epsilon is the machine
        precision. Otherwise tau = max(`tol`,10 epsilon, sqrt(n)epsilon)
        is used.

    maxitn : int
        The maximum number of iterations allowed.

    x : complex, array-like, shape (n)
        An initial approximation to the solution vector x.

    Returns
    -------
    x : complex, ndarray, shape (n)
        An improved approximation to the solution vector x.

    rnorm : float
        The final value of the residual norm ||r_k||, where k is the
        output value of `itn`.

    itn : int
        The number of iterations carried out.

    rdiag : float, ndarray, shape (n)
        The elements of the diagonal matrix D^-1, where D is the
        diagonal part of A. Note that since A is Hermitian the elements
        of D^-1 are necessarily real.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxitn` = *<value>*.

            Constraint: `maxitn` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` < 1.0.

        (`errno` 1)
            On entry, `method` != 'CG' or 'SYMMLQ': `method` =
            *<value>*.

        (`errno` 1)
            On entry, `omega` = *<value>*.

            Constraint: 0.0 < `omega` < 2.0.

        (`errno` 1)
            On entry, `precon` != 'N', 'J' or 'S': `precon` = *<value>*.

        (`errno` 1)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 2)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.

        (`errno` 3)
            The matrix A has a zero diagonal entry in row *<value>*.

        (`errno` 3)
            The matrix A has no diagonal entry in row *<value>*.

        (`errno` 4)
            The required accuracy could not be obtained. However, a
            reasonable accuracy has been achieved and further iterations
            could not improve the result.

        (`errno` 5)
            The solution has not converged after *<value>* iterations.

        (`errno` 6)
            The preconditioner appears not to be positive definite. The
            computation cannot continue.

        (`errno` 7)
            The matrix of the coefficients `a` appears not to be
            positive definite. The computation cannot continue.

        (`errno` 8)
            A serious error, code *<value>*, has occurred in an internal
            call. Check all function calls and array sizes. Seek expert
            help.

        (`errno` 9)
            The matrix A has a non-real diagonal entry in row *<value>*.
    """

    from ..base.sparse import complex_herm_solve_jacssor as b_complex_herm_solve_jacssor

    fun_name = 'naginterfaces.library.sparse.complex_herm_solve_jacssor'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _rdiag = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = b_complex_herm_solve_jacssor(
        method,
        precon,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        omega,
        _b,
        tol,
        maxitn,
        _x,
        _rdiag,
    )
    _x_py = _x
    _rnorm_py = _return_scalars.rnorm
    _itn_py = _return_scalars.itn
    _rdiag_py = _rdiag
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rnorm', 'itn', 'rdiag'],
    )(
        _x_py,
        _rnorm_py,
        _itn_py,
        _rdiag_py,
    )
    return _return_data

def direct_real_gen_setup(spec, n, icolzp, irowix, iprm):
    u"""
    Real sparse nonsymmetric linear systems, setup for
    ``direct_real_gen_lu``.

    For full information please refer to the NAG Library document for
    f11md

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mdf.html

    Parameters
    ----------
    spec : str, length 1
        Indicates the permutation to be applied.

        `spec` = 'N'

          The identity permutation is used.

        `spec` = 'U'

          The permutation in the `iprm` array is used, as supplied by
          you.

        `spec` = 'M'

          The permutation computed by the COLAMD algorithm is used

    n : int
        n, the order of the matrix A.

    icolzp : int, array-like, shape (max(1, (`n`+1)))
        The new column index array of sparse matrix A. See [the F11
        Introduction].

    irowix : int, array-like, shape (max(1, (`icolzp`[max(1,
    (`n`+1))-1]-1)))
        `irowix`[i-1] contains the row index in A for element A(i). See
        [the F11 Introduction].

    iprm : int, array-like, shape (max(1, (7*`n`)))
        The first `n` entries contain the column permutation if supplied
        by the user. This will be used if `spec` = 'U', and ignored
        otherwise. If used, it must consist of a permutation of all the
        integers in the range [0,(`n`-1)], the leftmost column of the
        matrix A denoted by 0 and the rightmost by `n`-1. Labelling
        columns in this way, `iprm`[i] = j means that column i-1 of A is
        in position j in A*P_c, where P_r AP_c = LU expresses the
        factorization to be performed.

    Returns
    -------
    iprm : int, ndarray, shape (max(1, (7*`n`)))
        The column permutation given or computed is returned in the
        second `n` entries. The rest of the array contains data
        structures that will be used by other functions in the suite.
        The function computes the column elimination tree for A and a
        post-order permutation on the tree. It then compounds the `iprm`
        permutation given or computed by the COLAMD algorthm with the
        post-order permutation and this permutation is returned in the
        first `n` entries. This whole array is needed by the LU
        factorization function ``direct_real_gen_lu`` and associated
        functions ``direct_real_gen_solve``, ``direct_real_gen_refine``
        and ``direct_real_gen_diag`` and should be passed to them
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            On entry, `spec` = *<value>*.

            Constraint: `spec` = 'N', 'U' or 'M'.

        (`errno` 2)
            Incorrect column permutations in array `iprm`.

        (`errno` 3)
            COLAMD algorithm failed.

        (`errno` 4)
            Incorrect specification of argument `icolzp`.

        (`errno` 5)
            Incorrect specification of argument `irowix`.
    """

    from ..base.sparse import direct_real_gen_setup as b_direct_real_gen_setup

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_setup'

    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _iprm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iprm'},
        iprm,
        copy=True,
    )
    b_direct_real_gen_setup(
        spec,
        n,
        _icolzp,
        _irowix,
        _iprm,
    )
    _iprm_py = _iprm
    return _iprm_py

def direct_real_gen_lu(irowix, a, iprm, thresh, nzlmx, nzlumx, nzumx):
    u"""
    LU factorization of real sparse matrix.

    For full information please refer to the NAG Library document for
    f11me

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mef.html

    Parameters
    ----------
    irowix : int, array-like, shape (:)
        The row index array of sparse matrix A. See [the F11
        Introduction].

    a : float, array-like, shape (:)
        The array of nonzero values in the sparse matrix A.

    iprm : int, array-like, shape (7*n)
        Contains the column permutation which defines the permutation
        P_c and associated data structures as computed by function
        ``direct_real_gen_setup``.

    thresh : float
        The diagonal pivoting threshold, t. At step j of the Gaussian
        elimination, if |A_{jj}| >= t*(max_{i >= j}|A_{ij}|), use A_{jj}
        as a pivot, otherwise use max_{i >= j}|A_{ij}|. A value of t = 1
        corresponds to partial pivoting, a value of t = 0 corresponds to
        always choosing the pivot on the diagonal (unless it is zero).

    nzlmx : int
        Indicates the available size of array `il`. The dimension of
        `il` should be at least 7*n+`nzlmx`+4. A good range for `nzlmx`
        that works for many problems is nnz to 8*nnz, where nnz is the
        number of nonzeros in the sparse matrix A. If, on exit, `errno`
        = 2, the given `nzlmx` was too small and you should attempt to
        provide more storage and call the function again.

    nzlumx : int
        Indicates the available size of array `lval`. The dimension of
        `lval` should be at least `nzlumx`.

    nzumx : int
        Indicates the available sizes of arrays `iu` and `uval`. The
        dimension of `iu` should be at least 2*n+`nzumx`+1 and the
        dimension of `uval` should be at least `nzumx`. A good range for
        `nzumx` that works for many problems is nnz to 8*nnz, where nnz
        is the number of nonzeros in the sparse matrix A. If, on exit,
        `errno` = 3, the given `nzumx` was too small and you should
        attempt to provide more storage and call the function again.

    Returns
    -------
    iprm : int, ndarray, shape (7*n)
        Part of the array is modified to record the row permutation P_r
        determined by pivoting.

    nzlumx : int
        If `errno` = 4, the given `nzlumx` was too small and is reset to
        a value that will be sufficient. You should then provide the
        indicated storage and call the function again.

    il : int, ndarray, shape (7*n+`nzlmx`+4)
        Encapsulates the sparsity pattern of matrix L.

    lval : float, ndarray, shape (`nzlumx`)
        Records the nonzero values of matrix L and some of the nonzero
        values of matrix U.

    iu : int, ndarray, shape (2*n+`nzumx`+1)
        Encapsulates the sparsity pattern of matrix U.

    uval : float, ndarray, shape (`nzumx`)
        Records some of the nonzero values of matrix U.

    nnzl : int
        The number of nonzero values in the matrix L.

    nnzu : int
        The number of nonzero values in the matrix U.

    flop : float
        The number of floating-point operations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nzumx` = *<value>*.

            Constraint: `nzumx` >= 1.

        (`errno` 1)
            On entry, `nzlumx` = *<value>*.

            Constraint: `nzlumx` >= 1.

        (`errno` 1)
            On entry, `nzlmx` = *<value>*.

            Constraint: `nzlmx` >= 1.

        (`errno` 1)
            On entry, `thresh` = *<value>*.

            Constraint: 0.0 <= `thresh` <= 1.0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` 2)
            Insufficient `nzlmx`.

        (`errno` 3)
            Insufficient `nzumx`.

        (`errno` 4)
            Insufficient `nzlumx`.

        (`errno` 5)
            The matrix is singular -- no factorization possible.
    """

    from ..base.sparse import direct_real_gen_lu as b_direct_real_gen_lu

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_lu'

    _iprm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iprm'},
        iprm,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_iprm, {'fun_name': fun_name, 'entity': 'iprm'}, exp_rank=1)[0]//7, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _nzlmx = _b_utils._EngineIntScalarType(
        nzlmx, {'fun_name': fun_name, 'entity': 'nzlmx'},
    ).to_ctypes()
    _nzlmx_py = _b_utils._EngineIntScalarType.to_py(_nzlmx)
    _il = _b_utils._EngineIntArrayType.empty_ndarray(
        (7*_n_py+_nzlmx_py+4,),
    )
    _nzlumx = _b_utils._EngineIntScalarType(
        nzlumx, {'fun_name': fun_name, 'entity': 'nzlumx'},
    ).to_ctypes()
    _nzlumx_py = _b_utils._EngineIntScalarType.to_py(_nzlumx)
    _lval = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nzlumx_py,),
    )
    _nzumx = _b_utils._EngineIntScalarType(
        nzumx, {'fun_name': fun_name, 'entity': 'nzumx'},
    ).to_ctypes()
    _nzumx_py = _b_utils._EngineIntScalarType.to_py(_nzumx)
    _iu = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*_n_py+_nzumx_py+1,),
    )
    _uval = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nzumx_py,),
    )
    _return_scalars = b_direct_real_gen_lu(
        _n,
        _irowix,
        _a,
        _iprm,
        thresh,
        _nzlmx,
        _nzlumx,
        _nzumx,
        _il,
        _lval,
        _iu,
        _uval,
    )
    _iprm_py = _iprm
    _nzlumx_py = _return_scalars.nzlumx
    _il_py = _il
    _lval_py = _lval[:_nzlumx_py]
    _iu_py = _iu
    _uval_py = _uval
    _nnzl_py = _return_scalars.nnzl
    _nnzu_py = _return_scalars.nnzu
    _flop_py = _return_scalars.flop
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iprm', 'nzlumx', 'il', 'lval', 'iu', 'uval', 'nnzl', 'nnzu', 'flop'],
    )(
        _iprm_py,
        _nzlumx_py,
        _il_py,
        _lval_py,
        _iu_py,
        _uval_py,
        _nnzl_py,
        _nnzu_py,
        _flop_py,
    )
    return _return_data

def direct_real_gen_solve(trans, iprm, il, lval, iu, uval, b):
    u"""
    Solution of real sparse simultaneous linear equations (coefficient
    matrix already factorized).

    For full information please refer to the NAG Library document for
    f11mf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mff.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether AX = B or A^TX = B is solved.

        `trans` = 'N'

          AX = B is solved.

        `trans` = 'T'

          A^TX = B is solved.

    iprm : int, array-like, shape (7*n)
        The column permutation which defines P_c, the row permutation
        which defines P_r, plus associated data structures as computed
        by ``direct_real_gen_lu``.

    il : int, array-like, shape (:)
        Records the sparsity pattern of matrix L as computed by
        ``direct_real_gen_lu``.

    lval : float, array-like, shape (:)
        Records the nonzero values of matrix L and some nonzero values
        of matrix U as computed by ``direct_real_gen_lu``.

    iu : int, array-like, shape (:)
        Records the sparsity pattern of matrix U as computed by
        ``direct_real_gen_lu``.

    uval : float, array-like, shape (:)
        Records some nonzero values of matrix U as computed by
        ``direct_real_gen_lu``.

    b : float, array-like, shape (n, nrhs)
        The n by nrhs right-hand side matrix B.

    Returns
    -------
    b : float, ndarray, shape (n, nrhs)
        The n by nrhs solution matrix X.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nrhs = *<value>*.

            Constraint: nrhs >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 2)
            Incorrect row permutations in array `iprm`.

        (`errno` 3)
            Incorrect column permutations in array `iprm`.
    """

    from ..base.sparse import direct_real_gen_solve as b_direct_real_gen_solve

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_solve'

    _iprm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iprm'},
        iprm,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_iprm, {'fun_name': fun_name, 'entity': 'iprm'}, exp_rank=1)[0]//7, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _il = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'il'},
        il,
        copy=False,
    )
    _lval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lval'},
        lval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iu'},
        iu,
        copy=False,
    )
    _uval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'uval'},
        uval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    b_direct_real_gen_solve(
        trans,
        _n,
        _iprm,
        _il,
        _lval,
        _iu,
        _uval,
        _nrhs,
        _b,
    )
    _b_py = _b
    return _b_py

def direct_real_gen_cond(norm, n, il, lval, iu, uval, anorm):
    u"""
    Estimate condition number of real matrix, matrix already factorized
    by ``direct_real_gen_lu``.

    For full information please refer to the NAG Library document for
    f11mg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mgf.html

    Parameters
    ----------
    norm : str, length 1
        Indicates whether kappa_1(A) or kappa_infinity (A) is to be
        estimated.

        `norm` = '1' or 'O'

          kappa_1(A) is estimated.

        `norm` = 'I'

          kappa_infinity (A) is estimated.

    n : int
        n, the order of the matrix A.

    il : int, array-like, shape (:)
        Records the sparsity pattern of matrix L as computed by
        ``direct_real_gen_lu``.

    lval : float, array-like, shape (:)
        Records the nonzero values of matrix L and some nonzero values
        of matrix U as computed by ``direct_real_gen_lu``.

    iu : int, array-like, shape (:)
        Records the sparsity pattern of matrix U as computed by
        ``direct_real_gen_lu``.

    uval : float, array-like, shape (:)
        Records some nonzero values of matrix U as computed by
        ``direct_real_gen_lu``.

    anorm : float
        If `norm` = '1' or 'O', the 1-norm of the matrix A.

        If `norm` = 'I', the infinity-norm of the matrix A.

        `anorm` may be computed by calling ``direct_real_gen_norm`` with
        the same value for the argument `norm`.

    Returns
    -------
    rcond : float
        An estimate of the reciprocal of the condition number of A.
        `rcond` is set to zero if exact singularity is detected or the
        estimate underflows. If `rcond` is less than machine precision,
        A is singular to working precision.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `anorm` = *<value>*.

            Constraint: `anorm` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'O' or 'I'.
    """

    from ..base.sparse import direct_real_gen_cond as b_direct_real_gen_cond

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_cond'

    _il = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'il'},
        il,
        copy=False,
    )
    _lval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lval'},
        lval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iu'},
        iu,
        copy=False,
    )
    _uval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'uval'},
        uval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = b_direct_real_gen_cond(
        norm,
        n,
        _il,
        _lval,
        _iu,
        _uval,
        anorm,
    )
    _rcond_py = _return_scalar
    return _rcond_py

def direct_real_gen_refine(trans, icolzp, irowix, a, iprm, il, lval, iu, uval, nrhs, b, x):
    u"""
    Refined solution with error bounds of real system of linear
    equations, multiple right-hand sides.

    For full information please refer to the NAG Library document for
    f11mh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mhf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether AX = B or A^TX = B is solved.

        `trans` = 'N'

          AX = B is solved.

        `trans` = 'T'

          A^TX = B is solved.

    icolzp : int, array-like, shape (n+1)
        The new column index array of sparse matrix A. See [the F11
        Introduction].

    irowix : int, array-like, shape (asize)
        The row index array of sparse matrix A. See [the F11
        Introduction].

    a : float, array-like, shape (asize)
        The array of nonzero values in the sparse matrix A.

    iprm : int, array-like, shape (7*n)
        The column permutation which defines P_c, the row permutation
        which defines P_r, plus associated data structures as computed
        by ``direct_real_gen_lu``.

    il : int, array-like, shape (:)
        Records the sparsity pattern of matrix L as computed by
        ``direct_real_gen_lu``.

    lval : float, array-like, shape (:)
        Records the nonzero values of matrix L and some nonzero values
        of matrix U as computed by ``direct_real_gen_lu``.

    iu : int, array-like, shape (:)
        Records the sparsity pattern of matrix U as computed by
        ``direct_real_gen_lu``.

    uval : float, array-like, shape (:)
        Records some nonzero values of matrix U as computed by
        ``direct_real_gen_lu``.

    nrhs : int
        nrhs, the number of right-hand sides in B.

    b : float, array-like, shape (n, `nrhs`)
        The n by nrhs right-hand side matrix B.

    x : float, array-like, shape (n, `nrhs`)
        The n by nrhs solution matrix X, as returned by
        ``direct_real_gen_solve``.

    Returns
    -------
    x : float, ndarray, shape (n, `nrhs`)
        The n by nrhs improved solution matrix X.

    ferr : float, ndarray, shape (`nrhs`)
        `ferr`[j-1] contains an estimated error bound for the jth
        solution vector, that is, the jth column of X, for j =
        1,2,...,nrhs.

    berr : float, ndarray, shape (`nrhs`)
        `berr`[j-1] contains the component-wise backward error bound
        beta for the jth solution vector, that is, the jth column of X,
        for j = 1,2,...,nrhs.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nrhs` = *<value>*.

            Constraint: `nrhs` >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 2)
            Incorrect row permutations in array `iprm`.

        (`errno` 3)
            Incorrect column permutations in array `iprm`.
    """

    from ..base.sparse import direct_real_gen_refine as b_direct_real_gen_refine

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_refine'

    _iprm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iprm'},
        iprm,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_iprm, {'fun_name': fun_name, 'entity': 'iprm'}, exp_rank=1)[0]//7, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _il = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'il'},
        il,
        copy=False,
    )
    _lval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lval'},
        lval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iu'},
        iu,
        copy=False,
    )
    _uval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'uval'},
        uval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs = _b_utils._EngineIntScalarType(
        nrhs, {'fun_name': fun_name, 'entity': 'nrhs'},
    ).to_ctypes()
    _nrhs_py = _b_utils._EngineIntScalarType.to_py(_nrhs)
    _ferr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nrhs_py,),
    )
    _berr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nrhs_py,),
    )
    b_direct_real_gen_refine(
        trans,
        _n,
        _icolzp,
        _irowix,
        _a,
        _iprm,
        _il,
        _lval,
        _iu,
        _uval,
        _nrhs,
        _b,
        _x,
        _ferr,
        _berr,
    )
    _x_py = _x
    _ferr_py = _ferr
    _berr_py = _berr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'ferr', 'berr'],
    )(
        _x_py,
        _ferr_py,
        _berr_py,
    )
    return _return_data

def direct_real_gen_matmul(trans, alpha, icolzp, irowix, a, b, beta, c=None):
    u"""
    Real sparse nonsymmetric matrix-matrix multiply, compressed column
    storage.

    For full information please refer to the NAG Library document for
    f11mk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mkf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix A is transposed.

        `trans` = 'N'

          alpha AB+beta C is computed.

        `trans` = 'T'

          alpha A^TB+beta C is computed.

    alpha : float
        alpha, the scalar factor in the matrix multiplication.

    icolzp : int, array-like, shape (n+1)
        The new column index array of sparse matrix A. See [the F11
        Introduction].

    irowix : int, array-like, shape (asize)
        The row index array of sparse matrix A. See [the F11
        Introduction].

    a : float, array-like, shape (asize)
        The array of nonzero values in the sparse matrix A.

    b : float, array-like, shape (n, m)
        The n by m matrix B.

    beta : float
        The scalar factor beta.

    c : None or float, array-like, shape (n, m), optional
        The n by m matrix C.

    Returns
    -------
    c : float, ndarray, shape (n, m)
        C is overwritten by alpha AB+beta C or alpha A^TB+beta C
        depending on the value of `trans`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.
    """

    from ..base.sparse import direct_real_gen_matmul as b_direct_real_gen_matmul

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_matmul'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_b, 'b'), (_c, 'c')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _beta = _b_utils._EngineFloat64ScalarType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
    ).to_ctypes()
    _beta_py = _b_utils._EngineFloat64ScalarType.to_py(_beta)
    if _beta_py != 0.0:
        if c is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'c'},
                'must not be None when beta != 0.0',
            )
    if c is None:
        _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py, _m_py),
            sorder=_m_sorder,
        )
    b_direct_real_gen_matmul(
        trans,
        _n,
        _m,
        alpha,
        _icolzp,
        _irowix,
        _a,
        _b,
        _beta,
        _c,
    )
    _c_py = _c
    return _c_py

def direct_real_gen_norm(norm, n, icolzp, irowix, a):
    u"""
    1-norm, infinity-norm, largest absolute element, real, square,
    sparse matrix.

    For full information please refer to the NAG Library document for
    f11ml

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mlf.html

    Parameters
    ----------
    norm : str, length 1
        Specifies the value to be returned in `anorm`.

        `norm` = '1' or 'O'

          The 1-norm ||A||_1 of the matrix is computed, that is max_{1
          <= j <= n}sum_i = 1^n|A_{ij}|.

        `norm` = 'I'

          The infinity-norm ||A||_infinity of the matrix is computed,
          that is max_{1 <= i <= n}sum_j = 1^n|A_{ij}|.

        `norm` = 'M'

          The value max_{1 <= i,j <= n}(|A_{ij}|) (not a norm).

    n : int
        n, the order of the matrix A.

    icolzp : int, array-like, shape (`n`+1)
        The new column index array of sparse matrix A. See [the F11
        Introduction].

    irowix : int, array-like, shape (asize)
        The row index array of sparse matrix A. See [the F11
        Introduction].

    a : float, array-like, shape (asize)
        The array of nonzero values in the sparse matrix A.

    Returns
    -------
    anorm : float
        The computed quantity relating the matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            On entry, `norm` = *<value>*.

            Constraint: `norm` = '1', 'O', 'I' or 'M'.
    """

    from ..base.sparse import direct_real_gen_norm as b_direct_real_gen_norm

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_norm'

    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = b_direct_real_gen_norm(
        norm,
        n,
        _icolzp,
        _irowix,
        _a,
    )
    _anorm_py = _return_scalar
    return _anorm_py

def direct_real_gen_diag(icolzp, a, iprm, il, lval, iu, uval):
    u"""
    Real sparse nonsymmetric linear systems, diagnostic for
    ``direct_real_gen_lu``.

    For full information please refer to the NAG Library document for
    f11mm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11mmf.html

    Parameters
    ----------
    icolzp : int, array-like, shape (n+1)
        The new column index array of sparse matrix A. See [the F11
        Introduction].

    a : float, array-like, shape (asize)
        The array of nonzero values in the sparse matrix A.

    iprm : int, array-like, shape (7*n)
        The column permutation which defines P_c, the row permutation
        which defines P_r, plus associated data structures as computed
        by ``direct_real_gen_lu``.

    il : int, array-like, shape (:)
        Records the sparsity pattern of matrix L as computed by
        ``direct_real_gen_lu``.

    lval : float, array-like, shape (:)
        Records the nonzero values of matrix L and some nonzero values
        of matrix U as computed by ``direct_real_gen_lu``.

    iu : int, array-like, shape (:)
        Records the sparsity pattern of matrix U as computed by
        ``direct_real_gen_lu``.

    uval : float, array-like, shape (:)
        Records some nonzero values of matrix U as computed by
        ``direct_real_gen_lu``.

    Returns
    -------
    rpg : float
        The reciprocal pivot growth factor
        max_j(||A_j||_infinity/||U_j||_infinity). If the reciprocal
        pivot growth factor is much less than 1, the stability of the LU
        factorization may be poor.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 0.

        (`errno` 2)
            Incorrect column permutations in array `iprm`.
    """

    from ..base.sparse import direct_real_gen_diag as b_direct_real_gen_diag

    fun_name = 'naginterfaces.library.sparse.direct_real_gen_diag'

    _iprm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iprm'},
        iprm,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_iprm, {'fun_name': fun_name, 'entity': 'iprm'}, exp_rank=1)[0]//7, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _il = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'il'},
        il,
        copy=False,
    )
    _lval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lval'},
        lval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iu = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iu'},
        iu,
        copy=False,
    )
    _uval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'uval'},
        uval,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = b_direct_real_gen_diag(
        _n,
        _icolzp,
        _a,
        _iprm,
        _il,
        _lval,
        _iu,
        _uval,
    )
    _rpg_py = _return_scalar
    return _rpg_py

def real_gen_matvec(trans, a, irow, icol, x, check='N'):
    u"""
    Real, sparse, nonsymmetric matrix-vector multiply.

    For full information please refer to the NAG Library document for
    f11xa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11xaf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix A is transposed.

        `trans` = 'N'

          y = Ax is computed.

        `trans` = 'T'

          y = A^Tx is computed.

    a : float, array-like, shape (nnz)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``real_gen_sort`` may be used to order the elements
        in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    x : float, array-like, shape (n)
        The vector x.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix A,
        values of n, nnz, `irow` and `icol` should be checked.

        `check` = 'C'

          Checks are carried on the values of n, nnz, `irow` and `icol`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    y : float, ndarray, shape (n)
        The vector y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.
    """

    from ..base.sparse import real_gen_matvec as b_real_gen_matvec

    fun_name = 'naginterfaces.library.sparse.real_gen_matvec'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_gen_matvec(
        trans,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def real_symm_matvec(a, irow, icol, x, check='N'):
    u"""
    Real sparse symmetric matrix-vector multiply.

    For full information please refer to the NAG Library document for
    f11xe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11xef.html

    Parameters
    ----------
    a : float, array-like, shape (nnz)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``real_symm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    x : float, array-like, shape (n)
        The vector x.

    check : str, length 1, optional
        Specifies whether or not the SCS representation of the matrix A,
        values of n, nnz, `irow` and `icol` should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow` and
          `icol`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    y : float, ndarray, shape (n)
        The vector y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.
    """

    from ..base.sparse import real_symm_matvec as b_real_symm_matvec

    fun_name = 'naginterfaces.library.sparse.real_symm_matvec'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_real_symm_matvec(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def complex_gen_matvec(trans, a, irow, icol, x, check='N'):
    u"""
    Complex sparse non-Hermitian matrix-vector multiply.

    For full information please refer to the NAG Library document for
    f11xn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11xnf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether or not the matrix A is conjugate transposed.

        `trans` = 'N'

          y = Ax is computed.

        `trans` = 'T'

          y = A^Hx is computed.

    a : complex, array-like, shape (nnz)
        The nonzero elements in the matrix A, ordered by increasing row
        index, and by increasing column index within each row. Multiple
        entries for the same row and column indices are not permitted.
        The function ``complex_gen_sort`` may be used to order the
        elements in this way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    x : complex, array-like, shape (n)
        The vector x.

    check : str, length 1, optional
        Specifies whether or not the CS representation of the matrix A,
        values of n, nnz, `irow` and `icol` should be checked.

        `check` = 'C'

          Checks are carried on the values of n, nnz, `irow` and `icol`.

        `check` = 'N'

          None of these checks are carried out.

        See also [Further Comments].

    Returns
    -------
    y : complex, ndarray, shape (n)
        The vector y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n^2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= n.

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.
    """

    from ..base.sparse import complex_gen_matvec as b_complex_gen_matvec

    fun_name = 'naginterfaces.library.sparse.complex_gen_matvec'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_gen_matvec(
        trans,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def complex_herm_matvec(a, irow, icol, x, check='N'):
    u"""
    Complex sparse Hermitian matrix-vector multiply.

    For full information please refer to the NAG Library document for
    f11xs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11xsf.html

    Parameters
    ----------
    a : complex, array-like, shape (nnz)
        The nonzero elements in the lower triangular part of the matrix
        A, ordered by increasing row index, and by increasing column
        index within each row. Multiple entries for the same row and
        column indices are not permitted. The function
        ``complex_herm_sort`` may be used to order the elements in this
        way.

    irow : int, array-like, shape (nnz)
        The row indices of the nonzero elements supplied in array `a`.

    icol : int, array-like, shape (nnz)
        The column indices of the nonzero elements supplied in array
        `a`.

    x : complex, array-like, shape (n)
        The vector x.

    check : str, length 1, optional
        Specifies whether or not the SCS representation of the matrix A,
        values of n, nnz, `irow` and `icol` should be checked.

        `check` = 'C'

          Checks are carried out on the values of n, nnz, `irow` and
          `icol`.

        `check` = 'N'

          None of these checks are carried out.

    Returns
    -------
    y : complex, ndarray, shape (n)
        The vector y.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `check` = *<value>*.

            Constraint: `check` = 'C' or 'N'.

        (`errno` 2)
            On entry, nnz = *<value>* and n = *<value>*.

            Constraint: nnz <= n*(n+1)/2.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, the location (`irow`[I-1],`icol`[I-1]) is a
            duplicate: I = *<value>*.

        (`errno` 3)
            On entry, `a`[i-1] is out of order: i = *<value>*.

        (`errno` 3)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 3)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= n.
    """

    from ..base.sparse import complex_herm_matvec as b_complex_herm_matvec

    fun_name = 'naginterfaces.library.sparse.complex_herm_matvec'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=False,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=False,
    )
    _y = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    b_complex_herm_matvec(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        check,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def sym_rcm(icolzp, irowix, lopts, mask=None):
    u"""
    Reverse Cuthill--McKee reordering of a sparse symmetric matrix in
    CCS format.

    For full information please refer to the NAG Library document for
    f11ye

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11yef.html

    Parameters
    ----------
    icolzp : int, array-like, shape (n+1)
        `icolzp` records the index into `irowix` which starts each new
        column.

    irowix : int, array-like, shape (nnz)
        The row indices corresponding to the nonzero elements in the
        matrix A.

    lopts : bool, array-like, shape (5)
        The options to be used by ``sym_rcm``.

        `lopts`[0] = True

          Row/column i of the matrix A will only be referenced if
          `mask`[i-1] != 0, otherwise `mask` will be ignored.

        `lopts`[1] = True

          The final permutation will not be reversed, that is, the
          Cuthill--McKee ordering will be returned. The bandwidth of the
          non-reversed matrix will be the same but the profile will be
          the same or larger.

        `lopts`[2] = True

          The matrix A will be checked for symmetrical sparsity pattern,
          otherwise not.

        `lopts`[3] = True

          The bandwidth and profile of the unpermuted matrix will be
          calculated, otherwise not.

        `lopts`[4] = True

          The bandwidth and profile of the permuted matrix will be
          calculated, otherwise not.

    mask : None or int, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `lopts`[0]= True: n; otherwise: 0.

        `mask` is only referenced if `lopts`[0] is True. A value of
        `mask`[i-1] = 0 indicates that the node corresponding to row or
        column i is not to be referenced. A value of `mask`[i-1] != 0
        indicates that the node corresponding to row or column i is to
        be referenced. In particular, rows and columns not referenced
        will not be permuted.

    Returns
    -------
    perm : int, ndarray, shape (:)
        This will contain the permutation vector that describes the
        permutation matrix P for the reordering of the matrix A. The
        elements of the permutation matrix P are zero except for the
        unit elements in row i and column `perm`[i-1], i = 1,2,...n.

    info : int, ndarray, shape (4)
        Statistics about the matrix A and the permuted matrix. The
        quantities below are calculated using any masking in effect
        otherwise the value zero is returned.

        `info`[0]

          The bandwidth of the matrix A, if `lopts`[3] = True.

        `info`[1]

          The profile of the matrix A, if `lopts`[3] = True.

        `info`[2]

          The bandwidth of the permuted matrix PAP^T, if `lopts`[4] =
          True.

        `info`[3]

          The profile of the permuted matrix PAP^T, if `lopts`[4] =
          True.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 1.

        (`errno` 3)
            On entry, `irowix`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `irowix`[i-1] <= n for all i.

        (`errno` 4)
            On entry, `icolzp`[*<value>*] = *<value>* and nnz =
            *<value>*.

            Constraint: 1 <= `icolzp`[i-1] <= nnz for all i.

        (`errno` 5)
            On entry, `icolzp`[0] = *<value>*.

            Constraint: `icolzp`[0] = 1.

        (`errno` 5)
            On entry, `icolzp`[n] = *<value>* and nnz = *<value>*.

            Constraint: `icolzp`[n] = nnz+1.

        (`errno` 6)
            On entry, the matrix A is not symmetric.

            Element (*<value>*,*<value>*) has no symmetric element.
    """

    from ..base.sparse import sym_rcm as b_sym_rcm

    fun_name = 'naginterfaces.library.sparse.sym_rcm'

    _icolzp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icolzp'},
        icolzp,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineIntArrayType.get_shape(_icolzp, {'fun_name': fun_name, 'entity': 'icolzp'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _irowix = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irowix'},
        irowix,
        copy=False,
    )
    _nnz_py = _b_utils._EngineIntArrayType.get_shape(_irowix, {'fun_name': fun_name, 'entity': 'irowix'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _lopts = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lopts'},
        lopts,
        copy=False,
    )
    _mask = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'mask'},
        mask,
        copy=False,
        allow_none=True,
    )
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _perm = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _info = _b_utils._EngineIntArrayType.empty_ndarray(
        (4,),
    )
    b_sym_rcm(
        _n,
        _nnz,
        _icolzp,
        _irowix,
        _lopts,
        _mask,
        _perm,
        _info,
    )
    _perm_py = _perm
    _info_py = _info
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['perm', 'info'],
    )(
        _perm_py,
        _info_py,
    )
    return _return_data

def real_gen_sort(n, a, irow, icol, dup, zer):
    u"""
    Real sparse nonsymmetric matrix reorder function.

    For full information please refer to the NAG Library document for
    f11za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11zaf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, array-like, shape (nnz)
        The nonzero elements of the matrix A. These may be in any order
        and there may be multiple nonzero elements with the same row and
        column indices.

    irow : int, array-like, shape (nnz)
        The row indices corresponding to the elements supplied in the
        array `a`.

    icol : int, array-like, shape (nnz)
        The column indices corresponding to the elements supplied in the
        array `a`.

    dup : str, length 1
        Indicates how elements in `a` with duplicate row and column
        indices are to be treated.

        `dup` = 'R'

          Duplicate entries are removed, only the first entry is kept.

        `dup` = 'S'

          The relevant values in `a` are summed.

        `dup` = 'F'

          The function fails with `errno` = 3 on detecting a duplicate.

    zer : str, length 1
        Indicates how elements in `a` with zero values are to be
        treated.

        `zer` = 'R'

          The entries are removed.

        `zer` = 'K'

          The entries are kept.

        `zer` = 'F'

          The function fails with `errno` = 4 on detecting a zero.

    Returns
    -------
    a : float, ndarray, shape (nnz)
        The nonzero elements ordered by increasing row index, and by
        increasing column index within each row. Each nonzero element
        has a unique row and column index.

    irow : int, ndarray, shape (nnz)
        The first nnz elements contain the row indices corresponding to
        the elements returned in the array `a`.

    icol : int, ndarray, shape (nnz)
        The first nnz elements contain the column indices corresponding
        to the elements returned in the array `a`.

    istr : int, ndarray, shape (`n`+1)
        `istr`[i-1]-1, for i = 1,2,...,`n`, is the starting address in
        the arrays `a`, `irow` and `icol` of row i of the matrix A.
        `istr`[`n`]-1 is the address of the last element in `a` plus
        one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'R', 'K' or 'F'.

        (`errno` 1)
            On entry, `dup` = *<value>*.

            Constraint: `dup` = 'R', 'S' or 'F'.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= `n`.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= `n`.

        (`errno` 3)
            On entry, a duplicate entry has been found in row i and
            column j: i = *<value>*, j = *<value>*.

        (`errno` 4)
            On entry, a zero entry has been found in row i and column j:
            i = *<value>*, j = *<value>*.
    """

    from ..base.sparse import real_gen_sort as b_real_gen_sort

    fun_name = 'naginterfaces.library.sparse.real_gen_sort'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalar = b_real_gen_sort(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        dup,
        zer,
        _istr,
    )
    _nnz_py = _return_scalar
    _a_py = _a[:_nnz_py]
    _irow_py = _irow[:_nnz_py]
    _icol_py = _icol[:_nnz_py]
    _istr_py = _istr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'istr'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _istr_py,
    )
    return _return_data

def real_symm_sort(n, a, irow, icol, dup, zer):
    u"""
    Real sparse symmetric matrix reorder function.

    For full information please refer to the NAG Library document for
    f11zb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11zbf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : float, array-like, shape (nnz)
        The nonzero elements of the lower triangular part of the real
        matrix A. These may be in any order and there may be multiple
        nonzero elements with the same row and column indices.

    irow : int, array-like, shape (nnz)
        The row indices corresponding to the elements supplied in the
        array `a`.

    icol : int, array-like, shape (nnz)
        The column indices corresponding to the elements supplied in the
        array `a`.

    dup : str, length 1
        Indicates how elements in `a` with duplicate row and column
        indices are to be treated.

        `dup` = 'R'

          Duplicate entries are removed, only the first entry is kept.

        `dup` = 'S'

          The relevant values in `a` are summed.

        `dup` = 'F'

          The function fails with `errno` = 3 on detecting a duplicate.

    zer : str, length 1
        Indicates how elements in `a` with zero values are to be
        treated.

        `zer` = 'R'

          The entries are removed.

        `zer` = 'K'

          The entries are kept.

        `zer` = 'F'

          The function fails with `errno` = 4 on detecting a zero.

    Returns
    -------
    a : float, ndarray, shape (nnz)
        The lower triangular nonzero elements ordered by increasing row
        index, and by increasing column index within each row. Each
        nonzero element has a unique row and column index.

    irow : int, ndarray, shape (nnz)
        The first nnz elements contain the row indices corresponding to
        the elements returned in the array `a`.

    icol : int, ndarray, shape (nnz)
        The first nnz elements contain the column indices corresponding
        to the elements returned in the array `a`.

    istr : int, ndarray, shape (`n`+1)
        `istr`[i-1]-1, for i = 1,2,...,`n`, is the starting address in
        the arrays `a`, `irow` and `icol` of row i of the matrix A.
        `istr`[`n`]-1 is the address of the last element in `a` plus
        one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'R', 'K' or 'F'.

        (`errno` 1)
            On entry, `dup` = *<value>*.

            Constraint: `dup` = 'R', 'S' or 'F'.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and
            `irow`[i-1] = *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= `irow`[i-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= `n`.

        (`errno` 3)
            On entry, a duplicate entry has been found in row I and
            column J: I = *<value>*, J = *<value>*.

        (`errno` 4)
            On entry, a zero entry has been found in row I and column J:
            I = *<value>*, J = *<value>*.
    """

    from ..base.sparse import real_symm_sort as b_real_symm_sort

    fun_name = 'naginterfaces.library.sparse.real_symm_sort'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalar = b_real_symm_sort(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        dup,
        zer,
        _istr,
    )
    _nnz_py = _return_scalar
    _a_py = _a[:_nnz_py]
    _irow_py = _irow[:_nnz_py]
    _icol_py = _icol[:_nnz_py]
    _istr_py = _istr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'istr'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _istr_py,
    )
    return _return_data

def real_rect_sort(m, a, irow, n=None, icol=None, istc=None, dup='S', zer='R'):
    u"""
    Sorts and merges elements of a real sparse rectangular matrix,
    represented in coordinate storage format, and provides the
    compressed column storage format of the result.

    For full information please refer to the NAG Library document for
    f11zc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11zcf.html

    Parameters
    ----------
    m : int
        m, the number of rows in the matrix A.

    a : float, array-like, shape (nnz)
        The nonzero elements of the matrix A. If column indices are
        supplied via `icol`, the elements may be in any order. If column
        starting addresses are supplied via `istc`, the elements must be
        ordered by increasing column index. There may be multiple
        nonzero elements with the same row and column indices.

    irow : int, array-like, shape (nnz)
        The row indices corresponding to the elements supplied in the
        array `a`.

    n : None or int, optional
        n, the number of columns in the matrix A.

        If you are providing column starting addresses via `istc` then
        your supplied value of `n` will be ignored and it will be
        inferred from that array instead.

    icol : None or int, array-like, shape (nnz), optional
        Must be used to supply the column indices corresponding to the
        elements supplied in the array `a`, when `a` is represented in
        coordinate storage format.

    istc : None or int, array-like, shape (n+1), optional
        Must be used to supply the starting address of each column, as
        supplied in the array `a`, when `a` is represented in compressed
        column storage format.

    dup : str, length 1, optional
        Indicates how elements in `a` with duplicate row and column
        indices are to be treated.

        `dup` = 'R'

          Duplicate entries are removed, only the first entry is kept.

        `dup` = 'S'

          The relevant values in `a` are summed.

        `dup` = 'F'

          The function fails with `errno` = 12 on detecting a duplicate.

    zer : str, length 1, optional
        Indicates how elements in `a` with zero values are to be
        treated.

        `zer` = 'R'

          The entries are removed.

        `zer` = 'K'

          The entries are kept.

        `zer` = 'F'

          The function fails with `errno` = 13 on detecting a zero.

    Returns
    -------
    a : float, ndarray, shape (nnz)
        The nonzero elements ordered by increasing column index, and by
        increasing row index within each column. Each nonzero element
        has a unique row and column index.

    irow : int, ndarray, shape (nnz)
        The row indices corresponding to the elements returned in the
        array `a`.

    icol : int, ndarray, shape (nnz)
        The column indices corresponding to the elements returned in the
        array `a`.

    istc : int, ndarray, shape (n+1)
        The starting address of each column, as returned in the array
        `a`. `istc`[n]-1 is the address of the last element in `a` plus
        one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 0.

        (`errno` 5)
            On entry, `dup` = *<value>*.

            Constraint: `dup` = 'R', 'S' or 'F'.

        (`errno` 6)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'R', 'K' or 'F'.

        (`errno` 7)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and `m` =
            *<value>*.

            Constraint: 1 <= `irow`[i-1] <= `m`.

        (`errno` 8)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and n =
            *<value>*.

            Constraint: if column indices are supplied, then 1 <=
            `icol`[i-1] <= n.

        (`errno` 9)
            On entry, `istc`[0] = *<value>*.

            Constraint: `istc`[0] = 1.

        (`errno` 10)
            On entry, i = *<value>*, `istc`[i-1] = *<value>* and
            `istc`[i] = *<value>*.

            Constraint: `istc`[i-1] <= `istc`[i], for i = 1,2,...,n

        (`errno` 11)
            On entry, n = *<value>*, `istc`[n] = *<value>* and nnz =
            *<value>*.

            Constraint: `istc`[n] = nnz+1.

        (`errno` 12)
            On entry, a duplicate entry has been found in row *<value>*
            and column *<value>*.

        (`errno` 13)
            On entry, a zero entry has been found in row *<value>* and
            column *<value>*.

        (`errno` 14)
            Exactly one of `icol` and `istc` must be provided.

        (`errno` 15)
            `n` or `istc` must be provided.
    """

    from ..base.sparse import real_rect_sort as b_real_rect_sort

    fun_name = 'naginterfaces.library.sparse.real_rect_sort'

    if (icol is not None) and istc is not None or icol is None and istc is None:
        _b_utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'Exactly one of icol and istc must be provided.',
            ],
            errcodes=(14,),
        ).manage()
    if n is None and istc is None:
        _b_utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'n or istc must be provided.',
            ],
            errcodes=(15,),
        ).manage()
    _istc = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'istc'},
        istc,
        copy=True,
        allow_none=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
        allow_none=True,
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    if n is None:
        _n = _b_utils._EngineIntScalarType(
            _b_utils._EngineIntArrayType.get_shape(_istc, {'fun_name': fun_name, 'entity': 'istc'}, exp_rank=1, allow_none=True)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
        ).to_ctypes()
        _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    else:
        _n = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_n'},
        ).to_ctypes()
        _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nnz_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    if icol is None:
        _icol = _b_utils._EngineIntArrayType.empty_ndarray(
            (_nnz_py,),
        )
    else:
        _icol = _utils._to_ndarray(
            {'fun_name': fun_name, 'entity': 'icol'},
            icol,
            copy=True,
        )
    if istc is None:
        _istc = _b_utils._EngineIntArrayType.empty_ndarray(
            (_n_py+1,),
        )
    if istc is None:
        _store = 1
        _store_py = _store
    else:
        _store = 2
        _store_py = _store
    _return_scalar = b_real_rect_sort(
        m,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _istc,
        _store,
        dup,
        zer,
    )
    _nnz_py = _return_scalar
    _a_py = _a[:_nnz_py]
    _irow_py = _irow[:_nnz_py]
    _icol_py = _icol[:_nnz_py]
    _istc_py = _istc
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'istc'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _istc_py,
    )
    return _return_data

def complex_gen_sort(n, a, irow, icol, dup, zer):
    u"""
    Complex sparse non-Hermitian matrix reorder function.

    For full information please refer to the NAG Library document for
    f11zn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11znf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, array-like, shape (nnz)
        The nonzero elements of the matrix A. These may be in any order
        and there may be multiple nonzero elements with the same row and
        column indices.

    irow : int, array-like, shape (nnz)
        The row indices corresponding to the elements supplied in the
        array `a`.

    icol : int, array-like, shape (nnz)
        The column indices corresponding to the elements supplied in the
        array `a`.

    dup : str, length 1
        Indicates how elements in `a` with duplicate row and column
        indices are to be treated.

        `dup` = 'R'

          Duplicate entries are removed, only the first entry is kept.

        `dup` = 'S'

          The relevant values in `a` are summed.

        `dup` = 'F'

          The function fails with `errno` = 3 on detecting a duplicate.

    zer : str, length 1
        Indicates how elements in `a` with zero values are to be
        treated.

        `zer` = 'R'

          The entries are removed.

        `zer` = 'K'

          The entries are kept.

        `zer` = 'F'

          The function fails with `errno` = 4 on detecting a zero.

    Returns
    -------
    a : complex, ndarray, shape (nnz)
        The nonzero elements ordered by increasing row index, and by
        increasing column index within each row. Each nonzero element
        has a unique row and column index.

    irow : int, ndarray, shape (nnz)
        The first nnz elements contain the row indices corresponding to
        the elements returned in the array `a`.

    icol : int, ndarray, shape (nnz)
        The first nnz elements contain the column indices corresponding
        to the elements returned in the array `a`.

    istr : int, ndarray, shape (`n`+1)
        `istr`[i-1]-1, for i = 1,2,...,`n`, is the starting address in
        the arrays `a`, `irow` and `icol` of row i of the matrix A.
        `istr`[`n`]-1 is the address of the last element in `a` plus
        one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'R', 'K' or 'F'.

        (`errno` 1)
            On entry, `dup` = *<value>*.

            Constraint: `dup` = 'R', 'S' or 'F'.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, i = *<value>*, `icol`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `icol`[i-1] >= 1 and `icol`[i-1] <= `n`.

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= `n`.

        (`errno` 3)
            On entry, a duplicate entry has been found in row I and
            column J: I = *<value>*, J = *<value>*.

        (`errno` 4)
            On entry, a zero entry has been found in row I and column J:
            I = *<value>*, J = *<value>*.
    """

    from ..base.sparse import complex_gen_sort as b_complex_gen_sort

    fun_name = 'naginterfaces.library.sparse.complex_gen_sort'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalar = b_complex_gen_sort(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        dup,
        zer,
        _istr,
    )
    _nnz_py = _return_scalar
    _a_py = _a[:_nnz_py]
    _irow_py = _irow[:_nnz_py]
    _icol_py = _icol[:_nnz_py]
    _istr_py = _istr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'istr'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _istr_py,
    )
    return _return_data

def complex_herm_sort(n, a, irow, icol, dup, zer):
    u"""
    Complex sparse Hermitian matrix reorder function.

    For full information please refer to the NAG Library document for
    f11zp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f11/f11zpf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    a : complex, array-like, shape (nnz)
        The nonzero elements of the lower triangular part of the complex
        matrix A. These may be in any order and there may be multiple
        nonzero elements with the same row and column indices.

    irow : int, array-like, shape (nnz)
        The row indices corresponding to the elements supplied in the
        array `a`.

    icol : int, array-like, shape (nnz)
        The column indices corresponding to the elements supplied in the
        array `a`.

    dup : str, length 1
        Indicates how elements in `a` with duplicate row and column
        indices are to be treated.

        `dup` = 'R'

          Duplicate entries are removed, only the first entry is kept.

        `dup` = 'S'

          The relevant values in `a` are summed.

        `dup` = 'F'

          The function fails with `errno` = 3 on detecting a duplicate.

    zer : str, length 1
        Indicates how elements in `a` with zero values are to be
        treated.

        `zer` = 'R'

          The entries are removed.

        `zer` = 'K'

          The entries are kept.

        `zer` = 'F'

          The function fails with `errno` = 4 on detecting a zero.

    Returns
    -------
    a : complex, ndarray, shape (nnz)
        The lower triangular nonzero elements ordered by increasing row
        index, and by increasing column index within each row. Each
        nonzero element has a unique row and column index.

    irow : int, ndarray, shape (nnz)
        The first nnz elements contain the row indices corresponding to
        the elements returned in the array `a`.

    icol : int, ndarray, shape (nnz)
        The first nnz elements contain the column indices corresponding
        to the elements returned in the array `a`.

    istr : int, ndarray, shape (`n`+1)
        `istr`[i-1]-1, for i = 1,2,...,`n`, is the starting address in
        the arrays `a`, `irow` and `icol` of row i of the matrix A.
        `istr`[`n`]-1 is the address of the last element in `a` plus
        one.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'R', 'K' or 'F'.

        (`errno` 1)
            On entry, `dup` = *<value>*.

            Constraint: `dup` = 'R', 'S' or 'F'.

        (`errno` 1)
            On entry, nnz = *<value>*.

            Constraint: nnz >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, I = *<value>*, `icol`[I-1] = *<value>* and
            `irow`[I-1] = *<value>*.

            Constraint: `icol`[I-1] >= 1 and `icol`[I-1] <= `irow`[I-1].

        (`errno` 2)
            On entry, i = *<value>*, `irow`[i-1] = *<value>* and `n` =
            *<value>*.

            Constraint: `irow`[i-1] >= 1 and `irow`[i-1] <= `n`.

        (`errno` 3)
            On entry, a duplicate entry has been found in row I and
            column J: I = *<value>*, J = *<value>*.

        (`errno` 4)
            On entry, a zero entry has been found in row I and column J:
            I = *<value>*, J = *<value>*.
    """

    from ..base.sparse import complex_herm_sort as b_complex_herm_sort

    fun_name = 'naginterfaces.library.sparse.complex_herm_sort'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _nnz_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=1)[0]
    _nnz = _b_utils._EngineIntScalarType(
        _nnz_py, {'fun_name': fun_name, 'entity': '_nnz'},
    ).to_ctypes()
    _irow = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irow'},
        irow,
        copy=True,
    )
    _icol = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'icol'},
        icol,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _istr = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py+1,),
    )
    _return_scalar = b_complex_herm_sort(
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        dup,
        zer,
        _istr,
    )
    _nnz_py = _return_scalar
    _a_py = _a[:_nnz_py]
    _irow_py = _irow[:_nnz_py]
    _icol_py = _icol[:_nnz_py]
    _istr_py = _istr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'irow', 'icol', 'istr'],
    )(
        _a_py,
        _irow_py,
        _icol_py,
        _istr_py,
    )
    return _return_data
